#!/usr/bin/perl -W
# use strict;

  our $beamline         = '23i';                        # biocat=18ID, gmca=23i, 23o, 23b
  our $undulator        = 'ID23ds';                     # biocat=ID18, gmca=ID23ds,...(?)

  our $shutter          = $beamline.':SH:mp:bo';        # shutter control
  our $shutter_readback = $beamline.':SH:mp:bo:rbk';    # shutter readdback
  our $shutter_open     = 1;                            # Digital I/O status for shutter open
  our $shutter_closed   = 1 - $shutter_open;		# Digital I/O status for shutter closed

# our $HUTCHshutterSTS  = $beamline.':MBSHI:status';	# endstation shutter status PV: 1=closed, 0=open
# our $HUTCHshutterOPE  = $beamline.':V2000:6';		# endstation shutter open PV
# our $HUTCHshutterCLO  = $beamline.':V2000:7';		# endstation shutter close PV
  our $HUTCHshutterST_OP= 'PA:23ID:STA_D_SDS_OPEN_PL';	# endstation shutter status PV:    1=open,   0=closed
  our $HUTCHshutterST_CL= 'PB:23ID:STA_D_SDS_CLSD_PL';	# endstation shutter status PV:    1=closed, 0=open,
  our $HUTCHshutterSTS  = '23ID:BLEPS:SDS_CLOSED';	# endstation shutter status PV:    1=closed, 0=open
  our $HUTCHshutterOPE  = '23ID:BLEPS:D_Shutter_Open';	# endstation shutter open PV
  our $HUTCHshutterCLO  = '23ID:BLEPS:D_Shutter_Close';	# endstation shutter close PV
  our $HUTCHshutterOpen = 0;
  our $HUTCHshutterClosed = 1 - $HUTCHshutterOpen;

# our $CshutterSTS      = $beamline.':IBSH:status';	# C-shutter status PV (IDin only): 1=closed, 0=open, 
# our $CshutterOPE      = $beamline.':V2000:4';		# C-shutter open PV   (IDin only)
# our $CshutterCLO      = $beamline.':V2000:5';		# C-shutter close PV  (IDin only)
  our $CshutterST_OP    = 'PA:23ID:STA_C_SCS_OPEN_PL';	# C-shutter status PV (IDin only): 0=open,   1=closed
  our $CshutterST_CL    = 'PB:23ID:STA_C_SCS_CLSD_PL';	# C-shutter status PV (IDin only): 1=closed, 0=open
  our $CshutterSTS      = '23ID:BLEPS:SCS_CLOSED';	# C-shutter status PV:             1=closed, 0=open
  our $CshutterOPE      = '23ID:BLEPS:C_Shutter_Open';	# C-shutter open PV   (IDin only)
  our $CshutterCLO      = '23ID:BLEPS:C_Shutter_Close';	# C-shutter close PV  (IDin only)
  our $CshutterOpen     = 0;
  our $CshutterClosed   = 1 - $CshutterOpen;

# our $FEshutter        = 'FE:23:ID:FEshutter';         # write: 1=open, 0=close (no write permissions and not used at GMCA)
# our $FEshutterSTS     = '23ID:FES:status';		# ID frontend shutter status PV (ACCEL): 0=open, 1=closed 
# our $FEshutterOPE     = '23ID:V2000:0';		# ID frontend shutter open PV
# our $FEshutterCLO     = '23ID:V2000:1';		# ID frontend shutter close PV
  our $FEshutterPOS     = 'EPS:23:ID:PS2:POSITION';	# ID frontend shutter status PV (APS): 0=moving, 1=closed, 2=opened, 3=error
  our $FEshutterST_OP   = 'PA:23ID:STA_A_FES_OPEN_PL';	# ID frontend shutter status PV:   1=open,   0=closed 
  our $FEshutterST_CL   = 'PB:23ID:STA_A_FES_CLSD_PL';	# ID frontend shutter status PV:   0=open,   1=closed 
  our $FEshutterSTS     = '23ID:BLEPS:FES_CLOSED';	# endstation shutter status PV:    1=closed, 0=open
  our $FEshutterOPE     = '23ID:BLEPS:A_Shutter_Open';	# ID frontend shutter open PV
  our $FEshutterCLO     = '23ID:BLEPS:A_Shutter_Close';	# ID frontend shutter close PV
  our $FEshutterOpen    = 0;
  our $FEshutterClosed  = 1 - $FEshutterOpen;

  our $STRUCK_MODEL     = 'SIS3820';			# supported: SIS3801 and SIS3820 (can be auto-detected if commented out)
  our $struckName       = $beamline.':s1';              # struck name (mcs=BioCAT, s1=GMCA)
  our $struck_ext       = ':mca';                       # struck extension for individual channesl (""=BioCAT, ":mca"=GMCA)
  our $struckChName     = $struckName.$struck_ext;	# struck name (mcs=BioCAT, s1:mca=GMCA)
  our $scanrecName      = $beamline.':2:scan1';         # scanrec name (scan1=BioCAT, 1:scan1=GMCA)
  our $scanrecAbort     = $beamline.':2:AbortScans';    # scanrec abort PV (AbortScans=BioCAT, 1:AbortScans=GMCA)
  our $scanrecPause     = $beamline.':2:scanPause';     # scanrec pause PV (scanPause=BioCAT, 1:scanPause=GMCA)
  our $joergerName      = $beamline.':scaler';          # Joerger scaler name (scaler=BioCAT & GMCA)
# our $bluiceLogPV      = $beamline.':FPE:cmd_input';	# where to write log_note, log_warn and log_error
  our $bluiceLogPV      = $beamline.':FPE:cmd_output256';# where to write log_note, log_warn and log_error
# our $JBLUICE_SERVER   = 'http://10.20.103.69:8001';   # bl1ws2-40g.gmca.aps.anl.gov
  our $JBLUICE_SERVER   = 'http://10.20.103.71:8001';   # bl1ws3-40g.gmca.aps.anl.gov
  our $robot            = $beamline.':R:';              # sample mounting robot prefix
  our $attenuatorPV     = $beamline.':bo0:ch';		# intensity attenuators channel number prefix in the PV
  our $attenuatrbPV     = $beamline.':rb0:ch';		# intensity attenuators channel number readback prefix in the PV
  our $nAttenuators     = 16;		                # number of intensity attenuators
  our $Attenuators_IN   = 0;				# Digital I/O state of intensity attenuators when IN
  our $axisCameraURL    = '192.168.1.21';		# Axis Video Server URL (sample, on videonet)
  our $axisCameraURL2   = '192.168.1.20';		# Axis Video Camera (hutch)
  our $mysqlHost        = 'bl1upper';			# MySQL host for this beamline
  our $mysqlUser        = 'dhs';			# MySQL user (BluIce)
  our $mysqlPass        = '';				# MySQL pass (BluIce)
  our $mysqlDb          = 'blc2004';			# MySQL DB (BluIce)
  our $HAVE_XYZ         = 1;				# comment out when using crystal XY & gonio St-SH
  our $PLATFORM_IS_64BIT = 1;				# what is this computer OS: 64 bit or 32?
  our $CAQTDM           = 0;				# use caqtdm instead of medm
  our $USE_FALCON       = 1;				# 0=Canberra, 1=Falcon
  our $have_KBM         = 1;				# does beamline have KBM?
  our $have_TF          = 0;				# does beamline have Transfocator?
  our $have_CRL         = 0;				# does beamline have fixed CRL box?

  if (defined &setEpicsRetry) {&setEpicsRetry();}       # retry no more than 20 times
  if (defined &setEZCA32or64) {&setEZCA32or64();}
