#!/usr/bin/perl -W
#--------------------------------------------------------------
# These are PERL beamline controls subs for CAEN AH501D electrometer
#                             Perl scripts
#
#     			Author: Sergey Stepanov
#
# Version-1.0:  2016/09/12 initial implementation
#--------------------------------------------------------------
# List of functions:
# ^^^^^^^^^^^^^^^^^
#  1.            $status = &setCAEN($caen,[$avertime,$range]);
#  2.            $status = &startCAEN($caen);
#  3.            $status = &stopCAEN($caen);
#  4.             $state = &getCAEN($caen);			#0=stopped, 1=acquiring, -1=error
#  5. ($status,$current) = &readCAEN($caen,[\@parts]);		#@parts=($sumX,$sumY,$diffX,$diffY,$posX,$posy)
#  6.   ($status,$range) = &getCAENrange($caen,['String']);
#  7.            $status = &setCAENrange($caen,$range);
#  8.   ($status,$label) = &getCAENrangeLabel($caen,$range);
#  9.          $avertime = &getCAENaverTime($caen);
# 10.            $status = &setCAENaverTime($caen,$avertime);
#--------------------------------------------------------------
# use warnings::unused;
  use strict;

#=================================================================

# sub setCAEN($caen,[$avertime,$range]) {
sub setCAEN($;$$) {			#returns ($status)

  my ($script, $caen, $avertime, $range, $status, $pv);

  $script   = 'setCAEN';
  $caen     = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"
  $avertime = shift(@_);  if (!defined $avertime) {$avertime = 0.1;}
  $range    = shift(@_);  if (!defined $range)    {$range    = 0;}

  if ($range < 0 || $range > 2) {
     print STDOUT '!!! '.$script.': incorrect RANGE parameter. Must be:'."\n"
		  .'  0: (+-2.5mA), 1: (+-2.5uA), or 2: (+-2.5nA)'."\n";
     return 1;
  }

  $status = &stopCAEN($caen);						#stop
  if ($status) {return $status;}

  $pv = $caen.'AcquireMode';
  $status = Pezca::PutStringOldCa($pv, 'Continuous');
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'Range';
  $status = Pezca::PutDoubleOldCa($pv, $range);				#set range
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'Geometry';
  $status = Pezca::PutStringOldCa($pv, 'Diamond');
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'ValuesPerRead';
  $status = Pezca::PutDoubleOldCa($pv, 1);
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'AveragingTime';
  $status = Pezca::PutDoubleOldCa($pv, $avertime);
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'NumChannels';
  $status = Pezca::PutStringOldCa($pv, '4');
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'ReadFormat';
# $status = Pezca::PutStringOldCa($pv, 'Binary');
  $status = Pezca::PutStringOldCa($pv, 'ASCII');
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'Resolution';
  $status = Pezca::PutStringOldCa($pv, '24 bits');
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'TriggerMode';
  $status = Pezca::PutStringOldCa($pv, 'Free run');
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $pv = $caen.'BiasState';
  $status = Pezca::PutStringOldCa($pv, 'On');
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $status = &startCAEN($caen);						#start
  return $status;
}

#=================================================================

# sub startCAEN($caen) {
sub startCAEN($) {							#returns ($status)

  my ($script, $caen, $status, $pv);

  $script = 'startCAEN';
  $caen   = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"

  $pv = $caen.'Acquire';
# $status = Pezca::PutDoubleOldCa($pv,1);
  $status = Pezca::PutStringOldCa($pv,'Acquire');
  &report_only($script, $status, $pv);
  return $status;
}

#=================================================================

# sub stopCAEN($caen) {
sub stopCAEN($) {						#returns ($status)

  my ($script, $caen, $status, $pv);

  $script = 'stopCAEN';
  $caen   = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"

  $pv = $caen.'Acquire';
# $status = Pezca::PutDoubleOldCa($pv,0);
  $status = Pezca::PutStringOldCa($pv,'Done');
  &report_only($script, $status, $pv);
  return $status;
}

#=================================================================

# sub getCAEN($caen) {
sub getCAEN($) {						#returns $state: 0=stopped, 1=acquiring, -1=error

  my ($script, $caen, $status, $state, $pv);

  $script = 'getCAEN';
  $caen   = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"

  $pv = $caen.'Acquire';
  ($status,$state) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     $state = -1;
  }
  return $state;
}

#=================================================================

# sub readCAEN($caen,[\@parts]) {		#@parts=($sumX,$sumY,$diffX,$diffY,$posX,$posy)
sub readCAEN($;\@) {				#returns ($status,$current) and optionally @parts

  my ($script, $caen, $current, $partsRef, $status);
  my ($pv, $val, @names, $name, $i);

  $script   = 'readCAEN';
  $caen     = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"
  $partsRef = shift(@_);

  $pv = $caen.'SumAll:MeanValue_RBV';
  ($status,$current) = Pezca::GetDouble($pv);
  if ($status) {&report_only($script, $status, $pv); $current=0; return $status;}

  if (defined $partsRef) {
     @$partsRef = ();				#@parts=($sumX,$sumY,$diffX,$diffY,$posX,$posy)
     $i = 0;
     @names = qw(SumX SumY DiffX DiffY PosX PosY);
     foreach $name (@names) {
        $pv = $caen.$name.':MeanValue_RBV';
        ($status,$val) = Pezca::GetDouble($pv);
        if ($status) {&report_only($script, $status, $pv); return $status;}
        @$partsRef[$i] = $val; $i++;
     }
  }
  return (0,$current);
}

#=================================================================

# sub getCAENrange($caen,['string']) {
sub getCAENrange($;$) {				#returns ($status,$range)

  my ($script, $caen, $valtype, $status, $pv, $range);

  $script  = 'getCAENrange';
  $caen    = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"
  $valtype = shift(@_);  if (!defined $valtype) {$valtype = 'double';}

  $pv = $caen.'Range_RBV';
  if    ($valtype =~ /^double$/i) {($status,$range) = Pezca::GetDouble($pv);}
  elsif ($valtype =~ /^string$/i) {($status,$range) = Pezca::GetString($pv);}
  else {print STDOUT '*** '.$script.': incorrect requsted value type "'.$valtype.'"'."\n"; $status=-1;}
  if ($status) {&report_only($script, $status, $pv); $range=-1;}

  return ($status,$range);
}

#=================================================================

# sub setCAENrange($caen,$range) {
sub setCAENrange($$) {				#returns ($status,$range)

  my ($script, $caen, $valtype, $status, $pv, $range);

  $script = 'setCAENrange';
  $caen   = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"
  $range  = shift(@_);

  $pv = $caen.'Range';
  if    ($range =~ /^[012]{1}$/)            {$status = Pezca::PutDoubleOldCa($pv,$range);}
  elsif ($range =~ /^\+\- 2\.5 [mun]{1}A$/) {$status = Pezca::PutStringOldCa($pv,$range);}
  else  {print STDOUT '*** '.$script.': incorrect range value "'.$range.'"'."\n"; return -1;}
  &report_only($script, $status, $pv);
  return $status;
}

#=================================================================

# sub getCAENrangeLabel($caen,$range) {
sub getCAENrangeLabel($$) {				#returns ($status,$label)

  my ($script, $caen, $range, $status, $pv, $fld, $label);

  $script = 'getCAENrangeLabel';
  $caen   = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"
  $range  = shift(@_);  
  $label = '';
  if (!defined $range) {
     print STDOUT '*** '.$script.': no range specified!'."\n";  return (-1,'');
  }
  elsif ($range !~ /^[012]$/) {
     print STDOUT '*** '.$script.': incorrect range='.$range.' instead of 0-2.'."\n";  return (-1,'');
  }
  $fld = qw(ZRST ONST TWST)[$range];

  $pv = $caen.'Range_RBV.'.$fld;
  ($status,$label) = Pezca::GetString($pv);
  if ($status) {&report_only($script, $status, $pv); $label='';}

  return ($status,$label);
}


#=================================================================

# sub getCAENaverTime($caen) {
sub getCAENaverTime($) {			#returns ($avertime)

  my ($script, $caen, $avertime, $status, $pv);

  $script   = 'getCAENaverTime';
  $caen     = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"
  $avertime = shift(@_);  if (!defined $avertime) {$avertime = 0.1;}

  $pv = $caen.'AveragingTime';
  ($status,$avertime) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv); 
     $avertime = 0;
  }
  return $avertime;
}


#=================================================================

# sub setCAENaverTime($caen,$avertime) {
sub setCAENaverTime($$) {			#returns ($status)

  my ($script, $caen, $avertime, $status, $pv);

  $script   = 'setCAENaverTime';
  $caen     = shift(@_);  if ($caen !~ /:$/) {$caen .= ':';}		#append ":"
  $avertime = shift(@_);  if (!defined $avertime) {$avertime = 0.1;}

  $status = &stopCAEN($caen);						#stop
  if ($status) {return $status;}

  $pv = $caen.'AveragingTime';
  $status = Pezca::PutDoubleOldCa($pv, $avertime);
  if ($status) {&report_only($script, $status, $pv); return $status;}

  $status = &startCAEN($caen);						#start
  return $status;
}
