#!/usr/bin/perl -W
#--------------------------------------------------------------
# These are CCD and PAD area detectors related subs for beamline controls
#                             Perl scripts
#
#     			Author: Sergey Stepanov
#
# Version-1.0:  2014/01/25 using DBI
#--------------------------------------------------------------
# List of functions:
# ^^^^^^^^^^^^^^^^^
#  1. ($ccdSerial,$status) = &getCCDserial();
#  2. ($ccdName,$status)   = &getCCDname();
#  3. ($ccdServer,$status) = &getCCDfpehost();
#--------------------------------------------------------------
# use warnings::unused;
  use strict;
# use DBI;			# mySQL interface (for getting Detector serial number)

#=================================================================

# sub getCCDserial() {
sub getCCDserial() {			#returns ($serial,$status)

### This routine returns the serial number of beamline area detector

  our ($mysqlHost, $mysqlDb, $mysqlUser, $mysqlPass);
  my ($script, $mysqlTbl, $id, $nrecords, $serial);
  my ($dsn, $dbh, $sth, $hashref);

  $script   = 'getCCDserial';

###############################
  use if ! defined &DBI, 'DBI';
###############################

  $mysqlTbl = 'Config';
  $id       = 'detector_serial';

  $dsn = 'DBI:mysql:host='.$mysqlHost.';database='.$mysqlDb;
  $dbh = DBI->connect($dsn,$mysqlUser,$mysqlPass,{PrintError=>0,});
  if (!defined $dbh) {
     print STDOUT '*** '.$script.': Cannot connect to mySQL server '.$mysqlHost."\n";
     return (undef,-1);
  }
  $sth = $dbh->prepare('SELECT * FROM '.$mysqlTbl.' WHERE id="'.$id.'"');
  $nrecords = $sth->execute;				#nrecords works for MySQL only. For SQLite result is def/undef only.
  if (!defined $nrecords) {
     print STDOUT '*** '.$script.': Failed reading ID='.$id
                        .' in table='.$mysqlDb
                        .' in DB='.$mysqlTbl
                        .' at mySQL server='.$mysqlHost."\n"
                 .'*** '.$script.': '.$sth->errstr."\n";
     $sth->finish();
     $dbh->disconnect();
     return (undef,-2);
  }
  elsif ($nrecords == 0) {
     print STDOUT '*** '.$script.': No records for ID='.$id
                        .' in table='.$mysqlDb
                        .' in DB='.$mysqlTbl
                        .' at mySQL server '.$mysqlHost."\n";
     $sth->finish();
     $dbh->disconnect();
     return (undef,-3);
  }
  elsif ($nrecords > 1) {
     print STDOUT '*** '.$script.': multiple records for ID='.$id
                        .' in table='.$mysqlDb
                        .' in DB='.$mysqlTbl
                        .' at mySQL server '.$mysqlHost."\n";
     $sth->finish();
     $dbh->disconnect();
     return (undef,-4);
  }
  $hashref = $sth->fetchrow_hashref();
  if (! defined $hashref->{'Value'}) {
     print STDOUT '*** '.$script.': No value for ID='.$id
                        .' in table='.$mysqlDb
                        .' in DB='.$mysqlTbl
                        .' at mySQL server '.$mysqlHost."\n";
     $sth->finish();
     $dbh->disconnect();
     return (undef,-5);
  }
  $serial = $hashref->{'Value'};
  $sth->finish();
  $dbh->disconnect();
  return ($serial,0);
}

#=================================================================

# sub getCCDname() {
sub getCCDname() {			#returns ($name,$status)

### This routine returns the name of beamline area detector (e.g. MARCCD or Pilatus)

  my ($script, $status, $serial, @marccdlist, @pilatuslist, @eigerlist);

  $script   = 'getCCDname';

  @marccdlist  = qw(015 013 0996);
  @pilatuslist = qw(2-0106 60-0122);
  @eigerlist   = qw(E-32-0104 DP-P073-04);

  ($serial, $status) = &getCCDserial();
  if ($status) {return ('unknown',$status);}

  if    (grep(/^$serial$/, @marccdlist))  {return ('marccd',$status);}
  elsif (grep(/^$serial$/, @pilatuslist)) {return ('pilatus',$status);}
  elsif (grep(/^$serial$/, @eigerlist))   {return ('eiger',$status);}
  else                                    {return ('unknown',-1);}
}

#=================================================================

# sub getCCDfpehost() {
sub getCCDfpehost() {			#returns ($fpehost,$status)

### This routine returns the name of beamline FPE host (e.g. bl1ppu, bl2epu, or bl3mar)

  our ($beamline);
  my ($script, $status, $ccdName, $pv, $fpehost);

  $script = 'getCCDfpehost';

  ($ccdName,$status) = &getCCDname();
  if ($status) {return ('unknown',$status);}
  
  if ($ccdName eq 'pilatus' || $ccdName eq 'eiger' || $ccdName eq 'marccd') {
     ### Here we need any PV hosted by FPE and common for both detectors:
     $pv = $beamline.':4:HOSTNAME';
     $fpehost = `cainfo -w 1 $pv | grep "^    Host:" 2>/dev/null`;
     $fpehost =~ s/^\s+Host:\s+//;
     $fpehost =~ s/:\d+\n*$//;
     if ($fpehost !~ /\.gmca\.aps\.anl\.gov$/) {return ('unknown',1);}
     return ($fpehost,0);
  } else {
     return ('unknown',1);
  }
}
