#!/usr/bin/perl -W
#--------------------------------------------------------------
# These are Joerger scaler related subs for beamline controls Perl scripts
#
#          NEW version with diiferent function arguments
#                    developed for GM/CA CAT
#
#     		     Author: Sergey Stepanov
#
# Version-2.0:  2004/07/28 using Pezca
# Version-2.1:  2014/03/28 switched most arrays passes by value to passing by pointer
#--------------------------------------------------------------
# List of functions:
# ^^^^^^^^^^^^^^^^^
# 0.       $nch = &JoergerNCH ($scaler);
# 1.  $cnt_time = &getJoergerTime ($scaler,['autocount']);
# 2.              &setJoergerTime($scaler, $cnt_time, ['autocount']);
# 3.              &setJoerger ($scaler, $cnt_time, [$struck]);
# 4.              &startJoerger ($scaler);
# 5.              &waitJoerger ($scaler,[$cnt_time]);
# 6. @Intensity = &runJoerger ($scaler, $cnt_time, $ninputs, @inputs);  -+ Normalized on time, but
# 7. @Intensity = &readJoerger ($scaler, $cnt_time, $ninputs, @inputs); -+ background not corrected.
# 8. @Intensity = &runJoergerRef ($scaler, $cnt_time, \@inputs);        -+ Normalized on time, but
# 9. @Intensity = &readJoergerRef ($scaler, $cnt_time, \@inputs);       -+ background not corrected.
# 10.    $yesno = &getJoergerAuto ($scaler);
# 11.             &Joerger_autoStart ($scaler);
# 12.             &Joerger_autoStop ($scaler);
# 13.             &printJoergerInputs ($LUN,$scaler, $ninputs, @inputs);
# 14.  @Offsets = &getJoergerOffsets ($file, $scaler);		#or: @Offsets = &getJoergerOffsets (\@Offsets, $scaler);
# 15.             &printJoergerOffsets ($LUN, $file, $scaler, \@Offsets, $ninputs, \@inputs);
# 16.             &printJoergerData ($LUN, $scaler, \@Offsets, $ninputs, \@inputs, \@Intensity);
# 17.   $status = &subtractJoergerOffsets(\@Intensity,\@inputs,\@Offsets);
#--------------------------------------------------------------
# use Pezca;

# use warnings::unused;
  use strict;
# no strict 'vars';
# no strict 'subs';
  no strict 'refs';

#=================================================================

# sub JoergerNCH ($scaler) {
sub JoergerNCH ($) {

  my ($script, $scaler, $status, $nch);
  $script = 'JoergerNCH';
  $scaler = $_[0];

# Determine the number of inputs:
  ($status,$nch) = Pezca::GetDouble($scaler.'.NCH');
  if ($status) {
     &report_exit($script, $status, $scaler.'.NCH');
     $nch = 0;
  }
  return $nch;
}

#=================================================================

# sub getJoergerTime ($scaler,['autocount']) {
sub getJoergerTime ($;$) {

  my ($script, $scaler, $status, $cnt_time, $auto, $ext);
  $script = 'getJoergerTime';
  $scaler = shift(@_);
  $auto   = shift(@_);	if (!defined $auto) {$auto='';}

  if ($auto =~ /^auto/i) {$ext = '.TP1';}		#auto count time
  else                   {$ext = '.TP';}		#normal count time

# Read the scaler count time:
  ($status,$cnt_time) = Pezca::GetDouble($scaler.$ext);
  if ($status) {
     &report_exit($script, $status, $scaler.$ext);
     $cnt_time = 0;
  }
  return $cnt_time;
}

#=================================================================

# sub setJoergerTime ($scaler, $cnt_time, ['autocount']) {
sub setJoergerTime ($$;$) {

  my ($script, $scaler, $cnt_time, $auto, $status, $ext);
  $script   = 'setJoergerTime';
  $scaler   = shift(@_);
  $cnt_time = shift(@_);
  $auto   = shift(@_);	if (!defined $auto) {$auto='';}

  if ($auto =~ /^auto/i) {$ext = '.TP1';}		#auto count time
  else                   {$ext = '.TP';}		#normal count time

# Set the scaler count time:
  $status = Pezca::PutDoubleOldCa($scaler.$ext,$cnt_time);
  &report_exit($script, $status, $scaler.$ext);
}

#=================================================================

# sub setJoerger ($scaler, $cnt_time, [$struck]) {
sub setJoerger ($$;$) {

  my ($script, $scaler, $cnt_time, $struck, $mode, $status);
  $script   = 'setJoerger';
  $scaler   = shift(@_);
  $cnt_time = shift(@_);
  $struck   = shift(@_);	#if Struck is specified (when Joerger is actuall Struck in this mode), then abort it

  if (defined $struck) {
     ($status, $mode) = Pezca::GetString($struck.':AcquireMode');
     if ($status) {
        ### On older versions of Struck driver the AcquireMode PV
        ### did not exist. Then we presume "Scaler".
        &report_only ($script, $status, $struck.':AcquireMode');
	$mode = 'Scaler';
     }
     if ($mode eq 'MCS') {
        $status = Pezca::PutDoubleOldCa($struck.':StopAll',1);
        &report_only ($script, $status, $struck.':StopAll');
     }
  }

# caput nnID:scaler1.CONT  OneShot=0 | AutoCount=1 (stop autocount)
  $status = Pezca::PutDoubleOldCa($scaler.'.CONT',0);
  &report_exit($script, $status, $scaler.'.CONT');

# caput nnID:scaler1.CNT  Done=0 | Count=1   (stop scaler)
  $status = Pezca::PutDoubleOldCa($scaler.'.CNT',0);
  &report_exit($script, $status, $scaler.'.CNT');

# caput nnID:scaler1.TP count_time
  $status = Pezca::PutDoubleOldCa($scaler.'.TP',$cnt_time);
  &report_exit($script, $status, $scaler.'.TP');

### The time interval over which the scaler bank is to count during a
### background-counting operation. (If any of inputs [2..n] are also
###  in preset mode, counting may stop before the time preset has been
###  reached.)
# caput nnID:scaler1.TP1 count_time
  $status = Pezca::PutDoubleOldCa($scaler.'.TP1',$cnt_time);
  &report_exit($script, $status, $scaler.'.TP1');
}

#=================================================================

# sub startJoerger ($scaler) {
sub startJoerger ($) {

  my ($script, $scaler, $status);
  $script = 'startJoerger';
  $scaler = shift(@_);

# caputOldCa nnID:scaler1.CNT  start=1 | stop=0
  $status = Pezca::PutDoubleOldCa($scaler.'.CNT',1);
  &report_exit($script, $status, $scaler.'.CNT');
}

#=================================================================

# sub waitJoerger ($scaler,[$cnt_time]) {
sub waitJoerger ($;$) {

  my ($script, $scaler, $cnt_time, $scalerCNT, $status);
  my ($max_wait, $wait_start, $wait_step, $start_time, $duration);
  $script = 'waitJoerger';
  $scaler   = shift(@_);
  $cnt_time = shift(@_);				#optional argument

  $max_wait   = 5;					#max wait after expected finish
  $wait_start = 0.20;					#wait start = 100ms
  $wait_step  = 0.05;					#wait step = 50ms

  if (!defined $cnt_time) {$cnt_time = 0} 	#optional parameter
  if ($cnt_time <= 0) {
### Read scaler cnt_time:
     ($status, $cnt_time) = Pezca::GetDouble($scaler.'.TP');
     if ($status) {
        &report_exit($script, $status, $scaler.'.TP');
	$cnt_time = 10;					#some large value for long waiting
     }
  }
  if ($wait_start < 0.5*$cnt_time) {$wait_start = 0.5*$cnt_time;}

  $max_wait += $cnt_time;

### Wait at the beginning to make sure that scaler started:
### (Sleep has 1s resolution; therefore we use 'select')
  $start_time = time();
  select (undef, undef, undef, $wait_start);		#sleep($wait_start);

### wait to finish:
  $status = &addCaMonitor($scaler.'.CNT');
  do {
     select (undef, undef, undef, $wait_step);		#sleep($wait_step);
     $duration = time() - $start_time;
     ($status,$scalerCNT) = Pezca::GetDouble($scaler.'.CNT');
  } while ($scalerCNT == 1 && $duration < $max_wait);

  if ($scalerCNT == 1) {
     printf STDOUT '*** '.$script.': timeout waiting for '.$scaler.' to finish!'."\n"
                  .'Counting time=%.2fs, waited=%.2fs'."\n\n",$cnt_time, $duration;
  }
}

#=================================================================

# sub runJoerger ($scaler, $cnt_time, $ninputs, @inputs) {
sub runJoerger ($$$@) {

  my ($scaler, $cnt_time, $i, $ninputs, @inputs, @Intensity);
  $scaler   = shift(@_);
  $cnt_time = shift(@_);
  $ninputs  = shift(@_);
  for ($i=0; $i<$ninputs; $i++) {$inputs[$i] = shift(@_);}

### Set Joerget count time
  &setJoergerTime ($scaler, $cnt_time);

### Start Joerger:
  &startJoerger ($scaler);

### Wait for Joerger to finish:
  &waitJoerger ($scaler);

### read scaler data
  @Intensity = &readJoerger ($scaler, $cnt_time, $ninputs, @inputs);
  return @Intensity;
}

#=================================================================

# sub readJoerger ($scaler, $cnt_time, $ninputs, @inputs) {
sub readJoerger ($$$@) {

  my ($script, $scaler, $cnt_time, $i, $input, $ninputs, $msg);
  my (@Intensity, $joerger_data, $maxJoerger, $status, $current_time);
  $script   = 'readJoerger';
  $scaler   = shift(@_);
  $cnt_time = shift(@_);
  $ninputs  = shift(@_);

### Determine the number of inputs:
  $maxJoerger = &JoergerNCH($scaler);

  if ($cnt_time <= 0) {
     ($status,$cnt_time) = Pezca::GetDouble($scaler.'.TP');
     &report_exit($script, $status, $scaler.'.TP');
     if ($cnt_time <= 0) {
        $current_time = scalar localtime;
        $msg = '*** '.$script.': '.$current_time."\n"
             . '*** '.$script.': Zero or negative '.$scaler.' counting time.'."\n";
        &press_key_exit($msg);
	$cnt_time = 1;
     }
  }
### read scaler data
  @Intensity = ();
  for ($i=0; $i<$ninputs; $i++) {
     $input = shift(@_);
     if ($input < 1 || $input > $maxJoerger) {
        $current_time = scalar localtime;
        $msg = '*** '.$script.': '.$current_time."\n"
             . '*** '.$script.': requested input='.$input.' not in the [1-'.$maxJoerger.'] range.'."\n";
        &press_key_exit($msg);
     }
     ($status,$joerger_data) = Pezca::GetDouble($scaler.'.S'.$input);
     if ($status) {
        &report_exit($script, $status, $scaler.'.S'.$input);
	$joerger_data = 0;
     }
     $Intensity[$i] = $joerger_data / $cnt_time;
  }
  return @Intensity;
}

#=================================================================

# sub runJoergerRef ($scaler, $cnt_time, \@inputs) {
sub runJoergerRef ($$\@) {

  my ($scaler, $cnt_time, $inputsRef, @Intensity);
  $scaler    = shift(@_);
  $cnt_time  = shift(@_);
  $inputsRef = shift(@_);

### Start Joerger:
  &startJoerger ($scaler);

### Wait for Joerger to finish:
  &waitJoerger ($scaler);

### read scaler data
  @Intensity = &readJoergerRef ($scaler, $cnt_time, $inputsRef);
  return @Intensity;
}

#=================================================================

# sub readJoergerRef ($scaler, $cnt_time, \@inputs) {
sub readJoergerRef ($$\@) {

  my ($script, $scaler, $cnt_time, $inputsRef, $i, $input, $ninputs, $msg);
  my (@Intensity, $joerger_data, $maxJoerger, $status, $current_time);
  $script    = 'readJoerger';
  $scaler    = shift(@_);
  $cnt_time  = shift(@_);
  $inputsRef = shift(@_);

### Determine the number of inputs:
  $maxJoerger = &JoergerNCH($scaler);
  $ninputs = @$inputsRef;

  if ($cnt_time <= 0) {
     ($status,$cnt_time) = Pezca::GetDouble($scaler.'.TP');
     if ($status) {
        &report_exit($script, $status, $scaler.'.TP');
	$cnt_time = 0;
     }
     if ($cnt_time <= 0) {
        $current_time = scalar localtime;
        $msg = '*** '.$script.': '.$current_time."\n"
             . '*** '.$script.': Zero or negative '.$scaler.' counting time.'."\n";
        &press_key_exit($msg);
	$cnt_time = 1;
     }
  }
### read scaler data
  @Intensity = ();
  for ($i=0; $i<$ninputs; $i++) {
     $input = @$inputsRef[$i];
     if ($input < 1 || $input > $maxJoerger) {
        $current_time = scalar localtime;
        $msg = '*** '.$script.': '.$current_time."\n"
             . '*** '.$script.': requested input='.$input.' not in the [1-'.$maxJoerger.'] range.'."\n";
        &press_key_exit($msg);
     }
     ($status,$joerger_data) = Pezca::GetDouble($scaler.'.S'.$input);
     if ($status) {
        &report_exit($script, $status, $scaler.'.S'.$input);
	$joerger_data = 0;
     }
     $Intensity[$i] = $joerger_data / $cnt_time;
  }
  return @Intensity;
}

#=================================================================

# sub getJoergerAuto ($scaler) {		#return $yesno=1|0
sub getJoergerAuto ($) {

  my ($script, $scaler, $status, $yesno);
  $script = 'getJoergerAuto';
  $scaler = $_[0];
# caput nnID:scaler1.CONT  OneShot=0 | AutoCount=1 (start autocount)
  ($status,$yesno) = Pezca::GetDouble($scaler.'.CONT');
  &report_exit($script, $status, $scaler.'.CONT');
  return $yesno;
}

#=================================================================

# sub Joerger_autoStart ($scaler) {
sub Joerger_autoStart ($) {

  my ($script, $scaler, $status);
  $script = 'Joerger_autoStart';
  $scaler = $_[0];
# caput nnID:scaler1.CONT  OneShot=0 | AutoCount=1 (start autocount)
  $status = Pezca::PutDoubleOldCa($scaler.'.CONT',1);
  &report_exit($script, $status, $scaler.'.CONT');
}

#=================================================================

# sub Joerger_autoStop ($scaler) {
sub Joerger_autoStop ($) {

  my ($script, $scaler, $status);
  $script = 'Joerger_autoStop';
  $scaler = $_[0];
# caput nnID:scaler1.CONT  OneShot=0 | AutoCount=1 (stop autocount)
  $status = Pezca::PutDoubleOldCa($scaler.'.CONT',0);
  &report_exit($script, $status, $scaler.'.CONT');
}

#=================================================================

# sub  printJoergerInputs ($LUN, $scaler, $ninputs, @inputs) {
sub  printJoergerInputs ($$$@) {

  my ($LUN, $scaler, $ninputs, $i, $input, $s);
  $LUN     = shift(@_);
  $scaler  = shift(@_);
  $ninputs = shift(@_);
  if ($ninputs > 1) {$s='s';} else {$s='';}
  printf $LUN '# Joerger scaler [%s] input%s: ',$scaler,$s;
  for ($i=0; $i<$ninputs; $i++) {
     $input = shift(@_);
     print $LUN ' '.$input;
  }
  print $LUN "\n";
}

#=================================================================

# sub getJoergerOffsets ($file, $scaler) {
# sub getJoergerOffsets (\$offsets, $scaler) {		#another possibility when offsets are already known
sub getJoergerOffsets ($$) {

  my ($script, $file, $offref, $scaler, @Offsets, $maxJoerger, $msg);
  my ($age_hours, $nOffsets, $handler, $current_time, $i);
  $script = 'getJoergerOffsets';
  $file   = $_[0];		if (!defined $file) {$file='';}
  $scaler = $_[1];
  $maxJoerger = &JoergerNCH($scaler);
  @Offsets = ();
  for ($i=0; $i<$maxJoerger; $i++) {$Offsets[$i] = 0.0;}	#zero offsets

  if (length($file) == 0) {					#filename is NOT specified:
### File name is NOT specified:
     print STDOUT '!!! '.$script.': Offsets file is not specified. Offsets are set to 0.'."\n";
  }
  elsif ($file =~ /^ARRAY\(0x[\da-fA-F]{1,8}\)$/) {		#this is not a file, but a reference to the offsets array
     $offref = $file;						#i.e. something like: "ARRAY(0x182bdac)"
     @Offsets = @$offref;
  }
  else {                     					#if file name is specified
     if (-f $file) {          					#if file exists
        $handler = open (BKG, '< '.$file);
        if (!defined $handler) {
           $current_time = scalar localtime;
           $msg = '*** '.$script.': '.$current_time."\n"
                . '*** '.$script.': cannot open scaler offsets file='.$file."\n";
           &press_key_exit($msg);
        }
        chomp (@Offsets = <BKG>);
        close(BKG);
        $age_hours = 24 * (-M $file);			#Returns file age in days
        if ($age_hours > 2.) {
           printf STDOUT '!!! '.$script.': offsets file is %.1f hour(s) old.'."\n", $age_hours;
        }
### Verify the number of inputs:
        $nOffsets = @Offsets;
        if ($maxJoerger != $nOffsets) {
           $current_time = scalar localtime;
           $msg = '*** '.$script.': '.$current_time."\n"
                . '*** '.$script.': Offsets number='.$nOffsets.' does not match scaler inputs='.$maxJoerger."\n";
           &press_key_exit($msg);
        }
     } else {                   				#filename is specified, but file does NOT exist
        print STDOUT '!!! '.$script.':'."\n";
        print STDOUT '        Offsets file ['.$file.'] is not found.'."\n";
        print STDOUT '        Offsets are set to 0.'."\n";
     }
  }
  return @Offsets;
}

#=================================================================

# sub printJoergerOffsets ($LUN, $file, $scaler, \@Offsets, $ninputs, \@inputs) {
sub printJoergerOffsets ($$$\@$\@) {

  my ($script, $LUN, $file, $scaler, $OffsetsRef, @Offsets, $ninputs, $msg);
  my ($inputsRef, $input, $maxJoerger, $i, $j, $offset, $current_time);
  $script     = 'printJoergerOffsets';
  $LUN        = shift(@_);
  $file       = shift(@_);         if (!defined $file) {$file='';}
  $scaler     = shift(@_);
  $OffsetsRef = shift(@_);
  $ninputs    = shift(@_);
  $inputsRef  = shift(@_);

### Determine the number of inputs:
  $maxJoerger = &JoergerNCH($scaler);
  for ($i=0; $i<$maxJoerger; $i++) {
     if (defined @$OffsetsRef[$i]) {$Offsets[$i] = @$OffsetsRef[$i];}
     else                          {$Offsets[$i] = 0;}
  }

  if (length($file) != 0) {			#if filename is specified
     print $LUN '# Offsets from '.$file.': ';
  } else {
     print $LUN '# Offsets (no BKG file): ';
  }
  for ($i=0; $i<$ninputs; $i++) {
     if (defined @$inputsRef[$i]) {
        $input = @$inputsRef[$i];
        if ($input >= 1 && $input <= $maxJoerger) {
           $j = $input-1;
           $offset = $Offsets[$j];
           $offset =~ s/\n//g;
           print $LUN ' '.$offset;
        } else {
           $current_time = scalar localtime;
           $msg = '*** '.$script.': '.$current_time."\n"
                . '*** '.$script.': incorrect input='.$input."\n";
           &press_key_exit($msg);
        }
     } else {
        $msg = '*** '.$script.': '.$current_time."\n"
             . '*** '.$script.': not enough inputs passed. Expected='.$ninputs.', received='.$i."\n";
        &press_key_exit($msg);
     }
  }
  print $LUN "\n";
}

#=================================================================

# sub  printJoergerData ($LUN, $scaler, \@Offsets, $ninputs, \@inputs, \@Intensity) {
sub  printJoergerData ($$\@$\@\@) {

  my ($script, $LUN, $scaler, $OffsetsRef, @Offsets, $msg);
  my ($ninputs, $inputsRef, $input, $IntensityRef, $Intensity);
  my ($maxJoerger, @Off, $i, $j, $current_time);
  $script       = 'printJoergerData';
  $LUN          = shift(@_);
  $scaler       = shift(@_);
  $OffsetsRef   = shift(@_);
  $ninputs      = shift(@_);
  $inputsRef    = shift(@_);
  $IntensityRef = shift(@_);

  $maxJoerger = &JoergerNCH($scaler);
  for ($i=0; $i<$maxJoerger; $i++) {
     if (defined @$OffsetsRef[$i]) {$Offsets[$i] = @$OffsetsRef[$i];}
     else                          {$Offsets[$i] = 0;}
  }
  for ($i=0; $i<$ninputs; $i++) {
     if (defined @$inputsRef[$i]) {
        $input = @$inputsRef[$i];
        if ($input >= 1 && $input<=$maxJoerger) {
           $j     = $input - 1;
           $Off[$i] = $Offsets[$j];
           $Off[$i] =~ s/\n//g;
        } else {
           $current_time = scalar localtime;
           $msg = '*** '.$script.': '.$current_time."\n"
                . '*** '.$script.': incorrect input='.$input."\n";
           &press_key_exit($msg);
        }
     } else {
        $msg = '*** '.$script.': '.$current_time."\n"
             . '*** '.$script.': not enough inputs passed. Expected='.$ninputs.', received='.$i."\n";
        &press_key_exit($msg);
     }
  }
  for ($i=0; $i<$ninputs; $i++) {
     if (defined @$IntensityRef[$i]) {
        $Intensity = @$IntensityRef[$i];
        $Intensity = $Intensity - $Off[$i];
        print $LUN ', '.$Intensity;
     } else {
        print $LUN ', undef';
     }
  }
}

#=================================================================

# sub subtractJoergerOffsets(\@Intensity,\@inputs,\@Offsets);
sub subtractJoergerOffsets (\@\@\@) {

  my ($script, $IntensityRef, $inputsRef, $OffsetsRef);
  my ($ninputs, $nIntensity, $nOffsets, $i, $offsetNr, $nNeg);
  $script = 'subtractJoergerOffsets';
  $IntensityRef = shift(@_);
  $inputsRef    = shift(@_);
  $OffsetsRef   = shift(@_);

no strict 'refs';
  $ninputs    = @$inputsRef;
  $nIntensity = @$IntensityRef;
  $nOffsets   = @$OffsetsRef;
  if ($ninputs < $nIntensity) {
     print STDOUT '*** '.$script.': not enough inputs: '.$ninputs.' < '.$nIntensity."\n\n";
     return 1;
  }
  if ($ninputs != $nIntensity) {
     print STDOUT '!!! '.$script.': more inputs than intensity: '.$ninputs.' > '.$nIntensity."\n\n";
     print STDOUT '!!! '.$script.': Not everything normalized?'."\n";
  }
  $nNeg = 0;
  for ($i=0; $i<$nIntensity; $i++) {
     $offsetNr = $$inputsRef[$i];
     if ($offsetNr < 1 || $offsetNr > $nOffsets) {
        print STDOUT '*** '.$script.': offset Nr '.$offsetNr.' not in proper range.'."\n\n";
        return 1;
     }
     $$IntensityRef[$i] = $$IntensityRef[$i] - $$OffsetsRef[$offsetNr-1];
     if ($$IntensityRef[$i] < 0.0) {$$IntensityRef[$i] = 0.0; $nNeg++;}
  }
  if ($nNeg > 0) {
     print STDOUT '!!! '.$script.': '.$nNeg.' inputs had intensity below offsets.'."\n";
     print STDOUT '!!! '.$script.': these intensities were zeroed.'."\n";
     print STDOUT '!!! '.$script.': You may need to re-measure Joerger offsets.'."\n\n";
  }
use strict 'refs';
  return 0;
}
