#!/usr/bin/perl -W
#--------------------------------------------------------------
# On-the-fly scans:
# ^^^^^^^^^^^^^^^^
# 01. ($status,$actPts) = &HDflyScan_PLC($assy,$axis,$begin,$end,$npts,$scanTime,
#                                        $medm,$fileBkg,\@inputs,[$scaler]);
#                                                               - Also returns GLOBALS:
#                                                                 @x1HD,
#                                                                 @timeHD,
#                                                                 @y1HD,...@y{$ninputs}HD,
#                                                                 @StruckOffsets
# 02.    ($status,$msg) = &HDflyScan_PLC_setup($assy,$axis,$begin,$end,$npts,$scanTime,[$scaler]);
#
# 03.    ($status,$msg) = &HDflyScan_PLC_wait($assy,$axis,$dwell);
#
# 04. ($status,$actPts) = &HDflyScan_PLC_read ($axis,\@inputs,\@Offsets,\@data,[$finalORfly]);
#                                                               -- fills \@data=(@positions,@timeline,@inputs)
#
#--------------------------------------------------------------
# use Pezca;
# use Math::Trig;               # for energy scans
# use Time::HiRes qw(usleep gettimeofday tv_interval);

# use warnings::unused;
  use strict;
# no strict 'vars';
# no strict 'subs';
# no strict 'refs';

#=================================================================

# sub HDflyScan_PLC ($assy,$axis,$begin,$end,$npts,$scanTime,$medm,$fileBkg,\@inputs,[$scaler]) {
  sub HDflyScan_PLC ($$$$$$$$\@;$) {

### Returns implicitly: ($status,$actPts)
### Returns GLOBAL:      @x1HD, @timeHD, @y1HD, ... @y{$ninputs}HD, @StruckOffsets

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext, @StruckOffsets);
  our ($HDSCAN_FLAGS);		#reference to a hash with various flags for HDSCAN
  our (@x1HD, @timeHD);		#GLOBAL RESULTs: positions and timeline
  my ($script, $assy, $axis, $begin, $end, $npts, $scanTime);
  my ($medm, $fileBkg, $ninputs, $inputsRef);
  my ($struck, $dwell, $actPts, $status, $message, @data, $name);
  my ($i, $j, $scaler);

  $script    = 'HDflyScan_PLC';
  $assy      = shift(@_);	# assembly to be scanned
  $axis      = shift(@_);	# particular axis in the assembly to be scanned
  $begin     = shift(@_);	# start scan position
  $end       = shift(@_);	# end scan position
  $npts      = shift(@_);	# number of points on fast scan
  $scanTime  = shift(@_);	# scan time in seconds
  $medm      = shift(@_);	# 1=open MEDM, 0=not
  $fileBkg   = shift(@_);	# struck background file ('auto' for automatic, '' for none)
  $inputsRef = shift(@_);	# Struck scaler inputs
  $scaler    = shift(@_);	# Struck as Joerger name

  if (!defined $HDSCAN_FLAGS->{'has run'})              {$HDSCAN_FLAGS->{'has run'}            = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter close'})   {$HDSCAN_FLAGS->{'skip shutter close'} = 0;}

# print STDOUT 'Enterting '.$script."\n";
  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }
  if (defined $shutter) {
     if ($HDSCAN_FLAGS->{'skip shutter close'} != 1) {
#       print STDOUT $script.': Closing shutter '.$shutter."\n";
        &closeShutter ($shutter,$shutter_readback);
     }
  }

  if (!defined $fileBkg)       {$fileBkg = '';}
  if ($fileBkg =~ /^auto$/i)   {$fileBkg = 'struck_Bkg.dat';}
### Read the scaler background offsets:
  if (!defined $StruckOffsets[0]) {			    #skip reading offsets if they were read in main
     if (length($fileBkg) > 0) {			    #if file name is specified
        print STDOUT $script.': Reading Struck scaler '.$struck.' offsets from '.$fileBkg."\n";
     }
     @StruckOffsets = &getStruckOffsets($fileBkg,$struck);  #we make it GLOBAL to access from main
  }

  $ninputs = @$inputsRef;
  if ($medm && $HDSCAN_FLAGS->{'has run'} == 0) {
     for ($i=0; $i<$ninputs; $i++) {&openStruckMedm($struck, $struck_ext, @$inputsRef[$i]);}
  }

  ($status,$message) = &HDflyScan_PLC_setup($assy,$axis,$begin,$end,$npts,$scanTime,$scaler);
  if ($status) {
     print STDOUT "\n",$message,"\n";
     return $status, 0;
  }

### Wait for scan to finish:
  ($status,$message) = &HDflyScan_PLC_wait($assy,$axis,$dwell);
  if ($status) {
     print STDOUT "\n",$message,"\n";
     return $status, 0;
  }
# &press_key_continue();

### Read scan data (fill \@data=(@positions,@timeline,@inputs)):
  ($status,$actPts) = &HDflyScan_PLC_read ($axis,$inputsRef,\@StruckOffsets,\@data,'final');
  if ($status) {
     print STDOUT "\n".$script.': error='.$status.' from HDflyScan_PLC_read.'."\n";
     return $status, 0;
  }

  @x1HD = ();						#GLOBAL RESULT: positions
  @timeHD = ();						#GLOBAL RESULT: timeline
  for ($j=0; $j<$actPts; $j++) {
     $x1HD[$j]   = $data[0][$j];
     $timeHD[$j] = $data[1][$j];
  }
  no strict 'refs';
  for ($i=1; $i<=$ninputs; $i++) {
     $name  = 'y'.$i.'HD';
     @$name = ();			#GLOBAL RESULT: data for input-$i
     for ($j=0; $j<$actPts; $j++) {
        @$name[$j] = $data[$i+1][$j];
     }
  }
  use strict 'refs';

# print STDOUT $script.': Scan data retrieved!'."\n";
  return $status,$actPts;

}	##HDflyScan


#=================================================================

# ($status,$message) = &HDflyScan_PLC_setup($assy,$axis,$begin,$end,$npts,$scanTime,[$scaler]);
  sub HDflyScan_PLC_setup($$$$$$;$) {

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext);
  our ($IGNORE_ERRORS, $report);
  our ($HDSCAN_FLAGS);		#reference to a hash with various flags for HDSCAN
  my ($script, $assy, $axis, $begin, $end, $npts, $dwell_, $delayTime);
  my ($struck, $int_ext, $iserased, $status, $message, $need2move);
  my ($scanRange, $l_speed, $h_speed, $beginAct);
  my ($scanTime, $ignore_errors, $estimPts, $maxPts);
  my ($cts2egu, $mtr, $encAddr, $dwell, $step, $step_encoder);
  my ($step_encoder_abs, $step_encoder_sign, $scaler);
  my ($DWELLTIMEMIN, $DWELLTIMEMAX, $STRUCK_EXTRA_TIME);
  my ($plc_enco_addr, $plc_enco_incr, $plc_incr_sign, $plc_ini_start);

  $script   = 'HDflyScan_PLC_setup';
  $assy     = shift(@_);	#assembly to be scanned
  $axis     = shift(@_);	#particular axis in the assembly to be scanned
  $begin    = shift(@_);	#start scan position
  $end      = shift(@_);	#end scan position
  $npts     = shift(@_);	#number of points on fast scan
  $scanTime = shift(@_);	#desired scantime in seconds
  $scaler   = shift(@_);	# Struck as Joerger name

# print STDOUT 'Enterting '.$script."\n";
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if (!defined $HDSCAN_FLAGS->{'has run'})              {$HDSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'has read mtrList'})     {$HDSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter close'})   {$HDSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter open'})    {$HDSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip calc speed'})      {$HDSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set speed'})       {$HDSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set slew'})        {$HDSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore speed'})   {$HDSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore mprg'})    {$HDSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip move to start'})   {$HDSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'return to start'})      {$HDSCAN_FLAGS->{'return to start'}      = 0;}

  $plc_enco_addr = 'm3605';			#encoder address
  $plc_enco_incr = 'm3604';                    #abs of encoder increment
  $plc_incr_sign = 'm3607';			#sign of encoder increment (0=positive, 1=negative)
  $plc_ini_start = 'm3606';			#set $plc_ini_start=1 to initialize & start PLC

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }
  if (defined $shutter) {
     if ($HDSCAN_FLAGS->{'skip shutter close'} == 0) {
#       print STDOUT $script.': Closing shutter='.$shutter."\n";
        &closeShutter ($shutter,$shutter_readback);
     }
  }

  ($DWELLTIMEMIN,$DWELLTIMEMAX) = &getStruckDwellMinMax($struck);
  $DWELLTIMEMIN = 1.0E-3;			#sec; PMAC PLC task restriction: 1kHz PLC rate
  $STRUCK_EXTRA_TIME = 3;			#time for Struck to count longer than motion
  $int_ext = 1;					#0=internal advance, 1=external

# print STDOUT $script.': Stopping motor='.$axis."\n";
  &abortAssy($assy);

  if ($npts < 3) {
     $message = '*** '.$script.': incorrect scan npts='.$npts;
     print STDOUT $message,"\n";
     return 1, $message;
  }

### Get $cts2egu, $mtr, $encAddr :
  if ($HDSCAN_FLAGS->{'has run'} == 0 && $HDSCAN_FLAGS->{'has read mtrList'} == 0) {
     ($cts2egu, $mtr, $encAddr) = &searchMotorList2($assy,$axis);
     if (!defined $cts2egu) {
        $message = '*** '.$script.': No HD-synchronized data in motorList for assy='.$assy.', axis='.$axis;
        return 1, $message;
     }
     $mtr                        = $beamline.':'.$mtr;
     $HDSCAN_FLAGS->{'cts2egu'}  = $cts2egu;
     $HDSCAN_FLAGS->{'motor'}    = $mtr;
     $HDSCAN_FLAGS->{'encAddr'}  = $encAddr;
  } else {
     $cts2egu  = $HDSCAN_FLAGS->{'cts2egu'};
     $mtr      = $HDSCAN_FLAGS->{'motor'};
     $encAddr  = $HDSCAN_FLAGS->{'encAddr'};
  }

# $HDSCAN_FLAGS->{'assy'}  = $assy;
# $HDSCAN_FLAGS->{'axis'}  = $axis;
  $HDSCAN_FLAGS->{'begin'} = $begin;
# $HDSCAN_FLAGS->{'end'}   = $end;
  $HDSCAN_FLAGS->{'step'}  = ($end - $begin) / $npts;

  $step_encoder = sprintf '%.2f', (32 * $HDSCAN_FLAGS->{'step'} / $cts2egu);
  $step_encoder_abs = abs($step_encoder);
  if ($step_encoder >=0) {$step_encoder_sign = 0;}
  else                   {$step_encoder_sign = 1;}
  print STDOUT $begin, ',  ', $end, ',  ', $npts, ',  ', $HDSCAN_FLAGS->{'step'},
               ',  ', $cts2egu, ',  ', $step_encoder, "\n\n\n\n";


#-----------------------------------------------------------------
  $dwell = $scanTime / $npts;
  if    ($dwell < $DWELLTIMEMIN) {
     printf STDOUT '!!! '.$script.': timePPT=%.3fs is below Struck limit=%.3fs'."\n", $dwell, $DWELLTIMEMIN;
     $dwell = $DWELLTIMEMIN;
  }
  elsif ($dwell > $DWELLTIMEMAX) {
     printf STDOUT '!!! '.$script.': timePPT=%.3fs exceeds Struck limit=%.3fs'."\n", $dwell, $DWELLTIMEMAX;
     $dwell = $DWELLTIMEMAX;
  }
  $scanTime = $npts * $dwell;

#-----------------------------------------------------------------
  if ($HDSCAN_FLAGS->{'has run'} == 0 || $HDSCAN_FLAGS->{'skip calc speed'} == 0) {
### Calculate scan speed for the motor:
#    print STDOUT $script.': Calculating slew-mode speed for axis='.$axis."\n";
     $ignore_errors = $IGNORE_ERRORS;
     $IGNORE_ERRORS = 1;
     $scanRange = abs ($end - $begin);
     ($l_speed, $h_speed) = &calcAxisSpeed ($assy, $axis, $scanRange, $scanTime);
     $IGNORE_ERRORS = $ignore_errors;
     if ($l_speed > $h_speed) {				#speed was too high
        printf STDOUT '!!! '.$script.': requested '.$axis.' speed=%.3f exceeds max=%.3f '
                            .'. Slowing scan down.'."\n", $l_speed, $h_speed;
        $l_speed  = $h_speed;
        $scanTime = $scanRange / $l_speed;
	$dwell_ = $dwell;
        $dwell = $scanTime / $npts;
        if ($dwell > $DWELLTIMEMAX) {
           $message = sprintf '!!! '.$script.': too high timePPT=%.3fs required with npts='.$npts, $dwell;
	   print STDOUT $message,"\n";
           return 1, $message;
#          $dwell = $DWELLTIMEMAX;
#          $scanTime = $npts * $dwell;
        }
        printf STDOUT '!!! '.$script.': timePPT changed from %.3fs to %.3fs'."\n", $dwell_, $dwell;
     }
     $HDSCAN_FLAGS->{'speed'} = $l_speed;
     $HDSCAN_FLAGS->{'dwell'} = $dwell;
  } else {
#    printf STDOUT '!!! '.$script.': Using '.$axis.' slew-mode speed=%.3f from previous scan'."\n", $HDSCAN_FLAGS->{'speed'};
#    print  STDOUT '!!! '.$script.': (This presumes same range and time)'."\n";
     $l_speed = $HDSCAN_FLAGS->{'speed'};
     $dwell   = $HDSCAN_FLAGS->{'dwell'};
  }

#-----------------------------------------------------------------
### Verify that we do not overflow Struck buffer:
  $maxPts = &getStruckNmax($struck, $struck_ext);
  if ($npts > $maxPts) {
     $message = '*** '.$script.': Required pts='.$npts.' exceeds'."\n".
                'Struck buffer='.$maxPts.'. Please, increase step size.';
     print STDOUT $message,"\n";
     return 1, $message;
  }
  $estimPts = $npts + int($STRUCK_EXTRA_TIME/$dwell + 0.5);
  if ($estimPts > $maxPts) {
     print STDOUT "\n".'!!! '.$script.': Estimated pts=='.$estimPts.' exceeds'."\n".
                    'Struck buffer='.$maxPts.'. Please, increase step size, as'."\n".
		    'otherwice scan may be aborted before motor arrives to the end.'."\n";
  }

#-----------------------------------------------------------------
  if ($HDSCAN_FLAGS->{'has run'} == 0) {
### Set and erase Struck scaler:
#    printf STDOUT $script.': Setting up Struck scaler '.$struck.' (dwell=.2fs)'."\n",$dwell;
     &setStruck ($struck, $struck_ext, $int_ext, 1, $maxPts, $scaler);
#    print STDOUT $script.': Erasing Struck scaler '.$struck."\n";
     &eraseStruck ($struck);
### Set motor:
#    print STDOUT $script.': Setting up motor '.$axis."\n";
     &configAssy ($assy);
  }
#-----------------------------------------------------------------
  $need2move = 0;
  if ($HDSCAN_FLAGS->{'skip move to start'}) {
     $beginAct = &getAxisAct ($axis);
     if (abs($beginAct - $begin) > 0.5*abs($step)) {$need2move=1;}
  }
  if ($HDSCAN_FLAGS->{'skip move to start'} == 0 || $need2move) {
### Move motor to the starting position:
     printf STDOUT $script.': Moving motor '.$axis.' to the starting position=%.3f'."\n",$begin;
### This is to recover speed from previouse unclean exit:
     if ($HDSCAN_FLAGS->{'has run'} == 0) {&restoreAxis ($assy, $axis);}	#restore high-speed & motion prg before start
     $status = &runMotor ($assy, $axis, $begin);
### Restore expected motion time to normal from local cache:
     if ($status) {
        $message = '*** '.$script.': failed moving '.$axis.' to the starting position'."\n";
	print STDOUT $message,"\n";
        return 1, $message;
     }
  }

### Setting slow speed for scanning:
# print STDOUT $script.': Setting up motor='.$axis.' speed to slew (speed='.$l_speed.')'."\n";
  if ($HDSCAN_FLAGS->{'has run'} == 0) {
#    print STDOUT 'Calling sendMtrCmd on mtr='.$mtr.', cmd='.$plc_enco_addr.'->X:'.$encAddr.',24'."\n\n";
     $status = &sendMtrCmd($mtr,$plc_enco_addr.'->X:'.$encAddr.',24','nowait');
     if ($status) {return 1, 'Error setting '.$plc_enco_addr.' (encoder address) for axis='.$axis.', motor='.$mtr;}
     print STDOUT 'Calling sendMtrCmd on mtr='.$mtr.', cmd='.$plc_enco_incr.'='.$step_encoder_abs."\n\n";
     $status = &sendMtrCmd($mtr,$plc_enco_incr.'='.$step_encoder_abs,'nowait');
     if ($status) {return 1, 'Error setting '.$plc_enco_incr.' (abs of encoder increment) for axis='.$axis.', motor='.$mtr;}
     print STDOUT 'Calling sendMtrCmd on mtr='.$mtr.', cmd='.$plc_incr_sign.'='.$step_encoder_sign."\n\n";
     $status = &sendMtrCmd($mtr,$plc_incr_sign.'='.$step_encoder_sign,'nowait');
     if ($status) {return 1, 'Error setting '.$plc_incr_sign.' (sign of encoder increment) for axis='.$axis.', motor='.$mtr;}
     $HDSCAN_FLAGS->{'step_encoder'} = $step_encoder;
     print STDOUT $script.': setting scanning speed & slew motion prg'."\n";
     &setAxisSlewScan ($assy, $axis, $l_speed);			#setMotionPrg+setAxisSpeed
  } else {
     if ($step_encoder ne $HDSCAN_FLAGS->{'step_encoder'}) {
        $status = &sendMtrCmd($mtr,$plc_enco_incr.'='.$step_encoder_abs,'nowait');
        if ($status) {return 1, 'Error setting '.$plc_enco_incr.' (abs of encoder increment) for axis='.$axis.', motor='.$mtr;}
        $status = &sendMtrCmd($mtr,$plc_incr_sign.'='.$step_encoder_sign,'nowait');
        if ($status) {return 1, 'Error setting '.$plc_incr_sign.' (sign of encoder increment) for axis='.$axis.', motor='.$mtr;}
        $HDSCAN_FLAGS->{'step_encoder'} = $step_encoder;
     }
     if (!$HDSCAN_FLAGS->{'skip set slew'} && !$HDSCAN_FLAGS->{'skip set speed'}) {
        print STDOUT $script.': setting scanning speed & slew motion prg'."\n";
        &setAxisSlewScan ($assy, $axis, $l_speed);			#setMotionPrg+setAxisSpeed
     }
     elsif ($HDSCAN_FLAGS->{'skip set slew'} && !$HDSCAN_FLAGS->{'skip set speed'}) {
        print STDOUT $script.': setting scanning speed only, but presuming slew motion prg'."\n";
        &setAxisSpeed ($assy, $axis, $l_speed); 		#this is faster (no motion prg change)
     }
     elsif (!$HDSCAN_FLAGS->{'skip set slew'} && $HDSCAN_FLAGS->{'skip set speed'}) {
        print STDOUT $script.': setting slew motion prg, but presuming scanning speed is set'."\n";
        &setMotionPrg($assy, 1); 				#this is faster (no speed change): 1=slew
     }
  }

### Setting monitors on motors motion and Struck:
  if ($HDSCAN_FLAGS->{'has run'} == 0 ) {
     $status = &addCaMonitor($struck.':Acquiring');
#    $status = &setAssyMonitors($assy);			#this was set before in runMotor
  }

### Read motor position at startup:
  $HDSCAN_FLAGS->{'beginAct'} = &getAxisAct ($axis);

### Initialize PLC:
  $status = &sendMtrCmd($mtr,$plc_ini_start.'=1','nowait');
  if ($status) {
     &abortAssy ($assy, 1, $axis);				#also restore speed and motion prg.
     return 1, 'Error setting '.$plc_ini_start.' (PLC start) for motor='.$mtr;
  }

#-----------------------------------------------------------------
  if (defined $shutter) {
     if ($HDSCAN_FLAGS->{'skip shutter open'} == 0) {
#       print STDOUT $script.': Opening shutter '.$shutter."\n";
        &openShutter ($shutter,$shutter_readback);
     }
  }

### Start Struck:
# print STDOUT $script.': Starting On-the-fly scan of '.$axis.' with '.$struck.'...'."\n";
  &runStruck ($struck, $iserased);		   	#0=EraseStart, 1=StartAll

  $delayTime = $dwell;
  if ($delayTime < 0.1) {$delayTime = 0.1;}		#delay motors start in order to catch all counts
  select(undef,undef,undef,$delayTime);			#sleep($wait_step);

### Start the motor:
  printf STDOUT $script.': Starting scan of '.$axis.' from %.3f to %.3f...'."\n",$begin,$end;
  &startMotor ($axis, $end);

  printf STDOUT "\n".$script.': HD-sync scan of '.$axis.' in progress.'
                    .' Estimated scan time %.3gs. Please wait...'."\n\n",$scanTime;

  return 0,'';

}  # end of HDflyScan_PLC_setup


#=================================================================


# sub HDflyScan_PLC_wait ($assy,$axis) {
  sub HDflyScan_PLC_wait($$) {

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext);
  our ($HDSCAN_FLAGS);			#reference to a hash with various flags for HDSCAN
  my ($script, $assy, $axis);
  my ($struck, $acquiring, $delayTime, $status);
  my ($wait_step, $wait_, $inpos, $runprg, $positioned);

  $script = 'HDflyScan_PLC_wait';
  $assy   = shift(@_);
  $axis   = shift(@_);

  if (!defined $HDSCAN_FLAGS->{'has run'})              {$HDSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'has read mtrList'})     {$HDSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter close'})   {$HDSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter open'})    {$HDSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip calc speed'})      {$HDSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set speed'})       {$HDSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set slew'})        {$HDSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore speed'})   {$HDSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore mprg'})    {$HDSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip move to start'})   {$HDSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'return to start'})      {$HDSCAN_FLAGS->{'return to start'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'dwell'})                {$HDSCAN_FLAGS->{'dwell'}                = 0;}

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }

### Wait for on-the-fly scan to finish (don't give up first 5*$delayTime):
  $wait_step  = 0.01;
  $wait_      = 0;
  $acquiring  = 1;
  $inpos      = 0;
  $runprg     = 1;
  $positioned = 0;
  $delayTime  = $HDSCAN_FLAGS->{'dwell'};
  if ($delayTime < 0.1) {$delayTime = 0.1;}		#delay motors start in order to catch all counts
  $delayTime = 5*$delayTime;

  while (($acquiring ==1 && ($runprg || $positioned<3)) || $wait_< $delayTime) {
     select(undef,undef,undef,$wait_step);		#sleep($wait_step);
     $wait_ += $wait_step;
     ($status,$acquiring) = Pezca::GetDouble($struck.':Acquiring');
     ($status,$inpos)     = Pezca::GetDouble($assy.':InPos');
     ($status,$runprg)    = Pezca::GetDouble($assy.':RunPrg');
     if ($inpos == 1) {$positioned++;}
     else             {$positioned=0;}
#    print STDOUT $script.': acquiring='.$acquiring.', runprg='.$runprg.', inpos='.$inpos
#                .', positioned='.$positioned.', wait_='.$wait_."\n";
  }
  printf STDOUT $script.': Scan finished! (acquiring='.$acquiring.', mtrRunPrg='.$runprg
               .', mtrInPos='.$positioned.', time=%.1fs)'."\n", $wait_;
  if (defined $shutter) {
     if ($HDSCAN_FLAGS->{'skip shutter close'} == 0) {
        &closeShutter ($shutter,$shutter_readback);
     }
  }
  if ($HDSCAN_FLAGS->{'skip restore mprg'} == 0 && $HDSCAN_FLAGS->{'skip restore speed'} == 0) {
     print STDOUT $script.': aborting motor, restoring speed and motion prg'."\n";
     &abortAssy ($assy, 1, $axis);			# also restore speed and motion prg.
  }
  elsif ($HDSCAN_FLAGS->{'skip restore mprg'} && $HDSCAN_FLAGS->{'skip restore speed'} == 0) {
     print STDOUT $script.': aborting motor, restoring speed, but keeping motion prg'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
     &setAxisHiSpeed ($assy, $axis);			# restore speed only
  }
  elsif ($HDSCAN_FLAGS->{'skip restore mprg'} == 0 && $HDSCAN_FLAGS->{'skip restore speed'}) {
     print STDOUT $script.': aborting motor, restoring motion prg, but keeping speed'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
     &setMotionPrg($assy, 3); 				#this is faster (no speed change): 3=position
  }
  else  {
     print STDOUT $script.': aborting motor only, but keeping motion prg and speed'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
  }

  print STDOUT $script.': aborting Struck'."\n";
  &abortStruck ($struck);

  if ($HDSCAN_FLAGS->{'has run'} == 0) {$HDSCAN_FLAGS->{'has run'} = 1;}

  return 0,' ';

}	##HDflyScan_PLC_wait


#=================================================================


# ($status,$actPts) = &HDflyScan_PLC_read ($axis,\@inputs,\@Offsets,\@data,[$finalORfly]);
  sub HDflyScan_PLC_read($\@\@\@;$) {

### Returns ($status, $actPts) and also fills in \@data (positions,timeline,inputs)
  our ($beamline, $struckName, $struck_ext, $IGNORE_ERRORS);
  our ($HDSCAN_FLAGS);		#reference to a hash with various flags for HDSCAN
  my ($script, $axis);
  my ($begin, $beginAct, $endAct, $step);
  my ($inputs_ref, $Offsets_ref, $data_ref, $finalORfly);
  my ($struck, $status, $actPts, $currPos, $currTime);
  my (@intensity, @struckTimer);
  my ($ninputs, $input, $i, $j);
  my ($range, $rangeErr, $rangeErr_Rel, $scanStep, $errorThreshold);

  $script      = 'HDflyScan_read';
  $axis        = shift(@_);
  $inputs_ref  = shift(@_);
  $Offsets_ref = shift(@_);
  $data_ref    = shift(@_);		#2D-array to fill with data (position+timer+ninputs)
  $finalORfly  = shift(@_);

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if (!defined $HDSCAN_FLAGS->{'has run'})              {$HDSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'has read mtrList'})     {$HDSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter close'})   {$HDSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter open'})    {$HDSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip calc speed'})      {$HDSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set speed'})       {$HDSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set slew'})        {$HDSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore speed'})   {$HDSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore mprg'})    {$HDSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip move to start'})   {$HDSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'return to start'})      {$HDSCAN_FLAGS->{'return to start'}      = 0;}

  if (!defined $finalORfly) {$finalORfly = 'flywithread';}
  if (($finalORfly !~ /^final$/i) &&				#abortStruck before reading
      ($finalORfly !~ /^fly$/i)   &&				#just read
      ($finalORfly !~ /^flywithread$/i)) {                      #pushStruckRead before reading
    print STDOUT '!!! '.$script.': incorrect flag finalORfly='.$finalORfly
                               .'. Presuming "flyWithRead"'."\n";
    $finalORfly = 'flywithread';
  }

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/) {$struck = $beamline.':'.$struck;}

### Start reverting motor to the beginning position
### in order to save time on the next scan:
  if ($finalORfly =~ /^final$/i) {
     $HDSCAN_FLAGS->{'has run'} = 1;
#    print STDOUT $script.': Scan finished!'."\n";
     $endAct = &getAxisAct ($axis);				#Read motor position at the end
#    printf STDOUT $script.': Motor end position = %.3f'."\n",$endAct;
     if ($HDSCAN_FLAGS->{'return to start'}) {
        $begin = $HDSCAN_FLAGS->{'begin'};
        printf STDOUT $script.': Initiating motor return to the beginning pos=%.3f'."\n", $begin;
        &startMotor ($axis, $begin);
     }
  }

  @$data_ref = ();
#-----------------------------------------------------------------
  $ninputs = @$inputs_ref;
# print STDOUT $script.': Reading Struck...'."\n";
### Read Struck timer:
# ($actPts, @struckTimer) = &readStruckTimer ($struck, $struck_ext, undef, $finalORfly);
  $actPts = &readStruckTimerRef ($struck, $struck_ext, \@struckTimer, undef, $finalORfly);
### Read motor position data:
# print STDOUT $script.': Reading positive/negative moves from Struck inputs '.$struckPs.'/'.$struckNg."\n";
# @encoderPs = &readStruckInputRaw ($struck, $struck_ext, $actPts, $struckPs, 'fly');
# @encoderNg = &readStruckInputRaw ($struck, $struck_ext, $actPts, $struckNg, 'fly');
### Read intensity data:
# print STDOUT $script.': Reading intensity from Struck inputs';
  for ($i=0; $i<$ninputs; $i++) {
     $input = @$inputs_ref[$i];
#    @intensity = &readStruckInput ($struck, $struck_ext, $actPts, $input, \@struckTimer, 'fly');
     $status = &readStruckInputRef ($struck, $struck_ext, $actPts, $input, \@struckTimer, \@intensity, 'fly');
     for ($j=0; $j<$actPts; $j++) {			#Correct for background:
        $$data_ref[$i+2][$j] = $intensity[$j] - @$Offsets_ref[$input-1];
     }
  }

### Erase Struck so that when calling HDflyScan next time, it will be already erased:
  if ($finalORfly =~ /^final$/i) {
     &eraseStruck ($struck);
  }
  $beginAct = $HDSCAN_FLAGS->{'beginAct'};
  $step     = $HDSCAN_FLAGS->{'step'};
#-----------------------------------------------------------------
### Calculate coordinates:
  $currTime = 0.;						#starting time
  for ($i=0; $i<$actPts; $i++) {
     $$data_ref[0][$i] = $beginAct + ($i+0.5)*$step;
### Timeline data:
     $$data_ref[1][$i] = $currTime + 0.5*$struckTimer[$i];	#time at the middle of Struck bin
     $currTime  += $struckTimer[$i];				#time at the end of Struck bin
  }
#-----------------------------------------------------------------
### Test the consistency of scan interval:
  if ($finalORfly =~ /^final$/i) {
     $currPos = $beginAct + $actPts*$step;
     $rangeErr = $currPos - $endAct;
     $range    = abs ($endAct - $beginAct);
     if ($range == 0) {
        print  STDOUT '!!! '.$script.': zero scan range or motor did not move!'."\n";
        return 0, $actPts;
     }
     $rangeErr_Rel = 100. * $rangeErr / $range;
     if ($actPts > 0) {$scanStep = $range / $actPts;}
     else             {$scanStep = $range;}
     if ($actPts > 0) {$errorThreshold = &maxval(1.,50./$actPts);}
     else             {$errorThreshold = 50.}
     if (abs($rangeErr_Rel) > $errorThreshold) {
        printf STDOUT '!!! '.$script.': END position error=%.5g (scan step=%.5g)'."\n",$rangeErr,$scanStep;
        printf STDOUT '!!! '.$script.': Scan  range  error=%.3g%s'."\n",$rangeErr_Rel,'%';
        printf STDOUT '!!! '.$script.': Start posit.(EPICS data) = %.5f'."\n",$beginAct;
        printf STDOUT '!!! '.$script.': End  posit. (EPICS data) = %.5f'."\n",$endAct;
        printf STDOUT '!!! '.$script.': End Theta (Struck calc.) = %.5f'."\n",$currPos;
        printf STDOUT '!!! '.$script.': Actual number of scan points = %d'."\n",$actPts;
        return -2,$actPts;
     }
  }
#-----------------------------------------------------------------

# print STDOUT $script.': Scan data retrieved!'."\n";
  return 0, $actPts;

}	##HDflyScan_PLC_read

