#!/usr/bin/perl -W
#--------------------------------------------------------------
# These are Struck scaler related subs for beamline controls Perl scripts
#
#     				Author: Sergey Stepanov
#
# Version-1.2:  2004/11/16 using Pezca
# Version-2.0:  2007/02/02 added ref-style functions
# Version-2.1:  2007/04/09 added HDflyScan and ENflyScan
# Version-2.2:  2007/04/09 added split ENflyScan and changed style of Struck reading
#                          (pushStruckRead,setStruckReadRate functions and $finalORfly flag)
# Version-2.3:  2011/02/08 converted HDflyScan and ENflyScan into a sequence of SCAN_setup,
#                          SCAN_wait and SCAN_read, introduced global hash pointers
#                          $HDSCAN_FLAGS and $ENSCAN_FLAGS
# Version-2.4:  2012/02/03 some changes for mca-7.1; in particular added waiting to
#                          complete after abort.
# Version-2.5:  2014/03/28 switched most of passing arrays by value to passing by pointer
# Version-2.6:  2014/06/18 added turning off autocount in setStruck
# Version-2.7:  2017/03/29 added motion tracking to HDflyScan_wait & ENflyScan_wait (like in runMotor)
# Version-2.8:  2019/02/12 added to &HDflyScan checking for overload, aferscan moving and posting progress arrays;
#                          for that purpose added optional arguments to &HDflyScan and&HDflyScan_wait 
# Version-2.9:  2021/08/25 adapted to new &runMono syntax
# Version-3.0:  2023/03/29 adapted to support the Measurement Computing Systems (MCS) USB-CTR08 scalers
#--------------------------------------------------------------
# List of functions:
# ^^^^^^^^^^^^^^^^^
#  1.        $maxinputs = &StruckNinputs([$struck]);
#  2.             $nmax = &getStruckNmax($struck, [$struck_ext]);
#  3.           $useMax = &getStruckNuseMax($struck);
#  4.             $nact = &getStruckNpts($struck, $struck_ext, [$finalORfly]);
#  5.             $mode = &getStruckMode($struck,[string]);			#MCS or Scaler
#  6.          $advance = &getStruckAdvance($struck,[string]);			#Internal/External
#  7.            $state = &getStruck($struck);					#1=Acquiring or 0=Done
#  8.                     &setStruck ($struck, $struck_ext, $int_ext, $dwell, $num_pts, [$scaler]);
#  9.           $status = &setStruckDwell ($struck, $dwell);
# 10.            $dwell = &getStruckDwell ($struck);
# 11.                     &runStruck ($struck, [$iserased]);   			#0=EraseStart, 1=StartAll
# 12.                     &eraseStruck ($struck);
# 13.                     &abortStruck ($struck);
# 14.                     &setStruckReadRate ($struck,$rate);
# 15.                     &pushStruckRead ($struck);
# 16.                     &waitStruck ($struck);
# 17.            $model = &getStruckModel($struck);
# 18. $struck_frequency = &getStruckFrequency($struck);
# 19.         (min,max) = &getStruckDwellMinMax($struck);
# 20. ($num_pts,@timer) = &readStruckTimerRaw ($struck, $struck_ext, [$read_pts,$finalORfly]);
# 21. ($num_pts,@timer) = &readStruckTimer ($struck, $struck_ext, [$read_pts,$finalORfly]);
# 22.          $num_pts = &readStruckTimerRawRef ($struck, $struck_ext, \@timer, [$read_pts,$finalORfly]);
# 23.          $num_pts = &readStruckTimerRef ($struck, $struck_ext, \@timer, [$read_pts,$finalORfly]);
# 24.             @data = &readStruckInputRaw ($struck, $struck_ext, $num_pts, $input, [$finalORfly]);
# 25.             @data = &readStruckInput ($struck, $struck_ext, $num_pts, $input, \@timer, [$finalORfly]);
# 26.           $status = &readStruckInputRawRef ($struck, $struck_ext, $num_pts, $input, \@data, [$finalORfly]);
# 27.           $status = &readStruckInputRef ($struck, $struck_ext, $num_pts, $input, \@timer, \@data, [$finalORfly]);
# 28.                     &readStruck ($struck, $struck_ext, $num_pts, $ninputs, \@inputs, [$finalORfly]);
#                                                               - returns GLOBAL arrays:
#                                                                 @StruckData${input}($num_pts)...
# 29.                     &printStruckInputs($LUN, $struck, $ninputs, @inputs);
# 30.          @offsets = &getStruckOffsets($file, $struck);			#or:  @offsets = &getStruckOffsets(\@offsets, $struck);
# 31.                     &printStruckOffsets($LUN, $file, $struck, \@offsets, $ninputs, \@inputs);
# 32.                     &printStruck_1pti($LUN, $struck, \@offsets, $pt, $input, \@data);
# 33.                     &printStruck_1pt ($LUN, $struck, \@offsets, $pt, $ninputs, \@inputs);
#                                                               - also uses GLOBAL:
#                                                                 @StruckData${input}($num_pts)...
# 34.                     &openStruckMedm($struck, $struck_ext, $input);
#
# On-the-fly scans:
# ^^^^^^^^^^^^^^^^
# 35. ($status,$actPts) =
#                         &HDflyScan($assy,$axis,$begin,$end,$npts,$scanTime,$mtrOut,$cts2egu,
#                                    $formula,$struckPs,$struckNg,$medm,$fileBkg,$ninputs,\@inputs,
#                                    [$scaler,$afterScan,$checkOverload,$postarrayPV,$post_interval]);
#                                                               - Also returns GLOBALS:
#                                                                 @x1HD,
#                                                                 @timeHD,
#                                                                 @y1HD,...@y{$ninputs}HD,
#                                                                 @StruckOffsets
# 36. ($status,$message,[$beginAct,$cts2egu,$mtrOut,$struckPs,$struckNg,$dwell]) =
#                         &HDflyScan_setup($assy,$axis,$begin,$end,$npts,$timePPT,[$scaler]);
#
# 37. ($status,$message)= &HDflyScan_wait($assy,$axis,$mtrOut,$dwell,$beginPos,$endPos,$npts,
#                                         [\@inputs,$postarrayPV,$post_interval,$i_plot,
#                                          $cts2egu,$struckPs,$struckNg,$checkOverload,$threshold]);
#
# 38. ($status,$actPts) = &HDflyScan_read ($axis,$beginAct,$cts2egu,$struckPs,$struckNg,
#                                          \@inputs,\@Offsets,\@data,[$finalORfly]);
#                                                               -- fills \@data=(@positions,@timeline,@inputs)
#
# 39. ($status,$actPts) =
#                         &ENflyScan($beginEnergy,$endEnergy,$npts,$scanTime,
#                                    $linkID,$undulator,$IDoffset,
#                                    $medm,$fileBkg,$ninputs,\@inputs,[$scaler]);
#                                                               - Also returns GLOBALS:
#                                                                 @x1EN,
#                                                                 @timeEN,
#                                                                 @y1EN,...@y{$ninputs}EN,
#                                                                 @StruckOffsets
#
# 40. ($status,$message,[$assy,$axisEnergy,$axisTheta,$beginThetaAct,$ESinTheta,
#                        $cts2egu,$mtrOut,$struckPs,$struckNg,$dwell]) =
#                         &ENflyScan_setup($beginEnergy,$endEnergy,$npts,$timePPT,
#                                         [$linkID,$undulator,$IDoffset,$scaler]);
#
# 41. ($status,$message)= &ENflyScan_wait ($assy,$axisEnergy,$mtrOut,$dwell,$beginEnergy,$endEnergy,$npts);
#
# 42. ($status,$actPts) = &ENflyScan_read ($axisTheta,$beginThetaAct,$ESinTheta,
#                                          $cts2egu,$struckPs,$struckNg,
#                                          \@inputs,\@Offsets,\@data,[$finalORfly]);
#                                                               -- fills \@data=(@energy,@timeline,@inputs)
#
#  Find-on-the-fly scans:
# ^^^^^^^^^^^^^^^^^^^^^^
# 40. ($status,$actPts,$pos) =
#               &findPosHD ($assy,$axis,$begin,$end,$npts,$scanTime,$medm,
#                           \@Offsets,$input,$target,$direction,\@data,[\%flags,$scaler]);
#                                                               -- fills \@data=(@positions,@timeline,@inputs)
#                                                               -- if scanTime=0, tries to use max.speed
#--------------------------------------------------------------
# use Pezca;
# use Math::Trig;               # for energy scans
# use Time::HiRes qw(usleep gettimeofday tv_interval);

# use warnings::unused;
  use strict;
# no strict 'vars';
# no strict 'subs';
# no strict 'refs';

#=================================================================

# sub StruckNinputs ([$struck]) {
sub StruckNinputs (;$) {		# - returns maximum number of inputs in Struck

  my ($script, $struck, $model, $MaxInputs);

  $script = 'StruckNinputs';
  $struck = shift(@_);

  if (defined $struck) {
     if ($struck =~ /:$/) {chop($struck);}
     $model = &getStruckModel($struck);
     if ($model =~ /^USB/i) {
        $MaxInputs = int(substr($model,-2));
     }
     else {
        ### Presumably the model is SIS3801 or SIS3820
        $MaxInputs = 32;
     }
  } else { 
     ### The default model is SIS3801 or SIS3820
     $MaxInputs = 32;
  }
  return $MaxInputs;
}

#=================================================================

# sub getStruckNmax ($struck, $struck_ext) {
sub getStruckNmax ($;$) {	# - returns maximum number of points in Struck

  my ($script, $struck, $struck_ext, $struckInputName, $status, $maxPts);

  $script     = 'getStruckNmax';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);

  ($status, $maxPts) = Pezca::GetDouble($struck.':MaxChannels');
  if ($status) {
     &report_only ($script, $status, $struck.':MaxChannels');
     $maxPts = 0;
  }
  if ($maxPts == 0 && defined $struck_ext) {				# for antique drivers
     $struckInputName = $struck.$struck_ext;
     ($status, $maxPts) = Pezca::GetDouble($struckInputName.'1.NMAX');
     if ($status) {
        &report_only ($script, $status, $struckInputName.'1.NMAX');
        $maxPts = 0;
     }
  }
  return $maxPts;
}

#=================================================================

# sub getStruckNuseMax ($struck) {
sub getStruckNuseMax ($) {	# - returns preset number of points to use in Struck

  my ($script, $struck, $status, $useMax);

  $script = 'getStruckNuseMax';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}

  ($status, $useMax) = Pezca::GetDouble($struck.':NuseAll');
  if ($status) {
     &report_exit ($script, $status, $struck.':NuseAll');
     $useMax = 0;
  }
  return $useMax;
}

#=================================================================

# sub getStruckNpts($struck, $struck_ext, [$finalORfly]) {
sub getStruckNpts($$;$) {
### - this function returns $npts, the number of points actually read
### before Struck stopped. To do it, it reads the first input (the
### clock) and determines when it becomes zero.

  my ($script, $struck, $struck_ext, $finalORfly);
  my ($num_pts, @timer);

  $script     = 'getStruckNpts';
  $struck     = shift(@_);
  $struck_ext = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

### Read struck timer (raw data):
  ($num_pts, @timer) = &readStruckTimerRaw ($struck, $struck_ext, undef, $finalORfly);

  if ($num_pts <= 0) {print STDOUT '!!! '.$script.': Struck ['.$struck.'] returned zero NPTS'."\n";}
  return $num_pts;
}

#=================================================================

# sub getStruckMode ($struck,[string]) {
sub getStruckMode ($;$) {			# - returns 0=MCS/1=Scaler

  my ($script, $struck, $status, $mode, $string);

  $script = 'getStruckMode';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $string = shift(@_);	if (!defined $string) {$string='double';}

  if ($string =~ /^string$/i) {
     ($status, $mode) = Pezca::GetString($struck.':AcquireMode');
  } else {
     ($status, $mode) = Pezca::GetDouble($struck.':AcquireMode');
  }
  if ($status) {
     ### On older versions of Struck driver the AcquireMode PV
     ### did not exist. Then we presume "MCS".
     &report_only ($script, $status, $struck.':AcquireMode');
     if ($string =~ /^string$/i) {$mode = 'MCS';}
     else                        {$mode = 0;}
  }
  return $mode;
}

#=================================================================

# sub getStruckAdvance ($struck,[string]) {
sub getStruckAdvance ($;$) {			# - returns 0=Internal/2=External

  my ($script, $struck, $status, $advance, $string);

  $script = 'getStruckAdvance';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $string = shift(@_);	if (!defined $string) {$string='double';}
  if ($string =~ /^string$/i) {
     ($status, $advance) = Pezca::GetString($struck.':ChannelAdvance');
  } else {
     ($status, $advance) = Pezca::GetDouble($struck.':ChannelAdvance');
  }
  if ($status) {
     &report_exit ($script, $status, $struck.':ChannelAdvance');
     $advance = 0;				#internal
  }
  return $advance;
}

#=================================================================

# sub getStruck ($struck) {
sub getStruck ($) {			# - returns 1=Acquiring or 0=Done

  my ($script, $struck, $status, $state);

  $script = 'getStruck';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}

  ($status, $state) = Pezca::GetDouble($struck.':Acquiring');
  if ($status) {
     &report_exit ($script, $status, $struck.':Acquiring');
     $state = 0;
  }
  return $state;
}

#=================================================================

# sub setStruck ($struck, $struck_ext, $int_ext, $dwell, $num_pts, [$scaler]) {
sub setStruck ($$$$$;$) {

  our $IGNORE_ERRORS;
  my ($script, $struck, $struck_ext, $int_ext, $dwell, $num_pts, $scaler);
  my ($status, $struckNMAX, $struckInputName, $current_time, $model);
  my ($DWELLTIMEMIN, $DWELLTIMEMAX, $acquiring, $mode, $msg);
  my (@usb, $usbctr, $frequency);

  $script     = 'setStruck';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $int_ext    = shift(@_);
  $dwell      = shift(@_);
  $num_pts    = shift(@_);
  $scaler     = shift(@_);      #if scaler is specified (when Struck is also used as Joerger), then abort it and turn off autocount

  $struckInputName = $struck.$struck_ext;
  $model = &getStruckModel($struck);

  if (defined $scaler) {
     $mode = &getStruckMode($struck,'string');
     if ($mode ne 'MCS') {
        if ($scaler =~ /:$/) {chop($scaler);}
        $status = Pezca::PutDoubleOldCa($scaler.'.CONT',0);	#stop autocount
        &report_only($script, $status, $scaler.'.CONT');
        select(undef,undef,undef,0.1);				#sleep 0.1s
        $status = Pezca::PutDoubleOldCa($scaler.'.CNT',0);	#stop count
        &report_only($script, $status, $scaler.'.CNT');
     }
  }

  $acquiring = &getStruck($struck);
  if ($acquiring) {
     &abortStruck ($struck);
# } else {
#    $mode = &getStruckMode($struck,'string');
#    if ($mode ne 'MCS') {		#we need to switch the boad to the MCS mode before erasing.
#       &runStruck ($struck,0);		#The only way to switch to MCS is to run (0=EraseStart)
#       select(undef,undef,undef,0.1);	#sleep 0.1s
#       &abortStruck ($struck);		#now abort
#    }
  }
  &eraseStruck ($struck);		#added 2012/02 for mca-7.1

### Read the maximum possible number of channels per input:
  $struckNMAX = &getStruckNmax($struck, $struck_ext);
  if ($num_pts > $struckNMAX) {
     $current_time = scalar localtime;
     $msg  = '*** '.$script.': '.$current_time."\n";
     $msg .= '*** '.$script.': The requested number of points='.$num_pts."\n";
     $msg .= '*** '.$script.': exceeds the Struck buffer='.$struckNMAX."\n";
     &press_key_exit($msg);
  }

  ($DWELLTIMEMIN,$DWELLTIMEMAX) = &getStruckDwellMinMax($struck);
  if ($dwell < $DWELLTIMEMIN) {
     $current_time = scalar localtime;
     $msg   = '*** '.$script.': '.$current_time."\n";
     $msg .= sprintf '*** '.$script.': Calculated time per point=%.3gs is under Struck limit of '
                                   .$DWELLTIMEMIN."s\n", $dwell;
     if ($IGNORE_ERRORS) {print STDOUT $msg;  $dwell=$DWELLTIMEMIN;}
     else                {&press_key_exit($msg);}
  }
  elsif ($dwell > $DWELLTIMEMAX) {
     $current_time = scalar localtime;
     $msg  ='*** '.$script.': '.$current_time."\n";
     $msg .= sprintf '*** '.$script.': Calculated time per point=%.3fs exceeds Struck limit of '
                                   .$DWELLTIMEMAX."s\n", $dwell;
     if ($IGNORE_ERRORS) {print STDOUT $msg;  $dwell=$DWELLTIMEMAX;}
     else                {&press_key_exit($msg);}
  }

### Added 2010.10.14 -- clear "preset realtime":
  $status = Pezca::PutDouble($struck.':PresetReal',0.0);
  &report_exit ($script, $status, $struck.':PresetReal');

### Added 2013.02.21 -- set InputMode=3:
### Changed 2023.03.30 - this must be for SIS only:
  if ($model =~ /^SIS38/i) {
     $status = Pezca::PutDouble($struck.':InputMode',3);
     &report_exit ($script, $status, $struck.':InputMode');
  }
  elsif ($model =~ /^USB-CTR/i) {
     $status = Pezca::PutDouble($struck.':TrigMode',3);
     &report_exit ($script, $status, $struck.':TrigMode');
     ### In order to have clock data in the 1st channel, we need to setup
     ### and start the USB-CTR pulse generator at the max frequency:
     $frequency = &getStruckFrequency($struck);
     ### We need the EPICS name of the USB-CTR device. Typically:
     ### $struck = 23b:ctr2:MCS;
     ### $usbctr = 23b:ctr2;
     @usb = split(':',$struck); pop(@usb); 		#remove the last element (e.g. "MCS")
     $usbctr = join(':',@usb);
     ### Stop USB-CTR pulse generator-1:
     $status = Pezca::PutDouble($usbctr.':PulseGen1Run',0);
     &report_exit ($script, $status, $usbctr.':PulseGen1Run');
     ### Set the max frequency:
     $status = Pezca::PutDouble($usbctr.':PulseGen1Frequency',$frequency);
     &report_exit ($script, $status, $usbctr.':PulseGen1Frequency');
     ### Start USB-CTR pulse generator-1:
     $status = Pezca::PutDouble($usbctr.':PulseGen1Run',1);
     &report_exit ($script, $status, $usbctr.':PulseGen1Run');
  }

  if ($int_ext == 0) {			#Internal
     $status = Pezca::PutDouble($struck.':ChannelAdvance',$int_ext);
     &report_exit ($script, $status, $struck.':ChannelAdvance');
     $status = Pezca::PutDouble($struck.':Prescale',1);
     &report_exit ($script, $status, $struck.':Prescale');
     $status = Pezca::PutDouble($struck.':Dwell',$dwell);
     &report_exit ($script, $status, $struck.':Dwell');
  }
  elsif ($int_ext == 1) {		#External
     $status = Pezca::PutDouble($struck.':ChannelAdvance',$int_ext);
     &report_exit ($script, $status, $struck.':ChannelAdvance');
     $status = Pezca::PutDouble($struck.':Prescale',$dwell);
     &report_exit ($script, $status, $struck.':Prescale');
     $status = Pezca::PutDouble($struck.':Dwell',0);
     &report_exit ($script, $status, $struck.':Dwell');
  }
  else {
     $current_time = scalar localtime;
     $msg  = '*** '.$script.': '.$current_time."\n";
     $msg .= '*** '.$script.': Incorrect Struck advance mode='.$int_ext."\n";
     $msg .= '*** '.$script.': -- must be 0 (Internal) or 1 (External)'."\n";
     &press_key_exit($msg);
  }
  $status = Pezca::PutDouble($struck.':NuseAll',$num_pts);
  &report_exit ($script, $status, $struck.':NuseAll');
}

#=================================================================

# sub setStruckDwell ($struck, $dwell) {
sub setStruckDwell ($$) {

  our $IGNORE_ERRORS;
  my ($script, $struck, $dwell, $status, $current_time, $msg);
  my ($DWELLTIMEMIN, $DWELLTIMEMAX);

### Attn: internal channel advance only!

  $script = 'setStruckDwell';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $dwell  = shift(@_);

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  ($DWELLTIMEMIN,$DWELLTIMEMAX) = &getStruckDwellMinMax($struck);

  &abortStruck ($struck);

  if ($dwell < $DWELLTIMEMIN) {
     $current_time = scalar localtime;
     $msg  = '*** '.$script.': '.$current_time."\n";
     $msg .= sprintf '*** '.$script.': Calculated time per point=%.3fs is under Struck limit of '
                                   .$DWELLTIMEMIN."s\n", $dwell;
     if ($IGNORE_ERRORS) {print STDOUT $msg;  $dwell=$DWELLTIMEMIN;}
     else                {&press_key_exit($msg);}
  }
  elsif ($dwell > $DWELLTIMEMAX) {
     $current_time = scalar localtime;
     $msg  = '*** '.$script.': '.$current_time."\n";
     $msg .= sprintf '*** '.$script.': Calculated time per point=%.3fs exceeds Struck limit of '
                                   .$DWELLTIMEMAX."s\n", $dwell;
     if ($IGNORE_ERRORS) {print STDOUT $msg;  $dwell=$DWELLTIMEMAX;}
     else                {&press_key_exit($msg);}
  }

  $status = Pezca::PutDouble($struck.':Dwell',$dwell);
  if ($IGNORE_ERRORS) {&report_only ($script, $status, $struck.':Dwell');}
  else                {&report_exit ($script, $status, $struck.':Dwell');}
  return $status;
}

#=================================================================

# sub getStruckDwell ($struck) {
sub getStruckDwell ($) {

  my ($script, $struck, $dwell, $status);

### Attn: internal channel advance only!

  $script = 'getStruckDwell';
  $struck = shift(@_);		if ($struck =~ /:$/) {chop($struck);}
  $dwell  = shift(@_);

  ($status, $dwell) = Pezca::GetDouble($struck.':Dwell');
  if ($status) {
     &report_exit ($script, $status, $struck.':Dwell');
     $dwell = 0;
  }
  return $dwell;
}

#=================================================================

# sub runStruck ($struck, [$iserased]) {
sub runStruck ($;$) {

  my ($script, $struck, $iserased, $status);

  $script   = 'runStruck';
  $struck   = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $iserased = shift(@_);
  if (!defined $iserased) {$iserased = 0;}

  if ($iserased) {
     $status = Pezca::PutDoubleOldCa($struck.':StartAll',1);
     &report_exit ($script, $status, $struck.':StartAll');
  }
  else  {
     $status = Pezca::PutDoubleOldCa($struck.':EraseStart',1);
     &report_exit ($script, $status, $struck.':EraseStart');
  }
}

#=================================================================

# sub eraseStruck ($struck) {
sub eraseStruck ($) {

  my ($script, $struck, $status);

  $script = 'eraseStruck';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}

  $status = Pezca::PutDoubleOldCa($struck.':EraseAll',0);
  $status = Pezca::PutDoubleOldCa($struck.':EraseAll',1);
  &report_exit ($script, $status, $struck.':EraseAll');
}

#=================================================================

# sub abortStruck ($struck) {
sub abortStruck ($) {

  my ($script, $struck, $status);
  my ($wait_step, $wait_, $wait_limit, $acquiring);

  $script = 'abortStruck';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}

  if (!defined $struck) {
    print STDOUT '*** '.$script.' no Struck PV specified.'."\n";
  }

  $status = Pezca::PutDoubleOldCa($struck.':StopAll',1);
  &report_exit ($script, $status, $struck.':StopAll');

### Wait for completion after abort (added 2012/02 for mca7.1):
  $acquiring  = 1;
  $wait_step  = 0.01;
  $wait_      = 0;
  $wait_limit = 5;		#max. wait for Struck to stop after receiving Abort
  $status = &addCaMonitor($struck.':Acquiring');
  do {
     select(undef,undef,undef,$wait_step);		#sleep($wait_step);
     $wait_ += $wait_step;
     ($status,$acquiring) = Pezca::GetDouble($struck.':Acquiring');
     if ($status) {$acquiring=1;}			#presume worse
  } while ($acquiring == 1 && $wait_ < $wait_limit);
  if ($acquiring) {
     print STDERR "\n".'*** '.$script.': Struck did not stop after aborting!'."\n\n";
     $status = Pezca::PutDoubleOldCa($struck.':StopAll',1);
  }
}

#=================================================================

# sub setStruckReadRate ($struck,$rate) {
sub setStruckReadRate ($$) {

  my ($script, $struck, $rate, $status);
  my (@rateList, @rateNums, $nrates);
  my ($diff, $diff_min, $indx_min, $i);

  $script = 'setStruckReadRate';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $rate   = shift(@_);

  $rate =~ s/second//i;				# strip out 'second'
  $rate =~ s/ //g;				# strip out spaces
  if ($rate =~ /^Passive$/i) {$rate = 0.;}
  @rateList = (0., 10., 5., 2., 1., 0.5, 0.2, 0.1);
  @rateNums = (0,   3,  4,  5,  6,   7,   8,   9);

  $nrates = @rateList;
  $diff_min = 999.;
  $indx_min = 0;

### Find the closest PV SCAN to the specified rate:
  if ($rate != 0.) {
     for ($i=1; $i<$nrates; $i++) {
        $diff = abs($rate-$rateList[$i])/$rateList[$i];
        if ($diff < $diff_min) {
           $diff_min = $diff;
           $indx_min = $i;
        }
     }
     $rate = $rateNums[$indx_min];
  }

  $status = Pezca::PutDoubleOldCa($struck.':ReadAll.SCAN',$rate);
  &report_exit ($script, $status, $struck.':ReadAll.SCAN');
}

#=================================================================

# sub pushStruckRead ($struck) {
sub pushStruckRead ($) {

  my ($script, $struck, $status);

  $script = 'pushStruckRead';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}

  $status = Pezca::PutDoubleOldCa($struck.':ReadAll.PROC',1);
  &report_exit ($script, $status, $struck.':ReadAll.PROC');
  select (undef,undef,undef,0.1);			#give Struck time to read
}

#=================================================================

# sub waitStruck ($struck) {
sub waitStruck ($) {

  our ($wait_max, $nonstop);
  my ($script, $struck, $status, $acquiring);
  my ($wait_step, $wait_, $wait_limit, $msg);
  my ($int_ext, $dwellTime, $presetTime);
  my ($channels2use, $countTime, $current_time);

  $script    = 'waitStruck';
  $struck    = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  if (!defined $wait_max) {$wait_max = 60;}
  if (!defined $nonstop)  {$nonstop  = 0;}

  $acquiring  = 1;
  $wait_step  = 0.05;
  $wait_      = 0;
  $wait_limit = $wait_max;

### Try to estimate the maximum counting time:
  ($status, $int_ext) = Pezca::GetDouble($struck.':ChannelAdvance');
  &report_exit ($script, $status, $struck.':ChannelAdvance');
  if ($int_ext == 0) {			#Internal
     ($status, $dwellTime) = Pezca::GetDouble($struck.':Dwell');
     &report_exit ($script, $status, $struck.':Dwell');
     ($status, $presetTime) = Pezca::GetDouble($struck.':PresetReal');
     &report_exit ($script, $status, $struck.':PresetReal');
     ($status, $channels2use) = Pezca::GetDouble($struck.':NuseAll');
     &report_exit ($script, $status, $struck.':NuseAll');
     $countTime = $dwellTime * $channels2use;
     if ($presetTime > $countTime) {$countTime = $presetTime;}
### Estimated wait time plus a few seconds overhead:
     if ($countTime != 0) {$wait_limit = $countTime+2;}
  }

  $status = &addCaMonitor($struck.':Acquiring');
  select(undef,undef,undef,1);				#wait 1s to ensure that Struck started
  $wait_ += 1;
  while ($acquiring == 1 && $wait_ < $wait_limit) {
     select(undef,undef,undef,$wait_step);		#sleep($wait_step);
     $wait_ += $wait_step;
     ($status,$acquiring) = Pezca::GetDouble($struck.':Acquiring');
  }
  &abortStruck($struck);

  if ($wait_ >= $wait_limit) {
     $current_time = scalar localtime;
     $msg  = '*** '.$script.': '.$current_time."\n";
     $msg .= sprintf '*** '.$script.': Struck scaler did not finish counting after %1.0fs'."\n",$wait_;
     if ($nonstop == 1) {$msg .= '*** ... continuing the script'."\n"; print STDOUT $msg;}
     else               {&press_key_exit($msg);}
  }
}

#=================================================================

# sub getStruckModel ($struck) {
sub getStruckModel ($) {	# - returns Struck model

  our ($STRUCK_MODEL);
  my ($script, $struck, $status, $model);

  $script = 'getStruckModel';
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}

  ($status, $model) = Pezca::GetString($struck.':Model');
  if ($status) {
     &report_only ($script, $status, $struck.':Model');
     ### Attention: older implementations of Struck driver do not have the "Model" PV.
     ### Then, define $STRUCK_MODEL in the beamline config
     if (defined $STRUCK_MODEL) {$model = $STRUCK_MODEL;}
     else                       {$model = 'SIS3801';}
     print STDOUT '!!! '.$script.': unable to read Struck model. Assuming ['.$model.']'."\n";
  }
  return $model;
}

#=================================================================

# sub getStruckFrequency ($struck) {
sub getStruckFrequency ($) {	# - returns Struck clock frequency

  my ($script, $struck, $frequency, $model);

  $script = 'getStruckFrequency';
  $struck = shift(@_);

  $model = &getStruckModel ($struck);
  if    ($model =~ /SIS3820/i) {$frequency = 50E+6;}
  elsif ($model =~ /SIS3801/i) {$frequency = 25E+6;}
  elsif ($model =~ /^USB/i)    {$frequency = 48E+6;}		#inputs cannot take more than 48MHz
  return $frequency;
}

#=================================================================

# sub getStruckDwellMinMax ($struck) {
sub getStruckDwellMinMax ($) {	# - returns Struck dwell limits

  my ($script, $struck, $dwellMin, $dwellMax, $model);

  $script = 'getStruckDwellMinMax';
  $struck = shift(@_);

  $model = &getStruckModel ($struck);

  if    ($model =~ /SIS3820/i) {		#SIS3820
     $dwellMin = 1.0E-6;			#sec; Struck limitation: 1us when all channels are used
     $dwellMax = 42.94967;			#sec; Struck limitation: 2^31=2,147,483,648/50,000,000=42.94967296
  }
  elsif ($model =~ /SIS3801/i) {		#SIS3801
     # In SIS3801 the internal channel advance clock
     # is 10MHz, not 25 MHz. (The channel 0 internal
     # counter clock is 25 MHz). See more details in
     # the Mark Rivers email of 2013/02/22
     $dwellMin = 4.0E-6;			#sec; Struck limitation: 4us when all channels are used
     $dwellMax = 1.67772;			#sec; Struck limitation: 2^24=16,777,216/10,000,000=1.6777216
  }
  elsif ($model =~ /^USB/i) {			#Measurement computing USB-CTR08
     # See: https://www.mccdaq.com/pdfs/manuals/usb-CTR08.pdf
     # is 10MHz, not 25 MHz. (The channel 0 internal
     # counter clock is 25 MHz). See more details in
     # the Mark Rivers email of 2013/02/22
     $dwellMin = 5.E-6;				#sec; determined empirically
     $dwellMax = 1000.;				#sec; seems to be no limitation
  }
  else {
     print STDOUT '*** '.$script.': unknown Struck model='.$model."\n\n";
     $dwellMin = 0;
     $dwellMax = 0;
  }
  return ($dwellMin,$dwellMax);
}

#=================================================================

# sub readStruckTimerRaw($struck, $struck_ext, [$read_pts, $finalORfly]) {
sub readStruckTimerRaw ($$;$$) {
### - returns: ($num_pts,@timer[$num_pts])
###   where:  $num_pts is number of points actually read by Struck before it stopped
###           @timer is a raw clock data of respective length

  our $EZCALONG;
  my ($script, $struck, $struck_ext, $read_pts, $finalORfly);
  my ($struckTimerName, $status, $arr_len, $num_pts, $i, $nonzero, @timer);

  $script     = 'readStruckTimerRaw';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $read_pts   = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  if (!defined $EZCALONG) {&setEZCA32or64();}
  if (!defined $read_pts) {$read_pts = 0;}
###if $read_pts undefined, then set to maximum:
  if ($read_pts == 0) {$read_pts = &getStruckNmax ($struck, $struck_ext);}

  if (!defined $finalORfly) {$finalORfly='final';}
  if (($finalORfly !~ /^final$/i) &&
      ($finalORfly !~ /^fly$/i)   &&
      ($finalORfly !~ /^flywithread$/i)) {
     print STDOUT '!!! '.$script.': incorrect flag finalORfly='.$finalORfly.'. Presuming "final"'."\n";
     $finalORfly='final';
  }
  if    ($finalORfly =~ /^final$/i)       {&abortStruck($struck);}
  elsif ($finalORfly =~ /^flywithread$/i) {&pushStruckRead($struck);}

  $struckTimerName = $struck.$struck_ext.'1';		#the first Struck input

  @timer = ();
  ($status,@timer) = Pezca::GetList($struckTimerName,$EZCALONG,$read_pts);
  &report_exit ($script, $status, $struckTimerName);

  $arr_len = @timer;
  if ($arr_len < $read_pts) {
     print STDOUT '!!! '.$script.': Warning: Struck returned incomplete timer array:'."\n";
     print STDOUT '!!! '.$script.': requested='.$read_pts.'pts, returned='.$arr_len.'pts.'."\n";
#    for ($i=$arr_len; $i<$num_read; $i++) {$timer[$i]=0.0;}
  }

### Search for last non-zero time:
  $nonzero = -1;
  for ($i=$read_pts-1; $i>=0; $i--) {
     if ($timer[$i] != 0) {$nonzero=$i; last;}
 }
  if ($nonzero >= 0) {
     $num_pts = $nonzero+1;
     $#timer  = $nonzero;					#truncate the array
#    splice(@timer,$nonzero+1);                                 #truncate the array - another method
     return $num_pts,@timer;
  } else {
     print STDOUT '!!! '.$script.': Struck returned empty timer array:'."\n";
     $num_pts = 0;
     $#timer  = 0;						#truncate the array
#    splice(@timer);		                                #truncate the array - another method
     return $num_pts,@timer;
  }
}

#=================================================================

# sub readStruckTimer ($struck, $struck_ext, [$read_pts, $finalORfly]) {
sub readStruckTimer ($$;$$) {
### - returns: ($num_pts,@timer[$num_pts])
###   where:  $num_pts is number of points actually read by Struck before it stopped
###           @timer is an array of respective length providing clock data in seconds

  my ($script, $struck, $struck_ext, $read_pts, $finalORfly);
  my ($struck_frequency, $status, $struck_dwell, $num_pts);
  my ($i, $j0, @timer);

  $script     = 'readStruckTimer';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $read_pts   = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  $struck_frequency = &getStruckFrequency($struck);	#25MHz or 50MHz

### Read struck timer:
  ($num_pts,@timer) = &readStruckTimerRaw ($struck, $struck_ext, $read_pts, $finalORfly);

### This is a backup for internal counting mode:
  ($status,$struck_dwell) = Pezca::GetDouble($struck.':Dwell');
  &report_exit ($script, $status, $struck.':Dwell');
  if (!defined $struck_dwell || $struck_dwell == 0) {$struck_dwell = 1E9;}

  $j0 = 0;
  for ($i=0; $i<$num_pts; $i++) {
     if ($timer[$i] > 0) {
        $timer[$i] = $timer[$i] / $struck_frequency;
     } else {
        $timer[$i] = $struck_dwell;		#correct (replace by Dwell)
        $j0++;
        if ($j0 <= 5) { 			#warn first 5 times only
           print STDOUT '!!! '.$script.': no time data in Struck channel='.$i.' of '.$num_pts."\n";
        }
     }
  }
  if ($j0 != 0) {
     print STDOUT '!!! '.$script.': time data in Struck was missing '.$j0.' times of '.$num_pts."\n";
  }

### Correct the 1st channel
### (Struck driver bug?):
### put it back to what
### it should if the deviation
### is less than 5 microseconds:
# if (abs($timer[0]-$struck_dwell) < 0.005) {
#    print STDOUT "\n".'!!! '.$script.': correcting 1st channel of Struck timer: was='
#                                    .$timer[0] now='.$struck_dwell."\n";
#    $timer[0] = $struck_dwell;
# }

  return $num_pts,@timer;
}

#=================================================================

# sub readStruckTimerRawRef ($struck, $struck_ext, \@timer, [$read_pts, $finalORfly]) {
sub readStruckTimerRawRef ($$\@;$$) {
### - returns: $num_pts -- the number of points actually read by Struck before it stopped

  our $EZCALONG;
  my ($script, $struck, $struck_ext, $timerRef, $read_pts, $finalORfly);
  my ($struckTimerName, $status, $arr_len, $num_pts, $i, $nonzero);

  $script     = 'readStruckTimerRawRef';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $timerRef   = shift(@_);
  $read_pts   = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  if (!defined $EZCALONG) {&setEZCA32or64();}
  if (!defined $read_pts) {$read_pts = 0;}
###if $read_pts undefined, then set to maximum:
  if ($read_pts == 0) {$read_pts = &getStruckNmax ($struck, $struck_ext);}

  if (!defined $finalORfly) {$finalORfly='final';}
  if (($finalORfly !~ /^final$/i) &&
      ($finalORfly !~ /^fly$/i)   &&
      ($finalORfly !~ /^flywithread$/i)) {
     print STDOUT '!!! '.$script.': incorrect flag finalORfly='.$finalORfly.'. Presuming "final"'."\n";
     $finalORfly='final';
  }
  if    ($finalORfly =~ /^final$/i)       {&abortStruck($struck);}
  elsif ($finalORfly =~ /^flywithread$/i) {&pushStruckRead($struck);}
# else  {
#    print STDOUT "\n".'!!! '.$script.': called with finalORfly='.$finalORfly."\n\n";
# }

  $struckTimerName = $struck.$struck_ext.'1';		#the first Struck input

  @$timerRef = ();
  ($status,@$timerRef) = Pezca::GetList($struckTimerName,$EZCALONG,$read_pts);
  &report_exit ($script, $status, $struckTimerName);

  $arr_len = @$timerRef;
  if ($arr_len < $read_pts) {
     print STDOUT '!!! '.$script.': Struck returned incomplete timer array:'."\n";
     print STDOUT '    requested='.$read_pts.'pts, returned='.$arr_len.'pts.'."\n";
#    for ($i=$arr_len; $i<$num_read; $i++) {@$timerRef[$i]=0.0;}
  }

### Search for last non-zero time:
  $nonzero = -1;
  for ($i=$read_pts-1; $i>=0; $i--) {
     #my $xxx;
     #if (defined @$timerRef[$i]) {$xxx = @$timerRef[$i];} else {$xxx = 'UNDEF';}
     #print STDOUT 'EZCALONG='.$EZCALONG.'  read_pts='.$read_pts.'  i='.$i.'  timerRef[i]='.$xxx."\n";
     if (@$timerRef[$i] != 0) {$nonzero=$i; last;}
  }
  if ($nonzero >= 0) {
     $num_pts    = $nonzero+1;
     $#$timerRef = $nonzero;					#truncate the array
#    splice(@$timerRef,$nonzero+1);                             #truncate the array - another method
  } else {
     print STDOUT '!!! '.$script.': Struck returned empty timer array:'."\n";
     $num_pts    = 0;
     $#$timerRef = 0;						#truncate the array
#    splice(@$timerRef);					#truncate the array - another method
  }
  return $num_pts;
}

#=================================================================

# sub readStruckTimerRef ($struck, $struck_ext, \@timer, [$read_pts, $finalORfly]) {
sub readStruckTimerRef ($$\@;$$) {
### - returns: $num_pts -- the number of points actually read by Struck before it stopped

  my ($script, $struck, $struck_ext, $timerRef, $read_pts, $finalORfly);
  my ($struck_frequency, $status, $struck_dwell, $num_pts, $i, $j0);

  $script     = 'readStruckTimerRef';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $timerRef   = shift(@_);
  $read_pts   = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  $struck_frequency = &getStruckFrequency($struck);	#25MHz or 50MHz

### Read struck timer:
  $num_pts = &readStruckTimerRawRef ($struck, $struck_ext, $timerRef, $read_pts, $finalORfly);

### This is a backup for internal counting mode:
  ($status,$struck_dwell) = Pezca::GetDouble($struck.':Dwell');
  &report_exit ($script, $status, $struck.':Dwell');
  if (!defined $struck_dwell || $struck_dwell == 0) {$struck_dwell = 1E9;}

  $j0 = 0;
  for ($i=0; $i<$num_pts; $i++) {
     if (@$timerRef[$i] > 0) {
        @$timerRef[$i] = @$timerRef[$i] / $struck_frequency;
     } else {
        @$timerRef[$i] = $struck_dwell;			#correct (replace by Dwell)
        $j0++;
        if ($j0 <= 5) { 				#warn first 5 times only
           print STDOUT '!!! '.$script.': no time data in Struck channel='.$i.' of '.$num_pts."\n";
        }
     }
  }
  if ($j0 != 0) {
     print STDOUT '!!! '.$script.': time data in Struck was missing '.$j0.' times of '.$num_pts."\n";
  }

### Correct the 1st channel
### (Struck driver bug?):
### put it back to what
### it should if the deviation
### is less than 5 microseconds:
# if (abs(@$timerRef[0]-$struck_dwell) < 0.005) {
#    print STDOUT "\n".'!!! '.$script.': correcting 1st channel of Struck timer: was='
#                                    .@$timerRef[0].' now='.$struck_dwell."\n";
#    @$timerRef[0] = $struck_dwell;
# }

  return $num_pts;
}

#=================================================================

# sub readStruckInputRaw ($struck, $struck_ext, $num_pts, $input, $finalORfly]) {
sub readStruckInputRaw ($$$$;$) {
### - returns array of raw @data[$num_pts] for a given input

  our $EZCALONG;
  my ($script, $struck, $struck_ext, $num_pts, $input, $finalORfly);
  my ($maxinputs, $struckInputName, $status, $msg);
  my ($arr_len, $current_time, $i, @data);

  $script     = 'readStruckInputRaw';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $num_pts    = shift(@_);
  $input      = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  if (!defined $EZCALONG)    {&setEZCA32or64();}
  if (!defined $finalORfly) {$finalORfly='final';}
  if (($finalORfly !~ /^final$/i) &&
      ($finalORfly !~ /^fly$/i)   &&
      ($finalORfly !~ /^flywithread$/i)) {
     print STDOUT '!!! '.$script.': incorrect flag finalORfly='.$finalORfly.'. Presuming "final"'."\n";
     $finalORfly='final';
  }
  if    ($finalORfly =~ /^final$/i)       {&abortStruck($struck);}
  elsif ($finalORfly =~ /^flywithread$/i) {&pushStruckRead($struck);}

  $maxinputs = &StruckNinputs($struck);
  if ($input < 1 || $input > $maxinputs) {
     $current_time = scalar localtime;
     $msg  = '*** '.$script.': '.$current_time."\n";
     $msg .= '*** '.$script.': Incorrect Struck input number='.$input."\n";
     $msg .= '*** '.$script.': -- must be in the 1--'.$maxinputs.' range.'."\n";
     &press_key_exit($msg);
  }

  $struckInputName = $struck.$struck_ext.$input;
### Read struck data:
  @data = ();
  ($status,@data) = Pezca::GetList($struckInputName,$EZCALONG,$num_pts);
  &report_exit ($script, $status, $struckInputName);

  $arr_len = @data;
  if ($arr_len < $num_pts) {
     print STDOUT '!!! '.$script.': Struck returned incomplete data array for input-'.$input.":\n";
     print STDOUT '    requested='.$num_pts.'pts, returned='.$arr_len.'pts.'."\n";
     for ($i=$arr_len; $i<$num_pts; $i++) {$data[$i]=0.0;}
  }

  return @data;
}

#=================================================================

# sub readStruckInput ($struck, $struck_ext, $num_pts, $input, \@timer, $finalORfly]) {
sub readStruckInput ($$$$\@;$) {
### - returns array @data[$num_pts] for a given input

  my ($script, $struck, $struck_ext, $num_pts, $input, $timerRef, $finalORfly);
  my (@timer, $nT, $i, $struck_frequency, $current_time, @data, $msg);

  $script     = 'readStruckInput';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $num_pts    = shift(@_);
  $input      = shift(@_);
  $timerRef   = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  $nT = @$timerRef;
  if ($num_pts > $nT) {
     $current_time = scalar localtime;
     $msg  = '*** '.$current_time."\n";
     $msg .= '*** '.$script.': requested npts='.$num_pts
                   .' exceeds the array length:'."\n"
                   .' @timer='.$nT."\n";
     &press_key_exit($msg);
  }
  for ($i=0; $i<$num_pts; $i++) {$timer[$i]=@$timerRef[$i];}

  $struck_frequency = &getStruckFrequency($struck);	#25MHz or 50MHz

### Read raw data:
  @data = &readStruckInputRaw ($struck, $struck_ext, $num_pts, $input, $finalORfly);

### Normalize data on the measurement time:
  if ($input != 1) {
     for ($i=0; $i<$num_pts; $i++) {
        if ($timer[$i] > 0) {$data[$i] = $data[$i] / $timer[$i];}
        else {
           $current_time = scalar localtime;
           $msg  = '*** '.$script.': '.$current_time."\n";
           $msg .= '*** '.$script.': Zero timer data in channel='.$i."\n";
           &press_key_exit($msg);
        }
     }
  } else {
     for ($i=0; $i<$num_pts; $i++) {$data[$i]=$data[$i]/$struck_frequency;}
  }
  return @data;
}

#=================================================================

# sub readStruckInputRawRef ($struck, $struck_ext, $num_pts, $input, \@data, $finalORfly]) {
sub readStruckInputRawRef ($$$$\@;$) {
### - returns $status and fills array of raw @data[$num_pts] for a given input

  our $EZCALONG;
  my ($script, $struck, $struck_ext, $num_pts, $input, $dataRef, $finalORfly);
  my ($maxinputs, $struckInputName, $status, $arr_len, $current_time, $i, $msg);

  $script     = 'readStruckInputRawRef';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $num_pts    = shift(@_);
  $input      = shift(@_);
  $dataRef    = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  if (!defined $EZCALONG)   {&setEZCA32or64();}
  if (!defined $finalORfly) {$finalORfly='final';}
  if (($finalORfly !~ /^final$/i) &&
      ($finalORfly !~ /^fly$/i)   &&
      ($finalORfly !~ /^flywithread$/i)) {
     print STDOUT '!!! '.$script.': incorrect flag finalORfly='.$finalORfly.'. Presuming "final"'."\n";
     $finalORfly='final';
  }
  if    ($finalORfly =~ /^final$/i)       {&abortStruck($struck);}
  elsif ($finalORfly =~ /^flywithread$/i) {&pushStruckRead($struck);}

  $maxinputs = &StruckNinputs($struck);
  if ($input < 1 || $input > $maxinputs) {
     $current_time = scalar localtime;
     $msg  = '*** '.$script.': '.$current_time."\n";
     $msg .= '*** '.$script.': Incorrect Struck input number='.$input."\n";
     $msg .= '*** '.$script.':   -- must be in the 1--'.$maxinputs.' range.'."\n";
     &press_key_exit($msg);
  }

  $struckInputName = $struck.$struck_ext.$input;
### Read struck data:
  @$dataRef = ();
  ($status,@$dataRef) = Pezca::GetList($struckInputName,$EZCALONG,$num_pts);
   &report_exit ($script, $status, $struckInputName);

  $arr_len = @$dataRef;
  if ($arr_len < $num_pts) {
     print STDOUT '!!! '.$script.': Struck returned incomplete data array for input-'.$input.":\n";
     print STDOUT '    requested='.$num_pts.'pts, returned='.$arr_len.'pts.'."\n";
     for ($i=$arr_len; $i<$num_pts; $i++) {@$dataRef[$i]=0.0;}
  }

  return $status;
}

#=================================================================

# sub readStruckInputRef ($struck, $struck_ext, $num_pts, $input, \@timer, \@data, $finalORfly]) {
sub readStruckInputRef ($$$$\@\@;$) {
### - returns $status and fills array @data[$num_pts] for a given input

  my ($script, $struck, $struck_ext, $num_pts);
  my ($input, $timerRef, $dataRef, $finalORfly, $msg);
  my ($struck_frequency, $current_time, $status, $i);

  $script     = 'readStruckInputRef';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $num_pts    = shift(@_);
  $input      = shift(@_);
  $timerRef   = shift(@_);
  $dataRef    = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  $struck_frequency = &getStruckFrequency($struck);	#25MHz or 50MHz

### This is checked inside readStruckInputRawRef 
# my ($maxinputs);
# $maxinputs = &StruckNinputs($struck);
# if ($input < 1 || $input > $maxinputs) {
#    $current_time = scalar localtime;
#    $msg  = '*** '.$script.': '.$current_time."\n";
#    $msg .= '*** '.$script.': Incorrect Struck input number='.$input."\n";
#    $msg .= '*** '.$script.':   -- must be in the 1--'.$maxinputs.' range.'."\n";
#    print STDOUT $msg;
# }
### Read raw data:
  $status = &readStruckInputRawRef ($struck, $struck_ext, $num_pts, $input, $dataRef, $finalORfly);

### Normalize data on the measurement time:
  if ($input != 1) {
     for ($i=0; $i<$num_pts; $i++) {
        if (@$timerRef[$i] > 0) {@$dataRef[$i] = @$dataRef[$i] / @$timerRef[$i];}
        else {
           $current_time = scalar localtime;
           $msg  = '*** '.$script.': '.$current_time."\n";
           $msg .= '*** '.$script.': Zero timer data in channel='.$i."\n";
           &press_key_exit($msg);
        }
     }
  } else {
     for ($i=0; $i<$num_pts; $i++) {@$dataRef[$i]=@$dataRef[$i]/$struck_frequency;}
  }
  return $status;
}

#=================================================================

# sub readStruck ($struck, $struck_ext, $num_pts, $ninputs, \@inputs, $finalORfly]) {
sub readStruck ($$$$\@;$) {
### - returns GLOBAL: @StruckData${input}[$num_pts], @StruckTimer[$num_pts]

  our @StruckTimer;
  my ($script, $struck, $struck_ext, $num_pts, $ninputs, $inputsRef, $finalORfly);
  my ($nI, $input, $name, $read_pts, $i, $msg, $current_time);

  $script     = 'readStruck';
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $struck_ext = shift(@_);
  $num_pts    = shift(@_);
  $ninputs    = shift(@_);
  $inputsRef  = shift(@_);
  $finalORfly = shift(@_);	#final/fly/flywithread; default is final, which aborts Struck

  $nI = @$inputsRef;
  if ($ninputs > $nI) {
     $current_time = scalar localtime;
     $msg  = '*** '.$current_time."\n";
     $msg .= '*** '.$script.': requested Struck inputs='.$ninputs
                   .' exceed the array length:'."\n"
                   .' @inputs='.$nI."\n";
     &press_key_exit($msg);
  }

### Read Struck timer:
  $read_pts = $num_pts;
  ($num_pts, @StruckTimer) = &readStruckTimer ($struck, $struck_ext, $read_pts, $finalORfly);
  if ($num_pts != $read_pts) {
     print STDOUT '!!! '.$script.': requested to read='.$read_pts.'pts, found='.$num_pts.'pts'."\n";
  }

### Read struck data:
  for ($i=0; $i<$ninputs; $i++) {
     $input = @$inputsRef[$i];
     $name = 'StruckData'.$input;
     no strict 'refs';  #-----------------------------------------------------------------------#
     @$name = &readStruckInput ($struck, $struck_ext, $num_pts, $input, \@StruckTimer,'fly');   #
     use strict 'refs'; #-----------------------------------------------------------------------#
  }
}

#=================================================================

# sub printStruckInputs ($LUN, $struck, $ninputs, @inputs) {
sub  printStruckInputs ($$$@) {

  my ($script, $LUN, $struck, $ninputs, $msg, $s);
  my ($inputs, $maxinputs, $current_time, $i);

  $script  = 'printStruckInputs';
  $LUN     = shift(@_);
  $struck  = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $ninputs = shift(@_);

  $maxinputs = &StruckNinputs($struck);

  if ($ninputs > 1) {$s='s';} else {$s='';}
  printf $LUN '# Struck scaler [%s] input%s: ',$struck,$s;
  for ($i=0; $i<$ninputs; $i++) {
     $inputs = shift(@_);
     if ($inputs < 1 || $inputs > $maxinputs) {
        $current_time = scalar localtime;
        $msg  = '*** '.$script.': '.$current_time."\n";
        $msg .= '*** '.$script.': Incorrect Struck input number='.$inputs."\n";
        $msg .= '*** '.$script.':   -- must be in range 1--'.$maxinputs."\n";
        &press_key_exit($msg);
     }
     print $LUN ' '.$inputs.' ';
  }
  print $LUN "\n";
}

#=================================================================

# sub getStruckOffsets ($file, $struck) {
# sub getStruckOffsets (\$offsets, $struck) {		#another possibility when offsets are already known
sub getStruckOffsets ($$) {		# - returns @StruckOffset[1:$maxninputs]

  my ($script, $file, $offref, $struck, $maxinputs, $msg);
  my ($age_hours, $nOffsets, $current_time, $i, @offsets);

  $script = 'getStruckOffsets';
  $file   = shift(@_);
  $struck = shift(@_);	if ($struck =~ /:$/) {chop($struck);}

  $maxinputs = &StruckNinputs($struck);
  for ($i=0; $i<$maxinputs; $i++) {$offsets[$i]=0.0;}

  if (!defined $file) {$file = '';}		#if file name is unspecified

  if (length($file) == 0) {			#if file name is NOT specified
### File name is NOT specified:
     print STDOUT '!!! '.$script.': Offsets file is not specified. Offsets are set to 0.'."\n";
  }
  elsif ($file =~ /^ARRAY\(0x[\da-fA-F]{1,8}\)$/) {		#this is not a file, but a reference to the offsets array
     $offref = $file;						#i.e. something like: "ARRAY(0x182bdac)"
     @offsets = @$offref;
  }
  else {                     			#filename is specified:
     if (-f $file) {				#if file exists:
        open (BKG, '< '.$file) || die 'Cannot open file='.$file."\n";
        chomp (@offsets = <BKG>);
        close(BKG);
        $age_hours = 24 * (-M $file);
        if ($age_hours > 2.) {
           printf STDOUT '!!! '.$script.': offsets file is %.1f hour(s) old.'."\n", $age_hours;
        }
### Verify the number of inputs:
        $nOffsets = @offsets;
        if ($nOffsets != $maxinputs) {
           $current_time = scalar localtime;
           $msg  = '*** '.$script.': '.$current_time."\n";
           $msg .= '*** '.$script.': Number of offsets='.$nOffsets.' in file'."\n";
           $msg .= '*** '.$script.': '.$file."\n";
           $msg .= '*** '.$script.': does not match struck inputs='.$maxinputs."\n";
           &press_key_exit($msg);
        }
     } else {                   			#filename is specified, but file does NOT exist
        print STDOUT '!!! '.$script.':'."\n";
        print STDOUT '   Offsets file="'.$file.'" is not found.'."\n";
        print STDOUT '   Offsets are set to 0.'."\n";
        for ($i=0; $i<$maxinputs; $i++) {$offsets[$i] = 0.0;}
     }
  }
  return @offsets;
}

#=================================================================

# sub printStruckOffsets($LUN, $file, $struck, \@offsets, $ninputs, \@inputs) {
sub printStruckOffsets($$$\@$\@) {

  my ($script, $LUN, $file, $struck, $offsetsRef, $ninputs, $inputsRef);
  my ($nO, $nI, @offsets, $input, $maxinputs, $i, $y_, $current_time, $msg);

  $script     = 'printStruckOffsets';
  $LUN        = shift(@_);
  $file       = shift(@_);
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $offsetsRef = shift(@_);
  $ninputs    = shift(@_);
  $inputsRef  = shift(@_);

  $nO = @$offsetsRef;
  $nI = @$inputsRef;
  $maxinputs = &StruckNinputs($struck);
  if ($ninputs > $nI) {
     $current_time = scalar localtime;
     $msg  = '*** '.$current_time."\n";
     $msg .= '*** '.$script.': requested Struck inputs='.$ninputs
                   .' exceed the array length:'."\n"
                   .' @inputs='.$nI."\n";
     &press_key_exit($msg);
  }
  if ($maxinputs > $nO) {
     $current_time = scalar localtime;
     $msg  = '*** '.$current_time."\n";
     $msg .= '*** '.$script.': requested Struck offsets='.$maxinputs
                   .' exceed the array length:'."\n"
                   .' @offsets='.$nO."\n";
     &press_key_exit($msg);
  }

  for ($i=0; $i<$maxinputs; $i++) {$offsets[$i] = @$offsetsRef[$i];}

  if (!defined $file) {$file = '';}		#if file name is unspecified
  if (length($file) != 0) {			#if file name is specified
     print $LUN '# Offsets from '.$file.': ';
  } else {
     print $LUN '# Offsets (no BKG file): ';
  }
  for ($i=0; $i<$ninputs; $i++) {
     $input = @$inputsRef[$i];
     if ($input < 1 || $input > $maxinputs) {
        $current_time = scalar localtime;
        $msg  = '*** '.$script.': '.$current_time."\n";
        $msg .= '*** '.$script.': Incorrect Struck input number='.$input."\n";
        $msg .= '*** '.$script.':   -- must be in range 1--'.$maxinputs."\n";
        &press_key_exit($msg);
     }
     $y_ = $offsets[$input-1];
     $y_ =~ s/\n//g;
     print $LUN ' '.$y_;
  }
  print $LUN "\n";
}

#=================================================================

# sub printStruck_1pti ($LUN, $struck, \@offsets, $pt, $input, \@data) {
sub printStruck_1pti ($$\@$$\@) {

  my ($script, $LUN, $struck, $offsetsRef, $pt, $input, $dataRef);
  my ($nOff, $nDat, $maxinputs, $y_, $y_bg, $y_corr);
  my ($out, $current_time, $msg);

  $script     = 'printStruck_1pti';
  $LUN        = shift(@_);
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $offsetsRef = shift(@_);
  $pt         = shift(@_);
  $input      = shift(@_);
  $dataRef    = shift(@_);

  $nOff = @$offsetsRef;
  $nDat = @$dataRef;
  $maxinputs = &StruckNinputs($struck);

  if ($maxinputs > $nOff) {
     $current_time = scalar localtime;
     $msg  = '*** '.$current_time."\n";
     $msg .= '*** '.$script.': requested offsets number='.$maxinputs
                   .' exceeds the array length:'."\n"
                   .' @Offsets='.$nOff."\n";
     &press_key_exit($msg);
  }
  if ($pt > $nDat-1) {
     $current_time = scalar localtime;
     $msg  = '*** '.$current_time."\n";
     $msg .= '*** '.$script.': requested data point number='.$pt
                   .' exceeds the array length:'."\n"
                   .' @data='.$nDat."\n";
     &press_key_exit($msg);
  }
  if ($input < 1 || $input > $maxinputs) {
     $current_time = scalar localtime;
     $msg  = '*** '.$script.': '.$current_time."\n";
     $msg .= '*** '.$script.': Incorrect Struck input number='.$input."\n";
     $msg .= '*** '.$script.':   -- must be in range 1--'.$maxinputs."\n";
     &press_key_exit($msg);
  }
  $y_     = @$dataRef[$pt];
  $y_bg   = @$offsetsRef[$input-1];
  $y_corr = $y_ - $y_bg;
  $out = sprintf '%12.5g', $y_corr;
# $out =~ s/ //g;
  $out =~ s/E /E\+/gi;
  print $LUN ', '.$out;
}

#=================================================================

# sub printStruck_1pt ($LUN, $struck, \@offsets, $pt, $ninputs, \@inputs) {
sub printStruck_1pt ($$\@$$@) {
### - also uses GLOBAL StruckData${input}($num_pts)...

  my ($script, $LUN, $struck, $offsetsRef, $pt, $ninputs, $inputsRef, $input);
  my ($nInp, $name, $current_time, $msg, $i);

  $script     = 'printStruck_1pt';
  $LUN        = shift(@_);
  $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
  $offsetsRef = shift(@_);
  $pt         = shift(@_);                                 #this counts from 0
  $ninputs    = shift(@_);
  $inputsRef  = shift(@_);

  $nInp = @$inputsRef;
  if ($ninputs > $nInp) {
     $current_time = scalar localtime;
     $msg  = '*** '.$current_time."\n";
     $msg .= '*** '.$script.': requested number of input='.$ninputs
                   .' exceeds the array length:'."\n"
                   .' @inputs='.$nInp."\n";
     &press_key_exit($msg);
  }

  for ($i=0; $i<$ninputs; $i++) {
     $input = @$inputsRef[$i];
     $name  = 'StruckData'.$input;
     &printStruck_1pti($LUN, $struck, $offsetsRef, $pt, $input, \@$name);
  }
}


#=================================================================

# sub openStruckMedm($struck, $struck_ext, $inputNo) {
  sub openStruckMedm($$$) {

### Open MEDM screen for given Struck mumber
### This currently works for UNIX only

  my ($script, $struck, $struck_ext, $inputNo);
  my ($status, $adl, $macro, $refdir);

  $script = 'openStruckMedm';

  if ($^O !~ /Win/i) { 							#if UNIX
     $struck     = shift(@_);	if ($struck =~ /:$/) {chop($struck);}
     $struck_ext = shift(@_);
     $inputNo    = shift(@_);

     $adl = 'gmcaApp/adl/mca_small.adl';

     if    (defined $ENV{PEZCA})     {$refdir = $ENV{PEZCA};}
     elsif (defined $ENV{FASTSCANS}) {$refdir = $ENV{FASTSCANS};}
     else                            {$refdir = './';}
     $refdir =~ s/\\/\//g;						#change \ to /
     $refdir =~ s/\/$//;						#remove end /
     $refdir =~ s/\/bin$//;						#remove end bin/
     $refdir =~ s/\/fastscans$//;					#remove end fastscans/
     $refdir =~ s/\/pezca$//;						#remove end pezca/
     $adl    = $refdir.'/'.$adl;
     $macro  = 'P='.$struck.',M='.$struck_ext.$inputNo;
     $status = &open_medm($adl,$macro);
  }
}


#=================================================================

# sub HDflyScan($assy,$axis,$begin,$end,$npts,$scanTime,$mtrOut,$cts2egu,
#               $formula,$struckPs,$struckNg,$medm,$fileBkg,$ninputs,\@inputs,
#               [$scaler,$afterScan,$checkOverload,$postarrayPV,$post_interval]) {
  sub HDflyScan($$$$$$$$$$$$$$\@;$$$$$) {

### Returns implicitly: ($status,$actPts)
### Returns GLOBAL:      @x1HD, @timeHD, @y1HD, ... @y{$ninputs}HD, @StruckOffsets

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext, @StruckOffsets);
  our ($HDSCAN_FLAGS);		#reference to a hash with various flags for HDSCAN
  our (@x1HD, @timeHD);		#GLOBAL RESULTs: positions and timeline
  my ($script, $assy, $axis, $begin, $end, $npts, $scanTime, $mtrOut, $cts2egu);
  my ($formula, $struckPs, $struckNg, $medm, $fileBkg, $ninputs, $inputsRef);
  my ($scaler ,$afterScan, $checkOverload, $postarrayPV, $post_interval);
  my ($struck, $timePPT, $dwell, $actPts, $status, $status1, $message, @data, $name);
  my ($beginAct, $axisIni, $i, $j, $i_plot, $name_plot, $scan_index);
  my ($OVERLOAD, $overload_detected, $threshold, @post_array, $narr, $min_val);

  $script        = 'HDflyScan';
  $assy          = shift(@_);	# assembly to be scanned
  $axis          = shift(@_);	# particular axis in the assembly to be scanned
  $begin         = shift(@_);	# start scan position
  $end           = shift(@_);	# end scan position
  $npts          = shift(@_);	# number of points on fast scan
  $scanTime      = shift(@_);	# scan time in seconds
  $mtrOut        = shift(@_);	# PMAC motor No that outputs to Struck (usually 32)
  $cts2egu       = shift(@_);	# count to engineering units scale factor for $axis
  $formula       = shift(@_);	# formula linking axis with mtrOut (e.g.: .5x+.5y)
  $struckPs      = shift(@_);	# struck input collecting positive direction pulses from $mtrOut
  $struckNg      = shift(@_);	# struck input collecting negative direction pulses from $mtrOut
  $medm          = shift(@_);	# 1=open MEDM, 0=not
  $fileBkg       = shift(@_);	# struck background file ('auto' for automatic, '' for none)
  $ninputs       = shift(@_);   #number of Struck inputs
  $inputsRef     = shift(@_);	#reference to Struck inputs array
  $scaler        = shift(@_);	#name of Struck as Joerger
  $afterScan     = shift(@_);	#0=stay 1=startpos 2=priorpos 3=maxpos 4,14=peakpos(noisy) 5,15=peakpos(smooth)
                                #6=minpos 7,17=valleypos(noisy) 8,18=valleypos(smooth) 10,110=gaussian 11,111=lorentzian
  $checkOverload = shift(@_);
  $postarrayPV   = shift(@_);   #array for posting partial & final data to JBluIce
  $post_interval = shift(@_);

  if (!defined $fileBkg)       {$fileBkg = '';}
  if (!defined $afterScan)     {$afterScan=0;}		#stay
  if (!defined $post_interval) {$post_interval = 2;}	#post interval in seconds
  if (!defined $checkOverload) {$checkOverload = 0;}

  if (!defined $HDSCAN_FLAGS->{'has run'})              {$HDSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter close'})   {$HDSCAN_FLAGS->{'skip shutter close'}   = 0;}

  if ($HDSCAN_FLAGS->{'has run'} == 0) {
     $HDSCAN_FLAGS->{'has read mtrList'} = 1;
     $HDSCAN_FLAGS->{'cts2egu'}          = $cts2egu;
     $HDSCAN_FLAGS->{'formula'}          = $formula;
     $HDSCAN_FLAGS->{'mtrOut'}           = $mtrOut;
     $HDSCAN_FLAGS->{'struckPs'}         = $struckPs;
     $HDSCAN_FLAGS->{'struckNg'}         = $struckNg;
  }

  $scan_index        = 0;                   	#scan index to save with the post (e.g. 11-vertical, 21-horizontal)
  $i_plot            = 1;			#the main plot = 1st input ($inputs[0])
  $name_plot         = 'y'.$i_plot.'HD';
  $threshold         = 0.95E+6;	                #threshold intensity for detecting overload (2013)
  $OVERLOAD          = 100000;	                #scan failure flag when overload detected
  $overload_detected = 0;	                #overload not detected yet

# print STDOUT 'Enterting '.$script."\n";
  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }
  if (defined $shutter) {
     if ($HDSCAN_FLAGS->{'skip shutter close'} == 0) {
#       print STDOUT $script.': Closing shutter '.$shutter."\n";
        &closeShutter ($shutter,$shutter_readback);
     }
  }

  if ($fileBkg =~ /^auto$/i)   {$fileBkg = 'struck_Bkg.dat';}
### Read the scaler background offsets:
  if (!defined $StruckOffsets[0]) {			    #skip reading offsets if they were read in main
     if (length($fileBkg) > 0) {			    #if file name is specified
        print STDOUT $script.': Reading Struck scaler '.$struck.' offsets from '.$fileBkg."\n";
     }
     @StruckOffsets = &getStruckOffsets($fileBkg,$struck);  #we make it GLOBAL to access from main
  }

  $timePPT = $scanTime / $npts;
  printf STDOUT $script.': Initializing. ScanTime=%.2fs,  Npts=%d, Dwell=%.2fs'."\n",$scanTime,$npts,$timePPT;

  if ($medm && $HDSCAN_FLAGS->{'has run'} == 0) {
     for ($i=0; $i<$ninputs; $i++) {&openStruckMedm($struck, $struck_ext, @$inputsRef[$i]);}
  }

  if ($HDSCAN_FLAGS->{'has run'} == 0) {
     ### Record initial axis position:
     $axisIni = &getAxisAct($axis,\$status);
     $HDSCAN_FLAGS->{'inipos'} = $axisIni;
  } else {
     if (defined $HDSCAN_FLAGS->{'inipos'}) {$axisIni = $HDSCAN_FLAGS->{'inipos'};}
     else                                   {$axisIni = $begin;}
  }

  ($status,$message,$beginAct,$cts2egu,
   $mtrOut,$struckPs,$struckNg,$dwell) = &HDflyScan_setup($assy,$axis,$begin,$end,$npts,$timePPT,$scaler);
  if ($status) {
     print STDOUT "\n".$script.': error='.$status.' from HDflyScan_setup.'."\n";
     print STDOUT "\n",$message,"\n";
     return $status, 0;
  }

### Wait for scan to finish (and optionally post progress arrays):
  ($status,$message) = &HDflyScan_wait($assy,$axis,$mtrOut,$dwell,$beginAct,$end,$npts,
                                       $inputsRef,$postarrayPV,$post_interval,$i_plot,
                                       $cts2egu,$struckPs,$struckNg,$checkOverload,$threshold);
  if ($status) {
     print STDOUT "\n".$script.': error='.$status.' from HDflyScan_wait.'."\n";
     print STDOUT "\n",$message,"\n";
#    return $status, 0;					#we cannot return here - first restore everything in HDflyScan_read
     if ($status == $OVERLOAD) {
        print STDOUT '!!! '.$script.': intensity overload detected. Returning to '.$axis.' start position for re-scanning...'."\n\n";
        &eraseStruck ($struck);
        &runMotor($assy,$axis,$begin,'forceAuto');
        return $OVERLOAD, 0;
     }
  }
  $status1 = $status;

### Read scan data (fill \@data=(@positions,@timeline,@inputs)):
  ($status,$actPts) = &HDflyScan_read ($axis,$beginAct,$cts2egu,$struckPs,$struckNg,
                                       $inputsRef,\@StruckOffsets,\@data,'final');
  if ($status) {
     print STDOUT "\n".$script.': error='.$status.' from HDflyScan_read.'."\n";
     return $status, 0;
  }
  elsif ($status1) {
     return $status1, 0;
  }

  @x1HD = ();						#GLOBAL RESULT: positions
  @timeHD = ();						#GLOBAL RESULT: timeline
  for ($j=0; $j<$actPts; $j++) {
     $x1HD[$j]   = $data[0][$j];
     $timeHD[$j] = $data[1][$j];
  }
  no strict 'refs';	#-----------------------#
  for ($i=1; $i<=$ninputs; $i++) {
     $name  = 'y'.$i.'HD';
     @$name = ();                               	#GLOBAL RESULT: data for input-$i
     for ($j=0; $j<$actPts; $j++) {
        @$name[$j] = $data[$i+1][$j];
        if (@$name[$j] >= $threshold) {$overload_detected++;}
     }
     if (defined $postarrayPV && $i == $i_plot) {
        $min_val = &minvalRef(\@$name);
        @post_array = ($scan_index,$actPts,$begin,$end,@x1HD);
        for ($j=0; $j<$actPts; $j++) {push @post_array,(@$name[$j]-$min_val);}
        $narr = @post_array;
        if ($narr < 2048) {
           $status = Pezca::PutList($postarrayPV, 'FLOAT', @post_array);
           if ($status) {
              &report_only($script, $status, $postarrayPV);
              print STDOUT '!!! '.$script.': error posting '.$narr.'-element array into PV='
                          .$postarrayPV.'. Further posting will be disabled.'."\n\n";
              undef $postarrayPV;
           }
        } else {
           print STDOUT '!!! '.$script.': cannot post '.$narr.'-element array into PV='
                       .$postarrayPV.'. Array is too long. Further posting will be disabled.'."\n\n";
           undef $postarrayPV;
        }
     }
  }
  use strict 'refs';    #-----------------------#
  $HDSCAN_FLAGS->{'has run'} = 1;
  print STDOUT $script.': Scan data retrieved!'."\n";

  if ($overload_detected > 0) {
     print STDOUT '!!! '.$script.': Intensity overload detected '.$overload_detected.' times.'."\n";
     if ($checkOverload) {
        print STDOUT '!!! '.$script.': intensity overload detected. Returning to '.$axis.' start position for re-scanning...'."\n\n";
        &runMotor($assy,$axis,$axisIni,'forceAuto');
        return $OVERLOAD,$actPts;
     } else {
        print STDOUT '!!! '.$script.':  Ignoring intensity overload.'."\n\n";
     }
  }
  
### -----------------------------------------------------
### afterScan:
### (see motor_subs.pl)
  no strict 'refs';  #-----------------------------------------------------------------------#
  &axis_afterScan($afterScan,\@x1HD,\@$name_plot,$begin,$end,$axisIni,$axis,$assy);          #
  use strict 'refs'; #-----------------------------------------------------------------------#
  print STDOUT '..'.$script.' finished!'."\n";
  if    ($status)                {return $status,$actPts;}
  elsif ($overload_detected > 0) {return $OVERLOAD,$actPts;}
  else                           {return $status,$actPts;}
}	##HDflyScan


#=================================================================

# ($status,$message,[$beginAct,$cts2egu,$mtrOut,$struckPs,$struckNg,$dwell]) =
#                         &HDflyScan_setup($assy,$axis,$begin,$end,$npts,$timePPT,[$scaler]);
  sub HDflyScan_setup($$$$$$;$) {

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext);
  our ($IGNORE_ERRORS, $report);
  our ($HDSCAN_FLAGS);		#reference to a hash with various flags for HDSCAN
  our ($EXTRA_MOTION_TIME);	#extra allowance for tracking because of virt.motor
  my ($script, $assy, $axis, $begin, $end, $npts, $timePPT, $delayTime);
  my ($cts2egu, $mtrOut, $formula, $struckPs, $struckNg, $dwell, $step);
  my ($struck, $int_ext, $iserased, $status, $message, $justLinked, $need2move);
  my ($scanRange, $l_speed, $h_speed, $formulaOffset, $beginAct, $scaler);
  my ($scanTime, $extra_motion_time, $ignore_errors, $estimPts, $maxPts);
  my ($DWELLTIMEMIN, $DWELLTIMEMAX, $STRUCK_EXTRA_TIME, $actpos);

  $script   = 'HDflyScan_setup';
  $assy     = shift(@_);	#assembly to be scanned
  $axis     = shift(@_);	#particular axis in the assembly to be scanned
  $begin    = shift(@_);	#start scan position
  $end      = shift(@_);	#end scan position
  $npts     = shift(@_);	#number of points on fast scan
  $timePPT  = shift(@_);	#desired scan time-per-point in seconds
  $scaler   = shift(@_);	#name of Struck as Joerger

# print STDOUT 'Enterting '.$script."\n";
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if (!defined $HDSCAN_FLAGS->{'has run'})              {$HDSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'has read mtrList'})     {$HDSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter close'})   {$HDSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter open'})    {$HDSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip calc speed'})      {$HDSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set speed'})       {$HDSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set slew'})        {$HDSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore speed'})   {$HDSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore mprg'})    {$HDSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip mtr link'})        {$HDSCAN_FLAGS->{'skip mtr link'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip mtr unlink'})      {$HDSCAN_FLAGS->{'skip mtr unlink'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip move to start'})   {$HDSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'return to start'})      {$HDSCAN_FLAGS->{'return to start'}      = 0;}
#
  if (!defined $HDSCAN_FLAGS->{'speed'})                {$HDSCAN_FLAGS->{'speed'}                = 0;}
  if (!defined $HDSCAN_FLAGS->{'cts2egu'})              {$HDSCAN_FLAGS->{'cts2egu'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'formula'})              {$HDSCAN_FLAGS->{'formula'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'mtrOut'})               {$HDSCAN_FLAGS->{'mtrOut'}               = 0;}
  if (!defined $HDSCAN_FLAGS->{'struckPs'})             {$HDSCAN_FLAGS->{'struckPs'}             = 0;}
  if (!defined $HDSCAN_FLAGS->{'struckNg'})             {$HDSCAN_FLAGS->{'struckNg'}             = 0;}

  if ($HDSCAN_FLAGS->{'has run'} == 0 && $HDSCAN_FLAGS->{'has read mtrList'} == 0) {
     ($cts2egu,$formula,$mtrOut,$struckPs,$struckNg) = &searchMotorList($assy,$axis,'harsync');
     if (!defined $cts2egu) {
        $message = '*** '.$script.': No HD-synchronized data in motorList for assy='.$assy.', axis='.$axis;
        return 1, $message;
     }
     $HDSCAN_FLAGS->{'cts2egu'}  = $cts2egu;
     $HDSCAN_FLAGS->{'formula'}  = $formula;
     $HDSCAN_FLAGS->{'mtrOut'}   = $mtrOut;
     $HDSCAN_FLAGS->{'struckPs'} = $struckPs;
     $HDSCAN_FLAGS->{'struckNg'} = $struckNg;
  } else {
     $cts2egu  = $HDSCAN_FLAGS->{'cts2egu'};
     $formula  = $HDSCAN_FLAGS->{'formula'};
     $mtrOut   = $HDSCAN_FLAGS->{'mtrOut'};
     $struckPs = $HDSCAN_FLAGS->{'struckPs'};
     $struckNg = $HDSCAN_FLAGS->{'struckNg'};
  }

  $HDSCAN_FLAGS->{'assy'}    = $assy;
# $HDSCAN_FLAGS->{'axis'}    = $axis;
# $HDSCAN_FLAGS->{'npts'}    = $npts;
  $HDSCAN_FLAGS->{'begin'}   = $begin;
  $HDSCAN_FLAGS->{'end'}     = $end;
# $HDSCAN_FLAGS->{'timePPT'} = $timePPT;

  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }
  if (defined $shutter) {
     if ($HDSCAN_FLAGS->{'skip shutter close'} == 0) {
#       print STDOUT $script.': Closing shutter='.$shutter."\n";
        $status = &closeShutter ($shutter,$shutter_readback);
        if ($status) {return 1,'*** '.$script.': Error closing fast shutter';}
     }
  }

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}

  ($DWELLTIMEMIN,$DWELLTIMEMAX) = &getStruckDwellMinMax($struck);
  $STRUCK_EXTRA_TIME = 3;				#time for Struck to count longer than motion

  &abortAssy($assy);
  $int_ext = 0;						#0=internal advance, 1=external

  if ($npts < 3) {
     $message = '*** '.$script.': incorrect scan npts='.$npts;
     print STDOUT $message,"\n";
     return 1, $message;
  }

  $dwell = $timePPT;
  if    ($dwell < $DWELLTIMEMIN) {
     printf STDOUT '!!! '.$script.': timePPT=%.3fs is below Struck limit=%.3fs'."\n", $dwell, $DWELLTIMEMIN;
     $dwell = $DWELLTIMEMIN;
  }
  elsif ($dwell > $DWELLTIMEMAX) {
     printf STDOUT '!!! '.$script.': timePPT=%.3fs exceeds Struck limit=%.3fs'."\n", $dwell, $DWELLTIMEMAX;
     $dwell = $DWELLTIMEMAX;
  }
  $timePPT  = $dwell;
  $scanTime = $npts * $dwell;

#-----------------------------------------------------------------
  if ($HDSCAN_FLAGS->{'has run'} == 0 || $HDSCAN_FLAGS->{'skip calc speed'} == 0) {
### Calculate scan speed for the motor:
#    print STDOUT $script.': Calculating slew-mode speed for axis='.$axis."\n";
     $ignore_errors = $IGNORE_ERRORS;
     $IGNORE_ERRORS = 1;
     $scanRange = abs ($end - $begin);
     ($l_speed, $h_speed) = &calcAxisSpeed ($assy, $axis, $scanRange, $scanTime);
     $IGNORE_ERRORS = $ignore_errors;
     if ($l_speed >= $h_speed) {				#speed was too high
        $l_speed  = $h_speed;
        $scanTime = abs($scanRange)/$l_speed;
        $dwell = $scanTime / $npts;
        if    ($dwell < $DWELLTIMEMIN) {
           $message = sprintf '!!! '.$script.': too low timePPT=%.3fs required with npts='.$npts, $dwell;
           print STDOUT $message,"\n";
#          return 1, $message;
           $dwell = $DWELLTIMEMIN;
        }
        elsif ($dwell > $DWELLTIMEMAX) {
           $message = sprintf '!!! '.$script.': too high timePPT=%.3fs required with npts='.$npts, $dwell;
           print STDOUT $message,"\n";
#          return 1, $message;
           $dwell = $DWELLTIMEMAX;
        }
        if ($dwell != $timePPT) {
           printf STDOUT '!!! '.$script.': timePPT changed from %.3fs to %.3fs'."\n", $timePPT, $dwell;
        }
     }
     $HDSCAN_FLAGS->{'speed'} = $l_speed;
     $HDSCAN_FLAGS->{'dwell'} = $dwell;
  } else {
#    printf STDOUT '!!! '.$script.': Using '.$axis.' slew-mode speed=%.3f from previous scan'."\n", $HDSCAN_FLAGS->{'speed'};
#    print  STDOUT '!!! '.$script.': (This presumes same range and time)'."\n";
     $l_speed = $HDSCAN_FLAGS->{'speed'};
     $dwell   = $HDSCAN_FLAGS->{'dwell'};
  }
  $timePPT  = $dwell;
  $scanTime = $npts * $dwell;

  $estimPts = $npts + int($STRUCK_EXTRA_TIME/$dwell + 0.5);
  $maxPts   = &getStruckNmax($struck, $struck_ext);
  if ($estimPts > $maxPts) {
     print STDOUT "\n".'!!! '.$script.': Estimated pts='.$estimPts.' exceeds'."\n".
                    'Struck buffer='.$maxPts.'. Please, increase step size,'."\n".
                    'Otherwice scan may be aborted before motor arrives to the end.'."\n";
     if ($npts > $maxPts) {
        $message =  '*** '.$script.': Required pts='.$npts.' exceeds'."\n".
                    'Struck buffer='.$maxPts.'. Please, increase step size.';
        print STDOUT $message,"\n";
        return 1, $message;
     }
  }

### Set Struck scaler:
  if ($HDSCAN_FLAGS->{'has run'} == 0) {
#    printf STDOUT $script.': Setting up Struck scaler '.$struck.' (dwell=.2fs)'."\n",$dwell;
     &setStruck ($struck, $struck_ext, $int_ext, $dwell, $maxPts, $scaler);
  }

### Erase struck:
  if ($HDSCAN_FLAGS->{'has run'} == 0) {			#this requires Struck to be erased at at end of prev.scan
#    print STDOUT $script.': Erasing Struck scaler '.$struck."\n";
     &eraseStruck ($struck);
  }
  $iserased = 1;

### Set motor:
  if ($HDSCAN_FLAGS->{'has run'} == 0) {
#    print STDOUT $script.': Setting up motor '.$axis."\n";
     &configAssy ($assy);
  }

  if ($HDSCAN_FLAGS->{'has run'} == 0 || $HDSCAN_FLAGS->{'skip mtr link'} == 0) {
### Link mtrOut to the assembly:
     &unlinkMtrOut($assy,$mtrOut);				#only needed with determineFormulaOffset2
     $formulaOffset = &determineFormulaOffset2($assy, $axis, $formula, $cts2egu);
     if    ($formulaOffset == 0) {&linkMtrOut ($assy, $mtrOut, $formula);}
     elsif ($formulaOffset > 0)  {&linkMtrOut ($assy, $mtrOut, $formula.'+'.$formulaOffset);}
     elsif ($formulaOffset < 0)  {&linkMtrOut ($assy, $mtrOut, $formula  .  $formulaOffset);}
     $justLinked = 1;
  } else {
     $justLinked = 0;
  }
#-----------------------------------------------------------------
  $need2move = 0;
  if ($HDSCAN_FLAGS->{'skip move to start'} && $justLinked == 0) {
     $beginAct = &getAxisAct ($axis);
     $step     = ($begin - $end) / $npts;
     if (abs($beginAct - $begin) > 0.5*abs($step)) {$need2move=1;}
  }
  if ($HDSCAN_FLAGS->{'skip move to start'} == 0 || $justLinked || $need2move) {
### Move motor to the starting position:
     printf STDOUT $script.': Moving motor '.$axis.' to the starting position=%.3f'."\n",$begin;
### This is to recover speed from previouse unclean exit:
     if ($HDSCAN_FLAGS->{'has run'} == 0) {&restoreAxis ($assy, $axis);}	#restore high-speed & motion prg before start
### Increase expected motion time due to linked motor#32:
     if (defined $EXTRA_MOTION_TIME) {$extra_motion_time = $EXTRA_MOTION_TIME;}
     $EXTRA_MOTION_TIME = 600;
     $status = &runMotor($assy, $axis, $begin);
### Restore expected motion time to normal from local cache:
     if (defined $extra_motion_time) {$EXTRA_MOTION_TIME = $extra_motion_time;}
     else                            {undef $EXTRA_MOTION_TIME;}
     $actpos = &getAxisAct($axis);
     if ($status) {
        $message = sprintf '*** '.$script.': failed moving '.$axis.' to the starting position=%.3f: act=%.3f'."\n",$begin,$actpos;
        print STDOUT $message,"\n";
        return 1, $message;
     } else {
        printf STDOUT $script.': motor '.$axis.' arrived at the starting position=%.3f (act=%.3f)'."\n",$begin,$actpos;
     }
  }

### Setting slow speed for scanning:
# print STDOUT $script.': Setting up motor='.$axis.' speed to slew (speed='.$l_speed.')'."\n";
  if ($HDSCAN_FLAGS->{'has run'} == 0) {
        print STDOUT $script.': setting scanning speed & slew motion prg'."\n";
        &setAxisSlewScan ($assy, $axis, $l_speed);			#setMotionPrg+setAxisSpeed
  } else {
     if ($HDSCAN_FLAGS->{'skip set slew'} == 0 && $HDSCAN_FLAGS->{'skip set speed'} == 0) {
        print STDOUT $script.': setting scanning speed & slew motion prg'."\n";
        &setAxisSlewScan ($assy, $axis, $l_speed);			#setMotionPrg+setAxisSpeed
     }
     elsif ($HDSCAN_FLAGS->{'skip set slew'} && $HDSCAN_FLAGS->{'skip set speed'} == 0) {
        print STDOUT $script.': setting scanning speed only, but presuming slew motion prg'."\n";
        &setAxisSpeed ($assy, $axis, $l_speed); 		#this is faster (no motion prg change)
     }
     elsif ($HDSCAN_FLAGS->{'skip set slew'} == 0 && $HDSCAN_FLAGS->{'skip set speed'}) {
        print STDOUT $script.': setting slew motion prg, but presuming scanning speed is set'."\n";
        &setMotionPrg($assy, 1); 				#this is faster (no speed change): 1=slew
     }
  }

### Setting monitors on motors motion and Struck:
  if ($HDSCAN_FLAGS->{'has run'} == 0 ) {
     $status = &addCaMonitor($struck.':Acquiring');
#    $status = &setAssyMonitors($assy);			#this was set before in runMotor
#    $status = &setAxisMonitors($axis);			#this was set before in runMotor
  }

### Read motor position at startup:
  $beginAct = &getAxisAct ($axis);

#-----------------------------------------------------------------
  if (defined $shutter) {
     if ($HDSCAN_FLAGS->{'skip shutter open'} == 0) {
#       print STDOUT $script.': Opening shutter '.$shutter."\n";
        $status = &openShutter ($shutter,$shutter_readback);
        if ($status) {return 1,'*** '.$script.': Error opening fast shutter';}
     }
  }

### Start Struck:
# print STDOUT $script.': Starting On-the-fly scan of '.$axis.' with '.$struck.'...'."\n";
  &runStruck ($struck, $iserased);		   	#0=EraseStart, 1=StartAll

  $delayTime = $dwell;
  if ($delayTime < 0.1) {$delayTime = 0.1;}		#delay motors start in order to catch all counts
  select(undef,undef,undef,$delayTime);			#sleep($wait_step);

### Start the motor:
  printf STDOUT $script.': Starting scan of '.$axis.' from %.3f to %.3f...'."\n",$beginAct,$end;
  $status = &startMotor ($axis, $end);
  if ($status) {return 1,'*** '.$script.': Error starting scan of '.$axis;}


  printf STDOUT "\n".$script.': HD-sync scan of '.$axis.' in progress.'
                    .' Estimated scan time %.3gs. Please wait...'."\n\n",$scanTime;

  return 0,'',$beginAct,$cts2egu,$mtrOut,$struckPs,$struckNg,$dwell;

}							# end of HDflyScan_setup


#=================================================================


# sub HDflyScan_wait ($assy,$axis,$mtrOut,$dwell,$beginPos,$endPos,$npts,
#                     [\@inputs,$postarrayPV,$post_interval,$i_plot,
#                      $cts2egu,$struckPs,$struckNg,$checkOverload,$threshold]) {
  sub HDflyScan_wait($$$$$$$;\@$$$$$$$$) {

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext, @StruckOffsets);
  our ($HDSCAN_FLAGS);					#reference to a hash with various flags for HDSCAN
  our (@x1HD, @timeHD, @y1HD);		                        #GLOBAL RESULTs: positions and timeline
  my ($script, $assy, $axis, $mtrOut);
  my ($dwell, $beginPos, $endPos, $npts);
  my ($inputsRef, $postarrayPV, $post_interval, $i_plot);
  my ($cts2egu, $struckPs, $struckNg, $checkOverload, $threshold);
  my ($struck, $delayTime, $status, $min_val);
  my ($name_plot, $npts_last, $time_epoch, $time_passed);
  my ($scan_index, $actPts, @data, @post_array, $narr, $j);
  my ($wait_step, $wait_, $positioned, @inputs);
  my ($togo, $range, $step, $tolerance, $message);
  my ($acquiring,   $inpos,   $runprg,   $actpos,   $ampena,   $limit);
  my ($acquiringPV, $inposPV, $runprgPV, $actposPV, $ampenaPV, $limitPV);
  my ($OVERLOAD, $overload_detected);

  $script        = 'HDflyScan_wait';
  $assy          = shift(@_);
  $axis          = shift(@_);
  $mtrOut        = shift(@_);
  $dwell         = shift(@_);
  $beginPos      = shift(@_);
  $endPos        = shift(@_);
  $npts          = shift(@_);
  $inputsRef     = shift(@_);
  $postarrayPV   = shift(@_);
  $post_interval = shift(@_);  if (!defined $post_interval) {$post_interval = 2;}
  $i_plot        = shift(@_);  if (!defined $i_plot)        {$i_plot        = 1;}
  $cts2egu       = shift(@_);
  $struckPs      = shift(@_);
  $struckNg      = shift(@_);
  $checkOverload = shift(@_);  if (!defined $checkOverload) {$checkOverload = 0;}
  $threshold     = shift(@_);  if (!defined $threshold)     {$threshold = 0.95E+6;}

  if (!defined $HDSCAN_FLAGS->{'has run'})              {$HDSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'has read mtrList'})     {$HDSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter close'})   {$HDSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter open'})    {$HDSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip calc speed'})      {$HDSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set speed'})       {$HDSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set slew'})        {$HDSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore speed'})   {$HDSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore mprg'})    {$HDSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip mtr link'})        {$HDSCAN_FLAGS->{'skip mtr link'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip mtr unlink'})      {$HDSCAN_FLAGS->{'skip mtr unlink'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip move to start'})   {$HDSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'return to start'})      {$HDSCAN_FLAGS->{'return to start'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'range tolerance'})      {$HDSCAN_FLAGS->{'range tolerance'}      = 0.05;}	# up to 5%
  if (!defined $HDSCAN_FLAGS->{'steps tolerance'})      {$HDSCAN_FLAGS->{'steps tolerance'}      = 5;}		# up to 5 steps

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }

### Wait for on-the-fly scan to finish (don't give up first 5*$delayTime):
  $range      = abs($endPos - $beginPos);
  $step       = $range;
  if ($npts > 1) {$step = $range/($npts-1);}
  else           {$step = $range;}
  $tolerance  = &minval($range*$HDSCAN_FLAGS->{'range tolerance'},$step*$HDSCAN_FLAGS->{'steps tolerance'});
  $acquiringPV = $struck.':Acquiring';
  $inposPV     = $assy.':InPos';
  $runprgPV    = $assy.':RunPrg';
  $actposPV    = $axis.':ActPos';
  $ampenaPV    = $axis.':AmpEna';
  if ($endPos >= $beginPos) {$limitPV=$axis.':PsLimSet';}
  else                      {$limitPV=$axis.':NgLimSet';}
  $wait_step         = 0.01;
  $wait_             = 0;
  $acquiring         = 1;
  $inpos             = 0;
  $runprg            = 1;
  $positioned        = 0;
  $OVERLOAD          = 100000;	                #scan failure flag when overload detected
  $overload_detected = 0;	                #overload not detected yet
  $delayTime         = $dwell;
  if ($delayTime < 0.1) {$delayTime = 0.1;}	#delay motors start in order to catch all counts
  $delayTime         = 5*$delayTime;
  $name_plot         = 'y'.$i_plot.'HD';
  $npts_last         = 0;
  $time_epoch        = time();           	#seconds since epoch
  @inputs            = (@$inputsRef[$i_plot-1]);
  $scan_index        = 0;                   	#scan index to save with the post (e.g. 11-vertical, 21-horizontal)

  while (($acquiring == 1 && ($runprg || $positioned < 3)) || $wait_ < $delayTime) {
     select(undef,undef,undef,$wait_step);	#sleep($wait_step);
     $wait_ += $wait_step;
     ($status,$acquiring) = Pezca::GetDouble($acquiringPV);
     if ($status) {&report_only($script,$status,$acquiringPV); $acquiring=1;}
     ($status,$inpos) = Pezca::GetDouble($inposPV);
     if ($status) {&report_only($script,$status,$inposPV);     $inpos=0;}
     ($status,$runprg) = Pezca::GetDouble($runprgPV);
     if ($status) {&report_only($script,$status,$runprgPV);    $runprg=1;}
     ($status, $actpos) = Pezca::GetDouble($actposPV);
     if ($status) {&report_only($script,$status,$actposPV);    $actpos=$beginPos;}
     ($status, $ampena) = Pezca::GetDouble($ampenaPV);
     if ($status) {&report_only($script,$status,$ampenaPV);    $ampena=0;}
     ($status, $limit) = Pezca::GetDouble($limitPV);
     if ($status) {&report_only($script,$status,$limitPV);     $limit=1;}
     if ($inpos == 1) {$positioned++;}
     else             {$positioned=0;}
     $togo = abs($endPos - $actpos);
     if ($npts > 0 && $wait_ > $delayTime && $range > 0 && $togo > 0.5*$range) {
        $positioned = 0;
     }
     if ($ampena == 0 || $limit == 1) {last;}
#    print STDOUT $script.': acquiring='.$acquiring.', runprg='.$runprg.', inpos='.$inpos
#                .', positioned='.$positioned.', wait_='.$wait_."\n";
     if (defined $postarrayPV || $checkOverload) {  #----Begin-the-post-array-bisiness------------
        $time_passed = time() - $time_epoch;
        if ($time_passed >= $post_interval) {
           $actPts = &getStruckNpts($struck,$struck_ext,'fly');
           if ($actPts > $npts_last+1) {
              $time_epoch = time();			#reset seconds since epoch
              $npts_last  = $actPts;
              ($status,$actPts) = &HDflyScan_read ($axis,$beginPos,$cts2egu,$struckPs,$struckNg,
                                                   \@inputs,\@StruckOffsets,\@data,'flywithread');
              if (! $status) {
                 @x1HD   = ();						#GLOBAL RESULT: positions
#                @timeHD = ();						#GLOBAL RESULT: timeline
                 @y1HD   = ();						#GLOBAL RESULT: intensities
                 for ($j=0; $j<$actPts; $j++) {
                    $x1HD[$j]   = $data[0][$j];
#                   $timeHD[$j] = $data[1][$j];
                    $y1HD[$j]   = $data[2][$j];
                    if ($checkOverload) {if ($y1HD[$j] >= $threshold) {$overload_detected++;}}
                 }
                 if (defined $postarrayPV) {
                    $min_val = &minvalRef(\@y1HD);
                    @post_array = ($scan_index,$npts_last,$beginPos,$endPos,@x1HD);
                    for ($j=0; $j<$actPts; $j++) {push @post_array,($y1HD[$j]-$min_val);}
                    $narr = @post_array;
                    if ($narr < 2048) {
                       $status = Pezca::PutList($postarrayPV, 'FLOAT', @post_array);
                       if ($status) {
                          &report_only($script, $status, $postarrayPV);
                          print STDOUT '!!! '.$script.': error posting '.$narr.'-element array into PV='
                                      .$postarrayPV.'. Further posting will be disabled.'."\n\n";
                          undef $postarrayPV;
                       }
                    } else {
                       print STDOUT '!!! '.$script.': cannot post '.$narr.'-element array into PV='
                                   .$postarrayPV.'. Array is too long. Further posting will be disabled.'."\n\n";
                       undef $postarrayPV;
                    }
                 }
                 if ($overload_detected > 0) {
                    print STDOUT '!!! '.$script.': Intensity overload detected '.$overload_detected.' times.'."\n";
                    $acquiring = 0; $runprg = 0; $positioned = 3;
                 }
              } else {
                 print STDOUT '!!! '.$script.': error reading fly scan data for posting.'."\n";
                 undef $postarrayPV;
              }
           }
        }
     }   #-------------------------------End-the-post-array-bisiness------------
  }
  printf STDOUT $script.': Scan finished!'
               .' (acquiring='.$acquiring.', mtrRunPrg='.$runprg
               .', mtrInPos='.$positioned.', ampEnabled='.$ampena
               .', limitHit='.$limit.', time=%.1fs'
               .', begin_pos=%.5f target_end=%.5f actual_end=%.5f)'."\n",
               $wait_, $beginPos, $endPos, $actpos;

  if (defined $shutter) {
     if ($HDSCAN_FLAGS->{'skip shutter close'} == 0) {
        &closeShutter ($shutter,$shutter_readback);
     }
  }
  if ($HDSCAN_FLAGS->{'skip restore mprg'} == 0 && $HDSCAN_FLAGS->{'skip restore speed'} == 0) {
     print STDOUT $script.': aborting motor, restoring speed and motion prg'."\n";
     &abortAssy ($assy, 1, $axis);			# also restore speed and motion prg.
  }
  elsif ($HDSCAN_FLAGS->{'skip restore mprg'} && $HDSCAN_FLAGS->{'skip restore speed'} == 0) {
     print STDOUT $script.': aborting motor, restoring speed, but keeping motion prg'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
     &setAxisHiSpeed ($assy, $axis);			# restore speed only
  }
  elsif ($HDSCAN_FLAGS->{'skip restore mprg'} == 0 && $HDSCAN_FLAGS->{'skip restore speed'}) {
     print STDOUT $script.': aborting motor, restoring motion prg, but keeping speed'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
     &setMotionPrg($assy, 3); 				#this is faster (no speed change): 3=position
  }
  else  {
     print STDOUT $script.': aborting motor only, but keeping motion prg and speed'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
  }

  print STDOUT $script.': aborting Struck'."\n";
  &abortStruck ($struck);

  if ($HDSCAN_FLAGS->{'skip mtr unlink'} == 0) {
     print STDOUT $script.': unlinking virtual motor'."\n";
     &unlinkMtrOut($assy,$mtrOut);
  }

  if ($HDSCAN_FLAGS->{'has run'} == 0) {$HDSCAN_FLAGS->{'has run'} = 1;}

  if ($overload_detected > 0) {
     return $OVERLOAD, 'Intensity overload detected '.$overload_detected.' times.';
  }
  elsif ($ampena == 0) {
     $message = sprintf '*** '.$script.': motor '.$axis.' did not scan properly.'."\n"
                       .' Amplifier '.$ampenaPV.' turned off (fatal following error?).'."\n"
                       .' Begin_pos=%.5f target_end=%.5f actual_end=%.5f',
                       $beginPos, $endPos, $actpos;
     return 1, $message;
  }
  elsif ($limit == 1) {
     $message = sprintf '*** '.$script.': motor '.$axis.' did not scan properly.'."\n"
                       .' Limit switch '.$limitPV.' is hit.'."\n"
                       .' Begin_pos=%.5f target_end=%.5f actual_end=%.5f',
                       $beginPos, $endPos, $actpos;
     return 1, $message;
  }
  elsif ($npts > 0 && $range > 0 && $togo > $tolerance) {
     $message = sprintf '*** '.$script.': motor '.$axis.' did not scan properly.'."\n"
                       .' Begin_pos=%.5f target_end=%.5f actual_end=%.5f',
                       $beginPos, $endPos, $actpos;
     return 1, $message;
  } else {
     return 0, '';
  }
}	##HDflyScan_wait


#=================================================================


# ($status,$actPts) = &HDflyScan_read ($axis,$beginAct,$cts2egu,$struckPs,$struckNg,
#                                      \@inputs,\@Offsets,\@data,[$finalORfly]);
  sub HDflyScan_read($$$$$\@\@\@;$) {

### Returns ($status, $actPts) and also fills in \@data (positions,timeline,inputs)
  our ($beamline, $struckName, $struck_ext, $IGNORE_ERRORS);
  our ($HDSCAN_FLAGS);		#reference to a hash with various flags for HDSCAN
  my ($script, $axis, $beginAct, $begin);
  my ($cts2egu, $struckPs, $struckNg, $mtrOut, $assy);
  my ($inputs_ref, $Offsets_ref, $data_ref, $finalORfly);
  my ($struck, $status, $endAct, $actPts);
  my ($currPos, $dxpos, $dxpos_max, $currTime);
  my (@intensity, @struckTimer, @encoderPs, @encoderNg);
  my ($ninputs, $input, $i, $j);
  my ($range, $rangeErr, $rangeErr_Rel, $scanStep, $errorThreshold);

  $script      = 'HDflyScan_read';
  $axis        = shift(@_);
  $beginAct    = shift(@_);
  $cts2egu     = shift(@_);
  $struckPs    = shift(@_);
  $struckNg    = shift(@_);
  $inputs_ref  = shift(@_);
  $Offsets_ref = shift(@_);
  $data_ref    = shift(@_);		#2D-array to fill with data (position+timer+ninputs)
  $finalORfly  = shift(@_);		#final/fly/flywithread; default is final, which aborts Struck

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if (!defined $HDSCAN_FLAGS->{'has run'})              {$HDSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $HDSCAN_FLAGS->{'has read mtrList'})     {$HDSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter close'})   {$HDSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip shutter open'})    {$HDSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip calc speed'})      {$HDSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set speed'})       {$HDSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip set slew'})        {$HDSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore speed'})   {$HDSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip restore mprg'})    {$HDSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip mtr link'})        {$HDSCAN_FLAGS->{'skip mtr link'}        = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip mtr unlink'})      {$HDSCAN_FLAGS->{'skip mtr unlink'}      = 0;}
  if (!defined $HDSCAN_FLAGS->{'skip move to start'})   {$HDSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $HDSCAN_FLAGS->{'return to start'})      {$HDSCAN_FLAGS->{'return to start'}      = 0;}

  if (!defined $finalORfly) {$finalORfly = 'flywithread';}
  if (($finalORfly !~ /^final$/i) &&				#abortStruck before reading
#     ($finalORfly !~ /^fly$/i)   &&				#just read
      ($finalORfly !~ /^flywithread$/i)) {                      #pushStruckRead before reading
    print STDOUT '!!! '.$script.': incorrect flag finalORfly='.$finalORfly
                               .'. Presuming "flyWithRead"'."\n";
    $finalORfly = 'flywithread';
  }

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/) {$struck = $beamline.':'.$struck;}

  if ($finalORfly =~ /^final$/i) {
#    print STDOUT $script.': Scan finished!'."\n";
     $HDSCAN_FLAGS->{'has run'} = 1;
     $endAct = &getAxisAct ($axis);				#Read motor position at the end
#    printf STDOUT $script.': Motor end position = %.3f'."\n",$endAct;
     if ($HDSCAN_FLAGS->{'return to start'}) {
### Start reverting motor to the beginning position
### in order to save time on the next scan:
        if (defined $HDSCAN_FLAGS->{'begin'}) {$begin = $HDSCAN_FLAGS->{'begin'};}
        else                                  {$begin = $beginAct;}
        printf STDOUT $script.': Initiating motor return to the beginning pos=%.3f'."\n", $begin;
        &startMotor ($axis, $begin);
     }
  }

  @$data_ref = ();
#-----------------------------------------------------------------
  $ninputs = @$inputs_ref;
# print STDOUT $script.': Reading Struck...'."\n";
### Read Struck timer:
  $actPts = &readStruckTimerRef ($struck, $struck_ext, \@struckTimer, undef, $finalORfly);
### Read motor position data:
# print STDOUT $script.': Reading positive/negative moves from Struck inputs '.$struckPs.'/'.$struckNg."\n";
  $status = &readStruckInputRawRef ($struck, $struck_ext, $actPts, $struckPs, \@encoderPs, 'fly');
  $status = &readStruckInputRawRef ($struck, $struck_ext, $actPts, $struckNg, \@encoderNg, 'fly');
### Read intensity data:
# print STDOUT $script.': Reading intensity from Struck inputs';
  for ($i=0; $i<$ninputs; $i++) {
     $input = @$inputs_ref[$i];
     $status = &readStruckInputRef ($struck, $struck_ext, $actPts, $input, \@struckTimer, \@intensity, 'fly');
     for ($j=0; $j<$actPts; $j++) {			#Correct for background:
        $$data_ref[$i+2][$j] = $intensity[$j] - @$Offsets_ref[$input-1];
     }
  }

### Erase Struck so that when calling HDflyScan next time, it will be already erased:
  if ($finalORfly =~ /^final$/i) {
     &eraseStruck ($struck);
  }
#-----------------------------------------------------------------
### Calculate coordinates:
  $currPos   = $beginAct;					#this is $axis
  $currTime  = 0.;						#starting time
  $dxpos_max = 0;
  for ($i=0; $i<$actPts; $i++) {
     $dxpos = ($encoderPs[$i]-$encoderNg[$i])*$cts2egu;		#convert encoder to xpos
     if (abs($dxpos) > $dxpos_max) {$dxpos_max = abs($dxpos);}
     $$data_ref[0][$i] = $currPos + 0.5*$dxpos;			#Calc.middle point of each interval
     $currPos += $dxpos;					#Calc.end point of each interval:
### Timeline data:
     $$data_ref[1][$i] = $currTime + 0.5*$struckTimer[$i];	#time at the middle of Struck bin
     $currTime  += $struckTimer[$i];				#time at the end of Struck bin
  }
# if ($actPts > 10 && $$data_ref[0][$actPts-1] == $$data_ref[0][0]) {
  if ($finalORfly =~ /^final$/i && $actPts > 10 && $dxpos_max == 0) {
     printf STDOUT '!!! '.$script.': Not getting encoder pulses'
                   .' from Struck='.$struck.' inputs '.$struckPs.' and '.$struckNg.'.'
                   .' Possible reasons: virtual moror is unlinked,'
                   .' or pulse converter is off or pulse converter'
                   .' is connected to incorrect Struck inputs'
                   .' (expecting %d and %d)'."\n"
                   .' cts2egu=%.3g, actPts=%d'."\n", $struckPs,$struckNg,$cts2egu,$actPts;
  }
#-----------------------------------------------------------------
### Test the consistency of scan interval:
  if ($finalORfly =~ /^final$/i) {
     $rangeErr = $currPos - $endAct;
     $range    = abs ($endAct - $beginAct);
     if ($range == 0) {
        print  STDOUT '!!! '.$script.': zero scan range or motor did not move!'."\n";
        return 0, $actPts;
     }
     $rangeErr_Rel = 100. * $rangeErr / $range;
     if ($actPts> 0) {$scanStep = $range / $actPts;}
     else            {$scanStep = $range;}
     if ($actPts > 0) {$errorThreshold = &maxval(1.,50./$actPts);}
     else             {$errorThreshold = 50.}
     if (abs($rangeErr_Rel) > $errorThreshold) {
        printf STDOUT '!!! '.$script.': END position error=%.5g (scan step=%.5g)'."\n",$rangeErr,$scanStep;
        printf STDOUT '!!! '.$script.': Scan  range  error=%.3g%s'."\n",$rangeErr_Rel,'%';
        printf STDOUT '!!! '.$script.': Start posit.(EPICS data) = %.5f'."\n",$beginAct;
        printf STDOUT '!!! '.$script.': End  posit. (EPICS data) = %.5f'."\n",$endAct;
        printf STDOUT '!!! '.$script.': End Theta (Struck calc.) = %.5f'."\n",$currPos;
        printf STDOUT '!!! '.$script.': Counts to Eng.units fctr = %.5g'."\n",$cts2egu;
        printf STDOUT '!!! '.$script.': Actual number of scan points = %d'."\n",$actPts;
        if (defined $HDSCAN_FLAGS->{'assy'} && defined $HDSCAN_FLAGS->{'mtrOut'}) {
           $mtrOut = $HDSCAN_FLAGS->{'mtrOut'};
           $assy   = $HDSCAN_FLAGS->{'assy'};
           printf STDOUT '!!! '.$script.': Virtual motor-%d link status=%d'."\n",$mtrOut,&isMtrOutLinked($assy,$mtrOut);
        } else {
           print  STDOUT '!!! '.$script.': Virtual motor or assembly information is lost'."\n";
        }
#       if (! $IGNORE_ERRORS) {
##         &press_key_continue();
           return -2,$actPts;
#       }
     }
  }
#-----------------------------------------------------------------

# print STDOUT $script.': Scan data retrieved!'."\n";
  return 0, $actPts;

}	##HDflyScan_read


#=================================================================

# sub ENflyScan($beginEnergy,$endEnergy,$npts,$scanTime,
#               $linkID,$undulator,$IDoffset,$medm,
#               $fileBkg,$ninputs,\@inputs,[$scaler]) {
  sub ENflyScan($$$$$$$$$$\@;$) {

### Returns implicitly: ($status,$actPts)
### Returns GLOBAL:      @x1EN, @timeEN, @y1EN, ... @y{$ninputs}EN, @StruckOffsets

###|----------------------------------------------------------------------|
###|The main difference of this scan from HDflyScan is that it is moving one |
###|axis (Energy), but reading another (Theta). At the end, after Theta   |
###|values are restored from encoder pulses, there is one extra step where|
###|they are recalculated into Energy.                                    |
###|In addition ENflyScan can roughly synchronize mono and undulator motions.|
###|----------------------------------------------------------------------|

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext, @StruckOffsets);
  our ($ENSCAN_FLAGS);		#reference to a hash with various flags for ENSCAN
  our (@x1EN, @timeEN);		#GLOBAL RESULTS: positions and timeline
  my ($script, $beginEnergy, $endEnergy, $npts, $scanTime, $linkID);
  my ($undulator, $IDoffset, $medm, $fileBkg, $ninputs, $inputsRef);
  my ($assy, $axisEnergy, $axisTheta, $ESinTheta, $beginThetaAct, $actPts);
  my ($cts2egu, $mtrOut, $struckPs, $struckNg, $scaler);
  my ($struck, $timePPT, $dwell, $message, $status, $status1, $i, $j, @data, $name);

  $script      = 'ENflyScan';
  $beginEnergy = shift(@_);	#start scan position
  $endEnergy   = shift(@_);	#end scan position
  $npts        = shift(@_);	#number of points on fast scan
  $scanTime    = shift(@_);	#scan time in seconds
  $linkID      = shift(@_);	#1=link undulator, 0=don't link.
  $undulator   = shift(@_);	#EPICS name of undulator
  $IDoffset    = shift(@_);	#offset of ID with respect to mono (in keV)
  $medm        = shift(@_);	#1=open MEDM, 0=not
  $fileBkg     = shift(@_);	#struck background file ('auto' for automatic, '' for none)
  $ninputs     = shift(@_);	#number of Struck inputs
  $inputsRef   = shift(@_);	#reference to the array of Struck inputs
  $scaler      = shift(@_);	#name of Struck as Joerger

  if (!defined $ENSCAN_FLAGS->{'has run'})              {$ENSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip shutter close'})   {$ENSCAN_FLAGS->{'skip shutter close'}   = 0;}

# print STDOUT 'Enterting '.$script."\n";
  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }
  if (defined $shutter) {
     if ($ENSCAN_FLAGS->{'skip shutter close'} == 0) {
#       print STDOUT $script.': Closing shutter '.$shutter."\n";
        &closeShutter ($shutter,$shutter_readback);
     }
  }

  if (!defined $fileBkg)       {$fileBkg = '';}
  if ($fileBkg =~ /^auto$/i)   {$fileBkg = 'struck_Bkg.dat';}
### Read the scaler background offsets:
  if (!defined $StruckOffsets[0]) {			    #skip reading offsets if they were read in main
     if (length($fileBkg) > 0) {			    #if file name is specified
        print STDOUT $script.': Reading Struck scaler '.$struck.' offsets from '.$fileBkg."\n";
     }
     @StruckOffsets = &getStruckOffsets($fileBkg,$struck);  #we make it GLOBAL to access from main
  }

  $timePPT = $scanTime / $npts;
  printf STDOUT $script.': Initializing. ScanTime=%.2fs,  Npts=%d, Dwell=%.2fs'."\n",$scanTime,$npts,$timePPT;

  if ($medm && $ENSCAN_FLAGS->{'has run'} == 0) {
     for ($i=0; $i<$ninputs; $i++) {&openStruckMedm($struck, $struck_ext, @$inputsRef[$i]);}
  }

  ($status,$message,
   $assy,$axisEnergy,$axisTheta,
   $beginThetaAct,$ESinTheta,$cts2egu,
   $mtrOut,$struckPs,$struckNg,$dwell) = &ENflyScan_setup($beginEnergy,$endEnergy,$npts,$timePPT,
                                                          $linkID,$undulator,$IDoffset,$scaler);
  if ($status) {
     print STDOUT "\n".$script.': error='.$status.' from ENflyScan_setup.'."\n";
     print STDOUT "\n",$message,"\n";
     return $status, 0;
  }

### Wait for scan to finish:
  ($status,$message) = &ENflyScan_wait ($assy,$axisEnergy,$mtrOut,$dwell,$beginEnergy,$endEnergy,$npts);
  if ($status) {
     print STDOUT "\n".$script.': error='.$status.' from ENflyScan_wait.'."\n";
     print STDOUT "\n",$message,"\n";
#    return $status, 0;					#we cannot return here - first restore everything in ENflyScan_read
  }
  $status1 = $status;

### Read scan data (fill \@data=(@positions,@timeline,@inputs)):
  ($status,$actPts) = &ENflyScan_read ($axisTheta,$beginThetaAct,$ESinTheta,
                                       $cts2egu,$struckPs,$struckNg,
                                       $inputsRef,\@StruckOffsets,\@data,'final');
  if ($status) {
     print STDOUT "\n".$script.': error='.$status.' from ENflyScan_read.'."\n";
     return $status, 0;
  }
  elsif ($status1) {
     return $status1, 0;
  }

  @x1EN = ();							#GLOBAL RESULT: positions
  @timeEN = ();							#GLOBAL RESULT: timeline
  for ($j=0; $j<$actPts; $j++) {
     $x1EN[$j]   = $data[0][$j];
     $timeEN[$j] = $data[1][$j];
  }
  no strict 'refs';		#-----------------------#
  for ($i=1; $i<=$ninputs; $i++) {                      #
     $name  = 'y'.$i.'EN';                              #
     @$name = ();					#	#GLOBAL RESULT: data for input-$i
     for ($j=0; $j<$actPts; $j++) {                     #
        @$name[$j] = $data[$i+1][$j];                   #
     }                                                  #
  }                                                     #
  use strict 'refs';		#-----------------------#

# print STDOUT $script.': Scan data retrieved!'."\n";
  return $status, $actPts;

}	##ENflyScan


#=================================================================

# ($status,$message,
# [$assy,$axisEnergy,$axisTheta,
# $beginThetaAct,$ESinTheta,$cts2egu,
# $mtrOut,$struckPs,$struckNg,$dwell]) = &ENflyScan_setup($beginEnergy,$endEnergy,$npts,$timePPT,
#                                                         [$linkID,$undulator,$IDoffset,$scaler]);
  sub ENflyScan_setup($$$$;$$$$) {

### Returns implicitly:
### ($status,$message,[$assy,$axisEnergy,$axisTheta,$beginThetaAct,$ESinTheta,$cts2egu,$mtrOut,$struckPs,$struckNg,$dwell])
###|----------------------------------------------------------------------|
###|The main difference of this scan from HDflyScan is that it is moving one |
###|axis (Energy), but reading another (Theta). At the end, after Theta   |
###|values are restored from encoder pulses, there is one extra step where|
###|they are recalculated into Energy.                                    |
###|----------------------------------------------------------------------|

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext);
  our ($IGNORE_ERRORS, $report);
  our ($ENSCAN_FLAGS);		#reference to a hash with various flags for ENSCAN
  our ($EXTRA_MOTION_TIME);	#extra allowance for tracking because
  our ($enmono13, @harmEn, @harmNum, @Elower, @Eupper, @Olower, @Oupper);		#make global to store SQL data between calls
  my ($script, $beginEnergy, $endEnergy, $npts, $timePPT, $delayTime);
  my ($cts2egu, $mtrOut, $formula, $struckPs, $struckNg, $dwell, $step);
  my ($struck, $int_ext, $iserased, $status, $message, $justLinked, $need2move);
  my ($scanRange, $l_speed, $h_speed, $formulaOffset, $beginThetaAct, $beginEnergyAct);
  my ($scanTime, $extra_motion_time, $ignore_errors, $estimPts, $maxPts);
  my ($DWELLTIMEMIN, $DWELLTIMEMAX, $STRUCK_EXTRA_TIME);
  my ($linkID, $undulator, $IDoffset, $beginGap, $endGap, $scaler);
  my ($assy, $axisEnergy, $axisTheta, $beginTheta, $endTheta, $ESinTheta);

  $script      = 'ENflyScan_setup';
  $beginEnergy = shift(@_);			#start scan position
  $endEnergy   = shift(@_);			#end scan position
  $npts        = shift(@_);			#desired scan step
  $timePPT     = shift(@_);			#desired scan time-per-point in seconds
  $linkID      = shift(@_);			#1=link undulator, 0=don't link.
  $undulator   = shift(@_);			#EPICS name of undulator
  $IDoffset    = shift(@_);			#offset of ID with respect to mono (in keV)
  $scaler      = shift(@_);			#name of Struck as Joerger

# print STDOUT 'Enterting '.$script."\n";
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if (!defined $ENSCAN_FLAGS->{'has run'})              {$ENSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $ENSCAN_FLAGS->{'has read mtrList'})     {$ENSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip shutter close'})   {$ENSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip shutter open'})    {$ENSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip calc speed'})      {$ENSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip set speed'})       {$ENSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip set slew'})        {$ENSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip restore speed'})   {$ENSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip restore mprg'})    {$ENSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip mtr link'})        {$ENSCAN_FLAGS->{'skip mtr link'}        = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip mtr unlink'})      {$ENSCAN_FLAGS->{'skip mtr unlink'}      = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip move to start'})   {$ENSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'return to start'})      {$ENSCAN_FLAGS->{'return to start'}      = 0;}
#
  if (!defined $ENSCAN_FLAGS->{'speed'})                {$ENSCAN_FLAGS->{'speed'}                = 0;}
  if (!defined $ENSCAN_FLAGS->{'cts2egu'})              {$ENSCAN_FLAGS->{'cts2egu'}              = 0;}
  if (!defined $ENSCAN_FLAGS->{'formula'})              {$ENSCAN_FLAGS->{'formula'}              = 0;}
  if (!defined $ENSCAN_FLAGS->{'mtrOut'})               {$ENSCAN_FLAGS->{'mtrOut'}               = 0;}
  if (!defined $ENSCAN_FLAGS->{'struckPs'})             {$ENSCAN_FLAGS->{'struckPs'}             = 0;}
  if (!defined $ENSCAN_FLAGS->{'struckNg'})             {$ENSCAN_FLAGS->{'struckNg'}             = 0;}
### Names of Bragg Angle and Energy axes in the fastscans DB
### (i.e. /home/gmca/epics/fastscans/data/motorList.dat).
  if (!defined $ENSCAN_FLAGS->{'keyEnergy'})            {$ENSCAN_FLAGS->{'keyEnergy'} = 'DCM_Energy';}	#GMCA default
  if (!defined $ENSCAN_FLAGS->{'keyTheta'})             {$ENSCAN_FLAGS->{'keyTheta'}  = 'DCM_Bragg'; }	#GMCA default

  if ($ENSCAN_FLAGS->{'has run'} == 0 && $ENSCAN_FLAGS->{'has read mtrList'} == 0) {
### Parameters $cts2egu,$formula,$mtrOut,$struckPs,$struckNg
### are only used for the Bragg angle axis:
     ($assy,$axisEnergy,$cts2egu,$formula,$mtrOut,$struckPs,$struckNg) = &readMotorList($ENSCAN_FLAGS->{'keyEnergy'});
     if (!defined $axisEnergy) {
        $message = '*** '.$script.': No HD-synchronized data in motorList for key='.$ENSCAN_FLAGS->{'keyEnergy'};
        print STDOUT $message,"\n";
        return 1, $message;
     }
     ($assy,$axisTheta, $cts2egu,$formula,$mtrOut,$struckPs,$struckNg) = &readMotorList($ENSCAN_FLAGS->{'keyTheta'});
     if (!defined $axisTheta) {
        $message = '*** '.$script.': No HD-synchronized data in motorList for key='.$ENSCAN_FLAGS->{'keyTheta'};
        print STDOUT $message,"\n";
        return 1, $message;
     }
     $assy       = $beamline.':'.$assy;
     $axisEnergy = $beamline.':'.$axisEnergy;
     $axisTheta  = $beamline.':'.$axisTheta;
     $ENSCAN_FLAGS->{'assy'}       = $assy;
     $ENSCAN_FLAGS->{'axisEnergy'} = $axisEnergy;
     $ENSCAN_FLAGS->{'axisTheta'}  = $axisTheta;
     $ENSCAN_FLAGS->{'cts2egu'}    = $cts2egu;
     $ENSCAN_FLAGS->{'formula'}    = $formula;
     $ENSCAN_FLAGS->{'mtrOut'}     = $mtrOut;
     $ENSCAN_FLAGS->{'struckPs'}   = $struckPs;
     $ENSCAN_FLAGS->{'struckNg'}   = $struckNg;
  } else {
     $assy       = $ENSCAN_FLAGS->{'assy'};
     $axisEnergy = $ENSCAN_FLAGS->{'axisEnergy'};
     $axisTheta  = $ENSCAN_FLAGS->{'axisTheta'};
     $cts2egu    = $ENSCAN_FLAGS->{'cts2egu'};
     $formula    = $ENSCAN_FLAGS->{'formula'};
     $mtrOut     = $ENSCAN_FLAGS->{'mtrOut'};
     $struckPs   = $ENSCAN_FLAGS->{'struckPs'};
     $struckNg   = $ENSCAN_FLAGS->{'struckNg'};
  }
  if ($ENSCAN_FLAGS->{'has run'} == 0) {
     $ESinTheta  = &getESinTheta($assy);
     $ENSCAN_FLAGS->{'ESinTheta'} = $ESinTheta;
  } else {
     $ESinTheta = $ENSCAN_FLAGS->{'ESinTheta'};
  }

# $ENSCAN_FLAGS->{'npts'}        = $npts;
  $ENSCAN_FLAGS->{'beginEnergy'} = $beginEnergy;
  $ENSCAN_FLAGS->{'endEnergy'}   = $endEnergy;
# $ENSCAN_FLAGS->{'timePPT'}     = $timePPT;

  if (!defined $linkID) {
     $linkID = 0;
     undef $undulator;
     undef $IDoffset;
  } else {
     $ENSCAN_FLAGS->{'undulator'} = $undulator;
     $ENSCAN_FLAGS->{'IDoffset'}  = $IDoffset;
  }
  $ENSCAN_FLAGS->{'linkID'}      = $linkID;

  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }
  if (defined $shutter) {
     if ($ENSCAN_FLAGS->{'skip shutter close'} == 0) {
#       print STDOUT $script.': Closing shutter '.$shutter."\n";
        &closeShutter ($shutter,$shutter_readback);
     }
  }

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}

  ($DWELLTIMEMIN,$DWELLTIMEMAX) = &getStruckDwellMinMax($struck);
  $STRUCK_EXTRA_TIME = 3;			#time for Struck to count longer than motion

  &abortAssy($assy);
  $int_ext = 0;					#0=internal advance, 1=external

  if ($npts < 3) {
     $message = '*** '.$script.': incorrect scan npts='.$npts;
     print STDOUT $message,"\n";
     return 1, $message;
  }

  $dwell = $timePPT;
  if    ($dwell < $DWELLTIMEMIN) {
     printf STDOUT '!!! '.$script.': timePPT=%.3fs is below Struck limit=%.3fs'."\n", $dwell, $DWELLTIMEMIN;
     $dwell = $DWELLTIMEMIN;
  }
  elsif ($dwell > $DWELLTIMEMAX) {
     printf STDOUT '!!! '.$script.': timePPT=%.3fs exceeds Struck limit=%.3fs'."\n", $dwell, $DWELLTIMEMAX;
     $dwell = $DWELLTIMEMAX;
  }
  $timePPT  = $dwell;
  $scanTime = $npts * $dwell;

###-----------------------------------------------------------------
  if ($ENSCAN_FLAGS->{'has run'} == 0 || $ENSCAN_FLAGS->{'skip calc speed'} == 0) {
### Calculate scan speed for the motor:
#    print STDOUT $script.': Calculating slew-mode speed for '.$axisTheta."\n";
     $ignore_errors = $IGNORE_ERRORS;
     $IGNORE_ERRORS = 1;
     $beginTheta = &E2theta($assy,$beginEnergy,$ESinTheta);	#scan begin in Theta
     $endTheta   = &E2theta($assy,$endEnergy,$ESinTheta);	#scan end in Theta
     $scanRange  = abs ($endTheta - $beginTheta);
     ($l_speed, $h_speed) = &calcAxisSpeed ($assy, $axisTheta, $scanRange, $scanTime);
     $IGNORE_ERRORS = $ignore_errors;
     if ($l_speed >= $h_speed) {				#speed was too high
        $l_speed  = $h_speed;
        $scanTime = abs($scanRange)/$l_speed;
        $dwell = $scanTime / $npts;
        if    ($dwell < $DWELLTIMEMIN) {
           $message = sprintf '!!! '.$script.': too low timePPT=%.3fs required with npts='.$npts, $dwell;
           print STDOUT $message,"\n";
#          return 1, $message;
           $dwell = $DWELLTIMEMIN;
        }
        elsif ($dwell > $DWELLTIMEMAX) {
           $message = sprintf '!!! '.$script.': too high timePPT=%.3fs required with npts='.$npts, $dwell;
           print STDOUT $message,"\n";
#          return 1, $message;
           $dwell = $DWELLTIMEMAX;
        }
        if ($dwell != $timePPT) {
            printf STDOUT '!!! '.$script.': timePPT changed from %.3fs to %.3fs'."\n", $timePPT, $dwell;
        }
     }
     $ENSCAN_FLAGS->{'speed'} = $l_speed;
     $ENSCAN_FLAGS->{'dwell'} = $dwell;
  } else {
#    printf STDOUT '!!! '.$script.': Using '.$axisTheta.' slew-mode speed=%.3f from previous scan'."\n",$ENSCAN_SPEED;
#    print  STDOUT '!!! '.$script.': (This presumes same range and time)'."\n";
     $l_speed = $ENSCAN_FLAGS->{'speed'};
     $dwell   = $ENSCAN_FLAGS->{'dwell'};
  }
  $timePPT  = $dwell;
  $scanTime = $npts * $dwell;

  $estimPts = $npts + int($STRUCK_EXTRA_TIME/$dwell + 0.5);
  $maxPts   = &getStruckNmax($struck, $struck_ext);
  if ($estimPts > $maxPts) {
     print STDOUT "\n".'!!! '.$script.': Estimated pts='.$estimPts.' exceeds'."\n".
                    'Struck buffer='.$maxPts.'. Please, increase step size,'."\n".
                    'Otherwice scan may be aborted before motor arrives to the end.'."\n";
     if ($npts > $maxPts) {
        $message =  '*** '.$script.': Required pts='.$npts.' exceeds'."\n".
                    'Struck buffer='.$maxPts.'. Please, increase step size.';
        print STDOUT $message,"\n";
        return 1, $message;
     }
  }

### Calculate scan speed for the ID:
  if ($linkID) {
     if ($ENSCAN_FLAGS->{'has run'} == 0) {
        ($status,$beginGap,$endGap) = &calcIDSpeed ($undulator, $beginEnergy+$IDoffset,
                                                    $endEnergy+$IDoffset, $scanTime);
        if ($status) {
           $message = '*** '.$script.': undulator speed setup failed';
           print STDOUT $message,"\n";
           return 1, $message;
        }
        $ENSCAN_FLAGS->{'beginGap'} = $beginGap;
        $ENSCAN_FLAGS->{'endGap'}   = $endGap;
     } else {
        $beginGap = $ENSCAN_FLAGS->{'beginGap'};
        $endGap   = $ENSCAN_FLAGS->{'endGap'};
     }
  }

### Set Struck scaler:
  if ($ENSCAN_FLAGS->{'has run'} == 0) {
#    print STDOUT $script.': Setting up Struck scaler '.$struck."\n";
     &setStruck ($struck, $struck_ext, $int_ext, $dwell, $maxPts, $scaler);
  }

### Erase struck:
  if ($ENSCAN_FLAGS->{'has run'} == 0) {
#    print STDOUT $script.': Erasing Struck scaler '.$struck."\n";
     &eraseStruck ($struck);
  }
  $iserased = 1;

### Set motor:
  if ($ENSCAN_FLAGS->{'has run'} == 0) {
#    print STDOUT $script.': Setting up motor '.$axisEnergy."\n";
     &configAssy ($assy);
  }

  if ($ENSCAN_FLAGS->{'has run'} == 0 || $ENSCAN_FLAGS->{'skip mtr link'} == 0) {
### Link mtrOut to the Theta-axis:
     &unlinkMtrOut($assy,$mtrOut);				#only needed with determineFormulaOffset2
     $formulaOffset = &determineFormulaOffset2($assy, $axisTheta, $formula, $cts2egu);
     if    ($formulaOffset == 0) {&linkMtrOut ($assy, $mtrOut, $formula);}
     elsif ($formulaOffset > 0)  {&linkMtrOut ($assy, $mtrOut, $formula.'+'.$formulaOffset);}
     elsif ($formulaOffset < 0)  {&linkMtrOut ($assy, $mtrOut, $formula  .  $formulaOffset);}
     $justLinked = 1;
  } else {
     $justLinked = 0;
  }
#-----------------------------------------------------------------
  $need2move = 0;
  if ($ENSCAN_FLAGS->{'skip move to start'} && $justLinked == 0) {
     $beginEnergyAct = &getAxisAct ($axisEnergy);
     $step           = ($beginEnergy - $endEnergy) / $npts;
     if (abs($beginEnergyAct - $beginEnergy) > 0.5*abs($step)) {$need2move=1;}
  }
  if ($ENSCAN_FLAGS->{'skip move to start'} == 0 || $justLinked) {
### Move motor to the starting position:
     printf STDOUT $script.': Moving '.$axisEnergy.' to the starting position=%.1feV'."\n",1000.*$beginEnergy;
### This is to recover speed from previouse unclean exit:
     if ($ENSCAN_FLAGS->{'has run'} == 0) {&restoreAxis ($assy, $axisEnergy);}	#restore high-speed & motion prg before start
### Increase expected motion time due to linked motor#32:
     if (defined $EXTRA_MOTION_TIME) {$extra_motion_time = $EXTRA_MOTION_TIME;}
     $EXTRA_MOTION_TIME = 600;
#    $status = &runMono($axisEnergy,$assy,$beginEnergy,$linkID,$undulator,$IDoffset);		#old format
     $status = &runMono($axisEnergy,$assy,$beginEnergy,
                        $linkID, $undulator, $IDoffset, 					#final (overwrites SQL)
                        undef, undef, undef,							#$syncMode=undef $syncStep=undef $switchID=undef
                        undef, undef,								#$attenuation=undef,$switchMono=undef
                        \$enmono13, \@harmEn, \@harmNum, 
                        \@Elower, \@Eupper, \@Olower, \@Oupper);
### Restore expected motion time to normal from local cache:
     if (defined $extra_motion_time) {$EXTRA_MOTION_TIME = $extra_motion_time;}
     else                            {undef $EXTRA_MOTION_TIME;}
     if ($status) {
        $message =  '*** '.$script.': failed moving '.$axisEnergy.' to the starting position';
        print STDOUT $message,"\n";
        return 1, $message;
     }
  }

### Setting slow speed for scanning:
# print STDOUT $script.': Setting up motor '.$axisTheta.' speed to slew (speed='.$l_speed.')'."\n";
  if ($ENSCAN_FLAGS->{'has run'} == 0) {
     print STDOUT $script.': setting scanning speed & slew motion prg'."\n";
     &setAxisSlewScan ($assy, $axisTheta, $l_speed);		#setMotionPrg+setAxisSpeed
  } else {
     if ($ENSCAN_FLAGS->{'skip set slew'} == 0 && $ENSCAN_FLAGS->{'skip set speed'} == 0) {
        print STDOUT $script.': setting scanning speed & slew motion prg'."\n";
        &setAxisSlewScan ($assy, $axisTheta, $l_speed);		#setMotionPrg+setAxisSpeed
     }
     elsif ($ENSCAN_FLAGS->{'skip set slew'} && $ENSCAN_FLAGS->{'skip set speed'} == 0) {
        print STDOUT $script.': setting scanning speed only, but presuming slew motion prg'."\n";
        &setAxisSpeed ($assy, $axisTheta, $l_speed); 		#this is faster (no motion prg change)
     }
     elsif ($ENSCAN_FLAGS->{'skip set slew'} == 0 && $ENSCAN_FLAGS->{'skip set speed'}) {
        print STDOUT $script.': setting slew motion prg, but presuming scanning speed is set'."\n";
        &setMotionPrg($assy, 1); 				#this is faster (no speed change): 1=slew
     }
  }

### Setting monitors on motors motion and stuck:
  if ($ENSCAN_FLAGS->{'has run'} == 0 ) {
     $status = &addCaMonitor($struck.':Acquiring');
#    $status = &setAssyMonitors($assy);			#this was set before in runMotor
#    $status = &setAxisMonitors($axisEnergy);		#this was set before in runMotor
     $status = &setAxisMonitors($axisTheta);
  }

### Read motor position at startup:
  $beginThetaAct = &getAxisAct ($axisTheta);

#-----------------------------------------------------------------
  if ($linkID) {
     print STDOUT $script.': Accelerating ID before the scan...'."\n";
     $status = &startIDScan($undulator,$beginGap,$endGap,$scanTime);
  }

  if (defined $shutter) {
     if ($ENSCAN_FLAGS->{'skip shutter open'} == 0) {
#       print STDOUT $script.': Opening shutter '.$shutter."\n";
        &openShutter ($shutter,$shutter_readback);
        sleep(1);			# 2010.11.06: TMP workaround for hight gains?
     }
  }

### Start Struck:
# print STDOUT $script.': Starting On-the-fly Energy scan with '.$struck.'...'."\n";
  &runStruck ($struck, $iserased);		   	#0=EraseStart, 1=StartAll

  $delayTime = $dwell;
  if ($delayTime < 0.1) {$delayTime = 0.1;}		#delay motors start in order to catch all counts
  select(undef,undef,undef,$delayTime);			#sleep($wait_step);

### Start the motor:
  printf STDOUT $script.': Starting scan of '.$axisEnergy.' from %.3f to %.3f...'."\n",$beginEnergy,$endEnergy;
  &startMotor ($axisEnergy, $endEnergy);

  printf STDOUT "\n".$script.': Energy HD-sync scan in progress. Estimated scan time %.3gs.'
                    .' Please wait...'."\n\n", $scanTime;

  return 0, '', $assy, $axisEnergy, $axisTheta, $beginThetaAct, $ESinTheta,
                $cts2egu, $mtrOut, $struckPs, $struckNg, $dwell;

}	##end of ENflyScan_setup


#=================================================================


# sub ENflyScan_wait ($assy,$axisEnergy,$mtrOut,$dwell,$beginEnergy,$endEnergy,$npts) {
  sub ENflyScan_wait($$$$$$$) {

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext);
  our ($ENSCAN_FLAGS);					#reference to a hash with various flags for ENSCAN
  my ($script, $assy, $axisEnergy, $mtrOut);
  my ($dwell, $beginEnergy, $endEnergy, $npts);
  my ($undulator, $IDoffset);
  my ($struck, $delayTime, $status);
  my ($wait_step, $wait_, $positioned);
  my ($togo, $range, $step, $tolerance, $message);
  my ($acquiring,   $inpos,   $runprg,   $actpos,   $ampena,   $limit);
  my ($acquiringPV, $inposPV, $runprgPV, $actposPV, $ampenaPV, $limitPV);

  $script      = 'ENflyScan_wait';
  $assy        = shift(@_);
  $axisEnergy  = shift(@_);
  $mtrOut      = shift(@_);
  $dwell       = shift(@_);
  $beginEnergy = shift(@_);
  $endEnergy   = shift(@_);
  $npts        = shift(@_);

  if (!defined $ENSCAN_FLAGS->{'has run'})              {$ENSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $ENSCAN_FLAGS->{'has read mtrList'})     {$ENSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip shutter close'})   {$ENSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip shutter open'})    {$ENSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip calc speed'})      {$ENSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip set speed'})       {$ENSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip set slew'})        {$ENSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip restore speed'})   {$ENSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip restore mprg'})    {$ENSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip mtr link'})        {$ENSCAN_FLAGS->{'skip mtr link'}        = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip mtr unlink'})      {$ENSCAN_FLAGS->{'skip mtr unlink'}      = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip move to start'})   {$ENSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'return to start'})      {$ENSCAN_FLAGS->{'return to start'}      = 0;}
  if (!defined $ENSCAN_FLAGS->{'range tolerance'})      {$ENSCAN_FLAGS->{'range tolerance'}      = 0.05;}	# up to 5%
  if (!defined $ENSCAN_FLAGS->{'steps tolerance'})      {$ENSCAN_FLAGS->{'steps tolerance'}      = 5;}		# up to 5 steps
#
  if (!defined $ENSCAN_FLAGS->{'linkID'})               {$ENSCAN_FLAGS->{'linkID'}                =0;}
  if (defined  $ENSCAN_FLAGS->{'undulator'}) {$undulator = $ENSCAN_FLAGS->{'undulator'};}
  if (defined  $ENSCAN_FLAGS->{'IDoffset'})  {$IDoffset  = $ENSCAN_FLAGS->{'IDoffset'};}

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }

### Wait for on-the-fly scan to finish (don't give up first 5*$delayTime):
  $range      = abs($endEnergy - $beginEnergy);
  if ($npts > 1) {$step = $range/($npts-1);}
  else           {$step = $range;}
  $tolerance  = &minval($range*$ENSCAN_FLAGS->{'range tolerance'},$step*$ENSCAN_FLAGS->{'steps tolerance'});
  $acquiringPV = $struck.':Acquiring';
  $inposPV     = $assy.':InPos';
  $runprgPV    = $assy.':RunPrg';
  $actposPV    = $axisEnergy.':ActPos';
  $ampenaPV    = $axisEnergy.':AmpEna';
  if ($endEnergy >= $beginEnergy) {$limitPV=$axisEnergy.':PsLimSet';}
  else                            {$limitPV=$axisEnergy.':NgLimSet';}
  $wait_step  = 0.01;
  $wait_      = 0;
  $acquiring  = 1;
  $inpos      = 0;
  $runprg     = 1;
  $positioned = 0;
  $delayTime  = $dwell;
  if ($delayTime < 0.1) {$delayTime = 0.1;}		#delay motors start in order to catch all counts
  $delayTime = 5*$delayTime;

  while (($acquiring == 1 && ($runprg || $positioned < 3)) || $wait_<$delayTime) {
     select(undef,undef,undef,$wait_step);		#sleep($wait_step);
     $wait_ += $wait_step;
     ($status,$acquiring) = Pezca::GetDouble($acquiringPV);
     if ($status) {&report_only($script,$status,$acquiringPV); $acquiring=1;}
     ($status,$inpos) = Pezca::GetDouble($inposPV);
     if ($status) {&report_only($script,$status,$inposPV);     $inpos=0;}
     ($status,$runprg) = Pezca::GetDouble($runprgPV);
     if ($status) {&report_only($script,$status,$runprgPV);    $runprg=1;}
     ($status, $actpos) = Pezca::GetDouble($actposPV);
     if ($status) {&report_only($script,$status,$actposPV);    $actpos=$beginEnergy;}
     ($status, $ampena) = Pezca::GetDouble($ampenaPV);
     if ($status) {&report_only($script,$status,$ampenaPV);    $ampena=0;}
     ($status, $limit) = Pezca::GetDouble($limitPV);
     if ($status) {&report_only($script,$status,$limitPV);     $limit=1;}
     if ($inpos == 1) {$positioned++;}
     else             {$positioned=0;}
     $togo = abs($endEnergy - $actpos);
     if ($npts > 0 && $wait_ > $delayTime && $range > 0 && $togo > 0.5*$range) {
        $positioned = 0;
     }
     if ($ampena == 0 || $limit == 1) {last;}
#    print STDOUT $script.': acquiring='.$acquiring.', runprg='.$runprg.', inpos='.$inpos
#                .', positioned='.$positioned.', wait_='.$wait_."\n";
  }
  printf STDOUT $script.': Scan finished!'
               .' (acquiring='.$acquiring.', mtrRunPrg='.$runprg
               .', mtrInPos='.$positioned.', ampEnabled='.$ampena
               .', limitHit='.$limit.', time=%.1fs'
               .', begin_energy=%.5f target_end=%.5f actual_end=%.5f)'."\n",
               $wait_, $beginEnergy, $endEnergy, $actpos;

  if (defined $shutter) {
     if ($ENSCAN_FLAGS->{'skip shutter close'} == 0) {
        &closeShutter ($shutter,$shutter_readback);
     }
  }
  if ($ENSCAN_FLAGS->{'skip restore mprg'} == 0 && $ENSCAN_FLAGS->{'skip restore speed'} == 0) {
     print STDOUT $script.': aborting motor, restoring speed and motion prg'."\n";
     &abortAssy ($assy, 1, $axisEnergy);		# also restore speed and motion prg.
  }
  elsif ($ENSCAN_FLAGS->{'skip restore mprg'} && $ENSCAN_FLAGS->{'skip restore speed'} == 0) {
     print STDOUT $script.': aborting motor, restoring speed, but keeping motion prg'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
     &setAxisHiSpeed ($assy, $axisEnergy);		# restore speed only
  }
  elsif ($ENSCAN_FLAGS->{'skip restore mprg'} == 0 && $ENSCAN_FLAGS->{'skip restore speed'}) {
     print STDOUT $script.': aborting motor, restoring motion prg, but keeping speed'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
     &setMotionPrg($assy, 3); 				#this is faster (no speed change): 3=position
  }
  else {
     print STDOUT $script.': aborting motor only, but keeping motion prg and speed'."\n";
     &abortAssy ($assy);  				# this is faster (no motion prg change)
  }

  print STDOUT $script.': aborting Struck'."\n";
  &abortStruck ($struck);

  if ($ENSCAN_FLAGS->{'linkID'}) {&abortIDScan($undulator);}

  if ($ENSCAN_FLAGS->{'skip mtr unlink'} == 0) {
     print STDOUT $script.': unlinking virtual motor'."\n";
     &unlinkMtrOut($assy, $mtrOut);
  }

  if ($ENSCAN_FLAGS->{'has run'} == 0) {$ENSCAN_FLAGS->{'has run'} = 1;}

  if ($ampena == 0) {
     $message = sprintf '*** '.$script.': energy '.$axisEnergy.' did not scan properly.'."\n"
                       .' Amplifier '.$ampenaPV.' turned off (fatal following error?).'."\n"
                       .' Begin_pos=%.5f target_end=%.5f actual_end=%.5f',
                       $beginEnergy, $endEnergy, $actpos;
     return 1, $message;
  }
  elsif ($limit == 1) {
     $message = sprintf '*** '.$script.': energy '.$axisEnergy.' did not scan properly.'."\n"
                       .' Limit switch '.$limitPV.' is hit.'."\n"
                       .' Begin_pos=%.5f target_end=%.5f actual_end=%.5f',
                       $beginEnergy, $endEnergy, $actpos;
     return 1, $message;
  }
  elsif ($npts > 0 && $range > 0 && $togo > $tolerance) {
     $message = sprintf '*** '.$script.': energy '.$axisEnergy.' did not scan properly.'."\n"
                       .' Begin_energy=%.5f target_end=%.5f actual_end=%.5f',
                       $beginEnergy, $endEnergy, $actpos;
     return 1, $message;
  } else {
     return 0, '';
  }
}	##ENflyScan_wait


#=================================================================


# sub ENflyScan_read ($axisTheta, $beginThetaAct, $ESinTheta,
#                     $cts2egu, $struckPs, $struckNg,
#                     \@inputs,\@Offsets,\@data,[$finalORfly]) {
  sub ENflyScan_read($$$$$$\@\@\@;$) {

### Returns ($status, $actPts) and also fills in \@data (energy,timeline,inputs)
  our ($beamline, $struckName, $struck_ext, $IGNORE_ERRORS);
  our ($ENSCAN_FLAGS);		#reference to a hash with various flags for ENSCAN
  my ($script, $axisTheta, $beginThetaAct, $begin, $ESinTheta);
  my ($cts2egu, $struckPs, $struckNg, $mtrOut, $assy);
  my ($inputs_ref, $Offsets_ref, $data_ref, $finalORfly);
  my ($struck, $status, $endThetaAct, $actPts);
  my ($currTheta, $theta, $dtheta, $dtheta_max, $currTime);
  my (@intensity, @timer, @encoderPs, @encoderNg);
  my ($ninputs, $input, $degree2rad, $i, $j);
  my ($range, $rangeErr, $rangeErr_Rel, $scanStep, $errorThreshold);
  my ($undulator, $IDoffset, $axisEnergy, $beginEnergy);

  $script        = 'ENflyScan_read';
  $axisTheta     = shift(@_);
  $beginThetaAct = shift(@_);
  $ESinTheta     = shift(@_);
  $cts2egu       = shift(@_);
  $struckPs      = shift(@_);
  $struckNg      = shift(@_);
  $inputs_ref    = shift(@_);
  $Offsets_ref   = shift(@_);
  $data_ref      = shift(@_);		#2D-array to fill with data (energy+timer+ninputs)
  $finalORfly    = shift(@_);		#final/fly/flywithread; default is final, which aborts Struck

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if (!defined $ENSCAN_FLAGS->{'has run'})              {$ENSCAN_FLAGS->{'has run'}              = 0;}
  if (!defined $ENSCAN_FLAGS->{'has read mtrList'})     {$ENSCAN_FLAGS->{'has read mtrList'}     = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip shutter close'})   {$ENSCAN_FLAGS->{'skip shutter close'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip shutter open'})    {$ENSCAN_FLAGS->{'skip shutter open'}    = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip calc speed'})      {$ENSCAN_FLAGS->{'skip calc speed'}      = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip set speed'})       {$ENSCAN_FLAGS->{'skip set speed'}       = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip set slew'})        {$ENSCAN_FLAGS->{'skip set slew'}        = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip restore speed'})   {$ENSCAN_FLAGS->{'skip restore speed'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip restore mprg'})    {$ENSCAN_FLAGS->{'skip restore mprg'}    = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip mtr link'})        {$ENSCAN_FLAGS->{'skip mtr link'}        = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip mtr unlink'})      {$ENSCAN_FLAGS->{'skip mtr unlink'}      = 0;}
  if (!defined $ENSCAN_FLAGS->{'skip move to start'})   {$ENSCAN_FLAGS->{'skip move to start'}   = 0;}
  if (!defined $ENSCAN_FLAGS->{'return to start'})      {$ENSCAN_FLAGS->{'return to start'}      = 0;}
#
  if (!defined $ENSCAN_FLAGS->{'linkID'})               {$ENSCAN_FLAGS->{'linkID'}                =0;}
#
  if (defined  $ENSCAN_FLAGS->{'undulator'})   {$undulator   = $ENSCAN_FLAGS->{'undulator'};}
  if (defined  $ENSCAN_FLAGS->{'IDoffset'})    {$IDoffset    = $ENSCAN_FLAGS->{'IDoffset'};}
  if (defined  $ENSCAN_FLAGS->{'assy'})        {$assy        = $ENSCAN_FLAGS->{'assy'};}
  if (defined  $ENSCAN_FLAGS->{'axisEnergy'})  {$axisEnergy  = $ENSCAN_FLAGS->{'axisEnergy'};}
  if (defined  $ENSCAN_FLAGS->{'beginEnergy'}) {$beginEnergy = $ENSCAN_FLAGS->{'beginEnergy'};}

  if (!defined $finalORfly) {$finalORfly = 'flywithread';}
  if (($finalORfly !~ /^final$/i) &&				#abortStruck before reading
#     ($finalORfly !~ /^fly$/i)   &&				#just read
      ($finalORfly !~ /^flywithread$/i)) {                      #pushStruckRead before reading
     print STDOUT '!!! '.$script.': incorrect flag finalORfly='.$finalORfly
                                .'. Presuming "flyWithRead"'."\n";
     $finalORfly = 'flywithread';
  }

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/) {$struck = $beamline.':'.$struck;}

  $degree2rad  = atan2(1.,1.)/45.;

  if ($finalORfly =~ /^final$/i) {
#    print STDOUT $script.': Scan finished!'."\n";
     $ENSCAN_FLAGS->{'has run'} = 1;
     $endThetaAct = &getAxisAct ($axisTheta);			#Read motor position at the end
#    printf STDOUT $script.': Bragg axis end position = %.3f'."\n",$endThetaAct;
     if ($ENSCAN_FLAGS->{'return to start'}) {
### Start reverting motor to the beginning position
### in order to save time on the next scan:
        if    (defined $beginEnergy) {$begin = $beginEnergy;}
        elsif (defined $assy)        {$begin = &theta2E($assy,$beginThetaAct,$ESinTheta);}
        if (defined $begin && defined $axisEnergy) {
           printf STDOUT $script.': Reverting Energy to the beginning=%.3f'."\n", $begin;
           &startMotor ($axisEnergy, $begin);
           if ($ENSCAN_FLAGS->{'linkID'}) {
              &startID($undulator,$beginEnergy+$IDoffset);
           }
        } else {
           print STDOUT "\n".'*** '.$script.': no data to restore initial energy.'."\n\n";
        }
     }
  }

  @$data_ref = ();
#-----------------------------------------------------------------
  $ninputs = @$inputs_ref;
# print STDOUT $script.': Reading Struck...'."\n";
### Read Struck timer:
  $actPts = &readStruckTimerRef ($struck, $struck_ext, \@timer, undef, $finalORfly);
### Read motor position data:
# print STDOUT $script.': Reading positive/negative Theta moves from Struck inputs '.$struckPs.'/'.$struckNg."\n";
  $status = &readStruckInputRawRef ($struck, $struck_ext, $actPts, $struckPs, \@encoderPs, 'fly');
  $status = &readStruckInputRawRef ($struck, $struck_ext, $actPts, $struckNg, \@encoderNg, 'fly');
### Read intensity data:
# print STDOUT $script.': Reading intensity from Struck inputs';
  for ($i=0; $i<$ninputs; $i++) {
     $input = @$inputs_ref[$i];
     $status = &readStruckInputRef ($struck, $struck_ext, $actPts, $input, \@timer, \@intensity, 'fly');
     for ($j=0; $j<$actPts; $j++) {			#Correct for background:
        $$data_ref[$i+2][$j] = $intensity[$j] - @$Offsets_ref[$input-1];
     }
  }

### Erase Struck so that when calling HDflyScan next time, it will be already erased:
  if ($finalORfly =~ /^final$/i) {
     &eraseStruck ($struck);
  }
#-----------------------------------------------------------------
### Calculate coordinates:
  $currTheta  = $beginThetaAct;					#this is $axisTheta
  $currTime   = 0.;						#starting time
  $dtheta_max = 0;
  for ($i=0; $i<$actPts; $i++) {
     $dtheta = ($encoderPs[$i]-$encoderNg[$i])*$cts2egu;	#convert encoder to theta
#    if ($finalORfly =~ /^final$/i && $actPts > 10) {print STDOUT $i,'  PS=',$encoderPs[$i],'  NG=',$encoderNg[$i],"\n";}
     if (abs($dtheta) > $dtheta_max) {$dtheta_max = abs($dtheta);}
     $theta  = $currTheta + 0.5*$dtheta;			#Calc.middle point of each interval
     $currTheta += $dtheta;					#Calc.end point of each interval:
### Recalculate coordinates from Theta into Energy:
     if ($theta != 0.) {
        $$data_ref[0][$i] = $ESinTheta/sin($theta*$degree2rad);
     } else {
        $$data_ref[0][$i] = 1.E+10;				#some big number
     }
### Timeline data:
     $$data_ref[1][$i] = $currTime + 0.5*$timer[$i];		#time at the middle of Struck bin
     $currTime  += $timer[$i];					#time at the end of Struck bin
  }
  if ($finalORfly =~ /^final$/i && $actPts > 10 && $dtheta_max == 0) {
     printf STDOUT '!!! '.$script.': Not getting encoder pulses'
                   .' from Struck='.$struck.' inputs '.$struckPs.' and '.$struckNg.'.'
                   .' Possible reasons: virtual moror is unlinked,'
                   .' or pulse converter is off or pulse converter'
                   .' is connected to incorrect Struck inputs'
                   .' (expecting %d and %d)'."\n"
                   .' cts2egu=%.3g, actPts=%d'."\n", $struckPs,$struckNg,$cts2egu,$actPts;
  }
#-----------------------------------------------------------------
### Test the consistency of scan interval:
  if ($finalORfly =~ /^final$/i) {
     $rangeErr = $currTheta - $endThetaAct;
     $range    = abs ($endThetaAct - $beginThetaAct);
     if ($range == 0) {
        print  STDOUT '!!! '.$script.': zero scan range or motor did not move!'."\n";
        return 0, $actPts;
     }
     $rangeErr_Rel = 100. * $rangeErr / $range;
     if ($actPts> 0) {$scanStep = $range / $actPts;}
     else            {$scanStep = $range;}
     if ($actPts > 0) {$errorThreshold = &maxval(1.,50./$actPts);}
     else             {$errorThreshold = 50.}
     if (abs($rangeErr_Rel) > $errorThreshold) {
        printf STDOUT '!!! '.$script.': END theta  error=%.3f degr. (scan step=%.5f degr.)'."\n", $rangeErr, $scanStep;
        printf STDOUT '!!! '.$script.': Scan range error=%.3f%s'."\n",$rangeErr_Rel,'%';
        printf STDOUT '!!! '.$script.': Start Theta (EPICS data) = %.5f degr.'."\n",$beginThetaAct;
        printf STDOUT '!!! '.$script.': End Theta   (EPICS data) = %.5f degr.'."\n",$endThetaAct;
        printf STDOUT '!!! '.$script.': End Theta (Struck calc.) = %.5f degr.'."\n",$currTheta;
        printf STDOUT '!!! '.$script.': Counts to Eng.units fctr = %.5g'."\n",$cts2egu;
        printf STDOUT '!!! '.$script.': Actual number of scan points = %d'."\n",$actPts;
        if (defined $ENSCAN_FLAGS->{'assy'} && defined $ENSCAN_FLAGS->{'mtrOut'}) {
           $mtrOut = $ENSCAN_FLAGS->{'mtrOut'};
           $assy   = $ENSCAN_FLAGS->{'assy'};
           printf STDOUT '!!! '.$script.': Virtual motor-%d link status=%d'."\n",$mtrOut,&isMtrOutLinked($assy,$mtrOut);
        } else {
           print  STDOUT '!!! '.$script.': Virtual motor or assembly information is lost'."\n";
        }
#       if (! $IGNORE_ERRORS) {
##         &press_key_continue();
           return -2,$actPts;
#       }
     }
  }
#-----------------------------------------------------------------

# print STDOUT $script.': Scan data retrieved!'."\n";
  return 0, $actPts;

}	##ENflyScan_read


#=================================================================


# 43. ($status,$actPts,$pos) = &findPosHD ($assy,$axis,$begin,$end,$npts,$scanTime,$medm,
#                                          \@Offsets,$input,$target,$direction,\@data,[\%flags,$scaler]);
#
# NOTES: -- fills \@data=(@positions,@timeline,@input)
#        -- if scanTime=0, tries to use max.speed
#        -- if return status#0 fills global $messageHDSCAN

  sub findPosHD ($$$$$$$\@$$$\@;\%$) {

### This function implements on-the-fly HD-sync scan with PMAC motors and Struck scaler.
### It analyzes scan data on the fly and when the intensity reaches $target (from either low
### or high depending on $direction) it aborts the scan and drives motor to the position where
### target intensity was reached.

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext);
  our ($findPos2ndTime, $HDSCAN_FLAGS, $IGNORE_ERRORS, $messageHDSCAN);
  my ($script, $assy, $axis, $begin, $end, $npts, $scanTime, $medm);
  my ($Offsets_ref, $input, $target, $direction, $data_ref, %dir, $flagsref);
  my ($DECREASING, $INCREASING, $h_speed, $l_speed, $cnt_time, $maxPts);
  my ($status,$message,$beginAct,$cts2egu,$mtrOut,$struckPs,$struckNg,$dwell);
  my ($struck, @inputs, $wait_step, $wait_, $delayTime, $acquiring, $scaler);
  my ($inpos, $runprg, $positioned, $nRec, $actPts, $scanInProgress, $j,  $t0);
  my ($found, $found_max, $y_curr, $position, $y_max, $y_min, $max_struck_usage);
  my ($pt_left, $pt_right, $pt_diff, $x_left, $x_right, $y_left, $y_right, $border);
  my ($scanstep, $pmac);

  $script      = 'findPosHD';
  $assy        = shift(@_);
  $axis        = shift(@_);
  $begin       = shift(@_);	#start scan position
  $end         = shift(@_);	#end scan position
  $npts        = shift(@_);	#number of points on fast scan
  $scanTime    = shift(@_);	#scan time in seconds (0=take scan at max speed)
  $medm        = shift(@_);	#1=open MEDM, 0=not
  $Offsets_ref = shift(@_);	#pointer to Struck offsets array
  $input       = shift(@_);	#Struck scaler input for detecting intensity
  $target      = shift(@_);	#target intensity to stop search
  $direction   = shift(@_);	#targeted intensity behavior: 0=decreasing, 1=increasing
  $data_ref    = shift(@_);	#pointer to the array where data will be stored (@positions,@timeline,@input)
  $flagsref    = shift(@_);	#reference to an optional hash with program flags (options)
  $scaler      = shift(@_);	#name of Struck as Joerger

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}
  if (!defined $flagsref->{'can change step'})     {$flagsref->{'can change step'}=0;}
  if (!defined $flagsref->{'max struck usage'})   {$flagsref->{'max struck usage'}=90;}

  if (!defined &gettimeofday || !defined &tv_interval) {
#    use Time::HiRes qw(gettimeofday tv_interval usleep time sleep);
  }

  $struck = $struckName;
  if ($struck !~ /^${beamline}:/)     {$struck = $beamline.':'.$struck;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }
  if (defined $shutter) {
     print STDOUT 'Closing shutter ['.$shutter.']'."\n";
     &closeShutter ($shutter,$shutter_readback);
  } else {
     print STDOUT '!!! '.$script.': Shutter is not used!'."\n";
  }
  $DECREASING = 0;
  $INCREASING = 1;
  $dir{$DECREASING} = 'Decreasing';
  $dir{$INCREASING} = 'Increasing';
  $status = 0;
  $messageHDSCAN = '';
  @$data_ref = ();
  print  STDOUT 'Entering '.$script.':'."\n";
  printf STDOUT '  motor='.$axis.', range=[%.3g : %.3g], npts=%d'."\n", $begin, $end, $npts;
  printf STDOUT '  scaler=Struck, input='.$input.', target_intensity=%.3g'."\n", $target;
  printf STDOUT '  search_time=%.3g, intensity_direction=', $scanTime;
  if ($direction eq $DECREASING || $direction eq $INCREASING) {
     print STDOUT $dir{$direction}."\n";
  } else {
     print STDOUT '**UNKNOWN**'."\n";
     $messageHDSCAN = 'Intensity search direction is **UNKNOWN**';
     return 1,0,0;
  }
#-----------------------------------------------------------------
### The globals below are defined in beamline.h:
  $maxPts = &getStruckNmax($struck, $struck_ext);
  if (!defined $findPos2ndTime) {$findPos2ndTime=0;}
#-----------------------------------------------------------------
### Begin parameters consistency tests

  if ($npts < 3) {
     $messageHDSCAN = 'Too low number of points in the scan, npts='.$npts."\n";
     print STDOUT '*** '.$script.': '.$messageHDSCAN;
     return 1,0,0;
  }
  $max_struck_usage = $flagsref->{'max struck usage'};
  if ($max_struck_usage > 1) {$max_struck_usage = $max_struck_usage/100.}
  if ($npts > $max_struck_usage*$maxPts && $flagsref->{'can change step'}) {
     printf STDOUT '!!! '.$script.': Too many scan points=%d (max=%d)'."\n"
        .'  Increasing scan step by  factor of %.2f'."\n", $npts,int($max_struck_usage*$maxPts),$npts/$maxPts;
        $npts = int($max_struck_usage*$maxPts);
  }

  $h_speed = &getAxisHiSpeed ($assy, $axis);
  if ($scanTime > 0) {
     $l_speed = abs($end - $begin) / $scanTime;
     if ($l_speed > $h_speed) {
        printf STDOUT '!!! '.$script.': too short scanTime=%.3gs is requested.'
                            .' Motor cannot run that fast!'."\n", $scanTime;
        $scanTime = abs($end - $begin) / $h_speed;
        $l_speed = $h_speed;
        printf STDOUT '!!! '.$script.': scan time increased to scanTime=%.3gs'."\n", $scanTime;
     }
  } else {						#auto-speed mode: set to maximum
     $scanTime = abs($end - $begin) / $h_speed;
     $l_speed = $h_speed;
     printf STDOUT $script.': automatic scan time calculated to be scanTime=%.3gs'."\n", $scanTime;
  }
  $cnt_time = $scanTime / $npts;
### End parameters consistency tests
#-----------------------------------------------------------------

  if ($findPos2ndTime == 0) {$HDSCAN_FLAGS->{'has run'}=0;}
### This also opens shutter and starts the scan:
  ($status,$message,$beginAct,$cts2egu,$mtrOut,$struckPs,$struckNg,$dwell) =
                          &HDflyScan_setup($assy,$axis,$begin,$end,$npts,$cnt_time,$scaler);
  if ($status) {
     print STDOUT '*** '.$script.':'."\n".$message."\n";
     $messageHDSCAN = $message;
     return $status, 0, 0;
  }

  &setStruckReadRate ($struck,'Passive');
  if ($medm) {&openStruckMedm($struck,$struck_ext,$input);}

### Monitor scan progress:
  @inputs     = ($input);
  $wait_step  = 0.01;
  if (defined &gettimeofday) {$t0 = [&gettimeofday];}
  else                       {$t0 = time();}
  $wait_      = 0;
  $acquiring  = 1;
  $inpos      = 0;
  $runprg     = 1;
  $positioned = 0;
  $found      = 0;					#counter for finding target
  $found_max  = 3;					#max times to check that the condition is satisfied
  $delayTime  = $dwell;
  if ($delayTime < 0.1) {$delayTime = 0.1;}		#delay motors start so Struck can catchup
  $delayTime = 5*$delayTime;
  $nRec       = 0;
  $y_max      = -1.E+37;
  $y_min      = +1.E+37;
  $scanInProgress = 1;
  while (1) {
     select(undef,undef,undef,$wait_step);		#sleep($wait_step);
#    $wait_ += $wait_step;
     if (defined &tv_interval) {$wait_ = &tv_interval($t0);}
     else                      {$wait_ = time() - $t0;}
     ($status,$acquiring) = Pezca::GetDouble($struck.':Acquiring');
     ($status,$inpos)     = Pezca::GetDouble($assy.':InPos');
     ($status,$runprg)    = Pezca::GetDouble($assy.':RunPrg');
     if ($inpos == 1) {$positioned++;}
     else             {$positioned=0;}
     if (($acquiring!=1 || ($runprg==0 && $positioned>=3)) && $wait_> $delayTime) {
        $scanInProgress = 0;
     }
     ($status,$actPts) = &HDflyScan_read ($axis,$beginAct,$cts2egu,$struckPs,$struckNg,
                                          \@inputs,$Offsets_ref,$data_ref,'flywithread');
     if ($actPts > $nRec) {
        $nRec = $actPts;
        $y_curr  = $$data_ref[2][$nRec-1];		#0=positions, 1=timer, 2=intensity
        if ($y_curr > $y_max) {$y_max = $y_curr;}
        if ($y_curr < $y_min) {$y_min = $y_curr;}
        if (($y_curr <= $target && $direction == $DECREASING) ||
            ($y_curr >= $target && $direction == $INCREASING)) {$found++;}	#position found once again
        else                                                   {$found=0;}	#skip intensity glitches
     }
     if ($scanInProgress == 0 || $found >= $found_max) {last;}
  }							#end of scan
  if (defined $shutter) {&closeShutter ($shutter,$shutter_readback);}
  &abortAssy ($assy, 1, $axis);				#also restore speed and motion prg.
  &abortStruck ($struck);
  &unlinkMtrOut($assy, $mtrOut);
  &setStruckReadRate ($struck,'2 second');
  if ($y_max < $y_min) {
     print  STDOUT '!!! '.$script.': Zero data points in scan data'."\n";
     $y_max = $y_min = 0;
  }
  if ($found < $found_max) {
     $messageHDSCAN = sprintf
                         'Scan finished before position found!'."\n"
                        .' --- Exit Flags ---'."\n"
                        .'   scaler acquiring='.$acquiring."\n"
                        .'   motor RunPrg='.$runprg."\n"
                        .'   motor InPos='.$positioned."\n"
                        .'   scan time=%.2fs'."\n"
                        .' --- Scan Stats ---'."\n"
                        .'   search satisfied '.$found.' times of required '.$found_max."\n"
                        .'   y_max=%1.0f, y_min=%1.0f, target=%1.0f'."\n",
                                                   $wait_, $y_max, $y_min, $target;
     print  STDOUT '!!! '.$script.': '.$messageHDSCAN;
     return 1,0,0;
  }
  printf STDOUT 'Search finished: scanPts='.$actPts.', time=%.2fs'."\n", $wait_;
  printf STDOUT ' (found='.$found.'/'.$found_max
               .', y_max=%1.0f, y_min=%1.0f target=%1.0f)'."\n",
                                                     $y_max, $y_min,$target;
### Check x-data for consistency: is the PMAC converter hooked?
  $scanstep = ($end-$begin)/$npts;
  if (abs($$data_ref[0][$actPts-1]-$$data_ref[0][0]) < 0.5*abs($scanstep)) {
     $pmac = &getPmac($assy);
     $messageHDSCAN = sprintf
                         'Scan X-data does not show changes:'."\n"
                        .'       scan points=%d'."\n"
                        .'         scan step=%.3f'."\n"
                        .'       x-data(beg)=%.3f'."\n"
                        .'       x-data(end)=%.3f'."\n"
                        .'   reported x-diff=%.3f'."\n"
                        .'   expected x-diff=%.3f'."\n\n"
                        .' *** Please check if link to Struck and pulse converter for pmac=%s are OK.'."\n",
                        $actPts, 
                        $scanstep,
                        $$data_ref[0][0], 
                        $$data_ref[0][$actPts-1],
                        ($$data_ref[0][$actPts-1]-$$data_ref[0][0]),
                        ($scanstep*$actPts),
                        $pmac;
     print  STDOUT '!!! '.$script.': '.$messageHDSCAN;
     return 1,0,0;
  }

### Re-search the target intensity from the left and right sides of data array:
  $pt_left  = 0;
  $pt_right = $actPts-1;
  for ($j=0; $j < $actPts; $j++) {
     $y_curr = $$data_ref[2][$j];
     if (($y_curr <= $target && $direction == $DECREASING) ||
         ($y_curr >= $target && $direction == $INCREASING)) {
        $pt_left = $j;
        $y_left  = $y_curr;
#       printf STDOUT $script.': found  left target point='.$pt_left
#                            .', intensity=%1.0f (target=%1.0f)'."\n", $y_left, $target;
        last;
     }
  }
  for ($j=$actPts-1; $j>=0; $j--) {
     $y_curr = $$data_ref[2][$j];
     if (($y_curr >= $target && $direction == $DECREASING) ||
         ($y_curr <= $target && $direction == $INCREASING)) {
        $pt_right = $j;
        $y_right  = $y_curr;
#       printf STDOUT $script.': found right target point='.$pt_right
#                            .', intensity=%1.0f (target=%1.0f)'."\n", $y_right, $target;
        last;
     }
  }
# for ($j=0; $j<$actPts; $j++) {printf STDOUT '%4d  %4.3g  %4.3g'."\n", $j,$$data_ref[0][$j],$$data_ref[2][$j];}

  $pt_diff = abs($pt_right-$pt_left);
  if ($pt_diff > 2) {
     print STDOUT '!!! '.$script.': left and right thresholds differ by '.$pt_diff
                        .' pts; left='.$pt_left.' right='.$pt_right."\n";
     print STDOUT '               Possible noise on the intensity data!'."\n";
  }
  $border = 0;
  if ($pt_left  == 0 || $pt_right == 0) {
     print STDOUT '!!! '.$script.': found point is at the beginning of scan interval'."\n";
     $border = 1;
  }
  if ($pt_left  == $actPts-1 || $pt_right == $actPts-1) {
     print STDOUT '!!! '.$script.': found point is at the end of scan interval'."\n";
     $border = 2;
  }
#  if ($border != 0) {
#     print STDOUT "\n\n";
#     for ($j=0; $j < $actPts; $j++) {printf STDOUT ' %1.0f',$$data_ref[2][$j];}
#     print STDOUT "\n\n";
#  }

  $x_left  = $$data_ref[0][$pt_left];
  $x_right = $$data_ref[0][$pt_right];
  printf STDOUT $script.': pt(left)=%d, x(left)=%.3g, pt(right)=%d. x(right)=%.3g'."\n",
                                                 $pt_left, $x_left, $pt_right, $x_right;
  if ($y_left != $y_right) {
     $position = $x_left + ($x_right - $x_left) * ($target - $y_left)
                                                / ($y_right - $y_left);
  } else {
     $position = 0.5*($x_left + $x_right);
  }
  printf STDOUT $script.': target_intensity=%1.0f for motor='.$axis
                       .' found at position=%.3g'."\n", $target, $position;
  $findPos2ndTime = 1;
  return 0,$actPts,$position;
}
