#!/usr/bin/perl -W
#--------------------------------------------------------------
# These are subs for fetching data from x-server.gmca.aps.anl.gov
#
#     			Author: Sergey Stepanov
#
# Version-1.0:  2018/03/28 using DBI
#--------------------------------------------------------------
# List of functions:
# ^^^^^^^^^^^^^^^^^
#  1. $FWHM = &getBraggPeakFWHM($energy,$crystal,$hkl,\$msg,['doublecrystal|Darwin','sigma|pi]); 	#returns $FWHM in urad or undef on error & status in $msg
#--------------------------------------------------------------
# use warnings::unused;
  use strict;
  use LWP::Simple;		# World-Wide Web library for Perl (libwww-perl)
  use LWP::UserAgent;

  our $xrayserver = 'http://x-server.gmca.aps.anl.gov';

#=================================================================

# sub getBraggPeakFWHM($energy,$crystal,$hkl,\$msg,['doublecrystal|Darwin','sigma|pi]) {
sub getBraggPeakFWHM($$$\$;$$) {								#returns $FWHM in urad or undef on error & status in $msg

  our ($xrayserver);
  my ($script, $energy, $crystal, $hkl, $msgref, $curve, $sgpi);
  my ($url, %FORM, $request, @ind, $i);
  my ($buffer, @content, $string, $keyword);

  $script = 'getBraggPeakFWHM';
  $url    = $xrayserver.'/cgi/x0h_form.exe';

  $energy  = shift(@_);
  $crystal = shift(@_);
  $hkl     = shift(@_);
  $msgref  = shift(@_);		$$msgref = '';
  $curve   = shift(@_);		if (!defined $curve) {$curve='Darwin';}
  $sgpi    = shift(@_);		if (!defined $sgpi)  {$sgpi='sigma';}

  if ($curve ne 'Darwin' && $curve ne 'doublecrystal') {
     $$msgref = '*** '.$script.': incorrect parameter curve='.$curve;
     return;
  }
  if ($sgpi ne 'sigma' && $sgpi ne 'sg' && $sgpi ne 'pi') {
     $$msgref = '*** '.$script.': incorrect parameter sigmapi='.$sgpi;
     return;
  }
  @ind = split(/[\s+,]/, $hkl);
  if (@ind != 3) {
     $$msgref = '*** '.$script.': incorrect parameter hkl='.$hkl;
     return;
  }
  foreach $i (@ind) {
     if (! &is_int($i)) {
        $$msgref = '*** '.$script.': incorrect parameter hkl='.$hkl.', index='.$i;
        return;
     }
  }
  $FORM{'xway'}  = 2;
  $FORM{'wave'}  = $energy;
  $FORM{'coway'} = 0;
  $FORM{'code'}  = $crystal;
 ($FORM{'i1'},$FORM{'i2'},$FORM{'i3'}) = @ind;
  $FORM{'df1df2'} = -1;					# auto DB
  $FORM{'detail'}  = 0;					# 0=no coords, 1=print coords
  $FORM{'modeout'} = 1;					# 0=html, 1=quasytext

  $request = '';
  foreach $i (keys %FORM) {$request .= '&'.$i.'='.$FORM{$i};}
  $request =~ s/^\&/\?/;				# replace 1st "&" by "?"
  $buffer = get($url.$request);
  if (!defined $buffer) {
     $$msgref = '*** '.$script.': failed to get response to: '.$url.$request;
     return;
  }
  @content = split /[\r\n]/, $buffer;			# split response into lines

  if ($curve eq 'Darwin') {$keyword = 'FWHMurad';}
  else                    {$keyword = 'FWHM2urad';}
  if ($sgpi eq 'sigma' || $sgpi eq 'sg') {$keyword .= '_sg';}
  else                                   {$keyword .= '_pi';}

  foreach $string (@content) {
     if ($string =~ m/$keyword=/) {
        $string =~ s/^.*$keyword=\s*//;
	$string =~ s/\s+$//;
        if (! &is_float($string)) {
           $$msgref = '*** '.$script.': incorrect response='.$string.' to request: '.$url.$request;
           return;
	} else {
	   return $string;				# FWHM
        }	
     }
  }
  $$msgref = '*** '.$script.': cannot find "'.$keyword.'" in response to: '.$url.$request;
     return;
}
