#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script displays the AllowedUsers string from /etc/ssh/sshd_config
#
# ATTENTION: this can only run via sudo -- add it to /etc/sudoers
#
#         	Author: Sergey Stepanov
#
# Version-1.0: 2013/03/17 initial implementation (parsing /etc/ssh/sshd_config)
# Version-2.0: 2013/04/08 added parsing /var/log/secure
# Version-2.1: 2016/01/27 added check that the script runs as root or via sudo
# Version-3.0: 2017/03/23 added checking vesion of NX client
# Version-3.1: 2017/07/06 replaced findkey & grep by findkey2 & read_entire_file
# Version-3.2: 2017/07/07 added check for invalid user and changed reading sshd_config & secure to read_entire_file
# Version-3.3: 2018/11/25 added printing expiration dates
# Version-3.4: 2019/04/06 added clientBrowser for detecting webplayer
# Version-3.5: 2019/06/24 added last_editor & host_uname (who changed the record last time)
# Version-3.6: 2020/05/04 added checking failed NX sessions
# Version-3.7: 2020/06/01 added showing time for how long the running session is disconnected
# Version-3.8: 2020/06/15 added analyzing nxhtd-error.log for login attempts from non-allowed IPs
# Version-4.0: 2020/08/31 added analyzing iptables rules for ssh
# Version-4.1: 2020/09/11 added cidr2iprange for iptables
# Version-4.2: 2020/10/29 added calls to ip2domain to resolve domain of IP blocks specified by users
# Version-4.3: 2021/01/30 added support for NX-7 (Redis)
# Version-4.4: 2021/02/21 changed output format and added some fields
# Version-4.5: 2021/03/17 fixed disconnected sessions
# Version-4.6: 2021/04/01 changed to using 'nxserver --history' instead of redis
#                         to find todays closed and failed sessions (much faster)
# Version-4.7: 2022/02/01 added ndays for reading logs; added iptables check
# Version-4.8: 2022/02/02 added $nxreal_state: printing "linked to" for any linked sesstion
# Version-4.9: 2022/02/03 added sending email per command line request
# Version-5.0: 2022/02/03 added &show_current_users_sql()
#--------------------------------------------------------------
  use Socket;                                                   #inet_ntoa (unpacks binary IP into d.d.d.d, legacy)
  use strict;
  use DBI;                                                      #added for 3.3 (reading users SQL data)
  use MIME::Lite;
  use warnings::unused;

  my ($PEZCA, $script, $key, $key2, $key3, $sshcfg, $exist);
  my (@ct, $mday, @months, $mnumb, $month, $year, $date, $today);
  my ($host, $nxdir, $nxserver, $nxrunning, $nxclosed, $nxfailed);
  my ($nxstart, $nxend, $nxsuspend, $nxdur, $nxdisc, @words, $nwords);
  my ($secure, $secure_old, @securelist, $beglog_epoch, $log_epoch);
  my ($htderr, %htdlist, $string, $rec, $user, $home, $ndays, $nbuf);
  my (@lines, $line, @buf, @connected, @disconnected, $mode, $msg);
  my (@remote, $nremote, $usr, $i, $j, $nsessions, $packed_ip, $ip);
  my ($nxuser, $nxip, $nxip2, $nxid, $nxversion, $nxOS, $nxtype);
  my ($nxclienttype, $nxbrowser, $nxreal, $nxerror, $nxreal_state);
  my ($bl, $id, $id_sql, $dbhost, $dbuser, $dbpass, $dtbase, $dbtable);
  my ($dbhandle, $sthandle, $hash, $nrecords, $epoch, $day, $status);
  my ($endstamp, $endepoch, $last_editor, $host_email, $host_uname);
  my ($iptbl_exists, $iptblcfg, $iptbllog, %ipblock, $ipv4_start, $ipv4_end);
  my ($ipspec, $ipdomain, $ip2domain, @dmn, $arg, $syntax, $resolve);
  my ($serverVersion, $serverMajor, $cmd, @seslist, $nses, $details);
  my ($begday_stamp, $begday_epoch, @buf2, $use_redis_method);
  my ($id2, $id2_sql, $id_sql_found);
  my (@inets, $inet, $gmcanet, $gmcaeth, $iptbeth, $str);
  my ($email, $ebuf, $erec, $elist, @users_sql);
  my ($iprange, $remote_user, $expire);

  select STDOUT; $|=1;					        #set unbuffered output
  select STDERR; $|=1;					        #set unbuffered output

  $script = $0;
  $script =~ s/\\/\//g;					        #change \ to /
  $script =~ s/^.*\///g;				        #remove everything before last /
  $script =~ s/\.pl$//i;

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;
  if ($PEZCA !~ /\/$/) {$PEZCA .='/';}
  do $PEZCA.'Common/common_subs.pl';

#---------------------------------------------------------------------
  if ($^O =~ /Win/i) {die $script.': cannot run under Windows.'."\n";}
  if    (defined $ENV{USER})     {$user = $ENV{USER};}
  elsif (defined $ENV{USERNAME}) {$user = $ENV{USERNAME};}
  else                           {$user = 'unknown';}
  if    (defined $ENV{HOME})     {$home = $ENV{HOME};}
  else                           {$home = 'unknown';}

### When initial user is not root, but he issued "su",
### the environment "USER" is preserved, but "HOME" is not.
  if ($user ne 'root' && $home !~ /^\/root/) {
     print STDOUT "\n".'User='.$user;
     print STDOUT "\n".'Home='.$home;
#    print STDOUT "\n".'This script should be executed as root or via sudo'."\n\n";
     print STDOUT "\n".'Re-launching as root via sudo...'."\n\n";
     exec('/usr/bin/sudo','-n',$0,@ARGV);
     exit;
  }
  if    (defined $ENV{HOSTNAME}) {$host = $ENV{HOSTNAME};}
  elsif (defined $ENV{HOST})	 {$host = $ENV{HOST};}
  else                           {$host=`hostname`; $host=~s/\n//g;}
  if (!defined $host) {print STDOUT '!!! '.$script.': HOSTNAME is undefined!'."\n\n"; $host='undef';}
  $host = lc($host);
  $host =~ s/\Q.gmca.aps.anl.gov\E$//;

  $resolve = 1;						#resolve domains for user-specified IP addresses
  $details = 0;						#resolve domains for user-specified IP addresses
  $email   = 0;
  $ndays   = 0;
  $elist   = 'sstepanov@anl.gov'
#          . ',makarov@anl.gov'
#          . ',mhilgart@anl.gov'
#          . ',qxu@anl.gov'
           ;
  $ebuf    = '';
  $syntax = 'Syntax: '.$script.' [showdomain|skipdomain] [details|short] [email] [ndays=n]'."\n";
  foreach $arg (@ARGV) {
     if    ($arg =~ /^showdomains?$/i) {$resolve = 1;}
     elsif ($arg =~ /^skipdomains?$/i) {$resolve = 0;}
     elsif ($arg =~ /^details?$/i)     {$details = 1;}
     elsif ($arg =~ /^short$/i)        {$details = 0;}
     elsif ($arg =~ /^email$/i)        {$email   = 1;}
     elsif ($arg =~ /^ndays=/i){
        $ndays = $arg;
        $ndays =~ s/^ndays=//i;
        if (! &is_int($ndays) || $ndays<0) {die $script.': Incorrect command line argument ['.$arg.'].'."\n\n".$syntax;}
     }
     else {die $script.': Incorrect command line argument ['.$arg.'].'."\n\n".$syntax;}
  }
#-----------------------------------------------------------------------
  $SIG{HUP}  = 'signal_handler';	#(1) hangup (see signum.h)
  $SIG{INT}  = 'signal_handler';	#(2) interrupt (see signum.h)
  $SIG{QUIT} = 'signal_handler';	#(3) quit (see signum.h)
# $SIG{KILL} = 'signal_handler';	#(9) this is what is sent by kill -9 -- unblockable
  $SIG{TERM} = 'signal_handler';	#(15) terminate -- sent by kill without paramaters
# $SIG{STOP} = 'signal_handler';	#(19) stop -- unblockable
#-----------------------------------------------------------------------
  if    ($host =~ /^bl1/) {$bl=1; $dbhost = 'bl1upper.gmca.aps.anl.gov'; $id = '23ID-D'; $id2 = '23ID-B';}
  elsif ($host =~ /^bl2/) {$bl=2; $dbhost = 'bl1upper.gmca.aps.anl.gov'; $id = '23ID-B'; $id2 = '23ID-D';}
  elsif ($host =~ /^bl3/) {$bl=3; $dbhost = 'bl3upper.gmca.aps.anl.gov'; $id = '23BM';}
  else                    {$bl=0;}
  if    ($host =~ /ws2/ || $host =~ /ws6/) {$day='first';}
  elsif ($host =~ /ws5/ || $host =~ /ws8/) {$day='second';}
# elsif ($host =~ /ws5/)                   {$day='second';}
  else                                     {$day='unknown'; $bl = 0;}
  if ($bl != 0) {
     $dbuser  = 'dhs';						#for SQL
     $dbpass  = '';                                             #for SQL
     $dtbase  = 'gmca_accounts';			        #for SQL
     $dbtable = 'accounts';					#for SQL
     $id_sql  = $id; $id_sql =~ s/^23//; 			#for SQL (e.g. $id='23ID-D';  $id_sql='ID-D')
     if ($bl == 1 || $bl == 2) {$id2_sql = $id2; $id2_sql =~ s/^23//;}	#(e.g. $id2='23ID-B'; $id2_sql='ID-B')
     $dbhandle = DBI->connect('DBI:mysql:host='.$dbhost.';database='.$dtbase,$dbuser,$dbpass,{mysql_connect_timeout=>2});
     if (!defined $dbhandle) {
        $erec = '*** '.$script.': cannot connect to mySQL DB='
                      .$dtbase.' on '.$dbhost."\n";
        print STDOUT $erec; $ebuf .= $erec;
        $bl = 0;
     }
  }
#---------------------------------------------------------------------
  $epoch = time();
  $sshcfg    = '/etc/ssh/sshd_config';
  if    (-e '/var/log/secure')   {$secure = '/var/log/secure';}	#we need to add here secure-date files too
  elsif (-e '/var/log/auth.log') {$secure = '/var/log/auth.log';}
  else {die $script.': cannot find ssh log.'."\n\n";}
  $nxdir     = '/usr/NX';
  $nxserver  = $nxdir.'/bin/nxserver';
  $nxrunning = $nxdir.'/var/db/running';
  $nxclosed  = $nxdir.'/var/db/closed';
  $nxfailed  = $nxdir.'/var/db/failed';
  $htderr    = $nxdir.'/var/log/nxhtd-error.log';
# if (! system('systemctl list-units --full -all | grep -Fq "iptables.service"')) {$iptbl_exists=1;}
  if (! system('systemctl is-enabled --quiet iptables.service 2>/dev/null'))      {$iptbl_exists=1;}
  else                                                                            {$iptbl_exists=0;}
  if ($iptbl_exists) {
     if    (-e '/etc/iptables/rules.v4')  {$iptblcfg  = '/etc/iptables/rules.v4';}
     elsif (-e '/etc/sysconfig/iptables') {$iptblcfg  = '/etc/sysconfig/iptables';}
     else  {die $script.': cannot find iptables config.'."\n\n";}
     if    (-e '/var/log/iptables.log')   {$iptbllog  = '/var/log/iptables.log';}
     elsif (-e '/var/log/kern.log')       {$iptbllog  = '/var/log/kern.log';}
     else  {die $script.': cannot find iptables log.'."\n\n";}
  }
  $ip2domain = &Which('ip2domain');
# $use_redis_method = 1;				#defined to use redis instead of history
  if ($ndays > 0) {$use_redis_method = 1;}

### ========================================================================
### Query SQL for currently allowed remote users:
### ========================================================================

  &show_current_users_sql();

### ========================================================================
### Parse /etc/ssh/sshd_config for AllowUsers:
### ========================================================================
  if (! -e $sshcfg) {&signal_handler($script.': file "'.$sshcfg.'" does not exist.'."\n");}
  if (! -r $sshcfg) {&signal_handler($script.': file "'.$sshcfg.'" is not readable.'."\n");}

  if (&read_entire_file($sshcfg,\@buf)) {&signal_handler($script.': cannot open "'.$sshcfg.'" for reading.'."\n");}
  $exist = 0;
  $key = 'AllowUsers';
  foreach $string (@buf) {
     $string =~ s/\t/ /g;	                                #replace tabs by spaces
     $string =~ s/\s+/ /g;	                                #compress spaces ("+" means match 1 or more times)
     $string =~ s/^ *//g;	                                #remove beginning-of-the-line spaces
     $string =~ s/ *$//g;	                                #remove end-of-the-line spaces
     $string =~ s/^#.*$//g;	                                #remove comment lines
     if ($string =~ /^${key}/) {
        if ($exist == 0) {
           $erec =  "\n".'## AllowUsers in ['.$sshcfg.']:'."\n";
           print STDOUT $erec; $ebuf .= $erec;
        }
        $exist++;
        $string =~ s/^${key} //;
        ### Split into permanent and temp
        @words = split(/DO_NOT_MODIFY_AFTER_HERE/,$string);
        $nwords = @words;
        if ($nwords == 0) {                                     #no temp users
           if ($string =~ /DO_NOT_MODIFY_AFTER_HERE/) {
              $erec = '  Temp: none'."\n";
              print STDOUT $erec; $ebuf .= $erec;
           }
        }
        elsif ($nwords == 1) {                                  #no temp users
           if ($string =~ /DO_NOT_MODIFY_AFTER_HERE/) {
              $words[1] = 'none';
              $nwords   = 2;
           }
        } 
        elsif ($nwords > 1) {                                   #there are some temp users
           $words[1] =~ s/^\s+//;			        #remove beginning-of-the-line spaces
        }
        if ($nwords > 1) {                                      #there are some temp users
           @remote = split(/\s+/,$words[1]);
           @remote = sort @remote;
           $nremote = @remote;
           if ($resolve && defined $ip2domain) {
              $erec = 'NOTE: if next step takes long time, run: "'.$script.'.pl skipdomain"'."\n";
              print STDOUT $erec; $ebuf .= $erec;
           }
           for ($i=0; $i<$nremote; $i++) {
              ($usr,$ipspec) = split/\@/,$remote[$i];
              if ($resolve && defined $ip2domain) {
                 if ($ipspec eq '127.0.0.1') {$ipdomain = 'localhost';}
                 else {$ipdomain = `$ip2domain $ipspec`; $ipdomain =~ s/\n//;}
                 if (length($ipdomain) > 0) {
                    @dmn = split(' ',$ipdomain);
                    @dmn = &uniq(@dmn);
                    $ipdomain = join(' ',@dmn);
                    $remote[$i] .= ' ('.$ipdomain.')';
                 }
              }
              for ($j=$i+1; $j<$nremote; $j++) {	        #group all IPs for given user
                 if ($remote[$j] =~ /^$usr/) {
                    ($usr,$ipspec) = split/\@/,$remote[$j];
                    $remote[$i] .= "\n".'        '.$remote[$j];
                    splice @remote, $j, 1;
                    $j--;
                    $nremote--;
                    if ($resolve && defined $ip2domain) {
                       if ($ipspec eq '127.0.0.1') {$ipdomain = 'localhost';}
                       else {$ipdomain = `$ip2domain $ipspec`; $ipdomain =~ s/\n//;}
                       if (length($ipdomain) > 0) {
                          @dmn = split(' ',$ipdomain);
                          @dmn = &uniq(@dmn);
                          $ipdomain = join(' ',@dmn);
                          $remote[$i] .= ' ('.$ipdomain.')';
                       }
                    }
                 }
              }
              if ($bl != 0) {
                 $id_sql_found = $id_sql;
                 $sthandle = $dbhandle->prepare('SELECT * FROM '.$dbtable.' where username="'.$usr.'" and beamline="'.$id_sql.'"');
                 $nrecords = $sthandle->execute();
                 if ((!defined $nrecords || $nrecords == 0) && 
                     ($bl == 1 || $bl == 2) && $day eq 'second') {
                    ### Second-day access is allowed for both bl1ws5 and bl2ws5:
                    $id_sql_found = $id2_sql;
                    $sthandle = $dbhandle->prepare('SELECT * FROM '.$dbtable.' where username="'.$usr.'" and beamline="'.$id2_sql.'"');
                    $nrecords = $sthandle->execute();
                 }
                 if (defined $nrecords && $nrecords > 0) {
                    $hash = $sthandle->fetchrow_hashref();
                    if (defined $hash->{$day.'_day_end'}) {
                       $endstamp = $hash->{$day.'_day_end'};
                       $endepoch = &timestamp2epoch($endstamp);
                       if ($endepoch !=0 && $endepoch < $epoch) {$endstamp .= ', EXPIRED!';}
                    } else {
                       #$erec = '!!! '.$script.': cannot find "'.$day.'_day_end" for user='.$usr."\n";
                       #print STDOUT $erec; $ebuf .= $erec;
                       $endstamp = 'no "'.$day.'_day_end" in SQL';
                    }
                    if (defined $hash->{'host_email'}) {
                       $host_email = $hash->{'host_email'};
                       $host_uname = $host_email;
                       $host_uname =~ s/\@.*$//;
                    } else {
                       #$erec = '!!! '.$script.': cannot find "host_email" for user='.$usr."\n";
                       #print STDOUT $erec; $ebuf .= $erec;
                       $host_email = 'unknown';
                       $host_uname = 'unknown';
                    }
                    if (defined $hash->{'last_editor'}) {
                       $last_editor = $hash->{'last_editor'};
                    } else {
                       #$erec = '!!! '.$script.': cannot find "last_editor" for user='.$usr."\n";
                       #print STDOUT $erec; $ebuf .= $erec;
                       $last_editor = 'unknown';
                    }
                 } else {
                    #$erec = '!!! '.$script.': cannot find SQL data in "'.$dbtable.'" for user='.$usr.' at BL='.$id_sql."\n";
                    #print STDOUT $erec; $ebuf .= $erec;
                    $endstamp = 'no user data in SQL for BL='.$id_sql;
                    if (($bl == 1 || $bl == 2) && $day eq 'second') {$endstamp .= ' and BL='.$id2_sql;}
                    $last_editor = 'unknown';
                    $host_email  = 'unknown';
                    $host_uname  = 'unknown';
                    $id_sql_found = 'none';
                 }
                 $remote[$i] .= '   <--Expiration: '.$endstamp.' (host='.$host_uname;
                 if ($last_editor ne $host_uname) {$remote[$i] .= '/edit='.$last_editor;}
                 if ($id_sql_found ne $id_sql) {$remote[$i] .= ', BL='.$id_sql_found.')';}
                 else                          {$remote[$i] .= ')';}
              }
           }
           if (length($words[0])>0) {
              $erec = 'Always: '.$words[0]."\n";
              print STDOUT $erec; $ebuf .= $erec;
           }
           $words[1] = join(' ',@remote);
           if (length($words[1])>0) {
              $erec = '  Temp: '.$remote[0]."\n";
              for ($i=1; $i<$nremote; $i++) {$erec .= '        '.$remote[$i]."\n";}
              print STDOUT $erec; $ebuf .= $erec;
           }
        } 
        elsif ($nwords == 1 && length($words[0])>0) {
           $erec = 'Always: '.$words[0]."\n";
           print STDOUT $erec; $ebuf .= $erec;
        }
     }
  }

  if (! $exist) {
     $erec = '!!! No '.$key.' in "'.$sshcfg.'". Access control via SSH is disabled.'."\n";
     print STDOUT $erec; $ebuf .= $erec;
  }

### ========================================================================
### Check that iptables is running and /etc/sysconfig/iptables is configured for right eth:
### ========================================================================
  if ($iptbl_exists) {
     if (! -e $iptblcfg) {&signal_handler($script.': file "'.$iptblcfg.'" does not exist.'."\n");}
     if (! -r $iptblcfg) {&signal_handler($script.': file "'.$iptblcfg.'" is not readable.'."\n");}

     if (&read_entire_file($iptblcfg,\@buf)) {&signal_handler($script.': cannot open "'.$iptblcfg.'" for reading.'."\n");}

### Check that iptables is running:
     if (! system('systemctl is-active --quiet iptables.service')) {
        $erec = "\n".'IPTABLES service is running OK.'."\n";
        print STDOUT $erec; $ebuf .= $erec;
     } else {
        $erec = "\n".'*** IPTABLES service is NOT running. ***'."\n\n";
        print STDOUT $erec; $ebuf .= $erec;
        $erec = 'Attempting to start IPTABLES service...'."\n";
        print STDOUT $erec; $ebuf .= $erec;
        if (system('systemctl start --quiet iptables.service')) {
           $erec = "\n".'*** Failed to start IPTABLES service. ***'."\n\n";
           print STDOUT $erec; $ebuf .= $erec;
        } 
        elsif (system('systemctl is-active --quiet iptables.service')) {
           $erec = "\n".'*** IPTABLES service did not start. ***'."\n\n";
           print STDOUT $erec; $ebuf .= $erec;
        } 
     }

     @inets   = split(/\n/, `ls -1 /sys/class/net`);
     $gmcanet = '164.54.103.';
     if ($host =~ /ws8$/) {$gmcanet = '164.54.143.';}
     foreach $inet (@inets) {
        $str = `ifconfig $inet`;
        if (index($str,$gmcanet) != -1) {$gmcaeth=$inet; last;}
     }
     if (defined $gmcaeth) {
        $erec = '"'.$gmcanet.'" is served by '.$gmcaeth.'.'."\n";
     } else {
        $erec = "\n".'*** Cannot find eth for the '.$gmcanet.' subnet ***'."\n";
        $gmcaeth = '*** unknown ***';
     }
     print STDOUT $erec; $ebuf .= $erec;
     foreach $string (@buf) {
        $string =~ s/\t/ /g;	                                #replace tabs by spaces
        $string =~ s/\s+/ /g;	                                #compress spaces ("+" means match 1 or more times)
        $string =~ s/^ *//g;	                                #remove beginning-of-the-line spaces
        $string =~ s/ *$//g;	                                #remove end-of-the-line spaces
        $string =~ s/^#.*$//g;	                                #remove comment lines
        ### Example:
        ###-A ACCEPT_GMCA -i p6p1 -p tcp -m state --state NEW -m multiport --dports 22,4443 -j ACCEPT
        ###-A ACCEPT_GMCA -i p6p1 -p udp -j ACCEPT
        if ($string =~ /^-A ACCEPT_GMCA -i /) {
           $string =~ s/^-A ACCEPT_GMCA -i //;
           $string =~ s/\s.*//g;
           $iptbeth = $string;
           last;         
        }
     }
     if (defined $iptbeth) {
        $erec = '"'.$gmcanet.'" is firewalled for '.$iptbeth.'.'."\n";
        if ($iptbeth ne $gmcaeth) {$erec .= "\n".'*** IPTABLES protects on WRONG eth. ***'."\n\n";}
        else                      {$erec .= 'IPTABLED firewall check passed OK.'."\n";}
     } else {
        $erec = "\n".'*** Cannot find protection chain for the '.$gmcanet.' subnet in '.$iptblcfg.' ***'."\n";
     }
     print STDOUT $erec; $ebuf .= $erec;

### ========================================================================
### Check /etc/sysconfig/iptables for ssh/nxhtd allowed IPs:
### ========================================================================
     $exist = 0;
     $erec = "\n".'## Allowed external SSH/NX/NXHTD IPs in ['.$iptblcfg.']:'."\n";
     print STDOUT $erec; $ebuf .= $erec;
     foreach $string (@buf) {
        $string =~ s/\t/ /g;	                                #replace tabs by spaces
        $string =~ s/\s+/ /g;	                                #compress spaces ("+" means match 1 or more times)
        $string =~ s/^ *//g;	                                #remove beginning-of-the-line spaces
        $string =~ s/ *$//g;	                                #remove end-of-the-line spaces
        $string =~ s/^#.*$//g;	                                #remove comment lines
        ### Example:
        ### -A INPUT -i p7p1 -p tcp -s 73.0.0.0/8   -m state --state NEW -m multiport --dports 22,4443 -j ACCEPT  -- OLD
        ### -A INPUT -s 128.194.145.0/24 -j ACCEPT_GMCA
        if ($string =~ /-A INPUT -s / &&
            $string =~ / -j ACCEPT_GMCA$/) {
           $exist++;
           $string =~ s/^-A INPUT -s //;
           $string =~ s/ -j ACCEPT_GMCA$//;
           $string =~ s/\s//g;
           $iprange = $string;
           $string = sprintf '  %-24s',$iprange;
           if ($iprange =~ /\//) {				#if CIDR
              ($status,$ipv4_start,$ipv4_end) = &cidr2iprange($iprange);
              if (! $status) {$string .= sprintf ' [%16s - %-16s]',$ipv4_start,$ipv4_end;}
              else           {$string .= sprintf ' [%16s   %-16s]',' ',' ';}
           } else {
              $string .= sprintf ' [%16s   %-16s]',' ',' ';
           }
           ($remote_user,$expire) = &find_user_expire($iprange,$day);
           if (length($remote_user) > 0) {$string .= sprintf ' - to %s (%s)',$expire,$remote_user;}
           $string .= "\n";
           print STDOUT $string; $ebuf .= $string;
        }
     }
     if (! $exist) {
        $string = '  none'."\n";
        print STDOUT $string; $ebuf .= $string;
     }
  } else {
     $erec = "\n".'!!! IPTABLES is not present. !!!'."\n\n";
     print STDOUT $erec; $ebuf .= $erec;
  }

### --------------------------------------------------------------------
  @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
  @ct = localtime(time);
  ($mday,$mnumb,$year) = ($ct[3],$ct[4]+1,$ct[5]+1900);
  $month = $months[$ct[4]];
  $date  = sprintf $month.' %2d ', $mday;
  $today = sprintf '%04d-%02d-%02d', $year,$mnumb,$mday;
  $begday_stamp = $today.' 00:00';			#beginning of the day
  $begday_epoch = &timestamp2epoch($begday_stamp);
  $beglog_epoch = $begday_epoch - $ndays * 24 * 3600;

### ========================================================================
### Parse /var/log/iptables.log for today's (+$ndays back) blocked logins:
### ========================================================================
  if ($iptbl_exists) {
     if (! -e $iptbllog) {&signal_handler($script.': file "'.$iptbllog.'" does not exist.'."\n");}
     if (! -r $iptbllog) {&signal_handler($script.': file "'.$iptbllog.'" is not readable.'."\n");}
     if (&read_entire_file($iptbllog,\@buf)) {&signal_handler($script.': cannot open "'.$iptbllog.'" for reading.'."\n");}
     $nbuf = @buf;
     ### --------------
     ### Example:
     #Aug 28 00:53:59 bl2ws2.gmca.aps.anl.gov kernel: [2125159.948305] IPTables-Dropped: IN=p7p1 OUT= MAC=00:07:43:07:2c:df:04:09:73:49:c5:00:08:00 SRC=47.184.168.185 DST=164.54.103.136 LEN=60 TOS=0x00 PREC=0x00 TTL=50 ID=3757 DF PROTO=TCP SPT=50344 DPT=22 WINDOW=5840 RES=0x00 SYN URGP=0 
     ### --------------
     $exist = 0;
     $key = ' IPTables-Dropped: ';
     for ($i=$nbuf-1; $i>=0; $i--) {
        $string    = $buf[$i];
        $log_epoch = &extract_epoch(\$string);
        if    ($log_epoch <= 1)			{next;}
        elsif ($log_epoch < $beglog_epoch)	{last;}
#       if ($string !~ /^${date}/)		{next;}
        if ($string !~ /${key}/)		{next;}
        if ($string !~ / SRC=/)			{next;}
        $string =~ s/^.* SRC=//;
        $string =~ s/ DST=.*$//;
        ### Skip Argonne scanners:
        if ($string =~ /^\Q164.54.\E/)		{next;}
        if ($string =~ /^\Q130.202.\E/)		{next;}
        if ($string =~ /^\Q146.137.\E/)		{next;}
        if ($string =~ /^\Q192.5.186.\E/)	{next;}
        if ($exist == 0) {
           if ($ndays == 0) {$erec = "\n".'## SSH/NXHTD logins (incl. hacks) blocked by iptables today ['.$iptbllog.']:'."\n";}
           else             {$erec = "\n".'## SSH/NXHTD logins (incl. hacks) blocked by iptables during last '.$ndays.' days ['.$iptbllog.']:'."\n";}
           print STDOUT $erec; $ebuf .= $erec;
        }
        $exist++;
        if (! exists($ipblock{$string})) {$ipblock{$string} = 1;}
        else                             {$ipblock{$string}++;}
#       print STDOUT $string,"\n";
     }
     if ($exist == 0) {
        if ($ndays == 0) {$erec = "\n".'## No SSH/NXHTD logins (incl. hacks) blocked by iptables today.'."\n";}
        else             {$erec = "\n".'## No SSH/NXHTD logins (incl. hacks) blocked by iptables during last '.$ndays.' days.'."\n";}
        print STDOUT $erec; $ebuf .= $erec;
     } else {
        foreach $key (keys %ipblock) {
           $erec = '  '.$key.' ('.$ipblock{$key}.' time(s))'."\n";
           print STDOUT $erec; $ebuf .= $erec;
        }
     }
  }

### ========================================================================
### Parse /var/log/secure for today's (+$ndays back) unsuccessful logins:
### ========================================================================
  if (! -e $secure) {&signal_handler($script.': file "'.$secure.'" does not exist.'."\n");}
  if (! -r $secure) {&signal_handler($script.': file "'.$secure.'" is not readable.'."\n");}
  if (&read_entire_file($secure,\@buf)) {&signal_handler($script.': cannot open "'.$secure.'" for reading.'."\n");}
### Now append last but one log file:
  @securelist = glob($secure.'-*');
  if (@securelist > 0) {
     @securelist = sort(@securelist);
#    foreach $secure (@securelist) {print STDOUT $secure,"\n";} exit;
     $secure_old = $securelist[-1];
     if (! &read_entire_file($secure_old,\@buf2)) {
#       $erec = 'Appending log '.$secure_old."\n";
#       print STDOUT $erec; $ebuf .= $erec;
        push(@buf, @buf2);
     }
  }     
  $nbuf = @buf;

### --------------
#Example:
#Jul  7 06:23:42 bl1ws2.gmca.aps.anl.gov sshd[30086]: Failed password for invalid user pi from 82.64.15.106 port 55429 ssh2
#Oct 28 06:32:02 bl1ws2.gmca.aps.anl.gov sshd[32153]: Failed none for invalid user default from 67.167.151.133 port 47286 ssh2
#Oct 30 07:37:27 bl1ws2.gmca.aps.anl.gov sshd[17406]: Invalid user springerlab from 130.199.251.4 port 11618
### --------------
  $exist = 0;
  $key  = ' Failed password for invalid user';
  $key2 = ' Failed none for invalid user';
  $key3 = ' Invalid user';
  for ($i=$nbuf-1; $i>=0; $i--) {
     $string    = $buf[$i];
     $log_epoch = &extract_epoch(\$string);
     if    ($log_epoch <= 1)		{next;}
     elsif ($log_epoch < $beglog_epoch)	{last;}
     if ($string !~ /sshd\[\d*\]:/)	{next;}
     if ($string !~ /${key}/  && 
         $string !~ /${key2}/ &&
         $string !~ /${key3}/)		{next;}
     if ($exist == 0) {
        if ($ndays == 0) {$erec = "\n".'## Today\'s rejected ssh logins (incl. hacks) due to invalid user ['.$secure.']:'."\n";}
        else             {$erec = "\n".'## Rejected ssh logins (incl. hacks) due to invalid user for the past '.$ndays.' days ['.$secure.']:'."\n";}
        print STDOUT $erec; $ebuf .= $erec;
     }
     $exist++;
     $string =~ s/ sshd\[\d*\]://;
     $string =~ s/ ssh2$//;
     $string =~ s/ port \d*$//;
     if    ($string =~ /${key}/)  {$string =~ s/${key}//;}
     elsif ($string =~ /${key2}/) {$string =~ s/${key2}//;}
     elsif ($string =~ /${key3}/) {$string =~ s/${key3}//;}
     @words = split(/\s+/,$string);
     $nwords = @words;
     if ($nwords > 4) {splice(@words,3,1); $nwords=@words;}	#remove gmca hostname (e.g. bl1ws2.gmca.aps.anl.gov)
     if ($words[$nwords-1] =~ /[a-zA-Z]/) {			#if the address is a hostname (not IP)
        $packed_ip = gethostbyname($words[$nwords-1]);		#this is core Perl function
        if (defined $packed_ip) {
           $ip = inet_ntoa($packed_ip);                         #converts binary packed IP to d.d.d.d (legacy function)
           $words[$nwords-1] .= ' ('.$ip.')';
        }
     }
     $string = join(' ',@words);
     $erec = $string."\n";
     print STDOUT $erec; $ebuf .= $erec;
  }
  if ($exist == 0) {
     if ($ndays == 0) {$erec = "\n".'## No today\'s rejected ssh logins due to invalid user (incl. hacks).'."\n";}
     else             {$erec = "\n".'## No rejected ssh logins (incl. hacks) due to invalid user durung the past '.$ndays.' days.'."\n";}
     print STDOUT $erec; $ebuf .= $erec;
  }

### --------------
#Example:
#Apr  8 18:17:12 bl1ws2 sshd[21327]: Failed password for tuser1 from 164.54.103.26 port 33608 ssh2
### --------------
  $exist = 0;
  $key  = ' Failed password for';
  for ($i=$nbuf-1; $i>=0; $i--) {
     $string    = $buf[$i];
     $log_epoch = &extract_epoch(\$string);
     if    ($log_epoch <= 1)		   {next;}
     elsif ($log_epoch < $beglog_epoch)	   {last;}
     if ($string !~ /sshd\[\d*\]:/)	   {next;}
     if ($string !~ /${key}/)		   {next;}
     if ($string =~ /${key} invalid user/) {next;}
     if ($exist == 0) {
        if ($ndays == 0) {$erec = "\n".'## Today\'s rejected ssh logins due to incorrect password ['.$secure.']:'."\n";}
        else             {$erec = "\n".'## Rejected ssh logins due to incorrect password for the past '.$ndays.' days ['.$secure.']:'."\n";}
        print STDOUT $erec; $ebuf .= $erec;
     }
     $exist++;
     $string =~ s/ sshd\[\d*\]://;
     $string =~ s/ ssh2$//;
     $string =~ s/ port \d*$//;
     $string =~ s/${key}//;
     @words = split(/\s+/,$string);
     $nwords = @words;
     if ($nwords > 4) {splice(@words,3,1); $nwords=@words;}	#remove gmca hostname (e.g. bl1ws2.gmca.aps.anl.gov)
     if ($words[$nwords-1] =~ /[a-zA-Z]/) {			#if the address is a hostname (not IP)
        $packed_ip = gethostbyname($words[$nwords-1]);		#this is core Perl function
        if (defined $packed_ip) {
           $ip = inet_ntoa($packed_ip);                         #converts binary packed IP to d.d.d.d (legacy function)
           $words[$nwords-1] .= ' ('.$ip.')';
        }
     }
     $string = join(' ',@words);
     $erec = $string."\n";
     print STDOUT $erec; $ebuf .= $erec;
  }
  if ($exist == 0) {
     if ($ndays == 0) {$erec = "\n".'## No today\'s rejected ssh logins due to incorrect password.'."\n";}
     else             {$erec = "\n".'## No rejected ssh logins due to incorrect password during past '.$ndays.' days.'."\n";}
     print STDOUT $erec; $ebuf .= $erec;
  }

### --------------
#Example:
#Apr  8 17:06:29 bl1ws2 sshd[18085]: User root from luch-old.aps.anl.gov not allowed because not listed in AllowUsers
### --------------
  $exist = 0;
  $key = ' not allowed because not listed in AllowUsers';
  for ($i=$nbuf-1; $i>=0; $i--) {
     $string    = $buf[$i];
     $log_epoch = &extract_epoch(\$string);
     if    ($log_epoch <= 1)		{next;}
     elsif ($log_epoch < $beglog_epoch)	{last;}
     if ($string !~ /sshd\[\d*\]:/)	{next;}
     if ($string !~ /${key}/)		{next;}
    if ($string =~ /: User root /)	{next;}
     if ($exist == 0) {
        if ($ndays == 0) {$erec = "\n".'## Today\'s rejected ssh logins due to user not listed in AllowUsers ['.$secure.']:'."\n";}
        else             {$erec = "\n".'## Rejected ssh logins due to user not listed in AllowUsers for the past '.$ndays.' days ['.$secure.']:'."\n";}
        print STDOUT $erec; $ebuf .= $erec;
     }
     $exist++;
     $string =~ s/ sshd\[\d*\]://;
     $string =~ s/${key}//;
     @words = split(/\s+/,$string);
     $nwords = @words;
     if ($nwords > 4) {splice(@words,3,2); $nwords=@words;}
     if ($words[$nwords-1] =~ /^[a-zA-Z]+$/) {			#if the address is a hostname (not IP)
        $packed_ip = gethostbyname($words[$nwords-1]);          #this is core Perl function
        if (defined $packed_ip) {
           $ip = inet_ntoa($packed_ip);				#converts binary packed IP to d.d.d.d (legacy function)
           $words[$nwords-1] .= ' ('.$ip.')';
        }
     }
     $string = join(' ',@words);
     $erec = $string."\n";
     print STDOUT $erec; $ebuf .= $erec;
  }
  if ($exist == 0) {
     if ($ndays == 0) {$erec = "\n".'## No today\'s rejected ssh logins due to user not listed in AllowUsers.'."\n";}
     else             {$erec = "\n".'## No rejected ssh logins due to user not listed in AllowUsers during the past '.$ndays.' days.'."\n";}
     print STDOUT $erec; $ebuf .= $erec;
  }

### = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = 
  if ($host !~ /bl[123]ws[256]$/ && $host ne 'px0') {
     &signal_handler("\n".'## No nxserver expected on this computer'."\n");
  }
  if (! -e $nxserver) {
     &signal_handler("\n".'## Did not find nxserver on this computer'."\n");
  } else {
     $serverVersion = `$nxserver --version 2>/dev/null | grep ' - Version ' | grep -v ' WARNING: '`; 
     $serverVersion =~ s/^.* - Version //;
     chomp $serverVersion;
     $serverMajor = $serverVersion;
     $serverMajor =~ s/\..*//g;
     if ($serverMajor !~ /^\d*$/) {
        $erec = '!!! Error determining NX server Generation='.$serverMajor.'. Assuming 6.'."\n";
        print STDOUT $erec; $ebuf .= $erec;
        $serverMajor = 6;
     }
  }

### ========================================================================
### Parse /usr/NX/var/log/nxhtd-error.log for today's (+$ndays back) unsuccessful logins:
### ========================================================================
  if (-e $htderr) {
     if (! -r $htderr)                     {&signal_handler($script.': file "'.$htderr.'" is not readable.'."\n");}
     if (&read_entire_file($htderr,\@buf)) {&signal_handler($script.': cannot open "'.$htderr.'" for reading.'."\n");}
     $nbuf = @buf;
### --------------
#Example:
#Jun 15 11:46:03 bl2ws6.gmca.aps.anl.gov httpd: [Mon Jun 15 11:46:03.367727 2020] [authz_core:error] [pid 11391:tid 139664971208448] [client 68.4.35.146:53780] AH01630: client denied by server configuration: /usr/NX/bin/nxwebclient, referer: https://bl2ws6-gmca.aps.anl.gov:4443/nxwebplayer?config=gmca_classic.nxs
### --------------
     $exist = 0;
     $key = ' client denied by server configuration';
     $rec = '';
     for ($i=$nbuf-1; $i>=0; $i--) {
        $string    = $buf[$i];
        $log_epoch = &extract_epoch(\$string);
        if    ($log_epoch <= 1)		   {next;}
        elsif ($log_epoch < $beglog_epoch) {last;}
        if ($string !~ / ${year}\] /)	   {next;}			#this year only
        if ($string !~ /${key}/)	   {next;}
        $exist++;
        $rec = $string;
        $string =~ s/${key}.*$//;
        $string =~ s/^.*\[client //;
        $string =~ s/:.*$//;
        if ($string =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) {
           if (defined $htdlist{$string}) {$htdlist{$string}++;}
           else                           {$htdlist{$string}=1;}
        } else {
           $erec = '!!! '.$script.': incorrect IP=['.$string.'] in ['.$htderr.'], record: '.$rec."\n";
           print STDOUT $erec; $ebuf .= $erec;
        }
     }
     if ($exist) {
        if ($ndays == 0) {$erec = "\n".'## Today\'s rejected NX WebPlayer logins due to IP not allowed (incl. hacks) ['.$htderr.']:'."\n";}
        else             {$erec = "\n".'## Rejected NX WebPlayer logins (incl. hacks) due to IP not allowed for the past '.$ndays.' days ['.$htderr.']:'."\n";}
        print STDOUT $erec; $ebuf .= $erec;
        foreach $key (keys %htdlist) {
           $erec = sprintf '%s %-15s %4d time(s)'."\n", (scalar localtime($log_epoch)), $key, $htdlist{$key};
           print STDOUT $erec; $ebuf .= $erec;
        }
     } else {
        if ($ndays == 0) {$erec = "\n".'## No today\'s rejected NX WebPlayer logins (incl. hacks) due to IP not allowed.'."\n";}
        else             {$erec = "\n".'## No rejected NX WebPlayer logins (incl. hacks) due to IP not allowed during the past '.$ndays.' days.'."\n";}
        print STDOUT $erec; $ebuf .= $erec;
     }
  } else {
     $erec = '!!! '.$script.': file=['.$htderr.'] not found.'."\n";
     print STDOUT $erec; $ebuf .= $erec;
  }

### ========================================================================
### Check today's (+$ndays back) closed NX sessions:
### ========================================================================
  @lines = ();
  if ($serverMajor > 6) {
     if ($ndays == 0) {$msg = '## Checking for Today\'s closed NX sessions...';}
     else             {$msg = '## Checking for closed NX sessions for the past '.$ndays.' days...';}
     print STDOUT "\n".$msg;
     $ebuf .= "\n".$msg."\n";
     if (defined $use_redis_method) {
        $cmd = $nxserver.' --redis cmd smembers closed';
        $string = `$cmd 2>/dev/null`;
        @seslist = split(/[\s\n]+/,$string);
        $nses = @seslist;
        for ($i=0; $i<$nses; $i++) {
           printf STDOUT "\r".$msg.' reading closed sessions DB %3d/%d',($i+1),$nses;
           $endepoch = &findkey3('endTime',$nxserver,'closed',$seslist[$i]);
           if ($endepoch >= $beglog_epoch) {push(@lines,$seslist[$i]);}			#this lists session IDs
        }
     } else {
#       $cmd = $nxserver.' --history | grep " '.$today.'" | grep " Finished "';
        $cmd = $nxserver.' --history | grep " '.$today.'"';				#we reuse history for failed too
        $string = `$cmd 2>/dev/null`;
#Example:
#1003 dkissick 76.224.236.184 C2DCE41DD4B6719C04837A700C546B9A 2021-03-29 10:11:30 Finished localhost:4000 unix-xsession-default
#1001 agulick                 1181FF25AB3D082CFED4B90A3E5FB8CF 2021-04-01 08:00:06 Failed   localhost:4000 unix-xsession-default
# 0     1          2                        3                      4          5        6          7                 8
        @seslist = split(/\n/,$string);
        $nses = @seslist;
        for ($i=0; $i<$nses; $i++) {
           @words = split(/\s+/,$seslist[$i]);
           if    (@words >= 8 && $words[6] =~ /^Finished$/) {push(@lines,$words[3]);}	#this lists session IDs
           elsif (@words >= 7 && $words[5] =~ /^Finished$/) {push(@lines,$words[2]);}	#this lists session IDs
        }
     }
     $nses = @lines;
     if ($ndays == 0) {$erec = '  ...'.$nses.' sessions closed today.            '."\n";}
     else             {$erec = '  ...'.$nses.' sessions closed for the past '.$ndays.' days.'."\n";}
     print STDOUT "\r".$msg.$erec;
     $ebuf .= $erec;
  }
  elsif ($serverMajor <= 6 && -e $nxclosed) {
     $string = sprintf '/usr/bin/find '.$nxclosed.' -maxdepth 1 -type f -daystart -mtime -%d', ($ndays+1);
     $string = `$string`;
     @lines = split(/\n/,$string);							#this lists session files
  }
  if (@lines > 0) {
     if ($ndays == 0) {$erec = "\n".'## Today\'s closed NX sessions:'."\n";}
     else             {$erec = "\n".'## Closed NX sessions for the past '.$ndays.' days:'."\n";}
     print STDOUT $erec; $ebuf .= $erec;
     $nsessions = 0;
     foreach $line (@lines) {
        if ($serverMajor > 6) {
           if (&read_nx_redis($nxserver,'closed',$line,\@buf) != 0) {next;}
        } else {
           if ($line !~ /^$nxclosed\/sessionId/)    {next;}
           if (&read_entire_file($line,\@buf) != 0) {next;}
        }
        $nxip         = &findkey2('foreignAddres',\@buf);
        $nxuser       = &findkey2('userName',\@buf);
        $nxid         = &findkey2('sessionId',\@buf);
        $nxclienttype = &findkey2('clientType',\@buf);					#nxclient|nxwebclient
        $nxversion    = &findkey2('clientVersion',\@buf);
        $nxOS         = &findkey2('clientPlatform',\@buf);				#mac|linux|windows|notset
        if ($nxOS eq 'notset') {
           $nxOS      = &findkey2('platform',\@buf);
           if    ($nxOS =~ /linux/i) {$nxOS = 'linux';}
           elsif ($nxOS =~ /mac/i)   {$nxOS = 'macosx';}
           elsif ($nxOS =~ /win/i)   {$nxOS = 'winnt';}
           else                      {$nxOS = 'notset';}
        }
        $nxtype       = &findkey2('type',\@buf);					#virtualDefault|virtualAttach|nodeConnectionMonitor
        $nxbrowser    = &findkey2('clientBrowser',\@buf);
        $nxstart      = &findkey2('startTime',\@buf);					#startTime is epoch
        if (length($nxstart) == 0) {
           $nxstart   = &findkey2('creationTime',\@buf);
        }
        $nxsuspend    = &findkey2('suspendTime',\@buf);					#suspendTime is epoch
        $nxend        = &findkey2('endTime',\@buf);					#endTime is epoch
        if (length($nxstart) > 0 && length($nxend) > 0) {
           $nxdur = sprintf '%.1fm',($nxend-$nxstart)/60.;
        } else {
           $nxdur = '-';
        }
        if (length($nxstart) > 0) {
           @ct        = localtime($nxstart);
           $nxstart   = sprintf '%04d/%02d/%02d %02d:%02d', $ct[5]+1900,$ct[4]+1,$ct[3],$ct[2],$ct[1];
        } else {
           $nxstart   = '-';
        }
        if (length($nxsuspend) > 0) {
           @ct        = localtime($nxsuspend);
           $nxsuspend = sprintf '%04d/%02d/%02d %02d:%02d', $ct[5]+1900,$ct[4]+1,$ct[3],$ct[2],$ct[1];
        } else {
           $nxsuspend = '-';
        }
        if (length($nxend) > 0) {
           @ct        = localtime($nxend);
           $nxend   = sprintf '%04d/%02d/%02d %02d:%02d', $ct[5]+1900,$ct[4]+1,$ct[3],$ct[2],$ct[1];
        } else {
           $nxend   = '-';
        }
        if (length($nxip) == 0) {$nxip = '-';}
        if ($nxip =~ /\Q127.0.0.1\E/ || $nxtype =~ /nodeConnectionMonitor/) {next;}
        if ($nxip eq '-'             && $nxclienttype eq 'nxwebclient')     {next;}
        if ($details) {
           $msg = sprintf '>--- user=%-10s ip=%-15s ID=%s'."\n"
                         .'    start=[%s] discon=[%s] end=[%s]'."\n"
                         .' duration=%s'."\n"
                         .'   client=%s:', 
                         $nxuser, $nxip, $nxid, 
                         $nxstart, $nxsuspend, $nxend, 
                         $nxdur, $nxOS;
        } else {
#          $msg = sprintf 'user=%-10s ip=%-15s start=[%s] discon=[%s] end=[%s] dur=%s client=%s:',
           $msg = sprintf 'user=%-10s ip=%-15s start=[%s] end=[%s] dur=%s client=%s:',
                         $nxuser, $nxip, 
#                        $nxstart, $nxsuspend, $nxend, 
                         $nxstart, $nxend, 
                         $nxdur, $nxOS;
        }
        if ($nxclienttype ne 'nxwebclient') {$msg .= $nxversion."\n";}
        else                                {$msg .= $nxbrowser."\n";}
        print STDOUT $msg;
        $ebuf .= $msg;
        $nsessions++;
     }
     if ($nsessions == 0) {
        $erec = '  none'."\n";
        print STDOUT $erec; $ebuf .= $erec;
     }
  }

### ========================================================================
### Check today's (+$ndays back) failed NX sessions:
### ========================================================================
  @lines = ();
  if ($serverMajor > 6) {
     if ($ndays == 0) {$msg = '## Checking for Today\'s failed NX sessions...';}
     else             {$msg = '## Checking for failed NX sessions during the past '.$ndays.' days...';}
     print STDOUT "\n".$msg;
     $ebuf .= "\n".$msg."\n";
     if (defined $use_redis_method) {
        $cmd = $nxserver.' --redis cmd smembers failed';
        $string = `$cmd 2>/dev/null`;
        @seslist = split(/[\s\n]+/,$string);
        $nses = @seslist;
        for ($i=0; $i<$nses; $i++) {
           printf STDOUT "\r".$msg.' reading failed sessions DB %3d/%d',($i+1),$nses;
           $endepoch = &findkey3('endTime',$nxserver,'failed',$seslist[$i]);
           if ($endepoch >= $beglog_epoch) {push(@lines,$seslist[$i]);}			#this lists session IDs
        }
     } else {
#       $cmd = $nxserver.' --history | grep " '.$today.'" | grep " Failed "';		#we reuse history from closed (Finished)
#       $cmd = $nxserver.' --history | grep " '.$today.'"';				#we reuse history from closed (Finished)
#       $string = `$cmd 2>/dev/null`;							#we reuse history from closed (Finished)
#       @seslist = split(/\n/,$string);							#we reuse history from closed (Finished)
        $nses = @seslist;
        for ($i=0; $i<$nses; $i++) {
           @words = split(/\s+/,$seslist[$i]);
           if    (@words >= 8 && $words[6] =~ /^Failed$/) {push(@lines,$words[3]);}	#this lists session IDs
           elsif (@words >= 7 && $words[5] =~ /^Failed$/) {push(@lines,$words[2]);}	#this lists session IDs
        }
     }

     $nses = @lines;
     if ($ndays == 0) {$erec = '  ...'.$nses.' sessions failed today.            '."\n";}
     else             {$erec = '  ...'.$nses.' sessions failed during the past '.$ndays.' days.'."\n";}
     print STDOUT "\r".$msg.$erec;
     $ebuf .= $erec;
  }
  elsif ($serverMajor <= 6 && -e $nxfailed) {
     $string = sprintf '/usr/bin/find '.$nxfailed.' -maxdepth 1 -type f -daystart -mtime -%d', ($ndays+1);
     $string = `$string`;
     @lines = split(/\n/,$string);							#this lists session files
  }
  if (@lines > 0) {
     if ($ndays == 0) {$erec = "\n".'## Today\'s failed NX sessions:'."\n";}
     else             {$erec = "\n".'## Failed NX sessions during the past '.$ndays.' days:'."\n";}
     print STDOUT $erec; $ebuf .= $erec;
     $nsessions = 0;
     foreach $line (@lines) {
        if ($serverMajor > 6) {
           if (&read_nx_redis($nxserver,'failed',$line,\@buf) != 0) {next;}
        } else {
           if ($line !~ /^$nxfailed\/sessionId/) {next;}
           if (&read_entire_file($line,\@buf) != 0) {next;}
        }
        $nxip         = &findkey2('foreignAddres',\@buf);
        $nxuser       = &findkey2('userName',\@buf);
        $nxid         = &findkey2('sessionId',\@buf);
        $nxclienttype = &findkey2('clientType',\@buf);					#nxclient|nxwebclient
        $nxversion    = &findkey2('clientVersion',\@buf);
        $nxOS         = &findkey2('clientPlatform',\@buf);				#mac|linux|windows|notset
        if ($nxOS eq 'notset') {
           $nxOS      = &findkey2('platform',\@buf);
           if    ($nxOS =~ /linux/i) {$nxOS = 'linux';}
           elsif ($nxOS =~ /mac/i)   {$nxOS = 'macosx';}
           elsif ($nxOS =~ /win/i)   {$nxOS = 'winnt';}
           else                      {$nxOS = 'notset';}
        }
        $nxtype       = &findkey2('type',\@buf);					##virtualDefault|virtualAttach|nodeConnectionMonitor
        $nxbrowser    = &findkey2('clientBrowser',\@buf);
        $nxstart      = &findkey2('startTime',\@buf);					#startTime is epoch
        if (length($nxstart) == 0) {
           $nxstart   = &findkey2('creationTime',\@buf);
        }
        $nxsuspend    = &findkey2('suspendTime',\@buf);					#suspendTime is epoch
        $nxend        = &findkey2('endTime',\@buf);					#endTime is epoch
        $nxerror      = &findkey2('error',\@buf);					#error string is URL-encoded
        $nxerror      = &urldecode($nxerror);
        $nxerror     =~ s/^.* NXSERVER //;						#remove header
        $nxerror     =~ s/Connection to the session \'${nxid}\' //;
        $nxerror     =~ s/ \'${nxid}\'//;
        $nxerror     =~ s/[\r\n]//g;

        if (length($nxstart) > 0 && length($nxend) > 0) {
           $nxdur = sprintf '%.1fm',($nxend-$nxstart)/60.;
        } else {
           $nxdur = '-';
        }
        if (length($nxstart) > 0) {
           @ct        = localtime($nxstart);
           $nxstart   = sprintf '%04d/%02d/%02d %02d:%02d', $ct[5]+1900,$ct[4]+1,$ct[3],$ct[2],$ct[1];
        } else {
           $nxstart   = '-';
        }
        if (length($nxsuspend) > 0) {
           @ct        = localtime($nxsuspend);
           $nxsuspend = sprintf '%04d/%02d/%02d %02d:%02d', $ct[5]+1900,$ct[4]+1,$ct[3],$ct[2],$ct[1];
        } else {
           $nxsuspend = '-';
        }
        if (length($nxend) > 0) {
           @ct        = localtime($nxend);
           $nxend   = sprintf '%04d/%02d/%02d %02d:%02d', $ct[5]+1900,$ct[4]+1,$ct[3],$ct[2],$ct[1];
        } else {
           $nxend   = '-';
        }
        if (length($nxip) == 0) {$nxip = '-';}
        if ($nxip =~ /\Q127.0.0.1\E/ || $nxtype =~ /nodeConnectionMonitor/) {next;}
        if ($nxip eq '-'             && $nxclienttype eq 'nxwebclient')     {next;}
        if ($details) {
           $msg = sprintf '>--- user=%-10s ip=%-15s ID=%s'."\n"
                         .'    start=[%s] discon=[%s] end=[%s]'."\n"
                         .' duration=%s'."\n"
                         .'   client=%s:', 
                $nxuser, $nxip, $nxid, 
                $nxstart, $nxsuspend, $nxend, 
                $nxdur, $nxOS;
        } else {
#          $msg = sprintf 'user=%-10s ip=%-15s start=[%s] discon=[%s] end=[%s] dur=%s client=%s:',
           $msg = sprintf 'user=%-10s ip=%-15s start=[%s] end=[%s] dur=%s client=%s:',
                         $nxuser, $nxip, 
#                        $nxstart, $nxsuspend, $nxend, 
                         $nxstart, $nxend, 
                         $nxdur, $nxOS;
        }
        if ($nxclienttype ne 'nxwebclient') {$msg .= $nxversion."\n";}
        else                                {$msg .= $nxbrowser."\n";}
        if ($details && length($nxerror)>0) {$msg .= '    error='.$nxerror."\n";}
        print STDOUT $msg;
        $ebuf .= $msg;
        $nsessions++;
     }
     if ($nsessions == 0) {
        $erec = '  none'."\n";
        print STDOUT $erec; $ebuf .= $erec;
     }
  }

### ========================================================================
### Check today's running NX sessions:
### ========================================================================
  if (-e $nxserver) {
     $string = `$nxserver --list | grep '^[0-9]'`;
### Output:
# 0          1            2                    3                         4
#1003    rfischetti 130.202.237.200 AB033F99DA75AF4C01772D336987E7D6 localhost:4000
#1001    rfischetti 130.202.237.200 2E6E1DAB888FDD9E5249D2ED63F42C8D localhost:4000
#1002    sergey     73.36.101.192   0641B0AF93B5CA28BD80815FE636B732 localhost:4000
     @lines = split(/\n/,$string);
     @connected = ();
     @disconnected = ();
#    $erec = "\n".'## Currently running NX sessions:'."\n";
#    print STDOUT $erec; $ebuf .= $erec;
     foreach $line (@lines) {
        @words = split(/\s+/,$line);
        if (@words == 5 && $words[0] =~ /^[0-9]*$/) {
           $nxuser = $words[1];
           $nxip   = $words[2];
           $nxid   = $words[3];
           if ($serverMajor > 6) {
              if (&read_nx_redis($nxserver,'running',$nxid,\@buf) != 0) {next;}
           } else {
              if (&read_entire_file($nxrunning.'/sessionId{'.$nxid.'}',\@buf) != 0) {next;}
           }
           if ($nxip eq '-') {
              $nxip2 = &findkey2('foreignAddres',\@buf);
              if (length($nxip2) >= 7 && $nxip2 ne '127.0.0.1') {$nxip = $nxip2;}
           }
### Find if the session is secondary to something:
           $nxreal = &findkey2('RealConnected',\@buf);
           if (length($nxreal) == 0) {
              $nxreal = &findkey2('MainSession',\@buf);
              $nxreal =~ s/\s+.*$//;							#take only first session
           }
           if (length($nxreal) == 0) {
              $nxreal = &findkey2('directlyConnectedSessions',\@buf);
              $nxreal =~ s/\s+.*$//;							#take only first session
           }
           if (length($nxreal) > 0) {
              @buf2 = ();
              $nxreal_state = '';
              if ($serverMajor > 6) {
                 if (length($nxreal_state) == 0) {
                    if (&read_nx_redis($nxserver,'running',$nxreal,\@buf2) == 0) {$nxreal_state='running';}
                 }
                 if (length($nxreal_state) == 0) {
                    if (&read_nx_redis($nxserver,'closed',$nxreal,\@buf2) == 0)  {$nxreal_state='closed';}
                 }
                 if (length($nxreal_state) == 0) {
                    if (&read_nx_redis($nxserver,'failed',$nxreal,\@buf2) == 0)  {$nxreal_state='failed';}
                 }
              }
              elsif ($serverMajor <= 6) {
                 if (length($nxreal_state) == 0) {
                    if (&read_entire_file($nxrunning.'/sessionId{'.$nxreal.'}',\@buf2) == 0) {$nxreal_state='running';}
                 }
                 if (length($nxreal_state) == 0) {
                    if (&read_entire_file($nxclosed.'/sessionId{'.$nxreal.'}',\@buf2) == 0)  {$nxreal_state='closed';}
                 }
                 if (length($nxreal_state) == 0) {
                    if (&read_entire_file($nxfailed.'/sessionId{'.$nxreal.'}',\@buf2) == 0)  {$nxreal_state='failed';}
                 }
              }
              if (length($nxreal_state) > 0) {
                 if ($nxip eq '-' || length($nxip) == 0) {$nxip = &findkey2('foreignAddres',\@buf2);}	#use IP of related session
#                @buf = @buf2;
              }
           }
           $mode = &findkey2('status',\@buf);						#Connected|Disconnected|Failed|Closed
           if ($mode eq 'Disconnected' && length($nxreal) > 0) {
              $mode = &findkey2('status',\@buf2);					#Connected|Disconnected|Failed|Closed
           }
           $nxclienttype = &findkey2('clientType',\@buf);				#nxclient|nxwebclient
           $nxversion    = &findkey2('clientVersion',\@buf);
           $nxOS         = &findkey2('clientPlatform',\@buf);				#mac|linux|windows|notset
           if ($nxOS eq 'notset') {
              $nxOS      = &findkey2('platform',\@buf);
              if    ($nxOS =~ /linux/i) {$nxOS = 'linux';}
              elsif ($nxOS =~ /mac/i)   {$nxOS = 'macosx';}
              elsif ($nxOS =~ /win/i)   {$nxOS = 'winnt';}
              else                      {$nxOS = 'notset';}
           }
           $nxbrowser    = &findkey2('clientBrowser',\@buf);
           $nxstart      = &findkey2('startTime',\@buf);				#startTime is epoch
           $nxsuspend    = &findkey2('suspendTime',\@buf);
           if (length($nxstart) == 0) {
              $nxstart   = &findkey2('creationTime',\@buf);
           }
           $nxend        = time();							#epoch
           if (length($nxstart) > 0) {
              $nxdur     = sprintf '%.1fm',($nxend-$nxstart)/60.;
              @ct        = localtime($nxstart);
              $nxstart   = sprintf '%04d/%02d/%02d %02d:%02d', $ct[5]+1900,$ct[4]+1,$ct[3],$ct[2],$ct[1];
           } else {
              $nxdur     = '-';
              $nxstart   = '-';
           }
           if (length($nxsuspend) > 0) {
              $nxdisc    = sprintf '%.1fm',($nxend-$nxsuspend)/60.;
              @ct        = localtime($nxsuspend);
              $nxsuspend = sprintf '%04d/%02d/%02d %02d:%02d', $ct[5]+1900,$ct[4]+1,$ct[3],$ct[2],$ct[1];
           } else {
              $nxdisc    = '-';
              $nxsuspend = '-';
           }
           if ($details) { #-------------------------------------------- detailed report
              $msg = sprintf     '>--- user=%-10s ip=%-15s ID=%s'."\n"
                                .'    start=[%s] duration=%s'."\n",
                                 $nxuser, $nxip, $nxid, 
                                 $nxstart, $nxdur;
              if ($mode !~ /^Connected$/i) {
                 $msg .= sprintf '   discon=[%s] unused=%s'."\n",
                                 $nxsuspend, $nxdisc;
              }
              $msg .= sprintf    '   client=%s:', $nxOS;
           } else { #--------------------------------------------------- short repost
              $msg = sprintf     'user=%-10s ip=%-15s ID=%s start=[%s] dur=%s',
                                  $nxuser, $nxip, $nxid, 
                                  $nxstart, $nxdur;
              if ($mode !~ /^Connected$/i) {
                 $msg .= sprintf ' discon=[%s] unused=%s'."\n",
                                  $nxsuspend, $nxdisc;
              }
              $msg .= sprintf ' client=%s:', $nxOS;
           } #----------------------------------------------------------end reports fork
           if ($nxclienttype ne 'nxwebclient') {$msg .= $nxversion;}
           else                                {$msg .= $nxbrowser;}
           if (length($nxreal) > 0) {
              if ($details) {$msg .= sprintf "\n".'   linked=%s (%s)'."\n", $nxreal, $nxreal_state;}
              else          {$msg .= sprintf ' => ID=%s (%s)'."\n",         $nxreal, $nxreal_state;}
           } else {$msg .= "\n";}
#
           if ($mode !~ /^Connected$/i) {push(@disconnected,$msg);}
           else                         {push(@connected,$msg);}
        }
     }
     $erec = "\n".'## Running sessions with currently disconnected NX users:'."\n";
     if (@disconnected > 0) {foreach $msg (@disconnected) {$erec .= $msg;}}
     else                   {$erec .= '  none'."\n";}
     print STDOUT $erec; $ebuf .= $erec;
     $erec = "\n".'## Running sessions with currently connected NX users:'."\n";
     if (@connected > 0) {foreach $msg (@connected) {$erec .= $msg;}}
     else                {$erec .= '  none'."\n";}
     print STDOUT $erec; $ebuf .= $erec;
  }
### ------------------------------------------------------------------------
  &signal_handler("\n");

#==========================================================================

sub show_current_users_sql () {						#user or *
  my ($sthandle, $nrecords, $hash);
  my ($begstamp, $begepoch, $endstamp, $endepoch);
  my ($username, $fullname, $host_uname, $host_email);
  my ($beamline, $remote_ip, $d, $i);
  @users_sql = ();			# this is defined in main for global use

  if ($bl != 0) {
     if ($day ne 'second') {$d=$day;} else {$d='first';}
     $sthandle = $dbhandle->prepare('SELECT * FROM '.$dbtable.' where beamline="'.$id_sql.'"');
     $nrecords = $sthandle->execute();
     if (! defined $nrecords) {$nrecords = 0;}
     for ($i=0; $i<$nrecords; $i++) {
        $hash = $sthandle->fetchrow_hashref();
#
        if (! defined $hash->{$day.'_day_end'}) {next;}
        $endstamp = $hash->{$day.'_day_end'};
        $endepoch = &timestamp2epoch($endstamp);
        if ($endepoch < $epoch) {next;}
#
        if (! defined $hash->{$d.'_day_start'}) {next;}
        $begstamp = $hash->{$d.'_day_start'};
        $begepoch = &timestamp2epoch($begstamp);
        if ($begepoch > $epoch) {next;}
#
        if (! defined $hash->{'enable_remote'}) {next;}
        if ($hash->{'enable_remote'} !~ /remote/i) {next;}
#
        push(@users_sql,$hash);
     }
     if (($bl == 1 || $bl == 2) && $day eq 'second') {
        ### Second-day access is allowed for both beamlines:
        $sthandle = $dbhandle->prepare('SELECT * FROM '.$dbtable.' where beamline="'.$id2_sql.'"');
        $nrecords = $sthandle->execute();
        if (! defined $nrecords) {$nrecords = 0;}
        for ($i=0; $i<$nrecords; $i++) {
           $hash = $sthandle->fetchrow_hashref();
#
           if (! defined $hash->{$day.'_day_end'}) {next;}
           $endstamp = $hash->{$day.'_day_end'};
           $endepoch = &timestamp2epoch($endstamp);
           if ($endepoch < $epoch) {next;}
#
           if (! defined $hash->{$d.'_day_start'}) {next;}
           $begstamp = $hash->{$d.'_day_start'};
           $begepoch = &timestamp2epoch($begstamp);
           if ($begepoch > $epoch) {next;}
#
           if (! defined $hash->{'enable_remote'}) {next;}
           if ($hash->{'enable_remote'} !~ /remote/i) {next;}
#
           push(@users_sql,$hash);
        }
     }
     $nrecords = @users_sql;
     if ($nrecords > 0) {
        print STDOUT "\n".'## Remote users currently enabled in SQL for this host:'."\n";
        foreach $hash (@users_sql) {
           $username = $hash->{'username'};
           $fullname = $hash->{'full_name'};
           $begstamp = $hash->{$day.'_day_start'};
           $endstamp = $hash->{$day.'_day_end'};
           $beamline = $hash->{'beamline'};
           if (defined $hash->{'remote_ip_list'}) {$remote_ip  = $hash->{'remote_ip_list'};} else {$remote_ip  = 'none';}
           if (defined $hash->{'last_editor'})    {$host_uname = $hash->{'last_editor'};}    else {$host_uname = 'unknown';}
           if (defined $hash->{'host_email'})     {$host_email = $hash->{'host_email'};}     else {$host_email = 'unknown';}
#          print STDOUT '  user='.$username.' ('.$fullname.'), beamline='.$beamline.', host='.$host_uname."\n"
#                      .'     end='.$endstamp.', remote IPs='.$remote_ip."\n";
           printf STDOUT '  user=%-14s BL=%s end=%s IPs=%s'."\n",$username,$beamline,$endstamp,$remote_ip;
        }
     } else {
        print STDOUT "\n".'## No remote users currently enabled in SQL for this host.'."\n";
     }
   }
}

#==========================================================================

sub find_user_expire($$) {						#$iprange,$endday
  ### Call: ($user,$expire) = &find_user_expire($iprange,$endday);
  my ($iprange, $endday, $hash);
  my (@ips, $i);
  $iprange = shift(@_);
  $endday  = shift(@_);
  foreach $hash (@users_sql) {
     if (! defined $hash->{'remote_ip_list'}) {next;}
     @ips = split(',',$hash->{'remote_ip_list'});
     foreach $i (@ips) {
        if (&ipstar2cidr($i) =~ /\Q$iprange\E/) {
           return ($hash->{'username'},$hash->{$endday.'_day_end'});
        }
     }
  }
  return ('','');
}

#==========================================================================

sub ipstar2cidr ($) {					#returns $cidr for 1.2.*.* and 1.2.3.*
  my ($ip, $x1, $x2);

  $ip = shift(@_);
  $x1 = '.*';
  $x2 = '.*.*';
  if    ($ip =~ /\Q$x2\E$/) {$ip =~ s/\Q$x2\E$/\.0\.0\/16/;}
  elsif ($ip =~ /\Q$x1\E$/) {$ip =~ s/\Q$x1\E$/\.0\/24/;}
  return $ip;
}

#==========================================================================

sub findkey2($\@) {						#$key,\@buf
  my ($key,$bufref,$line);
  ($key,$bufref) = @_;
  foreach $line (@$bufref) {
     if ($line =~ /^$key=/) {
        $line =~ s/^$key=//;
        return $line;
     }
  }
  return '';
}

#==========================================================================

sub findkey3($$$$) {						#$key,$nxserver,'running|closed|failed',$session)
  my ($key,$nxserver,$sesfolder,$session);
  my ($status, @lines, $line);
  ($key,$nxserver,$sesfolder,$session) = @_;
  $status = &read_nx_redis($nxserver,$sesfolder,$session,\@lines);
  if ($status) {return '';}
  foreach $line (@lines) {
     if ($line =~ /^$key=/) {
        $line =~ s/^$key=//;
        return $line;
     }
  }
  return '';
}

#==========================================================================

sub urlencode ($) {
  my $s = shift;
  $s =~ s/ /+/g;
  $s =~ s/([^A-Za-z0-9\+-])/sprintf("%%%02X", ord($1))/seg;
  return $s;
}

#==========================================================================

sub urldecode {
  my $s = shift;
  $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
  $s =~ s/\+/ /g;
  return $s;
}

#==========================================================================

sub cidr2iprange ($) {					#returns ($status, $start_ipv4, $end_ipv4)
# Taken from:
# https://www.perl.com/article/creating-ip-address-tools-from-scratch/
  my ($cidr, $start_ipv4, $prefixlen, $end_ipv4, @bytes);
  my ($start_decimal, $bits_remaining, $end_decimal);

  $cidr = shift(@_);
  ($start_ipv4, $prefixlen) = split /\//, $cidr;
  if (!defined $prefixlen || length($prefixlen) == 0) {return -1;}
  @bytes = split /\./, $start_ipv4;
  $start_decimal = $bytes[0] * 2**24 + $bytes[1] * 2**16 + $bytes[2] * 2**8 + $bytes[3];
  $bits_remaining = 32 - $prefixlen;
  $end_decimal = $start_decimal + 2 ** $bits_remaining - 1;
  @bytes = unpack 'CCCC', pack 'N', $end_decimal;
  $end_ipv4 = join '.', @bytes; 
  return (0,$start_ipv4,$end_ipv4);
}

#==========================================================================

sub uniq {
# Remove duplicates from an array.
# Taken from: https://stackoverflow.com/questions/7651/how-do-i-remove-duplicate-items-from-an-array-in-perl
    my %seen;
    grep !$seen{$_}++, @_;
}

#==========================================================================

# sub read_nx_redis($nxserver,$sesfolder,$session,\@lines) {
sub read_nx_redis($$$\@) {
  my ($nxserver, $sesfolder, $session, $linesref);
  my ($cmd, $buffer, @buf, $nbuf, $i);

  $nxserver  = shift(@_);
  $sesfolder = shift(@_);
  $session   = shift(@_);
  $linesref  = shift(@_);

  @$linesref = ();
  if ($sesfolder ne 'running' && 
      $sesfolder ne 'closed'  && 
      $sesfolder ne 'failed') {
     print STDOUT '*** '.$script.': incorrect NX Redis folder ['.$sesfolder.']'."\n";
     return 1;
  }
  $cmd = $nxserver.' --redis cmd hgetall '.$sesfolder.'.'.$session;
  $buffer = `$cmd 2>/dev/null`;
  if (${^CHILD_ERROR_NATIVE}) {
     print STDOUT '*** '.$script.': Error executing '.$cmd."\n";
     return 1;
  }
  if (length($buffer) == 0) {return 0;}
  @buf = split(/\s+/,$buffer);
  $nbuf = @buf;
  if ($nbuf == 0 || $nbuf != 2*int($nbuf/2)) {
     print STDOUT '*** '.$script.': incorrect words number='.$nbuf.' in the output of cmd=['.$cmd.']'."\n"
                 .'    output buffer=['.$buffer.']'."\n";
     return 1;
  }
  for ($i=0; $i<$nbuf-1; $i+=2) {
     push @$linesref,$buf[$i].'='.$buf[$i+1];
  }
# foreach $buffer (@$linesref) {print STDOUT $buffer,"\n";}; exit;
  return 0;
}

#==========================================================================

sub extract_epoch(\$) {
### Extract epoch from the strings like this:
#Aug 28 00:53:59 bl2ws2.gmca.aps.anl.gov kernel: [2125159.948305] IPTables-Dropped: IN=p7p1 OUT= MAC=00:07:43:07:2c:df:04:09:73:49:c5:00:08:00 SRC=47.184.168.185 DST=164.54.103.136 LEN=60 TOS=0x00 PREC=0x00 TTL=50 ID=3757 DF PROTO=TCP SPT=50344 DPT=22 WINDOW=5840 RES=0x00 SYN URGP=0 
#Jul  7 06:23:42 bl1ws2.gmca.aps.anl.gov sshd[30086]: Failed password for invalid user pi from 82.64.15.106 port 55429 ssh2
#Apr  8 18:17:12 bl1ws2 sshd[21327]: Failed password for tuser1 from 164.54.103.26 port 33608 ssh2
#Apr  8 17:06:29 bl1ws2 sshd[18085]: User root from luch-old.aps.anl.gov not allowed because not listed in AllowUsers
#Jun 15 11:46:03 bl2ws6.gmca.aps.anl.gov httpd: [Mon Jun 15 11:46:03.367727 2020] [authz_core:error] [pid 11391:tid 139664971208448] [client 68.4.35.146:53780] AH01630: client denied by server configuration: /usr/NX/bin/nxwebclient, referer: https://bl2ws6-gmca.aps.anl.gov:4443/nxwebplayer?config=gmca_classic.nxs
  my ($script, $strref, @words, $nwords, $year, $mon, $month, $day, $tim);
  my (@ct, @months, $year_curr, $month_curr, $stamp, $epoch, @matches);

  $script = 'extract_epoch';
  @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
  @ct     = localtime(time);
  ($month_curr,$year_curr) = ($ct[4]+1,$ct[5]+1900);

  $strref = shift(@_);
  @words = split(/\s+/,$$strref);
  $nwords = @words;
  if ($nwords < 3) {return -1;}
  $mon = $words[0];
  $day = $words[1];
  $tim = $words[2];
  if (! grep {/^\Q$mon\E$/} @months) {
#    print STDOUT '!!! '.$script.': unexpected month="'.$mon.'" in str="'.$$strref.'"'."\n";
     return -1;
  }
  @matches = grep {$months[$_] =~ /^$mon$/} 0..$#months;			#find index of matching element
  $month = $matches[0];
  if (! &is_int($month)) {return -1;}
  $month++;
  if ($month_curr >= $month) {$year = $year_curr;}
  else                       {$year = $year_curr-1;}
  if (! &is_int($day)) {return -1;}
  $stamp = sprintf '%04d-%02d-%02d %s',$year,$month,$day,$tim;
  $epoch = &timestamp2epoch($stamp);
# print STDOUT substr($$strref,0,40),"\n";
# printf STDOUT 'Date=%s, epoch=%s'."\n", $stamp, $epoch;
  return $epoch;
}

#=================================================================

# sub sendmail ($subject,$msg,$email_list) {
sub sendmail ($$$) {

  my ($subject, $msg, $email_list, $mail);

  $subject    = shift(@_);
  $msg        = shift(@_);
  $email_list = shift(@_);

  $mail = MIME::Lite->new(
    From     => 'sstepanov@anl.gov',
    To       => $email_list,
    Subject  => $subject,
    Type     => 'multipart/mixed',
  );
  $mail->attach(
    Type     => 'TEXT', 			#case-sensitive TEXT/BINARY/AUTO
    Encoding => 'quoted-printable',
    Data     => $msg."\n",
  );
  $mail->send();
  print STDOUT 'Mail sent to '.$email_list.' .'."\n";
}

#==========================================================================

sub signal_handler (;$) {				                        #($signal)
  $erec = shift(@_);
  if (defined $erec) {print STDOUT $erec; $ebuf .= $erec;}
  if ($email) {&sendmail($script.' on '.$host,$ebuf,$elist);}
  exit;
}
