#!/usr/bin/perl -W
#--------------------------------------------------------------
#
#    This script converts selected fileglob (e.g. *.for) from DOS-866 to
#    Win1251 encoding. It also converts pseudo-graphics, which is only
#    feasible in DOS, and replaces tabs with spaces. To accomplish that,
#    the script consecutively callsa notabs -> nosymgrf -> r123
#
#     		Author: Sergey Stepanov
#
# Version-1.0:  2013/08/08
#--------------------------------------------------------------
  use strict;

  my ($PEZCA, $script, $syntax, $narg, $status, $i, $n);
  my (@filelist, @failed, $file, $name, $ext, $nfailed);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/common_subs.pl';

  select STDOUT; $|=1;			# set unbuffered output

  $script = &makeScriptName();
  $syntax   = "\n".'Syntax: '.$script.' fileglob'."\n";
  $narg = @ARGV;
  if ($narg == 0) {die $syntax;}

### ATTENTION: fileglob is something like: xxx.*  or aa*bb.ccc
###            to verify fileglob on Windows/DOS:  dir /b xxx.*
###            to verify fileglob on Linux/Unux:   ls -1 xxx.*
### On different platforms PERL interprets files globs in the command line differently:
### on Windows it simply passes it to the script as a single argument, while
### on UNIX the script receives a parsed list (i.e. many arguments instead of one).

  @filelist = ();
  @failed   = ();
  if ($^O =~ /Win/i) {							#Windows
     for ($i=0; $i<$narg; $i++) {push @filelist, glob($ARGV[$i]);}
  } else {								#Linux
     for ($i=0; $i<$narg; $i++) {push @filelist, $ARGV[$i];}
  }

  if (!defined $filelist[0]) {
     die "\n".$script.': No files found for given fileglob.'."\n";
  }

  foreach $file (@filelist) {
     print STDOUT "\n".$script.': converting file='.$file."\n\n";

     if (!(-f $file)) {
	print STDOUT "\n".'!!! '.$script.': file='.$file.' does not exist, skipping...'."\n\n";
	push @failed, $file;
	next;
     }
     if (!(-w $file)) {
	$n = chmod 0777, $file;
	if ($n == 0) {
	   print STDOUT "\n".'!!! '.$script.': file='.$file.' is not writable, skipping...'."\n\n";
	   push @failed, $file;
	   next;
	}
     }
     ($name,$ext) = &analyzeFilename($file);

     $status = system('notabs',$file);
     if ($status) {
	print STDOUT "\n".'!!! '.$script.': "notabs" failed on file='.$file.', skipping...'."\n\n";
        push @failed, $file;
	next;
     }
     unlink $name.'.bak';

     $status = system('nosymgrf',$file);
     if ($status) {
	print STDOUT "\n".'!!! '.$script.': "nosymgrf" failed on file='.$file.', skipping...'."\n\n";
        push @failed, $file;
	next;
     }
     unlink $name.'.bak';

     $status = system('r123','1to2',$file);
     if ($status) {
	print STDOUT "\n".'!!! '.$script.': "r123" failed on file='.$file.', skipping...'."\n\n";
        push @failed, $file;
	next;
     }
     unlink $name.'.bak';
  }
  $nfailed = @failed;
  if ($nfailed > 0) {
     print STDOUT "\n".$script.': the following files failed:'."\n";
     foreach $file (@failed) {print STDOUT $file,"\n";}
  }
  print STDOUT "\n".$script.': finished!';
