#!/usr/bin/perl -W
#--------------------------------------------------------------
#       This script is a beam attenuation server
#
#     		Author: Sergey Stepanov
#
# Version-1.0:  2009/05/21 using Pezca
# Version-1.1:  2009/06/03 added protection from multiple instances
#--------------------------------------------------------------
  use Pezca;
  use DBI;
# use warnings::unused;
  use strict;

  our ($beamline, $ATTENUATION_USE_FACTOR);
  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($PEZCA, $script, $syntax, $narg, $arg, $i, @beamlinelist, $status);
  my ($attenuationRqs, $attenuationAct, $attenuationSet, $attenuationUnits);
  my ($energyPV, $attenuationRqsPV, $attenuationActPV, $attenuationSetPV);
  my ($attenuationUnitsPV, $attenuationMsgPV, $attenuationNew);
  my ($msg, $msg_time, $elapsed, $units);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/attenuation_subs_v2.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;			# set unbuffered output
  select STDERR; $|=1;			# set unbuffered output

  $script  = &makeScriptName();
  $syntax = "\n".'Syntax: '.$script.' [beamline]'."\n";
  @beamlinelist = ('23i', '23o', '23b', '23d');
  $NONINTERACTIVE = 1;
  $USE_ZENITY_FOR_EXIT = 1;
###-----------------------------------------------------------------
### Protection from running on multiple computers:
  &check_Nrunning_and_host_allowed();
#----------------------------------------------------------------
  $narg = @ARGV;
  if ($narg > 1) {&press_key_exit($syntax);}

###------------------------------------------------------------------------
### Parse command line arguments:

  for ($i=0; $i<$narg; $i++ ) {
     $arg = lc $ARGV[$i];
     if (grep(/^$arg$/, @beamlinelist)) {
        if ($arg ne $beamline) {
           $beamline = $arg;
           print STDOUT "\n".'!!! Cross-operation for beamline '.$beamline."\n\n";
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
	}
     }
     else {
        &press_key_exit("\n".'*** '.$script.': Incorrect command line.'.$syntax);
     }
  }

  $SIG{INT}  = 'signal_handler';
  $SIG{QUIT} = 'signal_handler';

  $energyPV           = $beamline.':MO:E:ActPos';
  $attenuationActPV   = $beamline.':attenuationAct';
  $attenuationRqsPV   = $beamline.':attenuationRqs';
  $attenuationSetPV   = $beamline.':attenuationSet';
  $attenuationUnitsPV = $beamline.':attenuationUnits';
  $attenuationMsgPV   = $beamline.':attenuationMsg';

  &addCaMonitor($energyPV);
  &addCaMonitor($attenuationRqsPV);
  &addCaMonitor($attenuationUnitsPV);
  $msg_time = time();
###------------------------------------------------------------------------

### Initialization:
  ($status,$attenuationUnits) = Pezca::GetDouble($attenuationUnitsPV);
  if ($status) {
     $msg = substr('Cannot get attenuation',0,39);
     Pezca::PutString($attenuationMsgPV, $msg);
     $msg_time = time();
     $attenuationUnits = 0;			#0=factor(x) 1=percent(%)
  }
  if ($attenuationUnits == 0) {$ATTENUATION_USE_FACTOR = 1;}
  else                        {$ATTENUATION_USE_FACTOR = 0;}

  ($status,$attenuationAct) = &getAttenuation1();
  if ($status) {
     $msg = substr('Cannot get attenuation',0,39);
     Pezca::PutString($attenuationMsgPV, $msg);
     $msg_time = time();
     $attenuationAct = 0;
  }
  $attenuationRqs = $attenuationAct;
  $attenuationSet = 0;
  $status = Pezca::PutDouble($attenuationActPV,$attenuationAct);
  $status = Pezca::PutDouble($attenuationRqsPV,$attenuationRqs);
  $status = Pezca::PutDouble($attenuationSetPV,$attenuationSet);

### Monitor loop
  while(1) {
     select(undef,undef,undef,0.2);				#sleep(..);
     $units = $attenuationUnits;
     ($status,$attenuationUnits) = Pezca::GetDouble($attenuationUnitsPV);
     if ($status) {$attenuationUnits = $units;}
     if ($attenuationUnits != $units) {
        if ($attenuationUnits == 0) {$ATTENUATION_USE_FACTOR = 1;}
        else                        {$ATTENUATION_USE_FACTOR = 0;}
        ($status,$attenuationAct) = &getAttenuation1();
        if ($status) {
           $msg = substr('Cannot get attenuation',0,39);
           Pezca::PutString($attenuationMsgPV, $msg);
           $msg_time = time();
           $attenuationAct = 0;
        }
        $attenuationRqs = $attenuationAct;
        $attenuationSet = 0;
        $status = Pezca::PutDouble($attenuationActPV,$attenuationAct);
        $status = Pezca::PutDouble($attenuationRqsPV,$attenuationRqs);
        $status = Pezca::PutDouble($attenuationSetPV,$attenuationSet);
     } else {
        ($status,$attenuationSet) = Pezca::GetDouble($attenuationSetPV);
        if ($status) {$attenuationSet = 0;}
        if ($attenuationSet) {
           ($status,$attenuationRqs) = Pezca::GetDouble($attenuationRqsPV);
           if ($status) {
              $msg = substr('Cannot get new attenuation',0,39);
              Pezca::PutString($attenuationMsgPV, $msg);
              $msg_time = time();
           } else {
	      ($status,$attenuationNew) = &setAttenuation1($attenuationRqs);
	      if ($status) {
                 $msg = substr('Cannot set attenuation',0,39);
                 Pezca::PutString($attenuationMsgPV, $msg);
                 $msg_time = time();
	      } else {
                 $attenuationAct = $attenuationNew;
	      }
	   }
           $attenuationSet = 0;
           $status = Pezca::PutDouble($attenuationSetPV,$attenuationSet);
        } else {
           ($status,$attenuationNew) = &getAttenuation1();
           if ($status) {
              $msg = substr('Cannot get attenuation',0,39);
              Pezca::PutString($attenuationMsgPV, $msg);
              $msg_time = time();
	   } else {
              $attenuationAct = $attenuationNew;
	   }
        }
        $status = Pezca::PutDouble($attenuationActPV,$attenuationAct);
     }
#
     $elapsed = time() - $msg_time;
     if ($elapsed > 5) {				#cleanup msg string after 5s
	$msg = '';
        Pezca::PutString($attenuationMsgPV, $msg);
        $msg_time = time();
     }
  }

#=================================================================

sub signal_handler () {
  my $signame = shift;
  print STDOUT "\n".' *** '.$script.': Caught a SIG'.$signame."\n";
  &clearAllCaMonitors();
  exit;
}

