#!/usr/bin/perl -W
# script for mounting eSATA external disk using udisks utility
# Apr 16 2012
##############################################################
# ATTENTION: for this to work, udisks must be allowed to mount
# drives without root permissions. Edit config file:
#
#vi /usr/share/polkit-1/actions/org.freedesktop.udisks.policy
#
# .. and change:
#      <allow_active>auth_admin_keep</allow_active>
# to:
#      <allow_active>yes</allow_active>
#############################################################

  use strict;

  my ($PEZCA, @devices, @params, @mounted, @words, @lines);
  my (%PAR, $device, $line, $nmount, $string);
  my (@dm, @uuid);

  select STDOUT; $|=1;				#set unbuffered output
  select STDERR; $|=1;				#set unbuffered output

# %ENV = (%ENV, &get_login_env_orig());					#older method
  &get_login_env();							#implemented 2019.02
  foreach my $key (keys %ENV) {if ($key =~ /BASH_FUNC_module/) {delete $ENV{$key};}}

  if (!defined $ENV{PEZCA}) {die 'PEZCA undefined';}
  $PEZCA = $ENV{PEZCA}; $PEZCA =~ s/\\/\//g;	#replace \ by /
  if ($PEZCA !~ /\/$/) {$PEZCA .= '/';}		#append /
  do $PEZCA.'Common/common_subs.pl';

# (@devices) = `/bin/ls -1 /dev/sd*`;                   #this way @devices contain "\n"
  @devices = split (/\n/, `/bin/ls -1 /dev/sd*`);

### find all /dev/sd* devices, which are already mounted:
  @lines = split (/\n/, `/bin/mount | grep "^\/dev\/sd" 2>/dev/null`);
  @mounted = ();
  foreach $line (@lines) {
     @words = split (/\s+/, $line);
     push (@mounted, $words[0]);
  }
# print STDOUT 'Mounted: '."@mounted\n";

### find multipath devices:
# (@dm) = `/bin/ls -1 /dev/dm*`;			#this way @dm contain "\n"
  @dm = split (/\n/, `/bin/ls -1 /dev/dm* 2>/dev/null`);

### find uuid of multipath devices:
  @uuid = ();
  foreach $device (@dm) {
     &getPAR($device,\%PAR);
     if (defined $PAR{'uuid'}) {push (@uuid, $PAR{'uuid'});}
  }
# print STDOUT 'UUID: '."@uuid\n";

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  $nmount = 0;
  foreach  $device (@devices) {

     ### check if the device in mounted list
     if (grep(/^${device}$/, @mounted)) {next;}		#already mounted

     ### fill udisks info for the device
     &getPAR($device,\%PAR);

     ### verify that device is not a part of multipath devices:
     $string = $PAR{'uuid'};
     if (defined $string) {
        if (grep(/^${string}$/, @uuid)) {next;}		#this device is a part of multipath
     }

     ### check if the device has allowed file system:
#    print $PAR{'type'},"\n";
     $string = $PAR{'type'};
     if (defined $string    &&
        $string !~ /^ntfs/  &&
        $string !~ /^vfat/  &&
        $string !~ /^hfs/   &&
        $string !~ /^exfat/ &&
        $string !~ /^xfs/   &&
        $string !~ /^ext/)  {next;}

     ### attempt to mount the device:
     print 'Attempting to mount '.$device.' (type='.$string.')...'."\n";
     $string = `/usr/bin/udisks  --mount $device`;
     if ($?) {
        print '.. failure='.$?."\n",
              'Result: ',"\n",$string,"\n";
     } else {
        $nmount++;
     }
  }
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  if ($nmount == 0) {print 'No devices mounted at this time'."\n";}
  exit 0;


#==========================================================================================
sub getPAR($\%) {
  my ($device, $PARref, @params, $string, @words, $key, $udisks);
  $device = shift(@_);
  $PARref = shift(@_);

  ### fill udisks info for the device
  if    (defined &Which('udisks'))    {@params = split (/\n/, `udisks --show-info $device`);}
# elsif (defined &Which('udisksctl')) {@params = split (/\n/, `udisksctl info -b $device`);}
# else  {die "\n".'Please install udisks (Centos6) or udisks2 (Centos7) or oldudisks (Centos7, nux-desktop repo)'."\n";}
  else  {die "\n".'Please install udisks (Centos6) or oldudisks (Centos7, nux-desktop repo)'."\n";}

  %$PARref = ();
  foreach $string (@params) {
     $string =~ s/^\s+//;				#remove firs two spaces at the beginning of line
     @words = split(/:/,$string);		#we are interested in first ":" only - will merge the rest
     $key = $words[0];
     splice @words,0,1;
     $string = join(':',@words);
     if    (length($string) > 0) {
        $$PARref{$key} = $string;
     }
     elsif ($key =~ /^\/dev\/disk\/by-uuid\//) {
        $string = $key; 
        $string =~ s/^\/dev\/disk\/by-uuid\///;
	$key = 'uuid';
        $$PARref{$key} = $string;
     }
  }
}

#==========================================================================================
sub get_login_env_orig {					#more stable to weird environment

### Taken from http://www.perlmonks.org/?node_id=33628

  if ($^O =~ /^Win/i) {return;}					#this function does not work on Windows (no getpwuid)
### 'my' creates a new variable, 'local'
### temporarily amends the value of a variable
  local %ENV;							#do not comment out, this is required
  my $shell = shift || (getpwuid($<))[8];               	#getpwuid does not exist on Windows
  my $env = `echo env | perl -e 'exec {"$shell"} -sh'`;
  if (wantarray) {						#this is a Perl function (type of context the sub is used)
     my @pieces = ($env =~ m/^(.*?)=((?:[^\n\\]|\\.|\\\n)*)/gm);
     s/\\(.)/$1/g foreach @pieces;
     return @pieces;
  } else {
     return $env;
  }
}

#==========================================================================================
sub get_login_env {                                             #Sergey: my version 2019.02 tested with crontab
  my ($shell, $env, @lines, $line, $key, $val);
  if ($^O =~ /^Win/i) {return;}					#this function does not work on Windows (no getpwuid)
  $shell = shift || (getpwuid($<))[8];		               	#getpwuid does not exist on Windows
  $env = `$shell -lc env`;
  @lines = split(/\n/,$env);
  foreach $line (@lines) {
     if ($line =~ /=/) {
        ($key,$val) = split("=",$line,2);
        if ($key !~ /BASH_FUNC_module/ && length($key)>0) {$ENV{$key} = $val;}
     }
  }
}
