#!/usr/bin/perl -W
#--------------------------------------------------------------
#
#    This script finds non-ASCII symbols in given text file
#    Non-ASCII symbols are those with code < 32 or > 127.
#
#     		Author: Sergey Stepanov
#
# Version-1.0:  2018/03/19
#--------------------------------------------------------------
  use strict;

  my ($script, $syntax, $narg, @buf, $l, $line, @na, $char);
  my ($min, $max, $c, $i, @exclu, $e, $exclude);

  select STDERR; $|=1;			# set unbuffered output
  select STDOUT; $|=1;			# set unbuffered output

  $min = 32;
  $max = 127;
  @exclu = (9, 10, 13);			#tab, LF, CR
# push(@exclu, 11);			#VT
# push(@exclu, 12);			#FF

  $script = $0 =~ s/^.*[\/\\]//gr;
  $syntax   = "\n".'Syntax: '.$script.' filename'."\n";
  $narg = @ARGV;
  if ($narg != 1) {die $syntax;}

  if (&read_entire_file($ARGV[0],\@buf)) {die $script.': failed to read '.$ARGV[0];}
  
# foreach $line (@buf) {
#    foreach $char (split //, $line) {
  for ($l=0; $l<@buf; $l++) {
     $line = $buf[$l];
     @na = ();
     for ($i=0; $i<length($line); $i++) {
        $char = substr($line,$i,1);
        $c = ord($char);
        if ($c<$min || $c>$max) {
           $exclude = 0;
	   foreach $e (@exclu) {
	      if ($c == $e) {$exclude=1; last;}
           }
	   if (! $exclude) {push(@na,$i+1);}
        }
     }
     if (@na > 0) {print STDOUT "Non-ASCII at pos. {@na} in line[".($l+1)."]=$line\n";}
  }
  exit;

#==========================================================================

# sub read_entire_file($file,\@lines) {
sub read_entire_file($\@) {
# use Fcntl;					#not used (defines O_RDONLY constant (O_RDONLY=0,O_WRONLY=1,O_RDWR=2))
  my ($file, $linesref, $buffer, $status);

  $file     = shift(@_);
  $linesref = shift(@_);

  @$linesref = ();
  $status = sysopen(FH, $file, 0);		#0 is used instead of O_RDONLY defined in "use Fcntl";
  if (!defined $status) {print STDOUT '*** '.$script.': Could not open '.$file."\n"; return 1;}
  sysread(FH, $buffer, -s FH);
  close FH;
  @$linesref = split(/\r?\n/,$buffer);
  return 0;
}
