#!/usr/bin/perl -W
#---------------------------------------------------------------------------
# This script attempt to fix a PMAC Coordinate System (PCS), which somehow
# got stuck in "Moving". It loops over all motors, finds those, which are
# "moving", and sends them "Kill", "Enable", "Stop"
#
#                       Author: Sergey Stepanov
#
# Version-1.0: 2012/07/20 using Pezca
# Version-1.1: 2013/05/24 changed to use "getAssyDataFromTsub";
#                         added printing of act & rqs positions
#---------------------------------------------------------------------------
# Test: fixPCS.pl 23i:COL:St:
#---------------------------------------------------------------------------
  use strict;
  use Pezca;

  our ($beamline, $NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($PEZCA, $script, $syntax, @beamlinelist, $bl, $narg, $status);
  my ($assy, @axes, @drives, @motors, $nmotors, $i, $confprg);
# my (@mtNumbers, $pmac);
  my ($inpos, $runprg, $mtr, $drv);
  my ($actDrv, $actMtr, $rqsDrv, $rqsMtr);
  my ($debug);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/motor_subs.pl';
  do $PEZCA.'/Common/home_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';

  select STDOUT; $|=1;					#set unbuffered output
  select STDERR; $|=1;					#set unbuffered output

  $SIG{'HUP'}  = 'signal_handler';			#(1) hangup (see signum.h)
  $SIG{'INT'}  = 'signal_handler';			#(2) interrupt (see signum.h)
  $SIG{'QUIT'} = 'signal_handler';			#(3) quit (see signum.h)
# $SIG{'KILL'} = 'signal_handler';			#(9) this is what is sent by kill -9 -- unblockable
  $SIG{'TERM'} = 'signal_handler';			#(15) terminate -- sent by kill without paramaters
# $SIG{'STOP'} = 'signal_handler';			#(19) stop -- unblockable
#-----------------------------------------------------------------
  $NONINTERACTIVE      = 1;				#don't wait for any answers
  $USE_ZENITY_FOR_EXIT = 1;

# $debug = 1;						#comment out for normal operations
  if (!defined $debug) {$debug=0;}

  $script = &makeScriptName();
  $syntax = "\n\n".'Syntax: '.$script.' PMACassemblyName'."\n";
  @beamlinelist = ('23i:', '23o:', '23b:', '23d:');

  $narg = @ARGV;

  if    ($narg == 0) {&press_key_exit('*** '.$script.': no cmdline arguments!'.$syntax);}
  elsif ($narg >  1) {&press_key_exit('*** '.$script.': too many cmdline arguments!'.$syntax);}
  $assy = $ARGV[0];
  $assy =~ s/:$//;
  foreach $bl (@beamlinelist) {
     if ($assy =~ /^$bl/) {
        $beamline = $bl;
        $beamline =~ s/:$//;
        $assy =~ s/^$bl//;
        last;
     }
  }
  if (defined $beamline) {do $PEZCA.'/Common/beamline_'.$beamline.'.pl';}
  else                   {do $PEZCA.'/Common/beamline.pl';}

  print STDOUT 'beamline='.$beamline.'   assy='.$assy."\n";
# no strict 'refs';
# $confprg = 'getAssyDataFromConfigTcl';
# $status = &getAssyDataFromConfigTcl($beamline,$assy,\$pmac,\@axes,\@drives,\@motors,\@mtNumbers);
# $confprg = 'getAssyDataFromTsub';
  $status = &getAssyDataFromTsub($beamline.':'.$assy,\@axes,\@drives,\@motors);
# use strict 'refs';
  if ($status) {
     &press_key_exit('*** '.$script.': '.$confprg.' returned status='
                           .$status.' for beamline='.$beamline.' assy='.$assy."\n");
  }
  print STDOUT "\n".'Search parameters: beamline='.$beamline.' assy='.$assy."\n\n";
  print STDOUT 'Found:'."\n";
  print STDOUT '     assy='.$assy."\n";
# print STDOUT '     axes='."(@{axes})\n";
  print STDOUT '   drives='."(@{drives})\n";
  print STDOUT '   motors='."(@{motors})\n";
# print STDOUT '  numbers='."(@{mtNumbers})\n";				#not defined in 'getAssyDataFromTsub'
# print STDOUT '     pmac='.$pmac."\n";					#not defined in 'getAssyDataFromTsub'

  $assy = $beamline.':'.$assy;
# $pmac = $beamline.':'.$pmac;
  $nmotors = @motors;
  if ($debug) {print STDOUT '  nmotors='.$nmotors."\n";}
  for ($i=0; $i<$nmotors; $i++) {
     if ($drives[$i] !~ /^$beamline/) {$drives[$i] = $beamline.':'.$drives[$i];}
     if ($motors[$i] !~ /^$beamline/) {$motors[$i] = $beamline.':'.$motors[$i];}
     if ($debug) {printf STDOUT '  motor%d=%s'."\n",($i+1),$motors[$i];}
     if ($debug) {printf STDOUT '  drive%d=%s'."\n",($i+1),$drives[$i];}
  }
  print STDOUT ' setting monotors for assy='.$assy."\n";
  $status = &setAssyMonitors ($assy);
  if ($debug) {print STDOUT '   setAssyMonitors status='.$status."\n";}
  if ($status) {
     &press_key_exit('*** '.$script.': cannot set monitors for assy='.$assy."\n"
                           .'EPICS_CA_ADDR_LIST='.$ENV{EPICS_CA_ADDR_LIST});}

  $inpos = &getInPos ($assy,\$status);
  if ($debug) {print STDOUT ' assy='.$assy.'   inpos='.$inpos."\n";}
  if ($inpos) {
     &press_key_exit($script.': assy='.$assy.' is positioned. Do not see a problem.',0,'info');
  }
  $runprg = &getRunPrg ($assy,\$status);
  if ($debug) {print STDOUT ' assy='.$assy.'  runprg='.$runprg."\n";}
  if ($runprg) {
     &press_key_exit($script.': assy='.$assy.' is being moved by a motion program. Do not see a problem.',0,'info');
  }

### We are here if assembly is not positioned and not running a program.

  print STDOUT $script.': OK, found that assy='.$assy.' is not positioned.'."\n";

### First try a simple abort
  print STDOUT $script.': trying simple abort...'."\n";
  &abortAssy ($assy);
  select(undef,undef,undef,0.4);			# sleep
  $inpos = &getInPos ($assy,\$status);
  if ($inpos) {
     &press_key_exit($script.': assy='.$assy.' is positioned. Simple aborting fixed the problem.',0,'info');
  }

  print STDOUT $script.': abort did not help. Checking individual motors...'."\n";
  for ($i=0; $i<$nmotors; $i++) {
     $mtr = $motors[$i];
     $drv = $drives[$i];
     $inpos = &getMtrInPos ($mtr);
     if (! $inpos) {
        $actMtr = &getAxisAct($mtr,\$status);     if ($status) {$actMtr='unknown';}
        $actDrv = &getAxisAct($drv,\$status);     if ($status) {$actDrv='unknown';}
        $rqsMtr = &getAxisActRps($mtr,\$status);  if ($status) {$rqsMtr='unknown';}
        $rqsDrv = &getAxisActRps($drv,\$status);  if ($status) {$rqsDrv='unknown';}
        print STDOUT $script.': motor='.$mtr.' is not positioned.'."\n"
                            .'RQS: mtr='.$rqsMtr." \t ".'drv='.$rqsDrv."\n"
                            .'ACT: mtr='.$actMtr." \t ".'drv='.$actDrv."\n"
                            .'Trying to fix..'."\n";

        &stopMotor($mtr);				# sends kill/enable/jogstop
        $inpos = &getInPos ($assy,\$status);
        if ($inpos) {
           &press_key_exit($script.': assy='.$assy.' is positioned. Aborting'
                                  .' motor='.$mtr.' fixed the problem.',0,'info');
        } else {
           print STDOUT $script.': Did not help. Continuing...'."\n";
        }
     }
  }
  $inpos = &getInPos ($assy,\$status);
  if ($inpos) {
     &press_key_exit($script.': assy='.$assy.' is positioned.'
                            .' Unknown step fixed the problem.',0,'info');
  } else {
     &press_key_exit($script.': assy='.$assy.' is still NOT positioned.'
                            .' Please call Sergey or Oleg.',1,'error');
  }


#==========================================================================

sub signal_handler ($) {
  my $signame = shift(@_);
  print STDOUT "\n".$script.': *** Caught a SIG'.$signame."\n\n";
  &clearAllCaMonitors();
  print STDOUT $script.'('.$signame.'): Done!'."\n\n";
  exit;
}
