#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script queries link state of motor 32 in given PMAC
#
#     		Author: Sergey Stepanov
#
# Version-1.0:  2013/11/29  initial implemenation
#--------------------------------------------------------------
  use strict;
  use Pezca;

  our ($beamline, $NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($PEZCA, $script, $syntax, @beamlinelist, $narg, $i, $arg);
  my ($pm_specified, $bl_specified, $status, @assyNames, @assyNums, $nAssy);
  my ($pmac, $mtrNo, $linkState, $assy, $pcs, $formula);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/motor_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;			# set unbuffered output
  select STDERR; $|=1;			# set unbuffered output

  $script = &makeScriptName();
  $syntax = "\n".'Syntax: '.$script.' pmac [beamline]'."\n";
  @beamlinelist = ('23i', '23o', '23b', '23d');

  $USE_ZENITY_FOR_EXIT = 1;
  $NONINTERACTIVE = 1;

  $SIG{INT}  = 'signal_handler';
  $SIG{QUIT} = 'signal_handler';

  $narg = @ARGV;
  if ($narg == 0 || $narg > 2) {&press_key_exit($syntax);
}
  for ($i=0; $i<$narg; $i++) {
     $arg = $ARGV[$i];
     if (grep(/^${arg}$/, @beamlinelist)) {
	if (! defined $bl_specified) {
	   $bl_specified = 1;
           if ($arg ne $beamline) {
              $beamline = $arg;
              print STDOUT "\n".'!!! '.$script.':  Cross-beamline operation for '.$beamline."\n\n";
              do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
	   }
	} else {
           &press_key_exit('*** '.$script.': duplicate beamline specification in cmdline='.$ARGV[$i].$syntax);
	}
     }
     elsif ($arg =~ /pmac/i) {
	if (! defined $pm_specified) {
	   $pm_specified = 1;
           $pmac = $arg;
           $arg = substr($pmac,0,3);
           if (grep(/^${arg}$/, @beamlinelist)) {		#pmac contain beamline specification
	      if (! defined $bl_specified) {			#we cannot have beamline specs via pmac & separate
	         $bl_specified = 1;
                 if ($arg ne $beamline) {
                    $beamline = $arg;
                    print STDOUT "\n".'!!! '.$script.':  Cross-beamline operation for '.$beamline."\n\n";
                    do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
	         }
	      } else {
                 &press_key_exit('*** '.$script.': duplicate beamline specification in cmdline='.$ARGV[$i].$syntax);
	      }
           }
	} else {
           &press_key_exit('*** '.$script.': duplicate pmac specification in cmdline='.$ARGV[$i].$syntax);
	}
     }
     else {
        &press_key_exit('*** '.$script.':  Unrecognized parameter '.$arg.' in cmdline.'.$syntax);
     }
  }

  if (!defined $pmac) {
     &press_key_exit('*** '.$script.':  No PMAC specified in cmdline.'.$syntax);
  } else {
     $pmac =~ s/^${beamline}//;
     $pmac =~ s/://g;
     $pmac = $beamline.':'.$pmac.':';
  }

  $status = 0;
  $mtrNo = &getMtrOutNo_Pmac($pmac,\$status);
  if ($status) {&press_key_exit('*** '.$script.': Failed to determine mtrOut for PMAC='.$pmac);}
  else         {print STDOUT 'Auto-determined mtrOut='.$mtrNo.'  for PMAC='.$pmac."\n\n";}

  $status = &getAssyList4Pmac ($beamline,$pmac,\@assyNames,\@assyNums);
  if ($status) {&press_key_exit('*** '.$script.': Failed to get assemblies list for PMAC='.$pmac);}

  print STDOUT $script.': PMAC='.$pmac.' contains the following assemblies:'."\n";
  $nAssy = @assyNames;
  for ($i=0; $i<$nAssy; $i++) {
     printf STDOUT '%3d  %s'."\n", $assyNums[$i], $assyNames[$i];
  }

  $linkState = &isMtrOutLinked($assyNames[0], $mtrNo);		#returns 1/0/-N
  if ($linkState == 1) {
     $assy = $assyNames[0];
     $pcs  = $assyNums[0];
  }
  elsif ($linkState < 0) {
     $pcs = -$linkState;
     for ($i=0; $i<$nAssy; $i++) {
	if ($assyNums[$i] == $pcs) {$assy=$assyNames[$i]; last;}
     }
     if (!defined $assy) {
	&press_key_exit('*** '.$script.': Failed to find name of pcs='.$pcs.' in PMAC='.$pmac);
     }
  }
  if ($linkState != 0) {
     $formula = &getLinkedFormula_Pcs($assy,$mtrNo);
     &press_key_exit($script.': mtrOut='.$mtrNo.' in PMAC='.$pmac
                        .' is linked to assy='.$assy
                        .' (PCS='.$pcs.'), formula='.$formula,0,'info');
  } else {
     &press_key_exit($script.': mtrOut='.$mtrNo.' in PMAC='.$pmac
                        .' is not linked to any assembly.',0,'info');
  }
  &clearAllCaMonitors();
  exit;

#=================================================================

sub signal_handler {
# local($sig) = @_;
  my $sig = shift;
  &press_key_exit('*** Caught a SIG'.$sig);
}

