#!/usr/bin/perl -W
#--------------------------------------------------------------
# This GUI script is aimed at emulating listAssemblies.tcl.
# It lists all beamline assemblies per PMAC along with their 
# PCS number and axes.
#
#                   Author: Sergey Stepanov
#
# Version 1.0  2019/02/20 initial implementation.
# Version 1.1  2020/02/07 added configNN.tcl check for duplicate pcs numbers in PMAC
#--------------------------------------------------------------
  use Tk;
  use Pezca;
  use strict;
  use warnings;

  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT, $IGNORE_ERRORS, $beamline);
  my ($PEZCA, $script, $syntax, $arg, $narg, @beamlinelist);
  my (@list, $nassy, $i, $j, $k, $m);
  my (@plst, $npl, $naxes, $ndrives, $str);
  my (@pmacs, $npmacs, $pmac);
  my ($redcolor, $pinkcolor, $bluecolor, $whitecolor);
  my ($mw, @blufr, @frame);

  if (!defined $ENV{PEZCA}) {
     die 'Please define the PEZCA environment. It must point to the PERL scripts depository';
  }
  $PEZCA = $ENV{PEZCA};
  $PEZCA =~ s/\\/\//g;
  if ($PEZCA !~ /\/$/) {$PEZCA .='/';}

# $beamline = '23i';

  do $PEZCA.'Common/motor_subs.pl';
  do $PEZCA.'Common/common_subs.pl';
  if (!defined $beamline) {do $PEZCA.'Common/beamline.pl';}
  else                    {do $PEZCA.'Common/beamline_'.$beamline.'.pl';}

  select(STDERR);  $|=1;				# Flush output after each print
  select(STDOUT);  $|=1;				# Flush output after each print

  $script = &makeScriptName();
  $syntax = "\n\n".'Syntax: '.$script.' [beamline]'."\n\n";
  @beamlinelist  = ('23i:', '23o:', '23b:', '23d:');
  $narg = @ARGV;
  if ($narg > 0) {print STDOUT $script.': started with cmdline = '."@ARGV\n\n";}
  else           {print STDOUT $script.': started with no cmdline arguments.'."\n\n";}

  $NONINTERACTIVE      = 1;
  $USE_ZENITY_FOR_EXIT = 1;
  $IGNORE_ERRORS       = 1;

  foreach $arg (@ARGV) {
     if (grep(/^${arg}:?$/, @beamlinelist)) {
        $arg =~ s/:$//;						#trim ':'
        if ($arg ne $beamline) {
           print STDOUT "\n".'!!! '.$script.': Cross-beamline operation: controlling '.$arg.' from '.$beamline."\n\n";
           $beamline = $arg;
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
	}
     } else {
        &press_key_exit('*** '.$script.': Unrecognized cmdline parameter "'.$arg.'"'.$syntax);
     }
  }

  $nassy = &generateComponentList($beamline,\@list);
  if ($nassy < 0) {
     &press_key_exit('*** '.$script.': error='.$nassy.' from generateComponentList');
  }
# for ($i=0; $i<$nassy; $i++) {
#    $naxes = $list[$i]->{'naxes'};  $ndrives = $list[$i]->{'ndrives'};  $nmotors = $list[$i]->{'nmotors'};
#    printf STDOUT "\n".'--------------'."\n".'label=%s'."\n",$list[$i]->{'label'};
#    printf STDOUT 'assy=%s, pmac=%s, pcs=%s, naxes=%s, ndrives=%s, nmotors=%s'."\n",
#                  $list[$i]->{'assy'},$list[$i]->{'pmac'},$list[$i]->{'pcs'},$naxes,$ndrives,$nmotors;
#    for ($j=0; $j<$naxes;   $j++) {$m = $j+1;  printf STDOUT '    axis%d=%s'."\n", $m, $list[$i]->{'axis'.$m};}
#    for ($j=0; $j<$ndrives; $j++) {$m = $j+1;  printf STDOUT '   drive%d=%s'."\n", $m, $list[$i]->{'drive'.$m};}
#    for ($j=0; $j<$nmotors; $j++) {$m = $j+1;  printf STDOUT '   motor%d=%s, no=%d'."\n", $m, $list[$i]->{'motor'.$m},$list[$i]->{'mtr'.$m.'_no'};}
# }
  @pmacs = ();
  for ($i=0; $i<$nassy; $i++) {
     $pmac = $list[$i]->{'pmac'};
     if (! grep {/^$pmac$/} @pmacs) {push @pmacs,$pmac;}
  }
  @pmacs = sort @pmacs;
  $npmacs = @pmacs;
# for ($i=0; $i<$npmacs; $i++) {print STDOUT $pmacs[$i],"\n";}
  
  $redcolor    = '#FF0000';
  $pinkcolor   = '#FFC0CE';
  $bluecolor   = '#5959E5';
  $whitecolor  = '#FFFFFF';
# $greencolor  = '#00BC00';
# $goldcolor   = '#D0D000';
# $yellowcolor = '#E0E000';
# $lightcolor  = '#FDFDFD';
### Create a GUI:
  $mw = MainWindow->new;
                $mw->Label(-text =>'Beamline='.$beamline, 
                           -anchor=>'w',
                           -borderwidth=>1,
                           -relief=>'groove',
                           -background=>$redcolor,
                           -foreground=>$whitecolor,)->grid(-row=>0, -column=>0, -pady=>0, -padx=>0, -columnspan=>$npmacs, -sticky=>'nsew',);
  for ($j=0; $j<$npmacs; $j++) {
     $blufr[$j] = $mw->Frame(-borderwidth=>2,
                           -relief=>'flat',
                           -background=>$bluecolor,)->grid(-row=>1, -column=>$j, -pady=>0, -padx=>0, -sticky=>'nsew',);
     $frame[$j] = $blufr[$j]->Frame(-borderwidth=>0,
                           -relief=>'flat')->pack(-side => 'top',-pady=>0,-padx=>0,-fill=>'both',-expand =>1,);
     $frame[$j]->Label(-text =>$pmacs[$j], 
                           -anchor=>'center',
                           -borderwidth=>0,
                           -relief=>'flat',
                           -background=>$pinkcolor,)->pack(-side => 'top',-pady=>0,-padx=>0,-fill=>'x');
     @plst = ();
     for ($i=0; $i<$nassy; $i++) {
        if ($list[$i]->{'pmac'} eq $pmacs[$j]) {push @plst,$list[$i];}
     }
     $npl = @plst;
     @plst = sort {$a->{'pcs'} <=> $b->{'pcs'}} @plst;

### Check pmac's assemblies list for duplicate pcs numbers:
     if ($npl > 1) {
        for ($i=1; $i<$npl; $i++) {
	   if ($plst[$i]{'pcs'} == $plst[$i-1]{'pcs'}) {
	      $str = '*** '.$script.': Assemblies '.$plst[$i]{'assy'}
	      .' and '.$plst[$i-1]{'assy'}.' in '.$pmacs[$j]
	      .' have the same number='.$plst[$i]{'pcs'}
	      .'. Check config'.$beamline.'.tcl!';
	      &zenity_error($str);
	   }
        }
     } 

     for ($i=0; $i<$npl; $i++) {
        $str = sprintf '%02d %s (', $plst[$i]->{'pcs'}, $plst[$i]->{'assy'};
        $naxes   = $plst[$i]->{'naxes'};  
        $ndrives = $plst[$i]->{'ndrives'};
        if ($naxes > 0) {
           for ($k=0; $k<$naxes; $k++) {
              $m = $k+1;  
              $str .= sprintf '%s ', $plst[$i]->{'axis'.$m};
           }
        } else {
           for ($k=0; $k<$ndrives; $k++) {
              $m = $k+1;  
              $str .= sprintf '%s ', $plst[$i]->{'drive'.$m};
           }
        }
        $str =~ s/\s+$/\)/;
#       $frame[$j]->Label(-text=>$str,                                  #this does not allow copying 
#                          -anchor=>'w',
#                          -borderwidth=>0,
#                          -relief=>'flat',)->pack(-side => 'top',-pady=>0,-padx=>0,-fill=>'x');
        $frame[$j]->Entry(-text=>$str, 
                          -state=>'readonly',
                          -readonlybackground=>'',
                          -width=>-1,
                          -borderwidth=>0,
                          -relief=>'flat',)->pack(-side => 'top',-pady=>0,-padx=>0,-fill=>'x');
     }
  }

  $mw->protocol('WM_DELETE_WINDOW', \&goodbye);

  $mw->resizable( 0, 0 );
  $mw->focus();

  $mw->repeat(1000,\&refresh_gui);

  MainLoop;

#==========================================================================
sub refresh_gui() {
  if (Exists($mw)) {$mw->update;}
}

#==========================================================================
sub goodbye() {
# require Tk::ErrorDialog;
  &clearAllCaMonitors();
# $mw->destroy;
# Tk::exit;
  exit 0;
}
