#!/usr/bin/perl -W
#----------------------------------------------------------------
# This script start terminal with environment specific to a PX
#                      program by user choice.
#
#	      	 	Author: Sergey Stepanov
#
# Version:	1.0 	2016/06/10 initial implementation
# Version:	1.1 	2017/01/27 introduced simple environment
# Version:	1.2 	2017/06/27 added autorun option
# Version:	1.3	2019/04/02 added serial-x (crystfel and anaconda)
#----------------------------------------------------------------

  use strict;
  use Tk;
  use Tk::Dialog;
# use warnings::unused;

### Declare variables ---------------------------------------------
  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT, $IGNORE_ERRORS);
  our ($mw, $prgFrame, $prgTitle, @prgRadio);
  our ($optFrame, @shellRadio, @autoRadio);

  my ($PEZCA, $script, $shell, @xterm, $px, $debug, $inline, $autorun);
  my (@prg, $nprg, @names, @selection, @shells, $nshell, @modes, $mode);
  my ($arg, $i, $rqs, $syntax, @autolbl, $mygroups);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;
  if ($PEZCA !~ /\/$/) {$PEZCA .='/';}

  select STDERR; $|=1;					#set unbuffered output
  select STDOUT; $|=1;					# set unbuffered output

  do $PEZCA.'Common/common_subs.pl';

# $debug = 1;						#comment out for operations

  if (! defined $debug) {$debug = 0;}
  if ($^O =~ /Win/i && ! $debug) {die 'This script cannot run under Windows OS';}

  $script = &makeScriptName();

  if	(defined &Which('gnome-terminal')) {@xterm = qw(gnome-terminal --disable-factory -t);}
  elsif (defined &Which('mate-terminal'))  {@xterm = qw(mate-terminal  --disable-factory -t);}
  else  				   {@xterm = qw(xterm +cm -sb -rightbar -sl 5000 -T);}

  if (defined $ENV{PX}) {$px=$ENV{PX};} else {$px='/mnt/software/px';}

  @shells  = qw(bash csh);
  $nshell  = @shells;
  @modes   = qw(simple advanced);
  @autolbl = ('open shell', 'autorun');
  $syntax  = "\n".'Syntax:'."\n".$script.' [program program ...]'
                 .' [bash|csh] [simple|advanced] [autorun] [help|?]'."\n\n";
  $inline = 0;						#exec in separate/same window
  $autorun= 0;

### env1: apply before sourcing the script, env2: apply after sourcing the script
  @prg = (
	  {name=>'autoPROC',	  dir=>$px.'/autoPROC/',		bash=>'setup.sh',	    csh=>'setup.csh',	      auto=>'process',  env1=>'',},
	  {name=>'Phenix',	  dir=>$px.'/phenix/',			bash=>'phenix_env.sh',	    csh=>'phenix_env.csh',    auto=>'phenix',   env1=>'',},
	  {name=>'DIALS',	  dir=>$px.'/dials/',			bash=>'dials_env.sh',	    csh=>'dials_env.csh',     auto=>'',         env1=>'',},
	  {name=>'CCTBX',	  dir=>$px.'/cctbx/',			bash=>'cctbx_env.sh',	    csh=>'cctbx_env.csh',     auto=>'',         env1=>'',},
#         {name=>'CCP4',	  dir=>$px.'/CCP4/ccp4/setup-scripts/',	bash=>'ccp4.setup-sh',	    csh=>'ccp4.setup-csh',    auto=>'ccp4i',    env2=>'PYTHONDIST=$CCP4/pythondist',},
#	  {name=>'Labelit',	  dir=>$px.'/labelit/labelit_build/',	bash=>'setpaths.sh',	    csh=>'setpaths.csh',      auto=>'',         env1=>'',},
#	  {name=>'XIA2',	  dir=>$px.'/xia2/',			bash=>'setup.sh',	    csh=>'setup.csh',	      auto=>'xia2',     env1=>'XIA2_ROOT=$PX/xia2',},
#	  {name=>'ARP/wARP',	  dir=>$px.'/arp_warp/',		bash=>'arpwarp_setup.bash', csh=>'arpwarp_setup.csh', auto=>'arp_warp', env1=>'',},
#	  {name=>'dTREK',	  dir=>$px.'/DTREK/bin/',		bash=>'DTREK_LOGIN.sh',	    csh=>'DTREK_LOGIN',	      auto=>'',         env1=>'DTREK_LOGIN_DIR=$PX/DTREK/bin',},
#	  {name=>'MAIN',	  dir=>$px.'/MAIN/',			bash=>'.login_BASH',	    csh=>'.login_CSH',	      auto=>'main',     env1=>'',},
#	  {name=>'Solve/Resolve', dir=>$px.'/solve/',			bash=>'',		    csh=>'',		      auto=>'',         env1=>'CCP4_OPEN=UNKNOWN SYMOP=$PX/solve/lib/solve/symop.lib SYMINFO=$PX/solve/lib/solve/syminfo.lib SOLVEDIR=$PX/solve/lib/solve SOLVETMPDIR=~/. PATH=$PX/solve/bin:$PATH',},
 	  {name=>'CNS_solve',	  dir=>$px.'/cns_solve/',		bash=>'cns_solve_env_bash', csh=>'cns_solve_env',     auto=>'',         env1=>'',},
#	  {name=>'XtalView',	  dir=>$px.'/XtalView/',		bash=>'XtalView.env.bash',  csh=>'XtalView.env',      auto=>'',         env1=>'',},
	  {name=>'Serial-X',	  dir=>$px.'/serial-x/',		bash=>'serialx-setup.sh',   csh=>undef,               auto=>'',         env1=>'',},
 	 );
  $nprg = @prg;
  for ($i=0; $i<$nprg; $i++) {
      if (! -e $prg[$i]{dir} && ! $debug) {
	 print STDOUT $script.': directory='.$prg[$i]{dir}
                    .' for pkg='.$prg[$i]{name}.' not found. Excluding.'."\n";
	 splice (@prg,$i,1);
	 $nprg--;
	 $i--;
      }
      elsif (! -r $prg[$i]{dir}) {
	 print STDOUT $script.': directory='.$prg[$i]{dir}
                    .' for pkg='.$prg[$i]{name}.' is not readble. Excluding.'."\n";
         if ($prg[$i]{name} =~ /autoPROC/i) {
	    $mygroups = `/usr/bin/groups`;
	    if ($mygroups !~ / nonproprietary/) {
	       print STDOUT $script.': the reason why you cannot access pkg='.$prg[$i]{name}
                           .' is because you are not a member of nonproprietary group.'."\n";
            }
	 }
	 splice (@prg,$i,1);
	 $nprg--;
	 $i--;
      } 
      else {
	 push(@names,$prg[$i]{name})
      }
  }
  if ($nprg < 1) {die $script.': none PX packages found.';}
# print STDOUT "@names\n";
  for ($i=0; $i<$nprg; $i++) {$selection[$i]=0;}
  $syntax .= 'Available programs: '."@{names}\n";

  $rqs = 0;
  foreach $arg (@ARGV) {
     if (grep(/^\Q$arg\E$/i, @names)) {
	for ($i=0; $i<$nprg; $i++) {
	   if ($names[$i] =~ /^$arg$/i) {$rqs++; $selection[$i]=1; last; }
	}
     }
     elsif (grep(/^\Q$arg\E$/, @shells)) {
	$shell = $arg;
     }
     elsif (grep(/^\Q$arg\E$/, @modes)) {
	$mode = $arg;
     }
     elsif ($arg =~ /^autorun$/) {
	$autorun = 1;
     }
     elsif ($arg =~ /^-{0,2}help$/i || $arg =~ /^-{0,2}\?$/) {
        print STDOUT $syntax;
	exit;
     }
     else {
        print STDOUT '*** '.$script.': unrecornized command line argument "'
                     .$arg.'".'.$syntax;
	exit;
     }
  }

  if (!defined $shell) {$shell=$shells[0];}
  if (!defined $mode)  {$mode=$modes[0];}

  if ($rqs > 0) {
     if ($rqs == 1) {$inline=1;}
     &start_selected_shells();
     exit;
  }

  &draw_main();

### ---------------------------------------------------------------
### Subs ----------------------------------------------------------
### ---------------------------------------------------------------

#=================================================================

sub  draw_main () {
  my ($i, $btnbkgcolor, $bpresscolor, $optcolor, $opresscolor, $btnWidth, $txt);

  $btnbkgcolor = '#0097FF';
  $bpresscolor = '#00D8FF';
  $optcolor    = '#FFFF00';
  $opresscolor = '#CCCC00';
  $btnWidth    = 24;

  $mw = MainWindow->new;

  $txt = 'Many PX software packages include the same components'."\n"
       . 'that may have different versions and then conflict with each'."\n"
       . 'other. While CCP4 is available by default, this tool starts a'."\n"
       . 'shell with environment prepared for selected package only.';

  $prgTitle = $mw->Label(-borderwidth=>2,
			 -text =>$txt
                        )->grid(-row=>0, -pady=>1, -padx=>2,);

  $prgFrame = $mw->Frame(-borderwidth=>2,
                        )->grid(-row=>1, -pady=>1, -padx=>2,);

  print STDOUT 'nprg='.$nprg."\n";
  for ($i=0; $i<$nprg; $i++) {
     $selection[$i] = 0;
     $prgRadio[$i] = $prgFrame->Checkbutton(-text=>$prg[$i]{name},
                                          -variable=>\$selection[$i],
                                          -onvalue =>1,
					  -offvalue=>0,
 		                          -command =>[\&reconfig, $i],
			                  -indicatoron=>0,
					  -foreground=>'yellow',
					  -activeforeground=>'white',
                                          -background=>$btnbkgcolor,
                                          -selectcolor=>$bpresscolor,
                                          -activebackground=>$btnbkgcolor,
                                          -width=>$btnWidth,
                                          -command=>\&start_selected_shells,
                        )->grid(-row=>$i,);
  }

  if ($mode =~ /^advanced$/) {
     $optFrame = $mw->Frame(-borderwidth=>0,
                        )->grid(-row=>2, -column=>0, -pady=>9, -padx=>9,);
     for ($i=0; $i<$nshell; $i++) {
        $shellRadio[$i] = $optFrame->Checkbutton(-text=>$shells[$i],
                                        -variable=>\$shell,
                                        -onvalue=>$shells[$i],
					-offvalue=>$shells[1-$i],
			                -indicatoron=>0,
                                        -background=>$optcolor,
                                        -selectcolor=>$opresscolor,
                                        -activeforeground=>'blue',
                                        -activebackground=>$optcolor,
			                -width=>int($btnWidth/2),
                        )->grid(-row=>0, -column=>$i);
     }
     for ($i=0; $i<2; $i++) {
        $autoRadio[$i] = $optFrame->Checkbutton(-text=>$autolbl[$i],
                                        -variable=>\$autorun,
                                        -onvalue=>$i,
					-offvalue=>1-$i,
			                -indicatoron=>0,
                                        -background=>$optcolor,
                                        -selectcolor=>$opresscolor,
                                        -activeforeground=>'blue',
                                        -activebackground=>$optcolor,
			                -width=>int($btnWidth/2),
                        )->grid(-row=>1, -column=>$i);
     }
  }

  MainLoop;
}

#--------------------------------------------------------------------------
sub reconfig($) {
  my ($current, $i);
  $current = shift (@_);
  if ($selection[$current] == 0) {return;}
### This is for selecting only one item:
  for ($i=0; $i<$nprg; $i++) {if ($i != $current) {$selection[$i]=0;}}
}

#--------------------------------------------------------------------------
sub start_selected_shells {
  my ($i);
  for ($i=0; $i<$nprg; $i++) {
     if ($selection[$i] != 0) {
        $selection[$i] = 0;
	if ($autorun && length($prg[$i]{auto}) > 0) {
           print STDOUT "\n".$script.': starting '.$prg[$i]{name}."\n";
	} else {
           print STDOUT "\n".$script.': opening shell for '.$prg[$i]{name}."\n";
	}
        &start_shell($prg[$i]{name},$shell,$prg[$i]{auto},$autorun);
     }
  }
}

#--------------------------------------------------------------------------
#sub start_shell($name,[$shell,$autoname,$autorun]) {
sub start_shell($;$$$) {
#	  {name=>'d*TREK',	  dir=>$px.'/DTREK/bin/', bash=>'DTREK_LOGIN.sh', csh=>'DTREK_LOGIN',	env1=>'DTREK_LOGIN_DIR=$PX/DTREK/bin',},
#	  {name=>'Solve/Resolve', dir=>$px.'/solve/',	  bash=>'',		  csh=>'',		env1=>'CCP4_OPEN=UNKNOWN SYMOP=$PX/solve/lib/solve/symop.lib SYMINFO=$PX/solve/lib/solve/syminfo.lib SOLVEDIR=$PX/solve/lib/solve SOLVETMPDIR=~/. PATH=$PATH:$PX/solve/bin',},
  my ($name, $shell, $i, @cmd, $scr, $env, $dir);
  my (@wrd1, @wrd2, $w, $str, $autoname, $autorun);
  $name     = shift (@_);
  $shell    = shift (@_); if (!defined $shell)    {$shell = $shells[0];}
  $autoname = shift (@_); if (!defined $autoname) {$autoname = '';}
  $autorun  = shift (@_); if (!defined $autorun)  {$autorun  = 0;}

  for ($i=0; $i<$nprg; $i++) {
     if ($prg[$i]{name} eq $name) {
        @cmd  = (@xterm, $name.'_shell', '-e');
        $dir  = $prg[$i]{dir};
        if (defined $prg[$i]{$shell}) {
           $scr  = $prg[$i]{$shell};
           if ($prg[$i]{env1}) {
              $env  = $prg[$i]{env1};
              @wrd1 = split /\s+/, $env;
	   } else {
	      @wrd1 = ();
	   }
           if ($prg[$i]{env2}) {
              $env  = $prg[$i]{env2};
              @wrd2 = split /\s+/, $env;
	   } else {
	      @wrd2 = ();
	   }
        } else {
           print STDOUT '!!! No environment for '.$name.' and '.$shell."\n";
	   next;
	}
#-------------------------
#       $str = 'env -i'
#            . ' PATH=/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin'
#            . ' PX='.$px
#	     . ' HOME='.$ENV{HOME}
#	     . ' HOSTNAME='.$ENV{HOSTNAME}
#	     . ' USERNAME='.$ENV{USER}
#	     . ' HOST='.$ENV{HOSTNAME}
#	     . ' USER='.$ENV{USER}
#	     . ' DISPLAY='.$ENV{DISPLAY};
#       foreach $w (@words) {$str .= ' '.$w;}
#       $str .= ' '.$shell;
#       if ($shell eq 'bash') {
#	   if (length($scr) > 0) {$str .= ' -c \"source '.$dir.$scr.'; bash\"';}
#	   if (length($scr) > 0) {$str .= ' --rcfile '.$dir.$scr;}
#	   if (length($scr) > 0) {$str .= ' --init-file '.$dir.$scr;}
#	   else                  {$str .= ' --norc';}
#       }
#       elsif ($shell eq 'csh') {
#          if (length($scr) > 0) {$str .= ' -c \"source '.$dir.$scr.'; csh\"';}
#	   else                  {$str .= ' -f';}
#       }
#-------------------------
        $str = $shell.' -c \'';
        if    ($shell eq 'bash') {foreach $w (@wrd1) {$str .= 'export '.$w.'; ';} }
        elsif ($shell eq 'csh')  {foreach $w (@wrd1) {$w =~ s/=/ /; $str .= 'setenv '.$w.'; ';}}
	if (length($scr) > 0) {$str .= 'source '.$dir.$scr.'; ';}
        if    ($shell eq 'bash') {foreach $w (@wrd2) {$str .= 'export '.$w.'; ';} }
        elsif ($shell eq 'csh')  {foreach $w (@wrd2) {$w =~ s/=/ /; $str .= 'setenv '.$w.'; ';}}
	if ($autorun && length($autoname) > 0) {$str .= $autoname.'; \'';}
	else                                   {$str .= $shell.'\'';}
        if (! $inline) {
	   push(@cmd, '"'.$str.'"', '&');
	   $str = join(' ',@cmd);
#	   $str = '/usr/bin/nohup '.$str;
	}
        print STDOUT 'CMD: '.$str."\n";
        if (! $inline) {
	   if ($^O !~ /Win/i) {system($str);}
	   last;
	} else {
           if ($^O !~ /Win/i) {exec($str);}
	   exit;
	}
     }
  }
}
