#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script is a frontend to GNUplot. It plots specified file(s) on screen.
# The script can plot multiple files on the same screen or a series of screens.
# For plotting multiple columns of the same file on one plot and producing
# plots as images, see plot.pl.
#
#     		     Author: Sergey Stepanov
# v.1.0 2011/02/18 ininital implementation
# v.1.1 2019/12/12 added -save option to save plots as images
#--------------------------------------------------------------
  use strict;

  our ($GNUPLOT_VERSION);
  my ($PEZCA, $script, $narg, $arg, $syntax, @filelist, @ff, $file);
  my ($xCol, $yCol, $status, $i, $nfiles, $cmd, $pause, $delay, $single);
  my ($lines, $save, $ext);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  if (-f $PEZCA.'/Common/gnuplot_subs.pl') {do $PEZCA.'/Common/gnuplot_subs.pl';}
  if (-f $PEZCA.'/Common/common_subs.pl')  {do $PEZCA.'/Common/common_subs.pl';}

  select STDOUT; $|=1;			# set unbuffered output
  select STDERR; $|=1;			# set unbuffered output

  $script  = &makeScriptName();
  $narg = @ARGV;
  $syntax = "\n".'Syntax: '.$script.' [-pause] [-delay=nn] [-singles] [-lines] [-save=PNG|GIF|JPG|JPEG] [x:y] <fileglob> <fileglob> ...'."\n\n";
  if ($narg>0) {$syntax .= '  Received: '.$script." @{ARGV}\n\n";}

  if ($narg < 1) {
    print STDOUT "\n".'*** '.$script.': Incorrect command line'."\n".$syntax;
    exit;
  }

  @filelist = ();
  $pause  = 0;				# no "Hit key to continue" before next plot
  $delay  = 1;				# delay in seconds before next plot
  $single = 0;				# plot all on one
  $lines  = 0;				# plot with lines instead of lines+boxes
  $xCol   = 1;				# default X-column number
  $yCol   = 2;				# default Y-column number

  for ($i=0; $i<$narg; $i++) {
     $arg = $ARGV[$i];
     if    ($arg =~ /^-pause/i)                {$pause=1;}
     elsif ($arg =~ /^-singles?$/i)            {$single=1;}
     elsif ($arg =~ /^-lines?$/i)              {$lines=1;}
     elsif ($arg =~ /^[0-9]{1,2}:[0-9]{1,2}$/) {($xCol,$yCol)=split /:/, $arg;}
     elsif ($arg =~ /^-delay=/i) {
	$arg =~ s/^-delay=//i;
	if ($arg =~ /^[0-9]*$/ && $arg > 0) {$delay=$arg;}
	else {print STDOUT '*** '.$script.': incorrect cmdline parameter '.$ARGV[$i]."\n";}
     }
     elsif ($arg =~ /^-save=/i) {
	$arg =~ s/^-save=//i;
        if ($arg=~/^png$/i || $arg=~/^gif$/i || $arg=~/^jpg$/i || $arg=~/^jpeg$/i) {$save=1; $ext=lc($arg);}
        else {print STDOUT '*** '.$script.': incorrect cmdline parameter '.$ARGV[$i]."\n";}
     }
     else {
        @ff = glob $arg;
        push (@filelist, @ff);
     }
  }
  $nfiles = @filelist;
  $i = 0;
  while ($i<$nfiles) {
     if (!(-f $filelist[$i])) {
        print STDERR '*** '.$script.': file='.$filelist[$i].' does not exist!'."\n";
	splice @filelist,$i,1;
	$nfiles--;
     } else {
        $i++;
     }
  }
  if ($nfiles < 1) {
     print STDERR "\n".'*** '.$script.': nothing to plot!'."\n\n";
     exit;
  }

  if ($single) {
     for ($i=0; $i<$nfiles; $i++) {
        $status = &open_gnuplot(undef,undef,undef,['-persist']);
        if (!$status) {die '*** '.$script.': Cannot open pipe to gnuplot'."\n";}
        sleep 1;
        ### Plot data without data points boxes:
        if ($lines) {
           if ($GNUPLOT_VERSION >= 4) {print GNUPLOT 'set style data lines'."\n";}
           else                       {print GNUPLOT 'set data style lines'."\n";}
        }
        print STDOUT $script.': Plotting file='.$filelist[$i]."\n";
        print GNUPLOT 'plot "'.$filelist[$i].'" using '.$xCol.':'.$yCol."\n";
        if ($pause) {&press_key_continue();}
        else        {sleep $delay;}
        if ($save) {&image_gnuplot($filelist[$i].'.'.$ext,undef,undef,1,1);}
        else       {&close_gnuplot();}
     }
#
  } else {
#
     $status = &open_gnuplot(undef,undef,undef,['-persist']);
     if (!$status) {die '*** '.$script.': Cannot open pipe to gnuplot'."\n";}
     sleep 1;
     ### Plot data without data points boxes:
     if ($lines) {
        if ($GNUPLOT_VERSION >= 4) {print GNUPLOT 'set style data lines'."\n";}
        else                       {print GNUPLOT 'set data style lines'."\n";}
     }
     for ($i=0; $i<$nfiles; $i++) {
        print STDOUT $script.': Plotting file='.$filelist[$i]."\n";
        if ($i == 0) {$cmd='plot';} 
        else         {$cmd='replot';}
        print GNUPLOT $cmd.' "'.$filelist[$i].'" using '.$xCol.':'.$yCol."\n";
        if ($pause) {
           if ($^O =~ /Win/i) {print GNUPLOT 'pause -1 "Hit any key to continue"'."\n";}	#Win32
           else               {&press_key_continue();}						#Unix
        } else {
           sleep $delay;
        }
     }
     if ($save) {&image_gnuplot($filelist[0].'.'.$ext,undef,undef,1,1);}
     else       {&close_gnuplot();}
  }
  exit;
