#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script runs a permanent loop copying PV-1 to PV-2
#
#     		     Author: Sergey Stepanov
#--------------------------------------------------------------
  use strict;
  use Pezca;

  my ($PEZCA, $script, $syntax, $narg);
  my ($pv1, $pv2, $val, $val1, $status,$sleeptime);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/common_subs.pl';

  select STDERR; $|=1;			# set unbuffered output
  select STDOUT; $|=1;			# set unbuffered output

  $script = &makeScriptName();
  $syntax = "\n\n".'Syntax: '.$script.' PVin PVout'."\n\n";
  $sleeptime = 0.05;

  $narg = @ARGV;
  if ($narg == 0) {
     $pv1 = '23i:scaler2_calc5.VAL';
     $pv2 = 'ID23:SteeringGoodness';
  }
  elsif ($narg == 2) {
     $pv1 = $ARGV[0];
     $pv2 = $ARGV[1];
  }
  else {
     print STDOUT "\n".' *** Incorrect number of command-line arguments!'.$syntax;
     exit;
  }

  $SIG{INT}  = 'signal_handler';
  $SIG{QUIT} = 'signal_handler';

  print STDOUT 'PVin  = '.$pv1."\n";
  print STDOUT 'PVout = '.$pv2."\n";

  $status = &addCaMonitor($pv1);
  &report_exit($script,$status,$pv1);

  $val1 = -1.E-37;
  while(1) {
     select(undef,undef,undef,$sleeptime);		# sleep(..);
     ($status,$val) = Pezca::GetDouble($pv1);
     &report_exit($script,$status,$pv1);
     if ($val != $val1) {
	$status = Pezca::PutDoubleOldCa($pv2,$val);
        &report_exit($script,$status,$pv2);
	$val1 = $val;
     }
  }
  &clearAllCaMonitors();
  exit;

#=================================================================

sub signal_handler {
# local($sig) = @_;
  my ($sig, $MIDcoord);
  $sig = shift;
  print STDOUT "\n".'*** Caught a SIG'.$sig."\n";
  &clearAllCaMonitors();
  print STDOUT 'Finished!'."\n";
  exit;
}
