#!/usr/bin/perl -W
#--------------------------------------------------------------
#  This script loads Lock-In Amplifier (LIA) parameters from a data file
#
#     	Authors: Sergey Stepanov & Sioan Zohar
#
# Version-1.0:  2014/06/09 using Pezca
#--------------------------------------------------------------
  use strict;
  use Pezca;

  our ($beamline, $NONINTERACTIVE, $IGNORE_ERRORS, $USE_ZENITY_FOR_EXIT);
  my ($PEZCA, @beamlinelist, $script, $syntax, $narg, $snap, $i, $arg);
  my ($lockin_asyn_PV, $fc, $status);

  our ($phase, $amplitude, $frequency, $gain,$timeConstant, $display);
  our ($lineFilter, $filterRollOff, $frontOutput);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDERR; $|=1;			# set unbuffered output
  select STDOUT; $|=1;			# set unbuffered output

  $NONINTERACTIVE      = 1;		# set as "1" to skip all dialogs
  $IGNORE_ERRORS       = 1;		# ignore EZCA errors
  $USE_ZENITY_FOR_EXIT = 1;		# use graphical popups

  @beamlinelist = ('23i', '23o', '23b', '23d');
  $script       = &makeScriptName();
  $syntax       = "\n\n".'Syntax: '.$script.' snapFile [beamline]'."\n";

  $narg = @ARGV;

  if ($NONINTERACTIVE == 0) {undef $NONINTERACTIVE;}

  for ($i=0; $i<$narg; $i++ ) {
     $arg = $ARGV[$i];
     if (grep(/^$arg$/i, @beamlinelist)) {
        if ($arg ne $beamline) {
           print STDOUT "\n".'!!! '.$script.': Cross-beamline operation: controlling '.$arg.' from '.$beamline."\n\n";
           $beamline = $arg;
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
	}
     }
     else {
        $snap = $ARGV[$i];
	print STDOUT $script.': snap file = '.$snap."\n";
     }
  }

  if (!defined $snap) {&press_key_exit('*** '.$script.': please specify a snap file to restore from!'.$syntax);}
  if (! -e $snap)     {&press_key_exit('*** '.$script.': cannot open '."\n".$snap.$syntax);}

  print STDOUT "\n";
  print STDOUT 'This script restores LIA settings from a previously recorded snap file'."\n";
  print STDOUT '     beamline = '.$beamline."\n";
  print STDOUT '     snapFile = '.$snap."\n";
# &ask_to_continue();

  $lockin_asyn_PV   = $beamline.':feedback:serialLIA';
### Different command for each beam line because different model lock-ins
  $fc = 'DDEF';    if ($beamline eq '23i') {$fc .= '1';}

### Source SNAP file (get the values to be loaded into LIA):
  $status = do $snap;
  if (!defined $status) {
     &press_key_exit('*** '.$script.': syntax error on '."\n".$snap."\n\n".$@);
  }
# unless ($status = do $snap) {
#    warn $script.': Could not parse '.$snap.": $@" if $@;
#    warn $script.': Could not do '   .$snap.": $!" unless defined $status;
#    warn $script.': Could not run '  .$snap        unless $return;
# }

  &putPV($lockin_asyn_PV.'.TMOD', 'Write');

  if (defined $phase)         {print STDOUT 'phase='.$phase."\n";                  &putPV($lockin_asyn_PV.'.AOUT', 'PHAS '.$phase);}
  if (defined $amplitude)     {print STDOUT 'amplitude='.$amplitude."\n";          &putPV($lockin_asyn_PV.'.AOUT', 'SLVL '.$amplitude);}
  if (defined $frequency)     {print STDOUT 'frequency='.$frequency."\n";          &putPV($lockin_asyn_PV.'.AOUT', 'FREQ '.$frequency);}
  if (defined $gain)          {print STDOUT 'gain='.$gain."\n";                    &putPV($lockin_asyn_PV.'.AOUT', 'SENS '.$gain);}
  if (defined $timeConstant)  {print STDOUT 'time constant='.$timeConstant."\n";   &putPV($lockin_asyn_PV.'.AOUT', 'OFLT '.$timeConstant);}
  if (defined $filterRollOff) {print STDOUT 'filter Rolloff='.$filterRollOff."\n"; &putPV($lockin_asyn_PV.'.AOUT', 'OFSL '.$filterRollOff);}
  if (defined $lineFilter)    {print STDOUT 'line filter='.$lineFilter."\n";       &putPV($lockin_asyn_PV.'.AOUT', 'ILIN '.$lineFilter);}
  if (defined $frontOutput)   {print STDOUT 'front output='.$frontOutput."\n";     &putPV($lockin_asyn_PV.'.AOUT', $fc.' '.$frontOutput);}

  &putPV($lockin_asyn_PV.'.TMOD','Write/Read');

  &press_key_exit($script.': Finished!',0,'info');


#==========================================================================
sub putPV ($$) {

  my ($pv2change, $new_pv_value, $ca_error, $maxStrLen);

  $pv2change    = shift (@_);
  $new_pv_value = shift (@_);

  $maxStrLen = 39;		  	# maximum length of EPICS string PV

  $ca_error = Pezca::PutString ($pv2change, substr($new_pv_value,0,$maxStrLen) );
  &report_only($script, $ca_error, $pv2change);
  select(undef,undef,undef,0.1);
  return $ca_error;
}
