#!/usr/bin/perl -W
#--------------------------------------------------------------
#
# This script recalculates energy differences for best focus
# into x differences
#
#                       Author: Sergey Stepanov
#
# Version-1.0:  2021/11/22	initial implementation
#--------------------------------------------------------------
  use Pezca;
  use DBI;

  use strict;
  use warnings;
# use warnings::unused;

  select STDERR; $|=1;			# set unbuffered output
  select STDOUT; $|=1;			# set unbuffered output

  our (@CRLconfig, @CRLconfig_full);	#cached array of the latest CRL config (what is normally read from SQL)
  our ($beamline);
  my ($PEZCA, $script, $E, $device);
  my ($focusRqs, $focusAct, @InOut);
  my ($nrecords, $exclude, $i);
  my ($E_exclude, $r_exclude);
  my ($focusTS, $focusV, $focusH, $ferrV, $ferrH);
  my (@En, @Ev, @Eh, @X, $nE);
  my ($n_in, $str, @IO, @rv, @rh);

  if (!defined $ENV{PEZCA}) {
     die 'Please define the PEZCA environment. It must point to the PERL scripts repository';
  }
  $PEZCA = $ENV{PEZCA};
  $PEZCA =~ s/\\/\//g;
  if ($PEZCA !~ /\/$/) {$PEZCA .='/';}

  $beamline = '23i';
  if (-e $PEZCA.'Common/common_subs.pl') {
     do $PEZCA.'Common/common_subs.pl';
     do $PEZCA.'Common/motor_subs.pl';
     do $PEZCA.'Common/mono_subs.pl';
     do $PEZCA.'Common/crl_subs.pl';
     if (defined $beamline) {do $PEZCA.'/Common/beamline_'.$beamline.'.pl';}
     else                   {do $PEZCA.'/Common/beamline.pl';}
  } else {die '*** Cannot find PEZCA subs in '.$PEZCA."\n";}
  
  $script = &makeScriptName('noext');
  if (!defined $E) {$E=&getEnergy();}
  if ($E <= 0) {die '*** '.$script.': Incorrect energy='.$E."\n";}
### If max range is undefined, it is calculated automatically:
# if (!defined $r) {$r = 0;}

  $device    = 'CRL-2020';
  $E_exclude = 26;			#exclude small-radius lenses below this energy in keV
  $r_exclude = 50;			#any lenses with this or smaller radius will be excluded an smaller energies
  $focusRqs  = 1538.80;

### Test experimental data -- recalculate experimentally determined
### energy for best focus into focus deviation:
  print STDOUT "\n".'---------------------------------------------------'
              ."\n".'Recalculating experimentally determined energy for best focus into shifts:'."\n";
  #values used in the first version of the paper
 #@En = qw(6.5      7.0      7.5      8.0      8.5      9.0       9.5       10.0     10.5     11.0      11.5     12.0              12.5     13.0     13.5     14.0     16.0     16.5     17.0     17.5     18.0     18.5     19.0);
 #@Ev = qw(6.556    7.035    7.560    8.059    8.531    9.050     9.503     10.033   10.551   11.049    11.533   12.030            12.511   12.991   13.481   13.980   16.000   16.520   17.055   17.556   18.060   18.575   18.966);
 #@Eh = qw(6.491    7.015    7.537    8.043    8.531    9.015     9.503     10.013   10.528   11.029    11.520   12.022            12.510   12.991   13.488   14.005   16.125   16.454   17.026   17.482   18.063   18.524   19.059);
 #@X  = qw(-5.115   5.938    2.413    10.637   7.814    0.358     -0.788    -0.629   -0.517   0.424     0.101    0.028             -0.127   0.030    0.280    0.046    0.036    -0.336   0.070    -0.048   -0.002   0.011    -0.051);
  @En = qw(6.5      7.0      7.5      8.0      8.5      9.0       9.5       10.0     10.5     11.0      11.5     12.0     12.0     12.5     13.0     13.5     14.0     16.0     16.5     17.0     17.5     18.0     18.5     19.0);
  @Ev = qw(6.54043  7.02577  7.55704  8.05344  8.53055  9.03305   9.52805   10.0149  10.5393  11.0371   11.5221  12.0446  12.0307  12.5114  12.9912  13.4815  13.9798  15.9998  16.5205  17.0549  17.5568  18.0604  18.5785  19.4616);
  @Eh = qw(6.49129  7.01479  7.53668  8.04274  8.53055  9.01485   9.50330   10.0134  10.5173  11.0153   11.5097  12.0282  12.0222  12.5097  12.9898  13.4870  14.0032  16.1101  16.4575  17.0279  17.4907  18.0564  18.4810  19.1882);
  @X  = qw(-5.115   5.938    2.413    10.637   7.814    0.358     -0.788    -0.629   -0.517   0.424     0.101    0.028    0.028    -0.127   0.030    0.280    0.046    0.036    -0.336   0.070    -0.048   -0.002   0.011    -0.051);
#V-fit and H-fit errors (keV)
# @rv = qw(0.009322 0.002270 0.004271 0.004140 34.5911  0.0048445 0.0067468 0.002980 0.003756 0.0033290 0.004068 0.004498 0.003730 0.002279 0.003752 0.003756 0.004837 0.003960 0.004627 0.007310 0.007995 0.005799 0.0165214 500.961);
# @rh = qw(0.004263 0.002126 0.003326 0.003157 0.002086 0.001674  0.0028716 0.002274 0.002757 0.0024958 0.003744 0.006919 0.002202 0.003759 0.004215 0.005498 0.014229 447.886  0.160009 4.99966  21.1552  124.253  3633.28   349.487);
  @rv = qw(0.009322 0.002270 0.004271 0.004140 na       0.0048445 0.0067468 0.002980 0.003756 0.0033290 0.004068 0.004498 0.003730 0.002279 0.003752 0.003756 0.004837 0.003960 0.004627 0.007310 0.007995 0.005799 0.0165214 na);
  @rh = qw(0.004263 0.002126 0.003326 0.003157 0.002086 0.001674  0.0028716 0.002274 0.002757 0.0024958 0.003744 0.006919 0.002202 0.003759 0.004215 0.005498 0.014229 na       0.160009 na       na       na       na        na);

# @IO = qw(11011011011010 11111111111111 ...);
  $nE = @En;
  @CRLconfig = ();
  $nrecords = &readCRLconfigSQLpopulated($device,\@CRLconfig);
  @CRLconfig_full = @CRLconfig;							#full set including 50um lenses
  $exclude = 0;
  for ($i=0; $i<$nE; $i++) {
     if ($En[$i] <= $E_exclude && $exclude == 0) {				#need to exclude small lenses
        $nrecords = &excludeCRLradius($r_exclude,$device);
        if ($nrecords > 0) {
           $exclude = 1;
        } else {
           print STDOUT 'Eliminating small lenses failed for E='.$En[$i];
           @CRLconfig = @CRLconfig_full;					#restore full set including 50um lenses
           $exclude = 1;
        }
     }
     elsif ($En[$i] > $E_exclude && $exclude == 1) {				#need to add back small lenses
        @CRLconfig = @CRLconfig_full;						#restore full set including 50um lenses
        $exclude = 0;
     }
     $nrecords = @CRLconfig;
     ### The matchTransfocatorFocus4 function finds a combination of lens
     ### In/Out that the transfocator server presumably used during the
     ### measurements. If the combination was different (for example if
     ### a different lens matching algorith was used), comment out the
     ### matchTransfocatorFocus4 call and call the InOut_str2arr with
     ### your combination as in the example below.
     ($focusAct,@InOut) = &matchTransfocatorFocus4($focusRqs,$En[$i],$device);
#    $n_in = &InOut_str2arr('11011011011010',\@InOut);
#    if (length($IO[$i]) != $nrecords) {die 'Incorrect string length for '.$IO[$i]."\n";}
#    $n_in = &InOut_str2arr($IO[$i],\@InOut);
     $focusTS = &calcTransfocatorFocus4(\@InOut,$En[$i],$device);
     $focusV  = &calcTransfocatorFocus4(\@InOut,$Ev[$i],$device);
     if ($rv[$i] ne 'na') {
       $ferrV = &calcTransfocatorFocus4(\@InOut,$Ev[$i]+0.5*$rv[$i],$device)
              - &calcTransfocatorFocus4(\@InOut,$Ev[$i]-0.5*$rv[$i],$device);
     } else {
       $ferrV = 99.999;
     }
     $focusH  = &calcTransfocatorFocus4(\@InOut,$Eh[$i],$device);
     if ($rh[$i] ne 'na') {
       $ferrH = &calcTransfocatorFocus4(\@InOut,$Eh[$i]+0.5*$rh[$i],$device)
              - &calcTransfocatorFocus4(\@InOut,$Eh[$i]-0.5*$rh[$i],$device);
     } else {
       $ferrH = 99.999;
     }
     ($str,$n_in) = &InOut_arr2str(\@InOut);
     printf STDOUT 'E=%-5.2fkeV focus(Bruls)=%-7.2fmm IN/OUT=(%s)'
                  .' dV(mm)=%-7.2f dH(mm)=%-7.2f'
                  .' dV(%s)=%-6.2f dH(%s)=%-6.2f'
                  .' errV(mm)=%-7.4f errH(mm)=%-7.4f'."\n", 
                  $En[$i], $focusTS, $str,
                  ($focusV-$focusTS), ($focusH-$focusTS),
                  '%',100*($focusV-$focusTS)/$focusRqs,
                  '%',100*($focusH-$focusTS)/$focusRqs,
                  $ferrV, $ferrH;
  }                   

  exit;
