#!/usr/bin/perl -W
#--------------------------------------------------------------
# This is a converter of Russian filenames into translaterated
# English. The original names are stored in UTF8. So, the script
# first converts them into Win-1251 and then  transliterates.
# Spaces are converted into "_".
#
#                     Author: Sergey Stepanov
#
# Version-1.0:  2018/10/21
#--------------------------------------------------------------
  use strict;
  use Encode qw(from_to is_utf8);

  my ($script, $narg, @filelist, @plist, $nfiles, $file, $i);

  select STDOUT; $|=1;				#set unbuffered output
  select STDERR; $|=1;				#set unbuffered output

  $script = $0 =~ s/^.*[\/\\]//gr;
  $narg = @ARGV;


### ATTENTION: fileglob is something like: xxx.*  or aa*bb.ccc
###            to verify fileglob on Windows/DOS:  dir /b xxx.*
###            to verify fileglob on Linux/Unux:   ls -1 xxx.*
  if ($narg < 1) {die 'Syntax: '.$script.' fileglob';}

### On different platforms PERL interprets fileglobs in the command line differently:
### on Windows it simply passes it to the script as a single argument, while
### on UNIX the script receives a parsed list (i.e. many arguments instead of one).
### A way to work around it would be to put the fileglob in quotes, but it would
### be something hard to remember. So, here we presume that when PERL sees only
### one argument, then it is a fileglob; otherwise the command line arguments
### except for the last two list the files corresponding to the glob:

  @filelist = ();
     for ($i=0; $i<$narg; $i++) {
        if ($ARGV[$i] =~ /[\*\?]/) {
           @plist = glob($ARGV[$i]);
           push (@filelist,@plist);
        } else {
           push (@filelist,$ARGV[$i]);
	}
     }
### Loop over files: ######################################################
  $nfiles = @filelist;
  for ($i=0; $i<$nfiles; $i++) {
     $file = $filelist[$i];
     if (-e $file) {
        &from_to($file, 'utf8', 'cp1251');
        &translit(\$file,1);				#second argument is the "safe" flag
        print STDOUT $filelist[$i],' => ',$file,"\n";
        rename($filelist[$i],$file);
     } else {
        print STDOUT '*** '.$script.': cannot find ['.$file.']'."\n";
     }
  }
  exit;

#==============================================================
sub translit (\$;$) {
  my ($stref, $safe, @tbl, $nchar, $st, $i, $c, $n);
  ### This is transcoding table from Win-1251 to Translit
  ### Additional two letters which are not included are
  ### "Yo" (168) and "yo" (184). We map them to "E" (197)
  ### and "e" (229).
  ### For more details see https://en.wikipedia.org/wiki/Windows-1251
  @tbl = ('A',    #00 192   A
          'B',    #01 193   B
          'V',    #02 194   V
          'G',    #03 195   G
          'D',    #04 196   D
          'E',    #05 197   E
          'Zh',   #06 198   Zhe
          'Z',    #07 199   Z
          'I',    #08 200   I
          'Y',    #09 201   Y
          'K',    #10 202   K
          'L',    #11 203   L
          'M',    #12 204   M
          'N',    #13 205   N
          'O',    #14 206   O
          'P',    #15 207   P
          'R',    #16 208   R
          'S',    #17 209   S
          'T',    #18 210   T
          'U',    #19 211   U
          'F',    #20 212   F
          'H',    #21 213   H
          'Ts',   #22 214   Tse
          'Ch',   #23 215   Che
          'Sh',   #24 216   She
          'Ch',   #25 217   Sche
          "'",    #26 218   Bolshoi miagkij znak
          'I',    #27 219   bI (Boldhoe "bI")
          "'",    #28 220   Bolshoi tverdij znak
          'E',    #29 221   E (kak "Ekvator")
          'Ju',   #30 222   Ju
          'Ja',   #31 223   Ja
          'a',    #32 224   a
          'b',    #33 225   b
          'v',    #34 226   v
          'g',    #35 227   g
          'd',    #36 228   d
          'e',    #37 229   e
          'zh',   #38 230   zhe
          'z',    #39 231   z
          'i',    #40 232   i
          'y',    #41 233   y
          'k',    #42 234   k
          'l',    #43 235   l
          'm',    #44 236   m
          'n',    #45 237   n
          'o',    #46 238   o
          'p',    #47 239   p
          'r',    #48 240   r
          's',    #49 241   s
          't',    #50 242   t
          'u',    #51 243   u
          'f',    #52 244   f
          'h',    #53 245   h
          'ts',   #54 246   tse
          'ch',   #55 247   che
          'sh',   #56 248   she
          'ch',   #57 249   sche
          "'",    #58 250   malenkij miagkij znak
          'i',    #59 251   bi (malenkoe "bI")
          "'",    #60 252   malenkij tverdij znak
          'e',    #61 253   e (kak "ekvator")
          'ju',   #62 254   ju
          'ja');  #63 255   ja
  $stref = shift(@_);
  $safe  = shift(@_);
  if (!defined $safe) {$safe=0;}
  if ($safe =~ /^safe$/i) {$safe=1;}

  $nchar = length($$stref);
  $st = '';
  for ($i=0; $i<$nchar; $i++) {
     $c = substr($$stref,$i,1);
     $n = ord($c);
     if    ($n >= 192 && $n <= 255) {$c=$tbl[$n-192]; if($safe) {$c=~s/\'//g;}}
     elsif ($n == 168)              {$c=$tbl[197-192];}
     elsif ($n == 184)              {$c=$tbl[229-192];}
     elsif ($c eq ' ')              {$c='_';}
     $st .= $c;
  }
  $$stref = $st;
}
