#!/usr/bin/perl -W
#--------------------------------------------------------------
#     This script cleans vector collection data for JBluIce.
#
#     	            	Author: Sergey Stepanov
#
# Version-1.0: 2010/09/27 using Pezca
#--------------------------------------------------------------
  use strict;
  use Pezca;
  use DBI;

  our ($beamline, $IGNORE_ERRORS, $NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($PEZCA, $script, @beamlinelist, $narg, $arg, $i, $status, $bl_specified);
  my ($syntax, $msg, $nRuns, $vectorEnablePV, $vectorEnable, $pv, $val, @cleans);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;					#set unbuffered output

  $script = &makeScriptName();
  $script =~ s/\.pl$//i;
  $syntax = "\n".'Syntax: '.$script.' [beamline]'."\n\n";

  $IGNORE_ERRORS = 1;
  $NONINTERACTIVE = 1;
  $USE_ZENITY_FOR_EXIT = 1;

  $narg         = @ARGV;
  @beamlinelist = ('23i', '23o', '23b', '23d');
  $nRuns        = 16;
  @cleans       = ();

  $SIG{HUP}  = 'signal_handler';	#(1) hangup (see signum.h)
  $SIG{INT}  = 'signal_handler';	#(2) interrupt (see signum.h)
  $SIG{QUIT} = 'signal_handler';	#(3) quit (see signum.h)
# $SIG{KILL} = 'signal_handler';	#(9) this is what is sent by kill -9 -- unblockable
  $SIG{TERM} = 'signal_handler';	#(15) terminate -- sent by kill without paramaters
# $SIG{STOP} = 'signal_handler';	#(19) stop -- unblockable

  for ($i=0; $i<$narg; $i++) {
     $arg = $ARGV[$i];
     $arg =~ s/:$//;
     $arg =~ s/^[-]?//;
     if (grep(/^${arg}$/, @beamlinelist)) {
	if (defined $bl_specified) {
### This will exit automatically if $NONINTERACTIVE=1
           &press_key_exit("\n".'*** '.$script.': Duplicate beamline parameter in cmdline.'.$syntax);
	}
	$bl_specified = 1;
        if ($arg ne $beamline) {
           $beamline = $arg;
           print STDOUT "\n".'!!! '.$script.': Cross-beamline operation for beamline='.$beamline."\n\n";
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
	}
     }
     else {
### This will exit automatically if $NONINTERACTIVE=1
        &press_key_exit("\n".'*** '.$script.': Unrecognized cmdline parameter ['.$ARGV[$i].']'.$syntax);
     }
  }

  $vectorEnablePV = $beamline.':bi:enableVectorCollect';
  ($status,$vectorEnable) = Pezca::GetDouble($vectorEnablePV);
  if ($status) {
     &tellBluIce($script.': cannot connect to pv='.$vectorEnablePV,'error');
     &report_only($script, $status, $vectorEnablePV);
     exit 1;
  }
  if ($vectorEnable == 0) {
     &tellBluIce($script.': vector collection is already disabled','warn');
     $USE_ZENITY_FOR_EXIT = 0;
     &press_key_exit("\n".'*** '.$script.': vector collection is already disabled');
  } else {
     &tellBluIce($script.': disabling vector collection');
     print STDOUT $script.': disabling vector collection'."\n";
     $status=Pezca::PutDouble($vectorEnablePV,0);
  }
  for ($i=0; $i<$nRuns; $i++) {
     $pv = $beamline.':bi:r'.$i.':start_set';
     ($status,$val) = Pezca::GetDouble($pv);
     if ($status) {
        &tellBluIce($script.': cannot connect to pv='.$pv,'error');
        &report_only($script, $status, $pv);
	$val = 0;
     }
     if ($val) {
	if (!defined $msg) {
           $msg = 'Do you also want to erase all data collection vectors?';
           if (&zenity_question($msg) == 0) {exit 2;}	#this will return "512" to calling perl script
        }
	print STDOUT $script.': erasing vector collection data for run='.$i."\n";
        push (@cleans, $i);
	$status=Pezca::PutDouble($pv,0);
        $pv = $beamline.':bi:r'.$i.':start_x_mm';  $status=Pezca::PutDouble($pv,0);
        $pv = $beamline.':bi:r'.$i.':start_y_mm';  $status=Pezca::PutDouble($pv,0);
        $pv = $beamline.':bi:r'.$i.':start_z_mm';  $status=Pezca::PutDouble($pv,0);
        $pv = $beamline.':bi:r'.$i.':end_set';     $status=Pezca::PutDouble($pv,0);
        $pv = $beamline.':bi:r'.$i.':end_x_mm';    $status=Pezca::PutDouble($pv,0);
        $pv = $beamline.':bi:r'.$i.':end_y_mm';    $status=Pezca::PutDouble($pv,0);
        $pv = $beamline.':bi:r'.$i.':end_z_mm';    $status=Pezca::PutDouble($pv,0);
     }
  }
  $USE_ZENITY_FOR_EXIT = 0;
  if (defined $msg) {
     &tellBluIce($script.': the following vectors are cleaned: ('."@{cleans}".')');
     &press_key_exit("\n".$script.': the following vectors are cleaned: ('."@{cleans}".')');
  } else {
     &tellBluIce($script.': no vectors cleaning was required');
     &press_key_exit("\n".$script.': no vectors cleaning was required');
  }

#=================================================================

sub signal_handler ($;$) {		#($signal,$code)
  my $signame = shift(@_);
  my $code    = shift(@_);
  if ($signame ne 'quit') {print STDOUT "\n".' *** Caught a SIG'.$signame."\n\n";}
  else                    {print STDOUT "\n".' !!! cleaning up before quit'."\n\n";}

  &clearAllCaMonitors();
  print STDOUT 'Done!'."\n\n";
  if (defined $code) {exit $code;}	#this will return '256*n' to calling perl script (when n is small)
  else               {exit 1;}		#this will return '256' to calling perl script
}
