#!/usr/bin/perl -w
#--------------------------------------------------------------
# This script is a simple emulator of Zenity (GTK+ dialog displayer),
# especially for MS Windows
#
#                   Author: Sergey Stepanov
#
#  Version-1: 2007/10/02
#  Version-2: 2009/03/11   Added question dialog
#--------------------------------------------------------------
  use Tk;
# use Getopt::Long;
  use strict;

  my ($syntax, $title, $type, $buttons, $msg, $arg, $cmd);
  my ($mw, $response);
# my ($status, $terror, $tinfo, $twarning, $tnotification, $tquestion, $tyesno);

### ATTN-1: 'question' displays OK/Cancel;
### ATTN-2: 'yesno' is an extention to standard zenity syntax:it replaces OK/Cancel with Yes/No
  $syntax = "Syntax: zenity.pl --title='title' --error|info|warning|notification|question|yesno --text='msg'";


### For UNIX:
  if ($^O !~ /Win/i) {
     foreach $arg (@ARGV) {
        if (($arg =~ /^-{0,2}title=/i) && (!defined $title)) {
           $arg =~ s/^-{0,2}title=//i;
           $title = $arg;
        }
        elsif (($arg =~ /^-{0,2}text=/i) && (!defined $msg)) {
           $arg =~ s/^-{0,2}text=//i;
           $msg = $arg;
        }
        elsif (($arg =~ /^-{0,2}error$/i) && (!defined $type)) {
           $type = 'error';
        }
        elsif (($arg =~ /^-{0,2}info$/i) && (!defined $type)) {
           $type = 'info';
        }
        elsif (($arg =~ /^-{0,2}warning$/i) && (!defined $type)) {
           $type = 'warning';
        }
        elsif (($arg =~ /^-{0,2}notification$/i) && (!defined $type)) {
           $type = 'info';
        }
        elsif (($arg =~ /^-{0,2}question$/i) && (!defined $type)) {
           $type = 'question';
	   $buttons = 'okcancel';
        }
        elsif (($arg =~ /^-{0,2}yesno$/i) && (!defined $type)) {
           $type = 'question';
	   $buttons = 'yesno';
        }
        else {
           print STDOUT 'zenity.pl: ignoring unrecognized or dupplicate argument "'.$arg.'"'."\n";
           print STDOUT $syntax."\n";
        }
     }
#    $status = GetOptions ("title=s"      => \$title,          # string
#                          "text=s"       => \$msg,            # string
#                          "error"        => \$terror,         # flag
#                          "info"         => \$tinfo,          # flag
#                          "warning"      => \$twarning,       # flag
#                          "notification" => \$tnotification); # flag
#                          "question"     => \$tquestion);     # flag
#                          "yesno"        => \$tyesno);        # flag
#    if (defined $title        ) {print STDOUT "        title = ${title}\n";}
#    if (defined $msg          ) {print STDOUT "         text = ${msg}\n";}
#    if (defined $terror       ) {print STDOUT "        error = ${terror}\n";}
#    if (defined $tinfo        ) {print STDOUT "         info = ${tinfo}\n";}
#    if (defined $twarning     ) {print STDOUT "      warning = ${twarning}\n";}
#    if (defined $tnotification) {print STDOUT " notification = ${tnotification}\n";}
#    if (defined $tquestion)     {print STDOUT "     question = ${tquestion}\n";}
#    if (defined $tyesno)        {print STDOUT "        yesno = ${tyesno}\n";}
  }
  else {
### The above options do not process arguments with quotes on MS Windows.
### Therefore, we merge all the argumemts in one string and then analyze it:
     $cmd = '';
     foreach $arg (@ARGV) {
#       print STDOUT 'arg = '.$arg."\n";
        $cmd = $cmd.' '.$arg;
     }
#    print STDOUT $cmd."\n";
     if    ($cmd =~ /-{1,2}error /i)        {$type = 'error';}
     elsif ($cmd =~ /-{1,2}info /i)         {$type = 'info';}
     elsif ($cmd =~ /-{1,2}warning /i)      {$type = 'warning';}
     elsif ($cmd =~ /-{1,2}notification /i) {$type = 'info';}
     elsif ($cmd =~ /-{1,2}question /i)     {$type = 'question'; $buttons='okcancel';}
     elsif ($cmd =~ /-{1,2}yesno /i)        {$type = 'question'; $buttons='yesno';}

     if ($cmd =~ /-{1,2}title=/i) {
        $title = $cmd;
        $title =~ s/^.*-{1,2}title=//i;
        if ($title =~ /^\'/) {			# if in single quotes
           $title =~ s/^\'//;
           $title =~ s/\'.*$//;			#crop any characters after closing quote
        }
        elsif ($title =~ /^\"/) {		#if in double quotes
           $title =~ s/^\"//;
           $title =~ s/\".*$//;			#crop any characters after closing quote
        }
        else {
           $title =~ s/ .*$//;			#not in quotes: trail everything after the first space
        }
#	print STDOUT 'TITLE: '.$title."\n";
     }
     if ($cmd =~ /-{1,2}text=/i) {
        $msg = $cmd;
        $msg =~ s/^.*-{1,2}text=//i;
	if ($msg =~ /^\'/) {			#if in single quotes
           $msg =~ s/^\'//;
           $msg =~ s/\'.*$//;			#crop any characters after closing quote
        }
        elsif ($msg =~ /^\"/) {			#if in double quotes
           $msg =~ s/^\"//;
           $msg =~ s/\".*$//;			#crop any characters after closing quote
        }
        else {
           $msg =~ s/ .*$//;			#not in quotes: trail everything after the first space
        }
        $msg =~ s/ /  /g;			#this improves readability: single spaces too tiny
#	print STDOUT 'MSG: '.$msg."\n";
     }
  }

  if (!defined $title) {$title = '';}
  if (!defined $type)  {$type  = 'info';}
  if (!defined $msg) {
     print STDOUT 'zenity.pl: no message specified'."\n";
     print STDOUT $syntax."\n";
     exit 1;
  }
### Replace \r by carriage return:
# print STDOUT "$msg\n";
  $msg =~ s/\\\\n/\n/g;
  $msg =~ s/\\n/\n/g;
  $msg =~ s/\\r/\n/g;

  $mw = MainWindow->new();
# $mw->title($title);
# $mw = MainWindow->new(-title => $title);
# Mainwindow: sizex/y, positionx/y
# $mw->geometry("220x150+100+120");
  $mw->withdraw();				                #hide main window
# $mw->deiconify();
# $mw->raise();

  if ($type =~ /^question$/i) {
     $response = $mw->messageBox(-title=>$title,
#                                -background=>$btnbkgcolor,
#                                -activebackground=>$btnbkgcolor,
#                                -foreground=>$btnbkgcolor,
#                                -activeforeground=>$btnbkgcolor,
#                                -default=>'No',
#                                -default=>'Cancel',
                                 -message=>$msg,
                                 -wrap=>0,
                                 -type=>$buttons,               #buttons: 'okcancel' or 'yesno'
                                 -icon=>'question');
     if ($response =~ /^Yes$/i ||
         $response =~ /^OK$/i) {print "OK\n";     exit 0;}
     else                      {print "Cancel\n"; exit 1;}
  }
  else {
     $mw->messageBox(-title=>$title,
                     -message=>$msg,
                     -wrap=>0,
                     -type=>'ok',
                     -icon=>$type);                             #type: 'error', 'info', 'warning'
     exit 0;
  }

  MainLoop;

###-------------------------------------------------------------------
# sub bye_action {
#	print "Goodbye\n";
# }
