package gov.anl.gmca.analysis;

import gov.anl.gmca.analysis.DB2Html.RESULT_PAGE;
import gov.anl.gmca.analysis.helpers.MiscUtils;
import gov.anl.gmca.analysis.helpers.WdgProtectedRunnableDialog;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.aps.jca.dbr.DBR;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.*;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.io.*;
import java.util.regex.Pattern;

public class CTabBrowser implements ClientListener {
	public static boolean usePV = true; // use PV or pool for updating

	private String bluiceDir = Main.getAppDir() != null ? Main.getAppDir(): "/mnt/software/bluice";
	private final String helpHtml = bluiceDir + "/java/docs/dataproc/dataproc_manual.html";
	private final String helpPdf = bluiceDir + "/java/docs/dataproc/dataproc_manual.pdf";
	private volatile Integer poolInterval = 1000; // Milliseconds
	private boolean autoRefresh = true;
	private boolean updateAvailable = false;
	private CTabFolder folder;
	private String html;
	private Result currentStates;
	private RESULT_PAGE pageType;
	private Composite c;
	private Shell cshell;
	private CTabItem item;
	private Browser browser;
	boolean recreateOnReload = true;
	private ToolItem settings, kamo;
	// this is used so that user can wake up wait immediately when user select Analysis Tab 
	private final Object lockObj = new Object();

	CTabBrowser(final CTabFolder folder, final String html, RESULT_PAGE pageType) {
		this.folder = folder;
		this.html = html;
		this.pageType = pageType;
		
		this.currentStates = null; 
		if (pageType == RESULT_PAGE.PROCESSING_RESULTS || pageType == RESULT_PAGE.SCREENING_RESULTS) {
			this.currentStates = new DB2Html(pageType).getStates();
		}

		this.item = new CTabItem(folder, SWT.NONE);
		this.c = new Composite(folder, SWT.NONE);
		this.cshell = c.getShell();
		item.setControl(c);
		//c.setLayout(new FillLayout());
		GridLayout gl = new GridLayout();
		gl.numColumns = 1;
		gl.makeColumnsEqualWidth = false;
		gl.marginHeight = 0;
		gl.marginWidth = 0;
		c.setLayout(gl);
		
		browser = new Browser(c, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessVerticalSpace = true;
		browser.setLayoutData(gd);
		
		addContent();
		addBrowserListeners();
		copyManual();

		boolean toolbarHidden = false;
		if (!toolbarHidden) {
			createToolBar(c);
		}
		
		folder.setSelection(item);
		
		Client.addListener(this);
	}
	
	private void copyManual() {
		// copy html and pdf manuals to user directory 
		String destDir = getPrimaryDir();
		File wdir = new File(destDir);
		if (!wdir.exists() || !wdir.canWrite()) {
			System.out.println(destDir + " does not exist or not writable, skip saving manuals.");
			return;
		}
		MiscUtils.copyFile(helpHtml, destDir+"/dataproc_manual.html");
		MiscUtils.copyFile(helpPdf, destDir+"/dataproc_manual.pdf");
	}

	public RESULT_PAGE getPageType() { return this.pageType; }
	public CTabItem getItem() { return this.item; }
	public CTabFolder getFolder() { return this.folder; }
	public Browser getBrowser() { return this.browser; }

	private void addBrowserListeners() {

		browser.addLocationListener(new UrlTaskHandler(this));

		browser.addOpenWindowListener(e -> {
			e.browser = new CTabBrowser(folder, null, pageType).addContent();
			//Log.error(" new window listener " + pageType + e.location);
		});

		browser.addVisibilityWindowListener(new VisibilityWindowListener() {
			@Override
			public void hide(WindowEvent e) {
				Browser browser = (Browser) e.widget;
				Shell shell = browser.getShell();
				shell.setVisible(false);
			}

			@Override
			public void show(WindowEvent e) {
				Browser browser = (Browser) e.widget;
				final Shell shell = browser.getShell();
				if (e.location != null)
					shell.setLocation(e.location);
				if (e.size != null) {
					Point size = e.size;
					shell.setSize(shell.computeSize(size.x, size.y));
				}
				shell.open();
			}

		});

		browser.addCloseWindowListener(e -> {
			Browser browser1 = (Browser) e.widget;
			Shell shell = browser1.getShell();
			shell.close();
		});

		// keyboard shortcuts for convenience
		if (pageType != RESULT_PAGE.URL && html != null) {
			browser.addKeyListener(new KeyListener() {

				@Override
				public void keyReleased(KeyEvent e) {
					if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'r')) {
						refreshPage();
					} else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'x')) {
						// processing gui
						new WdgDataProcDialog(cshell, 0, pageType).open();
					} else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'h')) {
						// open help page
						new CTabBrowser(folder, helpHtml, RESULT_PAGE.URL);
					} else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 's')) {
						// staff page
						new CTabBrowser(folder, "https://www.gmca.aps.anl.gov/staff/", RESULT_PAGE.URL);
					} else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'c')) {
						// chat page
						//new CTabBrowser(folder, "http://bl3ws7:3000", RESULT_PAGE.URL);
					} else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 't')) {
						// globus file transfer
						new CTabBrowser(folder, "https://www.globus.org/app/transfer", RESULT_PAGE.URL);
					} else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'u')) {
						// uap page
						new CTabBrowser(folder, "https://gmca-uap.aps.anl.gov/", RESULT_PAGE.URL);
					} else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'd')) {
						deleteResultsByUser();
					} else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'q')) {
						// testing
						// WdgScreeningTasks.exitBluice();
					}
				}

				@Override
				public void keyPressed(KeyEvent e) {}
			});
		}
	}

	public Browser addContent() {
		if (pageType != RESULT_PAGE.URL && html != null) {
			item.setFont(GUI.FONT_10_BOLD);
			browser.setText(html);
			item.setText(pageType.getPageTitle());
			item.setShowClose(false);
			item.setToolTipText(pageType.getPageTitle());
			addContextMenu();

			if (!usePV) {
				attachUpdater();
			} else {
				update_monitor();
				attachUpdater_pv();
			}
		} else if (pageType == RESULT_PAGE.URL && html != null) {
			item.setFont(GUI.FONT_10);
			item.setText(html);
			browser.setUrl(html);
			item.setShowClose(true);
		} else {
			item.setFont(GUI.FONT_10);
			item.setShowClose(true);
		}

		return browser;
	}

	private void attachUpdater() {
		// start a thread to update the content periodically
		Thread updateThread = new Thread() {
			public void run() {
				while (true) {
					synchronized (lockObj) {
						try {
							lockObj.wait(poolInterval); 		//Thread.sleep(poolInterval);
						} catch (InterruptedException e) {
							e.printStackTrace();
							return;
						}
					}
					
					if (item.isDisposed() || folder.isDisposed() || browser.isDisposed()) {
						return; 
					}

					final Result newStates;
					final String newHtml;

					final String title_final = pageType.getPageTitle();

					newStates = new DB2Html(pageType).getStates();
					if (!MiscUtils.isStateEqual(currentStates, newStates)) {
						newHtml = new DB2Html(pageType).html();
						html = newHtml;
						savePage(null);
						Display.getDefault().asyncExec(new Runnable() {
							@Override
							public void run() {
								if (autoRefresh) {
									currentStates = newStates;
									browser.setText(newHtml);
									item.setText(title_final);
									Log.note(title_final + " reloaded due to new content.");
								} else {
									item.setText(title_final);
								}
							}
						});
					} // end of outer if
					
				}
			}
		};
		// background thread
		updateThread.setDaemon(true);
		updateThread.start();
	}

	// update page based on PV monitor
	private void attachUpdater_pv() {
		// start a thread to update the content periodically
		Thread updateThread = new Thread() {
			public void run() {
				while (true) {
					synchronized (lockObj) {
						try {
							lockObj.wait(poolInterval); 		//Thread.sleep(poolInterval);
						} catch (InterruptedException e) {
							e.printStackTrace();
							return;
						}
					}

					if (item.isDisposed() || folder.isDisposed() || browser.isDisposed()) {
						return;
					}

					final String newHtml;

					final String title_final = pageType.getPageTitle();

					if (updateAvailable) {
						newHtml = new DB2Html(pageType).html();
						html = newHtml;
						savePage(null);
						Display.getDefault().asyncExec(new Runnable() {
							@Override
							public void run() {
								if (autoRefresh) {
									updateAvailable = false;
									browser.setText(newHtml);
									item.setText(title_final);
									Log.note(title_final + " reloaded due to new content.");
								} else {
									item.setText(title_final);
								}
							}
						});
					} // end of outer if

				}
			}
		};
		// background thread
		updateThread.setDaemon(true);
		updateThread.start();
	}


	// update page content with a pv monitor
	void update_monitor() {

		Epics.EpicsValueMonitor monitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				final String _pvName = pvName;
				final DBR _value = value;

				if (_pvName.equals(PVLookup.PV_ANALYSIS.pipelineStatus.toString())) {
					final String pipelineStatus = Epics.dbrToString(_value);

					// update the right page
					if ((pipelineStatus.contains("PROCESS") && pageType.equals(RESULT_PAGE.PROCESSING_RESULTS))
						|| (pipelineStatus.contains("STRATEGY") && pageType.equals(RESULT_PAGE.SCREENING_RESULTS)))
					{
						updateAvailable = true;
					}
				}
			}
		};
		Epics.addMonitor(PVLookup.PV_ANALYSIS.pipelineStatus, monitor);
	}

	private void closeTabs() {
		CTabItem[] ctitems = folder.getItems();
		for (int i = 2; i < ctitems.length; i++) {
			ctitems[i].dispose();
		}
	}
	
	// Add context menu
	private void addContextMenu() {
		Menu popupMenu = new Menu(browser);

		// section 1
		MenuItem refreshItem = new MenuItem(popupMenu, SWT.CASCADE);
		refreshItem.setText("Reload page");
		refreshItem.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				refreshPage();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});

		MenuItem saveItem = new MenuItem(popupMenu, SWT.NONE);
		saveItem.setText("Save a snapshot && Open in Firefox");
		saveItem.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				String primaryDir;
				try {
					primaryDir = DevDataDir.getPrimary();
				} catch (NullPointerException e) {
					primaryDir = System.getProperty("user.home");
				}
				
				FileDialog dialog = new FileDialog(cshell, SWT.SAVE);
				dialog.setFilterNames(new String[] { "html files", "All Files (*.*)" });
				dialog.setFilterExtensions(new String[] { "*.html", "*.*" });
				dialog.setFilterPath(primaryDir);
				dialog.setFileName(pageType.getPageTitle().replace(" ", "-") + ".html");
				String outfile = dialog.open();

				if (outfile == null) {
					return;
				}

				savePage(outfile);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		MenuItem closeItem = new MenuItem(popupMenu, SWT.NONE);
		closeItem.setText("Close all but summary tabs");
		closeItem.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				closeTabs();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});

		new MenuItem(popupMenu, SWT.SEPARATOR);

		// staff only table
		if (System.getProperty("user.name").equals("gmca")) {
			MenuItem cleanItemStaff = new MenuItem(popupMenu, SWT.CASCADE);
			cleanItemStaff.setText("Clear analysis tables -- all users");
			cleanItemStaff.addSelectionListener(new SelectionListener() {
				@Override
				public void widgetSelected(SelectionEvent event) {
					event.doit = false;

					new WdgProtectedRunnableDialog(popupMenu.getShell(), new Runnable() {
						@Override
						public void run() {
							clearProcResultsTables();
							refreshPage();
						}
					}).open();
				}

				@Override
				public void widgetDefaultSelected(SelectionEvent arg0) {
				}
			});
			cleanItemStaff.setEnabled(false);
		} else {
			MenuItem cleanItem = new MenuItem(popupMenu, SWT.CASCADE);
			cleanItem.setText("Delete all my processing results");
			cleanItem.addSelectionListener(new SelectionListener() {
				@Override
				public void widgetSelected(SelectionEvent event) {
					event.doit = false;
					deleteResultsByUser();
				}

				@Override
				public void widgetDefaultSelected(SelectionEvent arg0) {
				}
			});
			cleanItem.setEnabled(false);
		}

		browser.setMenu(popupMenu);
	}
	
	private void deleteResultsByUser() {
		boolean delete = false;
		String msg = "All the gathered processing statistics will be deleted."
				+ "\n\nIf you need this information, please save them to "
				+ "\n files with non-default names before deleting."
				+ "\n Processing files and collected images will not affected."
				+ "\n\nClick OK and then provide staff password to delete anyway.";
	
		MessageDialog dialog = new MessageDialog(cshell, "Warning", null, msg, MessageDialog.WARNING, new String[] { IDialogConstants.OK_LABEL, IDialogConstants.CANCEL_LABEL }, 0);
		dialog.setBlockOnOpen(true);
		int return_code = dialog.open(); 
		if (return_code == 0) 
			delete = true;

		if (delete) {
			new WdgProtectedRunnableDialog(cshell, new Runnable() {
				@Override
				public void run() {
					// clearProcResultsByUser();
					clearProcResultsByUserAndType();
					refreshPage();
				}
			}).open();
		}
	}

	private void startProgram(String program) { // default no logging
		startProgram(program, false);
	}
	
	private void startProgram(String program, boolean logOutput) {
		String wdir = getCurrentCollectDir();
		ExecTask task = ExecTask.new_ExecTask(program, program, logOutput);
		task.setWorkingDir(wdir);
		task.start();
	}
	
	private void startTerminal() {
		String wdir = getDirFromTitle();
		String cmd = String.format("%s --working-directory=%s", "mate-terminal", wdir);
		Log.note("started terminal: " +cmd);
		ExecTask.asyncExec(cmd, wdir, false);
	}
	
	// toolbar replicate the function of context menu
	private void createToolBar(Composite composite) {
		ToolBar toolbar = new ToolBar(composite, SWT.BORDER | SWT.HORIZONTAL);
		toolbar.setFont(GUI.FONT_10_BOLD);

		composite.setBackground(GUI.COLOR_BLUE_BUTTON);
		toolbar.setBackground(GUI.COLOR_BLUE_BUTTON);
		
		ToolItem forward = new ToolItem(toolbar, SWT.PUSH);
		forward.setText("Forward");
		
		if (!browser.isForwardEnabled()) {
			forward.setEnabled(false);
		}
		
		ToolItem back = new ToolItem(toolbar, SWT.PUSH);
		back.setText("Back");
		if (!browser.isBackEnabled()) {
			back.setEnabled(false);
		} else {
			back.setEnabled(true);
		}
		
		ToolItem reload = new ToolItem(toolbar, SWT.PUSH);
		reload.setText("Reload");
		reload.setToolTipText("Reload current page content.");
		
		ToolItem cleanTabs = new ToolItem(toolbar, SWT.PUSH);
		cleanTabs.setText("CloseTabs");
		cleanTabs.setToolTipText("Close all tabs except for the two left \"Summary of ... results\" tab.");
		
		ToolItem peelOut = new ToolItem(toolbar, SWT.PUSH);
		peelOut.setText("PeelOut");
		peelOut.setToolTipText("Launch a separate processing window outside bluice.");
		
		browser.addLocationListener(new LocationListener() {
			
			@Override
			public void changing(LocationEvent arg0) {}
			
			@Override
			public void changed(LocationEvent e) {
				Browser b = (Browser)e.widget;
				back.setEnabled(b.isBackEnabled());
				forward.setEnabled(b.isForwardEnabled());	
				if (b.getUrl().contains("about:blank")) {
					recreateOnReload = true;
				} else {
					recreateOnReload = false;
				}
			}
		});
		
		forward.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				browser.forward();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		back.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				browser.back();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		reload.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (recreateOnReload) {
					refreshPage();
				} else {
					browser.refresh();
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		cleanTabs.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				closeTabs();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		peelOut.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				boolean external = false;
				if (!external) {
					TabAnalysisNG.newDetached();
				} else { 
					// run jbluice-process as a stand alone so that it does not interfere with bluice operations.
					startProgram("jbluice-process");
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		new ToolItem(toolbar, SWT.SEPARATOR);

		ToolItem process = new ToolItem(toolbar, SWT.PUSH |SWT.CENTER);
		process.setToolTipText("Launch gui for data processing using supported pipelines.");
		if (pageType == RESULT_PAGE.SCREENING_RESULTS) {
			process.setText("Strategy");
		} else {
			process.setText("Process");
		}

		process.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				new WdgDataProcDialog(cshell, 0, pageType).open();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		new ToolItem(toolbar, SWT.SEPARATOR);
		
		ToolItem hkl = new ToolItem(toolbar, SWT.PUSH);
		hkl.setText("HKL3000");
		hkl.setToolTipText("Launch HKL in the current collect dir or $HOME");
		hkl.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				startProgram("/mnt/software/px/HKL/HKL3000");
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		ToolItem xds = new ToolItem(toolbar, SWT.PUSH);
		xds.setText("XDSGUI");
		xds.setToolTipText("Launch xdsgui in the current collect dir or $HOME");

		xds.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				startProgram("/mnt/software/px/XDS/xdsgui");
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		ToolItem dui = new ToolItem(toolbar, SWT.PUSH);
		dui.setText("DIALS");
		dui.setToolTipText("Launch DIALS in the current collect dir or $HOME");
		dui.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				startProgram("dui");
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		
		ToolItem mosflm = new ToolItem(toolbar, SWT.PUSH);
		mosflm.setText("iMOSFLM");
		mosflm.setToolTipText("Launch iMosflm in the current collect dir or $HOME");
		
		mosflm.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				startProgram("/mnt/software/px/CCP4/ccp4/bin/imosflm");
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		
		kamo = new ToolItem(toolbar, SWT.PUSH);
		kamo.setText("KAMO");
		kamo.setToolTipText(String.format("Launch KAMO to process data @ %s as you collect, \nassuming small wedge data.", getPrimaryDir()));
		kamo.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				String datasetList = getPrimaryDir() + "/datasets.txt";
				if (new File(datasetList).exists()) {
					KAMO kamo = new KAMO(toolbar.getShell());
					kamo.run();		
				} else {
					String msg = "no datasets found in " + datasetList + ". Please collect some data first.";
					Log.error(msg);
					WdgDataProcDialog.NBDisplayMessage(cshell, msg);
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		ToolItem terminal = new ToolItem(toolbar, SWT.PUSH);
		terminal.setText("Terminal");
		terminal.setToolTipText("Launch a terminal in the current collect dir or $HOME");

		terminal.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				startTerminal();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		new ToolItem(toolbar, SWT.SEPARATOR);

		settings = new ToolItem(toolbar, SWT.FLAT|SWT.ARROW|SWT.UP);
		settings.setText("Setting");
		settings.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
                Menu menu = new Menu(cshell, SWT.POP_UP);

                MenuItem item0 = new MenuItem(menu, SWT.PUSH);
                item0.setText("Disable background data processing");
                MenuItem item1 = new MenuItem(menu, SWT.PUSH);
                item1.setText("Enable background data processing (put all processing in a separate tree)");
                MenuItem item2 = new MenuItem(menu, SWT.PUSH);
                item2.setText("Enable background data processing (put processing data close to image data)");
                
                Point loc = toolbar.getLocation();
                Rectangle rect = toolbar.getBounds();

                Point mLoc = new Point(loc.x-1, loc.y+rect.height);
                menu.setLocation(cshell.getDisplay().map(toolbar.getParent(), null, mLoc));
                menu.setVisible(true);
				
                setSetting(item0, 0);
                setSetting(item1, 1);
                setSetting(item2, 2);
                
			}
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		if (!Client.isMaster()) {
			settings.setEnabled(false);
			kamo.setEnabled(false);
		}

		new ToolItem(toolbar, SWT.SEPARATOR);
		
		ToolItem help = new ToolItem(toolbar, SWT.PUSH);
		help.setText("Help");
		help.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				new CTabBrowser(folder, helpHtml, RESULT_PAGE.URL);
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		GridData data = new GridData(SWT.FILL, SWT.FILL, true,false);
		data.heightHint = 30;

		toolbar.setLayoutData(data);

	}
	
	public void setSetting(MenuItem item, int value) {
        item.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				DevUserConfig.get().setXDSProcessing(value);
				Log.note("Background processing flag is set as: "+value);
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}

	public static void deleteDataTableState() {
		String directory = System.getProperty("user.home") + "/.local/share/webkit/databases";
		File dir = new File(directory);
		if (!dir.exists()) {
			return;
		}
		for(File file: dir.listFiles()) {
		    if (!file.isDirectory()) {
		        file.delete();
		    }
		}
	}

	public String getDirFromTitle() {
		String url = browser.getUrl();
		url = url.replace("file://", "");

		// ignore in page anchors, assuming that # is not used as part of file/directory name
		if (url.contains("#")) {
			url = url.substring(0,  url.indexOf("#"));
		}
		
		if (url.startsWith("/")) {
			File url_f = new File(url);
			if (url_f.isFile()) {
				url = url.substring(0,  url.lastIndexOf("/"));
			} 
		} else {
			url = System.getProperty("user.home");
		}
		
		return url;
	}
	
	
	public static String getCurrentCollectDir() {
		String defaultDir = System.getProperty("user.home");
		
		String currCollectDir;
		
		try {
			currCollectDir = ModelCollectRunList.get().getCurrentRunModel().getFullDirectory();
		} catch (Exception e) {
			Log.warn("unable to get current collection directory, default to home.");
			currCollectDir = defaultDir;
		}
		
		// guard against weird value return by runList in offline mode
		if (!currCollectDir.startsWith("/")) {
			currCollectDir = defaultDir;
		}
		
		// check the directory is readable and writable
		if( !(new File(currCollectDir).canRead() && new File(currCollectDir).canWrite())) {
			currCollectDir = defaultDir;
		}

		return currCollectDir;
	}
	
	public static String getPrimaryDir() {
		try {
			String dataDir = DevDataDir.getPrimary();
			if (dataDir != null) {
				return dataDir;
			} else {
				return System.getProperty("user.home");
			}
		} catch (Exception e) {
			Log.note("Unable to obtain the primary directory.");
			return System.getProperty("user.home");
		}
	}
	
	// wipe data for the current user
	public static void clearProcResultsByUser() {
		String user = System.getProperty("user.name");
		// make sure delete on cascade is on for the associated tables, otherwise this would fails
		Database.exec(DBName.USERDATA, String.format("delete from pipelinestatus where username='%s';", user));
		Log.note(String.format("data for user %s in analysis tables were deleted.", user));
	}

	public void clearProcResultsByUserAndType() {
		String user = System.getProperty("user.name");
		// make sure delete on cascade is on for the associated tables, otherwise this would fails
		if (pageType == RESULT_PAGE.SCREENING_RESULTS) {
			Database.exec(DBName.USERDATA, String.format("delete from pipelinestatus where username='%s' and pipeline like '%%_strategy';", user));
		} else {
			Database.exec(DBName.USERDATA, String.format("delete from pipelinestatus where username='%s' and pipeline not like '%%_strategy';", user));
		}
		Log.note(String.format("data for user %s in analysis tables were deleted.", user));
	}
	
	// wipe everything, all user data
	public static void clearProcResultsTables() {
		Database.exec(DBName.USERDATA, "truncate screenstrategyresults;");
		Database.exec(DBName.USERDATA, "truncate dataprocessresults;");
		Database.exec(DBName.USERDATA, "delete from pipelinestatus;");
		Log.note("All analysis tables were reinitialized.");
	}
	
	// refresh page
	public void refreshPage() {
		html = new DB2Html(pageType).html();
		currentStates = new DB2Html(pageType).getStates();
		browser.setText(html);
		item.setText(pageType.getPageTitle());
		savePage(null);
	}

	// save page
	private void savePage(String outfile) {
		// only master can write to avoid concurrent writing of the same file
		if (outfile == null && !Client.isMaster()) {
			return;
		} 
		
		boolean openFirefox = false;
		if (outfile!=null) {
			openFirefox = true;
		}

		String htmlWithRelPath = html;
		if (outfile == null) {
			String dir = System.getProperty("user.home");
			try {
				dir = DevDataDir.getPrimary();
				// convert absolute path in html to relative ones
				htmlWithRelPath = html.replaceAll("file:\\/{2,4}"+dir.substring(1),".");
			} catch (Exception e) {
				Log.warn("unable to get the collection directory, default to home.");
			}
			outfile = dir + "/" + pageType.getPageTitle().replace(" ", "-") + ".html";
		} else {
			// TODO this code is not foolproof, infer primary directory when user manually saves file
			String odir = outfile.substring(0,outfile.lastIndexOf("/"));
			odir = MiscUtils.rtrim(odir, "/");
			Pattern primaryPattern = Pattern.compile("23(ID[BD]|BM)_\\d{4}_\\d{2}_\\d{2}"
					+ "(_hdf|_cbf|_mccd)?"
					+ "$");
			if (primaryPattern.matcher(odir).find()) {
				htmlWithRelPath = html.replaceAll("file:\\/{2,4}"+odir.substring(1),".");
			}
		}

		try (Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outfile), "utf-8"))) {
			writer.write(htmlWithRelPath);
			Log.note("File: " + outfile + " saved.");
			if (openFirefox) {
				startProgram(String.format("firefox %s", outfile));
			}

		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	// given in seconds
	public void setPoolInterval(Integer interval) {
		if (interval <= 0) { // turn it off
			interval = Integer.MAX_VALUE;
		} else if (interval <= 24 * 3600) {
			interval *= 1000;
		} else {
			interval = 24 * 3600;
		}
		synchronized (lockObj) {
			this.poolInterval = interval;
			//Log.note("set refresh rate for " + pageType + " to " + this.poolInterval/1000 + " sec");
			lockObj.notify();
		}
	}

	public void setAutoRefresh(boolean autoRefesh) {
		this.autoRefresh = autoRefesh;
	}

	@Override
	public void masterChanged(boolean isMaster) {
		Log.note("Master changed "+ isMaster);
		GUI.asyncExec(new Runnable() {
			@Override
			public void run() {
				kamo.setEnabled(isMaster);
				settings.setEnabled(isMaster);
			}
		});
		
	}

}
