package gov.anl.gmca.analysis;

import gov.anl.gmca.analysis.DB2Html.RESULT_PAGE;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.WdgMain;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.syslog.Log;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabFolder2Listener;
import org.eclipse.swt.custom.CTabFolderEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class TabAnalysisNG extends Tab {
	
	public static int lastParentIndex = 0;
	CTabBrowser processingResults, screenResults;
	private static Integer longPoolInterval=3600;
	private static Integer shortPoolInterval=2;
	private static Integer peelOutID=0;
	private CTabFolder folderNG;
	
	private static void changeTabHighlight(CTabFolder folder) {
		folder.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				folder.setSelectionBackground(GUI.COLOR_BLUE_BUTTON);
				folder.setSelectionForeground(GUI.COLOR_RED);
				int selectionIndex =  folder.getSelectionIndex();
				if (selectionIndex == 0 || selectionIndex == 1 ) {
					lastParentIndex = selectionIndex;
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		folder.addCTabFolder2Listener(new CTabFolder2Listener() {
			
			@Override
			public void showList(CTabFolderEvent arg0) {}
			
			@Override
			public void restore(CTabFolderEvent arg0) {}
			
			@Override
			public void minimize(CTabFolderEvent arg0) {}
			
			@Override
			public void maximize(CTabFolderEvent arg0) {}
			
			@Override
			public void close(CTabFolderEvent e) {
				folder.setSelection(lastParentIndex);
			}
		});
	}

	private static TabAnalysisNG inst = null;
	public static TabAnalysisNG get() { return inst; }

	public TabAnalysisNG(Composite parent, int style) {
		super(parent, style);
		inst=this;
		GridLayout gd = new GridLayout(1, false);
		gd.marginHeight = 0;
		gd.marginWidth = 0;
		setLayout(gd);

		folderNG = new CTabFolder(this, SWT.NONE);
		folderNG.setFont(GUI.FONT_SAMPLE_TABS);
		folderNG.setLayoutData(new GridData(GridData.FILL_BOTH));

		changeTabHighlight(folderNG);

		createPages(folderNG);
		
	}

	public static void setVisibleTab(int tabIndex) {
		TabAnalysisNG ta = TabAnalysisNG.get();
		if (ta != null) {
			GUI.asyncExec(new Runnable(){public void run()
			{
				WdgMain.setVisibleTab(WdgMain.TAB.AnalysisNG);
				ta.folderNG.setSelection(tabIndex);
			}});
		} else {
			Log.error("TabAnalysisNG does not exist.");
		}
	}

	public void save() { // force update and save
		screenResults.refreshPage();
		processingResults.refreshPage();
	}

	public static void newDetached(int tab_index) {
		peelOutID += 1;
		GUI.asyncExec(new Runnable() {

			@Override
			public void run() {
				Display display = GUI.getDisplay();
				Shell shell = new Shell(display);
				shell.setText("Analysis -- peelOut window " + peelOutID);
				shell.setLayout(new FillLayout());

				CTabFolder tabFolder = new CTabFolder(shell, SWT.BORDER);
				tabFolder.setFont(GUI.FONT_SAMPLE_TABS);

				changeTabHighlight(tabFolder);

				new CTabBrowser(tabFolder, new DB2Html(RESULT_PAGE.PROCESSING_RESULTS).html(),
						RESULT_PAGE.PROCESSING_RESULTS).setPoolInterval(3);
				new CTabBrowser(tabFolder, new DB2Html(RESULT_PAGE.SCREENING_RESULTS).html(),
						RESULT_PAGE.SCREENING_RESULTS).setPoolInterval(2);
				tabFolder.setSelection(tab_index);

				shell.open();
			}
		});
	}

	public static void newDetached() {
		newDetached(0);
	}

	@Override
	public String getTabName() {
		return "Analysis";
	}

	private void createPages(CTabFolder tabFolder) {
		// create browser in a tab folder, each item holds a browser
		try {
			processingResults = new CTabBrowser(tabFolder, new DB2Html(RESULT_PAGE.PROCESSING_RESULTS).html(),
					RESULT_PAGE.PROCESSING_RESULTS);
			screenResults = new CTabBrowser(tabFolder, new DB2Html(RESULT_PAGE.SCREENING_RESULTS).html(),
					RESULT_PAGE.SCREENING_RESULTS);

			screenResults.setPoolInterval(longPoolInterval);
			processingResults.setPoolInterval(longPoolInterval);
			tabFolder.setSelection(0);

		} catch (NullPointerException e) {
			Log.note("unable to start AnalysisNG Tab, skipped.");
		}
	}

	// main function for running stand alone
	public static void main(String[] args) {
		Epics.connect();

		Display display = GUI.getDisplay(); // new Display();
		Shell shell = new Shell(display, SWT.SHELL_TRIM);
		shell.setText("jBluIce Data Processing -- main window");
		
		GridLayout shellLayout = new GridLayout();
		shellLayout.marginHeight=0;
		shellLayout.marginWidth=0;
		shellLayout.horizontalSpacing=0;
		shellLayout.verticalSpacing=0;
		shell.setLayout(shellLayout);

		shell.addListener(SWT.Close, new Listener() {
			
			@Override
			public void handleEvent(Event event) {
				if (MessageDialog.openConfirm(shell, "Please confirm", "Are you sure to exit?\n\nJobs already started will continue to run in background.")) {
					event.doit = true;
				} else { 
					event.doit = false;
				}
			}
		});

		CTabFolder tabFolder = new CTabFolder(shell, SWT.BORDER);
		tabFolder.setFont(GUI.FONT_SAMPLE_TABS);

		GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		gd.verticalAlignment = GridData.FILL;
		gd.horizontalAlignment = GridData.FILL;
		tabFolder.setLayoutData(gd);

		/*
		shell.addListener(SWT.Resize, new Listener() {
			@Override
			public void handleEvent(Event arg0) {
				System.out.println("Shell size= " + shell.getBounds() + ", tabFolder Size= " + tabFolder.getBounds());
				System.out.println("grid data= " + tabFolder.getLayoutData());
			}
		});
		*/
	
		new CTabBrowser(tabFolder, 
				new DB2Html(RESULT_PAGE.PROCESSING_RESULTS).html(), 
				RESULT_PAGE.PROCESSING_RESULTS).setPoolInterval(3);
		new CTabBrowser(tabFolder, 
				new DB2Html(RESULT_PAGE.SCREENING_RESULTS).html(), 
				RESULT_PAGE.SCREENING_RESULTS).setPoolInterval(2);

		changeTabHighlight(tabFolder);
		tabFolder.setSelection(0);

		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
		//Epics.disconnectMonitors();
		System.exit(0);
	}
	
	@Override
	public void notifyIsVisible(boolean isVisible) {
		if (isVisible) {
			screenResults.setPoolInterval(shortPoolInterval);
			processingResults.setPoolInterval(shortPoolInterval);		
		} else {
			screenResults.setPoolInterval(longPoolInterval);
			processingResults.setPoolInterval(longPoolInterval);		
		}
	}
}
