package gov.anl.gmca.analysis;

import gov.anl.gmca.analysis.DB2Html.RESULT_PAGE;
import gov.anl.gmca.analysis.helpers.FindImagePrefixes;
import gov.anl.gmca.analysis.helpers.MiscUtils;
import gov.anl.gmca.analysis.helpers.Spacegroup;
import gov.anl.gmca.analysis.helpers.ValidateUserInput;
import gov.anl.gmca.common.ui.NBDirectoryDialog;
import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.io.hw.Database;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

public class WdgDataProcDialog extends Dialog implements Runnable {
	private Composite container;
	private int pipelinestatus_id;
	private RESULT_PAGE type;
	private String sampleName = "";
	private String sampleNumber = "";
	private String ImageDirectory = "";
	private String start = "";
	private String end = "";
	private String filenamePrefix = "";
	private String pipeline = "gmcaproc";
	private String OutputDirectory = "";
	private String spacegroup = "";
	private String unitcell = "";
	private String highResolutionCutoff = "2.0";
	private String refhkl = "";
	private boolean anomalous = true;
	private boolean smallWedges = false;
	private String njobs = "4";
	private String nproc = "8";
	private String runner = autoProcessData.RUNNER;
	private boolean recursive = false;
	private String XDScmds = "";
	private String spacegroup_choices = "";
	private String imagePrefixHints = "";
	private String exclude_dirs="";
	private String include_dirs="";
	
	private Text txtExcludeDirs;
	private Text txtIncludeDirs;
	
	Composite cDirFilter;
	GridData gdDirFilter;
	
	Group gImageFilter;
	GridData gdImageFilter;
	
	Group gProcessParams;
	GridData gdProcessParams;
	
	Composite compRef;
	
	private Text txtImageDirectory;
	private Text txtstart;
	private Combo comboEnd;
	private Combo comboFilenamePrefix;
	private Combo combopipeline;
	private Text txtOutputDirectory;
	private Combo comboSpacegroup;
	private Text txtunitcell;
	private Combo combohighResolutionCutoff;
	private Text txtrefhkl;
	private Button btnrefhkl;
	private Text txtSampleName;
	private Button toggleanomalous;
	private Button toggleSmallWedges;
	private Button toggleRecursive;
	private Combo comboXDScmds;
	private Shell parentShell;
	
	protected WdgDataProcDialog(Shell parentShell, int id, RESULT_PAGE type) {
		super(parentShell);
		this.parentShell = parentShell;
		setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(false);
		this.pipelinestatus_id = id;
		this.type = type; // track which page invoked this since the query is different

		if (this.type == RESULT_PAGE.SCREENING_RESULTS) {
			getValuesFromScreenDB();
		} else {
			getValuesFromProcessDB();
		}

		// in case of failure, default to current collection dir, or home dir
		if (ImageDirectory.equals("")) {
			ImageDirectory = CTabBrowser.getCurrentCollectDir();
		}

		// in case of multiple directories saved by quickProcess
		if (ImageDirectory.contains(":")) {
			ImageDirectory = ImageDirectory.substring(0, ImageDirectory.indexOf(":"));
		}
		 
		imagePrefixHints = new FindImagePrefixes(ImageDirectory).getPrefixHints();

	}

	private String trim(String fld) {
		if (fld == null || fld.equals("NA") || fld.equals("None") || fld.isEmpty()) {
			fld = "";
		}
		fld = fld.trim();
		return fld;
	}

	private void getValuesFromScreenDB() {
		String query = "SELECT p.sampleName, p.sampleNumber, p.imagedir, "
				+ "d.workdir, d.spacegroup, d.unitcell, d.pointgroup_choices "
				+ "FROM pipelinestatus p left join screenstrategyresults d " + "on p.id=d.pipelinestatus_id where p.id="
				+ pipelinestatus_id + ";";

		Result result = Database.exec(DBName.USERDATA, query);

		if (result.isOK() && result.numRows() > 0) {
			HashMap<String, String> row0 = result.getRow(0);
			sampleName = trim(row0.get("sampleName"));
			sampleNumber = trim(row0.get("sampleNumber"));
			ImageDirectory = trim(row0.get("imagedir"));
			OutputDirectory = trim(row0.get("workdir"));
			spacegroup = trim(row0.get("spacegroup"));
			unitcell = trim(row0.get("unitcell"));
			spacegroup_choices = trim(row0.get("pointgroup_choices"));
		} else {
			Log.note("no result found in screening results table " + result.error);
		}
	}

	private void getValuesFromProcessDB() {

		String query = "SELECT p.sampleName, p.sampleNumber, " 
				+ "d.imagedir, d.start, d.end, d.prefix, d.pipeline, "
				+ "d.workdir, d.spacegroup, d.unitcell, d.highresolution, d.spacegroup_choices "
				+ "FROM pipelinestatus p left join dataprocessresults d " + "on p.id=d.pipelinestatus_id where p.id="
				+ pipelinestatus_id + ";";

		Result result = Database.exec(DBName.USERDATA, query);

		if (result.isOK() && result.numRows() > 0) {
			HashMap<String, String> row = result.getRow(0);
			sampleName = trim(row.get("sampleName"));
			sampleNumber = trim(row.get("sampleNumber"));
			ImageDirectory = trim(row.get("imagedir"));
			start = trim(row.get("start"));
			end = trim(row.get("end"));
			filenamePrefix = trim(row.get("prefix"));
			pipeline = trim(row.get("pipeline"));
			OutputDirectory = trim(row.get("workdir"));
			spacegroup = trim(row.get("spacegroup")).replaceAll(" ", "");
			unitcell = trim(row.get("unitcell"));
			highResolutionCutoff = trim(row.get("highresolution"));
			spacegroup_choices = trim(row.get("spacegroup_choices"));
		} else {
			Log.note("no result found in processing results table " + result.error);
		}
	}

	protected Control createDialogArea(Composite parent) {
		this.container = (Composite) super.createDialogArea(parent);

		GridLayout layout = new GridLayout(4, true);
		layout.verticalSpacing = 10;
		layout.marginHeight = 10;
		layout.marginWidth = 10;
		layout.makeColumnsEqualWidth = false;

		container.setLayout(layout);

		createContent();

		return container;
	}

	public void createContent() {
		//banner();
		imageDirectoryDialog();
		outputDirectoryDialog();
		imageFilter(); 
		processParameters();
	}

	public void banner() {
		Label banner = new Label(container, SWT.CENTER);
		GridData griddata = new GridData(GridData.FILL_HORIZONTAL);
		griddata.verticalSpan = 3;
		griddata.horizontalSpan = 4;
		banner.setLayoutData(griddata);
		banner.setFont(new Font(container.getDisplay(), "Arial", 11, SWT.BOLD));
		banner.setText(
				"Only the image directory parameter is mandatory.\n " + "Other fields are optional (can be empty). \n"
						+ "If unit cell is given, you must also specify space group as well.");
	}

	public void imageDirectoryDialog() {
		Group compImageDir = new Group(container, SWT.BORDER);
		compImageDir.setBackground(GUI.COLOR_SAMPLE_BG);
		compImageDir.setText("image directory (required)");
		compImageDir.setLayout(new GridLayout(5, false));
		GridData gdImageDir = new GridData(GridData.FILL_HORIZONTAL);
		gdImageDir.horizontalSpan = 4;
		compImageDir.setLayoutData(gdImageDir);
		
		//new Label(compImageDir, SWT.NONE).setText("Image Directory (*):");

		txtImageDirectory = new Text(compImageDir, SWT.SINGLE | SWT.BORDER);
		GridData griddata = new GridData(GridData.FILL_HORIZONTAL);
		griddata.horizontalSpan = 2;
		txtImageDirectory.setLayoutData(griddata);
		txtImageDirectory.setText(ImageDirectory);
		txtImageDirectory.setToolTipText(new FindImagePrefixes(ImageDirectory).getImageListHints());
		
		Button button = new Button(compImageDir, SWT.PUSH);
		button.setText("Choose");

		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				// nonblocking ui
				String dir = null;
				try {
					NBDirectoryDialog dlg = new NBDirectoryDialog(container.getShell(), SWT.NONE);
					dlg.setFilterPath(txtImageDirectory.getText());
					dlg.setText("Choose image directory");
					dlg.setMessage("Select a directory containing diffraction image");
					dir = dlg.openStr();
				} catch (Exception e) {
					DirectoryDialog dlg = new DirectoryDialog(container.getShell(), SWT.NONE);
					dlg.setFilterPath(txtImageDirectory.getText());
					dlg.setText("Choose image directory");
					dlg.setMessage("Select a directory containing diffraction image");
					dir = dlg.open();
				}
				if (dir != null) {
					txtImageDirectory.setText(dir);
					String subdir = String.format("%s/%s", MiscUtils.analysisNGSubDirectory, "repo");
					String outdir = MiscUtils.getProcessDir(dir, subdir).replaceAll("//", "/");
					txtOutputDirectory.setText(outdir);
					if (!Disk.isDirectory(outdir)) {
						txtOutputDirectory.setForeground(GUI.COLOR_RED);
					} else {
						txtOutputDirectory.setForeground(GUI.COLOR_BLACK);
					}
				}
				
				System.out.println("recursive=" + recursive + " " + toggleRecursive.getSelection());
				container.getShell().pack();
				
				FindImagePrefixes fp = new FindImagePrefixes(dir);
				imagePrefixHints = fp.getPrefixHints();

				comboFilenamePrefix.setToolTipText(imagePrefixHints);
				comboFilenamePrefix.setItems(new String[] {"", imagePrefixHints});
				txtImageDirectory.setToolTipText(fp.getImageListHints());
				txtSampleName.setText(guessSampleName(txtImageDirectory.getText()) + "_rp");
			}
		});
		
		toggleRecursive = new Button(compImageDir, SWT.CHECK);
		toggleRecursive.setText("Recursive");
		toggleRecursive.setSelection(recursive);

		toggleRecursive.setToolTipText("Parsing image directory RECURSIVELY to look for data to process, USE WITH CAUTION.");

		toggleRecursive.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				recursive = !recursive;
				System.out.println("recursive=" + recursive + " " + toggleRecursive.getSelection());
				gdDirFilter.exclude = !recursive;
				cDirFilter.setVisible(recursive);
				container.getShell().pack();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});

		if (this.type == RESULT_PAGE.SCREENING_RESULTS) {
			toggleRecursive.setEnabled(false);
		}
	}
	
	// choose output directory
	public void outputDirectoryDialog() {
		Group gOutput = new Group(container, SWT.BORDER);
		gOutput.setText("Output");
		gOutput.setLayout(new GridLayout(4, false));
		GridData gdOutput = new GridData(GridData.FILL_HORIZONTAL);
		gdOutput.horizontalSpan = 4;
		gOutput.setLayoutData(gdOutput);	

		// output dir
		new Label(gOutput, SWT.NONE).setText("OutDir:");

		txtOutputDirectory = new Text(gOutput, SWT.BORDER);
		GridData griddata = new GridData(GridData.FILL_HORIZONTAL);
		griddata.horizontalSpan = 2;
		txtOutputDirectory.setLayoutData(griddata);
		txtOutputDirectory.setToolTipText("This parameter sets the root directory for the processing job."
										+ " The processing program will create it if it does not exist.");

		String subdir = String.format("%s/%s", MiscUtils.analysisNGSubDirectory, "repo");
		String defaultDirectory = MiscUtils.getProcessDir(ImageDirectory, subdir).replaceAll("//", "/");

		txtOutputDirectory.setText(defaultDirectory);

		// text is red if the directory doesn't exist
		if (!Disk.isDirectory(defaultDirectory)) {
			txtOutputDirectory.setForeground(GUI.COLOR_RED);			
		} else {
			txtOutputDirectory.setForeground(GUI.COLOR_BLACK);			
		}
		
		txtOutputDirectory.addModifyListener(new ModifyListener() {
			
			@Override
			public void modifyText(ModifyEvent arg0) {
				String currentText = txtOutputDirectory.getText();

				//Log.note(currentText + " exist? " + Disk.isDirectory(currentText));
				Color fgColor = GUI.COLOR_BLACK;
				try
				{
					if (!Disk.isDirectory(currentText)) {
						fgColor = GUI.COLOR_RED;
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
				txtOutputDirectory.setForeground(fgColor);				
			}
		});

		Button button = new Button(gOutput, SWT.PUSH);
		button.setText("Choose");

		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				String dir = null;
				try {
					NBDirectoryDialog dlg = new NBDirectoryDialog(container.getShell(), SWT.NONE);

					dlg.setFilterPath(txtOutputDirectory.getText());
					dlg.setText("Choose output directory");

					dlg.setMessage("Select a directory to write processing files...");
					dir = dlg.openStr();
				} catch (Exception e) {
					// standalone mode since above code crashes, TODO ugly, need to figure out why
					// above code crashes
					DirectoryDialog dlg = new DirectoryDialog(container.getShell(), SWT.NONE);

					dlg.setFilterPath(txtOutputDirectory.getText());
					dlg.setText("Choose output directory");

					dlg.setMessage("Select a directory to write processing files...");
					dir = dlg.open();
				}

				if (dir != null) {
					txtOutputDirectory.setText(dir);
				}

			}
		});

		// samplename
		new Label(gOutput, SWT.NONE).setText("Sample Name:");
		txtSampleName = new Text(gOutput, SWT.BORDER);
		GridData griddata2 = new GridData(GridData.FILL_HORIZONTAL);
		griddata2.horizontalSpan = 2;
		txtSampleName.setLayoutData(griddata2);
		txtSampleName.setText(this.sampleName + "_rp");
		txtSampleName.setToolTipText(
				"(Optional String) This is the string displayed in the samplename column, specify a name so that you can identify your job later.");

	
	}

	private String[] getEndChoices() {
		if (type == RESULT_PAGE.PROCESSING_RESULTS) { 
			return new String[] {"", "500", "50%", "75%", "100%"};
		} else {
			return new String[] {""};
		}
	}
	
	// show prefix, start, end
	public void imageFilter() {
		gImageFilter = new Group(container, SWT.BORDER);
		gImageFilter.setText("image filter");
		gImageFilter.setLayout(new GridLayout(2, false));
		gdImageFilter = new GridData(GridData.FILL_HORIZONTAL);
		gdImageFilter.horizontalSpan = 4;
		gdImageFilter.grabExcessHorizontalSpace = true;
		gImageFilter.setLayoutData(gdImageFilter);	

		// directory filtering for recursive mode
		dirsFilter();

		GridData griddata = new GridData(GridData.FILL_HORIZONTAL);
		griddata.horizontalSpan = 1;
		new Label(gImageFilter, SWT.NONE).setText("Start image no:");
		txtstart = new Text(gImageFilter, SWT.BORDER);
		txtstart.setLayoutData(griddata);
		txtstart.setText("");
		txtstart.setToolTipText(
				"(Optional) By default, the data processing will process all images in the specified image directory. User can choose a subset of images to process by providing the start, end image numbers, and filename prefix.\n\nPrevious run used: "
						+ start);

		Label e = new Label(gImageFilter, SWT.NONE);
		e.setText("End image no:");
		//e.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, true));
		
		comboEnd = new Combo(gImageFilter, SWT.BORDER);
		comboEnd.setLayoutData(griddata);
		comboEnd.setItems(getEndChoices());
		comboEnd.setToolTipText(
				"(Optional) By default, the data processing will process all images collected in the specified image directory. User can choose a set of of images to processing by providing the start, end number, and filename prefix.\n\nPrevious run used: "
						+ end);

		new Label(gImageFilter, SWT.FILL).setText("Filename prefix:");
		GridData griddata3 = new GridData(GridData.FILL_HORIZONTAL);
		griddata3.widthHint = 1;
		comboFilenamePrefix = new Combo(gImageFilter, SWT.BORDER);
		comboFilenamePrefix.setLayoutData(griddata3);
		comboFilenamePrefix.setItems(new String[] {"", imagePrefixHints});
		comboFilenamePrefix.setToolTipText(imagePrefixHints + "\n\nPrevious run used: " + filenamePrefix);
	}

	//filter subdirectory
	public void dirsFilter() {
		cDirFilter= new Composite(gImageFilter, SWT.NONE);
		cDirFilter.setLayout(new GridLayout(2, false));
		gdDirFilter = new GridData(GridData.FILL_HORIZONTAL);
		gdDirFilter.horizontalSpan = 2;
		cDirFilter.setLayoutData(gdDirFilter);	

		Label s = new Label(cDirFilter, SWT.NONE);
		s.setText(" - Exclude dirs:");
		txtExcludeDirs = new Text(cDirFilter, SWT.NONE);
		txtExcludeDirs.setText("process;raster;screen");
		txtExcludeDirs.setToolTipText("(Optional) set subdirectories to exclude");

		Label e = new Label(cDirFilter, SWT.NONE);
		e.setText(" - Include dirs:");
		
		txtIncludeDirs = new Text(cDirFilter, SWT.NONE);
		txtIncludeDirs.setText("");
		txtIncludeDirs.setToolTipText("(Optional) set subdirectories to include");

		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		txtIncludeDirs.setLayoutData(gd);
		txtExcludeDirs.setLayoutData(gd);

		gdDirFilter.exclude = true;
		cDirFilter.setVisible(recursive);
	}

	private String[] getPiplineSelections() {
		String[] pipelines = { 
				"gmcaproc", "gmcaproc w optimization", "autoproc", 
				"xia2_dials", "xia2_xds", "xia2_dials_aimless",
				"kamo_xds", "kamo_dials",
				"fastdp",
				"generate XDS.INP only", };

		// disable autoPROC if disabled in uap
		if (!Util.isUserNonProprietary()) {
			Log.warn("autoPROC disabled since " + Util.getUserLogin() + " is designated as proprietary.");
			pipelines = new String[] { 
					"gmcaproc", "gmcaproc w optimization", 
					"xia2_dials", "xia2_xds", "xia2_dials_aimless",
					"kamo_xds", "kamo_dials",
					"fastdp", 
					"generate XDS.INP only", };
		}
		
		if (this.type == RESULT_PAGE.SCREENING_RESULTS) {
			pipelines = new String[] {"xds_strategy", "mosflm_strategy", "labelit_strategy", "dials_strategy"};
			this.njobs = "1";
			this.nproc = "2";
			this.runner = "local";
		}
		
		return pipelines;
	}
	
	private String[] getHresPresets(String pipeline) {
		if (type == RESULT_PAGE.SCREENING_RESULTS) {
			return new String[] { "", "3.8", "2.0" };
		}
		
		if (pipeline.startsWith("gmca")) {
			return new String[] { "", String.valueOf(highResolutionCutoff), "default: cchalf~30%", "preset: <i/sigmai>~1.5", "preset: <i/sigmai>~2.0" };
		} else {
			return new String[] { "", String.valueOf(highResolutionCutoff)};
		}
	}
	
	private String getSpaceGroup(String str) {
		str = trim(str);

		String[] flds = str.split(" ");
		return trim(flds[0]).toLowerCase();
	}

	private String[] getSpacegroupChoices() {
		String[] spg_choices;
		if (spacegroup_choices.contains(";")) {
			spg_choices = spacegroup_choices.split(";");
		} else {
			spg_choices = Spacegroup.SOHNCKE_SGS;
		}

		// prepend a "" at the front
		String[] prepended_spg_choices = new ArrayList<String>() {
			{
				add("");
				addAll(Arrays.asList(spg_choices));
			}
		}.toArray(new String[0]);

		return prepended_spg_choices;
	}

	public void processParameters() {
		gProcessParams = new Group(container, SWT.BORDER);
		gProcessParams.setText("processing params");
		gProcessParams.setLayout(new GridLayout(2, false));
		gdProcessParams = new GridData(GridData.FILL_HORIZONTAL);
		gdProcessParams.horizontalSpan = 4;
		gdProcessParams.grabExcessHorizontalSpace = true;
		gProcessParams.setLayoutData(gdProcessParams);	
		
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		new Label(gProcessParams, SWT.NONE).setText("Pipeline:");
		
		combopipeline = new Combo(gProcessParams, SWT.BORDER | SWT.READ_ONLY);
		combopipeline.setLayoutData(data);

		combopipeline.setItems(getPiplineSelections());
		combopipeline.select(0); // default
		combopipeline.setToolTipText("Select processing pipeline for data processing or indexing, gmcaproc or mosflm is chosen by default.");
		
		combopipeline.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				String p = combopipeline.getText();
				combohighResolutionCutoff.setItems(getHresPresets(p));
				// extra command only useful for gmcaproc or generate XDS.INP
				if (p.startsWith("gmca") || p.startsWith("generate") || p.startsWith("xds_strategy")) {
					comboXDScmds.setEnabled(true);
					txtrefhkl.setEnabled(true);
					btnrefhkl.setEnabled(true);
				} else {
					comboXDScmds.select(0);
					comboXDScmds.setEnabled(false);
					txtrefhkl.setEnabled(false);
					btnrefhkl.setEnabled(false);
				}
				
				if (p.startsWith("kamo")) {
					toggleSmallWedges.setEnabled(true);
				} else {
					toggleSmallWedges.setEnabled(false);
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		// anomalous 
		Composite g2 = new Composite(gProcessParams, SWT.NONE);
		g2.setLayout(new GridLayout(2, false));
		GridData gd2 = new GridData(GridData.FILL_HORIZONTAL);
		gd2.horizontalSpan = 4;
		gd2.grabExcessHorizontalSpace = true;
		g2.setLayoutData(gd2);	
		
		
		GridData gd1 = new GridData(GridData.FILL_HORIZONTAL);
		gd1.horizontalSpan = 1;
		toggleanomalous = new Button(g2, SWT.CHECK);
		toggleanomalous.setText("Native Data");
		toggleanomalous.setToolTipText("By default, the data will be processed assuming I+ != I-. Select if you have native data.");
		toggleanomalous.setLayoutData(gd1);
		
		toggleanomalous.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				anomalous = !anomalous;
				System.out.println("anomalous =  "  + anomalous + " selection=" +  !toggleanomalous.getSelection());
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});
		
		// small wedges 
		//GridData gd2 = new GridData(GridData.FILL_HORIZONTAL);
		//gd1.horizontalSpan = 1;
		toggleSmallWedges = new Button(g2, SWT.CHECK);
		toggleSmallWedges.setText("Small wedges");
		toggleSmallWedges.setToolTipText("Select small wedges if you collect data in many crystals of small wedges, e.g. 10 degrees");
		toggleSmallWedges.setLayoutData(gd1);
		toggleSmallWedges.setEnabled(false);
		
		toggleSmallWedges.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				smallWedges = !smallWedges;
				System.out.println("smallWedges =  "  + smallWedges + " selection=" +  !toggleSmallWedges.getSelection());
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
		});		
		
		
		// high resolution cutoff
		Label hr = new Label(gProcessParams, SWT.NONE);
		hr.setText("High res:");
		//hr.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, true));
		
		combohighResolutionCutoff = new Combo(gProcessParams, SWT.BORDER);
		combohighResolutionCutoff.setItems(getHresPresets(pipeline));
		combohighResolutionCutoff.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (combohighResolutionCutoff.getText().contains("preset: ")
					|| combohighResolutionCutoff.getText().contains("default: ") ) {
					combohighResolutionCutoff.setText(combohighResolutionCutoff.getText().replace("default: ", "").replace("preset: ", ""));
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		combohighResolutionCutoff.select(0); // default
		combohighResolutionCutoff.setToolTipText("(Optional) By default, gmcaproc (and autoPROC) will choose resolution cutoff at CC1/2 ~ 30% "
				+ "(as suggested in aimless). You may set your own high resolution cutoff, or *choose* other presets (modified presets will be ignored).");
		
		// symmetry
		new Label(gProcessParams, SWT.NONE).setText("Symm:");
		comboSpacegroup = new Combo(gProcessParams, SWT.BORDER);
		comboSpacegroup.setItems(getSpacegroupChoices());
		comboSpacegroup.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				String s = getSpaceGroup(comboSpacegroup.getText());
				comboSpacegroup.setText(s); // display only the selected space group
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		comboSpacegroup.addListener(SWT.DefaultSelection, new Listener() {
			@Override
			public void handleEvent(Event arg0) {
				String s = getSpaceGroup(comboSpacegroup.getText());
				if (s != null && !s.isEmpty() && !ValidateUserInput.isValidSpaceGroup(s)) {
					String msg = "Invalid space group.";
					NBDisplayMessage(container.getShell(), msg);
				}	
			}
		});
		
		comboSpacegroup.setToolTipText(
				"(Optional) Space group symbol, hit return key to check your input is valid. \n\nPrevious run result: "
						+ spacegroup);


		// unit cell
		new Label(gProcessParams, SWT.NONE).setText("Unit cell:");
		txtunitcell = new Text(gProcessParams, SWT.BORDER);
		GridData griddatau = new GridData(GridData.FILL_HORIZONTAL);
		griddatau.horizontalSpan = 1;
		txtunitcell.setLayoutData(griddatau);
		txtunitcell.setText("");
		txtunitcell.addListener(SWT.DefaultSelection, new Listener() {
			public void handleEvent(Event e) {
				String u = txtunitcell.getText();
				if (u != null && !u.isEmpty() && !ValidateUserInput.isValidUnitCell(u)) {
					String msg = "Unit cell is defined by 6 numbers separated by space or comma";
					NBDisplayMessage(container.getShell(), msg);
				}
			}
		});
		txtunitcell.setToolTipText(
				"(Optional) define 6 cell parameters separate by space, usually not needed.\n\nPrevious run result: "
						+ spacegroup + " " + unitcell);

		// reference HKL
		compRef = new Composite(gProcessParams, SWT.NONE);
		compRef.setLayout(new GridLayout(3, false));
		GridData gdCompRef = new GridData(GridData.FILL_HORIZONTAL);
		gdCompRef.horizontalSpan = 4;
		compRef.setLayoutData(gdCompRef);	
		
		// reference HKL
		new Label(compRef, SWT.NONE).setText("Reference data:");

		txtrefhkl = new Text(compRef, SWT.SINGLE | SWT.BORDER);
		GridData griddata = new GridData(GridData.FILL_HORIZONTAL);
		txtrefhkl.setLayoutData(griddata);
		txtrefhkl.setText("");
		txtrefhkl.setToolTipText(
				"(Optional) a XDS or SCALEPACK formatted of previously processed (partial) data, used only in XDS processing as follows (check XDS keyword REFERENCE_DATA_SET):\n"
						+ "CASE 1: new data proccessing will try to match the indexing scheme of the provided data.\n"
						+ "CASE 2: XDS index/strategy will use the provide data as partial dataset to calculate strategy (i.e multi-strategy).");

		btnrefhkl = new Button(compRef, SWT.PUSH);
		btnrefhkl.setText("Choose");

		btnrefhkl.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				String file = null;
				try {
					NBFileDialog dlg = new NBFileDialog(container.getShell(), SWT.NONE);

					dlg.setFilterPath(txtImageDirectory.getText());
					dlg.setText("Choose a reference hkl file");

					String[] filterExt = { "*.HKL", "*.ahkl", "*.sca", "*.SCA", "*.*" };
					dlg.setFilterExtensions(filterExt);
					file = dlg.openStr();
				} catch (Exception e) {
					FileDialog dlg = new FileDialog(container.getShell(), SWT.NONE);

					dlg.setFilterPath(txtImageDirectory.getText());
					dlg.setText("Choose a reference hkl file");

					String[] filterExt = { "*.HKL", "*.ahkl", "*.sca", "*.SCA", "*.*" };
					dlg.setFilterExtensions(filterExt);
					file = dlg.open();
				}

				if (file != null) {
					txtrefhkl.setText(file);
					
					if (file.endsWith(".HKL") || file.endsWith(".ahkl")) {
						if (!MiscUtils.isXDS_ASCII_HKLFormat(file)) {
							txtrefhkl.setText("");
							NBDisplayMessage(parentShell, "The file provided is not in a valid format.");
							return;
						} 
						
						if (MiscUtils.isXDS_ASCII_HKLFileNative(file)) {
							if (!toggleanomalous.getSelection()) {
								toggleanomalous.setSelection(true);
								anomalous = !anomalous;
							}
						}
					}
				}
			}
		});
		
		// additional xds commands
		new Label(gProcessParams, SWT.NONE).setText("XDS Commands:");
		GridData griddata4 = new GridData(GridData.FILL_HORIZONTAL);
		griddata4.horizontalSpan = 1;
		griddata4.widthHint = 3;
		comboXDScmds = new Combo(gProcessParams, SWT.BORDER);
		comboXDScmds.setLayoutData(griddata4);

		String fixDistance = "Fix distance for integration: REFINE(IDXREF)=BEAM AXIS ORIENTATION CELL;REFINE(INTEGRATE)=BEAM ORIENTATION CELL";
		String disableRefineForIntegration = "Disable refine for integration: DELPHI=10.0;REFINE(INTEGRATE)=!";
		String usingSpotsFromAllImages = "Using spots from all images for indexing (recommended): !SPOT_RANGE=DATA_RANGE";
		String usingSpotsFrom3WedgesOf5Images = "Using spots from 3 wedges of 5 images: !SPOT_RANGE=3_WEDGES_OF_5IMAGES";
		String usingSpotsFromEvenlySpacedImages = "Using spots from 3-degree images per 15 degree rotation: !SPOT_RANGE=3DEGREE_EVERY_15DEGREE";
		String[] presets = { "", 
				usingSpotsFromAllImages, 
				usingSpotsFrom3WedgesOf5Images, 
				usingSpotsFromEvenlySpacedImages, 
				fixDistance, 
				disableRefineForIntegration};
		comboXDScmds.setItems(presets);
		comboXDScmds.select(0); // default
		comboXDScmds.setToolTipText("Additional XDS commands that may be useful for difficult cases, type xds keyword=value pairs separated by semicolon");
		
		comboXDScmds.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				String selected = comboXDScmds.getText(); 
				String[] fields;
				if (selected.contains(": ")) {
					fields = selected.split(": ");
					comboXDScmds.setText(fields[1]);
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		//Log.note("pipeline=" + pipeline + " pageType=" + type + " p=" + combopipeline.getText());

		// turn off xds command by default except when xds related stuff was selected
		//if (type == RESULT_PAGE.SCREENING_RESULTS) {
		//	comboXDScmds.setEnabled(false);
		//}
	}

	// common parts
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Set parameters for data reprocessing");
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		// buttons set up to ignore return events in fields, which is used for
		// validation
		// https://stackoverflow.com/questions/1047144/globally-disable-enter-return-key-in-a-swt-dialog
		Button cancel = createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
		cancel.setText("Cancel");
		setButtonLayoutData(cancel);

		Button ok = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, false);
		ok.setText("Start processing");
		setButtonLayoutData(ok);
	}

	// TODO Validation of user input
	private void saveInput() {
		ImageDirectory = txtImageDirectory.getText();
		start = txtstart.getText();
		end = comboEnd.getText();
		filenamePrefix = comboFilenamePrefix.getText();
		pipeline = combopipeline.getText();
		OutputDirectory = txtOutputDirectory.getText();
		spacegroup = getSpaceGroup(comboSpacegroup.getText());
		unitcell = txtunitcell.getText();
		highResolutionCutoff = combohighResolutionCutoff.getText();
		refhkl = txtrefhkl.getText();
		XDScmds = comboXDScmds.getText();
		sampleName = txtSampleName.getText();
		if (recursive) {
			include_dirs = txtIncludeDirs.getText();
			exclude_dirs = txtExcludeDirs.getText();
		} else {
			include_dirs = "";
			exclude_dirs = "";
		}
	}

	public String guessSampleName(String path) {
		// given a pathname, guess the sample name
		while (path.endsWith("/")) {
			path = path.substring(0, path.length() - 1);
			if (path.length() < 1) {
				return "";
			}
		}
		String[] parts = path.replace("//", "/").split("/");
		int len = parts.length;
		if (len >= 2 && (parts[len - 1].startsWith("cbf") || parts[len - 1].startsWith("hdf"))) {
			return parts[len - 2];
		}

		return parts[len - 1];
	}

	// nonblocking display message
	public static void NBDisplayMessage(Shell shell, String msg) {
		MessageDialog dialog = new MessageDialog(shell, "Warning", null, msg, MessageDialog.WARNING,
				new String[] { IDialogConstants.OK_LABEL }, 0);
		dialog.setBlockOnOpen(false);
		dialog.open();
	}

	private boolean validateInputs() {
		// check if there are anything to process
		// sanity check
		File imgdir = new File(this.ImageDirectory);
		if (!imgdir.exists()) {
			String msg = "directory does not exist, must provide a valid directory with images.";
			NBDisplayMessage(this.getShell(), msg);
			return false;
		}

		FindImagePrefixes fp = new FindImagePrefixes(ImageDirectory);
		Integer nImages = fp.nImages;
		if (this.type == RESULT_PAGE.SCREENING_RESULTS && nImages < 1 ) {
			String msg = "No images found for indexing. Change directory and retry.";
			NBDisplayMessage(this.getShell(), msg);
			return false;
		}

		if (this.type == RESULT_PAGE.PROCESSING_RESULTS && nImages < 10 && 
				!this.recursive 
				&& !(new File(ImageDirectory + "/datasets.txt").exists() && this.pipeline.startsWith("kamo"))) {
			String msg = String.format("Not enough images found(%d, <10) for processing. Change directory, or click recursive option.",
					nImages);
			NBDisplayMessage(this.getShell(), msg);
			return false;
		}

		// validating other fields
		if (spacegroup != null && !spacegroup.isEmpty() && !ValidateUserInput.isValidSpaceGroup(spacegroup)) {
			String msg = "Invalid space group.";
			NBDisplayMessage(container.getShell(), msg);
			return false;
		}

		if (unitcell != null && !unitcell.isEmpty() && !ValidateUserInput.isValidUnitCell(unitcell)) {
			String msg = "Unit cell is defined by 6 numbers separated by space or comma.";
			NBDisplayMessage(container.getShell(), msg);
			return false;
		}

		if (!unitcell.isEmpty() && spacegroup.isEmpty()) {
			String msg = "Unit cell is defined but space group is not, please define space group consistent with the cell first.";
			NBDisplayMessage(container.getShell(), msg);
			return false;
		}

		if (unitcell.isEmpty() && !spacegroup.isEmpty() && pipeline.startsWith("kamo")) {
			String msg = "Space group is defined but unit cell is not, KAMO needs both specified, or neither.";
			NBDisplayMessage(container.getShell(), msg);
			return false;
		}
		

		if (!ValidateUserInput.isEmptyOrInteger(start)) {
			String msg = "start frame number field should be empty or an integer.";
			NBDisplayMessage(container.getShell(), msg);
			return false;
		}

		if (!ValidateUserInput.isEmptyOrInteger(end)) {
			String msg = "end frame number field should be empty, an integer, or percentage.";
			NBDisplayMessage(container.getShell(), msg);
			return false;
		}

		if (!ValidateUserInput.isPercentage(end) && ValidateUserInput.isInteger(start)
				&& ValidateUserInput.isInteger(end) && (Integer.valueOf(start) > Integer.valueOf(end))) {
			String msg = "start frame number should be smaller than end frame number.";
			NBDisplayMessage(container.getShell(), msg);
			return false;
		}

		if (ValidateUserInput.isNumeric(highResolutionCutoff)) {
			Double highres = Double.valueOf(highResolutionCutoff);
			if (highres > 10 || highres < 0.6) {
				String msg = "high resolution cutoff value is unreasonable.";
				NBDisplayMessage(container.getShell(), msg);
				return false;
			}
		}

		// if filename prefix is defined, are there any images matching it?
		if (filenamePrefix != null && !filenamePrefix.isEmpty()) {
			if (this.recursive) {
				return true;
			} // for recurive parsing, disable this check

			SortedSet<String> prefixes = fp.getPrefixes();
			boolean match = false;
			for (String prefix : prefixes) {
				if (prefix.startsWith(filenamePrefix) || filenamePrefix.startsWith(prefix)) {
					match = true;
				}
			}

			if (!match) {
				String msg = "filename prefix not properly defined, since no images matching it.";
				NBDisplayMessage(container.getShell(), msg);
				return false;
			}
		}

		return true;
	}

	public void okPressed() {
		// new TestSGE().tests();

		System.out.println(String.format(
				"processing parameter by user: imagedir=%s " + "outdir=%s " + "start=%s " + "end=%s " + "filename=%s "
						+ "pipeline=%s " + "spg=%s " + "unitcell=%s " + "highres=%s " + "anomalous=%s " + "refhkl=%s ",
				ImageDirectory, OutputDirectory, start, end, filenamePrefix, pipeline, spacegroup, unitcell,
				highResolutionCutoff, anomalous, refhkl));

		// Log.error("qxu: " + Util.isUserNonProprietary("qxu"));
		// Log.error(Util.getUserLogin() + ": " + Util.isUserNonProprietary());

		saveInput();

		// validation
		if (!validateInputs()) {
			return;
		}

		if (OutputDirectory == null || OutputDirectory.isEmpty()) {
			OutputDirectory = ImageDirectory + "/process";
		}
		File of = new File(OutputDirectory);
		if (!of.exists()) {
			of.mkdirs();
		}

		String beamline = Util.getBID();

		PipelineOptions p = new PipelineOptions.PipelineBuilder(ImageDirectory, OutputDirectory)
				.sampleName(sampleName)
				.sampleNumber(sampleNumber)
				.anomalous(anomalous)
				.smallWedges(smallWedges)
				.start(start)
				.end(end)
				.spacegroup(spacegroup)
				.unitcell(unitcell)
				.filenamePrefix(filenamePrefix)
				.pipeline(pipeline)
				.njobs(this.njobs)
				.nproc(this.nproc)
				.runner(this.runner)
				.highResolutionCutoff(highResolutionCutoff)
				.refhkl(refhkl)
				.beamline(beamline)
				.recursive(recursive)
				.excludeDirs(exclude_dirs)
				.includeDirs(include_dirs)
				.xdscmds(XDScmds)
				.build();

		SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd-hh-mm-ss");
		String strDate = datetimeFormatter.format(new Date());
		String joblabel = String.format("%s-%s-%s", "reprocess", pipeline, strDate);
		new Thread(new CmdProcessData(p, joblabel)).start();
		this.close();

		String msg = "You job is submitted to the computing queue.\n\nUpdate will occur periodically, \nyou will not see your job status immediately in the results table.";
		Log.note(msg);

		NBDisplayMessage(parentShell, msg);
	}

	@Override
	public void run() {
		this.open();
	}
}
