package gov.anl.gmca.analysis.helpers;

import gov.anl.gmca.analysis.WdgDataProcDialog;
import gov.anl.gmca.collect.DevCollectRun;
import gov.anl.gmca.collect.DevCollectRunList;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DevRunList.RUN_LABEL_SETTING;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.WdgMain;
import gov.anl.gmca.gui.WdgMain.TAB;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;

// export strategy result to a new run
public class ExportStrategy {
	int id;
	Double startosc = null;
	Double endosc = null;
	Double delta = null;
	Double distance = null;
	boolean createNewRun = true;
	String returnMessage = null;
	static Double ENDPADDING = 5.0;
	static Double STARTADDING = 0.0;
	
	public ExportStrategy(Double startosc, Double endosc, Double delta, Double distance, boolean createNewRun) {
		this.createNewRun = createNewRun;

		this.startosc = startosc;
		this.endosc = endosc;
		this.delta = delta;
		this.distance = distance;

		this.returnMessage = this.exportStrategy();
		Log.note(this.returnMessage);
		WdgDataProcDialog.NBDisplayMessage(GUI.getShell(), this.returnMessage);
	}

	public String getExportMessage() {
		return returnMessage;
	}
	
	public ExportStrategy(Double startosc, boolean createNewRun) {
		this(startosc, null, null, null, createNewRun);
	}
	
	public ExportStrategy(Double startosc, Double endosc, boolean createNewRun) {
		this(startosc, endosc, null, null, createNewRun);
	}
	
	public ExportStrategy(StrategyResults sr, boolean createNewRun) {
		this(sr.getOscStart(), sr.getOscEnd(), sr.getOscDelta(), sr.getDetectorDistance(), createNewRun);
	}
	
	public ExportStrategy(int id, boolean createNewRun) {
		this(new StrategyResults(id), createNewRun);
	}

	private String exportStrategy() {	
		if (!Client.isMaster()) {
			return "The client is not the master, export not allowed.";
		}

		Log.note(String.format("export strategy to run: start=%s, end=%s, delta=%s, detector distance=%s",
				this.startosc, this.endosc, this.delta, this.distance));

		if (this.startosc == null) {
			return "Failed to obtain strategy results from database.";
		}

		// apply padding to the start & end
		this.startosc = this.startosc + STARTADDING;
		if (endosc == null) {
			this.endosc = this.startosc + 180;
		} else {
			this.endosc += ENDPADDING;
		}


		try {
			ModelCollectRunList runlist = ModelCollectRunList.get();
			int targetIndex = createNewRun ? runlist.getRunCount() + 1 : runlist.getCurrentRunIndex();
			
			if (createNewRun) {
				if (targetIndex < ModelCollectRunList.MAX_COUNT) {
					DevCollectRunList listDev = DevCollectRunList.get();
					DevCollectRunList.getIndex(runlist.getRunCount()).copyRunContents(targetIndex, RUN_LABEL_SETTING.INCREMENT_SRC,
							true, false); // copy contents of the last one
		
					// create new run
					DevCollectRun.setStartFrameLabel(targetIndex, 1);
					DevCollectRun.setStartAngle(targetIndex, this.startosc);

					DevCollectRun.setEndAngle(targetIndex, endosc);
					
					if (this.delta != null) {
						DevCollectRun.setDelta(targetIndex, this.delta);
					} 
					
					if (this.distance != null) {
						DevCollectRun.setDistance(targetIndex, distance);
					}

					// increase run count to load the new run
					listDev.setRunCount(targetIndex, false);
					listDev.setCurrentRun(targetIndex, true);
					Log.note(String.format("export strategy to run (with padding): start=%.1f, end=%.1f, delta=%.2f, detector distance=%.1f",
							this.startosc, this.endosc, this.delta, this.distance));

					// run it
					// ModelCollectRunList.get().startCollection();
				} else {
					return "Exceeded run limit, export failed.";
				}
			} else {
				ModelCollectRun run = (ModelCollectRun) ModelCollectRunList.get().getRun(targetIndex);
				run.dev().setStartFrameLabel(1, false);
				run.dev().setStartAngle(this.startosc, false);
				run.dev().setDelta(this.delta, false);
				run.dev().setDistance(this.distance, false);
				run.fid.changeEndAngle(this.endosc); // sends update
			}
			WdgMain.setVisibleTab(TAB.Collect); // change view to CollectTab
			return String.format("Strategy exported to new run, padded with %s, %s deg. at ends.", STARTADDING, ENDPADDING);
		} catch (NullPointerException e) {
			e.printStackTrace();
		}
		return "Strategy export failed.";
	}

	@Override
	public String toString() {
		return "Strategy Results: osc start=" + startosc + " end=" + endosc + " osc_range="+delta + " dist="+distance;
	}

}
