package gov.anl.gmca.analysis.helpers;

import gov.anl.gmca.analysis.autoProcessData;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.syslog.Log;

import java.io.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class FileListToTemplateStr {
	// this class convert a list of image file names into a template string, does not support marccd style filenames
	public static String imageFilenamePatternStr = "^(.+_)(\\d{3,6}\\.|master\\.)(h5|cbf)$";
	public static Pattern imageFilenamePattern = Pattern.compile(imageFilenamePatternStr);
	private int nSweeps = 0;
	
	ArrayList<String> imgList;
	String imgDir;

	public FileListToTemplateStr(ArrayList<String> imgList, String imgDir) {
		this.imgList = imgList;
		this.imgDir = imgDir;
	}

	public FileListToTemplateStr(int runNumber) {
		try {
			this.imgList = autoProcessData.getImageFilenames(runNumber);
			this.imgDir = ((ModelCollectRun) ModelCollectRunList.get().getRun(runNumber)).getFullDirectory();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public int getNSweeps() {
		if (nSweeps == 0) {
			String templates = getTemplateStr(true, ";");
			Log.note("data filename templates=" + templates + " nSweeps="+nSweeps);
		}
		return nSweeps;
	}
	
	public String getTemplateStr(boolean includeFullPath, String sep) {
		if (imgList == null || imgList.isEmpty())
			return null;

		String fullpath = "";
		if (includeFullPath && imgDir != null) {
			fullpath = imgDir + "/";
		}
		fullpath = fullpath.replaceAll("/+", "/");
		
		Collections.sort(imgList);

		ArrayList<String> saved_templateStartEnd = new ArrayList<String>();
		ArrayList<Integer> sweep_nums = new ArrayList<>();
		String saved_template = null;

		for (int i = 0; i < imgList.size(); i++) {
			String file = imgList.get(i);

			Matcher m = imageFilenamePattern.matcher(file);

			if (!m.matches()) {
				Log.warn(String.format("Unknown image name: %s", file));
				continue;
			}
			m.reset();
			m.find();
			
			String prefix = m.group(1);

			String seq = m.group(2).replace(".", "");
			if (seq.equals("master"))
				continue;

			Integer iseq = Integer.parseInt(seq);
			sweep_nums.add(iseq);
			String suffix = m.group(3);

			String template = null;
			if (suffix != null) { // if there is suffix
				template = String.format("%s%s.%s", prefix, seq.replaceAll("\\d", "?"), suffix);
			} else { // no suffix, e.g. prefix_name.001
				template = String.format("%s.%s%s", prefix, seq.replaceAll("\\d", "?"), "");
			}

			if (i == 0) {
				saved_template = template;
			}

			if (!template.equals(saved_template)) {
				String templateStartEnd = String.format("%s%s,%d,%d", fullpath, saved_template, sweep_nums.get(0),
						sweep_nums.get(sweep_nums.size() - 2));
				saved_templateStartEnd.add(templateStartEnd);
				saved_template = template;

				Integer last = sweep_nums.get(sweep_nums.size() - 1);
				sweep_nums.clear();
				sweep_nums.add(last);
			}

		}
		String templateStartEnd = String.format("%s%s,%d,%d", fullpath, saved_template, sweep_nums.get(0),
				sweep_nums.get(sweep_nums.size() - 1));
		saved_templateStartEnd.add(templateStartEnd);
		nSweeps = saved_templateStartEnd.size();
		return String.join(sep, saved_templateStartEnd);
	}
	
	public String getTemplateStr(boolean includeFullPath) {
		return getTemplateStr(includeFullPath, "\n");
	}
	
	public void getTemplateStrAsFile(boolean includeFullPath) {
		/*
		if (!Client.isMaster()) {
			return;
		}
		*/
		
		String datasets = getTemplateStr(includeFullPath);

		if (datasets == null) {
			return;
		}
		
		String saveDir = null;
		try {
			saveDir = DevDataDir.getPrimary();
		} catch (Exception e) {
			Log.warn("unable to get the collection directory, default to home.");
		}
		
		if (saveDir == null) {
			saveDir = System.getProperty("user.home");
		}

		String outfile = saveDir + "/" + "datasets.txt";
		
		// append to datasets.txt
		try (Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outfile, true), "utf-8"))) {
			writer.write(datasets+"\n"); 
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		System.out.println("saved file: " + outfile);
	}

	public static void main(String... args) {
		ArrayList<String> ilist = new ArrayList<>();
		ilist.add("X_000999.cbf");
		ilist.add("X_001003.cbf");
		ilist.add("Y_00003.cbf");
		ilist.add("Y_00003.h5");
		ilist.add("Y_master.h5");
		ilist.add("Y.00003");
		ilist.add("X_00002.cbf");
		ilist.add("X_00005.cbf");

		FileListToTemplateStr f = new FileListToTemplateStr(ilist, "/x");
		System.out.println(f.getTemplateStr(true));
		f.getTemplateStrAsFile(true);
	}
}
