package gov.anl.gmca.analysis.helpers;

import com.google.gson.Gson;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.Util;
import org.eclipse.jface.window.Window;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.util.HashMap;

public class MiscUtils {
	public static String analysisSubDirectory = "legacy";
	public static String analysisNGSubDirectory = DBConfig.getBoolean("processing_enable_analysis_legacy")?"analysis":"";
	
	public static String replace (String source, String os, String ns) {
	    if (source == null) {
	        return null;
	    }
	    int i = 0;
	    if ((i = source.indexOf(os, i)) >= 0) {
	        char[] sourceArray = source.toCharArray();
	        char[] nsArray = ns.toCharArray();
	        int oLength = os.length();
	        StringBuilder buf = new StringBuilder (sourceArray.length);
	        buf.append (sourceArray, 0, i).append(nsArray);
	        i += oLength;
	        int j = i;
	        // Replace all remaining instances of oldString with newString.
	        while ((i = source.indexOf(os, i)) > 0) {
	            buf.append (sourceArray, j, i - j).append(nsArray);
	            i += oLength;
	            j = i;
	        }
	        buf.append (sourceArray, j, sourceArray.length - j);
	        source = buf.toString();
	        buf.setLength (0);
	    }
	    return source;
	}
	
	public static String replaceOnce (String source, String os, String ns) {
	    if (source == null) {
	        return null;
	    }
	    int i = 0;
	    if ((i = source.indexOf(os, i)) >= 0) {
	        char[] sourceArray = source.toCharArray();
	        char[] nsArray = ns.toCharArray();
	        int oLength = os.length();
	        StringBuilder buf = new StringBuilder (sourceArray.length);
	        buf.append (sourceArray, 0, i).append(nsArray);
	        i += oLength;
	        int j = i;

	        buf.append (sourceArray, j, sourceArray.length - j);
	        source = buf.toString();
	        buf.setLength (0);
	    }
	    return source;
	}
	
	// utility functions
	public static boolean isStateEqual(Result r1, Result r2) {
		if (!r1.isOK() || !r2.isOK()) {
			return false;
		}

		if (r1.numRows() == r2.numRows()) {
			for (int i = 0; i < r1.numRows(); i++) {
				// NB map comparison, assuming that the rows are ordered 
				if (!r1.getRow(i).equals(r2.getRow(i))) {
					return false;
				}
			}
			return true;
		} 
		return false;
	}
	
	// generate json output from hashmap
	public static String hash2json(HashMap<String, String> row) {
		Gson gson = new Gson();
		String json = gson.toJson(row, HashMap.class);
		return json;
	}

	// turn Result into a json array
	public static String result2json(Result result, String jsonfile) {
		if (result.numRows() == 0) {
			return "[]";
		}
		StringBuilder sb = new StringBuilder();
		sb.append("[\n");
		String delimiter = "";
		for (HashMap<String, String> row: result.getRows()) {
			String json = MiscUtils.hash2json(row);
			sb.append(delimiter);
			sb.append(json);
			delimiter = ",\n";
		}
		sb.append("]\n");

		String jsonArray = sb.toString();
		if (jsonfile != null) {
			try {
				FileWriter fw = new FileWriter(jsonfile);
				fw.write(sb.toString());
				fw.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return jsonArray;
	}
	
	public static String readFile(String path, Charset encoding) throws IOException {
		byte[] encoded = Files.readAllBytes(Paths.get(path));
		return new String(encoded, encoding);
	}
	
	// read XDS_ASCII to find out whether the data is native or anomalous
	public static boolean isXDS_ASCII_HKLFileNative(String path) {
		try {
			if (readFile(path, Charset.defaultCharset()).contains("FRIEDEL'S_LAW=TRUE")) {
				return true;
			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return false;
	}

	// check whether the file is XDS_ASCII format
	public static boolean isXDS_ASCII_HKLFormat(String path) {
		try {
			if (readFile(path, Charset.defaultCharset()).contains("!FORMAT=XDS_ASCII")) {
				return true;
			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return false;
	}
	
	// given image directory, figure out where is the processRoot
	// same level at imageDir or a subDir in imageDir
	/**
	 * @param imageDir
	 * @return
	 */
	public static String getProcessDir(String imageDir) {
		// given gmca style image directory, suggests a processing directory
		// by insert "/process/" into middle of the path

		//		Pattern pattern = Pattern.compile("(^\\/.*/23ID[BD]_\\d{4}_\\d{2}_\\d{2}.{0,4}\\/)(.*)");
		//		Matcher m = pattern.matcher(imageDir);
		//		if (m.matches() && m.groupCount() == 2) {
		//			String dataRoot = m.group(1);
		//			String subDir = m.group(2);
		//			return String.format("%s/%s/%s/%s", dataRoot, "process", subDir, "auto").replaceAll("//","/");
		//		}
		//		return String.format("%s/%s/%s", imageDir, "process", "auto");
		
		// check whether user has chosen the alternate directory structure first
		if (Client.isMaster()) { 
			try {
				Integer bgProcMode = DevUserConfig.get().getXDSProc(); // Epics.getInteger(PV_RUNS.bgProc);
				if (bgProcMode == 1) { 
					String imageRoot = DevDataDir.getPrimary(); 
					String procRoot = imageDir.replace(imageRoot, imageRoot + "/process").replaceAll("//", "/");

					if (procRoot.endsWith("collect")) {
						procRoot = procRoot.substring(0, procRoot.lastIndexOf("collect"));
					} else if ((procRoot.endsWith("collect/"))) {
						procRoot = procRoot.substring(0, procRoot.lastIndexOf("collect/"));
					}
					Log.note("alternate processing root dir: " + procRoot);
					return procRoot;
				}
			} catch (Exception e) {
				Log.note("unable to get processing root directory...");
				e.printStackTrace();
			}
		}

		// regular method of putting processing directory along the images
		String processRootDir;
		imageDir = imageDir.replace("//", "/");
		if (imageDir.endsWith("/")) {
			imageDir = imageDir.substring(0, imageDir.lastIndexOf("/"));
		}
		if (imageDir.endsWith("collect") || imageDir.endsWith("collection") ||
				imageDir.endsWith("screen") || imageDir.endsWith("screening")) {
			processRootDir=imageDir.substring(0, imageDir.lastIndexOf("/"));
		} else {
			processRootDir = imageDir;
		}
		processRootDir = String.format("%s/%s", processRootDir, "process");
		return processRootDir;
	}
	
	public static String getProcessDir(String imageDir, String subdir) {
		String outdir = String.format("%s/%s", getProcessDir(imageDir), subdir);
		File oDir = new File(outdir);
		if (!oDir.exists()) {
			try {
				oDir.mkdirs();
			} catch (Exception e) {
				e.printStackTrace();
				Log.warn("unable to make directory: "+outdir);
			}
		}
		return outdir;
	}
	
	public static String getComputeNode() {
		boolean localmodel = true;
		if (localmodel)
			return null;
		
		// ignore for the time being since remote execution does not work consistently
		String bl = Util.getBID();
		switch (bl) {
		case "23o":
			return "bl2ws6";
		case "23i":
			return "bl1ws6";
		case "23b":
			return "bl3ws6";
		default:
			return "bl3ws6";
		}
	}
	
	public static String rtrim(String is, String trim) {
		if (is == null || is.isEmpty() || trim.isEmpty()) {
			return is;
		}
		while (is.endsWith(trim)) {
			int i = is.lastIndexOf(trim);
			is = is.substring(0, i);
			if (is.length() == 0) {
				return "";
			}
 		}
		return is;
	}

	public static String trimCell(String cell) {
		//make cell string shorter
		try {
			cell = cell.trim();
			String[] cells = cell.split("\\s+");
			for (int i = 0; i < cells.length; i++) {
				Double value = Double.valueOf(cells[i]);
				if (i < 3) {
					cells[i] = String.format("%.1f", value);
				} else {
					cells[i] = String.format("%.0f", value);
				}
			}
			return String.join(",", cells);
		} catch (Exception e) {
			e.printStackTrace();
			return cell;
		}
	}

	public static void copyFile(String src, String dest) {
		File destination = new File(dest);
		if (destination.exists()) {
			System.out.println(dest + " exists. Skip copy.");
			return;
		}
		
		File source = new File(src);
		if (!source.exists()) {
			System.out.println(src + " does not exist. Fail to copy.");
			return;
		}

		try {
			Files.copy(source.toPath(), destination.toPath());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	public static void main(String... strings) {
		String a="a/13432da////";
		System.out.println(rtrim(a, "/"));
	}
	
}
