package gov.anl.gmca.analysis.helpers;

import gov.anl.gmca.analysis.CmdProcessData;
import gov.anl.gmca.analysis.PipelineOptions;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.widgets.Shell;

// this class run strategy based on user selected space group 
// and export the results to a new Collection tab

public class StrategyWithSpaceGroup {
	Shell shell;
	int timeout = 120;
	
	private String getProcessingRootDir(Integer id) {
		String query = String.format("select workdir from pipelinestatus where id = %d;", id);
		Result r = Database.exec(DBName.USERDATA, query);
		if (!r.isOK() || r.numRows() != 1) {
			Log.error("failed to processing root directory for pipelinestatus_id=" + id);
			return null;
		}
		
		return r.getRow(0).get("workdir");
	}
	
	public StrategyWithSpaceGroup(String[] params, Shell shell) {
		this.shell = shell;
		
		System.out.println(params.length);
		if (params.length < 8) {
			Log.error("no enough information to proceed with Strategy calculation");
			return;
		}
		
		Integer id = Integer.parseInt(params[0]);
		String sampleName = params[1];
		String pipeline = params[2];
		String imgdir = params[3];
		String workdir = params[4];
		Boolean anomalous = params[5].equals("1"); 
		String spg = params[6];
		String defaultSpg = params[7];

		String beamline = Util.getBID();
		
		workdir = getProcessingRootDir(id);
		if (workdir == null) {
			workdir = params[4];
		}
		//String rootDir = MiscUtils.getProcessDir(imgdir, MiscUtils.analysisNGSubDirectory);
		//workdir = String.format("%s/%s/indexing/%s", rootDir, sampleName, pipeline);
		
		if (spg.toLowerCase().equals(defaultSpg.toLowerCase())) {
			new ExportStrategy(id, true);
			return;
		}

		PipelineOptions p = new PipelineOptions.PipelineBuilder(imgdir, workdir)
				.sampleName(sampleName)
				.anomalous(anomalous)
				.pipeline(pipeline)
				.spacegroup(spg)
				.njobs("1")
				.nproc("2")
				.runner("local")
				.beamline(beamline)
				.build();
		
		CmdProcessData proc = new CmdProcessData(p, spg, true);
		Thread t = new Thread(proc);
		t.start();
		
		//ThreadMonitorDialog tmd = new ThreadMonitorDialog(shell, t, timeout);
		//tmd.setBlocking(true);

		ThreadMonitorDialog tmd = new ThreadMonitorDialog(shell, t, timeout, new Runnable() {
			@Override
			public void run() {
				new ExportStrategy(new StrategyResults(sampleName, spg, 4), true);
			}
		});

		tmd.setBlocking(false);
		tmd.setMessage(String.format("You have selected a different point group %s as compared to suggested %s, "
				+ "which requires recalculation.\n "
				+ "This calculation usually takes 30-60sec for mosflm or xds. Please wait. \n\n"
				+ "This window will close itself if results do not return in %d sec, \n"
				+ "or you may abort the export operation now, the job you started will continue to run, \n"
				+ "you can check the result in the screening table later.", spg, defaultSpg, timeout));
		tmd.open();

	}

}
