package gov.anl.gmca.analysis.helpers;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

// this call ask user for password before executing the requested callback, a runnable
// two constructors
// 1) 1st is used in blocking mode, password status is passed through passwordOK
// example of use:
//  WdgProtectedRunnableDialog dlg = new WdgProtectedRunnableDialog(shell);
//  if (dlg.open() == Window.OK) {
//	if (dlg.isPasswordOK()) {
//		new WdgEditPucks(shell, true).open();  //blocking 
//		updateWidgetState(); // ui updates dependent on the success of the previous step
//	 }
//  }
// 2) 2nd constructor is nonblocking, the callback supplied is run async after password check success
//
public class WdgProtectedRunnableDialog extends Dialog {
	private Text usernameField;
	private Text passwordField;
	private Runnable callback;
	private boolean passwordOK = false;
	private boolean blocking = false;

	public WdgProtectedRunnableDialog(Shell parentShell) {
		super(parentShell);
		this.blocking = true;
		setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(this.blocking);
		this.callback = null;
	}
	
	public WdgProtectedRunnableDialog(Shell parentShell, Runnable callback) {
		super(parentShell);
		this.blocking = false;
		setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(blocking);
		this.callback = callback;
	}

	protected Control createDialogArea(Composite parent) {
		Composite comp = (Composite) super.createDialogArea(parent);

		GridLayout layout = (GridLayout) comp.getLayout();
		layout.numColumns = 2;

		Label usernameLabel = new Label(comp, SWT.RIGHT);
		usernameLabel.setText("Username: ");

		usernameField = new Text(comp, SWT.SINGLE);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		usernameField.setLayoutData(data);
		usernameField.setText("gmca");
		usernameField.setEnabled(false);

		Label passwordLabel = new Label(comp, SWT.RIGHT);
		passwordLabel.setText("Password: ");

		passwordField = new Text(comp, SWT.SINGLE | SWT.PASSWORD);
		data = new GridData(GridData.FILL_HORIZONTAL);
		passwordField.setLayoutData(data);

		return comp;
	}

	protected void createButtonsForButtonBar(Composite parent) {
		super.createButtonsForButtonBar(parent);
	}

	protected void okPressed() {
		String pwd = passwordField.getText();
		if (pwd != null) {
			passwordOK = Util.checkPassword("gmca", pwd);
			if (passwordOK) {
				runCallback();
				//Log.note("password ok, executed callback.");
			} else {
				Log.note("invalid password");
			}
		} 
		this.close();
	}
	
	private void runCallback() {
		if (this.callback !=null) {
			GUI.asyncExec(this.callback);
		} 
	}

	public boolean isPasswordOK() {
		return passwordOK;
	}
}