package gov.anl.gmca.collect;

import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.collect.ModelCollectRunList.CollectLeftTab;
import gov.anl.gmca.collect.ModelCollectRunList.CollectState;
import gov.anl.gmca.common.DevEpicsUpdate;
import gov.anl.gmca.common.DevRunList;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.io.PVLookup.PV_RUNS;
import gov.aps.jca.dbr.DBR;

import java.util.ArrayList;

public class DevCollectRunList extends DevEpicsUpdate implements DevRunList
{
	protected ModelCollectRunList runList;
	
	protected ArrayList<DevCollectRun> runDevs = new ArrayList<DevCollectRun>();
	
	// TODO: remove one of these
	public DevRun getRunDev(int runIndex) { return (DevRun)runDevs.get(runIndex); }
	public static DevCollectRun getIndex(int index) { return inst.runDevs.get(index); }
	
	protected static DevCollectRunList inst = null;
	public static DevCollectRunList get() { return inst; }
	
	public DevCollectRunList(ModelCollectRunList _runList)
	{
		runList=_runList;
		runList.devRunList = this;
		
		inst = this;

		for (PVName pvName : PV_RUNS.values())
		{
			if (pvName != PV_RUNS.currentLeftTab)
			{
				registerPV(pvName);
			}
		}
		setUpdatePV(PV_RUNS.updated);
		
		Epics.addMonitor(PV_RUNS.currentLeftTab, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				CollectLeftTab leftTab = null;
				
				int iLeftTab = Epics.dbrToInt(value);
				int numTabs = CollectLeftTab.values().length;
				if (iLeftTab>=0 && iLeftTab<numTabs) leftTab = CollectLeftTab.values()[iLeftTab];
				
				if (null != leftTab)
				{
    				runList.setLeftTab(leftTab);
    				runList.llLeftTab.notifyListeners(leftTab);
				}
			}
		});
	}
	
	public void connect()
	{
		registrationComplete();
		//while (!connected()) Util.sleep(10);
	}
	
	public int 		getCurrentRunIndex() 		{ return Epics.getInteger(PV_RUNS.currentRun); }
	public int 		getRunCount()				{ return Epics.getInteger(PV_RUNS.runCount); }
	
	@Override
	protected void sendNotification()
	{
		CollectLeftTab		leftTab					= CollectLeftTab.values()[Epics.getInteger(PV_RUNS.currentLeftTab)];
		int 				runCount 				= Epics.getInteger(PV_RUNS.runCount);
		int 				currentRun 				= Epics.getInteger(PV_RUNS.currentRun);
		
		// TODO: This should be moved to ModelCollectRunList.validateAndFix()
		if (currentRun>runCount)
		{
			setCurrentRun(runCount,true);
			return;
		}

		// set these two variables in an order so runCount is never less than currentRun
		if (runList.getCurrentRunIndex() > runCount)
		{
			runList.setCurrentRun(currentRun);
			runList.setRunCount(runCount);
		}
		else
		{
			runList.setRunCount(runCount);
			runList.setCurrentRun(currentRun);
		}
		
		runList.setBinMode(BinMode.values()[Epics.getInteger(PV_RUNS.binMode)]);
		runList.setCollectState(CollectState.values()[Epics.getInteger(PV_RUNS.collectState)]);
		runList.setLeftTab(leftTab);
		runList.notifyListeners();
	}
	
    public boolean setRunCount(int runCount, boolean sendUpdate)
    {
    	//if (!runList.validateRunCount(runCount)) return false;
    	
    	// if this is uncommented, this case may happen:
    	// 1) local+remote size agree: 9
    	// 2) local size quickly goes to 8 then 7
    	// 3) remote size goes to 8 and causes an update
    	// 4) now a run tab has to be created to go from 7 to 8
    	// 5) tab 8 has no run model, it's been deleted
    	//
    	//runList.setRunCount(runCount);
    	//runList.notifyListeners();
    	
		Epics.putInteger(PV_RUNS.runCount, runCount);
		if (sendUpdate) sendUpdate();
		
    	return true;
    }
    
    public boolean setCurrentRun(int currentRun, boolean sendUpdate)
    {
    	// this returns false when the star tab is clicked, because the callback didn't come
    	// back yet to update the runCount
    	//if (!runList.validateCurrentRun(currentRun)) return false;
    	
    	//runList.setCurrentRun(currentRun);
    	//runList.notifyListeners();
    	
		Epics.putInteger(PV_RUNS.currentRun, currentRun);
		if (sendUpdate) sendUpdate();
		
		return true;
    }

	public void setCollectState(CollectState state, boolean sendUpdate)
	{
		runList.setCollectState(state);
		runList.notifyListeners();
		
		Epics.putInteger(PV_RUNS.collectState, state.ordinal());
		if (sendUpdate) sendUpdate();
	}
	
	public void setBinMode(BinMode mode, boolean sendUpdate)
	{
		runList.setBinMode(mode);
		runList.notifyListeners();
		
		Epics.putInteger(PV_RUNS.binMode, mode.ordinal());
		if (sendUpdate) sendUpdate();
	}
	
	// left tab is updated independently of the other PVs and is not linked to the update PV
	public void setCurrentLeftTab(CollectLeftTab currentLeftTab)
	{
		if (null == currentLeftTab) throw new IllegalArgumentException("null should not be passed to setCurrentLeftTab");
		
		runList.setLeftTab(currentLeftTab);
		runList.llLeftTab.notifyListeners(currentLeftTab);
		
		Epics.putInteger(PV_RUNS.currentLeftTab, -1);
		Epics.putInteger(PV_RUNS.currentLeftTab, currentLeftTab.ordinal());
	}
}
