package gov.anl.gmca.collect;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.client_server.CounterState;
import gov.anl.gmca.gui.state.SWServer;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_FPE;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;

import java.io.*;
import java.util.LinkedList;
import java.util.concurrent.atomic.AtomicInteger;

public class DevSimFPE extends DevFPE
{
    private FPEState fpeState = FPEState.idle;

    @Override public FPEState getFPEState()
    {
        return fpeState;
    }

    public boolean checkFPEState(FrameList frameList, boolean nonFPEChecks) { return true; }

    private void changeState(FPEState newState) {
        fpeState = newState;
        llFPEState.notifyListeners();
    }

    @Override public boolean isIdle()
    {
        return fpeState==FPEState.idle;
    }
    @Override public boolean isCacheIdle()
    {
        return isIdle();
    }

    private boolean stopFlag = false;
    @Override public void collectPartition(FrameList frameList)
    {
        stopFlag = false;
        Log.note(String.format("Collecting %d frames...",frameList.frameCount));
        changeState(FPEState.blu_ice_updates);
        for (int i=0;i<frameList.frameCount;i++) {
            if (stopFlag) break;

            String src = String.format("/mnt/beegfs/testdata/QXU/AEF_Se_MalA_1_2_1_%06d.cbf", (i+1)%999);
            //String src = String.format("/mnt/beegfs/testdata/PC/PC_%06d.cbf", (i+1)%999);
            String dest = String.format("%s/%s%06d.cbf", frameList.directory.replaceAll("/$",""), frameList.prefix, frameList.firstFrameLabel+i);
            ProcessBuilder pb = new ProcessBuilder(new String[]{"/bin/cp", src, dest});
            //pb.directory(new File(""));
            try {
                Process p = pb.start();
                p.waitFor();
            } catch (Exception e) { e.printStackTrace(); }
            handleFramePV(frameList.firstFrameIndex+i+1, false);
            Util.sleep((int)(frameList.exposureTime*1000));
        }
        changeState(FPEState.idle);
        Log.note("Done!");
        frameList.result = FrameListResult.SUCCEEDED;
    }

    @Override public void abort()
    {
        stopFlag=true;
    }

    @Override public boolean pause(boolean enforcePauseSpacing)
    {
        stopFlag=true;
        return true;
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////// interface implementation ////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////

    public DevSimFPE()
    {
        super("FPE");
    }
}
