package gov.anl.gmca.collect;

import gov.anl.gmca.common.Redis;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.spotfinder.DBImageStats;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.util.Point4D;
import org.apache.commons.lang3.ArrayUtils;

import java.nio.FloatBuffer;
import java.util.Arrays;
import java.util.Collections;

public class MeshParams
{
   ////////////////////////////////////////////// change methods ////////////////////////////////////////////////////

    public static void changeRasterRun(ModelCollectRun run, int rasterRunIndex)
    {
        try {
            ModelRasterRun rrun = (ModelRasterRun) ModelRasterRunList.get(ModelRasterState.RasterListType.INTER).getRun(rasterRunIndex);
            if (null != rrun) {
                Point4D center = rrun.grid().getCenter();
                if (rrun.getRunType() == ModelRasterRun.RunType.SONICC && !rrun.getSONICCBin())
                    center = rrun.getSONICCRefPos();
                rrun.centerGrid(center, null);
            }
        } catch (Exception e) { e.printStackTrace(); }

        run.dev().setRasterRun(rasterRunIndex, false);
        run.updateMeshParameters(rasterRunIndex, true);
    }
    public static void changeOutput(ModelCollectRun run, DevSFType.SPOTFINDER_OUTPUT selectedOutput)
    {
        if (null!=selectedOutput) {
            run.dev().setMeshParam(selectedOutput.ordinal(), false);
            setDefaultFilter(run, selectedOutput, true);
        }
    }
    private static void setDefaultFilter(ModelCollectRun collectRun, SPOTFINDER_OUTPUT output, boolean update)
    {
        ModelRasterRun rasterRun = (ModelRasterRun)ModelRasterRunList.get(ModelRasterState.RasterListType.INTER).getRun(collectRun.getRasterRun());
        Double newDefault = getDefaultFilterForParam(rasterRun.getDirectorySuffix(), rasterRun.getFileRoot(), rasterRun.getRunLabel(), output.toString());
        if (null!=newDefault) collectRun.dev().setMeshFilter(newDefault, false);
        if (update) collectRun.dev().sendUpdate();
    }
    public static void changeNumbering(ModelCollectRun run, int numberingIndex)
    {
        run.dev().setRasterNumbering(numberingIndex, true);
    }
    public static void changeCombineFlag(ModelCollectRun run, boolean combine)
    {
        run.dev().setMeshCombine(combine, true);
    }
    public static void changeFilter(ModelCollectRun run, double filter) {
        run.dev().setMeshFilter(filter, true);
        //updateDisplay();
    }

    //////////////////////////////////////////////// helpers ////////////////////////////////////////////////////////

    public static Double getMaxValueForParam(String rel_dir, String prefix, int runLabel, String output)
    {
        FloatBuffer data = DBImageStats.load(Redis.image_stats(rel_dir, prefix, runLabel, output));
        float max = DBImageStats.max(data,0);
        return (Double)(double)max;
    }
    public static Double getDefaultFilterForParam(String rel_dir, String prefix, int runLabel, String output)
    {
        Double maxVal = getMaxValueForParam(rel_dir, prefix, runLabel, output);
        if (null==maxVal) return null;
        return (double)Math.round(0.7 * maxVal);
    }
}
