package gov.anl.gmca.collect;

import gov.anl.gmca.collect.DBImportSites.ImportSite;
import gov.anl.gmca.collect.RunCalculator.Frame;
import gov.anl.gmca.collect.ui.TabCollect;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.DevRunList.RUN_LABEL_SETTING;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.Redis;
import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.DevXYZ;
import gov.anl.gmca.raster.*;
import gov.anl.gmca.raster.ModelRasterHits.Hit;
import gov.anl.gmca.raster.ModelRasterSiteList.Site;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.calc.HitFinder;
import gov.anl.gmca.raster.calc.RasterGeometry;
import gov.anl.gmca.sample.WdgCamera;
import gov.anl.gmca.spotfinder.DBImageStats;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Point4D;
import gov.anl.gmca.util.Util;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.eclipse.swt.SWT;

import java.io.FileInputStream;
import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.Collections;

public class ModelCollectRun implements ModelRunList.ModelRun
{
	//////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////// Listener //////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////
	
	public interface Listener
	{
		public void runChanged(ModelCollectRun run);
	}
	
	ArrayList<Listener> listenerList = new ArrayList<Listener>();
	public void addListener(Listener listener) { listenerList.add(listener); }
	public void removeListener(Listener listener) { while (listenerList.remove(listener)) {} }
	
	protected void notifyListeners()
	{
		runList.notifyCurrentRunListeners();
		for (Listener l: listenerList)
		{
			l.runChanged(this);
		}
	}

	//////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////// change methods ///////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////
	
	protected Thread changeThread = null;
	protected void handleChange()
	{
		if (!validateAndFix()) return;
		
		boolean sequenceChange = didSequenceVariablesChange();
		boolean parameterChange = didParameterVariablesChange();
		boolean collectionRunning = (null != runList.cmd && runList.cmd.isAlive());
		
		// detect modifications during collection
		//if (collectionRunning && parameterChange) System.out.println("Parameter changed while collect thread is running!");
		//if (collectionRunning && sequenceChange)  System.out.println("Sequence changed while collect thread is running!");

		// delay sequence recalc during collection
		if (sequenceChange)
		{
    		if (!collectionRunning)
    		{
    			sequence.recalc();
    		}
    		else
    		{
    			if (null == changeThread || !changeThread.isAlive())
    			{
        			changeThread = new Thread(new Runnable(){public void run()
        			{
            			while (null != runList.cmd && runList.cmd.isAlive())
            			{
            				Util.sleep(10);
               			}
        				
            			sequence.recalc();
            			notifyListeners();
        			}}, "Collect Run Change");
        			changeThread.start();
    			}
    		}
		}
		
		notifyListeners();
	}
	
	// sequence variables
	// note: the only variables that aren't sequence or parameter values are runState and nextFrame
	protected String lastPrefix, lastDirectory;
	protected double lastStartAngle, lastDelta, lastStartAngle2;
	protected ArrayList<Double> lastEnergies;
	protected ArrayList<Boolean> lastSelectedSites;
	protected int lastStartFrameLabel, lastEndFrameLabel, lastRunLabel;
	protected double lastWedgeSize;
	protected boolean lastInverseOn;
	protected Point3D lastStartPoint, lastEndPoint;
	protected boolean lastReverseDir;
	protected double lastEndAngle, lastStepSize;
	protected int lastFramesPerSite, lastOverlap;
	protected boolean lastStrategyEnabled;
	protected ScanMode lastScanMode;
	protected SetupStep lastSetupStep;
	protected int lastNextFrame;
	protected double lastMeshFilter;
	protected boolean lastMeshCombine;
	protected int lastRasterRun;
	protected SPOTFINDER_OUTPUT lastOutput;
	
	protected boolean didSequenceVariablesChange()
	{
		// TODO: is this necessary? it could return the wrong value
		if (null == getEnergies() || getEnergies().size()==0) return false;
		
		String prefix = getFileRoot();
		double startAngle = getStartAngle();
		double startAngle2 = getStartAngle2();
		boolean strategyEnabled = getStrategyEnabled();
		ArrayList<Double> energies = getEnergies();
		ArrayList<Boolean> selectedSites = getSelectedSites();
		int startFrameLabel = getStartFrameLabel();
		int endFrameLabel = getEndFrameLabel();
		int runLabel = getRunLabel();
		double delta = getDelta();
		double wedgeSize = getWedgeSize_deg();
		boolean inverseOn = getInverseOn();
		Point3D startPoint = getPt1();
		Point3D endPoint = getPt2();
		boolean reverseDir = getReverseDir();
		double endAngle = getEndAngle();
		double stepSize = getStepSize_mm();
		int framesPerSite = getFramesPerSite();
		int overlap = getOverlap();
		ScanMode scanMode = getScanMode();
		SetupStep setupStep = getSetupStep();
		int nextFrame = getNextFrame();
		double meshFilter = getMeshFilter();
		boolean meshCombine = getMeshCombine();
		int rasterRun = ModelRasterState.get().getCurrentRunModel().getRunIndex();
		SPOTFINDER_OUTPUT sf_output = ModelRasterState.get().getDiffDisplayParameter();
		
		boolean changed = false;
		String whatChanged = "";
		
		if (prefix 			!= lastPrefix)			{ changed = true; whatChanged += "prefix "; 			}
		if (startAngle 		!= lastStartAngle)		{ changed = true; whatChanged += "startAngle "; 		}
		if (startAngle2		!= lastStartAngle2)		{ changed = true; whatChanged += "startAngle2 "; 		}
		if (strategyEnabled != lastStrategyEnabled) { changed = true; whatChanged += "strategyEnabled "; 	}
		if (startFrameLabel	!= lastStartFrameLabel)	{ changed = true; whatChanged += "startFrameLabel "; 	}
		if (endFrameLabel	!= lastEndFrameLabel)	{ changed = true; whatChanged += "endFrameLabel "; 		}
		if (runLabel		!= lastRunLabel)		{ changed = true; whatChanged += "runLabel "; 			}
		if (delta			!= lastDelta)			{ changed = true; whatChanged += "delta "; 				}
		if (wedgeSize		!= lastWedgeSize)		{ changed = true; whatChanged += "wedgeSize "; 			}
		if (inverseOn		!= lastInverseOn)		{ changed = true; whatChanged += "inverseOn "; 			}
		if (startPoint		!= lastStartPoint)		{ changed = true; whatChanged += "startPoint ";			}
		if (endPoint		!= lastEndPoint)		{ changed = true; whatChanged += "endPoint ";			}
		if (reverseDir		!= lastReverseDir)		{ changed = true; whatChanged += "reverseDir ";			}
		if (endAngle		!= lastEndAngle)		{ changed = true; whatChanged += "endAngle ";			}
		if (stepSize		!= lastStepSize)		{ changed = true; whatChanged += "stepSize ";			}
		if (framesPerSite	!= lastFramesPerSite)	{ changed = true; whatChanged += "framesPerSite ";		}
		if (overlap 		!= lastOverlap)			{ changed = true; whatChanged += "overlap ";			}
		if (!doubleArrayEquals(energies,lastEnergies))			  { changed = true; whatChanged += "energies "; 			}
		if (!booleanArrayEquals(selectedSites,lastSelectedSites)) { changed = true; whatChanged += "selectedSites ";}
		if (scanMode		!= lastScanMode)		{ changed = true; whatChanged += "scanMode ";			}
		if (setupStep		!= lastSetupStep)		{ changed = true; whatChanged += "setupStep ";			}
		if (nextFrame		!= lastNextFrame)		{ changed = true; whatChanged += "nextFrame "; 			}
		if (meshFilter		!= lastMeshFilter)		{ changed = true; whatChanged += "meshFilter "; 			}
		if (meshCombine		!= lastMeshCombine)		{ changed = true; whatChanged += "meshCombine "; 			}
		if (sf_output		!= lastOutput)			{ changed = true; whatChanged += "sf_output"; }
		if (rasterRun		!= lastRasterRun)		{ changed = true; whatChanged += "rasterRun"; }

		//System.out.println("didSequenceVariablesChange() returning "+changed+" changed="+whatChanged);
		
		if (!changed) return false;

		lastPrefix 			= prefix;
		lastStartAngle 		= startAngle;
		lastStartAngle2 	= startAngle2;
		lastStrategyEnabled = strategyEnabled;
		lastStartFrameLabel	= startFrameLabel;
		lastEndFrameLabel	= endFrameLabel;
		lastRunLabel		= runLabel;
		lastDelta			= delta;
		lastWedgeSize		= wedgeSize;
		lastInverseOn		= inverseOn;
		lastStartPoint		= startPoint;
		lastEndPoint		= endPoint;
		lastReverseDir		= reverseDir;
		lastEndAngle		= endAngle;
		lastStepSize		= stepSize;
		lastFramesPerSite	= framesPerSite;
		lastOverlap			= overlap;
		lastEnergies 		= new ArrayList<Double>(energies);
		lastSelectedSites	= new ArrayList<Boolean>(selectedSites);
		lastScanMode		= scanMode;
		lastSetupStep		= setupStep;
		lastNextFrame		= nextFrame;
		lastMeshFilter		= meshFilter;
		lastMeshCombine		= meshCombine;
		lastOutput			= sf_output;
		lastRasterRun		= rasterRun;
		
		return true;
	}
	
	// parameter values
	double lastBeamstop, lastExposureTime, lastDistance, lastAttenuation;
	Axis lastAxis;
	protected boolean didParameterVariablesChange()
	{
		double beamstop = getBeamstop_mm();
		double expTime = getExposureTime();
		double distance = getdistance_mm();
		double atten = getAttenuation_factors();
		Axis axis = getAxis();
		
		boolean changed = false;
		String whatChanged = "";

		if (beamstop		!= lastBeamstop)		{ changed = true; whatChanged += "beamstop ";		}
		if (expTime			!= lastExposureTime)	{ changed = true; whatChanged += "exposureTime ";	}
		if (distance		!= lastDistance)		{ changed = true; whatChanged += "distance ";		}
		if (atten			!= lastAttenuation)		{ changed = true; whatChanged += "attenuation ";	}
		if (!axis			.equals(lastAxis))		{ changed = true; whatChanged += "axis ";			}
		
		//System.out.println("didParameterVariablesChange() returning "+changed+" changed="+whatChanged);
		
		if (!changed) return false;

		lastBeamstop		= beamstop;
		lastExposureTime	= expTime;
		lastDistance		= distance;
		lastAttenuation		= atten;
		lastAxis			= axis;
		
		return true;
	}
	
	protected boolean doubleArrayEquals(ArrayList<Double> a, ArrayList<Double> b)
	{
		//System.out.println("Comparing "+a+" to "+b);
		
		if (null==a && null==b) return true;
		if (null==a || null==b) return false;
		if (a.size()!=b.size()) return false;
		
		for (int i=0;i<a.size();i++)
		{
			if (!Util.nearlyEquals(a.get(i),b.get(i),4)) return false;
		}
		return true;
	}
	
	protected boolean booleanArrayEquals(ArrayList<Boolean> a, ArrayList<Boolean> b)
	{
		if (null==a && null==b) return true;
		if (null==a || null==b) return false;
		if (a.size()!=b.size()) return false;
		
		for (int i=0;i<a.size();i++)
		{
			if (a.get(i) != b.get(i)) return false;
		}
		return true;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////// run attributes ////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public 		enum 		RunState 						{ INACTIVE, COLLECTING, PAUSING, PAUSED, COMPLETE }
	protected 	RunState 	runState =						RunState.INACTIVE;
	public		RunState 	getRunState() 					{ return runState; }
	public 		void 		setRunState(RunState runState) 	{ this.runState=runState; }
	
	public 		enum 		SetupStep 						{ SITE_SETUP, COLLECT_PARAM };
	protected 	SetupStep 	setupStep =						SetupStep.COLLECT_PARAM;
	public		SetupStep 	getSetupStep() 					{ return setupStep; }
	public 		void 		setSetupStep(SetupStep ss) 		{ this.setupStep=ss; }
	
	protected	int			runLabel 						= 0;
	public		int			getRunLabel()					{ return runLabel;																}
	public 		void 		setRunLabel(int rl)				{ runLabel = rl;																}
	
	protected	String		fileRoot 						= "";
	public		String		getFileRoot()					{ return fileRoot;																}
	public 		void 		setFileRoot(String fr)			{ fileRoot = fr; 																}
	
	protected	String		directorySuffix 				= "";
	public		String		getDirectorySuffix()			{ return directorySuffix;														}
	public 		void 		setDirectorySuffix(String dir)	{ directorySuffix = dir; 														}
	public		String		getFullDirectory()
	{
		return DevDataDir.addToPrimary(directorySuffix);
	}
	
	protected	double		beamstop_mm						= Double.NaN;
	public		double		getBeamstop_mm()				{ return beamstop_mm;															}
	public 		void 		setBeamstop_mm(double d)		{ if (!Util.nearlyEquals(beamstop_mm,d,3)) { beamstop_mm= d; 					}}

	protected	double		attenuation_factors				= Double.NaN;
	public		double		getAttenuation_factors()		{ return attenuation_factors;													}
	public 		void 		setAttenuation_factors(double d){ if (!Util.nearlyEquals(attenuation_factors,d,3)) { attenuation_factors = d; 	}}
	
	public enum Axis 		{ GONIO_OMEGA, GONIO_PHI };
	protected	Axis		axis = Axis.GONIO_OMEGA;
	public		Axis		getAxis()						{ return axis; }
	public		void		setAxis(Axis axis)				{ this.axis=axis; }
	
	protected	double		delta 							= Double.NaN;
	public		double		getDelta() 						{ return delta; 																}
	public 		void 		setDelta(double d)				{ if (!Util.nearlyEquals(delta,d,3)) { delta = d; updateEndFrame(); 			}}

	protected	double		exposureTime 					= Double.NaN;
	public		double		getExposureTime() 				{ return exposureTime; 															}
	public 		void 		setExposureTime(double d)		{ if (!Util.nearlyEquals(exposureTime,d,3)) { exposureTime = d; 				}}

	protected	double		beamWidth_um 					= Double.NaN;
	public		double		getBeamWidth_um() 				{ return beamWidth_um; 															}
	public 		void 		setBeamWidth_um(double d)		{ beamWidth_um = d; }

	protected	double		beamHeight_um 					= Double.NaN;
	public		double		getBeamHeight_um() 				{ return beamHeight_um; 															}
	public 		void 		setBeamHeight_um(double d)		{ beamHeight_um = d; }

	protected	boolean		strategyEnabled					= false;
	public		boolean		getStrategyEnabled()			{ return strategyEnabled; }
	public		void		setStrategyEnabled(boolean b)	{ strategyEnabled = b; }

	protected	int			rasterRun						= 0;
	public		int			getRasterRun()					{ return rasterRun; }
	public		void		setRasterRun(int i)				{ rasterRun = i; }

	protected	double		startAngle	 					= Double.NaN;
	public		double		getStartAngle() 				{ return startAngle; 															}
	public 		void 		setStartAngle(double d)			{ if (!Util.nearlyEquals(startAngle,d,3)) { startAngle = d; updateEndFrame(); 	}}

	protected	double		startAngle2	 					= Double.NaN;
	public		double		getStartAngle2() 				{ return startAngle2; 															}
	public 		void 		setStartAngle2(double d)		{ if (!Util.nearlyEquals(startAngle2,d,3)) { startAngle2 = d; }}

	protected	double		endAngle	 					= Double.NaN;
	public		double		getEndAngle() 					{ return endAngle; 																}
	public 		void 		setEndAngle(double d)			{ if (!Util.nearlyEquals(endAngle,d,3)) { endAngle = d; updateEndFrame();  		}}

	// end label is not set, only calculated
	protected	int			startFrameLabel	 					= 1;
	protected	int			endFrameLabel						= 2;
	public		int			getStartFrameLabel() 				{ return startFrameLabel; 													}
	public		int			getEndFrameLabel()					{ return endFrameLabel;														}
	public 		void 		setStartFrameLabel(int i)			{ if (i!=startFrameLabel) { startFrameLabel = i; updateEndFrame(); 			}}

	public static class Point3D
	{
		public Point3D(double x, double y, double z) { this.x=x; this.y=y; this.z=z; }
		public Point3D(double[] xyz) { this.x=xyz[0]; this.y=xyz[1]; this.z=xyz[2]; }
		public double x,y,z;
		public Point3D plus(Point3D rhs) { return new Point3D(x+rhs.x,y+rhs.y,z+rhs.z); }
		public Point3D minus(Point3D rhs) { return new Point3D(x-rhs.x,y-rhs.y,z-rhs.z); }
		public Point3D times(double d) { return new Point3D(x*d,y*d,z*d); }
		public double[] toArray() { return new double[]{x,y,z}; }
	}

	// null is a valid value here, it is checked for, and when null collection will occur on a single point
	protected	Point3D		pt1							= null;
	public		Point3D		getPt1()					{ return pt1; }
	public		void		setPt1(Point3D p) 			{ pt1 = p; }
	
	protected	Point3D		pt2							= null;
	public		Point3D		getPt2()					{ return pt2; }
	public		void		setPt2(Point3D p) 			{ pt2 = p; }

	protected	Point3D		pt3							= null;
	public		Point3D		getPt3()					{ return pt3; }
	public		void		setPt3(Point3D p) 			{ pt3 = p; }

	protected	Point3D		pt4							= null;
	public		Point3D		getPt4()					{ return pt4; }
	public		void		setPt4(Point3D p) 			{ pt4 = p; }

	public Double getVectorLength_mm()
	{
		if (null==pt1 || null==pt2)
		{
			return null;
		}
		else
		{
			double dx = pt2.x-pt1.x;
			double dy = pt2.y-pt1.y;
			double dz = pt2.z-pt1.z;
			double dist = Math.sqrt(dx*dx+dy*dy+dz*dz);
			return dist;
		}
	}
	
	protected	int			potentialVectorSites			= 0;
	public		int			getPotentialVectorSites()		{ return potentialVectorSites;	}
	public 		void 		setPotentialVectorSites(int vs)	{ potentialVectorSites = vs; 	}
	
	protected	int			importSiteCount					= 0;
	public		int			getImportSiteCount()			{ return importSiteCount;	}
	public 		void 		setImportSiteCount(int i)		{ importSiteCount = i; 	}
	
	protected	ListenerList<ModelCollectRun>		 		llNextFrame;
	protected	int			nextFrame 						= 0;
	public		int			getNextFrame()					{ return nextFrame;							}
	public 		void 		setNextFrame(int nf)			{ nextFrame = nf; llNextFrame.notifyListeners(); runList.llCurrentRunFrame.notifyListeners(); }

	protected	double		meshFilter	 					= 0;
	public		double		getMeshFilter()					{ return meshFilter;						}
	public 		void 		setMeshFilter(double d)			{ meshFilter = d; 							}

	protected	int			meshParam	 					= 3;
	public		int			getMeshParam()					{ return meshParam;							}
	public 		void 		setMeshParam(int i)				{ meshParam = i; 							}

	protected	int			meshSpotFinder 					= 1;
	public		int			getMeshSpotFinder()				{ return meshSpotFinder;					}
	public 		void 		setMeshSpotFinder(int i)		{ meshSpotFinder = i; 						}

	protected	int			rasterNumbering					= 1;
	public		int			getRasterNumbering()			{ return rasterNumbering;					}
	public 		void 		setRasterNumbering(int i)		{ rasterNumbering = i; 						}

	protected	boolean		meshCombine	 					= false;
	public		boolean		getMeshCombine()				{ return meshCombine;						}
	public 		void 		setMeshCombine(boolean b)		{ meshCombine = b; 							}

	protected	boolean		reverseDir 						= false;
	public		boolean		getReverseDir()					{ return reverseDir;						}
	public 		void 		setReverseDir(boolean rd)		{ reverseDir = rd; 							}

	protected	double		stepSize_mm						= Double.NaN;
	public		double		getStepSize_mm()				{ return stepSize_mm; 						}
	public		void		setStepSize_mm(double d)		{ stepSize_mm=d;  							}
	
	protected	int			framesPerSite 					= 0;
	public		int			getFramesPerSite()				{ return framesPerSite;						}
	public 		void 		setFramesPerSite(int i)			{ framesPerSite = i;						}

	protected	int			framesPerDir 					= 10000;
	public		int			getFramesPerDir()				{ return framesPerDir;					}
	public 		void 		setFramesPerDir(int i)			{ framesPerDir = i;						}

	protected	ArrayList<Boolean>	selectedSites			= new ArrayList<Boolean>(DevCollectRun.MAX_SITES);
	public		ArrayList<Boolean>	getSelectedSites()		{ return new ArrayList<Boolean>(selectedSites); }
	
	public boolean isSiteSelected(int number)
	{
		try
		{
			return selectedSites.get(number-1);
		}
		catch (IndexOutOfBoundsException e) { return false; }
	}
	
	public		void setSelectedSites(ArrayList<Boolean> al)
	{
		if (al.size()!=selectedSites.size()) seq().versionNumber++;
		else
		{
			for (int i=0;i<al.size();i++)
			{
				if (al.get(i)!=selectedSites.get(i)) { seq().versionNumber++; break; }
			}
		}
		selectedSites = al;
	}
	
	public ArrayList<ImportSite> getSelectedImportSiteObjects()
	{
		if (getScanMode() != ScanMode.IMPORT) return new ArrayList<ImportSite>();
		
		ArrayList<Boolean> 	    selectedSitesCopy 	= new ArrayList<Boolean>(selectedSites);
		ArrayList<ImportSite>   allSitesCopy	 	= DBImportSites.getAllImportSites();
		ArrayList<ImportSite> 	ret 				= new ArrayList<ImportSite>();
		
		for (ImportSite site : allSitesCopy)
		{
			if (site.index >= selectedSitesCopy.size()) break;
			if (selectedSitesCopy.get(site.index)) ret.add(site);
		}
		
		return ret;
	}
	
	public ArrayList<Site> getSelectedRasterSiteObjects()
	{
		if (getScanMode() != ScanMode.RASTER) return new ArrayList<Site>();
		
		ArrayList<Boolean> 	selectedSitesCopy 	= new ArrayList<Boolean>(selectedSites);
		ArrayList<Site> 	allSitesCopy 		= ModelRasterSiteList.get().getSiteListCopy();
		ArrayList<Site> 	ret 				= new ArrayList<Site>();
		
		for (Site site : allSitesCopy)
		{
			int siteIndex = site.index-1;
			if (siteIndex >= selectedSitesCopy.size()) break;
			if (selectedSitesCopy.get(siteIndex)) ret.add(site);
		}
		
		return ret;
	}
	
	protected	int			overlap							= 0;
	public		int			getOverlap()					{ return overlap; 							}
	public		void		setOverlap(int i)				{ overlap=i;								}
	
	protected	double		siteDelta_deg					= 0;
	public		double		getSiteDelta_deg()				{ return siteDelta_deg; 					}
	public		void		setSiteDelta_deg(double d)		{ siteDelta_deg=d; 							}
	
	protected	int			siteSkip						= 0;
	public		int			getSiteSkip()					{ return siteSkip; 					}
	public		void		setSiteSkip(int i)				{ siteSkip=i; 						}
	
	protected	boolean		siteCheck						= false;
	public		boolean		getSiteCheckEnable()			{ return siteCheck; 				}
	public		void		setSiteCheckEnable(boolean b)	{ siteCheck=b; 						}
	
	protected	int			savedFPS						= 0;
	public		int			getSavedFPS()					{ return savedFPS; 	}
	public		void		setSavedFPS(int i)				{ savedFPS=i;		}
	
	protected	int			savedOverlap					= 0;
	public		int			getSavedOverlap()				{ return savedOverlap;	}
	public		void		setSavedOverlap(int i)			{ savedOverlap=i;		}
	
	protected	double		savedEndAngle					= 0.0;
	public		double		getSavedEndAngle()				{ return savedEndAngle;	}
	public		void		setSavedEndAngle(double d)		{ savedEndAngle=d;		}
	
	protected	boolean		savedInverseBeam				= false;
	public		boolean		getSavedInverseBeam()			{ return savedInverseBeam;	}
	public		void		setSavedInverseBeam(boolean b)	{ savedInverseBeam=b;		}
	
	public		static enum	XDSProcMode
	{
		NONE, NATIVE, ANOMALOUS;
		public static XDSProcMode fromString(String s)
		{
			for (XDSProcMode mode : XDSProcMode.values())
			{
				if (mode.toString().toLowerCase().equals(s.toLowerCase())) return mode;
			}
			throw new IllegalArgumentException("Invalid XDSProcMode "+s);
		}
	};
	protected	XDSProcMode	xdsProcMode 					= XDSProcMode.NONE;
	public		XDSProcMode	getXDSProcMode()				{ return xdsProcMode;		}
	public 		void 		setXDSProcMode(XDSProcMode m)	{ xdsProcMode = m;			}
	
	protected	boolean		helicalMode 					= false;
	public		boolean		getHelicalMode()				{ return helicalMode;		}
	public 		void 		setHelicalMode(boolean hm)		{ helicalMode = hm;			}
	
	protected	boolean		inverseOn 						= false;
	public		boolean		getInverseOn()					{ return inverseOn;							}
	public 		void 		setInverseOn(boolean rd)		{ inverseOn = rd;							}

	protected	double		wedgeSize_deg					= Double.NaN;
	public		double		getWedgeSize_deg()				{ return wedgeSize_deg; 					}
	public		int			getWedgeSize_sites()
	{
		double fps = getSiteCheckEnable() ? getSavedFPS() : framesPerSite;
		return (int)Math.round(wedgeSize_deg/(fps*delta));
	}
	public		void		setWedgeSize_deg(double d)		{ wedgeSize_deg=d; 							}
	
	protected	double		distance_mm						= Double.NaN;
	public		double		getdistance_mm()				{ return distance_mm; 						}
	public		void		setdistance_mm(double d)		{ distance_mm=d; 							}

	protected	ArrayList<Double>	energies 				= new ArrayList<Double>();
	public		ArrayList<Double>	getEnergies()			{ return energies; }
	public		void				setEnergies(ArrayList<Double> e) { energies=e; }

	public enum OptionTab	{ COLLECT_PARAM, SITE_SETUP };
	protected	OptionTab	optionTab = OptionTab.COLLECT_PARAM;
	public		OptionTab	getOptionTab()					{ return optionTab; }
	public		void		setOptionTab(OptionTab ot)		{ optionTab = ot; }
	
	public enum ScanMode 	{ SINGLE, VECTOR, RASTER, IMPORT, MESH, JET };
	protected	ScanMode	scanMode = ScanMode.SINGLE;
	public		ScanMode	getScanMode()					{ return scanMode; }
	public		void		setScanMode(ScanMode m)			{ scanMode = m; }
	
	protected	String		importPath						= "";
	public		String		getImportPath()					{ return importPath; }
	public		void		setImportPath(String s)			{ importPath = s; }
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////// end attributes //////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public ArrayList<Point4D> getVectorSiteMotorPositions()
	{
		return new ArrayList<Point4D>();
	}
	
	// returns false if this run is invalid
	protected boolean lastValidateFailed = false;
	protected boolean validateAndFix()
	{
		if (lastValidateFailed)
		{
			lastValidateFailed = false;
			return true;
		}

		boolean isValid = true;

		if (runLabel != 0 && runIndex == 0)
		{
			runLabel = 0;
			dev().setRunLabel(0, false);
			isValid = false;
		}
		
		if (fileRoot.length()==0)
		{
			dev().setFileroot("test", false);
			isValid = false;
		}
		
		////////////////////////////////////////////////////////////////////////////////////////////////////
		///////////////////////////////////////// standard parameters //////////////////////////////////////
		////////////////////////////////////////////////////////////////////////////////////////////////////
		
		if (beamWidth_um < 1.0)
		{
			beamWidth_um = 20.0;
			dev().setBeamWidth_um(20.0, false);
			isValid = false;
		}
		
		if (beamHeight_um < 1.0)
		{
			beamHeight_um = 20.0;
			dev().setBeamHeight_um(20.0, false);
			isValid = false;
		}
		
		if (delta < 0.00001)
		{
			delta = 0.00001;
			dev().setDelta(0.00001, false);
			isValid = false;
		}

		// if there's less than one frame, or for run zero if there's more than one frame,
		// set endAngle for exactly one frame
		double angleDifference = endAngle - startAngle;
		if
		(
			angleDifference < delta - 0.001
			||
			(
				0 == runIndex &&
				Math.abs(angleDifference-delta) > .001
			)
		)
		{
			endAngle = startAngle+delta;
			dev().setEndAngle(startAngle+delta, false);
			isValid = false;
		}

		if (Math.abs( (angleDifference / delta) - Math.round(angleDifference / delta) ) > .0001)
		{
			endAngle = startAngle + (Math.round(angleDifference / delta) * delta);
			dev().setEndAngle(endAngle, false);
			isValid = false;
		}
		
		if (startFrameLabel < 1)
		{
			startFrameLabel = 1;
			dev().setStartFrameLabel(1, false);
			isValid = false;
		}
		
		updateEndFrame();
		int totalFrames = (endFrameLabel - startFrameLabel + 1);
		
		////////////////////////////////////////////////////////////////////////////////////////////////////
		/////////////////////////////////////////// vector parameters //////////////////////////////////////
		////////////////////////////////////////////////////////////////////////////////////////////////////
		
		if (stepSize_mm < 0.00001)
		{
			stepSize_mm = 0.00001;
			dev().setStepSize_mm(0.00001, false);
			isValid = false;
		}
		
		if (framesPerSite < 1)
		{
			framesPerSite = 1;
			dev().setFramesPerSite(1, false);
			isValid = false;
		}
		
		if (totalFrames>=1 && framesPerSite > totalFrames)
		{
			framesPerSite = totalFrames;
			dev().setFramesPerSite(totalFrames, false);
			isValid = false;
		}
		
		/*int correctNumberOfSites = (endFrameLabel-startFrameLabel+1)/;
		if (numberOfSites != correctNumberOfSites)
		{
			numberOfSites = correctNumberOfSites;
			dev().setNumberOfSites(correctNumberOfSites, false);
			isValid = false;
		}*/
	
		if (!isValid)
		{
			dev().sendUpdate();
		}
		
		lastValidateFailed = !isValid;
		return isValid;
	}
	
	protected void updateEndFrame()
	{
		endFrameLabel = (int)( (getEndAngle() - getStartAngle()) / getDelta() - 0.01 + getStartFrameLabel() );
	}
	
	public static class SequenceVariables
	{
		public Double 	startAngle	= null;
		public Double 	endAngle	= null;
		public Double 	delta		= null;
		public Integer 	startFrame	= null;
		public Integer 	endFrame	= null;
		public Double	wedgeSize	= null;
	}
	
	public int getSelectedSiteCount()
	{
		if (ScanMode.SINGLE == scanMode) return 1;
		
		int count = 0;
		for (Boolean b : selectedSites)
		{
			if (b) count++;
		}
		return count;
	}
	
	// just calculate based on other fields
	protected double calcEndAngle(int startFrame, int endFrame, double delta, double startAngle)
	{
		return (endFrame - startFrame + 1) * delta + startAngle;
	}
	
	// correct for end angle limits
	protected double getAllowedEndAngle(int startFrame, int endFrame, double delta, double startAngle)
	{
		Limits omegaLimits = DevPmacMotor.get("gonio_omega").getLimits();
		int newEndFrame = endFrame;
		
		double newEndAngle = calcEndAngle(startFrame,newEndFrame,delta,startAngle);
		while (newEndAngle > omegaLimits.getUpper())
		{
			newEndFrame--;
			if (newEndFrame==startFrame) break;
			newEndAngle = calcEndAngle(startFrame,newEndFrame,delta,startAngle);
		}
		return newEndAngle;
	}
	
	void selectAllVectorSites(boolean update)
	{
		ArrayList<Boolean> newSelectedSites = new ArrayList<Boolean>();
		for (int i=0; i<getPotentialVectorSites(); i++) newSelectedSites.add(true);
		dev().setSelectedSites(newSelectedSites, update);
	}

	void selectAllImportSites(Integer siteCount, boolean update)
	{
		if (null!=siteCount)
		{
			dev().setImportSiteCount(siteCount, false);
		}
		else
		{
			siteCount = getImportSiteCount();
		}
		
		ArrayList<Boolean> newSelectedSites = new ArrayList<Boolean>();
		for (int i=0; i<siteCount; i++) newSelectedSites.add(true);
		dev().setSelectedSites(newSelectedSites, update);
	}

	// either step size or site count can be kept
	public void fitSitesToVector(boolean keepStepSize, boolean useDirectSiteCount, boolean update)
	{
		Double length = getVectorLength_mm();
		if (null == length) return;

		double step;
		int siteCount;
		
		if (keepStepSize)
		{
			// for a given step size:
			step = getStepSize_mm();
			siteCount = (int)Math.floor(length/step)+1; // (0 length vector still needs 1 site)
		}
		else
		{
			int frameCount = RunCalculator.calcFrameCount(delta, startAngle, endAngle);
			
			// to fit all frames
			if (useDirectSiteCount)
				siteCount = potentialVectorSites;
			else
				siteCount = (int)Math.ceil((double)frameCount/(double)framesPerSite);
			
			step = (siteCount==1)?length:length/(siteCount-1);
		}
		
		dev().setStepSize_mm(step, false);
		dev().setPotentialVectorSites(siteCount, update);
	}
	
	void setVectorCollectParamDefaults(boolean update)
	{
		// copy variables so they are consistent during calculations
		double 	_delta 				= getDelta();
		int 	_NOS 				= getSelectedSiteCount();
		double	_startAngle			= startAngle;
		double	_endAngle			= endAngle;
		int		_frameCount			= RunCalculator.calcFrameCount(_delta, _startAngle, _endAngle);
		double	newEndAngle			= _endAngle;

		// fit frame count to sites by changing FPS
		int 	newFPS 				= (int)Math.ceil((double)_frameCount / _NOS);
		
		// increase frame count if needed so there is at least one frame per site
		if (_NOS > _frameCount)
		{
			newEndAngle = _startAngle + _NOS*_delta;
			newFPS = 1;
		}
		
		// set wedge to ~5 degrees
		double targetWedgeDeg = 5.0;
		int newWedgeSites = (int)Math.round(targetWedgeDeg / (newFPS*_delta));
		if (newWedgeSites>_NOS) newWedgeSites=_NOS; if (newWedgeSites<1) newWedgeSites=1;
		double newWedgeDeg = newWedgeSites*newFPS*_delta;

		// send to device
		dev().setEndAngle			(newEndAngle, false);
		dev().setFramesPerSite		(newFPS, false);
		dev().setWedgeSize			(newWedgeDeg, false);
		if (update) dev().sendUpdate();
	}
	
	void setRasterDefaults(boolean update)
	{
		setVectorCollectParamDefaults(update); // setup is identical for now
	}
	
	public static RunState getCorrectState(int currentFrame, int totalFrames)
	{
		RunState state = RunState.INACTIVE;
		if (currentFrame == totalFrames)	state = RunState.COMPLETE;
		else if (currentFrame>0)			state = RunState.PAUSED;
		return state;
	}
	
	public Point3D getUnitVector()
	{
		Point3D _startPoint = getReverseDir() ? getPt2() : getPt1();
		Point3D _endPoint = getReverseDir() ? getPt1() : getPt2();
		return getUnitVector(_startPoint, _endPoint);
	}
	
	public Point3D getUnitVector(Point3D startPoint, Point3D endPoint)
	{
		// calculate the unit vector
		double xd = (endPoint.x - startPoint.x);
        double yd = (endPoint.y - startPoint.y);
        double zd = (endPoint.z - startPoint.z);
        
        double length = Math.sqrt(xd*xd + yd*yd + zd*zd);
        double unitX = 0.0, unitY = 0.0, unitZ = 0.0;
        
        if (length > 0.0000001)
        {
        	unitX = xd/length;
        	unitY = yd/length;
        	unitZ = zd/length;
        }
        
        return new Point3D(unitX,unitY,unitZ);
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////// constructor ////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	protected RunCalculator sequence;
	public RunCalculator seq() { return sequence; }
	public ModelCollectRunList runList;
	protected int runIndex;
	public int getRunIndex() { return runIndex; }
	public DevCollectRun dev() { return (DevCollectRun)runList.devRunList.getRunDev(runIndex); }
	protected ClientListener clientListener;
	public FieldInteractionDependent fid;
	public FieldInteractionIndependent fii;
	
	public ModelCollectRun(ModelCollectRunList _runList, int _runIndex)
	{
		runList=_runList;
		runIndex=_runIndex;
		sequence = new RunCalculator(this);
		fid = new FieldInteractionDependent(this);
		fii = new FieldInteractionIndependent(this);
		llNextFrame = new ListenerList<ModelCollectRun>(this);
		
		clientListener = new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				if (isMaster &&
					DevFPE.get().isIdle() &&
					(RunState.COLLECTING == getRunState() || RunState.PAUSING == getRunState()) &&
					(null == runList.cmd || !runList.cmd.isAlive()))
				{
					RunState state = RunState.INACTIVE;
					
					try
					{
						ArrayList<Frame> frameList = seq().getFrameListCopy();
						if (null != seq() && null != frameList)
    					{
    						state = getCorrectState(nextFrame, frameList.size());
    					}
					} catch (Exception e) { e.printStackTrace(); }
					
					dev().setRunStatus(state, true);
				}
			}
		};
		Client.addListener(clientListener);
	}
	
	public void dispose()
	{
		Client.removeListener(clientListener);
	}
	
	@Override public void notifyStarTabClick() {}
	@Override public void notifyExistingRunClick() {}
	@Override public void notifySyncEnabled() {}
	
	public void copyRun()
	{
		// TODO: this should be moved somewhere else
		WdgRunTabs runTabs = TabCollect.get().getRunTabFolder();
		runTabs.remoteOpInProgress = true;
		runTabs.notifyListeners();
		
		int newRunCount = runList.getRunCount()+1;
		dev().copyRunContents(newRunCount, RUN_LABEL_SETTING.INCREMENT_LAST, false, false);
		DevCollectRun.setNextFrame(newRunCount, 0);
		DevCollectRun.setRunState(newRunCount, RunState.INACTIVE);
		runList.devRunList.setRunCount(newRunCount, false);
		runList.devRunList.setCurrentRun(newRunCount, true);
	}
	
	public void moveToVectorSite(int siteNumber)
	{
		Log.note("Moving to vector site "+siteNumber);
		
		int siteIndex = siteNumber-1;
		
		// calc motor positions
		Point3D siteMotorPos = new Point3D(pt1.x,pt1.y,pt1.z);
		Point3D unitVector = getUnitVector(pt1,pt2);
		siteMotorPos.x += siteIndex * getStepSize_mm() * unitVector.x;
		siteMotorPos.y += siteIndex * getStepSize_mm() * unitVector.y;
		siteMotorPos.z += siteIndex * getStepSize_mm() * unitVector.z;
		
		DevXYZ.syncMove(siteMotorPos, false);
	}
	
	public void moveToRasterSite(Site site)
	{
		DevXYZ.syncMove(site.motorPos.to3D(), false);
	}
	
	public void addSiteFromCurrentCenter()
	{
		Point4D currentMotorPos = RasterGeometry.getCurrentMotorPositions();
		int siteNumber = ModelRasterSiteList.get().getHighestSiteNumber()+1;
		ModelRasterSiteList.Site site = new ModelRasterSiteList.Site(siteNumber, -1, -1, -1, -1, -1, currentMotorPos, new Point3D(1,1,1));
		DBRasterSiteList.get().addSite(site, true);
	}
	
	public void importSites()
	{
		// ask for which file to import
		NBFileDialog dialog = new NBFileDialog(GUI.getShell(), SWT.OPEN);
		dialog.setFilterPath(System.getProperty("user.home"));
		dialog.setFilterExtensions(new String[] { "*.xlsx", "*.xls", "*.*" });
		String selectedPath = dialog.openStr();
		if (null==selectedPath) return;
		
		FileInputStream inp = null;
		try
		{
			// load the first worksheet
			inp = new FileInputStream(selectedPath);
			Workbook wbs = WorkbookFactory.create(inp);
			Sheet sheet = wbs.getSheetAt(0);
			if (null==sheet) return;
			
			// parse
			int siteNumber = ModelRasterSiteList.get().getHighestSiteNumber()+1;
			for (int rowIndex=1; rowIndex<10000; rowIndex++)
			{
				try
				{
					Row row = sheet.getRow(rowIndex);
					String siteLabel = row.getCell(0).toString();
					if (null==siteLabel || siteLabel.length()==0) break;
					double x_um = row.getCell(1).getNumericCellValue();
					double y_um = row.getCell(2).getNumericCellValue();
					
					Point4D siteMotorPos = WdgCamera.addXYZToMotorMM(RasterGeometry.getCurrentMotorPositions(), x_um, y_um, 0.0);
					ModelRasterSiteList.Site site = new ModelRasterSiteList.Site(siteNumber, -1, -1, -1, -1, -1, siteMotorPos, new Point3D(1,1,1));
					DBRasterSiteList.get().addSite(site, false);
				}
				catch (Exception e) { e.printStackTrace(); }
				siteNumber++;
			}
			DBRasterSiteList.get().updateAllRows();
		}
		catch (Exception e) { e.printStackTrace(); }
		finally
		{
			try { inp.close(); } catch (Exception e) {}
		}
	}
	public void calculateCrystalPositions()
	{
		// call David's script to calculate crystal positions
		
		ArrayList<ImportSite> siteList = new ArrayList<>();
		
		for (int i=0; i<5; i++)
		{
			ImportSite site = new ImportSite();
			site.collectRun = runIndex;
			site.index = i;
			Point4D currentPos = RasterGeometry.getCurrentMotorPositions();
			site.xyz = currentPos.to3D();
			site.xyz.z += -0.02*i;
			siteList.add(site);
		}
		DBImportSites.setSitesForRun(runIndex, siteList);
		
		selectAllImportSites(5, true);
	}

	public ArrayList<ModelRasterCell> getFilteredCells()
	{
		ModelRasterRun currentRasterRun = (ModelRasterRun)ModelRasterRunList.get(RasterListType.INTER).getRun(getRasterRun());
		ArrayList<ModelRasterCell> ret = new ArrayList<>();
		SPOTFINDER_OUTPUT selectedOutput = SPOTFINDER_OUTPUT.values()[getMeshParam()];
		FloatBuffer data = DBImageStats.load(Redis.image_stats(currentRasterRun.getDirectorySuffix(), currentRasterRun.getFileRoot(), currentRasterRun.getRunLabel(), selectedOutput.toString()));
		for (int frameIndex=0; frameIndex<data.capacity(); frameIndex++) {
			if (meshFilter <= data.get(frameIndex)) {
				ModelRasterCell cell = currentRasterRun.grid().getCellCopy(frameIndex);
				ret.add(cell);
			}
		}
		return ret;
	}

	public ArrayList<Hit> getHits()
	{
		ArrayList<Hit> ret = new ArrayList<>();

		ModelRasterRunList runList = ModelRasterRunList.get(RasterListType.INTER);
		if (null==runList) return ret;
		ModelRasterRun currentRasterRun = (ModelRasterRun)runList.getRun(getRasterRun());
		if (currentRasterRun.getRunState() != ModelRasterRun.RunState.PAUSED &&
			currentRasterRun.getRunState() != ModelRasterRun.RunState.COMPLETE)
			return ret;
		if (currentRasterRun.grid().getCenter()==null) return ret;

		HitFinder finder = new HitFinder(currentRasterRun.grid(), meshFilter, false);
		SPOTFINDER_OUTPUT selectedOutput = SPOTFINDER_OUTPUT.values()[getMeshParam()];
		String outputStr = currentRasterRun.getRunType()== ModelRasterRun.RunType.DIFFRACTION ? selectedOutput.toString() : "sonicc";
		finder.find(true, getMeshCombine(), outputStr);
		for (ArrayList<Integer> hitCells : finder.hits) {
			Hit hit = new Hit(ret.size()+1, currentRasterRun.getRunIndex(), -1, hitCells, selectedOutput);
			ret.add(hit);
		}
		Collections.sort(ret, Collections.reverseOrder());
		// renumber after sort
		for (int i=0;i<ret.size();i++) ret.get(i).hitNumber=i+1;
		return ret;
	}

	//
	// update raster run, mesh filter value, and collimator size to match the current raster run
	//
	public void updateMeshParameters(int referenceRasterRun, boolean sendUpdate)
	{
		if (!Client.isMaster()) {
			System.out.println("Client is not active, skipping mesh update");
			return;
		}

		if (getRunState()!=RunState.INACTIVE) {
			System.out.println("Run is not inactive, skipping mesh update");
			return;
		}

		ModelRasterRun rrun = referenceRasterRun==-1?ModelRasterState.get().getCurrentRunModel():(ModelRasterRun)ModelRasterRunList.get(RasterListType.INTER).getRun(referenceRasterRun);
		String output = SPOTFINDER_OUTPUT.values()[getMeshParam()].toString();
		if (rrun.getRunType()== ModelRasterRun.RunType.SONICC) output = "sonicc";

		int currentRRunIndex = rrun.getRunIndex();
		Double meshFilter = MeshParams.getDefaultFilterForParam(rrun.getDirectorySuffix(), rrun.getFileRoot(), rrun.getRunLabel(), output);
		if (null==meshFilter) meshFilter=0.0;
		int colli_um = (int)Math.round(rrun.getBeamWidth_um());
		dev().setRasterRun(currentRRunIndex, false);
		dev().setMeshFilter(meshFilter, false);
		dev().setBeamWidth_um(colli_um, false);
		dev().setBeamHeight_um(colli_um, false);

		Point4D gridCenter = rrun.grid().getCenter();
		if (null != gridCenter) {
			double rasterAngle = gridCenter.omega;
			double totalAngle = getEndAngle() - getStartAngle();
			double newStartAngle = rasterAngle - totalAngle / 2.0;
			double newEndAngle = rasterAngle + totalAngle / 2.0;
			dev().setStartAngle(newStartAngle, false);
			dev().setEndAngle(newEndAngle, false);
		}

		if (sendUpdate) dev().sendUpdate();
	}
}
