package gov.anl.gmca.collect;

import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.collect.ModelCollectRun.RunState;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.collect.ui.TabCollect;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevRunList.RUN_LABEL_SETTING;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.raster.ModelRasterSiteList;
import gov.anl.gmca.raster.ModelRasterSiteList.Site;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.sample.DevCollimatorCtrl;
import gov.anl.gmca.sample.DevSlider;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;

public class ModelCollectRunList extends ModelRunList
{
	/////////////////////////////////////////////////////// Variables ///////////////////////////////////////////
	
	public final static int MAX_COUNT = 16;

	// runCount=5 means runs zero through five are active
	// currentRun=5 means the sixth active run is current
	// getRun(5) returns the sixth run
	//
				protected	int				currentRun				= 0;
				public 		int 			getCurrentRunIndex()	{ return currentRun; }
	@Override	public		ModelCollectRun	getCurrentRunModel()	{ synchronized(runs) { if (currentRun>=runs.size()) return null; return runs.get(currentRun); } }

	protected 	ArrayList<ModelCollectRun> 	runs 								= new ArrayList<ModelCollectRun>();
	public 		int 						getRunCount() 						{ synchronized(runs) { return runs.size()-1; } }
	public 		ModelRun 					getRun(int runIndex) 				{ synchronized(runs) { return (ModelRun)runs.get(runIndex); } }
	public		ArrayList<ModelCollectRun>	getAllRunsCopy()
	{
		synchronized(runs) { return new ArrayList<ModelCollectRun>(runs); }
	}
	
	public 		enum 						CollectState 						{ INACTIVE, COLLECTING, PAUSING };
	protected 	CollectState 				collectState 						= CollectState.INACTIVE;
	public 		CollectState 				getCollectState() 					{ return collectState; 			}
	public		void						setCollectState(CollectState cs) 	{ collectState = cs; 			}

	protected 	BinMode 					binMode 							= BinMode.UM_72;
	public 		BinMode 					getBinMode() 						{ return binMode; 			}
	public		void						setBinMode(BinMode mode)			{ binMode = mode; 			}
	
	public		boolean						showVectorNumbers					= true;
	public 		boolean 					getShowVectorNumbers()				{ return showVectorNumbers; 	}
	public		void						setShowVectorNumbers(boolean b)		{ showVectorNumbers = b;		}
	
	public		static enum					CollectLeftTab						{ DIFFRACTION, VECTOR, STRATEGY; }
	protected	CollectLeftTab				leftTab								= CollectLeftTab.DIFFRACTION;
	public 		void 						setLeftTab(CollectLeftTab leftTab)	{ this.leftTab = leftTab; }
	public		CollectLeftTab				getLeftTab()						{ return leftTab; }
	
	protected CmdCollectSequence cmd = null;
	public boolean isCollectThreadAlive()
	{
		try {
			if (null!=cmd&&cmd.isAlive() && cmd.isRun0Done) {
				System.out.println("collect is alive but run 0 is done");
			}
			return (null != cmd && cmd.isAlive() && !cmd.isRun0Done);
		} catch (Exception e) {
			return false;
		}
	}
	DevCollectRunList devRunList;
	protected static ModelCollectRunList inst = null;
	public static ModelCollectRunList get() { return inst; }

	public ListenerList<ModelCollectRunList> llLeftTab;
	public ListenerList<ModelCollectRunList> llCurrentRunFrame;
	
	///////////////////////////////////////////////////////// Methods ////////////////////////////////////////////

	// when the run changes, notify frame listeners
	private int lastRunIndex = -1;
	@Override public void notifyCurrentRunListeners()
	{
		super.notifyCurrentRunListeners();
		
		int runIndex = getCurrentRunIndex();
		if (runIndex!=lastRunIndex)
		{
			lastRunIndex=runIndex;
			llCurrentRunFrame.notifyListeners();
		}
	}
	
	public ModelCollectRunList()
	{
		inst = this;
		llLeftTab = new ListenerList<ModelCollectRunList>(this);
		llCurrentRunFrame = new ListenerList<ModelCollectRunList>(this);
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				if (isMaster && CollectState.INACTIVE != getCollectState())
				{
					if (becomingMasterOnStartup)
					{
						// reset the flag here but don't call the handle method this time
						becomingMasterOnStartup = false;
					}
					else
					{
						handleCollectInProgress(true);
					}
				}
				else if (!isMaster && CollectState.INACTIVE != getCollectState())
				{
					handleBecomePassiveDuringCollect();
				}
			}
		});
	}

	public void setCurrentRun(int currentRun)
	{
		if (currentRun!=this.currentRun)
		{
			this.currentRun = currentRun;

			/*if (runs.size() > currentRun)
			{
				notifyCurrentRunListeners();
			}*/
		}
	}
	
	public void setRunCount(int newRunCount)
	{
		if (getRunCount()==newRunCount) return;
		
		if (!DBConfig.serverMode && null == TabCollect.get())
		{
			System.out.println("WARNING: returning on null collect tab");
			return;
		}

		// remove runs
		for (int runIndex = getRunCount(); runIndex > newRunCount; runIndex--)
		{
			// delete the model
			//runs.get(runIndex).dispose();
			runs.remove(runIndex);
			
			// delete the device
			devRunList.runDevs.get(runIndex).stopEpicsMonitors();
			devRunList.runDevs.remove(runIndex);
		}
		
		// add runs
		ArrayList<DevCollectRun> newDevices = new ArrayList<DevCollectRun>();
		for (int runIndex = getRunCount()+1; runIndex <= newRunCount; runIndex++)
		{
			// add the model
			ModelCollectRun run = new ModelCollectRun(this,runIndex);
			runs.add(run);
			
			// add the device
			DevCollectRun devRun = new DevCollectRun(runIndex, run);
			newDevices.add(devRun);
			devRunList.runDevs.add(devRun);
		}
		
		// update the gui
		if (!DBConfig.serverMode)
		{
			final int runCount = getRunCount();
			final int currentRunIndex = getCurrentRunIndex();

			Util.safeSyncExec(new Runnable() {
				public void run() {
					WdgRunTabs runTabs = TabCollect.get().getRunTabFolder();
					runTabs.configureTabs(runCount, currentRunIndex);
				}
			});
		}

		// connect the device to fill in the gui fields
		for (DevCollectRun devRun : newDevices)
		{
			devRun.connect();
		}
	}
	
	public void startCollection()
	{
		if (null != cmd && cmd.isAlive())
		{
			Log.error("Collection already in progress");
			Util.printStackTrace();
			return;
		}
		cmd = new CmdCollectSequence(this);
		cmd.start();
	}
	
	public void pauseCollection()
	{
		if (collectState == CollectState.COLLECTING)
		{
			DevFPE.get().pause(true);
			devRunList.setCollectState(CollectState.PAUSING,true);
		}
	}
	
	public void abortCollection()
	{
		if (collectState == CollectState.COLLECTING || collectState == CollectState.PAUSING)
		{
			devRunList.setCollectState(CollectState.PAUSING,true);
			cmd.abort();
		}
	}
	
    protected boolean validateRunCount(int runCount)
    {
		if (runCount<0)								{ return false; }
		if (runCount>ModelCollectRunList.MAX_COUNT)	{ return false; }
		return true;
    }
    
    protected boolean validateCurrentRun(int currentRun)
    {
		if (currentRun<0)							{ return false; }
		if (currentRun>getRunCount()) 				{ return false; }
		return true;
    }
    
	public void createRuns(ArrayList<Site> sites)
	{
		int runCount = ModelCollectRunList.get().getRunCount();
		
		if ((runCount+1) + sites.size() > MAX_COUNT)
		{
			Util.popupOk("Too many runs", "Adding "+sites.size()+" collect runs would exceed the run limit of "+MAX_COUNT+".");
			return;
		}
		
		int newRunLabel = DevCollectRun.getRunLabel(runCount)+1;
		int newIndex = runCount+1;
		
		for (Site site : sites)
		{
			// copy the last run
			DevCollectRunList.getIndex(runCount).copyRunContents(newIndex, RUN_LABEL_SETTING.SAME, true, false);
			DevCollectRun.setRunLabel(newIndex, newRunLabel);
			
			// created the selected sites array with one site selected
			ArrayList<Boolean> selectedSites = new ArrayList<Boolean>();
			int siteCount = ModelRasterSiteList.get().getHighestSiteNumber()+1;
			for (int i=0; i<siteCount; i++) { selectedSites.add(i==site.index-1); }
			DevCollectRun.setSelectedSites(newIndex, selectedSites);
			
			// fix FPS
			ModelCollectRun lastRun = (ModelCollectRun)ModelCollectRunList.get().getRun(runCount);
			int frameCount = (lastRun.getEndFrameLabel() - lastRun.getStartFrameLabel() + 1);
			DevCollectRun.setFramesPerSite(newIndex, frameCount); // assign all frames to 1 site
			
			// fix wedge size (multiple wedges per site not yet supported)
			DevCollectRun.setWedgeSize(newIndex, frameCount*lastRun.getDelta());

			// set scan mode to raster
			DevCollectRun.setScanMode(newIndex, ScanMode.RASTER);
			DevCollectRun.setSetupStep(newIndex, SetupStep.COLLECT_PARAM);
			
			// increment counters
			newIndex++;
			newRunLabel++;
		}

		// increase run count to load the new runs
		DevCollectRunList.get().setRunCount(runCount+sites.size(),false);
		DevCollectRunList.get().setCurrentRun(runCount+sites.size(),true);
	}

	private static volatile boolean becomingMasterOnStartup = false;
	public void handleCollectInProgressOnStartup()
	{
	}
	
	public void handleCollectInProgress(boolean showPopup)
	{
		if (!DevFPE.get().isIdle())
		{
			if (showPopup) Util.asyncPopupOk
			(
				"Collection is being paused",
				"This client has become master during a data collection.\n"+
				"Collection is being paused, and may be resumed from the collect tab."
			);
			
			Log.note("Collect recovery: Sending pause to FPE and waiting for partition completion");
			DevFPE.get().pause(false);
			
			while (!DevFPE.get().isIdle()) { Util.sleep(100); }
		}
		else
		{
			if (showPopup) Util.asyncPopupOk
			(
				"Collection recovery",
				"Collection has been aborted due to JBluIce closing before the end of the run.  "+
				"Collection may be resumed from the collect tab."
			);
		}
		
		// fix run states
		Log.note("Collect recovery: fixing run states");
		ModelCollectRun currentRun = getCurrentRunModel();
		RunState state = ModelCollectRun.getCorrectState(DevFPE.get().getCurrentFrameIndex(), currentRun.seq().getFrameListCopy().size());
		currentRun.dev().setRunStatus(state, true);
		devRunList.setCollectState(CollectState.INACTIVE,true);
		DevFPE.get().setETA(0);
		
		// ringlight off
		Log.note("Collect recovery: ringlight off");
		DevSlider ringLight = DevSlider.get("ring");
		ringLight.setIntensity(ringLight.getLowerLimit());

		// backlight in
		Log.note("Collect recovery: backlight in");
	    DevCollimatorCtrl.CCMove ccMove = DevCollimatorCtrl.get().newMove();
	    ccMove.setArguments("OUT backlightIN bluice");
	    //ccMove.setDest("backlight",		InOut.IN);
	    //ccMove.setDest("collimator",	InOut.OUT);
	    ccMove.start();

		Log.note("Collection inactive.");
	}
	
	private void handleBecomePassiveDuringCollect()
	{
		Util.asyncPopupOk
		(
			"Became passive during collect", 
			"Data collection was paused because another instance of JBluIce\n"+
			"became \"master\". To continue data collection, become\n"+
			"\"master by force\" and then click \"Collect\""
		);
	}
}
