package gov.anl.gmca.collect.graph;

import gov.anl.gmca.collect.graph.SFGraph.SpotFinderPoint;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import java.util.ArrayList;

// graph title bar and legend
//
public class WdgSFDisplay extends Composite
{
	private StyledText l1, l2;
	
	public WdgSFDisplay(Composite parent)
	{
		super(parent, SWT.NONE);
		
		GridLayout gl = new GridLayout(1,false);
		gl.verticalSpacing=0;
		gl.marginWidth=0;
		gl.marginHeight=0;
		setLayout(gl);
		
		Composite divider = new Composite(this, SWT.BORDER);
		divider.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint = 3;
		divider.setLayoutData(gd);
		
		l1 = new StyledText(this, SWT.WRAP); // wrap is needed for centering to work
		l1.setBackground(GUI.COLOR_WHITE);
		l1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l2 = new StyledText(this, SWT.WRAP);
		l2.setBackground(GUI.COLOR_WHITE);
		l2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	}

	public void updateText(SpotFinderPoint point)
	{
		try
		{
			l1.setText(point.filename+" ("+Util.printDouble(point.angle,2)+"\u00B0)");
			StyleRange range = new StyleRange(0, point.filename.length(), null, null, SWT.BOLD);
			l1.setStyleRange(range);
			l1.setLineAlignment(0, 1, SWT.CENTER);
			ranges = new ArrayList<StyleRange>();
			styleText = "";

			//for (SPOTFINDER_OUTPUT output : DevSFType.getConfig(SFRunType.COLLECT).id.getOutputs())
			SPOTFINDER_OUTPUT output = DevSFType.getConfig(SFRunType.COLLECT).id.defaultOutput;
			{
				Color c;
				if (output==SFGraph.getOutput1()) c=GUI.COLOR_BLUE2;
				else if (output==SFGraph.getOutput2()) c=GUI.COLOR_GREEN2;
				else c=GUI.COLOR_DARK_GRAY;

				styleText += output.shortName+": ";
				addStyleText(Util.printDouble((double)point.data, output.decimals), c, null, SWT.BOLD);
				
				if (null != point.inversePoint)
				{
					styleText += "/";
					addStyleText(Util.printDouble((double)point.inversePoint.data, output.decimals), GUI.COLOR_BLUE1, null, SWT.BOLD);
				}
				styleText += "  ";
			}
			
			l2.setText(styleText);
			l2.setStyleRanges(ranges.toArray(new StyleRange[0]));
			l2.setLineAlignment(0, 1, SWT.CENTER);
		}
		catch (Exception e) { e.printStackTrace(); }
	}
	
	private ArrayList<StyleRange> ranges;
	private String styleText;
	private void addStyleText(String text, Color fg, Color bg, int styleInt)
	{
		int start = styleText.length();
		styleText += text;
		int len = styleText.length()-start;
		ranges.add(new StyleRange(start, len, fg, bg, styleInt));
	}
}
