package gov.anl.gmca.collect.ui;

import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.Limits.LimitsResult;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

/**
 * A modified WdgControlEntry that hides itself and its unit label if it's not set
 */
public class WdgCollectEnergy extends WdgControlEntry
{
	protected Label label;
	protected Label unitLabel;
	
	protected boolean defaultVisible, isVisible;
	
	public WdgCollectEnergy(Composite parent, int style, boolean defaultVisible, EntryType type, ModelInterface mi)
	{
		super(parent, style, type, mi);
		setBackground(GUI.BGCOLOR);
		this.defaultVisible=defaultVisible;
		
		if (!defaultVisible) entry.setVisible(false);
	}
	
	public void setLabels(Label label, Label unitLabel)
	{
		this.label=label;
		this.unitLabel=unitLabel;
		
		if (!defaultVisible)
		{
			label.setVisible(false);
			unitLabel.setVisible(false);
		}
	}
	
	@Override
	public void setText(Object newVal)
	{
		if (null == newVal)
		{
			entry_setText(entry, "");
		}
		else
		{
			super.setText(newVal);
		}
	}
	
	public static ValidateResult energyValidate(int energyIndex, String s, Limits limits)
	{
		if (1 != energyIndex && s.equals("")) return new ValidateResult(true);
		
		try
		{
			double d = Double.parseDouble(s);

			if (1 != energyIndex && 0.0 == d) return new ValidateResult(true);

			if (null != limits)
			{
				LimitsResult result = limits.testValue(d);
				if (result.isOK())
				{
					return new ValidateResult(true);
				}
				else
				{
					return new ValidateResult(false, result.getReason());
				}
			}
			else
			{
				return new ValidateResult(true);
			}
		}
		catch (NumberFormatException nfe)
		{
			return new ValidateResult(false, s+" is not a valid number.");
		}
	}
	
	@Override
	public void setVisible(boolean visible)
	{
		label.setVisible(visible);
		entry.setVisible(visible);
		unitLabel.setVisible(visible);
	}
}
