package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.RunCalculator.Frame;
import gov.anl.gmca.common.ModelRunList.CurrentRunListener;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.SWTUtil;
import gov.anl.gmca.util.Util;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;

public class WdgCollectSequence extends Composite
{
	public static class FrameProvider implements ILazyContentProvider
	{
		private TableViewer viewer;
		private ArrayList<Frame> frames;
		
		public FrameProvider(TableViewer viewer) { this.viewer=viewer; }
		@Override public void dispose() {}
		@Override public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
			frames = (ArrayList<Frame>)newInput;
		}

		@Override public void updateElement(int index)
		{
			viewer.replace(frames.get(index), index);
		}
	}
	
	protected static Object[][] cellHeaders =
	{
		{ "Filename",	90 },
		{ "Angle",		60 },
		{ "Energy",		60 },
		{ "Site#",		50 },
		{ "Batch",		50 }
	};
	
	protected Group group;
	protected TableViewer tableViewer;
	private StackLayout stackLayout;
	private Composite jetModeDisplay;
	protected Table table;
	private Label dirDisplay, fileDisplay;
	private ModelCollectRun currentRun = null;
	private ArrayList<Frame> frameList = new ArrayList<Frame>();
	protected static WdgCollectSequence inst;
	
	public WdgCollectSequence(Composite parent, int style)
	{
		super(parent, style);
		inst = this;
		currentRun = ModelCollectRunList.get().getCurrentRunModel();
		stackLayout = new StackLayout();
		setLayout(stackLayout);
		setBackground(GUI.BGCOLOR);
		
		group = new Group(this,SWT.NONE);

		group.setFont(GUI.FONT_COLLECT_GROUP);
		group.setText("Run Sequence");
		group.setBackground(GUI.BGCOLOR);
		group.setLayout(new GridLayout(1,false));

		addMouseListener(new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent arg0) {}
			@Override public void mouseDown(MouseEvent me)
			{
				//Log.note("== me.x="+me.x+" me.y=" + me.y);
				if (me.y > 20) return;
				TabCollect.get().toggleSequenceSize();
			}
		});
		
		tableViewer = new TableViewer(group, SWT.VIRTUAL);
		table = tableViewer.getTable();
		table.setFont(GUI.FONT_COLLECT_SEQUENCE);
		table.setBackground(GUI.BGCOLOR);
		table.setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,true));
		for (int i=0;i<cellHeaders.length;i++)
		{
		    TableViewerColumn viewerColumn = new TableViewerColumn(tableViewer,SWT.CENTER);
			TableColumn column = viewerColumn.getColumn();
			column.setText((String)cellHeaders[i][0]);
			column.setWidth((Integer)cellHeaders[i][1]);
			column.setResizable(true);
			column.setMoveable(true);
			final int columnIndex=i;
			viewerColumn.setLabelProvider(new CellLabelProvider()
			{
		        @Override public void update(ViewerCell cell)
		        {
		        	Frame frame = (Frame)cell.getElement();
		        	String cellText;
		        	switch (columnIndex)
		        	{
		        	case 0:	cellText = frame.getDisplayFilename(); break;
		        	case 1:	cellText = Util.printDouble(frame.angle,2); break;
		        	case 2:	cellText = Util.printDouble(frame.energy,2); break;
		        	case 3:	cellText = frame.siteNumber>=0?Integer.toString(frame.siteNumber+1):""; break;
		        	case 4:	cellText = Integer.toString(frame.partitionIndex); break;
		        	default: throw new IllegalStateException();
		        	}
		        	if (frame.frameIndex==-1 && columnIndex!=0) cellText="";
		        	cell.setText(cellText);
		        	if (null!=currentRun) cell.setForeground(getTableItemColor(frame.frameIndex, currentRun.getNextFrame()));
			    }
			});
		}
		table.setHeaderVisible(true);
		tableViewer.setContentProvider(new FrameProvider(tableViewer));

		jetModeDisplay = new Composite(this, SWT.NONE);
		jetModeDisplay.setBackground(GUI.BGCOLOR);
		jetModeDisplay.setBackgroundMode(SWT.INHERIT_DEFAULT);
		jetModeDisplay.setLayout(new GridLayout(1,false));
		Label jetLabel = new Label(jetModeDisplay, SWT.CENTER);
		jetLabel.setText("Jet collect");
		jetLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Composite divider = new Composite(jetModeDisplay, SWT.NONE);
		divider.setBackground(GUI.COLOR_DARK_GRAY);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint = 1;
		divider.setLayoutData(gd);
		dirDisplay = new Label(jetModeDisplay, SWT.LEFT);
		dirDisplay.setFont(GUI.FONT_10_FIXED);
		dirDisplay.setText("Directory:    ---");
		dirDisplay.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		fileDisplay = new Label(jetModeDisplay, SWT.LEFT);
		fileDisplay.setFont(GUI.FONT_10_FIXED);
		fileDisplay.setText("Images taken: ---");
		fileDisplay.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		stackLayout.topControl = jetModeDisplay;
		layout(true);

		ModelCollectRunList.get().addCurrentRunListener(new CurrentRunListener()
		{
			@Override public void currentRunChanged(ModelRun run)
			{
				currentRun = ModelCollectRunList.get().getCurrentRunModel();
				updateContents();
			}
		});
		
		ModelCollectRunList.get().llCurrentRunFrame.addListener(new ListenerList.Listener<ModelCollectRunList>()
		{
			@Override public void handleEvent(ModelCollectRunList runList, Object data)
			{
				updateContents();
			}
		});
	}

	protected int lastFrame = -1;
	protected int lastRunIndex = -1;
	protected long lastSequenceVersion = 0;
	protected int lastNextFrame = -1;
	protected int colorCount = 0;
	protected boolean useWideLayout = false;
	
	public void redistributeColumns(boolean wideLayout)
	{
		useWideLayout = wideLayout;
		
		// update filename column width
		ModelCollectRun currentRun = ModelCollectRunList.get().getCurrentRunModel();
		if (null!=currentRun && null!=currentRun.seq())
		{
			ArrayList<Frame> frameList = new ArrayList<Frame>(currentRun.seq().getFrameListCopy());
			if (frameList.size()>0)
			{
				GC gc = new GC(inst);
				int textWidthPx = gc.textExtent(frameList.get(0).getDisplayFilename()).x;
				int margin = 10;
				cellHeaders[0][1] = textWidthPx+margin;
				gc.dispose();
			}
		}
		
		// for wide layout
		int totalWidth = getSize().x-15;
		int totalStandardWidth = 0;
		for (Object[] colData : cellHeaders) { totalStandardWidth += (Integer)colData[1]; }
		double multiplier = (double)totalWidth / (double)totalStandardWidth;
		
		for (int colIndex=0; colIndex<table.getColumnCount(); colIndex++)
		{
			int width = (Integer)cellHeaders[colIndex][1];
			
			if (wideLayout) width = (int)((double)width * multiplier);
			
			table.getColumn(colIndex).setWidth(width);
		}
	}
	
	private Color getTableItemColor(int frameIndex, int nextFrame)
	{
		if (frameIndex==-1) return GUI.COLOR_BLACK;
		if (frameIndex <  nextFrame) return GUI.COLOR_DARK_GRAY;
		if (frameIndex == nextFrame) return GUI.COLOR_DARK_RED3;
		if (frameIndex >  nextFrame) return GUI.COLOR_BLACK;
		return null;
	}
	
	private String getEndMarker(ModelCollectRun currentRun, int frameCount)
	{
		if (currentRun.getScanMode()==ScanMode.VECTOR && currentRun.getSetupStep() == SetupStep.SITE_SETUP)
		{
			return "";
		}
		else if (currentRun.getNextFrame() >= frameCount)
		{
			// if nextFrame is past the last frame, set the end label to "Complete" and make it black
			return "Complete";
		}
		else
		{
			// otherwise set it to "End"
			return "End";
		}
	}

	private Boolean currentShowJetDisplay = null;

	private void updateContents()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			ModelCollectRun currentRun = ModelCollectRunList.get().getCurrentRunModel();
			
			// refill table if needed
			if (lastSequenceVersion != currentRun.seq().getVersion() ||	lastRunIndex != currentRun.getRunIndex() || lastFrame != currentRun.getNextFrame())
			{
				Boolean showJetDisplay = currentRun.getScanMode() == ScanMode.JET;
				if (showJetDisplay != currentShowJetDisplay) {
					stackLayout.topControl = showJetDisplay ? jetModeDisplay : group;
					layout(true);
					currentShowJetDisplay = showJetDisplay;
				}

				int nextFrame = currentRun.getNextFrame();
				if (showJetDisplay) {
					int dirNum = nextFrame / currentRun.getFramesPerDir() + 1;
					int frameNum = nextFrame % currentRun.getFramesPerDir() + 1;
					dirDisplay.setText(Util.sprintf("Directory:    %03d", dirNum));
					fileDisplay.setText(Util.sprintf("Images taken: %06d", frameNum));
				}

				// get the current frame list and add the end marker
				frameList = currentRun.seq().getFrameListCopy();
				String endMarker = getEndMarker(currentRun, frameList.size());
				frameList.add(new Frame(endMarker, 0, 0, 0, null, -1 /*signals end marker*/, 0, -1, 0, false, false, 1, ScanMode.SINGLE, 0, "", 0));
				
				// send the list to the tableviewer
				tableViewer.setItemCount(frameList.size());
				tableViewer.setInput(frameList);
				tableViewer.refresh();

				// center the current frame
				SWTUtil.centerTableIndex(tableViewer.getTable(), nextFrame);

				// update variables to reflect a table refresh
				lastSequenceVersion = currentRun.seq().getVersion();
				lastRunIndex = currentRun.getRunIndex();
				lastFrame = nextFrame;
			}
			
			redistributeColumns(useWideLayout);
		}});
	}
}
