package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ui.cam.WdgImportCamera;
import gov.anl.gmca.common.DevRuntimeConfig;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.GrpWdgMove;
import gov.anl.gmca.hutch.WdgMotor;
import gov.anl.gmca.sample.DBCamera;
import gov.anl.gmca.sample.WdgCamera.CameraSize;
import gov.anl.gmca.sample.WdgCamera.CameraType;
import gov.anl.gmca.sample.WdgSampleRotate;
import gov.anl.gmca.sample.WdgSampleRotate.GetRotateN;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

public class WdgImportTab extends Composite
{
	WdgImportCamera repeatCamera;
	private double rotateNValue = 1.0;
	private WdgSampleRotate rotate;
	private Button checkboxShowNumbers, checkboxShowMarkers;
	private Label lblFiducialMarkersOffScreen, lblSitesOffScreen;
	
	public WdgImportTab(Composite parent)
	{
		super(parent, SWT.NONE);
		
		setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(2,false);
		gl.marginWidth = 0;
		setLayout(gl);
		
		repeatCamera = new WdgImportCamera(this, SWT.NONE, DBCamera.getImageURL(CameraType.HIGHRES, CameraSize.LARGE), CameraType.HIGHRES, CameraSize.LARGE);
		GridData gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		gd.horizontalSpan = 2;
		repeatCamera.setLayoutData(gd);
		
		lblFiducialMarkersOffScreen = new Label(this, SWT.CENTER);
		//lblFiducialMarkersOffScreen.setFont(GUI.FONT_12);
		lblFiducialMarkersOffScreen.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.CENTER,SWT.DEFAULT,true,false,2,1);
		lblFiducialMarkersOffScreen.setLayoutData(gd);

		lblSitesOffScreen = new Label(this, SWT.CENTER);
		lblSitesOffScreen.setBackground(GUI.BGCOLOR);
		//lblSitesOffScreen.setFont(GUI.FONT_12);
		gd = new GridData(SWT.CENTER,SWT.DEFAULT,true,false,2,1);
		lblSitesOffScreen.setLayoutData(gd);
		
		Composite zoomComp = new Composite(this, SWT.NONE);
		zoomComp.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.CENTER,SWT.BOTTOM,true,false);
		gd.widthHint = 220;
		zoomComp.setLayoutData(gd);
		GridLayout zoomLayout = new GridLayout(2,true);
		zoomLayout.horizontalSpacing=0;
		zoomLayout.marginWidth=20;
		zoomComp.setLayout(zoomLayout);
		
		GrpWdgMove moveGroup = new GrpWdgMove();
		
		WdgMotor wdgZoom = new WdgMotor(zoomComp,SWT.NONE,moveGroup,"Zoom",null, "camera_zoom_2", parent.getBackground(), new double[]{ 1, 2, 4, 6, 8, 10, 12 }, 2);
		wdgZoom.setBackground(parent.getBackground());
		wdgZoom.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1));

		Button startButton = new Button(zoomComp, SWT.PUSH);
		startButton.setText("Start");
		startButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		startButton.setFont(GUI.FONT_COLLECTRUN_BUTTON);
		moveGroup.setStartButton(startButton);
		startButton.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		Button cancelButton = new Button(zoomComp, SWT.PUSH);
		cancelButton.setText("Cancel");
		cancelButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		cancelButton.setFont(GUI.FONT_COLLECTRUN_BUTTON);
		moveGroup.setCancelButton(cancelButton);
		cancelButton.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		Composite rotateComp = new Composite(this, SWT.NONE);
		rotateComp.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.CENTER,SWT.BOTTOM,true,false);
		gd.widthHint = 220;
		rotateComp.setLayoutData(gd);
		GridLayout rotateLayout = new GridLayout(3,true);
		rotateLayout.marginWidth = 20;
		rotateComp.setLayout(rotateLayout);
		
		Group nGroup = new Group(rotateComp, SWT.NONE);
		nGroup.setBackground(parent.getBackground());
		nGroup.setText("Rotate +-n");
		nGroup.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData(SWT.FILL,SWT.DEFAULT,true,false, 3, 1);
		nGroup.setLayoutData(gd);
		
		nGroup.setLayout(new GridLayout(2,false));
		
		final WdgControlEntry rotateNEntry = new WdgControlEntry(nGroup, SWT.NONE, EntryType.RW_COMBO, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				rotateNValue = (Double)value;
			}
			@Override
			public double[] getFloatChoices()
			{
				return new double[]{.1,.5,1,5,10,30,45,60,90,180};
			}
			@Override public int getDecimals() 				{ return 2; 		}
			@Override public Limits getLimits() 			{ return DevPmacMotor.get("gonio_omega").getLimits(); }
		});
		rotateNEntry.setText(1.0);
		gd = new GridData();
		gd.widthHint = 100;
		rotateNEntry.setLayoutData(gd);
		rotateNEntry.setEnabled(true);
		
		Label nLabel = new Label(nGroup, SWT.NONE);
		nLabel.setText("deg");
		nLabel.setBackground(parent.getBackground());
		gd = new GridData();
		nLabel.setLayoutData(gd);
		
		GetRotateN getRotateN = new GetRotateN()
		{
			@Override public double getRotateN() { return rotateNValue; }
		};
		rotate = new WdgSampleRotate(rotateComp, SWT.HORIZONTAL, getRotateN);
		rotate.setLayoutData(new GridData(SWT.CENTER, SWT.DEFAULT, true, false, 3, 1));
		
		checkboxShowNumbers = new Button(this, SWT.CHECK);
		checkboxShowNumbers.setText("Show crystal numbers");
		checkboxShowNumbers.setBackground(parent.getBackground());
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, false, false);
		gd.horizontalSpan = 2;
		checkboxShowNumbers.setLayoutData(gd);
		checkboxShowNumbers.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se){}
			@Override public void widgetSelected(SelectionEvent se)
			{
				boolean showNumbers = checkboxShowNumbers.getSelection();
				DevRuntimeConfig.setImportShowNumbers(showNumbers);
			}
		});
		
		checkboxShowMarkers = new Button(this, SWT.CHECK);
		checkboxShowMarkers.setText("Show fiducial and crystal markers");
		checkboxShowMarkers.setBackground(parent.getBackground());
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, false, false);
		gd.horizontalSpan = 2;
		checkboxShowMarkers.setLayoutData(gd);
		checkboxShowMarkers.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se){}
			@Override public void widgetSelected(SelectionEvent se)
			{
				boolean showSites = checkboxShowMarkers.getSelection();
				DevRuntimeConfig.setImportShowSites(showSites);
			}
		});
	}
	
	public void connectListeners()
	{
		DevRuntimeConfig.llChange.addListener(new ListenerList.Listener<DevRuntimeConfig>() {
			@Override public void handleEvent(DevRuntimeConfig object, Object data) {
				GUI.asyncExec(new Runnable(){public void run() {
					updateCheckboxes();
				}});
			}});
		GUI.asyncExec(new Runnable(){public void run() {
			updateCheckboxes();
		}});
		
		/*repeatCamera.llMarkerVisibility.addListener(new ListenerList.Listener<WdgImportCamera>()
		{
			@Override public void handleEvent(WdgImportCamera object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run() {
					String text1 = "";
					int val1 = repeatCamera.fiducialMarkersOffScreen;
					if (val1>0) text1 = val1+" fiducial markers are off screen";
					lblFiducialMarkersOffScreen.setText(text1);
					
					String text2 = "";
					int val2 = repeatCamera.sitesOffScreen;
					if (val2>0) text2 = val2+" crystal positions are off screen";
					lblSitesOffScreen.setText(text2);
					
					layout(true);
				}});
			}
		});*/
	}
	
	private void updateCheckboxes()
	{
		boolean showSites = DevRuntimeConfig.getImportShowSites();
		checkboxShowMarkers.setSelection(showSites);
		checkboxShowNumbers.setSelection(DevRuntimeConfig.getImportShowNumbers());
		checkboxShowNumbers.setEnabled(showSites);
	}
	
	public void notifyIsVisible(boolean isVisible)
	{
		repeatCamera.notifyIsVisible(isVisible);
	}
}
