package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.MeshParams;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.NumberingParameter;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.spotfinder.DBImageStats;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_TYPE;
import gov.anl.gmca.users.ModelUserTable.*;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class WdgMeshSetup extends Composite
{
	private ModelCollectRun run;
	//private Slider slider;
	private Label displaySlider, lblParam, lblNumbering;
	private Combo comboRun, comboNumbering;
	private Combo comboParam;
	private Button checkboxCombine, filterAdd, filterSubtract;
	private Scale scale;

	public WdgMeshSetup(Composite parent, ModelCollectRun run)
	{
		super(parent, SWT.NONE);
		this.run=run;

		setBackground(GUI.BGCOLOR);
		setLayout(new GridLayout(1,false));

		// filter group
		Group group = new Group(this, SWT.NONE);
		group.setText("Filter");
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(2,false);
		gl.verticalSpacing=0;
		group.setLayout(gl);

		// run selector
		Label lblRun = new Label(group, SWT.NONE);
		lblRun.setText("Raster run: ");
		comboRun = new Combo(group, SWT.READ_ONLY);
		comboRun.add("0");
		comboRun.select(0);
		comboRun.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		lblNumbering = new Label(group, SWT.NONE);
		lblNumbering.setText("Numbering: ");
		comboNumbering = new Combo(group, SWT.READ_ONLY);
		comboNumbering.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		for (NumberingParameter np : NumberingParameter.values()) {
			comboNumbering.add(Util.capitalizeFirstLetters(np.toString()));
		}

		// combine option
		Label lblCombine = new Label(group, SWT.NONE);
		lblCombine.setText("Combine: ");
		checkboxCombine = new Button(group, SWT.CHECK);
		checkboxCombine.setSelection(false);
		checkboxCombine.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent selectionEvent) {
				MeshParams.changeCombineFlag(run, checkboxCombine.getSelection());
			}
			@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
		});

		// filter parameter
		lblParam = new Label(group, SWT.NONE);
		lblParam.setText("Filter: ");
		comboParam = new Combo(group, SWT.READ_ONLY);
		comboParam.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		for (SPOTFINDER_TYPE sfType : DevSFType.getActiveTypes())
		{
			for (SPOTFINDER_OUTPUT output : sfType.getOutputs()) {
				if (!output.shortName.toLowerCase().contains("res")) {
					comboParam.add(output.shortSF_Name);
				}
			}
		}

		// filter slider
		Label lblFilter = new Label(group, SWT.NONE);
		lblFilter.setText("Limit: ");
		lblFilter.setLayoutData(new GridData(SWT.DEFAULT, SWT.CENTER, false, true));

		Composite filterComp = new Composite(group, SWT.NONE);
		filterComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		filterComp.setBackground(GUI.BGCOLOR);
		gl = new GridLayout(3,false);
		gl.marginHeight=gl.marginWidth=0;
		gl.horizontalSpacing=gl.verticalSpacing=0;
		filterComp.setLayout(gl);

		filterSubtract = new Button(filterComp, SWT.PUSH);
		filterSubtract.setFont(GUI.FONT_10);
		filterSubtract.setText("-");
		GridData gd = new GridData();
		//gd.heightHint=gd.widthHint=20;
		filterSubtract.setLayoutData(gd);

		displaySlider = new Label(filterComp, SWT.CENTER);
		displaySlider.setText("  ");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		displaySlider.setLayoutData(gd);

		filterAdd = new Button(filterComp, SWT.PUSH);
		filterAdd.setFont(GUI.FONT_10);
		filterAdd.setText("+");
		gd = new GridData();
		//gd.heightHint=gd.widthHint=20;
		filterAdd.setLayoutData(gd);

		scale = new Scale(filterComp, SWT.HORIZONTAL|SWT.BORDER);
		scale.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
		scale.setMinimum(0);
		scale.setMaximum(50);
		scale.setIncrement(1);

		updateWidgets();
	}

	public void connectListeners()
	{
		ListenerList.Listener<DBImageStats> dbListener = new ListenerList.Listener<DBImageStats>()
		{
			@Override
			public void handleEvent(DBImageStats object, Object data)
			{
				updateWidgets();
			}
		};
		DBImageStats.llAnyChange.addListener(dbListener);

		//////////////////////////////////////// user input //////////////////////////////

		// raster run
		comboRun.addSelectionListener(new SelectionListener() {
			@Override public void widgetSelected(SelectionEvent selectionEvent) {
				int selected = comboRun.getSelectionIndex();
				if (selected != run.getRasterRun()) {
					MeshParams.changeRasterRun(run, selected);
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
		});

		// output
		comboParam.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent selectionEvent)
			{
				SPOTFINDER_OUTPUT selectedOutput = SPOTFINDER_OUTPUT.fromShortSF_Name(comboParam.getText());
				SPOTFINDER_OUTPUT currentOutput = SPOTFINDER_OUTPUT.values()[run.getMeshParam()];
				if (selectedOutput!=currentOutput)
					MeshParams.changeOutput(run, selectedOutput);
			}
			@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
		});

		// numbering
		comboNumbering.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				MeshParams.changeNumbering(run, comboNumbering.getSelectionIndex());
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		// combine

		// filter
		scale.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				double selectedVal = scale.getSelection();
				SPOTFINDER_OUTPUT output = SPOTFINDER_OUTPUT.values()[run.getMeshParam()];
				if (output.decimals>0) selectedVal /= 10.0;
				MeshParams.changeFilter(run, selectedVal);
				//System.out.println("Filter = "+scale.getSelection());
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		filterAdd.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				MeshParams.changeFilter(run, scale.getSelection()+1);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		filterSubtract.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				MeshParams.changeFilter(run, scale.getSelection()-1);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		//////////////////////////////////////// from epics //////////////////////////////

		// raster run list
		ModelRasterRunList.Listener listListener = new ModelRunList.Listener()
		{
			@Override
			public void runListChanged(ModelRunList runList)
			{
				updateWidgets();
			}
		};
		ListenerList.Listener<ModelRasterState> stateListener = new ListenerList.Listener<ModelRasterState>()
		{
			@Override
			public void handleEvent(ModelRasterState object, Object data)
			{
				updateWidgets();
			}
		};


		ModelRasterRunList.get(RasterListType.INTER).addListener(listListener);
		ModelRasterState.get().llCurrentRun.addListener(stateListener);
		updateWidgets();

		// all other changes are from the run itself
		WdgMeshSetup wdg = this;
		ModelCollectRun.Listener runListener = new ModelCollectRun.Listener()
		{
			@Override public void runChanged(ModelCollectRun run) { wdg.runChanged(); }
		};
		run.addListener(runListener);

		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent disposeEvent)
			{
				DBImageStats.llAnyChange.removeListener(dbListener);
				ModelRasterRunList.get(RasterListType.INTER).removeListener(listListener);
				ModelRasterState.get().llCurrentRun.removeListener(stateListener);
				run.removeListener(runListener);
			}
		});
	}

	public void runChanged() { updateWidgets(); }

	private int displayedOutputIndex=-1;
	public void updateWidgets()
	{
		if (!run.dev().connected()) return;

		GUI.asyncExec(()->
		{
			// run selection
			ModelRasterRunList runList = ModelRasterRunList.get(RasterListType.INTER);
			while (comboRun.getItemCount() < runList.getRunCount() + 1) {
				comboRun.add("");
			}
			while (comboRun.getItemCount() > runList.getRunCount() + 1) {
				comboRun.remove(comboRun.getItemCount() - 1);
			}
			for (int i = 0; i < comboRun.getItemCount(); i++) {
				comboRun.setItem(i, "" + ((ModelRasterRun) runList.getRun(i)).getRunLabel());
			}
			if (run.getRasterRun() > runList.getRunCount()) {
				MeshParams.changeRasterRun(run, runList.getRunCount());
			}
			if (comboRun.getSelectionIndex() != run.getRasterRun())
			{
				comboRun.select(run.getRasterRun());
			}

			// output selection
			SPOTFINDER_OUTPUT output = SPOTFINDER_OUTPUT.values()[run.getMeshParam()];
			if (displayedOutputIndex != output.ordinal()) {

				String[] items = comboParam.getItems();
				for (int i=0;i<items.length;i++)
					if (items[i].equals(output.shortSF_Name))
						comboParam.select(i);
				displayedOutputIndex = output.ordinal();
			}

			// numbering selection
			if (comboNumbering.getSelectionIndex() != run.getRasterNumbering()) comboNumbering.select(run.getRasterNumbering());

			// combine checkbox
			if (checkboxCombine.getSelection() != run.getMeshCombine()) checkboxCombine.setSelection(run.getMeshCombine());

			/////// filter, beam size, angles /////

			// set mesh defaults if raster run just finished, or clear filter otherwise
			ModelRasterRun rasterRun = ((ModelRasterRun)runList.getRun(run.getRasterRun()));
			RunState runState = rasterRun.getRunState();

			Double currentVal = null;
			boolean rasterComplete;
			if (rasterRun.getRunType()== ModelRasterRun.RunType.DIFFRACTION) rasterComplete = runState==RunState.COMPLETE;
			else rasterComplete = rasterRun.getSONICCBin();

			if (Client.isMaster()) {
				if (rasterComplete && run.getMeshFilter() < 0) {
					run.updateMeshParameters(run.getRasterRun(), true);
				} else if (!rasterComplete && run.getMeshFilter() >= 0) {
					currentVal = -1.0;
					run.dev().setMeshFilter(currentVal, true);
				}
			}

			// current val
			if (null==currentVal) currentVal = run.getMeshFilter();

			String selectedOutput = SPOTFINDER_OUTPUT.values()[run.getMeshParam()].toString();
			if (rasterRun.getRunType()== ModelRasterRun.RunType.SONICC) selectedOutput = "sonicc";

			// max val
			Double maxVal = 0.0;
			if (rasterComplete) maxVal = MeshParams.getMaxValueForParam(rasterRun.getDirectorySuffix(), rasterRun.getFileRoot(), rasterRun.getRunLabel(), selectedOutput);
			if (null==maxVal) maxVal=0.0;

			// set slider range, selection, and text display
			int displayDecimals = 0;
			if (displayDecimals>0) {
				scale.setMaximum((int)(maxVal*10.0));
				scale.setSelection((int)(currentVal*10.0));
				if (currentVal<0)
					displaySlider.setText("---");
				else
					displaySlider.setText(Util.sprintf("%.1f/%.1f", currentVal, maxVal));
			} else {
				scale.setMaximum((int)(double)maxVal);
				scale.setSelection((int)(double)currentVal);
				if (currentVal<0)
					displaySlider.setText("---");
				else
					displaySlider.setText(Util.sprintf("%d/%d", (int)(double)currentVal, (int) (double) maxVal));
			}

			// show/hide widgets
			boolean soniccMode = rasterRun.getRunType() != ModelRasterRun.RunType.DIFFRACTION;
			WdgControl.setControlVisible(lblNumbering, !soniccMode);
			WdgControl.setControlVisible(comboNumbering, !soniccMode);
			WdgControl.setControlVisible(lblParam, !soniccMode);
			WdgControl.setControlVisible(comboParam, !soniccMode);
			layout(true);

			updateEnableState(lastGEnable);
		});
	}

	private boolean lastGEnable = false;
	public void updateEnableState(boolean gEnable)
	{
		ModelRasterRun rasterRun = (ModelRasterRun)ModelRasterRunList.get(RasterListType.INTER).getRun(comboRun.getSelectionIndex());
		boolean enable = gEnable && (rasterRun.getRunState() == RunState.COMPLETE||rasterRun.getRunState()==RunState.PAUSED);
		boolean enableCombo = gEnable && !ModelRasterState.isRasterRunning();

		comboRun.setEnabled(enableCombo);
		comboParam.setEnabled(enable);
		comboNumbering.setEnabled(enable);
		checkboxCombine.setEnabled(enable);
		scale.setEnabled(enable);
		displaySlider.setEnabled(enable);
		filterSubtract.setEnabled(enable);
		filterAdd.setEnabled(enable);

		lastGEnable = gEnable;
	}

	public boolean updateModel()
	{
		return true;
	}
}
