package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DevMoveFloat;
import gov.anl.gmca.common.ModelRunList.CurrentRunListener;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgPositionDisplay;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.hutch.DevAttenuation;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.WdgResPredictor;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.WdgSpotFinderOptions;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class WdgMiddleColumn extends Composite
{
	protected WdgStartPause collectButtons;
	private WdgFPEStatus fpeStatus;
	private WdgShutterlessIndicator shutterlessIndicator;
	private WdgSpotFinderOptions sfOptions;
	
	public WdgMiddleColumn(Composite parent)
	{
		super(parent, SWT.NONE);
		
		setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(1,false);
		gl.marginBottom=0;
		setLayout(gl);
		
		collectButtons = new WdgStartPause(this, SWT.NONE);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		collectButtons.setLayoutData(gd);
		
		fpeStatus = new WdgFPEStatus(this);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		fpeStatus.setLayoutData(gd);
		
		Object[][] statList =
		{
				{"Gonio",			DevPmacMotor.get("gonio_omega")},
				{"Detector",		DevPmacMotor.get("detector_z")},
				{"Atten.",			DevAttenuation.get()},
				//{"Beamstop",		DevPmacMotor.get("beamstop_z")}
		};
		
		WdgPositionDisplay positionDisplay = new WdgPositionDisplay(this, SWT.NONE, "Current position", statList, GUI.FONT_COLLECT_GROUP);
		positionDisplay.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		positionDisplay.setLayoutData(gd);

		sfOptions = new WdgSpotFinderOptions(this, SFRunType.COLLECT);
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		sfOptions.setLayoutData(gd);
		
		final WdgResPredictor resPredictor = new WdgResPredictor(this, SWT.NONE, GUI.COLOR_GRAY, GUI.COLOR_BLUE, GUI.FONT_RESPREDICT_LABEL, 25);
		resPredictor.setGroupFont(GUI.FONT_COLLECT_GROUP);
		gd = new GridData();
		gd.heightHint = 140;
		gd.grabExcessHorizontalSpace=true;
		gd.horizontalAlignment=SWT.FILL;
		resPredictor.setLayoutData(gd);
		
		shutterlessIndicator = new WdgShutterlessIndicator(this, true);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		shutterlessIndicator.setLayoutData(gd);
		
		// The hutch tab uses rounded user-specified values to calculate the predicted resolutions,
		// so here too we have to do the same rounding
		DevPmacMotor.get("detector_2t").addListener(new DevMoveFloat.PositionListener()
		{
			@Override public void handleActPosChange(DevMoveFloat dev, final double newPos)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					double rounded_2t = Math.round(newPos*100.0)/100.0;
					resPredictor.setDetector2t_deg(rounded_2t, true);
				}});
			}
		});
		
		ModelCollectRunList.get().addCurrentRunListener(new CurrentRunListener()
		{
			@Override
			public void currentRunChanged(ModelRun _run)
			{
				final ModelCollectRun run = (ModelCollectRun)_run;
				
				if (null == run || null == run.getEnergies() || 0 == run.getEnergies().size()) return;
				
				GUI.asyncExec(new Runnable(){public void run()
				{
					double energy_eV = run.getEnergies().get(0)*1000.0;
					double distance_mm = run.getdistance_mm();
					
					double rounded_distance = Math.round(distance_mm*1000.0)/1000.0;
					double rounded_energy = Math.round(energy_eV*10.0)/10.0;
					
    				resPredictor.setDetectorZ_mm(rounded_distance, false);
    				resPredictor.setBeamEnergy_eV(rounded_energy, true);
				}});
			}
		});
		
		updateVisibility();
	}
	
	private void updateVisibility()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			WdgControl.setControlVisible(shutterlessIndicator, DetectorConfig.isDectris());
			getParent().layout(true);
		}});
	}

	public void connectListeners()
	{
		shutterlessIndicator.connectListeners();

		if (null!=sfOptions) {
			sfOptions.connectListeners();
			Client.addListener(new ClientListener()
			{
				@Override
				public void masterChanged(boolean isMaster)
				{
					GUI.asyncExec(() -> sfOptions.setEnabled(isMaster));
				}
			});
		}
	}
}
