package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.DevCollectRun;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevAttenuation;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.sample.DevCollimatorCtrl;
import gov.anl.gmca.sample.ModelBeamsize;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.spotfinder.SFData.SFType;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Util;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import java.util.ArrayList;

public class WdgRunButtons extends Composite implements SelectionListener, ClientListener
{
	protected Button copyButton, updateButton, deleteButton, resetButton;
	protected WdgRunTabs runTabs;
	protected WdgRunTab runTab;
	protected DevCollectRun runDev;
	protected ModelCollectRun run;
	
	protected WdgRunTabs.Listener runTabsListener = null;

	public WdgRunButtons(Composite parent, int style, WdgRunTabs _runTabs, WdgRunTab _runTab, ModelCollectRun _run, DevCollectRun _runDev)
	{
		super(parent, style);
		runTabs=_runTabs;
		runTab=_runTab;
		runDev=_runDev;
		run=_run;
		
		setLayout(new FillLayout());
		setBackground(parent.getBackground());
		
		copyButton	 =  WidgetFactory.newButton(this,	SWT.PUSH,	"Copy",		GUI.FONT_COLLECTRUN_BUTTON,	GUI.COLOR_BLUE_BUTTON);
		updateButton = 	WidgetFactory.newButton(this,	SWT.PUSH, 	"Update",	GUI.FONT_COLLECTRUN_BUTTON,	GUI.COLOR_BLUE_BUTTON);
		deleteButton = 	WidgetFactory.newButton(this,	SWT.PUSH, 	"Delete",	GUI.FONT_COLLECTRUN_BUTTON,	GUI.COLOR_BLUE_BUTTON);
		resetButton	 = 	WidgetFactory.newButton(this,	SWT.PUSH, 	"Reset",	GUI.FONT_COLLECTRUN_BUTTON,	GUI.COLOR_BLUE_BUTTON);

		copyButton.setToolTipText("Copy this run's contents to a new run tab");
		updateButton.setToolTipText("Overwrites motor positions for this run with the current hardware positions");
		deleteButton.setToolTipText("Delete this run");
		resetButton.setToolTipText("Reset this run's current position to the first frame");
		
		copyButton.setEnabled(false);
		updateButton.setEnabled(false);
		deleteButton.setEnabled(false);
		resetButton.setEnabled(false);
		
		copyButton.addSelectionListener(this);
		updateButton.addSelectionListener(this);
		deleteButton.addSelectionListener(this);
		resetButton.addSelectionListener(this);
		
		Client.addListener(this);
		
		runTabsListener = new WdgRunTabs.Listener()
		{
			@Override public void update()
			{
				updateEnableState();
			}
		};
		runTabs.addListener(runTabsListener);
	}
	
	public void removeListeners()
	{
		Client.removeListener(this);
		runTabs.removeListener(runTabsListener);
	}
	
	@Override public void widgetSelected(SelectionEvent se)
	{
		if (copyButton == se.getSource())		copyRun();
		if (updateButton == se.getSource())		updateValues();
		if (deleteButton == se.getSource())		runTabs.deleteTab(runTab.tabIndex, SFType.COLLECT, true);
		if (resetButton == se.getSource())		run.fii.reset(false, true);
	}

	protected void copyRun()
	{
		run.copyRun();
	}
	
	// called when clicking the 'update' button
	// ported from TCL, run_handle_update_button
	protected void updateValues()
	{
		if (run.getDelta()<0.01) runDev.setDelta(1.0, false);
		if (run.getExposureTime()<0.01) runDev.setExposureTime(1.0, false);
		if (run.getWedgeSize_deg()<1.0 && run.getScanMode()==ScanMode.SINGLE)
		{
			runDev.setWedgeSize(180.0, false);
		}
		
		Point2D beamsize = new Point2D(0,0);
		if (Util.beamSizeCalculated()) {
			int colli_um = DevCollimatorCtrl.getCollimatorSize().size_um;
			beamsize.x = beamsize.y = colli_um;
		} else {
			beamsize = ModelBeamsize.getCurrentBeamsize();
		}
		if (run.getScanMode()!=ScanMode.MESH) {
			runDev.setBeamWidth_um(beamsize.x, false);
			runDev.setBeamHeight_um(beamsize.y, false);
		}

		double newStartAngle = DevPmacMotor.get("gonio_omega").getPosition();
		double oldStartAngle = run.getStartAngle();

		if (0 == run.getRunIndex())
		{
			runDev.setStartAngle(newStartAngle, false);
			
			double endAngle = newStartAngle + run.getDelta();
			runDev.setEndAngle(endAngle, false);
			
			double upperGonioLimit = DevPmacMotor.get("gonio_omega").getLimits().getUpper();
			if (newStartAngle + 90.0 <= upperGonioLimit)
			{
				runDev.setStartAngle2(newStartAngle + 90.0, false);
			}
			else
			{
				runDev.setStartAngle2(newStartAngle - 90.0, false);
			}
		}
		else
		{
			// move end angle by the same amount start angle is moved, so the
			// frame count stays the same
			double newEndAngle = (newStartAngle-oldStartAngle)+run.getEndAngle();

			// skip for mesh mode, angles are handled by updateMeshParameters()
			if (run.getScanMode()!=ScanMode.MESH) {
				double upperGonioLimit = DevPmacMotor.get("gonio_omega").getLimits().getUpper();
				if (newEndAngle <= upperGonioLimit) {
					runDev.setStartAngle(newStartAngle, false);
					runDev.setEndAngle(newEndAngle, false);
				} else {
					Log.warn("Updating end angle would exceed upper limit (" + upperGonioLimit + ").  Please reduce gonio angle.");
				}
			}
		}

		if (run.getScanMode()==ScanMode.MESH)
			run.updateMeshParameters(-1, false);

		runDev.setDistance(DevPmacMotor.get("detector_z").getPosition(), false);
		runDev.setAttenuation_factors(DevAttenuation.get().getFactors(), false);
		runDev.setBeamstop_mm(DevPmacMotor.get("beamstop_z").getPosition(), false);

		if (0 ==run.getRunLabel())
		{
			ArrayList<Double> energies = new ArrayList<Double>();
			energies.add((Double)DevPmacMotor.get("mo_En_E").getPosition());
			runDev.setEnergies(energies,false);
		}
		else
		{
			double inflectionEnergy = 	DevScan.get().getInflection();
			double peakEnergy = 		DevScan.get().getPeak();
			double remoteEnergy = 		DevScan.get().getRemote();
			
			if (inflectionEnergy 	>= 1000.0)		inflectionEnergy 	/= 1000.0;
			if (peakEnergy 			>= 1000.0)		peakEnergy 			/= 1000.0;
			if (remoteEnergy 		>= 1000.0)		remoteEnergy 		/= 1000.0;
			
			ArrayList<Double> madEnergyList = new ArrayList<Double>();
			madEnergyList.add(peakEnergy);
			madEnergyList.add(inflectionEnergy);
			madEnergyList.add(remoteEnergy);

			ArrayList<Double> energies = new ArrayList<Double>();
			
			int index = 1;
			for (Double energy : madEnergyList)
			{
				if (energy > 0.0)
				{
					energies.add(energy);
					index++;
				}
			}
			if (1 == index)
			{
				energies.add((Double)DevPmacMotor.get("mo_En_E").getPosition());
			}
			run.fii.changeEnergies(energies,false);
		}
		
		runDev.sendUpdate();
	}
	
	@Override public void widgetDefaultSelected(SelectionEvent e) {}

	public void updateEnableState()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (isDisposed()) return;
			
			boolean remoteOpInProgress = false;
			if (null != TabCollect.get() && null != TabCollect.get().getRunTabFolder())
			{
				remoteOpInProgress = TabCollect.get().getRunTabFolder().getRemoteOpInProgress();
			}
			
			boolean enableCopy = true;
			if (0 == run.getRunIndex() || 15 == run.getRunIndex()) enableCopy = false;
			if (run.runList.getRunCount()>=15) enableCopy=false;
			
    		if (Client.isMaster())
    		{
    			switch(run.getRunState())
    			{
    			case INACTIVE:
        			copyButton.setEnabled(enableCopy);
        			updateButton.setEnabled(true);
        			deleteButton.setEnabled(!remoteOpInProgress && 0 != run.getRunIndex());
        			resetButton.setEnabled(0 != run.getNextFrame());
        			break;
    			case COLLECTING:
    			case PAUSING:
        			copyButton.setEnabled(false);
        			updateButton.setEnabled(false);
        			deleteButton.setEnabled(false);
        			resetButton.setEnabled(false);
    				break;
    			case PAUSED:
        			copyButton.setEnabled(enableCopy);
        			updateButton.setEnabled(false);
        			deleteButton.setEnabled(false);
        			resetButton.setEnabled(0 != run.getNextFrame());
    			case COMPLETE:
        			copyButton.setEnabled(enableCopy);
        			updateButton.setEnabled(false);
        			deleteButton.setEnabled(!remoteOpInProgress && 0 != run.getRunIndex());
        			resetButton.setEnabled(true);
    			}
    		}
    		else
    		{
    			copyButton.setEnabled(false);
    			updateButton.setEnabled(false);
    			deleteButton.setEnabled(false);
    			resetButton.setEnabled(false);
    		}
		}});
	}
	
	@Override
	public void masterChanged(boolean isMaster)
	{
		updateEnableState();
	}
}
