package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.OptionTab;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.syslog.Log;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgRunSubtabHeader extends Composite
{
	private		Label				lblSiteSetup, lblCollectParam;
	private		ModelCollectRun		run;
	private		WdgRunTab	runTab;
	
	public WdgRunSubtabHeader(Composite parent, ModelCollectRun run)
	{
		super(parent, SWT.NONE);
		this.run=run;
		this.runTab=(WdgRunTab)parent;
		
		GridLayout gl = new GridLayout(2,true);
		gl.marginHeight=0;
		gl.marginWidth=0;
		gl.horizontalSpacing=0;
		setLayout(gl);

		lblCollectParam = new Label(this, SWT.BORDER | SWT.CENTER);
		lblCollectParam.setText("Collect Param.");
		lblCollectParam.setFont(GUI.FONT_10);
		lblCollectParam.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		lblSiteSetup = new Label(this, SWT.BORDER | SWT.CENTER);
		lblSiteSetup.setText("Site Setup");
		lblSiteSetup.setFont(GUI.FONT_10);
		lblSiteSetup.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		setSubTab(OptionTab.COLLECT_PARAM);
	}
	
	public void connectListeners()
	{
		// sub-tab listeners
		if (null != lblSiteSetup) lblSiteSetup.addMouseListener(new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent e) {}
			@Override public void mouseUp(MouseEvent e) {}
			@Override public void mouseDown(MouseEvent e)
			{
				handleTabHeaderClick(OptionTab.SITE_SETUP);
			}
		});
		if (null != lblCollectParam) lblCollectParam.addMouseListener(new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent e) {}
			@Override public void mouseUp(MouseEvent e) {}
			@Override public void mouseDown(MouseEvent e)
			{
				handleTabHeaderClick(OptionTab.COLLECT_PARAM);
			}
		});
	}
	
	public void handleTabHeaderClick(OptionTab clickedTab)
	{
		if (clickedTab == OptionTab.SITE_SETUP && (run.getScanMode() == ScanMode.SINGLE || run.getScanMode() == ScanMode.JET))
		{
			//Log.note("Site setup is not applicable to standard collect mode.");
			return;
		}
		
		if (clickedTab == OptionTab.SITE_SETUP && run.getRunIndex() == 0)
		{
			Log.note("Site setup is not applicable to the snapshot tab.");
			return;
		}
		
		/*if (clickedTab == OptionTab.COLLECT_PARAM &&
			run.getScanMode() != ScanMode.SINGLE &&
			run.getSetupStep() == SetupStep.SITE_SETUP)
		{
			Log.note("Click 'Finish Site Setup' to continue");
			return;
		}*/
		
		run.dev().setOptionTab(clickedTab, true);
	}
	
	// called only from GUI thread
	public void setSubTab(OptionTab subTab)
	{
		Label lblUnselected, lblSelected;
		
		switch (subTab)
		{
		case SITE_SETUP:
			lblUnselected = lblCollectParam;
			lblSelected = lblSiteSetup;
			break;
			
		case COLLECT_PARAM:
			lblSelected = lblCollectParam;
			lblUnselected = lblSiteSetup;
			break;
			
		default:
			return;
		}
		
		lblSelected.setForeground(GUI.COLOR_BLACK);
		lblSelected.setBackground(GUI.COLOR_WHITE);
		lblUnselected.setForeground(GUI.COLOR_DARK_GRAY4);
		lblUnselected.setBackground(GUI.COLOR_LIGHT_GRAY);
		
		if (run.getScanMode() == ScanMode.SINGLE || run.getScanMode() == ScanMode.JET || run.getRunIndex() == 0)
		{
			lblSiteSetup.setText("---");
		}
		else
		{
			lblSiteSetup.setText("Site Setup");
		}
	}
	
	public void runChanged()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			setSubTab(run.getOptionTab());
		}});
	}
	
	// listeners will be disposed of along with label widgets
	public void removeListeners() {}
}
