package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.DevCollectRun;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.OptionTab;
import gov.anl.gmca.collect.ModelCollectRun.RunState;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DevRunList.DevRun;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.common.ui.GTabItem;
import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgRunTab extends Composite implements ClientListener, ModelCollectRun.Listener, ModelCollectRunList.Listener
{
	protected WdgRunTabs 			collectRunTabs;
	protected GTabItem 				tabItem;
	protected ModelCollectRun 		run;
	protected DevCollectRun 		runDev;
	protected int 					tabIndex;
	public 	  int 					getTabIndex() 		{ return tabIndex; }
	
	private   StackLayout 			stack;
	private   Composite 			subTabComp;
	private   WdgVectorSetup		vectorSetup;
	//private	  WdgRasterSiteSetup	rasterSetup;
	//private   WdgImportSetup		repeatSetup;
	private	  WdgMeshSetup			meshSetup;
	private   WdgCollectParams		collectParams;
	private	  WdgScanMode			scanModeSelector;
	
	protected Label 				runHeader;
	protected WdgRunButtons 		runButtons;
	private	  WdgRunSubtabHeader	setupTabs;
	
	public static class Factory implements WdgRunTabs.WdgRunTabFactory
	{
		@Override public Composite createRunTab(Composite parent, int style, int runIndex, ModelRun modelRun, DevRun devRun, WdgRunTabs runTabs, GTabItem tabItem)
		{
			return new WdgRunTab(parent,style,runIndex,(ModelCollectRun)modelRun,(DevCollectRun)devRun,runTabs,tabItem);
		}
	}
	
	public WdgRunTab(Composite parent, int style, int _tabIndex, ModelCollectRun _run, DevCollectRun _runDev, WdgRunTabs _collectRunTabs, GTabItem _tabItem)
	{
		super(parent,style);
		this.run=_run; this.runDev=_runDev; this.collectRunTabs=_collectRunTabs; this.tabItem=_tabItem; this.tabIndex=_tabIndex;
		
		compSetup();
		createRunHeader();
		createScanModeSelector();
		createTabHeader();
		createSubTabs();
		
		connectListeners();
	}

	private void compSetup()
	{
		setBackground(GUI.BGCOLOR);
		
		GridLayout layout = new GridLayout(1,false);
		layout.verticalSpacing=2;
		setLayout(layout);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		setLayoutData(gd);
	}
	
	private void createRunHeader()
	{
		runHeader = new Label(this, SWT.NONE);
		runHeader.setText("--loading--");
		runHeader.setBackground(GUI.BGCOLOR);
		runHeader.setFont(GUI.FONT_COLLECTRUN_HEADER);
		runHeader.setAlignment(SWT.CENTER);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		runHeader.setLayoutData(gd);
		
		///////////////////// run buttons /////////////////////
		
		runButtons = new WdgRunButtons(this,SWT.NONE, collectRunTabs, this, run, runDev);
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		runButtons.setLayoutData(gd);
	}
	
	private void createScanModeSelector()
	{
		if (0 != tabIndex)
		{
			scanModeSelector = new WdgScanMode(this, run);
			scanModeSelector.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 1, 1));
		}
	}
	
	private void createTabHeader()
	{
		setupTabs = new WdgRunSubtabHeader(this, run);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 20;
		setupTabs.setLayoutData(gd);
	}
	
	private void createSubTabs()
	{
		subTabComp = new Composite(this,SWT.NONE);
		subTabComp.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		subTabComp.setLayoutData(gd);
		stack = new StackLayout();
		stack.marginWidth=0;
		stack.marginHeight=0;
		subTabComp.setLayout(stack);
		
		vectorSetup =  new WdgVectorSetup(subTabComp, run);
		//rasterSetup = new WdgRasterSiteSetup(subTabComp, run);
		//repeatSetup = new WdgImportSetup(subTabComp, run);
		meshSetup = new WdgMeshSetup(subTabComp, run);
		collectParams = new WdgCollectParams(subTabComp, run, tabIndex);
	}
	
	protected static void createSpacer(Composite parent, int height, int hSpan)
	{
		Composite spacerComp = new Composite(parent, SWT.NONE);
		spacerComp.setBackground(parent.getBackground());
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = height;
		gd.horizontalSpan = hSpan;
		spacerComp.setLayoutData(gd);
	}
	
	private void connectListeners()
	{
		Client.addListener(this);
		run.addListener(this);
		run.runList.addListener(this);
		setupTabs.connectListeners();
		meshSetup.connectListeners();

		// dispose listener
		final WdgRunTab runTab = this;
		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent e)
			{
				Client.removeListener(runTab);
				runButtons.removeListeners();
				run.removeListener(runTab);
				run.runList.removeListener(runTab);
				setupTabs.removeListeners();
			}
		});
	}
	
	protected static GridData newGDEntry()
	{
		GridData gd = new GridData();
		gd.widthHint = 270;
		gd.grabExcessHorizontalSpace = true;
		return gd;
	}
	protected static GridData newGDLeftLabel()
	{
		GridData gd = new GridData();
		gd.widthHint = 95;
		gd.horizontalSpan = 2;
		return gd;
	}
	protected static GridData newGDRightLabel()
	{
		GridData gd = new GridData();
		gd.widthHint = 47;
		gd.horizontalSpan = 2;
		return gd;
	}

	@Override
	public void masterChanged(boolean isMaster)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			updateEnableState();
		}});
	}
	
	@Override
	public void runListChanged(ModelRunList runList)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			updateEnableState();
		}});
	}
	
	private OptionTab displayedSubTab = null;
	private ScanMode displayedSubTabScanMode = null;
	private void setSubTab(OptionTab subTab)
	{
		ScanMode scanMode = run.getScanMode();
		if (displayedSubTab==subTab && displayedSubTabScanMode==scanMode) return;
		displayedSubTab=subTab;
		displayedSubTabScanMode=scanMode;
		
		if (subTab == OptionTab.SITE_SETUP)
		{
			if (scanMode == ScanMode.VECTOR)		stack.topControl = vectorSetup;
			//else if (scanMode == ScanMode.RASTER)	stack.topControl = rasterSetup;
			//else if (scanMode == ScanMode.IMPORT)	stack.topControl = repeatSetup;
			else if (scanMode == ScanMode.MESH)		stack.topControl = meshSetup;
			else									stack.topControl = null;
		}
		else
		{
			stack.topControl = collectParams;
		}
		
		subTabComp.layout(true);

		setupTabs.setSubTab(subTab);
	}

	protected void updateEnableState()
	{
		if (isDisposed()) return;
		
		boolean gEnable =
		(
			Client.isMaster() &&
			run.getRunState() == RunState.INACTIVE
		);
		boolean isVectorSet =
		(
				null != run.getPt1() &&
				null != run.getPt2()
		);
		boolean isSiteDeltaEnabled = run.getSiteCheckEnable();
		
		if (null!=scanModeSelector)		scanModeSelector	.setEnabled(gEnable);
		
		if (null!=collectParams)		collectParams		.updateEnableState(gEnable,isVectorSet,isSiteDeltaEnabled);
		if (null!=vectorSetup)			vectorSetup			.updateEnableState(gEnable,isVectorSet,isSiteDeltaEnabled);
		//if (null!=rasterSetup)			rasterSetup			.updateEnableState(gEnable);
		//if (null!=repeatSetup)			repeatSetup			.updateEnableState(gEnable);
		if (null!=meshSetup)			meshSetup			.updateEnableState(gEnable);

		runButtons.updateEnableState();
	}
	
	// color scheme from SSRL BLU-ICE 3
	protected Color getTabColor(int runIndex, RunState state)
	{
		if (0 == runIndex)					return GUI.COLOR_BLACK;
		if (state == RunState.PAUSED) 		return GUI.COLOR_BROWNRED;
		if (state == RunState.COLLECTING)	return GUI.COLOR_RED;
		if (state == RunState.INACTIVE)		return GUI.COLOR_BLUE;
		if (state == RunState.COMPLETE)		return GUI.COLOR_DARK;
		
		throw new IllegalArgumentException("Unexpected state: "+state);
	}
	
	protected ScanMode displayedScanMode = ScanMode.SINGLE;
	
	@Override public void runChanged(final ModelCollectRun run)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (isDisposed()) return;
			
			// run header
			collectRunTabs.setTabColor(tabItem, getTabColor(run.getRunIndex(), run.getRunState()));
			collectRunTabs.setTabText(tabItem, " "+run.getRunLabel()+" ");
			runHeader.setText("Run "+run.getRunLabel()+" ("+run.getRunState().toString().toLowerCase()+")");

			// mode selector
			if (0 != run.getRunIndex())
			{
				ScanMode newScanMode = run.getScanMode();
				displayedScanMode = newScanMode;
				scanModeSelector.select(run.getScanMode());
			}

			setSubTab(run.getOptionTab());
			
			if (null != vectorSetup)		setupTabs.runChanged();
			if (null != collectParams) 		collectParams.runChanged(run);
			if (null != vectorSetup)		vectorSetup.runChanged(run);
			//if (null != rasterSetup)		rasterSetup.runChanged();
			///if (null != repeatSetup)		repeatSetup.runChanged(); // not needed
			if (null != meshSetup)			meshSetup.runChanged();

			updateEnableState();
		}});
	}
	
	public boolean directModelCopy()
	{
		if (!vectorSetup.updateModel())		return false;
		//if (!rasterSetup.updateModel())		return false;
		if (!meshSetup.updateModel())		return false;
		if (!collectParams.updateModel())	return false;
		
		return true;
	}
}
