package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgScanMode extends Composite
{
	private Button btnSingle, btnVector, btnRaster, btnRepeat, btnMesh, btnJet;
	private ModelCollectRun run;
	
	public WdgScanMode(Composite parent, ModelCollectRun run)
	{
		super(parent, SWT.NONE);
		this.run=run;
		
		setLayout(new FillLayout());
		setBackground(GUI.BGCOLOR);
		setBackgroundMode(SWT.INHERIT_FORCE);
		
		Group scanModeGroup = new Group(this, SWT.NONE);
		scanModeGroup.setText("Collection mode");
		scanModeGroup.setFont(GUI.FONT_COLLECT_GROUP);
		scanModeGroup.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(2,true);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.verticalSpacing=0;
		scanModeGroup.setLayout(gl);
		
		btnSingle = new Button(scanModeGroup, SWT.RADIO);
		btnSingle.setBackground(GUI.BGCOLOR);
		btnSingle.setText("Standard");
		btnSingle.setFont(GUI.FONT_10);
		btnSingle.setSelection(true);
		btnSingle.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		btnVector = new Button(scanModeGroup, SWT.RADIO);
		btnVector.setBackground(GUI.BGCOLOR);
		btnVector.setText("Vector");
		btnVector.setFont(GUI.FONT_10);
		btnVector.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		/*btnRaster = new Button(scanModeGroup, SWT.RADIO);
		btnRaster.setBackground(GUI.BGCOLOR);
		btnRaster.setText("Raster");
		btnRaster.setFont(GUI.FONT_10);
		btnRaster.setEnabled(false);
		btnRaster.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		btnRepeat = new Button(scanModeGroup, SWT.RADIO);
		btnRepeat.setBackground(GUI.BGCOLOR);
		btnRepeat.setText("Repeat");
		btnRepeat.setFont(GUI.FONT_10);
		btnRepeat.setEnabled(false);
		btnRepeat.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));*/

		btnMesh = new Button(scanModeGroup, SWT.RADIO);
		btnMesh.setBackground(GUI.BGCOLOR);
		btnMesh.setText("Mesh");
		btnMesh.setFont(GUI.FONT_10);
		btnMesh.setEnabled(false);
		btnMesh.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		if (Main.SHOW_JET_MODE)
		{
			btnJet = new Button(scanModeGroup, SWT.RADIO);
			btnJet.setBackground(GUI.BGCOLOR);
			btnJet.setText("Jet");
			btnJet.setFont(GUI.FONT_10);
			btnJet.setEnabled(false);
			btnJet.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		}

		connectListeners();
	}
	
	private void connectListeners()
	{
		btnSingle.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (btnSingle.getSelection()) run.fii.handleScanModeChange(ScanMode.SINGLE);
			}
		});
		btnVector.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (btnVector.getSelection()) run.fii.handleScanModeChange(ScanMode.VECTOR);
			}
		});
		/*btnRaster.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (btnRaster.getSelection()) run.fii.handleScanModeChange(ScanMode.RASTER);
			}
		});
		btnRepeat.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (btnRepeat.getSelection()) run.fii.handleScanModeChange(ScanMode.IMPORT);
			}
		});*/
		btnMesh.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (btnMesh.getSelection()) run.fii.handleScanModeChange(ScanMode.MESH);
			}
		});
		if (null!=btnJet) btnJet.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (btnJet.getSelection()) run.fii.handleScanModeChange(ScanMode.JET);
			}
		});
	}
	
	@Override public void setEnabled(boolean enable)
	{
		btnSingle.setEnabled(enable);
		btnVector.setEnabled(enable);
		//btnRaster.setEnabled(false);
		//btnRepeat.setEnabled(false);
		if (Main.SHOW_MESH_MODE)
			btnMesh.setEnabled(enable);
		else
			btnMesh.setEnabled(false);
		if (null!=btnJet) btnJet.setEnabled(enable);
	}
	
	public void select(ScanMode mode)
	{
		btnSingle.setSelection(mode==ScanMode.SINGLE);
		btnVector.setSelection(mode==ScanMode.VECTOR);
		//btnRaster.setSelection(mode==ScanMode.RASTER);
		//btnRepeat.setSelection(mode==ScanMode.IMPORT);
		btnMesh.setSelection(mode==ScanMode.MESH);
		if (null!=btnJet) btnJet.setSelection(mode==ScanMode.JET);
	}
}
