package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ModelRunList.CurrentRunListener;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgShutterlessIndicator extends Composite
{
	private Boolean globalEnable = null;
	private	Double currentDelta = null;
	private ScanMode currentRunScanMode = null;
	private Boolean currentRunSoniccMode = null;
	private boolean isCollectIndicator = false;
	private boolean isNonHelicalVector = false;
	private Integer currentRunLabel = null;
	private Label label;
	private Composite bgComp;
	
	private static final Color enabledBackground = GUI.COLOR_GREEN;
	private static final Color enabledForeground = GUI.COLOR_WHITE;
	private static final Color disabledBackground = GUI.COLOR_DARK_GRAY;
	private static final Color disabledForeground = GUI.COLOR_WHITE;
	private static final Font labelFont = GUI.FONT_10_BOLD;
	
	public WdgShutterlessIndicator(Composite parent, boolean isCollectIndicator)
	{
		super(parent, SWT.NONE);
		this.isCollectIndicator = isCollectIndicator;
		
		setBackground(GUI.COLOR_BLACK);
		
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth=gl.marginHeight=1;
		setLayout(gl);

		bgComp = new Composite(this, SWT.NONE);
		gl = new GridLayout(1,false);
		gl.marginWidth=gl.marginHeight=1;
		bgComp.setLayout(gl);
		bgComp.setBackground(GUI.BGCOLOR);
		bgComp.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		label = new Label(bgComp, SWT.CENTER);
		label.setText("Shutterless: ---");
		label.setFont(labelFont);
		label.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		label.setLayoutData(gd);
		
		globalEnable = DevUserConfig.get().getShutterlessEnable();
		updateLabel();
	}
	
	public void connectListeners()
	{
		DevUserConfig.get().ll.addListener(new ListenerList.Listener<DevUserConfig>()
		{
			@Override public void handleEvent(DevUserConfig dev, Object data)
			{
				boolean newState = dev.getShutterlessEnable();
				if (newState != globalEnable)
				{
					globalEnable = newState;
					updateLabel();
				}
			}
		});
		
		if (isCollectIndicator)
		{
			ModelCollectRunList.get().addCurrentRunListener(new CurrentRunListener()
			{
				@Override public void currentRunChanged(ModelRun run)
				{
					isNonHelicalVector = ((ModelCollectRun)run).seq().getIsNonHelicalVector();
					currentRunScanMode = ((ModelCollectRun)run).getScanMode();
					currentRunLabel = ((ModelCollectRun)run).getRunLabel();
					updateLabel();
				}
			});
		}
		else
		{
			ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
			{
				@Override public void handleEvent(ModelRasterState object, Object runObject)
				{
					ModelRasterRun run = ModelRasterState.get().getCurrentRunModel();
					currentDelta = run.getDelta();
					currentRunSoniccMode = run.getRunType() == ModelRasterRun.RunType.SONICC;
					updateLabel();
				}
			});
		}
	}

	private void updateLabel()
	{
		WdgShutterlessIndicator wdg = this;
		GUI.asyncExec(new Runnable(){public void run()
		{
			try
			{
				boolean hide = false;
				boolean enable = false;
				String text = null;
				if (isCollectIndicator)
				{
					if (null == currentRunScanMode) return;
					
					if (null!=currentRunLabel && currentRunLabel==0)
					{
						text = "Shutterless mode does not apply to run 0";
					}
					else if (globalEnable)
					{
						switch (currentRunScanMode)
						{
						case SINGLE:
						case JET:
							enable=true;
							text="Collect run "+currentRunLabel+" will be collected in shutterless mode.";
							break;
						case VECTOR:
							if (isNonHelicalVector)
							{
								enable=true;
								text="Collect run "+currentRunLabel+" will be collected in shutterless mode within each site.";
								break;
							}
							else
							{
								enable=true;
								text="Collect run "+currentRunLabel+" will be collected in shutterless helical mode.";
								//text = "Collect run "+currentRunLabel+" will be collected in shuttered mode.\nShutterless is not currently supported in helical mode.";
							}
							break;
						case RASTER:
							enable=true;
							text = "Collect run "+currentRunLabel+" will be collected in shutterless mode within each raster site.";
							break;
						}
					}
					else
					{
						text = "The global shutterless flag is disabled, all images will be taken in shuttered mode.";
					}
				}
				else // raster
				{
					if (null == currentDelta) return;

					if (currentRunSoniccMode) {
						hide = true;
					}
					else if (globalEnable)
    				{
						if (DBConfig.getBoolean("raster_helical") || currentDelta==0.0)
						{
	    					enable = true;
	    					text = "Each raster row will be collected in shutterless mode.";
						}
						else
						{
							text = Util.sprintf("Shutterless raster is only supported with still frames at this time. (Delta is set to %.3f in the current run)",currentDelta);
						}
    				}
    				else
    				{
    					text = "The global shutterless flag is disabled, each raster cell will be taken in shuttered mode.";
    				}
				}
				
				// text
				String enableText = enable ? "On" : "Off";
				label.setText("Shutterless: "+enableText);
				
				// tooltip
				label.setToolTipText(text);
				
				// colors
				bgComp.setBackground(enable ? enabledBackground : disabledBackground);
				label.setBackground(enable ? enabledBackground : disabledBackground);
				label.setForeground(enable ? enabledForeground : disabledForeground);

				WdgControl.setControlVisible(wdg, !hide);
				wdg.getParent().layout(true);
			}
			catch (Exception e) { e.printStackTrace(); }
		}});
	}
}
