package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;

public class WdgSiteList extends Composite implements gov.anl.gmca.collect.ModelCollectRun.Listener
{
	protected Table table;
	protected ModelCollectRun run;
	private Label countLabel;
	private Text fromField, toField;
	private Button removeButton, addButton/*, btnSelectNone, btnSelectAll*/;
	private String siteText = "", siteTextCapital = "";
	
	public WdgSiteList(Composite parent, final ModelCollectRun run, String siteText)
	{
		super(parent, SWT.NONE);
		
		this.run = run;
		this.siteText = siteText;
		this.siteTextCapital = siteText.substring(0, 1).toUpperCase()+siteText.substring(1);
		
		setBackground(GUI.BGCOLOR);
		setLayout(new GridLayout(3,false));
		
		VerifyListener numberVerify = new VerifyListener()
		{
			@Override public void verifyText(VerifyEvent ve)
			{
				if (ve.character==SWT.NONE || ve.character==SWT.BS || ve.character==SWT.DEL || ve.character==SWT.LF) return;
				
				if (ve.character<'0' && ve.character>'9')
				{
					Log.note("Only numbers are allowed in the "+siteText+" omit fields");
					ve.doit=false;
					return;
				}
			}
		};
		
		fromField = new Text(this,SWT.BORDER);
		fromField.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		fromField.addVerifyListener(numberVerify);
		
		Label toLabel = new Label(this, SWT.NONE);
		toLabel.setBackground(GUI.BGCOLOR);
		toLabel.setText(" to ");
		
		toField = new Text(this,SWT.BORDER);
		toField.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		toField.addVerifyListener(numberVerify);

		Composite buttonComp = new Composite(this, SWT.NONE);
		buttonComp.setBackground(GUI.BGCOLOR);
		buttonComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
		GridLayout gl = new GridLayout(2,true);
		gl.marginWidth=0;
		gl.marginHeight = 0;
		buttonComp.setLayout(gl);
		
		removeButton = new Button(buttonComp, SWT.PUSH);
		removeButton.setText("Deselect");
		removeButton.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		removeButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				setSelection(false);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		addButton = new Button(buttonComp, SWT.PUSH);
		addButton.setText("Select");
		addButton.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		addButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				setSelection(true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		table = new Table(this, SWT.BORDER);
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 3, 1));
		
		countLabel = new Label(this, SWT.CENTER);
		countLabel.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
		countLabel.setBackground(GUI.BGCOLOR);
		countLabel.setText("---");
		
		Composite selectComp = new Composite(this, SWT.NONE);
		selectComp.setBackground(getBackground());
		selectComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
		gl = new GridLayout(2,true);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		selectComp.setLayout(gl);
		
		/*btnSelectAll = new Button(selectComp, SWT.PUSH);
		btnSelectAll.setText("Select All");
		btnSelectAll.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		btnSelectAll.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				ArrayList<Boolean> allSites = new ArrayList<Boolean>();
				for (int i=0;i<run.getPotentialVectorSites();i++) allSites.add(true);
				run.dev().setSelectedSites(allSites, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		btnSelectNone = new Button(selectComp, SWT.PUSH);
		btnSelectNone.setText("Select None");
		btnSelectNone.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		btnSelectNone.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				ArrayList<Boolean> noSites = new ArrayList<Boolean>();
				for (int i=0;i<run.getPotentialVectorSites();i++) noSites.add(false);
				run.dev().setSelectedSites(noSites, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});*/
		
		// update GUI on PV change
		run.addListener(this);
	}

	private void setSelection(boolean selectFlag)
	{
		String fromText = fromField.getText();
		String toText = toField.getText();
		if (fromText.trim().length()==0) fromText=toText;
		if (toText.trim().length()==0) toText=fromText;

		Integer fromIndex=null,toIndex=null;
		try
		{
			fromIndex = Integer.parseInt(fromText)-1;
			toIndex = Integer.parseInt(toText)-1;
		}
		catch (NumberFormatException nfe)
		{
			nfe.printStackTrace(); // shouldn't happen, fields are constrained to numbers only
		}
		finally
		{
			fromField.setText("");
			toField.setText("");
			if (null==fromIndex||null==toIndex) return;
		}
		
		if (fromIndex>toIndex)
		{
			Util.asyncPopupError(siteTextCapital+" number error","From "+siteText+" number ("+(fromIndex+1)+") can't be greater than to "+siteText+" number ("+(toIndex+1)+")");
			return;
		}
		if (fromIndex<0)
		{
			Util.asyncPopupError(siteTextCapital+" number error","From "+siteText+" number ("+(fromIndex+1)+") can't be less than 1");
			return;
		}
		boolean isVector = (run.getScanMode() == ScanMode.VECTOR);
		int NOS = isVector ? run.getPotentialVectorSites() : run.getImportSiteCount();
		if (toIndex+1>NOS)
		{
			Util.asyncPopupError(siteTextCapital+" number error","To "+siteText+" number ("+(toIndex+1)+") can't be greater than the total number of "+siteText+"s ("+NOS+")");
			return;
		}
		
		ArrayList<Boolean> al = run.getSelectedSites();
		for (int i=fromIndex; i<=toIndex; i++)
		{
			al.set(i, selectFlag);
		}
		run.dev().setSelectedSites(al, true);
	}
	
	protected long lastSequenceVersion = 0;
	protected int lastRunIndex = -1;
	protected boolean sitesDisplayed = true;

	private void updateOmissionList()
	{
		if (null == table || table.isDisposed()) return;
		if (run.getScanMode() != ScanMode.VECTOR && run.getScanMode() != ScanMode.IMPORT) return;
		if (run.getScanMode() == ScanMode.VECTOR && run.getHelicalMode()) return;
		boolean isVector = (run.getScanMode() == ScanMode.VECTOR);
		
		
		
		// If a site is selected or deselected, this increments the sequence version
		// As does, of course, a change in the number of sites
		if (run.seq().getVersion()!=lastSequenceVersion || run.getRunIndex()!=lastRunIndex)
		{
			table.removeAll();
			int NOS = isVector ? run.getPotentialVectorSites() : run.getImportSiteCount();
			int selectedCount = run.getSelectedSiteCount();
			boolean allSitesSelected = true;
			
			Integer firstIndexOmittedThisBatch = null;
			for (int siteIndex=0;siteIndex<NOS;siteIndex++)
			{
				if (!run.isSiteSelected(siteIndex+1))
				{
					allSitesSelected = false;
					if (null==firstIndexOmittedThisBatch)
					{
						firstIndexOmittedThisBatch=siteIndex;
					}
				}
				else
				{
					if (null!=firstIndexOmittedThisBatch)
					{
						setOmitText(firstIndexOmittedThisBatch, siteIndex-1);
						firstIndexOmittedThisBatch = null;
					}
				}
			}
			if (null!=firstIndexOmittedThisBatch)
			{
				setOmitText(firstIndexOmittedThisBatch, NOS-1);
			}
			if (allSitesSelected)
			{
				TableItem ti = new TableItem(table, SWT.NONE);
				if (NOS>0)
					ti.setText("All "+siteText+"s selected");
				else if (run.getScanMode()==ScanMode.VECTOR)
					ti.setText("Vector not defined");
				else if (run.getScanMode()==ScanMode.IMPORT)
					ti.setText("Crystals not defined");
			}
			countLabel.setText(selectedCount+" of "+NOS+" "+siteText+"s selected");
		}
		
		lastSequenceVersion = run.seq().getVersion();
		lastRunIndex = run.getRunIndex();
	}
	
	private void setOmitText(Integer firstIndexOmitted, Integer lastIndexOmitted)
	{
		TableItem ti = new TableItem(table, SWT.NONE);
		if (firstIndexOmitted.equals(lastIndexOmitted))
			ti.setText(Integer.toString(firstIndexOmitted+1)+" omitted");
		else
			ti.setText((firstIndexOmitted+1)+"-"+(lastIndexOmitted+1)+" omitted");
	}
	
	@Override public void runChanged(final ModelCollectRun run)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			updateOmissionList();
		}});
	}
	
	public void clearToFromFields()
	{
		fromField.setText("");
		toField.setText("");
	}
	
	@Override public void setEnabled(boolean enable)
	{
		// clear to/from fields on enable transitions
		if (toField.getEnabled() != enable) toField.setText("");
		if (fromField.getEnabled() != enable) fromField.setText("");
		
		toField.setEnabled(enable);
		fromField.setEnabled(enable);
		removeButton.setEnabled(enable);
		addButton.setEnabled(enable);
		//btnSelectNone.setEnabled(enable);
		//btnSelectAll.setEnabled(enable);
		table.setEnabled(enable);
	}	
}
