package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.collect.ETA;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.RunState;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

public class WdgStartPause extends Composite
{
	protected Button btnCollect, btnPause;
	private Composite compCollect;
	//protected Button btnAbort;
	
	public WdgStartPause(Composite parent, int style)
	{
		super(parent, style);
		setLayout(new GridLayout());
		setBackground(parent.getBackground());

		////////////////////////////////////////////////////////////////////////////////////////////////////////////
		///////////////////////////////////////////// the three buttons ////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		compCollect = new Composite(this, SWT.NONE);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace=true;
		gd.widthHint = 106;
		compCollect.setLayoutData(gd);
		compCollect.setLayout(new FillLayout());
		
		btnCollect = WidgetFactory.newButton(compCollect, SWT.PUSH, "Collect", GUI.FONT_COLLECT_BUTTON, GUI.COLOR_BLUE_BUTTON);
		btnCollect.setToolTipText("Start collection for the current and subsequent runs");
		btnCollect.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				System.out.println("UserInput: Collect tab start button clicked");
				
				// during setFocus(), if an entry widget has focus, it will lose focus
				// and submit its value to the model before this method returns, guaranteeing
				// that the models will have up-to-date values for collection
				btnCollect.setFocus();
				
				ModelCollectRunList.get().startCollection();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		btnCollect.addMouseTrackListener(new MouseTrackAdapter()
		{
			@Override public void mouseEnter(MouseEvent e)
			{
				btnCollect.setToolTipText(ETA.getCollectTooltip());
			}
		});
		
		btnPause = WidgetFactory.newButton(this, SWT.PUSH, "Pause", GUI.FONT_COLLECT_BUTTON, GUI.COLOR_BLUE_BUTTON);
		btnPause.setToolTipText("Wait for the current frame to finish, then stop collection");
		gd = new GridData(); gd.horizontalAlignment = SWT.CENTER; gd.grabExcessHorizontalSpace=true; gd.widthHint = 106; btnPause.setLayoutData(gd); 
		btnPause.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				ModelCollectRunList.get().pauseCollection();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});

		/*btnAbort = WidgetFactory.newButton(this, SWT.PUSH, "Abort", GUI.FONT_COLLECT_BUTTON, GUI.COLOR_DARK_YELLOW);
		gd = new GridData(); gd.horizontalAlignment = SWT.CENTER; gd.grabExcessHorizontalSpace=true; gd.widthHint = 106; btnAbort.setLayoutData(gd); 
		btnAbort.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				DevShutter.get("shutter").close();
				DevFPE.get().sendAbort();
				
				if (null != ModelCollectRunList.get())
				{
					ModelCollectRunList.get().abortCollection();
				}
				if (null != ModelRasterState.get())
				{
					ModelRasterState.get().getCurrentRunModel().abort();
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});*/
		
		////////////////////////////////////////////////////////////////////////////////////////////////////////////
		///////////////////////////////////////////////// listeners ////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////////////////////////////////////////////

		ModelCollectRunList.get().addListener(new ModelRunList.Listener(){@Override public void runListChanged(ModelRunList runList)
		{
			updateButtonState();
		}});
		ModelCollectRunList.get().addCurrentRunListener(new ModelRunList.CurrentRunListener(){@Override public void currentRunChanged(ModelRun run)
		{
			updateButtonState();
		}});
		DevFPE.get().llFPEState.addListener(new ListenerList.Listener<DevFPE>()
		{
			@Override public void handleEvent(DevFPE object, Object data)
			{
				updateButtonState();
			}
		});
		AppState.addListener(AppState.Operation.START_COLLECT, new AppState.Listener()
		{
			@Override public void stateChanged(Operation op, boolean operationAllowed)
			{
				updateButtonState();
			}
		});
	}

	protected void updateButtonState()
	{
		boolean enableCollect	= false;
		boolean enablePause		= false;
		String 	pauseText		= "Pause";
		String	reasonText		= "";
		boolean	reasonIsError	= false;
		
		if (!Client.isMaster())
		{
			reasonText = "Client is not master.  Use the menu option\nNetwork->Become master by force to begin.";
			reasonIsError = true;
		}
		else
		{
			switch (ModelCollectRunList.get().getCollectState())
			{
			case INACTIVE:
				ModelCollectRun currentRun = ModelCollectRunList.get().getCurrentRunModel();
				if (null != currentRun)
				{
					enableCollect = (RunState.COMPLETE != currentRun.getRunState());
					if (!enableCollect) reasonText = "Run "+currentRun.getRunLabel()+" is complete";
					if (currentRun.getScanMode() != ScanMode.SINGLE &&
						currentRun.getScanMode() != ScanMode.JET &&
						currentRun.getSetupStep() == SetupStep.SITE_SETUP)
					{
						// don't allow clicking collect if sites aren't set up
						// in vector or raster mode
						enableCollect = false;
						reasonText = "Run "+currentRun.getRunLabel()+" is in site setup";
						reasonIsError = true;
					}
					if (enableCollect && currentRun.seq().getFrameListCopy().size()==0) {
						enableCollect = false;
						reasonText = "Run "+currentRun.getRunLabel()+" has zero frames";
						reasonIsError = true;
					}
				}
				break;
			
			case COLLECTING:
				enablePause=true;
				break;
			
			case PAUSING:
				pauseText="Pausing...";
				break;
			}
		}
		
		if (!AppState.isOperationAllowed(Operation.START_COLLECT) && enableCollect)
		{
			enableCollect = false;
			reasonText = "JBluIce is busy.  Check the application state by clicking the state display at the bottom left.";
		}
		
		if (!DevFPE.get().isRunning() && enableCollect)
		{
			enableCollect = false;
			reasonText = "FPE is not running.  Click the detector icon to start.";
		}
		
		if (!DevFPE.get().isIdle() && enableCollect)
		{
			enableCollect = false;
			reasonText = "FPE is not idle.";
		}
		
		final boolean f_enableCollect=enableCollect, f_enablePause=enablePause;
		final String f_pauseText=pauseText, f_reasonText=reasonText;
		GUI.asyncExec(new Runnable(){public void run()
		{
			btnCollect.setEnabled(f_enableCollect);
			btnPause.setEnabled(f_enablePause);
			btnPause.setText(f_pauseText);
			compCollect.setToolTipText(f_reasonText);
		}});
	}
}
