package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.DevCollectRun;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.Point3D;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.collect.ui.WdgSetPoint.SetPointActions;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgControlCheckbox;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

public class WdgVectorSetup extends Composite
{
	protected	WdgSetPoint			setPointStart, setPointEnd;
	protected	WdgControlCheckbox 	reverseDirCheckbox, helicalCheckbox;
	protected	Label 				reverseDirLabel,stepSizeLabel, siteCountLabel, helicalLabel;
	protected	WdgControlEntry 	stepSizeEntry, siteCountEntry;
	protected	Label 				reverseDirUnits,stepSizeUnits,siteCountUnits;
	private		WdgSiteList 		vectorSiteList;
	private		WdgCommitButton		commitButton;
	private		ModelCollectRun		run;
	private		Group				siteListGroup;
	private		Button				btnFitToVector;
	
	public WdgVectorSetup(Composite parent, final ModelCollectRun run)
	{
		super(parent, SWT.NONE);
		this.run=run;
		final DevCollectRun runDev = run.dev();
		
		setBackground(GUI.BGCOLOR);
		GridLayout paramLayout = new GridLayout(5,false);
		paramLayout.marginWidth = 0;
		paramLayout.marginHeight = 0;
		paramLayout.verticalSpacing = 0;
		setLayout(paramLayout);
		
		GridData gd;
		
		SetPointActions startPtActions = new SetPointActions() {
			@Override public void setPoint(Point3D point) 	{ run.fii.changeStartPoint(point,true); }
			@Override public void unsetPoint() 				{ run.fii.changeStartPoint(null,true); }
			@Override public Point3D getModelValue() 		{ return run.getPt1(); }
		};
		setPointStart = new WdgSetPoint(this, "Start point: ", run, startPtActions);
		setPointStart.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 5, 1));
		
		SetPointActions endPtActions = new SetPointActions() {
			@Override public void setPoint(Point3D point) 	{ run.fii.changeEndPoint(point,true); }
			@Override public void unsetPoint() 				{ run.fii.changeEndPoint(null,true); }
			@Override public Point3D getModelValue() 		{ return run.getPt2(); }
		};
		setPointEnd = new WdgSetPoint(this, "End point: ", run, endPtActions);
		setPointEnd.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 5, 1));
		
		reverseDirLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Reverse:", "Reverse vector direction"); reverseDirLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		reverseDirCheckbox = new WdgControlCheckbox(this, SWT.NONE, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ runDev.setReverseDir((Boolean)value,true);	}
		});
		reverseDirCheckbox.setLayoutData(WdgRunTab.newGDEntry());
		reverseDirUnits = WidgetFactory.newLabel(this, SWT.LEFT, ""); reverseDirUnits.setLayoutData(WdgRunTab.newGDRightLabel());
		
		helicalLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Helical:", "Frames will be evenly spaced along the vector and will not go past the endpoint"); helicalLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		helicalCheckbox = new WdgControlCheckbox(this, SWT.NONE, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				run.fii.changeHelicalMode((Boolean)value, true);
			}
		});
		helicalCheckbox.setLayoutData(WdgRunTab.newGDEntry());
		WidgetFactory.newLabel(this, SWT.LEFT, "").setLayoutData(WdgRunTab.newGDRightLabel());
		
		stepSizeLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Step size:"); stepSizeLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		stepSizeEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.fii.changeStepSize_mm((Double)value/1000.0,true);	}
			@Override public Object getModelValue()			{ return run.getStepSize_mm()*1000.0;			}
			@Override public Limits getLimits() 			{ return new Limits(0.1,1000,"step size"); 		}
			@Override public int getDecimals() 				{ return 2; 									}
		});
		stepSizeEntry.setLayoutData(WdgRunTab.newGDEntry());
		stepSizeUnits = WidgetFactory.newLabel(this, SWT.LEFT, "\u00B5m"); stepSizeUnits.setLayoutData(WdgRunTab.newGDRightLabel());
		
		siteCountLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "# of sites:"); siteCountLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		siteCountEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.fii.changeSiteCount((Integer)value, true);			}
			@Override public Object getModelValue()			{ return run.getPotentialVectorSites();			}
			@Override public Limits getLimits() 			{ return new Limits(1,1000,"site count"); 		}
		});
		siteCountEntry.setLayoutData(WdgRunTab.newGDEntry());
		siteCountUnits = WidgetFactory.newLabel(this, SWT.LEFT, ""); siteCountUnits.setLayoutData(WdgRunTab.newGDRightLabel());
		
		WdgRunTab.createSpacer(this, 10, 5);
		
		btnFitToVector = new Button(this, SWT.PUSH);
		btnFitToVector.setText("Fit sites to vector");
		btnFitToVector.setBackground(GUI.COLOR_SAMPLE_BG);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.horizontalSpan=5;
		gd.widthHint = 160;
		btnFitToVector.setLayoutData(gd);
		btnFitToVector.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				run.fitSitesToVector(false, true, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		WdgRunTab.createSpacer(this, 3, 5);
		
		commitButton = new WdgCommitButton(this, run);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.horizontalSpan=5;
		gd.widthHint = 162;
		commitButton.setLayoutData(gd);
		
		WdgRunTab.createSpacer(this, 10, 5);
		
		siteListGroup = new Group(this, SWT.NONE);
		siteListGroup.setText("Vector sites");
		siteListGroup.setBackground(getBackground());
		gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 5;
		siteListGroup.setLayoutData(gd);
		siteListGroup.setLayout(new GridLayout(1,false));
		
		vectorSiteList = new WdgSiteList(siteListGroup, run, "site");
		gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan=5;
		vectorSiteList.setLayoutData(gd);
	}
	
	public void updateEnableState(boolean gEnable, boolean isVectorSet, boolean isSiteDeltaEnabled)
	{
		boolean areSitesSetup 	= (SetupStep.COLLECT_PARAM == run.getSetupStep());
		boolean helical			= run.getHelicalMode();
		
		setPointStart			.setEnabled(gEnable && !areSitesSetup);
		setPointEnd				.setEnabled(gEnable && !areSitesSetup);
		reverseDirCheckbox		.setEnabled(gEnable && !areSitesSetup && isVectorSet);
		helicalCheckbox			.setEnabled(gEnable && !areSitesSetup);
		commitButton			.setEnabled(gEnable && isVectorSet && 0!=run.getSelectedSiteCount());
		btnFitToVector			.setEnabled(gEnable && !areSitesSetup && isVectorSet);
		
		/*WdgControl.setControlVisible(stepSizeLabel, 	!helical);
		WdgControl.setControlVisible(stepSizeEntry, 	!helical);
		WdgControl.setControlVisible(stepSizeUnits, 	!helical);
		WdgControl.setControlVisible(siteCountLabel, 	!helical);
		WdgControl.setControlVisible(siteCountEntry, 	!helical);
		WdgControl.setControlVisible(siteCountUnits, 	!helical);*/

		WdgControl.setControlVisible(siteListGroup, 	!helical);
		
		vectorSiteList			.setEnabled(gEnable && !areSitesSetup && isVectorSet);
		stepSizeEntry			.setEnabled(gEnable && !areSitesSetup && isVectorSet && !helical);
		siteCountEntry			.setEnabled(gEnable && !areSitesSetup && isVectorSet && !helical);
	}
	
	public void runChanged(ModelCollectRun run)
	{
		reverseDirCheckbox		.setSelected(run.getReverseDir());
		helicalCheckbox			.setSelected(run.getHelicalMode());
		stepSizeEntry			.setText(run.getStepSize_mm()*1000.0);
		siteCountEntry			.setText(run.getPotentialVectorSites());
		commitButton.runChanged();
	}
	
	public boolean updateModel()
	{
		if (!reverseDirCheckbox.updateModel()) 		return false;
		if (!helicalCheckbox.updateModel()) 		return false;
		if (!stepSizeEntry.updateModel())			return false;
		
		return true;
	}
}
