package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.RunCalculator.Frame;
import gov.anl.gmca.collect.ui.cam.WdgVectorCamera;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ModelRunList.CurrentRunListener;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.GrpWdgMove;
import gov.anl.gmca.hutch.WdgMotor;
import gov.anl.gmca.sample.DBCamera;
import gov.anl.gmca.sample.WdgCamera.CameraSize;
import gov.anl.gmca.sample.WdgCamera.CameraType;
import gov.anl.gmca.sample.WdgSampleRotate;
import gov.anl.gmca.sample.WdgSampleRotate.GetRotateN;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;

public class WdgVectorTab extends Composite
{
	private		WdgVectorCamera		vectorCamera;
	private		WdgSampleRotate 	rotate;
	private		Label 				vectorLengthLabel, stepSizeLabel;
	private		Button 				checkboxShowVectorNumbers;
	private		Link				siteInfoLink;
	private		Composite			siteInfoContainer;
	
	protected double rotateNValue = 1.0;
	
	public WdgVectorTab(Composite parent)
	{
		super(parent, SWT.NONE);
		
		setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(2,false);
		gl.marginWidth = 0;
		setLayout(gl);
		
		siteInfoContainer = new Composite(this, SWT.BORDER);
		siteInfoContainer.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1));
		gl = new GridLayout(1,false);
		gl.marginHeight=0;
		siteInfoContainer.setLayout(gl);
		siteInfoContainer.setBackground(GUI.COLOR_LIGHT_GRAY);
		
		siteInfoLink = new Link(siteInfoContainer, SWT.NONE);
		siteInfoLink.setLayoutData(new GridData(SWT.CENTER, SWT.DEFAULT, true, false));
		siteInfoLink.setBackground(GUI.COLOR_LIGHT_GRAY);
		siteInfoLink.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				Util.asyncPopupOk(savedNewMsg, popupMsg);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		WdgControl.setControlVisible(siteInfoContainer, false);

		vectorCamera = new WdgVectorCamera(this, SWT.NONE, DBCamera.getImageURL(CameraType.HIGHRES, CameraSize.LARGE), CameraType.HIGHRES);
		GridData gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		gd.horizontalSpan = 2;
		vectorCamera.setLayoutData(gd);
		
		Composite zoomComp = new Composite(this, SWT.NONE);
		zoomComp.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.CENTER,SWT.BOTTOM,true,false);
		gd.widthHint = 220;
		zoomComp.setLayoutData(gd);
		GridLayout zoomLayout = new GridLayout(2,true);
		zoomLayout.horizontalSpacing=0;
		zoomLayout.marginWidth=20;
		zoomComp.setLayout(zoomLayout);
		
		GrpWdgMove moveGroup = new GrpWdgMove();
		
		WdgMotor wdgZoom = new WdgMotor(zoomComp,SWT.NONE,moveGroup,"Zoom",null,"camera_zoom_2", parent.getBackground(), new double[]{ 1, 2, 4, 6, 8, 10, 12 }, 2);
		wdgZoom.setBackground(parent.getBackground());
		wdgZoom.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1));

		Button startButton = new Button(zoomComp, SWT.PUSH);
		startButton.setText("Start");
		startButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		startButton.setFont(GUI.FONT_COLLECTRUN_BUTTON);
		moveGroup.setStartButton(startButton);
		startButton.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		Button cancelButton = new Button(zoomComp, SWT.PUSH);
		cancelButton.setText("Cancel");
		cancelButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		cancelButton.setFont(GUI.FONT_COLLECTRUN_BUTTON);
		moveGroup.setCancelButton(cancelButton);
		cancelButton.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		Composite rotateComp = new Composite(this, SWT.NONE);
		rotateComp.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.CENTER,SWT.BOTTOM,true,false);
		gd.widthHint = 220;
		rotateComp.setLayoutData(gd);
		GridLayout rotateLayout = new GridLayout(3,true);
		rotateLayout.marginWidth = 20;
		rotateComp.setLayout(rotateLayout);
		
		Group nGroup = new Group(rotateComp, SWT.NONE);
		nGroup.setBackground(parent.getBackground());
		nGroup.setText("Rotate +-n");
		nGroup.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData(SWT.FILL,SWT.DEFAULT,true,false, 3, 1);
		nGroup.setLayoutData(gd);
		
		nGroup.setLayout(new GridLayout(2,false));
		
		final WdgControlEntry rotateNEntry = new WdgControlEntry(nGroup, SWT.NONE, EntryType.RW_COMBO, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				String s = (String)value;
				try {
					rotateNValue = Double.parseDouble(s);
				}
				catch (NumberFormatException nfe) {
					Log.warn("Invalid angle: "+s);
				}
			}
			@Override
			public double[] getFloatChoices()
			{
				return new double[]{.1,.5,1,5,10,30,45,60,90,180};
			}
			@Override public int getDecimals() 				{ return 2; 		}
			@Override public Limits getLimits() 			{ return DevPmacMotor.get("gonio_omega").getLimits(); }
		});
		rotateNEntry.setText(1.0);
		gd = new GridData();
		gd.widthHint = 100;
		rotateNEntry.setLayoutData(gd);
		rotateNEntry.setEnabled(true);
		
		Label nLabel = new Label(nGroup, SWT.NONE);
		nLabel.setText("deg");
		nLabel.setBackground(parent.getBackground());
		gd = new GridData();
		nLabel.setLayoutData(gd);
		
		rotate = new WdgSampleRotate(rotateComp, SWT.HORIZONTAL, new GetRotateN()
		{
			@Override public double getRotateN() { return rotateNValue; }
		});
		rotate.setLayoutData(new GridData(SWT.CENTER, SWT.DEFAULT, true, false, 3, 1));
		
		stepSizeLabel = new Label(this, SWT.CENTER);
		stepSizeLabel.setText("Step size = n/a");
		stepSizeLabel.setBackground(parent.getBackground());
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		stepSizeLabel.setLayoutData(gd);
		
		vectorLengthLabel = new Label(this, SWT.CENTER);
		vectorLengthLabel.setText("Vector length = n/a");
		vectorLengthLabel.setBackground(parent.getBackground());
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		vectorLengthLabel.setLayoutData(gd);
		
		checkboxShowVectorNumbers = new Button(this, SWT.CHECK);
		checkboxShowVectorNumbers.setText("Show vector site numbers");
		checkboxShowVectorNumbers.setBackground(parent.getBackground());
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, false, false);
		gd.horizontalSpan = 2;
		checkboxShowVectorNumbers.setLayoutData(gd);
		checkboxShowVectorNumbers.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se){}
			@Override public void widgetSelected(SelectionEvent se)
			{
				boolean showNumbers = checkboxShowVectorNumbers.getSelection();
				ModelCollectRunList.get().setShowVectorNumbers(showNumbers);
			}
		});
		checkboxShowVectorNumbers.setSelection(ModelCollectRunList.get().getShowVectorNumbers());
	}
	
	private String savedNewMsg = "", popupMsg = "";
	
	public void connectListeners()
	{
		rotate.connectListeners();
		
		ModelCollectRunList.get().addCurrentRunListener(new CurrentRunListener()
		{
			@Override public void currentRunChanged(ModelRun run)
			{
				ModelCollectRun cr = (ModelCollectRun)run;
				ArrayList<Frame> frameList = cr.seq().getFrameListCopy();
				
				/////////////////////// site info link /////////////////////
				
				// determine if all sites are completely filled
				int highestSiteIndexUsed = 0, siteCount=cr.getSelectedSiteCount(), lastSiteIndex=siteCount-1;
				int lastUsedSiteFramesUsed = 0;
				int framesPerSite = cr.getFramesPerSite();
				for (int frameIndex=0; frameIndex<frameList.size(); frameIndex++)
				{
					int siteIndex = frameList.get(frameIndex).siteNumber;
					lastUsedSiteFramesUsed++;
					
					if (highestSiteIndexUsed<siteIndex)
					{
						// new site: increment site count, reset partial frame count
						highestSiteIndexUsed=siteIndex;
						lastUsedSiteFramesUsed=1;
					}
				}
				
				// set the notification message
				String newMsg = "";
				popupMsg = "";
				if (cr.getSetupStep() == SetupStep.COLLECT_PARAM)
				{
					int suggestedEndFrame = siteCount*framesPerSite + cr.getStartFrameLabel()-1;
					
					if (highestSiteIndexUsed!=lastSiteIndex)
					{
						boolean partiallyFilledSite = (lastUsedSiteFramesUsed!=framesPerSite);
						int unusedSiteCount = lastSiteIndex-highestSiteIndexUsed;
						
						newMsg = "Unused sites exist";
						popupMsg = "The last "+unusedSiteCount+" sites are not filled with frames.\n";
						if (partiallyFilledSite) popupMsg += "Additionally, one site is partially filled.\n";
						popupMsg += "To fill them, set the end frame to "+suggestedEndFrame+".";
					}
					else if (lastUsedSiteFramesUsed < framesPerSite)
					{
						newMsg = "Last site not filled";
						popupMsg = "The last site has less frames than the others.\n";
						popupMsg += "To fill the last site, increase the end frame to "+suggestedEndFrame+".";
					}
				}
				
				// show or hide the link widget
				final String f_newMsg = newMsg;
				if (!savedNewMsg.equals(newMsg))
				{
					GUI.asyncExec(new Runnable(){public void run()
					{
						boolean showInfo = (f_newMsg.length()>0);
						siteInfoLink.setText(f_newMsg + " <a>(more info)</a>");
						WdgControl.setControlVisible(siteInfoContainer, showInfo);
						siteInfoContainer.layout(true);
						layout(true);
						savedNewMsg = f_newMsg;
					}});
				}
				
				//////////////////////// vector length /////////////////////
				
				String vText = "Vector length = ";
				Double vectorLength = cr.getVectorLength_mm();
				if (null == vectorLength) 	vText += "n/a";
				else						vText += Math.round(vectorLength*1000.0)+" \u00B5m";
				final String f_vText = vText;
				GUI.asyncExec(new Runnable(){public void run()
				{
					vectorLengthLabel.setText(f_vText);
				}});
				
				////////////////////////// step size ////////////////////////
				
				String ssText = "Step size = ";
				Double stepSizeMM = cr.getStepSize_mm();
				if (null == stepSizeMM)	ssText += "n/a";
				else
				{
					double stepSizeUm = stepSizeMM * 1000.0;
					int numDecimals = stepSizeUm >= 100 ? 0 : stepSizeUm >= 10 ? 1 : stepSizeUm >= 1 ? 2 : stepSizeUm >= .1 ? 3 : 4;
					String ssNumText = Util.printDouble(stepSizeUm, numDecimals); 
					ssText += ssNumText+" \u00B5m";
				}
				final String f_ssText = ssText;
				GUI.asyncExec(new Runnable(){public void run()
				{
					stepSizeLabel.setText(f_ssText);
				}});
			}
		});
	}

	public void notifyIsVisible(boolean isVisible)
	{
		vectorCamera.notifyIsVisible(isVisible);
	}
}
