package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.collect.DevAdxvFile;
import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;
import java.util.HashMap;

public class ExternalViewer
{
	public int index;
	public String name;
	public String cmdLine;
	
	private static ArrayList<ExternalViewer> viewerList;
	
	public static void init()
	{
		try
		{
			viewerList = new ArrayList<>();
	    	Result result = Database.exec(DBName.CONFIG, "select * from ImageViewer order by `index`");
			for (int i=0;i<result.numRows();i++)
			{
				HashMap<String,String> row = result.getRow(i);
				ExternalViewer v = new ExternalViewer();
				v.index = Integer.parseInt(row.get("index"));
				v.name = row.get("name");
				v.cmdLine = row.get("cmdLine");
				viewerList.add(v);
			}
		}
		catch (Exception e) { e.printStackTrace(); Log.error("Couldn't load external viewer list"); }
	}
	
	public static ArrayList<ExternalViewer> getAll()
	{
		if (null==viewerList) init();
		return viewerList;
	}
	
	public static ExternalViewer get(int index)
	{
		return viewerList.get(index);
	}
	
	public static void launch(String imagePath)
	{
		DevAdxvFile.update(imagePath);
		
		String displayCmd = DevUserConfig.get().getDisplayProgram().cmdLine;
		
		// add the current filename to the command
		if (null==imagePath)
		{
			imagePath = DevFPE.get().getImageViewerFile().getFullPath();
		}
		if (imagePath==null) imagePath="";
		if (displayCmd.contains("%s")) // we don't send filenames to all viewers
		{
			displayCmd = Util.sprintf(displayCmd, imagePath);
		}
		
		// TODO: don't rely on parsing name
		if (displayCmd.contains("dials_viewer") && displayCmd.contains("--spots"))
		{
			String highResLimit = Util.sprintf("%.2f",ModelRasterState.get().getCurrentRunModel().getHighResLimit_A());
			String lowResLimit =  Util.sprintf("%.2f",ModelRasterState.get().getCurrentRunModel().getLowResLimit_A());
			displayCmd += " spotfinder.filter.d_min="+highResLimit;
			displayCmd += " spotfinder.filter.d_max="+lowResLimit;
		}

		(ExecTask.new_ExecTask(displayCmd,"viewer",false,false)).start();
	}
}
