package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.widgets.Label;

public class ListenerResDisplay implements MouseMoveListener, MouseTrackListener
{
	private WdgDiffViewer viewer;
	
	public ListenerResDisplay(WdgDiffViewer viewer)
	{
		this.viewer = viewer;
		
		Label label = new Label(viewer,SWT.BORDER);
		label.setBackground(GUI.COLOR_WHITE);
		label.setForeground(GUI.COLOR_BLACK);
		label.setFont(GUI.FONT_DIFFIMAGE_MSG);
		label.setLocation(0, viewer.model.titleHeight);
		label.setSize(viewer.labelWidth, viewer.labelHeight);
		label.setVisible(false);
		label.setText("Loading...");
		viewer.label = label;
		
		viewer.addMouseMoveListener(this);
		viewer.addMouseTrackListener(this);
	}
	
	@Override public void mouseMove(MouseEvent e)
	{
		if (!viewer.lPan.inPanDrag)
		{
			updateMessage(e.x, e.y-viewer.model.titleHeight);
		}
	}
	
	@Override public void mouseEnter(MouseEvent e)
	{
		// not sure why asyncExec is needed here, but without it, JBI freezes on mouse exit/enter
		GUI.asyncExec(new Runnable(){public void run()
		{
			viewer.label.setVisible(true);
		}});
	}

	@Override
	public void mouseExit(MouseEvent e)
	{
		// see note in mouseEnter
		GUI.asyncExec(new Runnable(){public void run()
		{
			viewer.label.setVisible(false);
		}});
	}

	// ported from Diffimage.tcl's handle_mouse_motion
	// x and y are relative to the image's top left
	protected void updateMessage(int x, int y)
	{
		if (null == viewer.model.currentImage)
		{
			viewer.label.setText("Loading...");
			return;
		}
		
		// convert mouse position from px to mm
		double mouseMMFromLeft = x * viewer.model.currentImage.mmPerPixel;
		double mouseMMFromTop = y * viewer.model.currentImage.mmPerPixel;
		
		// calc mouse distance from beam center
		double deltaX_mm = mouseMMFromLeft - viewer.model.currentImage.beam_center_mm_from_left;
		double deltaY_mm = mouseMMFromTop - viewer.model.currentImage.beam_center_mm_from_top;
		double radius_mm = Math.sqrt(deltaX_mm*deltaX_mm + deltaY_mm*deltaY_mm);

		double twoTheta = Math.atan(radius_mm / viewer.model.currentImage.distance_mm);
		double dSpacing = viewer.model.currentImage.wavelength_A / (2*Math.sin(twoTheta/2.0));

		viewer.label.setText(Util.sprintf("%.2f A", dSpacing));
	}

	@Override public void mouseHover(MouseEvent arg0) {}
}
