package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.collect.DevFPE.ViewerFileInfo;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LoadAdjacentFrame
{
	private WdgDiffViewer viewer;
	private Pattern cbfPattern;
	
	public LoadAdjacentFrame(WdgDiffViewer viewer)
	{
		this.viewer = viewer;
		cbfPattern = Pattern.compile("(\\S+)_(\\d+).cbf");
	}
	
	public void loadNext() 		{ loadAdjacentFrame(+1);	}
	public void loadPrevious()	{ loadAdjacentFrame(-1); 	}
	
	/////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////
	
	protected void loadAdjacentFrame(int increment)
	{
		String description = increment==1?"next":"previous";
		String currentFilename = viewer.model.currentFilename.filename;
		String nextDir = DevFPE.get().getImageViewerDir();
		String nextFilename = "";

		// determine the next or previous filename
		Matcher cbfMatcher = cbfPattern.matcher(currentFilename);
		if (cbfMatcher.matches())
		{
			String prefix = cbfMatcher.group(1);
			int fileNumber = Integer.parseInt(cbfMatcher.group(2));
			nextFilename = Util.sprintf("%s_%06d.cbf", prefix, fileNumber+increment);
		}
		String nextFile = nextDir+"/"+nextFilename;
		
		// if incremented filename doesn't exist, search the sorted directory listing for
		// valid filenames in the next or previous direction
		if (!Disk.exists(nextFile))
		{
			String[] dirListing = null;
		    
			// read current directory listing and sort
		    if(!Disk.canRead(nextDir)) {
		    	Log.note(nextDir + "don't have read permissions");
		    	return;
		    }
		    dirListing = Disk.getDir(nextDir);
		    Arrays.sort(dirListing, 0, dirListing.length);

		    // find the current filename in the list
			int i = 0;
			for (; i < dirListing.length; i++)
			{
				if(currentFilename.equals(dirListing[i])) {
					break;
				}
			}
			
			// look for any valid filenames in the sorted direction (+ or -)
			boolean fileFound = false;
			while (i+increment < dirListing.length && i+increment >= 0)
			{
				if(cbfPattern.matcher(dirListing[i+increment]).matches())
				{
					fileFound = true;
					break;
				}
				else
					i+=increment;
			} 
			if(fileFound)
				nextFilename = dirListing[i+increment];
			else
				return;
		}
		if(nextFilename.equals(currentFilename)) return;
		nextFile = nextDir+"/"+nextFilename;
		
		if (!Disk.exists(nextFile))
		{
			String Description = description.substring(0,1).toUpperCase()+description.substring(1);
			Log.warn(Description+" file \""+nextFilename+"\" does not exist.");
			return;
		}
		
		// either set the local filename or set the FPE PV
		if (viewer.model.followFPE)
		{
			DevFPE.get().setImageViewerFilePath(nextDir,nextFilename);
		}
		else
		{
			viewer.model.currentFilename = new ViewerFileInfo(nextFilename,nextDir,0);
			viewer.model.currentFullPath = nextDir+"/"+nextFilename;
			viewer.model.requestReload();
		}
	}
}
