package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.EpicsDevice;
import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.common.ui.WdgControlEntry.ModelInterface;
import gov.anl.gmca.diffimage.*;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.options.*;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class WdgDiffControls extends Composite
{
	private int buttonHeight = 24;
	private int heightMargin = 2;
	
	protected WdgDiffViewer diffViewer;
	private Button openButton, updateButton, forwardButton, backwardButton, resetButton, smallspotsButton;
	Button resetSFButton;
	private Button zoomInButton, zoomOutButton, contrastDown, contrastUp;
	WdgControlEntry zoomEntry, contrastEntry;
	
	public WdgDiffControls(Composite parent, int style, int isHorizontal)
	{
		super(parent, style|SWT.BORDER);
		
		if(isHorizontal == 1)
		{
			GridLayout gl = new GridLayout(6,false);
			gl.marginHeight=heightMargin;
			gl.marginWidth=0;
			setLayout(gl);
		}
		else setLayout(new GridLayout(1,false));
		
		// open
		Composite openComp = new Composite(this, SWT.NONE);
		openComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth=0;
		gl.marginHeight=heightMargin;
		gl.horizontalSpacing = 0;
		openComp.setLayout(gl);
		openComp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		
		Label openLabel = new Label(openComp, SWT.CENTER);
		openLabel.setBackground(GUI.BGCOLOR);
		openLabel.setFont(GUI.FONT_COLLECT_BUTTON2);
		openLabel.setText("File");
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalSpan = 3;
		openLabel.setLayoutData(gd);
		
		openButton = new Button(openComp, SWT.PUSH);
		openButton.setFont(GUI.FONT_11);
		openButton.setText("Open");
		openButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.heightHint = buttonHeight;
		openButton.setLayoutData(gd);
		
		// contrast
		Composite contrastComp = new Composite(this, SWT.NONE);
		contrastComp.setBackground(GUI.BGCOLOR);
		gl = new GridLayout(3, false);
		gl.marginHeight=heightMargin;
		contrastComp.setLayout(gl);
		contrastComp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		
		Label contrastLabel = new Label(contrastComp, SWT.CENTER);
		contrastLabel.setFont(GUI.FONT_COLLECT_BUTTON2);
		contrastLabel.setBackground(GUI.BGCOLOR);
		contrastLabel.setText("Contrast");
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		gd.horizontalSpan = 3;
		contrastLabel.setLayoutData(gd);
		
		int buttonWidth=27, buttonHeight=30, entryWidth = 50;
		
		contrastDown = new Button(contrastComp, SWT.ARROW|SWT.LEFT);
		contrastDown.setBackground(GUI.COLOR_BLUE_BUTTON);
		contrastDown.setFont(GUI.FONT_COLLECT_BUTTON2);
		gd = new GridData();
		gd.widthHint = buttonWidth;
		gd.heightHint = buttonHeight;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		contrastDown.setLayoutData(gd);
		
		contrastEntry = new WdgControlEntry(contrastComp, SWT.NONE, EntryType.TEXT, new ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				try {
					diffViewer.model.setContrast(Double.parseDouble((String) value));
				} catch (Exception e) { e.printStackTrace(); }
			}
		});
		contrastEntry.setBackground(GUI.COLOR_WHITE);
		contrastEntry.setFont(GUI.FONT_10);
		contrastEntry.setEnabled(true);
		contrastEntry.setText(DetectorConfig.isDectris() ? 10.0 : 200.0);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = entryWidth;
		contrastEntry.setLayoutData(gd);
		
		contrastUp = new Button(contrastComp, SWT.ARROW|SWT.RIGHT);
		contrastUp.setBackground(GUI.COLOR_BLUE_BUTTON);
		contrastUp.setFont(GUI.FONT_COLLECT_BUTTON2);
		gd = new GridData();
		gd.widthHint = buttonWidth;
		gd.heightHint = buttonHeight;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		contrastUp.setLayoutData(gd);
		
		// zoom
		Composite zoomComp = new Composite(this, SWT.NONE);
		zoomComp.setBackground(GUI.BGCOLOR);
		gl = new GridLayout(3, false);
		gl.marginHeight=heightMargin;
		zoomComp.setLayout(gl);
		zoomComp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		
		Label zoomLabel = new Label(zoomComp, SWT.CENTER);
		zoomLabel.setBackground(GUI.BGCOLOR);
		zoomLabel.setFont(GUI.FONT_COLLECT_BUTTON2);
		zoomLabel.setText("Zoom");
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalSpan = 3;
		zoomLabel.setLayoutData(gd);
		
		zoomOutButton = new Button(zoomComp, SWT.ARROW|SWT.LEFT);
		zoomOutButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		zoomOutButton.setFont(GUI.FONT_COLLECT_BUTTON2);
		gd = new GridData();
		gd.widthHint = buttonWidth;
		gd.heightHint = buttonHeight;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		zoomOutButton.setLayoutData(gd);
		
		zoomEntry = new WdgControlEntry(zoomComp, SWT.NONE, EntryType.TEXT, new ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				try {
					diffViewer.model.setZoom(Double.parseDouble((String)value), false);
				} catch (Exception e) { e.printStackTrace(); }
			}
			@Override public int getDecimals() { return 2; }
		});
		zoomEntry.setEnabled(true);
		zoomEntry.setFont(GUI.FONT_10);
		zoomEntry.setBackground(GUI.COLOR_WHITE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = entryWidth;
		zoomEntry.setLayoutData(gd);
		
		zoomInButton = new Button(zoomComp, SWT.ARROW|SWT.RIGHT);
		zoomInButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		zoomInButton.setFont(GUI.FONT_COLLECT_BUTTON2);
		gd = new GridData();
		gd.widthHint = buttonWidth;
		gd.heightHint = buttonHeight;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		zoomInButton.setLayoutData(gd);
		
		Composite stepComp = new Composite(this, SWT.NONE);
		stepComp.setBackground(GUI.BGCOLOR);
		gl = new GridLayout(2, false);
		gl.marginHeight=heightMargin;
		stepComp.setLayout(gl);
		stepComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		Label stepLabel = new Label(stepComp, SWT.CENTER);
		stepLabel.setBackground(GUI.BGCOLOR);
		stepLabel.setFont(GUI.FONT_COLLECT_BUTTON2);
		stepLabel.setText("Step");
		gd = new GridData();
		gd.horizontalSpan=2;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		stepLabel.setLayoutData(gd);
		
		backwardButton = new Button(stepComp, SWT.PUSH);
		backwardButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		backwardButton.setFont(GUI.FONT_COLLECT_BUTTON2);
		backwardButton.setText("-");
		gd = new GridData();
		gd.widthHint = buttonWidth;
		gd.heightHint = buttonHeight;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		backwardButton.setLayoutData(gd);
		
		forwardButton = new Button(stepComp, SWT.PUSH);
		forwardButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		forwardButton.setFont(GUI.FONT_COLLECT_BUTTON2);
		forwardButton.setText("+");
		gd = new GridData();
		gd.widthHint = buttonWidth;
		gd.heightHint = buttonHeight;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		forwardButton.setLayoutData(gd);
		
		// stop button
		Composite followComp = new Composite(this, SWT.NONE);
		followComp.setBackground(GUI.BGCOLOR);
		gl = new GridLayout(1, false);
		gl.marginWidth=0;
		gl.marginHeight=heightMargin;
		followComp.setLayout(gl);
		followComp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		
		Label followLabel = new Label(followComp, SWT.CENTER);
		followLabel.setBackground(GUI.BGCOLOR);
		followLabel.setFont(GUI.FONT_COLLECT_BUTTON2);
		followLabel.setText("Stop");
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalSpan = 3;
		followLabel.setLayoutData(gd);
		
		updateButton = new Button(followComp, SWT.CHECK);
		updateButton.setBackground(GUI.BGCOLOR);
		updateButton.setFont(GUI.FONT_COLLECT_BUTTON2);
		//updateButton.setText("Stop");
		updateButton.setToolTipText("Stops the diffraction viewer from following data collection");
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		gd.heightHint = buttonHeight;
		updateButton.setLayoutData(gd);
		
		// reset button
		Composite resetComp = new Composite(this, SWT.NONE);
		resetComp.setBackground(GUI.BGCOLOR);
		gl = new GridLayout(2, false);
		gl.marginWidth=0;
		gl.marginHeight=heightMargin;
		gl.horizontalSpacing=0;
		resetComp.setLayout(gl);
		resetComp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		
		Label resetLabel = new Label(resetComp, SWT.CENTER);
		resetLabel.setBackground(GUI.BGCOLOR);
		resetLabel.setFont(GUI.FONT_COLLECT_BUTTON2);
		resetLabel.setText("Reset");
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalSpan = 2;
		resetLabel.setLayoutData(gd);
		
		resetButton = new Button(resetComp, SWT.PUSH);
		resetButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		resetButton.setFont(GUI.FONT_COLLECT_BUTTON2);
		resetButton.setText("\u21BA");
		resetButton.setToolTipText("Reset pan, zoom and contrast values to default");
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		gd.heightHint = buttonHeight;
		resetButton.setLayoutData(gd);
		
		resetSFButton = new Button(resetComp, SWT.PUSH);
		resetSFButton.setBackground(GUI.BGCOLOR);
		resetSFButton.setFont(GUI.FONT_COLLECT_BUTTON2);
		resetSFButton.setText("\u25B2");
		resetSFButton.setToolTipText("Restore SpotFinder graph");
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		gd.heightHint = buttonHeight;
		resetSFButton.setLayoutData(gd);
		
		/////////////////////////// second row ///////////////////////////
		
		Composite secondRow = new Composite(this, SWT.NONE);
		secondRow.setBackground(GUI.BGCOLOR);
		gl = new GridLayout(5,false);
		gl.marginHeight=0;
		secondRow.setLayout(gl);
		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 6;
		secondRow.setLayoutData(gd);
		
		Composite spacer = new Composite(secondRow, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint=1;
		gd.widthHint=1;
		spacer.setLayoutData(gd);

		WdgImgSrvSelector imgSrvSelector = new WdgImgSrvSelector(secondRow);
		WdgControl.setControlVisible(imgSrvSelector,false);

		smallspotsButton = new Button(secondRow, SWT.CHECK);
		smallspotsButton.setBackground(GUI.BGCOLOR);
		smallspotsButton.setFont(GUI.FONT_COLLECT_BUTTON2);
		smallspotsButton.setText("Small spots");
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		gd.heightHint = buttonHeight;
		smallspotsButton.setLayoutData(gd);
		smallspotsButton.setSelection(true);

		Composite spacer2 = new Composite(secondRow, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint=1;
		gd.widthHint=1;
		spacer2.setLayoutData(gd);

		WdgViewerSelector viewerComp = new WdgViewerSelector(secondRow);

		Composite spacer3 = new Composite(secondRow, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint=1;
		gd.widthHint=1;
		spacer3.setLayoutData(gd);

		WdgControl.setControlVisible(resetSFButton, false);
	}
	
	public void connectListeners(final WdgDiffViewer diffViewer)
	{
		this.diffViewer = diffViewer;

		smallspotsButton.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
				ImageServerImage.showSmallSpots = smallspotsButton.getSelection();
				diffViewer.model.requestReload();
			}
		});

		openButton.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0)
			{
				Shell fileShell = new Shell(SWT.NONE);
				NBFileDialog fd = new NBFileDialog(fileShell,SWT.OPEN);
				fd.setFilterPath(DevDataDir.getPrimary());
				String fullpath = fd.openStr();
				if (null != fullpath)
				{
    				String fileString = Disk.getName(fullpath);
    				String dirString = fullpath.substring(0,fullpath.length() - fileString.length() - 1);
				
    				// TODO: use file name patterns instead
    				boolean extensionOK = false;
    				String lowercaseFile = fileString.toLowerCase();
    				if (lowercaseFile.endsWith(".cbf")) extensionOK = true;
    				else
    				{
    					boolean endsInNumbers = true;
    					for (int i=lowercaseFile.length()-5;i<lowercaseFile.length();i++)
	    				{
	    					if (!Character.isDigit(lowercaseFile.charAt(i))) endsInNumbers = false;
	    				}
    					if (endsInNumbers) extensionOK = true;
    				}
    				
    				if (extensionOK)
    				{
    					((DevFPE)EpicsDevice.getDevice("FPE")).setImageViewerFilePath(dirString,fileString);
    				}
    				else
    				{
    					Util.asyncPopupError("Unsupported file type", "File type not supported: "+fileString);
    				}
				}
				fileShell.dispose();
			}
		});
		
		contrastDown.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				double currentContrast = diffViewer.model.getContrast();
				double newContrast;
				
				if 		(currentContrast > 1000.0) 	newContrast = currentContrast - 500.0;
				else if	(currentContrast > 125.0) 	newContrast = currentContrast - 100.0;
				else if (currentContrast > 35.0) 	newContrast = currentContrast - 25.0;
				else if (currentContrast > 10.0) 	newContrast = currentContrast - 5.0;
				else if (currentContrast > 2.0)		newContrast = currentContrast - 2.0;
				else if (currentContrast > 1.0)		newContrast = 1.0;
				else 	{ Log.note("Contrast minimum is 1.0"); return; }
				
				if (newContrast >= 0.0)
				{
					contrastEntry.setText(newContrast);
					diffViewer.model.setContrast(newContrast);
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		contrastUp.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				double currentContrast = diffViewer.model.getContrast();
				double newContrast;
				
				if		(currentContrast < 2.0)		newContrast = currentContrast + 1.0;
				else if	(currentContrast < 10.0)	newContrast = currentContrast + 2.0;
				else if	(currentContrast < 25.0) 	newContrast = currentContrast + 5.0;
				else if (currentContrast < 100.0) 	newContrast = currentContrast + 25.0;
				else if (currentContrast < 900.0) 	newContrast = currentContrast + 100.0;
				else if	(currentContrast < 1000.0) 	newContrast = 1000.0;
				else 								newContrast = currentContrast + 500.0;
				
				contrastEntry.setText(newContrast);
				diffViewer.model.setContrast(newContrast);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});

		zoomOutButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				double newZoom = diffViewer.model.getZoom() / 2.0;
				if (newZoom >= 0.1)
				{
					diffViewer.model.setZoom(newZoom, true);
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});

		zoomInButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				double newZoom = diffViewer.model.getZoom() * 2.0;
				if (newZoom >= 0.1)
				{
					diffViewer.model.setZoom(newZoom, true);
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		zoomEntry.setText(diffViewer.model.getZoom());
		
		backwardButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				diffViewer.model.loadAdjacent.loadPrevious();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});

		forwardButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				diffViewer.model.loadAdjacent.loadNext();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		updateButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				diffViewer.model.setFollowFPE(!updateButton.getSelection());
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		resetButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				contrastEntry.setText(ModelDiffViewer.DEFAULT_CONTRAST);
				diffViewer.model.setContrast(ModelDiffViewer.DEFAULT_CONTRAST);
				diffViewer.model.showFullImage();
				diffViewer.model.setPercentX(0.5);
				diffViewer.model.setPercentY(0.5);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		resetSFButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				diffViewer.diffTab.setSFGraphVisible(true, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}
}
