package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.graph.SFGraph;
import gov.anl.gmca.collect.graph.WdgSFDisplay;
import gov.anl.gmca.collect.graph.WdgSFGraph;
import gov.anl.gmca.collect.graph.WdgSFGraph.GraphSource;
import gov.anl.gmca.collect.graph.WdgSFGraph.LegendItem;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ui.Tab.VisibilityListener;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.io.PVLookup.PV_RUN;
import gov.anl.gmca.io.PVLookup.PV_RUNS;
import gov.anl.gmca.spotfinder.DBImageStats;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Point2D;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class WdgDiffTab extends Composite implements VisibilityListener
{
	public WdgDiffViewer mainDiffViewer, zoomDiffViewer;
	public WdgSFGraph spotFinderGraph;
	public WdgImageOptions imageOptions;
	public WdgSFDisplay sfDisplay;
	private boolean isZoomVisible = false;
	private StackLayout stack;
	private Composite upperComp, lowerComp;
	private CTabFolder lowerFolder;
	private Composite titleComp;
	private Label titleLabel;
	private Color bgColor = GUI.COLOR_WHITE;
	private Color textColor = GUI.COLOR_BLACK;
	private Font titleFont = GUI.FONT_11_BOLD;
			SashForm sashForm = null;
	WdgDiffControls diffControls;
	private PVName sf_pv;
	public GraphSource graphSource;
	
	public WdgDiffTab(Composite parent, boolean inCollectTab)
	{
		super(parent, SWT.NONE);
		
		graphSource = inCollectTab?GraphSource.COLLECT:GraphSource.SAMPLE;
		setBackground(GUI.BGCOLOR);
		
		setLayout(new FillLayout());
		sashForm = new SashForm(this, SWT.VERTICAL);
		
		upperComp = new Composite(sashForm, SWT.BORDER);
		upperComp.setBackground(GUI.BGCOLOR);
		upperComp.setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.verticalSpacing=0;
		upperComp.setLayout(gl);
		
		diffControls = new WdgDiffControls(upperComp, SWT.NONE, 1);
		diffControls.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		mainDiffViewer = new WdgDiffViewer(upperComp, this, diffControls.zoomEntry, true);
		mainDiffViewer.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 4;
		mainDiffViewer.setLayoutData(gd);
		diffControls.connectListeners(mainDiffViewer);
		
		sfDisplay = new WdgSFDisplay(upperComp);
		sfDisplay.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		setSFDisplayVisible(false);
		
		lowerComp = new Composite(sashForm, SWT.BORDER);
		gl = new GridLayout(1,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.verticalSpacing=0;
		lowerComp.setLayout(gl);
		lowerComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		createTitleBar(lowerComp);
		spotFinderGraph = new WdgSFGraph(lowerComp, this);
		spotFinderGraph.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		/*zoomDiffViewer = new WdgDiffViewer(lowerFolder, this, null, false);
		zoomDiffViewer.addMouseListener(new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent arg0) {}
			@Override public void mouseDown(MouseEvent arg0)
			{
				setZoomVisible(false);
			}
		});
		setZoomVisible(false);*/

		//imageOptions = new WdgImageOptions(lowerFolder, this);
		sashForm.setWeights(new int[]{75,25});
		
		if (!DBConfig.getBoolean("spotfinder_enable_collect"))
		{
			// TODO: show or hide tab instead of hiding entire composite
			//WdgControl.setControlVisible(lowerComp, false);
			//layout(true);
		}
		
		sf_pv = inCollectTab ? PV_RUNS.hideCollectSF : PV_RUNS.hideSampleSF;
	}
	
	private void createTitleBar(Composite parent)
	{
		titleComp = new Composite(parent, SWT.NONE);
		titleComp.setBackground(bgColor);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		titleComp.setLayoutData(gd);
		GridLayout gl = new GridLayout(3, false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		titleComp.setLayout(gl);
		
		titleLabel = new Label(titleComp, SWT.LEFT);
		titleLabel.setFont(titleFont);
		titleLabel.setBackground(bgColor);
		titleLabel.setForeground(textColor);
		titleLabel.setText("");
		titleLabel.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		Composite hSpacer = new Composite(titleComp, SWT.NONE);
		hSpacer.setBackground(GUI.COLOR_WHITE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint=1;
		gd.widthHint=1; 
		hSpacer.setLayoutData(gd);
		
		// clickable legend
		/*for (final LegendItem item : LegendItem.values())
		{
    		final Label l = new Label(titleComp, SWT.RIGHT);
    		l.setText("\u25a0"+item.output.shortName);
    		l.setBackground(bgColor);
    		l.setForeground(item.plotColor);
    		l.setLayoutData(new GridData(SWT.RIGHT, SWT.DEFAULT, false, false));
    		item.setLabel(graphSource, l);
    		
    		final Runnable handleClick = new Runnable(){public void run()
    		{
    			boolean newVisibility = !item.isVisible.get(graphSource);
				item.setVisible(graphSource, newVisibility, true);
				
				WdgSFGraph.redrawAll();
    		}};
    		
    		l.addMouseListener(new MouseListener()
    		{
				@Override public void mouseDoubleClick(MouseEvent arg0) {}
				@Override public void mouseUp(MouseEvent arg0) {}
				@Override public void mouseDown(MouseEvent arg0)
				{
					handleClick.run();
				}
			});
		}
		DevSFType.get(DevSFType.SFRunType.COLLECT).llChange.addListener(new ListenerList.Listener<DevSFType>()
		{
			@Override public void handleEvent(DevSFType object, Object data)
			{
				GUI.asyncExec(()->updateLegendItems());
			}
		});
		updateLegendItems();*/

		// minimize button
		Label l = new Label(titleComp, SWT.RIGHT|SWT.BORDER);
		l.setText(" \u25BC ");
		l.setToolTipText("Minimize SpotFinder graph");
		l.setBackground(bgColor);
		l.setForeground(GUI.COLOR_BLACK);
		l.setLayoutData(new GridData(SWT.RIGHT, SWT.DEFAULT, false, false));
		l.addMouseListener(new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent arg0) {}
			@Override public void mouseDown(MouseEvent arg0)
			{
				setSFGraphVisible(false, true);
			}
		});
	}

	private void updateLegendItems()
	{
		for (LegendItem li : LegendItem.values()) {
			li.output = li == LegendItem.LABEL1 ? SFGraph.getOutput1() : SFGraph.getOutput2();
			try {
				for (Label l : li.label.get(GraphSource.SAMPLE)) {
					l.setText("\u25a0" + li.output.shortName);
				}
				for (Label l : li.label.get(GraphSource.COLLECT)) {
					l.setText("\u25a0" + li.output.shortName);
				}
			} catch (NullPointerException npe) {}
			titleComp.layout(true);
		}
	}
	
	private Boolean graphShown = null;
	public void setSFGraphVisible(boolean visible, boolean updateDevice)
	{
		if (null!=graphShown && graphShown == visible) return;
		graphShown = visible;
		
		if (updateDevice) Epics.putInteger(sf_pv, visible?0:1);
		
		if (!visible)
		{
			sashForm.setMaximizedControl(upperComp);
			
			// graph is minimized, show restore button
			WdgControl.setControlVisible(diffControls.resetSFButton, true);
			diffControls.resetSFButton.getParent().layout(true);
		}
		else
		{
			sashForm.setMaximizedControl(null);
			
			// graph is restored, hide restore button
			WdgControl.setControlVisible(diffControls.resetSFButton, false);
			diffControls.resetSFButton.getParent().layout(true);
		}
	}
	
	public void connectListeners()
	{
		addControlListener(new ControlListener()
		{
			@Override public void controlMoved(ControlEvent arg0) {}
			@Override public void controlResized(ControlEvent ce)
			{
				//((GridData)lowerComp.getLayoutData()).heightHint = getBounds().height/4;
				layout(true);
			}
		});
		
		ModelCollectRunList.get().addListener(new ModelRunList.Listener()
		{
			@Override public void runListChanged(ModelRunList runList)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateTitle();
					spotFinderGraph.redraw();
				}});
			}
		});

		Executors.newSingleThreadScheduledExecutor().scheduleWithFixedDelay(()->
		{
			GUI.asyncExec(()->
			{
				updateTitle();
				spotFinderGraph.redraw();
			});
		}, 0, 2000, TimeUnit.MILLISECONDS);

		Epics.addMonitor(sf_pv, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					setSFGraphVisible(Epics.dbrToInt(value)==1?false:true, false);
				}});
			}
		});
		setSFGraphVisible(Epics.getInteger(sf_pv)==1?false:true, false);
	}
	
	void updateTitle()
	{
		try
		{
			/*for (LegendItem item : LegendItem.values())
			{
	    		if (null!=item.label.get(graphSource))
	    		for (Label l : item.label.get(graphSource))
	    		{
	    			if (!l.isDisposed()) WdgControl.setControlVisible(l, !isZoomVisible);
	    		}
			}*/
			
			if (isZoomVisible)
			{
				titleLabel.setText("Zoomed view (click to return to graph)");
				titleComp.layout(true);
			}
			else
			{
				String titleString = "SpotFinder";
				
				// TODO: do this somehow without direct EPICS access
				// the problem is the run number isn't loaded on startup when this is
				// first set
				int currentRunLabel = Epics.getInteger(PV_RUN.runLabel.run(ModelCollectRunList.get().getCurrentRunIndex()));
				titleString += ": Run "+currentRunLabel;
				
				titleLabel.setText(titleString);
				titleComp.layout(true);
			}
		}
		catch (Exception e) { e.printStackTrace(); }
	}
	
	public void setZoomVisible(boolean zoomVisible)
	{
		this.isZoomVisible = zoomVisible;
		updateTitle();
		
		// TODO: update for folder
		//zoomDiffViewer.notifyIsVisible(zoomVisible);
		//stack.topControl = zoomVisible ? zoomDiffViewer : spotFinderGraph;
		//stackComp.layout(true);
	}

	public void setSFDisplayVisible(boolean vis)
	{
		WdgControl.setControlVisible(sfDisplay, vis);
		upperComp.layout(true);
	}
	
	public void notifyIsVisible(boolean isVisible)
	{
		mainDiffViewer.notifyIsVisible(isVisible);
	}
	
	void updateZoom(Point2D zoomPctCoords)
	{
		zoomDiffViewer.model.titleHeight=0;
		zoomDiffViewer.model.imgWidth=zoomDiffViewer.getBounds().width;
		zoomDiffViewer.model.imgHeight=zoomDiffViewer.getBounds().height;
		zoomDiffViewer.model.currentFilename=mainDiffViewer.model.currentFilename;
		zoomDiffViewer.model.currentFullPath=mainDiffViewer.model.currentFullPath;
		zoomDiffViewer.model._percentX=zoomPctCoords.x;
		zoomDiffViewer.model._percentY=zoomPctCoords.y;
		zoomDiffViewer.model._zoom=8;
		zoomDiffViewer.model.requestUpdate();
	}
}
