package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.diffimage.DiffractionImage;
import gov.anl.gmca.util.SwingUtil;
import org.eclipse.swt.graphics.Image;

import java.awt.*;
import java.awt.image.BufferedImage;

public class ZoomPanContrast
{
	public static class Histogram
	{
		int[] values = new int[256];
		int maxVal=0;
	}
	
	public static Histogram getHistogram(DiffractionImage image)
	{
		//ImageData id = image.imageData;
		
		// histogram is stored in 256 bins regardless of actual color depth
		Histogram ret = new Histogram();
		
		/*for (int i=0;i<id.width;i++)
		{
			for (int j=0;j<id.height;j++)
			{
				// get the intensity for that pixel
				int c = id.getPixel(i, j);
				int val = c&0x000000FF;
				
				// add one to that value bin, and update the maximum
				ret.values[val]++;
				if (ret.values[val]>ret.maxVal) ret.maxVal=ret.values[val];
			}
		}*/
		
		return ret;
	}
	
	// apply operations to input DiffractionImage and return an SWT Image ready to draw to screen
	//
	public static Image process(DiffractionImage di, double percentX, double percentY, double gray, double zoom, int screenX, int screenY)
	{
		// if we're using the image server, don't process, just return the already processed image
		if (DBConfig.getBoolean("imgsrv_enable"))
		{
			return di.screenImg;
		}
		
		int imgWidth = di.origWidth();
		int imgHeight = di.origHeight();
		
		// find the larger ratio since that will be the one determining how much to scale down
		// the image for it to fit
		double ratioX = (double)imgWidth / (double)screenX;
		double ratioY = (double)imgHeight / (double)screenY;
		double maxRatio = ratioX>ratioY?ratioX:ratioY;
		
		// scale factor for the image to get it to fit in the screen
		double zoomRatioReference = 1.0/maxRatio;
		
		// size of the image as it will be drawn
		int scaledWidth = (int)(imgWidth * zoomRatioReference * zoom);
		int scaledHeight = (int)(imgHeight * zoomRatioReference * zoom);
		
		// center of the viewport
		int vpCenterX = screenX/2, vpCenterY = screenY/2;
		
		// top left of image in viewport coordinates
		int x = vpCenterX + (int)(-1.0 * percentX * scaledWidth);
		int y = vpCenterY + (int)(-1.0 * percentY * scaledHeight);

		// update contrast
		di.setContrast(gray);
		
		// use getScaledInstance, which is better when zooming out
		/*java.awt.Image awtImage = di.contrastImg.getScaledInstance(scaledWidth, scaledHeight, java.awt.Image.SCALE_SMOOTH);
		BufferedImage displayImg = new BufferedImage(awtImage.getWidth(null),awtImage.getHeight(null), BufferedImage.TYPE_INT_ARGB);
		Graphics g = displayImg.getGraphics();
		g.drawImage(awtImage, 0, 0, null);
		g.dispose();*/
		
		// create a viewport image
		BufferedImage displayImg = new BufferedImage(screenX, screenY, java.awt.image.BufferedImage.TYPE_INT_ARGB);
		
		// draw the image, scaling as needed
		Graphics2D g2d = displayImg.createGraphics();
		g2d.setRenderingHint
		(
			RenderingHints.KEY_INTERPOLATION,
			RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR
		);
		g2d.drawImage(di.contrastImg, x, y, x+scaledWidth, y+scaledHeight, 0, 0, imgWidth, imgHeight, null);
		//Log.note("Drawing "+imgWidth+"x"+imgHeight+" to "+x+","+y+" "+scaledWidth+"x"+scaledHeight+" screen="+screenX+"x"+screenY);
		g2d.dispose();

		// convert to SWT
		Image swtImage = SwingUtil.AWTImageToSWTImage(displayImg);
		return swtImage;
	}
}
