package gov.anl.gmca.common;

import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.util.ListenerList;
import gov.aps.jca.dbr.DBR;

import java.io.File;

public class DevDataDir
{
	public static ListenerList<DevDataDir> llDirChange;

	private static String 	dir;
	public	static String	getPlain()		{ return dir;						}
	public	static String	getCBF()		{ return dir+"_cbf";				}
	public	static String	getHDF()		{ return dir+"_hdf";				}
	public 	static void 	set(String s)	{ Epics.put(PV_BLUICE.data_dir, s); }
	
	// where files are initially written
	public static String getFPEDir()
	{
		switch (DetectorConfig.getType())
		{
		case EIGER: 	return getCBF();
		case PILATUS: 	return getCBF();
		case MAR: 		return getPlain();
		default: throw new IllegalArgumentException("Unknown detector type: "+DetectorConfig.getType());
		}
	}

	public static String getRelative(String fullDir)
	{
		if (fullDir.startsWith(getCBF())) {
			if (fullDir.length()==getCBF().length()) return "";
			else return fullDir.substring(getCBF().length()+1);
		} else {
			return fullDir;
		}
	}
	
	// where all extra files go
	public 	static String getPrimary()
	{
	    switch (DetectorConfig.getType())
	    {
	    case EIGER: 
			switch (DevUserConfig.get().getImageFormat())
			{
			case HDF_ONLY:
			case HDF_PRIMARY:
				return getHDF();
			case CBF_ONLY:
			case CBF_PRIMARY:
				return getCBF();
			default:
				throw new IllegalArgumentException();
			}
	    case PILATUS:
	    	return getCBF();
	    case MAR:
	    	return getPlain();
	    default:
	    	throw new IllegalArgumentException();
	    }
	}
	
	public static String addToPrimary(String dir)
	{
		return getPrimary()+File.separator+dir;
	}
	public static String addToFPEDir(String dir)
	{
		return getFPEDir()+File.separator+dir;
	}
	
	public DevDataDir()
	{
		llDirChange = new ListenerList<DevDataDir>(this);
		
		Epics.addMonitor(PV_BLUICE.data_dir, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				dir=Epics.dbrToString(value);
				llDirChange.notifyListeners();
			}
		});
	}
}
