package gov.anl.gmca.common;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.GUIStatus;

import java.util.HashSet;

// This class supports additional functionality when the move can be
// defined by a floating-point number
public abstract class DevMoveFloat extends DevMove
{
	public DevMoveFloat(String motorID)
	{
		super(motorID);
	}
	
	public MoveCmd startRelativeMove(double offset)
	{
		return startMove(getPosition()+offset);
	}
	
	public static final double DEFAULT_INPOS_BAND = 0.010;
	
	public double getInPosBand() { return DEFAULT_INPOS_BAND; }
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////// limits /////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	public abstract Limits getLimits();

	/////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////// listener ////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	public interface Listener
	{
		public abstract void handleStateChange(State state);
		public abstract void handleActPosChange(DevMoveFloat dev, double newPos);
		public abstract void handleRqsPosChange(double newPos);
		public abstract void handleUnitsChange(String units);
	}
	
	public abstract static class PositionListener implements Listener
	{
		@Override public void handleStateChange(State state) {}
		@Override public void handleRqsPosChange(double newPos) {}
		@Override public void handleUnitsChange(String units) {}
	}
	
	protected HashSet<Listener> listenerSet = new HashSet<Listener>();
	public void addListener(final Listener listener)
	{
		listenerSet.add(listener);
		final DevMoveFloat dev = this;
		if (GUIStatus.inGUIMode) GUI.asyncExec(new Runnable(){public void run()
		{
			listener.handleActPosChange(dev, getPosition());
			listener.handleRqsPosChange(getRqsPos());
			listener.handleStateChange(getState());
		}});
		//listener.handleUnitsChange(getUnits());
	}
	
	public void removeListener(Listener listener)
	{
		listenerSet.remove(listener);
	}

	public abstract double getPosition();
	public abstract double getRqsPos();

	/////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////// convert Object methods to double ///////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	@Override public boolean moveChecks(Object dest)
	{
		// check if move is allowed
		/*if (!AppState.isOperationAllowed(getOperation()))
		{
			Log.note(getDeviceName()+" move skipped, operation not allowed at this time.");
			return false;
		}*/
		
		return moveChecks(((Double)dest).doubleValue());
	}
	public abstract boolean moveChecks(double dest);

	@Override public void moveCmd(Object dest, MoveResult result)
	{
		moveCmd(((Double)dest).doubleValue(),result);
	}
	public abstract void moveCmd(double dest, MoveResult result);
}
