package gov.anl.gmca.common;

import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;

import java.util.ArrayList;
import java.util.HashMap;

// subclasses may choose to use _syncExec() and isRunning(), or override them and run tasks
// however they choose.  The requirements are:
//
// 1) Call ll.notifyListeners() once isRunning() will return true, then again after it will return false
// 2) isRunning() should return true if any thread or task is running

public abstract class DevScript
{
	protected static HashMap<String,DevScript> devMap = new HashMap<String,DevScript>();
	public static ArrayList<DevScript> getAll() { return new ArrayList<DevScript>(devMap.values()); }

	public ListenerList<DevScript> ll;
	protected ExecTask task = null;
	protected String devName = "";
	
	public boolean isRunning()
	{
		return (null != task && task.isRunning());
	}
	
	protected void _syncExec()
	{
		if (isRunning())
		{
			Log.error(getClass().getName()+" is already running, exec aborted.");
			return;
		}

		task.start();
		ll.notifyListeners();
		
		task.waitFor();
		ll.notifyListeners();
	}
	
	protected void _asyncExec()
	{
		if (isRunning())
		{
			Log.error(getClass().getName()+" is already running, exec aborted.");
			return;
		}
		
		new Thread(new Runnable(){public void run()
		{
			_syncExec();
		}}, "Script").start();
	}
	
	public void stopExec()
	{
		if(task != null) task.asyncStop();
	}
	
	public DevScript(String devName)
	{
		ll = new ListenerList<DevScript>(this);
	
		this.devName = devName;
		
		if (devMap.containsKey(devName))
		{
			throw new IllegalArgumentException("Script device "+devName+" already exists!");
		}
		
		devMap.put(devName, this);
	}
	
	public String getDeviceName()
	{
		return devName;
	}
}
