package gov.anl.gmca.common;

import gov.anl.gmca.util.ListenerList;

import java.util.HashMap;
import java.util.concurrent.Semaphore;

// tells listeners if any DevMove objects are in local moves
public class ModelMotorStatus
{
	public static ListenerList<ModelMotorStatus> ll = new ListenerList<ModelMotorStatus>(null);

	protected static HashMap<DevMove,Boolean> isMovingMap = new HashMap<DevMove,Boolean>();
	protected static Semaphore mapLock = new Semaphore(1);
	
	public static class MotorStatus
	{
		public DevMove dev;
		public boolean isMoving;
		public MotorStatus(DevMove dm, boolean im) { this.dev=dm; this.isMoving=im; }
	}
	
	protected static void updateStatus(DevMove dev, boolean isMoving)
	{
		boolean stateChange = false;
		
		mapLock.acquireUninterruptibly();
		try
		{
    		if (isMovingMap.get(dev)!=isMoving)
    		{
    			isMovingMap.put(dev, isMoving);
    			stateChange = true;
    		}
		}
		catch (Exception e) { e.printStackTrace(); }
		finally
		{
			mapLock.release();
		}
		
		if (stateChange) ll.notifyListeners(new MotorStatus(dev,isMoving));
	}
	
	public static void init()
	{
		for (DevMove dev : DevMove.getAll())
		{
			isMovingMap.put(dev,dev.inLocalMove());
			
			dev.llMoveStartStop.addListener(new ListenerList.Listener<DevMove>()
			{
				@Override public void handleEvent(DevMove dev, Object data)
				{
					updateStatus(dev, dev.getMoving());
				}
			});
		}
	}
}
