package gov.anl.gmca.common;

import java.util.ArrayList;

public abstract class ModelRunList
{
	public interface ModelRun
	{
		public void notifyStarTabClick();
		public void notifyExistingRunClick();
		public void notifySyncEnabled();
	}
	
	public static enum RunType { COLLECT, INTER_RASTER, AUTO_RASTER };
	
	abstract public ModelRun getRun(int runIndex);
	
	abstract public void setRunCount(int newCount);
	abstract public void setCurrentRun(int newCount);

	abstract public int getRunCount();
	abstract public int getCurrentRunIndex();
	
	public interface Listener
	{
		public void runListChanged(ModelRunList runList);
	}
	
	protected ArrayList<Listener> listenerList	= new ArrayList<ModelRunList.Listener>();
	public void addListener(Listener listener) { listenerList.add(listener); listener.runListChanged(this); }
	public void	removeListener(Listener listener) { while (listenerList.remove(listener)) {} }
	public void notifyListeners()
	{
		ArrayList<Listener> listenerListCopy = new ArrayList<Listener>(listenerList);
		for (Listener l : listenerListCopy)
		{
			if (listenerList.contains(l)) l.runListChanged(this);
		}
		notifyCurrentRunListeners();
	}

	public interface CurrentRunListener
	{
		public abstract void currentRunChanged(ModelRun run);
	}
	
	protected ArrayList<CurrentRunListener> currentRunListenerList = new ArrayList<CurrentRunListener>();
	public void addCurrentRunListener(CurrentRunListener listener) { currentRunListenerList.add(listener); }
	public void removeCurrentRunListener(CurrentRunListener listener) { while (currentRunListenerList.remove(listener)){} }
	public void notifyCurrentRunListeners()
	{
		ModelRun currentRun = getCurrentRunModel();
		if (currentRun==null)
		{
			System.out.println("current run model is null!");
		}
		else
		{
			for (CurrentRunListener crl : new ArrayList<CurrentRunListener>(currentRunListenerList))
			{
				crl.currentRunChanged(currentRun);
			}
		}
	}
	
	public abstract ModelRun getCurrentRunModel();
}
