package gov.anl.gmca.common;

import gov.anl.gmca.io.Database;
import org.apache.commons.lang3.*;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

import java.io.File;

public class Redis
{
    public static String getRedisHost()
    {
        String redisServer = DBConfig.get("redis_server").strip();
        System.out.println("Redis host is "+redisServer);
        return redisServer;
    }

    public static int getRedisPort()
    {
        boolean isLocal = getRedisHost().equals("127.0.0.1");
        int ret = isLocal ? 8003 : 6379;
        System.out.println("Redis port is "+ret);
        return ret;
    }

    public static JedisPool pool = null, pyBluIcePool = null;
    public static Jedis get() {
        if (null==pool) pool = new JedisPool(new JedisPoolConfig(), getRedisHost(), getRedisPort());;
        return pool.getResource();
    }
    public static Jedis getPyBluIce() {
        if (null==pyBluIcePool) {
            Database.Result dbResult = Database.exec(Database.DBName.CONFIG, "select location from Locations where name='redis'");
            String redisServer = dbResult.getRow(0).get("location");
            String redisHost = redisServer.split(":")[0];
            int redisPort = Integer.parseInt(redisServer.split(":")[1]);
            pyBluIcePool = new JedisPool(new JedisPoolConfig(), redisHost, redisPort);
        };
        return pyBluIcePool.getResource();
    }

    public static String clients_prefix = "bluice:clients#";
    public static String clients(String client_id) { return clients_prefix+client_id; }

    public static String busy_states = "bluice:busy:states";
    public static String busy_allowed_op = "bluice:busy:allowed_op";
    public static String auto_beamAlign = "bluice:auto:beamAlign";
    public static String server_fpe = "bluice:server:fpe";
    public static String collect_state = "bluice:collect:state";
    public static String config = "bluice:config";
    public static String raster_state = "bluice:raster:state";
    public static String hutch_motors = "bluice:hutch:motor#";
    public static String sonicc_state = "bluice:sonicc:state";

    public static String hutch_motor(String name) { return hutch_motors+name; }

    public static String collect_run(int runIndex) { return "bluice:collect:runs#"+runIndex; }
    public static String raster_run(int runIndex) { return "bluice:raster:runs#"+runIndex; }
    public static String image_stats(String relDir, String prefix, int runLabel, String output) {
        String _relDir = StringUtils.stripStart(relDir,"/");
        return String.format("bluice:collect:image_stats#%s:%s_%d:%s", _relDir, prefix, runLabel, output);
    }
    public static String image_info(String relDir, String prefix) {
        return String.format("bluice:collect:img_info#%s:%s", relDir, prefix);
    }

    public static void reset(boolean flushall)
    {
        String cmd = "redis_defaults.py";
        ProcessBuilder pb;
        if (flushall) {
            pb = new ProcessBuilder(cmd, "flushall");
        } else {
            pb = new ProcessBuilder(cmd);
        }
        pb.directory(new File(System.getProperty("application.home")+"/tools/scripts"));
        try {
            Process p = pb.start();
            p.waitFor();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
