package gov.anl.gmca.common.client_server;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import com.mashape.unirest.http.exceptions.UnirestException;
import gov.anl.gmca.common.client_server.ui.DlgServerView;
import gov.anl.gmca.gui.GUIStatus;
import gov.anl.gmca.io.Web;
import gov.anl.gmca.io.Web.WebConnection;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

public class RPC
{
	public static enum RPCResult { NOT_STARTED, RUNNING, OK, ERROR; }
	public static class RPCCommand
	{
		public RPCCommand(RPCResult result) { this.result=result; }
		public RPCResult result;
	}
	
	public static RPCCommand sendCommand(String ... cmd)
	{
		// prepare request
		String baseURL = Util.getServerURL();
		String url = baseURL+"/cmd_req";
		Map<String,Object> fields = new HashMap<>();
		for (String c : cmd) {
			int colonIndex = c.indexOf(':');
			fields.put(c.substring(0, colonIndex), c.substring(colonIndex+1));
		}

		// log the request
		String urlAsText = url+" ";
		for (String key : fields.keySet()) urlAsText += key+"="+fields.get(key)+" ";
		System.out.println(urlAsText);
		if (GUIStatus.inGUIMode) DlgServerView.get().cmdSent(urlAsText);

		// send the request
		HttpResponse<String> resp;
		try {
			resp = Unirest.post(url)
					.fields(fields)
					.asString();

			if (200 != resp.getStatus()) {
				Log.error(String.format("Received HTTP code %d for URL %s", resp.getStatus(), url));
				return new RPCCommand(RPCResult.ERROR);
			}
		} catch (UnirestException e) {
			e.printStackTrace();
			return new RPCCommand(RPCResult.ERROR);
		}

		// parse and log the reponse
		String responseText=resp.getBody();
		String cmdID = new String(responseText).trim();
		if (GUIStatus.inGUIMode) DlgServerView.get().cmdReceived("Command received, ID="+cmdID);

		new Thread(new Runnable(){public void run() {

			while (true) {
				Util.sleep(200);

				// get status update
				String statusURL = baseURL + "/cmd_stat?cmd_id=" + cmdID;
				WebConnection wc = Web.get().openConnection(statusURL);
				wc.waitForThread();
				String resultText = new String(wc.content);

				// show in GUI log
				if (GUIStatus.inGUIMode) DlgServerView.get().cmdReceived(resultText);

				// break if status is no longer RUNNING
				StringTokenizer st = new StringTokenizer(resultText, " ");
				while (st.hasMoreTokens()) {
					String token = st.nextToken();
					StringTokenizer st2 = new StringTokenizer(token, "=");
					while (st2.hasMoreTokens()) {
						String token2 = st2.nextToken();
						if (token2.equals("result")) {
							String status = st2.nextToken();
							if (!status.equals("RUNNING")) {
								return;
							}
						}
					}
				}
			}
		}}).start();

		return new RPCCommand(RPCResult.OK);
	}
}
