package gov.anl.gmca.common.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

public class GButton extends Canvas
{
	private PaintListener paintListener;
	private MouseListener mouseListener;
	private MouseTrackListener mouseTrackListener;
	private String text;
	private int style;
	private Color borderColor, interiorBG, textBorder;
	private boolean rounded=false, underline=false;
	private boolean isEnabled = true, isMouseHovering = false, isMouseDown = false;
	public ListenerList<GButton> llSelected;
	
	@Override public void setForeground(Color c) { super.setForeground(c); redraw(); }
	@Override public void setBackground(Color c) { super.setBackground(c); redraw(); }
	public void setTextBorder(Color c) { textBorder = c; redraw(); }
	public void setBorderColor(Color c) { borderColor = c; redraw(); }
	public void setRounded(boolean r) { rounded=r; redraw(); }
	public void setInteriorBackground(Color c) { interiorBG = c; redraw(); }
	public void setUnderline(boolean underline) { this.underline=underline; redraw(); }
	
	public GButton(Composite parent, int style)
	{
		super(parent, SWT.NONE);
		this.style=style;
		borderColor=GUI.COLOR_DARK_GRAY;
		interiorBG = getBackground();
		llSelected = new ListenerList<>(this);
		
		paintListener = new PaintListener()
		{
			@Override public void paintControl(PaintEvent pe)
			{
				draw(pe.gc);
			}
		};
		addPaintListener(paintListener);
		
		mouseListener = new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent me)
			{
				if (me.button==1) handleMouseUp();
			}
			@Override public void mouseDown(MouseEvent me)
			{
				if (me.button==1) handleMouseDown();
			}
		};
		addMouseListener(mouseListener);
		
		mouseTrackListener = new MouseTrackListener()
		{
			@Override public void mouseEnter(MouseEvent arg0)
			{
				handleMouseEnter();
			}
			@Override public void mouseExit(MouseEvent arg0)
			{
				handleMouseExit();
			}
			@Override public void mouseHover(MouseEvent arg0) {}
		};
		addMouseTrackListener(mouseTrackListener);
	}
	
	private void draw(GC gc)
	{
		Rectangle bounds = getBounds();
		
		if (isEnabled)
		{
			//bgColor = isMouseHovering ? GUI.COLOR_LIGHT_GRAY : GUI.COLOR_SAMPLE_BG;
		}
		//gc.setForeground(bgColor);
		//Point extent = gc.textExtent(text);
		
		gc.setBackground(getBackground());
		gc.fillRectangle(0, 0, getBounds().width, getBounds().height);
		
		/*if ((style&SWT.BORDER)!=0)
		{
			gc.setBackground(interiorBG);
			gc.setForeground(borderColor);
			gc.setLineWidth(1);
			gc.fillRoundRectangle(0, 0, bounds.width-1, bounds.height-1, 20, 20);
			gc.drawRoundRectangle(0, 0, bounds.width-1, bounds.height-1, 20, 20);
		}*/
		
		if (null!=textBorder)
		{
			gc.setForeground(textBorder);
			for (int x=-1;x<=1;x++)
			for (int y=-1;y<=1;y++)
			{
				gc.drawString(text, 4+x, 4+y, true);
			}
		}
		
		// underline
		Point extent = gc.textExtent(text);
		int yOffset=-3;
		if (underline)
		{
			gc.setForeground(GUI.COLOR_BLACK);
			gc.drawLine(4, 4+extent.y+yOffset, 4+extent.x, 4+extent.y+yOffset);
			gc.setForeground(GUI.COLOR_DARK_GRAY);
			gc.drawLine(4, 4+extent.y+yOffset+1, 4+extent.x, 4+extent.y+yOffset+1);
		}
		
		// text
		gc.setForeground(getForeground());
		gc.drawString(text, 4, 4, true);
	}

	private void handleMouseDown()
	{
		if (isEnabled)
		{
			isMouseDown = true;
			redraw();
		}
	}
	
	private void handleMouseUp()
	{
		if (isMouseDown)
		{
			isMouseDown = false;
			redraw();
			llSelected.notifyListeners();
		}
	}
	
	private void handleMouseEnter()
	{
		isMouseHovering = true;
		redraw();
	}

	private void handleMouseExit()
	{
		isMouseHovering = false;
		redraw();
	}

	public void setText(String text)
	{
		this.text = text;
		getParent().layout();
		redraw();
	}
	
	public void setImage(Image image)
	{
		
	}
	
	@Override public void setEnabled(boolean enable)
	{
		isEnabled = enable;
		redraw();
	}
	
	@Override public Point computeSize(int wHint, int hHint)
	{
		return getSize();
	}
	
	@Override public Point computeSize(int wHint, int hHint, boolean changed)
	{
		return getSize();
	}
	
	@Override public Point getSize()
	{
		GC gc = new GC(GUI.getDisplay());
		Point extent = gc.textExtent(text);
		gc.dispose();
		
		extent.x += 7;
		extent.y += 8;
		
		return extent;
	}
}
