package gov.anl.gmca.common.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.io.Disk.File;
import gov.anl.gmca.io.Disk.IOException;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;
import java.util.Collections;

// This dialog replaces the system file and directory selection dialogs
//
// It supports directory creation and navigation by direct text entry
//
// It is subclassed by the file- and directory-specific dialog classes.
//
public abstract class NBDiskDialog extends NBDialog
{
	protected String currentDir = null;
	protected String returnString = "";
	
	public void setFilterPath(String path)
	{
		if (!Disk.isDirectory(path))
		{
			Log.warn("Directory "+path+" doesn't exist, defaulting to home directory");
			currentDir = System.getProperty("user.home");
		}
		else
		{
			currentDir = path;
		}
	}
	public void setMessage(String s) { dialogTitle = s; }
	
	protected abstract boolean showFile(String filename);
	protected abstract boolean returnDir();
	protected abstract boolean returnValueMustExist();
	
	protected Text text;
	protected Table table;
	protected Label label, locationLabel;
	protected Button btnCreateDir, btnCancel, btnOK;
	protected int lastSelectedIndex=0;
	protected Composite comp;
	
	public NBDiskDialog(Shell parentShell, int style)
	{
		super(parentShell, SWT.APPLICATION_MODAL|SWT.DIALOG_TRIM);
		this.parentShell=parentShell;
		shellStyle |= SWT.RESIZE;
	}
	
	@Override protected void createContents(Composite parent)
	{
		createWidgets(parent);
		fillTable();
		connectListeners();
	}
	
	private void createWidgets(Composite parent)
	{
		///////////////////////////////////////////////////////////////////////
		///////////////////////  parent composite /////////////////////////////
		///////////////////////////////////////////////////////////////////////
		
		comp = new Composite(parent, SWT.NONE);
		comp.setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gl = new GridLayout(1,false);
		gl.marginHeight = 10;
		gl.marginWidth = 10;
		comp.setLayout(gl);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.minimumHeight = 500;
		gd.minimumWidth = 500;
		comp.setLayoutData(gd);
		
		///////////////////////////////////////////////////////////////////////
		//////////////////////  current directory display /////////////////////
		///////////////////////////////////////////////////////////////////////
		
		Composite displayComp = new Composite(comp, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		displayComp.setLayoutData(gd);
		gl = new GridLayout(2, false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		gl.horizontalSpacing=5;
		displayComp.setLayout(gl);
		
		label = new Label(displayComp, SWT.NONE);
		label.setText(currentDir);
		label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		btnCreateDir = new Button(displayComp, SWT.PUSH);
		gd = new GridData();
		btnCreateDir.setLayoutData(gd);
		btnCreateDir.setText("Create Folder");
		
		///////////////////////////////////////////////////////////////////////
		////////////////////////////  text entry //////////////////////////////
		///////////////////////////////////////////////////////////////////////
		
		Composite textComp = new Composite(comp, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		textComp.setLayoutData(gd);
		gl = new GridLayout(2, false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		gl.horizontalSpacing=5;
		textComp.setLayout(gl);
		
		locationLabel = new Label(textComp, SWT.NONE);
		locationLabel.setText("Location:");
		
		text = new Text(textComp, SWT.BORDER);
		text.setFont(GUI.FONT_DEFAULT);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		text.setLayoutData(gd);

		///////////////////////////////////////////////////////////////////////
		////////////////////////  directory listing ///////////////////////////
		///////////////////////////////////////////////////////////////////////
		
		table = new Table(comp, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.widthHint = 1;
		gd.heightHint = 1;
		table.setLayoutData(gd);
		
		/*String[] headers = new String[]{"Filename","Size","Modified"};
		for (String header : headers)
		{
			TableColumn tc = new TableColumn(table, SWT.NONE);
			tc.setText(header);
		}
		table.setHeaderVisible(true);*/
		
		///////////////////////////////////////////////////////////////////////
		///////////////////////  Filename filters /////////////////////////////
		///////////////////////////////////////////////////////////////////////

		createFilterCombo(comp);
		
		///////////////////////////////////////////////////////////////////////
		//////////////////////  OK & Cancel buttons ///////////////////////////
		///////////////////////////////////////////////////////////////////////
		
		Composite buttonComp = new Composite(comp, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		buttonComp.setLayoutData(gd);
		gl = new GridLayout(3, false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		gl.horizontalSpacing=5;
		buttonComp.setLayout(gl);

		Composite spacer = new Composite(buttonComp, SWT.NONE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint=1;
		spacer.setLayoutData(gd);
		
		btnCancel = new Button(buttonComp, SWT.PUSH);
		gd = new GridData();
		gd.widthHint=75;
		btnCancel.setLayoutData(gd);
		btnCancel.setText("Cancel");
		
		btnOK = new Button(buttonComp, SWT.PUSH);
		gd = new GridData();
		gd.widthHint=75;
		btnOK.setLayoutData(gd);
		btnOK.setText("OK");
		
		text.setFocus();
	}

	private Point mousePos = new Point(0,0);
		
	protected void connectListeners()
	{
		btnCreateDir.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				showCreateDir();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		text.addModifyListener(new ModifyListener()
		{
			@Override public void modifyText(ModifyEvent e)
			{
				String currentText = text.getText();
				if (currentText.startsWith("/"))
				{
					returnString = currentText;
				}
				else
				{
					returnString = currentDir+"/"+currentText;
				}
			}
		});
		text.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent e)
			{
				handleTraverse(e.detail);
			}
		});
		
		// when a row is selected
		table.addMouseMoveListener(new MouseMoveListener()
		{
			@Override public void mouseMove(MouseEvent e)
			{
				//Log.note("pos="+e.x+","+e.y);
				mousePos.x=e.x;
				mousePos.y=e.y;
			}
		});

		table.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				Point selectionInfo = Util.getClickedTableRowAndColumn(table, mousePos.x, mousePos.y);
				if (null==selectionInfo) return;
				TableItem selected = table.getItem(selectionInfo.x);
				String selectedFile = selected.getText();
				boolean isDir = Disk.get().isDirectory(currentDir+"/"+selectedFile);
				
				// for a file...
				if (!isDir)
				{
					// in a file dialog...
					if (!returnDir())
					{
						// allow it and update the return value
						lastSelectedIndex = table.getSelectionIndex();
						text.setText(selected.getText());
						returnString = currentDir+"/"+selected.getText();
					}
					else // for a file in a dir dialog
					{
						// selection is disallowed elsewhere
					}
				}
				else // for a directory
				{
					// always allow selection
					lastSelectedIndex = table.getSelectionIndex();
					
					// but only update the text field and the return value in a dir dialog
					if (returnDir())
					{
						text.setText(selectedFile);
						returnString = currentDir+"/"+selected.getText();
					}
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		table.addMouseListener(new MouseListener()
		{
			@Override public void mouseUp(MouseEvent e) {}
			@Override public void mouseDown(MouseEvent e) {}
			@Override public void mouseDoubleClick(MouseEvent e)
			{
				TableItem[] selected = table.getSelection();
				if (null != selected && selected.length>0)
				{
					String currentText = selected[0].getText();
					File clickedFile = new File(currentDir+"/"+currentText);
					if (clickedFile.isDirectory())
					{
						try
						{
							cd(clickedFile.getCanonicalPath());
							returnString = currentDir+"/"+text.getText();
						}
						catch (IOException ioe)
						{
							ioe.printStackTrace();
						}
					}
					else
					{
						if (!returnDir())
						{
							returnString = currentDir+"/"+currentText;
							submit(SWT.OK);	
						}
					}
				}
			}
		});
		
		btnCancel.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				submit(SWT.CANCEL);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		btnOK.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				submit(SWT.OK);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	}

	protected void handleTraverse(int keyCode)
	{
		if (keyCode == SWT.TRAVERSE_ESCAPE) submit(SWT.CANCEL);
		if (keyCode == SWT.TRAVERSE_RETURN) submit(SWT.OK);
	}
	
	protected void createFilterCombo(Composite parent) {}
	
	protected void fillTable()
	{
		// get list from disk
		String[] list = Disk.get().getDir(currentDir);

		// create sorted file and dir lists
		ArrayList<String> dirList = new ArrayList<String>(), fileList = new ArrayList<String>();
		if (null != list)
		{
			for (String file : list)
			{
				if (Disk.get().isDirectory(currentDir+"/"+file)) dirList.add(file);
				else fileList.add(file);
			}
		}
		Collections.sort(dirList);
		Collections.sort(fileList);

		dirList.add(0,"..");
		
		// populate table
		table.removeAll();
		for (final String file : dirList)
		{
			TableItem item = new TableItem(table, SWT.NONE);
			item.setText(0,file.toString());
			item.setImage(GUI.IMAGE_FOLDER);
		}
		for (String file : fileList)
		{
			if (showFile(file))
			{
				TableItem item = new TableItem(table, SWT.NONE);
				item.setText(0,file.toString());
				item.setImage(GUI.IMAGE_TEXT);
				if (returnDir()) { item.setForeground(GUI.COLOR_DARK_GRAY); }
			}
		}
	}
	
	private TableItem createDirItem;
	private Text createDirText;
	private TableEditor createDirEditor;
	private static final String createDirMessage = "Type name of new folder";
	
	private void showCreateDir()
	{
		createDirItem = new TableItem(table, SWT.NONE, 1);
		createDirEditor = new TableEditor(table);
		
		createDirText = new Text(table, SWT.BORDER);
		createDirText.setText(createDirMessage);
		createDirText.selectAll();
		createDirText.setFocus();
		createDirText.addFocusListener(new FocusListener()
		{
			@Override public void focusLost(FocusEvent e)
			{
				closeCreateDir();
			}
			@Override public void focusGained(FocusEvent e) {}
		});
		createDirText.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent e)
			{
				if (e.detail == SWT.TRAVERSE_ESCAPE) { closeCreateDir(); return; }
				if (e.detail == SWT.TRAVERSE_RETURN) { createDir(); return; }
			}
		});
		
		createDirEditor.grabHorizontal = true;
		createDirEditor.minimumHeight = 26;
		createDirEditor.verticalAlignment = SWT.TOP;
		createDirEditor.setEditor(createDirText, createDirItem, 0);
	}
	
	private void createDir()
	{
		String newDirName = createDirText.getText();
		if (newDirName.indexOf(createDirMessage)!=0)
		{
			String newDirFullPath = currentDir+"/"+newDirName;
			if (Disk.get().exists(newDirFullPath))
			{
				Util.popupError("", "The folder could not be created\n\nError creating directory: File exists");
			}
			else
			{
				if (Disk.get().mkdir(newDirFullPath))
				{
					cd(newDirFullPath);
					returnString = currentDir + "/"+text.getText();
				}
			}
		}
		closeCreateDir();
	}
	
	protected void cd(String newPath)
	{
		try
		{
			newPath = (new File(newPath)).getCanonicalPath();
		}
		catch (IOException ioe) { ioe.printStackTrace(); }
		
		currentDir = newPath;
		returnString = currentDir;
		label.setText(currentDir);
		if (returnDir()) text.setText("");
		fillTable();
	}
	
	private void closeCreateDir()
	{
		if (null!=createDirText) 	createDirText.dispose();
		if (null!=createDirItem) 	createDirItem.dispose();
		if (null!=createDirEditor)	createDirEditor.dispose();
	}
	
	private void submit(int value)
	{
		returnValue = value;
		closeDialog();
	}
	
	public String openStr()
	{
		super.open();
		if (null!=returnValue && returnValue==SWT.OK) return returnString;
		return null;
	}
	
	@Override public int open() { throw new IllegalStateException("use openStr() for NBDiskDialog"); }
}
