package gov.anl.gmca.common.ui;

import gov.anl.gmca.io.Disk;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

public class NBFileDialog extends NBDiskDialog
{
	private String currentFile = "";
	public void setFileName(String s) { currentFile = s; }
	
	private int currentFilterIndex = 0;

	private String[] filterNames = new String[0];
	public void setFilterNames(String[] names) { filterNames = names; }
	
	private String[] filterExtensions = new String[0];
	public void setFilterExtensions(String[] extensions) { filterExtensions = extensions; }

	protected Combo filterCombo;
	
	// the two modes are "save" and "open"
	private boolean saveMode = false;
	
	@Override protected boolean returnDir() 			{ return false; }
	@Override protected boolean returnValueMustExist()	{ return !saveMode;  }

	public NBFileDialog(Shell parentShell, int style)
	{
		super(parentShell, style);
		
		saveMode = (0 != (style & SWT.SAVE));
		dialogTitle = "Choose a file";
	}

	@Override protected void createContents(Composite parent)
	{
		super.createContents(parent);
		
		if (null != currentFile)
		{
			text.setText(currentFile);
			text.setSelection(0,currentFile.length());
		}
		
		if (saveMode)
		{
			locationLabel.setText("Name:");
		}
	}
	
	@Override protected void handleTraverse(int keyCode)
	{
		if (!saveMode && keyCode == SWT.TRAVERSE_RETURN)
		{
			if (Disk.get().isDirectory(returnString))
			{
				cd(returnString);
				return;
			}
		}
		
		super.handleTraverse(keyCode);
	}

	
	@Override protected void createFilterCombo(Composite parent)
	{
		if (filterExtensions.length==0) return;

		filterCombo = new Combo(parent, SWT.READ_ONLY);
		for (int i=0; i<filterExtensions.length;i++)
		{
			String filterText = filterExtensions[i];
			if (filterNames.length-1 >= i) filterText = filterNames[i];
			filterCombo.add(filterText);
		}
		
		GridData gd = new GridData(SWT.RIGHT, SWT.DEFAULT, false, false);
		filterCombo.setLayoutData(gd);
		filterCombo.select(0);
	}
	
	@Override protected void connectListeners()
	{
		super.connectListeners();
		
		filterCombo.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				currentFilterIndex = filterCombo.getSelectionIndex();
				fillTable();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	}
	
	@Override public String openStr()
	{
		if (filterExtensions.length==0)
		{
			filterExtensions = new String[]{"*.*"};
			filterNames = new String[]{"All Files (*.*)"};
		}
		String ret = super.openStr();
		if (!Disk.isDirectory(ret)) return ret;
		return null;
	}

	@Override protected boolean showFile(String fullPath)
	{
		if (filterExtensions.length-1 >= currentFilterIndex)
		{
			String filter = filterExtensions[currentFilterIndex];
			if (filter.equals("*.*"))
			{
				return true;
			}
			else if (filter.indexOf("*")==0)
			{
				String extension = filter.substring(1);
				boolean match = fullPath.endsWith(extension);
				return match;
			}
			else
			{
				throw new IllegalArgumentException("Unsupported extension: "+filter);
			}
		}
		else
		{
			throw new IllegalStateException("Filter index "+currentFilterIndex+" not specified");
		}
	}
}
