package gov.anl.gmca.common.ui;

import gov.anl.gmca.common.EpicsDevice;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import java.util.ArrayList;

public abstract class WdgControl extends Composite
{
	protected static ArrayList<WdgControl> controlList = new ArrayList<WdgControl>();
	
	public static abstract class ModelInterface
	{
		public abstract void setValue(Object value);
		public void updateModel(Object value) {}
		public EpicsDevice getDevice()		{ return null;			}
		public Object getDeviceValue() 		{ return null;			} // used to highlight when entries differ from motor positions
		public Object getModelValue()		{ return null;			} // used to determine whether and update should be sent
		public Limits getLimits() 			{ return null; 			}
		public int getDecimals() 			{ return 0; 			}
		public String[] getChoices() 		{ return null; 			}
		public double[] getFloatChoices() 	{ return null; 			}
		public String formatValue(Object o) { if (!(o instanceof Double)) return o.toString(); if ((Double)o==Double.POSITIVE_INFINITY) return "\u221E"; return Util.printDouble((Double)o,getDecimals()); }
	}
	
	protected final ModelInterface mi;
	public ModelInterface getMI() { return mi; }
	
	public WdgControl(Composite parent, int style, ModelInterface _mi)
	{
		super(parent, style);
		mi=_mi;
		controlList.add(this);
	}
	
	public static void setControlVisible(Control control, boolean isVisible)
	{
		if (null == control) return;
		
		Object ld = control.getLayoutData();
		if (ld instanceof GridData)
		{
			((GridData)ld).exclude = !isVisible;
		}
		control.setVisible(isVisible);
	}
	
	// returns false if the user input is invalid
	public abstract boolean updateModel();

	public static void addListenerToControlRecursively(Control c, MouseListener ml)
	{
		c.addMouseListener(ml);
		if (c instanceof Composite)
			for (Control cc : ((Composite)c).getChildren())
				addListenerToControlRecursively(cc, ml);
	}
	public static void addMoveListenerToControlRecursively(Control c, MouseMoveListener mml)
	{
		c.addMouseMoveListener(mml);
		if (c instanceof Composite)
			for (Control cc : ((Composite)c).getChildren())
				addMoveListenerToControlRecursively(cc, mml);
	}
}
