package gov.anl.gmca.common.ui;

import gov.anl.gmca.common.DevMoveFloat;
import gov.anl.gmca.common.EpicsDevice.State;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import java.util.HashMap;

/**
 * A non-interactive two-column display that shows motor names on the left and their positions on the right.
 * 
 * Labels and PV names are passed in through the constructor.
 */
public class WdgPositionDisplay extends Composite
{
	protected HashMap<DevMoveFloat,Label> statLabels = new HashMap<DevMoveFloat,Label>();
	
	public WdgPositionDisplay(Composite parent, int style, String title, Object[][] statList, Font groupFont)
	{
		super(parent,style);
		
		setLayout(new FillLayout());
		
		Group statusGroup = new Group(this,SWT.NONE);
		statusGroup.setFont(groupFont);
		statusGroup.setBackground(parent.getBackground());
		statusGroup.setText(title);
		statusGroup.setLayout(new GridLayout(2,false));
		
		for (int i=0;i<statList.length;i++)
		{
			Label lblName = new Label(statusGroup,SWT.NONE);
			lblName.setBackground(GUI.BGCOLOR);
			lblName.setText(statList[i][0]+" =");
			lblName.setFont(GUI.FONT_POSITION_DISPLAY);
			lblName.setAlignment(SWT.RIGHT);
			GridData gd = new GridData();
			gd.widthHint = 75;
			gd.grabExcessVerticalSpace = true;
			gd.verticalAlignment = SWT.FILL;
			gd.grabExcessHorizontalSpace = true;
			gd.horizontalAlignment = SWT.RIGHT;
			lblName.setLayoutData(gd);
			
			Label lblStat = new Label(statusGroup,SWT.SHADOW_OUT|SWT.RIGHT);
			lblStat.setBackground(GUI.BGCOLOR);
			lblStat.setText("  -----  ");
			lblStat.setFont(GUI.FONT_POSITION_DISPLAY);
			lblStat.setAlignment(SWT.LEFT);
			gd = new GridData();
			gd.grabExcessHorizontalSpace = true;
			gd.horizontalAlignment = SWT.FILL;
			gd.grabExcessVerticalSpace = true;
			gd.verticalAlignment = SWT.FILL;
			lblStat.setLayoutData(gd);
			statLabels.put((DevMoveFloat)statList[i][1],lblStat);
		}

		for (int i=0;i<statList.length;i++)
		{
			final DevMoveFloat dev = (DevMoveFloat)statList[i][1];
			
			updateLabel(dev);

			dev.addListener(new DevMoveFloat.Listener()
			{
				@Override public void handleUnitsChange(String units) {}
				@Override public void handleStateChange(State state) {}
				@Override public void handleRqsPosChange(double newPos) {}
				@Override public void handleActPosChange(DevMoveFloat dev, double newPos)
				{
					updateLabel(dev);
				}
			});
		}
	}
	
	protected void updateLabel(DevMoveFloat dev)
	{
		String sValue = "";
		
		double d = dev.getPosition();

		sValue = Util.sprintf("%-8.3f", d);
		if (sValue.charAt(0) != '-') sValue = " "+sValue;
		if (sValue.equals("-0.000")) sValue = " 0.000";
		if (sValue.length()>9)
		{
			sValue = Util.sprintf(" %8.3g",d);
		}
		
		final String f_sValue = sValue;
		final Label statLabel = statLabels.get(dev);
		
		GUI.asyncExec(new Runnable() { public void run()
		{
		   if (!statLabel.isDisposed())
		   {
			   statLabel.setText(f_sValue);
		   }
		}});
	}
}
