package gov.anl.gmca.common.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;

public class WdgSegmentedToggle extends Composite implements PaintListener, ControlListener, MouseListener
{
	private String leftText, rightText;
	private Point leftExtent, rightExtent;
	private int selectedIndex = -1;
	public int getSelectedIndex() { return selectedIndex; }
	private Font font = GUI.FONT_10_BOLD;
	public ListenerList<WdgSegmentedToggle> llChanged, llClicked;
	
	public WdgSegmentedToggle(Composite parent)
	{
		super(parent, SWT.NONE);
		llChanged = new ListenerList<>(this);
		llClicked = new ListenerList<>(this);
		
		addPaintListener(this);
		addControlListener(this);
		addMouseListener(this);
	}
	
	public void setText(String leftText, String rightText)
	{
		this.leftText=leftText; this.rightText=rightText;
		recalc();
	}
	
	public void setSelection(int index)
	{
		if (index!=selectedIndex)
		{
			selectedIndex=index;
			llChanged.notifyListeners(selectedIndex);
			redraw();
		}
	}
	
	private int maxExtentX, leftOffsetX, rightOffsetX, dividerX;
	private int marginX=5, marginY=5, arcSize=8;
	private Point drawLeftText, drawRightText;
	
	private void recalc()
	{
		// text size
		GC gc = new GC(GUI.getDisplay());
		gc.setFont(font);
		leftExtent = gc.textExtent(leftText);
		rightExtent = gc.textExtent(rightText);
		gc.dispose();

		// use the higher horizontal extent for sizing
		maxExtentX = Math.max(leftExtent.x,rightExtent.x);
		leftOffsetX = maxExtentX-leftExtent.x;
		rightOffsetX = maxExtentX-rightExtent.x;
		
		// adjust margins to fill requested size
		// totalX = maxExtentX*2 + marginX*4
		// marginX = 0.25 * (totalX - maxExtentX*2)
		marginX = (int)(0.25 * (getSize().x - maxExtentX*2));
		// totalY = extentY + marginY*2
		// marginY = 0.5 * (totalY - extentY)
		marginY = (int)(0.5 * (getSize().y - leftExtent.y));
		
		// coordinates for drawing
		dividerX = maxExtentX + 2*marginX;
		drawLeftText = new Point(marginX+leftOffsetX+1,marginY);
		drawRightText = new Point(dividerX+marginX+rightOffsetX,marginY);
		
		// draw the new text
		redraw();
	}

	@Override public void paintControl(PaintEvent e)
	{
		GC gc = e.gc;
		
		gc.setBackground(getParent().getBackground());
		gc.fillRectangle(0, 0, getSize().x, getSize().y);
		
		Color fgColor	= GUI.COLOR_WHITE;
		Color activeBgL = GUI.COLOR_RED;
		Color activeBgR = GUI.COLOR_GREEN;
		Color passiveBg = GUI.COLOR_GRAY;
		
		// left side background
		gc.setBackground(selectedIndex==0 ? activeBgL : passiveBg);
		gc.setClipping(0, 0, dividerX, getSize().y);
		gc.fillRoundRectangle(0, 0, getSize().x-1, getSize().y-1, arcSize, arcSize);

		// right side background
		gc.setBackground(selectedIndex==1 ? activeBgR : passiveBg);
		gc.setClipping(dividerX, 0, getSize().x, getSize().y);
		gc.fillRoundRectangle(0, 0, getSize().x-1, getSize().y-1, arcSize, arcSize);

		// outlines
		gc.setClipping(0, 0, getSize().x, getSize().y);
		gc.setForeground(GUI.COLOR_BLACK);
		gc.drawRoundRectangle(0, 0, getSize().x-1, getSize().y-1, arcSize, arcSize);
		gc.drawLine(dividerX, 0, dividerX, getSize().y);

		// text
		gc.setForeground(fgColor);
		gc.drawString(leftText, drawLeftText.x, drawLeftText.y, true);
		gc.drawString(rightText, drawRightText.x, drawRightText.y, true);
	}

	@Override public Point computeSize(int wHint, int hHint)
	{
		return getSize();
	}
	
	@Override public Point computeSize(int wHint, int hHint, boolean changed)
	{
		return getSize();
	}
	
	@Override public Point getSize()
	{
		return new Point(110,28);
	}
	
	@Override public void controlMoved(ControlEvent arg0) {}
	@Override public void controlResized(ControlEvent e)
	{
		recalc();
	}

	@Override public void mouseDoubleClick(MouseEvent e) {}

	private int clickedIndex=-1;
	@Override public void mouseDown(MouseEvent e)
	{
		clickedIndex = e.x < dividerX ? 0 : 1;
	}

	@Override public void mouseUp(MouseEvent e)
	{
		if (e.x>0&&e.y>0&&e.x<getSize().x&&e.y<getSize().y)
		{
			selectedIndex=clickedIndex;
			llClicked.notifyListeners(selectedIndex);
			redraw();
		}
		clickedIndex = -1;
	}
}
