/**
 *  @author spothineni
 *  
 *  this class runs MySQL queries on DataProcResults table form user_data database
 *  somehow dataprocessing thread locks are ended up this this class because it just created
 *  only once, refer to lock and lock1 objects which are used in InitDataprocessing
 *  
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.dataproc.ModelDataProc.DataProcField;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_ANALYSIS;
import gov.anl.gmca.log.db.CollectEvents;
import gov.anl.gmca.log.db.DataProcEvents;
import gov.anl.gmca.log.db.DataProcEvents.DataProcParam;
import gov.anl.gmca.syslog.Log;
import gov.aps.jca.dbr.DBR;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

public class DevReprocessing {
	protected static final String tableName = "DataProcResults";
	protected static final boolean UPDATE_CURRENT_INDEX_ON_RESULT_CREATION = true;
	protected static final boolean SHOW_DATAPROC_TAB_ON_RESULT_CREATION = true;

	protected static boolean doesTableExist() {
		Database.Result result = Database.exec(DBName.USERDATA,
				"show tables like '" + tableName + "'");
		boolean exists = (result.numRows() > 0);
		return exists;
	}

	public synchronized static boolean verifyTableExists() {
		if (!doesTableExist()) {
			System.out.println("Creating DataProc results table");
			String createCmd = "CREATE TABLE `" + tableName + "` ("
					+ "`sampleNumber` int(11) NOT NULL,"
					+ "`firstImg` int(11) NOT NULL,"
					+ "`lastImg` int(11) NOT NULL,"
					+ "`unitcell` varchar(512) default '',"
					+ "`spacegroup` varchar(512) default '',"
					+ " `timestamp` varchar(512) default '');";
			Database.exec(DBName.USERDATA, createCmd);
		}

		return doesTableExist();
	}

	protected boolean connected = false;
	protected Epics.EpicsValueMonitor dbUpdatedMonitor;

	public static DevReprocessing get() {
		return inst;
	}

	protected static DevReprocessing inst = null;

	public DevReprocessing() {
		inst = this;
	}

	public void verifyConnected() {
		if (connected)
			return;

		updateAllRows();

		Epics.addMonitor(PV_ANALYSIS.currentDataProcIndex,
				new Epics.EpicsValueMonitor() {
					@Override
					public void valueChanged(String pvName, DBR value) {
						ModelDataProcList.get().setCurrentDataProcIndex(
								Epics.dbrToInt(value));
						ModelDataProcList.get().ll.notifyListeners();
					}
				});

		dbUpdatedMonitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				int intVal = Epics.dbrToInt(value);
				if (-1 != intVal)
					handlePVUpdate(intVal);
			}
		};
		Epics.addMonitor(PV_ANALYSIS.dataprocUpdated, dbUpdatedMonitor);

		connected = true;
	}

	public void setResult(int sampleNumber, ModelDataProcParams dataprocResult,
			boolean update) {
		verifyTableExists();
		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		// if (0==result.numRows())
		{
			Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (sampleNumber, sampleName, state, workdir, images, imageSet, log, warning) "
							+ "values (" + sampleNumber + ", '"
							+ dataprocResult.sampleName + "', '"
							+ dataprocResult.state + "', '"
							+ dataprocResult.workDir + "', '"
							+ dataprocResult.images + "', '"
							+ dataprocResult.imageSet + "', '"
							+ dataprocResult.log + "', '"
							+ dataprocResult.warning + "')");
			newResultCreated = true;
		}
		
		if (update) {

			// update the GUI directly
			updateRow(sampleNumber, true);
			if (newResultCreated && UPDATE_CURRENT_INDEX_ON_RESULT_CREATION) {
				int last = ModelDataProcList.get().getDataProcList().size() - 1;
				DevDataProc.setCurrentDataProcIndex(last);
			}
			// send the PV update signal
			sendRowUpdate(sampleNumber);
		}
	}

	public void setDataProcResult(int sampleNumber,
			ModelDataProcParams dataprocResult, String imageSet, boolean update) {
		verifyTableExists();

		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber + " and imageSet='" + imageSet + "'");
		if (0 == result.numRows()) {
			Database.exec(DBName.USERDATA, "insert into `" + tableName
					+ "` (sampleNumber,state) " + "values (" + sampleNumber
					+ ",'DONE')");
			newResultCreated = true;
		}

		String updateCmd = "update `" + tableName + "` set ";
		String DATE_FORMAT_NOW = "yyyy-MM-dd-HH:mm:ss";
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
		String timestamp = sdf.format(date);

		updateCmd += "state='" + dataprocResult.state;
		updateCmd += "', images='" + dataprocResult.images;
		updateCmd += "', software='" + dataprocResult.software;
		updateCmd += "', scalingsoftware='" + dataprocResult.scalingsoftware;
		updateCmd += "', collectType='" + dataprocResult.collectType;
		updateCmd += "', collectType='" + dataprocResult.firstImage;
		updateCmd += "', warning=\"" + dataprocResult.warning;
		updateCmd += "\", lowresolution='" + dataprocResult.lowresolution;
		updateCmd += "', lowresolution_inner='"
				+ dataprocResult.lowresolution_inner;
		updateCmd += "', lowresolution_outer='"
				+ dataprocResult.lowresolution_outer;
		updateCmd += "', highresolution='" + dataprocResult.highresolution;
		updateCmd += "', highresolution_inner='"
				+ dataprocResult.highresolution_inner;
		updateCmd += "', highresolution_outer='"
				+ dataprocResult.highresolution_outer;
		updateCmd += "', unitcell='" + dataprocResult.unitCell;
		updateCmd += "', spacegroup='" + dataprocResult.spaceGroup;
		updateCmd += "', rmerge='" + dataprocResult.rMerge;
		updateCmd += "', completeness='" + dataprocResult.completeness;
		updateCmd += "', anom_completeness='"
				+ dataprocResult.anom_completeness;
		updateCmd += "', multiplicity='" + dataprocResult.multiplicity;
		updateCmd += "', anom_multiplicity='"
				+ dataprocResult.anom_multiplicity;
		updateCmd += "', isigma='" + dataprocResult.iSigma;
		updateCmd += "', rmerge_inner='" + dataprocResult.rMerge_inner;
		updateCmd += "', completeness_inner='"
				+ dataprocResult.completeness_inner;
		updateCmd += "', anom_completeness_inner='"
				+ dataprocResult.anom_completeness_inner;
		updateCmd += "', multiplicity_inner='"
				+ dataprocResult.multiplicity_inner;
		updateCmd += "', anom_multiplicity_inner='"
				+ dataprocResult.anom_multiplicity_inner;
		updateCmd += "', isigma_inner='" + dataprocResult.iSigma_inner;
		updateCmd += "', rmerge_outer='" + dataprocResult.rMerge_outer;
		updateCmd += "', completeness_outer='"
				+ dataprocResult.completeness_outer;
		updateCmd += "', anom_completeness_outer='"
				+ dataprocResult.anom_completeness_outer;
		updateCmd += "', multiplicity_outer='"
				+ dataprocResult.multiplicity_outer;
		updateCmd += "', anom_multiplicity_outer='"
				+ dataprocResult.anom_multiplicity_outer;
		updateCmd += "', isigma_outer='" + dataprocResult.iSigma_outer;
		updateCmd += "', scalalog='" + dataprocResult.scalaLog;
		updateCmd += "', truncatelog='" + dataprocResult.truncateLog;
		updateCmd += "', subdir='" + dataprocResult.subdir;
		updateCmd += "', timestamp='" + timestamp;
		updateCmd += "' where sampleNumber=" + sampleNumber + " and imageSet='"
				+ imageSet + "'";

		Database.exec(DBName.USERDATA, updateCmd);

		if (update) {
			// send the PV update signal
			sendRowUpdate(sampleNumber);

			/* update GM/CA database */
			if (imageSet.contains("Final_"))
				updateDataProcEvent(sampleNumber);
		}
	}

	protected void updateRow(int row, String subset, boolean update) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row
						+ " and imageSet='" + subset + "'");

		ModelDataProc dataProcResult;
		dataProcResult = sqlRowToResult(result.getRow(0));
		dataProcResult.imageSets = getImageSets(result.getRow(0));
		ModelDataProcList.get().setRow(dataProcResult, row, true);
		ModelDataProcList.get().ll.notifyListeners(row);
	}

	protected void updateRow(int row, boolean update) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row);
		if (result.numRows() < 1)
			return;
		ModelDataProc dataProcResult;
		dataProcResult = sqlRowToResult(result.getRow(result.numRows() - 1));
		dataProcResult.imageSets = getImageSets(result.getRow(0));
		ModelDataProcList.get().setRow(dataProcResult, row, true);
		ModelDataProcList.get().ll.notifyListeners(row);
	}

	protected void updateAllRows() {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " group by sampleNumber");
		if (result == null || result.numRows() < 1)
			return;
		System.out.println("Number of rows in the DataProcResults table "
				+ result.numRows());
		ModelDataProcList.get().clearResults();

		for (HashMap<String, String> row : result.getRows()) {
			ModelDataProc dataProcResult = sqlRowToResult(row);
			dataProcResult.imageSets = getImageSets(row);
			if (null != dataProcResult)
				ModelDataProcList.get().setRow(dataProcResult,
						Integer.parseInt(row.get("sampleNumber")), false);
		}
		ModelDataProcList.get().ll.notifyListeners(-2);
	}

	private ArrayList<String> getImageSets(HashMap<String, String> row) {
		final Database.Result result3 = Database.exec(
				DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber="
						+ row.get("sampleNumber"));
		
		ArrayList<String> imageSets = new ArrayList<String>();
		for (HashMap<String, String> row2 : result3.getRows()) {
			for (String sqlCol : row2.keySet()) {
				if (sqlCol.equals("imageSet")) {
					imageSets.add(row2.get(sqlCol));
				}
			}
		}
		return (imageSets);
	}

	protected ModelDataProc sqlRowToResult(HashMap<String, String> row) {
		HashMap<DataProcField, String> fieldMap = new HashMap<DataProcField, String>();

		for (String sqlCol : row.keySet()) {
			for (DataProcField field : DataProcField.values()) {
				if (field.getDBString().equals(sqlCol)) {
					fieldMap.put(field, row.get(sqlCol));
				}
			}
		}
		return new ModelDataProc(fieldMap);
	}

	protected void updateDataProcEvent(int num) {

		HashMap<DataProcParam, String> fieldMapStr = new HashMap<DataProcParam, String>();
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + num);
		if (result.numRows() < 1)
			return;
		HashMap<String, String> row = result.getRow(result.numRows() - 1);

		for (String sqlCol : row.keySet()) {
			for (DataProcParam fieldStr : DataProcParam.values()) {
				if (fieldStr.getDBField().fieldName.equals(sqlCol)) {
					fieldMapStr.put(fieldStr, row.get(sqlCol));
				}
			}
		}
		final Database.Result result1 = Database.exec(DBName.GMCADATA,
				"select * from " + CollectEvents.GMCA_COLLECT_DATA_TABLE);
		fieldMapStr.put(DataProcParam.COLLECT_NUMBER,
				Integer.toString(result1.numRows()));
		DataProcEvents.writeEvent(fieldMapStr);
	}

	public int getRowCount() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " group by sampleNumber");
		count = result.numRows();
		return count;
	}

	protected void handlePVUpdate(int pvValue) {
		verifyTableExists();
		if (pvValue >= 0) {
			System.out.println("Updating row " + pvValue);
			updateRow(pvValue, true);
		} else if (-2 == pvValue) {
			System.out.println("Updating all rows " + pvValue);
			updateAllRows();
		}
	}

	protected void sendRowUpdate(int sampleNumber) {
		Epics.putInteger(PV_ANALYSIS.dataprocUpdated, -1);
		Epics.putInteger(PV_ANALYSIS.dataprocUpdated, sampleNumber);
	}

	public static int getRows() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		count = result.numRows();
		return count;
	}

	public static void setCurrentDataProcIndex(int i) {
		Epics.putInteger(PV_ANALYSIS.currentDataProcIndex, i);
	}

	public void deleteAllResults() {
		Log.note("Deleting all strategy results");

		verifyTableExists();

		setCurrentDataProcIndex(0);

		Database.exec(DBName.USERDATA, "delete from " + tableName);
		sendRowUpdate(-2);
	}

	public void saveAllResults(String excelFile) {
		// WritableWorkbook workbook = Workbook.createWorkbook(new
		// File(excelFile));
		Workbook wbs;
		
		try {
			OutputStream inp = new FileOutputStream(excelFile);

			wbs = new HSSFWorkbook();

			Sheet s = wbs.createSheet();
			// declare a row object reference
			Row r = null;
			// declare a cell object reference
			Cell c = null;
			
			r = s.createRow(0); // write the column names

			final Database.Result result = Database.exec(DBName.USERDATA,
					"select * from " + tableName);
			if (result.numRows() < 1) {
				inp.close();
				return;
			}

			for (DataProcField field : DataProcField.values()) {

				c = r.createCell(field.ordinal());
				c.setCellValue(field.getDBString());
			}
			// write data
			int k = 1;
			int j = 0;
			for (HashMap<String, String> row : result.getRows()) {

				Row r1 = s.createRow(k);
				j = 0;
				for (DataProcField field : DataProcField.values()) {
					c = r1.createCell(j);
					for (String sqlCol : row.keySet()) {
						if (field.getDBString().equals(sqlCol))
							c.setCellValue(row.get(sqlCol));
					}
					j++;
				}
				k++;
			}
			wbs.write(inp);
			inp.close();
		} catch (IOException ex) {
			Log.error("Excel export failed :" + ex.getMessage());
		}
	}

	public String getWorkingDir(int sampNumber) {
		final Database.Result result3 = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber="
						+ sampNumber);
		String workDir = "";
		for (HashMap<String, String> row2 : result3.getRows()) {
			for (String sqlCol : row2.keySet()) {
				if (sqlCol.equals("workdir")) {
					workDir = row2.get(sqlCol);
				}
			}
		}

		return workDir;
	}
}
