/**
 * @author spothineni
 * 
 * This class used by Settings sub-tab to persist the data into MYSQL
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_ANALYSIS;
import gov.aps.jca.dbr.DBR;

import java.util.HashMap;

public class DevXDSSettings {
	protected static final String tableName = "XDSSettings";

	protected static boolean doesTableExist() {
		Database.Result result = Database.exec(DBName.USERDATA,
				"show tables like '" + tableName + "'");
		boolean exists = (result.numRows() > 0);
		return exists;
	}

	public synchronized static boolean verifyTableExists() {
		if (!doesTableExist()) {
			System.out.println("Creating DataProc results table");
			String createCmd = "CREATE TABLE `" + tableName + "` ("
					+ "`XDSparam` varchar(512) default '',"
					+ "`value` varchar(1024) default '');";
			Database.exec(DBName.USERDATA, createCmd);
		}

		return doesTableExist();
	}

	protected boolean connected = false;
	protected Epics.EpicsValueMonitor dbUpdatedMonitor;

	public static DevXDSSettings get() {
		return inst;
	}

	protected static DevXDSSettings inst = null;

	public DevXDSSettings() {
		inst = this;
	}

	public void verifyConnected() {
		if(verifyTableExists())
			setResult();
		
		if (connected)
			return;
		
		//updateAllRows();

		Epics.addMonitor(PV_ANALYSIS.updateXDSparams,
				new Epics.EpicsValueMonitor() {
					
				@Override
				public void valueChanged(String pvName, DBR value) {
					updateAllRows();
					}
		});

		
		Epics.addMonitor(PV_ANALYSIS.updateXDSparams, dbUpdatedMonitor);

		connected = true;
	}

	public void setResult() {
		
		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "`");
		if (0==result.numRows())
		{
			for(ModelXDSSettings.DefaultParam xdsParam : ModelXDSSettings.params) {
				Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (XDSparam, value) "
							+ "values ('" + xdsParam.getXDSParam() + "', '"
							+ xdsParam.getValue() + "')");
				
				ModelXDSSettings.paramValues.put(xdsParam, xdsParam.getValue());
			}
			newResultCreated = true;
		} else {
			updateAllRows();
		}
	}
	
	public void setResult(ModelXDSSettings.DefaultParam param, boolean update) {
		verifyTableExists();
		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "`");
		if (0==result.numRows())
		{
			for(ModelXDSSettings.DefaultParam xdsParam : ModelXDSSettings.params) {
				Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (XDSparam, value) "
							+ "values (" + xdsParam.getXDSParam() + ", '"
							+ xdsParam.getValue() + "')");
			
			}
			newResultCreated = true;
		} else {
			String updateCmd = "update `" + tableName + "` set ";
			updateCmd += "XDSparam='" + param;
			updateCmd += "', value='" + ModelXDSSettings.changedValues.get(param)+"'";
			updateCmd += " where XDSparam='"+param+"'";
			Database.exec(DBName.USERDATA, updateCmd);
		}
		
		ModelXDSSettings.get().ll.notifyListeners();
		if(update)
			setCurrentDataProcIndex(1);
	}
	protected void updateAllRows() {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		System.out.println("Number of rows " + result.numRows());
		
		for (HashMap<String, String> row : result.getRows()) {
			for (ModelXDSSettings.DefaultParam id : ModelXDSSettings.params) {

				 if(id.getXDSParam().equals((String)row.get("XDSparam"))) {
					//System.out.println(id.getXDSParam()+"="+row.get(id.getXDSParam()));
				 	ModelXDSSettings.paramValues.put(id,row.get("value"));
				 	
				 	if(!row.get("value").equals(id.getValue())) {
				 		ModelXDSSettings.changedValues.put(id,row.get("value"));
				 	}
				 }
			}	
		}
		ModelXDSSettings.get().ll.notifyListeners();
	}

	public static int getRows() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		count = result.numRows();
		return count;
	}

	public static void setCurrentDataProcIndex(int i) {
		Epics.putInteger(PV_ANALYSIS.updateXDSparams, 0);
		Epics.putInteger(PV_ANALYSIS.updateXDSparams, 1);
	}
}
