package gov.anl.gmca.dataproc;

import gov.anl.gmca.io.Disk;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.io.*;
import java.util.ArrayList;
import java.util.StringTokenizer;

public class ModelAimlessSummary {
	String workDir, subDir;
	String highresolution, highresolution_inner;

	public ModelAimlessSummary() {

	}

	public void processResult(String _workDir,
			ModelDataProcParams _modelDataProcParams) throws Exception {
		String outputFile = "", xdsLog = "";
		int pos1, pos2, pos3, pos4;
		workDir = _workDir;

		/* Read XDS log file */
		xdsLog = _modelDataProcParams.log;

		/* Sub directory where graph images are stored */
		/*if(!_modelDataProcParams.inverseOn && _modelDataProcParams.overlap.equals("0") && !_modelDataProcParams.collectType.equals("Raster")) {
			subDir = workDir + "/" + _modelDataProcParams.sampleName + "_"
				+ (_modelDataProcParams.software?"fast_dp":"GMCAproc") + "_"
				+ _modelDataProcParams.imageSet + "/";
		} else {
			subDir = workDir + "/" + CmdDataProc.tempLogDir+ "/";
		}*/

		subDir = _modelDataProcParams.subdir;

		/* aimless log */
		outputFile = subDir + "/"
				+ _modelDataProcParams.sampleName + "_aimless.log";
		_modelDataProcParams.truncateLog = subDir + "/"
				+ _modelDataProcParams.sampleName + "_ctruncate.log";
		_modelDataProcParams.aimlessLog = outputFile;

		/*fast_dp or GMCAproc log */
		File xdsFile = new File(xdsLog);
		if (!xdsFile.exists()) {
			Log.error(xdsLog + " not found, automated processing failed.");
			_modelDataProcParams.state = "ERROR";
			_modelDataProcParams.warning = xdsLog
					+ " doesnt exists check data!!!";
			return;
		} else {
			String xdsContent = readLogFile(xdsLog);
			if (_modelDataProcParams.log.contains("fast_dp")) {
				if (xdsContent.contains("error:")) {
					pos1 = xdsContent.lastIndexOf("error:");
					_modelDataProcParams.warning = xdsContent.substring(pos1).replace("\"", "");
					if(xdsContent.contains("Fast DP error: 1")) {
						_modelDataProcParams.warning = "Autoindexing failed";
					}
					if(xdsContent.contains("Autoindexing error:")) {
						_modelDataProcParams.warning = xdsContent.substring(xdsContent.indexOf("Autoindexing error:"));
					}
					_modelDataProcParams.state = "ERROR";
					return;
				}
			} else {
				if (xdsContent.contains("!!! ERROR ")) {
					pos1 = xdsContent.lastIndexOf("!!! ERROR ");
					int posNewline = 0;
					if(pos1 > 0) {
						posNewline = xdsContent.indexOf("\n", pos1);
						if(posNewline > pos1)
							_modelDataProcParams.warning = xdsContent.substring(pos1, posNewline).replace("\"", "");
					}
					_modelDataProcParams.state = "ERROR";
					return;
				}
			}
		}

		Log.note("reading " + outputFile);

		/* reading aimless log for data quality parameters */
		File scalaFile = new File(outputFile);

		if (!scalaFile.exists()) {
			Log.note(_modelDataProcParams.aimlessLog
					+ " doesnt exists,scaling not successful, check XDS log file for errors!!!");
			_modelDataProcParams.warning = _modelDataProcParams.aimlessLog
					+ " doesnt exists, scaling not successful, check XDS log file for errors!!!";
			return;
		}

		File truncateFile = new File(outputFile);

		if (!truncateFile.exists()) {
			Log.note(_modelDataProcParams.truncateLog
					+ " doesnt exists, scaling not successful, check data!!!");
			_modelDataProcParams.warning = _modelDataProcParams.truncateLog
					+ " doesnt exists, scaling not successful, check data!!!";
		}

		/* Read scala log file */
		String content = readLogFile(outputFile);

		_modelDataProcParams.scalaParams = "N/A";
		pos1 = content.indexOf("Overall  InnerShell  OuterShell");

		pos2 = content.indexOf("Average unit cell:", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Average unit cell doesnt find in aimless log";
			System.out.println("Reading "+_modelDataProcParams.aimlessLog+" Average unit cell doesnt find in aimless log");
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);
		String unitCell = content.substring(pos2 + 20, pos3);
		StringTokenizer tok = new StringTokenizer(unitCell);
		unitCell = tok.nextToken() + " ";
		unitCell = unitCell + " " + tok.nextToken();
		unitCell = unitCell + " " + tok.nextToken();
		unitCell = unitCell + " " + tok.nextToken();
		unitCell = unitCell + " " + tok.nextToken();
		unitCell = unitCell + " " + tok.nextToken();

		_modelDataProcParams.unitCell = unitCell;

		pos2 = content.indexOf("Space group:", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Space group doesnt find in aimless log";
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);
		_modelDataProcParams.spaceGroup = content.substring(pos2 + 13, pos3);

		pos2 = content.indexOf("maximum resolution", pos3);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "maximum resolution doesnt find in aimless log";
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);
		_modelDataProcParams.resolution = content.substring(pos2 + 24, pos3);

		pos2 = content.indexOf("High resolution limit", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "High resolution limit doesnt find in aimless log";
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String highres = content.substring(pos2 + 24, pos3);
		tok = new StringTokenizer(highres);
		String highres_ovearall = tok.nextToken();
		String highres_innershell = tok.nextToken();
		String highres_outershell = tok.nextToken();

		_modelDataProcParams.highresolution = highres_ovearall;
		_modelDataProcParams.highresolution_inner = highres_innershell;
		_modelDataProcParams.highresolution_outer = highres_outershell;

		highresolution = highres_ovearall;
		highresolution_inner = highres_innershell;
		pos2 = content.indexOf("Low resolution limit", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Low resolution limit doesnt find in aimless log";
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String lowres = content.substring(pos2 + 20, pos3);
		tok = new StringTokenizer(lowres);
		String lowres_ovearall = tok.nextToken();
		String lowres_innershell = tok.nextToken();
		String lowres_outershell = tok.nextToken();

		_modelDataProcParams.lowresolution = lowres_ovearall;
		_modelDataProcParams.lowresolution_inner = lowres_innershell;
		_modelDataProcParams.lowresolution_outer = lowres_outershell;
		// resolution = highres_ovearall;

		pos2 = content.indexOf("Rmerge ", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Rmerge doesnt find in aimless log";
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String rmerge = content.substring(pos2 + 25, pos3);
		tok = new StringTokenizer(rmerge);
		String rmerge_ovearall = tok.nextToken();
		String rmerge_innershell = tok.nextToken();
		String rmerge_outershell = tok.nextToken();

		_modelDataProcParams.rMerge = rmerge_ovearall;
		_modelDataProcParams.rMerge_inner = rmerge_innershell;
		_modelDataProcParams.rMerge_outer = rmerge_outershell;

		pos2 = content.indexOf("Rpim ", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Rmerge doesnt find in aimless log";
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String rpim = content.substring(pos2 + 25, pos3);
		tok = new StringTokenizer(rpim);
		String rpim_ovearall = tok.nextToken();
		String rpim_innershell = tok.nextToken();
		String rpim_outershell = tok.nextToken();

		_modelDataProcParams.rPim = rpim_ovearall;
		_modelDataProcParams.rPim_inner = rpim_innershell;
		_modelDataProcParams.rPim_outer = rpim_outershell;

		/* Reading CC1/2 */
		pos2 = content.indexOf("Mn(I) half-set correlation CC(1/2) ", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Mn(I) half-set correlation CC(1/2) doesnt find in aimless log";
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String cchalf = content.substring(pos2 + 36, pos3);
		tok = new StringTokenizer(cchalf);
		String cchalf_ovearall = tok.nextToken();
		String cchalf_innershell = tok.nextToken();
		String cchalf_outershell = tok.nextToken();

		_modelDataProcParams.cchalf = cchalf_ovearall;
		_modelDataProcParams.cchalf_inner = cchalf_innershell;
		_modelDataProcParams.cchalf_outer = cchalf_outershell;

		pos2 = content.indexOf("Completeness", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Completeness doesnt find in aimless log";
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String complete = content.substring(pos2 + 15, pos3);
		tok = new StringTokenizer(complete);
		String complete_ovearall = tok.nextToken();
		String complete_innershell = tok.nextToken();
		String complete_outershell = tok.nextToken();

		_modelDataProcParams.completeness = complete_ovearall;
		_modelDataProcParams.completeness_inner = complete_innershell;
		_modelDataProcParams.completeness_outer = complete_outershell;

		pos2 = content.indexOf("Anomalous completeness", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Anomalous completeness doesnt find in scala log";
			System.out.println("Reading "+_modelDataProcParams.aimlessLog+" Anomalous completeness doesnt find in scala log");
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String anom_complete = content.substring(pos2 + 25, pos3);
		tok = new StringTokenizer(anom_complete);
		String anom_complete_ovearall = tok.nextToken();
		String anom_complete_innershell = tok.nextToken();
		String anom_complete_outershell = tok.nextToken();

		_modelDataProcParams.anom_completeness = anom_complete_ovearall;
		_modelDataProcParams.anom_completeness_inner = anom_complete_innershell;
		_modelDataProcParams.anom_completeness_outer = anom_complete_outershell;

		pos2 = content.indexOf("Multiplicity", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Multiplicity doesnt find in scala log";
			System.out.println("Reading "+_modelDataProcParams.aimlessLog+" Multiplicity doesnt find in scala log");
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String multi = content.substring(pos2 + 15, pos3);
		tok = new StringTokenizer(multi);
		String multi_ovearall = tok.nextToken();
		String multi_innershell = tok.nextToken();
		String multi_outershell = tok.nextToken();

		_modelDataProcParams.multiplicity = multi_ovearall;
		_modelDataProcParams.multiplicity_inner = multi_innershell;
		_modelDataProcParams.multiplicity_outer = multi_outershell;

		pos2 = content.indexOf("Anomalous multiplicity", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Anomalous multiplicity doesnt find in scala log";
			System.out.println("Reading "+_modelDataProcParams.aimlessLog+" Anomalous multiplicity doesnt find in scala log");
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String anom_multi = content.substring(pos2 + 25, pos3);
		tok = new StringTokenizer(anom_multi);
		String anom_multi_ovearall = tok.nextToken();
		String anom_multi_innershell = tok.nextToken();
		String anom_multi_outershell = tok.nextToken();

		_modelDataProcParams.anom_multiplicity = anom_multi_ovearall;
		_modelDataProcParams.anom_multiplicity_inner = anom_multi_innershell;
		_modelDataProcParams.anom_multiplicity_outer = anom_multi_outershell;

		pos2 = content.indexOf("Mean((I)/sd(I))", pos1);
		if (pos2 < 0 || pos1 < 0) {
			//_modelDataProcParams.warning = "Mean doesnt find in scala log";
			System.out.println("Reading "+_modelDataProcParams.aimlessLog+" Mean doesnt find in scala log");
			_modelDataProcParams.warning = "Automated data processing failed";
			_modelDataProcParams.state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String isigma = content.substring(pos2 + 15, pos3);
		tok = new StringTokenizer(isigma);
		String isigma_ovearall = tok.nextToken();
		String isigmai_innershell = tok.nextToken();
		String isigmai_outershell = tok.nextToken();

		_modelDataProcParams.iSigma = isigma_ovearall;
		_modelDataProcParams.iSigma_inner = isigmai_innershell;
		_modelDataProcParams.iSigma_outer = isigmai_outershell;

		_modelDataProcParams.scalaParams = content.substring(pos1, pos2 + 20);





		/* Generate Graphs for analysis tab*/
		pos1 = content.indexOf("N  1/d^2    Dmid   Rmrg");
		pos2 = content.indexOf("$$", pos1);
		pos3 = content.indexOf("$$", pos2 + 10);

		pos4 = content.indexOf("\n", pos1);
		String header = content.substring(pos1, pos4);

		String resolutionTable = content.substring(pos2 + 3, pos3);
		if (resolutionTable.equals("") || resolutionTable == null)
			return;

		BufferedWriter out = new BufferedWriter(new FileWriter(subDir
				+ "/resolutionTable1.txt", false));

		out.write(header);
		out.write("\n");
		out.write(resolutionTable);
		out.close();

		out = new BufferedWriter(new FileWriter(workDir+ "/"+CmdDataProc.tempLogDir+ "/plot1.txt", true));

		out.write(subDir + "/resolutionTable1.txt");
		out.write("\n");
		out.close();
		/* write png files */

		writeGnuplotPNG("Rmerge vs Resolution", "Resolution (A)", "Rmerge", 3,
				4, workDir + "/"+CmdDataProc.tempLogDir+ "/plot1.txt", subDir + "/rmergevsresolution", 1);

		writeGnuplotPNG("Isigma vs Resolution", "Resolution (A)", "I/Sigma", 3,
				13, workDir + "/"+CmdDataProc.tempLogDir+ "/plot1.txt", subDir + "/isigmavsresolution",
				2);

		writeGnuplotPNG("Rmeas vs Resolution", "Resolution (A)", "Rmeas", 3, 7,
				workDir + "/"+CmdDataProc.tempLogDir+ "/plot1.txt", subDir + "/rmeasvsresolution", 1);

		pos1 = content.indexOf("N  1/d^2    Dmid    Nmeas");
		pos2 = content.indexOf("$$", pos1);
		pos3 = content.indexOf("$$", pos2 + 10);

		pos4 = content.indexOf("\n", pos1);
		header = content.substring(pos1, pos4);

		resolutionTable = content.substring(pos2 + 3, pos3);
		if (resolutionTable.equals("") || resolutionTable == null)
			return;

		out = new BufferedWriter(new FileWriter(subDir
				+ "/resolutionTable2.txt", false));

		out.write(header);
		out.write("\n");
		out.write(resolutionTable);
		out.close();

		out = new BufferedWriter(new FileWriter(workDir + "/"+CmdDataProc.tempLogDir+ "/plot2.txt", true));

		out.write(subDir + "/resolutionTable2.txt");
		out.write("\n");
		out.close();

		writeGnuplotPNG("Completeness vs Resolution", "Resolution (A)",
				"Completeness", 3, 8, workDir + "/"+CmdDataProc.tempLogDir+ "/plot2.txt", subDir
						+ "/completevsresolution", 3);

		writeGnuplotPNG("Multiplicity vs Resolution", "Resolution (A)",
				"Multiplicity", 3, 9, workDir + "/"+CmdDataProc.tempLogDir+ "/plot2.txt", subDir
						+ "/multivsresolution", 3);
	}

	protected void writeGnuplotPNG(String title, String xaxis, String yaxis,
			int col1, int col2, String fileName, String pngFile,
			int isRvalue) {
		BufferedWriter plotIntBS;
		try {
			plotIntBS = new BufferedWriter(new FileWriter(subDir
					+ "/vsresolution.sh", false));

			/* write gnupplot file */
			plotIntBS.append("set title \"" + title + "\" "+ "font \",11\"" + "\n");
			plotIntBS.append("set xlabel \"" + xaxis + "\" " + "font \",10\"" + "\n");
			//if (xaxis.equals("Resolution (A)"))
			//	plotIntBS.append("set xrange [" + highresolution + ":"
			//			+ highresolution_inner + "] reverse\n");

			/* apply boundaries */
			if (isRvalue == 1)
				plotIntBS.append("set yrange [0:0.5]\n");
			if (isRvalue == 2)
				plotIntBS.append("set yrange [0:30]\n");  // TODO if data contains  20 to 50 for i/sigma?

			plotIntBS.append("set ylabel \"" + yaxis + "\" font \",10\"" + "\n");
			plotIntBS.append("set terminal png size 320, 240 enhanced font \"/usr/share/fonts/liberation/LiberationSans-Regular.ttf, 8\"\n");
			plotIntBS.append("unset key\n");
			plotIntBS.append("set grid x\n");
			plotIntBS.append("set grid y\n");
			plotIntBS.append("set xtics rotate by 90 offset 0,-1.5\n");
			plotIntBS.append("set output \"" + pngFile + ".png\"" + "\n");
			plotIntBS.append("plot ");
			ArrayList<String> lines = null;

			// lines = Files.readAllLines(Paths.get(fileName),
			// Charset.defaultCharset());
			lines = Disk.readText(fileName);

			int i = 0;
			if (lines.size() > 0) {
				for (i = 0; i <= lines.size() - 2; i++) {
					plotIntBS.append("\"");
					plotIntBS.append(lines.get(i) + "\" using " + col2 + ":xticlabels("
							+ col1 + ")  with linespoints lw 2 ps 1 pt 4 title \"" + subDir
							+ "\"" + ",\\" + "\n");
				}
				plotIntBS.append("\"");
				plotIntBS.append(lines.get(i) + "\" using " + col2 + ":xticlabels(" + col1
						+ ")  with linespoints lw 2 ps 1 pt 4 title \"" + subDir + "\"");
			}

			if (plotIntBS != null)
				plotIntBS.close();

			Util.sleep(100);

			BufferedWriter plotIntBS1;
			try {
				plotIntBS1 = new BufferedWriter(new FileWriter(subDir
						+ "/run_gnuplot.sh", false));

				plotIntBS1.write("export GNUPLOT_DEFAULT_GDFONT=\n");
				plotIntBS1.write("/usr/bin/gnuplot "+subDir+"/vsresolution.sh\n");

				if (plotIntBS1 != null)
					plotIntBS1.close();

			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			ExecTask task = ExecTask.new_ExecTask("bash "+subDir
					+ "/run_gnuplot.sh", "", false, false);
			task.start();
			task.waitFor();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected String readLogFile(String logFile) {

		byte[] buffer = new byte[(int) new File(logFile).length()];
		BufferedInputStream f = null;
		try {
			f = new BufferedInputStream(new FileInputStream(logFile));
			f.read(buffer);
			f.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			if (f != null)
				try {
					f.close();
				} catch (IOException ignored) {
				}
		}

		return new String(buffer);
	}
}
