/**
 * @author spothineni
 * 
 * Model for Settings sub-tab of Analysis tab
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.util.ListenerList;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;


public class ModelXDSSettings {
	
	public static ListenerList<ModelXDSSettings> ll;
	/* Table data, one entry for each row */
	public static List<DefaultParam> params = new ArrayList<DefaultParam>();
	public static Map<DefaultParam, String> paramValues = new TreeMap<DefaultParam, String>();
	public static Map<DefaultParam, String> changedValues = new TreeMap<DefaultParam, String>();
	
	public static boolean takeXDSSettings = true;
	
	public static enum DefaultParam {
		
												// 	default 			value, 		helptext
		SPACE_GROUP_NUMBER 						("SPACE_GROUP_NUMBER", "0", "spacegroup will take a number, 0 if unknown"),
		UNIT_CELL_CONSTANTS  					("UNIT_CELL_CONSTANTS", "70 80 90 90 90 90", "unitcell Ex : 70 80 90 90 90 90, put correct values if known"),

		INCLUDE_RESOLUTION_RANGE 				("INCLUDE_RESOLUTION_RANGE", "50.0 0.0", "include resolution range Ex : 50.0 1.2"),
		EXCLUDE_RESOLUTION_RANGE 				("EXCLUDE_RESOLUTION_RANGE", "3.93 3.87", "exclude resolution range Ex : 3.2 3.3"),

		FREIDELS_LAW 							("FREIDELS_LAW", "TRUE", "This acts only on the CORRECT step : Ex : "),

		TRUSTED_REGION 							("TRUSTED_REGION", "0.0 1.4", "partially use corners of detectors; 1.41421=full use"),
		VALUE_RANGE_FOR_TRUSTED_DETECTOR_PIXELS ("VALUE_RANGE_FOR_TRUSTED_DETECTOR_PIXELS", "6000.0 30000.0", "often 7000 or 8000 is ok"),
		AIR 									("AIR", "", ""),
		BACKGROUND_PIXEL 						("BACKGROUND_PIXEL", "", ""),
		BACKGROUND_RANGE 						("BACKGROUND_RANGE", "", ""),
		BEAM_DIVERGENCE 						("BEAM_DIVERGENCE", "", ""),
		BEAM_DIVERGENCE_ESD 					("BEAM_DIVERGENCE_E.S.D.", "", ""),
		CLUSTER_RADIUS 							("CLUSTER_RADIUS", "", ""),
		CORRECTIONS 							("CORRECTIONS", "", ""),
		CUT 									("CUT", "", ""),
		DATA_RANGE 								( "DATA_RANGE", "", ""),
		DATA_RANGE_FIXED_SCALE_FACTOR 			("DATA_RANGE_FIXED_SCALE_FACTOR", "", ""),
		DELPHI 									("DELPHI", "", ""),
		DETECTOR_DISTANCE 						("DETECTOR_DISTANCE", "", ""),
		DIRECTION_OF_DETECTOR_XAXIS 			("DIRECTION_OF_DETECTOR_X-AXIS", "1 0 0", ""),
		DIRECTION_OF_DETECTOR_YAXIS 			("DIRECTION_OF_DETECTOR_Y-AXIS", "0 1 0", ""),
		DIRECTION_OF_SEGMENT_XAXIS 				("DIRECTION_OF_SEGMENT_X-AXIS", "1 0 0", ""),
		DIRECTION_OF_SEGMENT_YAXIS 				("DIRECTION_OF_SEGMENT_Y-AXIS", "0 1 0", ""),
		INDEX_ERROR 							("INDEX_ERROR", "", ""),
		INDEX_MAGNITUDE 						("INDEX_MAGNITUDE", "", ""),
		INDEX_ORIGIN 							("INDEX_ORIGIN", "", ""),
		INDEX_QUALITY 							("INDEX_QUALITY", "", ""),
		MAX_CELL_ANGLE_ERROR 					("MAX_CELL_ANGLE_ERROR", "", ""),
		MAX_CELL_AXIS_ERROR 					("MAX_CELL_AXIS_ERROR", "", ""),
		MAX_FAC_Rmeas 							("MAX_FAC_Rmeas", "", ""),
		MAXIMUM_ERROR_OF_SPINDLE_POSITION 		("MAXIMUM_ERROR_OF_SPINDLE_POSITION", "", ""),
		MAXIMUM_ERROR_OF_SPOT_POSITION 			("MAXIMUM_ERROR_OF_SPOT_POSITION", "", ""),
		MAXIMUM_NUMBER_OF_STRONG_PIXELS 		("MAXIMUM_NUMBER_OF_STRONG_PIXELS", "", ""),
		MIN_RFL_Rmeas 							("MIN_RFL_Rmeas", "", ""),
		MINIMUM_FRACTION_OF_BACKGROUND_REGION 	( "MINIMUM_FRACTION_OF_BACKGROUND_REGION", "", ""),
		MINIMUM_FRACTION_OF_INDEXED_SPOTS 		("MINIMUM_FRACTION_OF_INDEXED_SPOTS", "", ""),
		MINIMUM_ISIGMA 							("MINIMUM_I/SIGMA", "", ""),
		MINIMUM_NUMBER_OF_PIXELS_IN_A_SPOT 		("MINIMUM_NUMBER_OF_PIXELS_IN_A_SPOT", "3", "default of 6 is sometimes too high"),
		MINIMUM_VALID_PIXEL_VALUE 				("MINIMUM_VALID_PIXEL_VALUE", "0", ""),
		MINIMUM_ZETA 							("MINIMUM_ZETA", "", ""),
		MINPK 									("MINPK", "", ""),
		NBATCH 									("NBATCH", "", ""),
		NBX 									("NBX", "", ""),
		NBY 									("NBY", "", ""),
		NUMBER_OF_PROFILE_GRID_POINTS_ALONG_ALPHA_BETA ("NUMBER_OF_PROFILE_GRID_POINTS_ALONG_ALPHA/BETA", "13", ""),
		NUMBER_OF_PROFILE_GRID_POINTS_ALONG_GAMMA ("NUMBER_OF_PROFILE_GRID_POINTS_ALONG_GAMMA", "13", ""),
		OFFSET 									("OFFSET", "", ""),
		ORGX 									("ORGX ", "", ""),
		ORGY 									("ORGY", "", ""),
		PATCH_SHUTTER_PROBLEM 					("PATCH_SHUTTER_PROBLEM", "", ""),
		PROFILE_FITTING 						("PROFILE_FITTING", "", ""),
		REFERENCE_DATA_SET 						("REFERENCE_DATA_SET", "", ""),
		REFINE_CORRECT 							("REFINE(CORRECT)", "", ""),
		REFINE_IDXREF 							("REFINE(IDXREF)", "", ""),
		REFINE_INTEGRATE 						("REFINE(INTEGRATE)", "CELL BEAM ORIENTATION ! AXIS DISTANCE", ""),
		REFINE_SEGMENT 							("REFINE_SEGMENT", "", ""),
		REFLECTING_RANGE 						("REFLECTING_RANGE", "", ""),
		REFLECTING_RANGE_ESD 					("REFLECTING_RANGE_E.S.D.", "", ""),
		REFLECTIONS_CORRECTION_FACTOR 			("REFLECTIONS_CORRECTION_FACTOR", "", ""),
		REIDX 									("REIDX", "", ""),
		REJECT_ALIEN 							("REJECT_ALIEN", "", ""),
		RESOLUTION_SHELLS 						("RESOLUTION_SHELLS", "", ""),
		ROFF 									("ROFF", "", ""),
		ROTATION_AXIS 							("ROTATION_AXIS", "", ""),
		SEGMENT 								("SEGMENT", "", ""),
		SEGMENT_DISTANCE 						("SEGMENT_DISTANCE", "", ""),
		SEGMENT_ORGX 							("SEGMENT_ORGX", "", ""),
		SEGMENT_ORGY 							("SEGMENT_ORGY", "", ""),
		SEPMIN 									("SEPMIN", "3", "minimum separation between spots"),
		SIGNAL_PIXEL 							("SIGNAL_PIXEL", "", ""),
		SPOT_MAXIMUMCENTROID 					("SPOT_MAXIMUM-CENTROID", "", ""),
		SPOT_RANGE 								("SPOT_RANGE", "", ""),
		STARTING_ANGLES_OF_SPINDLE_ROTATION 	("STARTING_ANGLES_OF_SPINDLE_ROTATION", "", ""),
		STOE_CALIBRATION_PARAMETERS 			("STOE_CALIBRATION_PARAMETERS", "", ""),
		STRICT_ABSORPTION_CORRECTION 			("STRICT_ABSORPTION_CORRECTION","FALSE", "absorption correction factors in the CORRECT step of XDS. The parameter value can be TRUE or FALSE."),
		STRONG_PIXEL 							("STRONG_PIXEL", "4", "COLSPOT: only use strong reflections (default is 3)"),
		TEST 									("TEST", "1", " "),
		TEST_RESOLUTION_RANGE 					("TEST_RESOLUTION_RANGE", "", ""),
		TOFF 									("TOFF", "", ""),
		TOTAL_SPINDLE_ROTATION_RANGES 			("TOTAL_SPINDLE_ROTATION_RANGES", "", ""),
		UNIT_CELL_AAXIS 						("UNIT_CELL_A-AXIS", "", ""),
		UNIT_CELL_BAXIS 						("UNIT_CELL_B-AXIS", "", ""),
		UNIT_CELL_CAXIS 						("UNIT_CELL_C-AXIS", "", ""),
		UNTRUSTED_ELLIPSE 						("UNTRUSTED_ELLIPSE", " ", ""),
		UNTRUSTED_QUADRILATERAL 				("UNTRUSTED_QUADRILATERAL", " ", ""),
		UNTRUSTED_RECTANGLE 					("UNTRUSTED_RECTANGLE", " ", ""),
		WFAC1 									("WFAC1", "", ""),
		XGEO_CORR 								("X-GEO_CORR", "", ""),
		YGEO_CORR 								("Y-GEO_CORR", "", "");
		//BRASS_PLATE_IMAGE 						( "BRASS_PLATE_IMAGE", "", ""),
		//DARK_CURRENT_IMAGE 						( "DARK_CURRENT_IMAGE", "", ""),
		//DETECTOR 								( "DETECTOR", " ", ""),
		//FRACTION_OF_POLARIZATION 				("FRACTION_OF_POLARIZATION", "0.98", ""),
		//HOLE_DISTANCE 							( "HOLE_DISTANCE", "", ""),
		//INCIDENT_BEAM_DIRECTION 				( "INCIDENT_BEAM_DIRECTION", "0 0 1", ""),
		//JOB 									("JOB", "ALL !XYCORR INIT COLSPOT IDXREF DEFPIX INTEGRATE CORRECT", ""),
		//MAXIMUM_NUMBER_OF_JOBS 				("MAXIMUM_NUMBER_OF_JOBS", "6", ""),
		//MAXIMUM_NUMBER_OF_PROCESSORS 			("MAXIMUM_NUMBER_OF_PROCESSORS", "6", ""),
		//MNHOLE 									( "MNHOLE", "", ""),
		//MXHOLE 									( "MXHOLE", "", ""),
		//NAME_TEMPLATE_OF_DATA_FRAMES 			( "NAME_TEMPLATE_OF_DATA_FRAMES", "", ""),
		//NX 										( "NX", " ", ""),
		//NY 										( "NY", " ", ""),
		//OSCILLATION_RANGE 						( "OSCILLATION_RANGE", "", ""),
		//OVERLOAD 								( "OVERLOAD", "1048576", ""),
		//POLARIZATION_PLANE_NORMAL 				("POLARIZATION_PLANE_NORMAL", "0 1 0", ""),
		//QX 										( "QX", " ", ""),
		//QY 										( "QY", " ", ""),
		//SECONDS 								( "SECONDS", " ", ""),
		//SENSOR_THICKNESS 						( "SENSOR_THICKNESS", "1", ""),
		//SILICON 								( "SILICON", "", ""),
		//STARTING_ANGLE 							("STARTING_ANGLE", "", ""),
		//STARTING_FRAME 							("STARTING_FRAME", "", ""),
		//XRAY_WAVELENGTH 						( "X-RAY_WAVELENGTH", "", ""),

		
		String xdsparam;
		String val;
		String helptxt;

		String getXDSParam() {
			return xdsparam;
		}
		
		String getValue() {
			return val;
		}
		
		DefaultParam(String xdsparam, String val, String helptxt) {

			this.xdsparam = xdsparam;
			this.val = val;
			this.helptxt = helptxt;
		}
	}
	public static ModelXDSSettings inst = null;
	public static ModelXDSSettings get() {
		return inst;
	}
	public ModelXDSSettings() {
		inst = this;
		ll = new ListenerList<ModelXDSSettings>(this);
		for (DefaultParam param : DefaultParam.values()) {
			params.add(param);
			//paramValues.put(param, param.getValue());
		}
	}
	public  void resetXDSParameters() {
		for (DefaultParam param : DefaultParam.values()) {
			setXDSParamvalue(param, param.getValue());
		}
		DevXDSSettings.get().updateAllRows();
	}
	public void setXDSParamvalue(DefaultParam param, String val) {
		String value = (String)val;
		paramValues.put(param, value);
		changedValues.put(param, value);
		DevXDSSettings.get().setResult(param, true);
	}
}
