/**
 * @author spothineni
 * 
 * Analysis tab GUI
 * 
 * All the sub tabs are initted from this class
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.dataproc.ModelDataProc.DataProcField;
import gov.anl.gmca.dataproc.ModelDataProc.DataProcState;
import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets;
import gov.anl.gmca.dataproc.combinedatasets.WdgCombineDatasets;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.Resources;
import gov.anl.gmca.multistrategy.ModelReferenceData;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.io.File;
import java.util.HashMap;

public class WdgDataProc extends Composite implements
		ListenerList.Listener<ModelDataProcList>, ClientListener {
	
	private static final int NUMBER_OF_TABS = 6;
	public static final int TIMEWAIT_FOR_PROCESSINGJOB = 900; //secs
	
	protected static final ModelDataProc.DataProcField[] displayFields = {
			ModelDataProc.DataProcField.SAMPLE_NAME,
			ModelDataProc.DataProcField.WORKDIR,
			ModelDataProc.DataProcField.IMAGE_SET,
			ModelDataProc.DataProcField.IMAGES,
			ModelDataProc.DataProcField.WARNING,
			ModelDataProc.DataProcField.UNIT_CELL,
			ModelDataProc.DataProcField.SPACE_GROUP,
			ModelDataProc.DataProcField.LOWRESOLUTION,
			ModelDataProc.DataProcField.LOWRESOLUTION_INNER,
			ModelDataProc.DataProcField.LOWRESOLUTION_OUTER,
			ModelDataProc.DataProcField.HIGHRESOLUTION,
			ModelDataProc.DataProcField.HIGHRESOLUTION_INNER,
			ModelDataProc.DataProcField.HIGHRESOLUTION_OUTER,
			ModelDataProc.DataProcField.RMERGE,
			ModelDataProc.DataProcField.RMERGE_INNER,
			ModelDataProc.DataProcField.RMERGE_OUTER,
			ModelDataProc.DataProcField.RPIM,
			ModelDataProc.DataProcField.RPIM_INNER,
			ModelDataProc.DataProcField.RPIM_OUTER,
			ModelDataProc.DataProcField.CCHALF,
			ModelDataProc.DataProcField.CCHALF_INNER,
			ModelDataProc.DataProcField.CCHALF_OUTER,
			ModelDataProc.DataProcField.COMPLETE,
			ModelDataProc.DataProcField.COMPLETE_INNER,
			ModelDataProc.DataProcField.COMPLETE_OUTER,
			ModelDataProc.DataProcField.ANOM_COMPLETE,
			ModelDataProc.DataProcField.ANOM_COMPLETE_INNER,
			ModelDataProc.DataProcField.ANOM_COMPLETE_OUTER,
			ModelDataProc.DataProcField.MULTIPLICITY,
			ModelDataProc.DataProcField.MULTIPLICITY_INNER,
			ModelDataProc.DataProcField.MULTIPLICITY_OUTER,
			ModelDataProc.DataProcField.ANOM_MULTIPLICITY,
			ModelDataProc.DataProcField.ANOM_MULTIPLICITY_INNER,
			ModelDataProc.DataProcField.ANOM_MULTIPLICITY_OUTER,
			ModelDataProc.DataProcField.ISIGMA,
			ModelDataProc.DataProcField.ISIGMA_INNER,
			ModelDataProc.DataProcField.ISIGMA_OUTER };

	protected HashMap<DataProcField, Text> fieldTexts = new HashMap<DataProcField, Text>();
	protected Composite genComp, expandComp, graphComp;
	protected Composite controlComp, logComp;
	protected Composite[] procComp;
	protected Label positionLabel, stateLabel;
	protected Button truncateLogButton, scalaLogButton, xdsLogButton,
			exportButton;
	protected Button startButton, prevButton, nextButton, lastButton,
			multiButton;

	// sudhir
	protected TabFolder ctFolder;
	protected WdgDataProcTable stratTable = null;
	protected WdgReprocessing wdgReprocess = null;
	protected Combo softwareLabel;
	protected static final boolean UPDATE_CURRENT_INDEX_ON_RESULT_CREATION = true;
	protected static final boolean SHOW_DATAPROC_TAB_ON_RESULT_CREATION = true;
	protected boolean dontUpdate = false;
	Font buttonFont = GUI.FONT_12;
	Image imgGraph1, imgGraph2;
	CLabel lblGraph1, lblGraph2;
	Combo paramsGraph, paramsGraph1, valueGroups;
	Label processLabel;
	String imgBase = Main.getAppDir() + "/images/";
	String[] graphImages = new String[] { "rmergevsresolution.png",
			"rmeasvsresolution.png", "isigmavsresolution.png",
			"completevsresolution.png", "multivsresolution.png", };
	
	public WdgDataProc(Composite parent) {
		
		super(parent, SWT.BORDER);
		setBackground(GUI.BGCOLOR);
		setLayout(new GridLayout(1, false));
		
		procComp = new Composite[NUMBER_OF_TABS];
		
		genComp = new Composite(this, SWT.NONE);
		genComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		genComp.setLayout(gl);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		genComp.setLayoutData(gd);

		expandComp = new Composite(genComp, SWT.BORDER);
		expandComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(4, false);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		expandComp.setLayout(gl1);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		expandComp.setLayoutData(gd);

		createControls();

		graphComp = new Composite(genComp, SWT.NONE);
		graphComp.setBackground(GUI.BGCOLOR);
		GridLayout gl2 = new GridLayout(2, false);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		gl.makeColumnsEqualWidth = false;
		graphComp.setLayout(gl2);
		gd = new GridData(SWT.DEFAULT, SWT.FILL, false, true);
		gd.minimumWidth = 300;
		graphComp.setLayoutData(gd);

		createGraphs();
		createDataProcDisplay();
		createExportSolutionButtons();
	}

	public void connectListeners() {
		ModelDataProcList.get().ll.addListener(this);
		DevDataProc.get().verifyConnected();
		Client.addListener(this);

		updateLabels();

		stratTable.updateTable();
		ctFolder.setSelection(0);
	}

	protected void createExportSolutionButtons() {
		logComp = new Composite(expandComp, SWT.NONE);
		logComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(4, false);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		logComp.setLayout(gl);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		logComp.setLayoutData(gd);

		xdsLogButton = new Button(logComp, SWT.PUSH);
		xdsLogButton.setText("XDS Log");
		xdsLogButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		xdsLogButton.setLayoutData(gd);
		xdsLogButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				String filename = ModelDataProcList.get().getCurrentResult()
						.getField(DataProcField.LOG);

				// TODO: handle spaces
				File xdsFile = new File(filename);
				if (xdsFile.exists())
					ExecTask.new_ExecTask("nedit " + filename + "", "nedit",
							false, false).start();
				else
					Log.error(filename + " doesn't exists");
			}
		});

		scalaLogButton = new Button(logComp, SWT.PUSH);
		scalaLogButton.setText("AIMLESS Log");
		scalaLogButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		scalaLogButton.setLayoutData(gd);
		scalaLogButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				String filename = ModelDataProcList.get().getCurrentResult()
						.getField(DataProcField.SCALALOG);
				/* display it in qtrview will show the graphs */
				CmdLogview openFirefox = new CmdLogview(filename,"Logview AIMLESS Log");
				openFirefox.start();
			}
		});

		truncateLogButton = new Button(logComp, SWT.PUSH);
		truncateLogButton.setText("CTruncate Log");
		truncateLogButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		truncateLogButton.setLayoutData(gd);
		truncateLogButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				String filename = ModelDataProcList.get().getCurrentResult()
						.getField(DataProcField.TRUNCATELOG);
				/* display it in qtrview will show the graphs */
				CmdLogview openFirefox = new CmdLogview(filename, "Logview TRUNCATE Log");
				openFirefox.start();
			}
		});
	}

	protected void createDataProcDisplay() {
		
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true, 4, 1);

		ctFolder = new TabFolder(expandComp, SWT.BORDER);
		ctFolder.setFont(GUI.FONT_COLLECT_GROUP);
		ctFolder.setLayoutData(gd);
		
		/* Advanced tab for XDS parameters */
		procComp[0] = new Composite(ctFolder, SWT.BORDER);
		procComp[0].setBackground(GUI.BGCOLOR);
		procComp[0]
				.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		GridLayout  glOver= new GridLayout(1, false);
		glOver.verticalSpacing = 5;
		procComp[0].setLayout(glOver);
		
		new GridData(SWT.FILL, SWT.TOP, true, false);
		WdgXDSSettings wdgXDSSettings = new WdgXDSSettings(procComp[0], SWT.NONE);
		wdgXDSSettings.setLayoutData(gd);
		
		TabItem overrideTab1 = new TabItem(ctFolder, SWT.TOP);
		overrideTab1.setText("XDS Settings");
		overrideTab1.setControl(procComp[0]);
		
		/* Data Quality tab */
		procComp[1] = new Composite(ctFolder, SWT.BORDER);
		procComp[1].setBackground(GUI.COLOR_WHITE);
		
		GridLayout gl = new GridLayout(4, false);
		gl.verticalSpacing = 5;
		gl.horizontalSpacing = 15;
		procComp[1].setLayout(gl);
		
		procComp[1].setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 4,
				1));

		TabItem strategyTab = new TabItem(ctFolder, SWT.NONE);
		strategyTab.setText("Data Quality");
		strategyTab.setControl(procComp[1]);

		/* Reprocessing tab */
		procComp[2] = new Composite(ctFolder, SWT.BORDER);
		procComp[2].setBackground(GUI.BGCOLOR);
		procComp[2]
				.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		GridLayout glRe = new GridLayout(1, false);
		procComp[2].setLayout(glRe);

		new GridData(SWT.FILL, SWT.TOP, true, false);
		wdgReprocess = new WdgReprocessing(procComp[2], SWT.NONE);
		wdgReprocess.setLayoutData(gd);

		TabItem reprocessTab = new TabItem(ctFolder, SWT.NONE);
		reprocessTab.setText("Reprocessing");
		reprocessTab.setControl(procComp[2]);
		
		/* Combine Datasets tab : XSCALE */
		
		procComp[3] = new Composite(ctFolder, SWT.NONE);
		procComp[3].setBackground(GUI.BGCOLOR);
		procComp[3].setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		GridLayout glmerge = new GridLayout(1, false);
		procComp[3].setLayout(glmerge);

		new GridData(SWT.FILL, SWT.FILL, true, true);
		WdgCombineDatasets mergeD = new WdgCombineDatasets(procComp[3], SWT.NONE);
		mergeD.setLayoutData(gd);

		TabItem mergeTab = new TabItem(ctFolder, SWT.NONE);
		mergeTab.setText("MCS: Combine Datasets");
		mergeTab.setControl(procComp[3]);
		
		
		/* All Results tab */
		procComp[4] = new Composite(ctFolder, SWT.BORDER);
		procComp[4].setBackground(GUI.COLOR_WHITE);
		procComp[4].setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		GridLayout glall = new GridLayout(1, false);
		procComp[4].setLayout(glall);

		new GridData(SWT.FILL, SWT.FILL, true, true);
		stratTable = new WdgDataProcTable(procComp[4], SWT.NONE);
		stratTable.setLayoutData(gd);
		TabItem allTab = new TabItem(ctFolder, SWT.NONE);
		allTab.setText("All Results");
		allTab.setControl(procComp[4]);
		
		/* Help tab */
		procComp[5] = new Composite(ctFolder, SWT.BORDER);
		procComp[5].setBackground(GUI.COLOR_WHITE);
		procComp[5].setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		GridLayout glHelp = new GridLayout(1, false);
		procComp[5].setLayout(glHelp);
		
		new GridData(SWT.FILL, SWT.FILL, true, true);
		WdgProcHelp procHelp = new WdgProcHelp(procComp[5], SWT.NONE);
		procHelp.setLayoutData(gd);
		TabItem helpTab = new TabItem(ctFolder, SWT.NONE);
		helpTab.setText("Help");
		helpTab.setControl(procComp[5]);
		
		stateLabel = new Label(procComp[1], SWT.BORDER | SWT.CENTER);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 4, 1);
		stateLabel.setLayoutData(gd);
		stateLabel.setText("... Loading ...");

		Font font = GUI.FONT_11;

		for (ModelDataProc.DataProcField field : displayFields) {
			if (field.getDisplayLines() > 0) {
				gd = new GridData(SWT.FILL, SWT.CENTER, true, true);
				Label nameLabel = new Label(procComp[1], SWT.NONE);
				nameLabel.setBackground(GUI.COLOR_WHITE);
				nameLabel.setFont(font);
				nameLabel.setText(null == field ? "" : field.getUserString()
						+ ":");
				nameLabel.setLayoutData(gd);
			}
			if (field == ModelDataProc.DataProcField.IMAGE_SET) {
				gd = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
				gd.horizontalSpan = 3;
				gd.widthHint = 250;
				valueGroups = new Combo(procComp[1], SWT.READ_ONLY);
				valueGroups.setLayoutData(gd);

				valueGroups.addSelectionListener(new SelectionListener() {
					@Override
					public void widgetDefaultSelected(SelectionEvent se) {
					}

					@Override
					public void widgetSelected(SelectionEvent se) {
						
						DevDataProc
								.get()
								.updateRow(
										Integer.parseInt(ModelDataProcList
												.get()
												.getCurrentResult()
												.getField(
														ModelDataProc.DataProcField.SAMPLE_NUMBER)),
										valueGroups.getText(), true);
					}
				});
			} else {
				Text valueText = (field.getDisplayLines() > 2) ? new Text(
						procComp[1], SWT.WRAP | SWT.V_SCROLL | SWT.MULTI
								| SWT.NONE) : new Text(procComp[1], SWT.WRAP
						| SWT.MULTI);

				valueText.setFont(font);
				gd = new GridData(SWT.FILL, SWT.CENTER, true, true);
				if (field.getDisplayLines() > 1)
					gd.horizontalSpan = 3;
				if (field.getDisplayLines() > 2) {
					GC gc = new GC(procComp[1]);
					gc.setFont(font);
					int textHeight = gc.textExtent("1234567890").y;
					gd.heightHint = textHeight * field.getDisplayLines();
				}

				valueText.setEditable(false);
				valueText.setLayoutData(gd);

				if (null != field)
					fieldTexts.put(field, valueText);
				if (field == ModelDataProc.DataProcField.WARNING)
					valueText.setForeground(GUI.COLOR_RED);
				valueText.setBackground(GUI.COLOR_WHITE);

				if (field == ModelDataProc.DataProcField.SPACE_GROUP) {
					gd = new GridData();
					gd.widthHint = 100;
					Label blankText = new Label(procComp[1], SWT.NONE);
					blankText.setBackground(GUI.COLOR_WHITE);
					blankText.setLayoutData(gd);

					gd = new GridData();
					gd.widthHint = 100;
					Label overallText = new Label(procComp[1], SWT.RADIO);
					overallText.setText("Overall");
					overallText.setFont(GUI.FONT_11_BOLD);
					overallText.setBackground(GUI.COLOR_WHITE);
					overallText.setLayoutData(gd);

					gd = new GridData();
					gd.widthHint = 100;
					Label innerText = new Label(procComp[1], SWT.RADIO);
					innerText.setText("InnerShell");
					innerText.setFont(GUI.FONT_11_BOLD);
					innerText.setBackground(GUI.COLOR_WHITE);
					innerText.setLayoutData(gd);

					gd = new GridData();
					gd.widthHint = 100;
					Label outerText = new Label(procComp[1], SWT.RADIO);
					outerText.setText("OuterShell");
					outerText.setFont(GUI.FONT_11_BOLD);
					outerText.setBackground(GUI.COLOR_WHITE);
					outerText.setLayoutData(gd);
				}
			}
		}
		
		ctFolder.setSelection(1);
		ctFolder.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				int tab = ctFolder.getSelectionIndex();
				setButtonsVisible(ctFolder.getSelectionIndex() == 1);
				genComp.layout();
				procComp[tab].layout();
				
			}

			private void setButtonsVisible(boolean b) {
				WdgControl.setControlVisible(graphComp, b);
				WdgControl.setControlVisible(logComp, b);
				WdgControl.setControlVisible(controlComp, b);
				expandComp.layout();	
			}
			
		});
	}

	protected void createControls() {
		
		controlComp = new Composite(expandComp, SWT.NONE);
		controlComp.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
		controlComp.setLayoutData(gd);
		GridLayout gl = new GridLayout(6, false);
		gl.horizontalSpacing = 0;
		controlComp.setLayout(gl);

		int clearButtonWidth = 80;
		int moveButtonWidth = 70;

		startButton = new Button(controlComp, SWT.PUSH);
		startButton.setText("|<<");
		startButton.setFont(buttonFont);
		startButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = moveButtonWidth;
		startButton.setLayoutData(gd);
		startButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				ModelDataProc currentResult = ModelDataProcList.get()
						.getResultIndex(0);
				if (currentResult != null) {
					dontUpdate = true;
					DevDataProc.setCurrentDataProcIndex(0);
				}
			}
		});

		prevButton = new Button(controlComp, SWT.PUSH);
		prevButton.setText("<");
		prevButton.setFont(buttonFont);
		prevButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = moveButtonWidth;
		prevButton.setLayoutData(gd);
		prevButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {

				int prev = ModelDataProcList.get().getCurrentDataProcIndex() - 1;
				ModelDataProc currentResult = ModelDataProcList.get()
						.getResultIndex(prev);
				if (currentResult != null) {
					dontUpdate = true;
					DevDataProc.setCurrentDataProcIndex(prev);
				}
			}
		});

		nextButton = new Button(controlComp, SWT.PUSH);
		nextButton.setText(">");
		nextButton.setFont(buttonFont);
		nextButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = moveButtonWidth;
		nextButton.setLayoutData(gd);
		nextButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {

				int next = ModelDataProcList.get().getCurrentDataProcIndex() + 1;
				ModelDataProc currentResult = ModelDataProcList.get()
						.getResultIndex(next);
				if (currentResult != null) {
					dontUpdate = true;
					DevDataProc.setCurrentDataProcIndex(next);
				}
			}
		});

		lastButton = new Button(controlComp, SWT.PUSH);
		lastButton.setText(">>|");
		lastButton.setFont(buttonFont);
		lastButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = moveButtonWidth;
		lastButton.setLayoutData(gd);
		lastButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {

				int last = ModelDataProcList.get().getDataProcList().size() - 1;
				ModelDataProc currentResult = ModelDataProcList.get()
						.getResultIndex(last);
				if (currentResult != null) {
					dontUpdate = true;
					DevDataProc.setCurrentDataProcIndex(last);
				}
			}
		});

		positionLabel = new Label(controlComp, SWT.CENTER);
		positionLabel.setText("");
		positionLabel.setBackground(GUI.BGCOLOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.minimumWidth = 100;
		positionLabel.setLayoutData(gd);

		exportButton = new Button(controlComp, SWT.PUSH);
		exportButton.setText("Export All");
		exportButton.setToolTipText("save all processing results to excel");
		exportButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = clearButtonWidth;
		exportButton.setLayoutData(gd);
		exportButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				exportToSpreadsheet();
			}
		});
	}

	protected void createGraphs() {
		
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalAlignment = SWT.RIGHT;
		String[] params = new String[] { "Rmerge", "Rmeasured", "I/Sigma",
				"Completeness", "Multiplicity" };
		paramsGraph = new Combo(graphComp, SWT.RIGHT | SWT.READ_ONLY);
		paramsGraph.setItems(params);
		paramsGraph.setLayoutData(gd);
		paramsGraph.setText("Completeness");
		paramsGraph.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				loadGraphImage(imgGraph1, lblGraph1,
						graphImages[paramsGraph.getSelectionIndex()], 1);
			}
		});
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.verticalAlignment = SWT.CENTER;
		Label resLabel = new Label(graphComp, SWT.LEFT);
		resLabel.setBackground(GUI.BGCOLOR);
		resLabel.setText("Vs Resolution");
		resLabel.setLayoutData(gd);

		gd = new GridData();
		gd.horizontalSpan = 2;
		gd.horizontalAlignment = SWT.CENTER;

		imgGraph1 = new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/noimage.png"));
		lblGraph1 = new CLabel(graphComp, SWT.LEFT);
		lblGraph1.setBackground(GUI.BGCOLOR);
		lblGraph1.setImage(imgGraph1);
		lblGraph1.setSize(320, 240);
		lblGraph1.setLayoutData(gd);
		lblGraph1
				.setToolTipText("Double click on the graph to open it in ccp4 loggraph");
		lblGraph1.addMouseListener(new MouseListener() {

			@Override
			public void mouseDoubleClick(MouseEvent arg0) {
				showLogGraph();
			}

			@Override
			public void mouseDown(MouseEvent arg0) {
			}

			@Override
			public void mouseUp(MouseEvent arg0) {
			}

		});

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 2;
		gd.verticalAlignment = SWT.CENTER;
		Label helpLabel = new Label(graphComp, SWT.LEFT);
		helpLabel.setBackground(GUI.BGCOLOR);
		helpLabel
				.setText("(double click on the graph to analyse data with\n CCP4 loggraph)");
		helpLabel.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalAlignment = SWT.RIGHT;
		paramsGraph1 = new Combo(graphComp, SWT.RIGHT | SWT.READ_ONLY);
		paramsGraph1.setItems(params);
		paramsGraph1.setLayoutData(gd);
		paramsGraph1.setText("I/Sigma");
		paramsGraph1.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				loadGraphImage(imgGraph2, lblGraph2,
						graphImages[paramsGraph1.getSelectionIndex()], 2);
			}
		});

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.verticalAlignment = SWT.CENTER;
		Label resLabel1 = new Label(graphComp, SWT.LEFT);
		resLabel1.setBackground(GUI.BGCOLOR);
		resLabel1.setText("Vs Resolution");
		resLabel1.setLayoutData(gd);

		gd = new GridData();
		gd.horizontalSpan = 2;
		gd.horizontalAlignment = SWT.CENTER;
		imgGraph2 = new Image(GUI.getDisplay(),  Resources.get().getInputStream("/images/noimage.png"));
		lblGraph2 = new CLabel(graphComp, SWT.LEFT);
		lblGraph2.setBackground(GUI.BGCOLOR);
		lblGraph2.setImage(imgGraph1);
		lblGraph2.setSize(320, 240);
		lblGraph2.setLayoutData(gd);
		lblGraph2
				.setToolTipText("Double click on the graph to open it in ccp4 loggraph");
		lblGraph2.addMouseListener(new MouseListener() {

			@Override
			public void mouseDoubleClick(MouseEvent arg0) {
				showLogGraph();
			}

			@Override
			public void mouseDown(MouseEvent arg0) {
			}

			@Override
			public void mouseUp(MouseEvent arg0) {
			}

		});
		multiButton = new Button(graphComp, SWT.PUSH);
		multiButton.setText("Export XDS_ASCII.HKL\n to Multistrategy");
		multiButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.horizontalSpan = 2;
		gd.horizontalAlignment = SWT.CENTER;
		multiButton.setLayoutData(gd);
		multiButton
				.setToolTipText("Export XDS_ASCII.HKL file to Multi strategy sub-tab of the Collect tab");
		multiButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {

				String workDir = ModelDataProcList.get().getCurrentResult()
						.getField(DataProcField.SUBDIR);
				String resolution = ModelDataProcList.get().getCurrentResult()
						.getField(DataProcField.HIGHRESOLUTION);
				
				String filename = workDir + "/XDS_ASCII.HKL";
				File hkl = new File(filename);
				if (!hkl.exists()) {
					Log.error("HKL file doesn't find in " + workDir);
					return;
				} else {
					Log.note(filename+"\n exported to Multi Crystal Strategy sub-tab of Collect tab and Combine Datasets sub-tab");
				}

				ModelReferenceData.loadHKLFile(filename, false);
				ModelCombineDatasets.parseHKLFile(filename, resolution);
			}
		});

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 2;
		processLabel = new Label(graphComp, SWT.NONE);
		processLabel.setBackground(GUI.BGCOLOR);
		processLabel.setFont(GUI.FONT_12);
		processLabel.setText("Software pipeline:");
		processLabel.setLayoutData(gd);

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.horizontalSpan = 2;
		gd.minimumWidth = 150;
		softwareLabel = new Combo(graphComp, SWT.NONE);
		softwareLabel.setFont(GUI.FONT_11);
		softwareLabel.setLayoutData(gd);
		softwareLabel.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				
				ModelDataProcList.currentSoftwareSelection = softwareLabel.getText();
				
				DevDataProc
						.get()
						.updateRowSoftware(
								Integer.parseInt(ModelDataProcList
										.get()
										.getCurrentResult()
										.getField(
												ModelDataProc.DataProcField.SAMPLE_NUMBER)),
												true, softwareLabel.getText(),
												ModelDataProcList
												.get()
												.getCurrentResult()
												.getField(
														ModelDataProc.DataProcField.IMAGE_SET));
			}
		});
		
	}

	protected void setStateLabel(DataProcState ss) {
		if (null == ss) {
			stateLabel.setBackground(GUI.COLOR_LIGHT_GRAY);
			stateLabel.setForeground(GUI.COLOR_BLACK);
			stateLabel.setText("---");
			return;
		}

		switch (ss) {
		case INACTIVE:
			stateLabel.setBackground(GUI.COLOR_LIGHT_GRAY);
			stateLabel.setForeground(GUI.COLOR_BLACK);
			stateLabel.setText("Inactive");
			break;

		case ERROR:
			stateLabel.setBackground(GUI.COLOR_RED_BUTTON);
			stateLabel.setText("Error");
			break;

		case PROCESSING:
			stateLabel.setBackground(GUI.COLOR_DARK_CYAN);
			stateLabel.setForeground(GUI.COLOR_WHITE);
			stateLabel.setText("Processing");
			break;

		case DONE:
			stateLabel.setBackground(GUI.COLOR_DARK_GREEN);
			stateLabel.setForeground(GUI.COLOR_WHITE);
			stateLabel.setText("Done");

			break;
		}
	}

	@Override
	public void handleEvent(ModelDataProcList resultList, Object data) {
		updateLabels();
		stratTable.updateTable();
		// ctFolder.setSelection(0);
	}

	protected void updateLabels() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				ModelDataProcList resultList = ModelDataProcList.get();

				// position in list
				int listSize = resultList.getDataProcList().size();

				if (listSize >= 1) {
					positionLabel.setText((resultList.getCurrentDataProcIndex() + 1)
							+ " of " + listSize);
				} else {
					positionLabel.setText("---");
					valueGroups.removeAll();
					softwareLabel.removeAll();
				}

				// fields
				ModelDataProc currentResult = resultList.getCurrentResult();
				if (null != currentResult) {
					setStateLabel(currentResult.getState());

					for (DataProcField field : currentResult.getAllFields()
							.keySet()) {
						if (field == ModelDataProc.DataProcField.IMAGE_SET) {

							valueGroups.removeAll();
							for (Object s : currentResult.imageSets.toArray()) {
								if (valueGroups.indexOf((String) s) < 0) {
									valueGroups.add((String) s);
								}
							}
							if (dontUpdate) {
								dontUpdate = false;
							}
							valueGroups.setText(currentResult
									.getField(ModelDataProc.DataProcField.IMAGE_SET));
						}
						if (field == ModelDataProc.DataProcField.SOFTWARE) {
							softwareLabel.removeAll();
							for(Object s : currentResult.pipelines.toArray()) {
								if(softwareLabel.indexOf((String) s) < 0) {
									softwareLabel.add((String) s);
								}
								softwareLabel.setText(currentResult
										.getField(ModelDataProc.DataProcField.SOFTWARE));
							}							
						}

						Text text = fieldTexts.get(field);

						if (null != text) {
							String value = currentResult.getField(field);

							if (null == value) {
								text.setText("---");
							} else {
								if (!value.equals(""))
									text.setText(value);
								else
									text.setText("---");
							}
						}
					}
					// set graph image
					loadGraphImage(imgGraph1, lblGraph1,
							"completevsresolution.png", 1);
					loadGraphImage(imgGraph2, lblGraph2,
							"isigmavsresolution.png", 2);
					paramsGraph.select(3);
					paramsGraph1.select(2);
					
					WdgControl.setControlVisible(graphComp, true);
					WdgControl.setControlVisible(logComp, true);
					WdgControl.setControlVisible(controlComp, true);
					expandComp.layout();
					genComp.layout();
					
					String softwareUsed = ModelDataProcList.get()
							.getCurrentResult()
							.getField(DataProcField.SOFTWARE);
					String scalingSoftware = ModelDataProcList.get()
							.getCurrentResult()
							.getField(DataProcField.SCALINGSOFTWARE);
					
					String sampleName = ModelDataProcList.get()
							.getCurrentResult()
							.getField(DataProcField.SAMPLE_NAME);
					xdsLogButton.setText(softwareUsed + " Log");
					
					scalaLogButton.setText(scalingSoftware + " Log");
					multiButton.setEnabled(true);
					
					if(sampleName.equals("mcs")) {
						xdsLogButton.setText("XSCALE Log");
						truncateLogButton.setVisible(false);
						processLabel.setVisible(false);
						softwareLabel.setVisible(false);
					} else {
						truncateLogButton.setVisible(true);
						softwareLabel.setVisible(true);
						processLabel.setVisible(true);
					}
					
				} else {
					setStateLabel(null);
					//scalaLogButton.setEnabled(false);
					//truncateLogButton.setEnabled(false);
					//xdsLogButton.setEnabled(false);

					for (Text text : fieldTexts.values()) {
						if (null != text)
							text.setText("---");
					}
				}

				updateEnabled();
				ctFolder.setSelection(1);
				procComp[1].layout(true);
			}
		});
	}

	protected void loadGraphImage(Image imgGraph, CLabel lblGraph, String image, int index) {

		if (imgGraph != null)
			imgGraph.dispose();

		String filename = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.SUBDIR);
		
		Image tmpImg = null;
		if (filename == null || filename.equals("") || !filename.contains("/")) {
			tmpImg = new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/noimage.png"));
			lblGraph.setImage(tmpImg);
		} else {

			File graphFile = new File(filename + "/" + image);
			try {
				if (graphFile.exists() && graphFile.canRead()) {
					tmpImg = new Image(GUI.getDisplay(), filename + "/" + image);
					lblGraph.setImage(tmpImg);
				} else {
					tmpImg = new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/noimage.png"));
					lblGraph.setImage(tmpImg);
				}
			} catch (Exception e) {
				System.out.println("ERROR : " + image + "is not valid image format/ zero size file");
			}
		}
		
		if (index == 1) {
			imgGraph1 = tmpImg;
		} else if (index == 2) {
			imgGraph2 = tmpImg;
		}
	}

	protected void updateEnabled() {
		ModelDataProc currentResult = ModelDataProcList.get()
				.getCurrentResult();
		final DataProcState state = (null == currentResult) ? null
				: currentResult.getState();

		GUI.asyncExec(new Runnable() {
			public void run() {
				boolean enableButtons = (null != state && state == DataProcState.DONE);

				//scalaLogButton.setEnabled(enableButtons);
				//paramsGraph.setEnabled(enableButtons);
				//paramsGraph1.setEnabled(enableButtons);
				//truncateLogButton.setEnabled(enableButtons);
				//multiButton.setEnabled(enableButtons);

				boolean zeroResults = ModelDataProcList.get().getDataProcList()
						.size() == 0;
				boolean atStart = ModelDataProcList.get()
						.getCurrentDataProcIndex() <= 0;
				boolean atEnd = ModelDataProcList.get()
						.getCurrentDataProcIndex() >= ModelDataProcList.get()
						.getDataProcList().size() - 1;

				//xdsLogButton.setEnabled(Client.isMaster() && !zeroResults);
				//scalaLogButton.setEnabled(Client.isMaster() && !zeroResults);
				//truncateLogButton.setEnabled(Client.isMaster() && !zeroResults);

				//exportButton.setEnabled(Client.isMaster() && !zeroResults);
				//startButton.setEnabled(Client.isMaster() && !atStart && !zeroResults);
				//prevButton.setEnabled(Client.isMaster() && !atStart && !zeroResults);
				//nextButton.setEnabled(Client.isMaster() && !atEnd && !zeroResults);
				//lastButton.setEnabled(Client.isMaster() && !atEnd && !zeroResults);
			}
		});
	}

	protected void exportToSpreadsheet() {
		FileDialog dialog = new FileDialog(GUI.getShell(), SWT.SAVE);
		dialog.setFilterNames(new String[] { "Excel Files", "All Files (*.*)" });
		dialog.setFilterExtensions(new String[] { "*.xls" });

		String savePath = "";
		// savePath = System.getProperty("user.home");
		savePath = DevDataDir.getPrimary();
		dialog.setFilterPath(savePath);
		dialog.setFileName("DataProcessingResults.xls");

		ModelDataProcList.get().save(dialog.open());
	}

	private void showLogGraph() {
		
		String filename = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.SCALALOG);
		
		String prefix = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.SAMPLE_NAME);
		
		if (filename == null || filename.equals("") || !filename.contains("/")) {
			Log.error("Path to aimless log not found");
			return;
		}
		
		String currDir = filename.substring(0, filename.lastIndexOf("/"));
		currDir = currDir.replace(CmdDataProc.tempLogDir, "");
		
		filename = currDir + "/"+prefix+"_"+"aimless.log";
		
		// TODO: handle spaces
		File xdsFile = new File(filename);
		if (xdsFile.exists())
			ExecTask.new_ExecTask("loggraph " + filename + "", "loggraph",
					false, false).start();
		else
			Log.error(filename + " doesn't exists");
	}

	@Override
	public void masterChanged(boolean isMaster) {
		updateEnabled();
	}
}
