/**
 * @author spothineni
 * 
 * AllResults sub-tab of Analysis tab GUI
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.dataproc.ModelDataProc.DataProcField;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import java.util.ArrayList;

public class WdgDataProcTable extends Composite implements
		ListenerList.Listener<ModelDataProcList>, PaintListener {
	protected ArrayList<TableColumn> columnList = new ArrayList<TableColumn>();
	protected Table table = null;
	private Composite logComp;
	
	public WdgDataProcTable(Composite parent, int style) {
		super(parent, style);

		GridLayout gl = new GridLayout(1, true);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		gl.horizontalSpacing = 0;
		gl.verticalSpacing = 3;
		setLayout(gl);
		
		logComp = new Composite(this, SWT.NONE);
		logComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(1, true);
		gl1.marginWidth = 0;
		gl1.marginHeight = 0;
		logComp.setLayout(gl1);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		logComp.setLayoutData(gd);
		
		createTable();
		updateTable();
	}

	protected void createTable() {
		
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
		gd.widthHint = 630;
		table = new Table(logComp, SWT.NONE);
		table.setFont(GUI.FONT_DEFAULT);
		table.setLinesVisible(true);
		table.setLayoutData(gd);

		columnList.clear();
		for (DataProcField field : DataProcField.values()) {
			if(!field.getExportable())
				continue;
			if (field == DataProcField.SAMPLE_NUMBER)
				continue;
			TableColumn tc = new TableColumn(table, SWT.CENTER);
			tc.setText(field.dbString);
			tc.setWidth(130);
			columnList.add(tc);
		}
		table.setHeaderVisible(true);
		table.addMouseListener(new MouseListener() {
			public void mouseDoubleClick(MouseEvent arg0) {
				if (!Client.isMaster())
					return;
				int index = table.getSelectionIndex();
				DevDataProc.setCurrentDataProcIndex(index);
			}

			@Override
			public void mouseDown(MouseEvent e) {
			}

			@Override
			public void mouseUp(MouseEvent e) {
			}
		});
		addPaintListener(this);
	}

	public void updateTable() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				ModelDataProcList resultList = ModelDataProcList.get();

				table.removeAll();
				for (ModelDataProc cell : new ArrayList<ModelDataProc>(
						resultList.getDataProcList())) {

					TableItem ti = new TableItem(table, SWT.NONE);
					
					for (DataProcField field : cell.getAllFields().keySet()) {
						
						if (field == DataProcField.SAMPLE_NUMBER)
							continue;
						if (field == DataProcField.WARNING) {
							String value = cell.getField(field);
							if (value.length() > 20) {
								value = value.substring(0, 20);
								value = value.concat("...");
							}
							if (value != null)
								ti.setText(field.ordinal() - 1, value);
							else
								ti.setText(field.ordinal() - 1, "");
							continue;
						}
						String value = cell.getField(field);
						
						/* match with column name */
						int i=0;
						for(;i<table.getColumnCount();i++) {
							if(field.dbString.equals(table.getColumn(i).getText())) {
								break;
							}
						}
						
						if (value != null) {
							ti.setText(i, value);
						}
						else {						
							ti.setText(i, "");
						}
					}
				}
			}
		});
	}

	public void connectListeners() {
		ModelDataProcList.get().ll.addListener(this);
		updateTable();
	}

	@Override
	public void handleEvent(ModelDataProcList object, Object data) {
		// TODO Auto-generated method stub
		updateTable();
	}

	@Override
	public void paintControl(PaintEvent e) {
		// TODO Auto-generated method stub

	}
}
