/**
 * This scripts generates blend_input.dat file with all 
 * the XDS_ASCII.HKL files exported to CombineDatasets tab
 * and runs BLEND  in analysis mode (blend -a blend_input.dat)
 * 
 * More information on BLEND
 * http://www.ccp4.ac.uk/html/blend.html
 * 
 * Output Clusters.txt contains for which datasets to combine
 */

package gov.anl.gmca.dataproc.combinedatasets;

import gov.anl.gmca.common.Command;
import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets.HKLDataset;
import gov.anl.gmca.gridengine.CheckSubmitHost;
import gov.anl.gmca.gridengine.DRMAA;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.multistrategy.ModelReferenceData;
import gov.anl.gmca.syslog.Log;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;

public class CmdRunBLEND extends Command {
	
	public String workDir;
	private DevCombineDatasets devCombineDatasets = new DevCombineDatasets();
	@Override
	public void run() {
	
		BufferedWriter blendInput;
		workDir = workDir+"/BLEND";
		File dirFile = new File(workDir);
		if (!dirFile.exists()) {

			if (!dirFile.mkdirs()) {
				Log.error("Couldn't create directory " + workDir);
				return;
			}
		}
		ModelCombineDatasets.RESULTFILE = "";
		ModelCombineDatasets.WARNING = "Running BLEND....";

		devCombineDatasets.setResult(true);
		
		String blendInputfile = workDir  + "/blend_input.dat";
		try {
			blendInput = new BufferedWriter(new FileWriter(
					blendInputfile, false));
			
			for (int i = 0; i < ModelCombineDatasets.listHKLSets.size(); i++) {

				HashMap<HKLDataset, String> field = ModelCombineDatasets.get().listHKLSets
						.get(i);
				
				blendInput.write(field.get(HKLDataset.HKLFILE)+"\n");
			}
			
			blendInput.close();
		} catch (IOException e) {
			System.out.println("Error in writing BLEND input file:\n"+e.getMessage());
			return;
		}
		
		BufferedWriter blendScript;
		String blendScriptFile = workDir+"/run_blend.csh";
		try {
			blendScript = new BufferedWriter(new FileWriter(
					blendScriptFile, false));
			blendScript.write("#!/bin/csh\n");
			blendScript.write("# running BLEND in analysis mode from JBluIce interface\n");
			blendScript.write("# Alternatively you can run blend with input file "+ blendInputfile+ " in ccp4i interface\n");
			
			blendScript.write("set blend_path = `which blend`\n");
			blendScript.write("cd "+workDir+"\n");
			blendScript.write("$blend_path -a "+blendInputfile+"<< EOF >blend.log\nEND\nEOF\n");
			blendScript.close();
		}catch (IOException e) {
			System.out.println("Error in writing BLEND script file:\n"+e.getMessage());
			return;
		}
		
		/*check for gridengine environment and quit if not a submit host */
		boolean isSubmitHost = CheckSubmitHost.get().isSubmitHost();
		
		System.out.println("Gridengine env detected: "+isSubmitHost);
		if(!isSubmitHost) {
			Log.note("Grid Engine error: not a submit host");
			exitWithError();
			return;
		}
		
		try {
			Log.note("BLEND processing is in progress, check the dir. "
					+ workDir);
			
			Runtime.getRuntime().exec(
					"chmod u+x " + workDir + "/" + "run_blend.csh");
			
			DRMAA drmaa = null;
			boolean skip = true;
			try {
				drmaa = new DRMAA(workDir + "/" + "run_blend.csh", workDir,500);
			} catch (UnsatisfiedLinkError e) {
				e.printStackTrace();
				Log.warn("Grid Engine not running, script running locally");
				executeLocally();
				skip = false;
			}
			catch (Exception e) {
				
				exitWithError();
				e.printStackTrace();
				return;
			} // wait 300 sec before quitting job
			if(skip) {
				drmaa.start();
				int jobStatus = drmaa.waitFor();
			}
		} catch (Exception e1) {
			exitWithError();
			e1.printStackTrace();
		}	
		
		ModelCombineDatasets.RESULTFILE = this.workDir + "/CLUSTERS.txt";

		if (ModelCombineDatasets.WARNING.equals("Running BLEND..."))
			ModelCombineDatasets.WARNING = "DONE";
		if (ModelCombineDatasets.STATE.equals("PROCESSING"))
			ModelCombineDatasets.STATE = "DONE";
		
		devCombineDatasets.setResult(true);
		Log.note("BLEND processing finished");
	}
	private void executeLocally() {

		ExecTask task = ExecTask.new_ExecTask("csh " + workDir + "/"
				+ "run_blend.csh", "BLEND :",
				true, true);
		task.start();
		task.waitFor();
		Log.note("Processing with BLEND finished");
	}
	private void exitWithError() {
		ModelCombineDatasets.WARNING = "GridEngine/DRMAA exception";
		ModelCombineDatasets.STATE = "ERROR";
		/* load to ReferenceData as next crystal */
		ModelReferenceData.loadHKLFile(ModelCombineDatasets.RESULTFILE, true);
	}
}
