/**
 * Run MySQL queries for MultiStrategyCombineDatasets, HKLDatasetsList tables
 */

package gov.anl.gmca.dataproc.combinedatasets;

import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets.HKLDataset;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_STRATEGY;
import gov.anl.gmca.syslog.Log;
import gov.aps.jca.dbr.DBR;

import java.util.HashMap;

public class DevCombineDatasets {
	protected static final String tableName = "MultiStrategyCombineDatasets";
	protected static final String dataTableName = "HKLDatasetsList";

	protected static boolean doesTableExist() {
		Database.Result result = Database.exec(DBName.USERDATA,
				"show tables like '" + tableName + "'");
		Database.Result result1 = Database.exec(DBName.USERDATA,
				"show tables like '" + dataTableName + "'");
		boolean exists = (result.numRows() > 0 && result1.numRows() > 0);
		return exists;
	}
	
	/*UNIT_CELL 		("Unitcell", 100),
	COMPLETENESS	("Completeness", 50),
	RMEGRGE			("Rmege", 50),
	RPIM			("RPIM", 50),
	CCHALF			("CChalf", 50),
	ISIGMA			("ISigma", 50),
	MUMITIPLICITY	("Multiplicity", 50),
	ANOMCOMPLETENESS("AnomCompleteness", 50),
	ANOMMULTI		("AnomMulti", 50);*/
	
	public synchronized static boolean verifyTableExists() {
		if (!doesTableExist()) {
			System.out.println("Creating MultiStrategy results table");
			String createCmd = "CREATE TABLE `" + tableName + "` ("
					+ "`state` varchar(30) default '',"
					+ "`merge_result` varchar(5000) default '',"
					+ "`anomalous` boolean default false,"
					+ "`lowres` varchar(50) default '',"
					+ "`highres` varchar(50) default '',"
					+ "`warning` varchar(5000) default '',"
					+ "`dir` varchar(500) default '')";

			Log.note(createCmd);
			Database.exec(DBName.USERDATA, createCmd);

			System.out.println("Creating HKLDatasets table");
			createCmd = "CREATE TABLE `" + dataTableName + "` ("
					+ "`Selection` INT(10) default 1,"
					+ "`HKLFile` varchar(256) default '',"
					+ "`Resolution` varchar(16) default '',"
					+ "`Spacegroup` varchar(64) default '',"
					+ "`Unitcell` varchar(256) default '',"
					+ "`Completeness` varchar(16) default '',"
					+ "`Rmege` varchar(16) default '',"
					+ "`Rpim` varchar(16) default '',"
					+ "`CChalf` varchar(16) default '',"
					+ "`ISigma` varchar(16) default '',"
					+ "`Multiplicity` varchar(16) default '',"
					+ "`AnomCompleteness` varchar(16) default '',"
					+ "`AnomMulti` varchar(16) default '')";

			Log.note(createCmd);
			Database.exec(DBName.USERDATA, createCmd);
		}

		return doesTableExist();
	}

	protected boolean connected = false;
	protected Epics.EpicsValueMonitor dbUpdatedMonitor;

	public static DevCombineDatasets get() {
		return inst;
	}

	protected static DevCombineDatasets inst = null;

	public DevCombineDatasets() {
		inst = this;
	}

	public void verifyConnected() {
		if (connected)
			return;

		verifyTableExists();
		if (Main.SIM_LEVEL < 1)
			updateAllRows();
		Epics.addMonitor(PV_STRATEGY.mergeDatasetsUpdated,
				new Epics.EpicsValueMonitor() {
					@Override
					public void valueChanged(String pvName, DBR value) {
						int isR = Epics.dbrToInt(value);
						if (isR != -1) {
							updateAllRows();
							ModelCombineDatasets.get().ll.notifyListeners();
						}
					}
				});

		connected = true;
	}

	private void updateRow() {
		// TODO Auto-generated method stub
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);

		if (result.numRows() > 0) {
			for (HashMap<String, String> row : result.getRows()) {
				if (row.containsKey("state"))
					ModelCombineDatasets.STATE = row.get("state");
				if (row.containsKey("lowres"))
					ModelCombineDatasets.LowRes = row.get("lowres");
				if (row.containsKey("highres"))
					ModelCombineDatasets.HighRes = row.get("highres");
				if (row.containsKey("warning"))
					ModelCombineDatasets.WARNING = row.get("warning");
				if (row.containsKey("dir"))
					ModelCombineDatasets.DIR = row.get("dir");
				if (row.containsKey("merge_result"))
					ModelCombineDatasets.RESULTFILE = row.get("merge_result");
			}
		}

		ModelCombineDatasets.get().ll.notifyListeners();
	}

	public void setResultDatasets(boolean update,
			HashMap<HKLDataset, String> fieldMap) {
		verifyTableExists();
		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + dataTableName + "`");
		
		Database.exec(
				DBName.USERDATA,
				"insert into "
						+ dataTableName
						+ " (HKLFile, Resolution, Spacegroup, Unitcell, Completeness, Rmege, Rpim, "
						+ "CChalf, ISigma, Multiplicity, AnomCompleteness, AnomMulti) "
						+ "values ("
						+ "'"
						+ fieldMap.get(ModelCombineDatasets.HKLDataset.HKLFILE)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.RESOLUTION)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.SPACEGROUP)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.UNIT_CELL)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.COMPLETE)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.RMERGE)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.RPIM)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.CCHALF)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.ISIGMA)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.MULTIPLICITY)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.ANOM_COMPLETE)
						+ "', '"
						+ fieldMap
								.get(ModelCombineDatasets.HKLDataset.ANOM_MULTIPLICITY)
						+ "')");

		newResultCreated = true;

		if (update) {
			sendRowUpdate(result.numRows());
		}

	}

	public void setResult(boolean update) {
		verifyTableExists();
		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "`");
		if (0 == result.numRows()) {
			Database.exec(DBName.USERDATA, "insert into " + tableName
					+ " (merge_result, warning, dir, lowres, highres, state) "
					+ "values (" + "'" + ModelCombineDatasets.RESULTFILE + "', '"
					+ ModelCombineDatasets.WARNING + "', '"
					+ ModelCombineDatasets.DIR + "', '"
					+ ModelCombineDatasets.LowRes + "', '"
					+ ModelCombineDatasets.HighRes + "', '"
					+ ModelCombineDatasets.STATE + "')");
			newResultCreated = true;
		} else {
			Database.exec(DBName.USERDATA, "update " + tableName + " "
					+ "set merge_result='" + ModelCombineDatasets.RESULTFILE
					+ "' " + ", warning='" + ModelCombineDatasets.WARNING + "' "
					+ ", dir='" + ModelCombineDatasets.DIR + "' " + ", lowres='"
					+ ModelCombineDatasets.LowRes + "' " + ", highres='"
					+ ModelCombineDatasets.HighRes + "' " + ", state='"
					+ ModelCombineDatasets.STATE + "' ");

		}
		if (update)
			ModelCombineDatasets.get().ll.notifyListeners();
	}

	protected void updateAllRows() {
		verifyTableExists();
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		final Database.Result resultDatasets = Database.exec(DBName.USERDATA,
				"select * from " + dataTableName);
		if (result.numRows() > 0) {
			for (HashMap<String, String> row : result.getRows()) {
				if (row.containsKey("state"))
					ModelCombineDatasets.STATE = row.get("state");
				if (row.containsKey("warning"))
					ModelCombineDatasets.WARNING = row.get("warning");
				if (row.containsKey("dir"))
					ModelCombineDatasets.DIR = row.get("dir");
				if (row.containsKey("lowres"))
					ModelCombineDatasets.LowRes = row.get("lowres");
				if (row.containsKey("highres"))
					ModelCombineDatasets.HighRes = row.get("highres");
				if (row.containsKey("merge_result"))
					ModelCombineDatasets.RESULTFILE = row.get("merge_result");
			}
		}
		ModelCombineDatasets.listHKLSets.clear();
		if (resultDatasets.numRows() == 0) {
			return;
		}
		for (int i = 0; i < resultDatasets.numRows(); i++) {
			
			HashMap<HKLDataset, String> fieldMap = new HashMap<HKLDataset, String>();
			HashMap<String, String> row = resultDatasets.getRow(i);
			for (HKLDataset field : HKLDataset.values()) {
				for (String sqlCol : row.keySet()) {
					if (field.getDatasetString().equals(sqlCol))
						fieldMap.put(field, row.get(sqlCol));
				}
			}
			ModelCombineDatasets.listHKLSets.add(fieldMap);
		}
	}
	protected void updateSelectRow(int select, String hklFile) {
		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + dataTableName + "`");
		
		if(result.numRows() > 0) {
			Database.exec(DBName.USERDATA, "update " + dataTableName + " "
					+ "set Selection=" + select+ " where HKLFile=\""+ hklFile
					+ "\"");
		}
	}
	protected void removeRow(String hklFile) {
		final Database.Result resultDatasets = Database.exec(DBName.USERDATA,
				"delete from " + dataTableName + " where HKLFile=\"" + hklFile
						+ "\"");
		sendRowUpdate(0);
		ModelCombineDatasets.get().ll.notifyListeners();
	}

	protected void sendRowUpdate(int sampleNumber) {
		Epics.putInteger(PV_STRATEGY.mergeDatasetsUpdated, -1);
		Epics.putInteger(PV_STRATEGY.mergeDatasetsUpdated, sampleNumber);
	}

	public void deleteAllResults() {
		Log.note("Deleting all strategy results");

		verifyTableExists();

		Database.delete(DBName.USERDATA, tableName, null);
		Database.delete(DBName.USERDATA, dataTableName, null);
		updateAllRows();
		ModelCombineDatasets.get().ll.notifyListeners();
	}
}
