/**
 * GUI for CombineDatasets tab
 */

package gov.anl.gmca.dataproc.combinedatasets;

import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.ui.NBDirectoryDialog;
import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.dataproc.CmdLogview;
import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets.HKLDataset;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.multistrategy.DevReferenceData;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.io.File;
import java.util.HashMap;

public class WdgCombineDatasets extends Composite implements
		ListenerList.Listener<ModelCombineDatasets>, PaintListener,
		ClientListener {

	private Composite expandComp, fileComp;
	private Button runXDSButton, dirBrowse, xscaleLogButton, aimlessLogButton,
			addButton, runBLENDButton;
	private Text dirLabelText, warningText, lowResLimitEntry,
			highResLimitEntry;
	private Text mergeFileLabel;

	private Table table;
	boolean seqIsWide;
	boolean seqCurrentlyWide;
	private CmdCombineDatasets cmdCombineDatasets;
	private CmdRunBLEND cmdRunBLEND;

	protected static final ModelCombineDatasets.HKLDataset[] displayFields = {
			ModelCombineDatasets.HKLDataset.SELECT,
			ModelCombineDatasets.HKLDataset.HKLFILE,
			ModelCombineDatasets.HKLDataset.RESOLUTION,
			ModelCombineDatasets.HKLDataset.SPACEGROUP,
			ModelCombineDatasets.HKLDataset.UNIT_CELL,
			ModelCombineDatasets.HKLDataset.COMPLETE,
			ModelCombineDatasets.HKLDataset.RMERGE,
			ModelCombineDatasets.HKLDataset.RPIM,
			ModelCombineDatasets.HKLDataset.CCHALF,
			ModelCombineDatasets.HKLDataset.ISIGMA,
			ModelCombineDatasets.HKLDataset.MULTIPLICITY,
			ModelCombineDatasets.HKLDataset.ANOM_COMPLETE,
			ModelCombineDatasets.HKLDataset.ANOM_MULTIPLICITY
	};

	public WdgCombineDatasets(Composite parent, int style) {
		super(parent, style);
		ModelCombineDatasets.get().ll.addListener(this);
		DevCombineDatasets.get().verifyConnected();
		Client.addListener(this);

		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		gl.horizontalSpacing = 0;
		gl.verticalSpacing = 3;
		setLayout(gl);

		expandComp = new Composite(this, SWT.NONE);
		expandComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(8, false);
		expandComp.setLayout(gl1);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		expandComp.setLayoutData(gd);

		createCombineDatasets();
	}

	public void setResolution() {
		String lowStr = lowResLimitEntry.getText();
		String highStr = highResLimitEntry.getText();

		if (lowStr.equals("") || lowStr == null || highStr.equals("")
				|| highStr == null || lowStr.equals("-") || highStr.equals("-")) {
			DevReferenceData.get().setResult(true);
			return;
		}

		double low = Double.parseDouble(lowStr);
		double high = Double.parseDouble(highStr);

		if (low > 50.0) {
			Log.warn("Low resolution should be less than 50.0");
			DevReferenceData.get().setResult(true);
			return;
		}
		if (high < 0.0) {
			Log.warn("High resolution should be greater than 0.0");
			DevReferenceData.get().setResult(true);
			return;
		}
		if (low < high) {
			Log.warn("Value of Low Res. > High Res");
			DevReferenceData.get().setResult(true);
			return;
		}

		_setResolution(low, high);
	}

	public void _setResolution(double low, double high) {
		ModelCombineDatasets.LowRes = Double.toString(low);
		ModelCombineDatasets.HighRes = Double.toString(high);
		Log.note("High resolution limit " + high + " Low resolution limit "
				+ low);
		DevCombineDatasets.get().setResult(true);
	}

	private void createCombineDatasets() {

		fileComp = new Composite(expandComp, SWT.NONE);
		fileComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(7, false);
		fileComp.setLayout(gl1);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 8;
		gd.verticalSpan = 4;
		fileComp.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 8;
		final Group tableGrp = new Group(fileComp, SWT.NONE);
		tableGrp.setFont(GUI.FONT_COLLECT_GROUP);
		tableGrp.setText("To list Datasets press Export XDS_ASCII.HKL button on Data Quality tab or Manually load XDS_ASCII.HKL of XDS or .sca of HKL2000)");
		tableGrp.setBackground(GUI.BGCOLOR);
		tableGrp.setLayoutData(gd);
		tableGrp.setLayout(new GridLayout(1, false));
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 8;
		gd.heightHint = 150;
		gd.widthHint = 350;
		gd.verticalSpan = 4;

		table = new Table(tableGrp, SWT.CHECK | SWT.WRAP | SWT.BORDER
				| SWT.V_SCROLL | SWT.H_SCROLL);
		table.setLinesVisible(true);

		for (ModelCombineDatasets.HKLDataset field : displayFields) {
			TableColumn tc = new TableColumn(table, SWT.CHECK);

			String s = field.getDatasetString();
			tc.setText(s);
			tc.setWidth(field.getWidth());
			if (!field.equals(HKLDataset.HKLFILE)) {
				tc.setAlignment(SWT.CENTER);
			}
			if(field.equals(HKLDataset.CCHALF)) {
				tc.setText("CC1/2");
			}
		}
		table.setHeaderVisible(true);
		table.setLayoutData(gd);

		table.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {

				if (event.detail == SWT.CHECK) {
					final TableItem item = (TableItem) event.item;
					int idx = table.indexOf(item);

					if (item.getChecked() == true) {
						ModelCombineDatasets.listHKLSets.get(idx).put(
								HKLDataset.SELECT, "1");
					}
					else {
						ModelCombineDatasets.listHKLSets.get(idx).put(
								HKLDataset.SELECT, "0");
					}

					ModelCombineDatasets.updateList(ModelCombineDatasets.listHKLSets.get(idx));
				}
			}
		});

		Menu menu = new Menu(table); // where table1 is your table
		MenuItem item = new MenuItem(menu, SWT.PUSH);
		item.setText("remove");
		table.setMenu(menu);

		item.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					@Override
					public void run() {
						if (!Client.isMaster()) {
							Log.warn("MultiCrystalDatasets : This client is not master, remove not valid");
							return;
						}
						int ti = table.getSelectionIndex();
						ModelCombineDatasets.listHKLSets.remove(ti);
						ModelCombineDatasets.get().removeRow(
								table.getItem(ti).getText(1).replace("~", DevDataDir.getPrimary()));
					}

				});
			}
		});

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.horizontalSpan = 8;

		addButton = new Button(tableGrp, SWT.PUSH);
		addButton.setText("Import XDS_ASCII.HKL or .sca from HKL2000");
		addButton.setLayoutData(gd);
		addButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		addButton.setToolTipText("Add processed dataset for the list \nin the form of XDS_ASCII.HKL");

		addButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						Shell fileShell = new Shell(SWT.NONE);
						NBFileDialog dialog = new NBFileDialog(fileShell,
								SWT.OPEN);
						String path1 = DevDataDir.getPrimary();
						if (table.getItemCount() > 0) {
							TableItem firstFile = table.getItem(0);
							int size = ModelCombineDatasets.listHKLSets.size()-1;
							String first = ModelCombineDatasets.listHKLSets
									.get(size).get(HKLDataset.HKLFILE);
							path1 = first.substring(0, first.lastIndexOf("/"));
						}

						dialog.setFilterPath(path1);
						dialog.setFilterExtensions(new String[] { "*HKL",
								"*.hkl", "*.ahkl", "*.AHKL", "*.sca", "*.SCA", "*.*"});

						String path = dialog.openStr();
						if(path!= null)
							ModelCombineDatasets.parseHKLFile(path, "");
					}
				});

			}
		});

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.horizontalSpan = 8;

		Group grpResLimit = new Group(fileComp, SWT.CENTER);
		grpResLimit.setText("Res. Limit:");
		grpResLimit.setBackground(GUI.BGCOLOR);
		grpResLimit.setFont(GUI.FONT_COLLECT_GROUP);
		grpResLimit.setLayoutData(gd);
		grpResLimit
				.setToolTipText("Resolution limits for processing by XSCALE");

		GridLayout glRes = new GridLayout(7, false);
		grpResLimit.setLayout(glRes);

		gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		Label lowLabel = new Label(grpResLimit, SWT.NONE);
		lowLabel.setText("L");
		lowLabel.setBackground(GUI.BGCOLOR);
		lowLabel.setLayoutData(gd);

		gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, false);
		gd.horizontalSpan = 2;
		gd.widthHint = 80;

		lowResLimitEntry = new Text(grpResLimit, SWT.BORDER);
		lowResLimitEntry.setLayoutData(gd);

		lowResLimitEntry.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		lowResLimitEntry.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				setResolution();
			}
		});
		lowResLimitEntry.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					setResolution();
				}
			}
		});

		gd = new GridData(SWT.RIGHT, SWT.CENTER, true, false);

		Label highLabel = new Label(grpResLimit, SWT.NONE);
		highLabel.setText("H");
		highLabel.setBackground(GUI.BGCOLOR);
		highLabel.setLayoutData(gd);

		gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, false);
		gd.horizontalSpan = 2;
		gd.widthHint = 80;

		highResLimitEntry = new Text(grpResLimit, SWT.BORDER);
		highResLimitEntry.setLayoutData(gd);

		highResLimitEntry.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		highResLimitEntry.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				setResolution();
			}
		});
		highResLimitEntry.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					setResolution();
				}
			}
		});

		Label angLabel = new Label(grpResLimit, SWT.LEFT);
		angLabel.setText("\u212B");
		angLabel.setBackground(GUI.BGCOLOR);
		angLabel.setLayoutData(new GridData());
		
		
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 8;
		Label fileListLabel = new Label(fileComp, SWT.NONE);
		fileListLabel.setFont(GUI.FONT_COLLECT_GROUP);
		fileListLabel.setBackground(GUI.BGCOLOR);
		fileListLabel.setText("Select output directory to save combined XDS_ASCII.AHKL: ");
		fileListLabel.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 6;
		dirLabelText = new Text(fileComp, SWT.BORDER);
		dirLabelText.setLayoutData(gd);

		gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false);
		gd.horizontalSpan = 1;
		gd.widthHint = 80;
		dirBrowse = new Button(fileComp, SWT.PUSH);
		dirBrowse.setText("Dir.");
		dirBrowse.setLayoutData(gd);
		dirBrowse.setToolTipText("Select output directory where the results are stored");

		dirBrowse.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						NBDirectoryDialog dlg = new NBDirectoryDialog(
								new Shell(GUI.getDisplay()));
						String path1 = DevDataDir.getPrimary();
						String path2 = dirLabelText.getText();
						if (path2.equals("") || path2 == null)
							dlg.setFilterPath(path1);
						else
							dlg.setFilterPath(path2);

						String dir = dlg.openStr();
						if (dir != null) {
							// Set the text box to the new selection
							dirLabelText.setText(dir);
							ModelCombineDatasets.DIR = dir;
						}
						DevCombineDatasets.get().setResult(true);
					}
				});
			}
		});
		
		Composite buttonComp = new Composite(expandComp, SWT.NONE);
		buttonComp.setBackground(GUI.BGCOLOR);
		GridLayout gl2 = new GridLayout(8, false);
		buttonComp.setLayout(gl2);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 8;
		buttonComp.setLayoutData(gd);
		
		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.widthHint = 250;
		gd.horizontalSpan = 4;

		runXDSButton = new Button(buttonComp, SWT.PUSH );
		runXDSButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		runXDSButton.setText("Combine Datasets with XSCALE");
		runXDSButton.setLayoutData(gd);
		runXDSButton.setToolTipText("Combine Datasets with XSCALE");

		runXDSButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {

				if (ModelCombineDatasets.listHKLSets.size() < 1) {
					Log.error("Atleast one XDS_ASCII.HKL needed for processing");
					return;
				}

				String dir = dirLabelText.getText();

				if (dir.equals("") || dir == null) {
					Log.error("Selected working directory doesn't exist");
					return;
				}
				Log.note("XDS_ASCII.HKL file directory : " + dir);

				File hklFile = new File(dir + "/XSCALE/XDS_ASCII.AHKL");
				boolean isOk = true;
				if (hklFile.exists()) {
					isOk = Util.popupOkCancel("Overwrite", dir
							+ "/XSCALE/XDS_ASCII.AHKL"
							+ " exists. Do you want overwrite?");
				}
				boolean isOk1 = true;
				if (ModelCombineDatasets.listHKLSets.size() == 1) {
					isOk1 = Util.popupOkCancel("Nothing to scale","There is only one dataset in the list, nothing to scale"
							+ "Do you still want to proceed");
				}
				
				/* run XSCALE */
				if (isOk && isOk1) {
					cmdCombineDatasets = new CmdCombineDatasets();
					cmdCombineDatasets.workDir = dir;
					cmdCombineDatasets.start();
				}

			}
		});
		
		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.widthHint = 250;
		gd.horizontalSpan = 4;

		runBLENDButton = new Button(buttonComp, SWT.PUSH );
		runBLENDButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		runBLENDButton.setText("Analyse with BLEND");
		runBLENDButton.setLayoutData(gd);
		runBLENDButton.setEnabled(false);
		runBLENDButton.setToolTipText("Software BLEND will give clusters of datasets \n"
				+ "which can be combined for better data");

		runBLENDButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {

				if (ModelCombineDatasets.listHKLSets.size() < 3) {
					Log.error("Atleast three XDS_ASCII.HKL needed for processing");
					return;
				}

				String dir = dirLabelText.getText();

				if (dir.equals("") || dir == null) {
					Log.error("Selected working directory doesn't exist");
					return;
				}

				/* run BLEND */
				cmdRunBLEND = new CmdRunBLEND();
				cmdRunBLEND.workDir = dir;
				cmdRunBLEND.start();
			}
		});
		
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 8;

		Group resultGroup = new Group(expandComp, SWT.NONE);
		resultGroup.setFont(GUI.FONT_COLLECT_GROUP);
		resultGroup.setBackground(GUI.BGCOLOR);
		resultGroup.setText("XSCALE Results");
		
		GridLayout actionGrid = new GridLayout(8, true);
		resultGroup.setLayout(actionGrid);
		resultGroup.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 1;
		Label labelWarning = new Label(resultGroup, SWT.NONE);
		labelWarning.setBackground(GUI.BGCOLOR);
		labelWarning.setText("Warning");
		labelWarning.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 7;

		warningText = new Text(resultGroup, SWT.WRAP | SWT.MULTI | SWT.BORDER);
		warningText.setForeground(GUI.COLOR_RED);
		warningText.setLayoutData(gd);

		gd = new GridData(SWT.LEFT, SWT.FILL, true, true);
		gd.horizontalSpan = 1;
		Label labelCombine = new Label(resultGroup, SWT.NONE);
		labelCombine.setBackground(GUI.BGCOLOR);
		labelCombine.setText("Combined to:");
		labelCombine.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 7;
		gd.minimumHeight = 75;
		mergeFileLabel = new Text(resultGroup, SWT.MULTI | SWT.WRAP| SWT.BORDER);
		mergeFileLabel.setLayoutData(gd);

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.horizontalSpan = 4;
		
		xscaleLogButton = new Button(resultGroup, SWT.PUSH);
		xscaleLogButton.setText("XSCALE LOG");
		xscaleLogButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		xscaleLogButton.setLayoutData(gd);
		xscaleLogButton.setToolTipText("View XSCALE LOG");

		xscaleLogButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						String logfile = "XSCALE.LOG";
						String filename = ModelCombineDatasets.DIR + "/XSCALE/XSCALE.LP";

						// TODO: handle spaces
						File xdsFile = new File(filename);
						if (xdsFile.exists())
							ExecTask.new_ExecTask("nedit " + filename + "",
									"nedit", false, false).start();
						else
							Log.error(filename + " doesn't exists");
					}
				});
			}
		});

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.horizontalSpan = 4;
		aimlessLogButton = new Button(resultGroup, SWT.PUSH);
		aimlessLogButton.setText("AIMLESS LOG");
		aimlessLogButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		aimlessLogButton.setLayoutData(gd);
		aimlessLogButton.setToolTipText("View AIMLESS log for statistics");

		aimlessLogButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						String logfile = "aimless.log";
						String filename = ModelCombineDatasets.DIR
								+ "/POINTLESS_AIMLESS/mcs_aimless.log";

						// TODO: handle spaces
						File xdsFile = new File(filename);
						if (xdsFile.exists()) {
							
							CmdLogview openFirefox = new CmdLogview(filename,"qtrview AIMLESS MCS");
							openFirefox.start();
							
						} else
							Log.error(filename + " doesn't exists");
					}
				});
			}
		});

	}

	public void updateLabels() {
		
		GUI.asyncExec(new Runnable() {
			public void run() {

				dirLabelText.setText(ModelCombineDatasets.DIR);
				dirLabelText.setSelection(ModelCombineDatasets.DIR.length());

				table.removeAll();
				for (int i = 0; i < ModelCombineDatasets.listHKLSets.size(); i++) {
					if (table.getItemCount() / 4 > ModelCombineDatasets.listHKLSets
							.size())
						return;
					HashMap<HKLDataset, String> field = ModelCombineDatasets.listHKLSets.get(i);
					TableItem tableItem = new TableItem(table, SWT.WRAP);
					for (HKLDataset key : HKLDataset.values()) {
						if (key == HKLDataset.SELECT) {
							if (field.get(key).equals("1")) {
								tableItem.setChecked(true);
								tableItem.setBackground(GUI.COLOR_LIGHT_GREEN);
							} else {
								tableItem.setChecked(false);
								tableItem.setBackground(null);
							}
							continue;
						}
						if (field.get(key) != null) {
							String txt = field.get(key);
							String rStr = DevDataDir.getPrimary();
							if (key.equals(HKLDataset.HKLFILE))
								txt = txt.replace(rStr, "~");
							tableItem.setText(key.ordinal(), txt);
						}
					}
				}

				warningText.setText(ModelCombineDatasets.WARNING);
				mergeFileLabel.setText(ModelCombineDatasets.RESULTFILE.replace(
						"//", "/"));
				lowResLimitEntry.setText(ModelCombineDatasets.LowRes);
				highResLimitEntry.setText(ModelCombineDatasets.HighRes);
				
				updateButtons();
				
				expandComp.layout();
			}
		});
	}

	public void updateButtons() {

		final boolean isMaster = Client.isMaster();

		boolean isRunning = false;
		boolean isBlendRunning = false;
		if(cmdCombineDatasets != null)
			isRunning = cmdCombineDatasets.isAlive();
		//if(cmdRunBLEND != null)
			//isBlendRunning = cmdRunBLEND.isAlive();
		
		final boolean isR = isRunning;
		final boolean isB = isBlendRunning;
		GUI.asyncExec(new Runnable() {
			public void run() {

				addButton.setEnabled(isMaster);
				dirBrowse.setEnabled(isMaster);
				dirLabelText.setEnabled(isMaster);
				table.setEnabled(isMaster);
				runXDSButton.setEnabled(isMaster && !isR);
				//runBLENDButton.setEnabled(isMaster && !isB);
			}
		});
	}

	@Override
	public void masterChanged(boolean isMaster) {
		updateButtons();
	}

	@Override
	public void paintControl(PaintEvent arg0) {
	}

	@Override
	public void handleEvent(ModelCombineDatasets object, Object data) {
		updateLabels();
		updateButtons();
	}

}
