package gov.anl.gmca.diffimage;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import gov.anl.gmca.collect.viewer.ImgSrv;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;

import java.io.InputStream;
import java.util.concurrent.Future;

public class ImageServerImage extends DiffractionImage
{
	private static final int RETRY_COUNT = 2, RETRY_DELAY_MS = 500;
	private String fullpath;
	private double _percentX, _percentY;
	private int _gray;
	private double _zoom;
	private int width, height;
	public static boolean showSmallSpots = true;

	@Override public boolean hasValidImage() {return screenImg!=null;}
	@Override public int origWidth() { return 0; }
	@Override public int origHeight() { return 0; }
	@Override public void setContrast(double gray) { _gray=(int)gray; load(); }
	
	public ImageServerImage(String fullpath, double _percentX, double _percentY, int _gray, double _zoom, int width, int height)
	{
		this.fullpath=fullpath;
		this._percentX=_percentX; this._percentY=_percentY;
		this._gray=_gray; this._zoom=_zoom;
		this.width=width; this.height=height;
		
		int retriesLeft = RETRY_COUNT;
		//imageData = null;
		//while (null == imageData && retriesLeft>0)
		{
			retriesLeft--;
			//Util.sleep(500);
			load();
		}
		
		//if (null == returnedImage.image)
		{
			//title = "File doesn't exist: "+getPathAfterCommonDir(currentFullPath);
			//GUI.asyncExec(new Runnable(){public void run(){redraw();}});
		}
	}

	private HttpResponse<InputStream> httpGet(ImgSrv imgsrv)
	{
		// set up HTTP parameters
		String baseURL = imgsrv.base_url;
		String command = "getImage";
		String percentX = Double.toString(_percentX);
		String percentY = Double.toString(_percentY);
		String gray = Integer.toString(_gray);
		String zoom = Double.toString(_zoom);
		String sizeX = Integer.toString(width);
		String sizeY = Integer.toString(height);
		String userName = "gmca";
		String sessionId = "3282111111111111111";
		String smallSpots = showSmallSpots?"1":"0";

		// construct the URL
		String urlString =
				baseURL+
				command+
				"?fileName="+fullpath+
				"&percentX="+percentX+
				"&percentY="+percentY+
				"&gray="+gray+
				"&zoom="+zoom+
				"&sizeX="+sizeX+
				"&sizeY="+sizeY+
				"&userName="+userName+
				"&sessionId="+sessionId+
				"&smallspots="+smallSpots;

		try {
			// download
			System.out.println("Loading URL: " + urlString);

			Future<HttpResponse<InputStream>> resp = null;
			resp = Unirest.get(urlString).asBinaryAsync();
			while (!resp.isDone()) { Util.sleep(10); }
			HttpResponse<InputStream> is_resp = resp.get();
			if (200 != is_resp.getStatus()) {
				System.out.println("Server returned code " + is_resp.getStatus());
			}
			return is_resp;
		} catch (Exception e) { e.printStackTrace(); return null; }
	}

	private void load()
	{
		try {
			ImgSrv imgsrv = DevUserConfig.get().getImgSrv();
			HttpResponse<InputStream> imgResp = httpGet(imgsrv);
			HttpResponse<InputStream> headerResp = imgResp;

			// save header info that was sent in the HTTP header
			try {
				wavelength_A = Double.parseDouble(headerResp.getHeaders().get("wavelength").get(0));
				distance_mm = Double.parseDouble(headerResp.getHeaders().get("distance").get(0));
				beam_center_mm_from_left = -1.0 * Double.parseDouble(headerResp.getHeaders().get("originX").get(0));
				beam_center_mm_from_top = -1.0 * Double.parseDouble(headerResp.getHeaders().get("originY").get(0));
				mmPerPixel = Double.parseDouble(headerResp.getHeaders().get("pixelSize").get(0));
				exposureTime_sec = 0;//Double.parseDouble(headerResp.getHeaders().get("time").get(0));
				detector = headerResp.getHeaders().get("detectorTypeC64").get(0);
			} catch (Exception e) {
				e.printStackTrace();
			}

			// read the image
			ImageData imageData = new ImageData(imgResp.getRawBody());
			screenImg = new Image(GUI.getDisplay(), imageData);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
