package gov.anl.gmca.gui;

import com.mashape.unirest.http.Unirest;
import gov.anl.gmca.collect.*;
import gov.anl.gmca.common.*;
import gov.anl.gmca.common.client_server.DeviceMap;
import gov.anl.gmca.common.client_server.Popup;
import gov.anl.gmca.dataproc.DevDataProc;
import gov.anl.gmca.dataproc.ModelDataProcList;
import gov.anl.gmca.dataproc.combinedatasets.DevCombineDatasets;
import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.hutch.*;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup;
import gov.anl.gmca.io.PVLookup.PV_HUTCH_SHUTTER;
import gov.anl.gmca.io.PVLookup.PV_MISC;
import gov.anl.gmca.io.PVLookup.PV_UNDULATOR;
import gov.anl.gmca.multistrategy.DevMultiStrategy;
import gov.anl.gmca.multistrategy.DevReferenceData;
import gov.anl.gmca.multistrategy.ModelMultiStrategyList;
import gov.anl.gmca.multistrategy.ModelReferenceData;
import gov.anl.gmca.raster.*;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.cam.SONICCView;
import gov.anl.gmca.raster.util.DevMacroList;
import gov.anl.gmca.raster.util.ModelMacroList;
import gov.anl.gmca.sample.*;
import gov.anl.gmca.scan.DevEdgeDB;
import gov.anl.gmca.scan.DevEdgeScan;
import gov.anl.gmca.scan.DevMCA;
import gov.anl.gmca.screening.DevRobotWarmup;
import gov.anl.gmca.screening.DevScreeningTasks;
import gov.anl.gmca.screening.DevSilTable;
import gov.anl.gmca.screening.ModelScreeningTasks;
import gov.anl.gmca.spotfinder.DevSFOptions;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.SpotFinderQueue;
import gov.anl.gmca.strategy.DevStrategyResults;
import gov.anl.gmca.strategy.ModelStrategyResultList;
import gov.anl.gmca.users.DevUserTable;
import gov.anl.gmca.users.ModelUserTable;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;
import java.util.HashMap;

public class DeviceInit
{
	public static void init()
	{
		PVLookup.initWaveformPVs();
		DevRuntimeConfig.init();
		new DevUserConfig();
		Unirest.setTimeouts(3000,10000);

		////////////////////////////////////////////////////// set disabled PVs //////////////////////////////////////////////////

		if (Main.DISABLE_APS_PVS)
		{
			Epics.disablePV(PV_UNDULATOR.Busy_VAL, 							0);
			Epics.disablePV(PV_UNDULATOR.Energy_VAL, 						12.67);
			Epics.disablePV(PV_UNDULATOR.EnergySet_VAL, 					12.67);
			Epics.disablePV(PV_UNDULATOR.Start_VAL, 						0);
			Epics.disablePV(PV_UNDULATOR.Stop_VAL, 							0);
			Epics.disablePV(PV_UNDULATOR.HarmonicValue,						0);
			Epics.disablePV(PV_HUTCH_SHUTTER.stat.shutter("endStation"), 	0);
			Epics.disablePV(PV_HUTCH_SHUTTER.open.shutter("endStation"), 	0);
			Epics.disablePV(PV_HUTCH_SHUTTER.close.shutter("endStation"), 	0);
			Epics.disablePV(PV_HUTCH_SHUTTER.stat.shutter("aShutter"), 		0);
			Epics.disablePV(PV_HUTCH_SHUTTER.open.shutter("aShutter"), 		0);
			Epics.disablePV(PV_HUTCH_SHUTTER.close.shutter("aShutter"), 	0);
			Epics.disablePV(PV_MISC.apsCurrent,								0);
			Epics.disablePV(PV_MISC.apsShutterStat,							0);
			Epics.disablePV(PV_MISC.stationSearched,						0);
		}
		
		/////////////////////////////////////////////////// User and script tables ///////////////////////////////////////////////
		
		new ModelUserTable();
		DevUserTable dev = new DevUserTable();

		//ModelScriptTable scriptTable = new ModelScriptTable();
		//new DevScriptTable(scriptTable);
		
    	//////////////////////////////////////////////////////// Config /////////////////////////////////////////////////////
    	
		new DevDataDir();
		
    	//////////////////////////////////////////////////////// PMAC motors /////////////////////////////////////////////////////

		String crlMotors = Util.getBID().equals("23i") ? ",'crl_h','crl_v','crl_x'" : "";
    	Result pmaResult = Database.exec(DBName.CONFIG, "select * from PmacMotorAssembly where name in ('sample_x','sample_y','sample_z','gonio_omega','detector_z','detector_2t','beamstop_z','beam_size_x','beam_size_y','mo_En_E','camera_zoom','camera_zoom_2'"+crlMotors+")");
		if (true)//!Main.USE_SERVER)
		{
			for (int i=0;i<pmaResult.numRows();i++)
			{
				String devName = pmaResult.getRow(i).get("name");
				boolean useRunPrg = pmaResult.getRow(i).get("useRunPrg").contains("1");
				
				DeviceMap.pmacMotorMap.put(devName, new DevPmacMotor(devName, useRunPrg, pmaResult.getRow(i).get("inPosBand")));
			}
		}
		
    	////////////////////////////////////////////////////// PMAC assemblies ///////////////////////////////////////////////////

		for (int i=0;i<pmaResult.numRows();i++)
		{
			String assemblyName = pmaResult.getRow(i).get("assembly");
			
			// if this device is in an assembly and we didn't already create the assembly
			if (null != assemblyName && assemblyName.length() > 0 && null == DevPmacAssembly.get(assemblyName))
			{
				// gather all devices in this assembly
				ArrayList<DevPmacMotor> motorList = new ArrayList<DevPmacMotor>();
				for (int j=0;j<pmaResult.numRows();j++)
				{
					String assembly2 = pmaResult.getRow(j).get("assembly");
					if (null != assembly2 && assembly2.equals(assemblyName))
					{
						String motorName = pmaResult.getRow(j).get("name");
						motorList.add(DevPmacMotor.get(motorName));
					}
				}
				
				// create it
				if (null == DevPmacAssembly.get(assemblyName))
				{
					System.out.println("Creating assembly "+assemblyName+" with motors "+motorList);
					new DevPmacAssembly(assemblyName, motorList);
				}
			}
		}
		
		/////////////////////////////////////////////////////////// MCA //////////////////////////////////////////////////////////

		new DevMCA();
		
		///////////////////////////////////////////////////////// shutters ///////////////////////////////////////////////////////

		DevShutter.init();

		pmaResult = Database.exec(DBName.CONFIG, "select * from HutchShutter");
		for (int i=0;i<pmaResult.numRows();i++)
		{
			HashMap<String,String> row = pmaResult.getRow(i);
			
			String devName = row.get("Shutter");
			new DevHutchShutter(devName);
		}
		
		///////////////////////////////////////////////////////// attenuation //////////////////////////////////////////////////////
		
		new DevAttenuation();

		/////////////////////////////////////////////////////////// energy /////////////////////////////////////////////////////////
		
		boolean undulatorEnabled = DBConfig.get("unduflag").equals("1");
		if (undulatorEnabled)
		{
			if (Util.getBID().equals("23i")) new DevUndulator();
			if (Util.getBID().equals("23o")) new DevUndulator();
		}
		
		new DevEnergy();
		
		////////////////////////////////////////////////////////// databases ///////////////////////////////////////////////////////
		
		DBConfig.connectListener();
		DBConfig.update();
		if (Util.beamSizeCalculated()) DBBeamSize.init();
		new DevSFType(SFRunType.COLLECT);
		new DevSFType(SFRunType.RASTER);
		new DevEdgeDB();
		
		/////////////////////////////////////////////////////////// camera /////////////////////////////////////////////////////////
		
		DBCamera.update();

		////////////////////////////////////////////////////////// scripts /////////////////////////////////////////////////////////

		new DevAlignMinibeam();
		new DevAutocenter();
		new DevCollimatorCtrl();
		new DevEdgeScan();
		
		//////////////////////////////////////////////////////////// FPE ///////////////////////////////////////////////////////////

		if (Main.SIM_FPE)
			new DevSimFPE();
		else
			new DevFPE2();
		DevEiger.init();

		////////////////////////////////////////////////////////////lights ///////////////////////////////////////////////////////////

		DevSlider.init();

		/////////////////////////////////////////////////// Spotfinder and SafeImagDelete ///////////////////////////////////////////
		
		SpotFinderQueue.init();
		SafeImageDelete.init();
		
		////////////////////////////////////////////////////////// run lists ///////////////////////////////////////////////////////
		
		ModelCollectRunList collectRunList = new ModelCollectRunList();
		new DevCollectRunList(collectRunList);
		
		ModelRasterRunList interRunList = new ModelRasterRunList(RasterListType.INTER);
		new DevRasterRunList(interRunList).connect();
		
		ModelRasterRunList autoRunList = new ModelRasterRunList(RasterListType.AUTO);
		new DevRasterRunList(autoRunList).connect();
		
		ModelMacroList macroList = new ModelMacroList();
		new DevMacroList(macroList);
		
		// create the state object last since it connects listeners to the two lists
		ModelRasterState rasterState = new ModelRasterState();
		new DevRasterState(rasterState);
		
		new ModelRasterSiteList();
		new DBRasterSiteList();
		
		// Data collection strategy
		new ModelStrategyResultList();
		new DevStrategyResults();
		
		// Data processing with XDS/fast_dp
		new ModelDataProcList();
		new DevDataProc();
		
		//screening crystal scoring
		new DevScreeningTasks();
		new DevSilTable();
		
		//Multiple crystal strategy
		new ModelReferenceData();
		new ModelMultiStrategyList();
		new ModelCombineDatasets();
		new DevMultiStrategy();
		new DevReferenceData();
		new DevCombineDatasets();
		
		//Collect parameters database
		new ModelScreeningTasks();
		
		/////////////////////////////////////////////////////////  Misc /////////////////////////////////////////////////////////
		
		DevAPSStatus.init();
		DevADC.init();
		DevScriptInterface.init();
		new DetectorConfig();
		
		DevPinSensor.init();
		
		new DevKeithleyAmp();
		DevUserTable.get().connectListeners();
		
		new DevRobotWarmup();
		new ModelBeamsize();
		new DevBinMode();
		new DevMisc();
		DBImportSites.init();
		DevSFOptions.init();
		Popup.init();
		SONICCView.init();
	}
}
