package gov.anl.gmca.gui;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.Resources;
import gov.anl.gmca.syslog.Log;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.internal.gtk.OS;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import java.lang.Thread.UncaughtExceptionHandler;
import java.util.ArrayList;
import java.util.HashMap;

public class GUI
{
	static class InitDisplay
	{
		public Display display = null;
		
		public InitDisplay()
		{
			// use this block to trace SWT graphic resource leaks
			// (and comment out the last line of this function)
			
			/*DeviceData data = new DeviceData();
			data.tracking = true;
			display = new Display(data);
			org.eclipse.swt.tools.internal.Sleak sleak = new org.eclipse.swt.tools.internal.Sleak();
			sleak.open();*/
			
			//DeviceData data = new DeviceData();
			//data.debug = true;
			//display = new Display(data);
			
			// TODO: skip this when running in server mode
			try
			{
				display = new Display();
			}
			catch (Exception e) { e.printStackTrace(); }
		}
	}
	
	protected static Display display = null;
	protected static Shell shell;
	GUIThreadMonitor monitor;

	public static Display getDisplay()
	{
		if (null==display)
		{
			display = new InitDisplay().display;
			GUIStatus.inGUIMode = true;
		}
		return display;
	}
	public static Shell getShell() { return shell; }

	public final static int STARTING = 1;
	public final static int RUNNING = 2;
	public final static int STOPPING = 3;
	public static int appState = STARTING;
	
	public static final Color COLOR_BLACK = 			GUI.getDisplay().getSystemColor(SWT.COLOR_BLACK);
	public static final Color COLOR_BLUE = 				GUI.getDisplay().getSystemColor(SWT.COLOR_BLUE);
	public static final Color COLOR_CYAN = 				GUI.getDisplay().getSystemColor(SWT.COLOR_CYAN);
	public static final Color COLOR_DARK_BLUE = 		GUI.getDisplay().getSystemColor(SWT.COLOR_DARK_BLUE);
	public static final Color COLOR_DARK_CYAN = 		GUI.getDisplay().getSystemColor(SWT.COLOR_DARK_CYAN);
	public static final java.awt.Color COLOR_DARK_CYAN_AWT = new java.awt.Color(((float)COLOR_DARK_CYAN.getRed())/255f,((float)COLOR_DARK_CYAN.getGreen())/255f,((float)COLOR_DARK_CYAN.getBlue())/255f);
	public static final Color COLOR_DARK_GREEN = 		GUI.getDisplay().getSystemColor(SWT.COLOR_DARK_GREEN);
	public static final java.awt.Color COLOR_DARK_GREEN_AWT = new java.awt.Color(((float)COLOR_DARK_GREEN.getRed())/255f,((float)COLOR_DARK_GREEN.getGreen())/255f,((float)COLOR_DARK_GREEN.getBlue())/255f);
	public static final Color COLOR_DARK_GRAY = 		GUI.getDisplay().getSystemColor(SWT.COLOR_DARK_GRAY);
	public static final Color COLOR_DARK_RED = 			GUI.getDisplay().getSystemColor(SWT.COLOR_DARK_RED);
	public static final Color COLOR_RED = 				GUI.getDisplay().getSystemColor(SWT.COLOR_RED);
	public static final Color COLOR_YELLOW =			GUI.getDisplay().getSystemColor(SWT.COLOR_YELLOW);
	public static final Color COLOR_CHECKS =			new Color(GUI.getDisplay(), 255, 250, 227);
	public static final Color COLOR_GRAY = 				GUI.getDisplay().getSystemColor(SWT.COLOR_GRAY);
	public static final Color COLOR_WHITE = 			GUI.getDisplay().getSystemColor(SWT.COLOR_WHITE);
	public static final Color COLOR_MAGENTA =			GUI.getDisplay().getSystemColor(SWT.COLOR_DARK_MAGENTA);
	public static final java.awt.Color COLOR_MAGENTA_AWT =	new java.awt.Color(((float)COLOR_MAGENTA.getRed())/255f,((float)COLOR_MAGENTA.getGreen())/255f,((float)COLOR_MAGENTA.getBlue())/255f);
	public static final Color COLOR_DARK_YELLOW =		new Color(GUI.getDisplay(), 235, 235, 215);
	public static final Color COLOR_RED1 = 				new Color(GUI.getDisplay(), 251, 154, 153);
	public static final Color COLOR_RED2 = 				new Color(GUI.getDisplay(), 227, 26,  28);
	public static final Color COLOR_GREEN1 = 			new Color(GUI.getDisplay(), 178, 223, 138);
	public static final Color COLOR_GREEN2 = 			new Color(GUI.getDisplay(), 51,  160, 44);
	public static final Color COLOR_BLUE1 = 			new Color(GUI.getDisplay(), 166, 206, 227);
	public static final Color COLOR_BLUE2 = 			new Color(GUI.getDisplay(), 31,  120, 180);
	public static final Color COLOR_PURPLE = 			new Color(GUI.getDisplay(), 223, 0,   211);
	public static final java.awt.Color COLOR_PURPLE_AWT = new java.awt.Color(223f/255f,0,211f/255f);
	public static final Color BGCOLOR = 				new Color(GUI.getDisplay(), 211, 211, 211);
	// public static final Color COLOR_BLUE_BUTTON = 		new Color(GUI.getDisplay(), 192, 192, 255); // old ssrl blue button color
	public static final Color COLOR_BROWNRED = 			new Color(GUI.getDisplay(), 160, 53,  42);
	public static final java.awt.Color COLOR_BROWNRED_AWT = new java.awt.Color(160f/255f,53f/255f,42f/255f);
	public static final Color COLOR_BROWN = 			new Color(GUI.getDisplay(), 139, 0,   0);
	public static final Color COLOR_CYAN_BUTTON = 		new Color(GUI.getDisplay(), 173, 216, 230);
	public static final Color COLOR_DARK = 				new Color(GUI.getDisplay(), 119, 119, 119);
	public static final Color COLOR_DARK_GRAY2 = 		new Color(GUI.getDisplay(), 100, 100, 100);
	public static final Color COLOR_DARK_GRAY3 =		new Color(GUI.getDisplay(), 96,  96,  96 );
	public static final Color COLOR_DARK_GRAY4 =		new Color(GUI.getDisplay(), 70,  70,  70 );
	public static final Color COLOR_DARK_GRAY5 = 		new Color(GUI.getDisplay(), 169, 157, 140);
	public static final Color COLOR_DISABLED_GRAY =		new Color(GUI.getDisplay(), 145, 145, 145);
	public static final Color COLOR_SWT_BORDER = 		new Color(GUI.getDisplay(), 161, 157, 154);
	public static final Color COLOR_DARK_RED2 =			new Color(GUI.getDisplay(), 200, 80,  80 );
	public static final Color COLOR_DARK_RED3 =			new Color(GUI.getDisplay(), 165, 42,  42 );
	public static final Color COLOR_BRIGHT_GREEN =		new Color(GUI.getDisplay(), 0,   255, 0  );
	public static final Color COLOR_BRIGHT_GREEN2 =		new Color(GUI.getDisplay(), 40,  255, 75 );
	public static final java.awt.Color COLOR_BRIGHT_GREEN_AWT 		= new java.awt.Color(0,1,0);
	public static final java.awt.Color COLOR_BRIGHT_GREEN2_AWT 		= new java.awt.Color(40f/255f,1,75f/255f);
	public static final Color COLOR_BRIGHT_GREEN3 =		new Color(GUI.getDisplay(), 10,  215, 10 );
	public static final Color COLOR_GREEN =				new Color(GUI.getDisplay(), 0,   153, 0  );
	public static final Color COLOR_TEAL = 				new Color(GUI.getDisplay(), 0,   255, 255);
	public static final Color COLOR_LIGHT_BLUE = 		new Color(GUI.getDisplay(), 173, 216, 230);
	public static final java.awt.Color COLOR_LIGHT_BLUE_AWT = new java.awt.Color(173f/255f,216f/255f,230f/255f);
	public static final Color COLOR_LIGHT_BLUE2 = 		new Color(GUI.getDisplay(), 224, 224, 240);
	public static final Color COLOR_LIGHT_BLUE3 = 		new Color(GUI.getDisplay(), 239, 239, 248);
	public static final Color COLOR_LIGHT_BLUE4 = 		new Color(GUI.getDisplay(), 176, 176, 255);
	public static final Color COLOR_LIGHT_BLUE5 = 		new Color(GUI.getDisplay(), 22,   74, 128);
	public static final java.awt.Color COLOR_LIGHT_BLUE5_AWT = new java.awt.Color(22f/255f,74f/255f,128f/255f);
	public static final Color COLOR_LIGHT_BLUE6 = 		new Color(GUI.getDisplay(), 44,  148, 255);
	public static final java.awt.Color COLOR_LIGHT_BLUE6_AWT = new java.awt.Color(44f/255f,148f/255f,255f/255f);
	public static final Color COLOR_LIGHT_GRAY = 		new Color(GUI.getDisplay(), 235, 235, 235);
	public static final Color COLOR_LIGHT_GRAY2 = 		new Color(GUI.getDisplay(), 245, 245, 245);
	public static final Color COLOR_LIGHT_GREEN = 		new Color(GUI.getDisplay(), 200, 238, 200);
	public static final Color COLOR_LIGHT_RED = 		new Color(GUI.getDisplay(), 255, 200, 200);
	public static final Color COLOR_THICK_ORANGE =      new Color(GUI.getDisplay(), 255, 165,   0);
	public static final java.awt.Color COLOR_THICK_ORANGE_AWT =  new java.awt.Color(1f,165f/255f,0f);
	public static final Color COLOR_THICK_GREEN = 		new Color(GUI.getDisplay(),  84, 139,  84);
	public static final Color COLOR_MEDM_ORANGE =		new Color(GUI.getDisplay(), 238, 182, 43 );
	public static final Color COLOR_ORANGE = 			new Color(GUI.getDisplay(), 255, 117, 8  );
	public static final Color COLOR_PINK = 				new Color(GUI.getDisplay(), 255, 192, 203);
	public static final Color COLOR_PURPLE1 = 			new Color(GUI.getDisplay(), 255, 100, 241);
	public static final Color COLOR_PURPLE2 = 			new Color(GUI.getDisplay(), 240, 50,  225);
	public static final Color COLOR_PURPLE3 = 			new Color(GUI.getDisplay(), 223, 0,   211);
	public static final Color COLOR_RED_BUTTON = 		new Color(GUI.getDisplay(), 255, 170, 170);
	public static final Color COLOR_TITLE_BAR = 		new Color(GUI.getDisplay(), 60,	 133, 206);
	public static final Color COLOR_SAMPLE_BG = 		new Color(GUI.getDisplay(), 184, 211, 238); // #b8d3ee
	// added 12/13/17 based on most recent SSRL bluice
	// public static final Color COLOR_SSRL_BLUE_BUTTON = new Color(GUI.getDisplay(), 192, 192, 255); // #c0c0ff same as COLOR_BLUE_BUTTON
	// public static final Color COLOR_SSRL_RED_BUTTON = new Color(GUI.getDisplay(), 255, 170, 170); // #ffaaaa same as COLOR RED_BUTTON
	// public static final Color COLOR_SSRL_BG = new Color(GUI.getDisplay(), 211, 211, 211); // #d3d3d3, same as before
	public static final Color COLOR_GREEN_BUTTON = new Color(GUI.getDisplay(), 170, 255, 170);
	public static final Color COLOR_BLUE_BUTTON = COLOR_SAMPLE_BG; // current jbluice button color
	
	public static final Color COLOR_ALLCAM_BG =					COLOR_LIGHT_GRAY;
	public static final Color COLOR_ALLCAM_TAB_BG =				BGCOLOR;
	public static final Color COLOR_ALLCAM_TAB_FG =				COLOR_DARK_GRAY2;
	public static final Color COLOR_CAM_CROSSHAIR =				COLOR_RED;
	public static final java.awt.Color COLOR_CAM_CROSSHAIR_AWT = new java.awt.Color(1f, 0f, 0f, 200f/255f);
	public static final Color COLOR_CAM_BORDER =				COLOR_RED;
	public static final java.awt.Color COLOR_CAM_BORDER_AWT =	java.awt.Color.red;
	public static final Color COLOR_DEV_HEADER_BG =				COLOR_DARK_RED2;
	public static final Color COLOR_DEV_HEADER_FG =				COLOR_WHITE;
	public static final Color COLOR_DEFAULT_TEXT =				COLOR_BLACK;
	public static final Color COLOR_HUTCH_COMBO_MOVE =			COLOR_RED;
	public static final Color COLOR_MOTOR_ACTPOS_OFF =			COLOR_GRAY;
	public static final Color COLOR_MOTOR_ACTPOS_OFF_BORDER =	COLOR_DARK_GRAY;
	public static final Color COLOR_MOTOR_ACTPOS_OK =			COLOR_LIGHT_GREEN;
	public static final Color COLOR_MOTOR_ACTPOS_OK_BORDER =	COLOR_DARK_GREEN;
	public static final Color COLOR_MOTOR_ACTPOS_MOVE =			COLOR_RED;
	public static final Color COLOR_MOTOR_ACTPOS_MOVE_BORDER =	COLOR_DARK_RED;
	public static final Color COLOR_MOTOR_ACTPOS_ERR =			COLOR_ORANGE;
	public static final Color COLOR_MOTOR_ACTPOS_ERR_BORDER =	COLOR_RED;
	public static final Color COLOR_MOTOR_ACTPOS_FG =			COLOR_BLACK;
	public static final Color COLOR_PREDICT_RING_FG =			COLOR_DARK_GRAY3;
	public static final Color COLOR_PTBL_BOXOUTLINE =			COLOR_LIGHT_BLUE;
	public static final Color COLOR_PTBL_ATOMICZ_BG =			COLOR_RED_BUTTON;
	public static final Color COLOR_RASTER_ERROR =				COLOR_RED;
	public static final Color COLOR_RASTER_STATE_FG =			COLOR_DARK_GRAY4;
	public static final Color COLOR_RASTER_GRID =				COLOR_BLUE;
	public static final Color COLOR_RASTER_GRID_HLITE =			COLOR_BLUE;
	public static final Color COLOR_SCAN_BLUE_BUTTON =			COLOR_LIGHT_BLUE;
	public static final Color COLOR_SCAN_RED_BUTTON =			COLOR_RED_BUTTON;
	public static final Color COLOR_SCAN_SHADOW = 				COLOR_LIGHT_GRAY;
	public static final Color COLOR_STATUS_BG =					COLOR_LIGHT_BLUE2;
	public static final Color COLOR_STATUS_BG2 =				COLOR_LIGHT_BLUE3;
	
	private final static String fixedFont = "Bitstream Vera Sans Mono";
	
	public static final Font FONT_7_BOLD = 			new Font(GUI.getDisplay(),	"hlv", 			7,  SWT.BOLD);
	public static final Font FONT_8_FIXED =			new Font(GUI.getDisplay(),	fixedFont,		8,	SWT.NONE);
	public static final Font FONT_9_LUXI_BOLD =		new Font(GUI.getDisplay(),	"Luxi Sans",	9,	SWT.BOLD);
	public static final Font FONT_9 =	 			new Font(GUI.getDisplay(),	"hlv", 			9, 	SWT.NORMAL);
	public static final Font FONT_9_BOLD =			new Font(GUI.getDisplay(),	"hlv", 			9, 	SWT.BOLD);
	public static final java.awt.Font FONT_9_BOLD_AWT = new java.awt.Font("hlv",java.awt.Font.BOLD, 9);
	public static final Font FONT_9_FIXED =			new Font(GUI.getDisplay(),	fixedFont,		9,	SWT.NONE);
	public static final Font FONT_10_FIXED =		new Font(GUI.getDisplay(),	fixedFont,		10,	SWT.NONE);
	public static final Font FONT_11_FIXED =		new Font(GUI.getDisplay(),	fixedFont,		11,	SWT.NONE);
	public static final Font FONT_9_ARIAL_BOLD =	new Font(GUI.getDisplay(),	"Arial",		9,	SWT.BOLD);
	public static final Font FONT_10 = 				new Font(GUI.getDisplay(),	"hlv", 			10, SWT.NORMAL);
	public static final Font FONT_10_BOLD =			new Font(GUI.getDisplay(),	"hlv", 			10, SWT.BOLD);
	public static final Font FONT_11 = 				new Font(GUI.getDisplay(),	"hlv", 			11, SWT.NORMAL);
	public static final Font FONT_11_BOLD =			new Font(GUI.getDisplay(),	"hlv", 			11, SWT.BOLD);
	public static final Font FONT_11_ARIAL_BOLD =	new Font(GUI.getDisplay(),	"Arial",		11,	SWT.BOLD);
	public static final Font FONT_11_HELV_BOLD = 	new Font(GUI.getDisplay(),	"helvetica", 	11, SWT.BOLD);
	public static final Font FONT_12 = 				new Font(GUI.getDisplay(),	"hlv", 			12, SWT.NORMAL);
	public static final java.awt.Font FONT_12_AWT =	new java.awt.Font("hlv",java.awt.Font.PLAIN, 12);
	public static final Font FONT_12_BOLD = 		new Font(GUI.getDisplay(),	"hlv", 			12, SWT.BOLD);
	public static final java.awt.Font FONT_12_BOLD_AWT = new java.awt.Font("Sans",java.awt.Font.BOLD, 12);
	public static final Font FONT_12_FIXED = 		new Font(GUI.getDisplay(),	fixedFont, 		12, SWT.NORMAL);
	public static final Font FONT_12_HELV_BOLD = 	new Font(GUI.getDisplay(),	"helvetica", 	12, SWT.BOLD);
	public static final Font FONT_14 =		 		new Font(GUI.getDisplay(),	"hlv", 			14, SWT.NORMAL);
	public static final java.awt.Font FONT_14_AWT =	new java.awt.Font("hlv",java.awt.Font.PLAIN, 14);
	public static final Font FONT_14_BOLD = 		new Font(GUI.getDisplay(),	"hlv", 			14, SWT.BOLD);
	public static final Font FONT_18 = 				new Font(GUI.getDisplay(),	"hlv", 			18, SWT.NORMAL);
	public static final Font FONT_30 =				new Font(GUI.getDisplay(),	"hlv",30, SWT.BOLD);
	public static final Font FONT_80 =				new Font(GUI.getDisplay(),	"hlv",80, SWT.BOLD);
	public static final Font FONT_110 =				new Font(GUI.getDisplay(),	"hlv",110, SWT.BOLD);
	
	public static final Font FONT_CAMERA_MESSAGE =		FONT_11;
	public static final Font FONT_CAMERA_GROUP =		FONT_11_HELV_BOLD;
	public static final Font FONT_COLLECT_BUTTON = 		FONT_11;
	public static final Font FONT_COLLECT_BUTTON2 =     FONT_10_BOLD;
	public static final Font FONT_COLLECT_GROUP =		FONT_9_BOLD;
	public static final Font FONT_COLLECT_SEQUENCE = 	FONT_9_BOLD;
	public static final Font FONT_COLLECT_SITENUMBER = 	FONT_9_BOLD;
	public static final java.awt.Font FONT_COLLECT_SITENUMBER_AWT = FONT_9_BOLD_AWT;
	public static final Font FONT_COLLECT_STATUS = 		FONT_9_BOLD;
	public static final Font FONT_COLLECT_VECTORSTATE = FONT_9_BOLD;
	public static final Font FONT_COLLECTRUN_BUTTON =	FONT_9_BOLD;
	public static final Font FONT_COLLECTRUN_HEADER =	FONT_14_BOLD;
	public static final Font FONT_COLLECTRUN_LABEL =	FONT_9_BOLD;
	public static final Font FONT_COLLECTRUN_ENTRY =	FONT_12;
	public static final Font FONT_COLORSCALE =			FONT_9_BOLD;
	public static final Font FONT_DIFFIMAGE_TITLE =		FONT_11;
	public static final Font FONT_DIFFIMAGE_MSG =		FONT_11;
	public static final Font FONT_DEFAULT =				FONT_12;
	public static final java.awt.Font FONT_DEFAULT_AWT = FONT_14_AWT;
	public static final Font FONT_GRAPH_MESSAGE =		FONT_12;
	public static final Font FONT_HUTCH_GROUP =			FONT_12_HELV_BOLD;
	public static final Font FONT_HUTCH_BUTTON =		FONT_12_HELV_BOLD;
	public static final Font FONT_HUTCH_BUTTON_SMALL =	FONT_11_HELV_BOLD;
	public static final Font FONT_LARGE = 				FONT_18;
	public static final Font FONT_MEDIUM = 				FONT_14_BOLD;
	public static final Font FONT_LOG =					FONT_12;
	public static final Font FONT_MOTOR_ACTPOS =		FONT_9;
	public static final Font FONT_MOTOR_BUTTON =		FONT_9;
	public static final Font FONT_MOTOR_GROUP =			FONT_11_BOLD;
	public static final Font FONT_MOTOR_LABEL =			FONT_9;
	public static final Font FONT_MOTOR_RQSPOS =		FONT_9;
	public static final Font FONT_POSITION_DISPLAY =	FONT_9_FIXED;
	public static final Font FONT_PTBL_TINY =			FONT_7_BOLD;
	public static final Font FONT_PTBL_SMALL =			FONT_9;
	public static final Font FONT_PTBL_LABEL =			FONT_12_BOLD;
	public static final Font FONT_PTBL_HEADER =			FONT_18;
	public static final Font FONT_RASTERRESULTS_ERROR = FONT_12_BOLD;
	public static final Font FONT_RASTERRUN_HEADER = 	FONT_12_BOLD;
	public static final Font FONT_RASTERRUN_LABEL =		FONT_9_FIXED;
	public static final Font FONT_RASTER_OPTIONS =		FONT_11;
	public static final Font FONT_RESPREDICT_LABEL = 	FONT_11_BOLD;
	public static final Font FONT_RESPREDICT_LABEL_SM =	FONT_10_BOLD;
	public static final Font FONT_SAMPLE_BUTTONS = 		FONT_9_ARIAL_BOLD;
	public static final Font FONT_SAMPLE_LABEL =		FONT_9_LUXI_BOLD;
	public static final Font FONT_SAMPLE_ROTATE =		FONT_11;
	public static final Font FONT_SAMPLE_TABS = 		FONT_11_ARIAL_BOLD;
	public static final Font FONT_SCAN_AXIS_HORIZONTAL= FONT_11;
	public static final Font FONT_SCAN_AXIS_VERTICAL =	FONT_11_BOLD;
	public static final Font FONT_SCAN_CONTROL_BUTTON =	FONT_12_HELV_BOLD;
	public static final Font FONT_SCAN_BUTTON = 		FONT_11;
	public static final Font FONT_SCAN_BUTTON2 = 		FONT_12_BOLD;
	public static final Font FONT_SCAN_GROUP = 			FONT_12_BOLD;
	public static final Font FONT_SCAN_LABEL = 			FONT_11;
	public static final Font FONT_SCAN_TAB = 			FONT_14;
	public static final Font FONT_SCAN_STATE = 			FONT_12_BOLD;
	public static final Font FONT_SCAN_STEP = 			FONT_12;
	public static final Font FONT_SCAN_TEXT = 			FONT_11_BOLD;
	public static final Font FONT_SCAN_TITLE =			FONT_14_BOLD;
	public static final Font FONT_SCAN_STATUS =			FONT_9_BOLD;
	public static final Font FONT_SPLASH =				FONT_11_BOLD;
	public static final Font FONT_STATUS_DISPLAY = 		FONT_11_BOLD;
	public static final Font FONT_STATUS_LABEL = 		FONT_11;
	public static final Font FONT_STATUS_UNITS = 		FONT_11;
	public static final Font FONT_STATUS_BUTTON = 		FONT_9_BOLD;
	public static final Font FONT_USER_TABLE = 			FONT_12_FIXED;
	public static final Font FONT_USER_HEADER = 		FONT_12_BOLD;
	public static final Font FONT_SCREENING_STATUS=		FONT_10_BOLD;
	public static final Font FONT_SCRRENING_SAMPLE_30= 	FONT_30;
	public static final Font FONT_SCRRENING_SAMPLE_80= 	FONT_80;
	public static final Font FONT_SCRRENING_SAMPLE_110= FONT_110;
	
	
    public static final Image IMAGE_LOGO = 				new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/splash.gif"));
    public static final Image IMAGE_STOP =				new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/close.png"));
	public static final Image IMAGE_TEXT =				new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/text.png"));
	public static final Image IMAGE_FOLDER =			new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/folder.png"));
    
    //Screening Tab Images
    public static final Image image = 						new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/go1.png"));
	public static final Image imageblue = 					new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/bluebutton.png"));
	public static final Image imagegreen = 					new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/greenbutton.png"));
    public static final Image imagered = 					new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/redbutton.png"));
    public static final Image imagearrow = 					new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/gored.png"));
    public static final Image imageedit = 					new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/edit.png")); 
    public static final Image IMAGE_ENTER = 				new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/enter.png")); 
    public static final Image IMAGE_OPTIMIZE = 				new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/optimize.png"));
	public static final Image IMAGE_STRATEGYSCORE = 		new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/strategyscore.png")); 
	public static final Image STOP_SIGN = 					new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/stopsign.png"));
	public static final Image IMAGE_SPREADSHEET =			new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/spreadsheet29g.png"));
	public static final Image IMAGE_DIFFRACTION =			new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/diff_icon1.png"));
	public static final Image IMAGE_KEYBOARD =				new Image (GUI.getDisplay(), Resources.get().getInputStream("/images/keyboard3.png"));
	public static final Image pauseImage = 					new Image(GUI.getDisplay(),  Resources.get().getInputStream("/images/pausebutton.png"));
	public static final Image readyImage = 					new Image(GUI.getDisplay(),  Resources.get().getInputStream("/images/bluebutton1.png"));
	
	public static HashMap<String, Tab> tabMap = new HashMap<String, Tab>();

	protected WdgMain wdgMain = null;
	private static GUI inst = null;
	public static GUI get() { return inst; }
	
	public GUI()
	{
		//for (FontData fd : GUI.getDisplay().getFontList(null, true)) System.out.println(fd);
		wdgMain = new WdgMain();
		
		// connect the AppState listeners here - all the models should be created, so the
		// states will be correct on first notification
		AppState.init();
		
		inst = this;
	}

	public static UncaughtExceptionHandler eHandler = new UncaughtExceptionHandler()
	{
		public void uncaughtException(Thread t, Throwable e)
		{
			System.out.println("Caught exception "+e.getMessage());
			e.printStackTrace();
		}
	};
	
	public static void processMessages(int maxMessages)
	{
		int processed = 0;
		try
		{
			while (processed < maxMessages)
			{
				Epics.poll();
				if (!readAndDispatch()) return;
				processed++;
			}
		}
		catch (IllegalStateException e) {}
	}
	
	public static void processMessages(double maxSec)
	{
		long maxMs = (int)(maxSec*1000.0);
		
		long start = System.currentTimeMillis();
		try
		{
			while (readAndDispatch())
			{
				Epics.poll();
				long now = System.currentTimeMillis();
				if (now-start>maxMs) return;
			}
		}
		catch (IllegalStateException e) {}
	}
	
	protected volatile 			boolean 	messagePumpExited 		= false;
	
	public static void setFontSize(final int size)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			Log.note("Changing font size to "+size+"...");
			
			Font[] fonts = new Font[]{
			FONT_7_BOLD,FONT_8_FIXED,FONT_9_LUXI_BOLD,FONT_9,	
			FONT_9_BOLD,FONT_9_FIXED,FONT_9_ARIAL_BOLD,FONT_10, 			
			FONT_10_BOLD,FONT_11,FONT_11_BOLD,FONT_11_ARIAL_BOLD, 
			FONT_11_HELV_BOLD,FONT_12,FONT_12_BOLD,FONT_12_FIXED, 	
			FONT_12_HELV_BOLD,FONT_14,FONT_14_BOLD,FONT_18, 			
			FONT_80,FONT_110};
			
			/*for (Font font : fonts)
			{
				FontData[] fda = font.getFontData();
				for (FontData fd : fda)
				{
					fd.setHeight(size);
				}
			}*/
			
			ArrayList<Control> level = new ArrayList<Control>();
			level.add(shell);
			do
			{
				ArrayList<Control> nextLevel = new ArrayList<Control>();
				
				for (Control control : level)
				{
					/*FontData[] fda = control.getFont().getFontData();
					for (FontData fd : fda)
					{
						fd.setHeight(size);
					}*/
					control.setFont(GUI.FONT_7_BOLD);
					
					if (control instanceof Composite)
					{
						for (Control childControl : ((Composite)control).getChildren())
						{
							nextLevel.add(childControl);
						}
						((Composite)control).layout(true);
					}
					else
					{
						((Control)control).computeSize(1,1);
					}
					
				}
				
				level = nextLevel;
			}
			while (level.size()>0);
			
			Log.note("Font size changed to "+size+".");
		}});
	}
	
	public void mainLoop()
	{
		Thread.setDefaultUncaughtExceptionHandler(eHandler);

		while (!shell.isDisposed())
		{
			if (!readAndDispatch())
			{
				getDisplay().sleep();
			}
			if (null!=monitor) monitor.hangDetected = false;
		}
		messagePumpExited = true;
		
		getDisplay().dispose();

		appState = STOPPING;
	}
	
	public static boolean readAndDispatch()
	{
		// if the display event causes an exception, there was an event, so readAndDispatch() would return true
		boolean ret = true; 

		try
		{
			GUIThreadMonitor.handleRunnableStart();
			ret = getDisplay().readAndDispatch();
			GUIThreadMonitor.handleRunnableEnd();
		}
		catch (SWTException se)
		{
			if (-1 != se.getMessage().indexOf("Widget is disposed"))
			{
				System.out.println("Widget disposed event detected, silently printing to log.");
				se.printStackTrace();
			}
			else
			{
				throw se;
			}
		}
		catch (Throwable t)
		{
			t.printStackTrace();
		}
		return ret;
	}
	
	public static void asyncExec(final Runnable runnable)
	{
		final String trace = GUIThreadMonitor.logCall();
		getDisplay().asyncExec(new Runnable(){public void run()
		{
			GUIThreadMonitor.currentTrace = trace;
			try
			{
				runnable.run();
			}
			catch (Throwable t) { t.printStackTrace(); }
			GUIThreadMonitor.handleAppRunnableEnd();
		}});
		//display.asyncExec(runnable);
	}
}
