package gov.anl.gmca.gui;

import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList.CollectState;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.ui.WdgAppState;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.RasterState;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.AutoStep;
import gov.anl.gmca.scan.ModelScan.InterStep;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;

// tabs are created before this widget is created, so if they don't exist at this point
// they can be ignored
public class WdgCmdStatus extends WdgStatusElement
{
	public WdgCmdStatus(Composite parent, String labelName, String initialStatus, int statusWidth)
	{
		super(parent, labelName, null, initialStatus, statusWidth, false, null, null, null);
		
		status.setBackground(GUI.COLOR_STATUS_BG);
		
		AppState.stateLL.addListener(new ListenerList.Listener<AppState>()
		{
			@Override public void handleEvent(AppState object, Object data)
			{
				updateStatus();
			}
		});
		updateStatus();
		status.setText("---");
		
		// open read-only appstate dialog on click
		status.addMouseListener(new MouseListener()
		{
			private boolean mouseClicked=false;
			
			@Override public void mouseUp(MouseEvent e)
			{
				if (mouseClicked && e.x>=0 && e.y>=0 && e.x<status.getSize().x && e.y<status.getSize().y)
				{
					WdgAppState.open(true);
				}
				mouseClicked=false;
			}
			
			@Override public void mouseDown(MouseEvent e)
			{
				mouseClicked=true;
			}
			@Override public void mouseDoubleClick(MouseEvent e) {}
		});
	}
	
	protected void updateStatus()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
    		InterStep 		interStep		= null;
    		AutoStep 		autoStep		= null;
    		RasterState 	rasterState		= null;
    		CollectState 	collectState	= null;
    		String			statusText		= "Idle";
    		
    		// get the states from the different tabs
    		if (null != ModelScan.get())
    		{
    			interStep 	= ModelScan.get().getInterStep();
    			autoStep 	= ModelScan.get().getAutoStep();
    		}
    		if (null != ModelRasterState.get())
    		{
    			rasterState = ModelRasterState.get().getRasterState();
    		}
    		if (null != ModelCollectRunList.get())
    		{
    			collectState = ModelCollectRunList.get().getCollectState();
    		}
    
    		// set the text based on the states
    		int numActiveStates = 0;
    		if (null != autoStep && autoStep != AutoStep.NONE)
    		{
    			numActiveStates++;
    			switch (autoStep)
    			{
    			case EDGE_SCAN: 	statusText="Edge scan";		break;
    			case MOVE_ENERGY: 	statusText="Moving energy";	break;
    			case OPTIMIZE:		statusText="Opt. detector";	break;
    			case RETRACT:		statusText="Scan cleanup";	break;
    			case SPECTRUM:		statusText="Fl. spectrum";	break;
    			case STOPPING:		statusText="Stopping";		break;
    			default:			break;
    			}
    		}
    		else if (null != interStep && interStep != InterStep.IDLE)
    		{
    			numActiveStates++;
    			switch (interStep)
    			{
    			case CLEANUP:  		statusText="Scan cleanup";	break;
    			case EDGE:  		statusText="Edge scan";		break;
    			case OPT_DET:  		statusText="Opt. detector";	break;
    			case OPT_MCA:  		statusText="Opt. MCA";		break;
    			case SPECTRUM:  	statusText="Fl. spectrum";	break;
    			case STOPPING:  	statusText="Stopping";		break;
    			default:			break;
    			}
    		}
    		
    		if (null != rasterState && rasterState != RasterState.INACTIVE)
    		{
    			numActiveStates++;
    			switch (rasterState)
    			{
    			case RASTERING: 	statusText="Rastering";		break;
    			case STOPPING:  	statusText="Stopping";		break;
    			default:			break;
    			}
    		}
    		if (null != collectState && collectState != CollectState.INACTIVE)
    		{
    			numActiveStates++;
    			switch (collectState)
    			{
    			case COLLECTING: 	statusText="Collecting";	break;
    			case PAUSING: 		statusText="Stopping";		break;
    			default:			break;
    			}
    		}
    		
    		// print an error if multiple states are active
    		if (numActiveStates>1)
    		{
    			statusText = "ERROR";
    		}
    
    		// set the widget text
    		status.setText(statusText);
		}});
	}
}
