package gov.anl.gmca.gui;

import gov.anl.gmca.common.DevMoveFloat;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;

public class WdgEnergyStatus extends WdgStatusElement
{
	protected static enum UnitType
	{
		KEV("keV"),
		EV("eV"),
		A("\u212B");
		
		protected String text;
		UnitType(String text) { this.text=text; }
		@Override public String toString() { return text; }
	};
	
	protected UnitType currentUnits = UnitType.KEV;
	
	public WdgEnergyStatus(Composite parent, String labelName, String initialStatus, int statusWidth)
	{
		super(parent,
			  labelName,
			  null,
			  initialStatus,
			  statusWidth,
			  false,
			  null,
	  		  new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { return false; }
					@Override public String getString(Object value) { return ""; }
		      },
		      null);

		this.interpretStatus = new InterpretStatus() {
			@Override public boolean isValOK(Object value)  { return false; /* value is red always */ }
			@Override public String getString(Object value) { return generateString((Double)value); }
		};
		
		status.addMouseListener(new MouseListener()
		{
			@Override public void mouseDown(MouseEvent me)
			{
				  changeUnits();
			}

			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent arg0) {}
		});
		
		DevPmacMotor.get("mo_En_E").addListener(new DevMoveFloat.PositionListener()
		{
			@Override public void handleActPosChange(DevMoveFloat dev, double newPos)
			{
				setValue(newPos);
			}
		});
		setValue(DevPmacMotor.get("mo_En_E").getPosition());
	}
	
	protected void changeUnits()
	{
		switch (currentUnits)
		{
		case KEV:
			currentUnits = UnitType.EV;
			break;
		case EV:
			currentUnits = UnitType.A;
			break;
		case A:
			currentUnits = UnitType.KEV;
			break;
		}
		
		updateStatus();
	}
	
	protected String generateString(double keV)
	{
		switch (currentUnits)
		{
		case KEV:
			return Util.sprintf("%.3f ",keV)+currentUnits;
		case EV:
			return Util.sprintf("%d ",Math.round(1000.0*keV))+currentUnits;
		case A:
			return Util.sprintf("%.3f ", 12.398/keV)+currentUnits;
		default:
			return "";
		}
	}
}
