package gov.anl.gmca.gui.options;

import gov.anl.gmca.collect.viewer.ExternalViewer;
import gov.anl.gmca.collect.viewer.ImgSrv;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.*;
import gov.anl.gmca.util.ListenerList;
import gov.aps.jca.dbr.DBR;

public class DevUserConfig
{
	public static enum StrategyProgram { BEST, MOSFLM };
	public static enum ImageFormat { HDF_ONLY, CBF_ONLY, HDF_PRIMARY, CBF_PRIMARY };
	public static enum HDFCleanupMode { CLEAN_ALL, CLEAN_NON_COLLECT, CLEAN_NONE };
	
	public static boolean clearRasterRunsScr = true;
	public static String strategyProgram = "best";
	public static int processingPipeline = 0;
	
	public ListenerList<DevUserConfig> ll;
	
	public static DevUserConfig get() { return inst; }
	private static DevUserConfig inst;
	
	public DevUserConfig()
	{
		inst = this;
		ll = new ListenerList<DevUserConfig>(this);
		
		Epics.addMonitor(PV_R_RUNS.scrClear, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				clearRasterRunsScr = Epics.dbrToBoolean(value);
				ll.notifyListeners(clearRasterRunsScr);
			}
		});
		
		Epics.addMonitor(PV_RUNS.enableShutterless, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				ll.notifyListeners();
			}
		});
		
		Epics.addMonitor(PV_BLUICE.displayProgram, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				ll.notifyListeners();
			}
		});

		Epics.addMonitor(PV_BLUICE.imageServer, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				ll.notifyListeners();
			}
		});

		Epics.addMonitor(PV_SCR.strategyProgram, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				boolean strPro = Epics.dbrToBoolean(value);
				
				if(strPro)
					strategyProgram = "mosflm";
				else
					strategyProgram = "best";
				
				ll.notifyListeners(strPro);	
			}
		});
		
		Epics.addMonitor(PV_RUNS.bgProc, new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				processingPipeline = Epics.dbrToInt(value);
				ll.notifyListeners(processingPipeline);
		}});
	}
	
	public boolean 			getClearOnUnmount()							{ return Epics.getBoolean(PV_R_RUNS.scrClear);	 }
	public int 				getXDSProc()								{ return Epics.getInteger(PV_RUNS.bgProc); }
	public StrategyProgram 	getStrategyProgram()						{ return StrategyProgram.values()[Epics.getInteger(PV_SCR.strategyProgram)]; }
	public int 				getScalingSoftware()						{ return Epics.getInteger(PV_SCR.scalingSoftware);}
	public boolean			getShutterlessEnable()						{ return Epics.getBoolean(PV_RUNS.enableShutterless); }
	public ExternalViewer	getDisplayProgram()
	{
		try {
			return ExternalViewer.get(Epics.getInteger(PV_BLUICE.displayProgram));
		}
		catch (Exception e) {
			e.printStackTrace();
			return ExternalViewer.get(0);
		}
	}
	public ImgSrv	getImgSrv()
	{
		try {
			return ImgSrv.get(1);
		}
		catch (Exception e) {
			e.printStackTrace();
			return ImgSrv.get(0);
		}
	}
	public ImageFormat		getImageFormat()							{ return ImageFormat.CBF_PRIMARY; } // ImageFormat.values()[Epics.getInteger(PV_BLUICE.imageFormat)]; }
	public HDFCleanupMode	getHDFCleanupMode()							{ return HDFCleanupMode.values()[Epics.getInteger(PV_BLUICE.hdfCleanupMode)]; }
	
	public void 			setXDSProcessing(int enable)				{ Epics.putInteger(PV_RUNS.bgProc, enable); }
	public void 			setScalingSoftware(int i) 					{ Epics.putInteger(PV_SCR.scalingSoftware, i);}
	public void 			setStrategyProgram(StrategyProgram program) { Epics.putInteger(PV_SCR.strategyProgram, program.ordinal()); }
	public void 			setClearOnUnmount(boolean clear)			{ Epics.putInteger(PV_R_RUNS.scrClear, clear?1:0); }
	public void				setShutterlessEnable(boolean enable)		{ Epics.putInteger(PV_RUNS.enableShutterless, enable?1:0); }
	public void				setImageFormat(ImageFormat format)			{ Epics.putInteger(PV_BLUICE.imageFormat, format.ordinal()); }
	public void				setHDFCleanupMode(HDFCleanupMode mode)		{ Epics.putInteger(PV_BLUICE.hdfCleanupMode, mode.ordinal()); }
	public void				setDisplayProgram(int index)
	{
		if (getDisplayProgram().index != index)
			Epics.putInteger(PV_BLUICE.displayProgram, index);
	}
	public void				setImgSrv(int index)
	{
		if (getImgSrv().index != index)
			Epics.putInteger(PV_BLUICE.imageServer, index);
	}

	public void getMultiCrystalStrategy() { Epics.getInteger(PV_STRATEGY.showMultiStrategy); }
	public void setMultiCrystalStategy(int i) {	Epics.putInteger(PV_STRATEGY.showMultiStrategy, i);}
}
