package gov.anl.gmca.gui.options;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ui.WdgEnableDisable;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.util.ListenerList;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgCollectSetting
{
	private PVName pvName;
	private String dbKey;
	private String text;
	private Composite parent;
	WdgEnableDisable toggleWidget;
	private ListenerList.Listener<DBConfig> dbListener = null;
	private Epics.EpicsValueMonitor epicsListener = null;
	
	public WdgCollectSetting(Composite parent, String text, PVName pvName, String dbKey)
	{
		this.pvName = pvName;
		this.dbKey = dbKey;
		this.text = text;
		this.parent = parent;
		
		// left spacer
		WdgStaffOptions.addLeftSpacer(parent);
		
		// field label
		Label lblKey = new Label(parent, SWT.LEFT);
		GridData gd = new GridData();
		lblKey.setLayoutData(gd);
		lblKey.setText(text);

		// field control widget
		toggleWidget = new WdgEnableDisable(parent, "Enable", "Disable");
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.widthHint = 200;
		toggleWidget.setLayoutData(gd);

		// right spacer
		WdgStaffOptions.addRightSpacer(parent);
		
		connectListeners();
	}
	
	private void connectListeners()
	{
		// listen to the PV or DB key for changes
		if (null != pvName)
		{
			epicsListener = new Epics.EpicsValueMonitor()
			{
				@Override public void valueChanged(String pvName, final DBR value)
				{
					GUI.asyncExec(new Runnable(){public void run()
					{
						toggleWidget.setSelection(Epics.dbrToBoolean(value));
					}});
				}
			};
			Epics.addMonitor(pvName, epicsListener);
			toggleWidget.setSelection(Epics.getBoolean(pvName));
		}
		else if (null != dbKey)
		{
			dbListener = new ListenerList.Listener<DBConfig>()
			{
				@Override public void handleEvent(DBConfig object, Object data)
				{
					GUI.asyncExec(new Runnable(){public void run()
					{
						toggleWidget.setSelection(DBConfig.getBoolean(dbKey));
					}});
				}
			};
			DBConfig.llValueChange.addListener(dbListener);
			toggleWidget.setSelection(DBConfig.getBoolean(dbKey));
		}
		else throw new IllegalStateException("No PV or DB key given for the collect setting: "+text);

		// listen to the widget for changes to pass back to EPICS or SQL
		toggleWidget.llUserSelection.addListener(new ListenerList.Listener<WdgEnableDisable>()
		{
			@Override public void handleEvent(WdgEnableDisable object, Object data)
			{
				if (null != pvName) Epics.putInteger(pvName, toggleWidget.getSelection()?1:0);
				if (null != dbKey)  DBConfig.set(dbKey, toggleWidget.getSelection()?"1":"0");
			}
		});
		
		parent.addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent arg0)
			{
				removeListeners();
			}
		});
	}
	
	private void removeListeners()
	{
		if (null != dbKey)  DBConfig.llValueChange.removeListener(dbListener);
		if (null != pvName) Epics.removeMonitor(pvName, epicsListener);
	}
}
