package gov.anl.gmca.gui.options;

import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.ui.NBDirectoryDialog;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Disk.File;
import gov.anl.gmca.io.Disk.IOException;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class WdgCommonDir
{
	private Label label;
	private Text text;
	private Button button;
	private Composite parent;
	
	public WdgCommonDir(Composite parent, String title)
	{
		this.parent=parent;
		
		GridData gd;
		
		label = new Label(parent, SWT.NONE);
		label.setText(title);
		label.setFont(GUI.FONT_10_BOLD);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
		label.setLayoutData(gd);
		
		Composite comp = new Composite(parent, SWT.NONE);
		GridLayout gl = new GridLayout(1,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		comp.setLayout(gl);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
		comp.setLayoutData(gd);
		
		text = new Text(comp, SWT.BORDER | SWT.READ_ONLY);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 1, 1);
		text.setLayoutData(gd);
		text.setToolTipText("Data directory is not modifiable");
		
		/*button = new Button(comp, SWT.PUSH);
		button.setText("Select");
		gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false, 1, 1);
		button.setLayoutData(gd);*/
		
		Composite spacer = new Composite(parent, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
		gd.heightHint = 10;
		spacer.setLayoutData(gd);
		
		connectListeners();
	}
	
	private void connectListeners()
	{
		// don't allow focus - this prevents the cursor from showing up in the field
		if (null!=button) text.addFocusListener(new FocusListener()
		{
			@Override public void focusLost(FocusEvent fe) {}
			@Override public void focusGained(FocusEvent arg0)
			{
				button.setFocus();
			}
		});
		
		if (null!=button) button.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				NBDirectoryDialog dd = new NBDirectoryDialog(WdgControlPanel.get().getShell());
				dd.setFilterPath(DevDataDir.getPrimary());
				String newDir = dd.openStr();
				if (null != newDir)
				{
					File file = new File(newDir);
					try
					{
						if (!file.canRead())
						{
							Util.popupError("Invalid path","The selected directory is not readable:\n\n"+newDir);
							return;
						}
						if (!file.canWrite() || !file.canExec())
						{
							Util.popupError("Invalid path","The selected directory is not writable:\n\n"+newDir);
							return;
						}
						DevDataDir.set(file.getCanonicalPath());
					}
					catch (IOException ioe) { ioe.printStackTrace(); }
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		final ClientListener clientListener = new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				update();
			}
		};
		Client.addListener(clientListener);
		
		final Epics.EpicsValueMonitor epicsMonitor = new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				update();
			}
		};
		Epics.addMonitor(PV_BLUICE.data_dir,epicsMonitor);
		
		parent.addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent arg0)
			{
				Client.removeListener(clientListener);
				Epics.removeMonitor(PV_BLUICE.data_dir, epicsMonitor);
			}
		});
	}
	
	private void update()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			text.setEnabled(Client.isMaster());
			text.setText(Epics.getString(PV_BLUICE.data_dir));
			if (null!=button) button.setEnabled(Client.isMaster());
		}});
	}
}
