package gov.anl.gmca.gui.options;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.multistrategy.ModelMultiStrategyList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class WdgControlPanel
{
	private static WdgControlPanel inst = null;
	public static WdgControlPanel get() { if (null==inst) inst=new WdgControlPanel(); return inst; }
	public Shell getShell() { return shell; }
	
	Shell shell;
	WdgUserOptions userOptions;
	WdgUserActions userActions;
	WdgStaffOptions staffControls;
	Label bottomLabel;
	
	public WdgControlPanel() {}
	
	public void open()
	{
		if (null!=shell && !shell.isDisposed())
		{
			shell.moveAbove(GUI.getShell());
			shell.setFocus();
			return;
		}
		
		shell = new Shell(GUI.getDisplay());
		shell.setText("JBluIce-EPICS Control Panel");
		//shell.setBackground(GUI.BGCOLOR);
		
		createWidgets();
		
		int centerX = GUI.getShell().getLocation().x + GUI.getShell().getSize().x/2;
		int centerY = GUI.getShell().getLocation().y + GUI.getShell().getSize().y/2;
		
		int optionsWidth = 630;
		int optionsHeight = 750;
		
		int x = centerX - optionsWidth/2;
		int y = centerY - optionsHeight/2;
		
		shell.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent e)
			{
				if (e.detail == SWT.TRAVERSE_ESCAPE)
				{
					shell.close();
					shell.dispose();
				}
			}
		});
		
		shell.pack();
		shell.setLocation(x,y);
		shell.setSize(optionsWidth,optionsHeight);
		shell.setVisible(false);
		shell.open();
	}
	
	private void createWidgets()
	{
		GridData gd;

		// folder
		
		shell.setLayout(new GridLayout(2,false));
		final TabFolder tabFolder = new TabFolder(shell, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 2;
		tabFolder.setLayoutData(gd);
		tabFolder.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				if (tabFolder.getSelectionIndex()==2)
				{
					staffControls.setFocus();
					ModelMultiStrategyList.get().ll.notifyListeners();
					bottomLabel.setText("Press enter to apply text field changes");
				}
				else
				{
					bottomLabel.setText("Changes are applied instantly");
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		// user options
		TabItem userItem = new TabItem(tabFolder, SWT.NONE);
		userItem.setText("Options");
		userOptions = new WdgUserOptions(tabFolder);
		userItem.setControl(userOptions);
		
		// user actions
		TabItem actionItem = new TabItem(tabFolder, SWT.NONE);
		actionItem.setText("Actions");
		userActions = new WdgUserActions(tabFolder);
		actionItem.setControl(userActions);
		
		// staff controls
		TabItem staffItem = new TabItem(tabFolder, SWT.NONE);
		staffItem.setText("Staff");
		staffControls = new WdgStaffOptions(shell, tabFolder);
		staffItem.setControl(staffControls);
		
		// common label/button at bottom
		bottomLabel = new Label(shell, SWT.NONE);
		bottomLabel.setText("Changes are applied instantly");
		bottomLabel.setForeground(GUI.COLOR_DARK_GRAY);
		gd = new GridData(SWT.LEFT, SWT.CENTER, true, false);
		bottomLabel.setLayoutData(gd);

		Button closeButton = new Button(shell, SWT.PUSH);
		closeButton.setText("Close");
		gd = new GridData(SWT.RIGHT, SWT.DEFAULT, true, false);
		//gd.horizontalSpan = 2;
		gd.widthHint = 100;
		closeButton.setLayoutData(gd);
		closeButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				shell.close();
				shell.dispose();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	}
}
