package gov.anl.gmca.gui.options;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ui.WdgAppState;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.syslog.WdgLog;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;

public class WdgStaffOptions extends Composite
{
	private Text staffPwdEntry;
	private Composite beforePwdComp;
	private Composite afterPwdComp;
	private StackLayout staffStackLayout;
	private Shell cplShell;
	private WdgControlOption optMultiStrategy;
	WdgLog dbLog;
	private static WdgStaffOptions inst;
	public static WdgStaffOptions get() { return inst; }
	
	public WdgStaffOptions(Shell cplShell, TabFolder parent)
	{
		super(parent, SWT.NONE);
		inst=this;
		this.cplShell = cplShell;

		// stack layout to switch between controls
		staffStackLayout = new StackLayout();
		setLayout(staffStackLayout);

		// controls
		createBeforePwd();
		createAfterPwd();
		
		// set visible control to beforePwd
		staffStackLayout.topControl = beforePwdComp;
		layout();
	}

	private void createBeforePwd()
	{
		beforePwdComp = new Composite(this, SWT.NONE);
		
		GridLayout gl = new GridLayout(1,false);
		gl.verticalSpacing = 0;
		beforePwdComp.setLayout(gl);
		
		Composite spacer = new Composite(beforePwdComp, SWT.NONE);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		spacer.setLayoutData(gd);
		
		Composite pwdComp = new Composite(beforePwdComp, SWT.NONE);
		pwdComp.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
		pwdComp.setLayout(new GridLayout(1,false));
		
		Label label = new Label(pwdComp, SWT.CENTER);
		label.setText("Enter staff password:");
		label.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
		
		staffPwdEntry = new Text(pwdComp, SWT.BORDER|SWT.PASSWORD);
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		gd.widthHint = 100;
		staffPwdEntry.setLayoutData(gd);
		staffPwdEntry.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent e)
			{
				if (e.detail==SWT.TRAVERSE_RETURN)
				{
					handlePassword(staffPwdEntry.getText());
				}
				else if (e.detail==SWT.TRAVERSE_ESCAPE)
				{
					cplShell.close();
					cplShell.dispose();
				}
			}
		});
		
		Button button = new Button(pwdComp, SWT.CENTER|SWT.PUSH);
		button.setText("Submit");
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		gd.widthHint = 100;
		button.setLayoutData(gd);
		button.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handlePassword(staffPwdEntry.getText());
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		Composite spacer2 = new Composite(beforePwdComp, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		spacer2.setLayoutData(gd);
	}

	private void createAfterPwd()
	{
		// comp for the entire staff controls tab
		afterPwdComp = new Composite(this, SWT.NONE);
		afterPwdComp.setBackground(GUI.COLOR_BLACK);
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.verticalSpacing=2;
		afterPwdComp.setLayout(gl);
		
		// comp including the scrollbar
		ScrolledComposite scrolledComp = new ScrolledComposite(afterPwdComp, SWT.V_SCROLL);
		scrolledComp.setLayout(new FillLayout());
		scrolledComp.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		// comp for all scrollable contents
		Composite scrolledContents = new Composite(scrolledComp, SWT.NONE);
		scrolledContents.setBackground(GUI.COLOR_BLACK);
		gl = new GridLayout(1,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.verticalSpacing=2;
		scrolledContents.setLayout(gl);
		
		// comp to hold all entries
		Composite entryComp = new Composite(scrolledContents, SWT.NONE);
		entryComp.setLayout(new GridLayout(5,false));
		entryComp.setLayoutData(new GridData(GridData.FILL_BOTH));

		// non-DB widgets
		addMultiStrategyWidget(entryComp);
		addAppStateButton(entryComp);
		
		// first divider
		Composite divider = new Composite(entryComp, SWT.NONE);
		divider.setBackground(GUI.COLOR_BLACK);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 2;
		gd.horizontalSpan=5;
		divider.setLayoutData(gd);
		
		addCollectSettings(entryComp);
		
		// second divider
		Composite divider2 = new Composite(entryComp, SWT.NONE);
		divider2.setBackground(GUI.COLOR_BLACK);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 2;
		gd.horizontalSpan=5;
		divider2.setLayoutData(gd);
		
		// DB widgets
		for (String key : DBConfig.getKeyListCopy())
		{
			if (DBConfig.getFormat(key).type!=DBConfig.Type.HIDE)
			{
				new WdgDBEntry(entryComp, key);
			}
		}
		
		// mini log window for staff controls to track changes
		dbLog = new WdgLog(afterPwdComp, SWT.NONE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 60;
		dbLog.setLayoutData(gd);
		if (DBConfig.isConnected())
			dbLog.println("Connected to MySQL.", Log.STYLE_NOTE);
		else
			dbLog.println("Couldn't connect to MySQL", Log.STYLE_ERROR);
		
		// attach contents composite to the scroll bar
		Point prefSize = scrolledContents.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		scrolledComp.setMinSize(prefSize);
		scrolledComp.setContent(scrolledContents);
		scrolledComp.setExpandHorizontal(true);
		scrolledComp.setExpandVertical(true);
	}
	
	static Composite addLeftSpacer(Composite comp)
	{
		Composite spacer = new Composite(comp, SWT.NONE);
		GridData gd = new GridData(SWT.DEFAULT, SWT.FILL, false, true);
		gd.widthHint = 45;
		gd.heightHint = 10;
		gd.horizontalSpan = 2;
		spacer.setLayoutData(gd);
		return spacer;
	}
	
	static void addRightSpacer(Composite comp)
	{
		Composite spacer2 = new Composite(comp, SWT.NONE);
		GridData gd = new GridData();
		gd.widthHint = 45;
		gd.heightHint = 1;
		spacer2.setLayoutData(gd);
	}
	
	private void addAppStateButton(Composite comp)
	{
		addLeftSpacer(comp);
		
		Label label = new Label(comp, SWT.NONE);
		label.setText("Application State");
		GridData gd = new GridData();
		label.setLayoutData(gd);
		
		Button button = new Button(comp, SWT.PUSH);
		button.setLayoutData(new GridData(SWT.LEFT, SWT.DEFAULT, true, false));
		button.setText("Open viewer");
		button.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					WdgAppState.open(false);
				}});
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		addRightSpacer(comp);
	}
	
	private void addMultiStrategyWidget(Composite comp)
	{
		//addLeftSpacer(comp);
		
		// Multi strategy tab on collect
		/*optMultiStrategy = new WdgControlOption
		(
			comp,
			"Multi Crystal Strategy",
			new String[]{"Display","Hide"},
			new OptionHandler()
			{
				@Override public void handleSelection(int i)
				{
					DevUserConfig.get().setMultiCrystalStategy(i);
				}
			},
			false
		);*/
		
		//addRightSpacer(comp);
		
		/*ModelMultiStrategyList.get().ll.addListener(new ListenerList.Listener<ModelMultiStrategyList>()
		{
			@Override public void handleEvent(ModelMultiStrategyList object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					optMultiStrategy.select(ModelMultiStrategyList.get().getMultiStrategy());
				}});	
			}
		});*/
	}
	
	private void addCollectSettings(Composite parent)
	{
		// individual settings
		final ArrayList<WdgCollectSetting> wcsList = new ArrayList<WdgCollectSetting>();
		wcsList.add(new WdgCollectSetting(parent, "Endstation shutter", 			PV_BLUICE.enableEndShutterCheck, 	null));
		wcsList.add(new WdgCollectSetting(parent, "Wait for feedback", 				PV_BLUICE.waitForFeedback, 			null));
		wcsList.add(new WdgCollectSetting(parent, "Undulator offset", 				PV_BLUICE.enableEnergyGapCheck, 	null));
		wcsList.add(new WdgCollectSetting(parent, "Mono position", 					PV_BLUICE.enableEnergyMove, 		null));
		wcsList.add(new WdgCollectSetting(parent, "CCD/PAD detector distance", 		PV_BLUICE.enableDetectorMove, 		null));
		wcsList.add(new WdgCollectSetting(parent, "Sample environment positions",	PV_BLUICE.enableSampleEnvMove,		null));
		wcsList.add(new WdgCollectSetting(parent, "Cryojet monitor",				PV_BLUICE.enableCryojetMonitor,		null));
	}
	
	@Override public boolean setFocus()
	{
		return staffPwdEntry.setFocus();
	}
	
	private void handlePassword(String pwd)
	{
		boolean passwordOK = Util.checkPassword("gmca", pwd);
		if (!passwordOK)
		{
			Log.error("Invalid password");
			staffPwdEntry.setText("");
			return;
		}
		
		staffStackLayout.topControl = afterPwdComp;
		layout();		
	}
}
