package gov.anl.gmca.gui.options;

import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.raster.util.ClearSampleState;
import gov.anl.gmca.strategy.ModelStrategyResultList;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;

public class WdgUserActions extends Composite
{
	private WdgControlAction actionUndo, actionRedo, actionClearRobot, actionClearStrategy, actionClearState;
	private WdgControlAction actionClearMultiStartegy;
	private ClientListener clientListener;
	
	public WdgUserActions(TabFolder parent)
	{
		super(parent, SWT.NONE);
		
		GridLayout gl = new GridLayout(2, false);
		setLayout(gl);
		setBackgroundMode(SWT.INHERIT_FORCE);
		
		// user controls
		Composite controlsComp = new Composite(this, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.widthHint = 350;
		controlsComp.setLayoutData(gd);

		gl = new GridLayout(1,false);
		gl.verticalSpacing = 0;
		controlsComp.setLayout(gl);
		createControls(controlsComp);
		
		// listeners
		connectListeners();
	}
	
	private void createControls(Composite parent)
	{
		// title
		Label controlsTitle = new Label(parent, SWT.CENTER);
		controlsTitle.setText("Controls");
		controlsTitle.setFont(GUI.FONT_14_BOLD);
		controlsTitle.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Composite spacer = new Composite(parent, SWT.NONE);
		GridData gd = new GridData();
		gd.heightHint = 20;
		spacer.setLayoutData(gd);
		
		// p+c undo/redo
		Label pncLabel = new Label(parent, SWT.LEFT);
		pncLabel.setText("Point && Click");
		pncLabel.setFont(GUI.FONT_10_BOLD);
		pncLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		actionUndo = new WdgControlAction
		(
			parent,
			"Undo",
			new Runnable(){public void run()
			{
				Epics.put(PV_BLUICE.pointclick, "undo");
			}}
		);
		actionRedo = new WdgControlAction
		(
			parent,
			"Redo",
			new Runnable(){public void run()
			{
				Epics.put(PV_BLUICE.pointclick, "redo");
			}}
		);
		
		Composite spacer2 = new Composite(parent, SWT.NONE);
		gd = new GridData();
		gd.heightHint = 20;
		spacer2.setLayoutData(gd);
		
		// clear buttons
		Label clearLabel = new Label(parent, SWT.LEFT);
		clearLabel.setText("Clear operations");
		clearLabel.setFont(GUI.FONT_10_BOLD);
		clearLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		// clear loaded sample
		actionClearRobot = new WdgControlAction
		(
			parent,
			"Clear mounted sample",
			new Runnable(){public void run()
			{
				if(!Client.isMaster()) { Log.warn("This client is not master");return;}
				if(Util.popupOkCancel("Clear the mounted sample", "ATTENTION!!!! The mounted sample will be cleared, set to none. \nIf the sample is already mounted, please manually remove it."))
				{
					Epics.put(PV_ROBOT.cmd, "unload reset");
				}
			}}
		);
		
		// clear strategy results
		actionClearStrategy = new WdgControlAction
		(
			parent,
			"Clear strategy results",
			new Runnable(){public void run()
			{
				if(!Client.isMaster()) { Log.warn("This client is not master");return;}
				ModelStrategyResultList.get().save(System.getProperty("user.home")+"/strategyResults.xls");
				ModelStrategyResultList.get().deleteAll(); 
			}}
		);
		
		// clear sample state
		actionClearState = new WdgControlAction
		(
			parent,
			"Clear vector and raster",
			new Runnable(){public void run()
			{
				if (!Client.isMaster())
				{
					Log.note("Client is not master, sample-related clearing aborted.");
					return;
				}
				ClearSampleState.clearAllRelatedState("", true, true);
			}}
		);
		
		// clear multi startegy tables
		actionClearMultiStartegy = new WdgControlAction
		(
			parent,
			"Clear multi strategy",
			new Runnable(){public void run()
			{
				if (!Client.isMaster())
				{
					Log.note("Client is not master, sample-related clearing aborted.");
					return;
				}
				ModelCombineDatasets.get().deleteAll();
			}}
		);
	}
	
	private void connectListeners()
	{
		clientListener = new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateWidgets();
			}
		};
		
		Client.addListener(clientListener);

		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent de)
			{
				removeListeners();
			}
		});
		
		updateWidgets();
	}
	
	private void removeListeners()
	{
		Client.removeListener(clientListener);
	}
	
	private void updateWidgets()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			boolean isMaster = Client.isMaster();
			actionUndo			.setEnabled(isMaster);
			actionRedo			.setEnabled(isMaster);
			actionClearRobot	.setEnabled(isMaster);
			actionClearStrategy	.setEnabled(isMaster);
			actionClearState	.setEnabled(isMaster);
			actionClearMultiStartegy.setEnabled(isMaster);
		}});
	}
}
	