package gov.anl.gmca.gui.options;

import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.collect.DevCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList.CollectState;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.options.DevUserConfig.StrategyProgram;
import gov.anl.gmca.gui.options.WdgControlOption.OptionHandler;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.ListenerList.Listener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;

public class WdgUserOptions extends Composite
{
	private WdgControlOption /*optDiffViewer, */optXDSProc, optCutoff, optStrategyProgram, optAdaptiveScan, optBinMode, optShutterless, optHDFCleanup, optImageFormat;
	private WdgCommonDir optCommonDir;
	
	// listeners
	private ModelCollectRunList.Listener collectRunListListener;
	private ListenerList.Listener<DevUserConfig> userConfigListener;
	private ClientListener clientListener;
	
	public WdgUserOptions(TabFolder parent)
	{
		super(parent, SWT.NONE);
		
		GridLayout gl = new GridLayout(2, true);
		setLayout(gl);
		setBackgroundMode(SWT.INHERIT_FORCE);

		// widgets
		createTitle(this);
		createOptions(this);
		
		// listeners
		connectListeners();
	}

	private void createTitle(Composite parent)
	{
		// title
		Composite spacer = new Composite(parent, SWT.NONE);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
		gd.heightHint = 4;
		spacer.setLayoutData(gd);
		
		Label optionsTitle = new Label(parent, SWT.CENTER);
		optionsTitle.setText("JBluIce-EPICS Options");
		optionsTitle.setFont(GUI.FONT_14_BOLD);
		optionsTitle.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1));
		
		Composite spacer2 = new Composite(parent, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
		gd.heightHint = 20;
		spacer2.setLayoutData(gd);
	}
	
	
	private void createOptions(Composite parent)
	{
		// common dir
		//optCommonDir = new WdgCommonDir(parent,"Data directory");
		
		// left column
		Composite leftColumn = new Composite(parent, SWT.NONE);
		leftColumn.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		GridLayout gl = new GridLayout(1,false);
		gl.verticalSpacing = 0;
		//gl.marginLeft = 10;
		leftColumn.setLayout(gl);
		
		// right column
		Composite rightColumn = new Composite(parent, SWT.NONE);
		rightColumn.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		gl = new GridLayout(1,false);
		gl.verticalSpacing = 0;
		//gl.marginRight = 10;
		rightColumn.setLayout(gl);
		
		///////////////////////////////// left side ////////////////////////////
		
		// display: adxv/adxv+autoload/dtdisplay/albula
		/*optDiffViewer = new WdgControlOption
		(
			leftColumn,
			"Diffraction viewer",
			new String[]{"adxv","adxv -autoload","dtdisplay","albula"},
			new OptionHandler()
			{
				@Override public void handleSelection(int i)
				{
					DevUserConfig.get().setDisplayProgram(i);
				}
			}, true
		);*/
		
		// collect: data processing
		optXDSProc = new WdgControlOption(leftColumn, "Background data processing",
				new String[] { "Off (turn off background processing)",
						"On (separate processing dirs from data dirs)", 
						"On (proc dir close to image dir, under the sample dir)"},
				new OptionHandler() {
					@Override
					public void handleSelection(int i) {
						DevUserConfig.get().setXDSProcessing(i);
					}
				}, true);
		// collect: data processing
		/*optCutoff = new WdgControlOption
		(
			leftColumn,
				"GMCAproc resolution cutoff",
				new String[]{"no cutoff","0.5 sigma", "1 sigma"},
				new OptionHandler()
				{
					@Override public void handleSelection(int i)
					{
						//DevUserConfig.get().setXDSProcessing(i);
					}
				}, true
		);*/
		
		// strategy: best/mosflm
		optStrategyProgram = new WdgControlOption
		(
			leftColumn,
			"Strategy program",
			new String[]{"BEST","MOSFLM"},
			new OptionHandler()
			{
				@Override public void handleSelection(int i)
				{
					DevUserConfig.get().setStrategyProgram(StrategyProgram.values()[i]);
				}
			}, true
		);
		
		/*if (DetectorConfig.isEiger())
		{
			optImageFormat = new WdgControlOption
			(
				leftColumn,
				"Image output",
				//new String[]{"HDF only","CBF only", "Both, logs to HDF dir", "Both, logs to CBF dir"},
				new String[]{"HDF and CBF, other output to HDF dir", "HDF and CBF, other output to CBF dir"},
				new OptionHandler()
				{
					@Override public void handleSelection(int i)
					{
						DevUserConfig.get().setImageFormat(ImageFormat.values()[i+2]);
					}
				}, true
			);
		}*/

		///////////////////////////////// right side ////////////////////////////
		
		// edge scan: adaptive/fixed
		optAdaptiveScan = new WdgControlOption
		(
			rightColumn,
			"Edge scan mode",
			new String[]{"Adaptive","Fixed"},
			new OptionHandler()
			{
				@Override public void handleSelection(int i)
				{
					DevScan.get().setAdaptiveScan(i==0, true);
				}
			}, true
		);
		
		if (DetectorConfig.isMAR())
		{
			optBinMode = new WdgControlOption
			(
				rightColumn,
				"CCD bin mode for collect/screening",
				new String[]{"73\u00B5m pixel","146\u00B5m pixel"},
				new OptionHandler()
				{
					@Override public void handleSelection(int i)
					{
						DevCollectRunList.get().setBinMode(BinMode.values()[i+1], true);
					}
				}, true
			);
		}
		
		// shutterless enable/disable
		optShutterless = new WdgControlOption
		(
			rightColumn,
			"Shutterless Mode",
			new String[]{"Enable","Disable"},
			new OptionHandler()
			{
				@Override public void handleSelection(int i)
				{
					DevUserConfig.get().setShutterlessEnable(i==0);
				}
			}, true
		);
		
		/*if (DetectorConfig.isEiger())
		{
			optHDFCleanup = new WdgControlOption
			(
				rightColumn,
				"HDF5 header cleanup",
				new String[]{"Clean all master files","Clean non-collect master files","Leave originals"},
				new OptionHandler()
				{
					@Override public void handleSelection(int i)
					{
						DevUserConfig.get().setHDFCleanupMode(HDFCleanupMode.values()[i]);
					}
				}, true
			);
		}*/
	}

	private void connectListeners()
	{
		DBConfig.llValueChange.addListener(new Listener<DBConfig>()
		{
			@Override public void handleEvent(DBConfig object, Object data)
			{
				GUI.asyncExec(new Runnable(){@Override public void run()
				{
					updateWidgets();
				}});
			}
		});
		
		collectRunListListener = new ModelCollectRunList.Listener()
		{
			@Override public void runListChanged(final ModelRunList runList)
			{
				GUI.asyncExec(new Runnable(){@Override public void run()
				{
					updateWidgets();
				}});
			}
		};

		userConfigListener = new ListenerList.Listener<DevUserConfig>()
		{
			@Override public void handleEvent(DevUserConfig object, Object data)
			{
				updateWidgets();
			}
		};

		clientListener = new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateWidgets();
			}
		};
		
		ModelCollectRunList.get().addListener(collectRunListListener);
		DevUserConfig.get().ll.addListener(userConfigListener);
		Client.addListener(clientListener);

		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent de)
			{
				removeListeners();
			}
		});
		
		updateWidgets();
	}
	
	private void removeListeners()
	{
		ModelCollectRunList.get().removeListener(collectRunListListener);
		DevUserConfig.get().ll.removeListener(userConfigListener);
		Client.removeListener(clientListener);
	}
	
	private void updateWidgets()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			boolean isMaster = Client.isMaster();
			ModelCollectRunList crl = ModelCollectRunList.get();
			configureDisplayOptions();
			
			////////////////////////////////// enable or disable ////////////////////////////////
			
			//optDiffViewer		.setEnabled(isMaster);
			optXDSProc			.setEnabled(isMaster);
			//optCutoff			.setEnabled(isMaster);
			optStrategyProgram	.setEnabled(isMaster);
			optAdaptiveScan		.setEnabled(isMaster);
			optShutterless		.setEnabled(isMaster && DetectorConfig.isDectris());
			if (null!=optImageFormat) optImageFormat	.setEnabled(isMaster && DetectorConfig.isEiger());
			if (null!=optHDFCleanup)  optHDFCleanup		.setEnabled(isMaster && DetectorConfig.isEiger());
			if (null!=optBinMode)	  optBinMode		.setEnabled(isMaster && crl.getCollectState() == CollectState.INACTIVE);
			
			/////////////////////////////////////// values /////////////////////////////////////
			
			optXDSProc			.select(DevUserConfig.get().getXDSProc());
			//optCutoff			.select();
			optStrategyProgram	.select(DevUserConfig.get().getStrategyProgram().ordinal());
			optAdaptiveScan		.select(ModelScan.get().getAdaptiveScan()?0:1);
			optShutterless		.select(DevUserConfig.get().getShutterlessEnable()?0:1);
			if (null!=optImageFormat)
			{
				int i = DevUserConfig.get().getImageFormat().ordinal()-2;
				if (i>=0) optImageFormat.select(i);
			}
			if (null!=optHDFCleanup)  optHDFCleanup		.select(DevUserConfig.get().getHDFCleanupMode().ordinal());
			if (null!=optBinMode)	  optBinMode		.select(crl.getBinMode().ordinal()-1);
		}});
	}
	
	private void configureDisplayOptions()
	{
		//optDiffViewer.select(DevUserConfig.get().getDisplayProgram().index);
	}
}
