package gov.anl.gmca.gui.state;

import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.util.Util;

public abstract class Server
{
    ////////////////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////// Variables ////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////

    protected Thread thread = null;
    protected String url;
    protected String restartCmd;
    protected String serverName;
    protected abstract boolean update();

    ////////////////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////// Start thread //////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////

    protected boolean startReaderThread()
    {
        if (!verifyRunning()) return false;
        thread = new Thread(new Runnable() {public void run()
        {
            while (true)
            {
                Util.sleep(100);
                update();
            }
        }});
        thread.start();
        return true;
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////// Check server //////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////

    protected boolean restartServer()
    {
        ExecTask task = ExecTask.syncExec("/mnt/software/scripts/"+restartCmd, restartCmd, false);
        task.waitFor();
        boolean restartOK = task.exitValue==0;
        return restartOK;
    }

    protected void exitWithMessage(String message)
    {
        try {
            Main.closeSplash();
            Runtime.getRuntime().exec(new String[]{"zenity","--error","--text="+message+"  JBluIce will now exit."});
        } catch (Exception e) { e.printStackTrace(); }
    }

    protected boolean verifyRunning()
    {
        if (update()) return true;
        else
        if (!restartServer()) {
            exitWithMessage("The "+serverName+" did not respond and failed to restart.");
            return false;
        } else {
            long waitStartMs = System.currentTimeMillis();
            while (System.currentTimeMillis()-waitStartMs<20000) {
                if (update()) return true;
                Util.sleep(100);
            }
            exitWithMessage("The "+serverName+" restarted but is not responding.");
            return false;
        }
    }
}
