package gov.anl.gmca.hutch;

import gov.anl.gmca.collect.ModelCollectRun.Point3D;
import gov.anl.gmca.common.DevMove.MoveCmd;
import gov.anl.gmca.common.DevMove.MoveResult;
import gov.anl.gmca.common.DevMove.ResultType;
import gov.anl.gmca.hutch.DevPmacAssembly.AssemblyMove;

// combines XY assembly with Z motor if they are separate
// otherwise uses the combined XYZ assembly
//
public class DevXYZ
{
	public static MoveResult syncMove(Point3D dest, boolean waitFor)
	{
		if (DevPmacAssembly.get("sampleXY") != null)
		{
			// start xy move
			AssemblyMove xyMove = DevPmacAssembly.get("sampleXY").newMove();
			xyMove.setDest("sample_x", dest.x);
			xyMove.setDest("sample_y", dest.y);
			xyMove.start();
	
			// start z move
			MoveCmd zMove = DevPmacMotor.get("sample_z").startMove(dest.z);
	
			if (waitFor)
			{
				// wait for moves
				MoveResult xyResult = xyMove.waitFor();
				MoveResult zResult	= zMove.waitFor();
				
				// both ok
				if (xyResult.isOK() && zResult.isOK()) return new MoveResult(ResultType.SUCCESS);
				
				// one failed
				if (xyResult.isOK() && !zResult.isOK()) return zResult;
				if (!xyResult.isOK() && zResult.isOK()) return xyResult;
				
				// both failed - combine errors
				return new MoveResult(xyResult.type, xyResult.msg+"\n"+zResult.msg);
			}
			else
			{
				return new MoveResult(ResultType.SUCCESS);
			}
		}
		else if (DevPmacAssembly.get("sampleXYZ") != null)
		{
			// start xyz move
			AssemblyMove xyzMove = DevPmacAssembly.get("sampleXYZ").newMove();
			xyzMove.setDest("sample_x", dest.x);
			xyzMove.setDest("sample_y", dest.y);
			xyzMove.setDest("sample_z", dest.z);
			xyzMove.start();
	
			if (waitFor)
			{
				// wait for move
				MoveResult xyzResult = xyzMove.waitFor();
				
				// return the result
				return xyzResult;
			}
			else
			{
				return new MoveResult(ResultType.SUCCESS);
			}
		}
		else
		{
			return new MoveResult(ResultType.NOT_STARTED, "Neither sample motion assembly sampleXYZ nor sampleXY+sampleZ exists, couldn't start the move.");
		}
	}
}
