package gov.anl.gmca.hutch;

import gov.anl.gmca.common.DevMove.MoveCmd;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;

import java.util.ArrayList;

// a group of WdgMoves that are linked to the same start/cancel buttons
public class GrpWdgMove
{
	////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////// list //////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected ArrayList<WdgMove> wdgMoveList = new ArrayList<WdgMove>();
	public void add(WdgMove newWidget) { if (!wdgMoveList.contains(newWidget)) wdgMoveList.add(newWidget); }
	
	////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////// state /////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////

	protected boolean allMatch = true;
	public boolean doWidgetsMatch() { return allMatch; }
	public void updateButtonState()
	{
		if (null==startButton || null==cancelButton) return;
		
		GUI.asyncExec(new Runnable(){public void run()
		{
    		allMatch=true;
    		for (WdgMove widget : wdgMoveList)
    		{
    			if (!widget.matchesDevice()) allMatch = false;
    		}
    		
    		if (!Client.isMaster())
    		{
    			startButton.setEnabled(false);
    			cancelButton.setEnabled(false);
    		}
    		else
    		{
    			if (allMatch)
    			{
    				startButton.setEnabled(false);
    				cancelButton.setEnabled(false);
    			}
    			else
    			{
    				boolean enableStart = Client.isMaster(); // TODO: disallow motor moves at motor level?
    				startButton.setEnabled(enableStart);
    				cancelButton.setEnabled(true);
    			}
    		}
		}});
	}
	
	////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////// methods //////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected Button startButton, cancelButton, stopButton;
	
	public GrpWdgMove()
	{
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
    			updateButtonState();
			}
		});
	}
	
	public void setStartButton(Button startButton)
	{
		this.startButton = startButton;

		updateButtonState();
		
		startButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				handleStart();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}
	
	public void setCancelButton(Button startButton)
	{
		this.cancelButton = startButton;
		
		updateButtonState();
		
		cancelButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				handleCancel();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}
	
	public void setStopButton(Button startButton)
	{
		this.stopButton = startButton;
		
		updateButtonState();
		
		stopButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				handleStop();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}
	
	public void handleStart()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			final WdgMotor attenWidget = TabHutch.get().wdgAttenuation;
			WdgMotor energyWidget = TabHutch.get().wdgEnergy;
			
			if (!attenWidget.matchesDevice() && !energyWidget.matchesDevice())
			{
				Log.note("Simultaneous attenuation and energy moves requested, moving attenuation first.");
				
				final MoveCmd attenMove = attenWidget.move();
				
				new Thread(new Runnable(){public void run()
				{
					if (null != attenMove)
					{
						System.out.println("Waiting for attenuation move");
						attenMove.waitFor();
					}
					
            		GUI.asyncExec(new Runnable(){public void run()
            		{
                		for (WdgMove widget : wdgMoveList)
                		{
                			widget.move();
                		}
            		}});
				}}, "Combined energy move").start();
			}
			else
			{
        		for (WdgMove widget : wdgMoveList)
        		{
        			widget.move();
        		}
			}
		}});
	}
	
	public void handleCancel()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
    		for (WdgMove widget : wdgMoveList)
    		{
    			widget.cancel();
    		}
		}});
	}
	
	public void handleStop()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
        	for (WdgMove widget : wdgMoveList)
        	{
        		widget.stop();
        	}
		}});
	}
}
