package gov.anl.gmca.hutch;

import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Resources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import java.util.ArrayList;
import java.util.HashMap;

public class TabHutch extends Tab
{
	protected double[]
	                 omegaChoices = 		{ 0, 90, 180, 270 },
	                 twoThetaChoices =		{ 0, 1, 12 },
	                 beamstopChoices =		{ 35, 40, 45, 50, 55, 60, 65, 70 },
	                 beamSizeXChoices =		{ .025, .05, .075, .1, .2, 1, 2, 5 },
	                 beamSizeYChoices = 	beamSizeXChoices,
	                 energyChoices = 		{ 6, 8, 10, 12, 12.66, 14, 16, 18, 20 },
	                 attenuationChoices = 	{ 1, 5, 10, 20, 50, 100, 1000 };
	
	protected WdgMotor wdgDist, wdgEnergy, wdg2Theta, wdgAttenuation, wdgOmega;
	private WdgBeam beamToShutter, beamAfterShutter;
	public WdgAlignMiniBeam wdgAlignMiniBeam;
	
	public void setDefaultFocus()
	{
		//wdgOmega.comboRqsPos.setFocus();
		//wdgOmega.comboRqsPos.setSelection(new Point(0,0));
	}
	
	// base progress number for the hutch tab
	protected static final int HUTCH_PROGRESS = 40;
	final GrpWdgMove moveGroup = new GrpWdgMove();
	
	private static TabHutch inst = null;
	public static TabHutch get() { return inst; }
	
	public TabHutch(Composite parent, int style)
	{
		super(parent,style);
		inst = this;
		
		setBackground(GUI.BGCOLOR);

		setLayout(new GridLayout(3,false));
	
		Composite motorsComp = makeMotorsComp();
		motorsComp.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1);
		//gd.heightHint = 200;
		motorsComp.setLayoutData(gd);

		WdgAutomation auto = new WdgAutomation(this,SWT.NONE);
		gd = new GridData(SWT.LEFT, SWT.FILL, false, true);
		gd.widthHint=220;
		auto.setLayoutData(gd);

		Group alignGroup = new Group(this, SWT.NONE);
		alignGroup.setFont(GUI.FONT_HUTCH_GROUP);
		alignGroup.setBackground(GUI.BGCOLOR);
		alignGroup.setText("Beam alignment");
		GridLayout gl = new GridLayout();
		gl.marginHeight=0;
		gl.marginWidth=0;
		alignGroup.setLayout(gl);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.heightHint = 400;
		alignGroup.setLayoutData(gd);
		
	 	wdgAlignMiniBeam = new WdgAlignMiniBeam(alignGroup);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		wdgAlignMiniBeam.setLayoutData(gd);
	 	
		final WdgResPredictor resPredictor = new WdgResPredictor(this,SWT.MULTI,GUI.COLOR_LIGHT_BLUE,GUI.COLOR_RED,GUI.FONT_RESPREDICT_LABEL,60);
		resPredictor.setGroupFont(GUI.FONT_HUTCH_GROUP);
		gd = new GridData(SWT.RIGHT, SWT.FILL, false, true);
		gd.widthHint=275;
		resPredictor.setLayoutData(gd);

		wdg2Theta.addListener(new WdgMotor.Listener()
		{
			public void handleRqsPosChange(final double newRqsPos)
			{
    			GUI.asyncExec(new Runnable(){public void run()
    			{
    				resPredictor.setDetector2t_deg(newRqsPos, true);
    			}});
			}
		});
		
		wdgDist.addListener(new WdgMotor.Listener()
		{
			public void handleRqsPosChange(final double newRqsPos)
			{
    			GUI.asyncExec(new Runnable(){public void run()
    			{
    				resPredictor.setDetectorZ_mm(newRqsPos, true);
    			}});
			}
		});
		
		wdgEnergy.addListener(new WdgMotor.Listener()
		{
			public void handleRqsPosChange(final double newRqsPos)
			{
    			GUI.asyncExec(new Runnable(){public void run()
    			{
    				resPredictor.setBeamEnergy_eV(newRqsPos*1000.0, true);
    			}});
			}
		});
	}

	protected HashMap<String,Button> btnMap;
	private Image pilatusImage, marImage, currentDetImage;
	
	protected Composite makeMotorsComp()
	{
		final Composite motorsComp = new Composite(this,SWT.MULTI);
		motorsComp.setBackground(GUI.BGCOLOR);

		Button startButton = new Button(motorsComp, SWT.PUSH);
		startButton.setText("Start");
		startButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		startButton.setFont(GUI.FONT_HUTCH_BUTTON);
		startButton.setLocation(30,20);
		startButton.setSize(95,40);
		startButton.setEnabled(false);
		moveGroup.setStartButton(startButton);
		
		Button cancelButton = new Button(motorsComp, SWT.PUSH);
		cancelButton.setText("Cancel");
		cancelButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		cancelButton.setFont(GUI.FONT_HUTCH_BUTTON);
		cancelButton.setLocation(125,20);
		cancelButton.setSize(95,40);
		cancelButton.setEnabled(false);
		moveGroup.setCancelButton(cancelButton);
		
		Button stopButton = new Button(motorsComp, SWT.PUSH);
		stopButton.setVisible(false);
		stopButton.setText("Stop");
		stopButton.setBackground(GUI.COLOR_RED_BUTTON);
		stopButton.setFont(GUI.FONT_HUTCH_BUTTON);
		stopButton.setLocation(220,20);
		stopButton.setSize(95,40);
		moveGroup.setStopButton(stopButton);
		
		beamToShutter = new WdgBeam(motorsComp, true, false);
		beamToShutter.setLocation(0, 200);
		beamToShutter.setSize(64, 5);
		
		beamAfterShutter = new WdgBeam(motorsComp, true, true);
		beamAfterShutter.setLocation(398, 200);
		beamAfterShutter.setSize(232, 5);
		
		final int x=10,y=40;
		
		final Image imgGonio = new Image(GUI.getDisplay(),Resources.get().getInputStream("images/gonio.gif"));
		CLabel lblGonio = new CLabel(motorsComp,SWT.LEFT);
		lblGonio.setBackground(GUI.BGCOLOR);
		lblGonio.setImage(imgGonio);
		lblGonio.setMargins(0, 0, 0, 0);
		lblGonio.setLocation(x+436,y+77);
		lblGonio.setSize(imgGonio.getBounds().width, imgGonio.getBounds().height);
		
		Image imgFrontend = new Image(GUI.getDisplay(),Resources.get().getInputStream("images/frontend.gif"));
		CLabel lblFrontend = new CLabel(motorsComp,SWT.NONE);
		lblFrontend.setBackground(GUI.BGCOLOR);
		lblFrontend.setImage(imgFrontend);
		lblFrontend.setMargins(0, 0, 0, 0);
		lblFrontend.setLocation(x+50,y+72);
		lblFrontend.setSize(imgFrontend.getBounds().width, imgFrontend.getBounds().height);
		
		String pilatusImgPath = "images/Pilatus6M.png";
		String marImgPath = "images/q4_small.gif";
		pilatusImage = new Image(GUI.getDisplay(),Resources.get().getInputStream(pilatusImgPath));
		marImage = new Image(GUI.getDisplay(),Resources.get().getInputStream(marImgPath));
		currentDetImage = DetectorConfig.isDectris() ? pilatusImage : marImage;
		
		Point detXY = DetectorConfig.isDectris() ? new Point(833,103) : new Point(840,90);
		final CLabel lblDetector = new CLabel(motorsComp,SWT.NONE);
		lblDetector.setBackground(GUI.BGCOLOR);
		lblDetector.setMargins(0, 0, 0, 0);
		lblDetector.setLocation(x+detXY.x,y+detXY.y);
		lblDetector.setSize(currentDetImage.getBounds().width, currentDetImage.getBounds().height);
		lblDetector.setImage(currentDetImage);
		
		Image imgBeamstop = new Image(GUI.getDisplay(),Resources.get().getInputStream("images/beamstop.gif"));
		CLabel lblBeamstop = new CLabel(motorsComp,SWT.NONE);
		lblBeamstop.setBackground(GUI.BGCOLOR);
		lblBeamstop.setImage(imgBeamstop);
		lblBeamstop.setMargins(0, 0, 0, 0);
		lblBeamstop.setLocation(x+570,y+160);
		lblBeamstop.setSize(imgBeamstop.getBounds().width, imgBeamstop.getBounds().height);
		
		Image imgDistance = new Image(GUI.getDisplay(),Resources.get().getInputStream("images/distance.gif"));
		CLabel lblDistance = new CLabel(motorsComp,SWT.NONE);
		lblDistance.setBackground(GUI.BGCOLOR);
		lblDistance.setImage(imgDistance);
		lblDistance.setMargins(0, 0, 0, 0);
		lblDistance.setLocation(x+802,y+170);
		lblDistance.setSize(imgDistance.getBounds().width, imgDistance.getBounds().height);
		
		// Motor widgets
		wdgOmega = new WdgMotor(motorsComp, SWT.None, moveGroup, "Omega", null, "gonio_omega", GUI.BGCOLOR, omegaChoices, 3);
		wdgOmega.setLocation(x+310,y+20);
		wdgOmega.setSize(150,90);
		wdgOmega.moveAbove(lblGonio);
		wdgOmega.setDisplayUnits("deg");
		
	        wdgDist = new WdgMotor(motorsComp, SWT.None, moveGroup, "Distance", null, "detector_z", GUI.BGCOLOR, ModelPmacMotor.getDetectorDistanceOptions(), 3);
		wdgDist.setLocation(x+640,y+110);
		wdgDist.setSize(155,90);
		wdgDist.moveAbove(lblDetector);
		wdgDist.setDisplayUnits("mm");
		
		wdg2Theta = new WdgMotor(motorsComp, SWT.None, moveGroup, "2-Theta", null, "detector_2t", GUI.BGCOLOR, twoThetaChoices, 3);
		wdg2Theta.setLocation(x+740,y+5);
		wdg2Theta.moveAbove(lblDetector);
		wdg2Theta.setDisplayUnits("deg");

		WdgMotor wdgBeamstop = new WdgMotor(motorsComp, SWT.None, moveGroup, "Beamstop", null, "beamstop_z", GUI.BGCOLOR, beamstopChoices, 3);
		wdgBeamstop.setLocation(x+540,y+235);
		wdgBeamstop.moveAbove(lblFrontend);
		wdgBeamstop.setDisplayUnits("mm");

		WdgMotor wdgBeamWidth = new WdgMotor(motorsComp, SWT.None, moveGroup, "Width", null, "beam_size_x", GUI.BGCOLOR, beamSizeXChoices, 3);
		wdgBeamWidth.setLocation(x+180,y+215);
		wdgBeamWidth.moveAbove(lblFrontend);
		wdgBeamWidth.setDisplayUnits("mm");
		
		Label lblX = new Label(motorsComp, SWT.None);
		lblX.setText("x");
		lblX.setLocation(x+323,y+252);
		lblX.setSize(15,20);
		lblX.setBackground(GUI.BGCOLOR);
		lblX.moveAbove(wdgBeamWidth);

		WdgMotor wdgBeamHeight = new WdgMotor(motorsComp, SWT.None, moveGroup, "Height", null, "beam_size_y", GUI.BGCOLOR, beamSizeYChoices, 3);
		wdgBeamHeight.setLocation(x+343,y+215);
		wdgBeamHeight.moveAbove(lblFrontend);
		wdgBeamHeight.setDisplayUnits("mm");

		// add lower and upper limits to energy choices
		ArrayList<Double> energyChoices2 = new ArrayList<>();
		boolean lowerLimitFound=false, upperLimitFound=false;
		Limits limits = DevEnergy.getDevice().getLimits();
		for (double e : energyChoices) {
			double band = DevEnergy.getDevice().getInPosBand();
			if (Math.abs(e-limits.getLower()) < band) lowerLimitFound=true;
			if (Math.abs(e-limits.getUpper()) < band) upperLimitFound=true;
		}
		if (!lowerLimitFound) energyChoices2.add(limits.getLower());
		for (double e : energyChoices) energyChoices2.add(e);
		if (!upperLimitFound) energyChoices2.add(limits.getUpper());
		double[] energyChoices3 = new double[energyChoices2.size()];
		for (int i=0;i<energyChoices2.size();i++) energyChoices3[i]=energyChoices2.get(i);

		wdgEnergy = new WdgMotor(motorsComp, SWT.None, moveGroup, "Energy", null, "energy", GUI.BGCOLOR, energyChoices3, 4);
		wdgEnergy.setLocation(x+10,y+205);
		wdgEnergy.setSize(150,90);
		wdgEnergy.moveAbove(lblFrontend);
		wdgEnergy.setDisplayUnits("keV");

		wdgAttenuation = new WdgMotor(motorsComp, SWT.None, moveGroup, "Attenuation", null, "attenuation", GUI.BGCOLOR, attenuationChoices, 2);
		wdgAttenuation.setLocation(x+25,y+45);
		wdgAttenuation.setSize(160,90);
		wdgAttenuation.moveAbove(lblFrontend);
		wdgAttenuation.setDisplayUnits("x");
		
		return motorsComp;
	}
	
	private void updateWidgets()
	{
	 	/*DevAlignMinibeam.get().ll.addListener(new ListenerList.Listener<DevScript>(){@Override public void handleEvent(DevScript object, Object data)
	 	{
	 		updateEnableState();
	 	}});
		final boolean enableAlign 			= (Client.isMaster() && !DevAlignMinibeam.get().isRunning() && AppState.isOperationAllowed(Operation.OP_ALIGN_MINIBEAM));
		autoalignButton.setEnabled(enableAlign);*/
	}
	
	@Override public String getTabName() { return "Hutch"; }
}
