package gov.anl.gmca.hutch;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.DevMove.MoveCmd;
import gov.anl.gmca.common.DevMoveFloat;
import gov.anl.gmca.common.EpicsDevice.State;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.Limits.LimitsResult;
import gov.anl.gmca.common.client_server.RPC;
import gov.anl.gmca.common.client_server.StateEvents;
import gov.anl.gmca.common.client_server.StateMap;
import gov.anl.gmca.common.client_server.StateMap.StateMotor;
import gov.anl.gmca.common.ui.CCombo;
import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import java.util.ArrayList;
import java.util.HashSet;

public class WdgMotor extends WdgMove implements DevMoveFloat.Listener
{
	protected DevMoveFloat  dev;
	protected String		devName;
	
	protected Group			group			= null;
	protected Label			lblActPos 		= null;
	protected CCombo		comboRqsPos 	= null;
	protected Label			lblUnits 		= null;
	protected WdgMotorMenu	motorMenu 		= null;
	protected Composite		actposBorder	= null;
	protected Composite		actposBorder2	= null;
	
	protected double	widgetRqsPos = Double.NaN;
	
	protected Color bgColor;
	protected int decimals;
	
	public Group getGroup() 	{ return group; 		}
	public CCombo getActPos() 	{ return comboRqsPos; 	}
	public Label getLblUnits() 	{ return lblUnits; 		}
	
	public void setDisplayUnits(String units) {
		lblUnits.setText(units);
	}

	/////////////////////////////////////////////// listener ///////////////////////////////////////////
	
	public interface Listener
	{
		public abstract void handleRqsPosChange(double newPos);
	}
	
	protected HashSet<Listener> listenerSet = new HashSet<Listener>();
	public void addListener(Listener listener)
	{
		listenerSet.add(listener);
		notifyRqsPosListener(listener);
	}
	
	protected void notifyRqsPosListeners()
	{
    	for (final Listener l : listenerSet)
    	{
    		notifyRqsPosListener(l);
    	}
	}
	
	protected void notifyRqsPosListener(final Listener listener)
	{
		try
		{
			GUI.asyncExec(new Runnable(){public void run()
			{
				try
				{
					double d = Double.parseDouble(comboRqsPos.getText());
					listener.handleRqsPosChange(d);
				}
				catch (NumberFormatException nfe) {}
			}});
		}
		catch (NumberFormatException nfe) {}
	}

	//////////////////////////////////////////// actpos display ////////////////////////////////////////
	
	public WdgMotor(Composite parent, int style, final GrpWdgMove moveGroup, String title, DevMoveFloat _dev, String devName, Color _bgColor, double[] dropdownChoices, int _decimals)
	{
		super(parent,style, moveGroup);
		
		setLayout(new FillLayout());
	
		bgColor = _bgColor;
		decimals = _decimals;

		setBackground(bgColor);
		//setBackgroundMode(SWT.INHERIT_FORCE);
		
		if (null==_dev)	this.devName = devName;
		else			this.dev = _dev;
		
		group = new Group(this,SWT.NONE);
		group.setText(title);
		group.setFont(GUI.FONT_MOTOR_GROUP);
		group.setBackground(bgColor);
		group.setLayout(new GridLayout(2,false));
		GridData gd;
		
		//////////////////////////////////////////// actpos display ////////////////////////////////////////
		
		actposBorder2 = new Composite(group, SWT.NONE);
		actposBorder2.setBackground(GUI.COLOR_DARK_GREEN);
		GridLayout gl = new GridLayout(1,false);
		gl.marginHeight=1;
		gl.marginWidth=1;
		actposBorder2.setLayout(gl);
		gd = new GridData();
		gd.grabExcessVerticalSpace = false;
		gd.grabExcessHorizontalSpace = true;
		gd.verticalAlignment = SWT.TOP;
		gd.horizontalAlignment = SWT.FILL;
		actposBorder2.setLayoutData(gd);
		
		actposBorder = new Composite(actposBorder2, SWT.NONE);
		actposBorder.setBackground(GUI.COLOR_MOTOR_ACTPOS_OK);
		gl = new GridLayout(1,false);
		gl.marginHeight=2;
		gl.marginWidth=4;
		actposBorder.setLayout(gl);
		actposBorder.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		lblActPos = new Label(actposBorder, SWT.NONE);
		lblActPos.setText("Offline");
		lblActPos.setEnabled(true);
		lblActPos.setFont(GUI.FONT_MOTOR_ACTPOS);
		lblActPos.setBackground(GUI.COLOR_MOTOR_ACTPOS_OFF);
		lblActPos.setForeground(GUI.COLOR_MOTOR_ACTPOS_FG);
		lblActPos.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		lblUnits = new Label(group,SWT.None);
		lblUnits.setText("      ");
		lblUnits.setFont(GUI.FONT_MOTOR_LABEL);
		lblUnits.setBackground(bgColor);
		gd = new GridData();
		gd.verticalAlignment = SWT.CENTER;
		gd.horizontalAlignment = SWT.LEFT;
		gd.widthHint = 33;
		lblUnits.setLayoutData(gd);

		ArrayList<String> itemList = new ArrayList<String>();
		Limits limits;
		if (null!=dev) {
			limits = dev.getLimits();
		} else {
			StateMotor motor = StateMap.current.motorList.get(devName);
			limits = new Limits(motor.lowerLimit, motor.upperLimit, "");
		}
		for (int i=0; i<dropdownChoices.length; i++)
		{
			LimitsResult result = limits.testValue(dropdownChoices[i]);
			if (!result.isOK())
			{
				System.out.println("Skipping choice "+dropdownChoices[i]+" for device "+(null==dev?devName:dev.getDeviceName())+", "+result.getReason());
				continue;
			}
			itemList.add(Util.sprintf("%.0"+decimals+"f", dropdownChoices[i]));
		}

		//////////////////////////////////////////// rqspos combo /////////////////////////////////////////
		
		comboRqsPos = new CCombo(group,SWT.NONE);
		comboRqsPos.setItems(itemList.toArray(new String[]{}));
		comboRqsPos.setText("Offline");
		comboRqsPos.setEnabled(false);
		comboRqsPos.setFont(GUI.FONT_MOTOR_RQSPOS);
		gd = new GridData();
		gd.grabExcessVerticalSpace = false;
		gd.grabExcessHorizontalSpace = true;
		gd.verticalAlignment = SWT.TOP;
		gd.horizontalAlignment = SWT.FILL;
		comboRqsPos.setBackground(GUI.COLOR_WHITE);
		comboRqsPos.setLayoutData(gd);
		
		comboRqsPos.addModifyListener(new ModifyListener(){public void modifyText(ModifyEvent me) 
		{
			if (matchesDevice())
			{
				comboRqsPos.setForeground(GUI.COLOR_DEFAULT_TEXT);
			}
			else
			{
				comboRqsPos.setForeground(GUI.COLOR_RED);
			}
			moveGroup.updateButtonState();
			
			notifyRqsPosListeners();
		}});
		
		comboRqsPos.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent te)
			{
				if (te.detail == SWT.TRAVERSE_ESCAPE)
				{
					setWidgetRqsPos(widgetRqsPos);
				}
				else
				{
					submitRqsPos();
				}
			}
		});
		
		comboRqsPos.addFocusListener(new FocusListener()
		{
			@Override public void focusGained(FocusEvent fe) {}
			@Override public void focusLost(FocusEvent fe)
			{
				submitRqsPos();
			}
		});
		
		///////////////////////////////////////////////// motor menu /////////////////////////////////////////

		motorMenu = new WdgMotorMenu(group, dev);
		gd = new GridData();
		gd.verticalAlignment = SWT.CENTER;
		gd.horizontalAlignment = SWT.LEFT;
		gd.widthHint = 30;
		motorMenu.setLayoutData(gd);
		
		////////////////////////////////////////////////// end widgets ///////////////////////////////////////
		
		setSize(140,90);
		
		if (null==dev) {
			//StateEvents.addListener();
			StateEvents.addListener(
					"motorList."+devName+".actPos",
					(key, value) -> _handleActPosChange(Double.valueOf(value)));
			StateEvents.addListener(
					"motorList."+devName+".rqsPos",
					(key, value) -> handleRqsPosChange(Double.valueOf(value)));
			StateEvents.addListener(
					"motorList."+devName+".state",
					(key, value) -> handleStateChange(State.fromString(value)));
			StateEvents.addListener(
					"motorList."+devName+".units",
					(key, value) -> handleUnitsChange(value));
		}
		else {
			dev.addListener(this);
		}
		//System.out.println("Widget "+dev.getDeviceName()+" state="+dev.getState());
		
		// TODO: add for server device
		if (null!=dev) {
			Operation op = dev.getOperation();
			if (null != op)
			{
	    		AppState.addListener(op, new AppState.Listener()
	    		{
	    			@Override public void stateChanged(Operation op, boolean allowed)
	    			{
	    				// don't show highlight if slave
	    				final boolean showDisallowed = (!allowed && Client.isMaster());
	    				
	    				GUI.asyncExec(new Runnable(){public void run()
	    				{
	    					Color fg = showDisallowed?GUI.COLOR_YELLOW:GUI.COLOR_BLACK;
	    					group.setForeground(fg);
	    				}});
	    			}
	    		});
			}
		}
	}
	
	/////////////////////////////////////////////////////// methods ////////////////////////////////////////////

	protected boolean submitRqsPos()
	{
    	String inputString = comboRqsPos.getText();
    	try
    	{
    		double d = Double.parseDouble(inputString);
    		setWidgetRqsPos(d);
    		return true;
    	}
    	catch (NumberFormatException nfe)
    	{
    		setWidgetRqsPos(widgetRqsPos);
    		
    		if (inputString.length()>0)
    		{
        		NBMessageBox mb = new NBMessageBox(GUI.getShell(), SWT.APPLICATION_MODAL | SWT.ICON_ERROR|SWT.OK);
        		mb.setText("Invalid input");
        		mb.setMessage("\""+inputString+"\" is not a valid number.");
        		mb.open();
    		}
    		return false;
    	}
	}
	
	@Override public boolean matchesDevice()
	{
		// parse position field
		double newPos;
		try	{
			newPos = Double.parseDouble(comboRqsPos.getText());
		} catch (NumberFormatException nfe) { return false; }

		if (dev==null) {
			StateMotor motor = StateMap.current.motorList.get(devName);
			if (motor.state == State.OFFLINE) return true;
			
			// compare diff to inPosBand
			double currentPos = motor.actPos;
			return (Math.abs(newPos-currentPos) <= motor.inPosBand);
		} else {
			if (!dev.isOnline()) return true;
			double currentPos = dev.getPosition();
			
			// for attenuation, check that diff is < 5% of original
			if (dev.getDeviceName().equals("attenuation"))
			{
				return ((Math.abs(newPos-currentPos)/currentPos)<=0.05);
			
			// otherwise, compare to inPosBand
			} else {
				return (Math.abs(newPos-currentPos) <= dev.getInPosBand());
			}
		}
	}
	
	@Override
	public boolean isMoving()
	{
		if (null!=dev) { return dev.isMoving(); }
		else {
			StateMotor motor = StateMap.current.motorList.get(devName);
			return !motor.inPos;
		}
	}
	
	@Override
	public MoveCmd move()
	{
		if (!submitRqsPos()) return null; // fails for invalid input
		if (matchesDevice()) return null; // ignore near-zero moves

		if (null!=dev) {
			return dev.startMove(Double.parseDouble(comboRqsPos.getText()));
		} else {
			RPC.sendCommand("cmd:MotorMove", "motor:"+devName, "dest:"+comboRqsPos.getText());
			return null; // not used
		}
	}

	@Override
	public void cancel()
	{
		if (null!=dev) setWidgetRqsPos(dev.getPosition());
		else setWidgetRqsPos(StateMap.current.motorList.get(devName).actPos);
	}

	@Override
	public void stop()
	{
		RPC.sendCommand("cmd:Abort");
	}
	
	public void handleActPosChange(DevMoveFloat dev, final double newPos)
	{
		_handleActPosChange(newPos);
	}
	
	// for use by either Device or State listeners
	public void _handleActPosChange(final double newPos)
	{
		if (!isDisposed())
		{
			GUI.asyncExec(new Runnable(){public void run()
			{
    			lblActPos.setText(formatPosition(newPos));
    			
    			if (Double.isNaN(widgetRqsPos))
    			{
    				setWidgetRqsPos(newPos);
    			}
			}});
		}
	}
	
	protected void setWidgetRqsPos(final double pos)
	{
		widgetRqsPos = pos;
		GUI.asyncExec(new Runnable(){public void run()
		{
			comboRqsPos.setText(formatPosition(pos));
		}});
		notifyRqsPosListeners();
	}
	
	protected String formatPosition(double pos)
	{
		if (pos>=10000.0)
		{
			return Util.sprintf("%.02e",pos);
		}
		else
		{
			return Util.sprintf("%.0"+decimals+"f",pos);
		}
	}

	// from the device
	private double lastRqsPos = Double.MAX_VALUE;
	public void handleRqsPosChange(final double newPos)
	{
		if (null!=dev) {
			if (dev.getState() == State.MOVING) setWidgetRqsPos(newPos);
		} else {
			StateMotor motor = StateMap.current.motorList.get(devName);
			State motorState = motor.state;
			boolean significantChange = Math.abs(motor.rqsPos-lastRqsPos)>=motor.inPosBand;

			if (motorState==State.MOVING || significantChange) {
				lastRqsPos = motor.rqsPos;
				setWidgetRqsPos(newPos);
			}
		}
	}

	@Override public void handleStateChange(final State state)
	{
		if (isDisposed()) return;
		if (null==state) return;
		
		GUI.asyncExec(new Runnable(){public void run()
		{
    		switch (state)
    		{
    		case OFFLINE:
    			
                lblActPos.setBackground(GUI.COLOR_MOTOR_ACTPOS_OFF);
                actposBorder.setBackground(GUI.COLOR_MOTOR_ACTPOS_OFF);
                actposBorder2.setBackground(GUI.COLOR_MOTOR_ACTPOS_OFF_BORDER);
                
                lblActPos.setForeground(GUI.COLOR_BLACK);
                comboRqsPos.setEnabled(false);
                motorMenu.setEnabled(false);
    			break;
    			
    		case READY:
    			
                lblActPos.setBackground(GUI.COLOR_MOTOR_ACTPOS_OK);
                actposBorder.setBackground(GUI.COLOR_MOTOR_ACTPOS_OK);
                actposBorder2.setBackground(GUI.COLOR_MOTOR_ACTPOS_OK_BORDER);
                
                lblActPos.setForeground(GUI.COLOR_BLACK);
                comboRqsPos.setEnabled(true);
                motorMenu.setEnabled(true);
                break;
    			
    		case MOVING:

    			// when moving, rqspos is set to match the PV
    			if (dev==null) {
    				setWidgetRqsPos(StateMap.current.motorList.get(devName).rqsPos);
    			} else {
    				setWidgetRqsPos(dev.getRqsPos());
    			}
    			
                lblActPos.setBackground(GUI.COLOR_MOTOR_ACTPOS_MOVE);
                actposBorder.setBackground(GUI.COLOR_MOTOR_ACTPOS_MOVE);
                actposBorder2.setBackground(GUI.COLOR_MOTOR_ACTPOS_MOVE_BORDER);
                
                lblActPos.setForeground(GUI.COLOR_WHITE);
                comboRqsPos.setEnabled(true);
                motorMenu.setEnabled(true);
    			break;
    			
    		case ERROR:
    			
                lblActPos.setBackground(GUI.COLOR_MOTOR_ACTPOS_ERR);
                actposBorder.setBackground(GUI.COLOR_MOTOR_ACTPOS_ERR);
                actposBorder2.setBackground(GUI.COLOR_MOTOR_ACTPOS_ERR_BORDER);
                
                lblActPos.setForeground(GUI.COLOR_BLACK);
                comboRqsPos.setEnabled(false);
                motorMenu.setEnabled(false);
    			break;
    		}
    
    		if (matchesDevice())
    		{
    			comboRqsPos.setForeground(GUI.COLOR_DEFAULT_TEXT);
    		}
    		else
    		{
    			comboRqsPos.setForeground(GUI.COLOR_RED);
    		}
    		moveGroup.updateButtonState();
		}});
	}
	
	public void handleUnitsChange(String units)
	{
		GUI.asyncExec(new Runnable() {public void run()
		{
			if (!isDisposed()) getLblUnits().setText(units);
		}});
	}
}
