package gov.anl.gmca.io;

import gov.anl.gmca.gui.Main;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;

// runs sync/async system commands with optional logging and callbacks
public abstract class ExecTask
{
	public static enum ManagedScript
	{
		CollimatorCtrl,
		AlignMinibeam;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////// begin listener /////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public interface Listener
	{
		public void taskFinished();
	}
	protected ArrayList<Listener> listenerList = new ArrayList<Listener>();
	public void addListener(Listener listener) { listenerList.add(listener); }
	protected void notifyTaskFinished() { for (Listener l : listenerList) l.taskFinished(); }
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////// end listener //////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected String workingDir = null;
	public void setWorkingDir(String workingDir) { this.workingDir = workingDir; }
	
	public ArrayList<String> stdout = new ArrayList<String>();
	public ArrayList<String> stderr = new ArrayList<String>();
	public String stdoutString() { return concat(stdout); }
	public String stderrString() { return concat(stderr); }

	public String inputFilename;
	public String inputText;
	public int exitValue;
	
	protected Process proc = null;
	protected Thread thread = null;
	protected String cmdFilename = null;
	
	public boolean isRunning() { return (null != thread && thread.isAlive()); }
	
	public static ExecTask new_ExecTask(final String cmd, final String cmdFilename, final boolean logOutput)
	{
		if (Main.SIM_LEVEL == 2)	return new gov.anl.gmca.io.hw.ExecTask(cmd,cmdFilename,logOutput);
		else						return new gov.anl.gmca.io.hw.ExecTask(cmd,cmdFilename,logOutput);
	}
	
	public static ExecTask new_ExecTask(final String cmd, final String cmdFilename, final boolean logOutput, final boolean logError)
	{
		if (Main.SIM_LEVEL == 2)	return new gov.anl.gmca.io.sim.ExecTask(cmd,cmdFilename,logOutput, logError);
		else						return new gov.anl.gmca.io.hw.ExecTask(cmd,cmdFilename,logOutput, logError, true);
	}
	
	public static ExecTask new_ExecTask(final String cmd, final String cmdFilename, final boolean logOutput, final boolean logError, final boolean logToFile)
	{
		if (Main.SIM_LEVEL == 2)	return new gov.anl.gmca.io.sim.ExecTask(cmd,cmdFilename,logOutput, logError);
		else						return new gov.anl.gmca.io.hw.ExecTask(cmd,cmdFilename,logOutput, logError, logToFile);
	}
	
	public static ExecTask syncExec(String cmd, String cmdFilename, boolean logOutput)
	{
		ExecTask task = new_ExecTask(cmd,cmdFilename,logOutput);
		task.start();
		task.waitFor();
		return task;
	}
	
	public static ExecTask syncExec(String cmd, String cmdFilename, boolean logOutput, boolean logToFile)
	{
		ExecTask task = new_ExecTask(cmd,cmdFilename,logOutput,logOutput,logToFile);
		task.start();
		task.waitFor();
		return task;
	}
	
	public static ExecTask asyncExec(String cmd, String cmdFilename, boolean logOutput)
	{
		ExecTask task = new_ExecTask(cmd,cmdFilename,logOutput);
		task.start();
		return task;
	}

	public static ExecTask asyncExec(String cmd, String cmdFilename, boolean logOutput, boolean logError)
	{
		ExecTask task = new_ExecTask(cmd,cmdFilename,logOutput,logError);
		task.start();
		return task;
	}

	public void start()
	{
		thread.start();
	}
	
	public void asyncKill()
	{
		new Thread(new Runnable(){public void run()
		{
			syncKill();
		}
		}, "ExecTask asyncKill").start();
	}

	public abstract void syncKill();
	
	public void asyncStop()
	{
		new Thread(new Runnable(){public void run()
		{
			syncStop();
		}
		}, "ExecTask asyncStop").start();
	}
	
	public abstract void syncStop();
	
	public void waitFor()
	{
		while (thread.isAlive())
		{
			Util.sleep(10);
		}
	}

	protected String concat(ArrayList<String> al)
	{
		StringBuilder sb = new StringBuilder();
		for (String line : al)
			sb.append(line+"\n");
		return sb.toString();
	}
}
