package gov.anl.gmca.io;

import gov.anl.gmca.collect.DevBinMode;
import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.collect.DevCollectRunList;
import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList.CollectLeftTab;
import gov.anl.gmca.collect.ui.TabCollect;
import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.common.ui.WdgAppState;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevAlignMinibeam;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.spotfinder.SpotFinderQueue;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList.Listener;
import gov.anl.gmca.util.SaveState;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Shell;

import java.util.StringTokenizer;

public class Interpreter
{
	protected static boolean cmdOutputFirstCB=true, logMsgFirstCB=true;
	
	public static void connect()
	{
		DevFPE.get().commandListener.addListener(new Listener<DevFPE>()
		{
			@Override public void handleEvent(DevFPE object, Object data)
			{
				if (cmdOutputFirstCB) { cmdOutputFirstCB=false; return; }
				process((String)data);
			}
		});
	}
	
	public static void process(final String cmdLine)
	{
		System.out.println("Interpreter.process(): "+cmdLine);
		
		StringTokenizer st = new StringTokenizer(cmdLine);
		if (!st.hasMoreTokens()) return;
		
		final String cmd = st.nextToken();
		final String restOfLine = st.hasMoreTokens()?cmdLine.substring(cmd.length()+1):"";
		
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (cmd.equals("log_note"))
			{
				Log.note(restOfLine);
			}
			else if (cmd.equals("nbpopup"))
			{
				new Thread(new Runnable(){public void run()
				{
    				NBMessageBox mb = new NBMessageBox(GUI.getShell(),SWT.RETRY|SWT.CANCEL);
    				mb.setText("Shutter check");
    				mb.setMessage("The endstation shutter failed to open. "+
    							  "Click \"retry\" to try again, otherwise cancel.");
    				int result = mb.open();
    				Log.note("Result = "+(SWT.RETRY==result?"RETRY":"CANCEL"));
				}}, "Interpreter nbpopup").start();
			}
			else if (cmd.equals("create_exception"))
			{
				Shell s = new Shell(GUI.getDisplay());
				s.dispose();
				s.setEnabled(true);
			} 
			else if (cmd.equals("create_hang"))
			{
				int hangTimeSec = 10;
				final StringTokenizer st2 = new StringTokenizer(restOfLine);
				if (st2.hasMoreTokens())
				{
					try
					{
						hangTimeSec = Integer.parseInt(st2.nextToken());
					}
					catch (Exception e) { e.printStackTrace(); }
				}
				final int f_hangTimeSec = hangTimeSec;
				GUI.getDisplay().syncExec(new Runnable(){public void run() // syncExec is for create_hang command
				{
					Util.sleep(f_hangTimeSec*1000);
				}});
			}
			else if (cmd.equals("create_exception2"))
			{
				Shell s = new Shell(GUI.getDisplay());
				s.setLayout(new FillLayout());
				Button b = new Button(s,SWT.PUSH);
				b.setLayoutData(new GridData());
				s.pack();
				s.open();
			} 
			else if (cmd.equals("appstate"))
			{
				WdgAppState.open(false);
			} 
			else if (cmd.equals("log_warn"))
			{
				Log.warn(restOfLine);
			} 
			else if (cmd.equals("log_error"))
			{
				Log.error(restOfLine);
			} 
			else if (cmd.equals("popup_note"))
			{
				Util.popupOk("Note", restOfLine);
			}
			else if (cmd.equals("popup_error"))
			{
				if (!restOfLine.contains("responce")) {
						Util.popupError("Error", restOfLine);
				}
			}
			else if (cmd.equals("caput"))
			{
				int spaceIndex = restOfLine.indexOf(' ');
				if (spaceIndex<0) { Log.error("Syntax: caput [PV name] [value]"); return; }
				String pvName = restOfLine.substring(0, spaceIndex);
				String value = restOfLine.substring(spaceIndex+1);
				
				class CommandLinePV implements PVName
				{
					private String name;
					public CommandLinePV(String name) { this.name=name; }
					@Override public String toString() { return name; }
				}
				Epics.put(new CommandLinePV(pvName), value);
			}
			else if (cmd.equals("caget"))
			{
				String pvName = restOfLine;
				class CommandLinePV implements PVName
				{
					private String name;
					public CommandLinePV(String name) { this.name=name; }
					@Override public String toString() { return name; }
				}
				Log.note("caget "+pvName+" = "+Epics.getString(new CommandLinePV(pvName)));
			}
			else if (cmdLine.equals("start_operation pauseDataCollection"))
			{
				if (Client.isMaster())
				{
					ModelCollectRunList.get().pauseCollection();
				}
			}
			else if (cmdLine.equals("run_handle_start_button"))
			{
				if (Client.isMaster())
				{
					TabCollect.get().submitFields();
					ModelCollectRunList.get().startCollection();
				}
			}
			else if (cmdLine.equals("stog_become_master"))
			{
				ModelUserTable.get().devBecomeMaster();
			}
			else if (cmdLine.equals("stog_become_slave"))
			{
				ModelUserTable.get().devBecomeSlave();
			}
			else if (cmd.equals("exec"))
			{
				final StringTokenizer st2 = new StringTokenizer(restOfLine);
				if (st2.hasMoreTokens())
				{
					new Thread(new Runnable(){public void run()
					{
        				ExecTask task = ExecTask.new_ExecTask(restOfLine, st2.nextToken(), true);
        				task.start();
        				task.waitFor();
        				Log.note(restOfLine+" exited with value "+task.exitValue);
					}}, "Interpreter exec").start();
				}
			}
			else if (cmd.equals("g2eBecomeSlave"))
			{
				// listener is in DevUserTable for now
			}
			else if (cmd.equals("show_strategy"))
			{
				DevCollectRunList.get().setCurrentLeftTab(CollectLeftTab.STRATEGY);
			} 
			else if (cmd.equals("font_size"))
			{
				int fontSize;
				final StringTokenizer st2 = new StringTokenizer(restOfLine);
				if (st2.hasMoreTokens())
				{
					try
					{
						fontSize = Integer.parseInt(st2.nextToken());
						GUI.setFontSize(fontSize);
					}
					catch (Exception e) { e.printStackTrace(); }
				}
			} 
			else if (cmd.equals("cleanup_user_table"))
			{
				ModelUserTable.get().cleanupClientsOnAllWorkstations();
			}
			else if (cmd.equals("save_minibeam_log"))
			{
				DevAlignMinibeam.saveScanInfo();
			}
			else if (cmd.equals("save_state"))
			{
				SaveState.save();
			}
			else if (cmd.equals("load_state"))
			{
				SaveState.load();
			}
			else if (cmd.equals("sf_queue"))
			{
				SpotFinderQueue.startMonitor();
			}
			else if (cmd.equals("bin_mode"))
			{
				final StringTokenizer st2 = new StringTokenizer(restOfLine);
				if (st2.hasMoreTokens())
				{
					try
					{
						int modeIndex = Integer.parseInt(st2.nextToken());
						BinMode mode = BinMode.values()[modeIndex];
						DevBinMode.get().setMode(mode);
					}
					catch (Exception e) { e.printStackTrace(); }
				}
			}
			else
			{
				Log.note("Unknown command: "+cmdLine);
			}
		}});
	}
}
