package gov.anl.gmca.io.hw;

import gov.anl.gmca.io.hw.Epics.EpicsTask;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import gov.aps.jca.Channel;
import gov.aps.jca.Channel.ConnectionState;

import java.util.ArrayList;

public class EpicsStatus
{
	private static final int HANG_LIMIT_SEC = 2;
	
	protected static class PV
	{
		Channel ch;
		long creationTime;
		public PV(Channel ch, long creationTime) { this.ch=ch; this.creationTime=creationTime; }
		
		@Override public boolean equals(Object other)
		{
			if (!(other instanceof PV)) return false;
			return ((PV)other).ch == ch;
		}
	}
	
	public static void start()
	{
		new Thread(new Runnable(){public void run()
		{
			while(true)
			{
    			checkConnections();
    			checkForHangs();
    			Util.sleep(500);
			}
		}}, "Epics Status").start();
	}
	
	protected static ArrayList<PV> connectingPVList = new ArrayList<PV>();
	protected static ArrayList<PV> failedPVList = new ArrayList<PV>();
	
	public static void connectionInitiated(Channel pvChannel)
	{
		PV pv = new PV(pvChannel, System.currentTimeMillis());
		if (connectingPVList.contains(pv)) return;
		connectingPVList.add(pv);
	}
	
	protected static void checkConnections()
	{
		//System.out.println("Checking "+connectingPVList.size()+" connections");
		
		long now = System.currentTimeMillis();
		
		for (int i=0; i<connectingPVList.size(); i++)
		{
			PV pv = connectingPVList.get(i);
			if (pv.ch.getConnectionState() == ConnectionState.CONNECTED)
			{
				connectingPVList.remove(i);
				i--;
			}
			else
			{
				long timeSinceCreation = now - pv.creationTime;
				long timeoutMs = gov.anl.gmca.io.hw.Epics.CONNECT_TIMEOUT_MS;
				if (timeSinceCreation > timeoutMs)
				{
					Log.warn(pv.ch.getName()+" did not connect after "+(timeoutMs/1000.0)+" seconds, state = "+pv.ch.getConnectionState());
					connectingPVList.remove(i);
					i--;
					failedPVList.add(pv);
				}
			}
		}
	}
	
	protected static EpicsTask taskPrinted = null;
	
	protected static void checkForHangs()
	{
		long lcs = Epics.lastCallbackStart;
		if (0 != lcs)
		{
			long now = System.currentTimeMillis();
			int secSinceCBStart = (int)((now-lcs)/1000.0);
			
			if (secSinceCBStart > HANG_LIMIT_SEC)
			{
				EpicsTask currentTask = Epics.currentTask;
				
				if (null != currentTask)
				{
					if (taskPrinted == currentTask)
					{
						currentTask.printContinuingHangInfo();
					}
					else
					{
						currentTask.printFirstHangInfo();
						taskPrinted = currentTask;
					}
				}
			}
		}
		else
		{
			taskPrinted = null;
		}
	}
}
