package gov.anl.gmca.io.sim;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.util.Util;
import gov.aps.jca.JCALibrary;
import gov.aps.jca.configuration.DefaultConfiguration;
import gov.aps.jca.dbr.DBR;
import gov.aps.jca.dbr.DBR_Double;
import gov.aps.jca.dbr.DBR_Int;
import gov.aps.jca.dbr.DBR_String;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

public class Epics extends gov.anl.gmca.io.Epics
{
	@Override protected void i_addMonitor(final String pvName, final EpicsMonitor em)
	{
		try
		{
			imonMapLock.acquire();

			if (null == imonMap.get(pvName))
			{
				imonMap.put(pvName,new ArrayList<EpicsMonitor>());
			}
			
			GUI.asyncExec(new Runnable(){public void run()
			{
				// note that the value sent here could be out of date since it's cached
				if (null != em) em.valueChanged(pvName, objectToDBR(valueMap.get(pvName)));
			}});
			
			imonMap.get(pvName).add(em);
		}
		catch (Exception e) { e.printStackTrace(); }
		finally
		{
			imonMapLock.release();
		}
	}

	@Override protected boolean i_removeMonitor(String pvName, EpicsMonitor em)
	{
		return true;
	}

	protected HashMap<String,Object> valueMap = new HashMap<String,Object>();
	
	@Override protected double i_getDouble(String pvName)
	{
		Object val = valueMap.get(pvName);
		if (val instanceof Double) return (Double)val;
		if (val instanceof Integer) return (Integer)val;
		if (val instanceof String)
		{
			try
			{
				double ret = Double.parseDouble((String)val);
				return ret;
			}
			catch (NumberFormatException nfe) { return 0; }
		}
		if (val instanceof Double[]) return ((Double[])val)[0];
		if (val instanceof Integer[]) return (double)((Integer[])val)[0];
		
		if (val == null) return 0;
		
		System.out.println("WARNING: returning 0 for unknown type");
		return 0;
	}
	
	@Override protected int i_getCacheInteger(String pvName)
	{
		return i_getInteger(pvName);
	}
	
	@Override protected int i_getInteger(String pvName)
	{
		Object val = valueMap.get(pvName);
		if (val instanceof Double) return (int)Math.round((Double)val);
		if (val instanceof Integer) return (Integer)val;
		if (val instanceof String)
		{
			try
			{
				int ret = Integer.parseInt((String)val);
				return ret;
			}
			catch (NumberFormatException nfe) { return 0; }
		}
		return 0;
	}
	@Override protected short 		i_getEnum(String pvName)					{ return (short)i_getInteger(pvName); }
	@Override protected boolean 	i_getBoolean(String pvName)				{ return i_getInteger(pvName)==1; }
	
	@Override protected double[] 	i_getDoubleArray(String pvName)
	{
		Object val = valueMap.get(pvName);
		if (val instanceof Double) return new double[]{(Double)val};
		if (val instanceof Integer) return new double[]{(Integer)val};
		if (val instanceof String)
		{
			try
			{
				double ret = Double.parseDouble((String)val);
				return new double[]{ret};
			}
			catch (NumberFormatException nfe) { nfe.printStackTrace(); return new double[]{0.0}; }
		}
		if (val instanceof Double[])
		{
			Double[] dVal = (Double[])val;
			double[] ret = new double[dVal.length];
			for (int i=0;i<dVal.length;i++) ret[i]=dVal[i];
			return ret;
		}
		if (val instanceof Integer[])
		{
			Integer[] dVal = (Integer[])val;
			double[] ret = new double[dVal.length];
			for (int i=0;i<dVal.length;i++) ret[i]=dVal[i];
			return ret;
		}

		if (val == null) return new double[]{0.0};
		
		System.out.println("WARNING: returning 0 for unsupported EPICS array type");
		return new double[]{0};
	}
	
	@Override protected int[] i_getIntegerArray(String pvName)
	{
		Object val = valueMap.get(pvName);
		if (val instanceof Double) return new int[]{(int)Math.round((Double)val)};
		if (val instanceof Integer) return new int[]{(Integer)val};
		if (val instanceof String)
		{
			try
			{
				int ret = Integer.parseInt((String)val);
				return new int[]{ret};
			}
			catch (NumberFormatException nfe) { return new int[]{0}; }
		}
		if (val instanceof Double[])
		{
			Double[] dVal = (Double[])val;
			int[] ret = new int[dVal.length];
			for (int i=0;i<dVal.length;i++) ret[i]=(int)Math.round(dVal[i]);
			return ret;
		}
		if (val instanceof Integer[])
		{
			Integer[] dVal = (Integer[])val;
			int[] ret = new int[dVal.length];
			for (int i=0;i<dVal.length;i++) ret[i]=dVal[i];
			return ret;
		}

		return new int[]{0};
	}
	
	@Override protected short[] 	i_getEnumArray(String pvName) 			{ return new short[]{0}; }
	@Override protected String 	i_getString(String pvName, int decimals)
	{
		Object val = valueMap.get(pvName);
		if (val instanceof String) return (String)val;
		if (val instanceof Integer) return Integer.toString((Integer)val);
		if (val instanceof Double) return Util.printDouble((Double)val, decimals);
		if (null == val) return "";
		return "0";
	}
	
	@Override protected String i_getCacheString(String pvName, int decimals)
	{
		return i_getString(pvName,decimals);
	}

	
	@Override protected DBR i_getDBR(String pvName)
	{
		return new DBR_Int(new int[]{0});
	}
	

	@Override protected DBR i_getCacheDBR(String pvName)
	{
		return new DBR_Int(new int[]{0});
	}
	
	@Override protected void i_putInteger(final String pvName, final int value)	{ _put(pvName, value); }
	@Override protected void i_putBoolean(String pvName, boolean value)			{ _put(pvName, value?1:0); }
	@Override protected void i_putDouble(String pvName, double value)			{ _put(pvName, value); }
	@Override protected void i_putDoubleArray(String pvName, double[] values)
	{
		Double[] objValues = new Double[values.length];
		for (int i=0;i<values.length;i++) objValues[i]=values[i];
		_put(pvName, objValues);
	}
	@Override protected void i_putIntegerArray(String pvName, int[] values)
	{
		Integer[] objValues = new Integer[values.length];
		for (int i=0;i<values.length;i++) objValues[i]=values[i];
		_put(pvName, objValues);
	}
	@Override protected void i_put(String pvName, String[] value)				{ _put(pvName, value); }
	@Override protected void i_put(String pvName, String value)					{ _put(pvName, value); }
	@Override protected boolean i_isConnected(String pvName) 					{ return true; }
	
	protected void _put(final String pvName, final Object value)
	{
		System.out.println("caput "+pvName+" "+Epics.dbrToString(objectToDBR(value)));
		
		valueMap.put(pvName, value);
		
		final ArrayList<EpicsMonitor> monitorList = imonMap.get(pvName);
		if (null == monitorList) return;
		
		for (final EpicsMonitor monitor : monitorList)
		{
    		GUI.asyncExec(new Runnable(){public void run()
    		{
    			monitor.connectionChanged(pvName, true);
    			monitor.valueChanged(pvName, objectToDBR(value));
    		}});
		}
	}

	@Override protected void i_connect()
	{
		try
		{
			// JCA still needs to be initialized even if no real PVs are connected
			JCALibrary jca = JCALibrary.getInstance();
			DefaultConfiguration conf = new DefaultConfiguration("context");
			conf.setAttribute("class", JCALibrary.CHANNEL_ACCESS_JAVA);
			conf.setAttribute("addr_list", "");
			jca.createContext(conf);
		}
		catch(Exception ex) { ex.printStackTrace(); }
	}
	
	@Override protected void i_pendIO(int ms) {}
	@Override protected void i_poll() {}
	@Override protected void i_flushIO() {}
	@Override protected void i_startPollThread() {}
	@Override protected void i_disconnectMonitors() {}
	
	public DBR objectToDBR(Object obj)
	{
		if (obj instanceof Double) return new DBR_Double(new double[]{(Double)obj});
		if (obj instanceof Integer) return new DBR_Int(new int[]{(Integer)obj});
		if (obj instanceof String) return new DBR_String(new String[]{(String)obj});
		if (obj instanceof Double[])
		{
			Double[] objArray = (Double[])obj;
			double[] primitiveArray = new double[objArray.length];
			for (int i=0;i<objArray.length;i++) primitiveArray[i]=objArray[i]; 
			return new DBR_Double(primitiveArray);
		}
		if (obj instanceof Integer[])
		{
			Integer[] objArray = (Integer[])obj;
			int[] primitiveArray = new int[objArray.length];
			for (int i=0;i<objArray.length;i++) primitiveArray[i]=objArray[i]; 
			return new DBR_Int(primitiveArray);
		}
		if (obj instanceof String[])
		{
			String[] objArray = (String[])obj;
			return new DBR_String(objArray);
		}
		
		return new DBR_Double(new double[]{0.0});
	}

	@Override protected String[] i_getPVNames()
	{
		ArrayList<String> ret = new ArrayList<String>();
		for (String key : valueMap.keySet()) ret.add(key);
		Collections.sort(ret);
		return ret.toArray(new String[]{});
	}
	
	@Override protected void i_disablePV(PVName pvName, Object defaultValue) {}
	
	// in software mode, some PVs may not be set correctly, so return something that might work
	@Override protected int i_dbrToInt(DBR dbr)
	{
		try 				{ return gov.anl.gmca.io.Epics.d_dbrToInt(dbr); }
		catch (Exception e) { return 0;	}
	}
	@Override protected double i_dbrToDouble(DBR dbr)
	{
		try 				{ return gov.anl.gmca.io.Epics.d_dbrToDouble(dbr); }
		catch (Exception e) { return 0.0;	}
	}
	@Override protected int[] i_dbrToIntArray(DBR dbr)
	{
		try 				{ return gov.anl.gmca.io.Epics.d_dbrToIntArray(dbr); }
		catch (Exception e) { return new int[]{0};	}
	}
	@Override protected double[] i_dbrToDoubleArray(DBR dbr)
	{
		try 				{ return gov.anl.gmca.io.Epics.d_dbrToDoubleArray(dbr); }
		catch (Exception e) { return new double[]{0.0};	}
	}
	@Override protected String i_dbrToString(DBR dbr, int decimalsToKeep)
	{
		try 				{ return gov.anl.gmca.io.Epics.d_dbrToString(dbr,decimalsToKeep); }
		catch (Exception e) { return "0";	}
	}

	@Override protected boolean i_checkConnection(String pvName, int timeoutMs)
	{
		return true;
	}
	
	@Override protected int i_getEpicsTaskCount()
	{
		return 0;
	}
}
