package gov.anl.gmca.io.sim;

import gov.anl.gmca.common.DevMove.MoveCmd;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.io.sim.dev.*;
import gov.anl.gmca.scan.CmdChooch;

import java.util.StringTokenizer;

public class ExecTask extends gov.anl.gmca.io.ExecTask
{
	protected boolean stop = false;
	
	public ExecTask (final String cmd, final String cmdFilename, final boolean logOutput, final boolean logError)
	{
		init(cmd,cmdFilename,logOutput,logError);
	}
	
	public ExecTask (final String cmd, final String cmdFilename, final boolean logOutput)
	{
		init(cmd,cmdFilename,logOutput,true);
	}

	protected void init(final String cmd, final String cmdFilename, final boolean logOutput, final boolean logError)
	{
		System.out.println("EXEC: "+cmd);
		thread = new Thread(new Runnable(){public void run()
		{
			if (cmd.contains("svn"))
			{
				stdout.add("URL: svn+ssh://bl2upper/mnt/software/svnrepos/bluice/branches/SIM-MODE");
				stdout.add("Revision: 1");
				exitValue=0;
			}
			else if (cmd.contains("whoami"))
			{
				stdout.add("gmca");
				exitValue=0;
			}
			else if (cmd.contains("checkpw"))
			{
				stdout.add("0");
				exitValue=0;
			}
			else if (cmd.contains("hostname"))
			{
				stdout.add("bl3ws3.gmca.aps.anl.gov");
				exitValue=0;
			}
			else if (cmd.contains("bigEnergyMove"))
			{
				StringTokenizer st = new StringTokenizer(cmd);
				while (!st.nextToken().contains("bigEnergyMove")) {}
				try
				{
					double dest = Double.parseDouble(st.nextToken());
					MoveCmd monoMove = DevPmacMotor.get("mo_En_E").startMove(dest);
					monoMove.waitFor();
				}
				catch (NumberFormatException nfe) { nfe.printStackTrace(); }
			}
			else if (cmd.contains("edgeScan.pl"))
			{
				SimEdgeScan edgeScan = new SimEdgeScan();
				edgeScan.run();
			}
			else if (cmd.contains("alignMiniBeam.pl"))
			{
				SimAlignMiniBeam alignMiniBeam = new SimAlignMiniBeam();
				alignMiniBeam.run();
			}
			else if (cmd.contains("distl"))
			{
				StringTokenizer st = new StringTokenizer(cmd);
				st.nextToken();
				String filename = st.nextToken();
				
				SimDistl distl = new SimDistl(filename, stdout, stderr);
				distl.run();
			}
			else if (cmd.contains("autoraster_xrec.pl"))
			{
				SimAutoFind autoFind = new SimAutoFind();
				autoFind.run();
			}
			else if (cmd.indexOf(CmdChooch.choochDir+CmdChooch.choochFile)==0)
			{
				StringTokenizer st = new StringTokenizer(cmd);
				st.nextToken(); // chooch binary path
				
				if (!st.nextToken().equals("-e")) throw new IllegalArgumentException("Unsupported chooch argument order");
				String element=st.nextToken();
				
				if (!st.nextToken().equals("-a")) throw new IllegalArgumentException("Unsupported chooch argument order");
				String edge=st.nextToken();

				if (!st.nextToken().equals("-o")) throw new IllegalArgumentException("Unsupported chooch argument order");
				String efsPath=st.nextToken();

				//if (!st.nextToken().equals("-p")) throw new IllegalArgumentException("Unsupported chooch argument order");
				//String psPath=st.nextToken();

				String datPath=st.nextToken();
				
				SimChooch chooch = new SimChooch(element, edge, efsPath, null, datPath, stdout, stderr);
				chooch.run();
			}
		}}, "sim ExecTask "+cmdFilename);
	}
	
	@Override public void syncStop()
	{
		stop = true;
	}
	
	@Override public void syncKill()
	{
		stop = true;
	}
}
