package gov.anl.gmca.io.sim.dev;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Resources;
import gov.anl.gmca.io.Web.WebConnection;
import gov.anl.gmca.raster.calc.RasterGeometry;
import gov.anl.gmca.sample.DBCamera;
import gov.anl.gmca.sample.WdgCamera.CameraSize;
import gov.anl.gmca.sample.WdgCamera.CameraType;
import gov.anl.gmca.util.Point4D;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;

// camera images are taken centered at 1x zoom
public class SimCamera
{
	Object[][] fileLookup =
	{
		{ CameraType.LOWRES, 	CameraSize.SMALL, "sim-lowres-small.png" },
		{ CameraType.LOWRES, 	CameraSize.LARGE, "sim-lowres.png" },
		{ CameraType.HIGHRES, 	CameraSize.SMALL, "sim-highres-small.png" },
		{ CameraType.HIGHRES, 	CameraSize.LARGE, "sim-highres.png" },
		{ CameraType.HUTCH, 	CameraSize.SMALL, "sim-hutch-crystal-small.png" },
		{ CameraType.HUTCH, 	CameraSize.LARGE, "sim-hutch-crystal.png" },
	};

	public static SimCamera get() { return inst; }
	private static SimCamera inst = null;
	private Point4D refPos = null;
	
	public SimCamera()
	{
		inst = this;
	}
	
	public WebConnection getWebResponse(String urlString)
	{
		for (Object[] fileRow : fileLookup)
		{
			CameraType type = (CameraType)fileRow[0];
			CameraSize size = (CameraSize)fileRow[1];
			String rowUrl = DBCamera.getImageURL(type, size);
			
			if (rowUrl.equals(urlString))
			{
				String filename = (String)fileRow[2];
				InputStream is = generateImage(type, size, filename);
				return null;// new WebConnection(is);
			}
		}
		
		return null;
	}
	
	private Image procImage(Image diskImg)
	{
		if (null == refPos) refPos = RasterGeometry.getCurrentMotorPositions();

		Point4D currentPos = RasterGeometry.getCurrentMotorPositions();
		double[] posDiff = new double[]{refPos.x-currentPos.x,refPos.y-currentPos.y,refPos.z-currentPos.z,0.0};
		double[] screenXY = new double[]{0,0}; // FIXME: calcVisualXY(CameraType.HIGHRES, true, posDiff, 1.0);
		
		Image img = new Image(GUI.getDisplay(), diskImg.getBounds().width, diskImg.getBounds().height);
		GC imgGC = new GC(img);
		imgGC.drawImage(diskImg, (int)screenXY[0], (int)screenXY[1]);
		imgGC.dispose();
		
		return img;
	}
	
	private InputStream generateImage(CameraType type, CameraSize size, String filename)
	{
		String fullpath = "/images/sim/"+filename;
		InputStream imgStream = Resources.get().getInputStream(fullpath);
		
		if (CameraType.HIGHRES==type && CameraSize.LARGE==size)
		{
			// convert file to image
			ImageLoader loader = new ImageLoader();
			loader.load(imgStream);
			Image diskImg = new Image(GUI.getDisplay(), loader.data[0]);
			
			// process image
			Image procImg = procImage(diskImg);
			
			// convert image to input stream
			ByteArrayOutputStream os = new ByteArrayOutputStream();
			loader = new ImageLoader();
			loader.data = new ImageData[]{procImg.getImageData()};
			loader.save(os, SWT.IMAGE_PNG);
			InputStream ret = new ByteArrayInputStream(os.toByteArray());
			
			// return the stream
			return ret;
		}
		else
		{
			// return a direct stream to the resource
			return imgStream;
		}
	}
}
