package gov.anl.gmca.log;

import gov.anl.gmca.analysis.autoProcessData;
import gov.anl.gmca.analysis.autoProcessData.PROCESSING_MODE;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.dataproc.InitDataProcessing;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.*;
import gov.anl.gmca.log.db.DevFrameauditMsg;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * implements the LogTab UI and initiates
 * pre experimental log thread and data processing thread.
 */

public class TabLog extends Tab implements ClientListener {

	public static String userName = "";
	public static String sampleName = "";
	public static String projectName = "";
	public static String sampleXSize = "0";
	public static String sampleYSize = "0";
	public static String sampleZSize = "0";
	public static String comments = "";

	public static Text usernameLabel;
	public static Text sampleNameLabel;
	public static Text projectNameLabel;
	public static Text sampleXSizeLabel;
	public static Text sampleYSizeLabel;
	public static Text sampleZSizeLabel;

	public Label pindiodegainLabel;
	public Label i0gainLabel;

	protected Epics.EpicsValueMonitor monitor = null;
	
	static volatile boolean runChange = false;
	public static boolean scrLog = false;
	
	public static int[] countLog;
	public static boolean[] stopLog;
	
	private String[] storedRunStatus = new String[16];
	private String[] storedRunPrefix = new String[16];
	
	private Button saveLog;
	WritePreExperimentalLog log;
	InitDataProcessing initProcessing;
	int nLimit;
	int nStateSwitches = 0;
	
	public static void create() {
		Util.safeSyncExec(new Runnable() {
			public void run() {
				TabLog tl;
				try {
					tl = new TabLog(new Shell(GUI.getDisplay()), SWT.NONE);
					GUI.tabMap.put(tl.getTabName(), tl);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
	
	public TabLog(Composite parent, int style) {
		super(parent, style);
		Client.addListener(this);
		
		countLog = new int[ModelCollectRunList.MAX_COUNT];
		stopLog = new boolean[ModelCollectRunList.MAX_COUNT];
		
		Log.note("TabLog: started");
		setLayout(new GridLayout(1, false));
		setBackground(GUI.BGCOLOR);
		
		GridData gd = new GridData();
		Composite actionComp = new Composite(this, SWT.NONE);
		actionComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(5, false);
		gd.widthHint = 980;
		gd.widthHint = 600;
		actionComp.setLayout(gl);
		gd = new GridData();
		gd.verticalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		actionComp.setLayoutData(gd);

		constructProjectWdgs(actionComp, style);
		constrctSampleDetails(actionComp, style);
		constrctIntensityMonitors(actionComp, style);

		new WdgLogDataCollection(actionComp, style);

		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.horizontalSpan = 5;
		saveLog = new Button(actionComp, SWT.PUSH);
		saveLog.setBackground(GUI.COLOR_BLUE_BUTTON);
		saveLog.setText("Generate Log");
		saveLog.setLayoutData(gd);
		saveLog.setEnabled(false);
		saveLog.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent arg0) {
				WritePreExperimentalLog log = new WritePreExperimentalLog();
				log.writeLog();
			}

		});
		new DevFrameauditMsg();
		log = new WritePreExperimentalLog();
		
		Log.note("TabLog: done");
	}

	public void constructProjectWdgs(Composite parent, int style) {
		Group groupProject = new Group(parent, SWT.NONE);
		groupProject.setBackground(parent.getBackground());
		groupProject.setFont(GUI.FONT_HUTCH_GROUP);
		groupProject.setText("Project Detals");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		GridData gd = new GridData();
		gd.widthHint = 280;
		gd.heightHint = 160;
		// gd.grabExcessHorizontalSpace = true;
		// gd.horizontalAlignment = SWT.FILL;
		groupProject.setLayoutData(gd);
		groupProject.setLayout(new GridLayout(2, false));

		Label userLabel = new Label(groupProject, SWT.NONE);
		userLabel.setText("User Name :");
		userLabel.setBackground(parent.getBackground());
		gd = new GridData();
		userLabel.setLayoutData(gd);

		usernameLabel = new Text(groupProject, SWT.BORDER);
		userName = System.getProperty("user.name");
		usernameLabel.setText(userName);
		gd = new GridData();
		gd.widthHint = 35;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		usernameLabel.setLayoutData(gd);

		usernameLabel.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent arg0) {
				userName = usernameLabel.getText();
			}

		});
		Label projectLabel = new Label(groupProject, SWT.NONE);
		projectLabel.setText("Project Name :");
		projectLabel.setBackground(parent.getBackground());
		gd = new GridData();
		projectLabel.setLayoutData(gd);

		projectNameLabel = new Text(groupProject, SWT.BORDER);
		gd = new GridData();
		gd.widthHint = 35;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		projectNameLabel.setLayoutData(gd);

		projectNameLabel.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent arg0) {
				projectName = projectNameLabel.getText();
			}

		});
		Label dateLabel = new Label(groupProject, SWT.NONE);
		dateLabel.setText("Date :");
		dateLabel.setBackground(parent.getBackground());
		gd = new GridData();
		dateLabel.setLayoutData(gd);
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		Date date = new Date();

		Label dateDisplayLabel = new Label(groupProject, SWT.NONE);
		dateDisplayLabel.setText(dateFormat.format(date));
		dateDisplayLabel.setBackground(parent.getBackground());
		gd = new GridData();
		dateDisplayLabel.setLayoutData(gd);

		Label beamlineLabel = new Label(groupProject, SWT.NONE);
		beamlineLabel.setText("Beamline :");
		beamlineLabel.setBackground(parent.getBackground());
		gd = new GridData();
		beamlineLabel.setLayoutData(gd);

		String beamline = DBConfig.get("app_beamlineID");
		String beamlineID = "";
		if (beamline.equals("23i")) {
			beamlineID = "23ID-D";
		} else if (beamline.equals("23o")) {
			beamlineID = "23ID-B";
		} else if (beamline.equals("23b")) {
			beamlineID = "23BM";
		}
		Label beamlineDisplayLabel = new Label(groupProject, SWT.NONE);
		beamlineDisplayLabel.setText(beamlineID);
		beamlineDisplayLabel.setBackground(parent.getBackground());
		gd = new GridData();
		beamlineDisplayLabel.setLayoutData(gd);

		// get current runStatus for each run
		getAllRunStatus();
		
		monitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				final String _pvName = pvName;
				final DBR _value = value;

				if (_pvName.equals(PV_KEITHLEY.ActGain.index(3).toString())) {
					final int stat = Epics.dbrToInt(_value);
					GUI.asyncExec(new Runnable() {
						public void run() {
							pindiodegainLabel.setText("I0 :            " + getI0Gain(stat));
						}
					});
				}
				if (_pvName.equals(PV_KEITHLEY.ActGain.index(6).toString())) {
					final int stat = Epics.dbrToInt(_value);
					GUI.asyncExec(new Runnable() {
						public void run() {
							i0gainLabel.setText("Pindiode : " + getI0Gain(stat));
						}
					});
				}
				if (!Client.isMaster()) {
					return;
				}
				int isRaster = Epics.getInteger(PV_R_RUNS.rasterState);

				if (isRaster > 0) {
					return;
				}

				int isFrameaudit = Epics.getInteger(PV_FRAMEAUDIT.sts);

				if (_pvName.equals(PV_FPE.lastfilename.toString())) {

					String frame = Epics.dbrToString(_value);
					InitDataProcessing.frame_name = frame;

					if (frame.equals("") || frame.equals(" ")) {
						return;
					}

					if (frame.contains("_scr")) {
						writeScreeningLog(frame, isFrameaudit, -1, true);
					}
					int run = Epics.getInteger(PV_RUNS.currentRun);
					if (initProcessing != null) {
						countLog[run]++;
						log.countLog++;
					}
				}

				if (_pvName.equals(PV_SCR_RUN.runStatus.run(0).toString())) {
					String stat = Epics.dbrToString(_value);

					/*
					 * if(stat.equals("Collecting")) { String frame =
					 * Epics.getString(PV_FPE.lastfilename); startExeprimentalLog(frame,
					 * isFrameaudit, 0, true); }
					 */
				}

				for (int i = 0; i < 16; i++) {
					if (_pvName.equals(PV_RUN.runStatus.run(i).toString())) {
						int runNumber = i;
						String stat = Epics.dbrToString(_value);
						String frame = Epics.getString(PV_FPE.lastfilename);

						if (stat.equals("complete") || stat.equals("paused") || stat.equals("inactive")) {
							log.stopLog = true;
							if (initProcessing != null) {
								stopLog[i] = true;
								countLog[i] = 0;
							}
							log.countLog = 0;
						} else if (stat.equals("collecting")) {
							System.out.print("Analysis: Collect state for procesisng: " + stat + " for run" + i);
							InitDataProcessing.frame_name = frame;
							startExeprimentalLog(frame, isFrameaudit, i, true);

							startProcessingThread(frame, isFrameaudit, i, true); // also initiate processing
						}
						
						// run fastdp etc if data collection is paused 
						// TODO test
						if (stat.equals("paused") && DevUserConfig.get().getXDSProc() != 0) {
							Log.note("starting processing job when data collection is paused.");
							autoProcessData ap = new autoProcessData(i, PROCESSING_MODE.CUSTOM_COLLECTION);
							new Thread(ap).start();
						}
					}
				}
			}
		};
		
		for (int i = 0; i < 16; i++) {
			Epics.addMonitor(PV_RUN.runStatus.run(i), monitor);
		}
		Epics.addMonitor(PV_SCR_RUN.runStatus.run(0), monitor);
		Epics.addMonitor(PV_FRAMEAUDIT.msg, monitor);
		Epics.addMonitor(PV_RUNS.collectState, monitor);
		Epics.addMonitor(PV_FPE.lastfilename, monitor);
		Epics.addMonitor(PV_KEITHLEY.ActGain.index(3), monitor);
		Epics.addMonitor(PV_KEITHLEY.ActGain.index(6), monitor);
		Epics.addMonitor(PV_RUNS.currentRun, monitor);
		Epics.addMonitor(PV_RUNS.bgProc, monitor);
	}
	
	public void writeScreeningLog(String frame, int isFrameaudit, int runNumber, boolean isProcessing) {
		
		if (frame.contains("_scr")) {
			scrLog = true;

			/**
			 * Start pre experimental log thread for screening
			 * tab
			 */
				log.currentRunStr = Integer.toString(runNumber);
				Thread logThread =  new Thread(log, "scr Log");
				logThread.start();

			return;
		}
	}
	/*
	* Starts pre experimental log and data processing threads
	* 
	*/
	public void startExeprimentalLog (String frame, int isFrameaudit, int runNumber, boolean isProcessing) {
		int collectState = Epics.getInteger(PV_RUNS.collectState);
		System.out.println("Log: Overall collect state: "+collectState);
		
		runChange = false;
		log.stopLog = false;
		
		/**
		 * Start pre experimental log thread for collect tab
		 */
		
		log.currentRunStr = Integer.toString(runNumber);
		Thread logThread =  new Thread(log, "Collect Log Thread");
		logThread.start();
	}
	
	public void startProcessingThread (String frame, int isFrameaudit, int runNumber, boolean isProcessing) {
		if (!DBConfig.getBoolean("processing_enable_analysis_legacy")) {
			Log.note("AnalysisLegacy tab was disabled, turn on using option/processing_enable_analysis_legacy.");
			return;
		}
		
		int collectState = Epics.getInteger(PV_RUNS.collectState);
		System.out.println("Analysis: Overall collect state: "+collectState);
		
		if (collectState < 2) {
			
			if(runNumber == 0 ||
					((ModelCollectRun)ModelCollectRunList.get().getRun(runNumber)).getSiteCheckEnable())
				return;
			
			/**
			* Start data processing Thread 
			* 
			*/
			runChange = false;
				
			System.out.println("Analysis: New processing thread started for run "+runNumber);
			initProcessing = new InitDataProcessing();
			stopLog[runNumber] = false;
			initProcessing.runNumber = runNumber;
			countLog[runNumber]++;
			
			Thread processThread = new Thread(initProcessing, "Data Processing Thread for run "+runNumber);
			processThread.start();

		}
	}
	
	public void constrctSampleDetails(Composite parent, int style) {
		Group groupSample = new Group(parent, SWT.NONE);
		groupSample.setBackground(parent.getBackground());
		groupSample.setFont(GUI.FONT_HUTCH_GROUP);
		groupSample.setText("Sample Detials");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		GridData gd = new GridData();
		gd.widthHint = 280;
		gd.heightHint = 172;
		// gd.grabExcessHorizontalSpace = true;
		// gd.horizontalAlignment = SWT.FILL;
		groupSample.setLayoutData(gd);
		groupSample.setLayout(new GridLayout(2, false));

		Label sampleLabel = new Label(groupSample, SWT.NONE);
		sampleLabel.setText("Sample :");
		sampleLabel.setBackground(parent.getBackground());
		gd = new GridData();
		sampleLabel.setLayoutData(gd);

		sampleNameLabel = new Text(groupSample, SWT.BORDER);
		sampleNameLabel.setText("sample");
		gd = new GridData();
		sampleNameLabel.setLayoutData(gd);
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		sampleNameLabel.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent arg0) {
				sampleName = sampleNameLabel.getText();
			}

		});
		Label sampleXLabel = new Label(groupSample, SWT.NONE);
		sampleXLabel.setText("Sample Size X:");
		sampleXLabel.setBackground(parent.getBackground());
		gd = new GridData();
		sampleXLabel.setLayoutData(gd);

		sampleXSizeLabel = new Text(groupSample, SWT.BORDER);
		sampleXSizeLabel.setText("0");
		gd = new GridData();
		gd.widthHint = 35;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		sampleXSizeLabel.setLayoutData(gd);

		sampleXSizeLabel.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		sampleXSizeLabel.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent arg0) {
				sampleXSize = sampleXSizeLabel.getText();
			}

		});
		Label sampleYLabel = new Label(groupSample, SWT.NONE);
		sampleYLabel.setText("Sample Size Y :");
		sampleYLabel.setBackground(parent.getBackground());
		gd = new GridData();
		sampleYLabel.setLayoutData(gd);

		sampleYSizeLabel = new Text(groupSample, SWT.BORDER);
		sampleYSizeLabel.setText("0");
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		sampleYSizeLabel.setLayoutData(gd);
		sampleYSizeLabel.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent arg0) {

				sampleYSize = sampleYSizeLabel.getText();
			}

		});
		sampleYSizeLabel.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		sampleYSizeLabel.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent arg0) {
				sampleYSize = sampleYSizeLabel.getText();
			}

		});

		Label sampleZLabel = new Label(groupSample, SWT.NONE);
		sampleZLabel.setText("Sample Size Z : ");
		sampleZLabel.setBackground(parent.getBackground());
		gd = new GridData();
		sampleZLabel.setLayoutData(gd);

		sampleZSizeLabel = new Text(groupSample, SWT.BORDER);
		sampleZSizeLabel.setText("0");
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		sampleZSizeLabel.setLayoutData(gd);
		sampleZSizeLabel.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent arg0) {
				sampleZSize = sampleZSizeLabel.getText();
			}

		});
		sampleZSizeLabel.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
	}

	public void constrctIntensityMonitors(Composite parent, int style) {
		Group groupIntensity = new Group(parent, SWT.NONE);
		groupIntensity.setBackground(parent.getBackground());
		groupIntensity.setFont(GUI.FONT_HUTCH_GROUP);
		groupIntensity.setText("Intensity Monitors");
		GridData gd = new GridData();
		gd.heightHint = 160;
		gd.horizontalSpan = 2;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupIntensity.setLayoutData(gd);
		groupIntensity.setLayout(new GridLayout(3, false));

		new WdgParameter(groupIntensity, SWT.NONE,
				"APS Current : ", PV_MISC.apsCurrent, "mA", 0);
		new WdgParameter(groupIntensity, SWT.NONE, "DCM : ",
				PV_MISC.dcm, "V", 0);
		new WdgParameter(groupIntensity, SWT.NONE, "I0 : ",
				PV_MISC.localIZero, "V", 0);
		//new WdgParameter(groupIntensity, SWT.NONE,
		//		"Iabs : ", PV_MISC.Iabs, "V", 0);
		new WdgParameter(groupIntensity, SWT.NONE,
				"Attenuation : ", PV_BLUICE.attenuation, "factors", 0);

		Group groupGain = new Group(parent, SWT.NONE);
		groupGain.setBackground(parent.getBackground());
		groupGain.setFont(GUI.FONT_HUTCH_GROUP);
		groupGain.setText("Gain");
		gd = new GridData();
		gd.heightHint = 160;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupGain.setLayoutData(gd);
		groupGain.setLayout(new GridLayout(1, false));

		/*
		 * WdgParameter i0gain = new WdgParameter(groupGain, SWT.NONE, "I0 : ",
		 * "keithley6:ActGain", "", 0);
		 */
		gd = new GridData();
		i0gainLabel = new Label(groupGain, SWT.NONE);
		i0gainLabel.setText("I0 : ");
		i0gainLabel.setBackground(parent.getBackground());
		i0gainLabel.setLayoutData(gd);

		/*
		 * WdgParameter pindiodegain = new WdgParameter(groupGain, SWT.NONE,
		 * "Pindiode : ", "keithley3:ActGain", "", 0);
		 */
		gd = new GridData();
		pindiodegainLabel = new Label(groupGain, SWT.NONE);
		pindiodegainLabel.setText("Pindiode : ");
		pindiodegainLabel.setBackground(parent.getBackground());
		pindiodegainLabel.setLayoutData(gd);

	}

	public static String getI0Gain(int val) {
		if (val == 0)
			return "IE03";
		else if (val == 1)
			return "IE04";
		else if (val == 2)
			return "IE05";
		else if (val == 3)
			return "IE06";
		else if (val == 4)
			return "IE07";
		else if (val == 5)
			return "IE08";
		else if (val == 6)
			return "IE09";
		else if (val == 7)
			return "IE10";
		else
			return "";
	}

	public void updateEnableState() {
		Util.safeSyncExec(new Runnable() {
			public void run() {
				if (Client.isMaster()) {
					saveLog.setEnabled(true);
				} else {
					saveLog.setEnabled(false);
				}
			}
		});
	}

	private void getAllRunStatus() {
		for (int i = 0; i < 16; i++) {
			storedRunStatus[i] = Epics.getString(PV_RUN.runStatus.run(i));
			storedRunPrefix[i] = Epics.getString(PV_RUN.fileroot.run(i));
		}
	}
	
	@Override
	public void masterChanged(boolean isMaster) { 
		updateEnableState(); 
		getAllRunStatus();
	}
	
	@Override
	public String getTabName() { return "Log"; }
	
	@Override
	public void notifyIsVisible(boolean isVisible) {
		WdgParameter.isUpdate = isVisible;
	}
}
