package gov.anl.gmca.log;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.*;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class WdgLogDataCollection {
	GridData gd = new GridData();
	protected Epics.EpicsValueMonitor monitor = null;

	public static volatile Label beamX;
	public static volatile Label beamY;
	WdgParameter omegaStart, frameWidth, endAngle, exposureTime, wedge,
			currentRun, energy1;
	WdgParameter stepSize, framesPerSite, overlap;
	static Text commentLabel;
	Label collimatorLabel;
	Label frameauditLabel;
	static String comments = "";
	int previousRun;

	public WdgLogDataCollection(Composite parent, int style) {
		constrcutDataCollection(parent, style);
		constructGonio(parent, style);
		constructDetector(parent, style);
		constrcutVector(parent, style);
		constcructBeamstop(parent, style);
		constructBeamSize(parent, style);

		Group groupFrameaudit = new Group(parent, SWT.NONE);
		groupFrameaudit.setBackground(parent.getBackground());
		groupFrameaudit.setFont(GUI.FONT_HUTCH_GROUP);
		groupFrameaudit.setText("FrameAudit Message");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData();
		gd.widthHint = 250;
		gd.heightHint = 50;
		gd.horizontalSpan = 5;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupFrameaudit.setLayoutData(gd);
		groupFrameaudit.setLayout(new GridLayout(3, false));

		frameauditLabel = new Label(groupFrameaudit, SWT.WRAP);
		gd = new GridData();
		gd.horizontalSpan = 5;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		frameauditLabel.setLayoutData(gd);

		commentLabel = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		gd = new GridData();
		gd.heightHint = 150;
		gd.horizontalSpan = 5;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		commentLabel.setLayoutData(gd);

		commentLabel.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent arg0) {
				// TODO Auto-generated method stub
				comments = commentLabel.getText();
			}

		});
		previousRun = Epics.getInteger(PV_RUNS.currentRun);
		// previousRun = run1;
		monitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				final String _pvName = pvName;
				final DBR _value = value;
				int run = Epics.getInteger(PV_RUNS.currentRun);

				final PVName rpName = PV_RUN.fileroot.run(run);
				if (_pvName.equals(rpName.toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							final String fileName = Epics.getString(rpName);
							TabLog.sampleNameLabel.setText(fileName);
							if (!Client.isMaster())
								TabLog.projectNameLabel.setText("project_"
										+ fileName);
						}
					});
				}
				if (_pvName.equals(PV_RUN.startAngle.run(run).toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							double val = Math.round(Epics.dbrToDouble(_value) * 1000.0) / 1000.0;
							omegaStart.parVal.setText(Double.toString(val));
						}
					});
				}
				if (_pvName.equals(PV_RUN.endAngle.run(run).toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							double val = Math.round(Epics.dbrToDouble(_value) * 1000.0) / 1000.0;
							endAngle.parVal.setText(Double.toString(val));
						}
					});
				}
				if (_pvName.equals(PV_RUN.delta.run(run).toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							double val = Math.round(Epics.dbrToDouble(_value) * 1000.0) / 1000.0;
							frameWidth.parVal.setText(Double.toString(val));
						}
					});
				}
				if (_pvName.equals(PV_RUN.exposureTime.run(run).toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							double val = Math.round(Epics.dbrToDouble(_value) * 1000.0) / 1000.0;
							exposureTime.parVal.setText(Double.toString(val));
						}
					});
				}
				if (_pvName.equals(PV_RUN.wedgeSize.run(run).toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							double val = Math.round(Epics.dbrToDouble(_value) * 1000.0) / 1000.0;
							wedge.parVal.setText(Double.toString(val));
						}
					});
				}
				if (_pvName.equals(PV_RUNS.currentRun.toString())) {
					updateCollectParams();
				}
				if (_pvName.equals(PV_COLLIMATOR.ActivePreset.toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							collimatorLabel.setText(getColli());
						}
					});
				}
				if (_pvName.equals(PV_FRAMEAUDIT.msg.toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							frameauditLabel.setText(Epics.dbrToString(_value));
							updateBeamCenter(WritePreExperimentalLog.beamx,
									WritePreExperimentalLog.beamy);
						}
					});
				}
				if (_pvName.equals(PV_FPE.lastfilename.toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							updateBeamCenter(WritePreExperimentalLog.beamx,
									WritePreExperimentalLog.beamy);
						}
					});
				}
				if (_pvName.equals(PV_RUNS.currentRun.toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							int runCount = Epics.dbrToInt(_value);
							// String rmRun = Epics.addBID("bi:r"+previousRun);
							Epics.removeMonitor(
									PV_RUN.fileroot.run(previousRun), monitor);
							Epics.removeMonitor(
									PV_RUN.startAngle.run(previousRun), monitor);
							Epics.removeMonitor(PV_RUN.delta.run(previousRun),
									monitor);
							Epics.removeMonitor(
									PV_RUN.exposureTime.run(previousRun),
									monitor);
							Epics.removeMonitor(
									PV_RUN.wedgeSize.run(previousRun), monitor);
							Epics.removeMonitor(
									PV_RUN.directory256.run(previousRun),
									monitor);
							Epics.removeMonitor(PV_RUN.stepSize_um.startAngle
									.run(previousRun), monitor);
							Epics.removeMonitor(
									PV_RUN.framesPerSite.run(previousRun),
									monitor);
							Epics.removeMonitor(
									PV_RUN.startAngle.run(previousRun), monitor);
							Epics.removeMonitor(
									PV_RUN.overlap.run(previousRun), monitor);

							int run = Epics.getInteger(PV_RUNS.currentRun);
							// String rp = Epics.addBID("bi:r"+run);
							Epics.addMonitor(PV_RUN.fileroot.run(run), monitor);
							Epics.addMonitor(PV_RUN.startAngle.run(run),
									monitor);
							Epics.addMonitor(PV_RUN.delta.run(run), monitor);
							Epics.addMonitor(
									PV_RUN.exposureTime.fileroot.run(run),
									monitor);
							Epics.addMonitor(PV_RUN.wedgeSize.run(run), monitor);
							Epics.addMonitor(PV_RUN.directory256.run(run),
									monitor);

							Epics.addMonitor(PV_RUN.stepSize_um.run(run),
									monitor);
							Epics.addMonitor(PV_RUN.framesPerSite.run(run),
									monitor);
							Epics.addMonitor(PV_RUN.overlap.run(run), monitor);

							previousRun = run;
						}
					});
				}
			}
		};
		// String run =
		// Epics.get().getString(Epics.addBID("bi:runs:currentRun"));
		int run = Epics.getInteger(PV_RUNS.currentRun);
		Epics.addMonitor(PV_RUN.fileroot.run(run), monitor);
		Epics.addMonitor(PV_RUN.startAngle.run(run), monitor);
		Epics.addMonitor(PV_RUN.delta.run(run), monitor);
		Epics.addMonitor(PV_RUN.exposureTime.fileroot.run(run), monitor);
		Epics.addMonitor(PV_RUN.wedgeSize.run(run), monitor);
		Epics.addMonitor(PV_RUN.directory256.run(run), monitor);

		Epics.addMonitor(PV_RUN.stepSize_um.run(run), monitor);
		Epics.addMonitor(PV_RUN.framesPerSite.run(run), monitor);
		Epics.addMonitor(PV_RUN.overlap.run(run), monitor);

		Epics.addMonitor(PV_RUNS.currentRun, monitor);
		Epics.addMonitor(PV_COLLIMATOR.ActivePreset, monitor);
		Epics.addMonitor(PV_FRAMEAUDIT.msg, monitor);
		Epics.addMonitor(PV_FPE.lastfilename, monitor);

	}

	public void updateBeamCenter(double x, double y) {

		final double _x = x;
		final double _y = y;
		GUI.asyncExec(new Runnable() {
			public void run() {
				beamX.setText(Double.toString(_x));
				beamY.setText(Double.toString(_y));
			}
		});

	}

	public void updateCollectParams() {

		int run = Epics.getInteger(PV_RUNS.currentRun);
		// String run =
		// Epics.get().getString(Epics.addBID("bi:r"+run_+":runLabel"));
		// String rp = Epics.addBID("bi:r"+run);
		String _prefix = Epics.getString(PV_RUN.fileroot.run(run));
		String _startFrameLabel = Epics.getString(PV_RUN.startFrameLabel
				.run(run));
		String _spindle = Epics.getString(PV_RUN.axisMotor.run(run));
		String _omegaStart = Epics.getString(PV_RUN.startAngle.run(run));
		String _omegaEnd = Epics.getString(PV_RUN.endAngle.run(run));
		String _frameWidth = Epics.getString(PV_RUN.delta.run(run));
		String _nextFrame = Epics.getString(PV_RUN.nextFrame.run(run));

		String _exposureTime = Epics.getString(PV_RUN.exposureTime.run(run));
		String _wedgeSize = Epics.getString(PV_RUN.wedgeSize.run(run));
		String _inverseBeam = Epics.getString(PV_RUN.inverseOn.run(run));
		String _energy = Epics.getString(PV_RUN.energy1.run(run));

		String _stepSize = Epics.getString(PV_RUN.stepSize_um.run(run));
		String _framesPerSite = Epics.getString(PV_RUN.framesPerSite.run(run));
		String _overlap = Epics.getString(PV_RUN.overlap.run(run));

		omegaStart.setValue(_omegaStart);
		frameWidth.setValue(_frameWidth);
		endAngle.setValue(_omegaEnd);
		exposureTime.setValue(_exposureTime);
		wedge.setValue(_wedgeSize);
		currentRun.setValue(Integer.toString(run));
		energy1.setValue(_energy);

		stepSize.setValue(_stepSize);
		framesPerSite.setValue(_framesPerSite);
		overlap.setValue(_overlap);
	}

	public void constrcutDataCollection(Composite parent, int style) {
		Group groupCollection = new Group(parent, SWT.NONE);
		groupCollection.setBackground(parent.getBackground());
		groupCollection.setFont(GUI.FONT_HUTCH_GROUP);
		groupCollection.setText("Data Collection");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData();
		gd.widthHint = 200;
		gd.heightHint = 195;
		gd.verticalSpan = 2;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupCollection.setLayoutData(gd);
		groupCollection.setLayout(new GridLayout(3, false));

		int run = Epics.getInteger(PV_RUNS.currentRun);
		omegaStart = new WdgParameter(groupCollection, SWT.NONE,
				"Omega Start :", PV_RUN.startAngle.run(run), "deg", 1);

		frameWidth = new WdgParameter(groupCollection, SWT.NONE,
				"Frame Width :", PV_RUN.delta.run(run), "deg", 1);

		endAngle = new WdgParameter(groupCollection, SWT.NONE, "End Angle :",
				PV_RUN.endAngle.run(run), "deg", 1);

		exposureTime = new WdgParameter(groupCollection, SWT.NONE,
				"ExposureTime :", PV_RUN.exposureTime.run(run), "sec", 1);

		wedge = new WdgParameter(groupCollection, SWT.NONE, "Wedge :",
				PV_RUN.wedgeSize.run(run), "deg", 1);

		currentRun = new WdgParameter(groupCollection, SWT.NONE,
				"Current Run :", PV_RUNS.currentRun, "", 0);

		energy1 = new WdgParameter(groupCollection, SWT.NONE, "Energy :",
				PV_RUN.energy1.run(run), "deg", 1);

	}

	public void constrcutVector(Composite parent, int style) {
		Group groupCollection = new Group(parent, SWT.NONE);
		groupCollection.setBackground(parent.getBackground());
		groupCollection.setFont(GUI.FONT_HUTCH_GROUP);
		groupCollection.setText("Vector Parameters");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData();
		gd.widthHint = 200;
		gd.heightHint = 195;
		gd.verticalSpan = 2;
		gd.horizontalSpan = 2;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupCollection.setLayoutData(gd);
		groupCollection.setLayout(new GridLayout(3, false));
		int run = Epics.getInteger(PV_RUNS.currentRun);
		stepSize = new WdgParameter(groupCollection, SWT.NONE, "stepSize :",
				PV_RUN.stepSize_um.run(run), "um", 0);
		framesPerSite = new WdgParameter(groupCollection, SWT.NONE,
				"framesPerSite :", PV_RUN.framesPerSite.run(run), "", 0);
		overlap = new WdgParameter(groupCollection, SWT.NONE, "overlap :",
				PV_RUN.overlap.run(run), "", 0);

	}

	public void constructGonio(Composite parent, int style) {
		Group groupGonio = new Group(parent, SWT.NONE);
		groupGonio.setBackground(parent.getBackground());
		groupGonio.setFont(GUI.FONT_HUTCH_GROUP);
		groupGonio.setText("Gonio");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData();
		gd.widthHint = 200;
		gd.heightHint = 70;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupGonio.setLayoutData(gd);
		groupGonio.setLayout(new GridLayout(3, false));

		WdgParameter omegaStart = new WdgParameter(groupGonio, SWT.NONE,
				"Xtal-X :", PV_PMAC.ActPos.motor("sample_x"), "mm", 0);

		WdgParameter frameWidth = new WdgParameter(groupGonio, SWT.NONE,
				"Xtal-Y :", PV_PMAC.ActPos.motor("sample_y"), "mm", 0);

		WdgParameter endAngle = new WdgParameter(groupGonio, SWT.NONE,
				"Xtal-Z :", PV_PMAC.ActPos.motor("sample_z"), "mm", 0);
	}

	public void constructDetector(Composite parent, int style) {
		Group groupDetector = new Group(parent, SWT.NONE);
		groupDetector.setBackground(parent.getBackground());
		groupDetector.setFont(GUI.FONT_HUTCH_GROUP);
		groupDetector.setText("Detector");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData();
		gd.widthHint = 250;
		gd.heightHint = 70;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupDetector.setLayoutData(gd);
		groupDetector.setLayout(new GridLayout(3, false));

		WdgParameter distance = new WdgParameter(groupDetector, SWT.NONE,
				"Distance :", PV_PMAC.ActPos.motor("detector_z"), "mm", 0);

		WdgParameter twoTheta = new WdgParameter(groupDetector, SWT.NONE,
				"2Theta :", PV_PMAC.ActPos.motor("detector_2t"), "deg", 0);

	}

	public void constructBeamSize(Composite parent, int style) {
		Group groupBeamsize = new Group(parent, SWT.NONE);
		groupBeamsize.setBackground(parent.getBackground());
		groupBeamsize.setFont(GUI.FONT_HUTCH_GROUP);
		groupBeamsize.setText("Beam Size");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData();
		gd.widthHint = 250;
		gd.heightHint = 80;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupBeamsize.setLayoutData(gd);
		groupBeamsize.setLayout(new GridLayout(3, false));

		Label colliLabel = new Label(groupBeamsize, SWT.NONE);
		colliLabel.setText("Collimator :");
		colliLabel.setBackground(parent.getBackground());
		gd = new GridData();
		colliLabel.setLayoutData(gd);

		collimatorLabel = new Label(groupBeamsize, SWT.BORDER);
		gd = new GridData();
		gd.widthHint = 35;
		gd.horizontalSpan = 2;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		collimatorLabel.setLayoutData(gd);
		collimatorLabel.setText(getColli());
		collimatorLabel.setAlignment(SWT.CENTER);

		WdgParameter gWidth = new WdgParameter(groupBeamsize, SWT.NONE,
				"GuardSlitX :", PV_PMAC.ActPos.motor("beam_size_x"), "mm", 0);
		WdgParameter gHeight = new WdgParameter(groupBeamsize, SWT.NONE,
				"GuardSlitY :", PV_PMAC.ActPos.motor("beam_size_y"), "mm", 0);
	}

	public void constcructBeamstop(Composite parent, int style) {

		Group groupBeamCenter = new Group(parent, SWT.NONE);
		groupBeamCenter.setBackground(parent.getBackground());
		groupBeamCenter.setFont(GUI.FONT_HUTCH_GROUP);
		groupBeamCenter.setText("Beam Center");
		// groupProject.setFont(GUI.FONT_MOTOR_GROUP);
		gd = new GridData();
		gd.widthHint = 250;
		gd.heightHint = 80;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		groupBeamCenter.setLayoutData(gd);
		groupBeamCenter.setLayout(new GridLayout(4, false));

		Label beamXLabel = new Label(groupBeamCenter, SWT.NONE);
		beamXLabel.setText("Beam X :");
		gd = new GridData();
		beamXLabel.setLayoutData(gd);

		beamX = new Label(groupBeamCenter, SWT.NONE);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		beamX.setLayoutData(gd);

		Label beamYLabel = new Label(groupBeamCenter, SWT.NONE);
		beamYLabel.setText("Beam Y :");
		gd = new GridData();
		beamYLabel.setLayoutData(gd);

		beamY = new Label(groupBeamCenter, SWT.NONE);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		beamY.setLayoutData(gd);

		WdgParameter collimator = new WdgParameter(groupBeamCenter, SWT.NONE,
				"Beamstop :", PV_PMAC.ActPos.motor("beamstop_z"), "mm", 0);

	}

	public String getColli() {
		int _collimator = Epics.getInteger(PV_COLLIMATOR.ActivePreset);
		if (_collimator == 0) {
			return ("None");
		} else if (_collimator == 1) {
			return ("ScatterGuard");
		} else if (_collimator == 2) {
			return ("20um");
		} else if (_collimator == 3) {
			return ("10um");
		} else if (_collimator == 4) {
			return ("5um");

		} else
			return ("None");
	}
}
