package gov.anl.gmca.log.db;

import gov.anl.gmca.common.SQLDataType;
import gov.anl.gmca.common.Units;
import gov.anl.gmca.io.Database.DBField;
import gov.anl.gmca.log.db.DevEvents.EventParam;

import java.util.ArrayList;
import java.util.HashMap;

public class CollectEvents {
	
	public static final String GMCA_COLLECT_DATA_TABLE = "CollectEvents";
	
	public static enum CollectParam implements EventParam
	{
							// db string		// units			// datatype						// allowNull	// primary key
		SAMPLE_NUMBER		("sampleNumber",	Units.NONE,			SQLDataType.INT,				false,			false),
		COLLECT_NUMBER		("collectNumber",	Units.NONE,			SQLDataType.AUTOINC,			false,			true),
		USER_NAME			("userName",		Units.NONE,			SQLDataType.STRING,				false,			false),
		COLLECT_TYPE		("collectType",		Units.NONE,			SQLDataType.STRING,				false,			false),
		ROOT_DIR			("workdir",			Units.NONE,			SQLDataType.STRING,				false,			false),
		PREFIX				("prefix",			Units.NONE,			SQLDataType.STRING,				false,			false),
		NUMIMAGES			("numimages",		Units.NONE,			SQLDataType.STRING,				false,			false),
		COLLIMATOR			("collimator",		Units.NONE,			SQLDataType.STRING,				false,			false),
		BEAM_SIZE			("beamsize_um",		Units.NONE,			SQLDataType.STRING,				false,			false),
		SLIT_WIDTH			("slitWidth",		Units.NONE,			SQLDataType.STRING,				false,			false),
		SLIT_HEIGHT			("slitHeight",		Units.NONE,			SQLDataType.STRING,				false,			false),
		IZERO				("izero",			Units.NONE,			SQLDataType.STRING,				false,			false),
		ACTIVE_BS			("active_bs",		Units.NONE,			SQLDataType.STRING,				false,			false),
		APS_CURRENT			("apsCurrent",		Units.NONE,			SQLDataType.STRING,				false,			false),
		DCM					("dcm",				Units.NONE,			SQLDataType.STRING,				false,			false),
		ATTENUATION			("attenuation",		Units.NONE,			SQLDataType.STRING,				false,			false),
		MONO_ENERGY			("finalEnergy",		Units.NONE,			SQLDataType.STRING,				false,			false),
		XTAL_X				("crystalX",		Units.NONE,			SQLDataType.STRING,				false,			false),
		XTAL_Y				("crystalY",		Units.NONE,			SQLDataType.STRING,				false,			false),
		XTAL_Z				("crystalZ",		Units.NONE,			SQLDataType.STRING,				false,			false),
		FIRST_FRAME			("firstFrame",		Units.NONE,			SQLDataType.STRING,				false,			false),
		DET_DISTANCE		("detectorDistance",Units.MM,			SQLDataType.STRING,				false,			false),
		TWOTHETA			("twotheta",		Units.DEG,			SQLDataType.STRING,				false,			false),
		BEAMSTOP			("beamstop",		Units.MM,			SQLDataType.STRING,				false,			false),
		OMEGA_START			("omegaStart",		Units.DEG,			SQLDataType.STRING,				false,			false),
		OMEGA_END			("omegaEnd",		Units.DEG,			SQLDataType.STRING,				false,			false),
		FRAME_WIDTH			("frameWidth",		Units.DEG,			SQLDataType.STRING,				false,			false),
		EXP_TIME			("exposureTime",	Units.SEC,			SQLDataType.STRING,				false,			false),
		INV_BEAM			("inverseBeam",		Units.SEC,			SQLDataType.STRING,				false,			false),
		WEDGE				("wedge",			Units.DEG,			SQLDataType.STRING,				false,			false),
		ENERGY1				("energy1",			Units.KEV,			SQLDataType.STRING,				false,			false),
		ENERGY2				("energy2",			Units.KEV,			SQLDataType.STRING,				false,			false),
		ENERGY3				("energy3",			Units.KEV,			SQLDataType.STRING,				false,			false),
		XDSPROC				("xdsproc",			Units.NONE,			SQLDataType.STRING,				false,			false),
		STEP_SIZE			("stepsize",		Units.NONE,			SQLDataType.STRING,				false,			false),
		NUMSITES			("numsites",		Units.NONE,			SQLDataType.STRING,				false,			false),
		FRAMESPERSITE		("framepersite",	Units.NONE,			SQLDataType.STRING,				false,			false),
		OVERLAP				("overlap",			Units.NONE,			SQLDataType.STRING,				false,			false),
		SITECHECK			("sitecheck",		Units.NONE,			SQLDataType.STRING,				false,			false),
		SITEDELTA			("sitedelta",		Units.DEG,			SQLDataType.STRING,				false,			false),
		BEAMX				("beamX",			Units.MM,			SQLDataType.STRING,				false,			false),
		BEAMY				("beamY",			Units.MM,			SQLDataType.STRING,				false,			false),
		SIZEX				("sizeX",			Units.MM,			SQLDataType.STRING,				false,			false),
		SIZEY				("sizeY",			Units.MM,			SQLDataType.STRING,				false,			false),
		SIZEZ				("sizeZ",			Units.MM,			SQLDataType.STRING,				false,			false),
		FRAMEAUDIT_MEG		("frameauditmsg",	Units.NONE,			SQLDataType.STRING,				false,			false),
		COMMENT				("comment",			Units.NONE,			SQLDataType.STRING,				false,			false),
		TIMESTAMP			("timestamp",		Units.TIMESTAMP,	SQLDataType.TIMESTAMP,			false, 			false);
		
		public String dbString;
		public Units units;
		public SQLDataType dataType;
		public boolean allowNull;
		public boolean isPrimaryKey;
		
		CollectParam(String dbString, Units units, SQLDataType dataType, boolean allowNull, boolean isPrimaryKey)
		{
			this.dbString=dbString;
			this.units=units;
			this.dataType=dataType;
			this.allowNull=allowNull;
			this.isPrimaryKey=isPrimaryKey;
		}
		
		public DBField getDBField() { return new DBField(dbString, dataType, allowNull, isPrimaryKey, null); }
	};
	
	public static void writeEvent(HashMap<CollectParam,String> fieldmap)
	{
		// first create a beamline event
		//int beamlineEventNumber = BeamlineEvent.writeEvent();

		// then create the collect event
		ArrayList<HashMap<String,String>> rows = new ArrayList<HashMap<String,String>>(); 
		HashMap<String,String> row = new HashMap<String,String>();
		for (CollectParam param : CollectParam.values())
		{
			switch (param)
			{
			case SAMPLE_NUMBER:			DevEvents.addInt	(row, param, DevEvents.getSampleNumber());	break;
			case COLLECT_NUMBER:		DevEvents.addInt	(row, param, 0);	break;
			case USER_NAME:				
			case COLLECT_TYPE	:
			case ROOT_DIR	:
			case PREFIX	:
			case NUMIMAGES	:
			case COLLIMATOR	:
			case BEAM_SIZE	:
			case SLIT_WIDTH	:
			case SLIT_HEIGHT	:
			case IZERO	:
			case ACTIVE_BS :
			case APS_CURRENT	:
			case DCM	:
			case ATTENUATION	:
			case MONO_ENERGY	:
			case XTAL_X	:
			case XTAL_Y	:
			case XTAL_Z	:
			case FIRST_FRAME	:
			case DET_DISTANCE	:
			case TWOTHETA	:
			case BEAMSTOP	:
			case OMEGA_START	:
			case OMEGA_END	:
			case FRAME_WIDTH	:
			case EXP_TIME	:
			case INV_BEAM	:
			case WEDGE	:
			case ENERGY1	:
			case ENERGY2	:
			case ENERGY3	:
			case XDSPROC	:
			case STEP_SIZE	:
			case NUMSITES	:
			case FRAMESPERSITE	:
			case OVERLAP	:
			case SITECHECK	:
			case SITEDELTA	:
			case BEAMX	:
			case BEAMY	:
			case SIZEX	:
			case SIZEY	:
			case SIZEZ	:
			case FRAMEAUDIT_MEG	:
			case COMMENT	:
										DevEvents.addString	(row, param, fieldmap.get(param)); break;
			
			case TIMESTAMP:				DevEvents.addString	(row, param, SQLDataType.getCurrentTimeString()); break;
			default:					throw new IllegalArgumentException("Unknown collect event field: "+param);
			}
		}
		rows.add(row);
		
		DevEvents.addEvent(GMCA_COLLECT_DATA_TABLE, CollectParam.values(), rows);
	}
}
