package gov.anl.gmca.log.db;

import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBField;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.RESULT_ERROR;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_SCR;

import java.util.ArrayList;
import java.util.HashMap;

public class DevEvents
{
	////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////// interface for event parameters ///////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////
	
	public static interface EventParam
	{
		public DBField getDBField();
	}
	
	////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////// common methods for adding SQL events ///////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////
	
	public static int getSampleNumber() 		{ return Epics.getInteger(PV_SCR.sampleNumber); }
	public static void setSampleNumber(int n) 	{ Epics.putInteger(PV_SCR.sampleNumber, n); }
	
	// returns a generated key if there was one, otherwise null
	public static Integer addEvent(String tableName, EventParam[] params, ArrayList<HashMap<String,String>> rows)
	{
		Result result = Database.insert(DBName.GMCADATA, tableName, rows);
		Integer pk = result.getGeneratedKey();
		if (result.error == RESULT_ERROR.TABLE_DOESNT_EXIST)
		{
			createTable(tableName, params);
			Database.insert(DBName.GMCADATA, tableName, rows);
			pk = result.getGeneratedKey();
		}
		return pk;
	}
	
	public static void createTable(String tableName, EventParam[] values)
	{
		ArrayList<DBField> fields = new ArrayList<DBField>();
		
		for (EventParam eParam : values)
		{
			fields.add(eParam.getDBField());
		}
		
		Database.create(DBName.GMCADATA, tableName, fields);
	}
	
	////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////// helper methods for creating SQL updates ///////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////
	
	static void addDouble(HashMap<String,String> row, EventParam param, double d)
	{
		if (Double.isNaN(d)) d=0; // TODO: change this?
		row.put(param.getDBField().fieldName, Double.toString(d));
	}
	static void addInt(HashMap<String,String> row, EventParam param, int i)
	{
		row.put(param.getDBField().fieldName, Integer.toString(i));
	}
	static void addBool(HashMap<String,String> row, EventParam param, boolean b)
	{
		row.put(param.getDBField().fieldName, b?"1":"0");
	}
	static void addString(HashMap<String,String> row, EventParam param, String s)
	{
		row.put(param.getDBField().fieldName, s);
	}
}
