package gov.anl.gmca.log.db;

import gov.anl.gmca.common.SQLDataType;
import gov.anl.gmca.common.Units;
import gov.anl.gmca.io.Database.DBField;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_FRAMEAUDIT;
import gov.anl.gmca.log.db.DevEvents.EventParam;
import gov.aps.jca.dbr.DBR;

import java.util.StringTokenizer;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public class DevFrameauditMsg {
	
	public static enum FrameauditParam implements EventParam
	{
						// db string	// units			// datatype			// allowNull	// primary key
		FRAME_NAME		("Frame",		Units.NONE,			SQLDataType.STRING,	false,			false),
		I0_MAX			("I0_max",		Units.NONE,			SQLDataType.STRING,	false,			false),
		I0_SUM			("I0_sum",		Units.NONE,			SQLDataType.STRING,	false,			false),
		I0_KGAIN		("I0_kgain",	Units.NONE,			SQLDataType.STRING,	false,			false),
		I0_KOFF			("I0_koff",		Units.NONE,			SQLDataType.STRING,	false,			false),
		BS_MAX			("BS_max",		Units.NONE,			SQLDataType.STRING,	false,			false),
		BS_SUM			("BS_sum",		Units.NONE,			SQLDataType.STRING,	false,			false),
		BS_KGAIN		("BS_kgain",	Units.NONE,			SQLDataType.STRING,	false,			false),
		BS_KOFF			("BS_koff",		Units.NONE,			SQLDataType.STRING,	false,			false),
		BS_BKG			("BS_bkg",		Units.NONE,			SQLDataType.STRING,	false,			false);
		
		public String dbString;
		public Units units;
		public SQLDataType dataType;
		public boolean allowNull;
		public boolean isPrimaryKey;
		
		FrameauditParam(String dbString, Units units, SQLDataType dataType, boolean allowNull, boolean isPrimaryKey)
		{
			this.dbString=dbString;
			this.units=units;
			this.dataType=dataType;
			this.allowNull=allowNull;
			this.isPrimaryKey=isPrimaryKey;
		}
		
		public DBField getDBField() { return new DBField(dbString, dataType, allowNull, isPrimaryKey, null); }
		
	}
	protected Epics.EpicsValueMonitor monitor = null;
	public static ConcurrentMap<FrameauditParam,String> fieldMap = new ConcurrentHashMap<FrameauditParam,String>();
	
	public DevFrameauditMsg() {
		
		monitor = new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, DBR value) {
				
				final String _pvName=pvName;
				final DBR _value=value;
				int isFrameaudit = Epics.getInteger(PV_FRAMEAUDIT.sts);
				if(_pvName.equals(PV_FRAMEAUDIT.msg.toString()) && isFrameaudit == 1) {
					String msg = Epics.dbrToString(_value);
					StringTokenizer tok = new StringTokenizer(msg);
					while(tok.hasMoreTokens()) {
						String str = tok.nextToken();
						String[] msgField = str.split("=");
						if(msgField.length < 2)
							continue;
						for (FrameauditParam param : FrameauditParam.values()) {
							if(param.getDBField().fieldName.equals(msgField[0])) {
								fieldMap.put(param, msgField[1]);
							}
						}
						
					}
				}
			}
			
		};
		//Epics.addMonitor(PV_FRAMEAUDIT.msg, monitor);
	}	
}
