package gov.anl.gmca.log.db;

import gov.anl.gmca.common.SQLDataType;
import gov.anl.gmca.common.Units;
import gov.anl.gmca.io.Database.DBField;
import gov.anl.gmca.log.db.DevEvents.EventParam;

import java.util.ArrayList;
import java.util.HashMap;

public class StrategyEvents {
	
	public static final String GMCA_STRATEGY_DATA_TABLE = "StrategyEvents";
	public static enum StrategyParam implements EventParam
	{
								// db string			// units			// datatype						// allowNull	// primary key
		SAMPLE_NUMBER			("sampleNumber",		Units.NONE,			SQLDataType.INT,				false,			false),
		STRATEGY_NUMBER			("strategyNumber",		Units.NONE,			SQLDataType.AUTOINC,			false,			true),
		SCREEN_COLLECT_NUMBER	("screenCollectNumber",	Units.NONE,			SQLDataType.INT,				true,			false),
		COLLECT_ZERORUN_NUMBER	("collectZeroNumber",	Units.NONE,			SQLDataType.INT,				true,			false),
		IMAGES					("images",				Units.NONE,			SQLDataType.STRING,				false,			false),
		PREFIX					("sampleName",			Units.NONE,			SQLDataType.STRING,				false,			false),
		ROOT_DIR				("workdir",				Units.NONE,			SQLDataType.STRING,				false,			false),
		SCORE					("score",				Units.NONE,			SQLDataType.STRING,				false,			false),
		RESOLUTION				("resolution",			Units.NONE,			SQLDataType.STRING,				false,			false),
		MOSAICITY				("mosaicity",			Units.NONE,			SQLDataType.STRING,				false,			false),
		SP_GRP					("spaceGroup_short",	Units.NONE,			SQLDataType.STRING,				false,			false),
		SP_GRP_LONG				("spaceGroup",			Units.NONE,			SQLDataType.STRING,				false,			false),
		SOLUTION_NUMBER			("solutionNumber",		Units.NONE,			SQLDataType.STRING,				false,			false),
		OSC_START				("oscstart",			Units.NONE,			SQLDataType.STRING,				false,			false),
		OSC_END					("oscend",				Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_OSC_START			("anom_oscstart",		Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_OSC_END			("anom_oscend",			Units.NONE,			SQLDataType.STRING,				false,			false),
		COMPLETE				("complete",			Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_COMPLETE			("anom_complete",		Units.NONE,			SQLDataType.STRING,				false,			false),
		OSCDELTA				("oscdelta",			Units.NONE,			SQLDataType.STRING,				false,			false),
		DET_DISTANCE			("detectordistance",	Units.NONE,			SQLDataType.STRING,				false,			false),
		DET_WARNING				("detectorwarning",		Units.NONE,			SQLDataType.STRING,				false,			false),
		BEAMX					("beamx",				Units.NONE,			SQLDataType.STRING,				false,			false),
		BEAMY					("beamy",				Units.NONE,			SQLDataType.STRING,				false,			false),
		UNITCELL				("unitcell",			Units.NONE,			SQLDataType.STRING,				false,			false),
		RMSD					("rmsd",				Units.NONE,			SQLDataType.STRING,				false,			false),
		COMMENT					("comment",				Units.NONE,			SQLDataType.STRING,				false,			false),
		TIMESTAMP				("timestamp",			Units.TIMESTAMP,	SQLDataType.TIMESTAMP,			false, 			false);
		
		public String dbString;
		public Units units;
		public SQLDataType dataType;
		public boolean allowNull;
		public boolean isPrimaryKey;
		
		StrategyParam(String dbString, Units units, SQLDataType dataType, boolean allowNull, boolean isPrimaryKey)
		{
			this.dbString=dbString;
			this.units=units;
			this.dataType=dataType;
			this.allowNull=allowNull;
			this.isPrimaryKey=isPrimaryKey;
		}
		
		public DBField getDBField() { return new DBField(dbString, dataType, allowNull, isPrimaryKey, null); }
	};
	
	public static void writeEvent(HashMap<StrategyParam,String> fieldMap)
	{
		// first create a beamline event
		//int beamlineEventNumber = BeamlineEvent.writeEvent();

		// then create the strategy event
		ArrayList<HashMap<String,String>> rows = new ArrayList<HashMap<String,String>>(); 
		HashMap<String,String> row = new HashMap<String,String>();
		for (StrategyParam param : StrategyParam.values())
		{
			switch (param)
			{
			case 	SAMPLE_NUMBER:				DevEvents.addInt	(row, param, DevEvents.getSampleNumber());	break;
			case 	STRATEGY_NUMBER:			DevEvents.addInt	(row, param, 0);	break;
			case 	SCREEN_COLLECT_NUMBER:		
			case 	COLLECT_ZERORUN_NUMBER:
			case	IMAGES	:
			case	PREFIX	:
			case	ROOT_DIR	:
			case	SCORE	:
			case	RESOLUTION	:
			case	MOSAICITY	:
			case	SP_GRP	:
			case	SP_GRP_LONG	:
			case	SOLUTION_NUMBER	:
			case	OSC_START	:
			case	OSC_END	:
			case	ANOM_OSC_START	:
			case	ANOM_OSC_END	:
			case	COMPLETE	:
			case	ANOM_COMPLETE	:
			case	OSCDELTA	:
			case	DET_DISTANCE	:
			case	DET_WARNING	:
			case	BEAMX	:
			case	BEAMY	:
			case	UNITCELL	:
			case	RMSD	:
										DevEvents.addString	(row, param, fieldMap.get(param)); break;
			case COMMENT:				DevEvents.addString	(row, param, "");	break;
			case TIMESTAMP:				DevEvents.addString	(row, param, SQLDataType.getCurrentTimeString()); break;
			default:					throw new IllegalArgumentException("Unknown strategy event field: "+param);
			}
		}
		rows.add(row);
		DevEvents.addEvent(GMCA_STRATEGY_DATA_TABLE, StrategyParam.values(), rows);
	}
}
