/**
 * Generates a script for multi crystal strategy and submits to Grid Engine
 * 
 * the script inturn runs XPLAN strategy
 * 
 */

package gov.anl.gmca.multistrategy;

import gov.anl.gmca.common.Command;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.diffimage.MarImageHeader;
import gov.anl.gmca.gridengine.CheckSubmitHost;
import gov.anl.gmca.gridengine.DRMAA;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.StringTokenizer;

public class CmdMultiStrategy extends Command {

	public String workDir = "";
	public String referencedata = "";
	public String images = "";
	ExecTask task;
	int sampNumber;
	int startFrameLabel1;
	int startFrameLabel2;
	String xdsjob = "JOB=  XYCORR INIT COLSPOT IDXREF DEFPIX XPLAN";
	protected static MarImageHeader currentHeader = null;

	DevMultiStrategy devMultiStrategy = new DevMultiStrategy();
	ModelMultiStrategyParams multiStrategyResult = null;

	@Override
	public void run() {
		String filefilter = DBConfig
				.get("processing_file_filter");
		
		/* increment for each new strategy */
		sampNumber = devMultiStrategy.getRowCount() + 1;

		multiStrategyResult = new ModelMultiStrategyParams();
		
		StringTokenizer tok = new StringTokenizer(ModelReferenceData.IMAGES);
		String firstFile = tok.nextToken();

		File file = new File(firstFile);
		if (!file.exists()) {
			Log.error(firstFile + " doesn't exist for strategy");
			return;
		}
		
		int lastIndDot = firstFile.lastIndexOf('.') + 1;
		
		if(firstFile.contains(".cbf"))
			lastIndDot = firstFile.lastIndexOf('_') + 1;
		
		if (lastIndDot == -1) {
			Log.error(firstFile + " file extention problem");
			return;
		}
		
		int end = firstFile.length();
		if(firstFile.contains(".cbf"))
			end = firstFile.lastIndexOf('.');
		
		String fram = firstFile.substring(lastIndDot, end);
		String prefix = firstFile.substring(firstFile.lastIndexOf("/"), lastIndDot-1);
		
		if (fram.length() < 3 || fram.length() > 6) {
			Log.error(firstFile + " number of digits are limited to 3 to 6");
			return;
		}
		try {
			if(firstFile.contains(".cbf"))
				startFrameLabel1 = Integer.parseInt(fram);
			else
				startFrameLabel1 = Integer.parseInt(firstFile.substring(lastIndDot,
					firstFile.length()));
			
		} catch (Exception e) {
			Log.error(e.getMessage() + "\n" + firstFile
					+ " unsupported file extension");
			return;
		}
		
		String pre_framename = firstFile.substring(0, lastIndDot - 1);
		
		if(firstFile.contains(".cbf"))
			images = pre_framename + "_" + filefilter+".cbf";
		else
			images = pre_framename + "." + filefilter;
		
		String filenames = pre_framename+"." + filefilter;
		
		currentHeader = new MarImageHeader(firstFile.replace("//", "/"));
		currentHeader.getHeader();
		
		multiStrategyResult.workDir = this.workDir+ "/"+prefix+"_multistrategy";
		workDir = multiStrategyResult.workDir;
		multiStrategyResult.referencedata = this.referencedata;

		multiStrategyResult.state = "PROCESSING";

		File dirFile = new File(this.workDir);
		if (!dirFile.exists()) {

			if (!dirFile.mkdirs()) {
				Log.error("Couldn't create directory " + workDir);
				return;
			}
		}
		/* Create XDS input file */
		/*int val = writeXDSInput();
		if (val == 0) {
			Log.error("writing XDS file for strategy failed");
			return;
		}*/
		
		/* process XDS */
		
		multiStrategyResult.images = images;
		devMultiStrategy.setResult(sampNumber, multiStrategyResult, true);
		
		try {

			BufferedWriter runxdsOut = new BufferedWriter(new FileWriter(
					workDir + "/" + "run_multistrategy.sh", false));
			runxdsOut.write("#!/bin/bash -f \n");
			runxdsOut.write("# Running XDS \n");
			
			if(firstFile.contains(".cbf")) 
				filenames = pre_framename + "_" + filefilter+".cbf";
				
			runxdsOut.write("/mnt/software/px64/XDS/generate_XDS.INP \""+ filenames+"\"\n");
			//runxdsOut.write("egrep -v \'INCLUDE_RESOLUTION_RANGE|UNIT_CELL_CONSTANTS|SPACE_GROUP_NUMBER|JOB=|DATA_RANGE|SPOT_RANGE|BACKGROUND_RANGE\' XDS.INP >> XDS.INP_new\n");
                        runxdsOut.write("egrep -v \'INCLUDE_RESOLUTION_RANGE|^FRIEDEL|UNIT_CELL_CONSTANTS|SPACE_GROUP_NUMBER|JOB=|DATA_RANGE|SPOT_RANGE|BACKGROUND_RANGE\' XDS.INP >> XDS.INP_new\n");
			runxdsOut.write("mv XDS.INP_new XDS.INP\n");
			runxdsOut.write("echo \""+xdsjob+"\" >> XDS.INP\n");
			runxdsOut.write("echo \"STARTING_ANGLES_OF_SPINDLE_ROTATION= 0 180 5\" >> XDS.INP\n");
			runxdsOut.write("echo \"TOTAL_SPINDLE_ROTATION_RANGES="+ModelReferenceData.MINRANGE+" 30 "+ModelReferenceData.MINRANGE+"\" >> XDS.INP\n");
			
			if (!referencedata.equals("") && referencedata != null
					&& ModelReferenceData.ISREFERENCE == 1) {
				runxdsOut.write("echo \"INCLUDE_RESOLUTION_RANGE= "+ ModelReferenceData.RESOLUTION_LOW + " "
																+ ModelReferenceData.RESOLUTION_HIGH + "\" >> XDS.INP\n");
			
				runxdsOut.write("echo \"SPACE_GROUP_NUMBER="+ModelReferenceData.SPACE_GROUP+"\" >> XDS.INP\n");
				runxdsOut.write("echo \"UNIT_CELL_CONSTANTS="+ModelReferenceData.UNIT_CELL+"\" >> XDS.INP\n");
				runxdsOut.write("echo \"REFERENCE_DATA_SET="+referencedata +"\" >> XDS.INP\n");
			}
			
			runxdsOut.write("echo \"STARTING_ANGLE="+currentHeader.phi+"\" >> XDS.INP\n");
			runxdsOut.write("echo \"DATA_RANGE="+ startFrameLabel1+" "+ startFrameLabel1+"\" >> XDS.INP\n");
			runxdsOut.write("echo \"SPOT_RANGE="+ startFrameLabel1+" "+ startFrameLabel1+"\" >> XDS.INP\n");
			runxdsOut.write("echo \"BACKGROUND_RANGE="+ startFrameLabel1+" "+ startFrameLabel1+"\" >> XDS.INP\n");
                        runxdsOut.write("echo \"FRIEDEL'S_LAW="+ModelReferenceData.FRIEDELSLAW +"\" >> XDS.INP\n");
			
			runxdsOut.write("/mnt/software/px64/XDS/xds > XDS.LOG");
			runxdsOut.close();

			File setExec = new File(workDir + "/" + "run_multistrategy.sh");
			setExec.setExecutable(true);

		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		/*check for gridengine environment and quit if not a submit host */
		boolean isSubmitHost = CheckSubmitHost.get().isSubmitHost();
		System.out.println("Gridengine env detected: "+isSubmitHost);

		int jobStatus = 0;
		if (!isSubmitHost) {
			executeLocally(workDir + "/"+ "run_multistrategy.sh " + workDir);
			Log.warn("Grid Engine not available, strategy job running locally");

			task = ExecTask.new_ExecTask(workDir + "/"
					+ "run_multistrategy.sh " + workDir,
					"Multi Crystal strategy :", false, false);
			task.start();
			task.waitFor();

		} else {
			try {
				Log.note("XPLAN multi crystal strategy in progress, check the dir. "
						+ workDir);
				DRMAA drmaa;
				try {
					drmaa = new DRMAA(workDir + "/" + "run_multistrategy.sh",
							workDir, 900);
				} catch (UnsatisfiedLinkError e) {
					multiStrategyResult.warning = "GridEngine/DRMAA exception";
					multiStrategyResult.state = "ERROR";
					Log.error("Grid engine not initiated XPLAN job: DRMAA error");
					devMultiStrategy.setMultiStrategyResult(sampNumber,
							multiStrategyResult, true);
					e.printStackTrace();
					return;
				}
				catch (Exception e) {
			
					exitWithError(e);
					return;
				} // wait 900 sec before quitting job
				drmaa.start();
				jobStatus = drmaa.waitFor();
			} catch (Exception e1) {
				
				exitWithError(e1);
				return;
			}
		}

		try {
			multiStrategyResult.processResult(workDir);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/* check for multiple lattice or badly indexed spots */
		if (multiStrategyResult.state.equals("ERROR")) {
			xdsjob = "JOB= IDXREF\n";
			/*val = writeXDSInput();
			if (val == 0) {
				Log.error("wring XDS file for strategy failed");
				return;
			}*/

			Log.note("ERROR in XDS.LOG, checking for multiple lattice");
			task = ExecTask.new_ExecTask(Main.getAppDir()
					+ "/tools/scripts/multistrategy/run_multiplelattice.sh "
					+ workDir+" "+filenames, "Multi Crystal strategy :", false, false);
			
			task.start();
			task.waitFor();

			try {
				multiStrategyResult.processResult(workDir);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		if (multiStrategyResult.warning.equals("processing"))
			multiStrategyResult.warning = "DONE";
		if (multiStrategyResult.state.equals("PROCESSING"))
			multiStrategyResult.state = "DONE";

		devMultiStrategy.setMultiStrategyResult(sampNumber,
				multiStrategyResult, true);

		String savePath = "";
		savePath = DevDataDir.getPrimary();
		devMultiStrategy.saveAllResults(savePath + "/MultiStrategyResults.xls");
		Log.note("Multi Crystal Strategy calculation finished");
	}
	public void exitWithError(Exception e) {
		multiStrategyResult.warning = "GridEngine/DRMAA exception";
		multiStrategyResult.state = "ERROR";
		Log.error("Grid engine not initiated XPLAN job: DRMAA error");
		devMultiStrategy.setMultiStrategyResult(sampNumber,
				multiStrategyResult, true);
		e.printStackTrace();
		return;
	}
	public void executeLocally(String cmd) {
		
	}
	public int writeXDSInput() {
		/* Write XDS input file */
		String filefilter = DBConfig
				.get("file_filter");
		System.out.println("Writing XDS file");

		File xdsDirFile = new File(workDir);

		if (!xdsDirFile.exists()) {

			if (!xdsDirFile.mkdirs()) {
				Log.error("Couldn't create XDS directory " + workDir);
				return 0;
			}
		}
		int lastIndDot;
		String spot_range = "";

		StringTokenizer tok = new StringTokenizer(ModelReferenceData.IMAGES);
		String firstFile = tok.nextToken();

		File file = new File(firstFile);
		if (!file.exists()) {
			Log.error(firstFile + " doesn't exist for strategy");
			return 0;
		}
		lastIndDot = firstFile.lastIndexOf('.') + 1;
		if (lastIndDot == -1) {
			Log.error(firstFile + " file extention problem");
			return 0;
		}
		String fram = firstFile.substring(lastIndDot, firstFile.length());

		// looks useless, disabled by QX
		//if (fram.length()  != 5  && !fram.equalsIgnoreCase("cbf")) {
		//	Log.error(firstFile + " unsupported file extenstions");
		//	return 0;
		//}
		try {
			if(fram.equalsIgnoreCase("cbf"))
				startFrameLabel1 = Integer.parseInt(firstFile.substring(firstFile.lastIndexOf('_')+1, lastIndDot-1));
			else
				startFrameLabel1 = Integer.parseInt(firstFile.substring(lastIndDot,
					firstFile.length()));
		} catch (Exception e) {
			Log.error(e.getMessage() + "\n" + firstFile
					+ " unsupported file extension");
			return 0;
		}
		String pre_framename = firstFile.substring(0, lastIndDot - 1);
		if(fram.equalsIgnoreCase("cbf"))
			images = pre_framename + "_" + filefilter+".cbf";
		else
			images = pre_framename + "." + filefilter;
		tok = new StringTokenizer(ModelReferenceData.IMAGES);
		while (tok.hasMoreTokens()) {

			String fileStr = tok.nextToken();
			lastIndDot = fileStr.lastIndexOf('.') + 1;

			if (lastIndDot == -1) {
				Log.error(firstFile + " file extention problem");
				return 0;
			}

			fram = fileStr.substring(lastIndDot, (int) fileStr.length());
			// looks useless
			//if ( fram.length()  != 5  && !fram.equalsIgnoreCase("cbf")) {
			//	Log.error(fileStr + " unsupported file extenstions");
			//	return 0;
			//}
			int framNo;
			try {
				if(fram.equalsIgnoreCase("cbf"))
					framNo = Integer.parseInt(firstFile.substring(firstFile.lastIndexOf('_')+1, lastIndDot-1));
				else
					framNo = Integer.parseInt(fram);
			} catch (Exception e) {
				Log.error(e.getMessage() + "\n" + fileStr
						+ " file extesion should be a frame number");
				return 0;
			}
			images += " " + framNo;
			spot_range += "SPOT_RANGE= " + framNo + " " + framNo + "\n";
		}

		currentHeader = new MarImageHeader(firstFile.replace("//", "/"));
		currentHeader.getHeader();

		int orgx = (int) (currentHeader.displayOriginX * currentHeader.size1 / 300.0);
		int orgy = (int) (currentHeader.displayOriginY * currentHeader.size2 / 300.0);
		try {
			// Example file ~spothineni/testxds/XDS.INP on IDD
			BufferedWriter xdsOut = new BufferedWriter(new FileWriter(workDir
					+ "/" + "XDS.INP", false));
			String XDS_String = "!********** Example for MAR300 CCD-detector at APS Sector 23  GM/CA   *************\n"
					+ "!********** Edit parameters in this file for reprocessing  		   *************\n"
					+ "!**********************************************************************************\n"
					+ "!Standard settings for the MAR CCD that rarely need to be changed\n\n"
					+ "DETECTOR="
					+ currentHeader.detector
					+ "    "
					+ "MINIMUM_VALID_PIXEL_VALUE=0"
					+ "   "
					+ "OVERLOAD="
					+ currentHeader.overload
					+ "\n"
					+ "DIRECTION_OF_DETECTOR_X-AXIS= 1.0 0.0 0.0\n"
					+ "DIRECTION_OF_DETECTOR_Y-AXIS= 0.0 1.0 0.0\n"
					+ "TRUSTED_REGION=0.0 1.2 !Relative radii limiting trusted detector region\n\n"
					+ "!UNTRUSTED_RECTANGLE= 570 1920  1469 2048  ! rectangle: X1 X2   Y1 Y2\n"
					+ "!UNTRUSTED_ELLIPSE= 910 1110    1010 1210  ! ellipse enclosed by X1 X2 Y1 Y2\n"
					+ "!File name, access, format of dark-current (non-Xray background) image\n"
					+ "!DARK_CURRENT_IMAGE=../images/blank.tif    !hardly ever used\n\n"
					+ "MAXIMUM_NUMBER_OF_PROCESSORS=4     !<25;ignored by single cpu version of xds\n\n"
					+ "MAXIMUM_NUMBER_OF_JOBS=4  !Speeds-up COLSPOT & INTEGRATE on a Linux-cluster\n"
					+ "!SECONDS=0   !Maximum number of seconds to wait until data image must appear\n"
					+ "!TEST=1     !Test flag. 1,2 additional diagnostics and images\n"
					+ "!\n"
					+ "! Do not forget to define ORGX and ORGY, which are app. ORGX=NX/2, ORGY=NY/2\n"
					+ "!====================== JOB CONTROL PARAMETERS ===============================\n"
					+ xdsjob
					+ "\n"
					+ "!JOB= XYCORR INIT COLSPOT IDXREF DEFPIX\n"
					+ "!JOB= IDXREF\n"
					+ "!JOB= DEFPIX\n"
					+ "!JOB= XPLAN\n"
					+ "!====================== GEOMETRICAL PARAMETERS ===============================\n\n"
					+ "!NX=number of fast pixels (along X); QX=length of an X-pixel (mm)\n"
					+ "!NY=number of slow pixels (along Y); QY=length of a  Y-pixel (mm)\n"
					+ "NX="
					+ currentHeader.size1
					+ " NY="
					+ currentHeader.size2
					+ " QX="
					+ currentHeader.jpegPixelSize
					+ " QY="
					+ currentHeader.jpegPixelSize
					+ " !MARCCD 300mm version at APS 23ID\n\n"
					+ "# 149.65, 151,18\n"
					+ "#\n"
					+ "!ORGX and ORGY are often close to the image center, i.e. ORGX=NX/2, ORGY=NY/2\n"
					+ "ORGX="
					+ Integer.toString(orgx)
					+ " ORGY="
					+ Integer.toString(orgy)
					+ " !Detector origin (pixels).  ORGX=NX/2; ORGY=NY/2\n"
					+ "DETECTOR_DISTANCE= "
					+ currentHeader.distance
					+ "   !(mm)\n"
					+ "ROTATION_AXIS= 1.0 0.0 0.0\n"
					+ "OSCILLATION_RANGE="
					+ currentHeader.osc_range
					+ "  !degrees (>0)\n"
					+ "X-RAY_WAVELENGTH="
					+ currentHeader.wavelength
					+ "  !Angstroem\n"
					+ "INCIDENT_BEAM_DIRECTION=0.0 0.0 1.0\n"
					+ "FRACTION_OF_POLARIZATION=0.99 !default=0.5 for unpolarized beam;0.90 at DESY;\n"
					+ "POLARIZATION_PLANE_NORMAL= 0.0 1.0 0.0\n"
					+ "AIR=0.001        !Air absorption coefficient of x-rays\n\n"
					+ "!======================= CRYSTAL PARAMETERS =================================\n";
			if (!referencedata.equals("") && referencedata != null
					&& ModelReferenceData.ISREFERENCE == 1) {
				XDS_String += "SPACE_GROUP_NUMBER= "
						+ ModelReferenceData.SPACE_GROUP + "\n";
				XDS_String += "UNIT_CELL_CONSTANTS= "
						+ ModelReferenceData.UNIT_CELL + "\n";
			} else {
				XDS_String += "! SPACE_GROUP_NUMBER=89   !0 for unknown crystals; cell constants are ignored.\n";
				XDS_String += "! UNIT_CELL_CONSTANTS=  104.60   104.60    81.46  90.000  90.000  90.000\n";
			}
			XDS_String += "!Optional reindexing transformation to apply on reflection indices\n"
					+ "!REIDX=   0  0 -1  0  0 -1  0  0 -1  0  0  0\n"
					+ "!==================== DATA COLLECTION STRATEGY (XPLAN) ======================\n"
					+ "!                       !!! Warning !!!"
					+ "! If you processed your data for a crystal with unknown cell constants and\n"
					+ "! space group symmetry, XPLAN will report the results for space group P1.\n"
					+ "FRIEDEL'S_LAW=TRUE  !Default is TRUE.\n\n"
					+

					"STARTING_ANGLE= "
					+ currentHeader.phi
					+ " STARTING_FRAME= "
					+ startFrameLabel1
					+ "\n"
					+ "!used to define the angular origin about the rotation axis.\n"
					+ "!Default:  STARTING_ANGLE=  0 at STARTING_FRAME=first data image\n\n"
					+

					"!RESOLUTION_SHELLS=10 6 5 4 3 2 1.5 1.3 1.2\n\n"
					+

					"STARTING_ANGLES_OF_SPINDLE_ROTATION= 0 180 5\n\n"
					+

					"TOTAL_SPINDLE_ROTATION_RANGES="
					+ ModelReferenceData.MINRANGE
					+ " 30 "
					+ ModelReferenceData.MINRANGE + "\n\n";
			if (!referencedata.equals("") && referencedata != null
					&& ModelReferenceData.ISREFERENCE == 1)
				XDS_String += "REFERENCE_DATA_SET= " + referencedata
						+ " !Name of a reference data set (optional)\n\n";
			else
				XDS_String += "!REFERENCE_DATA_SET= XDS_ASCII.HKL !Name of a reference data set (optional)\n\n";

			XDS_String += "!==================== SELECTION OF DATA IMAGES ==============================\n"
					+ "!Generic file name, access, and format of data images\n"
					+ "NAME_TEMPLATE_OF_DATA_FRAMES= "
					+ pre_framename
					+ ".???? DIRECT  TIFF\n\n"
					+

					"DATA_RANGE="
					+ Integer.toString(startFrameLabel1)
					+ " "
					+ Integer.toString(startFrameLabel1)
					+ "      !Numbers of first and last data image collected\n"
					+ "! DATA_RANGE=1  60       !Numbers of first and last data image collected\n"
					+

					"BACKGROUND_RANGE= "
					+ Integer.toString(startFrameLabel1)
					+ " "
					+ Integer.toString(startFrameLabel1)
					+ " !Numbers of first and last data image for background\n"
					+

					spot_range
					+ " !First and last data image number for finding spots\n"
					+ "		       !Up to 10 SPOT_RANGE= parameters can be specified\n\n"
					+

					"!PROFILE_RANGE=1  9    !Up to 10 PROFILE_RANGE= parameters can be specified\n"
					+ "		       !Images used for automatic determination of\n"
					+ "		       !BEAM_DIVERGENCE, REFLECTING_RANGE and their E.S.D.'s\n\n"
					+

					"!====================== INDEXING PARAMETERS =================================\n"
					+ "!Never forget to check this, since the default 0 0 0 is almost always correct!\n"
					+ "!INDEX_ORIGIN= 0 0 0          ! used by \"IDXREF\" to add an index offset\n"
					+

					"!Additional parameters for fine tuning that rarely need to be changed\n"
					+ "!INDEX_ERROR=0.05 INDEX_MAGNITUDE=8 INDEX_QUALITY=0.8\n"
					+ "!SEPMIN=6.0 CLUSTER_RADIUS=3\n"
					+ "!MAXIMUM_ERROR_OF_SPOT_POSITION=3.0\n\n"
					+ "!MAXIMUM_ERROR_OF_SPINDLE_POSITION=2.0\n"
					+ "!MINIMUM_FRACTION_OF_INDEXED_SPOTS=0.5\n"
					+ "!============== DECISION CONSTANTS FOR FINDING CRYSTAL SYMMETRY =============\n"
					+ "!Decision constants for detection of lattice symmetry (IDXREF, CORRECT)\n"
					+ "!MAX_CELL_AXIS_ERROR=0.03 ! Maximum relative error in cell axes tolerated\n"
					+ "!MAX_CELL_ANGLE_ERROR=2.0 ! Maximum cell angle error tolerated\n"
					+ "\n"
					+ "!Decision constants for detection of space group symmetry (CORRECT).\n"
					+ "!Resolution range for accepting reflections for space group determination in\n"
					+ "!the CORRECT step. It should cover a sufficient number of strong reflections.\n"
					+ "!TEST_RESOLUTION_RANGE=8.0 4.5\n"
					+ "!MIN_RFL_Rmeas= 50 ! Minimum #reflections needed for calculation of Rmeas\n"
					+ "!MAX_FAC_Rmeas=2.0 ! Sets an upper limit for acceptable Rmeas\n"
					+ "REFINE(IDXREF)= DISTANCE BEAM AXIS ORIENTATION\n"
					+ "!================== CRITERIA FOR ACCEPTING REFLECTIONS ======================\n"
					+ "VALUE_RANGE_FOR_TRUSTED_DETECTOR_PIXELS= 6000 30000 !Used by DEFPIX\n"
					+ "			   !for excluding shaded parts of the detector.\n\n";
			if (!referencedata.equals("") && referencedata != null
					&& ModelReferenceData.ISREFERENCE == 1)
				XDS_String += "INCLUDE_RESOLUTION_RANGE= "
						+ ModelReferenceData.RESOLUTION_LOW + " "
						+ ModelReferenceData.RESOLUTION_HIGH + "\n";
			else
				XDS_String += "INCLUDE_RESOLUTION_RANGE=50.0 0.0 !Angstroem; used by DEFPIX,INTEGRATE,CORRECT\n\n";
			XDS_String += "!used by CORRECT to exclude ice-reflections\n"
					+ "!EXCLUDE_RESOLUTION_RANGE= 3.93 3.87 !ice-ring at 3.897 Angstrom\n"
					+ "!EXCLUDE_RESOLUTION_RANGE= 3.70 3.64 !ice-ring at 3.669 Angstrom\n"
					+ "!EXCLUDE_RESOLUTION_RANGE= 3.47 3.41 !ice-ring at 3.441 Angstrom\n"
					+ "!EXCLUDE_RESOLUTION_RANGE= 2.70 2.64 !ice-ring at 2.671 Angstrom\n"
					+ "!EXCLUDE_RESOLUTION_RANGE= 2.28 2.22 !ice-ring at 2.249 Angstrom\n"
					+ "MINIMUM_ZETA=0.05 !Defines width of 'blind region' (XPLAN,INTEGRATE,CORRECT)\n"
					+ "!WFAC1=1.0  !This controls the number of rejected MISFITS in CORRECT;\n"
					+ "	    	 !a larger value leads to fewer rejections.\n"
					+

					"!REJECT_ALIEN=20.0 ! Automatic rejection of very strong reflections\n\n"
					+

					"!============== INTEGRATION AND PEAK PROFILE PARAMETERS =====================\n"
					+ "!Specification of the peak profile parameters below overrides the automatic\n"
					+ "!determination from the images\n"
					+ "!Suggested values are listed near the end of INTEGRATE.LP\n"
					+ "!BEAM_DIVERGENCE=   0.80         !arctan(spot diameter/DETECTOR_DISTANCE)\n"
					+ "!BEAM_DIVERGENCE_E.S.D.=   0.080 !half-width (Sigma) of BEAM_DIVERGENCE\n"
					+ "!REFLECTING_RANGE=  0.780 !for crossing the Ewald sphere on shortest route\n"
					+ "!REFLECTING_RANGE_E.S.D.=  0.113 !half-width (mosaicity) of REFLECTING_RANGE\n"
					+

					"!NUMBER_OF_PROFILE_GRID_POINTS_ALONG_ALPHA/BETA=9 !used by: INTEGRATE\n"
					+ "!NUMBER_OF_PROFILE_GRID_POINTS_ALONG_GAMMA= 9     !used by: INTEGRATE\n\n"
					+

					"!CUT=2.0    !defines the integration region for profile fitting\n"
					+ "!MINPK=80.0 !minimum required percentage of observed reflection intensity\n"
					+ "!DELPHI= 5.0!controls the number of reference profiles and scaling factors\n\n"
					+

					"!======= PARAMETERS CONTROLLING CORRECTION FACTORS (used by: CORRECT) =======\n"
					+ "!MINIMUM_I/SIGMA=3.0 !minimum intensity/sigma required for scaling reflections\n"
					+ "!NBATCH=-1  !controls the number of correction factors along image numbers\n"
					+ "!REFLECTIONS/CORRECTION_FACTOR=50   !minimum #reflections/correction needed\n"
					+ "!PATCH_SHUTTER_PROBLEM=TRUE         !FALSE is default\n"
					+ "!STRICT_ABSORPTION_CORRECTION=TRUE  !FALSE is default\n"
					+ "!CORRECTIONS= DECAY MODULATION ABSORPTION\n\n"
					+

					"!=========== PARAMETERS DEFINING BACKGROUND AND PEAK PIXELS =================\n"
					+ "!STRONG_PIXEL=6.0                              !used by: COLSPOT\n"
					+ "!A 'strong' pixel to be included in a spot must exceed the background\n"
					+ "!by more than the given multiple of standard deviations.\n\n"
					+

					"!MAXIMUM_NUMBER_OF_STRONG_PIXELS=1500000       !used by: COLSPOT\n\n"
					+

					"!SPOT_MAXIMUM-CENTROID=3.0                     !used by: COLSPOT\n\n"
					+

					"!MINIMUM_NUMBER_OF_PIXELS_IN_A_SPOT=12          !used by: COLSPOT\n"
					+ "!This allows to suppress spurious isolated pixels from entering the\n"
					+ "!spot list generated by \"COLSPOT\".\n\n"
					+

					"!NBX=3  NBY=3  !Define a rectangle of size (2*NBX+1)*(2*NBY+1)\n"
					+ "!The variation of counts within the rectangle centered at each image pixel\n"
					+ "!is used for distinguishing between background and spot pixels.\n\n"
					+

					"!BACKGROUND_PIXEL=6.0                          !used by: COLSPOT,INTEGRATE\n"
					+ "!An image pixel does not belong to the background region if the local\n"
					+ "!pixel variation exceeds the expected variation by the given number of\n"
					+ "!standard deviations.\n"
					+

					"!SIGNAL_PIXEL=6.0                              !used by: INTEGRATE\n"
					+ "!A pixel above the threshold contributes to the spot centroid\n";

			xdsOut.write(XDS_String);
			xdsOut.close();
		} catch (IOException e) {
			Log.error("Failed to write XDS file for multi crystal startegy: "
					+ e.getMessage());
			return 0;
		}
		return 1;
	}
}
