/**
 * Handles MySQL queries for MultiStrategyReferenceData table
 */

package gov.anl.gmca.multistrategy;

import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_STRATEGY;
import gov.anl.gmca.syslog.Log;
import gov.aps.jca.dbr.DBR;

import java.util.HashMap;

public class DevReferenceData {
	protected static final String tableName = "MultiStrategyReferenceData";

	protected static boolean doesTableExist() {
		Database.Result result = Database.exec(DBName.USERDATA,
				"show tables like '" + tableName + "'");
		boolean exists = (result.numRows() > 0);
		return exists;
	}

	public synchronized static boolean verifyTableExists() {
		if (!doesTableExist()) {
			System.out.println("Creating MultiStrategy results table");
			String createCmd = "CREATE TABLE `" + tableName + "` ("
					+ "`isReference` int(11) default 1,"
					+ "`reference_data` varchar(500) default '',"
					+ "`min_range` int(11) default '5',"
					+ "`unit_cell` varchar(50) default '',"
					+ "`images` varchar(5000) default '',"
					+ "`spaceGroup` varchar(50) default '',"
					+ "`lowRes` varchar(30) default '',"
					+ "`highRes` varchar(30) default '')";

			Log.note(createCmd);
			Database.exec(DBName.USERDATA, createCmd);
		}

		return doesTableExist();
	}

	protected boolean connected = false;
	protected Epics.EpicsValueMonitor dbUpdatedMonitor;

	public static DevReferenceData get() {
		return inst;
	}

	protected static DevReferenceData inst = null;

	public DevReferenceData() {
		inst = this;
	}

	public void verifyConnected() {
		if (connected)
			return;

		verifyTableExists();
		updateRow();

		Epics.addMonitor(PV_STRATEGY.mergeDatasetsUpdated,
				new Epics.EpicsValueMonitor() {
					@Override
					public void valueChanged(String pvName, DBR value) {
						double isR = Epics.dbrToDouble(value);

						ModelCombineDatasets.get().ll.notifyListeners();
					}
				});

		connected = true;
	}

	private void updateRow() {
		// TODO Auto-generated method stub
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);

		if (result.numRows() > 0) {
			for (HashMap<String, String> row : result.getRows()) {
				if (row.containsKey("isReference"))
					ModelReferenceData.ISREFERENCE = Integer.parseInt(row
							.get("isReference"));
				if (row.containsKey("reference_data"))
					ModelReferenceData.REFERENCE_DATA = row
							.get("reference_data");
				if (row.containsKey("min_range"))
					ModelReferenceData.MINRANGE = Integer.parseInt(row
							.get("min_range"));
				if (row.containsKey("unit_cell"))
					ModelReferenceData.UNIT_CELL = row.get("unit_cell");
				if (row.containsKey("spaceGroup"))
					ModelReferenceData.SPACE_GROUP = Integer.parseInt(row
							.get("spaceGroup"));
				if (row.containsKey("images")) {
					ModelReferenceData.IMAGES = row.get("images");
					// Log.error(ModelReferenceData.IMAGES);
				}
				if (row.containsKey("lowRes"))
					ModelReferenceData.RESOLUTION_LOW = Double.parseDouble(row
							.get("lowRes"));
				if (row.containsKey("highRes"))
					ModelReferenceData.RESOLUTION_HIGH = Double.parseDouble(row
							.get("highRes"));
			}
		}

		ModelReferenceData.get().ll.notifyListeners();
	}

	public void setResult(boolean update) {
		verifyTableExists();
		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "`");
		if (0 == result.numRows()) {
			Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (isReference, reference_data, min_range, unit_cell, images, spaceGroup,lowRes, highRes) "
							+ "values (" + "'" + ModelReferenceData.ISREFERENCE
							+ "', '" + ModelReferenceData.REFERENCE_DATA
							+ "', '" + ModelReferenceData.MINRANGE + "', '"
							+ ModelReferenceData.UNIT_CELL + "', '"
							+ ModelReferenceData.IMAGES + "', '"
							+ ModelReferenceData.SPACE_GROUP + "', '"
							+ ModelReferenceData.RESOLUTION_LOW + "', '"
							+ ModelReferenceData.RESOLUTION_HIGH + "')");
			newResultCreated = true;
		} else {
			Database.exec(DBName.USERDATA, "update " + tableName + " "
					+ "set isReference='" + ModelReferenceData.ISREFERENCE
					+ "' " + ", reference_data='"
					+ ModelReferenceData.REFERENCE_DATA + "' "
					+ ", min_range='" + ModelReferenceData.MINRANGE + "' "
					+ ", unit_cell='" + ModelReferenceData.UNIT_CELL + "' "
					+ ", images='" + ModelReferenceData.IMAGES + "' "
					+ ", spaceGroup='" + ModelReferenceData.SPACE_GROUP + "' "
					+ ", lowRes='" + ModelReferenceData.RESOLUTION_LOW + "' "
					+ ", highRes='" + ModelReferenceData.RESOLUTION_HIGH + "' ");
		}
		if (update) {
			ModelReferenceData.get().ll.notifyListeners();
		}
	}
}
