/**
 * cache for multi crystal strategy results
 */

package gov.anl.gmca.multistrategy;

import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;
import java.util.Collections;

public class ModelMultiStrategyList {

	public ListenerList<ModelMultiStrategyList> ll;

	protected ArrayList<ModelMultiStrategy> resultList = new ArrayList<ModelMultiStrategy>();

	public ArrayList<ModelMultiStrategy> getMultiStrategyList() {
		return new ArrayList<ModelMultiStrategy>(resultList);
	}

	public ModelMultiStrategy getResultID(int requestedID) {
		for (ModelMultiStrategy result : getMultiStrategyList()) {
			if (result.getID() == requestedID)
				return result;
		}
		return null;
	}

	public ModelMultiStrategy getResultIndex(int index) {
		try {
			return resultList.get(index);
		} catch (IndexOutOfBoundsException ioobe) {
			return null;
		}
	}

	public ModelMultiStrategy getCurrentResult() {
		return getResultIndex(getCurrentMultiStrategyIndex());
	}

	public void clearResults() {
		resultList = new ArrayList<ModelMultiStrategy>();
	}

	public void setRow(ModelMultiStrategy result, int sampleNo, boolean update) {
		for (int i = 0; i < resultList.size(); i++) {
			if (resultList.get(i).getID() == sampleNo) {
				resultList.set(i, result);
				return;
			}
		}

		resultList.add(result);

		Collections.sort(resultList);

		if (update)
			ll.notifyListeners(result.getID());
	}

	protected int currentMultiStrategyIndex = 0;
	protected int showMultiStrategy = 1;

	public void setMultiStrategy(int show) {
		showMultiStrategy = show;
	}

	public int getMultiStrategy() {
		return showMultiStrategy;
	}

	public void setCurrentMultiStrategyIndex(int i) {
		currentMultiStrategyIndex = i;
	}

	public int getCurrentMultiStrategyIndex() {
		return currentMultiStrategyIndex;
	}

	protected static ModelMultiStrategyList inst = null;

	public static ModelMultiStrategyList get() {
		return inst;
	}

	public ModelMultiStrategyList() {
		inst = this;
		ll = new ListenerList<ModelMultiStrategyList>(this);
	}

	public void deleteAll() {
		new Thread(new Runnable() {
			public void run() {
				boolean okToDelete = Util.popupOkCancel(
						"Delete?",
						"OK to delete ALL ("
								+ resultList.size()
								+ ") strategy results?  This action cannot be undone.");
				if (okToDelete)
					DevMultiStrategy.get().deleteAllResults();
			}
		}).start();
	}

	public void save(String excelname) {
		final String excelFile = excelname;
		new Thread(new Runnable() {
			public void run() {
				DevMultiStrategy.get().saveAllResults(excelFile);
			}
		}).start();

	}
}
