/**
 * Model for ReferenceData tab
 * 
 */

package gov.anl.gmca.multistrategy;

import gov.anl.gmca.common.Command;
import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;

import java.io.*;
import java.util.StringTokenizer;

public class ModelReferenceData extends Command {

	public ListenerList<ModelReferenceData> ll;;
	protected static ModelReferenceData inst = null;
	public String PATH = "";
	public boolean isHighRes = false;
	public static int ISREFERENCE = 1;
	public static String REFERENCE_DATA = "";
	public static int MINRANGE = 5;
	public static String UNIT_CELL = "";
	public static String IMAGES = "";
	public static int SPACE_GROUP = 1;
	public static double RESOLUTION_LOW = 50.0;
	public static double RESOLUTION_HIGH = 0.0;
	public static String FRIEDELSLAW="TRUE";
	public static String[] UnitCell = { "a", "b", "c", "\u03B1", "\u03B2",
			"\u03B3" };

	public static ModelReferenceData get() {
		return inst;
	}

	public ModelReferenceData() {
		inst = this;
		ll = new ListenerList<ModelReferenceData>(this);
	}

	public static int loadHKLFile(String path, boolean isHigres) {
		if (path == null)
			return 1;
		if (path.equals("") || path.trim().equals("") || path == null)
			return 1;

		/* check if the file is XDS_ASCII.HKL */
		if (path.lastIndexOf(".") == -1) {
			Log.error("Please select XDS_ASCII.HKL/ or .sca file of HKL2000");
			return 1;
		}
		String extension = path.substring(path.lastIndexOf("."), path.length());

		if (extension.contains(".sca") || extension.contains(".SCA")) {
			readScaFile(path);
			path = ModelReferenceData.REFERENCE_DATA;
			// DevReferenceData.get().setResult(true);
		}

		/* Read XDS_ASCII.HKL for cell parameters */
		String content = ModelMultiStrategyParams.readLogFile(path, true);
		if (!content.contains("!FORMAT=XDS_ASCII")) {
			Log.error(path + " is not in XDS_ASCII.HKL file format");
			return 1;
		}

		int res1 = content.indexOf("!INCLUDE_RESOLUTION_RANGE");
		int pos1 = content.indexOf("!SPACE_GROUP_NUMBER");
		int pos2 = content.indexOf("!UNIT_CELL_CONSTANTS");

		if (pos2 < 0 || pos1 < 0) {
			Log.error("XDS_ASCII.HKL doesn't have cell parameter key words");
			return 1;
		}

		ModelReferenceData.REFERENCE_DATA = path;

		String res = "";
		StringTokenizer tok;
		if (res1 != -1) {
			res = content.substring(res1 + 27, content.indexOf("\n", res1));
			tok = new StringTokenizer(res);

			double low = Double.parseDouble(tok.nextToken());
			double high = Double.parseDouble(tok.nextToken());

			ModelReferenceData.RESOLUTION_LOW = low;
			ModelReferenceData.RESOLUTION_HIGH = high;
		}

		String spaceGroup = content.substring(pos1 + 21,
				content.indexOf("\n", pos1));
		spaceGroup = spaceGroup.trim();

		int spaceGroupNo = Integer.parseInt(spaceGroup);
		ModelReferenceData.SPACE_GROUP = spaceGroupNo;
		
		if (content.contains("FRIEDEL'S_LAW=FALSE")) {
			ModelReferenceData.FRIEDELSLAW = "FALSE";
		}
		
		String unitCell = content.substring(pos2 + 21,
				content.indexOf("\n", pos2));

		if (isHigres)
			ModelReferenceData.RESOLUTION_HIGH = 
					Double.parseDouble(ModelCombineDatasets.HighRes);

		ModelReferenceData.UNIT_CELL = getUnitCell(unitCell);
		DevReferenceData.get().setResult(true);
		return 0;
	}

	public static String getUnitCell(String unitCell) {

		StringTokenizer tok;
		tok = new StringTokenizer(unitCell);

		String trimCell = "";
		for (int i = 0; i < UnitCell.length; i++) {
			String cell = tok.nextToken();
			trimCell += cell + " ";
		}
		return (trimCell);
	}

	/*
	 * convert .sca file to XDS_ASCII.HKL file 
	 * !FORMAT=XDS_ASCII 
	 * !END_OF_HEADER
	 * 1 0 4 48445.  983.361 
	 * 1 0 5 2124.59 283.825 
	 * 1 0 6 17955.  493.356 
	 * 1 0 7 19328.  510.718 
	 * 1 0 8 87172.  3437.40 ... ... 
	 * !END_OF_DATA
	 */
	public static void readScaFile(String scaFile) {

		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(scaFile)));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			Log.error(scaFile + " not found");
			e.printStackTrace();
		}
		// Skip 2 lines.
		Log.note("Converting .sca file to .HKL of XDS");
		String sgNo = "1";
		try {
			for (int i = 0; i < 2; i++)
				reader.readLine();
			String cell = reader.readLine();

			ModelReferenceData.UNIT_CELL = getUnitCell(cell);
			StringTokenizer tok = new StringTokenizer(cell);

			for (int i = 0; i < UnitCell.length; i++)
				tok.nextToken();

			sgNo = tok.nextToken();
			// Log.error(sgNo);
			ModelReferenceData.SPACE_GROUP = DevMultiStrategy.get()
					.getNumberFromSpacegroup(sgNo);
			sgNo = Integer.toString(ModelReferenceData.SPACE_GROUP);

		} catch (IOException e) {
			// TODO Auto-generated catch block
			Log.error("Reading " + scaFile + " failed");
			e.printStackTrace();
		}

		/* Convert .sca file to XDS_ASCII.HKL file */
		String HKLFile = scaFile.substring(0, scaFile.lastIndexOf("."));
		String content = "";

		BufferedWriter out = null;
		try {
			out = new BufferedWriter(new FileWriter(HKLFile + ".HKL", false));
			out.write("!FORMAT=XDS_ASCII\n");
			out.write("!UNIT_CELL_CONSTANTS= " + ModelReferenceData.UNIT_CELL
					+ "\n");
			out.write("!SPACE_GROUP_NUMBER= " + sgNo + "\n");
			out.write("!NUMBER_OF_ITEMS_IN_EACH_DATA_RECORD= 5\n");
			out.write("!ITEM_H=1\n");
			out.write("!ITEM_K=2\n");
			out.write("!ITEM_L=3\n");
			out.write("!ITEM_IOBS=4\n");
			out.write("!ITEM_SIGMA(IOBS)=5\n");
			out.write("!END_OF_HEADER");
			out.write("\n");
			while ((content = reader.readLine()) != null) {
				out.write(content + "\n");
			}
			out.write("!END_OF_DATA");
			out.close();

		} catch (IOException e1) {
			Log.note("Writing .sca to .HKL failed");
			e1.printStackTrace();
		}
		try {
			reader.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		ModelReferenceData.REFERENCE_DATA = HKLFile + ".HKL";
	}

	@Override
	public void run() {

		loadHKLFile(this.PATH, this.isHighRes);
	}
}
