package gov.anl.gmca.raster;

import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.ModelRasterState.RasterState;
import gov.anl.gmca.raster.ui.TabRaster;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;
import java.util.HashMap;

public class ModelRasterRunList extends ModelRunList
{
	//////////////////////////////////////////// Variables ///////////////////////////////////////////////

	public static final int MAX_COUNT = 16;

	// These two methods refer to the current run index only for the run list it is called on
				protected	int							currentRun						= 0;
	@Override 	public 		int 						getCurrentRunIndex()			{ return currentRun;					}
	@Override	public		ModelRasterRun				getCurrentRunModel()
				{
					synchronized(runs)
					{
						if (currentRun<0) { System.out.println("currentRun="+currentRun+", returning null"); return null; }
						if (currentRun>=runs.size() && listType==RasterListType.INTER)
						{
							System.out.println("===runDebug=== currentRun="+currentRun+" runs.size()="+runs.size()+", returning null");
							Util.printStackTrace("runDebug");
							return null;
						}
						return runs.get(currentRun);
					}
				}
	
				protected 	ArrayList<ModelRasterRun> 	runs 							= new ArrayList<ModelRasterRun>();
	@Override	public		int 						getRunCount()					{ synchronized(runs) { return runs.size()-1; } }
	@Override	public 		ModelRun					getRun(int runIndex)			{ synchronized(runs) { return (ModelRun)runs.get(runIndex); } }
				public		ArrayList<ModelRasterRun>	getAllRunsCopy()				{ synchronized(runs) { return new ArrayList<ModelRasterRun>(runs); } }
	
				protected	DevRasterRunList			devRunList 						= null;
				protected	CmdRaster					cmdRaster						= null;
				public		boolean						isRasterThreadAlive()			{ return (null != cmdRaster && cmdRaster.isAlive()); }

	        	protected	static HashMap<RasterListType,ModelRasterRunList> instMap	= new HashMap<RasterListType,ModelRasterRunList>();
	        	public		static ModelRasterRunList	get(RasterListType type)		{ return instMap.get(type); }
		
	        	protected	RasterListType				listType						= null;
	        	public		RasterListType				getListType()					{ return listType; }
	        	
	//////////////////////////////////////////// Methods //////////////////////////////////////////////////
	
	public ModelRasterRunList(RasterListType type)
	{
		instMap.put(type,this);
		listType = type;
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				if (isMaster &&
					RasterState.INACTIVE != ModelRasterState.get().getRasterState() &&
					!isRasterThreadAlive())
				{
					crashRecovery();
				}
			}
		});
	}
	
	public void init()
	{
		// add run 0 model
		ModelRasterRun run = new ModelRasterRun(this,0);
		runs.add(run);
		
		// add run 0 device
		DevRasterRun devRun = new DevRasterRun(0, run);
		devRunList.runDevs.add(devRun);
		devRun.connect();
	}
	
	private void crashRecovery()
	{
		DevRasterState.get().setRasterState(RasterState.INACTIVE);
		
		/*Log.note("Crash detected in raster, processing skipped frames...");
		int processedFrames=0;
		
		try
		{
			ModelRasterRun currentRun = getCurrentRunModel();
			currentRun.dev().setRunStatus(RunState.INACTIVE,true);

			SFType sfType = listType==RasterListType.INTER ? SFType.INTER_RASTER : SFType.AUTO_RASTER;
			DBSpotFinder dbsf = DBSpotFinder.getDB(sfType, currentRun.getRunIndex());
			
			int fpeNextFrame = DevFPE.get().getCurrentFrameIndex();
			int bluiceNextFrame = currentRun.getNextFrame();
			int newNextFrame = bluiceNextFrame;
			
			// for each cell in the grid
			for (ModelRasterCell cell : currentRun.grid().getGridCalc().getCellListCopy())
			{
				// find cells that aren't done
				SFData.State cellState = dbsf.readCellFromDB(cell.getIndex()).state;
				if (cellState == SFData.State.DONE) continue;

				// this cell isn't done, did FPE take it?
				if (cell.getIndex() > fpeNextFrame) continue;
				
				// generate this image's filename
				int frameIndex = cell.getIndex();
				boolean isAutoRun = getListType()==RasterListType.AUTO;
				String fileRoot = currentRun.getFileRoot();
				int runLabel = currentRun.getRunLabel();
				int rowNumber = cell.row+1, columnIndex = cell.col;
				String imageName = ModelRasterCell.generateImageFilename(isAutoRun, frameIndex, fileRoot, runLabel, rowNumber, columnIndex);
				String imageFullPath = currentRun.getFullDirectory()+"/"+imageName;
				
				// FPE says it took the frame, is it written?
				System.out.println("Checking file path "+imageFullPath);
				if (!Disk.exists(imageFullPath)) continue;
				
				// cell was definitely taken but not processed, so process it
				CmdSpotFinder cmd = new CmdSpotFinderRemote(sfType, SFRunType.RASTER, this.currentRun, frameIndex, currentRun.getFullDirectory(), imageName, currentRun.getLowResLimit_A(), currentRun.getHighResLimit_A(), 3, DevSFType.getConfig(SFRunType.RASTER).id);
				SpotFinderQueue.submitTask(cmd);				

				// increment this run's next frame index
				newNextFrame = frameIndex+1;
				
				processedFrames++;
			}
			
			if (bluiceNextFrame != newNextFrame)
			{
				currentRun.dev().setNextFrame(newNextFrame, true);
			}
			DevFPE.get().setETA(0);
		}
		catch (Exception e) { e.printStackTrace(); }
		
		Log.note("Raster crash recovery complete, started processing for "+processedFrames+" frames.");*/
	}
	
	public void setCurrentRun(int currentRun)
	{
		if (currentRun!=this.currentRun)
		{
			this.currentRun = currentRun;
		}
		
		notifyListeners();
		notifyCurrentRunListeners();
	}
	
	private boolean firstSet = true;
	public void setRunCount(int newRunCount)
	{
		/*if (listType==RasterListType.INTER)
		{
			System.out.println("===runDebug=== setRunCount("+newRunCount+")");
			Util.printStackTrace("runDebug");
		}*/
		
		if (getRunCount()==newRunCount && !firstSet) return;
		firstSet=true;
		
		ArrayList<DevRasterRun> newDevices = new ArrayList<DevRasterRun>();
		synchronized(runs)
		{
			// remove runs
			for (int runIndex = getRunCount(); runIndex > newRunCount; runIndex--)
			{
				// delete the model
				runs.get(runIndex).stopMonitors();
				runs.remove(runIndex);
				
				// delete the device
				devRunList.runDevs.get(runIndex).stopEpicsMonitors();
				devRunList.runDevs.remove(runIndex);
			}
			
			// add runs
			for (int runIndex = getRunCount()+1; runIndex <= newRunCount; runIndex++)
			{
				// add the model
				ModelRasterRun run = new ModelRasterRun(this,runIndex);
				runs.add(run);
				
				// add the devices
				DevRasterRun devRun = new DevRasterRun(runIndex, run);
				newDevices.add(devRun);
				devRunList.runDevs.add(devRun);
			}
			
			//Log.note("=== new run count = "+newRunCount+" run list size = "+runs.size());
		}
		
		// update the gui
		if (TabRaster.get() != null && getListType() == RasterListType.INTER)
		{
    		final int runCount = getRunCount();
    		final int currentRunIndex = getCurrentRunIndex();
    
    		Util.safeSyncExec(new Runnable(){public void run()
    		{
    			TabRaster rasterTab = TabRaster.get();
    			WdgRunTabs runTabs = rasterTab.runTabs;
    			runTabs.configureTabs(runCount,currentRunIndex);
    		}});
		}
		else
		{
			// TODO: update auto GUI
		}
		
		// connect the device to fill in the gui fields
		for (DevRasterRun devRun : newDevices)
		{
			devRun.connect();
		}
		
		notifyListeners();
		//notifyCurrentRunListeners();
	}
	
    protected boolean validateRunCount(int runCount)
    {
		if (runCount<0)								{ return false; }
		if (runCount>ModelRasterRunList.MAX_COUNT)	{ return false; }
		return true;
    }
    
    protected boolean validateCurrentRun(int currentRun)
    {
		if (currentRun<0)							{ return false; }
		if (currentRun>getRunCount()) 				{ return false; }
		return true;
    }
    
    public static void start()
    {
		ModelRasterRun currentRun = ModelRasterState.get().getCurrentRunModel();
		if (currentRun.getRunType() == ModelRasterRun.RunType.DIFFRACTION)
		{
			ModelRasterRunList.get(RasterListType.INTER).getCurrentRunModel().start();
		}
		else if (currentRun.getRunType() == ModelRasterRun.RunType.SONICC)
		{
			//CmdSONICCAcquire.start();
			throw new IllegalStateException("SONICC not supported from this tab");
		}
    }
}
