package gov.anl.gmca.raster;

import gov.anl.gmca.collect.ModelCollectRun.Point3D;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Point4D;

import java.util.ArrayList;

public class ModelRasterSiteList
{
	public static ModelRasterSiteList get() { return inst; }
	public static ModelRasterSiteList inst = null;
	public ListenerList<ModelRasterSiteList> ll;
	
	protected ArrayList<Site> siteList = new ArrayList<Site>();
	public ArrayList<Site> getSiteListCopy()
	{
		ArrayList<Site> ret;
		synchronized(siteList)
		{
			ret = new ArrayList<Site>(siteList);
		}
		return ret;
	}
	public int getHighestSiteNumber()
	{
		int max=-1;
		synchronized(siteList)
		{
			for (Site site : siteList)
			{
				max = Math.max(max, site.index);
			}
		}
		return max;
	}
	
	public static class SiteSource
	{
		public String id;	// short version: "AO3H1"
		public String desc;	// long version:  "auto ortho run 3 hit 1"
		public SiteSource(String id, String desc) { this.id=id; this.desc=desc; }
	}
	
	public static class Site
	{
		public int index; // 1-based
		public int interRunIndex, interCell;
		public int fineRunIndex, orthoRunIndex, hitNumber;
		public Point4D motorPos;
		public Point3D size;
		
		public Site(int number, int interRunIndex, int interCell, int fineRunIndex, int orthoRunIndex, int hitNumber, Point4D motorPos, Point3D size)
		{
			this.index=number;
			this.interRunIndex=interRunIndex;
			this.interCell=interCell;
			this.fineRunIndex=fineRunIndex;
			this.orthoRunIndex=orthoRunIndex;
			this.hitNumber=hitNumber;
			this.motorPos=motorPos;
			this.size=size;
		}
		
		public int getMicronsCubed()
		{
			return (int)(size.x*size.y*size.z);
		}
		
		public SiteSource getSource()
		{
			if (interRunIndex>=0)
			{
				int interRunLabel = DevRasterRun.s_getRunLabel(RasterListType.INTER, interRunIndex);
				if (hitNumber>0)
				{
					return new SiteSource
					(
						"IR"+interRunLabel+"H"+hitNumber,
						"Interactive run "+interRunLabel+" hit "+hitNumber
					);
				}
				else
				{
					return new SiteSource
					(
						"IR"+interRunLabel+"C"+(interCell+1),
						"Interactive run "+interRunLabel+" cell "+(interCell+1)
					);
				}
			}
			else if (fineRunIndex>=0)
			{
				return new SiteSource
				(
					"AF"+fineRunIndex+"O"+orthoRunIndex+"H"+hitNumber,
					"Auto fine run "+fineRunIndex+" ortho run "+orthoRunIndex+" hit number "+hitNumber
				);
			}
			else
			{
				return new SiteSource
				(
					"AO"+orthoRunIndex+"H"+hitNumber,
					"Auto ortho run "+orthoRunIndex+" hit number "+hitNumber
				);
			}
		}

		public String getSizeString()
		{
			if (interRunIndex >= 0)
			{
				return 	(int)Math.round(size.x)+"x"+
						(int)Math.round(size.y);
			}
			else
			{
				return 	(int)Math.round(size.x)+"x"+
						(int)Math.round(size.y)+"x"+
						(int)Math.round(size.z);
			}
		}
	}
	
	public ModelRasterSiteList()
	{
		inst = this;
		ll = new ListenerList<ModelRasterSiteList>(this);
	}
	
	void addSite(Site site)
	{
		synchronized(siteList)
		{
			siteList.add(site);
			//Log.note("Added site "+site.getSource().desc);
		}
	}

	void clearRows(boolean notifyListeners)
	{
		synchronized(siteList)
		{
			siteList = new ArrayList<Site>();
		}
		if (notifyListeners) ll.notifyListeners();
	}
	
	public Site getSiteForHit(int currentHit, int interRunIndex, int autoRunIndex)
	{
		synchronized(siteList)
		{
			Site ret = null;
			for (Site site : siteList)
			{
				if (site.interRunIndex==interRunIndex && site.hitNumber==currentHit)
				{
					if (null!=ret) throw new IllegalStateException("Multiple sites matched hit "+currentHit+" interRun="+interRunIndex+" autoRun="+autoRunIndex);
					ret=site;
				}
			}
			return ret;
		}
	}
}
