package gov.anl.gmca.raster;

import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.common.ModelRunList.CurrentRunListener;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.util.ListenerList;

public class ModelRasterState
{
	// current run listener for both auto and inter lists, and also
	// updates on any config changes
	public		ListenerList<ModelRasterState>	llCurrentRun;
	
	public 		enum 						RasterState 					{ INACTIVE, RASTERING, STOPPING };
	protected 	RasterState 				rasterState 					= RasterState.INACTIVE;
	public 		RasterState 				getRasterState() 				{ return rasterState; 			}
	public		void						setRasterState(RasterState rs) 	{ if (rasterState != rs) { rasterState = rs; llCurrentRun.notifyListeners(); } }

	protected 	BinMode 					binMode 						= BinMode.UM_72;
	public 		BinMode 					getBinMode() 					{ return binMode; 			}
	public		void						setBinMode(BinMode bmode) 		{ if (binMode != bmode) { binMode = bmode; llCurrentRun.notifyListeners(); } }
	
	public		static enum					RasterListType					{ INTER, AUTO };
	protected	RasterListType				autoMode						= RasterListType.INTER;
	public		RasterListType				getAutoMode()					{ return autoMode; }
	protected	void						setAutoMode(RasterListType t)	{ if (autoMode != t) { autoMode = t; llCurrentRun.notifyListeners(); } }
	
	protected	boolean						fixLimits						= false;
	public		boolean						getFixLimits()					{ return fixLimits; }
	public		void						setFixLimits(boolean b) 		{ fixLimits = b; }
	
	protected	boolean						bShowBeamsize					= true;
	public		boolean						getShowBeamsize()				{ return bShowBeamsize; }
	public		void						setShowBeamsize(boolean b) 		{ bShowBeamsize = b; }
	
	protected	boolean						showCorrectedCounts					= true;
	public		boolean						getShowCorrectedCounts()			{ return showCorrectedCounts; }
	public		void						setShowCorrectedCounts(boolean b) 	{ showCorrectedCounts = b; }
	
	protected	SPOTFINDER_OUTPUT			diffDisplayParameter			= SPOTFINDER_OUTPUT.DIALS_SPOTS_NO_ICE;
	public		SPOTFINDER_OUTPUT			getDiffDisplayParameter()		{ return diffDisplayParameter; }
	public		void						setDiffDisplayParameter(SPOTFINDER_OUTPUT param) { diffDisplayParameter = param; llCurrentRun.notifyListeners(); }
	
	public		static enum					FluoDataParameter				{ NONE, FL_SIGNAL };
	protected	FluoDataParameter			fluoDisplayParameter			= FluoDataParameter.FL_SIGNAL;
	public		FluoDataParameter			getFluoDisplayParameter()		{ return fluoDisplayParameter; }
	public		void						setFluoDisplayParameter(FluoDataParameter param) { fluoDisplayParameter = param; }

	public		static enum					SONICCDisplay					{ NONE, SONICC };
	protected 	SONICCDisplay 				soniccDisplay					= SONICCDisplay.SONICC;
	public 		SONICCDisplay 				getSONICCDisplay()				{ return soniccDisplay; }
	public		void						setSONICCDisplay(SONICCDisplay sd) { soniccDisplay = sd; }

	public		static enum					NumberingParameter				{ NONE, CELL, HEATMAP };
	protected	NumberingParameter			numberingParameter				= NumberingParameter.CELL;
	public		NumberingParameter			getNumberingParameter()			{ return numberingParameter; }
	public		void						setNumberingParameter(NumberingParameter param) { numberingParameter = param; }
	
	public		ModelRasterRunList			getActiveRasterList()			{ return ModelRasterRunList.get(getAutoMode()); }

	/////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected	static ModelRasterState		inst							= null;
	public		static ModelRasterState		get()							{ return inst; }
	
	protected	ModelRasterRun				currentRun						= null;
	
	public ModelRasterState()
	{
		inst = this;
		
		llCurrentRun = new ListenerList<ModelRasterState>(this);
		
		ModelRasterRunList.get(RasterListType.INTER).addCurrentRunListener(new CurrentRunListener()
		{
			@Override public void currentRunChanged(final ModelRun run)
			{
				llCurrentRun.notifyListeners();
			}
		});
		ModelRasterRunList.get(RasterListType.AUTO).addCurrentRunListener(new CurrentRunListener()
		{
			@Override public void currentRunChanged(ModelRun run)
			{
				if (getCurrentRunModel()!=null) llCurrentRun.notifyListeners();
			}
		});
	}
	
	public void pause()
	{
		if (RasterState.RASTERING == getRasterState())
		{
			DevFPE.get().pause(true);
			DevRasterState.get().setRasterState(RasterState.STOPPING);
		}
	}
	
	public ModelRasterRun getCurrentRunModel()
	{
		return ModelRasterRunList.get(getAutoMode()).getCurrentRunModel();
	}
	
	public static boolean isRasterThreadAlive()
	{
		if (ModelRasterRunList.get(RasterListType.AUTO).isRasterThreadAlive()) return true;
		if (ModelRasterRunList.get(RasterListType.INTER).isRasterThreadAlive()) return true;
		return false;
	}
	
	public static boolean isRasterRunning()
	{
		try
		{
			ModelRasterRun currentRun = ModelRasterState.get().getCurrentRunModel();
			RunState currentRunState = currentRun.getRunState();
			int currentRunIndex = currentRun.getRunIndex();
			
			ModelRasterRunList activeList = ModelRasterState.get().getActiveRasterList();
			RasterListType activeListType = activeList.getListType();
			
			boolean rasterRunning =
			(
				RunState.RASTERING == currentRunState &&
				!(0 == currentRunIndex && RasterListType.AUTO == activeListType)
			);
			return rasterRunning;
		}
		catch (NullPointerException npe)
		{
			// print out the stack trace - this should never be reached
			//npe.printStackTrace();
			System.out.println("Null pointer exception from isRasterRunning(), returning false");
			return false;
		}
	}
}
