package gov.anl.gmca.raster.cam;

import gov.anl.gmca.common.Redis;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevEnergy;
import gov.anl.gmca.raster.*;
import gov.anl.gmca.raster.ModelRasterHits.HitSource;
import gov.anl.gmca.raster.ModelRasterState.NumberingParameter;
import gov.anl.gmca.raster.cam.ModelRasterVideo.VideoState;
import gov.anl.gmca.sample.DBBeamSize;
import gov.anl.gmca.sample.DBBeamSize.BeamSize;
import gov.anl.gmca.sample.WdgCamera.CameraType;
import gov.anl.gmca.spotfinder.DBImageStats;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Point4D;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.nio.FloatBuffer;
import java.util.ArrayList;

public class DrawStandard
{
	protected 	RasterMap 				rasterMap					= null;
	protected 	BufferedImage			imgBuffer 					= null;
	protected 	BufferedImage			lastCameraImage 			= null;

	final public static int 	HIT_RADIUS 				= 15;
	final static int 	HIT_BORDER				= 1;
	final static Color 	HIT_OUTER_BORDER_COLOR	= Color.black;
	final static Color 	HIT_INNER_BORDER_COLOR	= Color.white;
	final static Color 	HIT_TEXT_BORDER_COLOR	= Color.black;
	final static Color 	HIT_TEXT_COLOR			= Color.white;
	
	private ModelRasterVideo model;
	public DrawStandard(ModelRasterVideo model)
	{
		this.model = model;
		rasterMap = new RasterMap(model);
	}
	
	public void drawOverlays(Graphics2D g2)
	{
		try
		{
	    	// save motor positions and px2mm conversions once here so they are consistent while drawing
	    	model.currentFrameMotorPos = model.cam.getImageMotorPos().toArray();
        	model.current_mm2px.x = WdgRasterCamera.getConversion(CameraType.HIGHRES, SWT.HORIZONTAL);
        	model.current_mm2px.y = WdgRasterCamera.getConversion(CameraType.HIGHRES, SWT.VERTICAL);
	    	
			//PerfTimer pt = new PerfTimer("grid draw: ");
	    	drawColors(g2);
	    	drawCellOutlines(g2, false, false, true);
			drawCellOutlines(g2, true, false, false);
			drawCellOutlines(g2, false, true, false);
			drawGridNumbers(g2);
			drawContours(g2);
	   		drawHitLabels(g2);
	   		drawBeamsize(g2);
	   		drawAngleDiffMessage(g2, ModelRasterVideo.OMEGA_TOLERANCE_DEG);
	   		model.vcViewImages.draw.drawImageViewCell(g2);
	   		model.vcCellHover.draw.drawCellHover(g2);
	   		model.cam.drawCrosshairAndBorder(g2,model.cam.displayWidth,model.cam.displayHeight);
	   		model.vcCreateSite.draw.drawSiteHighlight(g2);
			//pt.printElapsedMs();
		}
		catch (Exception e) { e.printStackTrace(); }
	}
	
	public void drawColors(Graphics2D gr)
	{
		ModelRasterRun currentRun = model.getDisplayRun();
		if (null == currentRun || null==currentRun.grid().getCenter()) return;

		if (null==rasterMap) return;
		BufferedImage rmap = rasterMap.getImage();
		if (null==rmap) return;

		Point2D micronsTopLeft = new Point2D
		(
			currentRun.grid().getCorrectedLeftOffset_um(),
			currentRun.grid().getCorrectedTopOffset_um()
		);
		Point screenTopLeft = model.gridRelativeMicrons2Screen(micronsTopLeft);

		if (null==screenTopLeft) return;
		gr.drawImage(rmap, null, screenTopLeft.x+1, screenTopLeft.y+1);
	}

	public void drawAngleDiffMessage(Graphics2D gr, double tolerance_deg)
	{
		ModelRasterRun currentRun = model.getDisplayRun();
		Point4D currentRunCenter = currentRun.grid().getCenter();
		if (null != currentRunCenter)
		{
			double runOmega = currentRunCenter.omega;
			double currentOmega = model.cam.getImageMotorPos().omega;
			
			double omegaDiff = Math.abs(runOmega-currentOmega);
			if (omegaDiff > tolerance_deg)
			{
				String msg = Util.sprintf
				(
					"Current gonio angle differs from the grid's angle by %.1f\u00b0",
					omegaDiff
				);
				drawMessage(gr, msg);
			}
		}
	}

	public void drawMessage(Graphics2D gr, String msg)
	{
		int x=10,y=20;
		gr.setColor(Color.BLACK);
		gr.setFont(GUI.FONT_12_BOLD_AWT);
		gr.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		for (int i=-1;i<=1;i++)
			for (int j=-1;j<=1;j++)
			{
				gr.drawString(msg, x+i, y+j);
			}
		gr.setColor(Color.YELLOW);
		gr.drawString(msg, x, y);
	}
	
	public void drawBeamsize(Graphics2D gr)
	{
		if (!ModelRasterState.get().getShowBeamsize()) return;

		ModelRasterRun currentRun = model.getDisplayRun();
		if (null == currentRun) return;

		double beamWidth_mm, beamHeight_mm;
		if (!Util.beamSizeCalculated()) {
			beamWidth_mm = currentRun.getBeamWidth_um()/1000.0;
			beamHeight_mm = currentRun.getBeamHeight_um()/1000.0;
		} else {
			BeamSize bs = DBBeamSize.getBeamSize((int)Math.round(currentRun.getBeamWidth_um()), DevEnergy.getDevice().getPosition());
			beamWidth_mm = bs.width_um/1000.0;
			beamHeight_mm = bs.height_um/1000.0;
		}

		int beamWidth_px = (int)model.cam.mm2pxH(beamWidth_mm);
		int beamHeight_px = (int)model.cam.mm2pxV(beamHeight_mm);
		
		int xmid = model.cam.displayWidth/2;
		int ymid = model.cam.displayHeight/2;
		
    	gr.setPaint(new Color(1,0,0,200f/255f));
    	gr.setStroke(new BasicStroke(1));
		if (beamWidth_mm <= 0.024 || Util.beamSizeCalculated())
		{
			gr.drawOval(xmid-beamWidth_px/2, ymid-beamHeight_px/2, beamWidth_px, beamHeight_px);
		}
		else
		{
			gr.drawRect(xmid-beamWidth_px/2, ymid-beamHeight_px/2, beamWidth_px, beamHeight_px);
		}
	}
	
	protected void drawContours(Graphics2D gr)
	{
		// draw finished contours
		//Log.note("Drawing "+ModelRasterRunList.get().getCurrentRunModel().grid().getContours().size()+" contours");
		ModelRasterRun currentRun = model.getDisplayRun();
		if (ModelRasterVideo.showPolygonOutlines)
		{
			for (ModelContour contour : currentRun.grid().getContours())
			{
				drawContour(gr, contour, true);
			}
		}
		
		// draw the current contour
    	if (VideoState.CREATE_CONTOUR == model.getState())
    	{
    		drawContour(gr, model.vcCreatePolygon.getCurrentContour(), false);
    	}
		
		// draw the point for the current contour
		//gc.fillOval(mouseX-5, mouseY-5, 10, 10);
	}
	
	protected void drawContour(Graphics2D gr, ModelContour contour, boolean closeContour)
	{
		for (int drawOutline=1;drawOutline>=0;drawOutline--)
		{
			if (0==drawOutline)
			{
    			gr.setBackground(GUI.COLOR_LIGHT_BLUE_AWT);
    			gr.setColor(GUI.COLOR_LIGHT_BLUE_AWT);
    	    	gr.setStroke(new BasicStroke(3));
			}
			else
			{
    			gr.setBackground(Color.black);
    			gr.setColor(Color.black);
    	    	gr.setStroke(new BasicStroke(4));
			}
			
    		Point lastPoint = null;
			ModelRasterRun currentRun = model.getDisplayRun();
    		if (null == currentRun.grid().getCenter()) return;
    		
    		// draw points and lines for each point in the polygon
    		for (Point2D point : contour.points)
    		{
    			Point screenPoint = model.gridRelativeMicrons2Screen(point);
    			if (null==screenPoint) continue;
    			
    			if (null != lastPoint)
    			{
    				gr.drawLine(lastPoint.x, lastPoint.y, screenPoint.x, screenPoint.y);
    			}
    			
    			gr.fillOval((int)screenPoint.x-5, (int)screenPoint.y-5, 10, 10);
    			lastPoint = screenPoint;
    		}
    		
    		// close the polygon
    		if (null != lastPoint && contour.points.size() > 0 && closeContour)
    		{
    			Point2D point = contour.points.get(0);
    			
    			Point screenPoint = model.gridRelativeMicrons2Screen(point);
    			
        		gr.drawLine(lastPoint.x, lastPoint.y, screenPoint.x, screenPoint.y);
    		}
		}
	}
	
	public void drawCellOutlines(Graphics2D gr, boolean drawHitBorders, boolean drawHitOutlines, boolean drawGridLines)
	{
		ModelRasterRun currentRun = model.getDisplayRun();
		if (null == currentRun) return;
		ModelRasterGrid currentGrid = currentRun.grid();
		
		// TODO: this is a temporary fix to avoid drawing cells while
		// creating the first contour
		if (currentGrid.getContours().size()==0) return;
		
		// outline configuration
		int bw=4; 		// half border width
		int ow=3; 		// half outline width
		int wd=bw-ow;	// width difference
		
		if (drawHitBorders)
		{
			gr.setStroke(new BasicStroke(bw*2));
			gr.setColor(Color.blue);
		}
		else if (drawHitOutlines)
		{
			gr.setStroke(new BasicStroke(ow*2));
   			gr.setColor(GUI.COLOR_PURPLE_AWT);
		}
		else if (drawGridLines)
		{
			gr.setColor(new Color(0f,0f,200f/255f));
			gr.setStroke(new BasicStroke(1));
		}
		
		// draw the cells
	    for (ModelRasterCell cell : currentGrid.getGridCalc().getCellListCopy())
	    {
			Rectangle bounds = model.getCellScreenBounds(cell);
			if (null==bounds) continue;
			int x1 = bounds.x;
			int y1 = bounds.y;
			int x2 = bounds.x+bounds.width;
			int y2 = bounds.y+bounds.height;
			
			int hitNumber = ModelRasterHits.get().getHitNumber(new HitSource(cell));
			boolean isHit = hitNumber>=0;
			if (isHit && (drawHitBorders || drawHitOutlines))
			{
				boolean drawTop=false,drawBottom=false,drawLeft=false,drawRight=false;
				
				ModelRasterCell
    				topCell		=cell.grid().getCell(cell.getRow()-1,cell.getCol()),
    				bottomCell	=cell.grid().getCell(cell.getRow()+1,cell.getCol()),
    				leftCell	=cell.grid().getCell(cell.getRow(),cell.getCol()-1),
    				rightCell	=cell.grid().getCell(cell.getRow(),cell.getCol()+1);

				// set value to -2 for null cells, current hit number will be either valid or -1
    			int hitNumberTop 	= null==topCell		?-2:ModelRasterHits.get().getHitNumber(new HitSource(topCell));
    			int hitNumberBottom = null==bottomCell	?-2:ModelRasterHits.get().getHitNumber(new HitSource(bottomCell));
    			int hitNumberLeft	= null==leftCell	?-2:ModelRasterHits.get().getHitNumber(new HitSource(leftCell));
    			int hitNumberRight	= null==rightCell	?-2:ModelRasterHits.get().getHitNumber(new HitSource(rightCell));
    			
				if (hitNumberTop	!= hitNumber)	drawTop=true;
				if (hitNumberBottom	!= hitNumber)	drawBottom=true;
				if (hitNumberLeft	!= hitNumber)	drawLeft=true;
				if (hitNumberRight	!= hitNumber)	drawRight=true;
				
				if (drawHitBorders)
				{
        			x1 -= wd; y1 -= wd; x2 += wd; y2 += wd;
				}
				if (drawTop)		gr.drawLine(x1,y1,x2,y1);
				if (drawBottom)		gr.drawLine(x1,y2,x2,y2);
				if (drawLeft)		gr.drawLine(x1,y1,x1,y2);
				if (drawRight)		gr.drawLine(x2,y1,x2,y2);
			}
			else if (drawGridLines)
			{
				gr.drawRect(x1, y1, x2-x1, y2-y1);
			}
	    }
		gr.setStroke(new BasicStroke(1));
	}

	protected void drawHitLabels(Graphics2D gr)
	{
		ModelRasterRun currentRun = model.getDisplayRun();
		ModelRasterGrid currentGrid = currentRun.grid();
		
		// draw hit numbers
		int hit_draw_max = 500, hitIndex=0;
		for (ModelRasterHits.Hit hit : ModelRasterHits.get().getCopyOfHits(currentGrid.run()))
		{
			Point2D hitScreenXY = model.cam.motorMMtoCurrentScreenPx(hit.getMotorXYZO());
			drawNumberInCircle(gr, hitScreenXY.round(), Integer.toString(hit.getNumber()), GUI.COLOR_PURPLE_AWT);
			if (hitIndex++>hit_draw_max) break;
		}
	}
	
	public void drawNumberInCircle(Graphics2D g2, Point screenXY, String text, Color bgColor)
	{
		int ovalX=screenXY.x-HIT_RADIUS;
		int ovalY=screenXY.y-HIT_RADIUS;

		if (null!=bgColor) {
			g2.setColor(HIT_OUTER_BORDER_COLOR);
			g2.fillOval(ovalX-1, ovalY-1, HIT_RADIUS*2+2, HIT_RADIUS*2+2);
			g2.setColor(HIT_INNER_BORDER_COLOR);
			g2.fillOval(ovalX, ovalY, HIT_RADIUS*2, HIT_RADIUS*2);

			g2.setColor(bgColor);
			g2.fillOval
			(
				screenXY.x - HIT_RADIUS + HIT_BORDER,
				screenXY.y - HIT_RADIUS + HIT_BORDER,
				HIT_RADIUS * 2 - HIT_BORDER * 2,
				HIT_RADIUS * 2 - HIT_BORDER * 2
			);
		} else {
			g2.setColor(HIT_OUTER_BORDER_COLOR);
			g2.drawOval(ovalX-1, ovalY-1, HIT_RADIUS*2+2, HIT_RADIUS*2+2);
			g2.setColor(HIT_INNER_BORDER_COLOR);
			g2.drawOval(ovalX, ovalY, HIT_RADIUS*2, HIT_RADIUS*2);
		}
		
		Rectangle2D extent = g2.getFontMetrics().getStringBounds(text, g2);
		
		int strX=screenXY.x-((int)extent.getWidth())/2;
		int strY=screenXY.y+((int)extent.getHeight())/2;
		g2.setColor(HIT_TEXT_BORDER_COLOR);
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.drawString(text, strX-1, strY-1);
		g2.drawString(text, strX-1, strY+1);
		g2.drawString(text, strX+1, strY-1);
		g2.drawString(text, strX+1, strY+1);
		g2.setColor(HIT_TEXT_COLOR);
		g2.drawString(text, strX, strY);
	}
	
	public void drawGridNumbers(Graphics2D gr)
	{
		// TODO: this is a temporary fix to avoid drawing cells while
		// creating the first contour
		ModelRasterRun currentRun = model.getDisplayRun();
		if (null == currentRun) return;
		ModelRasterGrid currentGrid = currentRun.grid();
		if (currentGrid.getContours().size()==0) return;
		
		if (ModelRasterState.get().getNumberingParameter()==NumberingParameter.NONE) return;
		
		gr.setFont(GUI.FONT_DEFAULT_AWT);

		ArrayList<ModelRasterCell> cellList = currentGrid.getGridCalc().getCellListCopy();
		
		int halfCellWidthPx  = (int)Math.round(model.cam.mm2pxH(currentGrid.getCellWidth_um()/2.0/1000.0));
		int halfCellHeightPx = (int)Math.round(model.cam.mm2pxV(currentGrid.getCellHeight_um()/2.0/1000.0));

		FontMetrics metrics = gr.getFontMetrics();
		int sw = metrics.stringWidth(""+cellList.size());
		int sh = metrics.getHeight();
		int skipX = (int)Math.round((double)(sw-2)/(halfCellWidthPx*2.0));
		int skipY = (int)Math.round((double)(sh-4)/(halfCellHeightPx*2.0));
		if (skipX==0) skipX=1; if (skipY==0) skipY=1;
		
		int cellNumber = 1;
		String output = model.displayParam;
		FloatBuffer data = DBImageStats.load(Redis.image_stats(currentRun.getDirectorySuffix(), currentRun.getFileRoot(), currentRun.getRunLabel(), output));
		for (ModelRasterCell cell : cellList)
		{
			if (cell.getCol()%skipX!=0 || cell.getRow()%skipY!=0) { cellNumber++; continue; }
			
			//if (!ModelRasterRunList.get().getCurrentRunModel().grid().getGridCalc().getGridCalc().getCellList().get(cellNumber).getSelected()) continue;
			
			String cellString = "";
			boolean exceptionPrinted = false;
			try
			{
				switch (model.displayNumbering)
				{
				case NONE: break;
				case CELL: cellString = Integer.toString(cellNumber); break;
				case HEATMAP:
				{
					if (null==data) cellString = "---";
					else {
						cellString = Util.printDouble((double)data.get(cell.index), 0);
					}
					break;
				}
				}
			}
			catch (Exception e) { if (!exceptionPrinted) { e.printStackTrace(); exceptionPrinted=true; } }
			
			// calculate the middle of the cell using fixed cell size numbers
			// this ties the number position to the top and left cell borders to make it more stable
			Rectangle bounds = model.getCellScreenBounds(cell);
			if (null==bounds) continue;
			int x1 = bounds.x;
			int y1 = bounds.y;
			int x = x1 + halfCellWidthPx;
			int y = y1 + halfCellHeightPx;

			// center the text label
			Rectangle2D extent = gr.getFontMetrics().getStringBounds(cellString,gr);
			x -= ((int)extent.getWidth())/2;
			y += ((int)extent.getHeight())/2;

			// draw an outline
			gr.setColor(Color.black);
			gr.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			for (int a=-1;a<=1;a++)
			for (int b=-1;b<=1;b++)
			{
				if (a!=0||b!=0) gr.drawString(cellString, x+a, y+b);
			}
			gr.setColor(Color.white);
			
			gr.drawString(cellString, x, y);
			
			cellNumber++;
		}
	}
}
