package gov.anl.gmca.raster.cam;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.calc.RasterGeometry;
import gov.anl.gmca.raster.sonicc.SONICCImage;
import gov.anl.gmca.raster.ui.WdgSONICCTab;
import gov.anl.gmca.sample.WdgCamera;
import gov.anl.gmca.util.Point4D;
import gov.anl.gmca.util.SwingUtil;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;

import java.awt.*;
import java.awt.image.BufferedImage;

public class SONICCView
{
    public static BufferedImage imgFront;

    public static void init()
    {
        if (Util.getBeamlineNumberFromHostname()!=1 && !Main.SONICC_TEST_MODE) return;

        new Thread(new Runnable(){public void run()
        {
            while (true) {
                try {
                    updateImage();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                Util.sleep(100);
            }
        }},"SONICC Image").start();
    }

    public static void draw(GC gc, WdgRasterCamera cam, int targetWidth, int targetHeight)
    {
        if (Util.getBeamlineNumberFromHostname()!=1 && !Main.SONICC_TEST_MODE) return;

        // determine scaling to fit both dimensions to the target size
        int srcWidth=cam.getImgWidth(), srcHeight=cam.getImgHeight();
        float xScale = (float)targetWidth/(float)srcWidth;
        float yScale = (float)targetHeight/(float)srcHeight;
        cam.ratio = Math.min(xScale, yScale);
        cam.displayWidth = (int)(srcWidth*cam.ratio);
        cam.displayHeight = (int)(srcHeight*cam.ratio);

        // create the Swing image
        BufferedImage buf = null;
        Graphics2D bufG2 = null;

        // create a buffer with the scaled source image
        try
        {
            buf = new BufferedImage(cam.displayWidth, cam.displayHeight, BufferedImage.TYPE_4BYTE_ABGR);
            bufG2 = (Graphics2D)buf.getGraphics();

            // draw scaled imgFront to the buffer
            bufG2.setRenderingHint(RenderingHints.KEY_INTERPOLATION,RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
            if (null!=imgFront) bufG2.drawImage(imgFront, 0, 0, cam.displayWidth, cam.displayHeight, null);

            bufG2.setColor(Color.RED);
            bufG2.drawLine(cam.displayWidth/2,0,cam.displayWidth/2,cam.displayHeight);
            bufG2.drawLine(0,cam.displayHeight/2,cam.displayWidth,cam.displayHeight/2);

            // draw overlays
            cam.drawSimpleOverlays(bufG2);

            // determine draw range
            cam.drawX = Math.max(0, (targetWidth - cam.displayWidth)/2);
            cam.drawY = Math.max(0, (targetHeight - cam.displayHeight)/2);

            // fill areas outside image
            gc.setBackground(GUI.BGCOLOR);
            gc.fillRectangle(0, 0, targetWidth, cam.drawY);													// top
            gc.fillRectangle(0, cam.drawY+cam.displayHeight, targetWidth, targetHeight-cam.displayHeight-cam.drawY); 	// bottom
            gc.fillRectangle(0, 0, cam.drawX, targetHeight); 												// left
            gc.fillRectangle(cam.drawX+cam.displayWidth, 0, targetWidth-cam.displayWidth-cam.drawX, targetHeight); 		// right

            // paint frame centered on target image
            Image swtImg = null;
            try
            {
                swtImg = SwingUtil.AWTImageToSWTImage(buf);
                gc.setBackground(GUI.COLOR_GREEN);
                gc.drawImage(swtImg,cam.drawX,cam.drawY);
            }
            catch (Exception e) { e.printStackTrace(); }
            finally
            {
                if (null != swtImg && !swtImg.isDisposed()) swtImg.dispose();
            }
        }
        catch (Exception e) { e.printStackTrace(); }
        finally
        {
            if (null!=bufG2) bufG2.dispose();
        }

    }

    public static void updateImage()
    {
        ModelRasterRun currentRun = ModelRasterState.get().getCurrentRunModel();
        if (null==currentRun) return;
        Point4D ref_pos = currentRun.getSONICCRefPos();

        // load sonicc img
        SONICCImage img = null;
        if (null!=ref_pos)
        {
            img = new SONICCImage(WdgSONICCTab.channelString);
            if (Main.SONICC_TEST_MODE)
                img.loadFromDisk();
            else
                img.loadFromHTTP();

            boolean cropToRectangle = true;
            boolean cameraTransform = true;
            img.transform(ref_pos, cropToRectangle, cameraTransform);
            imgFront = SONICCImage.applyToHRImg(img.transformedImg, img.hrTopLeft);
        } else {
            imgFront = SONICCImage.applyToHRImg(null,null);
        }
    }
}
