package gov.anl.gmca.raster.cam;

import gov.anl.gmca.collect.ModelCollectRun.Point3D;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.DevXYZ;
import gov.anl.gmca.raster.ModelRasterCell;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ui.TabRaster;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

public class VCCellHover
{
	protected ModelRasterVideo model;
	public DrawCellHover draw;
	volatile ModelRasterCell hoverCell = null;
	
	public VCCellHover(final ModelRasterVideo model)
	{
		this.model=model;
		draw = new DrawCellHover(model);
	}
	
	void handleMouseMove(Point p)
	{
		ModelRasterRun run = ModelRasterState.get().getCurrentRunModel();
		ModelRasterCell newHoverCell=null;
		for (ModelRasterCell cell : run.grid().getGridCalc().getCellListCopy())
	    {
			Rectangle bounds = model.getCellScreenBounds(cell);
			if (null==bounds) continue;
			if (bounds.contains(p))
			{
				newHoverCell=cell;
			}
	    }
		boolean cellChanged = newHoverCell!=hoverCell;
		hoverCell = newHoverCell;
		if (cellChanged)
		{
			TabRaster.get().cellInfo.handleCellChange(hoverCell);
			model.cam.redraw();
		}
	}
	
	private void centerCell(Point3D pt)
	{
		ModelRasterRun run = ModelRasterState.get().getCurrentRunModel();
		
		if (null == run.grid().getCenter())
		{
			Log.error("Double-clicks are not allowed before rastering started.");
			return;
		}
		
		if (!Client.isMaster())
		{
			Log.error("Double-clicks are not allowed when slave");
			return;
		}
		
		if (!AppState.isOperationAllowed(Operation.MOVE_SAMPLE))
		{
			Log.error("Double-clicks are not allowed while other operations are in progress");
			return;
		}

		// start xyz move
		DevXYZ.syncMove(pt, false);
		
		// start omega move
		DevPmacMotor.get("gonio_omega").startMove(run.grid().getCenter().omega);
	}
}
