package gov.anl.gmca.raster.cam;

import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.cam.VCCreatePolygon.ContourType;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgCreatePolygon extends Composite
{
	protected final static Color buttonBgColor = GUI.COLOR_LIGHT_GRAY;
	protected final static Color bgColor = GUI.COLOR_DARK_GREEN;
	protected final static Color fgColor = GUI.COLOR_WHITE;
	
	protected final static String polygonText = "Click on the video to create a polygon";
	protected final static String rectangleText = "Rectangle: ";
	protected final static String polygonModeText = "Polygon";
	protected final static String polygonModeTooltip = "Enable clicking on the video to create a polygon";
	protected final static String rectangleModeText = "Grid";
	protected final static String rectangleModeTooltip = "Enable manual entry of a rectangular grid size";
	
	protected Button btnManual, btnRemoveLast, btnCancel, btnDone;
	protected Label label, widthLabel, heightLabel, offsetLabel;
	protected WdgControlEntry widthEntry, heightEntry, offsetEntry;
	protected Label fillLabel;
	private ModelRasterVideo model;
	
	public WdgCreatePolygon(Composite parent, int style, ModelRasterVideo model)
	{
		super(parent, SWT.BORDER);
		
		this.model=model;
		final ModelRasterVideo fModel = model;
		
		setBackground(bgColor);
		setForeground(fgColor);
		
		GridLayout gl = new GridLayout(10,false);
		gl.marginRight = 1;
		gl.marginHeight = 1;
		setLayout(gl);
		
		label = WidgetFactory.newLabel(this, SWT.NONE, polygonText);
		label.setBackground(bgColor);
		label.setForeground(fgColor);
		label.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, true));
		
		widthLabel = WidgetFactory.newLabel(this, SWT.NONE, " W:");
		widthLabel.setBackground(bgColor);
		widthLabel.setForeground(fgColor);
		widthLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, true));
		widthEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value){  int width_um = ((Double)value).intValue(); fModel.vcCreatePolygon.setRectangleWidth(width_um); }
			@Override public Object getModelValue()		{ return fModel.vcCreatePolygon.getRectangleWidth_um();	}
			@Override public Limits getLimits() 		{ return new Limits(1,1000,"cell width"); 	}
			@Override public int getDecimals() 			{ return 0; 								}
		});
		widthEntry.setEnabled(true);
		GridData gd = new GridData(SWT.LEFT, SWT.DEFAULT, false, false);
		gd.widthHint = 57;
		widthEntry.setLayoutData(gd);
		
		heightLabel = WidgetFactory.newLabel(this, SWT.NONE, " H:");
		heightLabel.setBackground(bgColor);
		heightLabel.setForeground(fgColor);
		heightLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, true));
		heightEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value){  int height_um = ((Double)value).intValue(); fModel.vcCreatePolygon.setRectangleHeight(height_um); }
			@Override public Object getModelValue()		{ return fModel.vcCreatePolygon.getRectangleHeight_um();	}
			@Override public Limits getLimits() 		{ return new Limits(1,1000,"cell height"); 	}
			@Override public int getDecimals() 			{ return 0; 								}
		});
		heightEntry.setEnabled(true);
		gd = new GridData(SWT.LEFT, SWT.DEFAULT, false, false);
		gd.widthHint = 57;
		heightEntry.setLayoutData(gd);
		
		offsetLabel = WidgetFactory.newLabel(this, SWT.NONE, " Offset:");
		offsetLabel.setBackground(bgColor);
		offsetLabel.setForeground(fgColor);
		offsetLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, true));
		offsetEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value){  int offset_um = ((Double)value).intValue(); fModel.vcCreatePolygon.setRectangleOffset(offset_um); }
			@Override public Object getModelValue()		{ return fModel.vcCreatePolygon.getRectangleOffset_um();	}
			@Override public Limits getLimits() 		{ return new Limits(-1000,1000,"grid offset"); 	}
			@Override public int getDecimals() 			{ return 0; 								}
		});
		offsetEntry.setEnabled(true);
		gd = new GridData(SWT.LEFT, SWT.DEFAULT, false, false);
		gd.widthHint = 57;
		offsetEntry.setLayoutData(gd);
		
		fillLabel = new Label(this, SWT.NONE);
		fillLabel.setBackground(bgColor);
		gd = new GridData();
		gd.grabExcessHorizontalSpace=true;
		gd.heightHint=1;
		gd.horizontalAlignment=SWT.FILL;
		fillLabel.setLayoutData(gd);
		
		btnManual = WidgetFactory.newButton(this, SWT.PUSH, rectangleModeText);
		btnManual.setToolTipText(rectangleModeTooltip);
		btnManual.setBackground(buttonBgColor);
		gd = new GridData(SWT.RIGHT, SWT.DEFAULT, false, false);
		gd.widthHint = 80;
		btnManual.setLayoutData(gd);
		btnManual.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				ContourType type = ContourType.values()[(fModel.vcCreatePolygon.getContourType().ordinal()==1?0:1)];//btnManual.getSelection()?ContourType.RECTANGLE:ContourType.POLYGON;
				fModel.vcCreatePolygon.setContourType(type);
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		btnRemoveLast = WidgetFactory.newButton(this, SWT.PUSH, "  -  ");
		btnRemoveLast.setToolTipText("Remove the last polygon point");
		btnRemoveLast.setBackground(buttonBgColor);
		btnRemoveLast.setLayoutData(new GridData(SWT.RIGHT, SWT.DEFAULT, false, false));
		btnRemoveLast.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				fModel.vcCreatePolygon.removeLastPoint();
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		btnCancel = WidgetFactory.newButton(this, SWT.PUSH, "Cancel");
		btnCancel.setBackground(buttonBgColor);
		btnCancel.setLayoutData(new GridData(SWT.RIGHT, SWT.DEFAULT, false, false));
		btnCancel.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				fModel.vcCreatePolygon.endPolygon();
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		btnDone = WidgetFactory.newButton(this, SWT.PUSH, "Done");
		btnDone.setBackground(buttonBgColor);
		btnDone.setLayoutData(new GridData(SWT.RIGHT, SWT.DEFAULT, false, false));
		btnDone.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				fModel.vcCreatePolygon.completeContour();
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		// this widget is created once and then shown/hidden,
		// so the listener doesn't need to be removed on the dispose event
		model.llStateChange.addListener(new ListenerList.Listener<ModelRasterVideo>()
		{
			@Override public void handleEvent(ModelRasterVideo object, Object data)
			{
				updateWidgets();
			}
		});
		updateWidgets();
	}
	
	protected void updateWidgets()
	{
		final ModelRasterVideo fModel = model;
		final ContourType type = fModel.vcCreatePolygon.getContourType();
		
		GUI.asyncExec(new Runnable(){public void run()
		{
			widthEntry.setText(fModel.vcCreatePolygon.getRectangleWidth_um());
			heightEntry.setText(fModel.vcCreatePolygon.getRectangleHeight_um());
			offsetEntry.setText(fModel.vcCreatePolygon.getRectangleOffset_um());
			
			int widgetCount = (type==ContourType.POLYGON)?6:11;
			((GridLayout)getLayout()).numColumns = widgetCount;
			
			label.setText(type==ContourType.POLYGON?polygonText:rectangleText);
			btnManual.setText(type==ContourType.POLYGON?rectangleModeText:polygonModeText);
			btnManual.setToolTipText(type==ContourType.POLYGON?rectangleModeTooltip:polygonModeTooltip);
			//btnRemoveLast.setEnabled(type==ContourType.POLYGON);
			
			boolean showEntries = (type==ContourType.RECTANGLE);
			
			WdgControl.setControlVisible(btnRemoveLast, !showEntries);
			WdgControl.setControlVisible(widthLabel, showEntries);
			WdgControl.setControlVisible(widthEntry, showEntries);
			WdgControl.setControlVisible(heightLabel, showEntries);
			WdgControl.setControlVisible(heightEntry, showEntries);
			WdgControl.setControlVisible(offsetLabel, showEntries);
			WdgControl.setControlVisible(offsetEntry, showEntries);
			
			layout(true);
		}});
	}
}
