package gov.anl.gmca.raster.cam;

import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.cam.ModelRasterVideo.VideoState;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import java.util.HashMap;

public class WdgRasterModeBar extends Composite
{
	private WdgPointAndClick pncBar;
	private WdgCreatePolygon createPolygonBar;
	private WdgCreateSite createSiteBar;
	private WdgViewImages viewImagesBar;
	private ModelRasterVideo camModel;
	private Composite currentBar = null;
	private Object[][] objMapping;
	private HashMap<VideoState, Composite> mapping = new HashMap<VideoState, Composite>();
	
	public WdgRasterModeBar(Composite parent, ModelRasterVideo model)
	{
		super(parent, SWT.NONE);
		this.camModel = model;
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth=gl.marginHeight=0;
		setLayout(gl);
		
		pncBar = new WdgPointAndClick(this, model);
		createPolygonBar = new WdgCreatePolygon(this, SWT.NONE, model);
		createSiteBar = new WdgCreateSite(this, model);
		viewImagesBar = new WdgViewImages(this, model);
		
		objMapping = new Object[][]
		{
			{ VideoState.POINT_AND_CLICK, pncBar },
			{ VideoState.CREATE_CONTOUR, createPolygonBar },
			{ VideoState.CREATE_SITE, createSiteBar },
			{ VideoState.VIEW_IMAGES, viewImagesBar }
		};
		for (Object[] mapRow : objMapping)
		{
			Composite bar = (Composite)mapRow[1];
			mapping.put((VideoState)mapRow[0], bar);
			bar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			WdgControl.setControlVisible((Composite)mapRow[1], false);
		}
		getParent().layout(true);
		
		connectListeners();
		showBar(mapping.get(model.getState()));
	}
	
	private void connectListeners()
	{
		camModel.llStateChange.addListener(new ListenerList.Listener<ModelRasterVideo>()
    	{
			@Override public void handleEvent(ModelRasterVideo model, Object data)
			{
				Composite newBar = mapping.get(model.getState());
				if (newBar != currentBar) showBar(newBar);
			}
    	});
	}
	
	private void showBar(Composite barToShow)
	{
		if (currentBar==barToShow) return;
		
		GUI.asyncExec(new Runnable(){public void run()
		{
			WdgControl.setControlVisible(currentBar,false);
			WdgControl.setControlVisible(barToShow,true);
			currentBar=barToShow;
			layout(true);
			getParent().layout(true);
		}});
	}
}
