package gov.anl.gmca.raster.ui;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.DevRasterRunList;
import gov.anl.gmca.raster.ModelRasterHits;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;

public class WdgAutoProgress extends Composite
{
	protected Table progressTable;
	protected ArrayList<TableColumn> columnList = new ArrayList<TableColumn>();
	protected ArrayList<TableItem> tableItems = new ArrayList<TableItem>();
	protected int displayedRuns = 0;
	
	public static enum AutoProgressHeader
	{
		ACTIVE		("A",		22,	"Which run is active"),
		RUN_NUMBER	("Run", 	45, "Run number"),
		STATUS		("State", 	60, "Run state"),
		TYPE		("Type",	70, "Run type"),
		REF			("Ref.",	70, "The run (R) and hit (H) numbers each run is refining"),
		HITS		("Hits",	42, "Hit count for each run");
		
		protected String title, tooltip;
		protected int width;
		
		AutoProgressHeader(String title, int width, String tooltip) { this.title=title; this.width=width; this.tooltip=tooltip; }
	};
	
	public WdgAutoProgress(Composite parent, int style)
	{
		super(parent,style);
		
		setLayout(new FillLayout());
		
		Group progressGroup = new Group(this, SWT.NONE);
		progressGroup.setText("Progress");
		progressGroup.setFont(GUI.FONT_DEFAULT);
		progressGroup.setLayout(new FillLayout());
		progressGroup.setBackground(GUI.BGCOLOR);
		
		progressTable = new Table(progressGroup,SWT.NONE);
		progressTable.setFont(GUI.FONT_DEFAULT);
		
		for (AutoProgressHeader header : AutoProgressHeader.values())
		{
			TableColumn tc = new TableColumn(progressTable, SWT.CENTER);
			tc.setText(header.title);
			tc.setWidth(header.width);
			tc.setToolTipText(header.tooltip);
			columnList.add(tc);
		}
		progressTable.setHeaderVisible(true);
		
		progressTable.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (!Client.isMaster())
				{
					Log.error("Changing runs is not allowed when slave");
					updateSelection();
					return;
				}
				
				if (!AppState.isOperationAllowed(Operation.START_RASTER))
				{
					Log.error("Changing runs is not allowed while other operations are in progress");
					updateSelection();
					return;
				}
				
				int runIndex = progressTable.getSelectionIndex()+1;
				
				final ModelRasterRun currentRun = (ModelRasterRun)ModelRasterRunList.get(RasterListType.AUTO).getRun(runIndex);
				currentRun.centerGrid(currentRun.grid().getCenter(), null);
				
				DevRasterRunList.get(RasterListType.AUTO).setCurrentRun(runIndex, true);
			}
		});
	}
	
	public void connectListeners()
	{
		ModelRasterRunList.get(RasterListType.AUTO).addListener(new ModelRasterRunList.Listener()
		{
			@Override public void runListChanged(ModelRunList runList)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateList();
				}});
			}
		});
		ModelRasterHits.get().ll.addListener(new ListenerList.Listener<ModelRasterHits>()
		{
			@Override public void handleEvent(ModelRasterHits object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateHits();
				}});
			}
		});
		
		/*ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(ModelRasterState state, Object data)
			{
				ModelRasterRun run = state.getCurrentRunModel();
				if (run.getRunList().getListType()!=RasterListType.AUTO) return;
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateList();
				}});
			}
		});*/
	}
	
	protected void updateSelection()
	{
		int currentRunIndex = ModelRasterRunList.get(RasterListType.AUTO).getCurrentRunIndex();
		
		if (0 == currentRunIndex)
		{
			progressTable.deselectAll();
		}
		else
		{
			progressTable.setSelection(currentRunIndex-1);
			progressTable.showSelection();
		}
	}
	
	// no concurrency issues, since this is only called from the GUI thread
	protected void updateList()
	{
		final ModelRasterRunList autoList = ModelRasterRunList.get(RasterListType.AUTO);
		
		int currentNumberOfRuns = autoList.getRunCount();
		
		while (currentNumberOfRuns > displayedRuns)
		{
			int tRunIndex = displayedRuns+1;
			
			final TableItem item = new TableItem(progressTable, SWT.NONE);
			item.setText(1,"Loading");
			
			// this listener is automatically disconnected if the run is deleted
			ListenerList.Listener<ModelRasterRun> runListener = new ListenerList.Listener<ModelRasterRun>()
			{
				@Override public void handleEvent(ModelRasterRun _run, Object data)
				{
					updateItem(_run, item);
				}
			};
			final ModelRasterRun run = (ModelRasterRun)autoList.getRun(tRunIndex);
			run.llRunChanged.addListener(runListener);
			updateItem(run,item);
			
			displayedRuns++;
		}
		
		while (currentNumberOfRuns < displayedRuns)
		{
			int indexToRemove = displayedRuns-1;
			progressTable.remove(indexToRemove);
			
			displayedRuns--;
		}
		
		for (int i=0;i<displayedRuns;i++)
		{
			boolean isActive = (i+1 == autoList.getCurrentRunIndex());
			progressTable.getItem(i).setText(AutoProgressHeader.ACTIVE.ordinal(),isActive?">":"");
		}

		updateSelection();
	}
	
	private void updateHits()
	{
		ArrayList<ModelRasterRun> runListCopy = ModelRasterRunList.get(RasterListType.AUTO).getAllRunsCopy();
		for (int i=0;i<progressTable.getItemCount()&&i<=runListCopy.size();i++)
		{
			TableItem item = progressTable.getItem(i);
			ModelRasterRun run = runListCopy.get(i+1);
			updateItem(run, item);
		}
	}
	
	protected void updateItem(final ModelRasterRun run, final TableItem item)
	{
		final ModelRasterRunList autoList = ModelRasterRunList.get(RasterListType.AUTO);
		
		GUI.asyncExec(new Runnable(){public void run()
		{
			boolean isActive = (run.getRunIndex() == autoList.getCurrentRunIndex());
			String statusString = run.getRunState().toShortString();
			String refString = (run.getReferenceRun()<=0)?"":("R"+run.getReferenceRun()+"H"+run.getReferenceHit());
			String runNumberString = Integer.toString(run.getRunLabel());
			int hitCount = ModelRasterHits.get().getCopyOfHits(run).size();
			Color hitCountColor = (0==hitCount)?GUI.COLOR_RED:GUI.COLOR_BLACK;
			
			item.setText(AutoProgressHeader.ACTIVE.ordinal(),		isActive?">":"");
			item.setText(AutoProgressHeader.HITS.ordinal(), 		Integer.toString(hitCount));
			item.setText(AutoProgressHeader.REF.ordinal(), 			refString);
			item.setText(AutoProgressHeader.RUN_NUMBER.ordinal(), 	runNumberString);
			item.setText(AutoProgressHeader.STATUS.ordinal(), 		statusString);
			item.setText(AutoProgressHeader.TYPE.ordinal(), 		run.getRefinementType().toString());
			
			item.setForeground(AutoProgressHeader.HITS.ordinal(), 	hitCountColor);
		}});
	}
}
