package gov.anl.gmca.raster.ui;

import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.collect.ETA;
import gov.anl.gmca.collect.ui.WdgShutterlessIndicator;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.cam.ModelRasterVideo;
import gov.anl.gmca.raster.cam.ModelRasterVideo.VideoState;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgRasterButtons extends WdgControl
{
	protected Button startButton, stopButton;
	private Composite compStart;
	private WdgShutterlessIndicator shutterlessIndicator;
	
	public WdgRasterButtons(Composite parent, int style)
	{
		super(parent, style, null);
		setLayout(new FillLayout());
		setBackground(GUI.BGCOLOR);
		setBackgroundMode(SWT.INHERIT_FORCE);
		
		Group buttonsGroup = new Group(this,SWT.SHADOW_IN);
		buttonsGroup.setFont(GUI.FONT_COLLECT_GROUP);
		buttonsGroup.setText("Control");
		
		buttonsGroup.setLayout(new GridLayout(1,false));
		buttonsGroup.setBackground(GUI.BGCOLOR);
		
		compStart = new Composite(buttonsGroup,SWT.NONE);
		compStart.setLayout(new FillLayout());
		compStart.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		startButton = new Button(compStart,SWT.NONE);
		startButton.setText("  Raster  ");
		startButton.setFont(GUI.FONT_DEFAULT);
		startButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		startButton.setEnabled(false);
		startButton.addMouseTrackListener(new MouseTrackAdapter()
		{
			@Override public void mouseEnter(MouseEvent e)
			{
				startButton.setToolTipText(ETA.getRasterTooltip());
			}
		});
		
		stopButton = new Button(buttonsGroup,SWT.NONE);
		stopButton.setToolTipText("Stop the raster operation after the current frame completes");
		stopButton.setText("  Pause  ");
		stopButton.setFont(GUI.FONT_DEFAULT);
		stopButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		stopButton.setEnabled(false);
		stopButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		shutterlessIndicator = new WdgShutterlessIndicator(buttonsGroup, false);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		shutterlessIndicator.setLayoutData(gd);
	}
	
	public void connectListeners()
	{
		startButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				// guarantee lose-focus event is processed for any entry widgets first
				startButton.setFocus();
				
				ModelRasterRunList.start();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		stopButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				ModelRasterState.get().pause();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		// update on auto run 0 change
		((ModelRasterRun)ModelRasterRunList.get(RasterListType.AUTO).getRun(0)).llRunChanged.addListener(new ListenerList.Listener<ModelRasterRun>()
		{
			@Override public void handleEvent(ModelRasterRun object, Object data)
			{
				updateButtonState();
			}
		});
		
		// update on current run change
		ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(ModelRasterState object, Object data)
			{
				updateButtonState();
			}
		});
		
		// update on start-raster-allowed change
		AppState.addListener(Operation.START_RASTER, new AppState.Listener()
    	{
			@Override public void stateChanged(Operation op, boolean allowed)
			{
				updateButtonState();
			}
    	});
		
		// update on camera mode change
		TabRaster.get().cam.model.llStateChange.addListener(new ListenerList.Listener<ModelRasterVideo>()
		{
			@Override public void handleEvent(ModelRasterVideo model, Object data)
			{
				updateButtonState();
			}
		});
		
		DevFPE.get().llFPEState.addListener(new ListenerList.Listener<DevFPE>()
		{
			@Override public void handleEvent(DevFPE object, Object data)
			{
				updateButtonState();
			}
		});

		updateButtonState();
		updateVisibility();
		shutterlessIndicator.connectListeners();
	}
	
	private void updateVisibility()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			System.out.println("shutterless indicator visibility = "+DetectorConfig.isDectris());
			WdgControl.setControlVisible(shutterlessIndicator, DetectorConfig.isDectris());
			layout(true);
		}});
	}
	
	protected void updateButtonState()
	{
		boolean enableRaster	= false;
		boolean enablePause		= false;
		String	reasonText		= "";
		String  startText		= "Start";
		String 	pauseText		= "Pause";

		ModelRasterRun currentRun = ModelRasterState.get().getCurrentRunModel();

		if (currentRun.getRunType() == ModelRasterRun.RunType.SONICC)
		{
			startText = "Start SONICC";
			if (!Client.isMaster())
			{
				reasonText = "Client is not master";
			} else {

				switch (ModelRasterState.get().getRasterState())
				{
					case INACTIVE:
						enableRaster = true;
						break;

					case RASTERING:
						enablePause=true;
						break;

					case STOPPING:
						pauseText="Stopping...";
						break;
				}
			}
			if (!AppState.isOperationAllowed(Operation.START_RASTER) && enableRaster)
			{
				enableRaster = false;
				reasonText = "Check application state";
			}
		} else {
			startText = "Raster";
			VideoState state = TabRaster.get().cam.model.getState();
			if (!Client.isMaster())
			{
				reasonText = "Client is not master";
			}
			else if (VideoState.POINT_AND_CLICK != state && VideoState.VIEW_IMAGES != state)
			{
				reasonText = "Raster is busy (mode="+state+")";
			}
			else
			{
				switch (ModelRasterState.get().getRasterState())
				{
				case INACTIVE:
					if (null != ModelRasterState.get().getCurrentRunModel())
					{
						enableRaster = (ModelRasterRun.RunState.COMPLETE != currentRun.getRunState());
						if (!enableRaster) reasonText = "Run "+currentRun.getRunLabel()+" is complete";
					}
					break;

				case RASTERING:
					enablePause=true;
					break;

				case STOPPING:
					pauseText="Stopping...";
					break;
				}
			}

			if (currentRun.isStarted()) startText = "Resume";

			if (!AppState.isOperationAllowed(Operation.START_RASTER) && enableRaster)
			{
				enableRaster = false;
				reasonText = "Check application state";
			}

			if (!currentRun.getAutoFind() && 0 == currentRun.grid().getContours().size() && enableRaster)
			{
				enableRaster = false;
				reasonText = "Click '+' to define a contour";
			}

			if (!DevFPE.get().isRunning() && enableRaster)
			{
				enableRaster = false;
				reasonText = "FPE is not running.  Click the detector icon to start.";
			}

			if (!DevFPE.get().isIdle() && enableRaster)
			{
				enableRaster = false;
				reasonText = "FPE is not idle.";
			}
		}

		final boolean f_enableRaster=enableRaster, f_enablePause=enablePause;
		final String f_pauseText=pauseText, f_startText=startText, f_reasonText=reasonText;
		GUI.asyncExec(new Runnable(){public void run()
		{
			startButton.setEnabled(f_enableRaster);
			startButton.setText(f_startText);
			stopButton.setEnabled(f_enablePause);
			stopButton.setText(f_pauseText);
			compStart.setToolTipText(f_reasonText);
		}});
	}

	@Override
	public boolean updateModel()
	{
		return true;
	}
}
