package gov.anl.gmca.raster.ui;

import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Resources;
import gov.anl.gmca.raster.DevRasterState;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.calc.RasterGeometry;
import gov.anl.gmca.raster.cam.ModelRasterVideo;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.WdgSpotFinderOptions;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgRasterOptions extends Composite
{
	private		Button					toggleShowOutline, toggleEnableBlending, toggleBeamsizeDisplay;
	protected 	Button 					btnSavePosition;
	private WdgSpotFinderOptions wdgSFOptions;
	protected 	static enum 			ButtonOp { CLEAR, SAVE };
	protected 	ButtonOp 				currentButtonOp;
	private		volatile static boolean bEnableBlending = true;
	public		static boolean			getEnableBlending() { return bEnableBlending; }
	
	private		Image 					imgShowNumbersOn 	= new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/showOutline.png"));
	private 	Image 					imgShowNumbersOff 	= new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/showOutlineBW.png"));
	
	private		Image 					imgShowSmoothingOn	= new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/showSmoothing.png"));
	private 	Image 					imgShowSmoothingOff	= new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/showSmoothingBW.png"));
	
	private		Image 					imgShowBeamsizeOn 	= new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/showBeamsize.png"));
	private 	Image 					imgShowBeamsizeOff 	= new Image(GUI.getDisplay(), Resources.get().getInputStream("/images/showBeamsizeBW.png"));

	
	///////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////// event handlers //////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////
	
	private void handleEnableNumbers(boolean enable)
	{
		ModelRasterVideo.setEnableNumbers(enable);
	}
	
	private void handleEnableBlending(boolean enable)
	{
		bEnableBlending = enable;
	}
	
	private void handleBeamsizeDisplay(boolean enable)
	{
		ModelRasterState.get().setShowBeamsize(enable);
	}
	
	private void handleBinMode(BinMode mode)
	{
		DevRasterState.get().setBinMode(mode);
	}
	
	private void handleSavePosition()
	{
		ModelRasterRun currentRun = ModelRasterRunList.get(RasterListType.INTER).getCurrentRunModel();
		
		switch (currentButtonOp)
		{
		case CLEAR: currentRun.dev().setUserPosition(null, true); break;
		case SAVE: 	currentRun.dev().setUserPosition(RasterGeometry.getCurrentMotorPositions(), true);
		}
	}

	protected void updateSaveButton()
	{
		ModelRasterRun currentRun = ModelRasterRunList.get(RasterListType.INTER).getCurrentRunModel();
		boolean inInterMode = 				(ModelRasterState.get().getAutoMode() 	== RasterListType.INTER);
		boolean offsetSet = 				(null != currentRun && null != currentRun.getUserPosition());
		boolean currentInterRunInactive = 	(RunState.INACTIVE == ModelRasterRunList.get(RasterListType.INTER).getCurrentRunModel().getRunState());
		
		final String buttonText = 		inInterMode ? (offsetSet ? "Clear" : "Save") : "---";
		currentButtonOp = 				inInterMode ? (offsetSet ? ButtonOp.CLEAR : ButtonOp.SAVE) : null;
		final boolean enableButton =	Client.isMaster() && inInterMode && currentInterRunInactive;
		
		GUI.asyncExec(new Runnable(){public void run()
		{
			btnSavePosition.setEnabled(enableButton);
			btnSavePosition.setText(buttonText);
		}});
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////// ctor and widget setup ///////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////
	
	public WdgRasterOptions(Composite parent)
	{
		super(parent, SWT.NONE);
		setLayout(new FillLayout());
		
		createWidgets();
	}

	private void createWidgets()
	{
		Group displayGroup = new Group(this, SWT.NONE);
		displayGroup.setBackground(GUI.BGCOLOR);
		displayGroup.setText("Options");
		displayGroup.setFont(GUI.FONT_COLLECT_GROUP);
		GridLayout gl = new GridLayout(1,false);
		gl.verticalSpacing = 0;
		displayGroup.setLayout(gl);
		
		Composite toggleButtons = new Composite(displayGroup, SWT.NONE);
		toggleButtons.setBackground(GUI.BGCOLOR);
		toggleButtons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		gl = new GridLayout(4,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		gl.horizontalSpacing=0;
		toggleButtons.setLayout(gl);
		
		toggleShowOutline = new Button(toggleButtons, SWT.TOGGLE);
		toggleShowOutline.setToolTipText("Show the polygon outline");
		toggleShowOutline.setImage(imgShowNumbersOn);
		toggleShowOutline.setSelection(true);
		
		toggleEnableBlending = new Button(toggleButtons, SWT.TOGGLE);
		toggleEnableBlending.setToolTipText("Blend colors in raster cells with neighbors");
		toggleEnableBlending.setImage(imgShowSmoothingOn);
		toggleEnableBlending.setSelection(true);
		
		toggleBeamsizeDisplay = new Button(toggleButtons, SWT.TOGGLE);
		toggleBeamsizeDisplay.setToolTipText("Show the beam size associated with the current raster grid");
		toggleBeamsizeDisplay.setImage(imgShowBeamsizeOn);
		toggleBeamsizeDisplay.setSelection(true);
		
		btnSavePosition = WidgetFactory.newButton(toggleButtons, SWT.PUSH, "Save");
		btnSavePosition.setFont(GUI.FONT_RASTER_OPTIONS);
		btnSavePosition.setBackground(GUI.COLOR_SAMPLE_BG);
		btnSavePosition.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		btnSavePosition.setToolTipText("Once saved, the current position will be restored when this run is returned to");
		btnSavePosition.setEnabled(false);

		Composite spacer = new Composite(displayGroup, SWT.NONE);
		spacer.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint=5;
		spacer.setLayoutData(gd);
		
		// TODO: make this listen to DetectorConfig changes
		WdgControl.setControlVisible(spacer, DetectorConfig.isDectris());

		wdgSFOptions = new WdgSpotFinderOptions(displayGroup, SFRunType.RASTER);
		wdgSFOptions.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		/////////////////////////////////// end display params ////////////////////////////////////
		
		/*Group binModeGroup = new Group(displayGroup, SWT.NONE);
		binModeGroup.setFont(GUI.FONT_COLLECT_GROUP);
		binModeGroup.setText("Raster CCD bin mode");
		binModeGroup.setBackground(GUI.BGCOLOR);
		binModeGroup.setLayout(new FillLayout());
		binModeGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		comboBinMode = new Combo(binModeGroup, SWT.READ_ONLY);
		comboBinMode.setFont(GUI.FONT_RASTER_OPTIONS);
		comboBinMode.add("73\u00B5m pixel");
		comboBinMode.add("146\u00B5m pixel");
		
		// TODO: make this listen to DetectorConfig changes
		WdgControl.setControlVisible(binModeGroup, !DetectorConfig.isDectris());*/
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////// listeners /////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////
	
	public void connectListeners()
	{
		wdgSFOptions.connectListeners();
		
		toggleShowOutline.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				boolean selected = toggleShowOutline.getSelection();
				toggleShowOutline.setImage(selected?imgShowNumbersOn:imgShowNumbersOff);
				handleEnableNumbers(selected);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		toggleEnableBlending.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				boolean selected = toggleEnableBlending.getSelection();
				toggleEnableBlending.setImage(selected?imgShowSmoothingOn:imgShowSmoothingOff);
				handleEnableBlending(selected);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		toggleBeamsizeDisplay.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				boolean selected = toggleBeamsizeDisplay.getSelection();
				toggleBeamsizeDisplay.setImage(selected?imgShowBeamsizeOn:imgShowBeamsizeOff);
				handleBeamsizeDisplay(selected);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		btnSavePosition.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				Log.note("handle save position clicked");
				handleSavePosition();
			}
		});
		ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(ModelRasterState state, Object data)
			{
				updateSaveButton();
			}
		});
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateSaveButton();
			}
		});
	}
}
