package gov.anl.gmca.raster.ui;

import gov.anl.gmca.common.Redis;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterCell;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.cam.VCViewImages;
import gov.anl.gmca.spotfinder.DBImageStats;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.spotfinder.SFData;
import gov.anl.gmca.spotfinder.SFData.SFImageData;
import gov.anl.gmca.spotfinder.SFData.State;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.SWTUtil;
import gov.anl.gmca.util.Util;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
//import org.eclipse.ui.internal.quickaccess.CamelUtil;

public class WdgRasterResultsTable extends Composite
{
	public static class SFDataProvider implements ILazyContentProvider
	{
		private TableViewer viewer;
		private ArrayList<SFData> frames;
		
		public SFDataProvider(TableViewer viewer) { this.viewer=viewer; }
		@Override public void dispose() {}
		@SuppressWarnings("unchecked")
		@Override public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
			frames = (ArrayList<SFData>)newInput;
		}

		@Override public void updateElement(int index)
		{
			viewer.replace(frames.get(index), index);
		}
	}
	
	private static final int HEADERS_MAX = DevSFType.SPOTFINDER_OUTPUT.values().length+2;
	
	private boolean[] visibleColumns = new boolean[HEADERS_MAX];
	protected Menu tableMenu;
	protected MenuItem itemCenterCell, itemCreateSite;
	protected Label titleLabel;
	
	private TableViewer tableViewer;
	private Table table;
	private ArrayList<SFImageData> cellList = new ArrayList<>();
	private HashMap<Integer,Integer> origIndexToSortedIndex = new HashMap<>();
	private HashMap<Integer,Integer> sortedIndexToOrigIndex = new HashMap<>();
	
	public WdgRasterResultsTable(Composite parent)
	{
		super(parent, SWT.NONE);
		createWidgets();
	}
	
	private int lastRunIndex=-1;
	private long lastListVersion=-1;
	private int sortColumnIndex=0;
	
	private String cellNumberString = "";
	private void createWidgets()
	{
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.horizontalSpacing=0;
		gl.verticalSpacing=0;
		setLayout(gl);
		
		tableViewer = new TableViewer(this, SWT.VIRTUAL|SWT.BORDER|SWT.MULTI);
		table = tableViewer.getTable();
		table.setLinesVisible(true);
		table.setFont(GUI.FONT_DEFAULT);
		table.setLayoutData(new GridData(GridData.FILL_BOTH));
		for (int i=0;i<HEADERS_MAX;i++)
		{
			visibleColumns[i]=true;
			
		    TableViewerColumn viewerColumn = new TableViewerColumn(tableViewer,SWT.CENTER);
			TableColumn column = viewerColumn.getColumn();
			if (i==0)
				column.setText("#");
			else if (i==1)
				column.setText("SONICC");
			else
				column.setText(DevSFType.SPOTFINDER_OUTPUT.values()[i-2].name);
			column.setWidth(i==0?50:125);
			column.setResizable(true);
			column.setMoveable(true);
			final int columnIndex=i;
			viewerColumn.setLabelProvider(new CellLabelProvider()
			{
		        @Override public void update(ViewerCell cell)
		        {
		        	SFImageData imgData = (SFImageData)cell.getElement();
		        	String cellText = "";
		        	ModelRasterRun run = ModelRasterState.get().getCurrentRunModel();
		        	if (columnIndex==0) cellText = ""+(imgData.frameIndex+1);
		        	else if (imgData.state!=State.CLEAR && columnIndex==1)
		        		cellText = Util.printDouble(imgData.soniccValue, 0);
		        	else if (imgData.state!=State.CLEAR)
		        	{
		        		SPOTFINDER_OUTPUT output = SPOTFINDER_OUTPUT.values()[columnIndex-2];
		        		Double val = imgData.values.get(output);
		        		if (null != val)
		        			cellText = Util.printDouble(val, output.decimals);
		        	}
		        	cell.setText(cellText);
			    }
			});
			viewerColumn.getColumn().addSelectionListener(new SelectionAdapter()
			{
				@Override public void widgetSelected(SelectionEvent e)
				{
					int dir = SWT.UP;
					if (table.getSortColumn()==viewerColumn.getColumn())
					{
						dir = table.getSortDirection()==SWT.DOWN ? SWT.UP : SWT.DOWN;
					}
					table.setSortDirection(dir);
					table.setSortColumn(viewerColumn.getColumn());
					sortColumnIndex=columnIndex;
					updateResults(true);
				}
			});
		}
		table.setHeaderVisible(true);
		tableViewer.setContentProvider(new SFDataProvider(tableViewer));
		createMenu();
	}

	private DevSFType.SPOTFINDER_TYPE displayedType=null;
	private ModelRasterRun.RunType displayedRunType=null;
	public void updateVisibleColumns()
	{
		ModelRasterRun run = ModelRasterState.get().getCurrentRunModel();
		boolean change = false;
		if (displayedRunType != run.getRunType()) change=true;
		displayedRunType = run.getRunType();
		if (run.getRunType()== ModelRasterRun.RunType.DIFFRACTION) {
			DevSFType.SPOTFINDER_TYPE type = DevSFType.getConfig(SFRunType.RASTER).id;
			if (type!=displayedType) change=true;
			displayedType=type;
			for (SPOTFINDER_OUTPUT output : SPOTFINDER_OUTPUT.values()) {
				// if a given output belongs to the current type, show it
				boolean show = (output.sf_type == type);
				visibleColumns[output.ordinal() + 2] = show;
			}
			visibleColumns[1] = false;
		} else {
			visibleColumns[1] = true;
			for (int i=2;i<HEADERS_MAX;i++) visibleColumns[i] = false;
		}
		if (change) GUI.asyncExec(()->redistributeColumns());
	}
	
	private boolean firstResize=true;
	public void connectListeners()
	{
		TabRaster.get().cam.model.vcViewImages.llSelectedCell.addListener(new ListenerList.Listener<VCViewImages>()
		{
			@Override public void handleEvent(VCViewImages object, Object data)
			{
				int selectedIndex = null==data?-1:((ModelRasterCell)data).getIndex();
				GUI.asyncExec(new Runnable(){public void run()
				{
					if (selectedIndex==-1)
					{
						table.deselectAll();
					}
					else
					{
						int mappedIndex = origIndexToSortedIndex.get(selectedIndex);
						table.setSelection(mappedIndex);
						table.showItem(table.getItem(mappedIndex));
					}
				}});
			}
		});
		
		addControlListener(new ControlAdapter()
		{
			@Override public void controlResized(ControlEvent ce)
			{
				redistributeColumns();
				if (firstResize && table.getClientArea().height>0)
				{
					showLastProcessedResult();
					firstResize=false;
				}
			}
		});
		ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(final ModelRasterState state, final Object data)
			{
				// if the table didn't change, return
				ModelRasterRun currentRun = ModelRasterState.get().getCurrentRunModel();
				int currentRunIndex = currentRun.getRunIndex();
				long currentListVersion = currentRun.grid().getGridCalc().getListVersion();
				boolean tableChanged = (currentRunIndex==lastRunIndex && currentListVersion==lastListVersion);
				lastRunIndex=currentRunIndex;
				lastListVersion=currentListVersion;
				
				if (tableChanged) updateResults(false);

				updateVisibleColumns();
			}
		});

		Executors.newSingleThreadScheduledExecutor().scheduleWithFixedDelay(()->
		{
			GUI.asyncExec(()->
			{
				updateResults(false);
			});
		}, 0, 1000, TimeUnit.MILLISECONDS);
		
		table.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (!Client.isMaster())
				{
					Log.note("Table selection not allowed unless client is master");
					table.deselectAll();
					return;
				}
				
				TableItem selectedItem = (TableItem)se.item;
				try
				{
					int selectedIndex = Integer.parseInt(selectedItem.getText(0))-1;
					TabRaster.get().cam.model.vcViewImages.setSelectedCell(selectedIndex);
				}
				catch (NumberFormatException nfe) {}
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		table.addKeyListener(new KeyListener()
		{
			@Override public void keyReleased(KeyEvent arg0) {}
			@Override public void keyPressed(KeyEvent ke)
			{
				if (ke.character>='0' && ke.character<='9')
				{
					cellNumberString += ke.character;
					try
					{
						int gotoCell = Integer.parseInt(cellNumberString);
						int mappedIndex = origIndexToSortedIndex.get(gotoCell-1);
						if (gotoCell>=1 && gotoCell<=table.getItemCount())
						{
							table.setSelection(mappedIndex);
							table.showItem(table.getItem(mappedIndex));
						}
					}
					catch (Exception e) { e.printStackTrace(); }
					Log.note("Cell#: "+cellNumberString);
				}
				if (ke.keyCode==SWT.ESC)
				{
					cellNumberString = "";
					TabRaster.get().cam.model.vcViewImages.setSelectedCell(null);
				}
			}
		});

		table.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent te)
			{
				if (te.detail==SWT.TRAVERSE_RETURN) cellNumberString = "";
			}
		});
		
		table.addMouseListener(new MouseListener()
		{
			@Override public void mouseUp(MouseEvent e) {}
			@Override public void mouseDown(MouseEvent e) {}
			@Override public void mouseDoubleClick(MouseEvent e)
			{
				ModelRasterCell selectedCell = getSelectedCell();
				if (null!=selectedCell) selectedCell.center();
			}
		});
		DevSFType.get(SFRunType.RASTER).llChange.addListener(new ListenerList.Listener<DevSFType>()
		{
			@Override public void handleEvent(DevSFType config, Object data)
			{
				updateVisibleColumns();
			}
		});
		updateVisibleColumns();
	}
	
	private ModelRasterCell getSelectedCell()
	{
		int selectedIndex = table.getSelectionIndex();
		if (selectedIndex<0) return null;
		int origIndex = sortedIndexToOrigIndex.get(selectedIndex);
		ModelRasterRun currentRun = ModelRasterState.get().getCurrentRunModel();
		ModelRasterCell cell = currentRun.grid().getGridCalc().getCellCopy(origIndex);
		return cell;
	}
	
	/*private void setColumnVisible(int columnIndex, boolean visible)
	{
		TableColumn column = table.getColumn(columnIndex);
		column.setWidth(visible ? 125 : 0);
		visibleColumns[columnIndex]=visible;
		redistributeColumns();
	}*/
	
	private void createMenu()
	{
		tableMenu = new Menu(GUI.getShell(), SWT.POP_UP);
		
		table.addListener(SWT.MenuDetect, new Listener()
		{
			public void handleEvent(Event event)
			{
				Point pt = GUI.getDisplay().map(null, table, new Point(event.x, event.y));
				Rectangle clientArea = table.getClientArea();
				boolean header = clientArea.y <= pt.y && pt.y < (clientArea.y + table.getHeaderHeight());
				if (!header) table.setMenu(tableMenu);
			}
		});
		
		tableMenu.addMenuListener(new MenuAdapter()
		{
			@Override public void menuShown(MenuEvent e)
			{
				updateMenu();
			}
		});
	}

	private void updateMenu()
	{
		SWTUtil.safeDispose(itemCenterCell);
		SWTUtil.safeDispose(itemCreateSite);
		
		ModelRasterCell cell = getSelectedCell();
		if (cell==null) return;
		
		itemCenterCell = new MenuItem(tableMenu, SWT.PUSH);
		itemCenterCell.setText("Center cell "+(cell.index+1));
		itemCenterCell.addSelectionListener(new SelectionAdapter()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				cell.center();
			}
		});
		itemCreateSite = new MenuItem(tableMenu, SWT.PUSH);
		itemCreateSite.setText("Create site from cell "+(cell.index+1));
		itemCreateSite.addSelectionListener(new SelectionAdapter()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
                cell.addSiteFromCell();
			}
		});
	}
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////// end setup //////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	private void redistributeColumns()
	{
		int totalWidth = getSize().x-20;
		
		// update column widths
		GC gc = new GC(this);
		int[] contentWidths = new int[HEADERS_MAX];
		
		// for cell numbers, use the highest number
		contentWidths[0] = gc.textExtent(""+cellList.size()).x;
		
		// use four digits for the spot total numbers
		for (int i=1;i<HEADERS_MAX;i++)
		{
			contentWidths[i]=gc.textExtent("9999").x;
		}
		
		gc.dispose();
		
		// set widths to zero for hidden columns
		for (int i=0;i<HEADERS_MAX;i++) if (!visibleColumns[i]) contentWidths[i]=0;
		
		// calculate multiplier
		int totalRequestedWidth = 0;
		for (int w : contentWidths) { totalRequestedWidth += w; }
		double multiplier = (double)totalWidth / (double)totalRequestedWidth;

		// set individual widths
		for (int colIndex=0; colIndex<table.getColumnCount(); colIndex++)
		{
			int columnWidth = (int)((double)contentWidths[colIndex] * multiplier);
			table.getColumn(colIndex).setWidth(columnWidth);
		}
	}
	
	private void createHitFromCells()
	{
		if (table.getSelectionCount()==0) return;
		
		/*ArrayList<Site> sites = new ArrayList<Site>();
		for (TableItem selectedItem : table.getSelection())
		{
			Site site = (Site)selectedItem.getData("site");
			sites.add(site);
		}
		
		ModelCollectRunList.get().createRuns(sites);*/
	}

	private void handleNextFrame(int nextFrame)
	{
		SWTUtil.centerTableIndex(table, nextFrame);
	}
	
	private class ImgDataComparator implements Comparator<SFImageData>
	{
		@Override public int compare(SFImageData id1, SFImageData id2)
		{
			int ret = 0;
			switch (sortColumnIndex)
			{
			case 0:
				ret = id1.frameIndex-id2.frameIndex; break;
			case 1:
				ret = (int)(id1.soniccValue*1000-id2.soniccValue*1000); break;
			default:
				SPOTFINDER_OUTPUT output = SPOTFINDER_OUTPUT.values()[sortColumnIndex-2];
				if (null==id1.values.get(output)||null==id2.values.get(output)) return 0;
				ret = (int)( (id1.values.get(output)-id2.values.get(output))*1000 );
			}
			if (table.getSortDirection()==SWT.DOWN) ret=-ret;
			return ret;
		}
	}
	private ImgDataComparator idComp = new ImgDataComparator();
	
	private void showLastProcessedResult()
	{
		int lastProcessedCell = 0;
		for (SFImageData id : cellList)
		{
			if (id.state == State.DONE) lastProcessedCell=Math.max(lastProcessedCell,id.frameIndex);
		}
		handleNextFrame(lastProcessedCell);
	}

	private HashMap<String,FloatBuffer> lastData = new HashMap<>();
	public void updateResults(boolean alwaysUpdate)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			try
			{
				if (null == table) return;
				
				// get the current list of cell data
				cellList = new ArrayList<>();
				ModelRasterRun currentRun = ModelRasterState.get().getCurrentRunModel();

				boolean soniccNotBinned = currentRun.getRunType()==ModelRasterRun.RunType.SONICC && !currentRun.getSONICCBin();

				if (!soniccNotBinned) {
					// get new data
					String dir = currentRun.getDirectorySuffix();
					String prefix = currentRun.getFileRoot();
					int runLabel = currentRun.getRunLabel();
					String[] outputs = new String[]{"dials:spots_no_ice", "dials:spots", "dials:total_int", "dozor:score", "dozor:res", "sonicc"};
					HashMap<String, FloatBuffer> data = new HashMap<>();
					boolean change = false;
					for (String output : outputs) {
						data.put(output, DBImageStats.load(Redis.image_stats(dir, prefix, runLabel, output)));
						if (!lastData.containsKey(output) || !DBImageStats.isSame(data.get(output), lastData.get(output))) {
							change = true;
						}
						lastData.put(output, data.get(output));
					}
					if (!change && !alwaysUpdate) return;

					// create SFImageData
					for (int cellIndex = 0; cellIndex < currentRun.grid().getSelectedCellCount(); cellIndex++) {
						SFImageData sfid = new SFImageData();
						boolean processed = false;
						if (currentRun.isStarted()) {
							try {
								double val = (double) data.get("dials:spots_no_ice").get(cellIndex);
								if (val >= 0) {
									processed = true;
									sfid.values.put(SPOTFINDER_OUTPUT.DIALS_SPOTS_NO_ICE, val);
								}
							} catch (IndexOutOfBoundsException e) {
							}
							try {
								double val = (double) data.get("dials:spots").get(cellIndex);
								if (val >= 0) {
									processed = true;
									sfid.values.put(SPOTFINDER_OUTPUT.DIALS_SPOT_TOTAL, val);
								}
							} catch (IndexOutOfBoundsException e) {
							}
							try {
								double val = (double) data.get("dials:total_int").get(cellIndex);
								if (val >= 0) {
									processed = true;
									sfid.values.put(SPOTFINDER_OUTPUT.DIALS_TOTAL_INT, val);
								}
							} catch (IndexOutOfBoundsException e) {
							}
							try {
								double val = (double) data.get("dozor:score").get(cellIndex);
								if (val >= 0) {
									processed = true;
									sfid.values.put(SPOTFINDER_OUTPUT.DOZOR_MAIN_SCORE, val);
								}
							} catch (IndexOutOfBoundsException e) {
							}
							try {
								double val = (double) data.get("dozor:res").get(cellIndex);
								if (val >= 0) {
									processed = true;
									sfid.values.put(SPOTFINDER_OUTPUT.DOZOR_RES, val);
								}
							} catch (IndexOutOfBoundsException e) {
							}
						}
						if (currentRun.getSONICCBin()) {
							try {
								double val = (double) data.get("sonicc").get(cellIndex);
								if (val >= 0) {
									processed = true;
									sfid.soniccValue = val;
								}
							} catch (IndexOutOfBoundsException e) {
							}
						}

						sfid.frameIndex = cellIndex;
						sfid.state = processed ? State.DONE : State.CLEAR;
						cellList.add(sfid);
					}
				}
			} catch (Exception e) { e.printStackTrace(); }
			
			// sort
			Collections.sort(cellList, idComp);
			origIndexToSortedIndex = new HashMap<>();
			sortedIndexToOrigIndex = new HashMap<>();
			for (int i=0;i<cellList.size();i++)
			{
				origIndexToSortedIndex.put(cellList.get(i).frameIndex,i);
				sortedIndexToOrigIndex.put(i,cellList.get(i).frameIndex);
			}
			
			// send the list to the tableviewer
			tableViewer.setItemCount(cellList.size());
			tableViewer.setInput(cellList);
			tableViewer.refresh();
			
			showLastProcessedResult();
		}});
	}
}
