package gov.anl.gmca.raster.ui;

import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.DevRasterRun;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

public class WdgRasterRunButtons extends Composite implements ClientListener
{
	protected Button reprocessButton, updateButton, deleteButton, resetButton;
	protected WdgRunTabs runTabs;
	protected WdgRasterRunTab runTab;
	protected DevRasterRun runDev;
	protected ModelRasterRun run;

	protected WdgRunTabs.Listener runTabsListener = null;
	protected ListenerList.Listener<ModelRasterRun> runListener = null;

	public WdgRasterRunButtons(Composite parent, int style, WdgRunTabs _runTabs, WdgRasterRunTab _runTab, ModelRasterRun _run, DevRasterRun _runDev)
	{
		super(parent, style);
		runTabs=_runTabs;
		runTab=_runTab;
		runDev=_runDev;
		run=_run;
		
		setLayout(new FillLayout());
		setBackground(parent.getBackground());
		
							reprocessButton = WidgetFactory.newButton(this,	SWT.PUSH,	"Reproc",	GUI.FONT_COLLECTRUN_BUTTON,	GUI.COLOR_BLUE_BUTTON);
							updateButton = 	WidgetFactory.newButton(this,	SWT.PUSH, 	"Update",	GUI.FONT_COLLECTRUN_BUTTON,	GUI.COLOR_BLUE_BUTTON);
		if (null!=runTabs) 	deleteButton = 	WidgetFactory.newButton(this,	SWT.PUSH, 	"Delete",	GUI.FONT_COLLECTRUN_BUTTON,	GUI.COLOR_BLUE_BUTTON);
							resetButton = 	WidgetFactory.newButton(this,	SWT.PUSH, 	"Reset",	GUI.FONT_COLLECTRUN_BUTTON,	GUI.COLOR_BLUE_BUTTON);

							reprocessButton.setToolTipText("Reprocess frames in this run with the current SpotFinder settings");
							updateButton.setToolTipText("Update hardware-based fields in this run to match hardware positions");
		if (null!=runTabs)	deleteButton.setToolTipText("Delete this run tab");
							resetButton.setToolTipText("Clear results for this run");
							
							reprocessButton.setEnabled(false);
							updateButton.setEnabled(false);
		if (null!=runTabs) 	deleteButton.setEnabled(false);
							resetButton.setEnabled(false);
		
		reprocessButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				run.handleReprocessButton();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		updateButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				run.handleUpdateButton();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		if (null!=deleteButton) deleteButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				run.handleDeleteButton();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		resetButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				run.handleResetButton();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		Client.addListener(this);
		
		runListener = new ListenerList.Listener<ModelRasterRun>()
		{
			@Override public void handleEvent(ModelRasterRun run, Object data)
			{
				updateEnableState();
			}
		};
		_run.llRunChanged.addListener(runListener);
		
		if (null != runTabs)
		{
    		runTabsListener = new WdgRunTabs.Listener()
    		{
    			@Override public void update()
    			{
    				updateEnableState();
    			}
    		};
    		runTabs.addListener(runTabsListener);
		}
	}
	
	public void removeListeners()
	{
		Client.removeListener(this);
		run.llRunChanged.removeListener(runListener);
		runTabs.removeListener(runTabsListener);
	}
	
	public void updateEnableState()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			// TODO: this shouldn't be necessary - are listeners not being disconnected?
			if (isDisposed()) return;

			boolean isSONICC = run.getRunType() == ModelRasterRun.RunType.SONICC;
			boolean remoteOpInProgress = false;
			if (null != TabRaster.get() && null != TabRaster.get().runTabs)
			{
				remoteOpInProgress = TabRaster.get().runTabs.getRemoteOpInProgress();
			}
			
    		if (Client.isMaster())
    		{
    			switch(run.getRunState())
    			{
    			case INACTIVE:
        									reprocessButton.setEnabled(false);
        									updateButton.setEnabled(!isSONICC);
        			if (null!=deleteButton) deleteButton.setEnabled(!remoteOpInProgress && 0 != run.getRunIndex());
        									resetButton.setEnabled(!isSONICC && 0 != run.getNextFrame());
        			break;
    			case RASTERING:
        									reprocessButton.setEnabled(false);
        									updateButton.setEnabled(false);
        			if (null!=deleteButton) deleteButton.setEnabled(false);
        									resetButton.setEnabled(false);
    				break;
    			case PAUSED:
        									reprocessButton.setEnabled(!isSONICC);
        									updateButton.setEnabled(false);
        			if (null!=deleteButton) deleteButton.setEnabled(false);
        									resetButton.setEnabled(!isSONICC && 0 != run.getNextFrame());
    			case COMPLETE:
        									reprocessButton.setEnabled(!isSONICC);
        									updateButton.setEnabled(false);
        			if (null!=deleteButton) deleteButton.setEnabled(!remoteOpInProgress && 0 != run.getRunIndex());
        									resetButton.setEnabled(true);
    			}
    		}
    		else
    		{
    									reprocessButton.setEnabled(false);
    									updateButton.setEnabled(false);
    			if (null!=deleteButton) deleteButton.setEnabled(false);
    									resetButton.setEnabled(false);
    		}
		}});
	}
	
	@Override
	public void masterChanged(boolean isMaster)
	{
		updateEnableState();
	}
}
