package gov.anl.gmca.raster.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgRotatedLabel;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.WdgMain;
import gov.anl.gmca.gui.WdgMain.TAB;
import gov.anl.gmca.raster.*;
import gov.anl.gmca.raster.ModelRasterRun.RefinementType;
import gov.anl.gmca.raster.ModelRasterSiteList.Site;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.ui.WdgRasterSiteList.ListType;
import gov.anl.gmca.raster.util.ClearSampleState;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList.Listener;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

import java.util.ArrayList;

public class WdgRasterSiteContainer extends Composite
{
	protected static Object[][] headers =
	{
		{"#",30},
		{"Source",70},
		{"Size (\u03BCm)",130}
	};
	
	protected Menu menu;
	protected MenuItem itemDescription, itemCenterInter, itemCenterCoarse, itemCenterFine, itemCenterOrtho, itemCreateRun, itemDeleteSite;
	protected Label titleLabel;
	private WdgRotatedLabel rotatedLabel;
	private Composite separator;
	private WdgRasterSiteList wdgSiteList;
	private SashForm form;
	
	public WdgRasterSiteContainer(Composite parent, int style)
	{
		super(parent, style);
		form = TabRaster.get().lowerForm;
		createWidgets();
		connectListeners();
	}
	
	private void connectListeners()
	{
		wdgSiteList.getTable().addMouseListener(new MouseListener()
		{
			@Override public void mouseUp(MouseEvent e) {}
			@Override public void mouseDown(MouseEvent e) {}
			@Override public void mouseDoubleClick(MouseEvent e)
			{
				int selectedIndex = wdgSiteList.getTable().getSelectionIndex();
				if (selectedIndex<0) return;
				
				if (!Client.isMaster()) Log.note("Client is not master, double-click ignored.");
				
				Site site = wdgSiteList.getSiteList().get(selectedIndex);
				if (site.orthoRunIndex>=0) centerSiteAndShowGrid(RefinementType.ORTHO, site);
				if (site.interRunIndex>=0) centerSiteAndShowGrid(null, site);
			}
		});
		
		ModelRasterSiteList.get().ll.addListener(new Listener<ModelRasterSiteList>()
		{
			@Override public void handleEvent(ModelRasterSiteList model, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateRotatedText();
					handleSiteCountChange(model.getSiteListCopy().size());
				}});
			}
		});
		handleSiteCountChange(ModelRasterSiteList.get().getSiteListCopy().size());
		
		addControlListener(new ControlAdapter()
		{
			@Override public void controlResized(ControlEvent e)
			{
				if (isMinimized) setMinimizedWeights();
			}
		});
	}

	private int lastSiteCount = -1;
	private void handleSiteCountChange(int newCount)
	{
		//Log.note("handleSiteCountChange: "+newCount);
		if (newCount>0 && (lastSiteCount==0 || lastSiteCount==-1))
		{
			setMinimized(false);
		}
		else if (newCount==0 && (lastSiteCount>0 || lastSiteCount==-1))
		{
			setMinimized(true);
		}
		lastSiteCount=newCount;
	}
	
	private void createWidgets()
	{
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.horizontalSpacing=0;
		gl.verticalSpacing=0;
		setLayout(gl);
		
		rotatedLabel = new WdgRotatedLabel(this, SWT.NONE);
		rotatedLabel.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		WdgControl.addListenerToControlRecursively(rotatedLabel, new MouseAdapter()
		{
			@Override public void mouseDown(MouseEvent e)
			{
				setMinimized(false);
			}
		});
		updateRotatedText();
		
		titleLabel = new Label(this, SWT.CENTER);
		titleLabel.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		titleLabel.setFont(GUI.FONT_DEFAULT);
		titleLabel.setText("Site List");
		titleLabel.addMouseTrackListener(new MouseTrackListener()
		{
			@Override public void mouseEnter(MouseEvent me)
			{
				titleLabel.setText(">> Site List >>");
			}
			@Override public void mouseExit(MouseEvent me)
			{
				titleLabel.setText("Site List");
			}
			@Override public void mouseHover(MouseEvent me) {}
		});
		titleLabel.addMouseListener(new MouseAdapter()
		{
			@Override public void mouseDown(MouseEvent e)
			{
				setMinimized(true);
			}
		});
		
		separator = new Composite(this, SWT.NONE);
		separator.setBackground(GUI.COLOR_DARK_GRAY5);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint=1;
		separator.setLayoutData(gd);
		
		wdgSiteList = new WdgRasterSiteList(this, SWT.NONE, headers, ListType.RASTER_LIST, null);
		wdgSiteList.setLayoutData(new GridData(GridData.FILL_BOTH));
		createMenu();
		
		setMinimized(true);
	}

	private int[] nonMinimizedWeights = new int[]{67,33};
	private Boolean isMinimized = null;
	public void setMinimized(Boolean minimize)
	{
		if (minimize==isMinimized) return;
		
		WdgControl.setControlVisible(rotatedLabel, 	minimize);
		WdgControl.setControlVisible(titleLabel, 	!minimize);
		WdgControl.setControlVisible(separator, 	!minimize);
		WdgControl.setControlVisible(wdgSiteList,	!minimize);
		layout(true);
		
		if (minimize)
		{
			if (null != isMinimized)
			{
				//Log.note("Setting nonMinimizedWeights to "+nonMinimizedWeights);
				nonMinimizedWeights = form.getWeights();
			}
			//Log.note("Setting weights to minimized");
			setMinimizedWeights();
		}
		else
		{
			//Log.note("setting weights to "+nonMinimizedWeights[0]+" "+nonMinimizedWeights[1]);
			GUI.asyncExec(new Runnable(){public void run()
			{
				form.setWeights(nonMinimizedWeights);
			}});
		}
		
		isMinimized=minimize;
	}
	
	private void setMinimizedWeights()
	{
		int minimizedWidth = 20;
		int[] newWeights = new int[]{getParent().getSize().x-minimizedWidth,minimizedWidth};
		if (newWeights[0]<=0) newWeights[0]=1;
		form.setWeights(newWeights);
	}
	
	private void createMenu()
	{
		menu = new Menu(GUI.getShell(), SWT.POP_UP);
		wdgSiteList.setMenu(menu);
		MenuListener menuListener = new MenuListener()
		{
			@Override public void menuShown(MenuEvent e)
			{
				setUpMenu();
			}
			@Override public void menuHidden(MenuEvent e) {}
		};
		menu.addMenuListener(menuListener);
	}
	
	private void setUpMenu()
	{
		for (MenuItem mi : menu.getItems()) mi.dispose();
		
		int selectedIndex = wdgSiteList.getTable().getSelectionIndex();
		if (selectedIndex<0) return;
		Site site = wdgSiteList.getSiteList().get(selectedIndex);
		
		itemDescription = new MenuItem(menu, SWT.PUSH);
		itemDescription.setEnabled(false);
		itemDescription.setText("Source: "+site.getSource().desc);
		
		new MenuItem(menu, SWT.SEPARATOR);
		
		if (site.interRunIndex>=0)
		{
			itemCenterInter = new MenuItem(menu, SWT.PUSH);
			if(site.interCell>=0)
			{
				itemCenterInter.setText("Center cell "+(site.interCell+1));
			}
			else
			{
				itemCenterInter.setText("Center hit "+(site.hitNumber));
			}
			itemCenterInter.addSelectionListener(new SelectionListener()
		    {
				@Override public void widgetSelected(SelectionEvent e)
				{
					centerSiteAndShowGrid(null, site);
				}
				@Override public void widgetDefaultSelected(SelectionEvent e) {}
			});
			itemCenterInter.setEnabled(Client.isMaster());
		}		
		if (site.interRunIndex<0)
		{
			itemCenterCoarse = new MenuItem(menu, SWT.PUSH);
			itemCenterCoarse.setText("Center coarse run");
			itemCenterCoarse.addSelectionListener(new SelectionListener()
		    {
				@Override public void widgetSelected(SelectionEvent e)
				{
					centerSiteAndShowGrid(RefinementType.COARSE, site);
				}
				@Override public void widgetDefaultSelected(SelectionEvent e) {}
			});
			itemCenterCoarse.setEnabled(Client.isMaster());
		}
		if (site.fineRunIndex>=0)
		{
			itemCenterFine = new MenuItem(menu, SWT.PUSH);
			itemCenterFine.setText("Center fine run");
			itemCenterFine.addSelectionListener(new SelectionListener()
		    {
				@Override public void widgetSelected(SelectionEvent e)
				{
					centerSiteAndShowGrid(RefinementType.FINE, site);
				}
				@Override public void widgetDefaultSelected(SelectionEvent e) {}
			});
			itemCenterFine.setEnabled(Client.isMaster());
		}
		if (site.orthoRunIndex>=0)
		{
			itemCenterOrtho = new MenuItem(menu, SWT.PUSH);
			itemCenterOrtho.setText("Center ortho run");
			itemCenterOrtho.addSelectionListener(new SelectionListener()
		    {
				@Override public void widgetSelected(SelectionEvent e)
				{
					centerSiteAndShowGrid(RefinementType.ORTHO, site);
				}
				@Override public void widgetDefaultSelected(SelectionEvent e) {}
			});
			itemCenterOrtho.setEnabled(Client.isMaster());
		}
		itemCreateRun = new MenuItem(menu, SWT.PUSH);
		itemCreateRun.setText("Create collect run from site "+site.index);
		itemCreateRun.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				createRunFromSites();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		itemCreateRun.setEnabled(Client.isMaster());
		
		itemDeleteSite = new MenuItem(menu, SWT.PUSH);
		itemDeleteSite.setText("Delete site "+site.getSource().desc);
		itemDeleteSite.addSelectionListener(new SelectionAdapter()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				ArrayList<Integer> runLabelsUsingSite = new ArrayList<>(), runIndicesUsingSite = new ArrayList<>();
				for (ModelCollectRun run : ModelCollectRunList.get().getAllRunsCopy())
				{
					if (run.getScanMode()==ScanMode.RASTER && run.isSiteSelected(site.index))
					{
						runLabelsUsingSite.add(run.getRunLabel());
						runIndicesUsingSite.add(run.getRunIndex());
					}
				}
				
				if (runLabelsUsingSite.size()>0)
				{
					// ask to delete the runs using the site
					String msg = "Site "+site.index+" is in use by ";
					msg += Util.commaSeparatedList("run", runLabelsUsingSite, true);
					
					msg += "\nProceed to reset these runs?";
					boolean okToDelete = Util.popupOkCancel("Site in use", msg);
					if (!okToDelete) { Log.note("Run reset declined, returning."); return; }

					// print a message with the deleted run labels and indices
					String deleteMsg = "Resetting runs (index|label): ";
					for (int i=0; i<runLabelsUsingSite.size(); i++)
					{
						deleteMsg += "("+runIndicesUsingSite.get(i)+"|"+runLabelsUsingSite.get(i)+") ";
					}
					Log.note(deleteMsg);

					// reset related runs
					ClearSampleState.resetCollectRunIndices(runIndicesUsingSite);
				}
				
				// delete the site and related hits
				DBRasterSiteList.get().deleteSite(site, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		itemDeleteSite.setEnabled(Client.isMaster());
	}
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////// end setup //////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////

	private void centerSiteAndShowGrid(RefinementType rType, Site site)
	{
		if (!AppState.isOperationAllowed(Operation.START_RASTER))
		{
			Log.note("Centering sites is not allowed at this time");
			return;
		}
		
		// show the correct run
		if (null == rType)
		{
			int refIndex = site.interRunIndex;
			if (-1 != refIndex)
			{
				DevRasterState.get().setAutoMode(RasterListType.INTER);
				DevRasterRunList.get(RasterListType.INTER).setCurrentRun(refIndex, true);
			}
		}
		else
		{
			int refIndex = -1;
			switch (rType)
			{
			case COARSE: 	refIndex = 1; break;
			case FINE:		refIndex = site.fineRunIndex; break;
			case ORTHO:		refIndex = site.orthoRunIndex; break;
			}
			
			if (-1 != refIndex)
			{
				DevRasterState.get().setAutoMode(RasterListType.AUTO);
				DevRasterRunList.get(RasterListType.AUTO).setCurrentRun(refIndex, true);
			}
		}

		// center
		ModelRasterRun.centerSite(site, rType);
	}

	private void createRunFromSites()
	{
		WdgMain.setVisibleTab(TAB.Collect);
		
		if (wdgSiteList.getTable().getSelectionCount()==0) return;
		
		ArrayList<Site> sites = new ArrayList<Site>();
		sites.add(wdgSiteList.getSiteList().get(wdgSiteList.getTable().getSelectionIndex()));
		
		ModelCollectRunList.get().createRuns(sites);
	}

	private void updateRotatedText()
	{
		ArrayList<Site> siteList = ModelRasterSiteList.get().getSiteListCopy();
		if (null==siteList) rotatedLabel.setText("Site List");
		else
		{
			int siteCount = siteList.size();
			rotatedLabel.setText("Site List ("+siteCount+")");
		}
	}
}
