package gov.anl.gmca.raster.ui;

import gov.anl.gmca.common.DevRunList.DevRun;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.common.ui.GTabItem;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.DevRasterRun;
import gov.anl.gmca.raster.ModelRasterGrid;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRun.RunType;
import gov.anl.gmca.raster.cam.ModelRasterVideo;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.ListenerList.Listener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgRunTabInter extends WdgRasterRunTab
{
	public int getTabIndex() { return tabIndex; }
	protected WdgRunTabs runTabs;
	protected GTabItem tabItem;
	
	protected Label runStateLabel;
	protected WdgRasterRunButtons runButtons;
	protected WdgControlMode controlMode;
	protected ListenerList.Listener<ModelRasterGrid> gridListener;
	
	public static class Factory implements WdgRunTabs.WdgRunTabFactory
	{
		@Override
		public Composite createRunTab(Composite parent, int style, int runIndex, ModelRun modelRun, DevRun devRun, WdgRunTabs runTabs, GTabItem tabItem)
		{
			return new WdgRunTabInter(parent,style,runIndex,(ModelRasterRun)modelRun,(DevRasterRun)devRun,runTabs,tabItem);
		}
	}
	
	public WdgRunTabInter(Composite parent, int style, int tabIndex, final ModelRasterRun run, DevRasterRun controller, WdgRunTabs runTabs, GTabItem tabItem)
	{
		super(parent, style, run, controller);
		
		this.tabIndex = tabIndex;
		this.runTabs=runTabs;
		this.tabItem=tabItem;
		
		setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(1,false);
		gl.marginBottom = 0;
		setLayout(gl);
		
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		setLayoutData(gd);

		setupRunStateLabel();
		setupRunButtons();
		setupModeSelector();
		setupParameterWidgets
		(
			this,
			PW.DirPrefix,
			PW.Contours,
			PW.CellAndBeamsize,
			PW.Time,
			PW.Delta,
			PW.Distance,
			PW.Attenuation,
			PW.Energy,
			PW.ROI,
			PW.ResLimits,
			PW.SONICC_Controls,
			PW.Macros
		);

		reconfigureWidgets(RunType.DIFFRACTION);
		
		final ListenerList.Listener<ModelRasterRun> runListener = new ListenerList.Listener<ModelRasterRun>()
		{
			@Override public void handleEvent(ModelRasterRun run, Object data)
			{
				handleRunChange(run);
			}
		};
		run.llRunChanged.addListener(runListener);
		
		gridListener = new Listener<ModelRasterGrid>()
		{
			@Override public void handleEvent(ModelRasterGrid grid, Object data)
			{
				gridChanged(grid);
			}
		};
		run.grid().llGridChanged.addListener(gridListener);
		
		
		final Listener<ModelRasterVideo> videoListener = new Listener<ModelRasterVideo>()
		{
			@Override public void handleEvent(ModelRasterVideo tab, Object data)
			{
				handleVideoEvent();
			}
		};
		TabRaster.get().cam.model.llStateChange.addListener(videoListener);
		
		final WdgRunTabInter runTab = this;
		
		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent e)
			{
				Client.removeListener(runTab);
				runButtons.removeListeners();
				run.llRunChanged.removeListener(runListener);
				run.getRunList().removeListener(runTab);
				run.grid().llGridChanged.removeListener(gridListener);
				TabRaster.get().cam.model.llStateChange.removeListener(videoListener);
			}
		});
	}
	
	public void connectListeners()
	{
		runTabs.connectListeners();
	}
	
	protected void setupRunButtons()
	{
		runButtons = new WdgRasterRunButtons(this,SWT.NONE, runTabs, this, run, devRun);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		runButtons.setLayoutData(gd);
	}
	
	protected void setupRunStateLabel()
	{
		runStateLabel = new Label(this, SWT.NONE);
		runStateLabel.setText("--loading--");
		runStateLabel.setBackground(GUI.BGCOLOR);
		runStateLabel.setAlignment(SWT.CENTER);
		runStateLabel.setFont(GUI.FONT_RASTERRUN_HEADER);
		runStateLabel.setForeground(GUI.COLOR_RASTER_STATE_FG);
		runStateLabel.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
	}

	protected void setupModeSelector()
	{
		controlMode = new WdgControlMode(this, SWT.NONE, run, devRun);
		controlMode.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		nonGridControls.add(controlMode);
	}
	
	@Override
	public void masterChanged(boolean isMaster)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			updateEnableState();
		}});
	}

	@Override
	public void runListChanged(ModelRunList runList)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			updateEnableState();
		}});
	}
	
	protected void handleVideoEvent()
	{
		updateEnableState();
	}

	protected RunType currentType = null;
	
	public void reconfigureWidgets(RunType newType)
	{
		if (newType == currentType) return;
		currentType = newType;

		switch (newType)
		{
		case DIFFRACTION:
			//controlMode.setSelection(ModelRasterRun.RunType.DIFFRACTION);
			WdgControl.setControlVisible(energyLabel, false);
			WdgControl.setControlVisible(energyEntry, false);
			WdgControl.setControlVisible(energyUnits, false);
			WdgControl.setControlVisible(deltaLabel, true);
			WdgControl.setControlVisible(deltaEntry, true);
			WdgControl.setControlVisible(deltaUnitLabel, true);
			WdgControl.setControlVisible(roi,false);
			WdgControl.setControlVisible(lblResLimit, true);
			WdgControl.setControlVisible(resLimitsComp, true);
			WdgControl.setControlVisible(resLimitsButtonComp, true);
			WdgControl.setControlVisible(lblPolygons,			true);
			WdgControl.setControlVisible(lblContourCount,		true);
			WdgControl.setControlVisible(polygonControlComp, 	true);
			WdgControl.setControlVisible(lblLoopZ, 				true);
			WdgControl.setControlVisible(loopMeasurementEntry,	true);
			WdgControl.setControlVisible(lblLoopZUnits,			true);
			WdgControl.setControlVisible(lblLoopCentered, 		false);
			WdgControl.setControlVisible(loopCenteredCheckbox,	false);
			WdgControl.setControlVisible(lblLoopCenteredUnits,	false);
			WdgControl.setControlVisible(attenLabel, true);
			WdgControl.setControlVisible(attenuationEntry, true);
			WdgControl.setControlVisible(attenUnitLabel, true);
			WdgControl.setControlVisible(timeLabel, true);
			WdgControl.setControlVisible(timeEntry, true);
			WdgControl.setControlVisible(timeUnitLabel, true);
			WdgControl.setControlVisible(distLabel, true);
			WdgControl.setControlVisible(distanceEntry, true);
			WdgControl.setControlVisible(distUnitLabel, true);
			WdgControl.setControlVisible(controlDir, true);
			WdgControl.setControlVisible(spacerComp, true);
			if (null!=beamsizeCombo) WdgControl.setControlVisible(beamsizeCombo, true);
			if (null!=beamSizeComp) WdgControl.setControlVisible(beamSizeComp, true);
			if (null!=cellSizeComp) WdgControl.setControlVisible(cellSizeComp, true);
			WdgControl.setControlVisible(beamSizeLabel, true);
			WdgControl.setControlVisible(beamSizeUnits, true);
			WdgControl.setControlVisible(cellSizeLabel, true);
			WdgControl.setControlVisible(cellSizeUnits, true);
			WdgControl.setControlVisible(wdgMacro, true);
			WdgControl.setControlVisible(binButton, false);
			WdgControl.setControlVisible(unbinButton, false);
			WdgControl.setControlVisible(soniccDisplayCombo, false);
			WdgControl.setControlVisible(channelCombo, false);
			break;
			
		case FLUO:
			//controlMode.setSelection(ModelRasterRun.RunType.SONICC);
			WdgControl.setControlVisible(energyLabel, true);
			WdgControl.setControlVisible(energyEntry, true);
			WdgControl.setControlVisible(energyUnits, true);
			WdgControl.setControlVisible(deltaLabel, false);
			WdgControl.setControlVisible(deltaEntry, false);
			WdgControl.setControlVisible(deltaUnitLabel, false);
			WdgControl.setControlVisible(roi,true);
			WdgControl.setControlVisible(lblResLimit, false);
			WdgControl.setControlVisible(resLimitsComp, false);
			WdgControl.setControlVisible(resLimitsButtonComp, false);
			WdgControl.setControlVisible(lblPolygons,			true);
			WdgControl.setControlVisible(lblContourCount,		true);
			WdgControl.setControlVisible(polygonControlComp, 	true);
			WdgControl.setControlVisible(lblLoopZ, 				false);
			WdgControl.setControlVisible(loopMeasurementEntry,	false);
			WdgControl.setControlVisible(lblLoopZUnits,			false);
			WdgControl.setControlVisible(lblLoopCentered, 		false);
			WdgControl.setControlVisible(loopCenteredCheckbox,	false);
			WdgControl.setControlVisible(lblLoopCenteredUnits,	false);
			WdgControl.setControlVisible(attenLabel, true);
			WdgControl.setControlVisible(attenuationEntry, true);
			WdgControl.setControlVisible(attenUnitLabel, true);
			WdgControl.setControlVisible(timeLabel, true);
			WdgControl.setControlVisible(timeEntry, true);
			WdgControl.setControlVisible(timeUnitLabel, true);
			WdgControl.setControlVisible(distLabel, true);
			WdgControl.setControlVisible(distanceEntry, true);
			WdgControl.setControlVisible(distUnitLabel, true);
			WdgControl.setControlVisible(controlDir, true);
			WdgControl.setControlVisible(spacerComp, true);
			if (null!=beamsizeCombo) WdgControl.setControlVisible(beamsizeCombo, true);
			if (null!=beamSizeComp) WdgControl.setControlVisible(beamSizeComp, true);
			if (null!=cellSizeComp) WdgControl.setControlVisible(cellSizeComp, true);
			WdgControl.setControlVisible(beamSizeLabel, true);
			WdgControl.setControlVisible(beamSizeUnits, true);
			WdgControl.setControlVisible(cellSizeLabel, true);
			WdgControl.setControlVisible(cellSizeUnits, true);
			WdgControl.setControlVisible(wdgMacro, true);
			WdgControl.setControlVisible(binButton, false);
			WdgControl.setControlVisible(unbinButton, false);
			WdgControl.setControlVisible(soniccDisplayCombo, false);
			WdgControl.setControlVisible(channelCombo, false);
			break;

		case SONICC:
			//controlMode.setSelection(ModelRasterRun.RunType.SONICC);
			WdgControl.setControlVisible(energyLabel, false);
			WdgControl.setControlVisible(energyEntry, false);
			WdgControl.setControlVisible(energyUnits, false);
			WdgControl.setControlVisible(deltaLabel, false);
			WdgControl.setControlVisible(deltaEntry, false);
			WdgControl.setControlVisible(deltaUnitLabel, false);
			WdgControl.setControlVisible(roi,false);
			WdgControl.setControlVisible(lblResLimit, false);
			WdgControl.setControlVisible(resLimitsComp, false);
			WdgControl.setControlVisible(resLimitsButtonComp, false);
			WdgControl.setControlVisible(lblPolygons,			false);
			WdgControl.setControlVisible(lblContourCount,		false);
			WdgControl.setControlVisible(polygonControlComp, 	false);
			WdgControl.setControlVisible(lblLoopZ, 				false);
			WdgControl.setControlVisible(loopMeasurementEntry,	false);
			WdgControl.setControlVisible(lblLoopZUnits,			false);
			WdgControl.setControlVisible(lblLoopCentered, 		false);
			WdgControl.setControlVisible(loopCenteredCheckbox,	false);
			WdgControl.setControlVisible(lblLoopCenteredUnits,	false);
			WdgControl.setControlVisible(attenLabel, false);
			WdgControl.setControlVisible(attenuationEntry, false);
			WdgControl.setControlVisible(attenUnitLabel, false);
			WdgControl.setControlVisible(timeLabel, false);
			WdgControl.setControlVisible(timeEntry, false);
			WdgControl.setControlVisible(timeUnitLabel, false);
			WdgControl.setControlVisible(distLabel, false);
			WdgControl.setControlVisible(distanceEntry, false);
			WdgControl.setControlVisible(distUnitLabel, false);
			WdgControl.setControlVisible(controlDir, false);
			WdgControl.setControlVisible(spacerComp, false);
			if (null!=beamsizeCombo) WdgControl.setControlVisible(beamsizeCombo, false);
			if (null!=beamSizeComp) WdgControl.setControlVisible(beamSizeComp, false);
			if (null!=cellSizeComp) WdgControl.setControlVisible(cellSizeComp, false);
			WdgControl.setControlVisible(beamSizeLabel, false);
			WdgControl.setControlVisible(beamSizeUnits, false);
			WdgControl.setControlVisible(cellSizeLabel, false);
			WdgControl.setControlVisible(cellSizeUnits, false);
			WdgControl.setControlVisible(wdgMacro, false);
			WdgControl.setControlVisible(binButton, true);
			WdgControl.setControlVisible(unbinButton, true);
			WdgControl.setControlVisible(soniccDisplayCombo, true);
			WdgControl.setControlVisible(channelCombo, true);

			break;
		}
		
		// recalculate widget layout
		paramComp.layout(true);
	}
	
	@Override protected void handleRunChange(final ModelRasterRun run)
	{
    	super.handleRunChange(run);
    	
    	GUI.asyncExec(new Runnable(){public void run()
    	{
    		runTabs.setTabText(tabItem, " "+run.getRunLabel()+" ");
    		String runState = run.getRunState().toString().toLowerCase();
    		if (run.getRunState()== ModelRasterRun.RunState.RASTERING && run.getRunType()==RunType.SONICC) runState = "acquiring";
    		runStateLabel.setText("Run "+run.getRunLabel()+" ("+runState+")");
   			reconfigureWidgets(run.getRunType());
    	}});
	}
}
