package gov.anl.gmca.raster.util;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.*;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.raster.ModelRasterSiteList.Site;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;

import java.util.ArrayList;

public class ClearSampleState
{
	public static ListenerList<ClearSampleState> llBeforeEachClear = new ListenerList<ClearSampleState>(null);
	
	public static boolean deleteInterSites(ModelRasterRun run, String opDescription, boolean promptForSites, boolean promptForRuns, boolean sendUpdate)
	{
		// find sites linked to this run
		ArrayList<Site> relatedSites = getLinkedSites(run);
		if (relatedSites.size()==0) return true;
		
		// prompt the user to delete
		if (promptForSites)
		{
			String msg = "All raster sites and setup in the collect tab related to this raster run will be deleted.  OK to delete?";
			boolean okToDelete = Util.popupOkCancel("Delete sites?", msg);
			if (!okToDelete)
			{
				Log.note("User aborted run deletion."); return false;
			}
		}

		ArrayList<ModelCollectRun> relatedRuns = getRelatedRuns(relatedSites);
		if (relatedRuns.size()>0 && !promptForSites && promptForRuns)
		{
			String msg = "All raster sites and setup in the collect tab related to this raster run will be deleted.  OK to delete?";
			boolean okToDelete = Util.popupOkCancel("Delete sites?", msg);
			if (!okToDelete)
			{
				Log.note("User aborted run deletion."); return false;
			}
		}

		// clear
		resetCollectRuns(relatedRuns);
		deleteSites(relatedSites, true, false, sendUpdate);
		
		return true;
	}
	
	public static boolean deleteAutoSitesAndHits(boolean prompt)
	{
		// find sites linked to this run
		ArrayList<Site> relatedSites = getLinkedSites(null);
		if (relatedSites.size()==0)
		{
			DBRasterHits.get().deleteAutoHits(true);
			return true;
		}
		
		// prompt to delete data and reset runs
		if (prompt)
		{
			String msg = "All raster sites and setup in the collect tab related to auto raster will be deleted.  OK to delete?";
			boolean okToDelete = Util.popupYesNo("Reset auto-raster", msg);
			if (!okToDelete) return false;
		}

		// clear
		ArrayList<ModelCollectRun> relatedRuns = getRelatedRuns(relatedSites);
		resetCollectRuns(relatedRuns);
		deleteSites(relatedSites, true, false, true);
		DBRasterHits.get().deleteAutoHits(true);
		
		return true;
	}
	
	public static ArrayList<Site> getLinkedSites(ModelRasterRun run)
	{
		ArrayList<Site> siteList = ModelRasterSiteList.get().getSiteListCopy();
		ArrayList<Site> relatedSites = new ArrayList<Site>();
		for (Site site : siteList)
		{
			if (null != run) // inter raster
			{
				if (site.interRunIndex==run.getRunIndex()) relatedSites.add(site);
			}
			else // auto raster
			{
				if (site.orthoRunIndex>=0) relatedSites.add(site);
			}
		}
		return relatedSites;
	}
	
	private static ArrayList<ModelCollectRun> getRelatedRuns(ArrayList<Site> sites)
	{
		ArrayList<ModelCollectRun> relatedRuns = new ArrayList<ModelCollectRun>();
		for (ModelCollectRun run : ModelCollectRunList.get().getAllRunsCopy())
		{
			for (Site site : sites)
			{
				if (run.getScanMode() == ScanMode.RASTER && run.isSiteSelected(site.index))
				{
					relatedRuns.add(run);
				}
			}
		}
		return relatedRuns;
	}
	
	public static void resetCollectRuns(ArrayList<ModelCollectRun> relatedRuns)
	{
		for (ModelCollectRun run : relatedRuns)
		{
			Log.note("Resetting collect run "+run.getRunLabel()+" (index "+run.getRunIndex()+")");
			run.fii.reset(true, true);
		}
	}
	
	public static void resetCollectRunIndices(ArrayList<Integer> runIndices)
	{
		ArrayList<ModelCollectRun> runs = new ArrayList<>();
		for (Integer index : runIndices) runs.add((ModelCollectRun)ModelCollectRunList.get().getRun(index));
		resetCollectRuns(runs);
	}
	
	// delete sites and any hits that are part of those sites
	public static void deleteSites(ArrayList<Site> sites, boolean logOutput, boolean promptUser, boolean update)
	{
		for (Site site : sites)
		{
			if (logOutput) Log.note("Deleting raster site "+site.index);
			DBRasterSiteList.get().deleteSite(site, false);
		}
		DBRasterSiteList.get().sendUpdate();
	}
	
	public static void clearAllRelatedState(String msg, boolean popupIfNoAction, final boolean promptToDelete)
	{
		// raster sites
		int rasterSiteCount = DBRasterSiteList.get().getSiteCount();
		final boolean rasterSitesNeedClearing = (rasterSiteCount>0);
		
		// raster hits
		int rasterHitCount = DBRasterHits.get().getHitCount();
		final boolean rasterHitsNeedClearing = (rasterHitCount>0);
		
		// auto raster
		final ModelRasterRun autoRun0 = (ModelRasterRun)ModelRasterRunList.get(RasterListType.AUTO).getRun(0);
		final boolean autoRasterNeedsClearing = autoRun0.getRunState() != RunState.INACTIVE || autoRun0.grid().getContours().size()>0;
		
		// interactive raster
		final ModelRasterRun interRun0 = (ModelRasterRun)ModelRasterRunList.get(RasterListType.INTER).getRun(0);
		final boolean interRasterRun0NeedsClearing = interRun0.getRunState()!=RunState.INACTIVE || interRun0.grid().getContours().size()>0;
		final int interRasterRunsAboveZero = ModelRasterRunList.get(RasterListType.INTER).getRunCount();
		
		// collect runs with vectors or sites
		final ArrayList<Integer> collectRunsWithVector = new ArrayList<Integer>();
		final ArrayList<Integer> collectRunsWithSites = new ArrayList<Integer>();
		for (ModelCollectRun run : new ArrayList<ModelCollectRun>(ModelCollectRunList.get().getAllRunsCopy()))
		{
			int siteCount = run.getSelectedRasterSiteObjects().size();
			if (siteCount>0) collectRunsWithSites.add(run.getRunIndex());
			
			boolean hasVector = run.getPt1()!=null || run.getPt2()!=null;
			if (hasVector) collectRunsWithVector.add(run.getRunIndex());
		}
		final boolean collectVectorRunsNeedClearing = (collectRunsWithVector.size()>0);
		final boolean collectSiteRunsNeedClearing = (collectRunsWithSites.size()>0);

		msg += "All raster sites and setup in the collect tab related to this sample will be deleted.  OK to delete?";
		
		final NBMessageBox msgBox = new NBMessageBox(GUI.getShell(), SWT.OK|SWT.CANCEL);
		msgBox.setText("Clearing sample-related state");
		msgBox.setMessage(msg);

		// deletion runnable
		Runnable deleteRunnable = new Runnable(){public void run()
		{
			boolean okToClear = (!promptToDelete || msgBox.getReturnValue()==SWT.OK);
			if (okToClear)
			{
				try
				{
					llBeforeEachClear.notifyListeners();
				}
				catch (Exception e) { e.printStackTrace(); }
				
				if (autoRasterNeedsClearing)
				{
					// reset auto-raster
					autoRun0.resetAutoRaster();
					autoRun0.grid().clearAllContours(false, false);
				}
				if (interRasterRun0NeedsClearing)
				{
					// reset run 0
					interRun0.resetInterRun(true);
					interRun0.grid().clearAllContours(false, false);
				}
				if (interRasterRunsAboveZero!=0)
				{
					// delete runs above run 0
					DevRasterRunList.get(RasterListType.INTER).setCurrentRun(0, false);
					DevRasterRunList.get(RasterListType.INTER).setRunCount(0, true);
				}
				for (int runIndex : collectRunsWithVector)
				{
					// reset collect runs with vectors
					((ModelCollectRun) ModelCollectRunList.get().getRun(runIndex)).fii.reset(true, true);
				}
				for (int runIndex : collectRunsWithSites)
				{
					// reset collect runs with sites
					((ModelCollectRun) ModelCollectRunList.get().getRun(runIndex)).fii.reset(true, true);
				}

				DBRasterHits.get().clearHits(true);
				DBRasterSiteList.get().clearSites(true);
				
				DevAutoFind.get().setComplete(false);
			}
		}};

		if (promptToDelete)
		{
			msgBox.setRunnableOnSubmit(deleteRunnable);
			new Thread(new Runnable(){public void run(){msgBox.open();}}).start();
		}
		else
		{
			deleteRunnable.run();
		}
	}
}
