package gov.anl.gmca.sample;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.syslog.Log;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.widgets.Label;

import java.util.HashMap;

public class DevInOut
{
	public enum InOut { IN, OUT };
	
	protected PVName inPV;
	protected double inVal = 0.0;
	String title;
	
	protected PVName outPV;
	protected double outVal = 0.0;
	
	protected PVName rqsPV;
	protected PVName actPV;
	
	protected int rqsVal;
	protected int actVal;
	
	protected PVName pvName;
	protected PVName motionPV;
	
	static HashMap<String,DevInOut> devMap = new HashMap<String,DevInOut>();
	
	protected Label wText;
	protected Label wINOUT;
	protected String status;
	static HashMap<String,String> deviceStatus = new HashMap<String,String>();
	protected Epics.EpicsValueMonitor monitor = null;
	public static String getStatus (String _title) { return (String) (deviceStatus.get(_title));}
	
	public DevInOut(String _title, PVName pv, PVName _inPV, PVName _outPV, PVName _motionPV) {
		
		pvName = pv;
		title = _title;
		inPV = _inPV;
		outPV = _outPV;
		motionPV = _motionPV;
		devMap.put(_title.toLowerCase(), this);
		
		monitor = new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String _pv, DBR value)
			{
				
				final String _pvName = _pv;
				final DBR _value = value;
				
				if (_pvName.equals(pvName.toString()) || _pvName.equals(motionPV.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						handleUpdateInOut(Epics.dbrToDouble(_value));	
					}
					});
				}
			}
			
		};
		Epics.addMonitor(pvName, monitor);
		Epics.addMonitor(motionPV, monitor);
		
	}
	
	public static DevInOut get(String devName)
	{
		if (!devMap.containsKey(devName.toLowerCase())) return null;
		return devMap.get(devName.toLowerCase());
	}
	
	public String getDeviceName()
	{
		return title;
	}
	
	public DevInOut(String _title, PVName _rqsPV, PVName _actPV) {
		title = _title;
		rqsPV = _rqsPV;
		actPV = _actPV;
		
		devMap.put(_title.toLowerCase(), this);
		
		monitor = new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, DBR value)
			{
				final String _pvName = pvName;
				final DBR _value = value;
				
				if (_pvName.equals(actPV.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						handleUpdateInOutDigital(_pvName, Epics.dbrToDouble(_value));
					}
					});
				}
					
			};
				
		};
		Epics.addMonitor(actPV, monitor);
	}
	public void changeInOutConfig (PVName newInPV, PVName newOutPV, PVName _pvName, double _value) {
		inPV = newInPV;
		outPV = newOutPV;
		handleUpdateInOut (_value);
	}
	public void handleUpdateInOut (double _value) {
		
			getCurrentInOut();
			boolean isOut;
			double pvVal = Epics.getDouble(pvName);

			if(Math.abs(pvVal-inVal) >= Math.abs(pvVal-outVal))
				isOut = true;
			else
				isOut = false;
		
			int motion_enabled = Epics.getInteger(motionPV);
		
				if (motion_enabled != 0 ) { 
					deviceStatus.put(title, "WAIT");
				}  else {
					
					if ( isOut ) {
						deviceStatus.put(title, "OUT");		
					} else {
						deviceStatus.put(title, "IN");
					}
				}
	}
	public void handleUpdateInOutDigital (String _pvName, double _value) {
		
		if (rqsPV.toString().equals("") || actPV.toString().equals(""))
		{
			Log.error("DevSample : reqPV/actPV is empty");
		}
		else 
		{
			getCurrentInOutDigital();
			int isIn;
			if (rqsVal == 0 && actVal == 0 ) {
				isIn = 0;
			} else if (rqsVal == 1 && actVal == 1 ) {
				isIn = 1;
			} else {
				isIn = 2;
			}
			
			if (isIn == 1) {
				deviceStatus.put(title, "IN");
				
			} else if(isIn == 0) {
				deviceStatus.put(title, "OUT");
				
			} else if(isIn == 2) {
				deviceStatus.put(title, "WAIT");
				
			} else {
			
			}
		}
		
	}
	public void getCurrentInOut () {
		if(inPV == null || outPV == null)
			return;
		if(!inPV.toString().equals(""))
			inVal = Epics.getDouble(inPV);
		else
			inVal = 0.0;
		if(!outPV.toString().equals(""))
			outVal = Epics.getDouble(outPV);
		else
			outVal = 0.0;
	}
	public void getCurrentInOutDigital () {
		rqsVal = Epics.getInteger(rqsPV);
		actVal = Epics.getInteger(actPV);
	}
}
