package gov.anl.gmca.sample;

import gov.anl.gmca.common.EpicsDevice;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.io.PVLookup.PV_MISC;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import gov.aps.jca.dbr.DBR;

import java.util.HashMap;

public class DevSlider extends EpicsDevice {

	public static void init()
	{
		new DevSlider("lowres");
		new DevSlider("highres");
		new DevSlider("ring");
		new DevSlider("back");

		new DevSlider("verticalpiezo");
		new DevSlider("horizontalpiezo");
	}

	PVName intPV;
	PVName tweakPV;
	PVName lowerLimitPV;
	PVName higherLimitPV;
	
	double intensity = 0.0;
	double scale = 0.01;
	double maxIntensity = 4.0;
	
	public ListenerList<DevSlider> ll;
	protected static HashMap<String,DevSlider> devMap = new HashMap<String, DevSlider>();
	
	public static DevSlider get(String sliderName) {
		return devMap.get(sliderName);
	}
	
	public ListenerList<DevSlider> getListenerList() {
		return ll;
	}

	public double getIntensity()
	{
		return Epics.getDouble(this.intPV);
	}

	public void setIntensity(double sliderVal) {
		double max = Epics.getDouble(this.higherLimitPV);
		double min = Epics.getDouble(this.lowerLimitPV);
		if(sliderVal <= max && sliderVal >= min)
			Epics.putDouble(this.intPV,  sliderVal);
		else
			Log.error(sliderVal+" not with in limits "+ min+ ":"+max + " for" + devMap.get(this));
	}
	
	public void setIntensity(double sliderVal, boolean tweak) {
		double scalePVVal = Epics.getDouble(tweakPV);
		double scaledValue = sliderVal*scalePVVal;
		Epics.putDouble(this.intPV, scaledValue);
	}
	
	// To do change the limits from PVs
	public double getLowerLimit() {
		return (Epics.getDouble(this.lowerLimitPV));
	}
	
	public double getHigherLimit() {
		return (Epics.getDouble(this.higherLimitPV));
	}
	
	public DevSlider(String sliderName) {
		
		super(sliderName);
		ll = new ListenerList<DevSlider>(this);
		
		// To do Move these PVs tp mysql
		
		if(sliderName.equals("lowres")) {
			this.intPV = PV_MISC.lowlightInt;
			this.tweakPV = PV_MISC.lowlightIntTweak;
			this.lowerLimitPV = PV_MISC.lowlightLowerLimit;
			this.higherLimitPV = PV_MISC.lowlightHigherLimit;
		}
		if(sliderName.equals("highres")) {
			this.intPV = PV_MISC.highlightInt;
			this.tweakPV = PV_MISC.highlightIntTweak;
			this.lowerLimitPV = PV_MISC.highlightLowerLimit;
			this.higherLimitPV = PV_MISC.highlightHigherLimit;
		}
		if(sliderName.equals("ring")) {
			this.intPV = PV_MISC.ringlightInt;
			this.tweakPV = PV_MISC.ringlightIntTweak;
			this.lowerLimitPV = PV_MISC.ringlightLowerLimit;
			this.higherLimitPV = PV_MISC.ringlightHigherLimit;
		}
		if(sliderName.equals("back")) {
			this.intPV = PV_MISC.backlightInt;
			this.tweakPV = PV_MISC.backlightIntTweak;
			this.lowerLimitPV = PV_MISC.backlightLowerLimit;
			this.higherLimitPV = PV_MISC.backlightHigherLimit;
		}
		if(sliderName.equals("verticalpiezo")) {
			
			this.intPV = PV_MISC.verticalPiezo;
			this.tweakPV = PV_MISC.verticalPiezoTweak;
			this.lowerLimitPV = PV_MISC.verticalPiezoLowerLimit;
			this.higherLimitPV = PV_MISC.verticalPiezoHigherLimit;
		}
		if(sliderName.equals("horizontalpiezo")) {
			
			this.intPV = PV_MISC.horizontalPiezo;
			this.tweakPV = PV_MISC.horizontalPiezoTweak;
			this.lowerLimitPV = PV_MISC.horizontalPiezoLowerLimit;
			this.higherLimitPV = PV_MISC.horizontalPiezoHigherLimit;
		}
		devMap.put(sliderName, this);
		
		Log.note(DevSlider.get(sliderName).intPV.toString());
		registerDevicePV(DevSlider.get(sliderName).intPV);
		registerDevicePV(DevSlider.get(sliderName).tweakPV);
		registrationComplete();
	}
	
	@Override
	protected void onAllPVsConnected() {}
	
	@Override
	protected void valueChangedAction(String pvName, DBR value) {
		if (pvName.equals(intPV.toString()))
			intensity = Epics.dbrToDouble(value);
		if (pvName.equals(tweakPV.toString()))
			scale = Epics.dbrToDouble(value);
		ll.notifyListeners();
	}
	
	@Override
	public void abort() {}
}
