package gov.anl.gmca.sample;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.sample.DevCollimatorCtrl.CCMove;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgSampleInOut implements ClientListener {
	
	Button deviceButton;
	Label deviceLabel;
	String title;
	
	protected PVName inPV;
	protected double inVal;
	
	protected PVName outPV;
	protected double outVal;
	
	protected PVName rqsPV;
	protected PVName actPV;
	
	protected int rqsVal;
	protected int actVal;
	
	protected PVName pvName;
	protected PVName motionPV;
	
	protected Label wText;
	protected Label wINOUT;
	protected String status;
	GridData gd;
	protected boolean digital = false;
	protected Epics.EpicsValueMonitor monitor = null;
	// for collimator and pindiode
	public WdgSampleInOut (Composite parent, int style, String _title, PVName pv, PVName _inPV, PVName _outPV, PVName _motionPV) {
		
		digital = false;
		pvName = pv;
		title = _title;
		inPV = _inPV;
		outPV = _outPV;
		motionPV = _motionPV;
		Client.addListener(this);
		construct(parent,style);
		
		monitor = new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(final String pvN, DBR value)
			{
				final String _pvName = pvN;
				final DBR _value = value;
				if (_pvName.equals(pvName.toString()) || _pvName.equals(motionPV.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						String buttonText = "";
						String lblText = "";
						Color tempColor = null;
						String posi = (String)DevInOut.deviceStatus.get(title);
						
						if(posi == null)
							return;
						if (posi.equals("IN")) {
							tempColor=GUI.COLOR_BRIGHT_GREEN;
							lblText = "IN";
							status = "IN";
							buttonText = "OUT";
						} else if(posi.equals("OUT")) {
							tempColor=GUI.COLOR_RED;
							lblText = "OUT";
							status = "OUT";
							buttonText = "IN";
						} else if(posi.equals("WAIT")) {
							tempColor = GUI.COLOR_SAMPLE_BG;
							lblText = "WAIT" ;
							status = "WAIT";
							buttonText = "WAIT";
						} else {
							
						}
						wINOUT.setForeground(tempColor);
						wINOUT.setText(lblText);
						deviceButton.setText(buttonText);
						/*	
						double pvVal = Math.round(Epics.get().getDouble(pvName)*100.0)/100.0;
						deviceLabel.setText("V : "+Double.toString(pvVal));
						if (!buttonText.equals("WAIT") && Client.isMaster() )
			    		{
							deviceButton.setEnabled(true);
							//TabSample.comboColli.setEnabled(true);
			    		}*/
					}
					});
				}
			};
			
		};
		Epics.addMonitor(pvName, monitor);
		Epics.addMonitor(motionPV, monitor);
		
	}
	// for beamstop and backlight Digital I/O
	public WdgSampleInOut (Composite parent, int style, String _title, PVName _rqsPV, PVName _actPV) {
		
		digital = true;
		title = _title;
		rqsPV = _rqsPV;
		actPV = _actPV;
		Client.addListener(this);
		construct(parent,style);
		
		monitor = new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, DBR value)
			{
				final String _pvName = pvName;
				final DBR _value = value;
				
				if (_pvName.equals(actPV.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						String buttonText = "";
						String lblText = "";
						Color tempColor = null;
	
						String posi = (String)DevInOut.deviceStatus.get(title);
	
						if(posi == null)
							return;
						if (posi.equals("IN")) {
							tempColor=GUI.COLOR_BRIGHT_GREEN;
							lblText = "IN";
							status = "IN";
							buttonText = "OUT";
						} else if(posi.equals("OUT")) {
							tempColor=GUI.COLOR_RED;
							lblText = "OUT";
							status = "OUT";
							buttonText = "IN";
						} else if(posi.equals("WAIT")) {
							tempColor = GUI.COLOR_SAMPLE_BG;
							lblText = "WAIT" ;
							status = "WAIT";
							buttonText = "WAIT";
						} else {
							
					}
					wINOUT.setForeground(tempColor);
					wINOUT.setText(lblText);
					deviceButton.setText(buttonText);
	
					/*
					if(!digital)
						deviceLabel.setText(Double.toString(Math.round(pvVal*100.0)/100.0));*/
					if (!buttonText.equals("WAIT") && Client.isMaster() )
		    		{
						//deviceButton.setEnabled(true);
		    		}
					}
					});
				}
					
			};
				
		};
		Epics.addMonitor(actPV, monitor);
	}
	public void construct(Composite parent, int style)
	{
			
		gd = new GridData();
		gd.widthHint = 70;
		gd.heightHint = 30;
		deviceButton = new Button(parent, SWT.PUSH);
		deviceButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		deviceButton.setLayoutData(gd);
		deviceButton.setEnabled(false);
		deviceButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub
				
				if (DevCollimatorCtrl.get().isRunning())
				{
					Util.popupOk("Script is running", "The collimator script is still running.  Please wait until it completes before starting a new move.");
					return;
				}
				
				GUI.asyncExec(new Runnable(){public void run()
				{
					//Log.note(title+" Moving "+ getStatus());
					deviceButton.setEnabled(false);
					for (TabSample tabSample : TabSample.get())
					{
						tabSample.comboColli.setEnabled(false);
					}
					
					final CCMove ccMove = constructMove(title.toLowerCase(), deviceButton.getText());
				    ccMove.start();
					
					new Thread(new Runnable(){public void run()
					{
						ccMove.waitFor();
						updateEnableState();
					}}, "collimator button").start();
				}
				});
			}
		});
		gd = new GridData();
		gd.widthHint = 50;
		gd.heightHint = 25;
		
		wINOUT = new Label(parent, SWT.NONE);
		wINOUT.setAlignment(SWT.CENTER);
		wINOUT.setFont(GUI.FONT_SAMPLE_TABS);
		wINOUT.setBackground(GUI.COLOR_DEFAULT_TEXT);
		wINOUT.setLayoutData(gd);
		wINOUT.setSize(50, 25);
		
		gd = new GridData();
		gd.widthHint = 50;
		gd.heightHint = 20;
		/*if(!digital)
		{
			deviceLabel = new Label(parent, SWT.BORDER);
			deviceLabel.setLayoutData(gd);
			deviceLabel.setBackground(parent.getBackground());
			deviceLabel.setAlignment(SWT.CENTER);
			deviceLabel.setSize(50, 20);
		}*/
	}
	
	protected CCMove constructMove(String devName, String strInOut)
	{
		boolean in = strInOut.toLowerCase().equals("in");
		String args = null;
		
		CCMove ccMove = DevCollimatorCtrl.get().newMove();
	    
		if (devName.equals("collimator"))
		{
			if (in)		args = "IN noBeamstop";
			else		args = "OUT noBacklight noBeamstop";
		}
		else if (devName.equals("pindiode"))
		{
			if (in)		args = "IN pindiodeIN noCollimator noBeamstop";
			else		args = "IN pindiodeOUT noCollimator";
		}
		else if (devName.equals("backlight"))
		{
			if (in)		args = "OUT pindiodeOUT";
			else		args = "IN backlightOUT noCollimator noBeamstop";
		}
		else if (devName.equals("beamstop"))
		{
			if (in)		args = "IN pindiodeOUT noCollimator";
			else		args = "OUT beamstopOUT noCollimator noBacklight";
		}
		
		ccMove.setArguments(args);
		return ccMove;
	}
	
	public String getStatus() {
		return status;
	}
	
	public void changeInOutConfig (PVName newInPV, PVName newOutPV) {
		inPV = newInPV;
		outPV = newOutPV;
	}
	
	public void updateEnableState()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
    		if (Client.isMaster() && AppState.isOperationAllowed(Operation.MOVE_COLL_CTRL))
    			deviceButton.setEnabled(true);
    		else
    			deviceButton.setEnabled(false);

		}});
	}
	@Override
	public void masterChanged(boolean isMaster) {
		// TODO Auto-generated method stub
		GUI.asyncExec(new Runnable(){public void run()
		{
			updateEnableState();
		}});
	}
}
