package gov.anl.gmca.sample;

import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList.CollectState;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.DevMoveFloat;
import gov.anl.gmca.common.DevMoveFloat.Listener;
import gov.anl.gmca.common.EpicsDevice;
import gov.anl.gmca.common.EpicsDevice.State;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

public class WdgSampleRotate extends Composite
{
	public abstract static class GetRotateN
	{
		public abstract double getRotateN();
	}
	
	Button gonio180, gonio90p, gonio90n, gonioNp, gonioNn;

	public WdgSampleRotate(Composite parent, int style, final GetRotateN grn)
	{
		super(parent, SWT.NONE);
		setBackground(parent.getBackground());
		
		int[] posX, posY, sizeX, sizeY;
		if (style == SWT.HORIZONTAL)
		{
			int width=56, height=27, startX=0, startY=0;
			posX = 	new int[]{ 	startX, 	startX+width, 	startX+width, 	startX+2*width, 	startX+2*width };
			posY = 	new int[]{ 	startY, 	startY, 		startY+height, 	startY, 			startY+height };
			sizeX = new int[]{	width, 		width,			width, 			width, 				width };
			sizeY = new int[]{	height*2, 	height, 		height, 		height, 			height };
			setSize(startX+width*3, startY+height*2);
		}
		else
		{
			int width=45, height=30, startX=5, startY=5;
			posX = 	new int[]{ 	startX, 	startX, 		startX+width, 	startX, 			startX+width };
			posY = 	new int[]{ 	startY, 	startY+height, 	startY+height, 	startY+2*height, 	startY+2*height };
			sizeX = new int[]{	width*2,	width, 			width, 			width, 				width };
			sizeY = new int[]{	height, 	height, 		height, 		height, 			height };
			setSize(startX+width*2, startY+height*3);
		}
		
		gonio180 = new Button(this,SWT.NONE);
		gonio180.setFont(GUI.FONT_SAMPLE_ROTATE);
		gonio180.setBackground(GUI.COLOR_BLUE_BUTTON);
		gonio180.setText("180");
		gonio180.setLocation(posX[0],posY[0]);
		gonio180.setSize(sizeX[0],sizeY[0]);
		gonio180.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				moveGonio180();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		gonio90p = new Button(this,SWT.NONE);
		gonio90p.setFont(GUI.FONT_SAMPLE_ROTATE);
		gonio90p.setText("+90");
		gonio90p.setBackground(GUI.COLOR_BLUE_BUTTON);
		gonio90p.setLocation(posX[1],posY[1]);
		gonio90p.setSize(sizeX[1],sizeY[1]);
		gonio90p.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				moveGonioN(90.0);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});

		gonio90n = new Button(this,SWT.NONE);
		gonio90n.setFont(GUI.FONT_SAMPLE_ROTATE);
		gonio90n.setText("-90");
		gonio90n.setBackground(GUI.COLOR_BLUE_BUTTON);
		gonio90n.setLocation(posX[2],posY[2]);
		gonio90n.setSize(sizeX[2],sizeY[2]);
		gonio90n.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				moveGonioN(-90.0);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});

		gonioNp = new Button(this,SWT.NONE);
		gonioNp.setFont(GUI.FONT_SAMPLE_ROTATE);
		gonioNp.setText("+n");
		gonioNp.setBackground(GUI.COLOR_BLUE_BUTTON);
		gonioNp.setLocation(posX[3],posY[3]);
		gonioNp.setSize(sizeX[3],sizeY[3]);
		gonioNp.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				moveGonioN(grn.getRotateN());
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});

		gonioNn = new Button(this,SWT.NONE);
		gonioNn.setFont(GUI.FONT_SAMPLE_ROTATE);
		gonioNn.setText("-n");
		gonioNn.setBackground(GUI.COLOR_BLUE_BUTTON);
		gonioNn.setLocation(posX[4],posY[4]);
		gonioNn.setSize(sizeX[4],sizeY[4]);
		gonioNn.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				moveGonioN(-1.0 * grn.getRotateN());
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	}
	
    protected boolean listenersConnected = false;
	
	public void connectListeners()
    {
	    if (listenersConnected) return;
			listenersConnected = true;

		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateEnabled();
			}
		});
		
		ModelCollectRunList.get().addListener(new ModelRunList.Listener()
		{
			@Override public void runListChanged(ModelRunList runList)
			{
				updateEnabled();
			}
		});

		DevPmacMotor.get("gonio_omega").addListener(new Listener()
		{
			@Override public void handleActPosChange(DevMoveFloat dev, double newPos) {}
			@Override public void handleRqsPosChange(double newPos) {}
			@Override public void handleUnitsChange(String units) {}
			@Override public void handleStateChange(State state)
			{
				updateEnabled();
			}
		});
		
		AppState.addListener(Operation.MOVE_SAMPLE, new AppState.Listener()
		{
			@Override public void stateChanged(Operation op, boolean allowed)
			{
				updateEnabled();
			}
		});
	}
	
	protected void updateEnabled()
	{
		final boolean doEnable = 
		(
			Client.isMaster()
			&&
			DevPmacMotor.get("gonio_omega").getState() == EpicsDevice.State.READY
			&&
			ModelCollectRunList.get().getCollectState() == CollectState.INACTIVE
			&&
			AppState.isOperationAllowed(Operation.MOVE_SAMPLE)
		);
		
		GUI.asyncExec(new Runnable(){public void run()
		{
    		gonio180.setEnabled(doEnable);
    		gonio90p.setEnabled(doEnable);
    		gonio90n.setEnabled(doEnable);
    		gonioNp.setEnabled(doEnable);
    		gonioNn.setEnabled(doEnable);
		}});
	}
	
	// ported from SampleWidget.tcl
	protected static int gonioDirection = 1;
	protected void moveGonio180()
	{
		double currentOmega = Math.round(DevPmacMotor.get("gonio_omega").getPosition());
		double dest = currentOmega;
		if (!Client.isMaster()) {
			Log.error("This client is not Master");
			return;
		}
			
	    // If the gonio is outside -360 to 360, move back towards that range
	    // and set the new direction regardless
		if (currentOmega > 360.1)
		{
			dest = Math.round(currentOmega) - 180.0;
			gonioDirection = -1;
		}
		else if (currentOmega < -360.1)
		{
			dest = Math.round(currentOmega) + 180.0;
			gonioDirection = 1;
		}
		else
		{
			// Use the old direction, change if -360/360 are hit
			dest = Math.round(currentOmega) + (gonioDirection * 180.0);

			if (dest > 360.1)
			{
				dest -= 360.0;
				gonioDirection = -1;
			}
			if (dest < -360.1)
			{
				dest += 360.0;
				gonioDirection = 1;
			}
	    }

		DevPmacMotor.get("gonio_omega").startMove(dest);
	}
	
	protected void moveGonioN(double n)
	{
		double currentOmega = DevPmacMotor.get("gonio_omega").getPosition();
		double roundedOmega = currentOmega;
		if (!Client.isMaster()) {
			Log.error("This client is not Master");
			return;
		}
			
		if (Math.abs(n) >= 1.0)
		{
			roundedOmega = Math.round(currentOmega);
		}
		else if (Math.abs(n) >= 0.1)
		{
			roundedOmega = Math.round(currentOmega*10.0)/10.0;
		}
		else if (Math.abs(n) >= 0.01)
		{
			roundedOmega = Math.round(currentOmega*100.0)/100.0;
		}
		else
		{
			roundedOmega = Math.round(currentOmega*1000.0)/1000.0;
		}

		DevPmacMotor.get("gonio_omega").startMove(roundedOmega + n);
	}
}
