package gov.anl.gmca.sample;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class WdgSampleSlider implements ClientListener {
	GridData gd;
	protected Label wText;
	public Slider wSlider;
	String sliderName = "";
	public int max; 
	
	protected Epics.EpicsValueMonitor monitor = null;
	public WdgSampleSlider(Composite parent, int style,  String _sliderName, String _tagName, int _max) {
		
		Client.addListener(this);
		sliderName = _tagName;
		max = _max;
		Group sliderGroup = new Group(parent, SWT.NONE);
		sliderGroup.setBackground(parent.getBackground());
		sliderGroup.setText(_sliderName);
		sliderGroup.setLayout(new GridLayout(2,false));
		gd = new GridData();
		gd.widthHint = 250;
		gd.horizontalSpan = 2;
		sliderGroup.setLayoutData(gd);
		
		gd = new GridData();
		gd.widthHint = 200;
		wSlider = new Slider(sliderGroup, SWT.NONE);
		wSlider.setBounds(10, 10, 200, 32);
		wSlider.setIncrement(5);
		//wSlider.setPageIncrement(5);
		wSlider.setThumb(1);
		//wSlider.setBackground(new Color(GUI.getDisplay(), 184,211,238));
		wSlider.setBackground(GUI.COLOR_LIGHT_GRAY);
		wSlider.setLayoutData(gd);
		
		if (Client.isMaster())
			wSlider.setEnabled(true);
		else
			wSlider.setEnabled(false);

		/*
		wSlider.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				handleSliderMove();
			}
			
		});
		*/

		wSlider.addListener(SWT.MouseUp, new Listener() {
			@Override
			public void handleEvent(Event arg0) {
				handleSliderMove();
			}
			
		});
		
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		wText = new Label(sliderGroup, SWT.NONE);
		wText.setBackground(GUI.BGCOLOR);
		wText.setText("0.005");
		wText.setLayoutData(gd);
		wText.setSize(50, 20);
		
		DevSlider.get(sliderName).ll.addListener(new ListenerList.Listener<DevSlider>() {

			@Override
			public void handleEvent(final DevSlider object, Object data) {

				GUI.asyncExec(new Runnable() {
					public void run() {
						double sliderVal = object.intensity;
						double scalePVVal = object.scale;

						wSlider.setMaximum((int) (max / scalePVVal) + 1);
						scalePVVal = 1 / scalePVVal;
						wSlider.setSelection((int) (sliderVal * scalePVVal));
						wText.setText(String.format("%.3f", sliderVal));
						//Log.error("DEBUG pv update: " + sliderName);
					}
				});
			}
		});
		DevSlider.get(sliderName).ll.notifyListeners();
	}
	
	public void handleSliderMove() {
		
		if (Client.isMaster())
		{
			int newPos = wSlider.getSelection();
			DevSlider.get(sliderName).setIntensity(newPos, true);
			//Log.error("DEBUG user triggered: " + sliderName);
		}
	}
	
	public void configurationUpdate(String key, String value) { }
	
	public void updateEnableState()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
	    	if (Client.isMaster() ) 
	    		wSlider.setEnabled(true);
	    	else 
	    		wSlider.setEnabled(false);
	    	
		}});
	}
	
	@Override
	public void masterChanged(boolean isMaster) {
		GUI.asyncExec(new Runnable() {
			public void run() {
				updateEnableState();
			}
		});
	}
	
}
