package gov.anl.gmca.scan;

import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.scan.ModelScan.ElementEdge;

import java.util.HashMap;
import java.util.HashSet;

public class DevEdgeDB
{
	public EdgeData getEdgeData(ElementEdge ee) { return dataMap.get(ee); }
	
	// look up all edges and associated data for a given element
	public HashMap<ElementEdge,EdgeData> getAllEdges(String requestedElement)
	{
		HashMap<ElementEdge,EdgeData> ret = new HashMap<ElementEdge,EdgeData>();
		
		for (ElementEdge ee : dataMap.keySet())
		{
			if (ee.element.toLowerCase().equals(requestedElement.toLowerCase()))
			{
				ret.put(ee, dataMap.get(ee));
			}
		}
		
		return ret;
	}
	
	public HashSet<String> getAllElements(boolean checkGraphFlag)
	{
		HashSet<String> ret = new HashSet<String>();
		for (ElementEdge ee : dataMap.keySet())
		{
			if (checkGraphFlag && !getEdgeData(ee).graphDisplay) continue;
			ret.add(ee.element);
		}
		return ret;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public final static String TABLE_NAME = "EdgeData";

	public static class EdgeData
	{
		public double edgeEnergy, fluoEnergy, fluoRange;
		public boolean graphDisplay;
	}
	
	protected HashMap<ElementEdge, EdgeData> dataMap = new HashMap<ElementEdge, EdgeData>();
	
	protected static DevEdgeDB inst = null;
	public static DevEdgeDB get() { return inst; }
	
	public DevEdgeDB()
	{
		inst = this;
		
		Database.Result result = Database.exec(DBName.CONFIG, "select * from "+TABLE_NAME);
		
		// get master values
		double masterFluoRange = Double.NaN;
		for (int i=0; i<result.numRows(); i++)
		{
			HashMap<String,String> row = result.getRow(i);
			
			String elementString 	= row.get("Element");
			String edgeString		= row.get("Edge");
			
			if (elementString.equals("master") && edgeString.equals("master"))
			{
				masterFluoRange 	= Double.parseDouble(row.get("FluoRange"));
			}
		}			
			
		// fill the table
		for (int i=0; i<result.numRows(); i++)
		{
			HashMap<String,String> row = result.getRow(i);
			EdgeData data = new EdgeData();
			
			String elementString 	= row.get("Element");
			String edgeString		= row.get("Edge");
			if (elementString.equals("master") || edgeString.equals("master")) continue;
			
			data.edgeEnergy 		= Double.parseDouble(row.get("EdgeEnergy"));
			data.fluoEnergy 		= Double.parseDouble(row.get("FluoEnergy"));
			
			if (row.get("FluoRange").equals("0"))
			{
				data.fluoRange = masterFluoRange;
			}
			else
			{
				data.fluoRange = Double.parseDouble(row.get("FluoRange"));
			}
			
			data.graphDisplay		= Integer.parseInt(row.get("graphDisplay"))==1;
			
			ElementEdge ee = new ElementEdge(elementString, edgeString);
			
			//System.out.println("Adding "+ee+" fluoRange = "+data.fluoRange+" fluoEnergy="+data.fluoEnergy);
			
			dataMap.put(ee, data);
		}
	}
}
