package gov.anl.gmca.scan;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.scan.ModelScan.InterStep;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ActionResult;
import gov.anl.gmca.util.Util;

public class DevOptimizeFluo
{
	protected 	static final String scriptFile 		= "optimizeFluorescence.pl";
	
	protected static ExecTask task = null;
	
	public static ActionResult run(boolean doMove)
	{
		// handled by CmdOptDetector.stop
		//WdgEmergencyStop.registerProcess(scriptFile);
		
		ActionResult ret;
		
		DevScan.get().setInterStep(InterStep.OPT_DET, true);
		
		if (Main.SIM_LEVEL>=1)		ret=runSim();
		else						ret=runScript(doMove);
		
		DevScan.get().setInterStep(InterStep.IDLE, true);
		
		return ret;
	}
	
	protected static ActionResult runScript(boolean doMove)
	{
		if (null != task && task.isRunning())
		{
			String err = scriptFile+" is already running, aborting.";
			Log.error(err);
			return new ActionResult(err);
		}
		
		String cmd = 
			Util.getPerlPrefix()+scriptFile+" "+
			DBConfig.get("app_beamlineID")+" ";
		
		if (doMove) cmd += "gotoEdgeSmall ";
		
		cmd += DBConfig.get("optimizeFluo_flags");
		
		task = ExecTask.new_ExecTask(cmd, scriptFile, true);
		task.start();
		task.waitFor();
		
		if (0 == task.exitValue)
		{
			DevScan.get().setAutoOptimized(true, true);
			return ActionResult.SUCCESS;
		}
		else
		{
			Log.error(scriptFile+" returned "+task.exitValue);
			return ActionResult.FAILURE;
		}
	}
	
	public static void stop()
	{
		if (task.isRunning()) task.asyncStop();
	}
	
	protected static ActionResult runSim()
	{
		Util.sleep(3000);
		DevScan.get().setAutoOptimized(true, true);
		return ActionResult.SUCCESS;
	}
}
