package gov.anl.gmca.scan;

import gov.anl.gmca.common.DevEpicsUpdate;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.io.PVLookup.PV_SCAN;
import gov.anl.gmca.io.PVLookup.PV_SCAN_TRACE;
import gov.anl.gmca.scan.ModelScan.*;
import gov.anl.gmca.scan.ui.ModelGraph;
import gov.anl.gmca.util.Point2D;
import gov.aps.jca.dbr.DBR;

import java.util.ArrayList;
import java.util.HashMap;

// Handles the state of the scan tab
//
public class DevScan extends DevEpicsUpdate
{
	private static DevScan inst;
	private ModelScan model; 
	public static DevScan get() { return inst; }

	public DevScan(ModelScan model)
	{
		super();
		inst = this;
		this.model = model;
		
		setUpdatePV(PV_SCAN.update);
		
		for (PVName pv : PV_SCAN.values())
		{
			registerPV(pv);
		}
		for (TRACE_ID trace_id : TRACE_ID.values())
		{
    		for (PV_SCAN_TRACE pv : PV_SCAN_TRACE.values())
    		{
    			registerPV(pv.id(trace_id));
    		}
		}
	}
	
	public void connect()
	{
		registrationComplete();
		
		Epics.addMonitor(PV_SCAN.displayedElements, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				model.setDisplayedElements(getDisplayedElements());
				ModelGraph.get().redraw(false, false, false, true, false, false);
				model.llDisplayedElements.notifyListeners();
			}
		});
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////// get methods ///////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////
	
	public double 		getEdgeTime()				{ return Epics.getDouble(PV_SCAN.time); }
	public double 		getSpectrumTime()			{ return Epics.getDouble(PV_SCAN.spectrumTime); }
	public double 		getSpectrumEnergy()			{ return Epics.getDouble(PV_SCAN.spectrumEnergy); }
	public double 		getEnergy()					{ return Epics.getDouble(PV_SCAN.edgeEnergy); }
	public double 		getScanCenter()				{ return Epics.getDouble(PV_SCAN.scanCenter); }
	public double 		getROILow()					{ return Epics.getDouble(PV_SCAN.roiLow); }
	public double 		getROIHigh()				{ return Epics.getDouble(PV_SCAN.roiHigh); }
	public double 		getInflection()				{ return Epics.getDouble(PV_SCAN.choochInflection); }
	public double 		getInfl_fP()				{ return Epics.getDouble(PV_SCAN.choochInflFp); }
	public double 		getInfl_fPP()				{ return Epics.getDouble(PV_SCAN.choochInflFpp); }
	public double 		getPeak()					{ return Epics.getDouble(PV_SCAN.choochPeak); }
	public double 		getPeak_fP()				{ return Epics.getDouble(PV_SCAN.choochPeakFp); }
	public double 		getPeak_fPP()				{ return Epics.getDouble(PV_SCAN.choochPeakFpp); }
	public double 		getRemote()					{ return Epics.getDouble(PV_SCAN.choochRemote); }
	public double 		getDefaultInflection()		{ return Epics.getDouble(PV_SCAN.defaultInflection); }
	public double 		getDefaultPeak()			{ return Epics.getDouble(PV_SCAN.defaultPeak); }
	public double 		getDefaultRemote()			{ return Epics.getDouble(PV_SCAN.defaultRemote); }
	public int 			getLeftTab()				{ return Epics.getInteger(PV_SCAN.currentTab); }
	public int			getRightTab()				{ return Epics.getInteger(PV_SCAN.rightTab); }
	public boolean 		getAutoOptimized()			{ return (1==Epics.getInteger(PV_SCAN.autoOptimized)); }
	public InterStep 	getInterStep()				{ return InterStep.values()[Epics.getInteger(PV_SCAN.interStep)]; }
	public AutoStep 	getAutoStep()				{ return AutoStep.values()[Epics.getInteger(PV_SCAN.autoStep)]; }
	public String 		getGraphTitle()             { return Epics.getString(PV_SCAN.graphTitle); }
	public String 		getGraphMessage()           { return Epics.getString(PV_SCAN.graphMessage); }
	public double		getGraphXMin()				{ return Epics.getDouble(PV_SCAN.graphXMin); }
	public double		getGraphXMax()				{ return Epics.getDouble(PV_SCAN.graphXMax); }
	public double		getDefaultXMin()			{ return Epics.getDouble(PV_SCAN.defaultXMin); }
	public double		getDefaultXMax()			{ return Epics.getDouble(PV_SCAN.defaultXMax); }
	public double		getGraphYMin(TRACE_ID id)	{ return Epics.getDouble(PV_SCAN_TRACE.graphYMin.id(id)); }
	public double		getGraphYMax(TRACE_ID id)	{ return Epics.getDouble(PV_SCAN_TRACE.graphYMax.id(id)); }
	public double		getDefaultYMin(TRACE_ID id)	{ return Epics.getDouble(PV_SCAN_TRACE.defaultYMin.id(id)); }
	public double		getDefaultYMax(TRACE_ID id)	{ return Epics.getDouble(PV_SCAN_TRACE.defaultYMax.id(id)); }
	public boolean		getTraceVisible(TRACE_ID id){ return Epics.getInteger(PV_SCAN_TRACE.isVisible.id(id))==1; }
	public TRACE_ID		getLeftAxisTrace()			{ int val = Epics.getInteger(PV_SCAN.leftAxisTrace); return val==-1?null:TRACE_ID.values()[val]; }
	public TRACE_ID		getRightAxisTrace()			{ int val = Epics.getInteger(PV_SCAN.rightAxisTrace); return val==-1?null:TRACE_ID.values()[val]; }
	public String		getGraphXLabel()			{ return Epics.getString(PV_SCAN.graphXLabel); }
	public String		getGraphYLabel(TRACE_ID id)	{ return Epics.getString(PV_SCAN_TRACE.graphYLabel.id(id)); }
	public String		getChoochPDFPath()			{ return Epics.getString(PV_SCAN.choochPDF); }
	public String		getPrefix()					{ return Epics.getString(PV_SCAN.prefix); }
	public String		getDirectory()				{ return Epics.getString(PV_SCAN.directory); }
	public String		getOtherElement()			{ return Epics.getString(PV_SCAN.otherElement); }
	public String		getOtherElement2()			{ return Epics.getString(PV_SCAN.otherElement2); }
	public ScanPlotMode getScanPlotMode()			{ return ScanPlotMode.values()[Epics.getInteger(PV_SCAN.plotMode)]; }
	public boolean		getAdaptiveScan()			{ return Epics.getInteger(PV_SCAN.adaptiveScan)==1; }
	public boolean		getShowAllEdges()			{ return Epics.getInteger(PV_SCAN.showAllEdges)==1; }
	
	public ElementEdge getElementEdge()
	{
		try									{ return new ElementEdge(Epics.getString(PV_SCAN.edge)); }
		catch (IllegalArgumentException e)	{ return null; }
	}
	
	public ArrayList<Point2D> getGraphTrace(TRACE_ID id)
	{
		ArrayList<Point2D> ret = new ArrayList<Point2D>();
		int nPoints = Epics.getInteger(PV_SCAN_TRACE.graphTraceCount.id(id));
		double[] xArray = Epics.getDoubleArray(PV_SCAN_TRACE.graphTraceX.id(id));
		double[] yArray = Epics.getDoubleArray(PV_SCAN_TRACE.graphTraceY.id(id));
		for (int i=0; i<nPoints && i<xArray.length && i<yArray.length; i++)
		{
			ret.add(new Point2D(xArray[i],yArray[i]));
		}
		return ret;
	}
	
	private HashMap<EDGE_IND,EI_STATE> getDisplayedElements()
	{
		HashMap<EDGE_IND,EI_STATE> ret = new HashMap<EDGE_IND,EI_STATE>();
		int[] val = new int[]{0};
		EDGE_IND ind=null; int i=0, stateIndex=0; EI_STATE state=null;
		try
		{
			val = Epics.getIntegerArray(PV_SCAN.displayedElements);
			for (i=0; i<EDGE_IND.values().length; i++)
			{
				ind = EDGE_IND.values()[i];
				stateIndex = val[i];
				state = EI_STATE.values()[stateIndex];
				ret.put(ind, state);
			}
		}
		catch (Exception e)
		{
			System.out.println("i="+i+" ind="+ind+" stateIndex="+stateIndex+" state="+state);
			String s = "val="; for (int j=0;j<val.length;j++) s+=val[j]+" "; System.out.println(s);
			ExecTask.syncExec("caget "+PV_SCAN.displayedElements, "caget", true);
			e.printStackTrace();
		}
		return ret;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////// set methods ///////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////
	
	public void setEdgeTime(double time, boolean update)				{ ModelScan.get().setEdgeTime(time);			Epics.putDouble(	PV_SCAN.time, time);                               if (update) sendUpdate(); } 
	public void setSpectrumTime(double time, boolean update)			{ ModelScan.get().setSpectrumTime(time);		Epics.putDouble(	PV_SCAN.spectrumTime, time);                       if (update) sendUpdate(); } 
	public void setEnergy(double energy, boolean update)				{ ModelScan.get().setEnergy(energy);			Epics.putDouble(	PV_SCAN.edgeEnergy, energy);                       if (update) sendUpdate(); } 
	public void setSpectrumEnergy(double energy, boolean update)		{ ModelScan.get().setSpectrumEnergy(energy);	Epics.putDouble(	PV_SCAN.spectrumEnergy, energy);                   if (update) sendUpdate(); } 
	public void setROILow(double d, boolean update)						{ ModelScan.get().setROILow(d);					Epics.putDouble(	PV_SCAN.roiLow, d);                                if (update) sendUpdate(); } 
	public void setROIHigh(double d, boolean update)					{ ModelScan.get().setROIHigh(d);				Epics.putDouble(	PV_SCAN.roiHigh, d);                               if (update) sendUpdate(); } 
	public void setInflection(double d, boolean update)					{ ModelScan.get().setInflection(d);				Epics.putDouble(	PV_SCAN.choochInflection, d);                      if (update) sendUpdate(); } 
	public void setInfl_fP(double d, boolean update)					{ ModelScan.get().setInfl_fP(d);				Epics.putDouble(	PV_SCAN.choochInflFp, d);                   	   if (update) sendUpdate(); } 
	public void setInfl_fPP(double d, boolean update)					{ ModelScan.get().setInfl_fPP(d);				Epics.putDouble(	PV_SCAN.choochInflFpp, d);                         if (update) sendUpdate(); } 
	public void setPeak(double d, boolean update)						{ ModelScan.get().setPeak(d);					Epics.putDouble(	PV_SCAN.choochPeak, d);                            if (update) sendUpdate(); } 
	public void setPeak_fP(double d, boolean update)					{ ModelScan.get().setPeak_fP(d);				Epics.putDouble(	PV_SCAN.choochPeakFp, d);                          if (update) sendUpdate(); } 
	public void setPeak_fPP(double d, boolean update)					{ ModelScan.get().setPeak_fPP(d);				Epics.putDouble(	PV_SCAN.choochPeakFpp, d);                         if (update) sendUpdate(); } 
	public void setRemote(double d, boolean update)						{ ModelScan.get().setRemote(d);					Epics.putDouble(	PV_SCAN.choochRemote, d);                          if (update) sendUpdate(); } 
	public void setDefaultInflection(double d, boolean update)			{ ModelScan.get().setDefaultInflection(d);		Epics.putDouble(	PV_SCAN.defaultInflection, d);                     if (update) sendUpdate(); } 
	public void setDefaultPeak(double d, boolean update)				{ ModelScan.get().setDefaultPeak(d);			Epics.putDouble(	PV_SCAN.defaultPeak, d);                           if (update) sendUpdate(); } 
	public void setDefaultRemote(double d, boolean update)				{ ModelScan.get().setDefaultRemote(d);			Epics.putDouble(	PV_SCAN.defaultRemote, d);                         if (update) sendUpdate(); } 
	public void setLeftTab(int i, boolean update)						{ ModelScan.get().setLeftTab(i);				Epics.putInteger(	PV_SCAN.currentTab, i);                            if (update) sendUpdate(); } 
	public void setRightTab(int i, boolean update)						{ ModelScan.get().setRightTab(i);				Epics.putInteger(	PV_SCAN.rightTab, i);                              if (update) sendUpdate(); } 
	public void setAutoOptimized(boolean b, boolean update)				{ ModelScan.get().setAutoOptimized(b);			Epics.putInteger(	PV_SCAN.autoOptimized, b?1:0);                     if (update) sendUpdate(); } 
	public void setInterStep(InterStep step, boolean update)			{ ModelScan.get().setInterStep(step);			Epics.putInteger(	PV_SCAN.interStep, step.ordinal());                if (update) sendUpdate(); } 
	public void setAutoStep(AutoStep step, boolean update)				{ ModelScan.get().setAutoStep(step);			Epics.putInteger(	PV_SCAN.autoStep, step.ordinal());                 if (update) sendUpdate(); } 
	public void setGraphTitle(String s, boolean update)         		{ ModelScan.get().setGraphTitle(s);				Epics.put(			PV_SCAN.graphTitle, s);                            if (update) sendUpdate(); } 
	public void setGraphMessage(String s, boolean update)         		{ ModelScan.get().setGraphMessage(s);			Epics.put(			PV_SCAN.graphMessage, s);                          if (update) sendUpdate(); } 
	public void setGraphXMin(double d, boolean update)					{ ModelScan.get().setGraphXMin(d);				Epics.putDouble(	PV_SCAN.graphXMin, d);                             if (update) sendUpdate(); } 
	public void setGraphXMax(double d, boolean update)					{ ModelScan.get().setGraphXMax(d);				Epics.putDouble(	PV_SCAN.graphXMax, d);                             if (update) sendUpdate(); } 
	public void setDefaultXMin(double d, boolean update)				{ ModelScan.get().setDefaultXMin(d);			Epics.putDouble(	PV_SCAN.defaultXMin, d);                           if (update) sendUpdate(); } 
	public void setDefaultXMax(double d, boolean update)				{ ModelScan.get().setDefaultXMax(d);			Epics.putDouble(	PV_SCAN.defaultXMax, d);                           if (update) sendUpdate(); } 
	public void setGraphYMin(TRACE_ID id, double d, boolean update)		{ ModelScan.get().setGraphYMin(id,d);			Epics.putDouble(	PV_SCAN_TRACE.graphYMin.id(id), d);                if (update) sendUpdate(); } 
	public void setGraphYMax(TRACE_ID id, double d, boolean update)		{ ModelScan.get().setGraphYMax(id,d);			Epics.putDouble(	PV_SCAN_TRACE.graphYMax.id(id), d);                if (update) sendUpdate(); } 
	public void setDefaultYMin(TRACE_ID id, double d, boolean update)	{ ModelScan.get().setDefaultYMin(id,d);			Epics.putDouble(	PV_SCAN_TRACE.defaultYMin.id(id), d);              if (update) sendUpdate(); } 
	public void setDefaultYMax(TRACE_ID id, double d, boolean update)	{ ModelScan.get().setDefaultYMax(id,d);			Epics.putDouble(	PV_SCAN_TRACE.defaultYMax.id(id), d);              if (update) sendUpdate(); } 
	public void setLeftAxisTrace(TRACE_ID id, boolean update)			{ ModelScan.get().setLeftAxisTrace(id);			Epics.putInteger(	PV_SCAN.leftAxisTrace, id==null?-1:id.ordinal());  if (update) sendUpdate(); } 
	public void setRightAxisTrace(TRACE_ID id, boolean update)			{ ModelScan.get().setRightAxisTrace(id);		Epics.putInteger(	PV_SCAN.rightAxisTrace, id==null?-1:id.ordinal()); if (update) sendUpdate(); } 
	public void setGraphXLabel(String s, boolean update)				{ ModelScan.get().setGraphXLabel(s);			Epics.put(			PV_SCAN.graphXLabel, s);                           if (update) sendUpdate(); } 
	public void setGraphYLabel(TRACE_ID id,String s, boolean update)	{ ModelScan.get().setGraphYLabel(id,s);			Epics.put(			PV_SCAN_TRACE.graphYLabel.id(id), s);              if (update) sendUpdate(); } 
	public void setChoochPDFPath(String s, boolean update)				{ ModelScan.get().setChoochPDFPath(s);			Epics.put(			PV_SCAN.choochPDF, s);                             if (update) sendUpdate(); } 
	public void setPrefix(String s, boolean update)         			{ ModelScan.get().setPrefix(s);					Epics.put(			PV_SCAN.prefix, s);                                if (update) sendUpdate(); } 
	public void setDirectory(String s, boolean update)         			{ ModelScan.get().setDirectory(s);				Epics.put(			PV_SCAN.directory, s);                             if (update) sendUpdate(); } 
	public void setOtherElement(String s, boolean update)       		{ ModelScan.get().setOtherElement(s);			Epics.put(			PV_SCAN.otherElement, s);                          if (update) sendUpdate(); } 
	public void setOtherElement2(String s, boolean update)       		{ ModelScan.get().setOtherElement2(s);			Epics.put(			PV_SCAN.otherElement2, s);                         if (update) sendUpdate(); } 
	public void setScanPlotMode(ScanPlotMode m, boolean update)			{ ModelScan.get().setScanPlotMode(m);			Epics.putInteger(	PV_SCAN.plotMode, m.ordinal());                    if (update) sendUpdate(); } 
	public void setAdaptiveScan(boolean b, boolean update)				{ ModelScan.get().setAdaptiveScan(b);			Epics.putInteger(	PV_SCAN.adaptiveScan, b?1:0);                      if (update) sendUpdate(); } 
	public void setScanCenter(double scanCenter, boolean update)		{ ModelScan.get().setScanCenter(scanCenter);	Epics.putDouble(	PV_SCAN.scanCenter, scanCenter);                   if (update) sendUpdate(); } 
	public void setShowAllEdges(boolean b, boolean update)				{ ModelScan.get().setShowAllEdges(b);			Epics.putInteger(	PV_SCAN.showAllEdges, b?1:0);	                   if (update) sendUpdate(); }

	public void setElementDisplayState(EDGE_IND edge, EI_STATE state, boolean clearOthers)                                                                                                                           
	{   
		// read existing state
		int[] displayedEdges = Epics.getIntegerArray(PV_SCAN.displayedElements);
		
		// set new state
		if (clearOthers) { for (int i=0;i<displayedEdges.length;i++) displayedEdges[i]=EI_STATE.OFF.ordinal(); }
		displayedEdges[edge.ordinal()] = state.ordinal();
		
		// send to EPICS
		Epics.putIntegerArray(PV_SCAN.displayedElements, displayedEdges);
	}
	
	public void setTraceVisible(TRACE_ID id, boolean b, boolean update)
	{
		ModelScan.get().setTraceVisible(id,b);
		Epics.putInteger(PV_SCAN_TRACE.isVisible.id(id), b?1:0);
		if (update) sendUpdate();
	}
	
	public void setGraphTrace(TRACE_ID id, ArrayList<Point2D> a, boolean update)
	{
		if (null == a) a = new ArrayList<Point2D>();
		
		ModelScan.get().setGraphTrace(id,a);
		
		double[] xArray = new double[a.size()], yArray = new double[a.size()];
		
		for (int i=0; i<a.size(); i++)
		{
			xArray[i] = a.get(i).x;
			yArray[i] = a.get(i).y;
		}
		Epics.putDoubleArray(PV_SCAN_TRACE.graphTraceX.id(id), xArray);
		Epics.putDoubleArray(PV_SCAN_TRACE.graphTraceY.id(id), yArray);
		Epics.putInteger(PV_SCAN_TRACE.graphTraceCount.id(id), 0);
		Epics.putInteger(PV_SCAN_TRACE.graphTraceCount.id(id), a.size());
		
		if (update) sendUpdate();
	}
	
	public void setElementEdge(ElementEdge ee, boolean update)
	{
		setEnergy(ee.getEnergy(), false);
		setScanCenter(ee.getEnergy(), false);
		
		model.setElementEdge(ee);
		Epics.put(PV_SCAN.edge, ee.toString());
		
		if (update) sendUpdate();
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////// update method //////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////
	
	@Override protected void sendNotification()
	{
		model.setEdgeTime              (getEdgeTime          ());   
		model.setSpectrumTime          (getSpectrumTime      ());   
		model.setEnergy                (getEnergy            ());   
		model.setSpectrumEnergy        (getSpectrumEnergy    ());   
		model.setScanCenter            (getScanCenter        ());   
		model.setROILow                (getROILow            ());   
		model.setROIHigh               (getROIHigh           ());   
		model.setInflection            (getInflection        ());   
		model.setInfl_fP               (getInfl_fP           ());   
		model.setInfl_fPP              (getInfl_fPP          ());   
		model.setPeak                  (getPeak              ());   
		model.setPeak_fP               (getPeak_fP           ());   
		model.setPeak_fPP              (getPeak_fPP          ());   
		model.setRemote                (getRemote            ());   
		model.setDefaultInflection     (getDefaultInflection ());   
		model.setDefaultPeak           (getDefaultPeak       ());   
		model.setDefaultRemote         (getDefaultRemote     ());   
		model.setLeftTab               (getLeftTab           ());   
		model.setRightTab              (getRightTab          ());   
		model.setAutoOptimized         (getAutoOptimized     ());   
		model.setInterStep             (getInterStep         ());   
		model.setAutoStep              (getAutoStep          ());   
		model.setGraphTitle            (getGraphTitle        ());   
		model.setGraphMessage          (getGraphMessage      ());   
		model.setGraphXMin             (getGraphXMin         ());   
		model.setGraphXMax             (getGraphXMax         ());   
		model.setDefaultXMin           (getDefaultXMin       ());   
		model.setDefaultXMax           (getDefaultXMax       ());   
		model.setGraphXLabel           (getGraphXLabel       ());   
		model.setChoochPDFPath         (getChoochPDFPath     ());   
		model.setPrefix                (getPrefix            ());   
		model.setDirectory             (getDirectory         ());   
		model.setOtherElement          (getOtherElement      ());   
		model.setOtherElement2         (getOtherElement2     ());   
		model.setScanPlotMode          (getScanPlotMode      ());
		model.setLeftAxisTrace         (getLeftAxisTrace     ());
		model.setRightAxisTrace        (getRightAxisTrace    ());
		model.setAdaptiveScan          (getAdaptiveScan      ());
		model.setElementEdge           (getElementEdge       ());
		model.setShowAllEdges		   (getShowAllEdges		 ());
		
		for (TRACE_ID id : TRACE_ID.values())
		{
			model.setGraphTrace		(id, getGraphTrace(id));
			model.setGraphYMin		(id, getGraphYMin(id));
			model.setGraphYMax		(id, getGraphYMax(id));
			model.setDefaultYMin	(id, getDefaultYMin(id));
			model.setDefaultYMax	(id, getDefaultYMax(id));
			model.setGraphYLabel	(id, getGraphYLabel(id));
			model.setTraceVisible	(id, getTraceVisible(id));
		}
		
		model.handleChange();
	}
	
	@Override public void sendUpdate()
	{
		//model.handleChange();
		super.sendUpdate();
	}
}
