package gov.anl.gmca.scan.ui;

import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.graphics.Transform;

public abstract class Layer
{
	protected ModelGraph graph;
	protected Image frontBuffer, backBuffer;
	
	public Layer(ModelGraph graph)
	{
		this.graph=graph;
	}
	
	// draw the buffer to the given GC
	public void draw(GC gc)
	{
		if (null != frontBuffer && null != _getBounds())
		{
			gc.drawImage(frontBuffer, _getBounds().x, _getBounds().y);
		}
	}
	
	// instead of drawing to a buffer and then drawing that to screen, this method
	// will draw directly to a given GC using a transform
	public void drawDirect(GC gc)
	{
		if (null == _getBounds()) return;
		
		Transform offsetTransform = new Transform(GUI.getDisplay());
		offsetTransform.translate(_getBounds().x, _getBounds().y);
		gc.setTransform(offsetTransform);
		gc.setClipping(0, 0, _getBounds().width, _getBounds().height);
		
		_update(gc);
		
		gc.setTransform(null);
		offsetTransform.dispose();
	}
	
	public void update()
	{
		if (null == _getBounds()) return;
		
		if (null != backBuffer && !backBuffer.isDisposed())
		{
			backBuffer.dispose();
			backBuffer = null;
		}
		
		if (null == _getBounds() || _getBounds().width<=0 || _getBounds().height<=0) return;
		
		if (null == backBuffer)
		{
			backBuffer = new Image(GUI.getDisplay(), _getBounds().width, _getBounds().height);
		}
		
		GC bufGC = new GC(backBuffer);
		_update(bufGC);
		bufGC.dispose();
		
		Image temp = backBuffer;
		backBuffer = frontBuffer;
		frontBuffer = temp;
	}
	
	public Rectangle getSize()
	{
		return new Rectangle(0,0,_getBounds().width,_getBounds().height);
	}
	
	protected abstract void _update(GC gc);
	protected abstract Rectangle _getBounds();
}
