package gov.anl.gmca.scan.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.ui.ModelGraph.Trace;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

public class LayerAxis extends Layer
{
	protected boolean isHorizontal, showArrows;
	protected int orientation, side;
	protected Color bgColor;
	
	public LayerAxis(ModelGraph graph, int orientation, int side, Color bgColor)
	{
		super(graph);
		this.orientation=orientation;
		this.side=side;
		this.isHorizontal = (SWT.HORIZONTAL==orientation);
		this.bgColor = bgColor;
		this.showArrows = false;
	}
	
	@Override protected Rectangle _getBounds()
	{
		if (SWT.HORIZONTAL == orientation)
		{
			return graph.getXScaleBounds();
		}
		else
		{
			if (side == SWT.LEFT  && null != graph.getLeftYScaleBounds())  return graph.getLeftYScaleBounds();
			if (side == SWT.RIGHT && null != graph.getRightYScaleBounds()) return graph.getRightYScaleBounds();
			return null;
		}
	}
	
	@Override
	protected void _update(GC gc)
	{
		if (!isHorizontal)
		{
			Trace trace = graph.getTrace(side);
			if (trace==null) return;
		}
		
		gc.setBackground(bgColor);
		gc.fillRectangle(getSize());
		drawScale(gc, isHorizontal);
	}
	
	protected void drawScale(GC gc, boolean isHorizontal)
	{
		int w = getSize().width-1;
		int h = getSize().height-1;
		int triSize = 10;
		
		if (isHorizontal)
		{
			// arrows on mouseover
			gc.setAntialias(SWT.ON);
			gc.setBackground(GUI.COLOR_BLACK);

			if (showArrows)
			{
				int[] leftTriOutline = new int[]{0,h/2,triSize,h/2-triSize/2,triSize,h/2+triSize/2};
				gc.fillPolygon(leftTriOutline);
				int[] rightTriOutline = new int[]{w,h/2,w-triSize,h/2-triSize/2,w-triSize,h/2+triSize/2};
				gc.fillPolygon(rightTriOutline);
			}
			
			// axis line
			gc.drawLine(0, 0, w, 0);

			// ticks and tick numbers
			for (Double x : graph.getXMajorTicks())		drawXTick(gc, x, 8, true);
			for (Double x : graph.getXMinorTicks())		drawXTick(gc, x, 5, false);
		}
		else
		{
			Trace trace = graph.getTrace(side);

			if (null != trace)
			{
				// arrows on mouseover
				gc.setAntialias(SWT.ON);
				gc.setBackground(GUI.COLOR_BLACK);
				
				if (showArrows)
				{
					int[] topTriOutline = new int[]{w/2,0,w/2-triSize/2,triSize,w/2+triSize/2,triSize};
					gc.fillPolygon(topTriOutline);
					int[] bottomTriOutline = new int[]{w/2,h,w/2-triSize/2,h-triSize,w/2+triSize/2,h-triSize};
					gc.fillPolygon(bottomTriOutline);
				}

				// axis
				gc.setBackground(GUI.COLOR_GRAY);
				if (side == SWT.LEFT)
				{
					int rightSideX = getSize().width-1;
					
					gc.setForeground(GUI.COLOR_SCAN_SHADOW);
					gc.drawLine(rightSideX, 0, rightSideX, getSize().height-1);
					gc.setForeground(colorLookup(trace.getID().getColor()));
					gc.drawLine(rightSideX-1, 0, rightSideX-1, getSize().height-1);
				}
				else
				{
					gc.setForeground(GUI.COLOR_SCAN_SHADOW);
					gc.drawLine(1, 0, 1, getSize().height-1);
					gc.setForeground(colorLookup(trace.getID().getColor()));
					gc.drawLine(0, 0, 0, getSize().height-1);
				}
				
				// ticks
				for (Double y : trace.getYMajorTicks())
				{
					drawYTick(gc, y, 8, true);
				}
			}
		}
	}
	
	protected void drawXTick(GC gc, double tickLocation, int tickLength, boolean drawText)
	{
		int tickSpacing = 2;
		gc.setFont(GUI.FONT_SCAN_AXIS_HORIZONTAL);

		// get tick location, return if it's out of bounds
		Integer screenX = graph.getScreenCoordX(tickLocation);
		if (null == screenX) return;

		// get text bounds, return if they go out of bounds
		String tickText = Util.sprintf("%.0f", tickLocation);
		Point extent = gc.textExtent(tickText);
		Point textTopLeft = new Point(screenX - extent.x/2, tickLength + tickSpacing);
		Point textBottomRight = new Point(textTopLeft.x+extent.x, textTopLeft.y+extent.y);
		
		// this used to work, but now GCs for images apparently have all zeroes for clipping regions
		//if (!gc.getClipping().contains(textTopLeft) || !gc.getClipping().contains(textBottomRight)) return;

		// draw
		gc.drawLine(screenX, 0, screenX, tickLength);
		if (drawText) gc.drawString(tickText, textTopLeft.x, textTopLeft.y, true);
	}
	
	protected void drawYTick(GC gc, double tickLocation, int tickLength, boolean drawText)
	{
		int tickSpacing = 2;
		Trace trace = graph.getTrace(side);

		// get screen location, return if it's out of bounds
		Integer tickScreenCoord = trace.getScreenCoordY(tickLocation);
		if (null == tickScreenCoord) return;
		
		// get text bounds, return if they go out of clipping bounds
		gc.setFont(GUI.FONT_SCAN_AXIS_VERTICAL);
		int decimals = trace.getYAxisLabelDecimals();
		String tickText = Util.sprintf("%."+decimals+"f", tickLocation);
		Point extent = gc.textExtent(tickText);
		int rightSide = getSize().width-2;
		int textX =
			(side == SWT.LEFT)
			? rightSide - tickLength - tickSpacing - extent.x
			: tickLength + tickSpacing;
		int textY = tickScreenCoord - extent.y/2;
		
		// see comment in x tick method
		//if (!gc.getClipping().contains(textX-1,textY-1) || !gc.getClipping().contains(textX+extent.x+1,textY+extent.y+1)) return;

		// draw tick
		if (side == SWT.LEFT)
		{
			gc.setForeground(GUI.COLOR_SCAN_SHADOW);
			gc.drawLine(rightSide-1, tickScreenCoord+1, rightSide - tickLength, tickScreenCoord+1);
			gc.setForeground(colorLookup(trace.getID().getColor()));
			gc.drawLine(rightSide, tickScreenCoord, rightSide - tickLength, tickScreenCoord);
		}
		else
		{
			gc.setForeground(GUI.COLOR_SCAN_SHADOW);
			gc.drawLine(1, tickScreenCoord+1, tickLength, tickScreenCoord+1);
			gc.setForeground(colorLookup(trace.getID().getColor()));
			gc.drawLine(0, tickScreenCoord, tickLength, tickScreenCoord);
		}
	
		// draw shadowed text
		if (drawText)
		{
			gc.setForeground(GUI.COLOR_SCAN_SHADOW);
			gc.drawString(tickText, textX+1, textY + 1, true);
			gc.drawString(tickText, textX-1, textY + 1, true);
			gc.drawString(tickText, textX-1, textY - 1, true);
			gc.drawString(tickText, textX+1, textY - 1, true);
			gc.setForeground(colorLookup(trace.getID().getColor()));
			gc.drawString(tickText, textX, textY, true);
		}
	}
	
	protected String formatYTickText(double d)
	{
		double absd = Math.abs(d);
		
		if (absd < Math.pow(10,-20)) return "0.00000";

		if (absd>=100000.0 || absd < 0.0001)
		{
			return Util.sprintf("%2.2e",d);
		}

		// 2 more decimal places could be printed, but they are left off for clarity
		// (12564.12 is unnecessary, 12564 is better)
		if (absd>=10000.0)	return Util.sprintf("%.0f",d);
		if (absd>=1000.0)	return Util.sprintf("%.1f",d);
		if (absd>=100.0)	return Util.sprintf("%.2f",d);
		if (absd>=10.0)		return Util.sprintf("%.3f",d);
		if (absd>=1.0)		return Util.sprintf("%.4f",d);
		if (absd>=0.1)		return Util.sprintf("%.5f",d);
		
		return Util.sprintf("%.6f",d);
	}
	
	private static final Color[][] lookup =
	{
		{ GUI.COLOR_GREEN,	GUI.COLOR_GREEN	},
		{ GUI.COLOR_BLUE,	GUI.COLOR_BLUE	},
		{ GUI.COLOR_RED, 	GUI.COLOR_DARK_RED	}
	};
	
	public static Color colorLookup(Color traceColor)
	{
		for (int i=0; i<lookup.length;i++)
		{
			if (lookup[i][0]==traceColor) return lookup[i][1];
		}
		return traceColor;
	}
}
