package gov.anl.gmca.scan.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.ui.ModelGraph.Trace;
import gov.anl.gmca.util.VerticalText;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

public class LayerLabels extends Layer
{
	protected Color bgColor;
	
	public LayerLabels(ModelGraph graph, Color bgColor)
	{
		super(graph);
		this.bgColor=bgColor;
	}

	@Override protected Rectangle _getBounds()
	{
		return new Rectangle(0,0,graph.getTotalSize().x, graph.getTotalSize().y);
	}
	
	@Override
	protected void _update(GC gc)
	{
		// fill background
		gc.setBackground(bgColor);
		gc.fillRectangle(getSize());

		// draw title
		drawTitle(gc, _getBounds());
		
		// draw labels
		drawAxisLabel
		(
			gc,
			graph.getGraphBounds().x + graph.getGraphBounds().width/2 - graph.getXLabelExtent().x/2,
			graph.getGraphBounds().y + graph.getGraphBounds().height + ModelGraph.SPACING + graph.getXScaleHeight() + ModelGraph.SPACING,
			SWT.BOTTOM,
			null
		);
		
		Trace leftTrace = graph.getTrace(SWT.LEFT), rightTrace = graph.getTrace(SWT.RIGHT);
		
		if (null != leftTrace)
		{
			Rectangle graphBounds = graph.getGraphBounds();
			Point yLabelExtent = leftTrace.getYLabelExtent();
			String err = "";
			
			if (null == graphBounds)		{ err += "graphBounds==null ";  }
			if (null == yLabelExtent)		{ err += "yLabelExtent==null "; }
			if (err.length()==0)
			{
				int x = ModelGraph.MARGIN;
				int y = graphBounds.y + graphBounds.height/2 + yLabelExtent.y/2;
				drawAxisLabel(gc,x,y,SWT.LEFT,leftTrace.getID().getColor());
			}
			else
			{
				System.out.println("Left trace axis drawing skipped: "+err);
			}
		}
		
		if (null != rightTrace)
		{
			Rectangle graphBounds = graph.getGraphBounds();
			Point yLabelExtent = rightTrace.getYLabelExtent();
			Rectangle yScaleBounds = graph.getRightYScaleBounds();
			String err = "";
			
			if (null == graphBounds)		{ err += "graphBounds==null ";  }
			if (null == yLabelExtent)		{ err += "yLabelExtent==null "; }
			if (null == yScaleBounds)		{ err += "yScaleBounds==null "; }
			if (err.length()==0)
			{
				int x = yScaleBounds.x + yScaleBounds.width + ModelGraph.SPACING + yLabelExtent.x;
				int y = graphBounds.y + graphBounds.height/2 - yLabelExtent.y/2;
				drawAxisLabel(gc,x,y,SWT.RIGHT,rightTrace.getID().getColor());
			}
			else
			{
				System.out.println("Right trace axis drawing skipped: "+err);
			}
		}
	}
	
	protected void drawTitle(GC gc, Rectangle bounds)
	{
		gc.setFont(GUI.FONT_SCAN_TITLE);
		Point titleExtent = gc.textExtent(graph.getTitle());
		gc.drawString(graph.getTitle(), bounds.width/2-titleExtent.x/2, ModelGraph.MARGIN);
	}
	
	protected void drawAxisLabel(GC gc, int x, int y, int side, Color fgColor)
	{
		gc.setForeground(GUI.COLOR_BLACK);
		gc.setBackground(bgColor);
		gc.setFont(GUI.FONT_SCAN_AXIS_HORIZONTAL);
		
		if (SWT.BOTTOM == side)
		{
			if (0!=graph.getXLabel().length())
			{
				gc.drawString(graph.getXLabel(), x, y);
			}
		}
		else
		{
			Trace leftTrace = graph.getTrace(SWT.LEFT), rightTrace = graph.getTrace(SWT.RIGHT);
			
			gc.setFont(GUI.FONT_SCAN_AXIS_VERTICAL);
			gc.setForeground(fgColor);				
			if (side == SWT.LEFT)
			{
				VerticalText.drawString(gc, leftTrace.getYLabel(), x, y, SWT.LEFT, GUI.COLOR_SCAN_SHADOW);
			}
			if (side == SWT.RIGHT)
			{
				VerticalText.drawString(gc, rightTrace.getYLabel(), x, y, SWT.RIGHT, GUI.COLOR_SCAN_SHADOW);
			}
		}
	}
}
