package gov.anl.gmca.scan.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.DevEdgeDB;
import gov.anl.gmca.scan.DevEdgeDB.EdgeData;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.EDGE_IND;
import gov.anl.gmca.scan.ModelScan.EI_STATE;
import gov.anl.gmca.scan.ModelScan.ElementEdge;
import gov.anl.gmca.scan.ui.ModelGraph.Bar;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import java.util.HashMap;

public class LayerOverlays extends Layer
{
	protected Bar barBeingDragged = null;
	protected Point hoverScreenPt = null;
	protected String hoverText = null;
	protected Double edgeDisplayX = null; // mouse x coord. in pixels
	
	//private static final Color[] tempEdgeColors = { GUI.COLOR_TEAL, GUI.COLOR_TEAL2, GUI.COLOR_TEAL3 };
	//private static final Color[] permEdgeColors = { GUI.COLOR_BLUE, GUI.COLOR_BLUE2, GUI.COLOR_BLUE3 };
	
	public LayerOverlays(ModelGraph graph)
	{
		super(graph);
	}
	
	@Override protected Rectangle _getBounds()
	{
		return graph.getGraphBounds();
	}
	
	@Override public void _update(GC gc)
	{
		drawBars(gc, true);
	}
	
	@Override public void draw(GC gc)
	{
		drawBars(gc, false);
		if (!drawZoomRectangle(gc))
		{
			drawHoverText(gc);
			drawEdgeDisplay(gc);
		}
	}
	
	protected void drawEdgeDisplay(GC gc)
	{
		if (edgeDisplayX != null)
		{
			gc.setLineWidth(ModelGraph.BAR_WIDTH);
			drawLine(gc, GUI.COLOR_BLUE, SWT.LINE_SOLID, edgeDisplayX, false);
		}
	}
	
	protected void drawHoverText(GC gc)
	{
		if (null != hoverScreenPt && null != hoverText && hoverText.length()>0)
		{
			Point extent = gc.textExtent(hoverText);
			int x = hoverScreenPt.x + 15;
			int y = hoverScreenPt.y - (extent.y/2);
			//Rectangle graphBounds = ModelGraph.get().getGraphBounds();
			//if (x+extent.x>=graphBounds.x+graphBounds.width) x = hoverScreenPt.x - 10 - extent.x;
			if (x+extent.x>=gc.getClipping().width) x = hoverScreenPt.x - 10 - extent.x;
			
			gc.setForeground(GUI.COLOR_BLACK);
			gc.setBackground(GUI.COLOR_WHITE);
			Rectangle textBounds = new Rectangle(x-2,y-2,extent.x+4,extent.y+4);
			gc.fillRectangle(textBounds);
			gc.drawRectangle(textBounds);
			gc.drawText(hoverText, x, y);
		}
	}
	
	protected void drawBars(GC gc, boolean useScreenCoord)
	{
		int oldWidth = gc.getLineWidth();
		gc.setLineWidth(ModelGraph.BAR_WIDTH);
		
		for (Bar bar : graph.getBars())
		{
			if (Double.isNaN(bar.x)) continue;
			
			drawLine(gc, bar.color, SWT.LINE_SOLID, bar.x, useScreenCoord);
		}
		
		for (EDGE_IND element : EDGE_IND.values())
		{
			EI_STATE displayState = graph.getElementDisplayState(element);
			
			Color color=null;
			if (ModelScan.get().getShowAllEdges())
			{
				color = GUI.COLOR_BLUE;
				displayState = EI_STATE.ON;
			}
			else switch (displayState)
			{
			case OFF:
				continue;
			case TEMP:
				color = GUI.COLOR_TEAL;
				break;
			case ON:
				color = GUI.COLOR_BLUE;
				break;
			}
			
			String elementString;
			switch (element)
			{
			case Other:		elementString = ModelScan.get().getOtherElement(); break;
			case Other2:	elementString = ModelScan.get().getOtherElement2(); break;
			default:		elementString = element.toString();
			}
			
			HashMap<ElementEdge,EdgeData> edges = DevEdgeDB.get().getAllEdges(elementString);
			int edgeNum = 0, lineStyle = SWT.LINE_DASH;
			for (ElementEdge ee : edges.keySet())
			{
				EdgeData data = edges.get(ee);
				double xCoord = data.fluoEnergy;
				boolean underCurrentEnergy = xCoord < ModelScan.get().getSpectrumEnergy();
				boolean displayFlag = DevEdgeDB.get().getEdgeData(ee).graphDisplay;
				
				//System.out.println(ee.toString()+" underCurrentEnergy="+underCurrentEnergy+" displayFlag="+displayFlag);
				
				if (underCurrentEnergy && displayFlag)
				{
					if (edgeNum==1 && displayState==EI_STATE.ON) color = GUI.COLOR_PURPLE;
					if (edgeNum==2 && displayState==EI_STATE.ON) color = GUI.COLOR_BRIGHT_GREEN3;
					drawLine(gc, color, lineStyle, xCoord, useScreenCoord);
					edgeNum++;
				}
			}
		}
		
		gc.setLineWidth(oldWidth);
		gc.setLineStyle(SWT.LINE_SOLID);
	}
	
	private void drawLine(GC gc, Color color, int lineStyle, double xCoord, boolean useScreenCoord)
	{
		if (xCoord < graph.getPlotXRange().xMin || graph.getPlotXRange().xMax < xCoord) return;
		
		Integer screenX = graph.getScreenCoordX(xCoord);
		if (null == screenX) return;
		
		gc.setForeground(color);
		gc.setLineStyle(lineStyle);
		
		int yMin = 0;
		int yMax = graph.getGraphBounds().height;

		int absYMin = graph.getGraphBounds().y + yMin;
		int absYMax = graph.getGraphBounds().y + yMax;
		
		// screenX is in LayerPlot coordinates
		int absoluteScreenX = screenX + graph.getGraphBounds().x;
		
		//System.out.println("Drawing bar: "+screenX+","+yMin+","+screenX+","+yMax);
		gc.drawLine
		(
			useScreenCoord ? screenX : absoluteScreenX,
			useScreenCoord ? yMin    : absYMin,
			useScreenCoord ? screenX : absoluteScreenX,
			useScreenCoord ? yMax    : absYMax
		);
	}
	
	private boolean drawZoomRectangle(GC gc)
	{
		Point topLeft = ListenerZoom.get().topLeft;
		Point bottomRight = ListenerZoom.get().bottomRight;
		if (null==topLeft || null==bottomRight) return false;
		
		Rectangle rect = new Rectangle(topLeft.x,topLeft.y,bottomRight.x-topLeft.x,bottomRight.y-topLeft.y);
		
		gc.setForeground(GUI.COLOR_BLACK);
		gc.drawRectangle(rect);
		return true;
	}
}
