package gov.anl.gmca.scan.ui;

import gov.anl.gmca.scan.DevEdgeDB;
import gov.anl.gmca.scan.DevEdgeDB.EdgeData;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.EDGE_IND;
import gov.anl.gmca.scan.ModelScan.EI_STATE;
import gov.anl.gmca.scan.ModelScan.ElementEdge;
import gov.anl.gmca.scan.ui.ModelGraph.Bar;
import gov.anl.gmca.scan.ui.ModelGraph.Trace;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.graphics.Point;

import java.util.HashMap;

public class ListenerBarDrag implements MouseListener, MouseMoveListener
{
	private static ListenerBarDrag inst;
	public static ListenerBarDrag get() { if (null==inst) inst=new ListenerBarDrag(); return inst; }

	public void addListeners(WdgGraph wdg)
	{
		wdg.addMouseListener(this);
		wdg.addMouseMoveListener(this);
	}
	
	@Override public void mouseDown(MouseEvent me)
	{
		ModelGraph model = ModelGraph.get();
		
		if (!model.getEnabled()) return;
		
		// was a bar clicked?
		Bar detectedBar = null;
		for (Bar bar : model.getBars())
		{
			Integer screenX = model.getScreenCoordX(bar.x);
			int relativeMouseX = me.x - model.getGraphBounds().x;
			//System.out.println("bar.x="+bar.x+" screenX="+screenX+" me.x="+me.x+" mouseX="+relativeMouseX);
			if (null != screenX && Math.abs(relativeMouseX - screenX) < (1 + ModelGraph.BAR_WIDTH))
			{
				detectedBar = bar;
			}
		}

		// was it a middle click and is no other bar being dragged?
		if (null != detectedBar && 2 == me.button)
		{
			if (0 == ModelScan.get().getLeftTab() &&
				(detectedBar == model.barROILow || detectedBar == model.barROIHigh))
			{
				Log.note("Changing ROI is not supported in auto mode");
			}
			else
			{
    			System.out.println("Detected middle-click on bar "+detectedBar);
    			WdgGraph.get().layerOverlays.barBeingDragged = detectedBar;
			}
		}
	}

	@Override
	public void mouseUp(MouseEvent me)
	{
		if (!ModelGraph.get().getEnabled()) return;
		
		if (2 == me.button)
		{
			ModelGraph graph = ModelGraph.get();
			Bar bar = WdgGraph.get().layerOverlays.barBeingDragged;
			DevScan dev = DevScan.get();
			
    		if (bar == graph.barROILow)		dev.setROILow		(graph.barROILow.x, true);
    		if (bar == graph.barROIHigh)	dev.setROIHigh		(graph.barROIHigh.x, true);
    		if (bar == graph.barInflection)	dev.setInflection	(graph.barInflection.x, true);
    		if (bar == graph.barPeak)		dev.setPeak			(graph.barPeak.x, true);
    		if (bar == graph.barRemote)		dev.setRemote		(graph.barRemote.x, true);
    		
    		WdgGraph.get().layerOverlays.barBeingDragged = null;
		}
	}
	
	@Override
	public void mouseMove(MouseEvent me)
	{
		// update ROI value on drag
		if (null != WdgGraph.get().layerOverlays.barBeingDragged)
		{
			Double graphCoordX = ModelGraph.get().getGraphCoordX(me.x-ModelGraph.get().getGraphBounds().x, false);
			if (null != graphCoordX)
			{
				// ensure bars don't cross over each other (high always >= low)
				if (WdgGraph.get().layerOverlays.barBeingDragged == ModelGraph.get().barROILow &&
					graphCoordX > ModelGraph.get().barROIHigh.x)
				{
					graphCoordX = ModelGraph.get().barROIHigh.x;
				}
				if (WdgGraph.get().layerOverlays.barBeingDragged == ModelGraph.get().barROIHigh &&
					graphCoordX < ModelGraph.get().barROILow.x)
				{
					graphCoordX = ModelGraph.get().barROILow.x;
				}
				
				WdgGraph.get().layerOverlays.barBeingDragged.x = graphCoordX;
				ModelGraph.get().listenerList.notifyListeners(); // to update the ROI text widgets
				WdgGraph.get().redraw();
			}
		}

		// show graph point coordinates on hover, but don't show point data during dragging
		// also don't update here if the edge display is active
		if (null == WdgGraph.get().layerOverlays.barBeingDragged && null == WdgGraph.get().layerOverlays.edgeDisplayX) 
		{
			// TODO: this is very inefficient - is it affecting responsiveness?
			
    		double smallestDistance = Double.MAX_VALUE;
    		Point closestScreenPt = null;
    		String closestPtText = null;
			int relativeMouseX = me.x - ModelGraph.get().getGraphBounds().x;
			int relativeMouseY = me.y - ModelGraph.get().getGraphBounds().y;
			
			for (Trace trace : ModelGraph.get().getTraces())
			{
				if (!trace.getVisible()) continue;
				
	    		for (Point2D graphPt : trace.getPoints())
	    		{
	    			Point screenPt = trace.getScreenCoord(graphPt.x,graphPt.y);
	    			
	    			if (null != screenPt)
	    			{
	        			double dx = Math.abs(relativeMouseX-screenPt.x);
	        			double dy = Math.abs(relativeMouseY-screenPt.y);
	        			double dist = Math.sqrt(dx*dx+dy*dy);
	        			
	        			if (dist < smallestDistance)
	        			{
	        				smallestDistance = dist;
	        				closestScreenPt = screenPt;
	        				closestPtText = Util.sprintf("(%.2f,%.2f)",graphPt.x,graphPt.y);
	        			}
	    			}
	    		}
			}
			
			HashMap<EDGE_IND,EI_STATE> eMap = new HashMap<EDGE_IND,EI_STATE>(ModelScan.get().getDisplayedElements());
			
			// search edges
			boolean edgeFound = false;
			for (EDGE_IND elem : eMap.keySet())
			{
				// to see which ones are selected
				if (ModelScan.get().getShowAllEdges() || eMap.get(elem) != EI_STATE.OFF)
				{
					String elementString = elem.toString();
					if (elem==EDGE_IND.Other) elementString = ModelScan.get().getOtherElement();
					if (elem==EDGE_IND.Other2) elementString = ModelScan.get().getOtherElement2();
					
					// and below the scan energy
					HashMap<ElementEdge,EdgeData> dataMap = DevEdgeDB.get().getAllEdges(elementString);
					for (ElementEdge ee : dataMap.keySet())
					{
						if (!DevEdgeDB.get().getEdgeData(ee).graphDisplay) continue;
						
						double flEnergy = dataMap.get(ee).fluoEnergy;
						
						if (flEnergy < ModelScan.get().getSpectrumEnergy())
						{
							// and then check them for mouse proximity
							double screenX = ModelGraph.get().getScreenCoordX(flEnergy);
							double dist = Math.abs(relativeMouseX - screenX);
							
		        			if (dist < 5.0)
		        			{
		        				smallestDistance = dist;
		        				closestScreenPt = new Point(relativeMouseX,relativeMouseY);
		        				if (edgeFound) closestPtText += "\n"; else closestPtText = "";
		        				closestPtText += Util.sprintf(ee.toString()+": %.2f eV",flEnergy);
		        				edgeFound = true;
		        			}
						}
					}
				}
			}
	    		
    		if (smallestDistance < 5.0)
    		{
    			if (null == WdgGraph.get().layerOverlays.hoverScreenPt || WdgGraph.get().layerOverlays.hoverScreenPt.x != closestScreenPt.x || WdgGraph.get().layerOverlays.hoverScreenPt.y != closestScreenPt.y)
    			{
    				WdgGraph.get().layerOverlays.hoverScreenPt = closestScreenPt;
    				WdgGraph.get().layerOverlays.hoverText = closestPtText;
    				WdgGraph.get().redraw();
    			}
    		}
    		else
    		{
    			if (null != WdgGraph.get().layerOverlays.hoverScreenPt)
    			{
    				WdgGraph.get().layerOverlays.hoverScreenPt = null;
    				WdgGraph.get().layerOverlays.hoverText = null;
        			WdgGraph.get().redraw();
    			}
    		}
		}
	}

	@Override public void mouseDoubleClick(MouseEvent e) {}
}
