package gov.anl.gmca.scan.ui;

import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan.TRACE_ID;
import gov.anl.gmca.scan.ui.ModelGraph.XRange;
import gov.anl.gmca.scan.ui.ModelGraph.YRange;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import java.util.HashMap;
import java.util.concurrent.atomic.AtomicBoolean;

public class ListenerScroll
{
	protected XRange 					graphXStart			= null;
	protected HashMap<TRACE_ID, YRange> graphYStart 		= new HashMap<TRACE_ID, YRange>();
	protected boolean 					dragInProgress 		= false;
	protected Point 					startPt				= null;
	protected Point 					endPt				= null;
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////// mouse event handlers ///////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	private void handleDragStart(Point screenPt, boolean ctrlDown)
	{
		boolean scaleDrag = false; //inScaleBounds(screenPt);
		boolean plotDrag = (inPlotBounds(screenPt) && ctrlDown);
		if (scaleDrag || plotDrag)
		{
			setShowArrows(true);
			ModelGraph.get().redraw(false, true, true, false, false, false); // redraw axes
			WdgGraph.get().update();
			
			dragInProgress = true;
			WdgGraph.get().layerPlot.setLineAA(false);

			// save starting ranges
			graphXStart = new XRange(ModelGraph.get().getReqXRange());
			for (TRACE_ID id : TRACE_ID.values())
			{
				graphYStart.put(id, new YRange(ModelGraph.get().getTrace(id).getReqYRange()));
			}
			
			if (ModelGraph.get().getXScaleBounds().contains(screenPt) || plotDrag)
			{
				startPt = screenPt;
			}
		}
	}
	
	private void handleDragEnd()
	{
		DevScan.get().sendUpdate();
		dragInProgress = false; startPt = null; endPt = null;
		WdgGraph.get().layerPlot.setLineAA(true);
		setShowArrows(false);
		
		ModelGraph.get().redraw(true, true, true, true, true, false);
		WdgGraph.get().update();
	}

	private void handleDragMove(Point screenPt)
	{
		// if we're in a drag but we started out of bounds...
		if (null == startPt)
		{
			// check if we've gone in bounds yet
			if (inScaleBounds(screenPt))
			{
				startPt = screenPt;
			}
		}
		else // we're already in bounds
		{
			endPt = screenPt;

			// how much have we dragged?
			double graphEndX = ModelGraph.get().getGraphCoordX(endPt.x, true);
			double graphStartX = ModelGraph.get().getGraphCoordX(startPt.x, true);
			double graphDeltaX = graphEndX - graphStartX;
			
			// add that to the starting range
			double newXMin = graphXStart.xMin - graphDeltaX;
			double newXMax = graphXStart.xMax - graphDeltaX;
			DevScan.get().setGraphXMin(newXMin, false);
			DevScan.get().setGraphXMax(newXMax, false);
			
			for (TRACE_ID id : TRACE_ID.values())
			{
				// check drag distance for y axes
				double graphEndY = ModelGraph.get().getTrace(id).getGraphCoordY(endPt.y, true);
				double graphStartY = ModelGraph.get().getTrace(id).getGraphCoordY(startPt.y, true);
				double graphDeltaY = graphEndY - graphStartY;
				
				// apply to starting y ranges
				double newYMin = graphYStart.get(id).yMin - graphDeltaY;
				double newYMax = graphYStart.get(id).yMax - graphDeltaY;
				DevScan.get().setGraphYMin(id, newYMin, false);
				DevScan.get().setGraphYMax(id, newYMax, false);
			}
			
			DevScan.get().sendUpdate();
			
			// redraw locally
			requestGUIUpdate();
		}
	}

	private void handleMouseMove(Point screenPt)
	{
		if (dragInProgress) handleDragMove(screenPt);
	}

	private AtomicBoolean updatePending = new AtomicBoolean(false);
	private AtomicBoolean updateDeferred = new AtomicBoolean(false);
	Runnable dragUpdate = new Runnable(){public void run()
	{
		ModelGraph.get().redraw(false, false, false, true, false, false);

		Rectangle plotBounds = ModelGraph.get().getGraphBounds();
		WdgGraph.get().redraw(plotBounds.x, plotBounds.y, plotBounds.width, plotBounds.height, false);
		WdgGraph.get().update(); // force update, don't allow redraws to be merged
		
		//System.out.println("pending=false");
		//updatePending.set(false);
	}};
	
	private void requestGUIUpdate()
	{
		/*if (updatePending.get())
		{
			System.out.println("Deferring update...");
			updateDeferred.set(true);
			return;
		}
		else
		{
			System.out.println("pending=true");
			updatePending.set(true);
			GUI.asyncExec(dragUpdate);
		}*/
		dragUpdate.run();
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////// arrow indicators ///////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	private boolean getShowArrows()
	{
		LayerAxis xAxis	= WdgGraph.get().layerAxisX; if (null!=xAxis) return xAxis.showArrows;
		return false;
	}
	
	private void setShowArrows(boolean show)
	{
		LayerAxis xAxis 	= WdgGraph.get().layerAxisX;		if (null!=xAxis) 		xAxis.showArrows=show;
		LayerAxis leftAxis 	= WdgGraph.get().layerAxisYLeft;	if (null!=leftAxis) 	leftAxis.showArrows=show;
		LayerAxis rightAxis = WdgGraph.get().layerAxisYRight;	if (null!=rightAxis) 	rightAxis.showArrows=show;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////// in-bounds checks ///////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	private boolean inScaleBounds(Point screenPt)
	{
		Rectangle xBounds 		= ModelGraph.get().getXScaleBounds();
		Rectangle leftYBounds 	= ModelGraph.get().getLeftYScaleBounds();
		Rectangle rightYBounds 	= ModelGraph.get().getRightYScaleBounds();
		
		if (null!=xBounds 		&& xBounds.contains(screenPt)) 		return true;
		if (null!=leftYBounds 	&& leftYBounds.contains(screenPt)) 	return true;
		if (null!=rightYBounds 	&& rightYBounds.contains(screenPt)) return true;
		
		return false;
	}
	
	private boolean inPlotBounds(Point screenPt)
	{
		return ModelGraph.get().getGraphBounds().contains(screenPt);
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////// ctor and related ///////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	private static ListenerScroll inst;
	public static ListenerScroll get() { if (null==inst) inst=new ListenerScroll(); return inst; }

	public ListenerScroll() {}
	
	public void addListeners(WdgGraph wdg)
	{
		wdg.addMouseListener(new MouseListener()
		{
			@Override public void mouseUp(MouseEvent e)
			{
				handleDragEnd();
			}
			@Override public void mouseDown(MouseEvent e) {}
			@Override public void mouseDoubleClick(MouseEvent e) {}
		});
		wdg.addMouseMoveListener(new MouseMoveListener()
		{
			@Override public void mouseMove(MouseEvent e)
			{
				handleMouseMove(new Point(e.x,e.y));
			}
		});
		wdg.addDragDetectListener(new DragDetectListener()
		{
			@Override public void dragDetected(DragDetectEvent e)
			{
				boolean ctrlDown = (e.stateMask & SWT.CTRL) != 0;
				handleDragStart(new Point(e.x,e.y), ctrlDown);
			}
		});
	}
}
