package gov.anl.gmca.scan.ui;

import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan.TRACE_ID;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

public class ListenerZoom
{
	private static ListenerZoom inst;
	public static ListenerZoom get() { if (null==inst) inst=new ListenerZoom(); return inst; }

	protected Point topLeft, bottomRight;
	protected boolean drag = false;
	
	public void addListeners(WdgGraph wdg)
	{
		wdg.addMouseListener(new MouseListener()
		{
			@Override public void mouseUp(MouseEvent e)
			{
				zoom();
				drag = false; topLeft = null; bottomRight = null;
				WdgGraph.get().redraw();
				WdgGraph.get().update();
			}
			@Override public void mouseDown(MouseEvent e) {}
			@Override public void mouseDoubleClick(MouseEvent e) {}
		});
		wdg.addMouseMoveListener(new MouseMoveListener()
		{
			@Override public void mouseMove(MouseEvent e)
			{
				if (drag)
				{
					Point screenPt = new Point(e.x,e.y);

					// if we're in a drag but we started out of bounds...
					if (null == topLeft)
					{
						// check if we've gone in bounds yet
						if (ModelGraph.get().getGraphBounds().contains(screenPt))
						{
							topLeft = screenPt;
						}
					}
					else // we're already in bounds
					{
						// only update to the boundaries, not farther
						Point corrected = new Point(screenPt.x,screenPt.y);
						Rectangle bounds = ModelGraph.get().getGraphBounds();
						if (corrected.x < bounds.x) corrected.x = bounds.x;
						if (corrected.x > bounds.x+bounds.width) corrected.x = bounds.x+bounds.width;
						if (corrected.y < bounds.y) corrected.y = bounds.y;
						if (corrected.y > bounds.y+bounds.height) corrected.y = bounds.y+bounds.height;
						
						// apply update and redraw zoom rectangle
						bottomRight = corrected;
						WdgGraph.get().redraw();
						WdgGraph.get().update(); // force update, don't allow redraws to be merged
					}
				}
			}
		});
		wdg.addDragDetectListener(new DragDetectListener()
		{
			@Override public void dragDetected(DragDetectEvent e)
			{
				if (0 != (e.stateMask & SWT.CTRL)) return;
				
				Point screenPt = new Point(e.x,e.y);
				
				if (ModelGraph.get().getGraphBounds().contains(screenPt))
				{
					drag = true;
					topLeft = screenPt;
				}
			}
		});
	}
	
	private void zoom()
	{
		if (null==topLeft||null==bottomRight) return;
		
		// get screen coordinates
		int screenXMin = Math.min(topLeft.x,bottomRight.x)-ModelGraph.get().getGraphBounds().x;
		int screenXMax = Math.max(topLeft.x,bottomRight.x)-ModelGraph.get().getGraphBounds().x;
		int screenYMin = Math.min(topLeft.y,bottomRight.y)-ModelGraph.get().getGraphBounds().y;
		int screenYMax = Math.max(topLeft.y,bottomRight.y)-ModelGraph.get().getGraphBounds().y;

		// convert X coordinates to graph units and send to device
		double xMin = ModelGraph.get().getGraphCoordX(screenXMin, false);
		double xMax = ModelGraph.get().getGraphCoordX(screenXMax, false);
		DevScan.get().setGraphXMin(xMin, false);
		DevScan.get().setGraphXMax(xMax, false);

		// for each trace
		for (TRACE_ID trace : TRACE_ID.values())
		{
			// convert Y coordinates to graph units and send to device
			double yMin = ModelGraph.get().getTrace(trace).getGraphCoordY(screenYMin, false);
			double yMax = ModelGraph.get().getTrace(trace).getGraphCoordY(screenYMax, false);
			DevScan.get().setGraphYMin(trace, yMax, false);
			DevScan.get().setGraphYMax(trace, yMin, false);
		}
		
		DevScan.get().sendUpdate();
	}
}
