package gov.anl.gmca.scan.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.EDGE_IND;
import gov.anl.gmca.scan.ModelScan.EI_STATE;
import gov.anl.gmca.scan.ModelScan.TRACE_ID;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Pair;
import gov.anl.gmca.util.Point2D;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

// This class is independent of JBluIce.  It calculates the positioning of the graph elements.
//
public class ModelGraph
{
	ListenerList<ModelGraph> listenerList;
	
	////////////////////////////////////////////// constants ////////////////////////////////////////////
	
	protected	static final int	SPACING						= 10;
	protected	static final int	MARGIN						= 10;
	protected	static final int	BAR_WIDTH					= 2;
	protected	static final double	PLOT_MARGIN					= 0.075;
	protected	static final double	MIN_RANGE					= 0.2;
	
	///////////////////////////////////////////// Trace class ///////////////////////////////////////////
	
	public class Trace
	{
		public Trace() {}
		
		protected	TRACE_ID				id;
		public		TRACE_ID				getID()			{ return id; }
		
		private 	ArrayList<Point2D> 	points				= new ArrayList<Point2D>();
		public		ArrayList<Point2D> 	getPoints()			{ return new ArrayList<Point2D>(points); }
		public		void setPoints(ArrayList<Point2D> pts) 	{ points = pts; }
		public		void addPoint(double x, double y)		{ points.add(new Point2D(x,y)); }
		
		private		boolean			isVisible				= false;
		public		boolean			getVisible()			{ return isVisible; }
		public		void			setVisible(boolean b)	{ isVisible = b; }
		
		protected	Point				yLabelExtent;
		public		Point				getYLabelExtent()	{ return yLabelExtent; }
		protected	ArrayList<Double> 	yMajorTicks			= null;
		protected	ArrayList<Double> 	yMinorTicks			= null;
		public		ArrayList<Double> 	getYMajorTicks()	{ return yMajorTicks; }
		public		ArrayList<Double> 	getYMinorTicks()	{ return yMajorTicks; }
		
		protected	YRange			reqYRange				= new YRange();
		public		YRange			getReqYRange()			{ return reqYRange; }
		
		protected	YRange			plotYRange				= new YRange();
		public		YRange			getPlotYRange()			{ return plotYRange; }
	
		protected	int				yAxisSide				= SWT.NONE;
		public		int				getYAxisSide()			{ return yAxisSide; }
		public		void			setYAxisSide(int side)	{ yAxisSide = side; }
		
		protected	String			yLabel					= "";
		public		String			getYLabel()				{ return yLabel; }
		public		void			setYLabel(String text)	{ if (null != text) yLabel = text; }
		
		public Point getScreenCoord(double graphX, double graphY)
		{
			Integer x = getScreenCoordX(graphX);
			Integer y = getScreenCoordY(graphY);
			if (null == x || null == y) return null;
			return new Point(x,y);
		}
		
		public Point2D getGraphCoord(int screenX, int screenY)
		{
			Double x = getGraphCoordX(screenX, false);
			Double y = getGraphCoordY(screenY, false);
			if (null == x || null == y) return null;
			return new Point2D(x,y);
		}
		
		public Integer getScreenCoordY(double graphY)
		{
			 if (null == graphBounds) return null;		
			 double percentY = (plotYRange.yMax-graphY)/(plotYRange.yMax-plotYRange.yMin);
			 int y = (int)(percentY * graphBounds.height);
			 return y;
		}
		
		public Double getGraphCoordY(int screenY, boolean returnOOB)
		{
			if (!returnOOB && (screenY < 0 || screenY > yScaleBoundsLeft.height+1)) return null;
			double percentY = (double)(screenY)/(double)yScaleBoundsLeft.height;
			double y = (1.0-percentY)*(plotYRange.yMax-plotYRange.yMin) + plotYRange.yMin;
			return y;
		}
		
		public void setYRange(double min, double max)
		{
			reqYRange.yMin=min;							reqYRange.yMax=max;
			plotYRange.yMin=validateRange(min,max).a;	plotYRange.yMax=validateRange(min,max).b;
			recomputeYTicks();
			listenerList.notifyListeners();
		}
		
		private int axisLabelDecimals = 0;
		public int getYAxisLabelDecimals() { return axisLabelDecimals; }
		
		protected void recomputeYTicks()
		{
			Pair<ArrayList<Double>,Integer> ret = looseLabel(reqYRange.yMin, reqYRange.yMax, 5);
			yMajorTicks = ret.a;
			axisLabelDecimals = ret.b;
			
			yMinorTicks = new ArrayList<Double>();
		}
	}
	
	protected	HashMap<TRACE_ID,Trace>	traceMap								= new HashMap<TRACE_ID,Trace>();
	public		Trace					getTrace(TRACE_ID id)					{ return traceMap.get(id); }
	public		Collection<Trace>		getTraces()								{ return traceMap.values(); }
	public 		void					setTrace(TRACE_ID id, Trace trace) 		{ trace.id=id; traceMap.put(id, trace);}

	public Trace getTrace(int side)
	{
		if (SWT.LEFT == side) return getTrace(ModelScan.get().getLeftAxisTrace());
		if (SWT.RIGHT == side) return getTrace(ModelScan.get().getRightAxisTrace());
		return null;
	}
	
	/////////////////////////////////////// non-calculated variables ////////////////////////////////////

	public static class XRange { public double xMin, xMax; public XRange(){}; public XRange(XRange xr) { if (null!=xr) { this.xMin=xr.xMin; this.xMax=xr.xMax; } } }
	public static class YRange { public double yMin, yMax; public YRange(){}; public YRange(YRange yr) { if (null!=yr) { this.yMin=yr.yMin; this.yMax=yr.yMax; } } }
	
	public static class Bar
	{
		public String name;
		public double x;
		public Color color;
		public String tclColor;
		public Bar(String name, double x, Color color, String tclColor) { this.name=name; this.x=x; this.color=color; this.tclColor=tclColor; }
		@Override public String toString() { return name+"@"+x; }
	}

	// when the related settings variables change, these bars are updated via the device
	// when a user drags the bars in the GUI, they are updated directly from the widget
	public		Bar					barROILow							= new Bar("ROILow", 	Double.NaN, GUI.COLOR_RED, "red");
	public		Bar					barROIHigh							= new Bar("ROIHigh", 	Double.NaN, GUI.COLOR_DARK_RED3, "brown");
	public		Bar					barInflection						= new Bar("Inflection", Double.NaN, GUI.COLOR_ORANGE, "orange");
	public		Bar					barPeak								= new Bar("Peak", 		Double.NaN, GUI.COLOR_DARK_GREEN, "darkgreen");
	public		Bar					barRemote							= new Bar("Remote",		Double.NaN, GUI.COLOR_BLUE, "blue");

	public	ArrayList<Bar>		bars								= new ArrayList<Bar>();
	public		ArrayList<Bar>		getBars()							{ return bars; }
	
	public		WdgGraph			graphWidget							= null;
	public		void				setWidget(WdgGraph widget)			{ this.graphWidget = widget; }
	
	protected	boolean				enabled								= false;
	public		boolean				getEnabled()						{ return enabled; }
	public		void				setEnabled(boolean b)				{ enabled=b; }
	
	protected	String				title								= "";
	public		String				getTitle()							{ return title; }
	public 		void				setTitle(String title)				{ if (null != title)  this.title = title; }
	
	protected boolean[] tempDisplay = new boolean[EDGE_IND.values().length];
	protected boolean[] permDisplay = new boolean[EDGE_IND.values().length];
	public EI_STATE getElementDisplayState(EDGE_IND ind)
	{
		if (permDisplay[ind.ordinal()]) return EI_STATE.ON;
		if (tempDisplay[ind.ordinal()]) return EI_STATE.TEMP;
		return EI_STATE.OFF;
	}
	public void setElementPermState(EDGE_IND edge, boolean showPerm)
	{
		permDisplay[edge.ordinal()] = showPerm;
	}
	public void setElementTempState(EDGE_IND edge, boolean showTemp)
	{
		tempDisplay[edge.ordinal()] = showTemp;
		redraw(false, false, false, false, false, false);
	}
	
	public void setXRange(double min, double max)
	{
		reqXRange.xMin=min;							reqXRange.xMax=max;
		plotXRange.xMin=validateRange(min,max).a;	plotXRange.xMax=validateRange(min,max).b;
		recomputeXTicks();
		listenerList.notifyListeners();
	}
	
	protected	String				xLabel								= "";
	public		String				getXLabel()							{ return xLabel; }
	public		void				setXLabel(String text)				{ if (null != text) xLabel = text; }
	
	protected	XRange			reqXRange							= new XRange();
	public		XRange			getReqXRange()						{ return reqXRange; }
	
	protected	String		message						= null;
	public		String		getMessage()				{ return message; }
	public		void		setMessage(String s)		{ message = s; }
	
	protected	Point		totalSize						= new Point(1,1);
	public		Point		getTotalSize()					{ return totalSize; }
	
	protected	Point		buttonSize						= new Point(130,110);
	public		Point		getButtonSize()					{ return buttonSize; }
	
	///////////////////////////////////////// calculated variables //////////////////////////////////////
	
	protected	Point		buttonPosition				= new Point(0,0);
	public		Point		getButtonPosition()			{ return buttonPosition; }
	
	protected	Point		xLabelExtent;
	public		Point		getXLabelExtent()			{ return xLabelExtent; }
	
	protected	int			xScaleHeight;
	public		int			getXScaleHeight()			{ return xScaleHeight; }
	protected	int			yScaleWidth;
	public		int			getYScaleWidth()			{ return yScaleWidth; }
	
	protected	int			titleHeight					= 0;
	
	protected	XRange		plotXRange					= new XRange();
	public		XRange		getPlotXRange()				{ return plotXRange; }
	
	protected	boolean		drawGuideLines				= true;
	public		boolean		getDrawGuideLines()			{ return drawGuideLines; }
	
	protected	Rectangle	xScaleBounds				= null;
	public		Rectangle	getXScaleBounds()			{ return xScaleBounds; }
	
	protected	Rectangle	yScaleBoundsLeft			= null;
	public		Rectangle	getLeftYScaleBounds()		{ return yScaleBoundsLeft; }
	
	protected	Rectangle	yScaleBoundsRight			= null;
	public		Rectangle	getRightYScaleBounds()		{ return yScaleBoundsRight; }
	
	protected	Rectangle	graphBounds					= null; // plotting area
	public		Rectangle	getGraphBounds()			{ return graphBounds; }
	
	protected	ArrayList<Double> xMajorTicks			= null;
	protected	ArrayList<Double> xMinorTicks			= null;
	public		ArrayList<Double> getXMajorTicks()		{ return xMajorTicks; }
	public		ArrayList<Double> getXMinorTicks()		{ return xMinorTicks; }

	/////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////// methods ////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected static ModelGraph inst;
	public static ModelGraph get() { return inst; }
	
	public ModelGraph()
	{
		if (null != inst) { Log.error("ModelGraph already created."); }
		inst=this;
		
		listenerList = new ListenerList<ModelGraph>(this);
		
		for (TRACE_ID id : TRACE_ID.values())
		{
			setTrace(id, new Trace());
		}
		
		bars.add(barROILow);
		bars.add(barROIHigh);
		bars.add(barInflection);
		bars.add(barPeak);
		bars.add(barRemote);
	}
	
	public Pair<Double,Double> validateRange(double min, double max)
	{
		if (min>max) { double temp=min; min=max; max=temp; }
		if (max-min<MIN_RANGE) { double mid = (min+max)/2.0; min=mid-MIN_RANGE/2.0; max=mid+MIN_RANGE/2.0; }
		
		double range = max-min;
		double vMin = min - PLOT_MARGIN * range;
		double vMax = max + PLOT_MARGIN * range;
		
		return new Pair<Double,Double>(vMin,vMax);
	}
	
	public void recalc(GC gc, Point totalSize)
	{
		this.totalSize = totalSize;
		
		// axis labels
		gc.setFont(GUI.FONT_SCAN_AXIS_HORIZONTAL);
		xLabelExtent = gc.textExtent(xLabel);
		
		xScaleHeight = 30;
		yScaleWidth = 65;
		
		Trace leftAxisTrace = getTrace(SWT.LEFT), rightAxisTrace = getTrace(SWT.RIGHT);
		for (Trace trace : traceMap.values())
		{
			trace.yLabelExtent = verticalTextExtent(gc, trace.yLabel);
		}
		int leftScaleWidth = null==leftAxisTrace?0:leftAxisTrace.yLabelExtent.x + SPACING + yScaleWidth + SPACING;
		int rightScaleWidth = null==rightAxisTrace?0:rightAxisTrace.yLabelExtent.x + SPACING + yScaleWidth + SPACING;
		
		// title height
		gc.setFont(GUI.FONT_SCAN_TITLE);
		titleHeight = gc.textExtent(title).y+MARGIN+SPACING;
		
		// graph area
		int graphX = MARGIN + leftScaleWidth;
		int graphY = titleHeight;
		graphBounds = new Rectangle
		(
			graphX,
			graphY,
			totalSize.x - graphX - MARGIN - rightScaleWidth - SPACING,
			totalSize.y - graphY - MARGIN - xLabelExtent.y - SPACING - xScaleHeight - SPACING
		);
		
		if (graphBounds.width <= 0 || graphBounds.height <= 0) return;

		// scales
		xScaleBounds = new Rectangle
		(
			graphBounds.x,
			graphBounds.y + graphBounds.height + SPACING,
			graphBounds.width+1,
			xScaleHeight
		);
		
		if (null == leftAxisTrace) yScaleBoundsLeft = null;
		else
		{
			yScaleBoundsLeft = new Rectangle
			(
				MARGIN + leftAxisTrace.yLabelExtent.x + SPACING,
				graphBounds.y,
				yScaleWidth,
				graphBounds.height+1
			);
		}
		
		if (null == rightAxisTrace) yScaleBoundsRight = null;
		else
		{
			yScaleBoundsRight = new Rectangle
			(
				graphBounds.x + graphBounds.width + SPACING,
				graphBounds.y,
				yScaleWidth,
				graphBounds.height+1
			);
		}
		
		// ticks
		recomputeXTicks();
		for (Trace trace : traceMap.values())
		{
			trace.recomputeYTicks();
		}
		
		// buttons
		buttonPosition = new Point(totalSize.x - buttonSize.x, totalSize.y - buttonSize.y);
	}
	
	// calculate tick locations
	protected static double[] majorMultipliers = { -2.5, -1.5, -.5, .5, 1.5, 2.5 };
	protected static double[] minorMultipliers = { -2, -1, 0, 1, 2 };
	
	protected void recomputeXTicks()
	{
		xMajorTicks = looseLabel(reqXRange.xMin, reqXRange.xMax, 10).a;
		xMinorTicks = new ArrayList<Double>();//computeTicks(minorMultipliers, xRange, xStart);
	}
	
	protected Point verticalTextExtent(GC gc, String s)
	{
		Point horizontalExtent = gc.textExtent(s);
		return new Point(horizontalExtent.y, horizontalExtent.x);
	}
	
	public Integer getScreenCoordX(double graphX)
	{
		if (null == graphBounds) return null;
		int x = (int)( ((graphX-plotXRange.xMin)/(plotXRange.xMax-plotXRange.xMin))*graphBounds.width );
		return x;
	}
	
	public Double getGraphCoordX(int screenX, boolean returnOOB)
	{
		if (!returnOOB && (screenX < 0 || screenX > xScaleBounds.width+1)) return null;
		double percentX = (double)(screenX)/(double)xScaleBounds.width;
		double x = percentX*(plotXRange.xMax-plotXRange.xMin) + plotXRange.xMin;
		return x;
	}
	
	public void redraw(final boolean redrawLabels, final boolean redrawXAxis, final boolean redrawYAxis, final boolean redrawPlot, final boolean recalc, final boolean drawDirect)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (recalc)
			{
				GC graphGC = new GC(graphWidget);
				recalc(graphGC, graphWidget.getSize());
				graphGC.dispose();
			}
			
			if (redrawLabels)	graphWidget.layerLabels.update();
			if (redrawXAxis) 	graphWidget.layerAxisX.update();
			if (redrawYAxis)	{ graphWidget.layerAxisYLeft.update(); if (null != getTrace(SWT.RIGHT)) graphWidget.layerAxisYRight.update(); }
			if (redrawPlot)
			{
				if (drawDirect)
				{
					if (null != graphWidget.buffer)
					{
						GC imgGC = new GC(graphWidget.buffer);
						graphWidget.layerPlot.drawDirect(imgGC);
						imgGC.dispose();
					}
				}
				else
				{
					graphWidget.layerPlot.update();
				}
			}

			if (!drawDirect) graphWidget.composite();
		}});
	}
	
	// From Paul Heckbert's "Nice Numbers for Graph Labels", Graphics Gems, pp 61-63
	// returns label x values and number of decimals to show
	public static Pair<ArrayList<Double>,Integer> looseLabel(double min, double max, double ntick)
	{
		double range = niceNumber(max-min, false);
		double d = niceNumber(range/(ntick-1),true);
		double graphMin = Math.floor(min/d)*d;
		double graphMax = Math.ceil(max/d)*d;
		int nfrac = Math.max(-1*(int)Math.floor(Math.log10(d)), 0);
		
		ArrayList<Double> ticks = new ArrayList<Double>();
		for (double x = graphMin; x <= graphMax + 0.5*d; x += d)
		{
			ticks.add(x);
		}
		return new Pair<ArrayList<Double>,Integer>(ticks, nfrac);
	}
	
	private static double niceNumber(double x, boolean round)
	{
		int exp = (int)Math.floor(Math.log10(x));
		double f = x / Math.pow(10.0, exp);
		
		double nf;
		
		if (round)
		{
			if 		(f<1.5) nf=1;
			else if (f<3.0) nf=2;
			else if (f<7.0) nf=5;
			else 			nf=10;
		}
		else
		{
			if 		(f<=1.0) nf=1;
			else if (f<=2.0) nf=2;
			else if (f<=5.0) nf=5;
			else 			 nf=10;
		}
		
		return nf * Math.pow(10.0, exp);
	}
	
	public Double getTraceValueAt(TRACE_ID traceID, double graphX)
	{
		ArrayList<Point2D> points = getTrace(traceID).getPoints();
		if (points.size()<2) return null;
		if (graphX < points.get(0).x) return null;
		if (graphX > points.get(points.size()-1).x) return null;
		
		double lastX = points.get(0).x;
		for (int i=1; i<points.size();i++)
		{
			double tx = points.get(i).x;
			if (lastX <= graphX && graphX <= tx)
			{
				double percent = (graphX-lastX)/(tx-lastX);
				double ret = (1.0-percent)*points.get(i-1).y + percent*points.get(i).y;
				return ret;
			}
			lastX = tx;
		}
		return null; // shouldn't get here
	}
}
