package gov.anl.gmca.scan.ui;

import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.ModelInterface;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgChoochSelect extends Composite
{
	private Button btnChoochClear, btnChoochDefault;
	private WdgControlEntry entryInflection, entryPeak, entryRemote;
	
	public WdgChoochSelect(Composite parent)
	{
		super(parent, SWT.NONE);
		
		setLayout(new FillLayout());
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		setLayoutData(gd);
		
		Group group = new Group(this, SWT.BORDER);
		group.setBackground(getBackground());
		GridLayout gl = new GridLayout(15,false);
		gl.horizontalSpacing = 0;
		group.setLayout(gl);
		
		btnChoochClear = new Button(group,SWT.PUSH);
		btnChoochClear.setText("Clear");
		btnChoochClear.setFont(GUI.FONT_SCAN_BUTTON);
		btnChoochClear.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		//gd.widthHint = 65;
		btnChoochClear.setLayoutData(gd);
		
		btnChoochClear.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				DevScan.get().setInflection(Double.NaN, false);
				DevScan.get().setPeak(Double.NaN, false);
				DevScan.get().setRemote(Double.NaN, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		btnChoochDefault = new Button(group,SWT.PUSH);
		btnChoochDefault.setText("Default");
		btnChoochDefault.setToolTipText("Reset inflection, peak, and remote values to those returned by chooch");
		btnChoochDefault.setFont(GUI.FONT_SCAN_BUTTON);
		btnChoochDefault.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		//gd.widthHint = 80;
		btnChoochDefault.setLayoutData(gd);
		
		btnChoochDefault.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				DevScan.get().setInflection(ModelScan.get().getDefaultInflection(), false);
				DevScan.get().setPeak(ModelScan.get().getDefaultPeak(), false);
				DevScan.get().setRemote(ModelScan.get().getDefaultRemote(), true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		entryInflection 	= WdgPlotControls.newEntry(group, "Inflection: ", 	"eV",	GUI.COLOR_ORANGE, new ModelInterface()
		{
			@Override public void 	setValue(Object value)	{ DevScan.get().setInflection((Double)value, true); }
			@Override public int 	getDecimals() 			{ return 2; }
			@Override public String formatValue(Object o)	{ if (o instanceof Double && Double.isNaN((Double)o)) return "unset"; return super.formatValue(o); }
		});
		entryPeak 			= WdgPlotControls.newEntry(group, "Peak: ", 			"eV",	GUI.COLOR_DARK_GREEN, new ModelInterface()
		{
			@Override public void 	setValue(Object value)	{ DevScan.get().setPeak((Double)value, true); }
			@Override public int 	getDecimals() 			{ return 2; }
			@Override public String formatValue(Object o)	{ if (o instanceof Double && Double.isNaN((Double)o)) return "unset"; return super.formatValue(o); }
		});
		entryRemote			= WdgPlotControls.newEntry(group, "Remote: ", 		"eV",	GUI.COLOR_BLUE, new ModelInterface()
		{
			@Override public void 	setValue(Object value)	{ DevScan.get().setRemote((Double)value, true); }
			@Override public int 	getDecimals() 			{ return 2; }
			@Override public String formatValue(Object o)	{ if (o instanceof Double && Double.isNaN((Double)o)) return "unset"; return super.formatValue(o); }
		});
		WdgPlotControls.newSpacer(group);
	}
	
	public void updateFields()
	{
		entryInflection.setText	(ModelScan.get().getInflection());
		entryPeak.setText		(ModelScan.get().getPeak());
		entryRemote.setText		(ModelScan.get().getRemote());
	}
	
	public void updateState(boolean isMaster, boolean scanInProgress)
	{
		btnChoochClear.setEnabled(isMaster);
		btnChoochDefault.setEnabled(isMaster);
		
		if (!entryInflection.getText().equals("unset"))	entryInflection.setEnabled(isMaster);	else entryInflection.setEnabled(false);
		if (!entryPeak.getText().equals("unset"))		entryPeak.setEnabled(isMaster);			else entryPeak.setEnabled(false);
		if (!entryRemote.getText().equals("unset"))		entryRemote.setEnabled(isMaster);		else entryRemote.setEnabled(false);
	}
}
