package gov.anl.gmca.scan.ui;

import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.DevEdgeDB;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.*;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Pair;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

public class WdgEdgeIndicators extends Composite
{
	private HashMap<EDGE_IND,Label> labels = new HashMap<EDGE_IND,Label>();
	private Combo combo, combo2;
	private Button allCheckbox, otherCheckbox, other2Checkbox;
	
	public WdgEdgeIndicators(Composite parent)
	{
		super(parent, SWT.NONE);
		
		createWidgets();
		connectListeners();
	}
	
	private void createWidgets()
	{
		setLayout(new FillLayout());
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		setLayoutData(gd);
		
		Group group = new Group(this, SWT.BORDER);
		group.setBackground(getBackground());
		GridLayout gl = new GridLayout(EDGE_IND.values().length+8,false); // +1 all checkbox, +2 other checkboxes, +5 fillers
		gl.horizontalSpacing=0;
		group.setLayout(gl);
		
		Composite leftFill = new Composite(group, SWT.NONE);
		leftFill.setBackground(getBackground());
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint=1;
		leftFill.setLayoutData(gd);
		
		allCheckbox = new Button(group, SWT.CHECK);
		allCheckbox.setText("All");
		allCheckbox.setToolTipText("Show edges for all elements");
		
		Composite fillComp = new Composite(group, SWT.NONE);
		fillComp.setBackground(getBackground());
		gd = new GridData();
		gd.heightHint=1;
		gd.widthHint=17;
		fillComp.setLayoutData(gd);
		
		for (final EDGE_IND element : EDGE_IND.values())
		{
			if (element==EDGE_IND.Other||element==EDGE_IND.Other2) continue; // no label for this one
			
			Label label = new Label(group, SWT.BORDER);
			label.setText(" "+element.toString()+" ");
			label.setBackground(GUI.COLOR_WHITE);
			setVisibility(element, label);
			labels.put(element, label);
		}
		
		Pair<Button,Combo> pair1 = createOtherCombo(group);
		otherCheckbox = pair1.a; combo = pair1.b;
		
		Pair<Button,Combo> pair2 = createOtherCombo(group);
		other2Checkbox = pair2.a; combo2 = pair2.b;
		
		Composite rightFill = new Composite(group, SWT.NONE);
		rightFill.setBackground(getBackground());
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint=1;
		rightFill.setLayoutData(gd);
	}
	
	private Pair<Button,Combo> createOtherCombo(Composite parent)
	{
		GridData gd;
		
		Composite comp = new Composite(parent, SWT.NONE);
		gd = new GridData();
		gd.widthHint=10;
		gd.heightHint=1;
		comp.setLayoutData(gd);
		
		Button button = new Button(parent, SWT.CHECK);
		button.setText("");
		gd = new GridData();
		gd.widthHint = 18;
		button.setLayoutData(gd);
		
		Combo combo = new Combo(parent, SWT.READ_ONLY|SWT.DROP_DOWN);
		combo.add("none");
		
		// find elements that don't have their own label
		int maxElements = 1000, elementIndex=0;
		HashSet<String> unorderedElementStrings = new HashSet<String>();
		for (String element : DevEdgeDB.get().getAllElements(true))
		{
			boolean alreadyShown=false;
			for (EDGE_IND ei : EDGE_IND.values())
			{
				if (ei.toString().toLowerCase().equals(element.toLowerCase()))
				{
					alreadyShown=true;
				}
			}
			if (!alreadyShown && elementIndex <= maxElements)
			{
				unorderedElementStrings.add(element);
				elementIndex++;
			}
		}
		
		// order them
		ArrayList<String> orderedElementStrings = new ArrayList<String>();
		for (int i=1; i<=103; i++)
		{
			String elemString = WdgPeriodicTable.get().getElement(i);
			if (unorderedElementStrings.contains(elemString)) orderedElementStrings.add(elemString);
		}
		
		// put them in the combo
		for (String elementString : orderedElementStrings)
		{
			combo.add(elementString);
		}
		
		combo.select(0);
		gd = new GridData(SWT.LEFT, SWT.DEFAULT, false, false);
		combo.setLayoutData(gd);
		
		return new Pair<Button,Combo>(button,combo);
	}
	
	private void setVisibility(EDGE_IND element, Label label)
	{
		boolean showElement = DevEdgeDB.get().getAllElements(true).contains(element.toString());
		if (!showElement)
		{
			System.out.println("Hiding element indicator "+element.toString()+" since graphDisplay is set to zero for all edges");
			WdgControl.setControlVisible(label, false);
		}
	}
	
	private void connectListeners()
	{
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateState();
				}});
			}
		});
		
		ModelScan.get().addListener(new ScanListener()
		{
			@Override public void scanChanged(ModelScan settings)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateFields();
					updateState();
				}});
			}
		});
		
		ModelScan.get().llDisplayedElements.addListener(new ListenerList.Listener<ModelScan>()
		{
			@Override public void handleEvent(ModelScan object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateFields();
					updateState();
				}});
			}
		});
		
		allCheckbox.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				DevScan.get().setShowAllEdges(allCheckbox.getSelection(), true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		for (final EDGE_IND element : EDGE_IND.values())
		{
			final Label label = labels.get(element);
			if (null==label) continue;
			
			label.addMouseTrackListener(new MouseTrackListener()
			{
				@Override public void mouseEnter(MouseEvent e)
				{
					boolean isOn = EI_STATE.ON == ModelScan.get().getDisplayedElements().get(element);
					if (!isOn)
					{
						setLabelState(label, EI_STATE.TEMP);
						ModelGraph.get().setElementTempState(element, true);
					}
				}
				@Override public void mouseExit(MouseEvent e)
				{
					boolean isOn = EI_STATE.ON == ModelScan.get().getDisplayedElements().get(element);
					setLabelState(label, isOn ? EI_STATE.ON : EI_STATE.OFF);
					ModelGraph.get().setElementTempState(element, false);
				}
				@Override public void mouseHover(MouseEvent e) {}
			});
			label.addMouseListener(new MouseListener()
			{
				@Override public void mouseDoubleClick(MouseEvent e) {}
				@Override public void mouseUp(MouseEvent e) {}
				@Override public void mouseDown(MouseEvent e)
				{
					EI_STATE currentState = ModelScan.get().getDisplayedElements().get(element);
					EI_STATE newState = null;
					switch (currentState)
					{
					case OFF:	newState = EI_STATE.ON; break;
					case TEMP:	newState = EI_STATE.ON; break;
					case ON:	newState = EI_STATE.OFF; break;
					}
					DevScan.get().setElementDisplayState(element, newState, false);
				}
			});
		}
		
		otherCheckbox.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handleCheckboxChange(otherCheckbox.getSelection(), combo, EDGE_IND.Other);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		other2Checkbox.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handleCheckboxChange(other2Checkbox.getSelection(), combo2, EDGE_IND.Other2);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		combo.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handleComboChange(combo, EDGE_IND.Other);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		combo2.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handleComboChange(combo2, EDGE_IND.Other2);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	}
	
	private void handleComboChange(Combo combo, EDGE_IND ind)
	{
		// set the dropdown element
		switch (ind)
		{
		case Other:  DevScan.get().setOtherElement(combo.getText(), false);		break;
		case Other2: DevScan.get().setOtherElement2(combo.getText(), false);	break;
		default:	throw new IllegalArgumentException();
		}
		
		// if a new element was selected, show it
		boolean elementSelected = (0!=combo.getSelectionIndex());
		EI_STATE state = elementSelected ? EI_STATE.ON : EI_STATE.OFF;
		DevScan.get().setElementDisplayState(ind, state, false);
	}
	
	private void handleCheckboxChange(boolean checked, Combo combo, EDGE_IND ind)
	{
		EI_STATE state = checked ? EI_STATE.ON : EI_STATE.OFF;
		if (combo.getSelectionIndex()==0) state = EI_STATE.OFF;
		DevScan.get().setElementDisplayState(ind, state, false);
	}
	
	public void updateFields()
	{
		allCheckbox.setSelection(ModelScan.get().getShowAllEdges());
		
		for (EDGE_IND elem : EDGE_IND.values())
		{
			if (elem == EDGE_IND.Other)
			{
				otherCheckbox.setSelection(ModelScan.get().getDisplayedElements().get(elem)!=EI_STATE.OFF);
				selectComboElement(combo, ModelScan.get().getOtherElement());
			}
			else if (elem == EDGE_IND.Other2)
			{
				other2Checkbox.setSelection(ModelScan.get().getDisplayedElements().get(elem)!=EI_STATE.OFF);
				selectComboElement(combo2, ModelScan.get().getOtherElement2());
			}
			else
			{
				setLabelState(labels.get(elem),ModelScan.get().getDisplayedElements().get(elem));
			}
		}
	}
	
	private void setLabelState(Label label, EI_STATE state)
	{
		switch (state)
		{
		case OFF:
			label.setBackground(GUI.COLOR_WHITE);
			label.setForeground(GUI.COLOR_BLACK);
			break;
		case TEMP:
			label.setBackground(GUI.COLOR_TEAL);
			label.setForeground(GUI.COLOR_BLACK);
			break;
		case ON:
			label.setBackground(GUI.COLOR_BROWN);
			label.setForeground(GUI.COLOR_WHITE);
			break;
		}
	}
	
	private void selectComboElement(Combo combo, String element)
	{
		int index=0;
		String[] items = combo.getItems();
		for (int i=0; i<items.length; i++)
		{
			if (items[i].equals(element)) index = i;
		}
		combo.select(index);
	}
	
	public void updateState()
	{
		boolean isMaster = Client.isMaster();
		boolean scanInProgress = 
    			(ModelScan.get().getAutoStep() != AutoStep.NONE ||
    			 ModelScan.get().getInterStep() != InterStep.IDLE);
		boolean gEnable = isMaster && !scanInProgress;
		boolean enableOtherControls = gEnable && !ModelScan.get().getShowAllEdges();
		
		for (Label label : labels.values())
		{
			label.setEnabled(enableOtherControls);
		}
		otherCheckbox.setEnabled(enableOtherControls);
		other2Checkbox.setEnabled(enableOtherControls);
		combo.setEnabled(enableOtherControls);
		combo2.setEnabled(enableOtherControls);
		
		allCheckbox.setEnabled(gEnable);
	}
}
