package gov.anl.gmca.scan.ui;

import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.TRACE_ID;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

public class WdgPlotSelect extends Composite
{
	private Button btnEdge, btnChooch, btnBoth;
	
	public WdgPlotSelect(Composite parent)
	{
		super(parent, SWT.NONE);
		
		setLayout(new FillLayout());
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		setLayoutData(gd);
		
		Group group = new Group(this, SWT.BORDER);
		group.setBackground(getBackground());
		group.setLayout(new GridLayout(4,false));
		
		Label plotSelectLabel = new Label(group, SWT.NONE);
		plotSelectLabel.setBackground(getBackground());
		plotSelectLabel.setText("Plot select: ");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		plotSelectLabel.setLayoutData(gd);
		
		btnEdge = new Button(group, SWT.RADIO);
		btnEdge.setBackground(getBackground());
		btnEdge.setText("Raw scan");
		btnEdge.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		btnEdge.setSelection(true);
		btnEdge.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				showTraces(new TRACE_ID[]{TRACE_ID.edge}, TRACE_ID.edge, null, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		btnChooch = new Button(group, SWT.RADIO);
		btnChooch.setBackground(getBackground());
		btnChooch.setText("f' and f\"");
		btnChooch.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		btnChooch.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				showTraces(new TRACE_ID[]{TRACE_ID.fPrime, TRACE_ID.fDoublePrime}, TRACE_ID.fPrime, TRACE_ID.fDoublePrime, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});

		btnBoth = new Button(group, SWT.RADIO);
		btnBoth.setBackground(getBackground());
		btnBoth.setText("All plots");
		btnBoth.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		btnBoth.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				showTraces(new TRACE_ID[]{TRACE_ID.edge, TRACE_ID.fPrime, TRACE_ID.fDoublePrime}, TRACE_ID.fPrime, TRACE_ID.fDoublePrime, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
	}
	
	public void updateFields()
	{
		boolean edgeVisible = ModelScan.get().getTraceVisible(TRACE_ID.edge);
		boolean fP_Visible 	= ModelScan.get().getTraceVisible(TRACE_ID.fPrime);
		boolean fPP_Visible = ModelScan.get().getTraceVisible(TRACE_ID.fDoublePrime);
		
		if 		( edgeVisible && !fP_Visible && !fPP_Visible)	{ btnEdge.setSelection(true);  btnChooch.setSelection(false); btnBoth.setSelection(false); }
		else if (!edgeVisible &&  fP_Visible &&  fPP_Visible)	{ btnEdge.setSelection(false); btnChooch.setSelection(true);  btnBoth.setSelection(false); }
		else if ( edgeVisible &&  fP_Visible &&  fPP_Visible)	{ btnEdge.setSelection(false); btnChooch.setSelection(false); btnBoth.setSelection(true);  }
	}
	
	public void updateState(boolean isMaster, boolean scanInProgress)
	{
		btnEdge.setEnabled		(isMaster && !scanInProgress);
		btnChooch.setEnabled	(isMaster && !scanInProgress);
		btnBoth.setEnabled		(isMaster && !scanInProgress);
	}
	
	public static void showTraces(TRACE_ID[] tracesToShow, TRACE_ID leftAxisTrace, TRACE_ID rightAxisTrace, boolean update)
	{
		for (TRACE_ID id : TRACE_ID.values())
		{
			boolean show = false;
			for (TRACE_ID iter : tracesToShow)
			{
				if (iter == id) show = true;
			}
			DevScan.get().setTraceVisible(id, show, false);
		}
		
		DevScan.get().setLeftAxisTrace(leftAxisTrace, false);
		DevScan.get().setRightAxisTrace(rightAxisTrace, update);
	}
}
