package gov.anl.gmca.scan.ui;

import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.ModelInterface;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.syslog.Log;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgROISelect extends Composite
{
	private WdgControlEntry entryROILow, entryROIHigh;
	private Button btnROIDefault;
	
	public WdgROISelect(Composite parent)
	{
		super(parent, SWT.NONE);
		
		setLayout(new FillLayout());
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		setLayoutData(gd);
		
		Group group = new Group(this, SWT.BORDER);
		group.setBackground(getBackground());
		group.setLayout(new GridLayout(10,false));
		
		btnROIDefault = new Button(group,SWT.PUSH);
		btnROIDefault.setText("Defaults");
		btnROIDefault.setFont(GUI.FONT_SCAN_BUTTON);
		btnROIDefault.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		gd.widthHint = 80;
		btnROIDefault.setLayoutData(gd);
		
		btnROIDefault.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				Log.note("Setting default ROI for "+ModelScan.get().getElementEdge());
				ModelScan.get().setDefaultROI();
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		entryROILow 	= WdgPlotControls.newEntry(group, "ROILow: ", 	"eV",	GUI.COLOR_RED, new ModelInterface()
		{
			@Override public void 	setValue(Object value)	{ DevScan.get().setROILow((Double)value, true); }
			@Override public int 	getDecimals() 			{ return 2; }
			@Override public String formatValue(Object o)	{ if (o instanceof Double && Double.isNaN((Double)o)) return "unset"; return super.formatValue(o); }
		});
		entryROIHigh 	= WdgPlotControls.newEntry(group, "ROIHigh: ", 	"eV",	GUI.COLOR_DARK_RED3, new ModelInterface()
		{
			@Override public void 	setValue(Object value)	{ DevScan.get().setROIHigh((Double)value, true); }
			@Override public int 	getDecimals() 			{ return 2; }
			@Override public String formatValue(Object o)	{ if (o instanceof Double && Double.isNaN((Double)o)) return "unset"; return super.formatValue(o); }
		});
		
		Composite fillComp = new Composite(group, SWT.NONE);
		fillComp.setBackground(this.getBackground());
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint = 1;
		fillComp.setLayoutData(gd);
	}
	
	public void updateFields()
	{
		entryROILow.setText		(ModelScan.get().getROILow());
		entryROIHigh.setText	(ModelScan.get().getROIHigh());
	}
	
	public void updateState(boolean isMaster, boolean scanInProgress)
	{
		btnROIDefault.setEnabled(isMaster);
		if (!entryROILow.getText().equals("unset"))		entryROILow.setEnabled(isMaster);		else entryROILow.setEnabled(false);
		if (!entryROIHigh.getText().equals("unset"))	entryROIHigh.setEnabled(isMaster);		else entryROIHigh.setEnabled(false);
	}
}
