package gov.anl.gmca.scan.ui;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.WdgControlDir;
import gov.anl.gmca.common.ui.WdgControlDir.PrefixDir;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.AutoStep;
import gov.anl.gmca.scan.ModelScan.InterStep;
import gov.anl.gmca.scan.ModelScan.ScanListener;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;

public class WdgScanControls extends Composite implements ModelScan.ScanListener
{
	public static final int CONTROLS_WIDTH = 242;
	public static final double ALLOWED_ENERGY_DEVIATION_EV = 50.0;
	
	protected ModelScan scanModel;
	protected DevScan scanDev;
	protected CTabFolder folder;
	protected Label interState;
	protected Button autoRescanButton, autoCleanupButton, autoStopButton;
	protected Button interStopButton;
	protected WdgControlEntry interSpectrumTimeEntry, interEdgeTimeEntry, autoTimeEntry, autoEdgeEnergyEntry, interEdgeEnergyEntry;
	protected Composite lblProgress1, lblProgress2, lblProgress3, lblProgress4, lblProgress5; 
	protected WdgControlDir interPrefixDirSelector, autoPrefixDirSelector;

	protected ArrayList<Control> controlList = new ArrayList<Control>();
	
	public WdgScanControls(Composite parent, int style, ModelScan scanModel, DevScan scanDev)
	{
		super(parent,style);
		this.scanModel=scanModel;
		this.scanDev=scanDev;

		scanModel.addListener(this);
		
		AppState.addListener(Operation.START_SCAN, new AppState.Listener()
		{
			@Override public void stateChanged(Operation op, boolean allowed)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateEnableState();
				}});
			}
		});
		
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		setLayout(gl);
		
		folder = new CTabFolder(this, SWT.TOP|SWT.BORDER);
		folder.setBackground(GUI.BGCOLOR);
		folder.setBackgroundMode(SWT.INHERIT_FORCE);
		folder.setFont(GUI.FONT_SCAN_TAB);
		folder.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		CTabItem tabAuto = new CTabItem(folder, SWT.NONE);
		tabAuto.setText("  Auto  ");
		
		Composite autoComp = new Composite(folder, SWT.NONE);
		gl = new GridLayout(1,false);
		gl.marginHeight = 15;
		gl.marginWidth = 5;
		gl.verticalSpacing = 15;
		autoComp.setLayout(gl);
		
		createAutoTab(autoComp);
		
   		tabAuto.setControl(autoComp);
    	
    	CTabItem tabInter = new CTabItem(folder, SWT.NONE);
		tabInter.setText("  Interactive  ");

		Composite interComp = new Composite(folder, SWT.NONE);
		gl = new GridLayout(1,false);
		gl.marginHeight = 15;
		gl.marginWidth = 5;
		gl.verticalSpacing = 15;
		interComp.setLayout(gl);
		
		createInterTab(interComp);
    		
    	tabInter.setControl(interComp);
    		
		folder.setSelection(0);
		
		folder.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (!Client.isMaster())
				{
					Log.note("Scan tab changes are not allowed while slave");
					folder.setSelection(ModelScan.get().getLeftTab());
					return;
				}
				if (InterStep.IDLE != ModelScan.get().getInterStep() ||
					AutoStep.NONE != ModelScan.get().getAutoStep())
				{
					Log.note("Scan tab changes are not allowed during scan operations");
					folder.setSelection(ModelScan.get().getLeftTab());
					return;
				}
				DevScan.get().setLeftTab(folder.getSelectionIndex(), true);
			}
		});
		
		ModelScan.get().addListener(new ScanListener()
		{
			@Override public void scanChanged(final ModelScan settings)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					folder.setSelection(settings.getLeftTab());
					updateEnableState();
				}});
			}
		});
		
		updateEnableState();
	}
	
	protected void createAutoTab(Composite parent)
	{
		Group autoGroup1 			= newGroup(parent);
   									  newLabelLeft(autoGroup1, 	"1. ");
   									  newLabelRight(autoGroup1, "Choose Edge --->");
   									  
        Group autoGroup2 			= newGroup(parent);
  								  	  newLabelLeft(autoGroup2,  "2. ");
  								  	  newLabelRight(autoGroup2, "Select filenames");
  								  	  newSpacer(autoGroup2);
  								  
      	autoPrefixDirSelector = new WdgControlDir(autoGroup2, SWT.NONE, null, 0, new WdgControlDir.ModelInterface()
      	{
			@Override public void setValue(Object value)
			{
				WdgControlDir.PrefixDir prefixDir = (WdgControlDir.PrefixDir)value;
				DevScan.get().setPrefix(prefixDir.prefix, false);
				DevScan.get().setDirectory(prefixDir.dir, true);
			}
			@Override public PrefixDir getPrefixDir()
			{
				return new PrefixDir(ModelScan.get().getPrefix(), ModelScan.get().getDirectorySuffix());
			}
		});
      
      	autoPrefixDirSelector.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
      	controlList.add(autoPrefixDirSelector);
    	
    	Group autoGroup3 			= newGroup(parent);
    								  newLabelLeft(autoGroup3, 	"3. ");
    								  newLabelRight(autoGroup3, "Do edge scan");

    	final Button autoButton2a 		= newButton3(autoGroup3, "Start Scan");
    	controlList.add(autoButton2a);
    	autoButton2a.setToolTipText("Record intensity of selected edge fluorescence as a function of X-ray energy");
    	
    	autoButton2a.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				autoButton2a.setFocus();
				scanModel.startAutoScan();
			}
		});
    	
    	autoRescanButton 		= newButton3(autoGroup3, "Rescan");
    	autoRescanButton.setToolTipText("Same as scan, but fluorescence signal optimization is skipped");
    	
    	autoRescanButton.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				autoRescanButton.setFocus();
				scanModel.restartAutoScan();
			}
		});
    	
        newSpacer(autoGroup3);
        
    	autoTimeEntry 				= newEntry(autoGroup3,	"Time:", "s/pt", false, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ scanDev.setEdgeTime((Double)value, true);									}
			@Override public Object getModelValue() 		{ return scanModel.getEdgeTime();										}
			@Override public Object getDeviceValue() 		{ return scanDev.getEdgeTime();											}
			@Override public Limits getLimits() 			{ return new Limits(0.1,100,"exposure time");										}
			@Override public int getDecimals() 				{ return 2; 														}
			@Override public boolean doHighlight(double d)  { return false;														}
		});
    	controlList.add(autoTimeEntry);

    	newSpacer(autoGroup3);
		
    	autoEdgeEnergyEntry 		= newEntry(autoGroup3,	"Center:", "eV", true, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				scanDev.setScanCenter((Double)value, true);
			}
			@Override public Object getModelValue() 		{ return scanModel.getScanCenter();}
			@Override public Object getDeviceValue() 		{ return scanDev.getScanCenter();											}
			@Override public Limits getLimits()
			{
				double defaultEnergy = scanModel.getElementEdge().getEnergy();
				return new Limits
				(
					defaultEnergy-ALLOWED_ENERGY_DEVIATION_EV,
					" is more than "+ALLOWED_ENERGY_DEVIATION_EV+" eV lower than the database value for the "+scanModel.getElementEdge().toString()+" edge, which is "+defaultEnergy,
					defaultEnergy+ALLOWED_ENERGY_DEVIATION_EV,
					" is more than "+ALLOWED_ENERGY_DEVIATION_EV+" eV higher than the database value for the "+scanModel.getElementEdge().toString()+" edge, which is "+defaultEnergy,
					true
				);
			}
			@Override public int getDecimals() 				{return 1;}
			@Override public boolean doHighlight(double d)
			{
				double defaultEnergy = scanModel.getElementEdge().getEnergy();
				return Math.abs(defaultEnergy-d)>0.5;
			}
		});
    	controlList.add(autoEdgeEnergyEntry);
    	
    	new WdgAutoProgress(parent, SWT.NONE);
    	
    	Group autoGroup4			= newGroup(parent);
    								  newLabelLeft(autoGroup4, "4. ");
    	autoCleanupButton			= newButton(autoGroup4, "Done with fluor.");
    	autoCleanupButton.setToolTipText("Retract fluor. detector and restore X-ray beam attenuation in case it was not done after scan");
    	
    	autoCleanupButton.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				scanModel.cleanupAutoScan();
			}
    	});

    	autoStopButton			= newButton2(parent, "Stop",	SWT.BOTTOM, GUI.COLOR_BLUE_BUTTON);
    	autoStopButton.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				scanModel.stopAutoScan();
			}
    	});
	}
	
	protected void createInterTab(Composite parent)
	{
		/*interState					= new Label(parent, SWT.CENTER);
		interState.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		interState.setFont(GUI.FONT_SCAN_STATE);
		interState.setText("Connecting...");*/
		
		Group interGroup1 			= newGroup(parent);
   									  newLabelLeft(interGroup1, 	"1. ");
   									  newLabelRight(interGroup1, "Choose Edge --->");
   									  
    	Group interGroup2 			= newGroup(parent);
    								  newLabelLeft(interGroup2, 	"2. ");
    								  newLabelRight(interGroup2, "Select filenames");
    								  
        newSpacer(interGroup2);
    								  
        interPrefixDirSelector = new WdgControlDir(interGroup2, SWT.NONE, null, 0, new WdgControlDir.ModelInterface()
        {
			@Override public void setValue(Object value)
			{
				WdgControlDir.PrefixDir prefixDir = (WdgControlDir.PrefixDir)value;
				DevScan.get().setPrefix(prefixDir.prefix, false);
				DevScan.get().setDirectory(prefixDir.dir, true);
			}
			@Override public PrefixDir getPrefixDir()
			{
				return new PrefixDir(ModelScan.get().getPrefix(), ModelScan.get().getDirectorySuffix());
			}
		});
        
        interPrefixDirSelector.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
        controlList.add(interPrefixDirSelector);
      	
   		Group interGroup3 			= newGroup(parent);
    								  newLabelLeft(interGroup3, 	"3. ");
    	final Button interButton3 	= newButton(interGroup3,	"Optimize fluor. signal");
    	interButton3.setToolTipText("Bring in fluor. detector and find optimum X-ray beam attenuation");
    	controlList.add(interButton3);
    	
    	interButton3.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				interButton3.setFocus();
				scanModel.startOptimizeDetector();
			}
    	});
    	
    	Group interGroup4 			= newGroup(parent);
    								  newLabelLeft(interGroup4, 	"4. ");
    	final Button interButton4	= newButton(interGroup4,	"Take fluor. spectrum");
    	interButton4.setToolTipText("Check what elements are present in your crystal");
    	controlList.add(interButton4);
    	
    	interButton4.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				interButton4.setFocus();
				scanModel.startSpectrum();
			}
		});
    	
    	newSpacer(interGroup4);
    	
    	interSpectrumTimeEntry		= newEntry(interGroup4,		"Time:", "sec", false, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ scanDev.setSpectrumTime((Double)value, true);								}
			@Override public Object getModelValue() 		{ return scanModel.getSpectrumTime();										}
			@Override public Object getDeviceValue() 		{ return scanDev.getSpectrumTime();											}
			@Override public Limits getLimits() 			{ return new Limits(0.1,100,"exposure time");										}
			@Override public int getDecimals() 				{ return 2; 														}
			@Override public boolean doHighlight(double d)  { return false;														}
		});
    	controlList.add(interSpectrumTimeEntry);

    	Group interGroup5 			= newGroup(parent);
    								  newLabelLeft(interGroup5, 	"5. ");
    	final Button interButton5 	= newButton(interGroup5,	"Optimize MCA");
    	controlList.add(interButton5);
    	
    	interButton5.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				interButton5.setFocus();
				scanModel.startOptimizeMCA();
			}
    	});
    	
    	Group interGroup6 			= newGroup(parent);
    								  newLabelLeft(interGroup6, 	"6. ");
    	final Button interButton6 	= newButton(interGroup6,	"Do edge scan");
    	interButton6.setToolTipText("Record intensity of selected fluor. as a function of X-ray energy");
    	controlList.add(interButton6);
    	
    	newSpacer(interGroup6);
    	
    	interEdgeTimeEntry			= newEntry(interGroup6,		"Time:", "s/pt", false, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ scanDev.setEdgeTime((Double)value, true);									}
			@Override public Object getModelValue() 		{ return scanModel.getEdgeTime();										}
			@Override public Object getDeviceValue() 		{ return scanDev.getEdgeTime();											}
			@Override public Limits getLimits() 			{ return new Limits(0.1,100,"exposure time");										}
			@Override public int getDecimals() 				{ return 2; 														}
			@Override public boolean doHighlight(double d)  { return false;														}
		});
    	controlList.add(interEdgeTimeEntry);
    	
    	newSpacer(interGroup6);
  	  
    	interEdgeEnergyEntry 		= newEntry(interGroup6,	"Center:", "eV", true, new WdgControlEntry.ModelInterface()
        {
        	@Override public void setValue(Object value)
        	{
        		scanDev.setScanCenter((Double)value, true);
        	}
        	@Override public Object getModelValue() 		{return scanModel.getScanCenter();}
        	@Override public Object getDeviceValue() 		{ return scanDev.getScanCenter();											}
        	@Override public Limits getLimits()
        	{
        		double defaultEnergy = scanModel.getElementEdge().getEnergy();
        		return new Limits
        		(
        			defaultEnergy-ALLOWED_ENERGY_DEVIATION_EV,
        			" is more than "+ALLOWED_ENERGY_DEVIATION_EV+" eV lower than the database value for the "+scanModel.getElementEdge().toString()+" edge, which is "+defaultEnergy,
        			defaultEnergy+ALLOWED_ENERGY_DEVIATION_EV,
        			" is more than "+ALLOWED_ENERGY_DEVIATION_EV+" eV higher than the database value for the "+scanModel.getElementEdge().toString()+" edge, which is "+defaultEnergy,
        			true
        		);
        	}
        	@Override public int getDecimals() 				{return 1;}
        	@Override public boolean doHighlight(double d)
        	{
        		double defaultEnergy = scanModel.getElementEdge().getEnergy();
        		return Math.abs(defaultEnergy-d)>0.5;
        	}
        });
        controlList.add(interEdgeEnergyEntry);
    
    	interButton6.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				interButton6.setFocus();
				scanModel.startEdgeScan();
			}
		});
    	
    	Group interGroup7			= newGroup(parent);
    								  newLabelLeft(interGroup7, 	"7. ");
    	Button interButton7			= newButton(interGroup7, 	"Done with fluor.");
    	interButton7.setToolTipText("Retract fluor. detector and restore X-ray beam attenuation");
    	controlList.add(interButton7);
    	
    	interButton7.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				scanModel.cleanupInteractiveScan();
			}
    	});
    	
    	interStopButton			= newButton2(parent, "Stop",	SWT.BOTTOM, GUI.COLOR_BLUE_BUTTON);
    	interStopButton.addSelectionListener(new SelectionListener()
    	{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				scanModel.stopInteractiveScan();
			}
    	});
	}
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////// helper methods //////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected Group newGroup(Composite parent)
	{
		Group group = new Group(parent, SWT.NONE);
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout gl = new GridLayout(2,false);
		gl.marginHeight = 5;
		gl.marginWidth = 5;
		gl.verticalSpacing = 8;
		group.setLayout(gl);
		return group;
	}
	
	protected Label newLabelLeft(Composite parent, String text)
	{
		Label label = new Label(parent, SWT.NONE);
		label.setFont(GUI.FONT_SCAN_STEP);
		GridData gd = new GridData(SWT.DEFAULT,SWT.CENTER,false,true);
		gd.widthHint = 19;
		label.setLayoutData(gd);
		label.setText(text);
		return label;
	}
	
	protected Label newLabelRight(Composite parent, String text)
	{
        Label label = new Label(parent, SWT.NONE);
        label.setFont(GUI.FONT_SCAN_STEP);
        label.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
        label.setText(text);
        return label;
	}
	
	protected Composite newSpacer(Composite parent)
	{
		Composite comp = new Composite(parent, SWT.NONE);
		comp.setLayoutData(new GridData(1,1));
		comp.setBackground(parent.getBackground());
		return comp;
	}

	protected Button newButton(Composite parent, String text)
	{
		Button button = new Button(parent, SWT.PUSH);
		button.setFont(GUI.FONT_DEFAULT);
		button.setBackground(GUI.COLOR_BLUE_BUTTON);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.widthHint = CONTROLS_WIDTH - 50;
		button.setLayoutData(gd);
		button.setText(text);
		return button;
	}
	
	protected Button newButton2(Composite parent, String text, int vAlign, Color color)
	{
    	Button button = new Button(parent, SWT.PUSH);
    	button.setText(text);
    	button.setFont(GUI.FONT_SCAN_BUTTON2);
    	button.setBackground(color);
    	GridData gd = new GridData(SWT.CENTER, vAlign, true, (vAlign==SWT.BOTTOM)?true:false);
    	gd.widthHint = CONTROLS_WIDTH - 120;
    	gd.heightHint = 35;
    	button.setLayoutData(gd);
    	return button;
	}
	
	protected Button newButton3(Composite parent, String text)
	{
		Button button = new Button(parent, SWT.PUSH);
		button.setFont(GUI.FONT_DEFAULT);
		button.setBackground(GUI.COLOR_BLUE_BUTTON);
		GridData gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false, 2, 1);
		gd.widthHint = CONTROLS_WIDTH - 80;
		button.setLayoutData(gd);
		button.setText(text);
		return button;
	}
	
	protected WdgControlEntry newEntry(Composite parent, String labelText, String unitText, boolean popupOnError, WdgControlEntry.ModelInterface mi)
	{
		if (null == mi)
		{
			mi = new WdgControlEntry.ModelInterface()
			{
				@Override public void setValue(Object value)
				{
				}
			};
		}
		
		Composite comp = new Composite(parent, SWT.NONE);
		comp.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
		GridLayout gl = new GridLayout(3,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		comp.setLayout(gl);
		
		Label label1 = new Label(comp, SWT.NONE);
		label1.setText(labelText);
		label1.setFont(GUI.FONT_DEFAULT);
		GridData gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		gd.widthHint = 54;
		label1.setLayoutData(gd);
		
		WdgControlEntry entry = new WdgControlEntry(comp, SWT.NONE, WdgControlEntry.EntryType.TEXT, popupOnError, mi);
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		gd.widthHint = 80;
		entry.setLayoutData(gd);

		Label label2 = new Label(comp, SWT.NONE);
		label2.setText(unitText);
		label2.setFont(GUI.FONT_DEFAULT);
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		gd.widthHint = 28;
		label2.setLayoutData(gd);
		
		return entry;
	}

	@Override
	public void scanChanged(ModelScan model)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			interSpectrumTimeEntry.setText(scanModel.getSpectrumTime());
			interEdgeTimeEntry.setText(scanModel.getEdgeTime());
			autoTimeEntry.setText(scanModel.getEdgeTime());
			//interState.setText(scanModel.getInterStep().toString());
			autoPrefixDirSelector.setPrefix(scanModel.getPrefix());
			autoPrefixDirSelector.setDir(scanModel.getDirectorySuffix());
			interPrefixDirSelector.setPrefix(scanModel.getPrefix());
			interPrefixDirSelector.setDir(scanModel.getDirectorySuffix());

			String energyText = "Default energy for "+scanModel.getElementEdge()+" is "+Util.sprintf("%.0f",scanModel.getElementEdge().getEnergy())+" eV";
			autoEdgeEnergyEntry.setText(scanModel.getScanCenter());
			interEdgeEnergyEntry.setText(scanModel.getScanCenter());
			autoEdgeEnergyEntry.setToolTipText(energyText);
			interEdgeEnergyEntry.setToolTipText(energyText);

			updateEnableState();
		}});
	}
	
	protected void updateEnableState()
	{
		boolean isMaster = Client.isMaster();
		
		boolean isInterIdle = (InterStep.IDLE == ModelScan.get().getInterStep());
		boolean isAutoIdle = (AutoStep.NONE == ModelScan.get().getAutoStep());
		boolean isScanIdle = isInterIdle && isAutoIdle;
		
		boolean isInterStopping = (InterStep.STOPPING == ModelScan.get().getInterStep());
		boolean isAutoStopping = (AutoStep.STOPPING == ModelScan.get().getAutoStep());
		boolean isStopping = isInterStopping || isAutoStopping;
		
		boolean autoOptimized = ModelScan.get().getAutoOptimized();
		
		for (Control control : controlList)
		{
			if (isMaster && isScanIdle && AppState.isOperationAllowed(Operation.START_SCAN))
			{
				control.setEnabled(true);
			}
			else
			{
				control.setEnabled(false);
			}
		}
		
		if (isMaster && isScanIdle && AppState.isOperationAllowed(Operation.START_SCAN))
		{
			autoRescanButton.setEnabled(autoOptimized);
			autoCleanupButton.setEnabled(autoOptimized);
		}
		else
		{
			autoRescanButton.setEnabled(false);
			autoCleanupButton.setEnabled(false);
		}
		
		if (isMaster)
		{
			if (isStopping)
			{
				autoStopButton.setText("Stopping...");
				interStopButton.setText("Stopping...");
				autoStopButton.setEnabled(false);
				interStopButton.setEnabled(false);
			}
			else
			{
				autoStopButton.setText("Stop Scan");
				interStopButton.setText("Stop Scan");
    			autoStopButton.setEnabled(!isScanIdle);
    			interStopButton.setEnabled(!isScanIdle);
			}
		}
		else
		{
			autoStopButton.setEnabled(false);
			interStopButton.setEnabled(false);
		}
	}
}
