/**
 *  SilTableModel is for accessing Web-Ice
 * This code is a part of Web-Ice examples of Crystal-Analysis server examples 
 * modified for JBluice
 * 
 * This is cache for spreadsheet displayed on the Screening tab
 * 
 */

package gov.anl.gmca.screening;

import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.io.hw.Database;
import gov.anl.gmca.syslog.Log;
import org.apache.http.entity.mime.content.ContentBody;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;

public class ModelSilTable
		implements org.xml.sax.ErrorHandler, IStructuredContentProvider, ITableLabelProvider, ContentBody {
	/* For parsing sil xml document and constructing a DOM object */
	private javax.xml.parsers.DocumentBuilderFactory dFactory = null;
	private javax.xml.parsers.DocumentBuilder dBuilder = null;

	/* Table data, one entry for each row */
	public Hashtable<Integer, ModelRowData> data = new Hashtable<Integer, ModelRowData>();
	private DevSilTable silTable = new DevSilTable();

	/* Keep track of the current event id for the loaded sil */
	public int curEventId = 0;

	public int silId = 0;
	public static boolean throwException = false;

	public ModelSilTable() throws Exception {
		/* Instantiate a DocumentBuilderFactory */
		dFactory = javax.xml.parsers.DocumentBuilderFactory.newInstance();

		/* Use the DocumentBuilderFactory to create a DocumentBuilder */
		dBuilder = dFactory.newDocumentBuilder();
		dBuilder.setErrorHandler(this);
	}

	public int getColumnCount() {
		return ModelRowData.getColumnCount();
	}

	public int getRowCount() {
		if (data == null)
			return 0;

		return data.size();
	}

	public String getColumnName(int col) {
		return ModelRowData.getColumnName(col);
	}

	public int getColumnIndex(String col) {
		String temp;
		int i;
		for (i = 0; i < ModelRowData.ColumnNames.length; i++) {
			temp = ModelRowData.ColumnNames[i];
			if (temp.equals(col)) {
				break;
			}
		}
		return i;
	}

	public Object getValueAt(int row, int col) {
		ModelRowData cell = (ModelRowData) data.get(row);
		if (cell == null)
			return null;
		if (col >= getColumnCount())
			return null;
		return cell.col[col];
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		/* Only allow editing the Comment column */
		if ((columnIndex < 0) || (columnIndex >= getColumnCount()))
			return false;

		if (getColumnName(columnIndex).equals("Comment"))
			return true;

		return false;
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex, boolean save) {
		try {

			/* Only allow editing the Comment column */
			if ((columnIndex < 0) || (columnIndex >= getColumnCount()))
				return;

			setCrystal(rowIndex, columnIndex, aValue, save);

		} catch (Exception e) {
			Log.note("Failed to set cell row = " + rowIndex + " col = " + columnIndex + " new value = "
					+ aValue.toString() + " " + e.getMessage());
		}
	}

	/**
	 * Extract value from a subnode of a crystal node, except for Images node. XML
	 * data for each crystal. <Crystal row="1" excelRow="3" selected="0">
	 * <ContainerID>unknown</ContainerID> <Port>A2</Port>
	 * <CrystalID>VINC</CrystalID> <Protein></Protein> <Comment></Comment>
	 * <FreezingCond></FreezingCond> <CrystalCond></CrystalCond> <Metal></Metal>
	 * <Priority></Priority> <Person></Person> <CrystalURL></CrystalURL>
	 * <ProteinURL></ProteinURL> <Directory>VINC</Directory>
	 * <SystemWarning></SystemWarning> <AutoindexImages>/data/penjitk/collect/SIM11
	 * -1/6377/Q315/VINC/VINC_001.mccd /data/penjitk/collect/SIM11-1/6377/Q315/VINC
	 * /VINC_002.mccd</AutoindexImages> <Score>0.7997053033554247</Score>
	 * <UnitCell>35.12 88.28 119.10 90.00 90.00 90.00</UnitCell>
	 * <Mosaicity>0.400000</Mosaicity> <Rmsr>0.029</Rmsr>
	 * <BravaisLattice>P222</BravaisLattice> <Resolution>1.81</Resolution>
	 * <ISigma></ISigma> <Images> <Group name="1">
	 * <Image name="VINC_001.mccd" jpeg="VINC_0deg_001.jpg" integratedIntensity=
	 * "N/A" numOverloadSpots="0" score="7" dir=
	 * "/data/penjitk/collect/SIM11-1/6377/Q315/VINC" resolution="2.4" iceRings="4"
	 * small="VINC_001_small.jpg" numSpots="N/A" spotShape="0.852" quality="" large=
	 * "VINC_001_large.jpg" diffractionStrength="4.2" medium="VINC_001_medium.jpg"
	 * spotfinderDir="/data/penjitk/webice/screening/6377/VINC/spotfinder" />
	 * </Group> <Group name="2">
	 * <Image name="VINC_002.mccd" jpeg="VINC_90deg_002.jpg" integratedIntensity=
	 * "N/A" numOverloadSpots="0" score="7" dir=
	 * "/data/penjitk/collect/SIM11-1/6377/Q315/VINC" resolution="2.6" iceRings="4"
	 * small="VINC_002_small.jpg" numSpots="N/A" spotShape="0.842" quality="" large=
	 * "VINC_002_large.jpg" diffractionStrength="4.3" medium="VINC_002_medium.jpg"
	 * spotfinderDir="/data/penjitk/webice/screening/6377/VINC/spotfinder" />
	 * </Group> <Group name="3"> </Group> </Images>
	 * <AutoindexDir>/data/penjitk/webice
	 * /screening/6377/VINC/autoindex</AutoindexDir> </Crystal>
	 */
	private Object getNodeValue(Element aCrystal, String colName) throws Exception {
		NodeList ll = aCrystal.getElementsByTagName(colName);
		if ((ll == null) || (ll.getLength() == 0))
			return null;

		if (colName.equals("Images")) {
			String img1 = getImage(aCrystal, "1");
			String img2 = getImage(aCrystal, "2");
			return new String(img1 + " " + img2).trim();

		} else {
			// Get first child of the given name
			Node node = ll.item(0);
			Text textNode = (Text) node.getFirstChild();
			if (textNode != null) {
				return (String) textNode.getNodeValue();
			}

		}
		return null;
	}

	/**
	 * Extract image name from Crystal/Images/Group[@name=which]/Image/@name
	 * <Crystal> <Images ...> <Group name="1" ...> <Image name="A1_001.mccd" ... />
	 * <Image name="A1_002.mccd" ... /> </Group> </Images> </Crystal>
	 */
	private String getImage(Element aCrystal, String whichGroup) throws Exception {
		NodeList ll = aCrystal.getElementsByTagName("Images");
		if ((ll == null) || (ll.getLength() == 0))
			return "";

		Element imagesNode = (Element) ll.item(0);

		ll = imagesNode.getElementsByTagName("Group");
		if ((ll == null) || (ll.getLength() == 0))
			return "";

		Element groupNode = null;
		for (int i = 0; i < ll.getLength(); ++i) {
			groupNode = (Element) ll.item(i);
			String groupName = groupNode.getAttribute("name");
			if (!groupName.equals(whichGroup))
				continue;
			NodeList mm = groupNode.getElementsByTagName("Image");
			if ((mm == null) || (mm.getLength() == 0))
				return "";
			Element imageNode = (Element) mm.item(0);
			Element imageNode1 = (Element) mm.item(1);
			if (imageNode != null && imageNode1 != null)
				return (imageNode.getAttribute("name") + " " + imageNode1.getAttribute("name"));
		}

		return "";
	}

	public void writeToExcel(String excelFile) throws IOException, InvalidFormatException {

		Workbook wbs;
		try {
			OutputStream inp = new FileOutputStream(excelFile);

			wbs = new HSSFWorkbook();

			Sheet s = wbs.createSheet();
			/* declare a row object reference */
			Row r = null;
			/* declare a cell object reference */
			Cell c = null;
			int i = 0;
			r = s.createRow(0); /* write the column names */
			for (int j = 1; j < ModelRowData.ColumnNames.length; j++) {

				c = r.createCell(j - 1);
				c.setCellValue(getColumnName(j));
			}
			// write data
			for (i = 1; i < data.size() + 1; i++) {
				Row r1 = s.createRow(i);
				for (int j = 1; j < ModelRowData.ColumnNames.length; j++) {
					c = r1.createCell(j - 1);
					c.setCellValue((String) getValueAt(i - 1, j));
				}
			}
			wbs.write(inp);
			inp.close();
		} catch (IOException ex) {
			Log.error("Excel export failed :" + ex.getMessage());
		}

	}

	/**
	 * Loading spreadsheet data as xml and notify Table to update GUI
	 */
	public void downloadSil(String excelFile) throws Exception {
		if (excelFile == null || excelFile.equals(""))
			return;

		writeToExcel(excelFile);
	}

	synchronized public void loadSilFromMysql() throws Exception {
		silTable.updateAllRows(this, true);
	}

	synchronized public String loadSilFromSpreadsheet(String excelFile, int isWebice) throws Exception {
		String xmlFileDir = System.getProperty("user.home") + "/bluice_temp";
		int mid = excelFile.lastIndexOf(".");
		int dir = excelFile.lastIndexOf("/");
		int isXlsx = 0;
		if (excelFile.substring(mid + 1, excelFile.length()).equals("xlsx")) {
			isXlsx = 1;
		}
		String newFile = excelFile.substring(dir + 1, mid) + ".xls";

		throwException = false;
		CmdExcel2Xml excelToXml = new CmdExcel2Xml();

		File dirFile = new File(xmlFileDir);
		if (!dirFile.exists()) {

			if (!dirFile.mkdirs()) {
				Log.error("ERROR; Couldn't create directory " + xmlFileDir);
				throw new Exception("ERROR; Couldn't create directory " + xmlFileDir);
			}
		}
		String xmlFileName = System.getProperty("user.home") + "/bluice_temp/sil.xml";
		try {
			excelToXml.convert(excelFile, xmlFileName);
		} catch (Exception e) {
			Log.error(e.getMessage());
			throw new Exception(e.getMessage());
		}
		StringBuffer warnings = new StringBuffer();
		File xmlFile = new File(xmlFileName);
		/* Use the DocumentBuilder to parse the XML input */
		Document silDoc = dBuilder.parse(xmlFile);

		Element root = silDoc.getDocumentElement();
		NodeList crystals = root.getElementsByTagName("Crystal");

		int cryscount = crystals.getLength() - excelToXml.rowCount;
		if (crystals.getLength() == 0) {
			throw new Exception("No sample list in spreadsheet");
		}
		if (cryscount != ModelScreening.getPinsPerPuck() * ModelScreening.NoofPucks) {
			Log.error("ALS/Rigaku puck type mismatch (or) \nRequired number of samples in the spreadsheet : "
					+ ModelScreening.getPinsPerPuck() * ModelScreening.NoofPucks + "\ncurrent spreadsheet has : "
					+ (crystals.getLength() - excelToXml.rowCount));
			throw new Exception("ALS/Rigaku puck type mismatch (or) \nRequired number of samples in the spreadsheet : "
					+ ModelScreening.getPinsPerPuck() * ModelScreening.NoofPucks + "\ncurrent spreadsheet has : "
					+ (crystals.getLength() - excelToXml.rowCount));
		}

		// VALIDATE & FIX
		Element dataNode = silDoc.getDocumentElement();
		Node child = dataNode.getFirstChild();
		if (child == null)
			throw new Exception("Empty spreadsheet");

		data.clear();
		silTable.deleteAllResults();

		// Lookup table for Port
		// to ensure that it's unique.
		Hashtable<String, Integer> portLookup = new Hashtable<String, Integer>();

		// Lookup table for CrystalID
		// to ensure that it's unique.
		Hashtable<String, Integer> idLookup = new Hashtable<String, Integer>();

		// Loop over each row.
		while (child != null) {

			if ((child instanceof Element) && child.getNodeName().equals("Crystal")) {

				// Validate Port. Can't fix this field.
				// If something is wrong we need to throw an exception.
				String port = excelToXml.validatePort((Element) child, portLookup, warnings);

				// Validate CrystalID field. Replace invalid chars with _.
				// Also make sure that crystalID is unique.
				excelToXml.validateCrystalID((Element) child, port, idLookup, warnings);

				// Validate Directory
				excelToXml.validateDirectory((Element) child, warnings);
			}

			child = child.getNextSibling();
		}

		if (throwException)
			throw new Exception(warnings.toString());
		// SAVE
		TransformerFactory tFactory = TransformerFactory.newInstance();
		Transformer transformer = tFactory.newTransformer();
		transformer.setOutputProperty(OutputKeys.VERSION, "1.0");
		transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
		DOMSource source = new DOMSource(dataNode);
		StreamResult result = new StreamResult(new FileOutputStream(xmlFileName));
		transformer.transform(source, result);

		for (int i = 0; i < cryscount; ++i) {
			Element aCrystal = (Element) crystals.item(i);
			
			// check the port matches row id
			String puckLabel = ModelScreening.indexToLabel(i).substring(0, 1);
			String port = getNodeValue(aCrystal, "Port").toString();
			
			if (port.trim().isEmpty() || !port.startsWith(puckLabel)) {
				NBMessageBox nm = new NBMessageBox(GUI.getShell(), SWT.ICON_WARNING | SWT.OK);
				nm.setText("Illegal port in spreadsheet.");
				nm.setMessage(String.format("port %s in the spreadsheet row %s  is illegal since we expect it to start with %s, \n please correct it (ports need to be ordered alphabetically A-Z) and reload.", port, i+1, puckLabel));
				nm.open();
				return "2";
			}
			
			ModelRowData cell = new ModelRowData(i);
			for (int j = 0; j < getColumnCount(); ++j) {
				Object str = getNodeValue(aCrystal, ModelRowData.getColumnName(j));
				String valStr = "";
				if (ModelRowData.getColumnName(j).equalsIgnoreCase("CrystalID")
						|| ModelRowData.getColumnName(j).equalsIgnoreCase("Directory")) {
					if (str == null) {
						valStr = getNodeValue(aCrystal, "Port").toString();
					} else if (str.toString().equals(" ") || str.toString().equals("")) {
						valStr = getNodeValue(aCrystal, "Port").toString();
					} else {
						if (str.toString().contains(" "))
							valStr = str.toString().replaceAll(" ", "");
						else
							valStr = str.toString();
					}
					valStr = valStr.toString().replaceAll("'", "");
					cell.setColumn(j, valStr);
				} else if (ModelRowData.getColumnName(j).equalsIgnoreCase("Score")) {
					if (str != null && !str.toString().equals("") && !str.toString().equals(" ")
							&& !str.equals("null")) {
						double valInt = Double.parseDouble(str.toString());
						valStr = Double.toString(Math.round(valInt * 100.0) / 100.0);
						valStr = valStr.toString().replaceAll("'", "");
						cell.setColumn(j, valStr);
					} else
						cell.setColumn(j, str);
				} else {
					if (str != null && !str.toString().equals("") && !str.toString().equals(" ")
							&& !str.equals("null")) {
						str = str.toString().replaceAll("'", "");

						cell.setColumn(j, str);
					} else
						cell.setColumn(j, "");
				}
				if (str != null) {
					/* check for old spreadsheet port number has only one letter */
					if (ModelRowData.getColumnName(j).equalsIgnoreCase("Port")) {
						if (str.toString().length() < 2) {
							return ("1");
						}
					}
				}
			}
			cell.prefixes.clear();
			cell.currentPrefix = "";
			data.put(cell.row, cell);
			silTable.setRow(cell, false);
		}

		silTable.updateAllRows(this, true);

		/* Save latest event id */
		String eventIdStr = root.getAttribute("eventId");
		try {
			curEventId = Integer.parseInt(eventIdStr);
		} catch (NumberFormatException e) {
			curEventId = 0;
		}
		return "0";
	}

	public void deleteAllTable() {
		silTable.deleteAllResults();
	}

	public void updateRowResults(String port, String prefix) {

		silTable.updateStrategy(this, port, prefix);
	}

	public int getCrystalSelection(int row) {

		silTable.updateRow(row, this);
		ModelRowData rowD = (ModelRowData) data.get(row);
		return rowD.selected;
	}

	public void setCrystalScreened(int rowIndex, int screened, String prefix, boolean update) {
		ModelRowData row = (ModelRowData) data.get(rowIndex);
		row.currentPrefix = prefix;
		row.prefixes.add(prefix);

		row.screened = screened;
		silTable.setRow(row, update);
	}

	public void clearCrystalScreened(int rowIndex, int screened, String prefix, boolean update) {
		ModelRowData row = (ModelRowData) data.get(rowIndex);
		row.currentPrefix = "";
		row.prefixes.clear();
		int i = 4;
		for (; i < ModelRowData.getColumnCount(); i++) {
			row.col[i] = "";
			i++;
		}
		row.screened = screened;
		data.put(rowIndex, row);
		silTable.clearRow(row, update);
	}

	public void setAllCrystals(boolean update) {
		silTable.updateAllRows(this, update);
	}

	public void setAllCrystals() {
		silTable.updateAllRows(this, true);
	}
	
	public void setCrystalSelection(int rowIndex, int select, int order, boolean update) {

		ModelRowData row = (ModelRowData) data.get(rowIndex);
		if (row == null)
			return;
		if (select == 0) {
			row.selected = 0;
			row.order = 0;
		} else {
			row.selected = 1;
			row.order = order;
		}

		// 8 mil sec per row, almost all used by DB query
		// long startTime = System.nanoTime();
		silTable.setRow(row, update);
		// long endTime = System.nanoTime();
		// Log.note("setRow took " + (endTime - startTime)/1000000 + " milsec.");
	}

	// QX combining multiple sql statements to increase speed
	// previous version run mysql query row by row, each takes 8 msec
	// the current version only run a few queries regardless size of the crystal list
	// and runs in constant time
	public void setCrystalSelections(ModelCrystalList crystalList, boolean update) {
		// long startTime = System.nanoTime();

		ArrayList<String> ids = new ArrayList<>();	// store sampleNumbers --- aka ids for update
		ArrayList<String> whens = new ArrayList<>();  // condition for update inside case
		ArrayList<String> tuples = new ArrayList<>(); // values for insert of the form (,,,)

		// single query and save the keys into a hash for lookup
		Database.Result r = Database.exec(DBName.USERDATA, "select sampleNumber from  SilTable;");
		Set<Integer> existingSampleNumbers = new HashSet<Integer>();
		if (r.numRows() > 0 ) {
			for (Map<String,String> row: r.getRows()) {
				existingSampleNumbers.add(Integer.valueOf(row.get("sampleNumber")));
			}
		}
		
		// aggregate mysql statement component
		int i = 1;
		for (Object crystal : crystalList.getAll()) {
			int rowIndex = Integer.parseInt((String) crystal);
			ModelRowData row = (ModelRowData) data.get(rowIndex);

			if (row == null)
				return;

			row.selected = 1;
			row.order = i;
			
			if (existingSampleNumbers.contains(rowIndex)) {
				ids.add(String.valueOf(rowIndex));
				whens.add(String.format("  WHEN sampleNumber = %s THEN '%s' ", rowIndex, row.order));
			} else {
				tuples.add("(" + row.row + ", '" + row.col[1] + "', '" + row.col[2] + "', '" + row.col[3] + "', '"
						+ row.col[4] + "', '" + row.col[6] + "', '" + row.col[7] + "', '" + row.col[9] + "', '"
						+ row.col[10] + "', '" + row.col[11] + "', '" + row.col[12] + "', '" + row.col[13] + "', '"
						+ row.col[14] + "', '" + row.col[8] + "')");
			} 
			
			i++;
		}

		// assembly mysql statements and run them
		if (!tuples.isEmpty()) {
			String createMultipleRows = String.format("insert into silTable (sampleNumber,Port,CrystalID,Directory,Comment,Resolution, Score, Images, IceRings, UnitCell, Mosaicity, Rmsd, BravaisLattice, Protein) values %s;", String.join(", ", tuples));
			//Log.note("bulk create = " + createMultipleRows);
			Database.exec(DBName.USERDATA, createMultipleRows);
		}

		if (!ids.isEmpty()) {
			String updateSelected = String.format("UPDATE SilTable SET selected='1' WHERE sampleNumber IN (%s)",
					String.join(",", ids));
			String updateOrdered = String.format(
					"UPDATE SilTable SET ordered= CASE \n%s\n  END \nWHERE sampleNumber IN (%s);",
					String.join("\n", whens), String.join(",", ids));
			Database.exec(DBName.USERDATA, updateSelected);
			Database.exec(DBName.USERDATA, updateOrdered);
		}
		
		if (update) {
			Epics.putInteger(PV_SCR.silUpdated, -2);
		}
	}

	public void unsetCrystalSelections(ArrayList<Integer> sampleNumbers, boolean update) {
		ArrayList<String> ids = new ArrayList<>();	// store sampleNumbers --- aka ids for update
		
		// aggregate mysql statement component
		for (int rowIndex : sampleNumbers) {
			ModelRowData row = (ModelRowData) data.get(rowIndex);

			if (row == null)
				return;

			row.selected = 0;
			row.order = 0;
			
			ids.add(String.valueOf(rowIndex));
		}

		if (!sampleNumbers.isEmpty()) {
			String updateSelectedOrdered = String.format("UPDATE SilTable SET selected='0', ordered='0' WHERE sampleNumber IN (%s)",
					String.join(",", ids));

			Database.exec(DBName.USERDATA, updateSelectedOrdered);
		}
		
		if (update) {
			Epics.putInteger(PV_SCR.silUpdated, -2);
		}
	}
	
	
	/**
	 * Set crystal data
	 */
	synchronized public void setCrystal(int rowIndex, int columnIndex, Object val, boolean save) throws Exception {
		ModelRowData row = (ModelRowData) data.get(rowIndex);
		if (row == null)
			return;

		String colName = getColumnName(columnIndex);
		row.setColumn(columnIndex, (String) val);

		if (save)
			silTable.setRow(row, true);
	}

	/**
	 * ErrorHandler method
	 */
	public void error(SAXParseException e) throws SAXException {
		System.out.println("in ErrorHandler:error: line = " + e.getLineNumber() + " col = " + e.getColumnNumber()
				+ e.getMessage());
		throw e;
	}

	/**
	 * ErrorHandler method
	 */
	public void fatalError(SAXParseException e) throws SAXException {
		System.out.println("in ErrorHandler:fatalError: line = " + e.getLineNumber() + " col = " + e.getColumnNumber()
				+ e.getMessage());
		throw e;
	}

	/**
	 * ErrorHandler method
	 */
	public void warning(SAXParseException e) throws SAXException {
		System.out.println("in ErrorHandler:warning: line = " + e.getLineNumber() + " col = " + e.getColumnNumber()
				+ e.getMessage());
		throw e;
	}

	@Override
	public Object[] getElements(Object arg0) {
		Object[] elements = new Object[data.size()];
		for (int i = 0; i < data.size(); i++) {
			elements[i] = data.get(i);
		}
		return elements;
	}

	@Override
	public String getColumnText(Object element, int columnIndex) {
		// TODO QX why element is null in the first place???
		if (element == null) {
			return null;
		}
		ModelRowData row = (ModelRowData) element;
		if (columnIndex >= ModelRowData.getColumnCount())
			return null;
		return (String) row.col[columnIndex];
	}

	@Override
	public void dispose() {
	}

	@Override
	public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
	}

	@Override
	public void addListener(ILabelProviderListener arg0) {
	}

	@Override
	public boolean isLabelProperty(Object arg0, String arg1) {
		return false;
	}

	@Override
	public void removeListener(ILabelProviderListener arg0) {
	}

	@Override
	public Image getColumnImage(Object element, int columnIndex) {
		return null;
	}

	@Override
	public String getFilename() {
		return null;
	}

	@Override
	public String getCharset() {
		return null;
	}

	@Override
	public long getContentLength() {
		return 0;
	}

	@Override
	public String getMediaType() {
		return null;
	}

	@Override
	public String getMimeType() {
		return null;
	}

	@Override
	public String getSubType() {
		return null;
	}

	@Override
	public String getTransferEncoding() {
		return null;
	}

	@Override
	public void writeTo(OutputStream arg0) throws IOException {
	}
}
