package gov.anl.gmca.screening.ui;

import gov.anl.gmca.common.DBConfig;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.*;

import java.util.Map;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.io.hw.Epics;
import gov.anl.gmca.screening.ModelScreening;
import gov.anl.gmca.screening.Pucks;
import gov.anl.gmca.syslog.Log;
import net.miginfocom.swt.MigLayout;

public class WdgEditPucks extends Dialog implements Runnable {
	private Composite container;
	Table table;
	Text name;
	Text owner;
	Combo type;
	boolean enablePuckOwnershipCheck = Epics.getBoolean(PV_SCR.enablePuckOwnership);
	private boolean blocking;
	private String[] puckLabels = new String[ModelScreening.NoofPucks];

	protected WdgEditPucks(Shell parentShell, boolean blocking) {
		super(parentShell);
		this.blocking = blocking;
		setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(this.blocking);
	}

	protected WdgEditPucks(Shell parentShell) {
		this(parentShell, false);
	}

	protected Control createDialogArea(Composite parent) {
		this.container = (Composite) super.createDialogArea(parent);
		MigLayout layout = new MigLayout("wrap 1");
		container.setLayout(layout);
		createContent();
		return container;
	}

	public void createContent() {
		banner();
		editPuckTable();
	}

	private void banner() {
		Label banner = new Label(container, SWT.CENTER);
		banner.setText("Edit puck-related configurations and set ownership.\n ");

		new Label(container, SWT.NONE).setText("No. of Pucks:");
		Combo comboPucks = new Combo(container, SWT.BORDER | SWT.READ_ONLY);
		String noOfPucks = DBConfig.get("screening_noofpucks");
		comboPucks.setItems(new String[] { noOfPucks, "6", "18", });
		comboPucks.select(0);
		comboPucks.setEnabled(false);

		Button toggleCheck = new Button(container, SWT.CHECK);

		toggleCheck.setText("Enable puck ownership check");
		toggleCheck.setSelection(enablePuckOwnershipCheck);

		toggleCheck.addMouseListener(new MouseListener() {
			@Override
			public void mouseUp(MouseEvent arg0) {
			}

			@Override
			public void mouseDown(MouseEvent arg0) {
				enablePuckOwnershipCheck = !enablePuckOwnershipCheck;
				Log.note("puck ownership enabled: " + enablePuckOwnershipCheck);
				Epics.putInteger(PV_SCR.enablePuckOwnership, enablePuckOwnershipCheck?1:0);
				table.setEnabled(enablePuckOwnershipCheck);
				name.setEnabled(enablePuckOwnershipCheck);
				type.setEnabled(enablePuckOwnershipCheck);
				owner.setEnabled(enablePuckOwnershipCheck);
			}

			@Override
			public void mouseDoubleClick(MouseEvent arg0) {
			}
		});
	}

	private void editPuckTable() {
		table = new Table(container, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);

		table.setHeaderVisible(true);
		String[] titles = { "DewarPosition", "PuckType", "PuckName", "PuckOwner" };

		for (int loopIndex = 0; loopIndex < titles.length; loopIndex++) {
			TableColumn column = new TableColumn(table, SWT.NULL);
			column.setText(titles[loopIndex]);
		}

		Result r = Database.exec(DBName.USERDATA, "select position, type, name, owner from Pucks;");
		int count = 0;
		for (Map<String, String> row : r.getRows()) {
			if (count >= ModelScreening.NoofPucks) {
				continue;
			}
			TableItem item = new TableItem(table, SWT.BORDER);
			int i =0;
			for (Map.Entry<String, String> entry : row.entrySet()) {
				item.setText(i, entry.getValue());
				i++;
			}
			puckLabels[count] = row.get("position") + " " + row.get("name");
			count ++;
		}

		for (int loopIndex = 0; loopIndex < titles.length; loopIndex++) {
			table.getColumn(loopIndex).pack();
		}

		new Label(container, SWT.CENTER).setText("Set Type Column");
		// type = new Text(container, SWT.NONE);
		type = new Combo(container, SWT.NONE);
		type.setItems(new String[] { "A", "R" });
		type.select(0);
		type.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent arg0) {
				for (TableItem i : table.getItems()) {
					i.setText(1, type.getText());
				}
			}
		});

		new Label(container, SWT.CENTER).setText("Set Puck Name Column");
		name = new Text(container, SWT.NONE);
		name.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent arg0) {
				for (TableItem i : table.getItems()) {
					i.setText(2, name.getText());
				}
			}
		});

		new Label(container, SWT.CENTER).setText("Set Owner Column");
		owner = new Text(container, SWT.NONE);
		owner.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent arg0) {
				for (TableItem i : table.getItems()) {
					i.setText(3, owner.getText());
				}
			}
		});

		table.setEnabled(enablePuckOwnershipCheck);
		name.setEnabled(enablePuckOwnershipCheck);
		type.setEnabled(enablePuckOwnershipCheck);
		owner.setEnabled(enablePuckOwnershipCheck);

		final TableEditor editor = new TableEditor(table);
		editor.horizontalAlignment = SWT.LEFT;
		editor.grabHorizontal = true;
		table.addListener(SWT.MouseDown, new Listener() {
			public void handleEvent(Event event) {
				Rectangle clientArea = table.getClientArea();
				Point pt = new Point(event.x, event.y);
				int index = table.getTopIndex();
				while (index < table.getItemCount()) {
					boolean visible = false;
					final TableItem item = table.getItem(index);

					for (int i = 1; i < table.getColumnCount(); i++) {
						// first column editing not allowed
						Rectangle rect = item.getBounds(i);
						if (rect.contains(pt)) {
							final int column = i;
							final Text text = new Text(table, SWT.NONE);
							Listener textListener = new Listener() {
								public void handleEvent(final Event e) {
									switch (e.type) {
									case SWT.FocusOut:
										item.setText(column, text.getText());
										text.dispose();
										break;
									case SWT.Traverse:
										switch (e.detail) {
										case SWT.TRAVERSE_RETURN:
											item.setText(column, text.getText());
											// FALL THROUGH
										case SWT.TRAVERSE_ESCAPE:
											text.dispose();
											e.doit = false;
										}
										break;
									}
								}
							};
							text.addListener(SWT.FocusOut, textListener);
							text.addListener(SWT.Traverse, textListener);
							editor.setEditor(text, item, i);
							text.setText(item.getText(i));
							text.selectAll();
							text.setFocus();
							return;
						}
						if (!visible && rect.intersects(clientArea)) {
							visible = true;
						}
					}
					if (!visible)
						return;
					index++;
				}
			}
		});

	}

	private void updateTable() {
		if (enablePuckOwnershipCheck) {
			int count = 0;
			for (TableItem i : table.getItems()) {
				String position = i.getText(0);
				String type = i.getText(1);
				String name = i.getText(2);
				String owner = i.getText(3);
				
				Pucks.update(position, type, name, owner);
				puckLabels[count] = position + "_" + name + " ";
				count ++;
			}
			Epics.putInteger(PV_SCR.pucksUpdated, 0);
			Epics.putInteger(PV_SCR.pucksUpdated, 1);
		}
	}

	// common parts
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Set parameters for data reprocessing");
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		// buttons set up to ignore return events in fields, which is used for
		// validation
		// https://stackoverflow.com/questions/1047144/globally-disable-enter-return-key-in-a-swt-dialog
		Button cancel = createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
		cancel.setText("Cancel");
		setButtonLayoutData(cancel);

		Button ok = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, false);
		ok.setText("Update");
		setButtonLayoutData(ok);
	}

	public void okPressed() {
		updateTable();
		this.close();
	}

	@Override
	public void run() {
		this.open();
	}
}
