/**
 * GUI for Wash/Rinse 
 */

package gov.anl.gmca.screening.ui;

import gov.anl.gmca.collect.ModelCollectRun.Point3D;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.DevMove.MoveCmd;
import gov.anl.gmca.common.ui.CCombo;
import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.DevXYZ;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_PMAC;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.screening.CmdRobot;
import gov.anl.gmca.screening.CmdScreening;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;

public class WdgRinseWash implements ClientListener {

	CCombo comboWash;

	private Combo comboShiftAnneal, comboAnnealOption, comboMoveAnneal,
			comboRinseWash, comboRinseWashTime;

	private Button goWash, cancelShiftButton, annealShiftButton;
	private CCombo comboWashTimes;
	private Thread washThread = null;

	static double omega, crystalX, crystalY, horizon;
	boolean isRinse = false;
	

	protected Epics.EpicsValueMonitor monitor = null;

	public WdgRinseWash(Composite parent, int style) {
		
		GridData gd;
		Client.addListener(this);
		String[] washOptions = {"wash"}; //  { "wash", "rinse" };
		
		
		Composite washRinseComp = new Composite(parent, SWT.NONE);
		washRinseComp.setBackground(GUI.BGCOLOR);
		washRinseComp.setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gl = new GridLayout(3, true);

		washRinseComp.setLayout(gl);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		washRinseComp.setLayoutData(gd);
		
		comboWash = new CCombo(washRinseComp, SWT.READ_ONLY);
		comboWash.setItems(washOptions);
		comboWash.select(0);
		comboWash.setEnabled(false);
		comboWash.setBackground(GUI.COLOR_WHITE);
		gd = new GridData();
		gd.widthHint = 60;
		gd.grabExcessVerticalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		comboWash.setLayoutData(gd);

		comboWash.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent arg0) {
				if (comboWash.getText().equals("rinse")) {
					comboWashTimes.setItems(new String[] {"1","2","3","4","5","6"});
					comboWashTimes.select(2);

				} else {
					comboWashTimes.setItems(new String[] {"1","2","3","4","5","6","7","8","9","10"});
					comboWashTimes.select(2);
				}
			}

		});
		
		comboWashTimes = new CCombo(washRinseComp, SWT.BORDER);
		comboWashTimes.setBackground(GUI.COLOR_WHITE);
//		comboWashTimes.setIncrement(1);
//		comboWashTimes.setMaximum(10);
//		comboWashTimes.setMinimum(1);
//		comboWashTimes.setSelection(3);
//		comboWashTimes.setEnabled(false);
		comboWashTimes.setItems(new String[] {"1","2","3","4","5","6","7","8","9","10"});
		comboWashTimes.select(2);
		
		gd = new GridData();
		gd.widthHint = 60;
		gd.grabExcessVerticalSpace = true;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		comboWashTimes.setLayoutData(gd);

		gd = new GridData();
		gd.widthHint = 60;
		goWash = new Button(washRinseComp, SWT.PUSH);
		goWash.setBackground(GUI.COLOR_BLUE_BUTTON);
		goWash.setText("Go");
		goWash.setLayoutData(gd);
		gd.grabExcessHorizontalSpace = false;
		gd.horizontalAlignment = SWT.FILL;
		goWash.setEnabled(false);
		goWash.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent arg0) {

				int robotStatus = Epics.getInteger(PV_ROBOT.pause);
				if (robotStatus == 1) {
					NBMessageBox messageBox = new NBMessageBox(GUI.getShell(),
							SWT.OK | SWT.ICON_ERROR);
					messageBox.setMessage("Robot not ready");
					messageBox.open();
					return;
				}

				String cmd = comboWash.getText();

				if (!cmd.equals("None")) {
					String mounted_ = Epics.getString(PV_ROBOT.mounted);
					if (mounted_.equals("none")) {
						Log.error("No sample mounted with Robot");
						return;
					}

					storeValues();

					comboWash.setEnabled(false);
					if (Client.isMaster())
						goWash.setEnabled(true);

					Epics.put(PV_SCR.CurrentTask, comboWash.getText());

					CmdRobot cmdRobo = new CmdRobot(cmd + " " + comboWashTimes.getText());
					
					washThread = new Thread(cmdRobo, "Robot command");
					washThread.start();
				}
			}

		});
		AppState.addListener(AppState.Operation.OP_RINSE_WASH,
				new AppState.Listener() {
					@Override
					public void stateChanged(Operation op, boolean allowed) {
						GUI.asyncExec(new Runnable() {
							public void run() {
								if (AppState
										.isOperationAllowed(Operation.OP_RINSE_WASH)
										&& !isRinse && Client.isMaster()) {
									goWash.setEnabled(true);
									comboWash.setEnabled(true);
									comboWashTimes.setEnabled(true);
								} else {
									goWash.setEnabled(false);
									comboWash.setEnabled(false);
									comboWashTimes.setEnabled(false);
								}
								updateEnableState();
							}
						});
					}
				});
		monitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {

				final String _pvName = pvName;
				final DBR _value = value;
				if (_pvName.equals(Epics.addBID("R:mounted"))) {
					GUI.asyncExec(new Runnable() {
						public void run() {

							if (Epics.dbrToString(_value).equals("none")) {
								comboWash.setEnabled(false);
								goWash.setEnabled(false);
								comboWashTimes.setEnabled(false);
							} else {
								if (Client.isMaster()) {
									if (washThread != null) {
										if (!washThread.isAlive()) {
											comboWash.setEnabled(true);
											goWash.setEnabled(true);
											comboWashTimes.setEnabled(true);
										}
									} else {
										comboWash.setEnabled(true);
										goWash.setEnabled(true);
										comboWashTimes.setEnabled(true);
									}

								}

							}
						}
					});
				}
				if (_pvName.equals(Epics.addBID("bi:scr:CurrentTask"))) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							int status_ = 0;
							String stat = Epics.dbrToString(_value);
							if (stat.contains("Rinsed")
									|| stat.contains("Washed")) {
								if (Client.isMaster()) {
									restoreValues();
									goWash.setEnabled(true);
									comboWash.setEnabled(true);
									comboWashTimes.setEnabled(true);
									status_ = 1;
									isRinse = false;
									if (!CmdScreening.stopThread)
										Epics.put(PV_SCR.CurrentTask,
												"RenameGroup");
								}

							}
							if (stat.contains("rinse") || stat.contains("wash")
									|| stat.contains("CenterXtal")) {
								if (Client.isMaster()) {
									goWash.setEnabled(false);
									comboWash.setText(comboWash.getText());
									comboWash.setEnabled(false);
									comboWashTimes.setEnabled(false);
									status_ = 0;
									isRinse = true;
								}
							}
							// TODO: disableExceptions
							/*
							 * AppState.screeningState = status_;
							 * AppState.llBusy.notifyListeners();
							 * AppState.llBusyOrSlave.notifyListeners();
							 */
						}
					});
				}

			}
		};

		Epics.addMonitor(PV_ROBOT.mounted, monitor);
		Epics.addMonitor(PV_SCR.CurrentTask, monitor);
	}

	public void storeValues() {
		omega = Epics.getDouble(PV_PMAC.ActPos.motor("gonio_omega"));
		crystalX = Epics.getDouble(PV_PMAC.ActPos.motor("sample_x"));
		crystalY = Epics.getDouble(PV_PMAC.ActPos.motor("sample_y"));
		horizon = Epics.getDouble(PV_PMAC.ActPos.motor("sample_z"));
	}

	public void restoreValues() {
		new Thread(new Runnable() {
			public void run() {
				/* start X-Y axis move */
				MoveCmd gonioMove = DevPmacMotor.get("gonio_omega").startMove(
						omega);

				/*
				 * AssemblyMove xyMove =
				 * DevPmacAssembly.get("sampleXYZ").newMove();
				 * xyMove.setDest("sample_x", crystalX);
				 * xyMove.setDest("sample_y", crystalY);
				 * xyMove.setDest("sample_z", horizon); xyMove.start();
				 */

				Point3D restoreCenter = new Point3D(crystalX, crystalY, horizon);
				DevXYZ.syncMove(restoreCenter, true);

				Util.sleep(200);
				// Epics.putDouble(PV_PMAC.RqsPos.motor("sample_z"),horizon);
				gonioMove.waitFor();
			}
		}, "WdgRinseWash restoreValues").start();

	}

	public void updateEnableState() {
		final String mounted = Epics.getString(PV_ROBOT.mounted);
		GUI.asyncExec(new Runnable() {
			public void run() {
				if (Client.isMaster()
						&& AppState.isOperationAllowed(Operation.OP_RINSE_WASH)) {
					if (!mounted.equals("none")) {
						goWash.setEnabled(true);
						comboWashTimes.setEnabled(true);
						comboWash.setEnabled(true);
					}

				} else {
					goWash.setEnabled(false);
					comboWashTimes.setEnabled(false);
					comboWash.setEnabled(false);

				}
			}
		});
	}

	@Override
	public void masterChanged(boolean isMaster) {
		// TODO Auto-generated method stub
		updateEnableState();
	}
}
