package gov.anl.gmca.server;

import gov.anl.gmca.common.client_server.StateMap.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import spark.Request;

import java.util.ArrayList;
import java.util.concurrent.ConcurrentHashMap;

// Tracks clients connected to this server
//
public class ClientTable
{
	public static ConcurrentHashMap<String,Client> table = new ConcurrentHashMap<>();
	public static final long CLIENT_TIMEOUT_SEC = 5;

	public static void init()
	{
		gov.anl.gmca.users.ModelUserTable.Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				
			}
		});
		new Thread(new Runnable(){public void run(){
			while(true) {
				cleanupClients();
				Util.sleep(1000);
			}
		}}, "Client table cleanup").start();
	}

	public static String getMasterID()
	{
		for (String key : table.keySet()) {
			Client c = table.get(key);
			if (c.isMaster) return c.clientID;
		}
		return null;
	}

	public static void handleRequest(Request req)
	{
		String clientID = req.queryParams("clientID");
		if (null==clientID) {
			// no action
		} else if (table.containsKey(clientID)) {
			table.get(clientID).isMaster = req.queryParams("isMaster").equals("1");
			table.get(clientID).lastUpdateSec = System.currentTimeMillis()/1000;
		} else {
			Client client = new Client();
			client.host = req.queryParams("host");
			client.isMaster = req.queryParams("isMaster").equals("1");
			client.username = req.queryParams("username");
			client.clientID = clientID;
			client.lastUpdateSec = System.currentTimeMillis() / 1000;
			synchronized (table) {
				table.put(clientID, client);
			}
		}
	}

	public static ArrayList<Client> getClientList()
	{
		synchronized (table)
		{
			ArrayList<Client> ret = new ArrayList<>();
			for (Client client : table.values()) ret.add(client);
			return ret;
		}
	}

	public static void remove(String clientID)
	{
		table.remove(clientID);
	}

	public static void cleanupClients()
	{
		synchronized (table) {
			long now = System.currentTimeMillis() / 1000;
			ArrayList<Client> toRemove = new ArrayList<>();
			for (Client client : table.values()) {
				if (client.lastUpdateSec + CLIENT_TIMEOUT_SEC < now) {
					toRemove.add(client);
				}
			}
			if (toRemove.size() > 0) {
				synchronized (table) {
					for (Client client : toRemove) {
						table.remove(client.clientID);
					}
				}
			}
		}
	}
	
	public static void becomeMaster(String clientID)
	{
		
	}
	
	public static boolean becomeSlave(String clientID)
	{
		synchronized (table) {
			if (!table.containsKey(clientID)) return false;
			table.get(clientID).isMaster = false;
			return true;
		}
	}
	
	public static void becomeMasterByForce(String clientID)
	{
		String masterID = null;
		synchronized (table) {
			for (Client client : table.values()) {
				if (client.isMaster) masterID = client.clientID;
				break;
			}
			if (null!=masterID) {
				table.get(masterID).isMaster=false;
			}
			table.get(masterID).isMaster=true;
		}
	}
}
