package gov.anl.gmca.server;

import gov.anl.gmca.collect.DevCollectRunList;
import gov.anl.gmca.collect.DevFPE2;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.*;
import gov.anl.gmca.common.client_server.DeviceMap;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.gui.state.SWServer;
import gov.anl.gmca.hutch.*;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup;
import gov.anl.gmca.raster.DevRasterRunList;
import gov.anl.gmca.raster.DevRasterState;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.sample.DBBeamSize;
import gov.anl.gmca.sample.DevCollimatorCtrl;
import gov.anl.gmca.sample.DevSlider;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.SpotFinderQueue;
import gov.anl.gmca.users.DevUserTable;
import gov.anl.gmca.users.ModelUserTable;
import gov.anl.gmca.util.Util;

import java.util.*;

public class DeviceInit
{
	public static Result pmaResult;

	public static void initMotors()
	{
		pmaResult = Database.exec(DBName.CONFIG,
				"select * from PmacMotorAssembly where name in "+
						"('sample_x','sample_y','sample_z','gonio_omega','detector_z',"+
						"'detector_2t','beamstop_z','beam_size_x','beam_size_y',"+
						"'mo_En_E','camera_zoom','camera_zoom_2')");

		for (int i=0;i<pmaResult.numRows();i++)
		{
			String devName = pmaResult.getRow(i).get("name");
			boolean useRunPrg = pmaResult.getRow(i).get("useRunPrg").contains("1");
			String inPosBandString = pmaResult.getRow(i).get("inPosBand");

			DeviceMap.pmacMotorMap.put(devName, new DevPmacMotor(devName, useRunPrg, inPosBandString));
		}
	}

	public static void initMotorAssemblies()
	{
		for (int i=0;i<pmaResult.numRows();i++)
		{
			String assemblyName = pmaResult.getRow(i).get("assembly");

			// if this device is in an assembly and we didn't already create the assembly
			if (null != assemblyName && assemblyName.length() > 0 && null == DevPmacAssembly.get(assemblyName))
			{
				// gather all devices in this assembly
				ArrayList<DevPmacMotor> motorList = new ArrayList<DevPmacMotor>();
				for (int j=0;j<pmaResult.numRows();j++)
				{
					String assembly2 = pmaResult.getRow(j).get("assembly");
					if (null != assembly2 && assembly2.equals(assemblyName))
					{
						String motorName = pmaResult.getRow(j).get("name");
						motorList.add(DevPmacMotor.get(motorName));
					}
				}

				// create it
				if (null == DevPmacAssembly.get(assemblyName))
				{
					System.out.println("Creating assembly "+assemblyName+" with motors "+motorList);
					new DevPmacAssembly(assemblyName, motorList);
				}
			}
		}
	}

	public static void init()
	{
		Epics.putInteger(PVLookup.PV_RUNS.collectState, ModelCollectRunList.CollectState.INACTIVE.ordinal());

		new DevUserConfig();
		DBBeamSize.init();

		DevSlider.init();
		new DevDataDir();
		new DevSFType(SFRunType.COLLECT);
		new DevSFType(SFRunType.RASTER);
		SpotFinderQueue.init();
		Main.FAKE_DATA=true;
		new DevFPE2();
		new DevKeithleyAmp();
		SWServer.get().init();
		Redis.reset(false);

		initMotors();
		initMotorAssemblies();

		///////////////////////////////////////////////////////// User table /////////////////////////////////////////////////////

		new ModelUserTable();
		DevUserTable dev = new DevUserTable();
		ClientTable.init();

		////////////////////////////////////////////////////////// scripts /////////////////////////////////////////////////////////

		new DevAlignMinibeam();
		new DevAutocenter();
		new DevCollimatorCtrl();
		//new DevEdgeScan();

		////////////////////////////////////////////////////////// run lists ///////////////////////////////////////////////////////

		ModelCollectRunList collectRunList = new ModelCollectRunList();
		new DevCollectRunList(collectRunList).connect();
		ModelRasterRunList interRunList = new ModelRasterRunList(RasterListType.INTER);
		new DevRasterRunList(interRunList).connect();
		ModelRasterRunList autoRunList = new ModelRasterRunList(RasterListType.AUTO);
		new DevRasterRunList(autoRunList).connect();
		ModelRasterState rasterState = new ModelRasterState();
		new DevRasterState(rasterState);


		///////////////////////////////////////////////////////// shutters ///////////////////////////////////////////////////////

		DevShutter.init();

		pmaResult = Database.exec(DBName.CONFIG, "select * from HutchShutter");
		for (int i=0;i<pmaResult.numRows();i++)
		{
			HashMap<String,String> row = pmaResult.getRow(i);

			String devName = row.get("Shutter");
			new DevHutchShutter(devName);
		}


		///////////////////////////////////////////////////////// attenuation //////////////////////////////////////////////////////

		new DevAttenuation();

		/////////////////////////////////////////////////////////// energy /////////////////////////////////////////////////////////

		boolean undulatorEnabled = DBConfig.get("unduflag").equals("1");
		if (undulatorEnabled)
		{
			if (Util.getBID().equals("23i")) new DevUndulator();
			if (Util.getBID().equals("23o")) new DevUndulator();
		}

		new DevEnergy();

		////////////////////////////////////////////////////////// databases ///////////////////////////////////////////////////////

		AppState.init();

		//DBConfig.connectListener();
		//DBConfig.update();
	}
}
