package gov.anl.gmca.server.workers;

import gov.anl.gmca.common.*;
import gov.anl.gmca.common.DevMove.*;
import gov.anl.gmca.common.client_server.RPC.*;
import gov.anl.gmca.hutch.*;
import gov.anl.gmca.server.*;
import gov.anl.gmca.util.*;

import java.util.*;

public class AssemblyMove extends Worker
{
    private String assembly;
    private ArrayList<Double> dest;
    public ArrayList<Double> actpos;

    public AssemblyMove(String assembly, ArrayList<Double> dest)
    {
        super("AssemblyMove");
        this.assembly = assembly;
        this.dest = dest;
        if (!assembly.equals("sampleXYZ")) {
            System.out.println("ERROR: Only assembly sampleXYZ is currently supported");
            throw new IllegalStateException();
        }
    }

    @Override
    public void run()
    {
        DevPmacAssembly dev = DevPmacAssembly.get(assembly);
        DevPmacAssembly.AssemblyMove move = dev.newMove();
        move.setDest("sample_x", dest.get(0));
        move.setDest("sample_y", dest.get(1));
        move.setDest("sample_z", dest.get(2));
        move.start();

        // wait for move
        result = RPCResult.RUNNING;
        MoveResult moveResult = move.waitFor();
        switch (moveResult.type)
        {
            case NOT_STARTED:
                result = RPCResult.NOT_STARTED;
                break;
            case INVALID:
            case ABORTED:
            case ERROR:
                result = RPCResult.ERROR;
                return;
            case SUCCESS:
                result = RPCResult.OK;
                return;

        }
    }

    @Override
    public String jsonStatus()
    {
        return "id="+id+" cmd="+cmd+" result="+result;
    }
}
