package gov.anl.gmca.server.workers;

import gov.anl.gmca.collect.CmdCollectJet;
import gov.anl.gmca.common.*;
import gov.anl.gmca.common.client_server.RPC.RPCResult;
import gov.anl.gmca.server.Worker;
import gov.anl.gmca.util.Util;
import redis.clients.jedis.*;

import java.util.Map;

// Required parameters:
//
// Directory, prefix
// Exposure time
// Frames per dir
//
// Optional:
//
// Skip sample env moves
// Det. dist.
// Atten.
// Beam size
// Energy
//
//
public class CollectJet extends Worker
{
    private Map<String,String> args;
    public CollectJet(Map<String,String> args)
    {
        super("CollectJet");
        if (null!=inst && inst.thread.isAlive()) {
            result = RPCResult.ERROR;
            return;
        }
        this.args=args;
        inst=this;
    }

    public static CollectJet inst = null;
    public static CollectJet get() { return inst; }
    public CmdCollectJet cmd;

    public void stop()
    {
        cmd.pause();
    }

    @Override
    public void run()
    {
        try {
            String dir = args.get("dir");
            String prefix = args.get("prefix");
            Double expTime_sec = Double.parseDouble(args.get("expTime_sec"));
            Integer framesPerDir = Integer.parseInt(args.get("framesPerDir"));
            Integer startFrame = Integer.parseInt(args.get("startFrame"));
            String subDir = args.get("subDir");
            int subDirLimit = 999;
            if (args.containsKey("subDirLimit")) {
                subDirLimit = Integer.parseInt(args.get("subDirLimit"));
            }
            cmd = new CmdCollectJet(dir, subDir, prefix, expTime_sec, framesPerDir, startFrame, subDirLimit);
            cmd.start();
            while (cmd.thread.isAlive()) {
                Util.sleep(100);
            }
            if (null != cmd.error) {
                System.out.println("ERROR: "+cmd.error);
                try (Jedis jedis = Redis.get()) {
                    jedis.hset(Redis.collect_state, "info", "ERROR: "+cmd.error);
                }
                result = RPCResult.ERROR;
            } else {
                result = RPCResult.OK;
            }
        } catch (Exception e) {
            e.printStackTrace();
            result = RPCResult.ERROR;
        }
    }

    @Override
    public String jsonStatus()
    {
        return "id="+id+" result="+result;
    }
}
