package gov.anl.gmca.spotfinder;

import gov.anl.gmca.common.*;
import gov.anl.gmca.util.*;
import gov.anl.gmca.util.ExecTask2.*;
import org.apache.commons.exec.*;
import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import gov.anl.gmca.collect.CmdCollectSequence;
import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.collect.HeaderCheck;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.spotfinder.DevSFType.SFConfig;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_TYPE;
import gov.anl.gmca.spotfinder.SFData.SFType;
import gov.anl.gmca.spotfinder.SpotFinderURL.SFURLInfo;
import gov.anl.gmca.syslog.Log;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.*;
import java.util.concurrent.*;

public class CmdSpotFinderRemote extends CmdSpotFinder
{
	private String remoteHost = "";
	public ArrayList<String> cmd;
	public String errMsg;
	private static int nextRemoteHostIndex = 0;
	private SFConfig sfConfig;
	public enum RESULT { NOT_STARTED, STARTED, SUCCESS, ABORTED, TIMEOUT_NO_FILE, TIMEOUT_PARTIAL_FILE, HEADER_CHECK_FAILED, SF_RETURN_ERROR, SF_PARSE_ERROR }
	
	public CmdSpotFinderRemote(SFType sfType, SFRunType sfRunType, int runIndex, int imageIndex, String dir, ArrayList<String> imgNames, String prefix, int runLabel, double lowLimit_A, double highLimit_A, int retries, SPOTFINDER_TYPE sf_type, ArrayList<HeaderCheck.Values> headerValueList, int timeoutMs)
	{
		super(sfType, sfRunType, runIndex, imageIndex, dir, imgNames, prefix, runLabel, lowLimit_A, highLimit_A, retries, sf_type, headerValueList, timeoutMs);
		isLocal = false;

		SFURLInfo info = new SFURLInfo();
		info.sfConfig 		= DevSFType.getConfig(sf_type);
		//info.remoteHost 	= getNextRemoteHost(info.sfConfig.hosts);
		info.imageDir 		= directory;
		info.cbfFilenames 	= imageNames;
		info.frameIndex		= imageIndex;
		info.lowLimit_A		= lowLimit_A;
		info.highLimit_A	= highLimit_A;
		info.sfType			= sfType;
		info.sfRunType		= sfRunType;
		info.timeoutMs		= timeoutMs;
		cmd = SpotFinderURL.generateURL(info);

		// save for output parsing
		this.sfConfig = info.sfConfig;
	}
	
	private static int imageNum=1;
	@Override protected void runSpotFinder()
	{
		//System.out.println("Starting SpotFinder processing for "+imageName+" type="+sf_type);

		try
		{
			// wait for files to appear on disk
			long waitStartMs = System.currentTimeMillis();
			boolean missingFile;
			do {
				Util.sleep(500);

				boolean timeout = false;
				if (stopFlag.get()) { saveResult(RESULT.ABORTED); return; }
				long waitSoFarMs = System.currentTimeMillis() - waitStartMs;
				if (waitSoFarMs > timeoutMs) {
					timeout = true;
				}

				missingFile = false;
				for (String filename : imageNames) {
					File f = new File(directory + "/" + filename);
					if (!(f.isFile() && f.length() > 2000)) {
						missingFile = true;
						if (timeout) {
							System.out.println("Timed out waiting for " + filename);
							saveResult(RESULT.TIMEOUT_NO_FILE);
							return;
						} else if (waitSoFarMs>=3000){
							System.out.println("Waiting for "+filename+" for "+waitSoFarMs+"ms");
						}
						break;
					}
				}
			}
			while (missingFile);

			// run header check
			if (null!=headerValueList && !Main.FAKE_DATA) {
				for (int i=0;i<imageNames.size();i++)
				{
					String result = HeaderCheck.check(directory + "/" + imageNames.get(i), headerValueList.get(i));
					if (!result.equals("OK")) {
						Util.popupError("Header check failed for "+imageNames.get(i), result);
						DevFPE.get().abort();
						saveResult(RESULT.HEADER_CHECK_FAILED);
						return;
					}
				}
			}

			long startMs = System.currentTimeMillis();
			int totalTimeoutSec = (int)DBConfig.getDouble("spotfinder_remote_retry_sec");
			while (true)
			{
				// run spot finder
				System.out.println(String.join(" ",cmd));
			    saveResult(RESULT.STARTED);
				ExecResult result = ExecTask2.execCmd(cmd, "TIMEOUT_MS="+timeoutMs, false, null, 0, true);
				if (result.getExitCode()!=0) sfFailed=true;

			    // read response
			    if (!sfFailed)
			    {
					cmdOutput = result.getLines();
					RESULT r = SpotFinderParser.parseAndSave(sf_type, cmdOutput, directory, prefix, runLabel, imageIndex);
					saveResult(r);
					if (r != RESULT.SUCCESS) sfFailed = true;
				}

			    // handle success or failure
			    if (sfFailed)
			    {
					// check for timeout
					long waitMs = System.currentTimeMillis()-startMs;
					if (waitMs > totalTimeoutSec*1000)
					{
						System.out.println("ERROR: SpotFinder for "+String.join(",",imageNames)+" timed out for batch starting with "+imageNames.get(0)+", giving up.");
						break;
					}

			    	if (retriesLeft<=0) {
						System.out.println("ERROR: SpotFinder for "+String.join(",",imageNames)+" out of retries, quitting.");
						break;
					}
			    	retriesLeft--;
			    }
			    else
			    {
			    	break; // success
			    }

				Util.sleep(10000); // wait 10 seconds before retrying
			}
		}
		catch (Exception e) { e.printStackTrace(); }
	}

	private void saveResult(RESULT result)
	{
		String rel_dir = DevDataDir.getRelative(directory);
		String sf_str = sfConfig.id.toRedisString();
		String key = Redis.image_stats(rel_dir,prefix,runLabel,"")+sf_str+":result";
		DBImageStats.save(key, imageIndex, result.ordinal());
	}

	private String allOutput()
	{
		return String.join("\n",cmdOutput);
	}
	
	private String getNextRemoteHost(List<String> hostList)
	{
		nextRemoteHostIndex++;
		nextRemoteHostIndex %= hostList.size();
		
		String ret = hostList.get(nextRemoteHostIndex);
		return ret;
	}
}