package gov.anl.gmca.spotfinder;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonSyntaxException;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.io.hw.Database;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.util.ListenerList;
import gov.aps.jca.dbr.DBR;
import org.jdbi.v3.core.Jdbi;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class DevSFOptions
{
	public static class SPOTFINDER_OPTION implements Serializable
	{
		private String url_key, spotfinder_id;
		private String eiger_default, pilatus_default, mar_default;
		private String help_text;
		public String getUrl_key() { return url_key; }
		public void setUrl_key(String url_key) { this.url_key = url_key; }
		public String getSpotfinder_id() { return spotfinder_id; }
		public void setSpotfinder_id(String spotfinder_id) { this.spotfinder_id = spotfinder_id; }
		public String getEiger_default() { return eiger_default; }
		public void setEiger_default(String eiger_default) { this.eiger_default = eiger_default; }
		public String getPilatus_default() { return pilatus_default; }
		public void setPilatus_default(String pilatus_default) { this.pilatus_default = pilatus_default; }
		public String getMar_default() { return mar_default; }
		public void setMar_default(String mar_default) { this.mar_default = mar_default; }
		public String getHelp_text() { return help_text; }
		public void setHelp_text(String help_text) { this.help_text = help_text; }
	}

	public static ListenerList<DevSFOptions> llCollectChange, llRasterChange;
	public static ListenerList<DevSFOptions> getListenerList(SFRunType sfRunType) { return sfRunType==SFRunType.COLLECT?llCollectChange:llRasterChange; }

	public static List<SPOTFINDER_OPTION> options = new ArrayList<>();
	public static HashMap<String,String> collectValues = new HashMap<>(), rasterValues = new HashMap<>();
	public static HashMap<String,String> stringMap = new HashMap<>();

	private static Gson gson = new GsonBuilder().create();
	private static Jdbi jdbi = Jdbi.create("jdbc:mysql://"+Database.getSqlHost()+"/blc2004?user=dhs");

	public static HashMap<String,String> getValues(SFRunType sfRunType)
	{
		return sfRunType==SFRunType.COLLECT?collectValues:rasterValues;
	}

	public static void init()
	{
		options = jdbi.withHandle(handle ->
		{
			return handle.createQuery("select * from SpotFinderOptions")
				.mapToBean(SPOTFINDER_OPTION.class)
				.list();
		});

		llCollectChange = new ListenerList<>(null);
		Epics.addMonitor(PVLookup.PV_BLUICE.sfCollectOptions, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				loadJson(SFRunType.COLLECT, Epics.dbrToString(value));
				llCollectChange.notifyListeners();
			}
		});
		loadJson(SFRunType.COLLECT, Epics.getString(PVLookup.PV_BLUICE.sfCollectOptions));
		llCollectChange.notifyListeners();

		llRasterChange = new ListenerList<>(null);
		Epics.addMonitor(PVLookup.PV_BLUICE.sfRasterOptions, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				loadJson(SFRunType.RASTER, Epics.dbrToString(value));
				llRasterChange.notifyListeners();
			}
		});
		loadJson(SFRunType.RASTER, Epics.getString(PVLookup.PV_BLUICE.sfCollectOptions));
		llRasterChange.notifyListeners();
	}

	private static void loadJson(SFRunType sfRunType, String json)
	{
		try {
			switch (sfRunType) {
				case COLLECT:
					collectValues = gson.fromJson(json, stringMap.getClass());
					if (null==collectValues) collectValues = new HashMap<>();
					break;
				case RASTER:
					rasterValues = gson.fromJson(json, stringMap.getClass());
					if (null==rasterValues) rasterValues = new HashMap<>();
					break;
			}
		} catch (JsonSyntaxException e) { e.printStackTrace(); }
	}

	public static PVName getOptionsPV(SFRunType sfRunType)
	{
		return sfRunType==SFRunType.COLLECT?PV_BLUICE.sfCollectOptions: PV_BLUICE.sfRasterOptions;
	}

	public static PVName getSFPV(SFRunType sfRunType)
	{
		return sfRunType==SFRunType.COLLECT?PV_BLUICE.collectSpotFinder: PV_BLUICE.rasterSpotFinder;
	}

	public static void setSettings(SFRunType sfRunType, HashMap<SPOTFINDER_OPTION,String> values)
	{
		HashMap<String,String> stringMap = new HashMap<>();
		for (SPOTFINDER_OPTION option : values.keySet()) stringMap.put(option.url_key,values.get(option));
		String json = gson.toJson(stringMap);
		Epics.put(getOptionsPV(sfRunType), json);
	}
}
