package gov.anl.gmca.spotfinder;

import com.github.wnameless.json.flattener.*;
import com.mashape.unirest.http.*;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.Redis;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.spotfinder.CmdSpotFinderRemote.*;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_TYPE;
import gov.anl.gmca.util.*;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.util.*;

public class SpotFinderParser
{
	public static CmdSpotFinderRemote.RESULT parseAndSave(SPOTFINDER_TYPE type, ArrayList<String> cmdOutput, String fullDir, String prefix, int runLabel, int firstImageRunSeqIndex)
	{
		switch (type)
		{
		case DIALS_SPOTFINDER:
			return parseNewDIALS(cmdOutput, fullDir, prefix, runLabel, firstImageRunSeqIndex);
		case DOZOR:
			return parseDozor(cmdOutput, fullDir, prefix, runLabel, firstImageRunSeqIndex);
		}
		throw new IllegalStateException("Unknown SF type: "+type.toString());
	}

	private static CmdSpotFinderRemote.RESULT parseDozor(ArrayList<String> cmdOutput, String fullDir, String prefix, int runLabel, int firstImageRunSeqIndex)
	{
		boolean success=false;
		for (String line : cmdOutput)
		{
			try {
				StringTokenizer st = new StringTokenizer(line," ");
				if (st.countTokens() != 4) continue;
				if (!st.nextToken().equals("dozor_result:")) continue;
				int imageIndex = Integer.parseInt(st.nextToken())-1+firstImageRunSeqIndex;
				float score = Float.parseFloat(st.nextToken());
				float res = Float.parseFloat(st.nextToken());
				success=true;

				String rel_dir = DevDataDir.getRelative(fullDir);
				String key = Redis.image_stats(rel_dir,prefix,runLabel,"");
				DBImageStats.save(key+"dozor:score", imageIndex, score);
				DBImageStats.save(key+"dozor:res", 	 imageIndex, res);
			} catch (Exception e) { return RESULT.SF_PARSE_ERROR; }
		}
		return success?RESULT.SUCCESS:RESULT.SF_PARSE_ERROR;
	}

	/*
	Example result:
	{
		"image": "/mnt/beegfs/testdata/PC/PC_000001.cbf",
		"d_min_distl_method_1": -1.0,
		"d_min_distl_method_2": -1.0,
		"estimated_d_min": -1.0,
		"n_spots_4A": 0,
		"n_spots_no_ice": 0,
		"n_spots_total": 0,
		"noisiness_method_1": -1.0,
		"noisiness_method_2": -1.0,
		"total_intensity": 0.0
	}
	Example error:
	{
		"image": "/mnt/beegfs/testdata/PC/PC_000001.cbf",
		"error": "dxtbx Internal Error: /scratch/jenkins_agent/workspace/dials_patch_release/build_dials/modules/dxtbx/src/dxtbx/boost_python/ext.cpp(182): DXTBX_ASSERT(nn == (slow * fast)) failure."
	}
	*/
	public static HashMap<Object,Object> objectMap = new HashMap<>();
	public static CmdSpotFinderRemote.RESULT parseNewDIALS(ArrayList<String> cmdOutput, String fullDir, String prefix, int runLabel, int firstImageRunSeqIndex)
	{
		Gson gson = new GsonBuilder().create();
		HashMap<Object, Object> result = gson.fromJson(cmdOutput.get(0), objectMap.getClass());
		if (result.containsKey("error")) {
			return RESULT.SF_PARSE_ERROR;
		}

		try {
			int imageIndex = firstImageRunSeqIndex;
			String rel_dir = DevDataDir.getRelative(fullDir);
			String key = Redis.image_stats(rel_dir, prefix, runLabel, "");
			DBImageStats.save(key + "dials:spots", imageIndex, Float.parseFloat(""+result.get("n_spots_total")));
			DBImageStats.save(key + "dials:spots_no_ice", imageIndex, Float.parseFloat(""+result.get("n_spots_no_ice")));
			DBImageStats.save(key + "dials:total_int", imageIndex, Float.parseFloat(""+result.get("total_intensity")));
			System.out.println(String.format("parseDIALS: image index %d stats=%.0f %.0f %.0f",
					imageIndex,
					Float.parseFloat(""+result.get("n_spots_total")),
					Float.parseFloat(""+result.get("n_spots_no_ice")),
					Float.parseFloat(""+result.get("total_intensity"))));
			return RESULT.SUCCESS;
		} catch (Exception e) {
			return RESULT.SF_PARSE_ERROR;
		}
	}

	// finds key/value pairs separated by colons, which DISTL and DIALS both use
	public static HashMap<String,String> parseDISTLorDIALS(ArrayList<String> cmdOutput)
	{
		HashMap<String,String> ret = new HashMap<>();
		
		// handle error with no spots
		if (containsDIALSError(cmdOutput)) return null;

	    for (String line : cmdOutput)
	    {
			if (line.indexOf(':') != -1)
			{
				StringTokenizer st = new StringTokenizer(line,":");
				if (!st.hasMoreTokens()) continue;
				String key = st.nextToken().trim();
				if (!st.hasMoreTokens()) continue;
				String val = st.nextToken().trim();
				ret.put(key,val);
			}
	    }
	    
	    return ret;
	}

	private static boolean containsDIALSError(ArrayList<String> output)
	{
		if (output.size()>0 && output.get(0).contains("Sorry")) return true;
		return false;
	}
}
