package gov.anl.gmca.spotfinder;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_TYPE;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgSpotFinderOptions extends Composite
{
	private Combo comboSFSelection;
	private Button btnSFParam, btnSFTest;
	private static final int ROW_HEIGHT = 18;
	private SFRunType sfRunType;
	
	public WdgSpotFinderOptions(Composite parent, SFRunType sfRunType)
	{
		super(parent, SWT.NONE);
		this.sfRunType=sfRunType;
		setLayout(new FillLayout());
		
		Group sfParamGroup = new Group(this, SWT.NONE);
		sfParamGroup.setBackground(GUI.BGCOLOR);
		sfParamGroup.setForeground(GUI.COLOR_BLACK);
		sfParamGroup.setFont(GUI.FONT_COLLECT_GROUP);
		sfParamGroup.setText("SpotFinder");
		GridLayout gl = new GridLayout(2,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		gl.verticalSpacing=0;
		gl.horizontalSpacing=0;
		sfParamGroup.setLayout(gl);
		
		comboSFSelection = new Combo(sfParamGroup, SWT.READ_ONLY | SWT.FLAT);
		for (String name : DevSFType.getNames(sfRunType))
		{
			comboSFSelection.add(name);
		}
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
		comboSFSelection.setLayoutData(gd);
		comboSFSelection.select(DevSFType.getConfigIndex(sfRunType));
		
		btnSFParam = new Button(sfParamGroup, SWT.PUSH);
		btnSFParam.setFont(GUI.FONT_10);
		btnSFParam.setText("Change settings");
		btnSFParam.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = ROW_HEIGHT;
		btnSFParam.setLayoutData(gd);
		
		btnSFTest = new Button(sfParamGroup, SWT.PUSH);
		btnSFTest.setFont(GUI.FONT_10);
		btnSFTest.setText("Test");
		btnSFTest.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = ROW_HEIGHT;
		btnSFTest.setLayoutData(gd);
		
		updateState();
	}
	
	public void connectListeners()
	{
		btnSFParam.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
			@Override public void widgetSelected(SelectionEvent e)
			{
				DlgSFOptions.get(sfRunType).open();
			}
		});
		
		btnSFTest.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
			@Override public void widgetSelected(SelectionEvent e)
			{
				new CmdSFTest(sfRunType).start();
			}
		});
		
		DevSFType.get(sfRunType).llChange.addListener(new ListenerList.Listener<DevSFType>()
		{
			@Override public void handleEvent(DevSFType object, Object data)
			{
				updateState();
			}
		});
		
		comboSFSelection.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				int i = comboSFSelection.getSelectionIndex();
				DevSFType.setConfigIndex(sfRunType, i);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateState();
			}
		});

		ModelCollectRunList.get().addCurrentRunListener((run)->updateState());
		ModelRasterState.get().llCurrentRun.addListener((obj,data)->updateState());
	}
	
	private void updateState()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			boolean isMaster = Client.isMaster();
			int sfTypeIndex = DevSFType.getConfigIndex(sfRunType);
			boolean enable;

			switch (sfRunType)
			{
			case COLLECT:
				ModelCollectRun currentRun = ModelCollectRunList.get().getCurrentRunModel();
				switch (currentRun.getScanMode())
				{
				case SINGLE:
				case VECTOR:
					enable = isMaster;
					break;
				case MESH:
					ModelRasterRun meshRasterRun = (ModelRasterRun)ModelRasterRunList.get(ModelRasterState.RasterListType.INTER).getRun(currentRun.getRasterRun());
					enable = isMaster && meshRasterRun.getRunType() != ModelRasterRun.RunType.SONICC;
					break;
				default:
					enable = isMaster;
					break;
				}
				break;
			case RASTER:
				ModelRasterRun currentRasterRun = ModelRasterState.get().getCurrentRunModel();
				switch (currentRasterRun.getRunType())
				{
				case DIFFRACTION:
					enable = isMaster;
					break;
				case SONICC:
					enable = false;
					break;
				default:
					enable = isMaster;
					break;
				}
				break;
			default:
				enable = isMaster;
				break;
			}

			comboSFSelection.select(sfTypeIndex);
			comboSFSelection.setEnabled(enable);
			btnSFParam.setEnabled(enable);
			btnSFTest.setEnabled(enable);
		}});
	}
}
