/**
 * This class writes input.xml and run_<image_name>.sh script
 * to sun WebIce csh scripts
 */

package gov.anl.gmca.strategy;

import gov.anl.gmca.analysis.helpers.MiscUtils;
import gov.anl.gmca.common.Command;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.diffimage.MarImageHeader;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.screening.DevSilTable;
import gov.anl.gmca.strategy.ModelAutoindexResult.IndexSolution;
import gov.anl.gmca.strategy.ModelAutoindexResult.IntegrationSolution;
import gov.anl.gmca.strategy.ModelLaueChoices.LaueSpaceGroups;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.StringTokenizer;

public class CmdCollectStrategy extends Command {

	public String masterFilename1;
	public String masterFilename2;
	public String cbfFilename1, cbfFilename2;
	public String workDir;
	public String imgDir;
	public String sampleName = "";
	private ExecTask task;
	private double detectorHeight = 300.0;
	private double webiceBeamX = 150.0;
	private double webiceBeamY = 150.0;
	private int sampleNo;
	private int sampNumber;
	boolean isScreening = false;
	public String strategyProgram = DevUserConfig.strategyProgram;
	protected static MarImageHeader firstImage = null;

	private DevStrategyResults devStrategyResults = new DevStrategyResults();
	
	private ModelAutoindexResult _autoindexResult;
	
	protected static final String cmdLabelit = Main.getAppDir()
			+ "/tools/scripts/webice/run_autoindex.csh";
	protected static final String scriptsDir = Main.getAppDir()
			+ "/tools/scripts/webice";

	public CmdCollectStrategy
			(
					String _workDir,
					String masterFileName1,
					String masterFileName2,
					String cbfFilename1,
					String cbfFilename2
			) {
		this.masterFilename1 = masterFileName1;
		this.masterFilename2 = masterFileName2;
		this.cbfFilename1 = cbfFilename1;
		this.cbfFilename2 = cbfFilename2;
		imgDir = _workDir;

		workDir = MiscUtils.getProcessDir(imgDir, MiscUtils.analysisSubDirectory) + "/" + cbfFilename1 + "_strategy";
		firstImage = new MarImageHeader(_workDir + "/" + cbfFilename1);
		firstImage.getHeader();
	}

	public void setScreening(boolean _isScreening, int _sampleNo,
							 String _sampleName) {
		isScreening = _isScreening;
		sampleNo = _sampleNo;
		sampleName = _sampleName;
	}

	public void setSampleName(String _sampleName) {
		sampleName = _sampleName;
	}

	@Override
	public void run() {
		
		_autoindexResult = new ModelAutoindexResult();

		/* check for working directory of strategy */
		Log.note("NB: workDir " + workDir);
		File dirFile = new File(workDir);
		if (!dirFile.exists()) {

			if (!dirFile.mkdirs()) {
				Log.error("ERROR: Couldn't create directory " + workDir);
				return;
			}
		}
		/* Run the webice scripts to show in collect tab */
		if (Main.SIM_LEVEL >= 1 || Main.FAKE_DATA) {
			Util.sleep(5000);
			if(masterFilename1.contains(".cbf")) {
				ExecTask.syncExec("cp /mnt/beegfs/testdata/QXU/AEF_Se_MalA_1_2_1_000001.cbf "
						+ imgDir + "/" + masterFilename1, null, false);
				ExecTask.syncExec("cp /mnt/beegfs/testdata/QXU/AEF_Se_MalA_1_2_1_000091.cbf "
						+ imgDir + "/" + masterFilename2, null, false);
			} else {
				ExecTask.syncExec("cp /mnt/staffhome/spothineni/strat.0001 "
					+ imgDir + "/" + masterFilename1, null, false);
				ExecTask.syncExec("cp /mnt/staffhome/spothineni/strat.0002 "
					+ imgDir + "/" + masterFilename2, null, false);
			}
		}

		StrategyParams strategyResult = new StrategyParams();
		ModelLaueChoices laueChoices = new ModelLaueChoices();

		//_autoindexResult.image1 = masterFilename1;
		//_autoindexResult.image2 = masterFilename2;
		_autoindexResult.image1 = cbfFilename1;
		_autoindexResult.image2 = cbfFilename2;
		_autoindexResult.workDir = workDir;
		_autoindexResult.sampleName = sampleName;
		_autoindexResult.isScreening = isScreening;
		
                Log.note("NB: get PV");
		sampNumber = Epics.getInteger(PV_SCR.dbUpdated);
		sampNumber = devStrategyResults.getRowCount() + 1;

		detectorHeight = DetectorConfig.getDimensionsMM().y;
		
		// pixels from top left
		webiceBeamX = firstImage.displayOriginX;
		webiceBeamY = firstImage.displayOriginY;
		
		// convert to mm
		//Point2D pixelSizeMM = DetectorConfig.getPixelSizeMM();
		//webiceBeamX *= pixelSizeMM.x;
		//webiceBeamY *= pixelSizeMM.y;
			
		_autoindexResult.state = "PROCESSING";

		String mount = Epics.getString(PV_ROBOT.mounted);

		if (isScreening) {

			_autoindexResult.screeningSample = mount;
			strategyResult.predictedResolution = "Processing";
			DevSilTable.get().setResult(sampleNo, _autoindexResult, strategyResult, true);
		}

		devStrategyResults.setResult(sampNumber, _autoindexResult, true);
		
		/* write index.xml using image header from imgsrv */
		Log.note("NB: write XML");
		writeInputXML();

		/* run Labeit.index on two images collected */
		Log.note("Strategy : Labelit.index started");

		BufferedWriter commandFileWriter;
		String commandFilePath = workDir + "/" + "run_" + cbfFilename1 + ".sh";
		try {
			commandFileWriter = new BufferedWriter(new FileWriter(commandFilePath, false));
			File commandFile = new File(commandFilePath);
			commandFile.setExecutable(true);
			commandFileWriter.write("#!/usr/bin/tcsh\n" + "tcsh -f " + cmdLabelit + " " + scriptsDir + " "
					+ workDir + " input.xml");
			commandFileWriter.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		/* submit the strategy job to Grid Engine using DRMAA if environment
		 * available, if not run locally
		 */
		int jobStatus = 0;

		//String cmd = "qsub -j y -sync y -wd " + workDir + " " + commandFilePath;
		String cmd = "sbatch --export=LD_PRELOAD= --wait --chdir " + workDir + " " + commandFilePath;

		//cmd = commandFilePath;
		task = ExecTask.new_ExecTask(cmd, "Autoindex-> Labelit ", false, false);
		task.start();
		task.waitFor();

		/*
		DRMAA drmaa;
		boolean isSubmitHost = CheckSubmitHost.get().isSubmitHost();
		System.out.println("Gridengine env detected?: "+isSubmitHost);
		//if (true) {//!isSubmitHost) {
		if (!isSubmitHost) {//!isSubmitHost) {
			task = ExecTask.new_ExecTask(commandFilePath, "Autoindex-> Labelit ", false, false);
			task.start();
			task.waitFor();
		} else {
			Log.note("Running strategy job with Grid Engine");
			try {
				drmaa = new DRMAA(commandFilePath, workDir, WdgStrategy.TIMEWAIT_FOR_STRATEYJOB);
				drmaa.start();
				jobStatus = drmaa.waitFor();
			} catch (NoClassDefFoundError  e) {
				System.out.println(e.getMessage());
				quitWithException(new Exception());
				return;
			} catch (UnsatisfiedLinkError e) {
				System.out.println(e.getMessage());
				quitWithException(new Exception());
				return;
			} catch (DrmaaException e) {
				System.out.println(e.getMessage());
				quitWithException(e);
				return;
			} catch (Exception e) {
				System.out.println(e.getMessage());
				quitWithException(e);
				return;
			} 
			
		}
		*/
		
		if (jobStatus == 1) {
			_autoindexResult.state = "ERROR";
			_autoindexResult.warning = "ERROR : Strategy job timed out";
			devStrategyResults.setAutoindexResult(sampNumber, _autoindexResult, true);
			return;
		} else {
			try {
				_autoindexResult.processResult(workDir);
				laueChoices.processResult(workDir);
				/* replace special characters in warning */
				if (_autoindexResult.warning.contains("'")) {
					_autoindexResult.warning = _autoindexResult.warning.replace("'", "");
				}
				
				/* write data to MySQL table StrategyResults */
				_autoindexResult.state = "DONE";
				if (_autoindexResult.warning.equals("processing")) {
					_autoindexResult.warning = "";
				}

				devStrategyResults.setAutoindexResult(sampNumber, _autoindexResult, true);
				
				if (isScreening) {
					if(strategyResult.predictedResolution.equals("Processing"))
						strategyResult.predictedResolution = "N/A";
					DevSilTable.get().setResult(sampleNo, _autoindexResult,
						strategyResult, true);
				}

				int writeToScreeningOnlyOnce = 0;
				/* retrieve the best solutions and store in mysql table */
				for (int i = 0; i < laueChoices.laueSpaceGroups.size(); i++) {
					LaueSpaceGroups laueSpaceGroup = (LaueSpaceGroups) laueChoices.laueSpaceGroups
							.get(i);
					IndexSolution indexSol = (IndexSolution) _autoindexResult
							.getIndexSolution(Integer
									.parseInt(laueSpaceGroup.solutionNum));

					if (indexSol == null)
						continue;
					if (indexSol.filter == true)
						continue;

					IntegrationSolution integrationSol = (IntegrationSolution) _autoindexResult
							.getIntegrationSolution(Integer
									.parseInt(laueSpaceGroup.solutionNum));
					FileStrategyResult _strategyResult = new FileStrategyResult();
					StringTokenizer tok = new StringTokenizer(
							laueSpaceGroup.SpaceGroups);
					String spaceGroup = tok.nextToken();
					
					/* R3:H spacegroup is actually H3 */
					if (spaceGroup.equals("R3:H"))
						spaceGroup = "H3";
					if (spaceGroup.equals("R32:H"))
						spaceGroup = "H32";

					int error = _strategyResult.read(workDir + "/"
							+ indexSol.strategyFile + "/strategy_summary_"
							+ spaceGroup + ".xml");

					if (error == 1) {
						error = _strategyResult.read(workDir + "/"
								+ indexSol.strategyFile
								+ "/strategy_summary.xml");
					}

					if (error == 1) {
						
						strategyResult = new StrategyParams();
						strategyResult.spaceGroup = "("
								+ laueSpaceGroup.solutionNum + ") "
								+ laueSpaceGroup.SpaceGroups + "-->"
								+ indexSol.unitCellA + "," + indexSol.unitCellB
								+ "," + indexSol.unitCellC + ","
								+ indexSol.unitCellAlpha + ","
								+ indexSol.unitCellBeta + ","
								+ indexSol.unitCellGamma;
						strategyResult.spaceGroup_short = spaceGroup;
						if (integrationSol != null
								&& integrationSol.solutionNum == indexSol.solutionNum)
							strategyResult.good = "yes";
						else
							strategyResult.good = "no";

						strategyResult.software = strategyProgram;
						strategyResult.solutionNum = laueSpaceGroup.solutionNum;
						strategyResult.unitCellA = indexSol.unitCellA;
						strategyResult.unitCellB = indexSol.unitCellB;
						strategyResult.unitCellC = indexSol.unitCellC;
						strategyResult.unitCellAlpha = indexSol.unitCellAlpha;
						strategyResult.unitCellBeta = indexSol.unitCellBeta;
						strategyResult.unitCellGamma = indexSol.unitCellGamma;
						if (i == laueChoices.laueSpaceGroups.size() - 1)
							devStrategyResults.setStrategyResult(sampNumber,
									_autoindexResult, strategyResult, true);
						else
							devStrategyResults.setStrategyResult(sampNumber,
									_autoindexResult, strategyResult, false);
						continue;
					}
					if (i == 0) {
						if (!_strategyResult.strategyResults.get(0).detectorWarning
								.equals("")) {
							_autoindexResult.warning += _strategyResult.strategyResults
									.get(0).detectorWarning;
							devStrategyResults.setAutoindexResult(sampNumber,
									_autoindexResult, false);
						}
					}
					for (int j = 0; j < _strategyResult.spaceGroups; j++) {
						//_strategyResult.strategyResults.get(j).software = strategyProgram;
						_strategyResult.strategyResults.get(j).solutionNum = indexSol.solutionNum;
						_strategyResult.strategyResults.get(j).rmsd = indexSol.rmsd;
						_strategyResult.strategyResults.get(j).spaceGroup = "("
								+ laueSpaceGroup.solutionNum + ") "
								+ laueSpaceGroup.SpaceGroups + "-->"
								+ indexSol.unitCellA + "," + indexSol.unitCellB
								+ "," + indexSol.unitCellC + ","
								+ indexSol.unitCellAlpha + ","
								+ indexSol.unitCellBeta + ","
								+ indexSol.unitCellGamma;
						_strategyResult.strategyResults.get(j).spaceGroup_short = spaceGroup;
						if (integrationSol != null) {
							_strategyResult.strategyResults.get(j).good = "yes";

							if (Integer.parseInt(indexSol.solutionNum) != _autoindexResult.indexResults
									.size())
								_autoindexResult.warning = "LABELIT best solution is "
										+ indexSol.solutionNum
										+ ". This is not highest symmetry solution";
							devStrategyResults.setAutoindexResult(sampNumber,
									_autoindexResult, false);
						} else
							_strategyResult.strategyResults.get(j).good = "no";
						
						strategyResult.software = _strategyResult.strategyResults.get(j).software;

						_strategyResult.strategyResults.get(j).unitCellA = indexSol.unitCellA;
						_strategyResult.strategyResults.get(j).unitCellB = indexSol.unitCellB;
						_strategyResult.strategyResults.get(j).unitCellC = indexSol.unitCellC;

						_strategyResult.strategyResults.get(j).unitCellAlpha = indexSol.unitCellAlpha;
						_strategyResult.strategyResults.get(j).unitCellBeta = indexSol.unitCellBeta;
						_strategyResult.strategyResults.get(j).unitCellGamma = indexSol.unitCellGamma;

						double startAngle = Double
								.valueOf(_strategyResult.strategyResults.get(j).oscStart);
						double endAngle = Double
								.valueOf(_strategyResult.strategyResults.get(j).oscEnd);

						int startAngleInt = (int) startAngle;
						int endAngleInt = (int) endAngle;

						if (startAngleInt == 0 && endAngleInt == 0) {
							_strategyResult.strategyResults.get(j).good = "no";
							_autoindexResult.warning = _strategyResult.strategyResults
									.get(j).bestWarn + "\n";
							
							devStrategyResults.setAutoindexResult(sampNumber,
									_autoindexResult, true);
						}

						_autoindexResult.state = "DONE";
						if (i == laueChoices.laueSpaceGroups.size() - 1)
							devStrategyResults.setStrategyResult(sampNumber,
									_autoindexResult,
									_strategyResult.strategyResults.get(j),
									true);
						else
							devStrategyResults.setStrategyResult(sampNumber,
									_autoindexResult,
									_strategyResult.strategyResults.get(j),
									false);
						writeToScreeningOnlyOnce++;
						
						if (isScreening
								&& writeToScreeningOnlyOnce == 1
								&& _strategyResult.strategyResults.get(j).good
										.equals("yes"))
							
							DevSilTable.get().setResult(sampleNo, _autoindexResult,
									_strategyResult.strategyResults.get(j),
									true);
					}
				}
				
				/* save the result to spreadsheet */
				ModelStrategyResultList.get().save(DevDataDir.getPrimary() + "/strategyResults_saved.xls");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		Log.note("Strategy calculation finished");
	}

	public double calculateResolution(double d) {
		try {
			double wavelength = firstImage.wavelength;
			double radius = detectorHeight / 2.0;
			double resolution = wavelength
					/ (2.0 * Math.sin(Math.atan(radius / d) / 2.0));
			long t = Math.round(resolution * 1000);
			resolution = t / 1000.0;
			return resolution;
		} catch (Exception e) {
			System.out.println("Autoindex -> strategy : calculation of resolution failed : "
							+ e.getMessage());
			return 1.94;
		}
	}

	public void writeInputXML()
	{
		StringBuffer buf = new StringBuffer();
		buf.append("<input>\n");
		buf.append("  <version>");
		buf.append(2.0);
		buf.append("</version>\n");
		buf.append("  <task name=\"run_autoindex.csh\">\n");
		buf.append("    <runName>");
		buf.append(1);
		buf.append("</runName>\n"); // change later to different runs
		buf.append("    <imageDir>");
		buf.append(imgDir);
		buf.append("</imageDir>\n");
		buf.append("    <host>");
		buf.append(DBConfig.get("webice_host"));
		buf.append("</host>\n");
		buf.append("    <port>");
		buf.append("61001");
		buf.append("</port>\n");
		buf.append("    <image>");
		buf.append(cbfFilename1);
		buf.append("</image>\n");
		buf.append("    <image>");
		buf.append(cbfFilename2);
		buf.append("</image>\n");
		buf.append("    <integrate>");
		buf.append("best");
		buf.append("</integrate>\n");
		buf.append("    <strategyMethod>");
		buf.append(strategyProgram);
		buf.append("</strategyMethod>\n");
		buf.append("    <generate_strategy>");
		buf.append("true");
		buf.append("</generate_strategy>\n");
		buf.append("    <beamCenterX>");
		buf.append(Double.toString(webiceBeamX));
		buf.append("</beamCenterX>\n");
		buf.append("    <beamCenterY>");
		buf.append(Double.toString(webiceBeamY));
		buf.append("</beamCenterY>\n");
		buf.append("    <distance>");
		buf.append(Double.toString(firstImage.distance));
		buf.append("</distance>\n");
		buf.append("    <wavelength>");
		buf.append(Double.toString(firstImage.wavelength));
		buf.append("</wavelength>\n");
		buf.append("    <detector>");
		buf.append(firstImage.detector);
		buf.append("</detector>\n");
		buf.append("    <detectorFormat>");
		buf.append("N/A");
		buf.append("</detectorFormat>\n");
		buf.append("    <detectorWidth>");
		buf.append(detectorHeight); // use larger dimension
		buf.append("</detectorWidth>\n");
		buf.append("    <detectorResolution>");
		buf.append(Double.toString(calculateResolution(firstImage.distance)));
		buf.append("</detectorResolution>\n");
		buf.append("    <exposureTime>");
		buf.append(Double.toString(firstImage.exposureTime));
		buf.append("</exposureTime>\n");
		buf.append("    <oscRange>");
		buf.append(firstImage.osc_range);
		buf.append("</oscRange>\n");
		buf.append("    <beamlineFile>");
		buf.append("");
		buf.append("</beamlineFile>\n");
		buf.append("    <dcsDumpFile>");
		buf.append("");
		buf.append("</dcsDumpFile>\n");
		buf.append("    <expType>");
		buf.append("Monochromatic");
		buf.append("</expType>\n");
		buf.append("    <mad");
		buf.append(" edge=\"");
		buf.append("");
		buf.append("\"");
		buf.append(" inflection=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append(" peak=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append(" remote=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append("/>\n");
		buf.append("    <laueGroup>");
		buf.append("unknown");
		buf.append("</laueGroup>\n");
		buf.append("    <unitCell");
		buf.append(" a=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append(" b=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append(" c=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append(" alpha=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append(" beta=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append(" gamma=\"");
		buf.append("0.0");
		buf.append("\"");
		buf.append("/>\n");
		buf.append("    <numHeavyAtoms>");
		buf.append("0");
		buf.append("</numHeavyAtoms>\n");
		buf.append("    <numResidues>");
		buf.append("0");
		buf.append("</numResidues>\n");
		buf.append("  </task>\n");
		buf.append("</input>\n");

		String filename = workDir + "/input.xml";
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(filename));
			String outText = buf.toString();
			out.write(outText);
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
	private void quitWithException(Exception e) {
		_autoindexResult.state = "ERROR";
		_autoindexResult.warning = "ERROR : Grid Engine not running, report to your host";
		devStrategyResults.setAutoindexResult(sampNumber,
				_autoindexResult, true);
		return;
	}
}
