/**
 * When ever user selects different solution than higher solution from the pull-down on 
 * Strategy sub-tab, strategy for that solution is generated by BEST/Mosflm.
 * 
 * This class runs already existing script strategy<sol_num>.sh
 */

package gov.anl.gmca.strategy;

import gov.anl.gmca.common.Command;
import gov.anl.gmca.diffimage.MarImageHeader;
import gov.anl.gmca.gridengine.CheckSubmitHost;
import gov.anl.gmca.gridengine.DRMAA;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.screening.CmdScreening;
import gov.anl.gmca.screening.DevSilTable;
import gov.anl.gmca.syslog.Log;

import java.io.File;
import java.util.StringTokenizer;

public class CmdUserStrategy extends Command {
	int sampleNumber;
	int solutionNumber;
	String workDir;
	String imgDir;
	ExecTask task;
	int detectorWidth = 300;
	public String strategyProgram = DevUserConfig.strategyProgram;
	boolean isScreening = false;
	int sampleNo = -1;
	String sampleName = "";
	String spaceGroup = "";
	String spaceGroupFull = "";
	String images = "";
	protected static MarImageHeader currentHeader = null;

	DevStrategyResults devStrategyResults = new DevStrategyResults();
	DevSilTable devSilTable = new DevSilTable();

	public CmdUserStrategy(String _workDir, int _sampleNum, int _solutionNum,
			String _sampleName, String _spaceGroup, String _spaceGroupFull,
			String _images) {

		sampleNumber = _sampleNum;
		solutionNumber = _solutionNum;
		imgDir = _workDir;
		workDir = _workDir;
		sampleName = _sampleName;
		spaceGroup = _spaceGroup;
		spaceGroupFull = _spaceGroupFull;
		images = _images;
	}

	@Override
	public void run() {

		File dirFile = new File(workDir);
		Log.note(workDir);
		if (!dirFile.exists()) {

			if (!dirFile.mkdirs()) {
				Log.error("ERROR: Couldn't create directory " + workDir);
				return;
			}
		}

		ModelAutoindexResult _autoindexResult = new ModelAutoindexResult();
		_autoindexResult.state = "PROCESSING";
		_autoindexResult.workDir = workDir;

		String currSamp = ModelStrategyResultList.get().getCurrentResult()
				.getField(ModelStrategyResult.StrategyField.SCREENING_SAMPLE);

		if (currSamp != null && !currSamp.equals("none")
				&& !currSamp.equals("") && !currSamp.equals("-1")) {
			sampleNo = CmdScreening.labelToindex(currSamp);
			devSilTable.setResult(sampleNo, _autoindexResult);
		}

		int sub = images.indexOf(" ");
		_autoindexResult.image1 = images.substring(0, sub);
		_autoindexResult.image2 = images.substring(sub + 1, images.length());
		_autoindexResult.warning = "Processing strategy for solution "
				+ solutionNumber;
		devStrategyResults.setResult(sampleNumber, _autoindexResult, false);
		DevStrategyResults.get().updateRow(sampleNumber, spaceGroupFull, false);

		/* run Labeit.index on two images collected */

		String solNum = Integer.toString(solutionNumber);
		while (solNum.length() < 2)
			solNum = "0" + solNum;
		String command = workDir + "/solution" + solNum + "/strategy" + solNum
				+ ".sh";
		String solDir = workDir + "/solution" + solNum;
		Log.note(command);

		/*check for gridengine environment and quit if not a submit host */
		boolean isSubmitHost = CheckSubmitHost.get().isSubmitHost();
		System.out.println("Gridengine env detected: "+isSubmitHost);
		
		/* submit the strategy job to Grid Engine using DRMAA if environment
		 * available, if not run locally
		 */
		int jobStatus = 0;
		//if (true) {//!isSubmitHost) {
		if (!isSubmitHost) { // !isSubmitHost) {

			Log.warn("Grid Engine not available, strategy job running locally");

			task = ExecTask.new_ExecTask(command, "Strategy-> Labelit ", false,
					false);
			task.start();
			task.waitFor();
		} else {
			try {
				Log.note("Running strategy job with Grid Engine");
				DRMAA drmaa = new DRMAA(command, solDir, WdgStrategy.TIMEWAIT_FOR_STRATEYJOB); // wait 180 sec
																// before
																// quitting job
				drmaa.start();
				jobStatus = drmaa.waitFor();
			} catch (Exception e) {
				_autoindexResult.state = "ERROR";
				_autoindexResult.warning = "GridEgine/DRMAA exception, job is not submitted";
				e.printStackTrace();
				jobStatus = 1;
			}
		}
		if (jobStatus == 1) {
			_autoindexResult.state = "ERROR";
			_autoindexResult.warning = "ERROR : Strategy job timed out";
			devStrategyResults.setAutoindexResult(sampleNumber,
					_autoindexResult, true);
			return;
		} else {

			/* parse labelit.out & strategy_summary.xml */

			try {
				_autoindexResult.processResult(workDir);
				_autoindexResult.sampleName = sampleName;
				/* replace special characters in warning */
				if (_autoindexResult.warning.contains("'"))
					_autoindexResult.warning = _autoindexResult.warning
							.replace("'", "");
				/* write data to MySQL table StrategyResults */
				_autoindexResult.state = "DONE";
				if (_autoindexResult.warning.equals("processing"))
					_autoindexResult.warning = "";
				devStrategyResults.setAutoindexResult(sampleNumber,
						_autoindexResult, true);
				/* retrieve the best solutions and store in mysql table */
				StringTokenizer tok = new StringTokenizer(spaceGroupFull);
				String spaceGroup_first = tok.nextToken();
				spaceGroup_first = tok.nextToken();

				if (spaceGroup.equals("R3:H"))
					spaceGroup = "H3";
				if (spaceGroup.equals("R32:H"))
					spaceGroup = "H32";

				FileStrategyResult _strategyResult = new FileStrategyResult();
				int error = _strategyResult.read(solDir + "/strategy_summary_"
						+ spaceGroup + ".xml");
				if (error == 1) {
					error = _strategyResult.read(solDir
							+ "/strategy_summary.xml");
				}

				if (error == 1) {
					_autoindexResult.state = "ERROR";
					_autoindexResult.warning = solDir + "/strategy_summary_"
							+ spaceGroup + ".xml not found";
					devStrategyResults.setAutoindexResult(sampleNumber,
							_autoindexResult, true);
				}

				if (!_strategyResult.strategyResults.get(0).detectorWarning
						.equals("")) {
					_autoindexResult.warning = _strategyResult.strategyResults
							.get(0).detectorWarning;
					devStrategyResults.setAutoindexResult(sampleNumber,
							_autoindexResult, false);
				}
				//_strategyResult.strategyResults.get(0).software = strategyProgram;
				_strategyResult.strategyResults.get(0).solutionNum = Integer
						.toString(solutionNumber);
				_strategyResult.strategyResults.get(0).good = "yes";
				_strategyResult.strategyResults.get(0).spaceGroup_short = spaceGroup;
				_strategyResult.strategyResults.get(0).sampleName = sampleName;
				double startAngle = Double
						.valueOf(_strategyResult.strategyResults.get(0).oscStart);
				double endAngle = Double
						.valueOf(_strategyResult.strategyResults.get(0).oscEnd);

				int startAngleInt = (int) startAngle;
				int endAngleInt = (int) endAngle;

				if (startAngleInt == 0 && endAngleInt == 0) {

					_autoindexResult.warning = _strategyResult.strategyResults
							.get(0).bestWarn + "\n";
					_strategyResult.strategyResults.get(0).detectorWarning = _strategyResult.strategyResults
							.get(0).bestWarn;
					Log.error(_autoindexResult.warning);
					devStrategyResults.setAutoindexResult(sampleNumber,
							_autoindexResult, false);
				}

				_autoindexResult.state = "DONE";
				devStrategyResults.setStrategyResult(sampleNumber,
						_autoindexResult,
						_strategyResult.strategyResults.get(0), false);
				DevStrategyResults.get().updateRow(sampleNumber,
						spaceGroupFull, true);

				if (!currSamp.equals("-1")
						&& sampleNo != -1
						&& currSamp != null
						&& !currSamp.equals("none")
						&& !currSamp.equals("")
						&& _strategyResult.strategyResults.get(0).good
								.equals("yes"))
					devSilTable.setResult(sampleNo, _autoindexResult,
							_strategyResult.strategyResults.get(0), true);

			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		Log.note("Strategy calculation finished");
	}
}
