/**
 *  this class runs MySQL queries on AutoindexResults,StrategyResults tables form user_data database
 *  
 */

package gov.anl.gmca.strategy;

import gov.anl.gmca.collect.DevCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList.CollectLeftTab;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.log.db.StrategyEvents;
import gov.anl.gmca.log.db.StrategyEvents.StrategyParam;
import gov.anl.gmca.screening.ModelScreening;
import gov.anl.gmca.strategy.ModelStrategyResult.StrategyField;
import gov.anl.gmca.syslog.Log;
import gov.aps.jca.dbr.DBR;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

public class DevStrategyResults {
	public static final String tableName = "AutoindexResults";
	public static final String resultTableName = "StrategyResults";
	protected static final boolean UPDATE_CURRENT_INDEX_ON_RESULT_CREATION = true;
	protected static final boolean SHOW_STRATEGY_TAB_ON_RESULT_CREATION = true;

	protected static boolean doesTableExist() {
		Database.Result result = Database.exec(DBName.USERDATA,
				"show tables like '" + tableName + "'");
		boolean exists = (result.numRows() > 0);
		return exists;
	}

	public synchronized static boolean verifyTableExists() {
		if (!doesTableExist()) {
			System.out.println("Creating Strategy results table");
			String createCmd = "CREATE TABLE `" + tableName + "` ("
					+ "`sampleNumber` int(11) NOT NULL,"
					+ "`sampleName` varchar(30) default '',"
					+ "`screeningSample` varchar(30) default '',"
					+ "`state` varchar(30) default '',"
					+ "`workdir` varchar(250) default '',"
					+ "`images` varchar(512) default '',"
					+ " `score` varchar(30) default '',"
					+ " `resolution` varchar(30) default '',"
					+ " `mosaicity` varchar(50) default '',"
					+ " `warning` varchar(5120) default '',"
					+ " `timestamp` varchar(512) default '',";

			createCmd += " PRIMARY KEY (`sampleNumber`)) DEFAULT CHARSET=utf8";
			// Log.note(createCmd);
			Database.exec(DBName.USERDATA, createCmd);
		}

		Database.Result result1 = Database.exec(DBName.USERDATA,
				"show tables like '" + resultTableName + "'");
		boolean exists1 = (result1.numRows() > 0);

		if (!exists1) {

			String createStrategyCmd = "CREATE TABLE `" + resultTableName
					+ "` (" + "`sampleNumber` int(11) NOT NULL,"
					+ "`sampleName` varchar(512) default '',"
					+ "`solutionNumber` int(11) default NULL,"
					+ "`software` varchar(50) default '',"
					+ "`good` varchar(11) default 'no',"
					+ " `oscstart` varchar(30) default '',"
					+ " `anom_oscstart` varchar(30) default '',"
					+ " `oscend` varchar(30) default '',"
					+ " `anom_oscend` varchar(30) default '',"
					+ " `continuous` varchar(30) default '',"
					+ " `complete` varchar(30) default '',"
					+ " `anom_complete` varchar(30) default '',"
					+ " `oscdelta` varchar(30) default '',"
					+ " `detectordistance` varchar(30) default '',"
					+ " `detectorwarning` varchar(512) default '',"
					+ " `spaceGroup_short` varchar(100) default '',"
					+ " `spaceGroup` varchar(100) default '',"
					+ " `resolution` varchar(30) default '',"
					+ " `unitcell` varchar(100) default '',"
					+ " `rmsd` varchar(30) default '',"
					+ " `beamx` varchar(30) default '',"
					+ " `beamy` varchar(30) default '')";

			// createStrategyCmd +=
			// " PRIMARY KEY (`sampleNumber`)) DEFAULT CHARSET=utf8";
			// Log.note(createStrategyCmd);
			Database.exec(DBName.USERDATA, createStrategyCmd);
		}

		return doesTableExist();
	}

	protected boolean connected = false;
	protected Epics.EpicsValueMonitor dbUpdatedMonitor;

	public static DevStrategyResults get() {
		return inst;
	}

	protected static DevStrategyResults inst = null;

	public DevStrategyResults() {
		inst = this;
	}

	public void verifyConnected() {
		if (connected)
			return;

		updateAllRows();

		Epics.addMonitor(PV_SCR.currentStrategyIndex,
				new Epics.EpicsValueMonitor() {
					@Override
					public void valueChanged(String pvName, DBR value) {
						ModelStrategyResultList.get().setCurrentStrategyIndex(
								Epics.dbrToInt(value));
						ModelStrategyResultList.get().ll.notifyListeners();
					}
				});

		dbUpdatedMonitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				int intVal = Epics.dbrToInt(value);
				if (-1 != intVal)
					handlePVUpdate(intVal);
			}
		};
		Epics.addMonitor(PV_SCR.dbUpdated, dbUpdatedMonitor);

		connected = true;
	}

	public void setResult(int sampleNumber,
			ModelAutoindexResult autoindexResult, boolean update) {
		verifyTableExists();
		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		if (0 == result.numRows()) {
			Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (sampleNumber, sampleName, screeningSample, state, workdir, images, warning) "
							+ "values (" + sampleNumber + ", '"
							+ autoindexResult.sampleName + "', '"
							+ autoindexResult.screeningSample + "', '"
							+ autoindexResult.state + "', '"
							+ autoindexResult.workDir + "', '"
							+ autoindexResult.image1 + " "
							+ autoindexResult.image2 + "','"
							+ autoindexResult.warning + "')");
			newResultCreated = true;
		} else {
			Database.exec(DBName.USERDATA, "update " + tableName + " "
					+ "set state='" + autoindexResult.state + "' "
					+ ", sampleName='" + autoindexResult.sampleName + "' "
					+ ", workdir='" + autoindexResult.workDir + "' "
					+ ", images='" + autoindexResult.image1 + " "
					+ autoindexResult.image2 + "' " + ", warning='"
					+ autoindexResult.warning + "' " + "where sampleNumber="
					+ sampleNumber);
		}
		if (update) {

			// update the GUI directly
			updateRow(sampleNumber);
			// Log.error("setResult "+sampleNumber);
			if (newResultCreated && UPDATE_CURRENT_INDEX_ON_RESULT_CREATION) {
				int last = ModelStrategyResultList.get().getStrategyList()
						.size() - 1;
				DevStrategyResults.setCurrentStrategyIndex(last);
			}
			if (newResultCreated && SHOW_STRATEGY_TAB_ON_RESULT_CREATION) {
				DevCollectRunList.get().setCurrentLeftTab(
						CollectLeftTab.STRATEGY);
			}
			// send the PV update signal
			sendRowUpdate(sampleNumber);
		}
	}

	public void setAutoindexResult(int sampleNumber,
			ModelAutoindexResult autoindexResult, boolean update) {
		verifyTableExists();

		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		if (0 == result.numRows()) {
			Database.exec(DBName.USERDATA, "insert into `" + tableName
					+ "` (sampleNumber,state) " + "values (" + sampleNumber
					+ ",'DONE')");
			newResultCreated = true;
		}

		String updateCmd = "update `" + tableName + "` set ";
		String DATE_FORMAT_NOW = "yyyy-MM-dd-HH:mm:ss";
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
		String timestamp = sdf.format(date);

		updateCmd += "state='" + autoindexResult.state;
		updateCmd += "', sampleName='" + autoindexResult.sampleName;
		updateCmd += "', workdir='" + autoindexResult.workDir;
		updateCmd += "', images='" + autoindexResult.image1 + " "
				+ autoindexResult.image2;
		updateCmd += "', score='" + autoindexResult.score;
		updateCmd += "', resolution='" + autoindexResult.resolution;
		updateCmd += "', mosaicity='" + autoindexResult.mosaicity;
		updateCmd += "', warning='" + autoindexResult.warning;
		updateCmd += "', timestamp='" + timestamp;
		updateCmd += "' where sampleNumber=" + sampleNumber;
		// Log.note(updateCmd);

		Database.exec(DBName.USERDATA, updateCmd);

		if (update) {
			// send the PV update signal
			// updateRow(sampleNumber);
			if (newResultCreated && SHOW_STRATEGY_TAB_ON_RESULT_CREATION) {
				DevCollectRunList.get().setCurrentLeftTab(
						CollectLeftTab.STRATEGY);
			}
			// update the GUI directly
			sendRowUpdate(sampleNumber);

			if (autoindexResult.state.equals("ERROR")
					|| autoindexResult.state.equals("DONE")) {
				updateStrategyEvent(sampleNumber, autoindexResult.isScreening);
			}
		}
	}

	public void setStrategyResult(int sampleNumber,
			ModelAutoindexResult autoindexResult,
			StrategyParams strategyResult, boolean update) {
		verifyTableExists();

		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + resultTableName + "` where sampleNumber="
						+ sampleNumber + " and solutionNumber="
						+ strategyResult.solutionNum
						+ " and spaceGroup_short='"
						+ strategyResult.spaceGroup_short + "'");
		String updateStrategyCmd = "";
		if (result.numRows() > 0) {
			updateStrategyCmd = "update `" + resultTableName + "` set ";
			String DATE_FORMAT_NOW = "yyyy-MM-dd-HH:mm:ss";
			Date date = new Date();
			SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
			String timestamp = sdf.format(date);

			updateStrategyCmd += "good='" + strategyResult.good;
			updateStrategyCmd += "', sampleName='" + autoindexResult.sampleName;
			updateStrategyCmd += "', software='" + strategyResult.software;
			updateStrategyCmd += "', oscStart='" + strategyResult.oscStart;
			updateStrategyCmd += "', anom_oscStart='"
					+ strategyResult.anom_oscStart;
			updateStrategyCmd += "', oscEnd='" + strategyResult.oscEnd;
			updateStrategyCmd += "', anom_oscEnd='"
					+ strategyResult.anom_oscEnd;
			updateStrategyCmd += "', continuous='" + strategyResult.cont_oscEnd;
			updateStrategyCmd += "', complete='" + strategyResult.completeness;
			updateStrategyCmd += "', anom_complete='"
					+ strategyResult.anom_completeness;
			updateStrategyCmd += "', oscDelta='" + strategyResult.oscDelta;
			updateStrategyCmd += "', detectordistance='"
					+ strategyResult.detectorDistance;
			updateStrategyCmd += "', detectorwarning='"
					+ strategyResult.detectorWarning;
			updateStrategyCmd += "', resolution='"
					+ strategyResult.predictedResolution;
			updateStrategyCmd += "', rmsd='" + autoindexResult.rms;
			updateStrategyCmd += "' where sampleNumber=" + sampleNumber
					+ " and solutionNumber=" + strategyResult.solutionNum
					+ " and spaceGroup_short='"
					+ strategyResult.spaceGroup_short + "'";
		} else {
			updateStrategyCmd = "insert into `" + resultTableName + "`";
			updateStrategyCmd += " (sampleNumber, sampleName, solutionNumber, good, software, oscStart, anom_oscStart, oscEnd, anom_oscEnd,";
			updateStrategyCmd += "continuous, complete, anom_complete, oscDelta, detectordistance, detectorwarning, spaceGroup_short, spaceGroup,";
			updateStrategyCmd += "resolution, unitcell, rmsd, beamx, beamy) ";

			updateStrategyCmd += "values ('" + sampleNumber + "','"
					+ autoindexResult.sampleName + "','"
					+ strategyResult.solutionNum + "','" + strategyResult.good
					+ "','" + strategyResult.software + "','"
					+ strategyResult.oscStart + "','"
					+ strategyResult.anom_oscStart + "','"
					+ strategyResult.oscEnd + "','"
					+ strategyResult.anom_oscEnd + "','"
					+ strategyResult.cont_oscEnd + "','"
					+ strategyResult.completeness + "','"
					+ strategyResult.anom_completeness + "','"
					+ strategyResult.oscDelta + "','"
					+ strategyResult.detectorDistance + "','"
					+ strategyResult.detectorWarning + "','"
					+ strategyResult.spaceGroup_short + "','"
					+ strategyResult.spaceGroup + "','"
					+ strategyResult.predictedResolution + " A" + "','"
					+ strategyResult.unitCellA + " " + strategyResult.unitCellB
					+ " " + strategyResult.unitCellC + " "
					+ strategyResult.unitCellAlpha + " "
					+ strategyResult.unitCellBeta + " "
					+ strategyResult.unitCellGamma + "','"
					+ autoindexResult.rms + "','" + autoindexResult.beamX
					+ "','" + autoindexResult.beamY + "')";
		}

		Database.exec(DBName.USERDATA, updateStrategyCmd);

		if (update) {
			// update the GUI directly
			// updateRow(sampleNumber);

			if (newResultCreated && SHOW_STRATEGY_TAB_ON_RESULT_CREATION) {
				DevCollectRunList.get().setCurrentLeftTab(
						CollectLeftTab.STRATEGY);
			}

			// send the PV update signal
			sendRowUpdate(sampleNumber);

			/* update GM/CA database */

			// if(strategyResult.good.equals("yes"))
			updateStrategyEvent(sampleNumber, autoindexResult.isScreening);
		}
	}

	protected void updateRow(int row) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row);
		final Database.Result result2 = Database.exec(DBName.USERDATA,
				"select * from " + resultTableName + " where sampleNumber="
						+ row + " and good='yes'");
		ModelStrategyResult strategyResult;
		if (result2.numRows() > 0 && result.numRows() > 0)
			strategyResult = sqlRowToResult(result.getRow(0), result2.getRow(0));
		else {
			if(result.numRows()  > 0)
				strategyResult = sqlRowToResult(result.getRow(0));
			else
				return;
		}
		strategyResult.spaceGroups = getSpaceGroups(row);
		ModelStrategyResultList.get().setRow(strategyResult, row, true);
		ModelStrategyResultList.get().ll.notifyListeners(row);
	}

	public void updateRow(int row, String spaceGroup, boolean update) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row);
		final Database.Result result2 = Database.exec(DBName.USERDATA,
				"select * from " + resultTableName + " where sampleNumber="
						+ row + " and spaceGroup='" + spaceGroup + "'");
		if (result2.numRows() > 0) {
			ModelStrategyResult strategyResult = sqlRowToResult(
					result.getRow(0), result2.getRow(0));
			strategyResult.spaceGroups = getSpaceGroups(row);
			ModelStrategyResultList.get().setRow(strategyResult, row, true);
		}

		if (update)
			ModelStrategyResultList.get().ll.notifyListeners(row);
	}

	protected ArrayList<String> getSpaceGroups(int row) {
		final Database.Result result3 = Database.exec(DBName.USERDATA,
				"select * from " + resultTableName + " where sampleNumber="
						+ row);

		ArrayList<String> spaceGroups = new ArrayList<>();
		for (HashMap<String, String> row2 : result3.getRows()) {
			for (String sqlCol : row2.keySet()) {
				if (sqlCol.equals("spaceGroup")) {
					spaceGroups.add(row2.get(sqlCol));
				}
			}
		}
		return (spaceGroups);
	}

	protected void updateAllRows() {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		System.out.println("Number of strategy rows " + result.numRows());
		ModelStrategyResultList.get().clearResults();

		for (HashMap<String, String> row : result.getRows()) {
			ModelStrategyResult strategyResult;
			final Database.Result result2 = Database.exec(DBName.USERDATA,
					"select * from " + resultTableName + " where sampleNumber="
							+ row.get("sampleNumber") + " and good='yes'");
			if (result2.numRows() > 0) {
				HashMap<String, String> row2 = result2.getRow(0);
				strategyResult = sqlRowToResult(row, row2);
			} else {
				strategyResult = sqlRowToResult(row);
			}
			strategyResult.spaceGroups = getSpaceGroups(Integer.valueOf(row
					.get("sampleNumber")));
			if (null != strategyResult)
				ModelStrategyResultList.get().setRow(strategyResult,
						Integer.parseInt(row.get("sampleNumber")), false);
		}
		ModelStrategyResultList.get().ll.notifyListeners(-2);
	}

	protected ModelStrategyResult sqlRowToResult(HashMap<String, String> row) {
		HashMap<StrategyField, String> fieldMap = new HashMap<StrategyField, String>();
		for (StrategyField field : StrategyField.values()) {
			boolean isStrat = true;
			for (String sqlCol : row.keySet()) {
				if (field.getDBString().equals(sqlCol)) {
					fieldMap.put(field, row.get(sqlCol));
					isStrat = false;
				}
			}
			if (isStrat)
				fieldMap.put(field, "");
		}
		return new ModelStrategyResult(fieldMap);
	}

	protected ModelStrategyResult sqlRowToResult(HashMap<String, String> row,
			HashMap<String, String> row2) {
		HashMap<StrategyField, String> fieldMap = new HashMap<StrategyField, String>();
		HashMap<StrategyParam, String> fieldMapStr = new HashMap<StrategyParam, String>();

		for (String sqlCol : row.keySet()) {
			for (StrategyField field : StrategyField.values()) {

				if (field.getDBString().equals(sqlCol)) {
					fieldMap.put(field, row.get(sqlCol));
				}
			}

			for (StrategyParam field : StrategyParam.values()) {
				if (field.getDBField().fieldName.equals(sqlCol))
					fieldMapStr.put(field, row.get(sqlCol));
			}
		}

		for (String sqlCol : row2.keySet()) {
			for (StrategyField field : StrategyField.values()) {
				if (field.getDBString().equals(sqlCol)) {
					fieldMap.put(field, row2.get(sqlCol));
				}
			}

			for (StrategyParam field : StrategyParam.values()) {
				if (field.getDBField().fieldName.equals(sqlCol))
					fieldMapStr.put(field, row.get(sqlCol));
			}
		}

		return new ModelStrategyResult(fieldMap);
	}

	protected void updateStrategyEvent(int num, boolean isScr) {
		HashMap<StrategyParam, String> fieldMapStr = new HashMap<StrategyParam, String>();
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + num);
		if (result.numRows() < 1)
			return;
		for (HashMap<String, String> row : result.getRows()) {
			for (String sqlCol : row.keySet()) {
				for (StrategyParam fieldStr : StrategyParam.values()) {
					if (fieldStr.getDBField().fieldName.equals(sqlCol)) {
						fieldMapStr.put(fieldStr, row.get(sqlCol));
					}
				}
			}
		}

		final Database.Result resultStr = Database.exec(DBName.USERDATA,
				"select * from " + resultTableName + " where sampleNumber="
						+ num + " and good='yes'");

		for (HashMap<String, String> row : resultStr.getRows()) {
			for (String sqlCol : row.keySet()) {
				for (StrategyParam fieldStr : StrategyParam.values()) {
					if (fieldStr.getDBField().fieldName.equals(sqlCol)) {
						fieldMapStr.put(fieldStr, row.get(sqlCol));
					}
				}
			}
		}

		if (isScr) {
			fieldMapStr.put(StrategyParam.PREFIX, ModelScreening.currentPrefix);
			fieldMapStr.put(StrategyParam.SCREEN_COLLECT_NUMBER,
					Integer.toString(num));
			fieldMapStr.put(StrategyParam.COLLECT_ZERORUN_NUMBER, "0");
		} else {
			fieldMapStr.put(StrategyParam.PREFIX, ModelCollectRunList.get()
					.getCurrentRunModel().getFileRoot());
			fieldMapStr.put(StrategyParam.SCREEN_COLLECT_NUMBER, "0");
			fieldMapStr.put(StrategyParam.COLLECT_ZERORUN_NUMBER,
					Integer.toString(num));
		}
		try {
			StrategyEvents.writeEvent(fieldMapStr);
		} catch(Exception e) {
			e.printStackTrace();
		} 
	}

	protected int getRowCount() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		count = result.numRows();
		return count;
	}

	protected void handlePVUpdate(int pvValue) {
		verifyTableExists();
		if (pvValue >= 0) {
			System.out.println("Updating row " + pvValue);
			updateRow(pvValue);
		} else if (-2 == pvValue) {
			System.out.println("Updating all rows " + pvValue);
			updateAllRows();
		}
	}

	protected void sendRowUpdate(int sampleNumber) {
		Epics.putInteger(PV_SCR.dbUpdated, -1);
		Epics.putInteger(PV_SCR.dbUpdated, sampleNumber);
	}

	public static int getRows() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		count = result.numRows();
		return count;
	}

	public static void setCurrentStrategyIndex(int i) {
		Epics.putInteger(PV_SCR.currentStrategyIndex, i);
	}

	public void deleteAllResults() {
		Log.note("Deleting all strategy results");

		verifyTableExists();

		setCurrentStrategyIndex(0);

		Database.delete(DBName.USERDATA, tableName, null);
		Database.delete(DBName.USERDATA, resultTableName, null);

		sendRowUpdate(-2);
	}

	public void saveAllResults(String excelFile) {
		
		Workbook wbs;
		ModelStrategyResultList resultList = ModelStrategyResultList.get();
		try {
			OutputStream inp = new FileOutputStream(excelFile);

			wbs = new HSSFWorkbook();

			Sheet s = wbs.createSheet();
			// declare a row object reference
			Row r = null;
			// declare a cell object reference
			Cell c = null;
			int i = 1;
			r = s.createRow(0); // write the column names
			for (StrategyField field : StrategyField.values()) {

				c = r.createCell(field.ordinal());
				c.setCellValue(field.getDBString());
			}
			// write data
			for (ModelStrategyResult cell : new ArrayList<ModelStrategyResult>(
					resultList.getStrategyList())) {

				Row r1 = s.createRow(i);
				for (StrategyField field : cell.getAllFields().keySet()) {
					String value = cell.getField(field);
					c = r1.createCell(field.ordinal());
					if (value != null)
						c.setCellValue(value);
					else
						c.setCellValue("");
				}
				i++;
			}
			wbs.write(inp);
			inp.close();
		} catch (IOException ex) {
			Log.error("Excel export failed :" + ex.getMessage());
		}
	}
}
