/**
 * This class parses strategy XML file generated by WebIce scripts
 * and saves the data collection strategy parameters
 * 
 * It used DOM XHL parser
 */

package gov.anl.gmca.strategy;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.io.File;
import java.util.Vector;

public class FileStrategyResult {
	public String mosaicity = "";
	public String score = "";
	public int spaceGroups = 0;
	public Vector<StrategyParams> strategyResults = new Vector<StrategyParams>();

	private javax.xml.parsers.DocumentBuilderFactory dFactory = null;
	private javax.xml.parsers.DocumentBuilder dBuilder = null;

	public FileStrategyResult() {
	}

	public int read(String strategyFile) throws Exception {
		// Instantiate a DocumentBuilderFactory.
		dFactory = javax.xml.parsers.DocumentBuilderFactory.newInstance();

		// Use the DocumentBuilderFactory to create a DocumentBuilder.
		dBuilder = dFactory.newDocumentBuilder();

		File xmlFile = new File(strategyFile);

		if (!xmlFile.exists()) {
			// Log.error(xmlFile+" doesn't exists");
			return 1;
		}

		// Use the DocumentBuilder to parse the XML input.
		Document silDoc = dBuilder.parse(xmlFile);
		silDoc.getDocumentElement().normalize();

		NodeList autoindex = silDoc.getElementsByTagName("autoindex");

		if (autoindex.getLength() == 0) {
			System.out.println("no strategy");
			return 2;
		}

		for (int i = 0; i < autoindex.getLength(); ++i) {
			Node fstNode = autoindex.item(i);
			if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				Element fstElmnt = (Element) fstNode;

				mosaicity = fstElmnt.getAttribute("mosaicity");
				score = fstElmnt.getAttribute("score");
				// wedge = getTextValue(strategy,"result");
				// attenuation = getTextValue(strategy,"attenuation");
			}
		}
		NodeList spaceG = silDoc.getElementsByTagName("spaceGroup");

		if (spaceG.getLength() == 0) {
			System.out.println("no spacegroup found");
			return 3;
		}
		spaceGroups = spaceG.getLength();
		for (int i = 0; i < spaceGroups; ++i) {
			Node fstNode = spaceG.item(i);
			StrategyParams strategySpaceGroup = new StrategyParams();
			if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				Element fstElmnt = (Element) fstNode;
				// strategySpaceGroup.spaceGroup =
				// fstElmnt.getAttribute("name");
			}

			NodeList uniqueStrategy = silDoc.getElementsByTagName("dcStrategy");
			int space = uniqueStrategy.getLength();

			Node spaceGroupVal = uniqueStrategy.item(2 * i);

			NodeList strategyParams = spaceGroupVal.getChildNodes();
			int str = strategyParams.getLength();
			Element fstElmnt;

			for (int k = 0; k < strategyParams.getLength(); ++k) {
				fstNode = strategyParams.item(k);
				if (fstNode != null)
					if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
						fstElmnt = (Element) fstNode;
						if (fstElmnt.getNodeName().equals("version"))
							strategySpaceGroup.software = fstElmnt
									.getAttribute("value");
						if (fstElmnt.getNodeName().equals("expType"))
							strategySpaceGroup.expType = fstElmnt
									.getAttribute("value");
						if (fstElmnt.getNodeName().equals("axis"))
							strategySpaceGroup.axis = fstElmnt
									.getAttribute("value");

						if (fstElmnt.getNodeName().equals("attenuation"))
							strategySpaceGroup.attenuation = fstElmnt
									.getAttribute("value");

						if (fstElmnt.getNodeName().equals("resolution")) {
							strategySpaceGroup.predictedResolution = fstElmnt
									.getAttribute("predicted");
							strategySpaceGroup.detectorResolution = fstElmnt
									.getAttribute("detector");
						}
						if (fstElmnt.getNodeName().equals("exposureTime")) {
							strategySpaceGroup.exposureTime = fstElmnt
									.getAttribute("value");
							strategySpaceGroup.exposureTimeWarn = fstElmnt
									.getAttribute("warning");
						}

						if (fstElmnt.getNodeName().equals("detectorZCorr")) {
							strategySpaceGroup.detectorDistance = fstElmnt
									.getAttribute("value");
							strategySpaceGroup.detectorWarning = fstElmnt
									.getAttribute("warning");
						}

						if (fstElmnt.getNodeName().equals("detector")) {
							strategySpaceGroup.detectorType = fstElmnt
									.getAttribute("type");
							strategySpaceGroup.detectorMode = fstElmnt
									.getAttribute("mode");
						}
						if (fstElmnt.getNodeName().equals("imageCount")) {
							strategySpaceGroup.completeness = fstElmnt
									.getAttribute("complete");
						}
						if (fstElmnt.getNodeName().equals("energy")) {
							strategySpaceGroup.energy1 = fstElmnt
									.getAttribute("value1");
							strategySpaceGroup.energy2 = fstElmnt
									.getAttribute("value2");
							strategySpaceGroup.energy3 = fstElmnt
									.getAttribute("value3");
						}

						if (fstElmnt.getNodeName().equals("beamstopZ")) {
							strategySpaceGroup.beamstop = fstElmnt
									.getAttribute("value");
						}
						if (fstElmnt.getNodeName().equals("best")) {
							strategySpaceGroup.bestWarn = fstElmnt
									.getAttribute("exposureTimeWarning");
						}
						if (fstElmnt.getNodeName().equals("osc")) {
							strategySpaceGroup.oscStart = fstElmnt
									.getAttribute("start");
							strategySpaceGroup.oscEnd = fstElmnt
									.getAttribute("end");
							strategySpaceGroup.oscDelta = fstElmnt
									.getAttribute("delta");
						}
						if (fstElmnt.getNodeName().equals("radDose")) {
							strategySpaceGroup.radDoseperImage = fstElmnt
									.getAttribute("perImage");
							;
							strategySpaceGroup.radDoseTotal = fstElmnt
									.getAttribute("total");
							;
							strategySpaceGroup.radDoseLimit = fstElmnt
									.getAttribute("limit");
							;
							strategySpaceGroup.radDiseMaxImages = fstElmnt
									.getAttribute("maxImagest");
							;
							strategySpaceGroup.radDoseWarning = fstElmnt
									.getAttribute("warning");
							;
						}
					}

				spaceGroupVal = uniqueStrategy.item(2 * i + 1);

				NodeList anomParams = spaceGroupVal.getChildNodes();
				str = strategyParams.getLength();
				for (int j = 0; j < anomParams.getLength(); ++j) {
					fstNode = anomParams.item(j);
					if (fstNode != null)
						if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
							fstElmnt = (Element) fstNode;
							if (fstElmnt.getNodeName().equals("osc")) {
								strategySpaceGroup.anom_oscStart = fstElmnt
										.getAttribute("start");
								strategySpaceGroup.anom_oscEnd = fstElmnt
										.getAttribute("end");
								strategySpaceGroup.cont_oscEnd = fstElmnt
										.getAttribute("cont");
							}

							if (fstElmnt.getNodeName().equals("imageCount")) {
								strategySpaceGroup.anom_completeness = fstElmnt
										.getAttribute("complete");
							}
						}
				}
			}
			strategyResults.add(strategySpaceGroup);
		}
		return 0;
	}
}
