/**
 * This class reads labelit.out for displaying solutions on the strategy tab
 * 
 */

package gov.anl.gmca.strategy;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.syslog.Log;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;

public class ModelAutoindexResult {

	public String warning = "processing";
	public String workDir = "";
	public String sampleName = "";
	public String screeningSample = "";
	public String image1 = "";
	public String image2 = "";
	public String score = "";
	public String spaceGroup = "";
	public String beamX = "";
	public String beamY = "";
	public String distance = "";
	public String resolution = "";
	public String mosaicity = "";
	public String rms = "";
	public String unitCell = "";
	public boolean isScreening = false;

	public String resultFile = "";
	public String state = "";

	public class IndexSolution {
		boolean good = false;
		boolean filter = false;
		String solutionNum = "";
		String metricFit = "";
		String metricFitUnit = "";
		String rmsd = "";
		String numSpots = "";
		String crystalSystemName = "";
		String crystalSystemSymbol = "";
		String unitCellA = "";
		String unitCellB = "";
		String unitCellC = "";
		String unitCellAlpha = "";
		String unitCellBeta = "";
		String unitCellGamma = "";
		String volume = "";

		String strategyFile = "";
	}

	public class IntegrationSolution {
		boolean good = false;
		String solutionNum = "";
		String spaceGroup = "";
		String beamX = "";
		String beamY = "";
		String distance = "";
		String resolution = "";
		String mosaicity = "";
		String rmsd = "";
		String unitCellA = "";
		String unitCellB = "";
		String unitCellC = "";
		String unitCellAlpha = "";
		String unitCellBeta = "";
		String unitCellGamma = "";
		String crystalSystemName = "";
		String crystalSystemSymbol = "";
	}

	public Vector<IndexSolution> indexResults = new Vector<IndexSolution>();
	public Vector<IntegrationSolution> integrationResults = new Vector<IntegrationSolution>();

	/**
	 * Parse labelit.out file : to get same functionality of Webice, labelit.out
	 * 
	 * Example labelit.out file /mnt/share1/user0/rs100817x1H_1_0.0001
	 * /mnt/share1/user0/rs100817x1H_1_0.0091 GOOD=492, OVERLAP=17, SPINDLE=55,
	 * ICE=36, OUTLIER=0 Total=600 total=492, inlier=233, fraction=196,
	 * outlier=63 GOOD=462, OVERLAP=16, SPINDLE=59, ICE=0, OUTLIER=63 Total=600
	 * total=462, inlier=271, fraction=184, outlier=7
	 * 
	 * LABELIT Indexing results: Beam center x 149.90mm, y 150.40mm, distance
	 * 249.70mm ; 80% mosaicity=0.70 deg.
	 * 
	 * Solution Metric fit rmsd #spots crystal_system unit_cell volume 
	 * :) 5 0.1225 dg 0.282 439 orthorhombic oP 31.99 53.67 74.66 90.00 90.00 90.00 128182
	 * ;( 4 0.1225 dg 0.285 440 monoclinic mP 	31.99 74.66 53.67 90.0090.05 90.00 128184 
	 * ;( 3 0.1018 dg 0.310 461 monoclinic mP 53.65 31.99 74.66 90.00 90.08 90.00 128149 
	 * ;( 2 0.0964 dg 0.282 439 monoclinic mP 31.99 53.66 74.66 90.00 90.09 90.00 128187 
	 * ;( 1 0.0000 dg 0.306 462 triclinic aP 31.99 53.63 74.67 90.08 90.09 90.05 128120
	 * 
	 * MOSFLM Integration results: 
	 * Solution SpaceGroup Beam x y distance Resolution Mosaicity RMS 
	 * :) 5 P222 150.12 150.61 249.79 1.67 0.700000 0.090 
	 * 	  1   P1 150.12 150.52 249.72 1.76 0.700000 0.111
	 * 
	 * 
	 */
	public void processResult(String workDir) throws Exception {
		String outputFile = "";
		outputFile = workDir + "/LABELIT/labelit.out";
		Log.note("reading labelit.out");

		File labelitFile = new File(outputFile);

		if (!labelitFile.exists()) {
			Log.note(outputFile + " doesn't exists");
			state = "ERROR";
			warning = outputFile + " doesnt exists";
			return;
		}

		byte[] buffer = new byte[(int) new File(outputFile).length()];
		BufferedInputStream f = null;
		try {
			f = new BufferedInputStream(new FileInputStream(outputFile));
			f.read(buffer);
		} finally {
			if (f != null)
				try {
					f.close();
				} catch (IOException ignored) {
				}
		}

		// Read spotfinder log file
		String content = new String(buffer);
		Log.note("Autoindex Result :\n" + content);
		/*
		 * if (content.length() > 200) { Log.error(" labelit result: " +
		 * content.substring(0, 200)); } else { Log.error(" labelit result: " +
		 * content); }
		 */

		spaceGroup = "N/A";
		resolution = "N/A";
		mosaicity = "N/A";
		rms = "N/A";
		unitCell = "N/A";

		int pos1 = 0;
		int pos2 = 0;

		int pos3 = -1;
		if ((pos3 = content.indexOf("MOSFLM HAS TERMINATED EARLY")) >= 0) {
			warning = "ERROR : MOSFLM HAS TERMINATED EARLY click on Solution button for more info.\n";
			//return;
		}
		if ((pos3 = content.indexOf("Non-zero two-theta")) >= 0) {
			warning = "ERROR : Non-zero two-theta, strategy not supported";
			return;
		}
		if ((pos3 = content.indexOf("pseudotranslation")) >= 0) {
			warning = "Labelit suggests pseudotranslation, click on Solution button for more info.\n";
		}
		if ((pos3 = content.indexOf("No indexing solution")) >= 0
				|| (pos3 = content.indexOf("No_Indexing_Solution")) >= 0) {
			warning = content.substring(pos3, content.length());
			return;
		}
		if ((pos3 = content.indexOf("INDEXING UNRELIABLE")) >= 0) {
			warning = "Labelit INDEXING UNRELIABLE :  click on Solution button for more info.\n";
			return;
		}
		if ((pos3 = content.indexOf("MosflmVersionError:")) > 0) {
			state = "ERROR";
			warning = content.substring(pos3, content.length());
			return;
			// throw new Exception(content.substring(pos3, content.length()));
		}
		if ((pos3 = content.indexOf("Spotfinder Problem:")) > 0) {
			if (pos3 + 50 < content.length()) {
				warning = content.substring(pos3, content.length());
				return;
				// throw new Exception(content.substring(pos3, pos3+120));
			} else {
				warning = content.substring(pos3);
				// throw new Exception(content.substring(pos3));
			}
		}
		// Labelit failed. Print out the error, assuming that it's on the last
		// line.
		if (content.indexOf("LABELIT Indexing results") < 0) {
			warning = content;
			return;
			// throw new Exception(content);
		}
		// Beam center X
		pos1 = content.indexOf("Beam center x");
		if (pos1 < 0) {
			warning = "Could not find text Beam center x in labelit.out";
			return;
			// throw new Exception("Could not find text 'Beam center x'");
		}

		pos2 = content.indexOf("mm", pos1);
		if (pos2 < 0) {
			warning = "Could not find text mm in labelit.out while reading beamX";
			return;
			// throw new Exception("Could not find text 'mm'");
		}

		beamX = content.substring(pos1 + 13, pos2).trim();

		// Beam center X
		pos1 = content.indexOf("y", pos2 + 2);
		if (pos1 < 0) {
			warning = "Could not find text y in labelit.out";
			return;
			// throw new Exception("Could not find text 'y'");
		}

		pos2 = content.indexOf("mm", pos1);
		if (pos2 < 0) {
			warning = "Could not find text mm in labelit.out while reading beamY";
			return;
			// throw new Exception("Could not find text 'mm'");
		}

		beamY = content.substring(pos1 + 1, pos2).trim();

		// Distance
		pos1 = content.indexOf("distance", pos2 + 2);
		if (pos1 < 0) {
			warning = "Could not find text distance in labelit.out";
			return;
			// throw new Exception("Could not find text 'distance'");
		}

		pos2 = content.indexOf("mm", pos1);
		if (pos2 < 0) {
			warning = "Could not find text mm in labelit.out while reading distance";
			return;
			// throw new Exception("Could not find text 'mm'");
		}

		distance = content.substring(pos1 + 8, pos2).trim();

		// mosaicity
		pos1 = content.indexOf("mosaicity=", pos2 + 2);
		if (pos1 < 0) {
			warning = "Could not find text ; while reading mosaicity from labelit.out";
			return;
			// throw new Exception("Could not find text ';'");
		}

		pos2 = content.indexOf(" deg.\n", pos1);
		if (pos2 < 0) {
			warning = "Could not find text end of line while reading mosaicity from labelit.out";
			return;
			// throw new Exception("Could not find text end of line");
		}

		mosaicity = content.substring(pos1 + 10, pos2).trim();

		// table header
		pos1 = content.indexOf("Solution", pos2 + 1);
		if (pos1 < 0) {
			warning = "Could not find Solution from Labelit";
			return;
			// throw new Exception("Could not find text 'Solution'");
		}

		pos2 = content.indexOf("\n", pos1);
		if (pos2 < 0) {
			warning = "labeit.out : Could not find end of line";
			return;
			// throw new Exception("Could not find text end of line");
		}
		
		StringTokenizer tok = new StringTokenizer(content.substring(pos1, pos2));

		Vector<String> headers = new Vector<String>();
		while (tok.hasMoreTokens()) {
			headers.add(tok.nextToken());
		}

		pos1 = pos2 + 1;

		pos2 = content.indexOf("MOSFLM Integration", pos1);
		if (pos2 < 0) {
			warning = "Could not find MOSFLM Integration";
			return;
			// throw new Exception("Could not find 'MOSFLM Integration'");
		}
		
		/* new requirement to check the beam deviation calculated by labelit */
		int posDev = content.indexOf("Correcting beam by");
		int posDevNewLine = content.indexOf("\n", posDev);
		if(posDev > 0) {
			tok = new StringTokenizer(content.substring(posDev, posDevNewLine));
			tok.nextToken();
			tok.nextToken();
			tok.nextToken();
			double dev = 0.0;
			try {
				dev = Double.parseDouble(tok.nextToken());
				if(dev > 2.0)
					warning += "The calculated beam center devition is more than 2.0\nPlease report this to your host";
			} catch (NumberFormatException e){
				System.out.println(e.getMessage());
			}
		}
		
		tok = null;
		tok = new StringTokenizer(content.substring(pos1, pos2), "\n");

		/* read all integration solutions */
		// Each row
		String cols[] = new String[15];
		while (tok.hasMoreTokens()) {
			StringTokenizer tok2 = new StringTokenizer(tok.nextToken());
			int i = 0;
			// each col
			while (tok2.hasMoreTokens()) {
				if (i < 15)
					cols[i] = tok2.nextToken();
				++i;
			}
			if (i < 14)
				throw new Exception(
						"Expect at least 14 columns for indexing results but got "
								+ i);

			IndexSolution res = new IndexSolution();

			res.good = false;
			if (i == 15) {
				if (cols[0].equals(":)"))
					res.good = true;
				// if (cols[0].equals("xx"))
				// res.filter = true;
				i = 1;
			} else {
				i = 0;
			}
			res.solutionNum = cols[i];
			++i;
			res.metricFit = cols[i];
			++i;
			res.metricFitUnit = cols[i];
			++i;
			res.rmsd = cols[i];
			++i;
			res.numSpots = cols[i];
			++i;
			res.crystalSystemName = cols[i];
			++i;
			res.crystalSystemSymbol = cols[i];
			++i;
			res.unitCellA = cols[i];
			++i;
			res.unitCellB = cols[i];
			++i;
			res.unitCellC = cols[i];
			++i;
			res.unitCellAlpha = cols[i];
			++i;
			res.unitCellBeta = cols[i];
			++i;
			res.unitCellGamma = cols[i];
			++i;
			res.volume = cols[i];
			++i;

			indexResults.add(res);
			res.strategyFile = "solution"
					+ String.format("%02d", Integer.valueOf(res.solutionNum));
			if (Integer.valueOf(res.solutionNum) == 1)
				break;

		}

		/* filter solution according to crystal system and unit cell */
		for (int i = 0; i < indexResults.size(); i++) {
			IndexSolution sol1 = (IndexSolution) indexResults.get(i);
			for (int j = i + 1; j < indexResults.size(); j++) {
				IndexSolution sol2 = (IndexSolution) indexResults.get(j);

				if (sol2.filter == true || sol2.good)
					continue;

				if (sol1.crystalSystemSymbol.equals(sol2.crystalSystemSymbol)) {
					double a1 = Math.abs(Double.valueOf(sol1.unitCellA)
							- Double.valueOf(sol2.unitCellA));
					double b1 = Double.valueOf(sol1.unitCellB)
							- Double.valueOf(sol2.unitCellB);
					double c1 = Double.valueOf(sol1.unitCellC)
							- Double.valueOf(sol2.unitCellC);
					;
					double alpha1 = Double.valueOf(sol1.unitCellAlpha)
							- Double.valueOf(sol2.unitCellAlpha);
					double beta1 = Double.valueOf(sol1.unitCellBeta)
							- Double.valueOf(sol2.unitCellBeta);
					double gamma1 = Double.valueOf(sol1.unitCellGamma)
							- Double.valueOf(sol2.unitCellGamma);

					double diff = Double.valueOf(DBConfig
							.get("filter_unitcell"));
					if (a1 < diff && b1 < diff && c1 < diff && alpha1 < diff
							&& beta1 < diff && gamma1 < diff) {
						((IndexSolution) indexResults.get(j)).filter = true;
					}
				}
			}
		}

		// Integration results
		pos1 = content.indexOf("Solution", pos2);
		if (pos1 < 0) {
			warning = "Could not find Solution";
			return;
			// throw new Exception("Could not find 'Solution'");
		}

		pos2 = content.indexOf("\n", pos1);
		if (pos2 < 0) {
			warning = "labeit.out : Could not find end of line";
			return;
			// throw new Exception("Could not find end of line");
		}

		tok = null;
		tok = new StringTokenizer(content.substring(pos2 + 1), "\n");

		cols = null;
		cols = new String[9];
		while (tok.hasMoreTokens()) {
			String str = tok.nextToken();
			StringTokenizer tok2 = new StringTokenizer(str);
			int i = 0;
			while (tok2.hasMoreTokens()) {
				cols[i] = tok2.nextToken();
				++i;
			}

			if (i < 8)
				throw new Exception(
						"Expect at least 8 columns for integeration result but got "
								+ i);

			IntegrationSolution res = new IntegrationSolution();

			res.good = false;
			if (i == 9) {
				if (cols[0].equals(":)")) {
					integrationResults.removeAllElements();
					res.good = true;
				}

				i = 1;

			} else {
				i = 0;
			}

			res.solutionNum = cols[i];
			++i;
			res.spaceGroup = cols[i];
			++i;
			res.beamX = cols[i];
			++i;
			res.beamY = cols[i];
			++i;
			res.distance = cols[i];
			++i;
			res.resolution = cols[i];
			++i;
			res.mosaicity = cols[i];
			++i;
			res.rmsd = cols[i];
			++i;

			StringBuffer tmpName = new StringBuffer();
			StringBuffer tmpSymbol = new StringBuffer();
			getCrystalSystem(res, res.solutionNum, tmpName, tmpSymbol);

			res.crystalSystemName = tmpName.toString();
			res.crystalSystemSymbol = tmpSymbol.toString();

			integrationResults.add(res);

			if (Integer.valueOf(res.solutionNum) == 1)
				break;
		}

		// Calculate score
		try {
			// copy the best solution
			IntegrationSolution intSol = (IntegrationSolution) integrationResults
					.get(0);
			mosaicity = Double.toString(Double.parseDouble(intSol.mosaicity));
			double mosaicityNum = Double.parseDouble(intSol.mosaicity);
			double rmsrNum = Double.parseDouble(intSol.rmsd);
			double resolutionNum = Double.parseDouble(intSol.resolution);

			// If we have poor mosaicity then the score
			// will be worse.
			double mosaicityPenalty = 1.0;
			int pos = content.indexOf("60%");
			if (pos > 0) {
				pos = content.indexOf("(POOR)", pos);
				if (pos > 0)
					mosaicityPenalty = 2.0;
			}

			double _score = 1.0 - (0.7 * Math.exp(-4.0 / resolutionNum))
					- (1.5 * rmsrNum) - (0.2 * mosaicityNum * mosaicityPenalty);

			int score_int = (int) (_score * 1000);
			score = String.valueOf(((double) score_int) / 1000.0);
			// warning = "Autoindexing done";

			System.out.println(" Finished processing result");
			System.out.println("UnitCell " + unitCell);
			System.out.println("Mosaicity " + mosaicity);
			System.out.println("Rmsr " + rms);
			System.out.println("BravaisLattice " + spaceGroup);
			System.out.println("Resolution " + resolution);
		} catch (Exception e) {
			warning = "Failed to calculate score";
			return;
		}

	}

	/**
	 */
	private void getCrystalSystem(IntegrationSolution res, String solNum,
			StringBuffer name, StringBuffer symbol) {
		for (int i = 0; i < indexResults.size(); ++i) {
			IndexSolution res1 = (IndexSolution) indexResults.elementAt(i);
			if (res1.solutionNum.equals(solNum)) {
				name.append(res.crystalSystemName);
				symbol.append(res.crystalSystemSymbol);
				res.unitCellA = res1.unitCellA;
				res.unitCellB = res1.unitCellB;
				res.unitCellC = res1.unitCellC;

				res.unitCellAlpha = res1.unitCellAlpha;
				res.unitCellBeta = res1.unitCellBeta;
				res.unitCellGamma = res1.unitCellGamma;
				return;
			}
		}
	}

	public IndexSolution getIndexSolution(int solNum) {
		IndexSolution indexSol = null;
		for (int j = 0; j < indexResults.size(); j++) {
			indexSol = (IndexSolution) indexResults.get(j);
			if (Integer.parseInt(indexSol.solutionNum) == solNum)
				break;
		}
		return indexSol;
	}

	public IntegrationSolution getIntegrationSolution(int solNum) {
		IntegrationSolution integrationSol = null;
		for (int j = 0; j < integrationResults.size(); j++) {
			integrationSol = (IntegrationSolution) integrationResults.get(j);
			if (Integer.parseInt(integrationSol.solutionNum) == solNum)
				break;
			else
				integrationSol = null;
		}
		return integrationSol;
	}
}
