/**
 * This class parses laue_choices.out to get the spacegroups of each solution
 * from labelit.index output
 */

package gov.anl.gmca.strategy;

import gov.anl.gmca.syslog.Log;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;

public class ModelLaueChoices {
	public class LaueSpaceGroups {
		String solutionNum = "";
		String crystal_system = "";
		String lauegroup = "";
		String reflectionsymmetry = "";
		String SpaceGroups = "";
	}

	public Vector laueSpaceGroups = new Vector();

	/*
	 * All possible Bravais lattices consistent with this data are listed here:
	 * bravais_choice LePage_delta crystal_system a b c alpha beta gamma 
	 * 
	 * 12 0.2928 deg hexagonal hP 36.71 36.71 187.32 90.00 90.00 120.00 
	 * 11 0.2928deg orthorhombic oC 36.73 63.54 187.32 90.00 90.00 90.00 
	 * 10 0.2928 deg monoclinic mC 63.54 36.73 187.32 90.00 89.97 90.00 
	 * 9 0.2691 deg orthorhombic oC 36.74 63.52 187.36 90.00 90.00 90.00 
	 * 8 0.2691 deg monoclinic mC 36.74 63.52 187.24 90.00 90.10 90.00 
	 * 7 0.2464 deg orthorhombic oC 36.65 63.68 187.43 90.00 90.00 90.00 
	 * 6 0.2464 deg monoclinic mP 36.65 187.43 36.73 90.00 119.91 90.00 
	 * 5 0.1979 deg monoclinic mC 36.65 63.68 187.58 90.00 89.85 90.00 
	 * 4 0.1747 deg monoclinic mC 63.52 36.74 187.40 90.00 89.77 90.00 
	 * 3 0.1635 deg monoclinic mC 36.73 63.54 187.32 90.00 89.76 90.00 
	 * 2 0.1485 deg monoclinic mC 63.68 36.65 187.19 90.00 90.20 90.00 
	 * 1 0.0000 deg triclinic aP 36.65 36.73 187.32 89.76 89.85 60.09
	 * 
	 * Laue groups and space groups associated with each lattice are:
	 * bravais_choice Laue-group Reflection-symmetry Space-groups 
	 * 
	 * 12 (hP) -3 3 P3 P31 P32 12 (hP) -3m1 321 P321 P3121 P3221 
	 * 12 (hP) -31m 312 P312 P3112 P3212 12 (hP) 6/m 6 P6 P61 P65 P62 P64 P63 
	 * 12 (hP) 6/mmm 622 P622 P6122P6522 P6222 P6422 P6322 
	 * 11 (oC) mmm 222 C2221 C222 
	 * 10 (mC) 2/m 2 C121 
	 * 9  (oC) mmm 222 C2221 C222 
	 * 8  (mC) 2/m 2 C121 
	 * 7  (oC) mmm 222 C2221 C222 
	 * 6  (mP) 2/m 2 P121 P1211 \
	 * 5  (mC) 2/m 2 C121 
	 * 4  (mC) 2/m 2 C121 
	 * 3  (mC) 2/m 2 C121 
	 * 2  (mC) 2/m 2 C121 
	 * 1  (aP) -1 1 P1
	 */
	public void processResult(String workDir) throws Exception {
		String outputFile = "";
		outputFile = workDir + "/LABELIT/laue_choices.out";
		Log.note("reading laue choices");

		File labelitFile = new File(outputFile);

		if (!labelitFile.exists()) {
			Log.note(outputFile + " doesn't exists");
			// state = "ERROR";
			// warning = outputFile + " doesnt exists";
			return;
		}
		
		// Read laue_choices.out log file
		byte[] buffer = new byte[(int) new File(outputFile).length()];
		BufferedInputStream f = null;
		try {
			f = new BufferedInputStream(new FileInputStream(outputFile));
			f.read(buffer);
		} finally {
			if (f != null)
				try {
					f.close();
				} catch (IOException ignored) {
				}
		}
		
		String content = new String(buffer);

		int pos1 = content
				.indexOf("bravais_choice  Laue-group  Reflection-symmetry  Space-groups");
		if (pos1 < 0) {
			Log.note("could not find bravais_choice");
			return;
			// throw new Exception("Could not find 'MOSFLM Integration'");
		}

		int pos2 = content.indexOf("\n", pos1);
		if (pos2 < 0) {
			// warning = "labeit.out : Could not find end of line";
			return;
			// throw new Exception("Could not find text end of line");
		}

		pos1 = pos2 + 1;

		StringTokenizer tok = null;
		tok = new StringTokenizer(content.substring(pos1, content.length()),
				"\n");

		String cols[] = new String[5];
		while (tok.hasMoreTokens()) {
			StringTokenizer tok2 = new StringTokenizer(tok.nextToken());
			int i = 0;
			// each col
			for (i = 0; i < 4; i++) {
				cols[i] = tok2.nextToken();
				// Log.note(cols[i]);
			}
			cols[i] = "";
			while (tok2.hasMoreTokens()) {
				cols[i] += " " + tok2.nextToken();
				// Log.note(cols[i]);
			}
			LaueSpaceGroups res = new LaueSpaceGroups();
			i = 0;
			res.solutionNum = cols[i];
			++i;
			res.crystal_system = cols[i];
			++i;
			res.lauegroup = cols[i];
			++i;
			res.reflectionsymmetry = cols[i];
			++i;
			res.SpaceGroups = cols[i];
			laueSpaceGroups.add(res);
		}
	}

}
