package gov.anl.gmca.users;

import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;

public class ModelUserTable
{
	////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////// client //////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public static class Client
	{
		public static String	getID()								{ return getUser()+"@"+getShortHostname()+":"+getPid(); }
		public static boolean 	isMaster() 							{ return isMaster; }
		public static int 		getPid() 							{ return pid; }
		public static String getHostname()
		{
			if (null==hostname) hostname = ExecTask.syncExec("hostname", "", false).stdoutString().trim();
			return hostname;
		}
		public static String getUser()
		{
			return user;
		}
		
		public static void addListener(ClientListener cl)
		{
			if (!listenerList.contains(cl)) listenerList.add(cl);
		}
		public static void removeListener(ClientListener cl)
		{
			while (listenerList.remove(cl)) {}
		}
		
		public static String	getShortHostname()
		{
			int dotIndex=getHostname().indexOf('.');
			if (-1 == dotIndex) return hostname;
			return hostname.substring(0, dotIndex);
		}
		
		static boolean 	isMaster 	= false;
		static int 		pid 		= Util.getPid();
		static int		slot		= -1;
		static String	hostname	= null;
		static String	user		= System.getenv("USER");
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////// listener /////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public static interface ClientListener
	{
		public void masterChanged(boolean isMaster);
	}

	protected static ArrayList<ClientListener> listenerList = new ArrayList<ClientListener>();
	protected static void notifyClientListeners()
	{
		for (ClientListener cl : new ArrayList<ClientListener>(listenerList))
		{
			cl.masterChanged(Client.isMaster);
		}
	}
	
	////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////// end public interface ///////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public void devBecomeMaster() 	{ Client.isMaster = true; 	notifyClientListeners(); }
	public void devBecomeSlave() 	{ Client.isMaster = false; 	notifyClientListeners(); }
	
	protected			ListenerList<ModelUserTable>	tableListenerList;
	final static		int 							MAX_CLIENTS 	= 10;
	protected			User[]							userTable 		= new User[MAX_CLIENTS];
	protected			int 							masterSlot;
	protected			boolean 						masterExists;
	protected static	ModelUserTable					inst			= null;
	
	static class User
	{
		boolean isMaster;
		int 	pid;
		String 	host;
		String 	user;
	}
	
	public ModelUserTable()
	{
		inst = this;
		
		tableListenerList = new ListenerList<ModelUserTable>(this);
		
		resetTable();
	}
	
	public static ModelUserTable get() { return inst; }
	
	public void resetTable()
	{
		for (int i=0; i<MAX_CLIENTS; i++)
		{
			userTable[i] = new User();
		}
	}

	public void cleanupClientsOnAllWorkstations()
	{
		Log.note("Updating clients table for all workstations");
		
		// checks all workstations and compares running JBluIce instances to
		// the clients table in EPICS, and removes lines in the clients table
		// if the clients are no longer running
		ExecTask.syncExec(Util.getPerlPrefix()+"cleanupJBluice.pl", "cleanupJBluice.pl", false);
	}

	public boolean masterExists()
	{
		DevUserTable.get().readClientTable();
		return (masterSlot >= 0);
	}
}
