package gov.anl.gmca.users;

import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class TabUsers extends Tab implements ListenerList.Listener<ModelUserTable>
{
	Text txtUserTable = null;
	Text txtBeamlineUser = null;
	
	public TabUsers(Composite parent, int style)
	{
		super(parent,style);
		
		setBackground(GUI.BGCOLOR);
		
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth = 50;
		setLayout(gl);
		GridData gd;
		
		Label lblHeader = new Label(this, SWT.NONE);
		lblHeader.setText("Users");
		lblHeader.setFont(GUI.FONT_USER_HEADER);
		lblHeader.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.CENTER;
		lblHeader.setLayoutData(gd);
		
		txtUserTable = new Text(this, SWT.MULTI | SWT.BORDER);
		txtUserTable.setFont(GUI.FONT_USER_TABLE);
		txtUserTable.setEditable(false);
		txtUserTable.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		gd.heightHint = 250;
		txtUserTable.setLayoutData(gd);
		
		Label lblBeamlineUser = new Label(this, SWT.NONE);
		lblBeamlineUser.setText("Beamline User");
		lblBeamlineUser.setFont(GUI.FONT_USER_HEADER);
		lblBeamlineUser.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.verticalIndent = 10;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.CENTER;
		lblBeamlineUser.setLayoutData(gd);
		
		txtBeamlineUser = new Text(this, SWT.BORDER);
		txtBeamlineUser.setFont(GUI.FONT_USER_TABLE);
		txtBeamlineUser.setEditable(false);
		txtBeamlineUser.setText("");
		txtBeamlineUser.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.CENTER;
		gd.widthHint = 200;
		txtBeamlineUser.setLayoutData(gd);
		
		DevUserTable.get().llCurrentUser.addListener(new ListenerList.Listener<DevUserTable>()
		{
			@Override public void handleEvent(DevUserTable object, Object data)
			{
				final String username = (String)data;

				GUI.asyncExec(new Runnable(){public void run()
				{
					txtBeamlineUser.setText(" "+username);
				}});
			}
		});
		txtBeamlineUser.setText(" "+DevUserTable.get().getCurrentUser());
		
		Label lblLoggedInAs = new Label(this, SWT.NONE);
		lblLoggedInAs.setText("Logged in as");
		lblLoggedInAs.setFont(GUI.FONT_USER_HEADER);
		lblLoggedInAs.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.verticalIndent = 10;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.CENTER;
		lblLoggedInAs.setLayoutData(gd);
		
		Text txtLoggedInAs = new Text(this, SWT.BORDER);
		txtLoggedInAs.setFont(GUI.FONT_USER_TABLE);
		txtLoggedInAs.setEditable(false);
		txtLoggedInAs.setText(" "+Util.getUserLogin());
		txtLoggedInAs.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.CENTER;
		gd.widthHint = 200;
		txtLoggedInAs.setLayoutData(gd);
		
		ModelUserTable.get().tableListenerList.addListener(this);
		DevUserTable.get().readClientTable();
	}
	
	@Override
	public void handleEvent(ModelUserTable object, Object data)
	{
		if (null == txtUserTable) return;
		
		final StringBuilder sb = new StringBuilder();
		
		sb.append("                 Pid          Host                  User\n");

		for (ModelUserTable.User user : ModelUserTable.get().userTable)
		{
			if (0 == user.pid) continue;
			
			if (user.isMaster)
			{
				sb.append("     Active ->   ");
			}
			else
			{
				sb.append("                 ");
			}
			
			sb.append(Util.sprintf("%-13d", user.pid));
			sb.append(Util.sprintf("%-22s", user.host));
			sb.append(user.user);
			sb.append("\n");
		}
	
		GUI.asyncExec(new Runnable(){public void run()
		{
			txtUserTable.setText(sb.toString());
		}});
	}

	@Override
	public String getTabName() { return "Users"; }
}
