package gov.anl.gmca.util;

import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.lang.Thread.UncaughtExceptionHandler;


public class DlgException implements UncaughtExceptionHandler
{
	protected static DlgException handler = new DlgException();
	public static UncaughtExceptionHandler getHandler() { return handler; }
	
	protected static class WdgExceptionDisplay extends Composite
	{
		WdgExceptionDisplay(Composite parent, int style, String errorString)
		{
			super(parent,style);
			
			GridLayout gl = new GridLayout(2,false);
            gl.marginWidth = 10;
            gl.marginHeight = 10;
            gl.verticalSpacing = 10;
            setLayout(gl);
            GridData gd = null;
        	
            Label lblIcon = new Label(this,SWT.NONE);
            lblIcon.setImage(Display.getDefault().getSystemImage(SWT.ICON_WARNING));
            gd = new GridData();
            lblIcon.setLayoutData(gd);
            
            Label lbl = new Label(this,SWT.NONE);
            lbl.setText("The following Java exception was not caught.  Please restart BluIce-EPICS.");
            gd = new GridData();
            lbl.setLayoutData(gd);
            
            Text text = new Text(this,SWT.MULTI|SWT.V_SCROLL|SWT.H_SCROLL);
            gd = new GridData();
            gd.horizontalSpan=2;
            gd.horizontalAlignment=SWT.FILL;
            gd.grabExcessHorizontalSpace=true;
            gd.verticalAlignment=SWT.FILL;
            gd.grabExcessVerticalSpace=true;
            text.setLayoutData(gd);
    
            text.setText(errorString);
		}
	}
	
	protected Shell shell = null;
	protected CTabFolder ctFolder = null;
	
	@Override
	public void uncaughtException(Thread t, Throwable throwable)
	{
        StringBuilder sb = new StringBuilder();
        sb.append(throwable.getMessage());
        sb.append("\n\n");
        for (StackTraceElement ste : throwable.getStackTrace())
        {
        	sb.append('\t');
        	sb.append(ste.toString());
        	sb.append('\n');
        }
        final String errorString = sb.toString();
        System.out.println(errorString);
        
		if (Util.atExit())
		{
			return;
		}
		
		Util.safeSyncExec(new Runnable(){public void run()
		{
			if (null == shell)
			{
				// open a new dialog shell
	            shell = new Shell(GUI.getShell(), SWT.NONE);
	            shell.setText("Uncaught Java Exception");
	            GridLayout gl = new GridLayout(1,false);
	            gl.marginWidth=0;
	            shell.setLayout(gl);
	            
	            // set up the notebook and add the first item
	            ctFolder = new CTabFolder(shell,SWT.NONE);
	            CTabItem ctItem = new CTabItem(ctFolder, SWT.NONE);
	            ctItem.setControl(new WdgExceptionDisplay(ctFolder, SWT.NONE, errorString));
	            GridData gd = new GridData();
	            gd.horizontalAlignment = SWT.FILL;
	            gd.verticalAlignment = SWT.FILL;
	            gd.grabExcessHorizontalSpace = true;
	            gd.grabExcessVerticalSpace = true;
	            ctFolder.setLayoutData(gd);
	            ctFolder.setSelection(0);
	            
	            // add the OK button
	            Button b1 = new Button(shell,SWT.NONE);
	            b1.setText(" OK ");
	            gd = new GridData();
	            gd.grabExcessHorizontalSpace=false;
	            gd.horizontalAlignment=SWT.RIGHT;
	            b1.setLayoutData(gd);
	            b1.addSelectionListener(new SelectionListener()
	            {
	    			public void widgetDefaultSelected(SelectionEvent arg0) {}
	    
	    			public void widgetSelected(SelectionEvent arg0)
	    			{
	    				shell.dispose();
	    				ctFolder = null;
	    			}
	    		});
	            
	            shell.pack();
	            shell.setSize(800,800);
	            
				shell.setDefaultButton(b1);
	            
	            shell.open();
			}
			else
			{
				shell.setText("Uncaught Java Exceptions");
				
				CTabItem ctItem = new CTabItem(ctFolder, SWT.NONE);
				ctItem.setControl(new WdgExceptionDisplay(ctFolder, SWT.NONE, errorString));
			}
		}});
	}
}