package gov.anl.gmca.util;

import java.io.*;
import java.util.*;

import org.apache.commons.exec.*;
import org.apache.commons.exec.environment.EnvironmentUtils;

/**
 * Execute helper using apache commons exed
 *
 * From: https://stackoverflow.com/questions/7340452/process-output-from-apache-commons-exec
 * Modified and reviewed by MH
 *
 * @author wf
 *
 */
public class ExecTask2
{
    public static class ExecResult extends LogOutputStream
    {
        public ExecResult(String logPrefix, boolean printOutput) { super(); this.logPrefix=logPrefix; this.printOutput=printOutput; }
        private String logPrefix;
        private boolean printOutput;
        private int exitCode;
        public int getExitCode() { return exitCode; }
        public void setExitCode(int exitCode) { this.exitCode = exitCode; }
        private final ArrayList<String> lines = new ArrayList<>();
        @Override protected void processLine(String line, int level) {
            lines.add(line); if (printOutput) System.out.println(logPrefix+": "+line);
        }
        public ArrayList<String> getLines() { return lines; }
    }
    public static boolean shutdownHookAdded = false;
    public static Set<DefaultExecutor> executors = new HashSet<>();

    public static void addHookIfNeeded()
    {
        if (!shutdownHookAdded) {
            Runtime.getRuntime().addShutdownHook(new Thread(()->{
                synchronized(executors) {
                    for (DefaultExecutor executor : executors) {
                        executor.getWatchdog().destroyProcess();
                    }
                }
            }));
            shutdownHookAdded = true;
        }
    }

    public static ExecResult execCmd(ArrayList<String> cmd, String env, boolean useExistingEnv, String workingDir, int exitValue, boolean printOutput) {

        // generate command line and environment variables
        CommandLine cl = new CommandLine(cmd.get(0));
        for (int i=1;i<cmd.size();i++) {
            cl.addArgument(cmd.get(i), false);
        }
        HashMap<String,String> envMap = useExistingEnv ? new HashMap<>(System.getenv()) : new HashMap<>();
        if (null!=env) {
            StringTokenizer st = new StringTokenizer(env,"=;");
            while (st.hasMoreTokens()) {
                String key = st.nextToken();
                String val = st.nextToken();
                envMap.put(key,val);
            }
        }
        DefaultExecutor executor = new DefaultExecutor();
        if (null!=workingDir) executor.setWorkingDirectory(new File(workingDir));
        executor.setExitValue(exitValue);
        ExecResult result = new ExecResult(cmd.get(0), printOutput);
        executor.setStreamHandler(new PumpStreamHandler(result,result));
        executor.setWatchdog(new ExecuteWatchdog(Long.MAX_VALUE));
        try {
            synchronized(executors) { executors.add(executor); addHookIfNeeded(); }
            int returnVal = executor.execute(cl, envMap);
            result.setExitCode(returnVal);
        } catch (Exception e) { e.printStackTrace(); }
        finally {
            synchronized(executors) { executors.remove(executor); }
        }
        return result;
    }
}
