package gov.anl.gmca.util;

import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;

import java.util.ArrayList;
import java.util.StringTokenizer;

public class ProcessHandling
{
	public static enum Signal { INT, TERM, KILL };
	
	// kills only child processes of this BluIce client
	// returns the number of processes killed
	public static int killall(String processName, Signal killSignal)
	{
		ArrayList<Integer> pidList = getChildPids(processName, true);
		for (Integer pid : pidList)
		{
			kill(pid, killSignal);
		}
		return pidList.size();
	}

	public static void kill(Integer pid, Signal killSignal)
	{
		Log.note("Sending SIG"+killSignal+" to pid "+pid);
		ExecTask.syncExec("kill -"+killSignal+" "+pid, null, false);
	}
	
	public static boolean isPidAlive(Integer targetPid)
	{
		ArrayList<String> ppidList = ExecTask.syncExec("ps -o pid=,ppid=,command= x",null,false).stdout;
		for (int i=0;i<ppidList.size();i++)
		{
			String line = ppidList.get(i);
			
			StringTokenizer st = new StringTokenizer(line);
			String pid = st.nextToken();
			try
			{
				int iPid = Integer.parseInt(pid);
				if (iPid==targetPid) return true;
			}
			catch (NumberFormatException nfe)
			{
				System.out.println("ERROR: Couldn't parse pid \""+pid+"\"");
			}
		}
		return false;
	}

	protected static class Process
	{
		public int pid;
		public String cmd;
		public ArrayList<Process> childPidList = new ArrayList<Process>();
		public Process(int pid)
		{
			this.pid=pid;
		}
	}
	
	// returns all child process IDs of this BluIce client with a matching command name
	public static ArrayList<Integer> getChildPids(String processName, boolean includeChildrenOfCmd)
	{
    	ArrayList<Integer> matchingPidList = new ArrayList<Integer>();
    	
		// get all processes owned by this user
		// output is with no header, list of processes "[pid] [ppid] [command]"
		ArrayList<String> psOutput = ExecTask.syncExec("ps -o pid=,ppid=,command= x",null,false).stdout;

		// construct the process tree from this app as root
		Process app = new Process(Util.getPid());
		ArrayList<Process> currentLevel = new ArrayList<Process>();
		ArrayList<Process> nextLevel = new ArrayList<Process>();
		ArrayList<Process> allChildProcesses = new ArrayList<Process>();
		currentLevel.add(app);
		
		int SEARCH_DEPTH_MAX = 20;
		
		for (int j=0; j<SEARCH_DEPTH_MAX; j++)
		{
    		for (int i=0;i<psOutput.size();i++)
    		{
    			String line = psOutput.get(i);
    			StringTokenizer st = new StringTokenizer(line);
    			Integer pid=null,ppid=null;
    			try
    			{
    				pid = Integer.parseInt(st.nextToken());
    				ppid = Integer.parseInt(st.nextToken());
    			}
    			catch (NumberFormatException nfe)
    			{
    				System.out.println("Error parsing line \""+line+"\"");
    				continue;
    			}
    			
    			for (Process proc : currentLevel)
    			{
    				if (ppid == proc.pid)
    				{
    	    			Process thisProcess = new Process(pid);
    	    			thisProcess.cmd = line;
    	    			
    					proc.childPidList.add(thisProcess);
    					nextLevel.add(thisProcess);
    					allChildProcesses.add(thisProcess);
    					
    					break;
    				}
    			}
    		}
    		
    		if (0 == nextLevel.size()) break;
    		
    		currentLevel = nextLevel;
    		nextLevel = new ArrayList<Process>();
		}
		
		// gather child processes that match, plus optionally their children
		for (Process p : allChildProcesses)
		{
			//System.out.println("Process "+p.cmd+" children="+p.childPidList);
			if (p.cmd.contains(processName))
			{
				matchingPidList.add(p.pid);
				
				if (includeChildrenOfCmd)
				{
					currentLevel = new ArrayList<Process>();
					nextLevel = new ArrayList<Process>();
					currentLevel.add(p);
					
					for (int i=0; i<SEARCH_DEPTH_MAX; i++)
					{
						for (Process _p : currentLevel)
						{
            				for (Process child : _p.childPidList)
            				{
            					matchingPidList.add(child.pid);
            					nextLevel.add(child);
            				}
						}
						
						if (0 == nextLevel.size()) break;
						
						currentLevel = nextLevel;
						nextLevel = new ArrayList<Process>();
					}
				}
			}
		}
    	
    	return matchingPidList;
	}
}
