package gov.anl.gmca.util;

import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.*;

public class VerticalText
{
	private static Transform origTransform;
	
	public static void drawString(GC gc, String text, int x, int y, int side, Color shadowColor)
	{
		if (null == text || text.length() == 0) return;
		
		GC imageGC = null;
		Image image = null;
		Transform tr = null;
		try
		{
			if (null == origTransform) origTransform = new Transform(GUI.getDisplay());
			gc.getTransform(origTransform);
			
			// create image to draw to
			Point horizontalExtent = gc.textExtent(text);
			image = new Image(GUI.getDisplay(), horizontalExtent.x+4, horizontalExtent.y+4);
			
			// create image GC and copy settings from passed-in GC
			imageGC = new GC(image);
			imageGC.setBackground(gc.getBackground());
			imageGC.fillRectangle(0,0,horizontalExtent.x+4,horizontalExtent.y+4);
			imageGC.setFont(gc.getFont());
			
			// draw text to image
			if (null != shadowColor)
			{
				imageGC.setForeground(shadowColor);
				imageGC.drawString(text,0,0,true);
				imageGC.drawString(text,0,2,true);
				imageGC.drawString(text,2,2,true);
				imageGC.drawString(text,2,0,true);
			}
			imageGC.setForeground(gc.getForeground());
			imageGC.drawString(text,1,1,true);
			
			// draw the image sideways
			tr = new Transform(GUI.getDisplay());
			gc.getTransform(tr);
			tr.rotate(side==SWT.LEFT ? 270 : 90);
			
			gc.setAdvanced(true);
			gc.setTransform(tr);
	
			// rotate x,y coordinates based on above rotation angle
			int tx = side==SWT.LEFT ? -y : +y;
			int ty = side==SWT.LEFT ? +x : -x;
			
			gc.drawImage(image, tx, ty);
	
			gc.setAdvanced(false);
			gc.setTransform(origTransform);
		}
		catch (Exception e) { e.printStackTrace(); }
		finally
		{
			if (null != imageGC && !imageGC.isDisposed()) imageGC.dispose();
			if (null != image && !image.isDisposed()) image.dispose();
			if (null != tr && !tr.isDisposed()) tr.dispose();
		}
	}
}
