#!/usr/bin/perl -W
# _CALLED_FROM_BLUICE_
#--------------------------------------------------------------
# This script implements automatic sample centering with video
# from Axis/Digital camera/server and XREC from victor lamzin
# (http://www.embl-hamburg.de/xrec)
#
#     	Authors: Sudhir Pothineni and Sergey Stepanov
#
# Version-1.0:  2006/05/16 using Pezca, LWP and PerlMagick
# Version-2.0:  2009/03/25 uses single camera at low zoom and high zoom
# Version-2.1:  2009/09/22 adapted to work with the collimatorCtrl script
# Version-2.2:  2011/01/07 change to output popups via JBluIce (SS)
#--------------------------------------------------------------
# Algorithm (due to Ward Smith):
#
# Insert backlight, remove collimator
#
# for Low res camera:
# 1 take six images with axis_snapshot at Omega 0,60,120,180,240,300 for lowzoom
# & label files image1,2,3,4,5,6
# 2 take nine images with digital camera at Omega 0 40 80 120 160 200 240 280 320
# 2 convert files from .jpg to .png
# 3 run xrec with saved parameter file i.e. xrec input_file output_file
# 4 if success - adjust x,y if not exit
#
#
#--------------------------------------------------------------
  use strict;
  use LWP::UserAgent;           # World-Wide Web library for Perl (libwww-perl)
  use Image::Magick;            # ImageMagick API for Perl
  use DBI;			# MySQL interface (query camera type)
  use File::Spec;		# uncomment only when using display option
# use File::Copy;		# to copy/move files (does not take wildcards)
  use List::Util qw(min max);	# min/max functions (for zooming and cropping)
# use Math::Trig;		# needed for mono Theta <-> Energy calculations
  use Pezca;

  our ($beamline, $HAVE_XYZ);
  our ($IGNORE_ERRORS, $NONINTERACTIVE, $USE_ZENITY_FOR_EXIT, $ZENITY_VIA_BLUICE);
  our ($image, $inpDir, $imageDir, $imgType, $imgDir);
  my ($PEZCA, $script, @beamlinelist, $arg, $tell_bluice);
  my (@anglesLow, @idxLow, @anglesHigh, @idxHigh, @afterCenter, @idxAfterCenter);
  my ($file, $lowzoom, $highzoom, $zoomAssyHigh, $zoomAssyLow, $zoomAxisHigh, $zoomAxisLow);
  my ($hilo, $xcenter, $ycenter, $radius, $angle, $status, $width, $height);
  my ($found, $xpixel, $ypixel, $zpixel, $video, $clickString, $motionTimeout);
  my ($bkgPrefix, $sleep_interval, $degr2rad, $nangles, $narg, $syntax);
  my ($omegaAssyPV, $omegaAxisPV, $pointclickPV);
  my (@ct,$sec,$min,$hour,$mday,$mon,$monN,$year,$dateStamp,$message);
  my (@argArray, @types, @ips, $type, $ip, $i);
  my ($targetangle, $reliability, $str);
  my ($inpfile, $r2dfile, $r3dfile, $filehandle, $rasterFile, $rasterResultFile);
  my (@axesXY, $assyXY, $axisSH, $assySH, $omegaTolerance, $naxes);
  my ($omegaMtrPV, $mtrNo, $rsp, $omegaJogSpeed, $omegaPrgSpeed, $omegaScalePV, $omegaScale);
  my ($virtualMtrNo, $formula);
  my ($backbright, $ringbright, $ringlightPV, $backlightPV);
  my ($filter, $nprocesses, $autocenterCmd);
  my ($completePV, $coarseContourPV, $loopZPV, $faceOnOmegaPV, $omegaPV, $autoCenterPV);
  my ($maskfile, $yvaluesfile);
  my ($maskyvalues, $maskhull, $raster, $maxval, $val);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if (-f '/etc/passwd') {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/video_subs.pl';
  do $PEZCA.'/Common/motor_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;			# set unbuffered output
  select STDERR; $|=1;			# set unbuffered output
  
  $SIG{'HUP'}  = 'signal_handler';			#(1) hangup (see signum.h)
  $SIG{'INT'}  = 'signal_handler';			#(2) interrupt (see signum.h)
  $SIG{'QUIT'} = 'signal_handler';			#(3) quit (see signum.h)
  $SIG{'KILL'} = 'signal_handler';			#(9) this is what is sent by kill -9 -- unblockable
  $SIG{'TERM'} = 'signal_handler';			#(15) terminate -- sent by kill without paramaters
  $SIG{'STOP'} = 'signal_handler';			#(19) stop -- unblockable
#-----------------------------------------------------------------

  printf STDOUT "\n%s\n", scalar(localtime);

  $IGNORE_ERRORS    = 1;		# don't exit on error
  $NONINTERACTIVE   = 1;		# don't wait for any answers

  @beamlinelist = ('23i:', '23o:', '23b:', '23d:');
  $script    = &makeScriptName();
  @anglesLow = (60, 120, 180, 240, 300, 360);
  @idxLow    = (0,  1,   2,   3,   4,   5);
  @anglesHigh = (40, 80, 120, 160, 200, 240, 280, 320, 360);
  @idxHigh   = (0, 1, 2, 3, 4, 5, 6, 7, 8);
  @afterCenter = (30, 60, 90, 120);
  @idxAfterCenter = (0, 1, 2, 3);

  $inpDir   = './';			# inpfile=${inpDir}${beamline}_xrec_${hilo}.inp
  $imageDir = './';
  $rasterFile = './';
  $rasterResultFile = './';
  $imgType  = 'png';
  $imgDir   = '/dev/shm/';
# $imgType   = 'tif';			# TIFF make smaller files than PNG
# $bkgPrefix = 'background';		# expansion: ${imageDir}${beamline}_${bkgPrefix}_${hilo}.${imgType}
  $bkgPrefix = '';			# set to '' if no background used
  $sleep_interval = 0.05;
  $motionTimeout = 10;
  $omegaTolerance = 1.0;                # in degrees
  $tell_bluice = 0;			# default if no cmdline argument is given

  $degr2rad = 45./atan2(1.,1.);		# atan2(1)=pi/4
  $nangles = $#anglesLow+1;
  $narg = $#ARGV + 1;
  $raster = 0;
  
  $syntax = "\n\n".'Syntax: '.$script.' [beamline] [skipcentering][bluice|nobluice]'."\n\n";
  foreach $arg (@ARGV) {
    if (grep(/^$arg:?$/, @beamlinelist)) {
      $arg =~ s/:$//;
      if ($arg ne $beamline) {
        $beamline = $arg;
        print STDOUT "\n".'!!! Cross operation for beamline '.$beamline.' requested!'."\n";
        do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
      }
    }
    elsif ($arg =~ /^skipcentering$/i)   {$raster = 1;}
    elsif ($arg =~ /^bluice$/i)   {$tell_bluice = 1;}
    elsif ($arg =~ /^nobluice$/i) {$tell_bluice = 0;}
    else {
      print STDOUT '*** '.$script.': Unknown : '.$ARGV[0]."\n";
      &press_key_exit('*** '.$script.': Unknown cmdline argument: '.$arg.$syntax);
    }
  }
  if ($tell_bluice) {$ZENITY_VIA_BLUICE = 1;}

  ### Protection from multiple instances:
  if (-f '/etc/passwd') {
#    $filter = '/bin/ps -ef | '
     $filter = '/bin/ps -eo user:19,pid,cmd | '
              .'/bin/grep -v grep | '
              .'/bin/grep -v edit | '
              .'/bin/grep -v term | '
              .'/bin/grep -v onsole | '
              .'/bin/grep ';
     $nprocesses  = `${filter} -c autocenter_xrec`;
     chomp $nprocesses;
     $nprocesses = $nprocesses - 1;				#ignore this script
     if ($nprocesses > 0) {
        $NONINTERACTIVE      = 1;				#do not wait for any answers
        $USE_ZENITY_FOR_EXIT = 1;
	&zenity_error_exit('*** '.$nprocesses.' instance(s) of '.$script.' not yet finished. Please try again.');
     }
  }
  $lowzoom = 8.0;
  $highzoom = 6.0;
  if ($beamline =~ /^23o/i) { $backbright = 3.0; $ringbright = 4.0; $lowzoom = 6.0;}
  else { $backbright = 2.60;  $ringbright = 5.0; }

  $zoomAssyHigh = $beamline.':TV:V1';
  $zoomAxisHigh = $beamline.':TV:ZM1';
  $zoomAssyLow  = $beamline.':TV:V2';
  $zoomAxisLow 	= $beamline.':TV:ZM2';
  $omegaAssyPV  = $beamline.':GO:Om';
  $omegaAxisPV  = $beamline.':GO:OM';
  $omegaMtrPV   = $beamline.':GO:mom';
  $omegaScalePV = $beamline.':GO:OM:Scale';

  $pointclickPV = $beamline.':bi:pointclick';

  @axesXY       = ($beamline.':GO:CX', $beamline.':GO:CY');
  $assyXY       = $beamline.':GO:Cr';
  if (defined $HAVE_XYZ) {
    push (@axesXY, $beamline.':GO:CZ');
  } else {
     $axisSH    = $beamline.':GO:SH';
     $assySH    = $beamline.':GO:St';
     if ($beamline eq '23b') {$assySH .= 'B';}
  }
  $naxes = @axesXY;


  $ringlightPV  = $beamline.':DAC1_4.VAL';
  $backlightPV  = $beamline.':DAC1_8.VAL';
  
### Auto raster PVs
  $completePV		= $beamline.':bi:autoFind:complete';
  $coarseContourPV	= $beamline.':bi:autoFind:faceOnOmega';
  $loopZPV 		= $beamline.':bi:autoFind:loopZ';	
  $faceOnOmegaPV	= $beamline.':bi:autoFind:coarseContour';
  $omegaPV		= $beamline.':GO:OM:ActPos';
  $autoCenterPV		= $beamline.':bi:autoCenterRun';

  if ($beamline eq '23b') {$assySH .= 'B';}

### update autocenterPV
  $val = `caput $beamline':bi:autoCenterRun' 1`;

  $omegaScale = Pezca::GetDouble($omegaScalePV);
  &set_omega_speed_4x();
####xrec

### Move collimator out:
  $status = system('perl', $PEZCA.'/collimatorCtrl.pl', 'OUT', $beamline,'ringlight='.$ringbright, 'backlight='.$backbright);
  if ($status) {
    if ($tell_bluice) {&tellBluIce('autoCntr: coll-OUT failed','error');}
    &handle_press_key_exit('*** '.$script.': Cannot move collimator out: status='.$status);
  }

  $status = &addCaMonitor ($pointclickPV, 'string');
  &report_only($script, $status, $pointclickPV);

  @ct = localtime(time);		#($sec,$min,$hour,$mday,$mon,$year$,$wday,$yday,$isdst)
  ($sec,$min,$hour,$mday,$monN,$year) = ($ct[0],$ct[1],$ct[2],$ct[3],$ct[4]+1,$ct[5]+1900);
  $mon  = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec')[$ct[4]];
  $dateStamp = sprintf '%04d%s%02d_%02d%02d',$year,$mon,$mday,$hour,$min;
# $dateStamp = scalar(localtime);
  print STDOUT 'Getting camera types from mySQL'."\n";
  $status = &getCameraTypes(\@types,\@ips);		#data order: HiRes/LoRes/Hutch
  if ($status) {&zenity_error_exit($script.': error='.$status.' from getCameraTypes');}

  printf STDOUT "\n%s\n", scalar(localtime);
  $xpixel = 0;
  $radius = 0;
  if($raster == 1) {
  	$targetangle = `caget -t $omegaPV`
  }
### Loop over cameras:
if ($raster == 0 ) {
foreach $hilo ('LoRes','LoRes','LoRes','HiRes') {

    ### Take snapshots:
    if ($tell_bluice) {&tellBluIce('autoCentering started ..');}
    # 423 pixels : size of pin length from the tip to copper stem
    if ($hilo =~ /^LoRes$/i) {
      if($xpixel > 423 || $xpixel < 100 || $radius > 230 ) {  }
      else { next; }
    }

    if    ($hilo =~ /^HiRes$/i) {$ip = $ips[0]; $type = $types[0];}
    elsif ($hilo =~ /^LoRes$/i) {$ip = $ips[1]; $type = $types[1];}
    else {&handle_press_key_exit('*** '.$script.': incorrect camera = '.$hilo);}
    $status = &runMotor ($omegaAssyPV, $omegaAxisPV, 0, 'checkAuto');
    if ($status) {
      if ($tell_bluice) {&tellBluIce('autoCenter: Omega failed','error');}
      &handle_press_key_exit('*** '.$script.': Omega failed.',$omegaAssyPV,$omegaAxisPV);
    }
    if ($hilo =~ /^LoRes$/i) {
      $i = 0;
      print STDOUT 'Moving zoom to '.$lowzoom."\n";
      $status = &runMotor ($zoomAssyLow, $zoomAxisLow, $lowzoom, 'checkAuto');
      if ($status) {
        if ($tell_bluice) {&tellBluIce('autoCenter: lowzoom failed','error');}
        &handle_press_key_exit('*** '.$script.': lowzoom failed.');
      }
      
      print STDOUT 'Moving zoom to '.$highzoom."\n";
      $status = &runMotor ($zoomAssyHigh, $zoomAxisHigh, $highzoom, 'checkAuto');
      if ($status) {
        if ($tell_bluice) {&tellBluIce('autoCenter: highzoom failed','error');}
        &handle_press_key_exit('*** '.$script.': highzoom failed.',$omegaAssyPV,$omegaAxisPV);
      }
      $status = Pezca::PutDouble($backlightPV, $backbright);
      &report_only($script,$status,$backlightPV);
      
      foreach $angle (@anglesLow) {
        print STDOUT 'Taking image-'.$i."\n";
        ($image, $width, $height) = &takeImage ($dateStamp,$hilo,$i,$ip,$type);

        printf STDOUT 'Starting motion of Omega to %3.2f degr.'."\n",$angle;

        $status = &startNMotors ($omegaAssyPV, 1, ($omegaAxisPV), ($angle), 'checkAuto');
        if ($status) {
          if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
          &handle_press_key_exit('*** '.$script.': Omega start failed.');
        }
        print STDOUT 'Saving image-'.$i."\n";
        &saveImage ($dateStamp,$hilo,$i,$ip,$type,$image);
        printf STDOUT 'Waiting for Omega to arrive at %3.2f degr.'."\n",$angle;

	$status = &trackMotorFast ($omegaAssyPV, $motionTimeout, $omegaAxisPV, $angle, $omegaTolerance, 'RunPrg');
        if ($status) {
          if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
          &handle_press_key_exit('*** '.$script.': Omega track failed.',$omegaAssyPV,$omegaAxisPV);
        }
        $i++;
      }
    }		#endof taking images with LoRes camera
    if ($hilo =~ /^HiRes$/i) {
      $i = 0;

      #if ($beamline =~ /^23o/i) {
	$status = Pezca::PutDouble($ringlightPV, 0.0);
	&report_only($script,$status,$ringlightPV);
      #}
      $status = Pezca::PutDouble($backlightPV, $backbright);
      &report_only($script,$status,$backlightPV);
      
      foreach $angle (@anglesHigh) {

        print STDOUT 'Taking image-'.$i."\n";
        ($image,$width, $height) = &takeImage ($dateStamp,$hilo,$i,$ip,$type);

        printf STDOUT 'Starting motion of Omega to %3.2f degr.'."\n",$angle;

        $status = &startNMotors ($omegaAssyPV, 1, ($omegaAxisPV), ($angle), 'checkAuto');
        if ( $status ) {
          if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
          &handle_press_key_exit('*** '.$script.': Omega start failed.');
        }
        print STDOUT 'Saving image-'.$i."\n";
        &saveImage ($dateStamp,$hilo,$i,$ip,$type,$image);
        printf STDOUT 'Waiting for Omega to arrive at %3.2f degr.'."\n",$angle;

	$status = &trackMotorFast ($omegaAssyPV, $motionTimeout, $omegaAxisPV, $angle, $omegaTolerance, 'RunPrg');
        if ($status) {
          if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
          &handle_press_key_exit('*** '.$script.': Omega track failed.');
        }
        $i++;
      }
    }		#endof taking images with HiRes camera
    printf STDOUT "\n%s\n", scalar(localtime);
### Execute XREC:
    $inpfile = $imgDir.$inpDir.$beamline.'_'.$dateStamp.'_'.$hilo.'.inp';
    
    $rasterResultFile = $imgDir.$inpDir.$beamline.'_'.$dateStamp.'_'.$hilo.'.rasterResult';
#   $r2dfile = $inpDir.$beamline.'_'.$dateStamp.'_'.$hilo.'.r2d';
    $r3dfile = $imgDir.$inpDir.$beamline.'_'.$dateStamp.'_'.$hilo.'.r3d';
### Settings common for both high-res and low-res cameras:
### Sudhir:
    if ($hilo =~ /^LoRes$/i) {
      @argArray = ( 	'LOOP_POSITION 2',
			'NUM_IMG 6',
			'PREALIGN',
#			'CENTRE_COORD 240',
#			'SET_CROSS',
			'DATA_START',
			'0 '             .$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[0].'.'.$imgType,
			$anglesLow[0].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[1].'.'.$imgType,
			$anglesLow[1].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[2].'.'.$imgType,
			$anglesLow[2].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[3].'.'.$imgType,
			$anglesLow[3].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[4].'.'.$imgType,
			$anglesLow[4].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[5].'.'.$imgType,
			'DATA_END');
    }
    if ($hilo =~ /^HiRes$/i) {
      @argArray = ( 	'LOOP_POSITION 2',
			'NUM_IMG 9',
#			'CENTRE_COORD 240',
#			'SET_CROSS',
			'DATA_START',
			'0 '              .$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[0].'.'.$imgType,
			$anglesHigh[0].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[1].'.'.$imgType,
			$anglesHigh[1].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[2].'.'.$imgType,
			$anglesHigh[2].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[3].'.'.$imgType,
			$anglesHigh[3].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[4].'.'.$imgType,
			$anglesHigh[4].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[5].'.'.$imgType,
			$anglesHigh[5].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[6].'.'.$imgType,
			$anglesHigh[6].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[7].'.'.$imgType,
			$anglesHigh[7].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[8].'.'.$imgType,
			'DATA_END');
    }
### If background file has to be used:
### Sudhir:
    if ($bkgPrefix ne '') {
#     push @argArray, 'BACK '.$beamline.'_'.$bkgPrefix.'_'.$overidehilo.'.'.$imgType;
    }
### Settings specific for low-res camera only:
### Sudhir:
    if ($hilo =~ /^LoRes$/i) {
      #push @argArray, 'PREALIGN';
    }

    printf STDOUT "\n%s\n", scalar(localtime);
### Generate INP-file for xrec:
    $filehandle = open (INP,'> '.$inpfile);
    if (!defined $filehandle) {
      &handle_press_key_exit('Cannot open "'.$r3dfile.'"');
    }
    select INP; $|=1;			# set unbuffered output
    foreach $arg (@argArray) {print INP $arg."\n";}
    close(INP);
    if ($tell_bluice) {&tellBluIce('XREC processing the images..');}
### Execute xrec:
    $autocenterCmd = '/home/gmca/shared/Sudhir/XREC/xrec '.$inpfile.' '.$r3dfile.' 2>/dev/null';
    $status = system($autocenterCmd);

#   if ($status || !(-e $r2dfile) || !(-e $r3dfile)) {
    if ($status || !(-e $r3dfile)) {
      &handle_press_key_exit('*** '.$script.': XREC failed.');
    }
    ($targetangle,$radius, $xcenter, $ycenter, $reliability) = &readRes3dFile($r3dfile);
    print STDOUT $targetangle.', '.$radius.', '.$xcenter.', '.$ycenter.', '.$reliability."\n";
    printf STDOUT "\n%s\n", scalar(localtime);

### Move Omega to angle specified by xrec:
    print STDOUT 'Moving Omega to '.$targetangle.' given by XREC'."\n";
#   $targetangle = $anglesLow[$nangles-1] - $targetangle;

    if ($targetangle <  0.)  {$targetangle = $targetangle + 360.;}
    if ($targetangle > 360.) {$targetangle = $targetangle - 360.;}
    $status = &runMotor ($omegaAssyPV, $omegaAxisPV, $targetangle,'checkAuto');
    if ( $status ) {
      if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
      &handle_press_key_exit('*** '.$script.': Omega failed.');
    }
    if ($tell_bluice) {&tellBluIce('Making Auto centering..');}
    if    ( $hilo =~ /^HiRes$/i ) {
      $str = sprintf 'hires Reliability: %2.1f%s',$reliability,'%';
      if ($tell_bluice) {&tellBluIce($str);}
    }
    elsif ( $hilo  =~ /^LoRes$/i ) {
      $str = sprintf 'lowres Reliability: %2.1f%s',$reliability,'%';
      if ($tell_bluice) {&tellBluIce($str);}
    }
### MoveXYhead conventions on cameras: 0=LowRes 1=HiRes  #
    if    ( $hilo =~ /^HiRes$/i )  { $video = 1; }
    elsif ( $hilo  =~ /^LoRes$/i ) { $video = 0; }
    else                           { $video = '';}

    if    ( $hilo =~ /^HiRes$/i ) { $xpixel  =  int($ycenter); }
    elsif ( $hilo  =~ /^LoRes$/i ) { $xpixel  =  int($ycenter); if ($ycenter < 1) { $xpixel = 0;} }  #offset lowest loop size by hampton 25um = 22 pixels horizontal
    else                    { $xpixel  =  int($ycenter);}

    $ypixel = int($xcenter+$radius);
    if($ypixel > $height){$ypixel=$height};
    if($ypixel < 1 || $radius < 1) { $xpixel = 0; $ypixel = 240; };
    $clickString = $video.' '.$xpixel.' '.$ypixel.' '.$width.' '.$height;

    print STDOUT 'Requesting Point & Click Server to process '.$pointclickPV.' '.$clickString."\n";
    $status = Pezca::PutString($pointclickPV,$clickString);
    if ($status) {
      if ($tell_bluice) {&tellBluIce('autoCenter: click failed','error');}
      &handle_press_key_exit('*** '.$script.': click failed.');
    }
### Now wait it to finish (while moveXYhead.pl finishes):
    print STDOUT 'Waiting Point & Click Server to finish'."\n";

    select(undef,undef,undef,0.2);    		    #give P&C server time to start motion
### Track XY positioner motion:
    if ($ypixel != 0 || defined $HAVE_XYZ ) {
      $status = &trackNMotors($assyXY,$motionTimeout,$naxes,@axesXY);
      if ($status) {				    #Report error to BluIce:
        if ($tell_bluice) {&tellBluIce('autoCenter: XY failed','error');}
        &handle_press_key_exit('*** '.$script.': Tracking XY detected timeout.');
      }
      print STDOUT $script.': Finished tracking '.$assyXY."\n";
    }
### Track support horizontal motion:
    if ($xpixel != 0 && !defined $HAVE_XYZ) {
      $status = &trackMotor($assySH,$motionTimeout,$axisSH);
      if ($status) {				    #Report error to BluIce:
        if ($tell_bluice) {&tellBluIce('autoCenter: SH failed','error');}
	&handle_press_key_exit('*** '.$script.': Tracking SH detected timeout.');
      }
      print STDOUT $script.': Finished tracking '.$assySH."\n";
    }
    print STDOUT 'Point & Click Server done'."\n";
  }	# end of the main loop over all cameras (foreach...)
} #end if if loop for skipcentering
### For Auto Raster colelct 4 images 
  $i = 0;
  $hilo = 'HiResRaster';
  $rasterFile = $imgDir.$inpDir.$beamline.'_'.$dateStamp.'_'.$hilo.'.raster';
  foreach $angle (@afterCenter) {

        print STDOUT 'Taking image-'.$i."\n";
        ($image,$width, $height) = &takeImage ($dateStamp,$hilo,$i,$ip,$type);

        printf STDOUT 'Starting motion of Omega to %3.2f degr.'."\n",$angle+$targetangle;

        $status = &startNMotors ($omegaAssyPV, 1, ($omegaAxisPV), ($angle+$targetangle), 'checkAuto');
        if ( $status ) {
          if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
          &handle_press_key_exit('*** '.$script.': Omega start failed.');
        }
        print STDOUT 'Saving image-'.$i."\n";
        &saveImage ($dateStamp,$hilo,$i,$ip,$type,$image);
        printf STDOUT 'Waiting for Omega to arrive at %3.2f degr.'."\n",$angle+$targetangle;

	$status = &trackMotorFast ($omegaAssyPV, $motionTimeout, $omegaAxisPV, $angle+$targetangle, $omegaTolerance, 'RunPrg');
        if ($status) {
          if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
          &handle_press_key_exit('*** '.$script.': Omega track failed.');
        }
        $i++;
  }
  $targetangle = int($targetangle);	
  @argArray = ( 	'LOOP_POSITION 2',
			'NUM_IMG 4',
			'DATA_START',
			$targetangle.' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxAfterCenter[0].'.'.$imgType,
			$afterCenter[0]+$targetangle.' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxAfterCenter[1].'.'.$imgType,
			$afterCenter[1]+$targetangle.' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxAfterCenter[2].'.'.$imgType,
			$afterCenter[2]+$targetangle.' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxAfterCenter[3].'.'.$imgType,
			'DATA_END');
			
  			
  printf STDOUT "\n%s\n", scalar(localtime);
### Generate INP-file for xrec:
    $filehandle = open (INP,'> '.$rasterFile);
    if (!defined $filehandle) {
      &handle_press_key_exit('Cannot open "'.$rasterFile.'"');
    }
    select INP; $|=1;			# set unbuffered output
    foreach $arg (@argArray) {print INP $arg."\n";}
    close(INP);
    if ($tell_bluice) {&tellBluIce('XREC processing the images..');}
### Execute xrec:
    $autocenterCmd = '/home/gmca/shared/Sudhir/XREC/XREC_DAVE '.$rasterFile.' '.$rasterResultFile.' 2>/dev/null';
    $status = system($autocenterCmd);

    if ($status || !(-e $rasterResultFile)) {
      &handle_press_key_exit('*** '.$script.': XREC failed.');
    }		

#### Find maximum faceon view from 4 images
   my $maxValues = `cat /dev/shm/masks2_max.txt`;
   my @maxVals = split(" ", $maxValues);
   my $maxi = $maxVals[0];
   my $maxImg = 0;
   
   foreach $maxval (1, 2, 3) {
   	if( $maxVals[$maxval] > $maxi) {
		$maxImg = $maxval;
	}  
   }

### Read controur files and Write into contour Pvs
  
  $maskhull = `cat /dev/shm/masks2_hull_$maxImg.txt`;
  my @numhull = split(" ", $maskhull);
  my $hullno =int($numhull[0]+3);
  $maskhull = $hullno." ".$width." ".$height." ".$maskhull;
  
  $maskyvalues= `cat /dev/shm/masks2_yvalues_$maxImg.txt`;
  my @numyvalues = split(" ", $maskyvalues);
  my $numy = @numyvalues;
  $maskyvalues = $numy." ".$width." ".$height." ".$maskyvalues;
  
  if ($maxImg != 0) {
  	$targetangle = $afterCenter[$maxImg-1]+$targetangle;
  }
  	
  $val = `caput $beamline':bi:autoFind:faceOnOmega' $targetangle`;
  
  $val = `caput -a $beamline':bi:autoFind:coarseContour' $maskhull`;
  $val = `caput -a $beamline':bi:autoFind:loopZ' $maskyvalues`;
  
  $status = &runMotor ($omegaAssyPV, $omegaAxisPV, $targetangle,'checkAuto');
    if ( $status ) {
      if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
      &handle_press_key_exit('*** '.$script.': Omega failed.');
    }
  &restore_omega();
  &clearAllCaMonitors();
  system ('mv /dev/shm/'.$beamline.'* /home/gmca/shared/Sudhir/XREC/ 2>/dev/null');
  if ($tell_bluice) {&tellBluIce('autoCenter: finished');}
  $val = `caput $beamline':bi:autoFind:complete' 1`;
  system('rm -rf /dev/shm/masks2*.txt /dev/shm/masks2*.jpg /dev/shm/diffmask*.jpg');
  
  ### update autocenterPV
  $val = `caput $beamline':bi:autoCenterRun' 0`;
  exit 0;

#=================================================================
# save image
sub saveImage ($$$) {
  our ($beamline, $axisURL, $imageDir, $imgType, $imgDir, $image);
  my ($dateStamp, $hilo, $idx, $ip, $type);

  $dateStamp = shift(@_);
  $hilo      = shift(@_);
  $idx       = shift(@_);
  $ip        = shift(@_);
  $type      = shift(@_);
  $image     = shift(@_);

  $file = $imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idx.'.'.$imgType;
  print STDOUT 'Camera-'.$hilo.': saving image as '.$file."\n";
  $image->Write($file);
}

#=================================================================

# sub takeImage ($dateStamp,$hilo,$idx,$ip,$type) {
sub takeImage ($$$) {
  our ($image, $beamline, $axisURL, $imageDir, $imgType, $imgDir);
  my  ($dateStamp, $hilo, $idx, $ip, $type);
  my  ($compression, $colorlevel, $rotation, $overidehilo);
  my  ($downsample, $display, $status);

  $dateStamp = shift(@_);
  $hilo      = shift(@_);
  $idx       = shift(@_);
  $ip        = shift(@_);
  $type      = shift(@_);

### Camera and image options:
  $compression = 0;		# can be 0, 25, 50, 75, 100
  $colorlevel  = 100;		# can be ??? /0 = grayscale, 100 = full color/
  $rotation    = 0;
  $overidehilo = 'HiRes';

  $downsample = 4;
# $display = 0;
# $status = &saveCameraSnap($file,$hilo,$display,-$downscale,$type,$ip,$compression,$colorlevel,$rotation);
  $status = &getCameraSnap(\$image,$overidehilo,-$downsample,$type,$ip,$compression,$colorlevel,$rotation);
  if ($status) {&zenity_error_exit($script.': saveCameraSnap failed on camera='.$overidehilo.' with status='.$status);}


### Convert the image to grayscale:
### (do we need it?)
# $image->Quantize(colorspace=>'gray');

  $width  = $image->Get('columns');
  $height = $image->Get('rows');

  return $image,$width, $height;
}

#=================================================================

# sub readRes3dFile($r3dfile) {
#sudhir
sub readRes3dFile($) {
  my ($r3dfile, $string, $filehandle, @stringWords);
  my (@stringwords1, @stringwords2, @stringwords3);
  my (@stringwords4, @stringwords5, @stringwords6);
  my ($flag, $found, $xc, $yc, $angle);
# my ($T, $GBA, $QSel, $QCentr, $QImage, $Score, $Comment);

  $r3dfile = shift(@_);
  $string = '';

  $filehandle = open (DAT,'< '.$r3dfile);
  print STDOUT 'Reading "'.$r3dfile.'"'."\n";
  if (!defined $filehandle) {
    &press_key_exit('*** readRes3dFile: Cannot open "'.$r3dfile.'"');
  }

  $string=<DAT>;
  @stringwords1 = split ' ', $string;
  $string=<DAT>;
  @stringwords2 = split ' ', $string;
  $string=<DAT>;
  @stringwords3 = split ' ', $string;
  $string=<DAT>;
  @stringwords4 = split ' ', $string;
  $string=<DAT>;
  @stringwords5 = split ' ', $string;
  $string=<DAT>;
  @stringwords6 = split ' ', $string;

close(DAT);
 # if( $stringWords[0] eq 'TARGET_ANGLE') {
  $targetangle = $stringwords1[1];
 # }
 # if( $stringWords[2] eq 'RADIUS') {
  $radius = $stringwords2[1];
 # }
 # if( $stringWords[4] eq 'Y_CENTRE') {
  $ypixel = $stringwords3[1];
 # }
 # if( $stringWords[6] eq 'X_CENTRE') {
  $xpixel = $stringwords4[1];
 # }
#  if( $stringWords[10] eq 'RELIABILITY') {
  $reliability = $stringwords5[1];
 # }
  print STDOUT "\n".'*** '.$stringwords1[0].'  '.$stringwords1[1].'  '.$targetangle."\n";
  print STDOUT "\n".'*** '.$stringwords2[0].'  '.$stringwords2[1].'  '.$radius."\n";
  print STDOUT "\n".'*** '.$stringwords3[0].'  '.$stringwords3[1].'  '.$ypixel."\n";
  print STDOUT "\n".'*** '.$stringwords4[0].'  '.$stringwords4[1].'  '.$xpixel."\n";
  print STDOUT "\n".'*** '.$stringwords5[0].'  '.$stringwords5[1].'  '.$reliability."\n";

  return ($targetangle,$radius, $xpixel, $ypixel, $reliability);
}

#=================================================================

sub set_omega_speed_4x() {

  $virtualMtrNo = 32;
  $mtrNo        = 5;				#hardcode to save time
# $mtrNo        = &getMtrNo($omegaMtrPV);	#we use hardcoded value
  if (&isMtrOutLinked($omegaAssyPV,$virtualMtrNo) == 1) {
### If the virtual motor is linked to Omega,
### then read the linking equation & unlink:
     print STDOUT 'Un-linking Omega from virtual motor'."\n";
     $formula = &readPcsRsp ($omegaAssyPV, '#'.$virtualMtrNo.'->');
     &unlinkMotor_Pcs($omegaAssyPV, $virtualMtrNo);
  }
### Read current Omega jog speed:

  $omegaJogSpeed = &readMtrRsp($omegaMtrPV,'I'.$mtrNo.'22','numeric');
  my $omegaJobSpeedDeg = int ($omegaJogSpeed * $omegaScale * 1000 + 0.5);
  print STDOUT 'Omega motor normal speed = '.$omegaJobSpeedDeg."\n";
### Set Omega program speed 4x normal:

  $omegaPrgSpeed = 4 * $omegaJogSpeed ;
  &sendMtrCmd($omegaMtrPV, 'I'.$mtrNo.'16='.$omegaPrgSpeed);
### Verify that the program speed has been set:
# $omegaPrgSpeed = &readMtrRsp($omegaMtrPV, 'I'.$mtrNo.'16','numeric');
  my $omegaPrgSpeedDeg = int($omegaPrgSpeed * $omegaScale * 1000 + 0.5);
  print STDOUT 'Omega motor prg high speed = '.$omegaPrgSpeedDeg."\n";
}

###########################################################################
sub pvwrite ($$;$) {

  our $beamline;
  my ($pv, $val, $type, $status);
  $pv   = shift(@_);
  $val  = shift(@_);
  $type = shift(@_);	if(!defined $type) {$type='double';}
  $pv = $beamline.':'.$pv;
  if    ($type =~ /^double$/i)   {$status = Pezca::PutDoubleOldCa($pv, $val);}
  elsif ($type =~ /^string$/i)   {$status = Pezca::PutStringOldCa($pv, $val);}
  elsif ($type =~ /^wfstring$/i) {$status = &PezcaPutStringWF($pv, $val);}
  else                           {&press_key_exit('*** '.$script.': incorrect PV type='.$type);}
  &report_only($script, $status, $pv);
}

#=================================================================

sub restore_omega() {

  if (defined $omegaAssyPV && defined $omegaAxisPV && defined $omegaPrgSpeed) {
     my $omegaJobSpeedDeg = int ($omegaJogSpeed * $omegaScale * 1000 + 0.5);
     print STDOUT 'Restoring Omega motor normal speed = '.$omegaJobSpeedDeg."\n";
### This should reset the program speed to the jog speed:
     &setAxisHiSpeed($omegaAssyPV,$omegaAxisPV);

     if (defined $formula) {
### Omega was initially linked to the
### virtual motor; restore the link:
        print STDOUT 'Re-linking Omega to virtual motor & syncing postions (formula='.$formula.")\n";
        &sendPcsCmd($omegaAssyPV, '#'.$virtualMtrNo.' homez');		#Zero motor-OUT position
        &sendPcsCmd($omegaAssyPV, '#'.$virtualMtrNo.'->'.$formula);	#Link using the formula
        &sendPcsCmd($omegaAssyPV, 'R');					#Sync the positions
     }
  }
}

#=================================================================

sub handle_press_key_exit($) {

  $message = shift(@_);
  
  ### update autocenterPV
  $val = `caput $beamline':bi:autoCenterRun' 0`;
  
  &restore_omega();
  &press_key_exit($message);
}

#=================================================================

sub signal_handler {
  my $signame = shift;
  if ($signame ne 'quit') {print STDOUT "\n".' *** Caught a SIG'.$signame."\n";}
  
  ### update autocenterPV
  $val = `caput $beamline':bi:autoCenterRun' 0`;
  
  &restore_omega();
  &clearAllCaMonitors();
  print STDOUT 'Done!'."\n\n";
  exit 1;
}
