import PyQt5.QtWidgets as qtw
import PyQt5.QtCore as qtcore

import sys
import urllib3
import datetime
import subprocess


class MainWindow(qtw.QMainWindow):

    def __init__(self):

        self.http = urllib3.PoolManager()
        self.urls = [
            'http://10.20.103.71:8005/state',
            'http://10.20.103.138:8001/state',
            'http://10.20.103.154:8005/counter'
        ]
        self.labels = [
            "ID-D JBluIce",
            "ID-B JBluIce",
            "ID-B StreamWriter"
        ]

        # main window
        qtw.QMainWindow.__init__(self)
        self.setMinimumSize(qtcore.QSize(640,480))
        self.setWindowTitle('HTTP-JSON monitor')

        # create empty tabs
        self.tabs = qtw.QTabWidget()
        tab_widgets = []
        for i in range(0, 3):

            # create the content widget
            w = qtw.QWidget()
            tab_widgets.append(w)

            # set layout
            w.layout = qtw.QGridLayout()
            w.layout.setColumnStretch(0, 5)
            w.layout.setColumnStretch(1, 1)
            w.layout.setColumnStretch(2, 1)
            w.layout.setColumnStretch(3, 1)
            w.setLayout(w.layout)

            # add widget to tab
            self.tabs.addTab(w, self.labels[i])

        self.setCentralWidget(self.tabs)

        # add text widgets
        self.text_widgets = []
        for i in range(0, 3):

            # create the widget
            tw = qtw.QTextEdit(tab_widgets[i])
            self.text_widgets.append(tw)

            # configure
            tw.setReadOnly(True)
            font = tw.font()
            font.setFamily('monospace')
            tw.document().setDefaultFont(font)

            # add to tab
            tab_widgets[i].layout.addWidget(tw, 0, 0, 1, 4)

        # add buttons
        for i in range(0, 3):

            # get the tab widget
            tw = tab_widgets[i]

            # spacer
            tw.spacer = qtw.QWidget()
            tw.layout.addWidget(tw.spacer, 1, 0)

            # add check
            tw.btn_check = qtw.QPushButton('Check')
            tw.btn_check.setStyleSheet('background-color:#99f')
            tw.btn_check.clicked.connect(self.check_server)
            tw.layout.addWidget(tw.btn_check, 1, 1)

            # add restart
            tw.btn_restart = qtw.QPushButton('Restart')
            tw.btn_restart.setStyleSheet('background-color:#f99')
            tw.btn_restart.clicked.connect(self.restart_server)
            tw.layout.addWidget(tw.btn_restart, 1, 2)

            # add close
            tw.btn_close = qtw.QPushButton('Close')
            tw.btn_close.clicked.connect(lambda: self.close())
            tw.layout.addWidget(tw.btn_close, 1, 3)

        qtcore.QTimer.singleShot(100, self.update)

    def keyPressEvent(self, event):
        if event.key() == qtcore.Qt.Key_Escape:
            self.close()


    def check_server(self):
        subprocess.Popen(["jbs-check", "popup"])


    def restart_server(self):
        subprocess.Popen(["jbs-restart", "popup"])


    def update(self):

        current_index = self.tabs.currentIndex()
        current_doc = self.text_widgets[current_index].document()
        timestamp = datetime.datetime.now().strftime('[%Y-%m-%d %H:%M:%S.%f]')
        success = False

        try:
            request = self.http.request('GET', self.urls[current_index])
            if (200 != request.status):
                current_doc.setHtml(timestamp+"<p>ERROR: Server returned code "+request.status)
            else:
                jsonText = request.data.decode('utf-8')
                jsonText = "<pre>"+jsonText+"</pre>"
                jsonText = jsonText.replace("\\n","<br>")
                current_doc.setHtml(timestamp+jsonText)
                success = True
        except Exception as e:
            current_doc.setHtml(timestamp+"<p>ERROR: "+str(e))

        qtcore.QTimer.singleShot(100 if success else 1000, self.update)


if __name__ == '__main__':
    app = qtw.QApplication(sys.argv)
    mainWin = MainWindow()
    mainWin.show()
    sys.exit(app.exec_())
