#!/bin/csh 
#
# Running XDS in parallel to data collection, not Generic, works only on 23ID beamlines @APS
#
# Usage : run_XDS.csh <file_prefix> <working_dir> <number of energies1/2/3> <first_framename> <fast_dp 0/1> <scala/aimless> 
#			<beamX> <beamY> <firstFrameNum> <lastFrameNum> <isInverse 0/1> <isFinal 0/1> <imageSet> <expType> <replace_dir_prefix> <unitCell> <spaceGroup>
#
# Sudhir Babu Pothineni, GM/CA CAT, APS, Argonne National Laboratory
# Ver 1 : running XDS, XSCALE, POINELESS, SCALA, TRUNCATE 10/18/2011
# Ver 2 : runs fast_dp 09/10/2012
# Ver 3 : scala/aimless
# Ver 4 : Non-standard data collection use by default XDS (inverseBeam, Vector, Raster) 05/21/2013
# Ver 5 : support for cbf format files 01/14/2014
# Ver 6 : replacing CTRUNCATE with TRUNCATE

#TODO : XML file for commandline parameters

#Check number of arguments

# set environment for XDS
setenv PATH ${PATH}:/mnt/software/px/XDS
# source the csh environment, sometimes GridEngine environment can't load automatically
source /mnt/software/px/cshrc_px
# source grid engine environment
# source /mnt/software/px/sge_settings.csh

if ($#argv  < 15 ) then
	echo "Wrong number of arguments for running fast_dp/GMCAproc"
	echo "Usage : run_XDS.csh <file_prefix> <working_dir> <number of energies1/2/3> <first_framename> <fast_dp 0/1> <scala/aimless>" 
	echo "        <beamX> <beamY> <firstFrameNum> <lastFrameNum> <isInverse 0/1> <isFinal 0/1> <imageSet> <expType> <replace_dir_prefix> <unitCell> <spaceGroup>"
	echo ""
	echo "Parameters <unitCell> <spaceGroup> are optional"
endif

echo "Host: "`hostname`
set tempLogDir = ".JBluIce"

# input from JBluIce#############################
set prefix = $1
set workDir = $2
set noofenergies = $3
set first_name = $4
set last_image = $5
set isFastDP = $6
set scale = $7
set beamX = $8
set beamY = $9
set firstFrameNum = $10
set lastFrameNum = $11
set isInverse = $12
set isFinal = $13
set imageSet = $14
set expType = $15
set imageDir = $16 

set unitcell = ""
set spacegroup = ""

if ($#argv == 18) then
	set unitcell="$17"
	set spacegroup=$18
endif
#################################################
#
#################################################
# WAITING for DATA ##############################
#################################################
# check for log file
set logfile=`echo $first_name | sed 's/cbf/log/g'`

# check file extension for cbf 
set ext = `echo $first_name | cut -d'.' -f2`

if ( $isFinal == 1 && "$ext" == "cbf") then
	# Sleep for 5 second before checking image file
	set eachSleep = 5
	# Total number of seconds spend in loop checking image file
	set totalSleep = 0
	set maxSleep = 90
	set moreThanMaxSleep = 300 # 5 minutes

	# MH: removing log check for now since Eiger(?) doesn't write it
	#echo `date +"%T"`
	# Loop to check image file until it becomes available
	# or timeout
	echo "waiting for log file $imageDir/$logfile"
	#while ($totalSleep < $maxSleep)
	#	if (! -e $imageDir/$logfile) then
    	#		sync
    	#		# Sleep before checking the file again
    	#		sleep $eachSleep
    	#		@ totalSleep += $eachSleep
	#	else
    	#		# Get out of the loop once we have found the file
    	#		echo `date +"%T"` "Wait time $totalSleep sec"
	#		set totalSleep = $moreThanMaxSleep
	#	endif
	#end

# MH: increasing sleep time to compensate for skipping log check
#sleep 10
sleep 20

echo `date +"%T"`
endif
####################################################
set NUMBER_OF_JOBS_CLUSTER = 32
set NUMBER_OF_CORES = 4

set scriptDir = `dirname $0`
set beamline = ""
set firstDataset = ""

cd $imageDir
echo "Image directory: "$imageDir
echo "Working directory: "$workDir
set files = 
set digits = 
if ("$ext" != "cbf") then
        echo "Images: ${prefix}*.??????"
	set files = `ls ${prefix}*.?????[0-9] | grep -v "_scr." | grep -v "_chk." | grep -v "_raster"| grep -v "(_0.)"`
else
        set digits = `echo $first_name | cut -d'.' -f1 |rev |cut -d'_' -f1 |rev`
        set ndigits = `echo ${digits} | awk '{print length($0)}'`

        if ($ndigits == 6 ) then
            echo "Images: ${prefix}*_??????.$ext"
        else
            echo "Images: ${prefix}*_?????.$ext"
        endif

	set files = `ls ${prefix}_*.$ext |grep -P "_\d{5,6}.$ext" | grep -v "_scr." | grep -v "_chk." | grep -v "_raster"| grep -Pv "_0_\d{5,6}"`
endif


@ i = 1
#set nofiles = `echo ${#files} - 1 | bc`
set nofiles = ${#files}

#####################################################
# write manual processing XDS.INP
if ( -e $workDir/../${prefix}_XDS) then
	rm -rf $workDir/../${prefix}_XDS
endif

mkdir $workDir/../${prefix}_XDS
cd $workDir/../${prefix}_XDS
#####################################################

if ("$ext" != "cbf") then
	$scriptDir/generate_XDS.INP "$imageDir/${prefix}.??????" > /dev/null
else
        if ($ndigits == 6) then
            $scriptDir/generate_XDS.INP "$imageDir/${prefix}_??????.cbf" > /dev/null
            echo $scriptDir/generate_XDS.INP "$imageDir/${prefix}_??????.cbf" 
        else
            $scriptDir/generate_XDS.INP "$imageDir/${prefix}_?????.cbf" > /dev/null
            echo $scriptDir/generate_XDS.INP "$imageDir/${prefix}_?????.cbf" 
        endif
endif


# change to working dir
cd $workDir

#copy documentation for data processing
#set docfile = "/mnt/software/bluice/docs/README_GMCA_AutomatedProcessing.txt"

#if( -e $docfile) then
#	cp -f $docfile ./
#endif

#initialize number of datasets
@ numberofdatasets = 0

#check for experimental type
if ( $expType != "Standard" || $isInverse == 1) then
	set isFastDP=1 
endif

#check if JBluIce XDS parameters file exists
set isSpaceGroup = ""
if ( -e .xdsparams) then
	set isSpaceGroup = `cat .xdsparams | grep "UNIT_CELL_CONSTANTS"`
endif

#create directory for combining multiple datasets, will be removed at the end for single dataset
if ( ! -e ${prefix}_COMBINED ) then
	mkdir ${prefix}_COMBINED
endif

#create directory for xscale and pointless, aimless
mkdir ${prefix}_COMBINED/XSCALE
mkdir ${prefix}_COMBINED/POINTLESS_AIMLESS

# created pointless script
if ( $#argv == 18 || "$isSpaceGroup" != "" ) then
	echo "pointless -c HKLOUT ${prefix}_xds.mtz << EOF" > ${prefix}_COMBINED/POINTLESS_AIMLESS/pointless.sh
else
	echo "pointless HKLOUT ${prefix}_xds.mtz << EOF" > ${prefix}_COMBINED/POINTLESS_AIMLESS/pointless.sh
endif

echo "SETTING SYMMETRY-BASED" >> ${prefix}_COMBINED/POINTLESS_AIMLESS/pointless.sh

echo "OUTPUT_FILE=XDS_ASCII.HKL" > ${prefix}_COMBINED/XSCALE/XSCALE.INP
echo "FRIEDEL'S_LAW=FALSE MERGE=FALSE" >> ${prefix}_COMBINED/XSCALE/XSCALE.INP
echo "MAXIMUM_NUMBER_OF_PROCESSORS=$NUMBER_OF_CORES" >> ${prefix}_COMBINED/XSCALE/XSCALE.INP
		
while ( $i < $nofiles )
	set file1 = $files[$i]
	set siteFirstFrame=$file1
	set num = 0
	if ( $expType != "Standard" || $isInverse == 1) then
		
		while( $i < $nofiles )
			set prefix1 =
			set number = 
			set number1 =
		
			if ("$ext" != "cbf") then
				set prefix1 = `echo $files[$i] | cut -d'.' -f1`
				set number = `echo $files[$i] | cut -d'.' -f2`
			else
				set nocbf = `echo $files[$i] | cut -d'.' -f1`
				set prefix1 = `echo $nocbf | rev | cut -d'_' -f2-10 | rev`
				set number = `echo $nocbf | rev | cut -d'_' -f1 | rev`
			endif
		
			set num = `echo $number | bc`
			@ i ++
		
			set file2 = $files[$i]
			if ("$ext" != "cbf") then
				set prefix2 = `echo $files[$i] | cut -d'.' -f1`
				set number1 = `echo $files[$i] | cut -d'.' -f2`
			else
				set nocbf = `echo $files[$i] | cut -d'.' -f1`
				set prefix2 = `echo $nocbf | rev | cut -d'_' -f2-10 | rev`
				set number1 = `echo $nocbf | rev | cut -d'_' -f1 | rev`
			endif
		
			set num1 = `echo $number1 | bc`
		
			if ( "$prefix1" != "$prefix2" ) then
				break
			endif
			
			if ( `echo $num+1 | bc` != $num1 ) then
				break
			else
				set num = $num1
			endif
		end
		
		
	else
		set i = $nofiles
		if ("$ext" != "cbf") then
			set prefix1 = `echo $files[$i] | cut -d'.' -f1`
			set number = `echo $files[$i] | cut -d'.' -f2`
		else
			set nocbf = `echo $files[$i] | cut -d'.' -f1`
			set prefix1 = `echo $nocbf | rev | cut -d'_' -f2-10 | rev`
			set number = `echo $nocbf | rev | cut -d'_' -f1 | rev`
		endif
		
		set num = `echo $number | bc`
	endif
	
	if ( $i == `echo $nofiles | bc` ) then
		#set num = `echo $num+1 | bc`
	endif
	#echo $i -1 | bc
	set s = `echo $i-1 | bc`
	
	if ("$ext" != "cbf") then
		set startFramePrefix=`echo $file1 | cut -d'.' -f1`
		set startFrame=`echo $file1 | cut -d'.' -f2`
		set startFrameName = `echo $file1 | cut -d'.' -f2`
		set startFrameName = `echo $startFrameName | bc`
	else
		set nocbf = `echo $file1 | cut -d'.' -f1`
		set startFramePrefix=`echo $nocbf | rev | cut -d'_' -f2-10 | rev`
		set startFrame=`echo $nocbf | rev | cut -d'_' -f1 | rev`
		set startFrameName = `echo $startFrame | bc`
	endif
	
	set endFrame=$num
	set endBack=`echo $startFrame+5 | bc`
	
	#image set
	
	if ( $numberofdatasets > 0) then
			echo "$startFrameName-$endFrame" >> imagesets
	else
			echo "$startFrameName-$endFrame" > imagesets
	endif
	
	# check for fast_dp/GMCAproc
	if ( $isFastDP == 0 && $isInverse != 1 ) then
		#if ("$expType" == "Standard" ) then
		#	set dir = ${first_name}_fast_dp_$imageSet
		#else
		#	set dir = ${siteFirstFrame}_fast_dp_$imageSet
		#endif
		set dir = ${prefix}_fast_dp_$imageSet
	else
		if ("$expType" == "Standard" && $isInverse != 1 ) then
			if ( $isFinal == 1 ) then
				set dir = ${prefix}_GMCAproc_Final_$startFrameName-$endFrame-$numberofdatasets
			else
				set dir = ${prefix}_GMCAproc_$startFrameName-$endFrame-$numberofdatasets
			endif
		else
			if ( $isFinal == 1 ) then
				set dir = ${prefix}_GMCAproc_Final_$startFrameName-$endFrame-$numberofdatasets
			else
				set dir = ${prefix}_GMCAproc_$startFrameName-$endFrame-$numberofdatasets
			endif
		endif
	endif
	
	if ( ! -e $dir ) then
		mkdir $dir
	endif
	
	# save first dataset directory
	if ( $numberofdatasets == 0) then
		set firstDataset = $dir;
	endif
	
	cd $dir
	
	# PHISTART
	
	set PHISTART =
	
	if ("$ext" != "cbf") then
		set SKIP = `echo "1024+256+128+256+44"| bc`
		set PHISTART = `od -t dI -j $SKIP -N 4 $imageDir/$siteFirstFrame | head -1 | awk '{print $2}'`
		set PHISTART = `echo $PHISTART/1000 | bc`
	else
		set PHISTART = `strings $imageDir/$siteFirstFrame | grep Start_angle | awk '{print $3}'`
		#echo "PHI START : $PHISTART"
	endif 
	
	# keeping orientation matrix for the next datasets
	set aaxis =
	set baxis =
	set caxis =
	echo "Number of datasets : "$numberofdatasets
	if ( $numberofdatasets > 0) then
		echo "../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL"
		if ( -e ../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL) then
			
			set aaxis = `cat ../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL | grep UNIT_CELL_A-AXIS= | awk '{print $2" "$3" "$4}'`
			set baxis = `cat ../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL | grep UNIT_CELL_B-AXIS= | awk '{print $2" "$3" "$4}'`
			set caxis = `cat ../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL | grep UNIT_CELL_C-AXIS= | awk '{print $2" "$3" "$4}'`
		endif	
	endif
	
	set totalSleep = 0
	set xdsfilepath = ""
	set xdspathforscale = ""
	# check for fast_dp/XDS
	if ( $isFastDP == 0 ) then
		repeatOnerror:
		if ("$expType" == "Standard" && $isInverse != 1 ) then
			
			if ($#argv == 18 ) then
			     #set localcell = `echo $unitcell | sed 's/ /,/g'`
			     echo "/mnt/software/px/fast_dp/bin/fast_dp -b $beamX,$beamY -j $NUMBER_OF_JOBS_CLUSTER -c $unitcell -s $spacegroup  -1 $firstFrameNum -N $lastFrameNum -S $scale $imageDir/$file1"
			    /mnt/software/px/fast_dp/bin/fast_dp -b $beamX,$beamY -j $NUMBER_OF_JOBS_CLUSTER -c $unitcell -s $spacegroup -1 $firstFrameNum -N $lastFrameNum -S $scale $imageDir/$file1
			else
			     echo "/mnt/software/px/fast_dp/bin/fast_dp -b $beamX,$beamY -j $NUMBER_OF_JOBS_CLUSTER -1 $firstFrameNum -N $lastFrameNum -S $scale $imageDir/$file1"
			    /mnt/software/px/fast_dp/bin/fast_dp -b $beamX,$beamY -j  $NUMBER_OF_JOBS_CLUSTER -1 $firstFrameNum -N $endFrame -S $scale $imageDir/$file1
			endif
		else
			echo "/mnt/software/px/fast_dp/bin/fast_dp -b $beamX,$beamY -j $NUMBER_OF_JOBS_CLUSTER -1 $startFrameName -N $endFrame -S $scale $imageDir/$file1"
			if ($#argv == 18 ) then
				#set localcell = `echo $unitcell | sed 's/ /,/g'`
				echo "/mnt/software/px/fast_dp/bin/fast_dp -b $beamX,$beamY -j $NUMBER_OF_JOBS_CLUSTER -c $unitcell -s $spacegroup -1 $startFrameName -N $endFrame -S $scale $imageDir/$file1"
				/mnt/software/px/fast_dp/bin/fast_dp -b $beamX,$beamY -j $NUMBER_OF_JOBS_CLUSTER -c $unitcell -s $spacegroup -1 $startFrameName -N $endFrame -S $scale $imageDir/$file1
			else
				/mnt/software/px/fast_dp/bin/fast_dp -b $beamX,$beamY -j $NUMBER_OF_JOBS_CLUSTER -1 $startFrameName -N $endFrame -S $scale $imageDir/$file1
			endif
		endif 
		
		# scaling by sudhir for reference (same as XDS case)
		#$scriptDir/scale.csh $scale $siteFirstFrame $#argv
		
		#if ( $isFinal == 1 ) then
		#Repeat above steps on images missing
		set imageMissing = `grep "Fast DP error: images missing" fast_dp.log`
		set maxSleep = 90
		if("$imageMissing" != "" && $totalSleep < $maxSleep) then
			sleep $eachSleep
    			@ totalSleep += $eachSleep
			goto repeatOnerror
		endif
		#endif
		#write the logfile
		echo `pwd` > ../subdirfile
		
	else	
		echo "Creating XDS.INP using generate_XDS.INP from XDS wiki"
		mkdir XDS
		cd XDS
		if ("$ext" != "cbf") then
			echo "$scriptDir/generate_XDS.INP $imageDir/${startFramePrefix}.??????"
			$scriptDir/generate_XDS.INP "$imageDir/${startFramePrefix}.??????" > /dev/null
		else
                        if ($ndigits == 6) then
				echo "$scriptDir/generate_XDS.INP $imageDir/${startFramePrefix}_??????.cbf"
				$scriptDir/generate_XDS.INP "$imageDir/${startFramePrefix}_??????.cbf" > /dev/null
                        else
				echo "$scriptDir/generate_XDS.INP $imageDir/${startFramePrefix}_?????.cbf"
				$scriptDir/generate_XDS.INP "$imageDir/${startFramePrefix}_?????.cbf" > /dev/null
                        endif
                        # copy a copy of XDS.INP for user
			mkdir -p $workDir/../${prefix}_XDS/$dir
			cp XDS.INP mkdir -p $workDir/../${prefix}_XDS/$dir
			#echo "$scriptDir/XDS_from_H5_GMCA.py $imageDir/../${startFramePrefix}_master.h5"
			#$scriptDir/XDS_from_H5_GMCA.py "$imageDir/../${startFramePrefix}_master.h5" > /dev/null
		endif
		
		echo "STARTING_ANGLE= $PHISTART STARTING_FRAME= $startFrameName" >> XDS.INP
		
		if ( $numberofdatasets > 0) then
			#echo "egrep -v 'DATA_RANGE|SPOT_RANGE|BACKGROUND_RANGE|JOB' XDS.INP > XDS.INP_new"
			egrep -v 'DATA_RANGE|SPOT_RANGE|BACKGROUND_RANGE|JOB' XDS.INP > XDS.INP_new
		else
			#echo "egrep -v 'DATA_RANGE|SPOT_RANGE|BACKGROUND_RANGE' XDS.INP > XDS.INP_new"
			egrep -v 'DATA_RANGE|SPOT_RANGE|BACKGROUND_RANGE' XDS.INP > XDS.INP_new
		endif
		
		if ($expType == "Standard" && $isInverse != 1 ) then
			set endBack=`echo $firstFrameNum+5 | bc`
			
			echo "DATA_RANGE= $firstFrameNum $lastFrameNum" >> XDS.INP_new
			set spotEndFrame = `echo "$firstFrameNum+($lastFrameNum - $firstFrameNum)/2" | bc`
			echo "SPOT_RANGE= $firstFrameNum $spotEndFrame" >> XDS.INP_new
			#echo "BACKGROUND_RANGE= $firstFrameNum $endBack" >> XDS.INP_new
		else
			#if ( $numberofdatasets > 0) then
			#	echo "$startFrameName-$endFrame" >> ../../imagesets
			#else
			#	echo "$startFrameName-$endFrame" > ../../imagesets
			#endif
			
			echo "DATA_RANGE= $startFrameName $endFrame" >> XDS.INP_new
			set spotEndFrame = `echo "$startFrameName+($endFrame - $startFrameName)/2" | bc`
			echo "SPOT_RANGE= $startFrameName $spotEndFrame" >> XDS.INP_new
			echo "BACKGROUND_RANGE= $startFrameName $endBack" >> XDS.INP_new
			
		endif
		
		#echo "REFINE(IDXREF)= BEAM ORIENTATION AXIS ! CELL DISTANCE" >> XDS.INP_new
			
		#echo "TRUSTED_REGION=0.0 1.0  ! partially use corners of detectors; 1.41421=full use" >> XDS.INP_new
	
		mv XDS.INP_new XDS.INP
		
		echo "MAXIMUM_NUMBER_OF_PROCESSORS=$NUMBER_OF_CORES" >> XDS.INP
		echo "MAXIMUM_NUMBER_OF_JOBS=$NUMBER_OF_JOBS_CLUSTER" >> XDS.INP
		
		# TODO remove existing parameters
		if ( -e ../../.xdsparams) then
			cat ../../.xdsparams >> XDS.INP
		endif
		
		#if ($#argv == 18) then
			#set localcell = `echo $unitcell | sed 's/,/ /g'`
			#echo "UNIT_CELL_CONSTANTS= $localcell" >> XDS.INP
			#echo "SPACE_GROUP_NUMBER= $spacegroup" >> XDS.INP
		#endif
		
		
		if ( $numberofdatasets > 0) then
			 if ( -e ../../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL) then
			 	set sg = `grep SPACE_GROUP_NUMBER ../../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL | awk '{print $2}'`
			 	set localunitcell = `grep UNIT_CELL_CONSTANTS ../../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL | awk '{print $2" "$3" "$4" "$5" "$6" "$7}'`
			 
			 	egrep -v 'SPACE_GROUP_NUMBER|UNIT_CELL_CONSTANTS|REFERENCE_DATA_SET' XDS.INP > XDS.INP_new
			 
			 	echo "SPACE_GROUP_NUMBER= $sg" >> XDS.INP_new
			 	echo "UNIT_CELL_CONSTANTS= $localunitcell" >> XDS.INP_new
			 	echo "REFERENCE_DATA_SET= ../../$firstDataset/XDS_REINTEGRATE/XDS_ASCII.HKL" >> XDS.INP_new
			 
			 	echo "JOB=  XYCORR INIT COLSPOT IDXREF DEFPIX XPLAN INTEGRATE CORRECT" >> XDS.INP_new
				echo "UNIT_CELL_A-AXIS= $aaxis" >> XDS.INP_new
				echo "UNIT_CELL_B-AXIS= $baxis" >> XDS.INP_new
				echo "UNIT_CELL_C-AXIS= $caxis" >> XDS.INP_new
							
				mv -f XDS.INP_new XDS.INP
			endif
		endif
		
		#RUN XDS
		$scriptDir/procXDS.csh ${siteFirstFrame}
		
		if (-e XDS_ASCII.HKL) then
		
			set xdsfilepath = ../../$dir/XDS/XDS_ASCII.HKL
			set xdspathforscale = ../XDS/XDS_ASCII.HKL
		
			set resol = `python $scriptDir/get_resolution_CORRECT.py`
			cd ..
			# run XDS refine with INTEGRATE CORRECT after copying GXPARM.XDS to XPARM.XDS
			if ($#argv < 18) then
			
				mkdir XDS_REINTEGRATE
				cd  XDS_REINTEGRATE
				cp ../XDS/XDS.INP .
				# get resolution from CORRECT.LP : using fast-dp module
			
				#mv CORRECT.LP CORRECT.LP.JOB=ALL
		
				# reintegrate http://strucbio.biologie.uni-konstanz.de/xdswiki/index.php/Optimisation
				cp ../XDS/GXPARM.XDS XPARM.XDS
				#mv CORRECT.LP CORRECT.LP.old
				egrep -v 'JOB=|REIDX' XDS.INP > XDS.INP.new
				echo "! JOB=XYCORR INIT COLSPOT IDXREF DEFPIX INTEGRATE CORRECT" > XDS.INP
				echo "JOB=XYCORR INIT DEFPIX INTEGRATE CORRECT" >> XDS.INP
				echo NUMBER_OF_PROFILE_GRID_POINTS_ALONG_ALPHA/BETA=13 >> XDS.INP ! default is 9
				echo NUMBER_OF_PROFILE_GRID_POINTS_ALONG_GAMMA=13      >> XDS.INP ! default is 9
				cat XDS.INP.new >> XDS.INP
				rm -rf XDS.INP.new
		
				if ($resol != "None") then
					if ( $resol != "" ) then
						echo "INCLUDE_RESOLUTION_RANGE=30.0 $resol" >> XDS.INP
					endif
				endif
			
				#RUN XDS
				/mnt/software/px/XDS/xds_par >& GMCAproc.LOG
			
				if (-e XDS_ASCII.HKL) then
					set xdsfilepath = ../../$dir/XDS_REINTEGRATE/XDS_ASCII.HKL
					set xdspathforscale = ../XDS_REINTEGRATE/XDS_ASCII.HKL
				endif
			
				#write the logfile
				echo `pwd` > ../../subdirfile
			endif
			
			cd ..
		endif
		
		#scale individual data sets
		$scriptDir/scale.csh  $scale $prefix $#argv $xdspathforscale
	endif
	
	# write XSCALE input files
	if ( "$xdsfilepath" != "" && $#argv < 18 ) then
		echo "XDSIN $xdsfilepath" >> ../${prefix}_COMBINED/POINTLESS_AIMLESS/pointless.sh
		echo "INPUT_FILE= $xdsfilepath" >> ../${prefix}_COMBINED/XSCALE/XSCALE.INP	
	endif
	
        if ($#argv < 18) then
	 	cd ..
        endif
	@ numberofdatasets ++
end

#run XSCALE
if ( $numberofdatasets > 1) then
	cd ${prefix}_COMBINED/XSCALE
	/mnt/software/px/XDS/xscale &
else
	rm -rf XSCALE.INP
endif

# Run pointeless & aimless
if ( $numberofdatasets > 1 && $noofenergies < 2) then
	cd ../POINTLESS_AIMLESS/
	cp -p ../XSCALE/XDS_ASCII.HKL XDS_ASCII.HKL
	set gmcalog = `cat ../../subdirfile`
	if ( -e $gmcalog/GMCAproc.LOG ) then
		cp -p $gmcalog/GMCAproc.LOG .
		
	endif
	# Merge multiple datasets using POINTLESS
	echo "EOF" >> pointless.sh
	chmod +x pointless.sh
	./pointless.sh  > ${prefix}_pointless.log
	chmod +x pointless.sh
	
	echo
	echo "Multiple datasets"
	echo "Scaling and merging data with $scale"
	echo "$scale hklin ${prefix}_xds.mtz hklout ${prefix}_$scale.mtz \\" > $scale.sh
	echo "<<EOF> ${prefix}_$scale.log" >> $scale.sh
	echo "anomalous on" >> $scale.sh
	echo "END" >> $scale.sh
	echo "EOF" >> $scale.sh
	chmod +x $scale.sh
	
	source $scale.sh
	#write the logfile
	echo `pwd` > ../../subdirfile
	
	echo "$scale finished. See ${prefix}_all.log"
else
	cd $dir
	if ( $isFastDP == 0 ) then
		cp -pf fast_dp.mtz ${prefix}_$scale.mtz
		cp -pf $scale.log ${prefix}_$scale.log
	else
		cp -pf ${prefix}_$scale.mtz ${prefix}_$scale.mtz
		cp -pf ${prefix}_$scale.log ${prefix}_$scale.log
	endif	
endif

if ( -e ${prefix}_$scale.mtz ) then
	echo "Calculating structure factors with CTRUNCATE"
	echo "/mnt/software/px/CCP4/ccp4/bin/ctruncate -mtzin ${prefix}_$scale.mtz -mtzout ${prefix}_ctruncate.mtz -colano '/*/*/[I(+),SIGI(+),I(-),SIGI(-)]' <<EOF> ${prefix}_ctruncate.log" > ctruncate.sh
	echo "END" >> ctruncate.sh
	echo "EOF" >> ctruncate.sh
	chmod +x ctruncate.sh
	
	source ctruncate.sh 

	echo "CTRUNCATE finished. See ${prefix}_ctruncate.log"
endif

#CLEANUP 
cd $workDir

echo `pwd`
if ( ! -e $workDir/$tempLogDir ) then
	mkdir $workDir/$tempLogDir
endif

if ( $numberofdatasets == 1 ) then
	rm -rf ${prefix}_COMBINED
endif

mv -f $workDir/imagesets $workDir/subdirfile $workDir/$tempLogDir/
mv -f $workDir/*.o* $workDir/*.e*  $workDir/$tempLogDir/

echo `date`

