#!/opt/anaconda/bin/python

import json
import sys
import redis
import socket
import argparse
import os


# Sets default values in Redis for JBluice/PyBluIce
# Resets existing values if "redis_defaults.py flushall" is run
# -MH

def parse_args():
    global args
    parser = argparse.ArgumentParser()
    parser.add_argument('--sim', action='store_true')
    args = parser.parse_known_args()[0]


def get_redis_host():
    if args.sim:
        return '127.0.0.1'
    host = socket.gethostname()
    if host[0:2]!='bl':
        print("Unrecognized hostname format: "+host)
        sys.exit(-1)
    beamline_index = int(host[2:3])-1
    return ['10.20.103.71','10.20.103.138','10.20.103.197'][beamline_index]


def get_redis_port():
    return 8003 if args.sim else 6379


parse_args()

# load defaults
defaults_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), './redis_defaults.json')
f = open(defaults_path)
defaults = json.loads(f.read())

# connect redis
redis_host = get_redis_host()
print('Connecting to '+redis_host)
rs_str = redis.Redis(host=redis_host, port=get_redis_port(), charset="utf-8", decode_responses=True)

# delete all existing keys if requested
if len(sys.argv) > 1 and sys.argv[1] == 'flushall':
    print('Flushing all existing data')
    rs_str.flushall()

key_set_count = 0
for toplevel_key_group in defaults:

    # add ranges to key groups (e.g. for runs)
    keys = []
    if '#' in toplevel_key_group:
        (group,range_info) = toplevel_key_group.split('#')
        if '-' in range_info:
            (range_start,range_end) = range_info.split('-')
            for r in range(int(range_start),int(range_end)):
                keys.append('{}#{}'.format(group, str(r)))
        else:
            item_list = range_info.split(',')
            for item in item_list:
                keys.append('{}#{}'.format(group, str(item)))
    else:
        keys.append(toplevel_key_group)

    # set defaults for all keys and hashes
    for top_level_key in keys:
        if type(defaults[toplevel_key_group])==str:
            #print('{} = "{}"'.format(top_level_key, defaults[toplevel_key_group]))
            rs_str.set(top_level_key, defaults[toplevel_key_group])
            key_set_count += 1
        elif type(defaults[toplevel_key_group])==dict:
            for hash_key in defaults[toplevel_key_group]:
                #print('{}${} = "{}"'.format(top_level_key, hash_key, defaults[toplevel_key_group][hash_key]))
                rs_str.hset(top_level_key, hash_key, defaults[toplevel_key_group][hash_key])
                key_set_count += 1
        else:
            #print('Skipping empty toplevel key '+toplevel_key_group)
            pass

print('{} default values set.'.format(key_set_count))
