#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Nov  1 10:29:43 2016

@author: qxu
"""
from __future__ import absolute_import
import subprocess
from collections import namedtuple
import logging

from . import utils

logger = logging.getLogger(__name__)


class Spacegroup:
    SPG_TABLE1 = {
        'p1': 1,
        'p2': 3,
        'p21': 4,
        'c2': 5,
        'p222': 16,
        'p2221': 17,
        'p2122': 17,
        'p2212': 17,
        'p21212': 18,
        'p21221': 18,
        'p22121': 18,
        'p212121': 19,
        'c2221': 20,
        'c222': 21,
        'f222': 22,
        'i222': 23,
        'i212121': 24,
        'p4': 75,
        'p41': 76,
        'p42': 77,
        'p43': 78,
        'i4': 79,
        'i41': 80,
        'p422': 89,
        'p4212': 90,
        'p4122': 91,
        'p41212': 92,
        'p4222': 93,
        'p42212': 94,
        'p4322': 95,
        'p43212': 96,
        'i422': 97,
        'i4122': 98,
        'p3': 143,
        'p31': 144,
        'p32': 145,
        'h3': 146,
        'r3': 146,
        'p312': 149,
        'p321': 150,
        'p3112': 151,
        'p3121': 152,
        'p3212': 153,
        'p3221': 154,
        'h32': 155,
        'r32': 155,
        'p6': 168,
        'p61': 169,
        'p65': 170,
        'p62': 171,
        'p64': 172,
        'p63': 173,
        'p622': 177,
        'p6122': 178,
        'p6522': 179,
        'p6222': 180,
        'p6422': 181,
        'p6322': 182,
        'p23': 195,
        'f23': 196,
        'i23': 197,
        'p213': 198,
        'i213': 199,
        'p432': 207,
        'p4232': 208,
        'f432': 209,
        'f4132': 210,
        'i432': 211,
        'p4332': 212,
        'p4132': 213,
        'i4132': 214
    }
    
    SPG_TABLE2 = { # hash of has to translate integration space to real space group
        1: ['p1'],  # p1
        3: ['p21', 'p2'],  # p2 and p21
        5: ['c2'],  # c2
        16: ['p212121', 'p2221', 'p2212', 'p2122', 'p21212', 'p21221', 'p22121', 'p222'],  # p222,p2221 p21212 p212121
        21: ['c2221', 'c222'],  # c2221 c222
        22: ['f222'],  # f222
        23: ['i222', 'i212121'],  # i222 and i212121
        75: ['p41', 'p42', 'p43', 'p4'],  # p41-p43
        89: ['p43212', 'p41212', 'p422', 'p4212', 'p4122', 'p4322', 'p4222', 'p42212'],  # p422 etc
        79: ['i4', 'i41'],  # i4 and i41
        97: ['i422', 'i4122'],  # i422 and i4122
        143: ['p3', 'p31', 'p32'],  # p3,p31,p32
        149: ['p312', 'p3112', 'p3212'],  # p312 p3112 p3212
        150: ['p321', 'p3121', 'p3221'],  # p321 p3121 p3221
        168: ['p6', 'p61', 'p62', 'p63', 'p64', 'p65'],  # p6 etc
        177: ['p622', 'p6122', 'p6222', 'p6322', 'p6422', 'p6522'],  # p622 etc
        146: ['h3'],  # r3
        155: ['h32'],  # r32
        195: ['p23', 'p213'],  # p23 and p213
        207: ['p432', 'p4132', 'p4232', 'p4332'],  # p432 etc
        196: ['f23'],  # f23
        209: ['f432', 'f4132'],  # f432 and f4132
        197: ['i23', 'i213'],  # i23/i213
        211: ['i432', 'i4132']  # i432 and i4132
    }

    LATTICE = {
        'aP': (1, 'Primitive Triclinic', [1], [1]),
        'mP': (2, 'Primitive Monoclinic', [3], [3, 4]),
        'mC': (3, 'C Centered Monoclinic', [5], [5]),
        'mI': (3, 'C Centered Monoclinic', [5], [5]),
        'oP': (4, 'Primitive Orthorhombic', [16], [16, 17, 18, 19]),
        'oC': (5, 'C Centered Orthorhombic', [21], [21, 20]),
        'oF': (6, 'F Centered Orthorhombic', [22], [22]),
        'oI': (7, 'I Centered Orthorhombic', [23], [23, 24]),
        'tP': (8, 'Primitive Tetragonal', [89, 75],
               [75, 76, 77, 78, 89, 90, 91, 92, 93, 94, 95, 96]),
        'tI': (9, 'I Centered Tetragonal', [97, 79], [79, 80, 97, 98]),
        'hP': (10, 'Primitive Hexagonal', [177, 168, 150, 149, 143],
               [143, 144, 145, 149, 150, 151, 152, 153, 154,
                168, 169, 170, 171, 172, 173, 177, 178, 179, 180, 181, 182]),
        'hR': (11, 'Primitive Rhombohedral', [155, 146], [146, 155]),
        'cP': (12, 'Primitive Cubic', [207, 195],
               [195, 198, 207, 208, 212, 213]),
        'cF': (13, 'F Centered Cubic', [209, 196], [196, 209, 210]),
        'cI': (14, 'I Centered Cubic', [211, 197], [197, 199, 211, 214])
    }

    HM = {
        'p1': 'P 1',
        'p2': 'P 1 2 1',
        'p21': 'P 1 21 1',
        'c2': 'C 1 2 1',
        'p222': 'P 2 2 2',
        'p2221': 'P 2 2 21',
        'p2122': 'P 21 2 2',
        'p2212': 'P 2 21 2',
        'p21212': 'P 21 21 2',
        'p21221': 'P 21 2 21',
        'p22121': 'P 2 21 21',
        'p212121': 'P 21 21 21',
        'c2221': 'C 2 2 21',
        'c222': 'C 2 2 2',
        'f222': 'F 2 2 2',
        'i222': 'I 2 2 2',
        'i212121': 'I 21 21 21',
        'p4': 'P 4',
        'p41': 'P 41',
        'p42': 'P 42',
        'p43': 'P 43',
        'i4': 'I 4',
        'i41': 'I 41',
        'p422': 'P 4 2 2',
        'p4212': 'P 4 21 2',
        'p4122': 'P 41 2 2',
        'p41212': 'P 41 21 2',
        'p4222': 'P 42 2 2',
        'p42212': 'P 42 21 2',
        'p4322': 'P 43 2 2',
        'p43212': 'P 43 21 2',
        'i422': 'I 4 2 2',
        'i4122': 'I 41 2 2',
        'p3': 'P 3',
        'p31': 'P 31',
        'p32': 'P 32',
        'h3': 'H 3',
        'r3': 'R 3',
        'p312': 'P 3 1 2',
        'p321': 'P 3 2 1',
        'p3112': 'P 31 1 2',
        'p3121': 'P 31 2 1',
        'p3212': 'P 32 1 2',
        'p3221': 'P 32 2 1',
        'h32': 'H 3 2',
        'r32': 'R 3 2',
        'p6': 'P 6',
        'p61': 'P 61',
        'p65': 'P 65',
        'p62': 'P 62',
        'p64': 'P 64',
        'p63': 'P 63',
        'p622': 'P 6 2 2',
        'p6122': 'P 61 2 2',
        'p6522': 'P 65 2 2',
        'p6222': 'P 62 2 2',
        'p6422': 'P 64 2 2',
        'p6322': 'P 63 2 2',
        'p23': 'P 2 3',
        'f23': 'F 2 3',
        'i23': 'I 2 3',
        'p213': 'P 21 3',
        'i213': 'I 21 3',
        'p432': 'P 4 3 2',
        'p4232': 'P 42 3 2',
        'f432': 'F 4 3 2',
        'f4132': 'F 41 3 2',
        'i432': 'I 4 3 2',
        'p4332': 'P 43 3 2',
        'p4132': 'P 41 3 2',
        'i4132': 'I 41 3 2'
    }

    def __init__(self):
        pass

    '''
    @staticmethod
    def get_point_groups_from_lattice(lattice, output_mode='symbol'):
        pgnums = Spacegroup.LATTICE.get(lattice)
        if pgnums:
            if output_mode == 'symbol':
                return [Spacegroup.get_symbol_from_spg_number(n) for n in pgnums[2]]
            else:
                return pgnums[2]
        return None
    '''

    @staticmethod
    def get_space_groups_from_lattice(lattice, sgtype='spacegroup', output_mode='symbol'):
        pgnums = Spacegroup.LATTICE.get(lattice)
        if pgnums:
            if sgtype == 'spacegroup':
                nums = pgnums[3][::-1]  # space group
            else:
                nums = pgnums[2]  # point group

            if output_mode == 'symbol':
                return [Spacegroup.get_symbol_from_spg_number(n) for n in nums]
            else:
                return nums
        return None

    @staticmethod
    def get_spg_number_from_symbol(symbol):
        if isinstance(symbol, int):
            return symbol

        symbol = symbol.lower().replace('(','').replace(')','').replace(' ', '')
        try:
            number = Spacegroup.SPG_TABLE1[symbol]
            return number
        except KeyError:
            logger.error("unknown space group symbol: {}.".format(symbol))
            return None
            
    @staticmethod
    def get_symbol_from_spg_number(number):
        """NB for p2221 and p21212 return only first one"""
        if number == 17:
            return 'p2221'
        elif number == 18:
            return 'p21212'
            
        for k, v in Spacegroup.SPG_TABLE1.items():
            if v == number:
                return k
        else:
            logger.error("unknown space group number: {}.".format(number))
            return None

    @staticmethod
    def get_laue_number_from_symbol(symbol):
        symbol = symbol.lower().replace("(","").replace(")","")
        for k, v in Spacegroup.SPG_TABLE2.items():
            if symbol in v:
                return k
        else:
            logger.error("unknown space group symbol: {}.".format(symbol))
            return None
     
    @staticmethod
    def get_spacegroups_from_laue_number(laue_number):
        try:
            return Spacegroup.SPG_TABLE2[laue_number]
        except KeyError:
            logger.error("invalid laue number: {}.".format(laue_number))
            return None

    @staticmethod
    def get_spacegroup_from_number(number):
        if isinstance(number, str):
            number = int(number)

        for k, v in Spacegroup.SPG_TABLE1.items():
            if v == number:
                return k
        else:
            logger.error("no spg symbol found for {}".format(number))
            return None

    @staticmethod
    def generate_table_from_sginfo():
        sginfo = '/mnt/share/qxu/bluice/tools/scripts/quickProcess/quickProcess/bin/sginfo_1_01_linux_redhat8.exe'
        spg_dict = {}
        for k, v in Spacegroup.SPG_TABLE2.items():
            for spg in v:
                arg = spg
                if spg == 'h3':
                    arg = 'r3'
                elif spg == 'h32':
                    arg = 'r32'
                out = subprocess.check_output([sginfo, arg], shell=False).decode("utf-8")
                number = utils.getNumbers(out.split('\n')[1].split()[2])[0]
                spg_dict[spg] = number
        return spg_dict

    @staticmethod
    def cell_enforcer(spg, cell):
        """make sure the cell parameter comply to the lattice"""
        if isinstance(spg, int):
            spg = Spacegroup.get_symbol_from_spg_number(spg)
        
        spg = Spacegroup.get_laue_number_from_symbol(spg)
        
        a, b, c, al, be, ga = cell
        # monoclinic
        if spg in [3, 5]:
            al = ga = 90.0
        
        # orthorhombic
        if spg in [16, 21, 22, 23]:
            al = be = ga = 90.0
        
        # tetragonal
        if spg in [75, 89, 79, 97]:
            a = b = (a+b) / 2.0
            al = be = ga = 90.0
        
        # trigonal or r3/r32 in h setting
        if spg in [143, 149, 150, 168, 177, 146, 155]:
            a = b = (a+b) / 2.0
            al = be = 90.0
            ga = 120.0

        # cubic
        if spg in [195, 207, 196, 209, 197, 211]:
            a = b = c = (a+b+c) / 3.0
            al = be = ga = 90.0

        return round(a, 2), round(b, 2), round(c, 2), al, be, ga

    @staticmethod
    def choose_spacegroup(spg1, spg2):
        """ given two space groups, choose the one with lower lattice/symm """
        lattice1 = Spacegroup.get_lattice_from_spacegroup(spg1)
        lattice2 = Spacegroup.get_lattice_from_spacegroup(spg2)
        if lattice1 == lattice2:
            if isinstance(spg1, str):
                spgn1 = Spacegroup.get_spg_number_from_symbol(spg1)
            else:
                spgn1 = spg1

            if isinstance(spg2, str):
                spgn2 = Spacegroup.get_spg_number_from_symbol(spg2)
            else:
                spgn2 = spg2
                
            return spg1 if spgn1 <= spgn2 else spg2
        else:
            nlattice1 = Spacegroup.LATTICE[lattice1][0]
            nlattice2 = Spacegroup.LATTICE[lattice2][0]
       
            return spg1 if nlattice1 <= nlattice2 else spg2

    @staticmethod
    def get_lattice_info(lattice_abbrev):
        lattice = namedtuple('LATTICE',
                             'lattice_abbrev, lattice_no, lattice_name, possible_laue_group_numbers, possible_space_group_numbers')
        try:
            values = Spacegroup.LATTICE[lattice_abbrev]
            logger.debug("= value = ", len(values))
            return lattice(lattice_abbrev, *values)
        except KeyError:
            logger.error("invalid laue lattice abbreviation: {}.".format(lattice_abbrev))
            return None

    @staticmethod
    def get_lattice_from_spacegroup(spg):
        if isinstance(spg, str):
            spg = Spacegroup.get_spg_number_from_symbol(spg)

        for k, v in Spacegroup.LATTICE.items():
            if spg in v[-1]:
                return k
        else:
            logger.error("no suitable lattice found, given spg = {}.".format(spg))
            return None

    @staticmethod
    def get_supported_space_groups():
        return sorted(Spacegroup.SPG_TABLE1.keys())

    @staticmethod
    def get_lowest_space_group_symbol_from_lattice(lattice):
        if lattice in Spacegroup.LATTICE:
            spg_num = min(Spacegroup.LATTICE[lattice][2])
            return Spacegroup.get_symbol_from_spg_number(spg_num)

    @staticmethod
    def similar_cell(cell1, cell2, epsilon=0.015):
        if len(cell1) != 6 or len(cell2) != 6:
            return False

        cell1 = [float(e) for e in cell1]
        cell2 = [float(e) for e in cell2]
        return all(abs(cell1[i] - cell2[i])/min(cell1[i], cell2[i]) <= epsilon
                   for i in range(6))

    @staticmethod
    def get_reindex_vector(spg):
        spg = spg.replace(" ", "").lower()
        reindex = 'h,k,l'
        new_spg = spg

        if spg in ['p22121', 'p2122']:
            reindex = 'k,l,h'
            new_spg = 'p2221'
            if spg == 'p22121':
                new_spg = 'p21212'
        elif spg in ['p21221', 'p2212']:
            reindex = 'l,h,k'
            new_spg = 'p2221'
            if spg == 'p21221':
                new_spg = 'p21212'

        return new_spg, reindex

    @staticmethod
    def is_cell_equal(cell1, cell2, error=0.1):
        if isinstance(cell1, str) and isinstance(cell2, str):
            try:
                cell1 = map(float, cell1.split())
                cell2 = map(float, cell2.split())
            except:
                return False

        try:
            if all(abs(x1-x2)/min(x1, x2) <= error for x1,x2 in zip(cell1, cell2)):
                return True
        except Exception:
            return False

        return False

    @staticmethod
    def is_pointgroup_equal(spg1, spg2):
        if isinstance(spg1, int):
            spg1 = Spacegroup.get_spacegroup_from_number(spg1)

        if isinstance(spg2, int):
            spg2 = Spacegroup.get_spacegroup_from_number(spg2)

        return Spacegroup.get_laue_number_from_symbol(spg1) == Spacegroup.get_laue_number_from_symbol(spg2)

    @staticmethod
    def cal_average_cell(cells):
        # expect each cell as a string of 3 or 6 field, separated by spaces
        import numpy as np
        cells = [cell for cell in cells if cell is not None]
        cells = [list(map(float, cell.split())) for cell in cells if len(cell.split()) in [3, 6]]
        cells = np.array(cells)
        avg_cell = np.mean(cells, axis=0).tolist()
        return avg_cell

if __name__ == '__main__':
    s = Spacegroup()
    print(s.get_laue_number_from_symbol('P2(1)2(1)2(1)'))
    print(s.get_spacegroups_from_laue_number(16))
    print(s.generate_table_from_sginfo())
    print(s.get_spg_number_from_symbol('p212121'))
    print(s.get_symbol_from_spg_number(17))
    print(s.cell_enforcer('i23', (12, 11, 1, 90, 90.3, 90.5)))
    print(s.get_lattice_info('aP').lattice_name)
    print(s.get_lattice_from_spacegroup('p212121'))
    print(s.get_lattice_from_spacegroup(1))
    print(s.get_supported_space_groups())
    print(s.get_laue_number_from_symbol('i213'))
    print(s.get_lattice_from_spacegroup('i23'))
    print(s.choose_spacegroup('p222','p212121'))
    print(s.choose_spacegroup('p422','p212121'))
    print(s.choose_spacegroup('p4','p3'))
    print(s.choose_spacegroup('p321','p6'))
    print(s.choose_spacegroup('p1', 'h3'))
    print(s.choose_spacegroup(1, 5))
    print(s.similar_cell([1, 2,3.1, 4,5,6], [1, 2, 3, 4,5, 6]))
    print(s.get_lowest_space_group_symbol_from_lattice('mI'))
    # print(s.get_point_groups_from_lattice('tI'))
    print(s.get_space_groups_from_lattice('tI', sgtype='pointgroup'))
    print(s.get_space_groups_from_lattice('tI', output_mode='number'))
    print("pg p222 = p212121? {}".format(s.is_pointgroup_equal('p222', 'p212121')))
    print("pg 17 = 20? {}".format(s.is_pointgroup_equal(17, 20)))

    cells = ["1 2 3 90 90 90", "1 2 30 90 90 90", "2 3 5 90 90 90"]
    print(s.cal_average_cell(cells))