import os
from operator import itemgetter, attrgetter
from math import ceil
from collections import Counter

import numpy as np
from scipy import stats


class real_time_peak_detection():
    """
    https://stackoverflow.com/questions/22583391/peak-signal-detection-in-realtime-timeseries-data/22640362#22640362
    """

    def __init__(self, array, lag, threshold, influence):
        self.y = list(array)
        self.length = len(self.y)
        self.lag = lag
        self.threshold = threshold
        self.influence = influence
        self.signals = [0] * len(self.y)
        self.filteredY = np.array(self.y).tolist()
        self.avgFilter = [0] * len(self.y)
        self.stdFilter = [0] * len(self.y)
        self.avgFilter[self.lag - 1] = np.mean(self.y[0:self.lag]).tolist()
        self.stdFilter[self.lag - 1] = np.std(self.y[0:self.lag]).tolist()

    def thresholding_algo(self, new_value):
        if new_value:
            self.y.append(new_value)
        i = len(self.y) - 1
        self.length = len(self.y)
        if i < self.lag:
            return 0
        elif i == self.lag:
            self.signals = [0] * len(self.y)
            self.filteredY = np.array(self.y).tolist()
            self.avgFilter = [0] * len(self.y)
            self.stdFilter = [0] * len(self.y)
            self.avgFilter[self.lag - 1] = np.mean(self.y[0:self.lag]).tolist()
            self.stdFilter[self.lag - 1] = np.std(self.y[0:self.lag]).tolist()
            return 0

        self.signals += [0]
        self.filteredY += [0]
        self.avgFilter += [0]
        self.stdFilter += [0]

        if abs(self.y[i] - self.avgFilter[i - 1]) > self.threshold * self.stdFilter[i - 1]:
            if self.y[i] > self.avgFilter[i - 1]:
                self.signals[i] = 1
            else:
                self.signals[i] = -1

            self.filteredY[i] = self.influence * self.y[i] + (1 - self.influence) * self.filteredY[i - 1]
            self.avgFilter[i] = np.mean(self.filteredY[(i - self.lag):i])
            self.stdFilter[i] = np.std(self.filteredY[(i - self.lag):i])
        else:
            self.signals[i] = 0
            self.filteredY[i] = self.y[i]
            self.avgFilter[i] = np.mean(self.filteredY[(i - self.lag):i])
            self.stdFilter[i] = np.std(self.filteredY[(i - self.lag):i])

        return self.signals[i]


def trend_detector(list_of_index, array_of_data, order=1):
    """
    :param list_of_index:
    :param array_of_data:
    :param order:
    :return: slope, negative decrease
    """
    result = np.polyfit(list_of_index, list(array_of_data), order)
    slope = result[-2]
    return float(slope)


def find_runs(x):
    """Find runs of consecutive items in an array.
    https://gist.github.com/alimanfoo/c5977e87111abe8127453b21204c1065
    """

    # ensure array
    x = np.asanyarray(x)
    if x.ndim != 1:
        raise ValueError('only 1D array supported')
    n = x.shape[0]

    # handle empty array
    if n == 0:
        return np.array([]), np.array([]), np.array([])

    else:
        # find run starts
        loc_run_start = np.empty(n, dtype=bool)
        loc_run_start[0] = True
        np.not_equal(x[:-1], x[1:], out=loc_run_start[1:])
        run_starts = np.nonzero(loc_run_start)[0]

        # find run values
        run_values = x[loc_run_start]

        # find run lengths
        run_lengths = np.diff(np.append(run_starts, n))

        return run_values, run_starts, run_lengths


def cal_exclude_ranges(l_spots, start_threshold=0, max_threshold=5, min_reject=0, max_reject=50, min_frames=3):
    """
    find a list of ranges, such that values in the range are all <= threshold, and range width >= min_frames
    :param max_reject: min percentage to reject
    :param min_reject: max percentage to reject
    :param l_spots: list of tuples (frame #, # of spots)
    :param threshold: threshold used for cutoff
    :param min_frames: minimum # of frames to be considered as an excluded range
    :return: list of tuple (start, end) defining range to be excluded
    
    # equivalent but less efficient implementation
    ranges = []
    count = 0
    in_range = False
    for i, data in enumerate(l_spots):
        index = data[0]
        value = data[1]
        if value <= threshold:
            in_range = True
            if count == 0:
                start = index

            count += 1
        else:
            if count >= min_frames and in_range:
                ranges.append((start, l_spots[i - 1][0]))

            count = 0
            in_range = False

    # end of the sequence
    if in_range and count >= min_frames:
        ranges.append((start, l_spots[i - 1][0]))

    print(l_spots, "\n", ranges)
    return ranges

    def range_cal(lis, n_first=50, window_size=10):
    na = np.array(lis)
    # si = na.argsort()[-n_first:]
    # sa = na[si]
    # print(na, "sa=", sa)
    mean = np.mean(na[:n_first])
    std = np.std(na[:n_first])

    x = (na - mean) / std
    print(" mean, std=", mean, std, "x = ", x)

    zscore = stats.zscore(na)
    print("na = ", na, " zscore=", zscore)

    # moving average
    w = np.ones(window_size, 'd')
    y = np.convolve(w / w.sum(), na, mode='valid')
    print(y, len(na), len(y))

    print([0 if x <= 1 else 1 for x in na])

    # values.sort()
    # print(values.tolist())
    # bottom10 = values[:10]
    # top10 = values[-10:]
    # std_top10 = np.std(top10)
    # mean_top10 = np.mean(top10)
    # std_bottom10 = np.std(bottom10)
    # mean_bottom10 = np.mean(bottom10)
    #
    # print(f"stats Mb={mean_bottom10} Mt={mean_top10} Std_b={std_bottom10} Std_t={std_top10} median = {median}")
    # SnR = (mean_top10 - mean_bottom10) / (0.5 * (std_bottom10 + std_top10))
    # print(SnR)
    #
    # print("===", values <= threshold)

    # min_index = frame_npixels[0][0]
    # max_index = frame_npixels[-1][0]
    # l_indices = range(min_index, max_index + 1)
    # print(l_spots)
    # print(trend_detector(l_indices, l_spots))
    # print(trend_detector(l_indices[:20], l_spots[:20]))

    # range_cal(l_spots)
    
    """
    values = np.array([v for i, v in l_spots])
    frame_indices = [i for i, v in l_spots]

    median = np.median(values)

    ranges = []
    threshold = start_threshold
    while threshold <= median and threshold <= max_threshold:
        mask_array = values <= threshold
        run_values, run_starts, run_lengths = find_runs(mask_array)

        rejected = 0
        for i, v in enumerate(run_values):
            if v == 1 and run_lengths[i] >= min_frames:
                start_frame = frame_indices[run_starts[i]]
                end_frame = frame_indices[run_starts[i] + run_lengths[i] - 1]
                rejected += run_lengths[i]
                ranges.append([start_frame, end_frame])

        r_percentage = 100.0 * rejected / mask_array.shape[0]
        if min_reject <= r_percentage <= max_reject:
            print(f"will reject {r_percentage} percentage frames: {ranges} using threshold {threshold}")
            return ranges

        threshold += 1.0
        ranges = []
        # print(threshold, run_values, run_starts, run_lengths)

    print("found no bad frames to reject")
    return ranges


def merge_ranges(ranges, min_gap=2):
    """merge two consecutive ranges if the gap <= min_gap, e.g. [[1,2],[4,5]] -->[1,5]"""
    if not ranges or len(ranges) == 1:
        return ranges

    new_ranges = []
    start = ranges[0][0]
    last = ranges[-1][1]
    for i, r in enumerate(ranges[:-1]):
        if int(ranges[i + 1][0]) - int(ranges[i][1]) > min_gap:
            end = ranges[i][1]
            new_ranges.append([start, end])
            start = ranges[i + 1][0]

    # last range if not merge
    new_ranges.append([start, last])

    return new_ranges


class SpotXDS:
    def __init__(self, wdir='.', spot_file='SPOT.XDS', colspot_lp='COLSPOT.LP'):
        self.wdir = wdir
        self.spot_file = spot_file
        self.colspot_lp = colspot_lp

    def get_exclude_ranges(self, method=1, start_threshold=100, min_frames=5, output='XDS'):
        frame_npixels = self.parse_colspot_lp()
        frame_nspots = self.parse_spot_xds_file()

        l_spots = []
        for i, x in frame_npixels:
            if i in frame_nspots:
                l_spots.append((i, float(frame_nspots[i])))
            else:
                l_spots.append((i, 0.0))

        range1 = cal_exclude_ranges(l_spots, start_threshold=start_threshold, min_frames=min_frames)
        range2 = cal_exclude_ranges(frame_npixels, start_threshold=start_threshold, min_frames=min_frames)
        # print(frame_npixels, "\n", l_spots)
        print(f"ranges = {range1} (method 1) or : {range2}  (method 2).")
        ranges = range1 if method == 1 else range2
        if output == 'XDS':
            return ";".join([f"EXCLUDE_DATA_RANGE= {r[0]} {r[1]}" for r in ranges])
        else:
            return ranges

    def parse_spot_xds_file(self):
        """
        :return: {frame #: # of strong spots}, NB not ordered
        """
        frame_intensity = []
        spot_file = os.path.join(self.wdir, self.spot_file)
        with open(spot_file, "r") as spot_xds:
            lines = spot_xds.readlines()
            for line in lines:
                x, y, z, i = line.split()[:4]
                frame_intensity.append([ceil(float(z)), float(i)])

        frame_nspots = Counter(i for i, x in frame_intensity)

        # for i in sorted(frame_nspots.keys()):
        #    print(i, frame_nspots[i])

        return frame_nspots

    def parse_colspot_lp(self):
        """
        :return: a list of (frame #, # of strong pixels)
        """
        frame_npixels = []
        lp_file = os.path.join(self.wdir, self.colspot_lp)
        with open(lp_file, "r") as spot_lp:
            lines = spot_lp.readlines()
            in_section = False
            for line in lines:
                if "FRAME #" in line:
                    in_section = True
                    continue

                if "-----" in line or len(line) < 10:
                    in_section = False
                    continue

                if in_section:
                    fields = line.split()
                    if len(fields) != 4:
                        print("parsing error: ", line)
                        continue

                    z, bg, n_strong_pixel, _ = fields
                    frame_npixels.append([int(z), int(n_strong_pixel)])

        # print(sorted(frame_npixels, key=itemgetter(0)))
        return frame_npixels


if __name__ == '__main__':
    x = SpotXDS(wdir='/mnt/beegfs/qxu/junk/gmcaproc-50p/s001_196802_1_00001_00210/XDS-p1')
    # x = SpotXDS(wdir='/mnt/beegfs/qxu/MUV_grease/xds')
    x = SpotXDS(wdir='/mnt/beegfs/qxu/23BM_2017_08_08/4z2x-p21212/test/gmcaproc/s003_196802_1_00001_00420/XDS-p1')
    x.get_exclude_ranges()
