from __future__ import absolute_import

import os
import logging

from ..common.spacegroup import Spacegroup
from .fileopener import FileOpener
from ..common import utils

logger = logging.getLogger(__name__)


class XdsAsciiParser(FileOpener):
    def __init__(self, wdir=".", filename='XDS_ASCII.HKL'):
        super(XdsAsciiParser, self).__init__(wdir, filename)

    def get_spg_cell(self, spg_type='number'):
        # parse XDS_ASCII.HKL to get space group (in symbol--str) and cell (list)
        # return two strings if there are results, otherwise, (None, None)
        spg_num = cell = None
        if os.path.isfile(self.filename):
            with open(self.filename) as hkl:
                for line in hkl:
                    if line.startswith("!SPACE_GROUP_NUMBER="):
                        spg_num = utils.getNumbers(line)[0]
                    if line.startswith("!UNIT_CELL_CONSTANTS="):
                        cell = utils.getNumbers(line)
                    if line.startswith("!END_OF_HEADER"):
                        break

        if spg_type == 'number':
            return spg_num, cell
        else:
            spg_symbol = Spacegroup.get_spacegroup_from_number(spg_num)
            return spg_symbol, cell