from __future__ import absolute_import

import logging
from .fileopener import FileOpener
from ..common.utils import get_outliers
from .table import Table

logger = logging.getLogger(__name__)


# parser for xdscc12 or xscalecc12
class XDSCC12Parser(FileOpener):
    def __init__(self, wdir=".", filename='xdscc12.sh.out'):
        super(XDSCC12Parser, self).__init__(wdir, filename)

    # extract CC1/2 related to "^a", "^b" etc
    def get_rows(self, key='a'):
        lines = self.parser.get_lines_startswith(key)
        if key == 'a' or key == 'd':
            lines2skip = 3
        else:
            lines2skip = 1
        return lines[lines2skip:]

    def get_outliers_using_a(self, method=1, negative_overall_CC=True):
        lines = self.get_rows("a")

        outliers = list()
        cc_all = list()
        for iset, line in enumerate(lines):
            fields = line.split()
            if len(fields) == 10:
                deltacc_this = float(fields[-5])
                deltacc_overall = float(fields[-1])
                cc_all.append(deltacc_overall)

            if deltacc_overall <= 0:
                outliers.append((iset+1, deltacc_overall))

        if outliers and method == 1:
            # reject iset that have negative overall cc value
            outliers = sorted(outliers, key=lambda x: x[1])
            if negative_overall_CC:
                outliers = [x[0] for x in outliers]
        else:
            outliers = [i+1 for i in get_outliers(cc_all, m=3, left_tail=True, indices=True)]
        return outliers


if __name__ == '__main__':
    p = XDSCC12Parser('/mnt/beegfs/qxu/23BM_2017_08_08/rlefkowitz/process/gmcaproc/scale-1', filename='xscalecc12.sh.out')
    print(p.get_outliers_using_a(method=2))
