
from __future__ import absolute_import
import logging, os
import numpy as np
import pandas as pd

logger = logging.getLogger(__name__)


class xlsReader:
    """read screening spreadsheet xls, figure out where is the model, sequence, for structure solution steps"""

    def __init__(self, xlsFile):
        self.xlsFile = xlsFile
        self.df = None
        if os.path.exists(xlsFile):
            self.df = pd.read_excel(self.xlsFile).replace(np.nan, '', regex=True)
        else:
            logger.error("{} does not exists.".format(self.xlsFile))

    def get_row(self, mounted_crystal):
        row = self.df[self.df.Port == mounted_crystal]
        if row.empty:
            logger.warning("no row matching requested crystal {} was found.".format(mounted_crystal))
        return row

    def get_element(self, mounted_crystal, column_name):
        row = self.get_row(mounted_crystal)

        if row.empty:
            return None

        column = row.get(column_name)

        # print(dir(column), type(column))
        if column is None:
            return None

        element = "".join(map(str, column.values))

        if element:
            return element

        # if values not found at this row, look else where protein name is the same
        #print("===", row.get("Protein").values)
        protein = "".join(map(str, row.get("Protein").values))
        if not protein or protein.replace(" ", "") is None:
            logger.info("protein is not defined")
            return None

        same_protein_rows = self.df[(self.df['Protein'] == protein) & (self.df[column_name] != '')]

        if not same_protein_rows.empty:
            element = same_protein_rows.iloc[0][column_name]

        return element

    def get_model_path(self, mounted_crystal):
        return self.get_element(mounted_crystal, 'ModelPath')

    def get_nmol(self, mounted_crystal):
        return self.get_element(mounted_crystal, 'NMol')

    def get_sequence_path(self, mounted_crystal):
        return self.get_element(mounted_crystal, 'SequencePath')

    def get_space_group(self, mounted_crystal):
        return self.get_element(mounted_crystal, 'Spacegroup')

    def get_metal(self, mounted_crystal):
        return self.get_element(mounted_crystal, 'Metal')

    def get_reference_dataset(self, mounted_crystal):
        return self.get_element(mounted_crystal, 'ReferenceDataSet')


if __name__ == '__main__':
    xls = xlsReader("/mnt/beegfs/qxu/ALS_template_20191124.xls")
    row = xls.get_row('A1')

    x = xls.get_element('A9', 'ModelPath')
    if x:
        print("model=", x)
    else:
        print("not found")

    print("not exist=", xls.get_element('A9', 'ModelPatha'))
    print("seq=", xls.get_sequence_path('A9'))
    print("mode=", xls.get_model_path('A10'))
    print("mode=", xls.get_element('A1', "Protein"))
    print("mode=", xls.get_element('A1', "Proteinx"))
    print("model=", xls.get_element('A3', "ModelPath"))
    print("model=", xls.get_element('A4', "ModelPath"))
    print("model=", xls.get_element('R1', "Spacegroup"))
