from __future__ import absolute_import

import logging

import numpy as np

from .fileopener import FileOpener
from ..common.utils import getNumbers, getElbowPoint

logger = logging.getLogger(__name__)


# parse XSCALE.1.INP generate by xscale_isocluster and identify outliers
class XSCALE1INP(FileOpener):
    """
    ...
    INPUT_FILE=../s020_test_1_00001_00900/XDS-p1/XDS_ASCII.HKL
    !new, old ISET=      1     20 length=CC*,angle,cluster=     0.919     1.4      1
    !INCLUDE_RESOLUTION_RANGE=00 00
    WEIGHT=     1.000
    ...
    """

    def __init__(self, wdir=".", filename='XSCALE.1.INP'):
        super(XSCALE1INP, self).__init__(wdir, filename)

    def get_outliers_using_angle(self, threshold=1.5, method='elbow'):
        blocks = self.parser.get_blocks_between('INPUT_FILE=', '!INCLUDE_RESOLUTION_RANGE=', iMatch=-1)

        i_angles = list()
        for block in blocks:
            for line in block:
                if 'length=CC*,angle,cluster= ' in line:
                    flds = getNumbers(line)
                    i_new = (flds[0])
                    i_old = int(flds[1])
                    ccstar = float(flds[2])
                    angle = float(flds[3])
                    i_cluster = int(flds[4])
                    if i_cluster > 0:
                        i_angles.append((i_old, angle))

        #print("i, angles=", i_angles)
        outliers = detect_outliers(i_angles, threshold=threshold, method=method)

        return outliers[::-1]


def detect_outliers(data, threshold=1.5, method='elbow'):
    score = np.array([x[1] for x in data])
    score_mean = np.mean(score)
    score_std = np.std(score)
    elbow_index = getElbowPoint(score)
    #elbow2_index = elbow2(score)
    #print("elbow_index=", elbow_index)

    outliers = list()

    # Z-score method
    for x in data:
        if 'z-score' == method.lower():
            z_score = (x[1] - score_mean) / score_std
            if z_score > threshold:
                outliers.append(x[0])
        elif 'elbow' == method.lower():
            if x[1] > score[elbow_index] and x[1] > threshold*score_mean:
                outliers.append(x[0])
        # elif 'elbow2' == method.lower():
        #     if elbow2_index >= len(score)//2 and x[1] > score[elbow2_index]:
        #         outliers.append(x[0])

    #print("a==", score_mean, score_std, elbow_index, elbow2_index, score[elbow_index], score[elbow2_index], outliers)
    return outliers


def elbow2(arr):
    diff = list()
    for i in range(1, len(arr)):
        l_mean = np.mean(arr[:i])
        r_mean = np.mean(arr[i:])
        diff.append(r_mean-l_mean)

    return (np.argmax(np.array(diff)))

if __name__ == '__main__':
    p = XSCALE1INP('/mnt/beegfs/qxu/bluice/tools/scripts/quickProcess/quickProcess/tests/', filename='XSCALE.1.INP')
    print(p.get_outliers_using_angle(method='elbow', threshold=1))
    p = XSCALE1INP('/mnt/beegfs/qxu/bluice/tools/scripts/quickProcess/quickProcess/tests/', filename='XSCALE.2.INP')
    print(p.get_outliers_using_angle(method='eblow', threshold=1))
    p = XSCALE1INP('/mnt/beegfs/qxu/bluice/tools/scripts/quickProcess/quickProcess/tests/', filename='XSCALE.3.INP')
    print(p.get_outliers_using_angle(method='elbow', threshold=1))