#! /bin/awk -f
#
# Create XDS-style input parameters starting with mosflm-style parameters
#								-James Holton
#
# also read in an A matrix (first three lines of a MOSFLM matrix file)
#
# use this script by just concatenating a mosflm script and matrix file
# on its standard input
#
#

BEGIN{}

a_star_x=="" && sprintf("%.8f",$1)==$1{
    # each column is a reciprocal-lattice vector
    print "# reading: ", $0
    a_star_x = $1; b_star_x = $2; c_star_x = $3; getline;
    print "# reading: ", $0
    a_star_y = $1; b_star_y = $2; c_star_y = $3; getline;
    print "# reading: ", $0
    a_star_z = $1; b_star_z = $2; c_star_z = $3; getline;

    if(sprintf("%.3f",$1)==$1) getline
    getline
    getline
    getline

    # use unit cell to figure out wavelength
    if(NF!=6) next
    a=$1;b=$2;c=$3;
    a_star = sqrt(a_star_x^2+a_star_y^2+a_star_z^2);
    b_star = sqrt(b_star_x^2+b_star_y^2+b_star_z^2);
    c_star = sqrt(c_star_x^2+c_star_y^2+c_star_z^2);

b_star_cross_c_star_x = b_star_y*c_star_z - b_star_z*c_star_y;
b_star_cross_c_star_y = b_star_z*c_star_x - b_star_x*c_star_z;
b_star_cross_c_star_z = b_star_x*c_star_y - b_star_y*c_star_x;

# reciprocal unit cell volume
V_star = a_star_x*b_star_cross_c_star_x + a_star_y*b_star_cross_c_star_y + a_star_z*b_star_cross_c_star_z;

a_x = b_star_cross_c_star_x/V_star
a_y = b_star_cross_c_star_y/V_star
a_z = b_star_cross_c_star_z/V_star

    matrix_a = sqrt(a_x^2+a_y^2+a_z^2)

    if(matrix_a>0) wave = lambda = a/matrix_a;
}

/^PROCESS/ && phi0==""{phi0=$6; osc=$8}

/^WAVE/{wave=lambda=$2}
/^BEAM/{beamx=$2;beamy=$3}
/^DIST/{dist=$NF}
/^TWOTHETA/{twotheta=$NF}
/^BEAM/{mos_beamy=$NF;mos_beamx=$(NF-1)}

/^SYMM/{SG=$2}
/^CELL/{a=$2;b=$3;c=$4;alpha=$5;beta=$6;gamma=$7}

/^LIMITS/{
   detsize=substr($0,index($0,"YMAX ")+5)+0
}

/QX= /{fpix=substr($0,index($0,"QX= ")+3)+0}
/NX= /{fpixels=substr($0,index($0,"NX= ")+3)+0}
/QY= /{spix=substr($0,index($0,"QY= ")+3)+0}
/NY= /{spixels=substr($0,index($0,"NY= ")+3)+0}

/ORGX= /{orgx=substr($0,index($0,"ORGX= ")+5)+0}
/ORGY= /{orgy=substr($0,index($0,"ORGY= ")+5)+0}

/^X-RAY_WAVELENGTH=/{
   test=substr($0,index($0,"=")+2);
   if(test+0>0)wave=test
}
/^STARTING_FRAME=/{
   test=substr($0,index($0,"=")+2);
   if(int(test+0)==test) firstnum=test
}
/^STARTING_ANGLE=/{
   test=substr($0,index($0,"=")+2);
   phi0=test
}
/OSCILLATION_RANGE=/{
   test=substr($0,index($0,"=")+2);
   if(test+0!=0) osc=test
}
/DETECTOR_DISTANCE=/{
   test=substr($0,index($0,"=")+2);
   if(test+0!=0) dist=test
}

/^SPACE_GROUP_NUMBER=/{
   test=substr($0,index($0,"=")+1);
   if(test+0>0)SGnum=test
}
/^UNIT_CELL_CONSTANTS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==6){
        a=w[1];b=w[2];c=w[3];alpha=w[4];beta=w[5];gamma=w[6];
    }
}

/^INCIDENT_BEAM_DIRECTION=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        beam_x=w[1];beam_y=w[2];beam_z=w[3];
    }
}
/^DIRECTION_OF_DETECTOR_X-AXIS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        detX_x=w[1];detX_y=w[2];detX_z=w[3];
    }
}
/^DIRECTION_OF_DETECTOR_Y-AXIS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        detY_x=w[1];detY_y=w[2];detY_z=w[3];
    }
}
/^ROTATION_AXIS=/{
    test=substr($0,index($0,"=")+1);
    n=split(test,w);
    if(n==3){
        spindle_x=w[1];spindle_y=w[2];spindle_z=w[3];
    }
}


/^UNIT_CELL_A-AXIS=/{a_x=$2;a_y=$3;a_z=$4}
/^UNIT_CELL_B-AXIS=/{b_x=$2;b_y=$3;b_z=$4}
/^UNIT_CELL_C-AXIS=/{c_x=$2;c_y=$3;c_z=$4}





END {
# volume of the unit cell is:
# V = a dot b cross c
# reciprocal cell volume:
# V_star = a_star.(b_star X c_star)

# scale factor to convert radians to degrees
RTD = 45.0/atan2(1,1);
DTR = 1/RTD;

# scale mosflm matrix by wavelength
if(! wave) wave =1;
a_star_x /= wave; b_star_x /= wave; c_star_x /= wave;
a_star_y /= wave; b_star_y /= wave; c_star_y /= wave;
a_star_z /= wave; b_star_z /= wave; c_star_z /= wave;


# calculate reciprocal cell vector lengths
a_star = sqrt(a_star_x^2+a_star_y^2+a_star_z^2);
b_star = sqrt(b_star_x^2+b_star_y^2+b_star_z^2);
c_star = sqrt(c_star_x^2+c_star_y^2+c_star_z^2);

# three cross products
a_star_cross_b_star_x = a_star_y*b_star_z - a_star_z*b_star_y;
a_star_cross_b_star_y = a_star_z*b_star_x - a_star_x*b_star_z;
a_star_cross_b_star_z = a_star_x*b_star_y - a_star_y*b_star_x;

b_star_cross_c_star_x = b_star_y*c_star_z - b_star_z*c_star_y;
b_star_cross_c_star_y = b_star_z*c_star_x - b_star_x*c_star_z;
b_star_cross_c_star_z = b_star_x*c_star_y - b_star_y*c_star_x;

c_star_cross_a_star_x = c_star_y*a_star_z - c_star_z*a_star_y;
c_star_cross_a_star_y = c_star_z*a_star_x - c_star_x*a_star_z;
c_star_cross_a_star_z = c_star_x*a_star_y - c_star_y*a_star_x;

# three dot products
a_star_dot_b_star = a_star_x*b_star_x + a_star_y*b_star_y + a_star_z*b_star_z;
a_star_dot_c_star = a_star_x*c_star_x + a_star_y*c_star_y + a_star_z*c_star_z;
b_star_dot_c_star = b_star_x*c_star_x + b_star_y*c_star_y + b_star_z*c_star_z;

# reciprocal unit cell volume
V_star = a_star_x*b_star_cross_c_star_x + a_star_y*b_star_cross_c_star_y + a_star_z*b_star_cross_c_star_z;
if(V_star<=0) V_star=1e-99

# direct-space cell volume
V = 1/V_star;

# reciprocal lattice vector "a_star" is defined as perpendicular to both b and c, and must also preserve volume
# converse is true for direct-space lattice: a is perpendicular to both b_star and c_star
# a = ( b_star cross c_star ) / V_star

a_x = b_star_cross_c_star_x/V_star
a_y = b_star_cross_c_star_y/V_star
a_z = b_star_cross_c_star_z/V_star

b_x = c_star_cross_a_star_x/V_star
b_y = c_star_cross_a_star_y/V_star
b_z = c_star_cross_a_star_z/V_star

c_x = a_star_cross_b_star_x/V_star
c_y = a_star_cross_b_star_y/V_star
c_z = a_star_cross_b_star_z/V_star

# calculate direct-space unit cell lengths
a = sqrt(a_x^2+a_y^2+a_z^2);
b = sqrt(b_x^2+b_y^2+b_z^2);
c = sqrt(c_x^2+c_y^2+c_z^2);

if(a<=0) a=1e-99;
if(b<=0) b=1e-99;
if(c<=0) c=1e-99;

# three dot products
a_dot_b = a_x*b_x + a_y*b_y + a_z*b_z;
a_dot_c = a_x*c_x + a_y*c_y + a_z*c_z;
b_dot_c = b_x*c_x + b_y*c_y + b_z*c_z;

# for fun, calculate the cell angles too
sinalpha = a_star*V/b/c;
sinbeta  = b_star*V/a/c;
singamma = c_star*V/a/b;
cosalpha = b_dot_c/b/c;
cosbeta  = a_dot_c/a/c;
cosgamma = a_dot_b/a/b;

if(sinalpha>1) sinalpha=1;
if(sinbeta>1) sinbeta=1;
if(singamma>1) singamma=1;
alpha = RTD*atan2(sinalpha,cosalpha);
beta  = RTD*atan2(sinbeta ,cosbeta);
gamma = RTD*atan2(singamma,cosgamma);

if(a_star<=0) a_star=1e-99;
if(b_star<=0) b_star=1e-99;
if(c_star<=0) c_star=1e-99;

# reciprocal cell angles
sinalpha_star = a*V_star/b_star/c_star;
sinbeta_star  = b*V_star/a_star/c_star;
singamma_star = c*V_star/a_star/b_star;
cosalpha_star = b_star_dot_c_star/b_star/c_star;
cosbeta_star  = a_star_dot_c_star/a_star/c_star;
cosgamma_star = a_star_dot_b_star/a_star/b_star;

alpha_star = RTD*atan2(sinalpha_star,cosalpha_star);
beta_star  = RTD*atan2(sinbeta_star ,cosbeta_star);
gamma_star = RTD*atan2(singamma_star,cosgamma_star);


#if(firstnum=="") firstnum = 1;
if(lastnum=="" && firstnum!="") lastnum = firstnum;
#if(pix=="") pix = 0.1024;
if(fpixels=="" && fpix != "" && detsize != ""){
    fpixels = spixels = 2**(1+int(log(detsize/pix)/log(2)));
}
#if(pixels=="") pixels = 3096;
#if(dist=="") dist = 250;
if(beam_z=="") beam_z = 1/wave;
if(detX_x=="") detX_x=1;
if(detY_y=="") { detY_x=0; detY_y=cos(twotheta*DTR) ; detY_z=sin(twotheta*DTR) }
if(detZ_y=="") { detZ_x=0; detZ_y=-sin(twotheta*DTR) ; detZ_z=cos(twotheta*DTR) }
if(spindle_x=="") spindle_x=1;

if(mos_beamx != "" && pix!="" && dist!=""){
    orgx=mos_beamy/pix;
    orgy=(mos_beamx+dist*sin(twotheta*DTR)/cos(twotheta*DTR))/pix
}
#if(orgx=="") orgx=pixels/2
#if(orgy=="") orgy=pixels/2

if(SGnum == "" && SG+0>0) SGnum = SG
if(SGnum == "") SGnum = SGnum_of(SG);

if(firstnum!="") print "STARTING_FRAME=", firstnum;
if(phi0!="") print "STARTING_ANGLE=", phi0;
if(osc!="") print "OSCILLATION_RANGE=", osc;
if(orgx!="")print "ORGX=", orgx,"ORGY=",orgy;
if(fpixels!="") print "NX=", fpixels,"NY=",spixels;
if(fpix!="") print "QX=",fpix,"QY=",spix;

if(lambda!="") print "X-RAY_WAVELENGTH=",lambda;
if(beam_z!="")print "INCIDENT_BEAM_DIRECTION=",beam_x+0,beam_y+0,beam_z+0;
if(dist!="") print "DETECTOR_DISTANCE=",dist; 
if(detX_x!="")print "DIRECTION_OF_DETECTOR_X-AXIS=", detX_x+0,detX_y+0,detX_z+0;
if(detY_x!="")print "DIRECTION_OF_DETECTOR_Y-AXIS=", detY_x+0,detY_y+0,detY_z+0;
if(spindle_x!="") print "ROTATION_AXIS=",spindle_x+0,spindle_y+0,spindle_z+0;

if(a>1e-99) {
    print "UNIT_CELL_CONSTANTS=", a,b,c,alpha,beta,gamma;
    print "SPACE_GROUP_NUMBER=", SGnum+0;

    # print out the real-space matrix the way XDS expects it
    print "UNIT_CELL_A-AXIS=",a_z,-a_y,a_x;
    print "UNIT_CELL_B-AXIS=",b_z,-b_y,b_x;
    print "UNIT_CELL_C-AXIS=",c_z,-c_y,c_x;
#    print "UNIT_CELL_A-AXIS=",a_z,a_x,a_y;
#    print "UNIT_CELL_B-AXIS=",b_z,b_x,b_y;
#    print "UNIT_CELL_C-AXIS=",c_z,c_x,c_y;
}

}



function SGnum_of(SG) {
    if(SG=="P1") SGnum = 1
    if(SG=="P-1") SGnum = 2
    if(SG=="P2") SGnum = 3
    if(SG=="P21") SGnum = 4
    if(SG=="C2") SGnum = 5
    if(SG=="Pm") SGnum = 6
    if(SG=="Pc") SGnum = 7
    if(SG=="Cm") SGnum = 8
    if(SG=="Cc") SGnum = 9
    if(SG=="P2/m") SGnum = 10
    if(SG=="P21/m") SGnum = 11
    if(SG=="C2/m") SGnum = 12
    if(SG=="P2/c") SGnum = 13
    if(SG=="P21/c") SGnum = 14
    if(SG=="C2/c") SGnum = 15
    if(SG=="P222") SGnum = 16
    if(SG=="P2221") SGnum = 17
    if(SG=="P21212") SGnum = 18
    if(SG=="P212121") SGnum = 19
    if(SG=="C2221") SGnum = 20
    if(SG=="C222") SGnum = 21
    if(SG=="F222") SGnum = 22
    if(SG=="I222") SGnum = 23
    if(SG=="I212121") SGnum = 24
    if(SG=="Pmm2") SGnum = 25
    if(SG=="Pmc21") SGnum = 26
    if(SG=="Pcc2") SGnum = 27
    if(SG=="Pma2") SGnum = 28
    if(SG=="Pca21") SGnum = 29
    if(SG=="Pnc2") SGnum = 30
    if(SG=="Pmn21") SGnum = 31
    if(SG=="Pba2") SGnum = 32
    if(SG=="Pna21") SGnum = 33
    if(SG=="Pnn2") SGnum = 34
    if(SG=="Cmm2") SGnum = 35
    if(SG=="Cmc21") SGnum = 36
    if(SG=="Ccc2") SGnum = 37
    if(SG=="Amm2") SGnum = 38
    if(SG=="Abm2") SGnum = 39
    if(SG=="Ama2") SGnum = 40
    if(SG=="Aba2") SGnum = 41
    if(SG=="Fmm2") SGnum = 42
    if(SG=="Fdd2") SGnum = 43
    if(SG=="Imm2") SGnum = 44
    if(SG=="Iba2") SGnum = 45
    if(SG=="Ima2") SGnum = 46
    if(SG=="Pmmm") SGnum = 47
    if(SG=="Pnnn") SGnum = 48
    if(SG=="Pccm") SGnum = 49
    if(SG=="Pban") SGnum = 50
    if(SG=="Pmma") SGnum = 51
    if(SG=="Pnna") SGnum = 52
    if(SG=="Pmna") SGnum = 53
    if(SG=="Pcca") SGnum = 54
    if(SG=="Pbam") SGnum = 55
    if(SG=="Pccn") SGnum = 56
    if(SG=="Pbcm") SGnum = 57
    if(SG=="Pnnm") SGnum = 58
    if(SG=="Pmmn") SGnum = 59
    if(SG=="Pbcn") SGnum = 60
    if(SG=="Pbca") SGnum = 61
    if(SG=="Pnma") SGnum = 62
    if(SG=="Cmcm") SGnum = 63
    if(SG=="Cmca") SGnum = 64
    if(SG=="Cmmm") SGnum = 65
    if(SG=="Cccm") SGnum = 66
    if(SG=="Cmma") SGnum = 67
    if(SG=="Ccca") SGnum = 68
    if(SG=="Fmmm") SGnum = 69
    if(SG=="Fddd") SGnum = 70
    if(SG=="Immm") SGnum = 71
    if(SG=="Ibam") SGnum = 72
    if(SG=="Ibca") SGnum = 73
    if(SG=="Imma") SGnum = 74
    if(SG=="P4") SGnum = 75
    if(SG=="P41") SGnum = 76
    if(SG=="P42") SGnum = 77
    if(SG=="P43") SGnum = 78
    if(SG=="I4") SGnum = 79
    if(SG=="I41") SGnum = 80
    if(SG=="P-4") SGnum = 81
    if(SG=="I-4") SGnum = 82
    if(SG=="P4/m") SGnum = 83
    if(SG=="P42/m") SGnum = 84
    if(SG=="P4/n") SGnum = 85
    if(SG=="P42/n") SGnum = 86
    if(SG=="I4/m") SGnum = 87
    if(SG=="I41/a") SGnum = 88
    if(SG=="P422") SGnum = 89
    if(SG=="P4212") SGnum = 90
    if(SG=="P4122") SGnum = 91
    if(SG=="P41212") SGnum = 92
    if(SG=="P4222") SGnum = 93
    if(SG=="P42212") SGnum = 94
    if(SG=="P4322") SGnum = 95
    if(SG=="P43212") SGnum = 96
    if(SG=="I422") SGnum = 97
    if(SG=="I4122") SGnum = 98
    if(SG=="P4mm") SGnum = 99
    if(SG=="P4bm") SGnum = 100
    if(SG=="P42cm") SGnum = 101
    if(SG=="P42nm") SGnum = 102
    if(SG=="P4cc") SGnum = 103
    if(SG=="P4nc") SGnum = 104
    if(SG=="P42mc") SGnum = 105
    if(SG=="P42bc") SGnum = 106
    if(SG=="I4mm") SGnum = 107
    if(SG=="I4cm") SGnum = 108
    if(SG=="I41md") SGnum = 109
    if(SG=="I41cd") SGnum = 110
    if(SG=="P-42m") SGnum = 111
    if(SG=="P-42c") SGnum = 112
    if(SG=="P-421m") SGnum = 113
    if(SG=="P-421c") SGnum = 114
    if(SG=="P-4m2") SGnum = 115
    if(SG=="P-4c2") SGnum = 116
    if(SG=="P-4b2") SGnum = 117
    if(SG=="P-4n2") SGnum = 118
    if(SG=="I-4m2") SGnum = 119
    if(SG=="I-4c2") SGnum = 120
    if(SG=="I-42m") SGnum = 121
    if(SG=="I-42d") SGnum = 122
    if(SG=="P4/mmm") SGnum = 123
    if(SG=="P4/mcc") SGnum = 124
    if(SG=="P4/nbm") SGnum = 125
    if(SG=="P4/nnc") SGnum = 126
    if(SG=="P4/mbm") SGnum = 127
    if(SG=="P4/mnc") SGnum = 128
    if(SG=="P4/nmm") SGnum = 129
    if(SG=="P4/ncc") SGnum = 130
    if(SG=="P42/mmc") SGnum = 131
    if(SG=="P42/mcm") SGnum = 132
    if(SG=="P42/nbc") SGnum = 133
    if(SG=="P42/nnm") SGnum = 134
    if(SG=="P42/mbc") SGnum = 135
    if(SG=="P42/mnm") SGnum = 136
    if(SG=="P42/nmc") SGnum = 137
    if(SG=="P42/ncm") SGnum = 138
    if(SG=="I4/mmm") SGnum = 139
    if(SG=="I4/mcm") SGnum = 140
    if(SG=="I41/amd") SGnum = 141
    if(SG=="I41/acd") SGnum = 142
    if(SG=="P3") SGnum = 143
    if(SG=="P31") SGnum = 144
    if(SG=="P32") SGnum = 145
    if(SG=="H3") SGnum = 146
    if(SG=="R3") SGnum = 146
    if(SG=="P-3") SGnum = 147
    if(SG=="H-3") SGnum = 148
    if(SG=="R-3") SGnum = 148
    if(SG=="P312") SGnum = 149
    if(SG=="P321") SGnum = 150
    if(SG=="P3112") SGnum = 151
    if(SG=="P3121") SGnum = 152
    if(SG=="P3212") SGnum = 153
    if(SG=="P3221") SGnum = 154
    if(SG=="H32") SGnum = 155
    if(SG=="R32") SGnum = 155
    if(SG=="P3m1") SGnum = 156
    if(SG=="P31m") SGnum = 157
    if(SG=="P3c1") SGnum = 158
    if(SG=="P31c") SGnum = 159
    if(SG=="H3m") SGnum = 160
    if(SG=="R3m") SGnum = 160
    if(SG=="H3c") SGnum = 161
    if(SG=="R3c") SGnum = 161
    if(SG=="P-31m") SGnum = 162
    if(SG=="P-31c") SGnum = 163
    if(SG=="P-3m1") SGnum = 164
    if(SG=="P-3c1") SGnum = 165
    if(SG=="H-3m") SGnum = 166
    if(SG=="R-3m") SGnum = 166
    if(SG=="H-3c") SGnum = 167
    if(SG=="R-3c") SGnum = 167
    if(SG=="P6") SGnum = 168
    if(SG=="P61") SGnum = 169
    if(SG=="P65") SGnum = 170
    if(SG=="P62") SGnum = 171
    if(SG=="P64") SGnum = 172
    if(SG=="P63") SGnum = 173
    if(SG=="P-6") SGnum = 174
    if(SG=="P6/m") SGnum = 175
    if(SG=="P63/m") SGnum = 176
    if(SG=="P622") SGnum = 177
    if(SG=="P6122") SGnum = 178
    if(SG=="P6522") SGnum = 179
    if(SG=="P6222") SGnum = 180
    if(SG=="P6422") SGnum = 181
    if(SG=="P6322") SGnum = 182
    if(SG=="P6mm") SGnum = 183
    if(SG=="P6cc") SGnum = 184
    if(SG=="P63cm") SGnum = 185
    if(SG=="P63mc") SGnum = 186
    if(SG=="P-6m2") SGnum = 187
    if(SG=="P-6c2") SGnum = 188
    if(SG=="P-62m") SGnum = 189
    if(SG=="P-62c") SGnum = 190
    if(SG=="P6/mmm") SGnum = 191
    if(SG=="P6/mcc") SGnum = 192
    if(SG=="P63/mcm") SGnum = 193
    if(SG=="P63/mmc") SGnum = 194
    if(SG=="P23") SGnum = 195
    if(SG=="F23") SGnum = 196
    if(SG=="I23") SGnum = 197
    if(SG=="P213") SGnum = 198
    if(SG=="I213") SGnum = 199
    if(SG=="Pm-3") SGnum = 200
    if(SG=="Pn-3") SGnum = 201
    if(SG=="Fm-3") SGnum = 202
    if(SG=="Fd-3") SGnum = 203
    if(SG=="Im-3") SGnum = 204
    if(SG=="Pa-3") SGnum = 205
    if(SG=="Ia-3") SGnum = 206
    if(SG=="P432") SGnum = 207
    if(SG=="P4232") SGnum = 208
    if(SG=="F432") SGnum = 209
    if(SG=="F4132") SGnum = 210
    if(SG=="I432") SGnum = 211
    if(SG=="P4332") SGnum = 212
    if(SG=="P4132") SGnum = 213
    if(SG=="I4132") SGnum = 214
    if(SG=="P-43m") SGnum = 215
    if(SG=="F-43m") SGnum = 216
    if(SG=="I-43m") SGnum = 217
    if(SG=="P-43n") SGnum = 218
    if(SG=="F-43c") SGnum = 219
    if(SG=="I-43d") SGnum = 220
    if(SG=="Pm-3m") SGnum = 221
    if(SG=="Pn-3n") SGnum = 222
    if(SG=="Pm-3n") SGnum = 223
    if(SG=="Pn-3m") SGnum = 224
    if(SG=="Fm-3m") SGnum = 225
    if(SG=="Fm-3c") SGnum = 226
    if(SG=="Fd-3m") SGnum = 227
    if(SG=="Fd-3c") SGnum = 228
    if(SG=="Im-3m") SGnum = 229
    if(SG=="Ia-3d") SGnum = 230
    if(SG=="P112") SGnum = 3
    if(SG=="P1121") SGnum = 4
    if(SG=="B2") SGnum = 5
    if(SG=="A2") SGnum = 5
    if(SG=="C21") SGnum = 5
    if(SG=="I2") SGnum = 5
    if(SG=="I21") SGnum = 5
    if(SG=="P11m") SGnum = 6
    if(SG=="P11b") SGnum = 7
    if(SG=="B11m") SGnum = 8
    if(SG=="B11b") SGnum = 9
    if(SG=="P112/m") SGnum = 10
    if(SG=="P1121/m") SGnum = 11
    if(SG=="B112/m") SGnum = 12
    if(SG=="P112/b") SGnum = 13
    if(SG=="P1121/b") SGnum = 14
    if(SG=="B112/b") SGnum = 15
    if(SG=="P2122") SGnum = 17
    if(SG=="P2212") SGnum = 17
    if(SG=="P21212a") SGnum = 18
    if(SG=="P21221") SGnum = 18
    if(SG=="P22121") SGnum = 18
    if(SG=="C2221a") SGnum = 20
    if(SG=="C222a") SGnum = 21
    if(SG=="F222a") SGnum = 22
    if(SG=="I222a") SGnum = 23
    if(SG=="Pmmn2") SGnum = 59
    if(SG=="P42212a") SGnum = 94
    if(SG=="I23a") SGnum = 197
    
    return SGnum
}




