#! /bin/awk -f

# convert XDS-style inputs into an XDS "XPARM.XDS" format
#								-James Holton 
#

BEGIN{if(! wave) wave=1;
      if(! pix)  pix=0.1024;
}

# potentially read in mosflm matrix
a_star_x=="" && sprintf("%.8g",$1)==$1{
    # each column is a reciprocal-lattice vector
    a_star_x = $1; b_star_x = $2; c_star_x = $3; getline;
    a_star_y = $1; b_star_y = $2; c_star_y = $3; getline;
    a_star_z = $1; b_star_z = $2; c_star_z = $3;
}


/^WAVE/{wave=$2}
/^BEAM/{beamx=$2;beamy=$3}
/^DIST/{dist=$NF}
/^TWOTHETA/{twotheta=$NF}
/^BEAM/{print;mos_beamy=$NF;mos_beamx=$(NF-1)}

/^SYMM/{SG=$2}
/^CELL/{a=$2;b=$3;c=$4;alpha=$5;beta=$6;gamma=$7}
/^LIMITS/{
   detsize=substr($0,index($0,"YMAX ")+5)+0
}


/QX= /{fpix=substr($0,index($0,"QX= ")+3)+0}
/NX= /{fpixels=substr($0,index($0,"NX= ")+3)+0}
/QY= /{spix=substr($0,index($0,"QY= ")+3)+0}
/NY= /{spixels=substr($0,index($0,"NY= ")+3)+0}

/ORGX= /{orgx=substr($0,index($0,"ORGX= ")+5)+0}
/ORGY= /{orgy=substr($0,index($0,"ORGY= ")+5)+0}


/^X-RAY_WAVELENGTH=/{
   test=substr($0,index($0,"=")+2);
   if(test+0>0)wave=test
}
/^STARTING_FRAME=/{
   test=substr($0,index($0,"=")+2);
   if(int(test+0)==test) firstnum=test
}
/^STARTING_ANGLE=/{
   test=substr($0,index($0,"=")+2);
   phi0=test
}
/OSCILLATION_RANGE=/{
   test=substr($0,index($0,"=")+2);
   if(test+0!=0) osc=test
}
/DETECTOR_DISTANCE=/{
   test=substr($0,index($0,"=")+2);
   if(test+0!=0) dist=test
}

/SPACE_GROUP_NUMBER=/{
   test=substr($0,index($0,"=")+2);
   if(test+0>0)SGnum=test
}
/UNIT_CELL_CONSTANTS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==6){
        a=w[1];b=w[2];c=w[3];alpha=w[4];beta=w[5];gamma=w[6];
    }
}


/INCIDENT_BEAM_DIRECTION=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        beam_x=w[1];beam_y=w[2];beam_z=w[3];
    }
}
/DIRECTION_OF_DETECTOR_X-AXIS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        detX_x=w[1];detX_y=w[2];detX_z=w[3];
    }
}
/DIRECTION_OF_DETECTOR_Y-AXIS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        detY_x=w[1];detY_y=w[2];detY_z=w[3];
    }
}
/ROTATION_AXIS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        spindle_x=w[1];spindle_y=w[2];spindle_z=w[3];
    }
}


/UNIT_CELL_A-AXIS=/{a_x=$2;a_y=$3;a_z=$4}
/UNIT_CELL_B-AXIS=/{b_x=$2;b_y=$3;b_z=$4}
/UNIT_CELL_C-AXIS=/{c_x=$2;c_y=$3;c_z=$4}



END {
# scale factor to convert radians to degrees
RTD = 45.0/atan2(1,1);
DTR = 1/RTD;



# calculate direct-space unit cell lengths
a = sqrt(a_x^2+a_y^2+a_z^2);
b = sqrt(b_x^2+b_y^2+b_z^2);
c = sqrt(c_x^2+c_y^2+c_z^2);


if(firstnum=="") firstnum = 1;
if(lastnum=="") lastnum = firstnum;
if(phi0=="") phi0 = 0;
if(osc=="") osc = 1;
if(pix=="") pix = 0.1024;
if(pixels=="") pixels = 3096;
if(dist=="") dist = 250;
if(beam_z=="") beam_z = 1/wave;
if(detX_x=="") detX_x=1;
if(detY_y=="") { detY_x=0; detY_y=cos(twotheta*DTR) ; detY_z=sin(twotheta*DTR) }
if(detZ_y=="") { detZ_x=0; detZ_y=-detY_z ; detZ_z=detY_y }
if(spindle_x=="") spindle_x=1;

if(mos_beamx != ""){
    orgx=mos_beamy/pix;
    orgy=(mos_beamx+dist*sin(twotheta*DTR)/cos(twotheta*DTR))/pix
}
if(orgx=="") orgx=pixels/2
if(orgy=="") orgy=pixels/2

if(version=="old"){
printf(" %5d %13.4f %9.4f %9.6f %9.6f %9.6f\n",firstnum,phi0,osc,spindle_x,spindle_y,spindle_z);
printf(" %14.6f %14.6f %14.6f %14.6f\n",wave,beam_x,beam_y,beam_z);
printf(" %9d %9d %11.6f %11.6f\n",pixels,pixels,pix,pix);
printf(" %14.6f %14.6f %14.6f\n",dist,orgx,orgy);
printf(" %14.6f %14.6f %14.6f\n",detX_x,detX_y,detX_z);
printf(" %14.6f %14.6f %14.6f\n",detY_x,detY_y,detY_z);
printf(" %14.6f %14.6f %14.6f\n",detZ_x,detZ_y,detZ_z);
printf(" %5d %11.4f %11.4f %11.4f %7.3f %7.3f %7.3f\n",SGnum,a,b,c,alpha,beta,gamma);
printf(" %14.6f %14.6f %14.6f\n",a_x,a_y,a_z);
printf(" %14.6f %14.6f %14.6f\n",b_x,b_y,b_z);
printf(" %14.6f %14.6f %14.6f\n",c_x,c_y,c_z);
}
else
{

print " XPARM.XDS    VERSION March 30, 2013"
printf(" %5d %13.4f %9.4f %9.6f %9.6f %9.6f\n",firstnum,phi0,osc,spindle_x,spindle_y,spindle_z);
printf(" %14.6f %14.6f %14.6f %14.6f\n",wave,beam_x,beam_y,beam_z);
printf(" %5d %11.4f %11.4f %11.4f %7.3f %7.3f %7.3f\n",SGnum,a,b,c,alpha,beta,gamma);
printf(" %14.6f %14.6f %14.6f\n",a_x,a_y,a_z);
printf(" %14.6f %14.6f %14.6f\n",b_x,b_y,b_z);
printf(" %14.6f %14.6f %14.6f\n",c_x,c_y,c_z);
printf(" %9d %9d %9d %11.6f %11.6f\n",1,fpixels,spixels,fpix,spix);
printf(" %14.6f %14.6f %14.6f\n",orgx,orgy,dist);
printf(" %14.6f %14.6f %14.6f\n",detX_x,detX_y,detX_z);
printf(" %14.6f %14.6f %14.6f\n",detY_x,detY_y,detY_z);
printf(" %14.6f %14.6f %14.6f\n",detZ_x,detZ_y,detZ_z);
printf(" %9d %9d %9d %9d %9d\n",1,1,fpixels,1,spixels);
printf(" %7.2f %7.2f %7.2f %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f\n",0,0,0,1,0,0,0,1,0);


}

}

