from __future__ import absolute_import
import argparse
import logging
import os
import sys

from quickProcess.tasks.scriptjob import ScriptJob
from quickProcess.common.utils import getNumbers

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger(__name__)


def get_args():
    """ command line options """
    parser = argparse.ArgumentParser(prog='')
    parser.add_argument('--data_dir', '-i',
                        required=True, help='parent directory for image files')
    parser.add_argument('--proc_dir', '-o',
                        required=True, help='parent directory for processing files')

    return parser.parse_args()


def walk(directory):
    if not os.path.isdir(directory):
        logger.info('{} does not exist'.format(directory))
        yield None

    for root, dirs, files in os.walk(directory):
        for dir in dirs:
            if 'cbf' == dir:
                yield os.path.join(root, dir)


def run_strategy(data_dir,
                 proc_dir='.',
                 sampleName='sampleName',
                 sampleNumber='sampleNumber',
                 pipeline='mosflm_strategy',
                 dry_run=False):
    wdir = proc_dir
    if not os.path.isdir(wdir):
        os.makedirs(wdir)

    job = ScriptJob(wdir, script_name='{}.sh'.format(pipeline))
    job.update_cmd("module load anaconda/python3\n"
                   "export PYTHONPATH=/home/qxu/work/quickProcess:$PYTHONPATH\n"
                   "python -m quickProcess.strategy"
                   " --data_dir={} --proc_dir={}"
                   " --pipeline={} "
                   " --nproc=1 --njobs=1 --log2db"
                   " --sampleName={} --sampleNumber={}"
                   " --native"
                   .format(data_dir, wdir, pipeline, sampleName, sampleNumber))
    if dry_run:
        job.write_script()
    else:
        job.run_sync()


def main():
    sys.path.append('/home/qxu/work/quickProcess')
    args = get_args()
    pipelines = ['xds_strategy', 'mosflm_strategy', 'dials_strategy', 'labelit_strategy']
    pipelines = ['labelit_strategy', 'mosflm_strategy']
    # pipelines = ['mosflm_strategy']
    count = 0
    maxcount = 1000
    for dir in walk(args.data_dir):
        print("processing: ", dir)
        subdir = dir.split(os.sep)[-2]
        sampleName = str(subdir)
        sampleNumber = getNumbers(sampleName)[-1]
        for pipeline in pipelines:
            proc_dir = os.path.join(args.proc_dir, subdir)
            run_strategy(dir,
                         proc_dir=proc_dir,
                         pipeline=pipeline,
                         sampleName=sampleName,
                         sampleNumber=sampleNumber, dry_run=False)
        count += 1
        if count >= maxcount:
            raise SystemExit(1)


if __name__ == '__main__':
    main()
