import socket
import sys, subprocess, time, os
from contextlib import closing
import psutil


class AdxvServer:
    def __init__(self, host='localhost', port=8100, adxv_path='/mnt/software/px/bin/adxv_nogl'):
        self.host = host
        self.port = port
        self.adxv_path = adxv_path
        self.cmd = "{} -socket {} -weak_data -small_spots -parallel".format(self.adxv_path, self.port)
        #self.cmd = "{} -socket {} -weak_data -small_spots -minicbf -nodisplay -parallel".format(self.adxv_path, self.port)

        if not self.is_running():
            self.run()
            time.sleep(1)
        else:
            print("adxv is already running, listening at port: {}".format(self.port))

    def run(self):
        logfile_handle = open(os.path.join(os.getenv("HOME"), "adxv.log"), "a")
        subprocess.Popen(self.cmd, shell=True, stdout=logfile_handle, stderr=logfile_handle)

    def is_running(self):
        for p in psutil.process_iter():
            if 'adxv' in p.name():
                if ('-socket' in p.cmdline()
                        and str(self.port) in p.cmdline()
                        and p.is_running()):
                    if p.username() == os.getenv('USER'):
                        return True
                    else:
                        print("Warning: the socket is openned by another user.")
                        return True
        return False

    '''
    # this call only works if starting from code, a second call confuses adxv
    def is_port_open(self):
        with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
            sock.settimeout(2)
            if sock.connect_ex((self.host, self.port)) == 0:
                print("Port is open")
                sock.close()
                return True
            else:
                print("Port is not open")
                return False

        #clientsocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        #clientsocket.settimeout(1)

        #port_status = False
        #try:
        #    result = clientsocket.connect_ex((self.host, self.port))
        #    if result == 0:
        #        port_status = True
        #except Exception as e:
        #    print(e)
        #finally:
        #    clientsocket.close()

        #return port_status
    '''


class AdxvSocket:
    """
    this code is copied from https://github.com/erikssod/adxv_class
    """

    def __init__(self, host='localhost', port=8100):

        self.clientsocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.clientsocket.connect((host, port))
            print('Connected to Host:Port - %s:%i' % (host, port))
        except Exception as e:
            print(e)

    def load_image(self, image_file):
        """
        Load an image file
        """
        payload = 'load_image %s\n' % (image_file)
        self.send(payload)

    def raise_window(self, window):
        """
        Raises a Window. <window> must be one of
        'Control', 'Image', 'Magnify', 'Line', or
        'Load'.
        """
        payload = 'raise_window %s\n' % (window)
        self.send(payload)

    def raise_image(self):
        """
        Raises image window; see raise_window for
        additional options but this seems like the
        most common one.
        """
        payload = 'raise_window Image\n'
        self.send(payload)

    def save_image(self, path_name_format):
        '''
        Save an image file (jpeg or tiff)
        '''
        payload = 'save_image %s\n' % (path_name_format)
        self.send(payload)

    def slab(self, N):
        """
        Display slab N
        """
        payload = 'slab %i\n' % (N)
        self.send(payload)

    def set_slab(self, N):
        """
        Same as slab, but don’t load the image
        """
        payload = 'set_slab %i\n' % (N)
        self.send(payload)

    def slabs(self, N):
        """
        Slab thickness to display
        """
        payload = 'slabs %i\n' % (N)
        self.send(payload)

    def set_slabs(self, N):
        """
        Same as slabs, but don’t load the image
        """
        payload = 'set_slabs %i\n' % (N)
        self.send(payload)

    def exit(self):
        """
        Exit Adxv
        """
        payload = 'exit\n'
        self.send(payload)

    def stride(self, N):
        """
        stride - sets Stride in the Load Window
        """
        payload = 'stride %i\n' % (N)
        self.send(payload)

    def increment_slabs(self):
        """
        increment_slabs - checks the +Slabs checkbox in the Load Window
        """
        payload = 'increment_slabs\n'
        self.send(payload)

    def increment_files(self):
        """
        increment_files - unchecks the +Slabs checkbox in the Load Window
        """
        payload = 'increment_files\n'
        self.send(payload)

    def contrast_min(self, N):
        """
        contrast_min - sets the min contrast value
        """
        payload = 'contrast_min %i\n' % (N)
        self.send(payload)

    def contrast_max(self, N):
        """
        contrast_max - sets the max contrast value
        """
        payload = 'contrast_max %i\n' % (N)
        self.send(payload)

    def close(self, window):
        if window in ['Control', 'Image', 'Magnify', 'Line', 'Load']:
            payload = 'lower_window %s\n' % (window)
            self.send(payload)

    def add_type(self, stype=0, color='red', radius=8):
        payload = 'define_type {} color {} radius {}\n'.format(stype, color, radius)
        self.send(payload)

    def rings(self, onoff):
        payload = 'rings {}\n'.format(onoff)
        self.send(payload)

    def remove_spots(self):
        spots = ['-100 -100 0', ]
        self.add_spots(spots)

    def add_spots(self, spots):
        nspots = len(spots)
        if not nspots:
            return

        nspots = len(spots)

        payload = 'load_spots {}\n'.format(nspots)
        self.send(payload)

        for spot in spots:
            payload = '{}\n'.format(spot)
            self.send(payload)

        payload = 'end_of_pack\n'
        self.send(payload)

    def send(self, payload):
        """
        Takes command, encodes it, and sends it down the socket.
        """
        try:
            #print("payload = %s" % (payload))
            self.clientsocket.sendall(payload.encode())
        except Exception as e:
            print(e)


if __name__ == '__main__':
    s = AdxvServer()


    print("server started.")

    adxv_client = AdxvSocket()
    startTime = time.time()
    for i in range(1000):
        adxv_client.load_image("/mnt/beegfs/qxu/23IDD-test/PC/run0020-tiny/PC_000100.cbf")
    elapsedTime = time.time() - startTime
    print("done, elapsed time ", elapsedTime, "sec")

    sys.exit(0)
    # adxv_client.load_image("/mnt/beegfs/qxu/23IDD-test/PC/run0020-smaller/PC_000104.cbf")
    # adxv_client.add_type()
    '''
      fs/px   ss/px (1/d)/nm^-1   Intensity  Panel
    1203.84 1084.50       1.15       74.91   0
    1220.05 1131.50       0.88       33.10   0
    1250.04 1139.50       0.83       39.54   0
    1253.33 1139.61       0.83      173.58   0
    1413.79 1179.72       1.11       63.89   0
    1415.22 1182.26       1.11       74.49   0
    1372.50 1321.73       0.73       70.61   0
    1139.40 1402.40       0.86       52.17   0
    1094.03 1458.50       1.25       35.31   0
    1239.50 1464.50       0.97       38.07   0
     871.50 2418.00       5.71       11.43   0
    '''

    spots = ["1203 1084 0",
             "1220 1131 0",
             "1250 1139 0",
             "1253 1139 0"]
    adxv_client.add_spots(spots)
    adxv_client.raise_image()
    adxv_client.close('Control')

    '''
    cmd = '/mnt/software/px/bin/adxv_nogl -padsmooth -small_spots -socket 8100'.split()
    #popen = subprocess.check_output(cmd, shell=True, stdout=subprocess.DEVNULL)
    popen = subprocess.Popen(cmd, shell=True, stdout=subprocess.DEVNULL)
    '''
    sys.exit(0)

    time.sleep(10)
    adxv_client = adxvSocket()





    adxv_client.load_image("/mnt/beegfs/qxu/23IDD-test/PC/run0020-smaller/PC_000104.cbf")
    print("done")
