import time, os, re
from watchdog.observers.polling import PollingObserver
from watchdog.events import FileSystemEventHandler, DirCreatedEvent, FileCreatedEvent
from multiprocessing import Process

from .smx_get_args import SMX_ARGS

class Handler(FileSystemEventHandler):
    def __init__(self, queue, count=0):
        super().__init__()
        self.task_queue = queue
        self.count = count

    def dispatch(self, event):
        if type(event) in [DirCreatedEvent] and os.path.basename(event.src_path).startswith('run'):
            # self.task_queue.put(event.src_path)  # new run created
            print("run {} found.".format(event.src_path))
        elif type(event) in [FileCreatedEvent] and re.search(r"_\d{5,8}.cbf$",
                                                             event.src_path):  # event.src_path.endswith('.cbf'):
            print("image {} found.".format(event.src_path))
            self.count += 1
            if SMX_ARGS.nimages == -1 or self.total <= SMX_ARGS.nimages:
                self.task_queue.put(event.src_path)  # new images created
            print("Total {} images found.".format(self.count))


class DirMonitor:
    def __init__(self, queue, directory=None, filelist=None):
        self.directory = directory
        self.filelist = filelist
        self.observer = PollingObserver()
        self.task_queue = queue
        self.total = 0
        # self.init_queue()

        initq = Process(target=self.init_queue)
        initq.start()

        self.process = None
        if self.directory:
            self.process = Process(target=self.monitor)
            self.process.start()

    def init_queue(self):
        if self.directory:
            for dirname, subdirlist, filelist in os.walk(self.directory):
                for f in sorted(filelist):
                    full_name = os.path.join(dirname, f)
                    if re.search(r"_\d{5,8}.cbf$", f) and os.path.isfile(full_name):
                        self.total += 1
                        if SMX_ARGS.nimages == -1 or self.total <= SMX_ARGS.nimages:
                            self.task_queue.put(full_name)
            print("Total {} images found.".format(self.total))

        if self.filelist:
            with open(self.filelist, 'r') as fh:
                files = fh.readlines()
                for f in files:
                    f = f.rstrip('\n')
                    if f.endswith('.cbf') and os.path.exists(f):
                        self.total += 1
                        if SMX_ARGS.nimages == -1 or self.total <= SMX_ARGS.nimages:
                            self.task_queue.put(f)
            print("Total {} images found.".format(self.total))

    def monitor(self):
        handler = Handler(self.task_queue, count=self.total)
        self.observer.schedule(handler, self.directory, recursive=SMX_ARGS.recursive)

        self.observer.start()
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            self.observer.stop()

        self.observer.join()

    def stop(self):
        if self.observer.is_alive():
            self.observer.stop()

        if self.process.is_alive():
            self.process.terminate()

    def is_alive(self):
        return self.process
