import os
import shutil
import time
from multiprocessing import Pool

from .cbf_header import PilatusHeader
from .smx_get_args import SMX_ARGS
from ..tasks.dozor import Dozor


def dozor(filelist, out_q, index_q, job_id, cbf_header, clean_up=False):
    tempdir = os.path.join(SMX_ARGS.proc_dir, "dozor_s{:07d}".format(job_id))

    if os.path.exists(tempdir):
        os.rmdir(tempdir)

    os.makedirs(tempdir)

    filter_it = Dozor(filelist,  header=cbf_header, wdir=tempdir)

    for hit in filter_it.get_hits():
        index_q.put(hit)

    for r in filter_it.get_all_results():
        out_q.put(r)

    if clean_up:
        shutil.rmtree(tempdir)


class PreProcessor:
    def __init__(self, preprocessor_q, index_q, out_q, chunk_size=8, pool_size=8):
        self.chunk_size = chunk_size
        self.task_queue = preprocessor_q
        self.out_queue = out_q
        self.index_queue = index_q
        self.pool = Pool(processes=pool_size)
        self.geometry = None

    def manage(self):
        job_id = 0
        try:
            filelist = list()
            waited_for = 0
            while True:
                if len(filelist) == self.chunk_size or (waited_for >= 5 and filelist):
                    job_id += 1

                    # generate geometry for the first image unless predefined
                    if not self.geometry and job_id == 1:
                        self.geometry = PilatusHeader(filelist[0]).header_dict

                    self.pool.apply_async(dozor, args=(filelist,
                                                       self.out_queue,
                                                       self.index_queue,
                                                       job_id,
                                                       self.geometry,
                                                       True
                                                       ))

                    filelist = list()
                    waited_for = 0

                if not self.task_queue.empty():
                    filelist.append(self.task_queue.get())
                    waited_for = 0
                else:
                    time.sleep(1)
                    waited_for += 1

        except:
            raise Exception("PreProcessor exception")

    def stop(self):
        if self.pool:
            self.pool.terminate()
