import sys

import numpy as np
import pyqtgraph as pg
from PyQt5 import QtGui, QtWidgets


class sinus_wave(QtGui.QWidget):
    def __init__(self):
        super(sinus_wave, self).__init__()
        self.init_ui()
        self.qt_connections()
        self.plotcurve = pg.PlotCurveItem()
        self.plotwidget.addItem(self.plotcurve)
        self.amplitude = 10
        self.t = 0
        self.updateplot()

        self.timer = pg.QtCore.QTimer()
        self.timer.timeout.connect(self.moveplot)
        self.timer.start(500)

    def init_ui(self):
        self.setWindowTitle('GMCA SMX Streamer')
        vbox = QtGui.QVBoxLayout()
        self.setLayout(vbox)

        self.plotwidget = pg.PlotWidget()
        vbox.addWidget(self.plotwidget)

        hbox = QtGui.QHBoxLayout()

        self.cellExplorerButton = QtGui.QPushButton("Cell explorer")
        self.adxvButton = QtGui.QPushButton("adxv")
        self.cheetahButton = QtGui.QPushButton("Run cheetah")

        self.showResolutionRings = QtGui.QPushButton("Show resolution ring")
        self.showResolutionRings.setCheckable(True)

        self.showPeaks = QtGui.QPushButton("Show peaks")
        self.showPeaks.setCheckable(True)

        self.showPredictions = QtGui.QPushButton("Show predictions")
        self.showPredictions.setCheckable(True)

        l_spot_radius = QtWidgets.QLabel("peak radius")
        self.spotRadius = QtWidgets.QLineEdit()
        # self.spotRadius.setFixedWidth(120)

        l_reflection_radius = QtWidgets.QLabel("reflection radius")
        self.reflectionRadius = QtWidgets.QLineEdit()
        # self.reflectionRadius.setFixedWidth(120)

        hbox.addWidget(self.cellExplorerButton)
        hbox.addWidget(self.adxvButton)
        hbox.addWidget(self.showPeaks)
        hbox.addWidget(l_spot_radius)
        hbox.addWidget(self.spotRadius)
        hbox.addWidget(self.showPredictions)
        hbox.addWidget(l_reflection_radius)
        hbox.addWidget(self.reflectionRadius)
        hbox.addWidget(self.showResolutionRings)
        hbox.addWidget(self.cheetahButton)

        self.showPeaks.toggled.connect(self.on_spots_toggle)
        self.showPredictions.toggled.connect(self.on_reflections_toggle)
        self.showResolutionRings.toggled.connect(self.on_resolution_toggle)

        vbox.addLayout(hbox)

        self.setGeometry(10, 10, 1000, 600)
        self.show()

    def on_spots_toggle(self):
        print("show spots=", self.showPeaks.isChecked())
        self.show_peaks_status = self.showPeaks.isChecked()

    def on_reflections_toggle(self):
        print("show reflections=", self.showPredictions.isChecked())
        self.show_reflections_status = self.showPredictions.isChecked()

    def on_resolution_toggle(self):
        print("resolution ring", self.showResolutionRings.isChecked())

    def qt_connections(self):
        self.cellExplorerButton.clicked.connect(self.on_increasebutton_clicked)
        self.adxvButton.clicked.connect(self.on_decreasebutton_clicked)

    def moveplot(self):
        self.t += 1
        self.updateplot()

    def updateplot(self):
        print
        "Update"
        data1 = self.amplitude * np.sin(np.linspace(0, 30, 121) + self.t)
        self.plotcurve.setData(data1)

    def on_increasebutton_clicked(self):
        print("Amplitude increased")
        self.amplitude += 1
        self.updateplot()

    def on_decreasebutton_clicked(self):
        print("Amplitude decreased")
        self.amplitude -= 1
        self.updateplot()


def main():
    app = QtGui.QApplication(sys.argv)
    app.setApplicationName('Sinuswave')
    ex = sinus_wave()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()
