from __future__ import absolute_import
import os
import logging


from .scriptjob import ScriptJob
from .xia2_report import Xia2Report

logger = logging.getLogger(__name__)


class Aimless(ScriptJob):
    def __init__(self, wdir='.',
                 script_name='aimless.sh',
                 script_text=None,
                 nproc=1,
                 runner='local',
                 pe_name='smp',
                 anomalous=True,
                 hklin='XDS_ASCII.mtz',
                 hklout='aimless.mtz',
                 xmlout='aimless.xml',
                 logfile='aimless.log',
                 extra_output=False,
                 xia2_report=False
                 ):
        super(Aimless, self).__init__(wdir,
                                      script_name,
                                      script_text,
                                      nproc,
                                      runner,
                                      pe_name)
        self.hklin = hklin
        self.hklout = hklout
        self.unmergedfile = None
        self.xmlout = xmlout
        self.logfile = logfile
        self.anomalous = anomalous
        self.extra_output = extra_output
        self.xia2_report = xia2_report
        self.report_dir = None

    def cmd_run_aimless_merge_only(self, highres=None):
        hklin = os.path.join(self.wdir, self.hklin)
        if os.path.isfile(hklin):
            self.unmergedfile = self.hklout.replace(".mtz", "_unmerged.mtz")
            unmergedsca = self.hklout.replace(".mtz", "_unmerged.sca")
            mergedsca = self.hklout.replace(".mtz", ".sca")

            if self.extra_output:
                cmd = "aimless hklin {} hklout {} xmlout {} hkloutunmerged {} " \
                      " scalepackunmerged {} scalepack {} > {} << eof\n".format(
                                                                                self.hklin,
                                                                                self.hklout,
                                                                                self.xmlout,
                                                                                self.unmergedfile,
                                                                                unmergedsca,
                                                                                mergedsca,
                                                                                self.logfile)
                cmd += "output merged unmerged scalepack unmerged\n"

            else:
                cmd = "aimless hklin {} hklout {} xmlout {} > {} << eof\n".format(
                                                                                self.hklin,
                                                                                self.hklout,
                                                                                self.xmlout,
                                                                                self.logfile)

            cmd += "sdcorrection norefine full 1 0 0\n"
            cmd += "run 1 all\n"
            cmd += "cycles 0\n"
            cmd += "bins 20\n"
            cmd += "scales constant\n"

            if self.anomalous:
                cmd += "anomalous on\n"
            else:
                cmd += "anomalous off\n"

            if highres:
                cmd += "resolution high {}\n".format(highres)

            cmd += "eof\n\n"

            self.script_text = cmd
            return self
        else:
            logger.error("{} does not exist".format(self.hklin))

    def run_aimless_merge_only(self, highres=None, timeout=1800):
        self.cmd_run_aimless_merge_only(highres=highres).run_sync(timeout=timeout)

        # using xia2.report to generate html report in background
        if self.xia2_report:
            self.report_dir = self.hklout.replace(".mtz", "-report")
            Xia2Report(self.unmergedfile,
                       unmergedfile_dir=self.wdir,
                       wdir=os.path.join(self.wdir, self.report_dir),
                       runner='local'
                       ).run_xia2_report(async_flag=True)
