from __future__ import absolute_import
import os
import logging

from .scriptjob import ScriptJob

logger = logging.getLogger(__name__)


class Dimple(ScriptJob):
    def __init__(self,
                 wdir='.',
                 script_name='dimple.sh',
                 script_text=None,
                 nproc=1,
                 runner='slurm',
                 pe_name='smp',
                 hklin_dir='.',
                 hklin_name='fast_dp.mtz',
                 model='model.pdb',
                 logfile='dimple.log'):

        super(Dimple, self).__init__(wdir,
                                     script_name,
                                     script_text,
                                     nproc,
                                     runner,
                                     pe_name)
        self.hklin_name = hklin_name
        self.hklin_dir = hklin_dir
        self.model = model
        self.hklin = os.path.join(self.hklin_dir, self.hklin_name)
        if not os.path.exists(self.hklin):
            raise RuntimeError("mtz file does not exist: {}".format(self.hklin))

        if not os.path.exists(model):
            raise RuntimeError("model file does not exist: {}".format(self.model))

        self.logfile = logfile

    def cmd_run_dimple(self):
        cmd = "cd {}\n".format(self.wdir)
        cmd += "ln -s {} {}\n".format(self.hklin, self.hklin_name)
        cmd += "dimple {} {} {}\n".format(self.hklin, self.model, self.wdir)
        self.script_text = cmd
        return self

    def report(self):
        pass

    def run(self, async_flag=False):
        logger.info("run dimple for difference map for locating ligand.")
        if async_flag:
            self.cmd_run_dimple().run_sync()
        else:
            return self.cmd_run_dimple().run_sync(timeout=1800)

        self.report()