from __future__ import absolute_import
import os
import logging

from .scriptjob import ScriptJob
from ..config.config import MORDA_SETUP_SH

logger = logging.getLogger(__name__)


class MoRDA(ScriptJob):
    def __init__(self,
                 wdir='.',
                 script_name='morda.sh',
                 script_text=None,
                 nproc=1,
                 sample_name=None,
                 seq_file=None,
                 runner='slurm',
                 pe_name='smp',
                 space_group_exploration=0,
                 hklin='fast_dp.mtz',
                 logfile='morda.log'):

        super(MoRDA, self).__init__(wdir,
                                     script_name,
                                     script_text,
                                     nproc,
                                     runner,
                                     pe_name)

        self.hklin = hklin
        self.sequence = seq_file
        if not (os.path.exists(self.hklin) and os.path.exists(self.sequence)):
            raise RuntimeError("mtz or sequence file does not exist: mtz={}, seq={}.".format(self.hklin, self.sequence))

        self.spacegroup_exploration = space_group_exploration
        self.sample_name = sample_name

        self.logfile = logfile

    def cmd_run_morda(self):
        cmd = MORDA_SETUP_SH

        spacegroup_alt = "-alt" if self.spacegroup_exploration else ""
        cmd += "morda -f {} -s {} {}\n".format(self.hklin, self.sequence, spacegroup_alt)
        self.script_text = cmd
        return self

    def report(self):
        pass

    def run(self, async_flag=False):
        logger.info("run MoRDA for molecular replacement.")
        if async_flag:
            self.cmd_run_morda().run_sync()
        else:
            return self.cmd_run_morda().run_sync(timeout=3600)

        self.report()
